// @license http://www.apache.org/licenses/LICENSE-2.0 Apache-2.0
// ^ for LibreJS (this has to be the first comment in the file)

// ==UserScript==
// @name              Image Max URL
// @namespace         http://tampermonkey.net/
// @version           0.13.1
// @description       Finds larger or original versions of images for 6500+ websites, including a powerful image popup feature
// @description:ko    6500개 이상의 사이트에 대해 고화질이나 원본 이미지를 찾아드립니다
// @description:fr    Trouve des images plus grandes ou originales pour plus de 6500 sites
// @description:es    Encuentra imágenes más grandes y originales para más de 6500 sitios
// @description:ru    Находит увеличенные или оригинальные версии изображений для более чем 6500 веб-сайтов
// @description:zh    为6500多个网站查找更大或原始图像
// @description:zh-CN 为6500多个网站查找更大或原始图像
// @description:zh-TW 為6500多個網站查找更大或原始圖像
// @description:zh-HK 為6500多個網站查找更大或原始圖像
// @author            qsniyg
// @homepageURL       https://qsniyg.github.io/maxurl/options.html
// @supportURL        https://github.com/qsniyg/maxurl/issues
// @icon              https://raw.githubusercontent.com/qsniyg/maxurl/b5c5488ec05e6e2398d4e0d6e32f1bbad115f6d2/resources/logo_256.png
// @include           *
// @grant             GM.xmlHttpRequest
// @grant             GM_xmlhttpRequest
// @grant             GM.setValue
// @grant             GM_setValue
// @grant             GM.getValue
// @grant             GM_getValue
// @grant             GM_registerMenuCommand
// @grant             GM_unregisterMenuCommand
// @grant             GM_addValueChangeListener
// @grant             GM_download
// @grant             GM_openInTab
// @grant             GM.openInTab
// @connect           *
// api.github.com is used for checking for updates (can be disabled through the "Check Updates" setting)
// @connect           api.github.com
// @run-at            document-start
// @license           Apache-2.0
// ==/UserScript==

// If you see "A userscript wants to access a cross-origin resource.", it's used for:
//   * Detecting whether or not the destination URL exists before redirecting
//   * API calls for various websites to find the larger image (e.g. for Flickr)
//     * You can control this with the "Rules using API calls" setting
//   * Downloading the image for the popup
//   * Querying a third-party library
//     * You can control this with the "Rules using 3rd-party libraries" setting
// Search for do_request if you want to see what the code does exactly.
//
// Please let me know if you have any questions or concerns regarding the script.

// Due to Greasyfork's 2MB limit, all comments within bigimage() had to be removed
// You can view the original source code here: https://github.com/qsniyg/maxurl/blob/master/userscript.user.js



var $$IMU_EXPORT$$;

// Disable linting because otherwise editing is incredibly slow
// jshint ignore: start
(function() {
	// Don't 'use strict', as it prevents nested functions
	//'use strict';

	var _nir_debug_ = false;

	if (_nir_debug_) {
		_nir_debug_ = {
			no_request: false,
			no_recurse: false,
			no_redirect: true
		};
	}

	var nullfunc = function(){};

	var is_extension = false;
	var is_webextension = false;
	var is_firefox_webextension = false;
	var extension_send_message = null;
	var extension_options_page = null;
	var is_extension_options_page = false;
	var is_options_page = false;
	var is_maxurl_website = false;
	var options_page = "https://qsniyg.github.io/maxurl/options.html";
	var preferred_options_page = options_page;
	var firefox_addon_page = "https://addons.mozilla.org/en-US/firefox/addon/image-max-url/";
	var userscript_update_url = "https://github.com/qsniyg/maxurl/blob/master/userscript_smaller.user.js?raw=true";
	var greasyfork_update_url = userscript_update_url;
	//var greasyfork_update_url = "https://greasyfork.org/scripts/36662-image-max-url/code/Image%20Max%20URL.user.js";
	var github_issues_page = "https://github.com/qsniyg/maxurl/issues";
	var current_version = null;
	var imagetab_ok_override = false;

	var get_window = function() {
		if (typeof(unsafeWindow) !== "undefined")
			return unsafeWindow || this.window || window;

		return this.window || window;
	};

	try {
		if (window.location.href.match(/^https?:\/\/qsniyg\.github\.io\/+maxurl\/+options\.html/) ||
			window.location.href.match(/^file:\/\/.*\/maxurl\/site\/options\.html/)) {
			is_options_page = true;
		}

		if (window.location.href.match(/^https?:\/\/qsniyg\.github\.io\/+maxurl\/+/) ||
			window.location.href.match(/^file:\/\/.*\/maxurl\/site\/(?:index|about|options)\.html/)) {
			is_maxurl_website = true;
		}
	} catch(e) {
	}

	try {
		var extension_manifest = chrome.runtime.getManifest();
		is_extension = extension_manifest.name === "Image Max URL";

		extension_options_page = chrome.runtime.getURL("extension/options.html");
		is_extension_options_page = window.location.href.replace(/[?#].*$/, "") === extension_options_page;
		is_options_page = is_options_page || is_extension_options_page;
		//options_page = extension_options_page; // can't load from website
		preferred_options_page = extension_options_page;

		if (is_extension) {
			is_webextension = true;
			if (navigator.userAgent.indexOf("Firefox") >= 0)
				is_firefox_webextension = true;

			current_version = extension_manifest.version;

			extension_send_message = function(message, respond) {
				message = deepcopy(message, {json:true});
				if (!respond)
					respond = nullfunc;
				return chrome.runtime.sendMessage(null, message, null, respond);
			};
		}
	} catch (e) {
	}

	var is_node = false;
	if ((typeof module !== 'undefined' && module.exports) &&
		typeof window === 'undefined' && typeof document === 'undefined') {
		is_node = true;
	}

	var is_scripttag = false;
	if (typeof imu_variable !== 'undefined' && (typeof(GM_xmlhttpRequest) === 'undefined' &&
												typeof(GM) === 'undefined'))
		is_scripttag = true;

	var is_userscript = false;
	if (!is_node && !is_scripttag && !is_extension)
		is_userscript = true;

	// https://stackoverflow.com/a/326076
	var check_in_iframe = function() {
		try {
			return window.self !== window.top;
		} catch (e) {
			return true;
		}
	};
	var is_in_iframe = check_in_iframe();
	var is_remote_possible = false;

	var is_interactive = is_extension || is_userscript;

	var userscript_manager = "unknown";
	var userscript_manager_version = "";
	if (is_userscript) {
		var gm_info = undefined;

		if (typeof GM_info === "function") {
			gm_info = GM_info();
		} else if (typeof GM_info === "object") {
			gm_info = GM_info;
		}

		if (typeof gm_info === "object") {
			if (gm_info.scriptHandler) {
				userscript_manager = gm_info.scriptHandler;
			}

			if (gm_info.version) {
				userscript_manager_version = gm_info.version;
			}
		} else if (typeof GM_fetch === 'function' && gm_info === null) {
			// Unfortunately FireMonkey currently doesn't implement GM_info's scriptHandler:
			//   https://github.com/erosman/support/issues/98#issuecomment-534671229
			// We currently have to rely on this hack
			userscript_manager = "FireMonkey";
			gm_info = {scriptHandler: userscript_manager};
		}

		if (_nir_debug_ && false) {
			console.log("GM_info", gm_info);
		}

		try {
			current_version = gm_info.script.version;
		} catch (e) {
			current_version = null;
		};
	}

	var get_random_text = function(length) {
		var text = "";

		while (text.length < length) {
			var newtext = Math.floor(Math.random() * 10e8).toString(26);
			text += newtext;
		}

		text = text.substr(0, length);
		return text;
	};

	var get_random_id = function() {
		return get_random_text(10) + Date.now();
	};

	var id_to_iframe = {};

	var get_frame_info = function() {
		return {
			id: current_frame_id,
			//url: window.location.href, // doesn't get updated for the iframe src's attribute?
			url: current_frame_url,
			size: [
				document.documentElement.scrollWidth,
				document.documentElement.scrollHeight
			]
		};
	};

	var find_iframe_for_info = function(info) {
		if (info.id in id_to_iframe)
			return id_to_iframe[info.id];

		var finish = function(iframe) {
			if (!iframe)
				return iframe;

			id_to_iframe[info.id] = iframe;
			return iframe;
		}

		var iframes = document.getElementsByTagName("iframe");
		var newiframes = [];
		for (var i = 0; i < iframes.length; i++) {
			if (iframes[i].src !== info.url)
				continue;

			newiframes.push(iframes[i]);
		}

		if (newiframes.length <= 1)
			return finish(newiframes[0]);

		iframes = newiframes;
		newiframes = [];
		for (var i = 0; i < iframes.length; i++) {
			if (iframes[i].scrollWidth !== info.size[0] ||
				iframes[i].scrollHeight !== info.size[1]) {
				continue;
			}

			newiframes.push(iframes[i]);
		}

		if (newiframes.length <= 1)
			return finish(newiframes[0]);

		// TODO: check cursor too
		return null;
	};

	var iframe_to_id = function(iframe) {
		for (var id in id_to_iframe) {
			if (id_to_iframe[id] === iframe)
				return id;
		}

		return null;
	};

	var id_to_iframe_window = function(id) {
		if (!(id in id_to_iframe))
			return null;

		try {
			if (id_to_iframe[id].contentWindow)
				return id_to_iframe[id].contentWindow;
		} catch (e) {
			// not allowed
			return false;
		}

		return id_to_iframe[id];
	}

	var remote_send_message = null;
	var remote_send_reply = null;
	var remote_reply_ids = {};
	var current_frame_id = null;
	var current_frame_url = null;

	var raw_remote_send_message = null;
	var remote_send_message = nullfunc;
	var remote_send_reply = nullfunc;
	var imu_message_key = "__IMU_MESSAGE__";

	if (is_extension) {
		raw_remote_send_message = function(to, message) {
			extension_send_message(message)
		};

		is_remote_possible = true;
	} else if (is_interactive) {
		if (is_in_iframe && window.parent) {
			id_to_iframe["top"] = window.parent;
		}

		raw_remote_send_message = function(to, message) {
			if (!to && is_in_iframe)
				to = "top"; // fixme?

			var specified_window;
			if (to && to in id_to_iframe) {
				specified_window = id_to_iframe_window(to);
				if (!specified_window) // not allowed
					return;
			}

			message.imu = true;

			var wrapped_message = {};
			wrapped_message[imu_message_key] = message;

			if (!specified_window) {
				for (var i = 0; i < window.frames.length; i++) {
					try {
						window.frames[i].postMessage(wrapped_message, "*");
					} catch (e) {
						// not allowed
						continue;
					}
				}
			} else {
				specified_window.postMessage(wrapped_message, "*");
			}
		};

		is_remote_possible = true;

		if (window.location.hostname === "cafe.daum.net") {
			// unfortunately they interpret all message events, leading to bugs in their website. thanks to solplparty on discord for noticing
			is_remote_possible = false;
		}
	}

	if (is_remote_possible) {
		current_frame_url = window.location.href;
		current_frame_id = get_random_id() + " " + current_frame_url;

		if (!is_in_iframe)
			current_frame_id = "top";

		remote_send_message = function(to, data, cb) {
			var id = undefined;

			if (cb) {
				id = get_random_id();
				remote_reply_ids[id] = cb;
			}

			//console_log("remote", data);
			raw_remote_send_message(to, {
				type: "remote",
				data: data,
				to: to,
				from: current_frame_id,
				response_id: id
			});
		};

		remote_send_reply = function(to, response_id, data) {
			raw_remote_send_message(to, {
				type: "remote_reply",
				data: data,
				response_id: response_id
			});
		};
	}

	var can_use_remote = function() {
		return is_remote_possible && settings.allow_remote;
	};

	var can_iframe_popout = function() {
		return can_use_remote() && settings.mouseover_use_remote;
	};

	var do_request_browser = function (request) {
		if (_nir_debug_) {
			console_log("do_request_browser", request);
		}

		var method = request.method || "GET";

		var xhr = new XMLHttpRequest();
		xhr.open(method, request.url, true);

		if (request.responseType)
			xhr.responseType = request.responseType;

		var do_final = function(override, cb) {
			if (_nir_debug_) {
				console_log("do_request_browser's do_final", xhr, cb);
			}

			var resp = {
				readyState: xhr.readyState,
				finalUrl: xhr.responseURL,
				responseHeaders: xhr.getAllResponseHeaders(),
				responseType: xhr.responseType,
				status: xhr.status, // file:// returns 0, tracking protection also returns 0
				statusText: xhr.statusText
			};

			resp.response = xhr.response;

			try {
				resp.responseText = xhr.responseText;
			} catch (e) {}

			cb(resp);
		};

		var add_handler = function(event, empty) {
			xhr[event] = function() {
				if (empty) {
					return request[event](null);
				}

				do_final({}, function(resp) {
					request[event](resp);
				});
			};
		};

		add_handler("onload");
		add_handler("onerror");
		add_handler("onprogress");
		add_handler("onabort", true);

		xhr.send(request.data);

		return {
			abort: function() {
				xhr.abort();
			}
		};
	};

	if (typeof XMLHttpRequest !== "function") {
		do_request_browser = null;
	}

	var extension_requests = {};

	var do_request_raw = null;
	if (is_extension) {
		do_request_raw = function(data) {
			var reqid;

			extension_send_message({
				type: "request",
				data: data
			}, function (response) {
				if (response.type !== "id") {
					console_error("Internal error: Wrong response", response);
					return;
				}

				reqid = response.data;

				extension_requests[reqid] = {
					id: reqid,
					data: data
				};
			});

			return {
				abort: function() {
					if (reqid === undefined) {
						console_error("abort() was called before the request was initialized");
						return;
					}

					extension_send_message({
						type: "abort_request",
						data: reqid
					});
				}
			};
		};
	} else if (typeof(GM_xmlhttpRequest) !== "undefined") {
		do_request_raw = GM_xmlhttpRequest;
	} else if (typeof(GM) !== "undefined" && typeof(GM.xmlHttpRequest) !== "undefined") {
		do_request_raw = GM.xmlHttpRequest;
	}

	var register_menucommand = nullfunc;
	var unregister_menucommand = nullfunc;
	var num_menucommands = 0;

	if (is_userscript) {
		if (typeof(GM_registerMenuCommand) !== "undefined") {
			register_menucommand = function(name, func) {
				num_menucommands++;

				var caption = "[" + num_menucommands + "] " + name;
				var id = GM_registerMenuCommand(caption, func);

				if (id === undefined || id === null)
					id = caption;
				return id;
			};
		}

		if (typeof(GM_unregisterMenuCommand) !== "undefined") {
			unregister_menucommand = function(id) {
				num_menucommands--;

				return GM_unregisterMenuCommand(id);
			};
		}
	}

	var open_in_tab = nullfunc;

	if (is_userscript) {
		if (typeof(GM_openInTab) !== "undefined") {
			open_in_tab = GM_openInTab;
		} else if (typeof(GM) !== "undefined" && typeof(GM.openInTab) !== "undefined") {
			open_in_tab = GM.openInTab;
		}

		if (open_in_tab !== nullfunc) {
			register_menucommand("Options", function() {
				// this gets run for every frame the script is injected in
				if (is_in_iframe)
					return;

				open_in_tab(options_page);
			});
		}
	}

	var open_in_tab_imu = function(imu, bg, cb) {
		if (is_extension) {
			extension_send_message({
				type: "newtab",
				data: {
					imu: imu,
					background: bg
				}
			}, cb);
		} else if (is_userscript && open_in_tab) {
			open_in_tab(imu.url, bg);
			if (cb) {
				cb();
			}
		}
	};

	var check_tracking_blocked = function(result) {
		// FireMonkey returns null for result if blocked
		// GreaseMonkey returns null for status if blocked
		if (!result || result.status === 0 || result.status === null) {
			if (result && result.finalUrl && /^file:\/\//.test(result.finalUrl))
				return false;

			return true;
		}
		return false;
	};

	var do_request = null;
	if (do_request_raw) {
		do_request = function(data) {
			if (_nir_debug_) {
				console_log("do_request", deepcopy(data));
			}

			if (!data.onerror)
				data.onerror = data.onload;

			// For cross-origin cookies
			if (!("withCredentials" in data)) {
				data.withCredentials = true;
			}

			var raw_request_do = do_request_raw;
			if (is_userscript && settings.allow_browser_request) {
				if (userscript_manager === "Falkon GreaseMonkey" ||
					// USI doesn't properly support blob responses: https://bitbucket.org/usi-dev/usi/issues/13/various-problems-with-gm_xmlhttprequest
					(userscript_manager === "USI" && data.need_blob_response)) {
					raw_request_do = do_request_browser;
					delete data.trackingprotection_failsafe;
				}
			}

			if (data.trackingprotection_failsafe && settings.allow_browser_request && do_request_browser) {
				var real_onload = data.onload;
				var real_onerror = data.onerror;

				var finalcb = function(resp, iserror) {
					if (_nir_debug_) {
						console_log("do_request's finalcb:", resp, iserror);
					}

					if (check_tracking_blocked(resp)) {
						// Workaround for a bug in FireMonkey where it calls both onload and onerror: https://github.com/erosman/support/issues/134
						data.onload = null;
						data.onerror = null;

						var newdata = shallowcopy(data);
						newdata.onload = real_onload;
						newdata.onerror = real_onerror;

						return do_request_browser(newdata);
					} else {
						if (iserror) {
							real_onerror(resp);
						} else {
							real_onload(resp);
						}
					}
				};

				data.onload = function(resp) {
					finalcb(resp, false);
				};

				data.onerror = function(resp) {
					finalcb(resp, true);
				};
			}

			if (data.responseType === "blob" && !settings.use_blob_over_arraybuffer) {
				(function(real_onload) {
					data.onload = function(resp) {
						var newresp = resp;

						if (resp.response) {
							newresp = shallowcopy(resp);
							newresp.response = new native_blob([resp.response]);
						}

						real_onload(newresp);
					};
				})(data.onload);

				data.responseType = "arraybuffer";
			}

			return raw_request_do(data);
		};
	}

	var get_cookies = null;
	if (is_extension) {
		get_cookies = function(url, cb) {
			if (settings.browser_cookies === false) {
				return cb(null);
			}

			extension_send_message({
				type: "getcookies",
				data: {url: url}
			}, function(message) {
				cb(message.data);
			});
		};
	}

	var cookies_to_httpheader = function(cookies) {
		// deduplication apparently isn't necessary (browser duplicates them too)

		var strs = [];
		for (var i = 0; i < cookies.length; i++) {
			var str = cookies[i].name + "=" + cookies[i].value;
			strs.push(str);
		}

		return strs.join("; ");
	};

	var bigimage_filter = function() {
		return true;
	};

	if (is_interactive) {
		bigimage_filter = function(url) {
			for (var i = 0; i < blacklist_regexes.length; i++) {
				if (blacklist_regexes[i].test(url))
					return false;
			}

			return true;
		}
	}

	var default_options = {
		fill_object: true,
		null_if_no_change: false,
		catch_errors: true,
		use_cache: true,
		use_api_cache: true,
		urlcache_time: 60*60,
		iterations: 200,
		exclude_problems: [
			"watermark",
			"smaller",
			"possibly_different",
			"possibly_broken"
		],
		exclude_videos: false,
		include_pastobjs: true,
		force_page: false,
		allow_thirdparty: false,
		allow_thirdparty_libs: true,
		filter: bigimage_filter,

		rule_specific: {
			deviantart_prefer_size: false,
			imgur_source: true,
			imgur_nsfw_headers: null,
			instagram_use_app_api: true,
			instagram_gallery_postlink: false,
			tiktok_no_watermarks: true,
			tumblr_api_key: null
		},

		do_request: do_request,
		get_cookies: get_cookies,
		host_url: null,
		document: null,
		window: null,
		element: null,

		cb: null
	};

	var default_object = {
		url: null,
		always_ok: false,
		likely_broken: false,
		can_head: true,
		head_ok_errors: [],
		head_wrong_contenttype: false,
		head_wrong_contentlength: false,
		need_blob: false,
		waiting: false,
		redirects: false,
		forces_download: false,
		is_private: false,
		is_original: false,
		norecurse: false,
		forcerecurse: false,
		bad: false,
		bad_if: [],
		fake: false,
		video: false,
		headers: {},
		referer_ok: {
			same_domain: false,
			same_domain_nosub: false
		},
		extra: {
			page: null,
			caption: null
		},
		filename: "",
		problems: {
			watermark: false,
			smaller: false,
			possibly_different: false,
			possibly_broken: false,
			possibly_upscaled: false
		}
	};

	//var options_page = "https://qsniyg.github.io/maxurl/options.html";

	// restore console.log for websites that remove it (twitter)
	// https://gist.github.com/Ivanca/4586071
	//var console_log = function(){ return window.console.__proto__.log.apply(console, arguments) } ;
	//var console_error = function(){ return window.console.__proto__.error.apply(console, arguments) } ;

	// since the userscript is run first, this generally shouldn't be a problem
	var console_log = console.log;
	var console_error = console.error;
	var console_warn = console.warn;
	var console_trace = console.trace;
	var JSON_stringify = JSON.stringify;
	var JSON_parse = JSON.parse;

	var native_functions = {};
	var get_native_functions = function(functions) {
		// thanks to tophf here: https://github.com/violentmonkey/violentmonkey/issues/944
		var result = {};
		var iframe = document.createElement("iframe");
		iframe.srcdoc = ""; //"javascript:0"
		document.documentElement.appendChild(iframe);
		var frame_window = iframe.contentWindow;

		for (var i = 0; i < functions.length; i++) {
			var func = functions[i];
			native_functions[func] = frame_window[func];
		}

		iframe.parentElement.removeChild(iframe);
	};

	if (is_interactive) {
		try {
			get_native_functions(["Blob", "atob", "btoa"]);
		} catch (e) {
			console_error(e);
		}
	}

	// native_functions returns iframe
	var our_EventTarget, EventTarget_addEventListener, EventTarget_removeEventListener;

	if (is_interactive) {
		our_EventTarget = EventTarget;
		EventTarget_addEventListener = our_EventTarget.prototype.addEventListener;
		EventTarget_removeEventListener = our_EventTarget.prototype.removeEventListener;
	}

	var our_addEventListener = function(element, event, handler, options) {
		// VM compatibility
		if (element === window && element.unsafeWindow)
			element = element.unsafeWindow;

		EventTarget_addEventListener.call(element, event, handler, options);
	};

	var our_removeEventListener = function(element, event, handler, options) {
		EventTarget_removeEventListener.call(element, event, handler, options);
	};

	var sanity_test = function(orig, correct, check) {
		if (!orig)
			return correct;

		if (check) {
			try {
				check(orig);
				return orig;
			} catch (e) {};
		}

		return correct;
	};

	var is_array = sanity_test(Array.isArray, function(x) { return x instanceof Array; });

	function is_element(x) {
		if (!x || typeof x !== "object")
			return false;

		if (("namespaceURI" in x) && ("ariaSort" in x) && ("nodeType" in x)) {
			return true;
		}

		// window
		if (typeof x.HTMLElement === "function" && typeof x.navigator === "object") {
			return true;
		}

		// very slow
		if (is_interactive) {
			if ((x instanceof Element) ||
				(x instanceof HTMLDocument) ||
				(x instanceof Window)) {
				return true;
			}
		}

		return false;
	}

	function is_iterable_object(x) {
		return typeof x === "object" && x !== null && !is_array(x) && !is_element(x);
	}

	var shallowcopy_obj = function(x) {
		result = {};

		for (var key in x) {
			result[key] = x[key];
		}
		return result;
	};

	if ("assign" in Object) {
		// FIXME: should this be kept? it's faster, but it causes issues with object instances, like dom rects. it works fine with plain objects though
		// it's about as good as JSON.parse(JSON.stringify(x)), but shallow
		shallowcopy_obj = function(x) {
			return Object.assign({}, x);
		};
	}

	function shallowcopy(x) {
		var result = x;

		if (!is_iterable_object(x)) {
			return result;
		} else if (is_array(x)) {
			result = [];
			for (var i = 0; i < x.length; i++) {
				var item = x[i];
				result.push(item);
			}
			return result;
		} else if (typeof x === "object") {
			return shallowcopy_obj(x);
		}

		return result;
	}

	function deepcopy(x, options) {
		if (!options)
			options = {};
		if (!options.history)
			options.history = [];

		var result;

		if (typeof x === "string" || x === null || typeof x === "undefined") {
			return x;
		} else if (is_element(x)) {
			if (options.json) {
				return undefined;
			} else {
				return x;
			}
		} else if (typeof x === "function") {
			if (options.json) {
				return undefined;
			} else {
				return x;
			}
		} else if (typeof x === "object") {
			if (array_indexof(options.history, x) >= 0)
				return;
			else
				options.history.push(x);

			if (is_array(x)) {
				result = [];
				for (var i = 0; i < x.length; i++) {
					var item = x[i];
					result.push(deepcopy(item, options));
				}
			} else {
				result = {};
				for (var key in x) {
					try {
						result[key] = deepcopy(x[key], options);
					} catch (e) {
						result[key] = x[key];
					}
				}
			}

			return result;
		} else {
			return x;
		}
	}

	var base64_decode_orig = nullfunc;
	if (is_node && typeof atob === 'undefined') {
		base64_decode_orig = function(a) {
			return Buffer.from(a, 'base64').toString('binary');
		};
	} else if (typeof atob !== 'undefined') {
		base64_decode_orig = native_functions.atob || atob;
	}

	// Some websites replace atob, so we have to provide our own implementation in those cases
	// https://stackoverflow.com/a/15016605
	// unminified version: https://stackoverflow.com/a/3058974
	var base64_decode_correct = function(s) {
		var e={},i,b=0,c,x,l=0,a,r='',w=String.fromCharCode,L=s.length;
		var A="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
		for(i=0;i<64;i++){e[A.charAt(i)]=i;}
		for(x=0;x<L;x++){
			c=e[s.charAt(x)];b=(b<<6)+c;l+=6;
			while(l>=8){((a=(b>>>(l-=8))&0xff)||(x<(L-2)))&&(r+=w(a));}
		}
		return r;
	};

	var base64_decode_test = function(func) {
		if (func("dGVzdA==") === "test") {
			return true;
		}

		return false;
	};

	var base64_decode = sanity_test(base64_decode_orig, base64_decode_correct, base64_decode_test);

	var base64_encode = nullfunc;
	if (typeof btoa !== 'undefined') {
		base64_encode = native_functions.btoa || btoa;
	}

	// https://www.mycomicshop.com/search?minyr=1938&maxyr=1955&TID=29170235
	// this site replaces Array.indexOf
	// cache Array.prototype.indexOf in case it changes while the script is executing
	var array_prototype_indexof = Array.prototype.indexOf;
	var array_indexof_orig = function(array, x) {
		return array_prototype_indexof.call(array, x);
	};

	var array_indexof_correct = function(array, x) {
		if (typeof array === "string") {
			// TODO: make sure Array.from is sane
			array = Array.from(array);
		}

		for (var i = 0; i < array.length; i++) {
			if (array[i] === x) {
				return i;
			}
		}

		return -1;
	};

	var array_indexof_check = function(func) {
		var test_array = ["a", "b"];
		var test_string = "abc";
		if (func(test_array, "not here") === -1 &&
			func(test_array, "b") === 1 &&
			func(test_string, "n") === -1 &&
			func(test_string, "b") === 1 &&
			func(test_string, "bc") === -1) {
			return true;
		}

		return false;
	};

	var array_indexof = sanity_test(array_indexof_orig, array_indexof_correct, array_indexof_check);

	var string_prototype_indexof = String.prototype.indexOf;
	var string_indexof_orig = function(string, x) {
		return string_prototype_indexof.call(string, x);
	};

	var string_indexof_correct = function(string, x) {
		if (x.length === 0)
			return 0;

		var x_i = 0;
		for (var i = 0; i < string.length; i++) {
			if (string.charAt(i) === x.charAt(x_i)) {
				if (x_i + 1 === x.length) {
					return i - x_i;
				} else {
					x_i++;
				}
			} else {
				x_i = 0;
			}
		}

		return -1;
	};

	var string_indexof_check = function(func) {
		var test_string = "abc";
		if (func(test_string, "n") === -1 &&
			func(test_string, "b") === 1 &&
			func(test_string, "bc") === 1 &&
			func(test_string, "bcz") === -1 &&
			func(test_string, "") === 0) {
			return true;
		}

		return false;
	};

	var string_indexof = sanity_test(string_indexof_orig, string_indexof_correct, string_indexof_check);

	var native_blob = nullfunc;
	if (typeof Blob !== "undefined") {
		native_blob = native_functions.Blob || Blob;
	}

	var serialize_event = function(event) {
		return deepcopy(event, {json: true});
	};

	// Note: None of this information is automatically sent anywhere, it's only displayed to the user when something crashes.
	var get_crashlog_info = function() {
		var ua = "";
		try {ua = window.navigator.userAgent} catch (e) {}

		var imu_version = "(!!!UNKNOWN, PLEASE FILL IN!!!)\n";
		try {imu_version = gm_info.script.version} catch (e) {}

		var keys = [
			"User agent: " + ua,
			"Is userscript: " + is_userscript,
			"Is addon: " + is_extension,
			"Image Max URL version: " + imu_version
		];

		if (is_userscript) {
			try {
				keys.push("Userscript manager: " + userscript_manager);
				keys.push("Userscript manager version: " + userscript_manager_version);
			} catch (e) {}
		}

		return keys.join("\n");
	};

	function overlay_object(base, obj) {
		if (typeof base === "function" || is_array(base))
			return obj; // FIXME?

		if (typeof base === "object") {
			if (typeof obj !== "object")
				return obj;

			for (var key in obj) {
				if (key in base) {
					base[key] = overlay_object(base[key], obj[key]);
				} else {
					base[key] = obj[key];
				}
			}

			return base;
		}

		return obj;
	}

	var parse_boolean = function(bool) {
		if (bool === "true" || bool === true || bool === 1)
			return true;

		if (bool === "false" || bool === false || bool === 0)
			return false;

		return;
	};

	// https://stackoverflow.com/a/25603630
	function get_language() {
		if (typeof navigator === "undefined")
			return "en";

		if (navigator.languages)
			return navigator.languages[0];

		return navigator.language || navigator.userLanguage;
	}

	var supported_languages = ["en", "ko", "fr"];

	var browser_language = "en";
	try {
		browser_language = get_language().replace(/-.*/, "").toLowerCase();
		if (array_indexof(supported_languages, browser_language) < 0)
			browser_language = "en";
	} catch (e) {
		console_error(e);
	}

	var strings = {
		"options_header": {
			"en": "Options",
			"ko": "설정"
		},
		"yes": {
			"en": "Yes",
			"ko": "예",
			"fr": "Oui"
		},
		"no": {
			"en": "No",
			"ko": "아니오",
			"fr": "Non"
		},
		"category_redirection": {
			"en": "Redirection",
			"ko": "리디렉션"
		},
		"Enable redirection": {
			"ko": "리디렉션 사용",
			"fr": "Activer la redirection"
		},
		"Add to history": {
			"ko": "브라우저 기록에 추가",
			"fr": "Ajouter à l'historique"
		},
		"Use GET if HEAD is unsupported": {
			"ko": "HEAD 지원되지 않으면 GET 사용",
			"fr": "Utiliser GET si HEAD n'est pas supporté"
		},
		"Try finding original page/caption": {
			"fr": "Essayer de trouver la page d'origine/sous-titre"
		},
		"Enable mouseover popup": {
			"en": "Enable image popup",
			"ko": "이미지 팝업 사용",
			"fr": "Activer le popup"
		},
		"Mouseover popup action": {
			"en": "Popup action",
			"ko": "이미지 팝업 작업"
		},
		"category_popup": {
			"en": "Popup",
			"ko": "팝업"
		},
		"subcategory_settings": {
			"en": "Settings"
		},
		"subcategory_ui": {
			"en": "UI"
		},
		"subcategory_trigger": {
			"en": "Trigger",
			"ko": "트리거",
			"fr": "Déclencheur"
		},
		"subcategory_open_behavior": {
			"en": "Open Behavior"
		},
		"subcategory_close_behavior": {
			"en": "Close Behavior"
		},
		"subcategory_behavior": {
			"en": "Popup Behavior"
		},
		"subcategory_video": {
			"en": "Video",
			"ko": "영상"
		},
		"subcategory_gallery": {
			"en": "Gallery"
		},
		"subcategory_popup_other": {
			"en": "Other",
			"fr": "Autre"
		},
		"New tab": {
			"ko": "새 탭",
			"fr": "Nouvel onglet"
		},
		"Mouseover popup trigger": {
			// FIXME is 트리거 correct?
			"en": "Popup trigger",
			"ko": "팝업 트리거",
			"fr": "Déclencheur du popup"
		},
		"Mouseover": {
			// FIXME
			"ko": "마우스"
		},
		"Key trigger": {
			"ko": "키 바인딩"
		},
		"Popup trigger key": {
			"ko": "팝업 키 바인딩"
		},
		"Popup trigger delay": {
			// FIXME?
			"ko": "팝업 작업 지연 시간"
		},
		"seconds": {
			"ko": "초"
		},
		"Popup UI": {
			"ko": "팝업 UI"
		},
		"Opacity": {
			"ko": "불투명",
			"fr": "Opacité"
		},
		"Gallery counter": {
			"ko": "갤러리 이미지 수",
			"fr": "Nombre d'images dans la galerie"
		},
		"Gallery counter max": {
			"ko": "갤러리 이미지 수의 최대값",
			"fr": "Nombre max d'images a compter pour la galerie"
		},
		"images": {
			"ko": "이미지"
		},
		"Options Button": {
			"ko": "설정 링크"
		},
		"Rotation Buttons": {},
		"Video": {
			"ko": "영상"
		},
		"Media": {
			"ko": "미디어"
		},
		"Both images and video": {
			"ko": "사진+영상"
		},
		"Keep popup open until": {
			"ko": "팝업 닫으려면"
		},
		"Any trigger is released": {
			"ko": "아무 키 놓습니다"
		},
		"All triggers are released": {
			"ko": "키 다 놓습니다"
		},
		"ESC/Close is pressed": {
			"ko": "ESC/닫기 누릅니다"
		},
		"Popup default zoom": {
			"ko": "확대/축소 기본값"
		},
		"Fit to screen": {
			"ko": "화면 크기에 맞춤",
			"fr": "Adapter a l'ecran"
		},
		"Full size": {
			"ko": "전체 크기",
			"fr": "Taille réelle"
			//"fr": "Pleine grandeur" // should it be this instead?
		},
		"Popup panning method": {
			"ko": "이미지 이동하려면"
		},
		"Movement": {
			"ko": "마우스 움직입니다"
		},
		"Drag": {
			"ko": "끕니다"
		},
		"Popup scroll action": {
			"ko": "마우스 휠 작업"
		},
		"Zoom": {
			"ko": "줌"
		},
		"Pan": {
			"ko": "이동"
		},
		"None": {
			"ko": "없다"
		},
		"Zoom behavior": {
			// FIXME?
			"ko": "줌 동작"
		},
		"Fit/Full": {
			"ko": "화면맞춤/전체"
		},
		"Incremental": {
			"ko": "증분",
			"fr": "Incrémentale"
		},
		"Popup position": {
			"ko": "팝업 위치"
		},
		"Mouse cursor": {
			"ko": "마우스 커서"
		},
		"Page middle": {
			"ko": "페이지 중간"
		},
		"Clicking image downloads": {
			"ko": "이미지 클릭하면 다운로드",
			"fr": "Cliquer l'image le télécharge"
		},
		"Popup for plain hyperlinks": {
			"ko": "일반적인 링크에도 팝업"
		},
		"Popup CSS style": {
			"ko": "팝업 CSS"
		},
		"category_rules": {
			"en": "Rules",
			"ko": "규칙",
			"fr": "Règles"
		},
		"Larger watermarked images": {
			"ko": "더 크지만 워터마크 있는 이미지"
		},
		"Smaller non-watermarked images": {
			"ko": "더 작지만 워터마크 없는 이미지"
		},
		"Possibly different images": {
			"ko": "다를 수 있는 이미지",
			"fr": "Images possiblement différentes"
		},
		"Possibly broken images": {
			"ko": "손상될 수 있는 이미지",
			"fr": "Images possiblement brisée"
		},
		"Rules using 3rd-party websites": {
			"ko": "3파티 사이트를 사용하는 규칙",
			"fr": "Règles utilisant des sites 3rd-party"
		},
		"Newsen": {
			"ko": "뉴스엔"
		},
		"subcategory_rule_specific": {
			"en": "Rule-specific"
		},
		"category_website": {
			"en": "Website",
			"ko": "웹사이트",
			"fr": "Site"
		},
		"Use userscript": {
			"ko": "유저스크립트 사용하기",
			"fr": "Utiliser ce userscript"
		},
		"Website image preview": {
			"ko": "링크 붙인 후 이미지 미리보기"
		},
		"saved_refresh_target": {
			"en": "Saved! Refresh the target page for changes to take effect",
			"ko": "저장됩니다. 번경사항 적용하려면 대상 웹페이지 다시 로드하십시오",
			"fr": "Enregistré! Actualiser la page que vous visitez pour que les changements prennent effet" // FIXME: this is a terrible translation
		},
		"saved_no_refresh": {
			"en": "Saved!",
			"ko": "저장됩니다",
			"fr": "Enregistré!"
		},
		"save": {
			"en": "Save",
			"ko": "저장",
			"fr": "Enregistrer"
		},
		"record": {
			"en": "Record"
			// "Enregistrer" also means "Record"
		},
		"cancel": {
			"en": "Cancel",
			"ko": "취소",
			"fr": "Annuler"
		},
		"Mouseover popup (%%1) is needed to display the original version": {
			"ko": "원본 이미지 보려면 팝업 (%%1) 필요합니다",
			"fr": "Popup (%%1) est nécessaire pour trouver la version originale"
		},
		"custom headers": {
			"ko": "특정 헤더",
			"fr": "en-têtes spéciales"
		},
		"forces download": {}, // TODO
		"Close": {
			"ko": "닫기",
			"fr": "Fermer"
		},
		"Previous": {
			"ko": "이전",
			"fr": "Image précédente"
		},
		"Next": {
			"ko": "다음",
			"fr": "Image suivante"
		},
		"Left Arrow": {
			"ko": "왼쪽 화살표",
			"fr": "Flèche gauche"
		},
		"Right Arrow": {
			"ko": "오른쪽 화살표",
			"fr": "Flèche droite"
		},
		"Blacklist": {
			"ko": "블랙리스트"
		},
		"Blacklist engine": {
			"ko": "블랙리스트 엔진"
		},
		"Simple (glob)": {
			"ko": "단순 (glob)"
		},
		"Regex": {
			"ko": "정규식"
		},
		"category_extension": {
			"en": "Extension"
		},
		"rotate_left_btn": {
			"en": "Rotate Left"
		},
		"rotate_right_btn": {
			"en": "Rotate Right"
		},
		"category_extra": {
			"en": "Buttons"
		},
		"subcategory_replaceimages": {
			"en": "Replace Images"
		},
		"subcategory_highlightimages": {
			"en": "Highlight Images"
		},
		"category_general": {
			"en": "General",
			"ko": "일반"
		},
		"Language": {
			"ko": "언어"
		}
	};

	function _(str) {
		if (typeof str !== "string") {
			return str;
		}

		var language = settings.language;

		if (str in strings) {
			if (language in strings[str]) {
				str = strings[str][language];
			} else if ("en" in strings[str]) {
				str = strings[str]["en"];
			}
		}

		var parts = [];
		var currentpart = "";
		for (var i = 0; i < str.length; i++) {
			if (str[i] == '%') {
				if ((i + 2) < str.length) {
					if (str[i + 1] == '%') {
						var num = parseInt(str[i + 2]);
						if (!isNaN(num)) {
							parts.push(currentpart);
							currentpart = "";
							parts.push(arguments[num]);
							i += 2;
							continue;
						}
					}
				}
			}

			currentpart += str[i];
		}

		parts.push(currentpart);
		return parts.join("");
	}

	var old_settings_keys = [
		"mouseover_trigger",
		"mouseover_use_fully_loaded_video",
		"mouseover_use_fully_loaded_image",
		"mouseover_close_on_leave_el",
		"mouseover_scroll_behavior",
		"mouseover_mask_styles"
	];

	var settings = {
		// thanks to decembre on github for the idea: https://github.com/qsniyg/maxurl/issues/14#issuecomment-530760246
		imu_enabled: true,
		language: browser_language,
		check_updates: true,
		check_update_interval: 24,
		// thanks to forefix on firefox for the idea: https://github.com/qsniyg/maxurl/issues/189
		dark_mode: false,
		settings_tabs: false,
		settings_visible_description: true,
		settings_show_disabled: true,
		settings_show_requirements: true,
		advanced_options: false,
		allow_browser_request: true,
		use_blob_over_arraybuffer: false,
		allow_live_settings_reload: true,
		allow_remote: true,
		disable_keybind_when_editing: true,
		redirect: true,
		redirect_history: true,
		canhead_get: true,
		redirect_force_page: false,
		print_imu_obj: false,
		redirect_disable_for_responseheader: false,
		mouseover: true,
		// thanks to blue-lightning on github for the idea: https://github.com/qsniyg/maxurl/issues/16
		mouseover_open_behavior: "popup",
		//mouseover_trigger: ["ctrl", "shift"],
		mouseover_trigger_behavior: "keyboard",
		// thanks to 894-572 on github for the idea: https://github.com/qsniyg/maxurl/issues/30
		mouseover_trigger_key: ["shift", "alt", "i"],
		mouseover_trigger_delay: 1,
		mouseover_trigger_mouseover: false,
		mouseover_trigger_prevent_key: ["shift"],
		// also thanks to blue-lightning: https://github.com/qsniyg/maxurl/issues/16
		mouseover_close_behavior: "esc",
		// thanks to acid-crash on github for the idea: https://github.com/qsniyg/maxurl/issues/14#issuecomment-436594057
		mouseover_close_need_mouseout: true,
		mouseover_jitter_threshold: 30,
		mouseover_cancel_popup_when_elout: true,
		mouseover_cancel_popup_with_esc: true,
		// thanks to remlap on discord for the idea: https://github.com/qsniyg/maxurl/issues/250
		mouseover_auto_close_popup: false,
		mouseover_auto_close_popup_time: 5,
		// thanks to decembre on github for the idea: https://github.com/qsniyg/maxurl/issues/14#issuecomment-530760246
		mouseover_use_hold_key: true,
		mouseover_hold_key: ["i"],
		mouseover_hold_position_center: false,
		// thanks to decembre on github for the idea: https://github.com/qsniyg/maxurl/issues/14#issuecomment-531549043
		//mouseover_close_on_leave_el: true,
		mouseover_close_el_policy: "both",
		// thanks to hans dokha on greasyfork for the idea: https://greasyfork.org/en/forum/discussion/71894/this-script-is-a-dream-come-true-just-1-thing
		mouseover_close_click_outside: false,
		// thanks to decembre on github for the idea: https://github.com/qsniyg/maxurl/issues/126
		mouseover_allow_partial: is_extension ? "media" : "video",
		mouseover_use_blob_over_data: false,
		mouseover_enable_notallowed: true,
		mouseover_notallowed_duration: 300,
		//mouseover_use_fully_loaded_image: is_extension ? false : true,
		//mouseover_use_fully_loaded_video: false,
		mouseover_minimum_size: 20,
		mouseover_exclude_backgroundimages: false,
		// thanks to decembre on github for the idea: https://github.com/qsniyg/maxurl/issues/14#issuecomment-530760246
		mouseover_exclude_page_bg: true,
		// thanks to Jin on discord for the idea
		mouseover_only_links: false,
		mouseover_exclude_sameimage: false,
		mouseover_exclude_imagetab: true,
		mouseover_video_controls: false,
		mouseover_video_controls_key: ["c"],
		mouseover_video_loop: true,
		mouseover_video_playpause_key: ["space"],
		mouseover_video_muted: false,
		mouseover_video_mute_key: ["m"],
		mouseover_video_volume: 100,
		mouseover_video_resume_from_source: false,
		mouseover_video_seek_amount: 10,
		mouseover_video_seek_left_key: ["shift", "left"],
		mouseover_video_seek_right_key: ["shift", "right"],
		mouseover_video_seek_vertical_scroll: false,
		mouseover_video_seek_horizontal_scroll: false,
		mouseover_video_speed_down_key: ["["],
		mouseover_video_speed_up_key: ["]"],
		mouseover_video_speed_amount: 0.25,
		mouseover_ui: true,
		mouseover_ui_opacity: 80,
		mouseover_ui_imagesize: true,
		mouseover_ui_zoomlevel: true,
		mouseover_ui_gallerycounter: true,
		mouseover_ui_gallerymax: 50,
		// thanks to pacep94616 on github for the idea: https://github.com/qsniyg/maxurl/issues/225
		mouseover_ui_gallerybtns: true,
		mouseover_ui_closebtn: true,
		mouseover_ui_optionsbtn: is_userscript ? true : false,
		mouseover_ui_downloadbtn: false,
		mouseover_ui_rotationbtns: false,
		mouseover_ui_caption: true,
		mouseover_ui_wrap_caption: true,
		mouseover_use_remote: false,
		mouseover_zoom_behavior: "fit",
		// thanks to decembre on github for the idea: https://github.com/qsniyg/maxurl/issues/14#issuecomment-531080061
		mouseover_zoom_custom_percent: 100,
		mouseover_pan_behavior: "drag",
		mouseover_movement_inverted: true,
		mouseover_drag_min: 5,
		mouseover_scrolly_behavior: "zoom",
		mouseover_scrollx_behavior: "gallery",
		// thanks to regis on discord for the idea
		scroll_override_page: false,
		// thanks to regis on discord for the idea
		scroll_zoom_origin: "cursor",
		scroll_zoom_behavior: "fitfull",
		// thanks to regis on discord for the idea
		scroll_incremental_mult: 1.25,
		mouseover_move_with_cursor: false,
		// thanks to regis on discord for the idea
		mouseover_move_within_page: true,
		zoom_out_to_close: false,
		// thanks to 07416 on github for the idea: https://github.com/qsniyg/maxurl/issues/20#issuecomment-439599984
		mouseover_position: "cursor",
		// thanks to decembre on github for the idea: https://github.com/qsniyg/maxurl/issues/14#issuecomment-531549043
		mouseover_prevent_cursor_overlap: true,
		mouseover_add_link: true,
		mouseover_add_video_link: false,
		mouseover_download: false,
		mouseover_hide_cursor: false,
		mouseover_hide_cursor_after: 0,
		// also thanks to 07416: https://github.com/qsniyg/maxurl/issues/25
		mouseover_links: false,
		// thanks to LoneFenris: https://github.com/qsniyg/maxurl/issues/25#issuecomment-482880122
		mouseover_only_valid_links: true,
		mouseover_allow_canvas_el: false,
		mouseover_allow_svg_el: true,
		mouseover_enable_gallery: true,
		mouseover_gallery_cycle: false,
		mouseover_gallery_prev_key: ["left"],
		mouseover_gallery_next_key: ["right"],
		mouseover_gallery_move_after_video: false,
		// thanks to acid-crash on github for the idea: https://github.com/qsniyg/maxurl/issues/20
		mouseover_styles: "",
		mouseover_fade_time: 100,
		mouseover_enable_mask_styles: false,
		mouseover_mask_styles2: "background-color: rgba(0, 0, 0, 0.5)",
		mouseover_mask_fade_time: 100,
		mouseover_ui_styles: "",
		// thanks to decembre on github for the idea: https://github.com/qsniyg/maxurl/issues/14#issuecomment-541065461
		mouseover_wait_use_el: false,
		mouseover_add_to_history: false,
		mouseover_download_key: [["s"], ["ctrl", "s"]],
		mouseover_open_new_tab_key: ["o"],
		mouseover_open_bg_tab_key: ["shift", "o"],
		mouseover_open_options_key: ["p"],
		mouseover_rotate_left_key: ["e"],
		mouseover_rotate_right_key: ["r"],
		mouseover_flip_horizontal_key: ["h"],
		mouseover_flip_vertical_key: ["v"],
		mouseover_zoom_in_key: [["+"], ["="], ["shift", "="]],
		mouseover_zoom_out_key: [["-"]],
		mouseover_zoom_full_key: ["1"],
		mouseover_zoom_fit_key: ["2"],
		mouseover_apply_blacklist: true,
		apply_blacklist_host: false,
		mouseover_matching_media_types: false,
		mouseover_support_pointerevents_none: true,
		website_inject_imu: true,
		website_image: true,
		extension_contextmenu: true,
		allow_video: true,
		allow_dash_video: false,
		allow_hls_video: false,
		allow_watermark: false,
		allow_smaller: false,
		allow_possibly_different: false,
		allow_possibly_broken: false,
		allow_possibly_upscaled: false,
		allow_thirdparty: false,
		allow_apicalls: true,
		allow_thirdparty_libs: is_userscript ? false : true,
		//browser_cookies: true,
		deviantart_prefer_size: false,
		imgur_source: true,
		instagram_use_app_api: true,
		instagram_gallery_postlink: false,
		tiktok_no_watermarks: true,
		// just a very small protection against github scraping bots :)
		tumblr_api_key: base64_decode("IHhyTXBMTThuMWVDZUwzb1JZU1pHN0NMQUx3NkVIaFlEZFU2V3E1ZUQxUGJNa2xkN1kx").substr(1),
		// thanks to LukasThyWalls on github for the idea: https://github.com/qsniyg/maxurl/issues/75
		bigimage_blacklist: "",
		bigimage_blacklist_engine: "glob",
		replaceimgs_enable_keybinding: false,
		replaceimgs_keybinding: ["shift", "alt", "r"],
		replaceimgs_auto: false,
		replaceimgs_replaceimgs: true,
		replaceimgs_addlinks: false,
		replaceimgs_replacelinks: false,
		replaceimgs_usedata: is_userscript ? true : false,
		replaceimgs_wait_fullyloaded: true,
		replaceimgs_totallimit: 8,
		replaceimgs_domainlimit: 2,
		highlightimgs_enable_keybinding: false,
		highlightimgs_keybinding: ["shift", "alt", "h"],
		highlightimgs_enable: false,
		highlightimgs_auto: "never",
		highlightimgs_onlysupported: true,
		highlightimgs_css: "outline: 4px solid yellow",

		// cache entries (not settings, but this is the most convenient way to do it)
		last_update_check: 0,
		last_update_version: null,
		last_update_url: null
	};
	var orig_settings = deepcopy(settings);

	var user_defined_settings = {};

	var settings_meta = {
		imu_enabled: {
			name: "Enable extension",
			description: "Globally enables or disables the extension",
			category: "general",
			// Userscript users can easily disable it from the userscript menu,
			//   and enabling it again isn't as trivial as it is for the extension
			extension_only: true,
			imu_enabled_exempt: true
		},
		language: {
			name: "Language",
			description: "Language for this extension",
			category: "general",
			options: {
				_type: "combo",
				en: {
					name: "English"
				},
				ko: {
					name: "한국어"
				},
				fr: {
					name: "Français"
				}
			},
			onedit: function() {
				run_soon(do_options);
			},
			imu_enabled_exempt: true
		},
		dark_mode: {
			name: "Dark mode",
			description: "Changes the colors to have light text on a dark background",
			category: "general",
			onedit: update_dark_mode,
			onupdate: update_dark_mode,
			imu_enabled_exempt: true
		},
		settings_visible_description: {
			name: "Description below options",
			description: "Shows the description below the options (otherwise the description is only shown when you hover over the option's name)",
			category: "general",
			subcategory: "settings",
			onedit: function() {
				run_soon(do_options);
			},
			imu_enabled_exempt: true
		},
		settings_show_disabled: {
			name: "Show disabled options",
			description: "If disabled, options that are disabled due to their requirements being unmet will not be displayed",
			category: "general",
			subcategory: "settings",
			onedit: function() {
				run_soon(do_options);
			},
			imu_enabled_exempt: true
		},
		settings_show_requirements: {
			name: "Requirements below disabled options",
			description: "If an option is disabled, the requirements to enable the option will be displayed below it",
			category: "general",
			subcategory: "settings",
			onedit: function() {
				run_soon(do_options);
			},
			requires: {
				settings_show_disabled: true
			},
			imu_enabled_exempt: true
		},
		check_updates: {
			name: "Check for updates",
			description: "Periodically checks for updates. If a new update is available, it will be shown at the top of the options page",
			category: "general",
		},
		check_update_interval: {
			name: "Update check interval",
			description: "How often to check for updates",
			category: "general",
			requires: {
				check_updates: true
			},
			type: "number",
			number_min: 1,
			number_int: true,
			number_unit: "hours",
		},
		advanced_options: {
			name: "Show advanced settings",
			description: "If disabled, settings that might be harder to understand will be hidden",
			category: "general",
			subcategory: "settings",
			onedit: function() {
				run_soon(do_options);
			},
			imu_enabled_exempt: true
		},
		settings_tabs: {
			name: "Use tabs",
			description: "If disabled, all settings will be shown on a single page",
			category: "general",
			subcategory: "settings",
			onedit: function() {
				run_soon(do_options);
			},
			imu_enabled_exempt: true
		},
		allow_browser_request: {
			name: "Allow using browser XHR",
			description: "This allows XHR requests to be run in the browser's context if they fail in the extension (e.g. when Tracking Protection is set to High)",
			category: "general",
			imu_enabled_exempt: true,
			advanced: true
		},
		use_blob_over_arraybuffer: {
			name: "Use `Blob` over `ArrayBuffer`",
			description: "Uses `Blob`s for XHRs instead of `ArrayBuffer`s. Keep this enabled unless your userscript manager doesn't support blob requests",
			category: "general",
			imu_enabled_exempt: true,
			advanced: true,
			hidden: true
		},
		allow_live_settings_reload: {
			name: "Live settings reloading",
			description: "Enables/disables live settings reloading. There shouldn't be a reason to disable this unless you're experiencing issues with this feature",
			category: "general",
			hidden: is_userscript && typeof GM_addValueChangeListener === "undefined",
			imu_enabled_exempt: true,
			advanced: true
		},
		disable_keybind_when_editing: {
			name: "Disable keybindings when editing text",
			description: "Disables IMU keybindings when key events are sent to an input area on the page",
			category: "general",
			imu_enabled_exempt: true,
			advanced: true
		},
		redirect: {
			name: "Enable redirection",
			description: "Redirect images opened in their own tab",
			category: "redirection"
		},
		redirect_history: {
			name: "Add to history",
			description: "Redirection will add a new entry to the browser's history",
			requires: {
				redirect: true
			},
			category: "redirection"
		},
		canhead_get: {
			name: "Use GET if HEAD is unsupported",
			description: "Use a GET request to check an image's availability, if the server does not support HEAD requests",
			requires: {
				redirect: true
			},
			category: "redirection",
			advanced: true
		},
		redirect_force_page: {
			name: "Try finding original page/caption",
			description: "Enables methods that use API calls for finding the original page or caption",
			example_websites: [
				"Flickr",
				"SmugMug",
				"..."
			],
			category: "rules"
		},
		print_imu_obj: {
			name: "Log IMU object to console",
			description: "Prints the full IMU object to the console whenever a popup/redirect is found",
			category: "rules",
			advanced: true
		},
		redirect_disable_for_responseheader: {
			name: "Disable when response headers need modifying",
			description: "This option works around Chrome's migration to manifest v3, redirecting some images to being force-downloaded",
			extension_only: true,
			hidden: true, // Doesn't seem to be needed?
			category: "redirection",
			advanced: true
		},
		mouseover: {
			name: "Enable mouseover popup",
			description: "Show a popup with the larger image when you mouseover an image with the trigger key held",
			category: "popup"
		},
		mouseover_open_behavior: {
			name: "Mouseover popup action",
			description: "Determines how the mouseover popup will open",
			// While it won't work for some images without the extension, let's not disable it outright either
			//extension_only: true,
			hidden: is_userscript && open_in_tab === nullfunc,
			options: {
				_type: "or",
				popup: {
					name: "Popup"
				},
				newtab: {
					name: "New tab"
				}
			},
			requires: {
				mouseover: true
			},
			category: "popup",
			subcategory: "open_behavior"
		},
		mouseover_trigger: {
			name: "Popup trigger",
			description: "Trigger key that, when held, will show the popup",
			options: {
				_type: "and",
				_group1: {
					_type: "and",
					ctrl: {
						name: "Ctrl"
					},
					shift: {
						name: "Shift"
					},
					alt: {
						name: "Alt"
					}
				},
				_group2: {
					_type: "or",
					delay_1: {
						name: "Delay 1s"
					},
					delay_3: {
						name: "Delay 3s"
					}
				}
			},
			requires: {
				mouseover: true
			},
			category: "popup",
			subcategory: "trigger"
		},
		mouseover_trigger_behavior: {
			name: "Mouseover popup trigger",
			description: "How the popup will get triggered",
			options: {
				mouse: {
					name: "Mouseover",
					description: "Triggers when your mouse is over the image"
				},
				keyboard: {
					name: "Key trigger",
					description: "Triggers when you press a key sequence when your mouse is over an image"
				},
				none: {
					name: "None",
					description: "Disables the popup from being triggered (useful if you only want to use the context menu item)",
					extension_only: true
				}
			},
			requires: {
				mouseover: true
			},
			category: "popup",
			subcategory: "trigger"
		},
		mouseover_trigger_key: {
			name: "Popup trigger key",
			description: "Key sequence to trigger the popup",
			type: "keysequence",
			requires: {
				mouseover: true,
				mouseover_trigger_behavior: "keyboard"
			},
			category: "popup",
			subcategory: "trigger"
		},
		mouseover_trigger_delay: {
			name: "Popup trigger delay",
			description: "Delay (in seconds) before the popup shows",
			requires: {
				mouseover: true,
				mouseover_trigger_behavior: "mouse"
			},
			type: "number",
			number_min: 0,
			number_unit: "seconds",
			category: "popup",
			subcategory: "trigger"
		},
		mouseover_trigger_mouseover: {
			name: "Use mouseover event",
			description: "Uses the mouseover event instead of mousemove to figure out where to trigger the popup. This more closely matches the way other image popup addons work, at the cost of configurability",
			requires: {
				mouseover_trigger_behavior: "mouse"
			},
			advanced: true,
			category: "popup",
			subcategory: "trigger"
		},
		mouseover_trigger_prevent_key: {
			name: "Popup prevention key",
			description: "Holding down this key will prevent the popup from being opened",
			requires: {
				mouseover: true,
				mouseover_trigger_behavior: "mouse"
			},
			type: "keysequence",
			category: "popup",
			subcategory: "trigger"
		},
		mouseover_allow_partial: {
			name: "Allow showing partially loaded",
			description: "This will allow the popup to open for partially loaded media",
			description_userscript: "This will allow the popup to open for partially loaded media, but this might break some images",
			requires: {
				mouseover: true,
				mouseover_open_behavior: "popup"
			},
			options: {
				_type: "or",
				video: {
					name: "Video"
				},
				media: {
					name: "Media",
					description: "Both images and video"
				},
				none: {
					name: "None"
				}
			},
			category: "popup",
			subcategory: "open_behavior"
		},
		mouseover_use_blob_over_data: {
			name: "Use `blob:` over `data:` URLs",
			description: "Blob URLs are more efficient, but aren't supported by earlier browsers. Some websites also block `blob:` URLs",
			requires: {
				mouseover_open_behavior: "popup"
			},
			category: "popup",
			subcategory: "open_behavior",
			advanced: true
		},
		mouseover_use_fully_loaded_image: {
			name: "Wait until image is fully loaded",
			description: "Wait until the image has fully loaded before displaying it",
			requires: {
				mouseover: true,
				mouseover_open_behavior: "popup"
			},
			category: "popup",
			subcategory: "open_behavior"
		},
		mouseover_use_fully_loaded_video: {
			name: "Wait until video is fully loaded",
			description: "Wait until the video has fully loaded before displaying it (this may significantly increase memory usage with larger videos)",
			requires: {
				mouseover: true,
				mouseover_open_behavior: "popup"
			},
			category: "popup",
			subcategory: "open_behavior"
		},
		mouseover_enable_notallowed: {
			name: "Enable `not-allowed` cursor",
			description: "If the image isn't supported, the mouse cursor will change to a `not-allowed` cursor for a brief duration",
			requires: {
				mouseover_trigger_behavior: "keyboard"
			},
			category: "popup",
			subcategory: "open_behavior"
		},
		mouseover_notallowed_duration: {
			name: "`not-allowed` cursor duration",
			description: "How long the `not-allowed` cursor should last",
			requires: {
				mouseover_enable_notallowed: true
			},
			type: "number",
			number_min: 0,
			number_int: true,
			number_unit: "ms",
			advanced: true,
			category: "popup",
			subcategory: "open_behavior"
		},
		mouseover_exclude_page_bg: {
			name: "Exclude page background",
			description: "Excludes the page background for the popup",
			requires: {
				mouseover: true,
				mouseover_exclude_backgroundimages: false
			},
			category: "popup",
			subcategory: "open_behavior"
		},
		mouseover_minimum_size: {
			name: "Minimum image size",
			description: "Smallest size acceptable for the popup to open (this option is ignored for background images)",
			requires: {
				mouseover: true
			},
			type: "number",
			number_min: 0,
			number_unit: "pixels",
			category: "popup",
			subcategory: "open_behavior"
		},
		mouseover_exclude_backgroundimages: {
			name: "Exclude `background-image`s",
			description: "Excludes `background-image`s for the popup. Might prevent the popup from working on many images",
			requires: {
				mouseover: true
			},
			disabled_if: {
				mouseover_trigger_mouseover: true
			},
			category: "popup",
			subcategory: "open_behavior"
		},
		mouseover_exclude_imagetab: {
			name: "Exclude image tabs",
			description: "Excludes images that are opened in their own tabs",
			requires: {
				mouseover: true,
				mouseover_trigger_behavior: "mouse"
			},
			category: "popup",
			subcategory: "open_behavior"
		},
		mouseover_exclude_sameimage: {
			name: "Exclude if image URL is unchanged",
			description: "Don't pop up if the new image is the same as the thumbnail image",
			requires: {
				mouseover: true
			},
			category: "popup",
			subcategory: "open_behavior"
		},
		mouseover_only_links: {
			name: "Only popup for linked images",
			description: "Don't pop up if the image isn't hyperlinked",
			requires: {
				mouseover: true
			},
			category: "popup",
			subcategory: "open_behavior"
		},
		mouseover_video_controls: {
			name: "Show video controls",
			description: "Shows native video controls. Note that this prevents dragging under Firefox",
			requires: {
				mouseover_open_behavior: "popup",
				allow_video: true
			},
			category: "popup",
			subcategory: "video"
		},
		mouseover_video_controls_key: {
			name: "Toggle video controls",
			description: "Key to toggle whether the video controls are shown",
			requires: {
				mouseover_open_behavior: "popup",
				allow_video: true
			},
			type: "keysequence",
			category: "popup",
			subcategory: "video"
		},
		mouseover_video_loop: {
			name: "Loop video",
			description: "Allows the video to automatically restart to the beginning after finishing playing",
			requires: {
				mouseover_open_behavior: "popup",
				allow_video: true
			},
			disabled_if: {
				mouseover_gallery_move_after_video: true
			},
			category: "popup",
			subcategory: "video"
		},
		mouseover_video_playpause_key: {
			name: "Play/pause key",
			description: "Key to toggle whether the video is playing or paused",
			requires: {
				mouseover_open_behavior: "popup",
				allow_video: true
			},
			type: "keysequence",
			category: "popup",
			subcategory: "video"
		},
		mouseover_video_muted: {
			name: "Mute video",
			description: "Mutes the video by default",
			requires: {
				mouseover_open_behavior: "popup",
				allow_video: true
			},
			category: "popup",
			subcategory: "video"
		},
		mouseover_video_mute_key: {
			name: "Toggle mute key",
			description: "Key to toggle whether the video is muted or unmuted",
			requires: {
				mouseover_open_behavior: "popup",
				allow_video: true
			},
			type: "keysequence",
			category: "popup",
			subcategory: "video"
		},
		mouseover_video_volume: {
			name: "Default volume",
			description: "Default volume for the video",
			requires: {
				mouseover_video_muted: false
			},
			type: "number",
			number_min: 0,
			number_max: 100,
			number_int: true,
			number_unit: "%",
			category: "popup",
			subcategory: "video"
		},
		mouseover_video_resume_from_source: {
			name: "Resume playback from source",
			description: "If enabled, playback will resume from where the source video left off",
			requires: {
				mouseover_open_behavior: "popup",
				allow_video: true
			},
			category: "popup",
			subcategory: "video"
		},
		mouseover_video_seek_amount: {
			name: "Seek amount",
			description: "Amount of time to seek forward/back when using the seek keys",
			requires: {
				mouseover_open_behavior: "popup",
				allow_video: true
			},
			type: "number",
			number_min: 0,
			number_unit: "seconds",
			category: "popup",
			subcategory: "video"
		},
		mouseover_video_seek_left_key: {
			name: "Seek left key",
			description: "Key to seek backwards in a video by the specified amount",
			requires: {
				mouseover_open_behavior: "popup",
				allow_video: true
			},
			type: "keysequence",
			category: "popup",
			subcategory: "video"
		},
		mouseover_video_seek_right_key: {
			name: "Seek right key",
			description: "Key to seek forwards in a video by the specified amount",
			requires: {
				mouseover_open_behavior: "popup",
				allow_video: true
			},
			type: "keysequence",
			category: "popup",
			subcategory: "video"
		},
		mouseover_video_seek_vertical_scroll: {
			name: "Vertical scroll seeks",
			description: "Scrolling vertically will seek the video forward/backward",
			requires: {
				mouseover_open_behavior: "popup",
				allow_video: true
			},
			category: "popup",
			subcategory: "video"
		},
		mouseover_video_seek_horizontal_scroll: {
			name: "Horizontal scroll seeks",
			description: "Scrolling horizontally will seek the video forward/backward",
			requires: {
				mouseover_open_behavior: "popup",
				allow_video: true
			},
			category: "popup",
			subcategory: "video"
		},
		mouseover_video_speed_down_key: {
			name: "Speed down key",
			description: "Key to speed the video down by a specified amount",
			requires: {
				mouseover_open_behavior: "popup",
				allow_video: true
			},
			type: "keysequence",
			category: "popup",
			subcategory: "video"
		},
		mouseover_video_speed_up_key: {
			name: "Speed up key",
			description: "Key to speed the video up by a specified amount",
			requires: {
				mouseover_open_behavior: "popup",
				allow_video: true
			},
			type: "keysequence",
			category: "popup",
			subcategory: "video"
		},
		mouseover_video_speed_amount: {
			name: "Speed up/down amount",
			description: "How many times faster/slower to speed the video up/down",
			requires: {
				mouseover_open_behavior: "popup",
				allow_video: true
			},
			type: "number",
			number_min: 0,
			number_unit: "x",
			category: "popup",
			subcategory: "video"
		},
		mouseover_ui: {
			name: "Popup UI",
			description: "Enables a UI on top of the popup",
			requires: {
				mouseover_open_behavior: "popup"
			},
			category: "popup",
			subcategory: "ui"
		},
		mouseover_ui_opacity: {
			name: "Opacity",
			description: "Opacity of the UI on top of the popup",
			requires: {
				mouseover_ui: true
			},
			type: "number",
			number_unit: "%",
			number_max: 100,
			number_min: 0,
			number_int: true,
			category: "popup",
			subcategory: "ui"
		},
		mouseover_ui_imagesize: {
			name: "Image size",
			description: "Displays the image size on top of the UI",
			requires: {
				mouseover_ui: true
			},
			category: "popup",
			subcategory: "ui"
		},
		mouseover_ui_zoomlevel: {
			name: "Zoom percent",
			description: "Displays the current zoom level on top of the UI",
			requires: {
				mouseover_ui: true
			},
			category: "popup",
			subcategory: "ui"
		},
		mouseover_ui_gallerycounter: {
			name: "Gallery counter",
			description: "Enables a gallery counter on top of the UI",
			requires: {
				mouseover_ui: true,
				mouseover_enable_gallery: true
			},
			category: "popup",
			subcategory: "ui"
		},
		mouseover_ui_gallerymax: {
			name: "Gallery counter max",
			description: "Maximum amount of images to check in the counter (this can be slightly CPU-intensive)",
			requires: {
				mouseover_ui_gallerycounter: true,
				mouseover_enable_gallery: true
			},
			type: "number",
			number_min: 0,
			number_unit: "images",
			category: "popup",
			subcategory: "ui"
		},
		mouseover_ui_gallerybtns: {
			name: "Gallery buttons",
			description: "Enables buttons to go left/right in the gallery",
			requires: {
				mouseover_ui: true,
				mouseover_enable_gallery: true
			},
			category: "popup",
			subcategory: "ui"
		},
		mouseover_ui_closebtn: {
			name: "Close Button",
			description: "Enables a button to close the popup",
			requires: {
				mouseover_ui: true
			},
			category: "popup",
			subcategory: "ui"
		},
		mouseover_ui_optionsbtn: {
			name: "Options Button",
			description: "Enables a button to go to this page",
			requires: {
				mouseover_ui: true
			},
			// While it works for the extension, it's more or less useless
			userscript_only: true,
			category: "popup",
			subcategory: "ui"
		},
		mouseover_ui_downloadbtn: {
			name: "Download Button",
			description: "Enables a button to download the image",
			requires: {
				mouseover_ui: true
			},
			category: "popup",
			subcategory: "ui"
		},
		mouseover_ui_rotationbtns: {
			name: "Rotation Buttons",
			description: "Enables buttons on the UI to rotate the image by 90 degrees",
			requires: {
				mouseover_ui: true
			},
			category: "popup",
			subcategory: "ui"
		},
		mouseover_ui_caption: {
			name: "Caption",
			description: "Shows the image's caption (if available) at the top",
			requires: {
				mouseover_ui: true
			},
			category: "popup",
			subcategory: "ui"
		},
		mouseover_ui_wrap_caption: {
			name: "Wrap caption text",
			description: "Wraps the caption if it's too long",
			requires: {
				mouseover_ui_caption: true
			},
			category: "popup",
			subcategory: "ui"
		},
		mouseover_close_behavior: {
			name: "Keep popup open until",
			description: "Closes the popup when the selected condition is met",
			options: {
				_type: "or",
				_group1: {
					any: {
						name: "Any trigger is released"
					}
				},
				_group2: {
					all: {
						name: "All triggers are released"
					}
				},
				_group3: {
					esc: {
						name: "ESC/Close is pressed"
					}
				}
			},
			requires: {
				mouseover_open_behavior: "popup",
				mouseover_trigger_behavior: "keyboard"
			},
			category: "popup",
			subcategory: "close_behavior"
		},
		mouseover_close_need_mouseout: {
			name: "Don't close until mouse leaves",
			description: "If true, this keeps the popup open even if all triggers are released if the mouse is still over the image",
			requires: [
				{mouseover_close_behavior: "any"},
				{mouseover_close_behavior: "all"}
			],
			category: "popup",
			subcategory: "close_behavior"
		},
		mouseover_jitter_threshold: {
			name: "Threshold to leave image",
			description: "How many pixels outside of the image before the cursor is considered to have left the image",
			requires: [
				{
					mouseover_open_behavior: "popup",
					mouseover_close_need_mouseout: true
				},
				{
					mouseover_open_behavior: "popup",
					mouseover_trigger_behavior: "mouse"
				}
			],
			type: "number",
			number_unit: "pixels",
			category: "popup",
			subcategory: "close_behavior"
		},
		mouseover_cancel_popup_when_elout: {
			name: "Leaving thumbnail cancels loading",
			description: "Cancels the current popup loading when the cursor has left the thumbnail image",
			requires: {
				mouseover: true,
				mouseover_trigger_behavior: "mouse"
			},
			category: "popup",
			subcategory: "close_behavior"
		},
		mouseover_cancel_popup_with_esc: {
			name: "ESC cancels loading",
			description: "Cancels the current popup loading if ESC is pressed",
			requires: {
				mouseover: true
			},
			category: "popup",
			subcategory: "close_behavior"
		},
		mouseover_auto_close_popup: {
			name: "Automatically close after timeout",
			description: "Closes the popup automatically after a specified period of time has elapsed",
			requires: {
				mouseover_open_behavior: "popup"
			},
			category: "popup",
			subcategory: "close_behavior"
		},
		mouseover_auto_close_popup_time: {
			name: "Timeout to close popup",
			description: "Amount of time to elapse before automatically closing the popup",
			requires: {
				mouseover_auto_close_popup: true
			},
			type: "number",
			number_min: 0,
			number_unit: "seconds",
			category: "popup",
			subcategory: "close_behavior"
		},
		mouseover_use_hold_key: {
			name: "Use hold key",
			description: "Enables the use of a hold key that, when pressed, will keep the popup open",
			requires: [
				{
					mouseover_trigger_behavior: "mouse",
					mouseover_open_behavior: "popup"
				},
				{
					mouseover_auto_close_popup: true,
					mouseover_open_behavior: "popup"
				},
			],
			category: "popup",
			subcategory: "close_behavior"
		},
		mouseover_hold_key: {
			name: "Hold key",
			description: "Hold key that, when pressed, will keep the popup open",
			requires: {
				mouseover_use_hold_key: true
			},
			type: "keysequence",
			category: "popup",
			subcategory: "close_behavior"
		},
		mouseover_hold_position_center: {
			name: "Center popup on hold",
			description: "Centers the popup to the middle of the page when the popup is held",
			requires: {
				mouseover_use_hold_key: true
			},
			category: "popup",
			subcategory: "close_behavior"
		},
		mouseover_close_on_leave_el: {
			name: "Close when leaving thumbnail",
			description: "Closes the popup when the mouse leaves the thumbnail element (won't close if the mouse instead moves to the popup)",
			requires: {
				mouseover_open_behavior: "popup",
				mouseover_trigger_behavior: "mouse",
				mouseover_position: "beside_cursor"
			},
			category: "popup",
			subcategory: "close_behavior"
		},
		mouseover_close_click_outside: {
			name: "Clicking outside the popup closes",
			description: "Closes the popup when the mouse clicks outside of it",
			requires: {
				mouseover_open_behavior: "popup"
			},
			category: "popup",
			subcategory: "close_behavior"
		},
		mouseover_close_el_policy: {
			name: "Close when leaving",
			description: "Closes the popup when the mouse leaves the thumbnail element, the popup, or either",
			requires: {
				mouseover_open_behavior: "popup",
				mouseover_trigger_behavior: "mouse"
			},
			options: {
				_type: "or",
				thumbnail: {
					name: "Thumbnail"
				},
				popup: {
					name: "Popup"
				},
				both: {
					name: "Both"
				}
			},
			category: "popup",
			subcategory: "close_behavior"
		},
		mouseover_wait_use_el: {
			name: "Use invisible element when waiting",
			description: "Creates an invisible element under the cursor when waiting for the popup instead of a style element (can improve performance on websites with many elements, but prevents the cursor from clicking anything while loading the popup)",
			requires: {
				mouseover: true
			},
			category: "popup",
			subcategory: "popup_other",
			advanced: true
		},
		mouseover_add_to_history: {
			name: "Add popup link to history",
			description: "Adds the image/video link opened through the popup to the browser's history",
			requires: {
				mouseover: true
			},
			category: "popup",
			subcategory: "popup_other",
			required_permission: "history",
			extension_only: true
		},
		allow_remote: {
			name: "Allow inter-frame communication",
			description: "Allows communication between frames in windows, improving support for keybindings",
			description_userscript: "Allows communication between frames in windows, improving support for keybindings. Can pose a fingerprinting risk when used through the userscript",
			requires: {
				mouseover: true
			},
			category: "general"
		},
		mouseover_use_remote: {
			name: "Pop out of frames",
			description: "Opens the popup on the top frame instead of within iframes. Still in beta",
			requires: {
				mouseover_open_behavior: "popup",
				allow_remote: true
			},
			category: "popup",
			subcategory: "open_behavior"
		},
		mouseover_zoom_behavior: {
			name: "Popup default zoom",
			description: "How the popup should be initially sized",
			options: {
				_type: "or",
				_group1: {
					fit: {
						name: "Fit to screen"
					}
				},
				_group2: {
					full: {
						name: "Full size"
					}
				},
				_group3: {
					custom: {
						name: "Custom size"
					}
				}
			},
			requires: {
				mouseover_open_behavior: "popup"
			},
			category: "popup",
			subcategory: "open_behavior"
		},
		mouseover_zoom_custom_percent: {
			name: "Custom zoom percent",
			description: "Custom percent to initially size the popup",
			type: "number",
			number_min: 0,
			number_unit: "%",
			requires: {
				mouseover_zoom_behavior: "custom"
			},
			category: "popup",
			subcategory: "open_behavior"
		},
		mouseover_pan_behavior: {
			name: "Popup panning method",
			description: "How the popup should be panned when larger than the screen",
			options: {
				_type: "or",
				movement: {
					name: "Movement",
					description: "The popup pans as you move your mouse"
				},
				drag: {
					name: "Drag",
					description: "Clicking and dragging pans the popup"
				}
			},
			requires: {
				mouseover_open_behavior: "popup"
			},
			category: "popup",
			subcategory: "behavior"
		},
		mouseover_movement_inverted: {
			name: "Invert movement",
			description: "Inverts the movement of the mouse. For example, if the mouse moves left, the popup moves right. If disabled, it feels more like the popup is being invisibly dragged.",
			requires: {
				mouseover_pan_behavior: "movement"
			},
			// It's doubtful many users will want this option enabled
			advanced: true,
			category: "popup",
			subcategory: "behavior"
		},
		mouseover_drag_min: {
			name: "Minimum drag amount",
			description: "How many pixels the mouse should move to start a drag",
			type: "number",
			number_min: 0,
			number_int: true,
			number_unit: "pixels",
			requires: {
				mouseover_pan_behavior: "drag"
			},
			category: "popup",
			subcategory: "behavior"
		},
		mouseover_scrolly_behavior: {
			name: "Vertical scroll action",
			description: "How the popup reacts to a vertical scroll/mouse wheel event",
			options: {
				_type: "or",
				_group1: {
					zoom: {
						name: "Zoom"
					},
					pan: {
						name: "Pan"
					},
					gallery: {
						name: "Gallery",
						requires: [{
							mouseover_enable_gallery: true
						}]
					}
				},
				_group2: {
					nothing: {
						name: "None"
					}
				}
			},
			requires: {
				mouseover_open_behavior: "popup"
			},
			category: "popup",
			subcategory: "behavior"
		},
		mouseover_scrollx_behavior: {
			name: "Horizontal scroll action",
			description: "How the popup reacts to a horizontal scroll/mouse wheel event",
			options: {
				_type: "or",
				_group1: {
					pan: {
						name: "Pan"
					},
					gallery: {
						name: "Gallery",
						requires: [{
							mouseover_enable_gallery: true
						}]
					},
					nothing: {
						name: "None"
					}
				}
			},
			requires: {
				mouseover_open_behavior: "popup"
			},
			category: "popup",
			subcategory: "behavior"
		},
		scroll_override_page: {
			name: "Override scroll outside of popup",
			description: "Scroll events performed outside of the popup are still acted on",
			requires: {
				mouseover_open_behavior: "popup"
			},
			category: "popup",
			subcategory: "behavior"
		},
		scroll_zoom_origin: {
			name: "Zoom origin",
			description: "The point on the image it's zoomed in/out from",
			options: {
				_type: "or",
				cursor: {
					name: "Cursor"
				},
				center: {
					name: "Center"
				}
			},
			requires: [
				{mouseover_scrollx_behavior: "zoom"},
				{mouseover_scrolly_behavior: "zoom"}
			],
			category: "popup",
			subcategory: "behavior"
		},
		scroll_zoom_behavior: {
			name: "Zoom behavior",
			description: "How zooming should work",
			options: {
				_type: "or",
				fitfull: {
					name: "Fit/Full",
					description: "Toggles between the full size, and fit-to-screen"
				},
				incremental: {
					name: "Incremental"
				}
			},
			requires: [
				{mouseover_scrollx_behavior: "zoom"},
				{mouseover_scrolly_behavior: "zoom"}
			],
			category: "popup",
			subcategory: "behavior"
		},
		scroll_incremental_mult: {
			name: "Incremental zoom multiplier",
			description: "How much to zoom in/out by (for incremental zooming)",
			type: "number",
			number_min: 1,
			number_unit: "x",
			category: "popup",
			subcategory: "behavior"
		},
		mouseover_move_with_cursor: {
			name: "Move with cursor",
			description: "Moves the popup as the cursor moves",
			requires: {
				mouseover_open_behavior: "popup"
			},
			category: "popup",
			subcategory: "behavior"
		},
		mouseover_move_within_page: {
			name: "Move within page",
			description: "Ensures the popup doesn't leave the page",
			requires: {
				mouseover_move_with_cursor: true
			},
			category: "popup",
			subcategory: "behavior"
		},
		zoom_out_to_close: {
			name: "Zoom out fully to close",
			description: "Closes the popup if you zoom out past the minimum zoom",
			requires: [
				{mouseover_scrollx_behavior: "zoom"},
				{mouseover_scrolly_behavior: "zoom"}
			],
			category: "popup",
			subcategory: "close_behavior"
		},
		mouseover_position: {
			name: "Popup position",
			description: "Where the popup will appear",
			options: {
				_type: "or",
				_group1: {
					cursor: {
						name: "Cursor middle",
						description: "Underneath the mouse cursor"
					}
				},
				_group2: {
					beside_cursor: {
						name: "Beside cursor"
					}
				},
				_group3: {
					center: {
						name: "Page middle"
					}
				}
			},
			requires: {
				mouseover_open_behavior: "popup"
			},
			category: "popup",
			subcategory: "open_behavior"
		},
		mouseover_prevent_cursor_overlap: {
			name: "Prevent cursor overlap",
			description: "Prevents the image from overlapping with the cursor",
			requires: {
				mouseover_position: "beside_cursor"
			},
			hidden: true, // no longer applicable with new beside_cursor implementation
			category: "popup",
			subcategory: "open_behavior"
		},
		mouseover_hide_cursor: {
			name: "Hide cursor over popup",
			description: "Hides the cursor when the mouse is over the popup",
			requires: {
				mouseover_open_behavior: "popup"
			},
			category: "popup",
			subcategory: "behavior"
		},
		mouseover_hide_cursor_after: {
			name: "Hide cursor after",
			description: "Hides the cursor over the popup after a specified period of time (in milliseconds), 0 always hides the cursor",
			requires: {
				mouseover_hide_cursor: true
			},
			type: "number",
			number_unit: "ms",
			number_int: true,
			number_min: 0,
			category: "popup",
			subcategory: "behavior"
		},
		mouseover_add_link: {
			name: "Link image",
			description: "Adds a link to the image in the popup",
			requires: {
				mouseover_open_behavior: "popup"
			},
			category: "popup",
			subcategory: "behavior"
		},
		mouseover_add_video_link: {
			name: "Link video",
			description: "Adds a link to the video in the popup",
			requires: {
				mouseover_open_behavior: "popup",
				allow_video: true
			},
			category: "popup",
			subcategory: "behavior"
		},
		mouseover_download: {
			name: "Clicking link downloads",
			description: "Instead of opening the link in a new tab, it will download the image/video instead",
			requires: [
				{
					mouseover_open_behavior: "popup",
					mouseover_add_link: true
				},
				{
					mouseover_open_behavior: "popup",
					mouseover_add_video_link: true
				},
			],
			category: "popup",
			subcategory: "behavior"
		},
		mouseover_download_key: {
			name: "Download key",
			description: "Downloads the image in the popup when this key is pressed",
			requires: {
				mouseover_open_behavior: "popup"
			},
			type: "keysequence",
			category: "popup",
			subcategory: "behavior"
		},
		mouseover_open_new_tab_key: {
			name: "Open in new tab key",
			description: "Opens the image in the popup in a new tab when this key is pressed",
			hidden: is_userscript && open_in_tab === nullfunc,
			requires: {
				mouseover_open_behavior: "popup"
			},
			type: "keysequence",
			category: "popup",
			subcategory: "behavior"
		},
		mouseover_open_bg_tab_key: {
			name: "Open in background tab key",
			description: "Opens the image in the popup in a new tab without switching to it when this key is pressed",
			hidden: is_userscript && open_in_tab === nullfunc,
			requires: {
				mouseover_open_behavior: "popup"
			},
			type: "keysequence",
			category: "popup",
			subcategory: "behavior"
		},
		mouseover_open_options_key: {
			name: "Open options key",
			description: "Opens this page in a new tab when this key is pressed",
			hidden: is_userscript && open_in_tab === nullfunc,
			requires: {
				mouseover_open_behavior: "popup"
			},
			type: "keysequence",
			category: "popup",
			subcategory: "behavior"
		},
		mouseover_rotate_left_key: {
			name: "Rotate left key",
			description: "Rotates the popup 90 degrees to the left",
			requires: {
				mouseover_open_behavior: "popup"
			},
			type: "keysequence",
			category: "popup",
			subcategory: "behavior"
		},
		mouseover_rotate_right_key: {
			name: "Rotate right key",
			description: "Rotates the popup 90 degrees to the right",
			requires: {
				mouseover_open_behavior: "popup"
			},
			type: "keysequence",
			category: "popup",
			subcategory: "behavior"
		},
		mouseover_flip_horizontal_key: {
			name: "Horizontal flip key",
			description: "Flips the image horizontally",
			requires: {
				mouseover_open_behavior: "popup"
			},
			type: "keysequence",
			category: "popup",
			subcategory: "behavior"
		},
		mouseover_flip_vertical_key: {
			name: "Vertical flip key",
			description: "Flips the image vertically",
			requires: {
				mouseover_open_behavior: "popup"
			},
			type: "keysequence",
			category: "popup",
			subcategory: "behavior"
		},
		mouseover_zoom_in_key: {
			name: "Zoom in key",
			description: "Incrementally zooms into the image",
			requires: {
				mouseover_open_behavior: "popup"
			},
			type: "keysequence",
			category: "popup",
			subcategory: "behavior"
		},
		mouseover_zoom_out_key: {
			name: "Zoom out key",
			description: "Incrementally zooms out of the image",
			requires: {
				mouseover_open_behavior: "popup"
			},
			type: "keysequence",
			category: "popup",
			subcategory: "behavior"
		},
		mouseover_zoom_full_key: {
			name: "Full zoom key",
			description: "Sets the image to be at a 100% zoom, even if it overflows the screen",
			requires: {
				mouseover_open_behavior: "popup"
			},
			type: "keysequence",
			category: "popup",
			subcategory: "behavior"
		},
		mouseover_zoom_fit_key: {
			name: "Fit screen key",
			description: "Sets the image to either be at a 100% zoom, or to fit the screen, whichever is smaller",
			requires: {
				mouseover_open_behavior: "popup"
			},
			type: "keysequence",
			category: "popup",
			subcategory: "behavior"
		},
		mouseover_links: {
			name: "Popup for plain hyperlinks",
			description: "Whether or not the popup should also open for plain hyperlinks",
			requires: {
				mouseover: true
			},
			category: "popup",
			subcategory: "open_behavior"
		},
		mouseover_only_valid_links: {
			name: "Only for links that look valid",
			description: "Enabling this option will only allow links to be popped up if they look valid (such as if they have a known image/video extension, or are explicitly supported)",
			requires: {
				mouseover_links: true
			},
			category: "popup",
			subcategory: "open_behavior"
		},
		mouseover_allow_canvas_el: {
			name: "Popup for `<canvas>`",
			description: "Allows `<canvas>` elements to be popped up as well. This will likely cause popups with any kind of web-based games, so it's recommended to keep this disabled",
			requires: {
				mouseover: true
			},
			category: "popup",
			subcategory: "open_behavior"
		},
		mouseover_allow_svg_el: {
			name: "Popup for `<svg>`",
			description: "Allows `<svg>` elements to be popped up as well. These are usually used for icons, and can occasionally cause problems for websites that overlay icons on top of images",
			requires: {
				mouseover: true
			},
			category: "popup",
			subcategory: "open_behavior"
		},
		mouseover_enable_gallery: {
			name: "Enable gallery",
			description: "Toggles whether gallery detection support should be enabled",
			requires: {
				mouseover_open_behavior: "popup"
			},
			category: "popup",
			subcategory: "gallery"
		},
		mouseover_gallery_cycle: {
			name: "Cycle gallery",
			description: "Going to the previous image for the first image will lead to the last image and vice-versa",
			requires: {
				mouseover_open_behavior: "popup",
				mouseover_enable_gallery: true
			},
			category: "popup",
			subcategory: "gallery"
		},
		mouseover_gallery_prev_key: {
			name: "Previous gallery item",
			description: "Key to trigger the previous gallery item",
			requires: {
				mouseover_open_behavior: "popup",
				mouseover_enable_gallery: true
			},
			type: "keysequence",
			category: "popup",
			subcategory: "gallery"
		},
		mouseover_gallery_next_key: {
			name: "Next gallery item",
			description: "Key to trigger the next gallery item",
			requires: {
				mouseover_open_behavior: "popup",
				mouseover_enable_gallery: true
			},
			type: "keysequence",
			category: "popup",
			subcategory: "gallery"
		},
		mouseover_gallery_move_after_video: {
			name: "Move to next when video finishes",
			description: "Moves to the next gallery item when a video finishes playing",
			requires: {
				mouseover_open_behavior: "popup",
				allow_video: true,
				mouseover_enable_gallery: true
			},
			category: "popup",
			subcategory: "gallery"
		},
		mouseover_styles: {
			name: "Popup CSS style",
			description: "Custom CSS styles for the popup",
			documentation: {
				title: "Documentation",
				value: [
					"Most valid CSS is supported, with these differences:",
					"<ul><li>Multiline comments (<code>/* ... */</code>) are currently not supported</li>",
					"<li>Single comments (<code>// ...</code>) are supported, but only at the beginning of a line</li>",
					"<li><code>%thumburl%</code> is the URL of the thumbnail image. For example, you could use it like this: <code>background-image: url(%thumburl%)</code><br />",
					"The URL is properly encoded, so quotes are not necessary (but not harmful either)</li>",
					"<li><code>%fullurl%</code> is the URL of the full image. If IMU fails to find a larger image, it will be the same as <code>%thumburl%</code></li>",
					"<li>Styles are <code>!important</code> by default</li></ul>"
				].join("\n")
			},
			type: "textarea",
			requires: {
				mouseover_open_behavior: "popup"
			},
			category: "popup",
			subcategory: "popup_other"
		},
		mouseover_fade_time: {
			name: "Popup fade",
			description: "Fade in/out time (in milliseconds) for the popup, set to 0 to disable",
			requires: {
				mouseover_open_behavior: "popup"
			},
			type: "number",
			number_min: 0,
			number_unit: "ms",
			category: "popup",
			subcategory: "popup_other"
		},
		mouseover_enable_mask_styles: {
			name: "Enable background CSS",
			description: "Toggles whether CSS styles for the background when the popup is active is enabled",
			requires: {
				mouseover_open_behavior: "popup"
			},
			category: "popup",
			subcategory: "popup_other"
		},
		mouseover_mask_styles2: {
			name: "Background CSS style",
			description: "CSS style for the background when the popup is active. See the documentation for Popup CSS style for more information (the thumb/full URL variables aren't supported here)",
			requires: {
				mouseover_open_behavior: "popup",
				mouseover_enable_mask_styles: true
			},
			type: "textarea",
			category: "popup",
			subcategory: "popup_other"
		},
		mouseover_mask_fade_time: {
			name: "Background fade",
			description: "Fade in/out time (in milliseconds) for the page background, set to 0 to disable",
			requires: {
				mouseover_open_behavior: "popup",
				mouseover_enable_mask_styles: true
			},
			type: "number",
			number_min: 0,
			number_unit: "ms",
			category: "popup",
			subcategory: "popup_other"
		},
		mouseover_ui_styles: {
			name: "Button CSS style",
			description: "Custom CSS styles for the popup's UI buttons. See the documentation for Popup CSS style for more information (the thumb/full URL variables aren't supported here)",
			type: "textarea",
			requires: {
				mouseover_open_behavior: "popup",
				mouseover_ui: true
			},
			category: "popup",
			subcategory: "ui"
		},
		mouseover_apply_blacklist: {
			name: "Don't popup blacklisted images",
			description: "This option prevents a popup from appearing altogether for blacklisted images",
			requires: {
				mouseover: true
			},
			category: "popup",
			subcategory: "open_behavior"
		},
		apply_blacklist_host: {
			name: "Apply blacklist for host websites",
			description: "This option prevents the script from applying any popups to host websites that are in the blacklist. For example, adding `twitter.com` to the blacklist would prevent any popup from opening on twitter.com. If disabled, this option only applies to image URLs (such as twimg.com), not host URLs",
			category: "popup",
			subcategory: "open_behavior"
		},
		mouseover_matching_media_types: {
			name: "Don't popup video for image",
			description: "This option prevents the popup from loading a video when the source was an image. Vice-versa is also applied",
			requires: {
				mouseover: true,
				allow_video: true
			},
			category: "popup",
			subcategory: "open_behavior"
		},
		mouseover_support_pointerevents_none: {
			name: "Support `pointer-events:none`",
			description: "Manually looks through every element on the page to see if the cursor is beneath them. Supports more images, but also results in a higher CPU load for websites such as Facebook.",
			requires: {
				mouseover: true
			},
			//advanced: true, // Commenting this out because the option is important
			category: "popup",
			subcategory: "open_behavior"
		},
		website_inject_imu: {
			name: "Use userscript",
			description: "Replaces the website's IMU instance with the userscript",
			userscript_only: true,
			category: "website"
		},
		website_image: {
			name: "Website image preview",
			description: "Enables a preview of the image on the Image Max URL website",
			userscript_only: true,
			requires: {
				website_inject_imu: true
			},
			category: "website"
		},
		extension_contextmenu: {
			name: "IMU entry in context menu",
			description: "Enables a custom entry for this extension in the right click/context menu",
			extension_only: true,
			category: "extension",
			imu_enabled_exempt: true
		},
		allow_video: {
			name: "Videos",
			description: "Allows videos to be returned",
			category: "rules",
			onupdate: update_rule_setting
		},
		allow_dash_video: {
			name: "Allow DASH videos",
			description: "Allows playback of DASH video streams. Some videos may not work with other websites due to hotlinking protection.",
			description_userscript: "Allows playback of DASH video streams. Some videos may not work with other websites due to hotlinking protection, and it may even break video playback for some websites. Use with caution.",
			category: "rules",
			requires: {
				allow_thirdparty_libs: true
			}
		},
		allow_hls_video: {
			name: "Allow HLS videos",
			description: "Allows playback of HLS video streams. Some videos may not work with other websites due to hotlinking protection.",
			description_userscript: "Allows playback of HLS video streams. Some videos may not work with other websites due to hotlinking protection, and it may even break video playback for some websites. Use with caution.",
			category: "rules",
			requires: {
				allow_thirdparty_libs: true
			}
		},
		allow_watermark: {
			name: "Larger watermarked images",
			description: "Enables rules that return larger images that include watermarks",
			category: "rules",
			example_websites: [
				"Stock photo websites"
			],
			onupdate: update_rule_setting
		},
		allow_smaller: {
			name: "Smaller non-watermarked images",
			description: "Enables rules that return smaller images without watermarks",
			category: "rules",
			onupdate: update_rule_setting
		},
		allow_possibly_different: {
			name: "Possibly different images",
			description: "Enables rules that return images that possibly differ",
			category: "rules",
			example_websites: [
				"YouTube video thumbnails"
			],
			hidden: true, // not currently used
			onupdate: update_rule_setting
		},
		allow_possibly_broken: {
			name: "Possibly broken images",
			description: "Enables rules that return images that are possibly broken",
			category: "rules",
			hidden: true, // not currently used
			onupdate: update_rule_setting
		},
		allow_possibly_upscaled: {
			name: "Possibly upscaled images",
			description: "Enables rules that return images that are possibly upscaled",
			category: "rules",
			onupdate: update_rule_setting
		},
		allow_thirdparty: {
			name: "Rules using 3rd-party websites",
			description: "Enables rules that use 3rd-party websites",
			category: "rules",
			example_websites: [
				"Newsen"
			],
			onupdate: update_rule_setting
		},
		allow_apicalls: {
			name: "Rules using API calls",
			description: "Enables rules that use API calls. Strongly recommended to keep this enabled",
			category: "rules",
			example_websites: [
				"Instagram",
				"Flickr",
				"..."
			],
			onupdate: update_rule_setting
		},
		allow_thirdparty_libs: {
			name: "Rules using 3rd-party libraries",
			description: "Enables rules that use 3rd-party libraries",
			description_userscript: "Enables rules that use 3rd-party libraries. There is a possible (but unlikely) security risk for the userscript version",
			category: "rules",
			example_websites: [
				"Sites using testcookie (slowAES)"
			],
			onupdate: function() {
				update_rule_setting();
				real_api_cache.clear();
			}
		},
		browser_cookies: {
			name: "Use browser cookies",
			description: "Uses the browser's cookies for API calls in order to access otherwise private data",
			category: "rules",
			example_websites: [
				"Private Flickr images"
			],
			// Until GM_Cookie is implemented
			extension_only: true,
			onupdate: update_rule_setting
		},
		deviantart_prefer_size: {
			name: "DeviantArt: Prefer size over original",
			description: "Prefers a larger (but not upscaled) thumbnail image over a smaller original animated image",
			category: "rule_specific",
			onupdate: update_rule_setting
		},
		imgur_source: {
			name: "Imgur: Use source image",
			description: "If a source image is found for Imgur, try using it instead",
			category: "rule_specific",
			onupdate: update_rule_setting
		},
		instagram_use_app_api: {
			name: "Instagram: Use native API",
			description: "Uses Instagram's native API if possible, requires you to be logged into Instagram",
			category: "rule_specific",
			onupdate: update_rule_setting
		},
		instagram_gallery_postlink: {
			name: "Instagram: Use albums for post thumbnails",
			description: "Queries Instagram for albums when using the popup on a post thumbnail",
			category: "rule_specific"
		},
		tiktok_no_watermarks: {
			name: "TikTok: Don't use watermarked videos",
			description: "Uses non-watermarked videos for TikTok if possible. This will introduce an extra delay when loading the video as two extra requests need to be performed.",
			category: "rule_specific",
			onupdate: update_rule_setting
		},
		tumblr_api_key: {
			name: "Tumblr: API key",
			description: "API key for finding larger images on Tumblr",
			category: "rule_specific",
			type: "lineedit",
			onupdate: update_rule_setting
		},
		bigimage_blacklist: {
			name: "Blacklist",
			description: "A list of URLs that are blacklisted from being processed",
			category: "rules",
			type: "textarea",
			onupdate: function() {
				update_rule_setting();
				create_blacklist_regexes();
			},
			onedit: function() {
				var errors = create_blacklist_regexes();

				var errordiv;

				try {
					errordiv = document.querySelector("#option_bigimage_blacklist .error");
					errordiv.innerText = "";
				} catch (e) {
				}

				if (errors) {
					for (var i = 0; i < errors.length; i++) {
						if (errordiv)
							errordiv.innerText += errors[i].message + "\n";

						console.error(errors[i]);
					}
				}
			},
			documentation: {
				title: "Documentation",
				value: [
					"The examples below are written for the simple (glob) engine, not the regex engine. The glob engine is generally based on the UNIX glob syntax.<br />",
					"<ul><br />",
					"<li><code>google.com</code> will block https://google.com/, https://www.google.com/, https://abcdef.google.com/, https://def.abc.google.com/, etc.</li>",
					"<li><code>abc.google.com</code> will block https://abc.google.com/, https://def.abc.google.com/, etc.</li>",
					"<li><code>*.google.com</code> will block https://www.google.com/, https://def.abc.google.com/, etc. but not https://google.com/</li>",
					"<li><code>google.*/</code> will block https://google.com/, https://www.google.co.uk, etc.</li>",
					"<li><code>http://google.com</code> will block http://google.com/, but not https://google.com/, http://www.google.com/, etc.</li>",
					"<li><code>google.com/test</code> will block https://google.com/test, https://www.google.com/test/abcdef, but not https://google.com/, etc.</li>",
					"<li><code>google.com/*/test</code> will block https://google.com/abc/test, but not https://google.com/test or https://google.com/abc/def/test</li>",
					"<li><code>google.com/**/test</code> will block https://google.com/abc/test, https://google.com/abc/def/test, https://google.com/abc/def/ghi/test, etc. but not https://google.com/test</li>",
					"<li><code>g??gle.com</code> will block https://google.com/, https://gaagle.com/, https://goagle.com/, etc.</li>",
					"<li><code>google.{com,co.uk}</code> will block https://google.com/ and https://google.co.uk/</li>",
					"<li><code>g[oau]ogle.com</code> will block https://google.com/, https://gaogle.com/, and http://www.guogle.com/</li>",
					"<li><code>g[0-9]ogle.com</code> will block https://g0ogle.com/, https://g1ogle.com/, etc. (up to https://g9ogle.com/)</li>",
					"</ul>"
				].join("\n")
			}
		},
		bigimage_blacklist_engine: {
			name: "Blacklist engine",
			description: "How the blacklist should be processed",
			category: "rules",
			options: {
				glob: {
					name: "Simple (glob)"
				},
				regex: {
					name: "Regex"
				}
			}
		},
		replaceimgs_enable_keybinding: {
			name: "Enable trigger key",
			description: "Enables the use of the trigger key to run it without needing to use the menu",
			category: "extra",
			subcategory: "replaceimages",
		},
		replaceimgs_keybinding: {
			name: "Trigger key",
			description: "Trigger keybinding that will run the Replace Images function",
			requires: {
				replaceimgs_enable_keybinding: true
			},
			type: "keysequence",
			category: "extra",
			subcategory: "replaceimages"
		},
		replaceimgs_auto: {
			name: "Automatically replace images",
			description: "Automatically replace images to larger versions on pages you view",
			warning: {
				"true": "This could lead to rate limiting or IP bans"
			},
			// Auto-updating is disabled due to the warning above
			needrefresh: true,
			category: "extra",
			subcategory: "replaceimages"
		},
		replaceimgs_usedata: {
			name: "Use data URLs",
			description: "Uses data:// URLs instead of image links. Disabling this may improve compatibility with some bulk image downloader extensions",
			category: "extra",
			subcategory: "replaceimages",
			imu_enabled_exempt: true
		},
		replaceimgs_wait_fullyloaded: {
			name: "Wait until image is fully loaded",
			description: "Waits until the image being replaced is fully loaded before moving on to the next image",
			category: "extra",
			subcategory: "replaceimages",
			requires: {
				replaceimgs_usedata: false
			},
			imu_enabled_exempt: true
		},
		replaceimgs_totallimit: {
			name: "Max images to process at once",
			description: "The maximum amount of images to process at once",
			type: "number",
			number_min: 1,
			number_int: true,
			number_unit: "images",
			category: "extra",
			subcategory: "replaceimages",
			imu_enabled_exempt: true
		},
		replaceimgs_domainlimit: {
			name: "Max images per domain at once",
			description: "The maximum amount of images per domain to process at once",
			type: "number",
			number_min: 1,
			number_int: true,
			number_unit: "images",
			category: "extra",
			subcategory: "replaceimages",
			imu_enabled_exempt: true
		},
		replaceimgs_replaceimgs: {
			name: "Replace images",
			description: "Replaces images to their larger versions when the button is pressed",
			category: "extra",
			subcategory: "replaceimages",
			imu_enabled_exempt: true
		},
		replaceimgs_addlinks: {
			name: "Add links",
			description: "Adds links around images if a link doesn't already exist",
			category: "extra",
			subcategory: "replaceimages",
			imu_enabled_exempt: true
		},
		replaceimgs_replacelinks: {
			name: "Replace links",
			description: "Replaces links if they already exist",
			category: "extra",
			subcategory: "replaceimages",
			requires: {
				replaceimgs_addlinks: true
			},
			imu_enabled_exempt: true
		},
		highlightimgs_enable_keybinding: {
			name: "Enable trigger key",
			description: "Enables the use of the trigger key to run it without needing to use the menu",
			category: "extra",
			subcategory: "highlightimages",
		},
		highlightimgs_keybinding: {
			name: "Trigger key",
			description: "Trigger keybinding that will run the Highlight Images function",
			requires: {
				highlightimgs_enable_keybinding: true
			},
			type: "keysequence",
			category: "extra",
			subcategory: "highlightimages"
		},
		highlightimgs_enable: {
			name: "Enable button",
			description: "Enables the 'Highlight Images' button",
			category: "extra",
			subcategory: "highlightimages",
			imu_enabled_exempt: true
		},
		highlightimgs_auto: {
			name: "Automatically highlight images",
			description: "Automatically highlights images as you view pages",
			options: {
				_type: "or",
				always: {
					name: "Always"
				},
				hover: {
					name: "Hover",
					description: "When hovering over an image"
				},
				never: {
					name: "Never"
				}
			},
			category: "extra",
			subcategory: "highlightimages"
		},
		highlightimgs_onlysupported: {
			name: "Only explicitly supported images",
			description: "Only highlights images that can be made larger or the original version can be found",
			requires: [
				{highlightimgs_enable: true},
				{highlightimgs_auto: "always"},
				{highlightimgs_auto: "hover"}
			],
			category: "extra",
			subcategory: "highlightimages"
		},
		highlightimgs_css: {
			name: "Highlight CSS",
			description: "CSS style to apply for highlight. See the documentation for Popup CSS style for more information (the thumb/full URL variables aren't supported here)",
			type: "textarea",
			requires: [
				{highlightimgs_enable: true},
				{highlightimgs_auto: "always"},
				{highlightimgs_auto: "hover"}
			],
			category: "extra",
			subcategory: "highlightimages",
			imu_enabled_exempt: true
		}
	};

	var option_to_problems = {
		allow_watermark: "watermark",
		allow_smaller: "smaller",
		allow_possibly_different: "possibly_different",
		allow_possibly_broken: "possibly_broken",
		allow_possibly_upscaled: "possibly_upscaled"
	};

	var categories = {
		"general": "category_general",
		"redirection": "category_redirection",
		"popup": "category_popup",
		"rules": "category_rules",
		"website": "category_website",
		"extension": "category_extension",
		"extra": "category_extra"
	};

	var subcategories = {
		"general": {
			"settings": "subcategory_settings"
		},
		"popup": {
			"trigger": "subcategory_trigger",
			"open_behavior": "subcategory_open_behavior",
			"close_behavior": "subcategory_close_behavior",
			"behavior": "subcategory_behavior",
			"gallery": "subcategory_gallery",
			"video": "subcategory_video",
			"ui": "subcategory_ui",
			"popup_other": "subcategory_popup_other"
		},
		"rules": {
			"rule_specific": "subcategory_rule_specific"
		},
		"extra": {
			"replaceimages": "subcategory_replaceimages",
			"highlightimages": "subcategory_highlightimages"
		}
	};

	for (var option in option_to_problems) {
		var problem = option_to_problems[option];
		settings[option] = array_indexof(default_options.exclude_problems, problem) < 0;
	}

	var settings_history = {};

	var js_map_available = false;
	var new_map = function() {
		var map = {};

		try {
			map = new Map();
			js_map_available = true;
		} catch (e) {
		}

		return map;
	};

	var map_set = function(map, key, value) {
		if (_nir_debug_) {
			console_log("map_set", map, deepcopy(key), deepcopy(value));
		}

		if (js_map_available) {
			map.set(key, value);
		} else {
			map[key] = value;
		}

		return value;
	};

	var map_get = function(map, key) {
		if (js_map_available) {
			return map.get(key);
		} else {
			return map[key];
		}
	};

	var map_has = function(map, key) {
		if (js_map_available) {
			return map.has(key);
		} else {
			return key in map;
		}
	};

	var map_remove = function(map, key) {
		if (js_map_available) {
			map.delete(key);
		} else {
			delete map[key];
		}
	};

	var map_foreach = function(map, cb) {
		if (js_map_available) {
			var keys = map.keys();
			for (var i = 0; i < keys.length; i++) {
				cb(keys[i], map.get(keys[i]));
			}
		} else {
			for (var key in map) {
				cb(key, map[key]);
			}
		}
	};

	var js_set_available = false;
	var new_set = function() {
		var set = [];

		try {
			set = new Set();
			js_set_available = true;
		} catch (e) {
		}

		return set;
	};

	var set_add = function(set, key) {
		if (js_set_available) {
			set.add(key);
		} else {
			if (!set_has(set, key)) {
				set.push(key);
			}
		}
	};

	var set_has = function(set, key) {
		if (js_set_available) {
			return set.has(key);
		} else {
			return array_indexof(set, key) >= 0;
		}
	};

	function Cache() {
		this.data = new_map();
		this.times = new_map();

		this.fetches = new_map();

		this.set = function(key, value, time) {
			if (_nir_debug_)
				console_log("Cache.set key:", key, ", time=" + time + ", value:", deepcopy(value));

			this.remove(key);

			map_set(this.data, key, value);

			if (typeof time === "number" && time > 0) {
				var cache = this;
				var timer = setTimeout(function() {
					cache.remove(key);
				}, time * 1000);

				// Ensures the process can exit in node.js
				if (is_node && "unref" in timer) {
					timer.unref();
				}

				map_set(this.times, key, {
					timer: timer,
					time: time
				});
			}
		};

		this.has = function(key) {
			var has_key = map_has(this.data, key);

			if (_nir_debug_)
				console_log("Cache.has key:", key, has_key);

			return has_key;
		};

		this.get = function(key) {
			var value = map_get(this.data, key);

			// TODO: maybe renew timeout per-get?
			if (_nir_debug_)
				console_log("Cache.get key:", key, deepcopy(value));

			return value;
		};

		this.fetch = function(key, done, fetcher) {
			var exists = map_has(this.data, key);

			if (_nir_debug_)
				console_log("Cache.fetch key:", key, ", exists=" + exists);

			if (!exists) {
				if (map_has(this.fetches, key)) {
					map_get(this.fetches, key).push(done);
				} else {
					map_set(this.fetches, key, []);

					fetcher(function(data, time) {
						if (time !== false)
							this.set(key, data, time);

						done(data);

						var our_fetches = map_get(this.fetches, key);
						for (var i = 0; i < our_fetches.length; i++) {
							our_fetches[i](data);
						}

						map_remove(this.fetches, key);
					}.bind(this));
				}
			} else {
				done(map_get(this.data, key));
			}
		};

		this.remove = function(key) {
			if (_nir_debug_)
				console_log("Cache.remove key:", key);

			if (map_has(this.times, key)) {
				clearTimeout(map_get(this.times, key).timer);
			}

			map_remove(this.times, key);
			map_remove(this.data, key);
		};

		this.clear = function() {
			if (_nir_debug_)
				console_log("Cache.clear");

			map_foreach(this.times, function(key, value) {
				clearTimeout(value);
			});

			this.times = new_map();
			this.data = new_map();
		};
	};

	var url_cache = new Cache();
	var real_api_cache = new Cache();
	var lib_cache = new Cache();
	var cookie_cache = new Cache();

	var urlparse = function(x) {
		return new URL(x);
	};

	if (is_node && typeof URL === 'undefined') {
		var url = require("url");
		urlparse = function(x) {
			var parsed = url.parse(x);
			parsed.searchParams = new Map();
			if (parsed.query) {
				parsed.query.split("&").forEach(function (query) {
					var splitted = query.split("=");
					parsed.searchParams.set(splitted[0], splitted[1]);
				});
			}
			return parsed;
		};
	}

	// https://stackoverflow.com/a/17323608
	function mod(n, m) {
		return ((n % m) + m) % m;
	}

	function urlsplit(a) {
		var protocol_split = a.split("://");
		var protocol = protocol_split[0];
		var splitted = protocol_split[1].split("/");
		var domain = splitted[0];
		var start = protocol + "://" + domain;
		return {
			protocol: protocol,
			domain: domain,
			url: a
		};
	}

	function urlnorm(a) {
		var protocol_split = a.split("://");
		var splitted = protocol_split[1].split("/");
		var newsplitted = [];
		for (var i = 0; i < splitted.length; i++) {
			if (splitted[i] === "..")
				newsplitted.pop();
			else
				newsplitted.push(splitted[i]);
		}

		return protocol_split[0] + "://" + newsplitted.join("/");
	}

	var is_valid_resource_url = function(url) {
		var match = url.match(/^([-a-z]+):/);
		if (match) {
			var valid_schemes = ["http", "https", "ftp", "data", "x-raw-image", "blob", "chrome", "file"];
			return valid_schemes.indexOf(match[1].toLowerCase()) >= 0;
		}

		return true;
	};

	function urljoin(a, b, browser) {
		if (b.length === 0)
			return a;
		if (b.match(/^[-a-z]*:\/\//) || b.match(/^(?:data|x-raw-image|blob|about|javascript):/))
			return b;

		var protocol_split = a.split("://");

		// FIXME? for URLs like about:blank
		if (protocol_split.length < 2) {
			return a;
		}

		var protocol = protocol_split[0];
		var splitted = protocol_split[1].split("/");
		var domain = splitted[0];
		var start = protocol + "://" + domain;

		if (!browser) {
			// simple path join
			// urljoin("http://site.com/index.html", "file.png") = "http://site.com/index.html/file.png"
			return a + "/" + b.replace(/^\/*/, "");
		} else {
			if (b.length >= 2 && b.slice(0, 2) === "//")
				return protocol + ":" + b;
			if (b.length >= 1 && b.slice(0, 1) === "/")
				return start + b;

			// to emulate the browser's behavior instead
			// urljoin("http://site.com/index.html", "file.png") = "http://site.com/file.png"
			if (!a.match(/\/$/))
				a = a.replace(/^([^?]*)\/.*?$/, "$1/");

			return urlnorm(a + b.replace(/^\/*/, ""));
			//return a.replace(/\/[^/]*$/, "/") + b.replace(/^\/*/, "");
			//return urlnorm(a.replace(/^([^?]*)\/.*?$/, "$1/") + b.replace(/^\/*/, ""));
		}
	}

	var fullurl = function(url, x) {
		if (x === undefined || x === null)
			return x;

		var a = document.createElement(a);
		a.href = x;
		return a.href;
	};

	var fillobj_urls = function(urls, obj) {
		var newobj = [];
		for (var i = 0; i < urls.length; i++) {
			var currentobj = deepcopy(obj);

			if (typeof urls[i] === "string") {
				currentobj.url = urls[i];
			} else {
				for (var key in urls[i]) {
					currentobj[key] = urls[i][key]
				}
			}

			newobj.push(currentobj);
		}

		return newobj;
	};

	var add_full_extensions = function(obj, extensions, prefer_order) {
		if (!extensions)
			extensions = [
				"jpg", "jpeg", "png", "gif", "webp",
				"JPG", "JPEG", "PNG", "GIF"
			];

		if (!is_array(obj)) {
			obj = [obj];
		}

		var result = [];

		for (var i = 0; i < obj.length; i++) {
			var currentobj = obj[i];
			var url = currentobj;
			if (typeof currentobj !== "string") {
				url = currentobj.url;
			}

			var regex = /(.*)\.([^/.]*?)([?#].*)?$/;
			if (!url.match(regex)) {
				result.push(currentobj);
				continue;
			}

			var ext = url.replace(regex, "$2");
			var basename = url.replace(regex, "$1");
			var query = url.replace(regex, "$3");

			//var result = [url];
			if (!prefer_order)
				result.push(currentobj);

			for (var i = 0; i < extensions.length; i++) {
				if (!prefer_order && ext === extensions[i])
					continue;

				var currenturl = basename + "." + extensions[i] + query;
				if (typeof currentobj === "string") {
					result.push(currenturl);
				} else {
					var newobj = deepcopy(currentobj);
					newobj.url = currenturl;
					result.push(newobj);
				}
			}

			if (prefer_order && array_indexof(result, currentobj) < 0)
				result.push(currentobj);
		}

		return result;
	};

	var add_extensions = function(url) {
		return add_full_extensions(url, ["jpg", "png"]);
	};

	var add_extensions_jpeg = function(url) {
		return add_full_extensions(url, ["jpeg", "png"]);
	};

	var add_extensions_with_jpeg = function(url) {
		return add_full_extensions(url, ["jpg", "jpeg", "png"]);
	};

	var add_extensions_gif = function(url) {
		return add_full_extensions(url, ["jpg", "png", "gif"]);
	};

	var add_extensions_upper = function(url) {
		return add_full_extensions(url, ["jpg", "png", "JPG", "PNG"]);
	};

	var add_extensions_upper_jpeg = function(url) {
		return add_full_extensions(url, ["jpg", "jpeg", "png", "JPG", "JPEG", "PNG"]);
	};

	var add_http = function(url) {
		if (!url.match(/^[a-z]+:\/\//))
			return "http://" + url;
		return url;
	};

	var force_https = function(url) {
		return url.replace(/^http:\/\//, "https://");
	};

	var decodeuri_ifneeded = function(url) {
		if (url.match(/^https?:\/\//))
			return url;
		if (url.match(/^https?%3[aA]/) || /^[^/]*%2[fF]/.test(url))
			return decodeURIComponent(url);
		if (url.match(/^https?%253[aA]/))
			return decodeURIComponent(decodeURIComponent(url));
		return url;
	};

	var encodeuri_ifneeded = function(url) {
		// TODO: improve
		if (string_indexof(url, "%") < 0) {
			return encodeURI(url);
		}

		return url;
	};

	// https://stackoverflow.com/a/10073788
	var zpadnum = function(n, width, z) {
		z = z || '0';
		n = n + '';
		return n.length >= width ? n : new Array(width - n.length + 1).join(z) + n;
	}

	// https://www.w3resource.com/javascript-exercises/javascript-string-exercise-28.php
	function hex_to_ascii(str1) {
		var hex = str1.toString();
		var str = '';
		for (var n = 0; n < hex.length; n += 2) {
			str += String.fromCharCode(parseInt(hex.substr(n, 2), 16));
		}
		return str;
	}

	function hex_to_numberarray(str) {
		var result = [];

		for (var i = 0; i < str.length; i += 2) {
			result.push(parseInt(str.substr(i, 2), 16));
		}

		return result;
	}

	function numberarray_to_hex(arr) {
		var str = "";

		for (var i = 0; i < arr.length; i++) {
			if (arr[i] < 16)
				str += "0";
			str += arr[i].toString(16);
		}

		return str;
	}

	function reverse_str(str) {
		return str.split("").reverse().join("");
	}

	function decode_entities(str) {
		return str
			.replace(/&nbsp;/g, " ")
			.replace(/&#([0-9]+);/g, function (full, num) { return String.fromCharCode(num); })
			.replace(/&amp;/g, "&");
	}

	function get_queries(url) {
		// TODO: handle things like: ?a=b&c=b#&d=e
		var querystring = url.replace(/^[^#]*?\?/, "");
		if (!querystring || querystring === url)
			return {};

		var queries = {};

		var splitted = querystring.split("&");
		for (var i = 0; i < splitted.length; i++) {
			var name = splitted[i];
			var value = true;

			var match = splitted[i].match(/^(.*?)=(.*)/);
			if (match) {
				name = match[1];
				value = match[2];
			}

			if (name.length === 0)
				continue;

			queries[name] = value;
		}

		return queries;
	}

	function stringify_queries(queries) {
		var queriesstr = [];

		for (var query in queries) {
			if (query.length === 0)
				continue;

			var current_query = query;
			if (queries[query] !== true) {
				current_query += "=" + queries[query];
			}

			queriesstr.push(current_query);
		}

		return queriesstr.join("&");
	}

	function remove_queries(url, queries) {
		if (!is_array(queries)) {
			queries = [queries];
		}

		var beforequery = url.replace(/^([^#]*?)\?(.*)$/, "$1");
		var afterquery = url.replace(/^([^#]*?)\?(.*)$/, "$2");

		// TODO: handle things like: ?a=b&c=b#&d=e

		// no query string
		if (beforequery === url)
			return url;

		var splitted = afterquery.split("&");
		var newsplitted = [];
		for (var i = 0; i < splitted.length; i++) {
			var property = splitted[i].replace(/^(.*?)=.*/, "$1");
			if (array_indexof(queries, property) < 0) {
				newsplitted.push(splitted[i]);
			}
		}

		if (newsplitted.length === 0) {
			afterquery = "";
		} else {
			afterquery = "?" + newsplitted.join("&");
		}

		return beforequery + afterquery;
	}

	function add_queries(url, queries) {
		var parsed_queries = get_queries(url);

		for (var query in queries) {
			parsed_queries[query] = queries[query];
		}

		var newquerystring = stringify_queries(parsed_queries);
		if (newquerystring) {
			return url.replace(/^([^#]*?)(?:\?.*)?$/, "$1?" + newquerystring);
		} else {
			return url;
		}
	}

	function fuzzify_text(str) {
		return str
			.replace(/(?:[-=_!?$#"'’‘”“]|\[|])/g, " ")
			.replace(/\s+/g, " ")
			.replace(/^\s+|\s+$/g, "");
	}

	var _version_compare_pad_0 = function(array, amount) {
		if (amount <= 0)
			return;

		for (var i = 0; i < amount; i++) {
			array.push(0);
		}
	};

	function version_compare(a, b) {
		var version_regex = /^[0-9]+(\.[0-9]+){0,}$/;
		if (!version_regex.test(a) ||
			!version_regex.test(b))
			return null;

		var a_split = a.split(".");
		var b_split = b.split(".");

		if (a_split.length !== b_split.length) {
			_version_compare_pad_0(a_split, b_split - a_split);
			_version_compare_pad_0(b_split, a_split - b_split);
		}

		for (var i = 0; i < a_split.length; i++) {
			var an = parseInt(a_split[i]);
			var bn = parseInt(b_split[i]);

			if (an < bn)
				return 1;
			if (an > bn)
				return -1;
		}

		return 0;
	}

	var check_updates_firefox = function(cb) {
		do_request({
			url: firefox_addon_page,
			method: "GET",
			onload: function(resp) {
				if (resp.readyState < 4)
					return;

				if (resp.status !== 200)
					return cb(null);

				var match = resp.responseText.match(/<script[^>]*id=["']redux-store-state["']\s*>\s*({.*?})\s*<\/script>/);
				if (!match) {
					return cb(null);
				}

				try {
					var json = JSON_parse(match[1]);
					var addoninfo = json.addons.byID["1003321"];
					var versionid = addoninfo.currentVersionId;
					var versioninfo = json.versions.byId[versionid];
					var version = versioninfo.version;
					var downloadurl = versioninfo.platformFiles.all.url.replace(/\?src=.*/, "?src=external-updatecheck");

					cb({
						version: version,
						downloadurl: downloadurl
					});
				} catch (e) {
					console_error("Unable to parse mozilla addon info", e);
					return cb(null);
				}
			}
		});
	};

	var check_updates_github = function(cb) {
		do_request({
			url: "https://api.github.com/repos/qsniyg/maxurl/tags",
			method: "GET",
			headers: {
				Referer: ""
			},
			onload: function(resp) {
				if (resp.readyState <4 )
					return;

				if (resp.status !== 200)
					return cb(null);

				try {
					var json = JSON_parse(resp.responseText);

					for (var i = 0; i < json.length; i++) {
						var version = json[i].name;

						if (!version.match(/^v[0-9.]+$/)) {
							continue;
						}

						return cb({
							version: version.replace(/^v([0-9])/, "$1")
						});
					}
				} catch (e) {
					console_error("Unable to parse github info", e);
				}

				return cb(null);
			}
		});
	};

	var check_updates = function(cb) {
		// Firefox blocks these requests
		if (false && is_firefox_webextension) {
			check_updates_firefox(function(data) {
				if (!data) {
					check_updates_github(cb);
				} else {
					cb(data);
				}
			});
		} else {
			check_updates_github(cb);
		}
	};

	var check_updates_if_needed = function() {
		// if last_update_check == 0, it means for some reason it's not able to store values
		if (!settings.imu_enabled || !settings.check_updates || !current_version || !settings.last_update_check) {
			return;
		}

		var update_check_delta = Date.now() - settings.last_update_check;
		if (update_check_delta > (settings.check_update_interval*60*60*1000)) {
			check_updates(function(data) {
				update_setting("last_update_check", Date.now());

				if (!data || !data.version)
					return;

				if (!data.downloadurl) {
					update_setting("last_update_url", null);
				} else {
					update_setting("last_update_url", data.downloadurl);
				}

				update_setting("last_update_version", data.version);
			});
		}
	};

	function _fuzzy_compare_rollover(a, b, lim) {
		if (a === b)
			return true;

		if (a - 1 === b || a + 1 === b)
			return true;

		for (var i = 0; i < lim.length; i++) {
			if (a === lim[i]) {
				if (b === 1)
					return true;
			} else if (b === lim[i]) {
				if (a === 1)
					return true;
			}
		}

		return false;
	}

	function _is_larger_rollover(a, b, end) {
		if (a === 1 && array_indexof(end, b) >= 0)
			return true;

		if (b === 1 && array_indexof(end, a) >= 0)
			return true;

		return false;
	}

	function fuzzy_date_compare(a, b) {
		if (a === b)
			return true;

		if (a.length !== 8 || b.length !== 8)
			return false;

		var a_d = parseInt(a.substr(6, 2));
		var b_d = parseInt(b.substr(6, 2));

		if (!_fuzzy_compare_rollover(a_d, b_d, [28, 29, 30, 31]))
			return false;

		var a_m = parseInt(a.substr(4, 2));
		var b_m = parseInt(b.substr(4, 2));

		var d_rollover = _is_larger_rollover(a_d, b_d, [28, 29, 30, 31]);
		if (a_m !== b_m) {
			if (!d_rollover)
				return false;
			if (!_fuzzy_compare_rollover(a_m, b_m, [12]))
				return false;
		}

		var a_y = parseInt(a.substr(0, 4));
		var b_y = parseInt(b.substr(0, 4));

		if (a_y !== b_y) {
			if (!d_rollover || !_is_larger_rollover(a_m, b_m, [12]))
				return false;
			if (!_fuzzy_compare_rollover(a_y, b_y, []))
				return false;
		}

		return true;
	}

	function run_soon(func) {
		setTimeout(func, 1);
	}

	// bug in chrome, see
	// https://github.com/qsniyg/maxurl/issues/7
	// https://our.umbraco.org/forum/using-umbraco-and-getting-started/91715-js-error-when-aligning-content-left-center-right-justify-in-richtext-editor
	if (is_node || true) {
		fullurl = function(url, x) {
			return urljoin(url, x, true);
		};
	}

	var blacklist_regexes = [];

	function update_rule_setting() {
		url_cache.clear();
	}

	function create_blacklist_regexes() {
		blacklist_regexes = [];
		var blacklist = settings.bigimage_blacklist || "";
		if (typeof blacklist !== "string") {
			console_warn("Invalid blacklist", blacklist);
			return;
		}

		blacklist = blacklist.split("\n");

		for (var i = 0; i < blacklist.length; i++) {
			var current = blacklist[i].replace(/^\s+|\s+$/, "");
			//console_log(current);
			if (current.length === 0)
				continue;

			if (settings.bigimage_blacklist_engine === "regex") {
				try {
					blacklist_regexes.push(new RegExp(current));
				} catch (e) {
					return [e];
				}
			} else if (settings.bigimage_blacklist_engine === "glob") {
				var newcurrent = "";
				var sbracket = -1;
				var cbracket = -1;
				for (var j = 0; j < current.length; j++) {
					if (sbracket >= 0) {
						if (current[j] === "]") {
							newcurrent += current.substr(sbracket, j - sbracket + 1);
							sbracket = -1;
						}
						continue;
					}

					if (cbracket >= 0) {
						if (current[j] === "}") {
							var options = current.substr(cbracket + 1, j - cbracket - 1).split(",");
							var newoptions = [];
							for (var k = 0; k < options.length; k++) {
								newoptions.push(options[k].replace(/(.)/g, "[$1]"));
							}
							if (newoptions.length > 0 && (newoptions.length > 1 || newoptions[0].length > 0))
								newcurrent += "(?:" + newoptions.join("|") + ")";
							cbracket = -1;
						}

						continue;
					}

					if (current[j] !== "*") {
						if (current[j] === "{") {
							cbracket = j;
						} else if (current[j] === "[") {
							sbracket = j;
						} else if (current[j] === "?") {
							newcurrent += "[^/]";
						} else if (current[j] === ".") {
							newcurrent += "\\.";
						} else {
							newcurrent += current[j];
						}
						continue;
					}

					var doublestar = false;
					if ((j + 1) < current.length) {
						if (current[j+1] === "*") {
							doublestar = true;
							j++;
						}
					}

					if (doublestar)
						newcurrent += ".+";
					else
						newcurrent += "[^/]+";
				}

				current = newcurrent;

				if (current[0] !== "*") {
					newcurrent = current.replace(/^[a-z]*:\/\//, "[a-z]+://");
					if (newcurrent !== current) {
						current = newcurrent;
					} else {
						current = "[a-z]+://[^/]*" + current;
					}
				}

				current = "^" + current;

				try {
					blacklist_regexes.push(new RegExp(current));
				} catch (e) {
					return [e];
				}
			}
		}

		//console_log(blacklist_regexes);
	}

	var parse_headers = function(headerstr) {
		var headers = [];

		var splitted = headerstr.split("\r\n");
		for (var i = 0; i < splitted.length; i++) {
			var header_name = splitted[i].replace(/^\s*([^:]*?)\s*:[\s\S]*/, "$1").toLowerCase();
			var header_value = splitted[i].replace(/^[^:]*?:\s*([\s\S]*?)\s*$/, "$1");

			if (header_name === splitted[i] || header_value === splitted[i])
				continue;

			var value_split = header_value.split("\n");
			for (var j = 0; j < value_split.length; j++) {
				headers.push({name: header_name, value: value_split[j]});
			}
		}

		if (_nir_debug_)
			console_log("parse_headers", headerstr, deepcopy(headers));

		return headers;
	};

	var headers_list_to_dict = function(headers) {
		var dict = {};

		for (var i = 0; i < headers.length; i++) {
			dict[headers[i].name.toLowerCase()] = headers[i].value;
		}

		return dict;
	};

	var headers_dict_to_list = function(headers) {
		var list = [];

		for (var header in headers) {
			list.push({name: header, value: headers[header]});
		}

		return list;
	};

	var parse_cookieheader = function(cookieheader) {
		var cookies = {};

		do {
			var match = cookieheader.match(/^\s*([^=]*?)\s*=\s*([^;]*?)\s*(?:;\s*(.*))?$/);
			if (!match)
				break;

			cookies[match[1]] = match[2];
			cookieheader = match[3];
		} while (cookieheader);

		if (_nir_debug_)
			console_log("parse_cookieheader", cookieheader, deepcopy(cookies));

		return cookies;
	};

	var create_cookieheader_from_headers = function(headers, cookieheader) {
		headers = parse_headers(headers);

		var cookies = {};
		for (var i = 0; i < headers.length; i++) {
			if (headers[i].name !== "set-cookie")
				continue;

			var cookie_match = headers[i].value.match(/^\s*([^=]*?)\s*=\s*([^;]*?)\s*;.*/);
			if (!cookie_match) {
				console_error("Unable to match cookie: ", headers[i]);
				continue;
			}

			cookies[cookie_match[1]] = cookie_match[2];
		}

		if (_nir_debug_)
			console_log("create_cookieheader_from_headers", headers, cookieheader, deepcopy(cookies));

		if (cookieheader) {
			var parsed = parse_cookieheader(cookieheader);

			for (var key in parsed) {
				if (!(key in cookies)) {
					cookies[key] = parsed[key];
				}
			}
		}

		var cookies_array = [];
		for (var key in cookies) {
			cookies_array.push(key + "=" + cookies[key]);
		}

		return cookies_array.join("; ");
	};

	var extmap = {
		"jpeg": "jpg"
	};

	var get_ext_from_contenttype = function(contenttype) {
		var split = contenttype.match(/^\s*\[?([^/]+)\/([^/]+)\]?\s*$/);

		if (!split)
			return null;

		if (split[0] !== "image" && split[0] !== "video")
			return null;

		if (split[1] in extmap)
			return extmap[split[1]];

		return split[1];
	};

	// https://stackoverflow.com/a/18639999
	var makeCRCTable = function() {
		var c;
		var crcTable = [];
		for (var n =0; n < 256; n++) {
			c = n;
			for (var k =0; k < 8; k++) {
				c = ((c&1) ? (0xEDB88320 ^ (c >>> 1)) : (c >>> 1));
			}
			crcTable[n] = c;
		}
		return crcTable;
	};

	var cached_crc_table = null;
	var crc32 = function(str) {
		var crcTable = cached_crc_table || (cached_crc_table = makeCRCTable());
		var crc = 0 ^ (-1);

		for (var i = 0; i < str.length; i++) {
			crc = (crc >>> 8) ^ crcTable[(crc ^ str.charCodeAt(i)) & 0xFF];
		}

		return (crc ^ (-1)) >>> 0;
	};

	var run_sandboxed_lib = function(fdata) {
		if (true) {
			return new Function(fdata + ";return lib_export;")();
		} else {
			// doesn't work unfortunately
			var frame = document.createElement('iframe');
			frame.srcdoc = ""; //"javascript:0"
			document.body.appendChild(frame);
			var result = frame.contentWindow.Function(fdata + ";return lib_export;")();
			frame.parentElement.removeChild(frame);

			return result;
		}
	};

	var lib_urls = {
		"testcookie_slowaes": {
			name: "testcookie_slowaes",
			url: "https://raw.githubusercontent.com/qsniyg/maxurl/5af5c0e8bd18fb0ae716aac04ac42992d2ddc2e5/lib/testcookie_slowaes.js",
			size: 31248,
			crc32: 2955697328,
			crc32_size: 4174899014
		},
		"dash": {
			name: "dash.all.debug",
			url: "https://github.com/qsniyg/maxurl/blob/c18941c401c344a9db868a41899cdc4522eebd64/lib/dash.all.debug.js?raw=true",
			size: 2192274,
			crc32: 3434155307,
			crc32_size: 2440872286
		},
		"hls": {
			name: "hls",
			url: "https://github.com/qsniyg/maxurl/raw/0ec7ead029e967854d839c0ee0f0fa30de3848d3/lib/hls.js",
			size: 694990,
			crc32: 1683555956,
			crc32_size: 1065995231
		}
	};

	var get_library = function(name, options, do_request, cb) {
		if (!options.allow_thirdparty_libs) {
			console_warn("Refusing to request library " + name + " due to 3rd-party library support being disabled");
			return cb(null);
		}

		if (!(name in lib_urls)) {
			console_error("Invalid library", name);
			return cb(null);
		}

		var lib_obj = lib_urls[name];

		if (is_scripttag) {
			return cb(null);
		} else if (is_node) {
			try {
				var lib = require("./lib/" + lib_obj.name + ".js");
				return cb(lib);
			} catch (e) {
				console.error(e);
				return cb(null);
			}
		}

		if (is_extension || is_userscript) {
			// Unfortunately in these cases it's less clean
			// Without building separate version of this for each use case, we have to fall back on "eval"ing (through `new Function`)
			lib_cache.fetch(name, cb, function (done) {
				if (is_extension) {
					// Thankfully in this case can query directly from the extension, removing possibility for XSS
					extension_send_message({
						type: "get_lib",
						data: {
							name: lib_obj.name
						}
					}, function (response) {
						if (!response || !response.data || !response.data.text)
							return done(null, 0); // 0 instead of false because it will never be available

						//done(new Function(response.data.text + ";return lib_export;")(), 0);
						done(run_sandboxed_lib(response.data.text), 0);
					});
				} else {
					// For the userscript, we have no other choice than to query and run arbitrary JS.
					// The commit is specified in lib_urls in order to prevent possible incompatibilities.
					// Unfortunately this still cannot prevent a very dedicated hostile takeover.
					// This is why we use the dual CRC32 check. Not bulletproof, but much better than nothing.
					// On my system it takes 6ms to do both CRC32 checks, so performance isn't an issue.

					do_request({
						method: "GET",
						url: lib_obj.url,
						headers: {
							Referer: ""
						},
						onload: function(result) {
							if (result.readyState !== 4)
								return;

							if (result.status !== 200) {
								console_error(result);
								return done(null, false);
							}

							if (result.responseText.length !== lib_obj.size) {
								console_error("Wrong response length for " + name + ": " + result.responseText.length + " (expected " + lib_obj.size + ")");
								return done(null, false);
							}

							var crc = crc32(result.responseText);
							if (crc !== lib_obj.crc32) {
								console_error("Wrong crc32 for " + name + ": " + crc + " (expected " + lib_obj.crc32 + ")");
								return done(null, false);
							}

							crc = crc32(result.reponseText + (lib_obj.size + ""));
							if (crc !== lib_obj.crc32_size) {
								console_error("Wrong crc32 #2 for " + name + ": " + crc + " (expected " + lib_obj.crc32_size + ")");
								return done(null, false);
							}

							done(run_sandboxed_lib(result.responseText), 0);
							//done(new Function(result.responseText + ";return lib_export;")(), 0);
						}
					});
				}
			});
		} else {
			return cb(null);
		}
	};

	var normalize_whitespace = function(str) {
		// https://stackoverflow.com/a/11305926
		return str
			.replace(/[\u200B-\u200D\uFEFF]/g, '')
			.replace(/[\u2800]/g, ' ');
	};

	var strip_whitespace = function(str) {
		if (!str || typeof str !== "string") {
			return str;
		}

		return normalize_whitespace(str)
			.replace(/^\s+/, "")
			.replace(/\s+$/, "");
	};

	var common_functions = {};
	common_functions.fetch_imgur_webpage = function(do_request, api_cache, headers, url, cb) {
		var cache_key = "imgur_webpage:" + url.replace(/^https?:\/\/(?:www\.)?imgur/, "imgur").replace(/[?#].*/, "");

		var apply_headers = false;

		var real_fetch = function(done) {
			var request_headers;
			if (apply_headers)
				request_headers = headers;

			do_request({
				url: url.replace(/^http:/, "https:"),
				method: "GET",
				headers: request_headers,
				onload: function(resp) {
					if (resp.readyState !== 4)
						return;

					if (resp.status !== 200) {
						console_error("Bad status for Imgur: " + resp.status);

						if (resp.status === 404) {
							return done({
								bad: true
							}, false);
						}

						return done(null, false);
					}

					var ogvideo, ogimage;

					var ogmatch = resp.responseText.match(/<meta\s+property=["']og:video["']\s+content=["'](.*?)["']/);
					if (ogmatch) {
						ogvideo = decode_entities(ogmatch[1]).replace(/\?.*/, "");
					}

					ogmatch = resp.responseText.match(/<meta\s+property=["']og:image["']\s+content=["'](.*?)["']/);
					if (ogmatch) {
						ogimage = decode_entities(ogmatch[1]).replace(/\?.*/, "");
					}

					var retobj = {
						ogvideo: ogvideo,
						ogimage: ogimage
					};

					var imageinfo;
					var match = resp.responseText.match(/\.\s*mergeConfig\s*\(\s*["']gallery["']\s*,\s*{[\s\S]+?image\s*:\s*({.*?})\s*,\s*\n/);
					if (!match) {
						var nsfwmatch = resp.responseText.match(/<a.*?btn-wall--yes.*?\.signin\(/);
						var msg = "Unable to find match for Imgur page";

						if (nsfwmatch) {
							msg += " (it's probably NSFW and you aren't logged in)";
						}

						console_warn(msg);

						if (headers && !apply_headers) {
							console_log("Retrying with custom headers");
							apply_headers = true;
							return real_fetch(done);
						}

						// Only cache it for 15 seconds (helpful if the user logs in)
						done(retobj, 15);
					} else {
						imageinfo = match[1];

						try {
							imageinfo = JSON_parse(imageinfo);
							retobj.imageinfo = imageinfo;

							if (imageinfo.source && imageinfo.hash && !imageinfo.is_album) {
								api_cache.set("imgur_imageinfo:" + imageinfo.hash, imageinfo, 6*60*60);
							}
						} catch (e) {
							console_error(e);
							console_log(match);
							imageinfo = undefined;
						}

						done(retobj, 6*60*60);
					};
				}
			});
		};

		api_cache.fetch(cache_key, cb, real_fetch);
	};

	common_functions.deviantart_page_from_id = function(do_request, api_cache, id, cb) {
		var cache_key = "deviantart_page_from_id:" + id;

		api_cache.fetch(cache_key, cb, function (done) {
			do_request({
				method: "GET",
				url: "http://fav.me/" + id,
				onload: function (result) {
					if (result.status !== 200) {
						console_log(cache_key, result);
						return done(null, false);
					}

					done(result, 60 * 60);
				}
			});
		});
	};

	common_functions.wix_image_info = function(url) {
		// https://images-wixmp-ed30a86b8c4ca887773594c2.wixmp.com/f/06653b48-43c3-403f-9d72-c1c5519db560/ddl6lkp-cde05779-5a0a-47d8-8d43-f31a063fd30e.jpg/v1/fill/w_623,h_350,q_100/into_the_light_by_pajunen_ddl6lkp-350t.jpg
		// https://images-wixmp-ed30a86b8c4ca887773594c2.wixmp.com/f/faa48d2d-12c2-43d1-bf23-b5e99857825b/ddo0eau-c742e0f9-07f9-4a22-8d47-933e9fd3fb2b.png/v1/crop/w_244,h_350,x_0,y_0,scl_0.066812705366922,q_70,strp/railway_road_to_the_stars_by_ellysiumn_ddo0eau-350t.jpg

		// https://images-wixmp-ed30a86b8c4ca887773594c2.wixmp.com/intermediary/f/e04c7e93-4504-4dbe-91f9-fd353fc145f2/dcx503r-30c02b72-c26d-4732-9c9f-14fcbc633aaa.jpg
		var match = url.match(/^[a-z]+:\/\/[^/]+\/+(?:intermediary\/+)?([if])\/+[-0-9a-f]{20,}\/+[^/]+(?:[?#].*)?$/);
		if (match) {
			if (match[1] === "i") {
				return {
					preview: true
				};
			} else {
				return {
					original: true
				};
			}
		}

		var match = url.match(/^[a-z]+:\/\/[^/]+\/+(?:intermediary\/+)?[if]\/+[-0-9a-f]{20,}\/+[^/]+\/+v1\/+(?:fit|fill|crop)\/+([^/]+)\/+[^/]+(?:[?#].*)?$/);
		if (!match) {
			return null;
		}

		var infostr = match[1];
		var splitted = infostr.split(",");

		var obj = {};

		for (var i = 0; i < splitted.length; i++) {
			if (splitted[i].match(/^[a-z]+$/)) {
				obj[splitted[i]] = true;
				continue;
			}

			var name = splitted[i].replace(/_.*$/, "");
			var value = splitted[i].replace(/^.*?_/, "");

			if (value.match(/^[-0-9.]+$/))
				value = parseFloat(value);

			obj[name] = value;
		}

		if (obj.w && obj.h) {
			obj.pixels = obj.w * obj.h;
		}

		return obj;
	};

	common_functions.wix_compare = function(url1, url2, prefer_size) {
		if (_nir_debug_)
			console_log("wix_compare", url1, url2, prefer_size);

		if (!url2)
			return url1;

		var info1 = common_functions.wix_image_info(url1);
		var info2 = common_functions.wix_image_info(url2);

		if (!info1 || !info2)
			return null;

		if (info1.original && (!prefer_size || !info2.preview))
			return url1;

		if (info2.original && (!prefer_size || !info1.preview))
			return url2;

		if (info1.pixels && info2.pixels) {
			if (info1.pixels > info2.pixels)
				return url1;
			else if (info2.pixels > info1.pixels)
				return url2;
			else if (info1.q && info2.q) {
				if (info1.q > info2.q) {
					return url1;
				} else if (info2.q > info1.q) {
					return url2;
				}
			}
		}

		return null;
	};

	common_functions.deviantart_fullimage = function(options, api_cache, src, id, cb) {
		// animated:
		// https://images-wixmp-ed30a86b8c4ca887773594c2.wixmp.com/i/db0d85b1-b8b9-4790-bef0-121edb2dce7d/ddabn1h-5342115a-06a6-4f89-9c54-a2843719553a.jpg/v1/fit/w_150,h_150,q_70,strp/spaghetti_high_by_f1x_2_ddabn1h-150.jpg
		// https://images-wixmp-ed30a86b8c4ca887773594c2.wixmp.com/i/db0d85b1-b8b9-4790-bef0-121edb2dce7d/ddabn1h-5342115a-06a6-4f89-9c54-a2843719553a.jpg -- 1920x1080, but original size says 1280x720 and is animated. however, it doesn't look upscaled either
		//   https://images-wixmp-ed30a86b8c4ca887773594c2.wixmp.com/f/db0d85b1-b8b9-4790-bef0-121edb2dce7d/ddabn1h-1d7ac558-1fa8-4943-a3b6-0ad8b06be106.gif
		common_functions.deviantart_page_from_id(options.do_request, api_cache, id, function(result) {
			if (!result) {
				return cb(null);
			}

			var obj = {
				url: null,
				waiting: false,
				extra: {
					page: result.finalUrl
				}
			};

			if (!src.match(/:\/\/[^/]*\/fake_image\//))
				obj.url = src;

			var deviationid = result.finalUrl.replace(/.*-([0-9]+)(?:[?#].*)?$/, "$1");
			if (deviationid === result.finalUrl)
				deviationid = null;

			// Support the redesign
			var initialstate = result.responseText.match(/window\.__INITIAL_STATE__\s*=\s*JSON\.parse\((".*")\);\s*(?:window\.|<\/script)/);
			if (initialstate && deviationid) {
				initialstate = JSON_parse(JSON_parse(initialstate[1]));
				//console_log(initialstate);

				try {
					var deviation = initialstate["@@entities"].deviation[deviationid];
					//console_log(deviation);

					if (deviation.title)
						obj.extra.caption = deviation.title;

					var maxurl = obj.url;

					var files = deviation.files;

					if (is_array(files)) {
						for (var i = files.length - 1; i >= 0; i--) {
							var current = files[i];
							var newurl = common_functions.wix_compare(current.src, maxurl, options.rule_specific.deviantart_prefer_size);
							if (newurl === current.src)
								maxurl = newurl;
						}
					} else if ("media" in deviation && is_array(deviation.media.types)) {
						var types = deviation.media.types;

						for (var i = types.length - 1; i >= 0; i--) {
							var link = null;

							var tokenid = 0;

							// https://images-wixmp-ed30a86b8c4ca887773594c2.wixmp.com/f/40458dca-4360-4b4b-8aca-ba831f8db36d/ddsdflz-9484b31e-187e-4761-9724-6853698242a7.png/v1/fill/w_712,h_1123,q_100/morning_sun_by_pegaite_ddsdflz-pre.png
							// https://www.deviantart.com/pegaite/art/Morning-Sun-833716295
							if ("r" in types[i] && types[i].r < deviation.media.token.length)
								tokenid = types[i].r;

							var tokenq = "?token=" + deviation.media.token[tokenid];

							if (types[i].c) {
								link = deviation.media.baseUri + "/" + types[i].c.replace("<prettyName>", deviation.media.prettyName) + tokenq;
							} else if (types[i].b) { // e.g. animated gifs
								link = types[i].b + tokenq;
							} else if (types[i].t === "fullview" && "r" in types[i]) {
								// TODO: improve check?
								link = deviation.media.baseUri + tokenq;
							}

							// Occasionally this exists for some images, where it instead has:
							// s: "https://st.deviantart.net/misc/noentrythumb-200.png" (for t: "social_preview")
							if (!link)
								continue;

							var newurl = common_functions.wix_compare(link, maxurl, options.rule_specific.deviantart_prefer_size);
							if (newurl === link)
								maxurl = newurl;
						}
					}

					obj.url = maxurl;

					var image_info = common_functions.wix_image_info(obj.url);
					if (image_info && image_info.original) {
						obj.is_original = true;
						return cb(obj);
					}
				} catch (e) {
					console_error(e);
				}
			}

			try {
				var hrefre = /href=["'](https?:\/\/www\.deviantart\.com\/+download\/+[0-9]+\/+[^/>'"]*?)["']/;
				var match = result.responseText.match(hrefre);
				if (!match) {
					console_error("No public download for " + src);

					// This will occasionally scale down the image
					// Broken for the site redesign
					if (false) {
						try {
							var hrefre = /<img[^>]*?src=["'](https?:\/\/(?:images-wixmp)[^>'"]*?)["'][^>]*class=["']dev-content-/g;
							var match = result.responseText.match(hrefre);
							if (match) {
								var maxres = 0;
								var maxurl = null;
								for (var i = 0; i < match.length; i++) {
									var whmatch = match[i].match(/width=["']?([0-9]+)/);
									var oururl = match[i].match(/\ssrc=['"](http[^'"]*)/);
									if (!oururl)
										continue;
									oururl = oururl[1];
									var base = 0;
									if (!whmatch)
										continue;
									base = parseInt(whmatch[1]);
									whmatch = match[i].match(/height=["']?([0-9]+)/);
									if (!whmatch)
										continue;
									base *= parseInt(whmatch[1]);

									if (base > maxres) {
										maxres = maxres;
										maxurl = oururl;
									}
								}

								if (maxurl &&
									maxurl.replace(/\?.*/) !== src.replace(/\?.*/)) {
									var compare = common_functions.wix_compare(maxurl, src);

									if (compare === maxurl) {
										obj.url = maxurl;
										obj.likely_broken = false;
										return cb(obj);
									}
								}
							} else {
								console_error("No image found (this is likely a bug)");
							}

							return cb(obj)
						} catch (e) {
							console_error(e);
							return cb(obj);
						}
					} else {
						return cb(obj);
					}
					return;
				}

				var href = match[1].replace("&amp;", "&");

				options.do_request({
					method: "HEAD",
					url: href,
					onload: function (result) {
						if (result.status !== 200 && result.status !== 405) {
							console_log("Error fetching DeviantArt download link:");
							console_log(result);
							return cb(obj);
						}

						var finalurl = result.finalUrl;
						if (finalurl.match(/^[a-z]+:\/\/(?:www\.)?deviantart\.com\/+users\/+outgoing\?/)) {
							finalurl = finalurl.replace(/^[a-z]+:\/\/(?:www\.)?deviantart\.com\/+users\/+outgoing\?/, "");
						}
						obj.url = finalurl;
						obj.likely_broken = false;
						return cb(obj);
					}
				});
			} catch (e) {
				console_error(e);
				return cb(obj);
			}
		});
	}

	common_functions.get_testcookie_cookie = function(options, api_cache, site, cb) {
		var cache_key = "testcookie_cookie:" + site;
		var do_request = options.do_request;

		api_cache.fetch(cache_key, cb, function (done) {
			get_library("testcookie_slowaes", options, options.do_request, function (lib) {
				if (!lib) {
					console_error(cache_key, "Unable to fetch patched slowAES library");
					return done(null, false);
				}

				do_request({
					method: "GET",
					url: site,
					headers: {
						Referer: "",
						Cookie: ""
					},
					onload: function (result) {
						if (result.readyState !== 4)
							return;

						if (result.status !== 200) {
							console_log(cache_key, result);
							return done(null, false);
						}

						var matches = result.responseText.match(/[a-z]=\s*toNumbers\(["'][0-9a-f]+["']/g);
						if (matches.length === 0) {
							console_log(cache_key, "Unable to find toNumbers match", result);
							return done(null, false);
						}

						var vartable = {};
						for (var i = 0; i < matches.length; i++) {
							var match = matches[i].match(/([a-z])=toNumbers\(["']([0-9a-f]+)["']/);
							vartable[match[1]] = hex_to_numberarray(match[2]);
						}

						var slowaesmatch = result.responseText.match(/slowAES\.decrypt\s*\(\s*(.)\s*,\s*(.)\s*,\s*(.)\s*,\s*(.)\s*\)/);
						if (!slowaesmatch) {
							console_log(cache_key, "Unable to find slowAES match", result);
							return done(null, false);
						}

						var cookiename = result.responseText.match(/document\.cookie\s*=\s*["']([^='"]*?)=/);
						if (!cookiename) {
							console_log(cache_key, "Unable to find cookie name match", result);
							return done(null, false);
						} else {
							cookiename = cookiename[1];
						}

						var getarg = function(x) {
							if (/[a-z]/.test(x))
								return vartable[x];
							return parseInt(x);
						};

						var cookievalue = numberarray_to_hex(lib.decrypt(
							getarg(slowaesmatch[1]), getarg(slowaesmatch[2]),
							getarg(slowaesmatch[3]), getarg(slowaesmatch[4])));

						done(cookiename + "=" + cookievalue, 6*60*60);
					}
				});
			});
		});
	};

	common_functions.instagram_username_from_sharedData = function(json) {
		if (json.username)
			return json.username;
		else {
			var entrydata = json.entry_data;

			if (entrydata.ProfilePage)
				return entrydata.ProfilePage[0].graphql.user.username;
			else
				return entrydata.PostPage[0].graphql.shortcode_media.owner.username;
		}
	};

	common_functions.instagram_get_imageid = function(image_url) {
		if (!image_url)
			return image_url;

		return image_url.replace(/.*\/([^/.]*)\.[^/.]*(?:[?#].*)?$/, "$1");
	};

	common_functions.instagram_parse_el_info = function(api_cache, do_request, use_app_api, info, cb) {
		var query_ig = function(url, cb) {
			// Normalize the URL to reduce duplicate cache checks
			url = url
				.replace(/[?#].*/, "")
				.replace(/([^/])$/, "$1/")
				.replace(/^http:/, "https:")
				.replace(/(:\/\/)(instagram\.com\/)/, "$1www.$2")
				.replace(/(:\/\/.*?)\/\/+/g, "$1/");

			var cache_key = "instagram_sharedData_query:" + url;
			api_cache.fetch(cache_key, cb, function (done) {
				do_request({
					method: "GET",
					url: url,
					onload: function (result) {
						if (result.readyState !== 4)
							return;

						try {
							var text = result.responseText;

							var regex1 = /window\._sharedData = *(.*?);?<\/script>/;
							var regex2 = /window\._sharedData *= *(.*?}) *;[\s]*window\.__initialDataLoaded/;

							var match = text.match(regex1);
							if (!match) {
								match = text.match(regex2);
							}

							var parsed = JSON_parse(match[1]);

							var regex3 = /window\.__additionalDataLoaded\(["'].*?["']\s*,\s*({.*?})\);?\s*<\/script>/;
							match = text.match(regex3);
							if (match) {
								var parsed1 = JSON_parse(match[1]);
								for (var key in parsed.entry_data) {
									if (is_array(parsed.entry_data[key])) {
										parsed.entry_data[key][0] = overlay_object(parsed.entry_data[key][0], parsed1);
									}
								}
							}

							// TODO: maybe check if parsed is correct before setting to cache?
							done(parsed, 60 * 60);
						} catch (e) {
							console_log("instagram_sharedData", result);
							console_error("instagram_sharedData", e);
							done(null, false);
						}
					}
				});
			});
		};

		var uid_from_sharedData = function(json) {
			if (json.id)
				return json.id;
			else {
				var entrydata = json.entry_data;

				if (entrydata.ProfilePage)
					return entrydata.ProfilePage[0].graphql.user.id;
				else
					return entrydata.PostPage[0].graphql.shortcode_media.owner.id;
			}
		};

		var username_to_uid = function(username, cb) {
			if (username.match(/^http/)) {
				username = username.replace(/^[a-z]+:\/\/[^/]*\/+(?:stories\/+)?([^/]*)(?:\/.*)?(?:[?#].*)?$/, "$1");
			}

			var cache_key = "instagram_username_uid:" + username;
			api_cache.fetch(cache_key, cb, function (done) {
				query_ig("https://www.instagram.com/" + username + "/", function (json) {
					try {
						done(uid_from_sharedData(json), 5*60);
					} catch (e) {
						console_error(cache_key, e);
						done(null, false);
					}
				});
			});
		};

		var uid_to_profile = function(uid, cb) {
			var cache_key = "instagram_uid_to_profile:" + uid;
			api_cache.fetch(cache_key, cb, function (done) {
				var url = "https://i.instagram.com/api/v1/users/" + uid + "/info/";
				app_api_call(url, function (result) {
					if (result.readyState !== 4)
						return;

					try {
						var parsed = JSON_parse(result.responseText).user;

						// 5 minutes since they can change their profile pic often
						done(parsed, 5 * 60);
					} catch (e) {
						console_log("instagram_uid_to_profile", result);
						console_error("instagram_uid_to_profile", e);
						done(null, false);
					}
				});
			});
		};

		var get_instagram_cookies = function(cb) {
			// For now, we'll disable this as it doesn't appear to be needed
			if (true) {
				cb(null);
			} else {
				var cookie_cache_key = "instagram";
				if (cookie_cache.has(cookie_cache_key)) {
					return cb(cookie_cache.get(cookie_cache_key));
				}

				if (options.get_cookies) {
					options.get_cookies("https://www.instagram.com/", function(cookies) {
						cookie_cache.set(cookie_cache_key, cookies);
						cb(cookies);
					});
				} else {
					cb(null);
				}
			}
		};

		var app_api_call = function (url, cb) {
			var headers = {
				//"User-Agent": "Instagram 10.26.0 (iPhone7,2; iOS 10_1_1; en_US; en-US; scale=2.00; gamut=normal; 750x1334) AppleWebKit/420+",
				"User-Agent": "Instagram 10.26.0 Android (23/6.0.1; 640dpi; 1440x2560; samsung; SM-G930F; herolte; samsungexynos8890; en_US)",
				"X-IG-Capabilities": "36oD",
				"Accept": "*/*",
				"Accept-Language": "en-US,en;q=0.8"
			};

			get_instagram_cookies(function(cookies) {
				if (cookies) {
					headers.Cookie = cookies_to_httpheader(cookies);
				}

				if (!use_app_api)
					headers.Cookie = "";

				do_request({
					method: "GET",
					url: url,
					headers: headers,
					onload: cb
				});
			});
		};

		var mediainfo_api = function(id, cb) {
			if (!use_app_api)
				return cb(null);

			var cache_key = "instagram_mediainfo:" + id;
			api_cache.fetch(cache_key, cb, function (done) {
				var url = "https://i.instagram.com/api/v1/media/" + id + "/info/";
				app_api_call(url, function (result) {
					if (result.readyState !== 4)
						return;

					if (result.status === 200) {
						var parsed = null;

						try {
							parsed = JSON_parse(result.responseText);
						} catch (e) {
							console_log("instagram_mediainfo", result);
							console_error("instagram_mediainfo", e);
						}

						if (parsed) {
							return done(parsed, 60 * 60);
						}
					} else {
						console_error(cache_key, result);
					}

					done(null, false);
				});
			});
		};

		var get_all_stories_api = function(uid, cb) {
			if (!use_app_api)
				return cb(null);

			var story_cache_key = "instagram_story_uid:" + uid;
			api_cache.fetch(story_cache_key, cb, function (done) {
				var url = "https://i.instagram.com/api/v1/feed/user/" + uid + "/reel_media/";
				app_api_call(url, function(result) {
					if (result.readyState !== 4)
						return;

					if (result.status !== 200) {
						console_log(story_cache_key, result);
						return done(null, false);
					}

					try {
						var parsed = JSON_parse(result.responseText);

						return done(parsed, 10*60);
					} catch(e) {
						console_log(story_cache_key, result);
						console_error(story_cache_key, e);
					}

					return done(null, false);
				});
			});
		};

		var story_api = function(picid, uid, cb) {
			var get_stories = function(cb) {
				get_all_stories_api(uid, function(result) {
					if (!result) {
						return cb(null);
					}

					try {
						var items = result.items;
						var all_images = [];
						var our_item = null;

						for (var i = 0; i < items.length; i++) {
							var item = items[i];

							var images = get_maxsize_app(item);
							if (images.length < 1) {
								console_warn("No images found for", item);
								continue;
							}
							var image = images[0];
							all_images.push(image);

							var item_picid = common_functions.instagram_get_imageid(image.src);
							api_cache.set("instagram_story_pic:" + item_picid, image, 6*60*60);

							if (image.video) {
								var item_vidid = common_functions.instagram_get_imageid(image.video);
								api_cache.set("instagram_story_pic:" + item_vidid, image, 6*60*60);
							}

							if (picid && (item_picid === picid || item_vidid === picid)) {
								our_item = image;
							}
						}

						if (picid) {
							if (our_item !== null)
								return cb(our_item);
						} else {
							return cb(all_images);
						}

						return cb(null);
					} catch (e) {
						console_log(cache_key, result);
						console_error(cache_key, e);
						return cb(null);
					}
				});
			};

			if (picid) {
				var cache_key = "instagram_story_pic:" + picid;
				api_cache.fetch(cache_key, cb, function (done) {
					get_stories(function(result) {
						if (result) {
							return done(result, 6*60*60);
						} else {
							return done(result, false);
						}
					});
				});
			} else {
				get_stories(cb);
			}
		};

		var profile_to_url = function(profile) {
			try {
				// Try using the app's API
				return profile.hd_profile_pic_url_info.url;
			} catch (e) {
				// Try using the normal browser json
				try {
					return profile.profile_pic_url;
				} catch (e) {
					console_error(e, profile);
					return null;
				}
			}
		};

		var request_profile = function(username, cb) {
			username_to_uid(username, function(uid) {
				if (!uid) {
					return cb(null);
				}

				uid_to_profile(uid, function(profile) {
					if (!profile) {
						return cb(null);
					}

					return cb(profile_to_url(profile));
				});
			});

			return {
				waiting: true
			};
		};

		var parse_caption = function(caption) {
			if (typeof caption === "string")
				return caption;

			if (caption.text)
				return caption.text;

			if (caption.edges && caption.edges.length > 0)
				return caption.edges[0].node.text;

			return null;
		};

		var get_caption = function(item) {
			if (item.caption)
				return parse_caption(item.caption);

			if (item.title)
				return parse_caption(item.title);

			if (item.edge_media_to_caption)
				return parse_caption(item.edge_media_to_caption);

			return undefined;
		};

		var image_in_objarr = function(image, objarr) {
			var imageid = common_functions.instagram_get_imageid(image);

			for (var i = 0; i < objarr.length; i++) {
				if (string_indexof(objarr[i].src, imageid) > 0)
					return objarr[i];
			}

			return null;
		};

		var get_maxsize_app = function(item) {
			var images = [];

			var parse_image = function (img) {
				var candidates = img.image_versions2.candidates;
				var maxsize = 0;
				var maxobj = null;
				for (var i = 0; i < candidates.length; i++) {
					var size = candidates[i].width * candidates[i].height;
					if (size > maxsize) {
						maxsize = size;
						maxobj = candidates[i];
					}
				}

				var image = null;
				if (maxobj !== null) {
					image = {
						src: maxobj.url,
						caption: get_caption(item),
						width: maxobj.width,
						height: maxobj.height
					};
				}

				if (image && img.video_versions) {
					maxsize = 0;
					maxobj = null;
					var videos = img.video_versions;

					for (var i = 0; i < videos.length; i++) {
						var size = videos[i].width * videos[i].height;
						if (size > maxsize) {
							maxsize = size;
							maxobj = videos[i];
						}
					}

					if (maxobj !== null) {
						image.video = maxobj.url;
						image.width = maxobj.width;
						image.height = maxobj.height;
					}
				}

				if (image !== null) {
					images.push(image);
				}
			};

			if ("carousel_media" in item) {
				for (var i = 0; i < item.carousel_media.length; i++) {
					parse_image(item.carousel_media[i]);
				}
			} else {
				parse_image(item);
			}

			return images;
		};

		var get_maxsize_graphql = function(media) {
			var images = [];

			var parse_image = function(node) {
				var image = node.display_src;
				if (!image)
					image = node.display_url;

				var width = 0, height = 0;
				if (node.dimensions) {
					width = node.dimensions.width;
					height = node.dimensions.height;
				}

				if (!image)
					return;

				var found_image = image_in_objarr(image, images);
				if (found_image) {
					var found_size = found_image.width * found_image.height;
					var our_size = width * height;

					// fixme: why is this check even here? it breaks width=0, height=0 videos
					if (our_size <= found_size || true)
						return;
				}

				if (node.video_url) {
					// width/height corresponds to the image, not the video
					width = 0;
					height = 0;
				}

				images.push({
					src: image,
					video: node.video_url,
					caption: get_caption(media),
					width: width,
					height: height
				});
			};

			if (media.edge_sidecar_to_children) {
				var edges = media.edge_sidecar_to_children.edges;
				for (var i = 0; i < edges.length; i++) {
					var edge = edges[i];
					if (edge.node)
						edge = edge.node;

					parse_image(edge);
				}
			}

			parse_image(media);

			return images;
		};

		var image_to_obj = function(image) {
			var extra = null;
			if (image.caption)
				extra = {caption: image.caption};

			if (image.video) {
				return [
					{url: image.video, video: true, extra: extra},
					{url: image.src, extra: extra}
				];
			} else {
				return {
					url: image.src,
					extra: extra
				};
			}
		};

		var request_post_inner = function(post_url, image_url, cb) {
			query_ig(post_url, function(json) {
				if (!json) {
					return cb(null);
				}

				try {
					var media = json.entry_data.PostPage[0].graphql.shortcode_media;

					mediainfo_api(media.id + "_" + media.owner.id, function(app_response) {
						var images = [];

						if (app_response !== null) {
							images = get_maxsize_app(app_response.items[0]);
						} else if (use_app_api) {
							console_log("Unable to use API to find Instagram image, you may need to login to Instagram");
						}

						var images_graphql = get_maxsize_graphql(media);

						if (images && images.length === images_graphql.length) {
							for (var i = 0; i < images.length; i++) {
								var app_size = images[i].width * images[i].height;
								var graphql_size = images_graphql[i].width * images_graphql[i].height;

								if (graphql_size > app_size) {
									//console_log("Using graphql image", images[i], images_graphql[i]);
									images[i] = images_graphql[i];
								}
							}
						} else {
							images = images_graphql;
						}

						if (image_url) {
							var image = image_in_objarr(image_url, images);
							if (image)
								return cb(image_to_obj(image));
						} else {
							return cb(images);
						}

						cb(null);
					});
				} catch (e) {
					console_error(e);
					cb(null);
				}
			});

			return {
				waiting: true
			};
		};

		var request_post = function(post_url, image_url, cb) {
			return request_post_inner(post_url, image_url, cb);
		};

		var request_stories = function(url, image_url, cb, all) {
			var username = url.replace(/.*\/stories\/+([^/]*).*$/, "$1");
			if (username === url)
				return null;

			username_to_uid(username, function(uid) {
				if (!uid) {
					return cb(null);
				}

				var image_id = common_functions.instagram_get_imageid(image_url);
				story_api(image_id, uid, function(result) {
					if (!result) {
						return cb(null);
					}

					var images = result;
					if (!is_array(images))
						images = [images];

					if (image_id) {
						var image = image_in_objarr(image_url, images);
						if (!image) {
							console_warn("Unable to find", image_url, "in", images);
							return cb(null);
						}

						return cb(image_to_obj(image));
					} else {
						return cb(images);
					}
				});
			});

			return {
				waiting: true
			};
		};

		var parse_single_el_info = function(info, cb) {
			var retval;

			if (info.type === "post") {
				if (info.all)
					info.image = null;

				retval = request_post(info.url, info.image, cb);
				if (retval)
					return retval;
			} else if (info.type === "profile") {
				retval = request_profile(info.url, cb);
				if (retval)
					return retval;
			} else if (info.type === "story") {
				if (info.all)
					info.image = null;

				retval = request_stories(info.url, info.image, cb);
				if (retval)
					return retval;
			}

			return retval;
		};

		var parse_el_info = function(info, cb) {
			var retval;

			for (var i = 0; i < info.length; i++) {
				retval = parse_single_el_info(info[i], cb);
				if (retval)
					return retval;
			}

			return retval;
		};

		return parse_el_info(info, cb);
	};

	common_functions.instagram_get_el_for_imageid = function(element) {
		var newel = element;
		if (newel.tagName === "SOURCE") {
			newel = newel.parentElement;
		}

		if (newel.tagName === "VIDEO" && newel.parentElement) {
			try {
				newel = newel.parentElement.querySelectorAll("img[srcset]");
				if (!newel || newel.length === 0)
					newel = element;
				else
					newel = newel[0];
			} catch (e) {
				console_error(e);
				newel = element;
			}
		} else {
			newel = element;
		}

		return newel;
	};

	common_functions.instagram_get_image_src_from_el = function(el) {
		if (el.tagName === "VIDEO") {
			// use the poster instead as the larger video urls differ
			return el.poster || el.src;
		}

		return el.src;
	}

	common_functions.instagram_find_el_info = function(document, element, host_url) {
		var possible_infos = [];

		var element_src = common_functions.instagram_get_image_src_from_el(element);

		// check for links first
		var current = element;
		while ((current = current.parentElement)) {
			if (current.tagName !== "A")
				continue;

			if (current.href.match(/:\/\/[^/]+\/+(?:[^/]+\/+)?p\//)) {
				// link to post
				possible_infos.push({
					type: "post",
					subtype: "link",
					url: current.href,
					image: element_src,
					element: current
				});
			} else if (current.href.match(/:\/\/[^/]+\/+[^/]+(?:\/+(?:[?#].*)?)?$/)) {
				// link to profile (e.g. for someone who comments on a post)
				possible_infos.push({
					type: "profile",
					subtype: "link",
					url: current.href,
					element: current
				});
			}
		}

		current = element;
		while ((current = current.parentElement)) {
			// profile image
			// a better way would be to check the username from the h2 > a (title, href, innerText)
			if (current.tagName === "HEADER") {
				var sharedData = null;

				// Still keep this code because this way we can know it exists?
				// We can't use this directly because the user might have switched the profile they're currently viewing
				if (true) {
					var scripts = document.getElementsByTagName("script");
					for (var i = 0; i < scripts.length; i++) {
						if (scripts[i].innerText.match(/^ *window\._sharedData/)) {
							sharedData = scripts[i].innerText.replace(/^ *window\._sharedData *= *({.*}) *;.*?/, "$1");
						}
					}

					if (!sharedData) {
						console_error("Shared data not found");
						continue;
					} else {
						sharedData = JSON_parse(sharedData);
					}
				}

				var url = host_url;
				if (url.match(/:\/\/[^/]+\/+p\//)) {
					var username;

					if (element.parentElement.tagName === "SPAN" && element.parentElement.getAttribute("role") === "link") {
						var as = current.getElementsByTagName("a");
						for (var i = 0; i < as.length; i++) {
							var a = as[i];
							var amatch = a.href.match(/^[a-z]+:\/\/[^/]+\/+([^/]+)\/*(?:[?#].*)?$/);
							if (amatch) {
								if (strip_whitespace(a.innerText).toLowerCase() === amatch[1].toLowerCase()) {
									username = strip_whitespace(a.innerText).toLowerCase();
									break;
								}
							}
						}
					}

					if (!username) {
						try {
							var h2 = current.querySelector("h2 > a");
							if (strip_whitespace(h2.innerText) === strip_whitespace(h2.title)) {
								username = h2.innerText;
							}
						} catch (e) {}
					}

					if (!username) {
						try {
							// There are 2 h1's, the first should be the username (the second is the person's "name")
							username = current.querySelector("section h1").innerText;
						} catch (e) {
							console_error(e);
						}
					}

					if (username) {
						url = "https://www.instagram.com/" + strip_whitespace(username);//common_functions.instagram_username_from_sharedData(sharedData);
					} else {
						url = null;
					}
				}

				if (url) {
					possible_infos.push({
						type: "profile",
						subtype: "page",
						url: url,
						element: current
					});
				}
			}

			// popup
			if ((current.tagName === "DIV" && current.getAttribute("role") === "dialog") ||
				// post page
				(current.tagName === "BODY" && host_url.match(/:\/\/[^/]*\/+(?:[^/]+\/+)?p\//))) {
				possible_infos.push({
					type: "post",
					subtype: current.tagName === "BODY" ? "page" : "popup",
					url: host_url,
					image: element_src,
					element: current
				});
			}

			// home
			if (current.tagName === "ARTICLE" && host_url.match(/:\/\/[^/]+\/+(?:[?#].*)?$/)) {
				var timeel = current.querySelector("a > time");
				if (timeel) {
					var href = timeel.parentElement.href;
					if (/:\/\/[^/]*\/+(?:[^/]+\/+)?p\//.test(href)) {
						possible_infos.push({
							type: "post",
							subtype: "home",
							url: href,
							image: element_src,
							element: current
						});
					}
				}
			}

			// stories
			// https://www.instagram.com/stories/hollyearl__/2271839116690161119/
			if (current.tagName === "BODY" && host_url.match(/:\/\/[^/]*\/+stories\/+([^/]*)(?:\/+[0-9]+)?\/*(?:[?#].*)?$/)) {
				// try to find image instead of video because video ids change for app stories
				var newel = common_functions.instagram_get_el_for_imageid(element);

				possible_infos.push({
					type: "story",
					url: host_url,
					image: newel.src,
					element: current
				});
			}
		}

		return possible_infos;
	};

	common_functions.get_twitter_caption = function(el) {
		var currentel = el;
		while ((currentel = currentel.parentElement)) {
			if (currentel.tagName === "ARTICLE") {
				var captiondiv = currentel.querySelectorAll("div[lang]");
				if (captiondiv && captiondiv.length === 1) {
					return captiondiv[0].innerText;
				}

				break;
			}
		}

		return null;
	};

	common_functions.get_twitter_video_tweet = function(el, window) {
		if (el.tagName !== "VIDEO" || !el.src.match(/^blob:/))
			return null;

		var poster = el.poster;
		if (!poster)
			return null;

		// note that the numbers here corresponds to the media id, not the tweet id, so it can't be used
		if (!/\/ext_tw_video_thumb\/+[0-9]+\/+pu\/+img\//.test(poster))
			return null;

		var href = window.location.href;

		// embedded video
		var match = href.match(/\/i\/+videos\/+tweet\/+([0-9]+)(?:[?#].*)?$/);
		if (match) {
			return {
				id: match[1]
			};
		}

		var currentel = el;
		while ((currentel = currentel.parentElement)) {
			if (currentel.tagName === "ARTICLE") {
				var our_as = currentel.querySelectorAll("a[role='link']");
				for (var i = 0; i < our_as.length; i++) {
					var our_href = our_as[i].href;
					if (!our_href)
						continue;

					var match = our_href.match(/\/status\/+([0-9]+)(?:\/+(?:retweets|likes)|\/*)(?:[?#].*)?$/);
					if (match) {
						return {
							id: match[1]
						};
					}
				}
				break;
			}
		}

		return null;
	};

	common_functions.get_snapchat_story = function(api_cache, do_request, username, cb) {
		var cache_key = "snapchat_story:" + username;
		api_cache.fetch(cache_key, cb, function(done) {
			do_request({
				method: "GET",
				url: "https://search.snapchat.com/lookupStory?id=" + username,
				headers: {
					"sec-fetch-dest": "empty",
					"sec-fetch-mode": "cors",
					"sec-fetch-site": "same-site",
					"Origin": "https://www.snapchat.com",
					"Referer": "https://www.snapchat.com/add/" + username // maybe use real url instead?
				},
				onload: function(resp) {
					if (resp.readyState !== 4)
						return;

					if (resp.status !== 200) {
						console_error(resp);
						return done(null, false);
					}

					try {
						var json = JSON_parse(resp.responseText);
						return done(json, 60); // story can change, so 60 seconds? or less?
					} catch (e) {
						console_error(e, resp);
						return done(null, false);
					}
				}
			});
		});
	};

	common_functions.snap_to_obj = function(snap) {
		var caption = null;

		// Apparently this isn't related to the caption?
		if (false) {
			caption = snap.snapTitle + snap.snapSubtitles;
			caption = caption.replace(/^\s*([\s\S]*)\s*$/, "$1");
		}

		return {
			url: snap.snapUrls.mediaUrl,
			extra: {
				caption: caption || null
			},
			need_blob: true
		};
	};

	common_functions.get_snapchat_info_from_el = function(el) {
		if (el.getAttribute("data-imu")) {
			return {
				username: el.getAttribute("data-username"),
				pos: parseInt(el.getAttribute("data-pos")),
				url: el.getAttribute("data-url")
			};
		}

		if (el.tagName !== "VIDEO" && el.tagName !== "IMG")
			return null;

		var current = el;
		while ((current = current.parentElement)) {
			if (current.tagName === "DIV" && current.getAttribute("role") === "presentation") {
				var username = "";

				var as = current.getElementsByTagName("a");
				for (var i = 0; i < as.length; i++) {
					var match = as[i].href.match(/^[a-z]+:\/\/story\.snapchat\.com\/+s\/+([^/?#]+)(?:[?#].*)?$/);
					if (match) {
						username = match[1];
						break;
					}
				}

				var pos = 0;

				var prevbutton = current.querySelector("#PrevButton");
				if (prevbutton) {
					pos = 1;
				}

				var nextbutton = current.querySelector("#NextButton");
				if (!nextbutton) {
					pos = -1;
				}

				return {
					username: username,
					pos: pos
				};
			}
		}

		return null;
	};

	common_functions.get_obj_from_snap_info = function(api_cache, do_request, info, cb) {
		if (!info) {
			return cb(null);
		}

		common_functions.get_snapchat_story(api_cache, do_request, info.username, function(data) {
			if (!data) {
				return cb(null);
			}

			if (info.pos === -1) {
				info.pos = data.snapList.length - 1;
			}

			return cb(common_functions.snap_to_obj(data.snapList[info.pos]));
		});
	};

	common_functions.get_tiktok_urlvidid = function(url) {
		var match = url.match(/^[a-z]+:\/\/[^/]+\/+(?:[0-9a-f]{32}\/+[0-9a-f]{8}\/+)?video\/+[^/]+\/+[^/]+\/+[^/]+\/+([0-9a-f]{32})\/*\?/);
		if (match)
			return match[1];

		return null;
	};

	common_functions.set_tiktok_vid_filename = function(obj) {
		var vidid = common_functions.get_tiktok_urlvidid(obj.url);
		if (vidid) {
			obj.filename = vidid + ".mp4"; // hack, but needed because firefox etc. doesn't automatically set the extension, because content-type isn't set
			return true;
		}

		return false;
	};

	common_functions.get_best_tiktok_url = function(api_cache, do_request, src, cb) {
		var get_tiktok_vidid_key = function(url) {
			var urlvidid = common_functions.get_tiktok_urlvidid(url);
			if (!urlvidid) {
				console_warn("Unknown video URL:", url);
				return null;
			}

			return "tiktok_vidid:" + urlvidid;
		};

		var query_tiktok_vidid = function(url, cb) {
			var cache_key = get_tiktok_vidid_key(url);
			if (!cache_key)
				return cb(null);

			api_cache.fetch(cache_key, cb, function(done) {
				var request_handle;
				var request_aborted = false;

				var progress_cb = function(resp) {
					if (request_aborted)
						return;

					if (!resp.responseText) {
						if (resp.readyState !== 4)
							return;

						// Tampermonkey has a bug with onprogress: https://github.com/Tampermonkey/tampermonkey/issues/906
						if (resp.loaded && resp.total && resp.status === 200)
							return;

						request_handle.abort();
						request_aborted = true;

						console_error(resp);
						return done(null, false);
					}

					var match = resp.responseText.match(/mdtacomment[\s\S]{10,400}vid:([0-9a-z]{32})/);
					// after that, it stores 0, 0, 0, 37, with 37 being the length of vid:..., is this related?
					if (!match) {
						if (resp.readyState !== 4)
							return;

						if (resp.readyState === 4) {
							request_handle.abort();
							request_aborted = true;

							console_warn("Unable to find video ID for", url);
							done(null, false);
						}
					} else {
						request_handle.abort();
						request_aborted = true;

						done(match[1], 24*60*60);
					}
				};

				request_handle = do_request({
					url: url,
					method: "GET",
					headers: {
						Referer: "https://www.tiktok.com/"
					},
					onprogress: progress_cb,
					onload: progress_cb
				});
			});
		};

		var get_nowatermark_for_vidid = function(vidid, cb) {
			var cache_key = "tiktok_watermarkfree:" + vidid;
			api_cache.fetch(cache_key, cb, function(done) {
				do_request({
					url: "https://api2.musical.ly/aweme/v1/playwm/?video_id=" + vidid,
					headers: {
						Referer: ""
					},
					method: "HEAD",
					onload: function(resp) {
						if (resp.readyState !== 4)
							return;

						// https://www.tiktok.com/@auliicravalho/video/6813323310521224454 - returns 302
						if (resp.status !== 200 && resp.status !== 302) {
							console_error(resp);
							return done(null, false);
						}

						return done(force_https(resp.finalUrl), 60*60);
					}
				});
			});
		};

		var get_newurl_if_changed = function(newsrc, src) {
			var old_urlvidid = common_functions.get_tiktok_urlvidid(src);

			if (!newsrc)
				newsrc = src;
			var new_urlvidid = common_functions.get_tiktok_urlvidid(newsrc);

			// to avoid infinite redirects
			if (new_urlvidid === old_urlvidid) {
				newsrc = src;
			}

			return newsrc;
		};

		query_tiktok_vidid(src, function(vidid) {
			if (!vidid) {
				return cb(null);
			}

			get_nowatermark_for_vidid(vidid, function(newsrc) {
				cb(get_newurl_if_changed(newsrc, src));
			});
		});
	};

	var get_domain_nosub = function(domain) {
		var domain_nosub = domain.replace(/^.*\.([^.]*\.[^.]*)$/, "$1");
		if (domain_nosub.match(/^co\.[a-z]{2}$/) ||
			domain_nosub.match(/^ne\.jp$/) || // stream.ne.jp
			domain_nosub.match(/^or\.jp$/) ||
			domain_nosub.match(/^com\.[a-z]{2}$/) ||
			domain_nosub.match(/^org\.[a-z]{2}$/) ||
			domain_nosub.match(/^net\.[a-z]{2}$/)) {
			domain_nosub = domain.replace(/^.*\.([^.]*\.[^.]*\.[^.]*)$/, "$1");
		}

		return domain_nosub;
	};

	function bigimage(src, options) {
		if (!src)
			return src;

		if (!src.match(/^(?:https?|x-raw-image):\/\//) && !src.match(/^(?:data|blob):/))
			return src;

		var origsrc = src;

		var url = urlparse(src);

		var protocol;
		var domain;
		var port;

		if (!src.match(/^(?:data|x-raw-image|blob):/)) {
			// to prevent infinite loops
			if (src.length >= 65535)
				return src;

			var protocol_split = src.split("://");
			protocol = protocol_split[0];
			var splitted = protocol_split[1].split("/");
			domain = splitted[0];

			port = domain.replace(/.*:([0-9]+)$/, "$1");
			if (port === domain)
				port = "";
			domain = domain.replace(/(.*):[0-9]+$/, "$1");
		} else {
			//protocol = src.replace(/^([-a-z]+):.*/, "$1");
			protocol = "data";
			domain = "";
			src = ""; // FIXME: this isn't great
		}

		var domain_nowww = domain.replace(/^www\./, "");
		var domain_nosub = get_domain_nosub(domain);

		var amazon_container = null;
		if (string_indexof(domain, ".amazonaws.com") >= 0) {
			// https://timely-api-public.s3.us-west-2.amazonaws.com/116743_phpg7Ixww_small.JPG
			// https://pixls-discuss.s3.dualstack.us-east-1.amazonaws.com/optimized/3X/0/5/0530ef1424b7bace746cd10d2bc26b5cd58d5e27_2_690x388.png
			// http://stezor-img-res.s3-website.eu-central-1.amazonaws.com/600x0/0eefab17-aa8b-4f8e-9017-7b9442d1ee9e
			if (domain.match(/^s3(?:-website)?(?:\.dualstack)?(?:[-.][-a-z0-9]+)?\.amazonaws\.com/))
				amazon_container = src.replace(/^[a-z]*:\/\/[^/]*\/([^/]*)\/.*/, "$1");
			else if (domain.match(/[^/]*\.s3(?:-website)?(?:\.dualstack)?(?:[-.][-a-z0-9]+)?\.amazonaws\.com/))
				amazon_container = src.replace(/^[a-z]*:\/\/([^/]*)\.s3(?:-website)?(?:\.dualstack)?(?:[-.][-a-z0-9]+)?\.amazonaws\.com\/.*/, "$1");
		}

		var googleapis_container = null;
		if (string_indexof(domain, ".googleapis.com") >= 0) {
			googleapis_container = src.replace(/^[a-z]*:\/\/[^/]*\/([^/]*)\/.*/, "$1");
		}

		var googlestorage_container = null;
		if (domain_nosub === "googleapis.com" &&
			(domain === "storage.googleapis.com" ||
			 domain.match(/\.storage\.googleapis\.com$/) ||
			 domain === "commondatastorage.googleapis.com" ||
			 domain.match(/\.commondatastorage\.googleapis\.com$/))) {
			if (domain.match(/\.[a-z]+\.googleapis\.com$/)) {
				googlestorage_container = src.replace(/^[a-z]*:\/\/([^/]*)\.[a-z]+\.googleapis\.com\/.*/, "$1");
			} else {
				googlestorage_container = src.replace(/^[a-z]*:\/\/[^/]*\/([^/]*)\/.*/, "$1");
			}
		}

		var host_domain = "";
		var host_domain_nowww = "";
		var host_domain_nosub = "";
		if (options.host_url) {
			host_domain = options.host_url.replace(/^[a-z]+:\/\/([^/]*?)(?::[0-9]+)?(?:\/.*)?$/,"$1");

			host_domain_nowww = host_domain.replace(/^www\./, "");
			host_domain_nosub = host_domain.replace(/^.*\.([^.]*\.[^.]*)$/, "$1");
			if (host_domain_nosub.match(/^co\.[a-z]{2}$/)) {
				host_domain_nosub = host_domain.replace(/^.*\.([^.]*\.[^.]*\.[^.]*)$/, "$1");
			}
		}

		try {
			if (options.document)
				document = options.document;
			if (options.window)
				window = options.window;
		} catch (e) {
			//console_warn("Failed to set document/window", e);
		}

		var problem_excluded = function(problem) {
			if (!options.exclude_problems)
				return false;

			if (typeof options.exclude_problems === "string" && options.exclude_problems === problem)
				return true;

			if (!is_array(options.exclude_problems))
				return false;

			return array_indexof(options.exclude_problems, problem) >= 0;
		};

		var api_cache = real_api_cache;
		if (options.use_api_cache === false) {
			api_cache = new Cache();
		}

		var newsrc, i, id, size, origsize, regex, match;

		// instart logic morpheus
		// test urls:
		// char - 5
		// https://c-6rtwjumjzx7877x24nrlncx2ewfspjwx2ehtr.g00.ranker.com/g00/3_c-6bbb.wfspjw.htr_/c-6RTWJUMJZX77x24myyux78x3ax2fx2fnrlnc.wfspjw.htrx2fzx78jw_stij_nrlx2f94x2f415044x2ftwnlnsfqx2fjzs-on-bts-wjhtwinsl-fwynx78yx78-fsi-lwtzux78-umtyt-z6x3fbx3d105x26vx3d05x26krx3doulx26knyx3dhwtux26hwtux3dkfhjx78x26n65h.rfwp.nrflj.yduj_$/$/$/$/$/$
		//   https://imgix.ranker.com/user_node_img/49/960599/original/eun-ji-won-recording-artists-and-groups-photo-u1?w=650&q=50&fm=jpg&fit=crop&crop=faces
		// https://c-6rtwjumjzx7877x24nrlncx2ewfspjwx2ehtr.g00.ranker.com/g00/3_c-6bbb.wfspjw.htr_/c-6RTWJUMJZX77x24myyux78x3ax2fx2fnrlnc.wfspjw.htrx2fzx78jw_stij_nrlx2f03x2f6698837x2ftwnlnsfqx2fmjj-hmzq-umtyt-z4x3fbx3d105x26vx3d05x26krx3doulx26knyx3dhwtux26hwtux3dkfhjx78x26n65h.rfwp.nrflj.yduj_$/$/$/$/$/$
		//   https://imgix.ranker.com/user_node_img/58/1143382/original/hee-chul-photo-u9?w=650&q=50&fm=jpg&fit=crop&crop=faces
		// https://c-6rtwjumjzx7877x24nrlncx2ewfspjwx2ehtr.g00.ranker.com/g00/3_c-6bbb.wfspjw.htr_/c-6RTWJUMJZX77x24myyux78x3ax2fx2fnrlnc.wfspjw.htrx2fzx78jw_stij_nrlx2f16x2f6757374x2ftwnlnsfqx2fmdzs-dtzsl-z7x3fbx3d105x26vx3d05x26krx3doulx26knyx3dhwtux26hwtux3dkfhjx78x26n65h.rfwp.nrflj.yduj_$/$/$/$/$/$
		//   https://imgix.ranker.com/user_node_img/61/1202829/original/hyun-young-u2?w=650&q=50&fm=jpg&fit=crop&crop=faces
		// https://c-7npsfqifvt0x24jnh-t-nto-dpnx2eblbnbjafex2eofu.g01.msn.com/g00/3_c-7x78x78x78.nto.dpn_/c-7NPSFQIFVT0x24iuuqtx3ax2fx2fjnh-t-nto-dpn.blbnbjafe.ofux2fufoboux2fbnqx2ffoujuzjex2fBBFFIbi.jnhx3fix3d2191x26x78x3d2031x26nx3d7x26rx3d71x26px3dgx26mx3dgx26yx3d665x26zx3d341x26j21d.nbslx3djnbhf_$/$/$/$/$
		//   https://img-s-msn-com.akamaized.net/tenant/amp/entityid/AAEEHah.img
		//
		// http://c-6rtwjumjzx7877x24bbbx2esfstanx78twx2ent.g00.tomshardware.com/g00/3_c-6bbb.ytrx78mfwibfwj.htr_/c-6RTWJUMJZX77x24myyux78x3ax2fx2fbbb.sfstanx78tw.ntx2fx40u6x2fHfhmjfgqjHXXx3fywfhpx26n65h.rfwp.qnsp.yduj_$/$/$
		//   https://www.nanovisor.io/@p1/CacheableCSS?track
		//
		// char - 8
		// https://c-5uwzmx78pmca09x24quoqfx2ezivsmzx2ekwu.g00.ranker.com/g00/3_c-5eee.zivsmz.kwu_/c-5UWZMXPMCA09x24pbbx78ax3ax2fx2fquoqf.zivsmz.kwux2fcamz_vwlm_quox2f25x2f716313x2fwzqoqvitx2fmuui-eibawv-x78mwx78tm-qv-bd-x78pwbw-c01x3fex3d903x26px3d903x26nqbx3dkzwx78x26kzwx78x3dnikmax26yx3d48x26nux3drx78ox26q98k.uizs.quiom.bgx78m_$/$/$/$/$/$
		//   https://imgix.ranker.com/user_node_img/47/938535/original/emma-watson-people-in-tv-photo-u23?w=125&h=125&fit=crop&crop=faces&q=60&fm=jpg
		//
		// http://c-6rtwjumjzx7877x24zlh-56x2ehfkjrtrx78yfynhx2ehtr.g00.cafemom.com/g00/3_c-6ymjx78ynw.hfkjrtr.htr_/c-6RTWJUMJZX77x24myyux78x3ax2fx2fzlh-56.hfkjrtrx78yfynh.htrx2fljsx2fhwtux2f705x2f695x2f25x2f7563x2f57x2f56x2f65x2f82x2fibx2futpmy32vnt27.uslx3fn65h.rfwpx3dnrflj_$/$/$/$/$/$/$/$/$/$/$/$/$
		//   https://ugc-01.cafemomstatic.com/gen/crop/250/140/70/2018/02/01/10/37/dw/pokht87qio72.png
		if (string_indexof(src, "/g00/") >= 0 && /\.g0[0-9]\./.test(domain)) {
			var str = "";
			//var i;

			// decode x[0-9][0-9] to \x[0-9][0-9]
			for (i = 0; i < src.length; i++) {
				if (src[i] == 'x') {
					var char = parseInt(src[i + 1] + src[i + 2], 16);
					str += String.fromCharCode(char);
					i += 2;
				} else {
					str += src[i];
				}
			}

			str = str.split("/").slice(5).join("/").split("$").slice(1).join("$");
			if (str && string_indexof(str, "://") < 10 && str[1] == str[2]) {
				var diff = mod(str.charCodeAt(0) - 'h'.charCodeAt(0), 26);

				// char - diff
				var str1 = "";
				for (i = 0; i < str.length; i++) {
					var code = str.charCodeAt(i);
					if (code > 47 && code < 58) {
						/* number */
						code = (mod((code - 48 - diff), 10) + 48);
					} else if (code > 64 && code < 91) {
						/* uppercase */
						code = (mod((code - 65 - diff),26) + 65);
					} else if (code > 96 && code < 123) {
						/* lowercase */
						code = (mod((code - 97 - diff),26) + 97);
					}
					str1 += String.fromCharCode(code);
				}

				var urlparts = str1;
				if (urlparts && string_indexof(urlparts, "http") === 0) {
					var $s = urlparts.replace(/.*?([$/]*)$/, "$1");
					if ($s !== urlparts && $s) {
						var count = $s.split("$").length - 1;
						if (count > 0) {
							// + 2 for http://
							var newurl = urlparts.split("/").slice(0, count + 2).join("/");

							// https://ugc-01.cafemomstatic.com/gen/crop/250/140/70/2018/02/01/10/37/dw/pokht87qio72.png?i10c.mark=image_$
							//newurl = newurl.split("&").slice(0,-1).join("&"); // remove &i10c.mark.link.type_...
							newurl = newurl.replace(/[?&]i10c\.mark[^/]*$/, "");

							if (newurl)
								return newurl;
						}
					}
				} else {
					console_log(urlparts);
				}
			}
		}

		// -- start bigimage --


		if (domain === "img.hankyung.com" ||
			domain === "img.tenasia.hankyung.com") {
			newsrc = src.replace(/(\/photo\/+[0-9]{6}\/+[0-9A-Z]{2}\.[0-9]+\.[0-9]+)-[0-9]*x[0-9]*(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
			if (newsrc !== src)
				return newsrc;

			if (src.match(/\.[0-9](\.[^/.]*)(?:[?#].*)?$/)) {
				newsrc = src.replace(/[?#].*$/, "");
				if (newsrc !== src)
					return newsrc;

				return [
					src.replace(/\.[0-9](\.[^/.]*)$/, ".4$1"),
					src.replace(/\.[0-9](\.[^/.]*)$/, ".1$1")
				];
			}
		}

		if (domain_nosub === "naver.net" ||
			domain_nosub === "pstatic.net") {
			if (string_indexof(domain, "gfmarket.") >= 0) {
				return src;
			}

			if (/\/img\.tvcast\/+pc\/+img\/+/.test(src)) {
				return {
					url: src,
					bad: "mask"
				};
			}

			if (domain.match(/tv[0-9]*\.search\.naver\.net/) ||
				domain.match(/tv[0-9]*\.pstatic\.net/)) {
				return src.replace(/.*\/thm\?.*?q=/, "");
			}

			if (src.match(/[?&]src=/)) {
				return decodeURIComponent(src.replace(/.*src=*([^&]*).*/, "$1")).replace(/^"*/, '').replace(/"$/, '');
			}






			if (domain.search(/^[-a-z0-9]*cafe[-a-z0-9]*\./) < 0 &&
				domain.search(/^img-pholar[-a-z0-9]*\./) < 0 &&
				domain.search(/^shopping-phinf[-a-z0-9]*\./) < 0 &&
				domain.search(/^dic.phinf.naver.net/) < 0 &&
				domain.search(/^musicmeta.phinf.naver.net/) < 0 && false)
				src = src.replace(/\?type=[^/]*$/, "?type=w1");
			else
				src = src.replace(/\?type=[^/]*$/, "");

			src = src.replace(/#[^/]*$/, "");

			if (domain.search(/^[-a-z0-9]*blog[-a-z0-9]*\./) < 0 &&
				domain.search(/^[-a-z0-9]*cafe[-a-z0-9]*\./) < 0 &&
				domain.search(/^[-a-z0-9]*news[-a-z0-9]*\./) < 0 &&
				domain.search(/^[-a-z0-9]*post[-a-z0-9]*\./) < 0 &&
				domain.search(/^[-a-z0-9]*v.phinf[-a-z0-9]*\./) < 0 &&
				domain.search(/^[-a-z0-9]*shopping.phinf[-a-z0-9]*\./) < 0 &&
				domain.search(/^[-a-z0-9]*musicmeta.phinf[-a-z0-9]*\./) < 0) {
				return src;
			}



			newsrc = src
				.replace(/postfiles[^/.]*\./, "blogfiles.")
				.replace(/m?blogthumb[^./]*/, "blogfiles")
				.replace(/blogfiles[^/.]*\./, "blogfiles.")
				.replace(/postfiles[^/.]*\./, "blogfiles.")

				.replace(/cafeptthumb[^./]*/, "cafefiles")

				.replace(/cafeskthumb[^./]*/, "cafefiles")
				.replace(/m?cafethumb[^./]*/, "cafefiles")
				.replace(/cafefiles[^/.]*\./, "cafefiles.")

				.replace(/mimgnews[^./]*/, "imgnews")

				.replace(/post\.phinf\./, "post-phinf.")
				.replace(/v\.phinf\./, "v-phinf.")
				.replace(/musicmeta\.phinf\./, "musicmeta-phinf.")
				.replace(/shopping\.phinf\./, "shopping-phinf.")
				.replace(/blogpfthumb\.phinf\./, "blogpfthumb-phinf.")


				.replace(/(:\/\/blogfiles\.)pstatic\.net\/+/, "$1naver.net/")
				.replace(/^https?(:\/\/[^/.]*(?:phinf|files)\.naver\.net\/)/, "http$1");
			if (newsrc !== src)
				return newsrc;

			return {
				url: src,
				headers: {
					Referer: ""
				},
				referer_ok: {
					same_domain_nosub: true
				}
			};
		}

		if ((domain_nosub === "daumcdn.net" ||
			 domain_nosub === "kakaocdn.net") &&
			string_indexof(src, "/thumb/") >= 0) {
			return decodeURIComponent(src.replace(/.*fname=([^&]*).*/, "$1"));
		}


		if (domain_nosub === "daumcdn.net" && /^t[0-9]*\./.test(domain)) {
			if (/\/cafe_image\/+fancafe\/+[0-9]+\/+fancafe-cheer-color-bg\./.test(src)) {
				return {
					url: src,
					bad: "mask"
				};
			}
		}

		if ((domain_nosub === "tistory.com" ||
			  domain_nosub === "daum.net") &&
			 (domain.match(/\.uf\.[a-z]+\.[a-z]+$/) ||
			  domain.match(/^cfs[0-9]*\./))) {
			return src
				.replace("/attach/", "/original/")
				.replace("/media/" , "/original/")
				.replace("/image/", "/original/")
				.replace(/\/[RTC][0-9]*x[0-9]*\//, "/original/");
		}

		if (domain_nosub === "daumcdn.net" &&
			domain.match(/^t[0-9]*\.daumcdn\.net/)) {
			return src.replace(/(\/cfile\/(?:tistory\/)?[0-9A-F]+)(?:\\?.*)$/, "$1?original");
		}

		if (domain_nosub === "daumcdn.net" &&
			domain.match(/^i[0-9]*\.media\./)) {
			return src.replace(/\/uf\/image\//, "/uf/original/");
		}

		if (domain_nosub === "daumcdn.net" &&
			domain.match(/^[mi][0-9]*\.daumcdn\.net/)) {

			return src.replace(/:\/\/[^/]*\/(cfile[0-9]+)\//, "://$1.uf.daum.net/");
		}

		if (domain_nosub === "daumcdn.net" && /^i[0-9]*\.tvpot\./.test(domain)) {
			return src.replace(/(\/thumb\.[a-z0-9]+)\.mini(?:[?#].*)?$/, "$1");
		}

		if (domain === "image.news1.kr") {
			newsrc = src.replace(/\/+dims\/.*/, "");
			if (newsrc !== src)
				return newsrc;

			newsrc = src
				.replace(/\/thumbnails\/(.*)\/thumb_[0-9]+x(?:[0-9]+)?(\.[^/.]*)$/, "/$1/original$2")
				.replace(/main_thumb\.jpg/, "original.jpg")
				.replace(/article.jpg/, "original.jpg")
				.replace(/no_water.jpg/, "original.jpg")
				.replace(/photo_sub_thumb.jpg/, "original.jpg")
				.replace(/section_top\.jpg/, "original.jpg")
				.replace(/high\.jpg/, "original.jpg");

			var filename = src.replace(/.*\/photos\/+[0-9]{4}\/+(?:[0-9]{1,2}\/+){2}([0-9]+)\/+.*$/, "$1");
			if (filename !== src) {
				return {
					url: newsrc,
					filename: filename,
					extra: {
						page: "http://news1.kr/photos/view/?" + filename
					}
				};
			}

			return newsrc;
		}

		if (string_indexof(domain, ".joins.com") >= 0) {
			newsrc = src.replace(/\.tn_[0-9]*\..*/, "");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "ir.joins.com" ||
			domain === "loki.zhibotie.net") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/.*?[?&]u=([^&]*).*$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nosub === "koreadaily.com" &&
			domain.match(/^thumb[0-9]*\./)) {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/tn\.dll.*?[?&]url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeURIComponent(newsrc);
		}

		if (domain === "uhd.img.topstarnews.net") {
			return src
				.replace("/file_attach_thumb/", "/file_attach/")
				.replace(/_[^/]*[0-9]*x[0-9]*_[^/]*(\.[^/]*)$/, "-org$1")
				.replace(/(-[0-9]*)(\.[^/]*)$/, "$1-org$2");
		}

		if (domain_nowww === "topstarnews.net" ||
			domain === "cdn.topstarnews.net" ||
			domain === "cds.topstarnews.net") {

			newsrc = src.replace(/:\/\/[^/]*\/(.*\/photo\/.*_[0-9]+)(\.[^/.]*)\/.*$/, "://www.topstarnews.net/$1_org$2");
			if (newsrc !== src)
				return newsrc;

			return src
				.replace(/_v[0-9]+(\.[^/.]*)$/, "_org$1")
				.replace(/(_[0-9]+)(\.[^/.]*)$/, "$1_org$2")
				.replace("/thumbnail/", "/photo/");
		}

		if (domain === "thumb.mt.co.kr" ||
			domain === "thumb.mtstarnews.com") {

			newsrc = src.replace(/\/+dims\/.*/, "");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/^[a-z]+:\/\/thumb\.([^/]*)\/+([^/]*\/+)?[0-9]{2}\/+([0-9]{4}\/+[0-9]{2}\/+[0-9]{8,}_[0-9]+\.)/, "http://image.$1/$2$3");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "menu.mt.co.kr" ||
			domain === "img.koreatimes.co.kr" ||
			domain_nosub === "myskcdn.com") {



			if (/\/images\/+bg_[a-z]+[0-9]+\.png(?:[?#].*)?$/.test(src))
				return {
					url: src,
					bad: "mask"
				};

			var obj = src.match(/\/thumb\/+(?:[0-9]+\/+){2,3}([0-9]+)\/[0-9]{10,}_/);
			if (obj && obj[1] !== "00") {
				var obj1_str = src.replace(/.*\/thumb\/+([0-9]+\/+[0-9]+\/+[0-9]+\/).*/, "$1").replace(/\//g, "");
				var obj1 = parseInt(obj1_str);
				var num = "00";
				if (obj1 < 20170526)
					num = "06";

				src = src.replace(/(\/thumb\/+(?:[0-9]+\/+){2,3})[0-9]+(\/+[0-9]{10,}_)/, "$1" + num + "$2");
			}

			return src
				.replace(/\/dims\/.*/, "");
		}

		if (domain === "tkfile.yes24.com") {
			return src.replace(/(\/upload2\/.*?)\/dims\/+.*/, "$1");
		}

		if (domain === "cdn.music-flo.com") {
			return src.replace(/(\/image\/.*?)(?:[?#].*)?$/, "$1");
		}

		if (domain === "static.discovery-expedition.com") {
			return src.replace(/(\/images\/.*)\/dims\/.*/, "$1");
		}

		if (domain === "img.enews24.cjenm.skcdn.com") {
			return src
				.replace(/\/News\/NewsThumbnail(\/[0-9]+\/)(?:[0-9]+_)?([0-9]+\.[^/.]*)$/, "/News/Contents$1$2")
				.replace(/(\/Photo\/Contents\/[0-9]+\/)[0-9]+_([0-9]+\.[^/.]*)$/, "$1$2");
		}

		if (domain_nosub === "stardailynews.co.kr" ||

			domain_nosub === "liveen.co.kr" ||
			domain_nosub === "ilyoseoul.co.kr" ||
			domain_nosub === "sportsq.co.kr" ||
			domain_nosub === "zenithnews.com" ||
			domain_nowww === "munhwanews.com" ||
			domain_nowww === "mhns.co.kr" ||
			domain_nowww === "ccdailynews.com" ||
			domain === "ph.kyeonggi.com" ||
			domain_nowww === "jemin.com" ||
			domain_nowww === "domin.co.kr" ||
			domain === "cdn.jejudomin.co.kr" ||
			domain === "ph.incheonilbo.com" ||
			domain_nowww === "hidomin.com" ||
			domain_nowww === "newsfreezone.co.kr" ||
			domain === "cdn.newsfreezone.co.kr" ||
			domain_nowww === "newsinside.kr" ||
			domain === "cdn.newsinside.kr" ||
			domain_nowww === "greenpostkorea.co.kr" ||
			domain_nowww === "egn.kr" ||
			domain_nowww === "whitepaper.co.kr" ||
			domain_nowww === "outdoornews.co.kr" ||
			domain_nowww === "shinailbo.co.kr" ||
			domain_nowww === "ngtv.tv" ||
			domain_nowww === "rnx.kr" ||
			domain_nowww === "intronews.net" ||
			domain_nowww === "hg-times.com" ||
			domain_nowww === "iemn.kr" ||
			domain_nosub === "newscj.com" ||
			domain_nowww === "ggilbo.com" ||
			domain_nowww === "bstoday.kr" ||
			domain === "interfootball.heraldcorp.com" ||
			domain_nowww === "ilyosisa.co.kr" ||
			domain_nowww === "ynnews.kr" ||
			domain_nowww === "starilbo.com" ||
			domain_nowww === "autoherald.co.kr" ||
			domain_nowww === "00news.co.kr" ||
			domain_nowww === "kstarfashion.com" ||
			domain_nowww === "inewspeople.co.kr" ||
			domain_nowww === "wonnews.co.kr" ||
			domain_nosub === "gukjenews.com" ||
			domain_nowww === "lunarglobalstar.com" ||
			domain_nowww === "sundayjournal.kr" ||
			domain_nowww === "stnsports.co.kr" ||
			domain_nowww === "thekpm.com" ||
			domain === "chunchu.yonsei.ac.kr" ||
			domain_nowww === "e2news.com" ||
			domain === "ph.spotvnews.co.kr" ||
			domain_nowww === "sisunnews.co.kr" ||
			domain_nowww === "kpinews.co.kr" ||
			domain_nowww === "4th.kr" ||
			domain === "cds.topdaily.kr" ||
			domain_nowww === "kntimes.co.kr" ||
			domain_nowww === "senmedia.kr" ||
			domain_nowww === "siminilbo.co.kr" ||
			domain_nowww === "sisafocus.co.kr" ||
			domain_nowww === "futurekorea.co.kr" ||
			domain_nowww === "dailygrid.net" ||
			domain_nowww === "datanet.co.kr" ||
			domain_nowww === "slist.kr" ||
			domain_nowww === "secondmirror.co.kr" ||
			domain_nowww === "ntoday.co.kr" ||
			domain_nowww === "seoulilbo.com" ||
			domain_nowww === "lecturernews.com" ||
			domain_nowww === "youthdaily.co.kr" ||
			domain_nowww === "mdaily.net" ||
			domain_nowww === "cctoday.co.kr" ||
			domain_nowww === "gnmaeil.com" ||
			domain_nosub === "mhnew.com" ||
			domain_nowww === "mediadale.com" ||
			domain === "viewers.heraldcorp.com" ||
			domain_nowww === "biztribune.co.kr" ||
			domain_nowww === "obsnews.co.kr" ||
			domain_nowww === "kyeonggi.com" ||
			domain_nowww === "iloveorganic.co.kr" ||
			domain_nowww === "newslock.co.kr" ||
			domain_nowww === "news33.net" ||
			domain_nowww === "mainnews.kr" ||
			domain_nowww === "ilemonde.com" ||
			domain_nowww === "canews.kr" ||
			domain_nowww === "newsflix.co.kr" ||
			domain_nowww === "gokorea.kr" ||
			domain_nowww === "womaneconomy.kr" ||
			domain_nowww === "apsk.co.kr" ||
			domain_nowww === "sporbiz.co.kr" ||
			domain_nowww === "lovesbeauty.co.kr" ||
			domain_nowww === "ikoreadaily.co.kr" ||
			domain_nowww === "interview365.com" ||
			domain_nowww === "newsa.co.kr" ||
			domain_nowww === "newstown.co.kr" ||
			src.match(/^[a-z]+:\/\/[^/]+\/+news\/+thumbnail\/+[0-9]{6}\/+[0-9]+_[0-9]+_[0-9]+_v?150\.[^/.]*(?:[?#].*)?$/)) {
			newsrc = src.replace(/\/thumbnail\/+([0-9]{6}\/+(?:[^/.]*_)?[0-9]+_[0-9]+_[0-9]+)(?:_orgBig)?_v?150(\.[^/.]*)(?:[?#].*)?$/, "/photo/$1$2");
			if (newsrc !== src)
				return add_extensions_upper_jpeg(newsrc);
		}

		if (domain === "shop.newsen.com") {
			return src.replace(/:\/\/[^/]*\/+data\/+thumb\/+[0-9]+_[0-9]+_([0-9][0-9][0-9][0-9])([0-9][0-9])([0-9][0-9])([^/]*)$/, "://photo.newsen.com/news_photo/$1/$2/$3/$1$2$3$4");
		}

		if (domain === "cdn.newsen.com" ||
			domain === "photo.newsen.com") {

			newsrc = src.replace(/_ts\.[^/._]*$/, ".jpg").replace("/mphoto/", "/news_photo/");
			if (newsrc !== src)
				return newsrc;

			if (string_indexof(src, "/main_photo/") >= 0) {
				newsrc = src.replace(/\/main_photo\/+(?:mobile\/+)?[^/]*_([0-9][0-9][0-9][0-9])([0-9][0-9])([0-9][0-9])([^/]*)$/, "/news_photo/$1/$2/$3/$1$2$3$4");
				if (newsrc !== src)
					return newsrc;
			}

			newsrc = src.replace(/\/resize\/+[-0-9]+x[-0-9]+\/+/, "/news_photo/");
			if (newsrc !== src)
				return newsrc;

			var extra = {};
			match = src.match(/cdn\.newsen\.com\/+newsen\/+news_photo\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}([0-9]{10,})_[0-9]+\.[^/.]*(?:[?#].*)?$/);
			var pageid = null;
			if (match ||
				(match = src.match(/photo\.newsen\.com\/+news_photo\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}([0-9]{10,})_[0-9]+\.[^/.]*(?:[?#].*)?$/))) {
				pageid = match[1];
				extra.page = "http://www.newsen.com/news_view.php?uid=" + match[1];
			}

			var obj = {
				url: src,
				headers: {
					Referer: "http://www.newsen.com/"
				},
				referer_ok: {
					same_domain_nosub: true
				},
				extra: extra
			};

			if (extra.page && options && options.cb && options.allow_thirdparty && options.do_request) {
				var get_imageid = function(url) {
					if (!url)
						return url;

					var match = url.match(/\/[0-9]{4}\/+(?:[0-9]{2}\/+){2}([0-9]{10,}_[0-9]+)\.[^/.]*(?:[?#].*)?$/);
					if (!match)
						return null;

					if (_nir_debug_)
						console_log("Image ID for " + url + ":", match[1]);

					return match[1];
				};

				var get_newsen_pageinfo = function(page, pageid, cb) {
					api_cache.fetch("newsen_pageinfo:" + pageid, cb, function(done) {
						options.do_request({
							method: "GET",
							url: extra.page,
							overrideMimeType: "text/html; charset=euc-kr",
							onload: function(result) {
								if (result.readyState !== 4)
									return;

								if (result.status !== 200) {
									return done(null, false);
								}

								var match = result.responseText.match(/<meta\s+property=["']og:title["']\s+content=["']([^'"]+)["']/);
								if (!match) {
									console_warn("Newsen title not found", result);
									return done(null, false);
								}

								var title = strip_whitespace(match[1]);
								console_log("Title: ", title);

								match = result.responseText.match(/<img\s+id=["']artImg["']\s+src=["'](?:https?:)?\/\/[a-z]+\.newsen\.com\/.*?["']/g);
								if (!match) {
									console_warn("Newsen images not found", result);
									return done(null, false);
								}

								return done({
									page: result.finalUrl,
									title: title,
									images: match
								}, 6*60*60);
							}
						});
					});
				};

				var get_imagenum_from_newsen_pageinfo = function(image, pageinfo) {
					var imageid = get_imageid(image);
					if (!imageid)
						return null;

					var imagenum;
					for (imagenum = 0; imagenum < pageinfo.images.length; imagenum++) {
						var i_imageid = get_imageid(pageinfo.images[imagenum]);
						if (!i_imageid)
							continue;

						if (i_imageid === imageid)
							break;
					}

					if (imagenum === pageinfo.images.length) {
						return null;
					}

					return imagenum;
				};

				var get_daum_search_results = function(title, cb) {
					api_cache.fetch("newsen_daum_search:" + title, cb, function(done) {
						var searchurl = "https://m.search.daum.net/search?w=news&q=" + encodeURIComponent(title) + "&enc=utf8";
						options.do_request({
							method: "GET",
							url: searchurl,
							onload: function (result) {
								if (result.readyState !== 4) {
									return;
								}

								if (result.status !== 200) {
									return done(null, false);
								}

								var entry_regex = /<a\s+class=["']info_item["']\s+href=["']((?:https?:)\/\/v\.media\.daum\.net\/+.*?)["'][^>]*?>\s*(?:<div\s+class=["']compo-.*?["']\s*>\s*)?<span\s+class=["']wrap_(?:tit|head)["']\s*>(?:\s*<strong\s+class=["']tit-g["']\s*>\s*)?([\s\S]*?)(?:\s*<\/strong>\s*)?<\/span>/g;
								var nonglobal_regex = new RegExp(entry_regex, "");
								var entries_match = result.responseText.match(entry_regex);
								if (!entries_match) {
									console_error("No search entries found", searchurl);
									console_log(result);
									return done(null, false);
								}

								var search_entries = [];

								for (var entry_i = 0; entry_i < entries_match.length; entry_i++) {
									var match = entries_match[entry_i].match(nonglobal_regex);
									if (!match) {
										console_error("Unable to match non-global regex (this shouldn't happen)");
										continue;
									}

									search_entries.push({
										title: decode_entities(match[2].replace(/<([a-z]+)(?:\s+.*?)?>(.*?)<\/\1>/g, "$2")),
										url: urljoin(result.finalUrl, match[1].replace(/[?#].*/, ""), true)
									});
								}

								if (search_entries.length === 0) {
									console_error("Zero search entries found", searchurl);
								}

								done(search_entries, 2*60*60);
							}
						});
					});
				};

				var get_daum_url_from_results = function(real_title, real_pageid, results) {
					if (!results)
						return null;

					var real_fuzzy = fuzzify_text(real_title).substr(0, 35);

					for (var i = 0; i < results.length; i++) {
						if (real_fuzzy !== fuzzify_text(results[i].title).substr(0, 35)) {
							continue;
						}

						var pageid = results[i].url.replace(/.*\/v\/([0-9]+)$/, "$1");
						if (pageid === results[i].url) {
							console_error("Unhandled URL: " + results[i].url);
							continue;
						}

						if (!fuzzy_date_compare(pageid.substr(0, 8), real_pageid.substr(0, 8))) {
							if (_nir_debug_) {
								console_log("Skipping:", pageid, real_pageid);
							}
							continue;
						}

						return results[i].url;
					}

					console_error("Unable to find result from search", real_title, results);

					return null;
				};

				var get_daum_pageinfo = function(daumurl, cb) {
					api_cache.fetch("newsen_daum_pageinfo:" + daumurl, cb, function(done) {
						options.do_request({
							method: "GET",
							url: daumurl,
							onload: function (result) {
								if (result.readyState !== 4)
									return;

								if (result.status !== 200) {
									return done(null, false);
								}

								var image_regex = /<img[^>]+dmcf-mtype=["']image["'][^>]+src=["']((?:https?:)\/\/[^/]+\.daumcdn\.net\/.*?)["']/g;
								var nonglobal_regex = new RegExp(image_regex, "");

								var obj = {
									url: result.finalUrl,
									images: []
								};

								var images_match = result.responseText.match(image_regex);
								if (!images_match) {
									return done(obj, 60*60);
								}

								for (var i = 0; i < images_match.length; i++) {
									var image_match = images_match[i].match(nonglobal_regex);
									if (!image_match)
										continue;

									obj.images.push(urljoin(result.finalUrl, image_match[1], true));
								}

								done(obj, 6*60*60);
							}
						});
					});
				};

				get_newsen_pageinfo(extra.page, pageid, function(pageinfo) {
					if (!pageinfo) {
						return options.cb(obj);
					}

					var title = pageinfo.title;
					var total_images = pageinfo.images.length;
					var imagenum = get_imagenum_from_newsen_pageinfo(src, pageinfo);
					if (imagenum === null) {
						return options.cb(obj);
					}

					get_daum_search_results(title, function(results) {
						var daumurl = get_daum_url_from_results(title, pageid, results);
						if (!daumurl)
							return options.cb(obj);

						console_log("Daum article URL:", daumurl);

						get_daum_pageinfo(daumurl, function(daum_pageinfo) {
							if (!daum_pageinfo)
								return options.cb(obj);

							if (daum_pageinfo.images.length !== total_images) {
								console_error("Image number mismatch");
								return options.cb(obj);
							}

							obj.url = daum_pageinfo.images[imagenum];
							obj.extra.page = daum_pageinfo.url;
							delete obj.headers.Referer;
							return options.cb(obj);
						});
					});
				});

				return {
					waiting: true
				};
			}

			return obj;
		}

		if (domain === "sccdn.chosun.com") {

			return src.replace(/\/([0-9]*_)(?:scr_)?([^._/]*)(?:_t)?(\.[^/.]*)$/, "/$1$2$3");
		}

		if (domain_nosub === "chosun.com" ||
			domain_nowww === "the-star.co.kr" ||
			domain === "digitalchosun.dizzo.com" ||
			domain_nosub === "chosunonline.com") {

			newsrc = src
				.replace("/simg_thumb/", "/simg_org/")
				.replace(/\/thumb_dir\/(.*)_thumb(\.[^/.]*)$/, "/img_dir/$1$2")
				.replace(/\/thumbnail\/(.*?)(?:_thumb)?(\.[^/.]*)$/, "/image/$1$2");

			if (newsrc !== src)
				return add_full_extensions(newsrc);

			id = src.match(/\/site\/+data\/+img_dir\/+([0-9]{4}\/+(?:[0-9]{2}\/+){2})([0-9]{10,})_[0-9]+\./);
			if (false && id) {
				var id_plus_one = parseInt(id[2]) + 1; // doesn't always work
				return {
					url: src,
					extra: {
						page: src.replace(/\/site\/+data\/+img_dir\/+.*/, "/site/data/html_dir/" + id[1] + (id_plus_one) + ".html")
					}
				};
			}
		}


		if (domain === "photo.hankooki.com") {




			newsrc = src.replace("/arch/photo/", "/arch/original/")
				.replace("/arch/thumbs/", "/arch/original/")
				.replace(/(\/newsphoto\/[^/]*\/)thumbs\//, "$1");
			if (newsrc !== src) {
				return newsrc.replace(/(.*\/)t_?([^/]*)$/, "$1$2");
			} else {
				return src;
			}
		}

		if (domain === "thumb.hankooki.com") {
			return src.replace(/:\/\/[^/]*\/+(.*\/+)[0-9]+x[0-9]+x[^/]*@([^/]*)(?:[?#].*)?$/,
							   "://photo.hankooki.com/$1$2");
		}

		if (domain_nosub === "ettoday.net") {
			return {
				url: src
					.replace(/\/[a-z]+([0-9]*\.[^/]*)(?:[?#].*)?$/, "/$1")
					.replace(/:\/\/cdn[0-9]*\./, "://static."),
				head_wrong_contenttype: true
			};
		}

		if (domain === "img.mbn.co.kr") {
			return src.replace(/_s[0-9]+x[0-9]+(\.[^/]*)$/, "$1");
		}

		if (domain === "imgmmw.mbn.co.kr") {
			return src.replace(/(\/storage\/+news\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[-0-9A-F]{30,})_[0-9]+(\.[^/.]*)*(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "image-gd.inews24.com") {
			return src.replace(/:\/\/[^/]*\/image[0-9]*\.php\?u=([^&]*).*/, "://image3.inews24.com$1");
		}

		if ((domain_nosub === "inews24.com" && /^img\.(?:lb|xml|[0-9]+)\./.test(domain)) ||
			(domain_nosub === "inews24.com" && /^image[0-9]*\./.test(domain)) ||
			(domain_nosub === "iwinv.net" && /^inews24\.(?:ext[0-9]+\.)?cache\./.test(domain))) {
			return src
				.replace(/(:\/\/[^/]+\/)(?:face\/+)?(?:[0-9]+dpi\/+){0,}(?:[0-9]*x[0-9]*\/+){0,}image_joy\//, "$1image_joy/")
				.replace(/\/image_joy\/+thumbnail\//, "/image_joy/");
		}

		if (domain_nosub === "wowkorea.jp" &&
			(string_indexof(src, ".wowkorea.jp/img") >= 0 ||
			 string_indexof(src, ".wowkorea.jp/upload") >= 0)) {

			newsrc = src.replace(/(\/upload\/+photoSpecial\/+[0-9]+\/+)re_([^/]*\.[^/.]*)(?:[?#].*)?$/, "$1$2");
			if (newsrc !== src)
				return newsrc;

			if (string_indexof(src, "/img/album/") < 0 &&
				!src.match(/\/upload\/news\/+[0-9]+\//)) {
				return src.replace(/([^/]*_)[a-z0-9]*(\.[^/.]*)$/, "$1l$2");
			}

			newsrc = src.replace(/_(?:[a-z0-9]|ss)(\.[^/.]*)$/, "$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nosub === "saostar.vn" &&
			domain.match(/img[0-9]*\.saostar\.vn/)) {
			newsrc = src
				.replace(/saostar.vn\/fb[0-9]+[^/]*(\/.*\.[^/.]*)\/[^/]*$/, "saostar.vn$1")
				.replace(/saostar.vn\/[a-z][0-9]+\//, "saostar.vn/")
				.replace(/saostar.vn\/[0-9]+x[0-9]+\//, "saostar.vn/");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nosub.match(/^google\./) &&
			src.match(/\/www\.google\.(?:[a-z]+\.)?[a-z]*\/url\?/)) {
			newsrc = src.replace(/.*url=([^&]*).*/, "$1");
			if (newsrc !== src && /^https?/.test(newsrc))
				return decodeURIComponent(newsrc);
		}

		if ((domain_nowww === "lipstickalley.com" ||
			 domain_nowww === "stephenking.com" ||
			 domain_nowww === "ambercutie.com" ||
			 string_indexof(domain, "forum.purseblog.com") >= 0) &&
			string_indexof(src, "/proxy.php?") >= 0) {
			newsrc = src.replace(/.*image=([^&]*).*/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "images.askmen.com") {
			return {
				url: src,
				head_wrong_contentlength: true
			};
		}

		if ((domain_nosub === "127.net" && string_indexof(domain, "nosdn.127.net") >= 0) ||
			domain === "pic-bucket.ws.126.net") { //lofter
			return {
				url: src.replace(/\?.*$/, ""),
				headers: {
					"Referer": "https://lofter.com"
				}
			};
		}

		if (domain === "board.makeshop.co.kr") {
			return src.replace(/\/[a-z]*::/, "/");
		}

		if (domain_nosub === "naver.jp" &&
			domain.match(/^rr\.img[0-9]*\.naver\./)) {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/mig.*?[?&]src=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeURIComponent(newsrc);
		}

		if (domain === "imgcc.naver.jp") {
			return src
				.replace(/\/thumb(\/[0-9]+x[0-9]+x[0-9a-f]+)_[0-9]+_[0-9]+(\.[^/.]*)$/, "$1$2")
				.replace(/\/[0-9]+\/[0-9]+\/*(?:[?&].*)?$/, "");
		}

		if (domain === "dimg.donga.com") {
			return src
				.replace(/\/[ca]\/(?:[0-9]+\/){4}/, "/")
				.replace(/\/i\/[0-9]+\/[0-9]+\/[0-9]+\//, "/");
		}

		if (domain === "imgpark.donga.com" && string_indexof(src, "/fileUpload/") >= 0) {
			return {
				url: src,
				headers: {
					Referer: "http://mlbpark.donga.com/"
				}
			};
		}

		if ((domain_nosub === "marishe.com" &&
			 domain.match(/s[0-9]\.marishe\.com/)) ||
			(domain_nowww === "klik.gr" && string_indexof(src, "/uploads_image/") >= 0) ||
			(domain_nowww === "ug-ts.ru" && /\/content\/+pics\//.test(src)) ||
			(domain_nowww === "eroticasearch.net" && string_indexof(src, "/content/pics/") >= 0) ||
			domain === "resource.breakingnews.mn") {
			newsrc = src.replace(/(\/[^/]*)_[0-9]+(\.[^/.]*)$/, "$1$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "klik.gr") {
			return {
				url: src,
				can_head: false
			};
		}


		if (domain === "cdn.hk01.com") {
			return src
				.replace(/(\/media\/images\/[^/]*\/)[^/]*\//, "$1org/")
				.replace(/\?.*/, "");
		}

		if (domain_nosub === "sinaimg.cn") {
			if (src.match(/:\/\/[^/]*\/+max(?:width|height)\.[0-9]+\//)) {
				return {
					url: src.replace(/(:\/\/[^/]*\/+)[^/]*\//, "$1original/"),
					can_head: false
				};
			}

			if (domain.match(/^ss/)) {
				src = src.replace(/\.sinaimg\.cn\/+[^/]*\/+([^/]*)\/*$/, ".sinaimg.cn/orignal/$1");
			} else {
				src = src.replace(/\.sinaimg\.cn\/+[^/]*\/+([^/]*)\/*$/, ".sinaimg.cn/large/$1");
			}

			if (domain.match(/^n\./)) {
				newsrc = src.replace(/(\/ent\/+[0-9]+_)img(\/+upload\/)/, "$1ori$2");
				if (newsrc !== src)
					return newsrc;
			}

			newsrc = src.replace(/(\/autoimg\/+serial\/+[0-9]{2}\/+[0-9]{2}\/+[0-9]+)_[0-9]+(\.[^/.]+)(?:[?#].*)?$/, "$1_src$2");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/\/slidenews\/+([^/_]*)_[^/_]*\//, "/slidenews/$1_img/"); // there's also _ori, but it seems to be smaller?
			if (newsrc !== src)
				return newsrc;

			return {
				url: src,
				headers: {
					Referer: "https://weibo.com/"
				},
				referer_ok: {
					same_domain: true
				}
			}
		}

		if (domain_nosub === "sina.com.cn" && domain.match(/^static[0-9]\.photo\.sina\.com\.cn/)) {
			return src.replace(/:\/\/static([0-9]*)\.photo\.sina\.com\.cn\//, "://ss$1.sinaimg.cn/");
		}

		if (domain === "sinaimg.acgsoso.com") {
			return src.replace(/:\/\/[^/]+\/+/, "://wx4.sinaimg.cn/");
		}

		if (domain === "thumbnail.egloos.net" ||
			domain === "thumb.egloos.net") {
			return src.replace(/^[a-z]+:\/\/thumb(?:nail)?\.egloos\.net\/[^/]*\/*/, "");
		}

		if (domain === "k.kakaocdn.net") {
			return src.replace(/\/img_[a-z]*\.([^./]*)$/, "/img.$1");
		}

		if (domain === "images.sportskhan.net" ||
			domain === "img.khan.co.kr") {
			return src
				.replace(/\/r\/[0-9]+x[0-9]+\//, "/")
				.replace(/\/[a-z]*_([0-9]+\.[a-z0-9A-Z]*)$/, "/$1")
				.replace(/\/c\/[0-9]*x[0-9]*\//, "/")
				.replace(/(\/news\/[0-9]{4}\/[0-9]{2}\/[0-9]{2}\/[0-9]{8}\.[0-9]+\.[0-9]+)[A-Z](\.[^/.]*)$/, "$1L$2")
				.replace(/\/photodb\//, "/PhotoDB/");
		}

		if (domain_nosub === "sbs.co.kr" &&
			domain.match(/^img[0-9]*\.sbs\.co\.kr/)) {
			newsrc = src.replace(/(\/[0-9]+)_[0-9v]+\.([a-z0-9A-Z]*)$/, "$1.$2");
			if (newsrc !== src)
				return newsrc;

			return src.replace(/(\/[^_]*)_[^/.]*(\.[^/.]*)$/, "$1_ori$2");
		}

		if (domain === "image.board.sbs.co.kr") {
			return src.replace(/-[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain_nosub === "edaily.co.kr" &&
			string_indexof(domain, "image.edaily.co.kr") >= 0 ||
			string_indexof(domain, "img.edaily.co.kr") >= 0) {

			return src
				.replace(/\/[a-z]_([A-Z0-9]+)\.([a-z0-9A-Z]*)$/, "/$1.$2")
				.replace(/(\/[A-Z0-9]+)[a-z]\.([a-z0-9A-Z]*)$/, "$1.$2");
		}

		if (domain === "media.glamour.com" ||

			domain === "assets.teenvogue.com" ||
			domain === "assets.vogue.com" ||
			domain === "media.vanityfair.com" ||
			domain === "media.gq.com" ||
			domain === "media.wmagazine.com" ||
			domain === "media.self.com" ||
			domain === "media.pitchfork.com" ||
			domain === "media.wired.com" ||
			domain === "media.golfdigest.com" ||
			domain === "media.architecturaldigest.com" ||
			domain === "media.cntraveler.com" ||
			domain === "media.allure.com" ||
			src.match(/:\/\/[^/]*\/photos\/[0-9a-f]{24}\/[^/]*\/[^/]*\/[^/]*$/)) {
			newsrc = src.replace(/\/[^/]*\/[^/]*\/([^/]*)$/, "/original/original/$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nosub === "cloudinary.com" ||
			domain === "images.playmates.com" ||
			domain === "images.playboy.com" ||
			(domain_nosub === "engoo.com" && domain.match(/^transcode[0-9]*\.app\./)) ||
			domain === "images.thehollywoodgossip.com" ||
			domain === "cdn.apartmenttherapy.info" ||
			domain === "image.thestartmagazine.com" ||
			domain === "cdn.substack.com" ||
			domain === "c.bellesa.co" ||
			domain === "images.haarets.co.il" ||
			domain === "assets.mocha.global" ||
			domain === "tbn.bidorbuy.co.za" ||
			domain === "resources.sonyliv.com" ||
			(domain_nosub === "minutemediacdn.com" && /^images[0-9]*\./.test(domain)) ||
			domain === "images.taboola.com") {
			if (src.search(/:\/\/[^/]*\/(?:[^/]*\/+)?(?:image\/+)?fetch\//) >= 0) {
				newsrc = src.replace(/.*?:\/\/[^/]*\/(?:[^/]*\/)?(?:image\/+)?fetch\/(?:.*?(?:\/|%2F))?([^/%]*(?::|%3A).*)/, "$1");
				if (newsrc.match(/^[^/:]*%3A/))
					newsrc = decodeURIComponent(newsrc);
				return newsrc;
			}

			newsrc = src.replace(/(\/iu\/[^/]*)\/.*?(\/v[0-9]*\/)/, "$1$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nosub === "jpimedia.uk" && domain.match(/^images(?:-[a-z])?\./)) {
			return src.replace(/^[a-z]+:\/\/[^/]*\/+imagefetch\/+.*?\/(https?:\/\/)/, "$1");
		}

		if ((domain_nosub === "cloudinary.com" &&
			 (string_indexof(domain, "res.cloudinary.com") >= 0 ||
			  domain.match(/res-[0-9]*\.cloudinary\.com/))) ||
			domain === "i.kinja-img.com") {
			newsrc = src
				.replace(/\/image\/upload\/s\-\-[^/]*\-\-\//, "/image/upload/")
				.replace(/\/iu\/s\-\-[^/]*\-\-\//, "/iu/")
				.replace(/\/image\/upload\/[^/]*_[^/]*\//, "/image/upload/")
				.replace(/\/image\/upload\/v[0-9]+\//, "/image/upload/")
				.replace(/(\/private_images\/)[^/]*\//, "$1c_limit/")
				.replace(/(\/image\/private\/)[^s][^-][^/]*\//, "$1c_limit/")
				.replace(/(:\/\/[^/]*\/)[^/]*\/(ch\/images\/[0-9]+\/[^/]*$)/, "$1$2");
			if (newsrc !== src) {
				return newsrc;
			}

		}

		if (domain === "fiverr-res.cloudinary.com" ||
			domain === "assets.lybrate.com") {
			return src.replace(/(:\/\/[^/]*\/+(?:images\/+)?)[a-z]_[^/]*\//, "$1");
		}

		if (domain === "images.complex.com") {
			return src.replace(/\/(images|image\/upload)\/[^/]*_[^/]*\//, "/$1/");
		}

		if (domain === "images.spot.im" ||
			domain_nowww === "fashionista.com" ||
			domain === "images.pigeonsandplanes.com" ||
			domain === "images.sftcdn.net" ||
			domain === "cdn.primedia.co.za" ||
			domain_nowww === "maxim.com" ||
			domain === "img.thedailybeast.com" ||
			domain === "alibaba.kumpar.com" ||
			domain === "5b0988e595225.cdn.sohucs.com" ||
			domain === "images-cdn.moviepilot.com" ||
			domain === "img.playbuzz.com" ||
			domain === "images.discerningassets.com" ||
			domain === "images.radio-canada.ca" ||
			domain === "img.wcdn.co.il" ||
			domain === "images.haarets.co.il" ||
			domain === "images.cdn.yle.fi" ||
			(domain_nowww === "sol.no" && string_indexof(src, "/img/") >= 0)||
			(domain === "prof.prepics-cdn.com" && string_indexof(src, "/image/upload/") >= 0) ||
			domain === "images.ezvid.com" ||
			domain === "img.peerspace.com" ||
			domain === "dhgywazgeek0d.cloudfront.net" ||
			domain === "assets.nuuvem.com" ||
			domain === "images.lanouvellerepublique.fr" ||
			domain === "images.fastcompany.net" ||
			domain === "dwgyu36up6iuz.cloudfront.net" ||
			(domain_nosub === "minutemediacdn.com" && domain.match(/^images[0-9]*\./)) ||
			(domain_nosub === "cloudinary.com" && string_indexof(domain, "res.cloudinary.com") >= 0 && string_indexof(src, "/images/") >= 0) ||
			domain === "media.stubhubstatic.com" ||
			domain === "img.promipool.de" ||
			domain === "blue.kumparan.com" ||
			(domain_nosub === "answcdn.com" && domain.match(/^file[0-9]*\./)) ||
			domain === "res.fashionsnap.com" ||
			domain === "user-images.strikinglycdn.com" ||
			domain === "novel-img.prepics-cdn.com" ||
			domain === "cdn.apartmenttherapy.info" ||
			domain === "cdn.domestika.org" ||
			(domain === "assets.charmboard.com" && string_indexof(src, "/images/") >= 0) ||
			domain === "image.spreadshirtmedia.com" ||
			domain === "c.bellesa.co" ||
			domain === "images.moviepilot.com") {
			return src
				.replace(/%2C/g, ",")
				.replace(/\/[a-z]+_[^/_,]+(?:,[^/]*)?\//, "/")
				.replace(/\/fl_any_format\.[^/]*\//, "/")
				.replace(/\/fl_keep_iptc[^/]*\//, "/")
				.replace("/t_mp_quality/", "/")
				.replace(/\/image\/+upload\/+t_[^/]*\//, "/image/upload/")
				.replace(/\/v[0-9]+\//, "/");
		}

		if (domain === "image.kkday.com") {
			return src.replace(/\/image\/+get\/+[^/]*(?:%2C|,)[^/]*\//, "/image/get/");
		}

		if (domain === "cdn.skim.gs") {
			return src
				.replace(/\/image\/upload\/[^/]*_[^/]*\//, "/image/upload/")
				.replace(/\/images\/[^/]*_[^/]*\//, "/images/");
		}

		if (((domain_nosub === "biography.com" ||
			  domain_nowww === "guitarworld.com" ||
			  domain_nowww === "guitaraficionado.com" ||
			  domain_nowww === "psneurope.com") &&
			 string_indexof(src, "/.image/") >= 0) ||
			src.match(/:\/\/[^/]*\/\.image\/[^/]*_[^/]*\/[A-Za-z-0-9]{24}\/[^/]*$/)) {
			newsrc = src.replace(/(\/.image)\/[^/]*(\/[^/]*\/[^/]*)$/, "$1$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "resources.mynewsdesk.com") {
			return src.replace(/(\/image\/+upload\/+)t_[^/]+\/+/, "$1");
		}

		if (domain_nosub === "popsugar-assets.com" ||
			domain_nosub === "onsugar.com") {
			newsrc = src.replace(/\/thumbor\/[^/]+\/(?:[0-9]+x[0-9]+:[0-9]+x[0-9]+\/)?(?:fit-in\/)?[^/]+\/(?:filters:[^/]+\/)?/, "/");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/\.(?:preview_)?(?:[a-z]*|_original)(?:_(?:[0-9x]+|wm))?(\/i\/[^/]*)$/, "$1");
			if (newsrc !== src)
				return newsrc;

			return src.replace(/\.(?:preview|x*large)(?:_(?:[0-9x]+|wm))?(\.[^/.]*)$/, "$1");
		}

		if (domain_nosub === "ceros.com" && domain.match(/^media[-.]/)) {
			newsrc = src.replace(/\?.*/, "");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "elleuk.cdnds.net") {
			newsrc = src.replace(/:\/\/.*\/[^/]*assets-elleuk-com-gallery-([0-9]*)-([^/]*)-([^-/.]*)\.[^-/.]*$/, "://assets.elleuk.com/gallery/$1/$2.$3");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nosub === "cdnds.net" &&
			!src.match(/\/[0-9]*x[0-9]*-[^/]*$/)) {

			newsrc = src
				.replace(/\/[0-9]+x[0-9]+\/gallery_/, "/")
				.replace(/\/[0-9]+x[0-9]+\/([^/]*)$/, "/$1")
				.replace(/\/[ML]\/([^/]*)$/, "/$1")
				.replace(/\/(?:landscape|hd-aspect|gallery)-([^/]*)$/, "/$1")
				.replace(/(\/[^/]*)\?[^/]*$/, "$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "redonline.cdnds.net") {
			return src.replace(/__[a-z]+(\.[^/.]*)$/, "$1");
		}

		if (domain === "img.usmagazine.com") {
			return src.replace(/(.*?[^:])\/[0-9]*-[^/]*\//, "$1/");
		}

		if (domain_nosub === "gannett-cdn.com" &&
			string_indexof(src, "/-mm-/") >= 0) {
			newsrc = src.replace(/.*?\/-mm-\/[0-9a-f]*\/[^/]*\/(http[^/]*)\/(.*)$/, "$1://$2");
			if (newsrc !== src)
				return newsrc;

			return src.replace(/\/-mm-\/.*?\/-\//, "/");
		}

		if (domain_nosub === "aolcdn.com") {
			var regex1 = /.*image_uri=([^&]*).*/;

			if (src.match(regex1)) {
				newsrc = decodeURIComponent(src.replace(/.*image_uri=([^&]*).*/, "$1"));
			} else if (src.match(/.*o\.aolcdn\.com\/images\//)) {
				newsrc = decodeURIComponent(src).replace(/.*o\.aolcdn\.com\/images\/[^:]*\/([^:/]*:.*)/, "$1");
			} else if (src.match(/^[a-z]+:\/\/[^/]*\/hss\/storage\/midas\//)) {
				return src.replace(/\/[0-9]+_([^/]*)$/, "/$1");
			}

			if (newsrc && newsrc !== src)
				return newsrc;
		}

		if (domain === "imagesvc.timeincapp.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/v3\/+[a-z]+\/+image\/?.*?[?&]url=(http[^&]*).*/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}


		if (domain_nosub === "photoshelter.com") {
			newsrc = src
				.replace(/\/img-get2\/([^/]*)\/(?:[a-z]+=[^/]*\/)*([^/]*)$/, "/img-get2/$1/fit=99999999999/$2")
				.replace(/\/img-get\/([^/]*)(?:\/[ts]\/[0-9]+\/(?:[0-9]+\/)?)?([^/]*)$/, "/img-get2/$1/fit=99999999999/$2")
				.replace(/\/+fit=[0-9x]+\/+fit=[0-9x]+/, "/fit=99999999999");

			if (newsrc !== src)
				return newsrc;

			match = src.match(/\/img-get2\/+(I[^/]{10,})\//);
			if (match && options.force_page) {
				id = match[1];
				var cache_key = "photoshelter_page:" + id;
				api_cache.fetch(cache_key, function(data) {
					if (data) {
						return options.cb({
							url: src,
							extra: {
								page: data
							}
						});
					} else {
						return options.cb(null);
					}
				}, function(done) {
					options.do_request({
						url: "https://www.photoshelter.com/image/" + id,
						method: "GET",
						onload: function(resp) {
							if (resp.readyState < 4)
								return;

							if (resp.status !== 200) {
								return done(null, false);
							}

							var url = resp.finalUrl;
							if (url.match(/\.photoshelter\.com\/+image\/+/)) {
								return done(url, 12*60*60);
							} else {
								return done(null, false);
							}
						}
					});
				});

				return {
					waiting: true
				};
			}
		}

		if (domain_nowww === "celebzz.com" &&
			string_indexof(src, "/wp-content/uploads/") >= 0) {
			newsrc = src.replace(/_thumbnail(\.[^/.]*)$/, "$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nosub === "images-amazon.com" ||

			domain_nosub === "ssl-images-amazon.com" ||
			domain_nosub === "media-amazon.com" ||
			domain_nosub === "media-imdb.com" ||
			domain === "in.in3w.info" ||
			domain === "i.gr-assets.com") {
			return {
				url: src
					.replace(/(\.[^/.]*)(?:\.[^/.]*_){1,}\1(?:[?#].*)?$/, "$1") // is this needed?
					.replace(/\.(?:[^/.]*_|_[^/.]+)\.([^./]*)$/, ".$1"), // for now this seems to work for all images
				is_original: true,
				can_head: false
			};
		}

		if (domain_nosub === "otapol.jp") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/img\/amazon\/size[0-9]+\/([^/]*)$/,
								 "https://images-na.ssl-images-amazon.com/images/I/$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "movpins.com") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/[a-z]+\/+(.)([a-zA-Z0-9]+(?:@+)?)(?:\.[^/]*)?\/[^/]*(\.[^/.]*)$/,
							   "https://ia.media-imdb.com/images/$1/$1$2$3");
		}



		if (domain === "cdn-img.instyle.com" ||

			domain === "static.independent.co.uk" ||
			domain === "static.standard.co.uk" ||
			/*domain.indexOf("www.billboard.com") >= 0 ||
			  string_indexof(domain, "www.harpersbazaararabia.com") >= 0 ||
			  string_indexof(domain, "www.etonline.com") >= 0 ||*/
			domain === "o.oystermag.com" ||
			/*domain.indexOf("www.metro.us") >= 0 ||
			  string_indexof(domain, "www.mtv.co.uk") >= 0 ||
			  string_indexof(domain, "www.grammy.com") >= 0 ||*/
			(domain_nosub === "thr.com" && domain.match(/cdn[0-9]*\.thr\.com/)) ||
			domain.match(/s[0-9]*\.ibtimes\.com/) ||
			src.match(/\/s3fs-public\/styles\/[^/]*\/public\//) ||
			domain === "media.pri.org" ||
			domain_nowww === "wwe.com" ||
			domain === "akm-img-a-in.tosshub.com" ||
			domain === "cdn.9razia.de" ||
			domain_nowww === "bravo.de" ||
			domain === "static.reservationvacances.com" ||
			domain === "cdn.okmag.de" ||
			domain_nowww === "soganhaber.com" ||
			domain_nowww === "favera.ru" ||
			domain === "cdn.businessinsider.es" ||
			(domain_nowww === "ru-24.ru" && string_indexof(src, "/_files/") >= 0) ||
			(domain_nowww === "selenagomez.com" && string_indexof(src, "/sites/") >= 0) ||
			(domain_nowww === "zizki.com" && string_indexof(src, "/styles/") >= 0) ||
			(domain_nowww === "rcf.fr" && string_indexof(src, "/sites/") >= 0) ||
			src.match(/\/sites\/[^/]*\/files2?\/styles\/[^/]*/) ||
			src.match(/\/sites\/[^/]*\/files2?\/[^/]*\/styles\/[^/]*/) ||
			src.match(/(?:(?:\/sites\/+[^/]*)?\/files\/+|\/sites\/+[^/]*\/+)imagecache\/+[^/]*|\/sites\/+[^/]*\/+public\/+styles\/+/) ||
			src.search(/\/files\/styles\/[^/]*\/(?:public|private)\//) >= 0 ||
			src.search(/\/files\/[^/]*\/styles\/[^/]*\/(?:public|private)\//) >= 0) {

			newsrc = src
				.replace(/\/styles\/+.*?\/+(?:public|private)\//, "/")
				.replace(/\/imagecache\/+[^/]*\/+(?:files\/)?/, "/")
				.replace(/\?.*$/, "");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "cdn-img.instyle.com") {
			newsrc = src.replace(/(\/images\/+[0-9]{4}\/+[0-9]{2}\/+[0-9]+-[^/]+)_0(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "etonline.com") {
			return {
				url: src,
				head_wrong_contentlength: true
			};
		}

		if (domain === "cdn.okmag.de" ||
			domain === "cdn.9razia.de") {
			return src.replace(/(:\/\/[^/]*\/)s\/[^/]*\/public\/(media\/)/, "$1$2");
		}

		if (domain === "img.elcomercio.pe" ||
			domain === "img.peru21.pe" ||
			domain_nowww === "elpais.com.co") {
			return src.replace(/\/files\/[^/]*\/uploads\//, "/uploads/");
		}

		if (domain === "media.voltron.voanews.com") {
			return src.replace(/\/styles\/+[^/]*\/+s3\/+/, "/");
		}

		if (domain_nowww === "trbimg.com") {
			return src.replace(/\/[0-9]*\/[0-9]*x[0-9]*\/*$/, "/").replace(/\/[0-9]*\/*$/, "/");
		}

		if ((domain_nosub === "blogspot.com" && string_indexof(domain, ".bp.blogspot.com") >= 0) ||

			((domain_nosub === "googleusercontent.com" ||
			  domain_nosub.match(/^google\./)) &&
			 (domain.match(/^lh[0-9]\./) ||
			  domain.match(/^gp[0-9]\./) ||
			  domain.match(/^ci[0-9]\./))) ||
			domain === "d2yal1mtmg1ts6.cloudfront.net" ||
			(domain_nosub === "blogger.com" && domain.match(/^bp[0-9]*\.blogger\.com/)) ||
			domain_nosub === "ggpht.com") {
			newsrc = src;

			if (!(/^[a-z]+:\/\/[^/]+\/+[^/?#=]{30,}=x[0-9]+-y[0-9]+-z[0-9]+-t[^-/?#]{23,}(?:[?#].*)?$/.test(src))) {
				newsrc = src
					.replace(/#.*$/, "")
					.replace(/\?.*$/, "")
					.replace(/\/[swh][0-9]*(-[^/]*]*)?\/([^/]*)$/, "/s0/$2")
					.replace(/(=[^/]*)?$/, "=s0?imgmax=0");
			}

			return {
				url: newsrc,
				can_head: false // 404
			};
		}

		if (domain === "ssl.gstatic.com") {
			if (/\/docs\/+common\/+[^/]+\.svg(?:[?#].*)?$/.test(src)) {
				return {
					url: src,
					bad: "mask"
				};
			}
		}

		if (host_domain === "docs.google.com" && options.element) {
			if (options.element.tagName.toUpperCase() === "SVG" && options.element.classList.contains("kix-embeddedobject-image")) {
				var images = options.element.getElementsByTagName("image");
				if (images.length === 1) {
					newsrc = images[0].getAttribute("xlink:href");
					if (newsrc && newsrc !== src)
						return newsrc;
				}
			}
		}

		if (domain_nowww === "star-tool.ru") {
			return src.replace(/^[a-z]+:\/\/[^/]+\/+([^/]+\/+[^/]+\/+[^/]+\/+[^/]+\/+[swh][0-9]*(?:-[^/]*]*)?\/+)/, "https://lh3.googleusercontent.com/$1");
		}

		if (domain_nowww === "thehackernews.com") {
			return src.replace(/^[a-z]+:\/\/[^/]+\/+images\/+((?:[^/]+\/+){4}[^/]+\/+[^/]+)(?:[?#].*)?$/, "https://1.bp.blogspot.com/$1");
		}

		if (domain_nosub === "googleusercontent.com" &&
			string_indexof(domain, "opensocial.googleusercontent.com") >= 0) {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/gadgets\/proxy.*?[?&]url=([^&]*).*?$/, "$1"));
		}


		if (domain === "cdn.narcity.com" ||
			domain_nowww === "narcity.com") {
			return src.replace(/(\/[^/.]*\.[^/._]+)_(?:facebook|[0-9]+x[0-9]+)\.[^/.]*(?:[?#].*)?$/, "$1");
		}

		if (domain === "images.vanityfair.it" ||
			domain === "images.glamour.it") {
			return src.replace(/(\/gallery\/[0-9]*\/)[^/]*\//, "$1Original/");
		}

		if (domain_nosub === "r29static.com" ||
			(domain_nosub === "refinery29.com" && domain.match(/^static[0-9]*\./))) {
			return src.replace(/(\/bin\/(?:entry|public|author)\/[^/]*)\/(?:[0-9]+,[0-9]+,[0-9]+,[0-9]+\/)?[^/]*(?:,[^/]*)?\/([^,]*)$/, "$1/x,100/$2");
		}

		if (domain === "img.huffingtonpost.com") {
			return src
				.replace(/\/asset\/[^/]*\/([^/.]*\.[^/.]*)$/, "/asset/$1")
				.replace(/\?[^/]*$/, "");
		}

		if (domain === "images.huffingtonpost.com" ||
			(domain_nowww === "kisax.com" && string_indexof(src, "/images/") >= 0) ||
			(domain_nosub === "nicepik.com" && domain.match(/^i[0-9]*\./) && string_indexof(src, "/files/") >= 0) ||
			(domain_nowww === "babylonbee.com" && /\/img\/+articles\//.test(src)) ||
			(domain_nowww === "womansdiary.gr" && src.match(/\/articles\/[0-9]+\/[-0-9a-f]+-thumb/))) {
			return src.replace(/-thumb(\.[^/.]*)$/, "$1");
		}

		if (domain === "i.huffpost.com" ||
			domain === "s-i.huffpost.com") {

			return src
				.replace(/(\/gadgets\/slideshows\/[0-9]*\/slide_[^/]*_)[a-z]*(\.[^/.]*)$/, "$1original$2")
				.replace(/(\/gen\/[0-9]*\/).*(\.[^/.?]*)(?:\?[^/]*)?$/, "$1original$2");
		}

		if ((domain_nosub === "washingtonpost.com" ||
			 domain_nowww === "statesman.com" ||
			 domain_nowww === "myajc.com" ||
			 domain_nowww === "ajc.com" ||
			 domain_nowww === "lastampa.it" ||
			 domain_nowww === "whio.com" ||
			 domain_nowww === "daytondailynews.com" ||
			 domain_nowww === "livemint.com" ||
			 domain === "c.o0bg.com" ||
			 domain_nosub === "bostonglobe.com" ||
			 domain_nosub === "hindustantimes.com") &&
			string_indexof(src, "/rf/") >= 0 ||
			src.match(/^[a-z]+:\/\/[^/]*\/rf\/+image_[^/]*\/+(?:[0-9]{4}-[0-9]{4}\/+)?(?:Pub|Wires|HT)\/+(?:[wp][0-9]+|Online|Web)\/+/)) {



			newsrc = src
				.replace(/(.*?:\/\/[^/]*\/)rf\/[^/]*\/(.*)$/, "$1rw/$2")
				.replace(/[?&].*$/, "")
				.replace(/\.r(\.[^/.]*)$/, "$1");
			if (newsrc !== src) {
				return newsrc;
			}

		}


		if (domain_nosub === "foxnews.com" &&
			domain.match(/^a[0-9]*\.foxnews\.com/)) {
			if (src.replace(/.*\/a[0-9]*\.foxnews\.com\/([^/]*).*/, "$1") !== "images.foxnews.com") {
				return src.replace(/.*\/a[0-9]*\.foxnews\.com\/(.*)\/[0-9]+\/[0-9]+\/([^/]*)$/, "http://$1/$2");
			}
			return src.replace(/(\/a[0-9]*\.foxnews\.com\/.*)\/[0-9]+\/[0-9]+\/([^/?]*)(?:\?.*)?$/, "$1/0/0/$2");
		}

		if (domain === "cdn.cliqueinc.com" ||
			domain_nosub === "cliqueimg.com") {
			return src
				.replace(/(\/[^/]*)\.[0-9]*x[0-9]*[^/.]*\.([^./]*)$/, "$1.$2")
				.replace(/\/cache\/posts\//, "/posts/");
		}

		if (domain_nosub === "hubstatic.com") {
			return src.replace(/_[^_/.]*\.([^/.]*)$/, ".$1");
		}

		if ((domain === "pbs.twimg.com" &&
			 /:\/\/[^/]+\/+(?:media|(?:card|ad|semantic_core)_img|ext_tw_video_thumb)\//.test(src)) ||
			(domain === "ton.twitter.com" &&
			 string_indexof(src, "/ton/data/dm/") >= 0)) {


			obj = {
				url: src
			};

			if (options && options.element && host_domain_nosub === "twitter.com") {
				var caption = common_functions.get_twitter_caption(options.element);
				if (caption) {
					obj.extra = {caption: caption};
				}
			}

			newsrc = src
				.replace(/:([^/?]+)(.*)?$/, "$2?name=$1")
				.replace(/(\?.*)\?name=/, "$1&name=");
			if (newsrc !== src) {
				obj.url = newsrc;
				return obj;
			}

			if (!(/\/(?:card|ad|semantic_core)_img\//.test(src))) {
				newsrc = src
					.replace(/(\/[^/.?]+)\?(.*?&)?format=([^&]*)(.*?$)?/, "$1.$3?$2$4")
					.replace(/\?&/, "?")
					.replace(/[?&]+$/, "");

				if (newsrc !== src) {
					obj.url = newsrc;
					return obj;
				}
			}

			var names = ["orig", "4096x4096", "large", "medium"];
			var baseobj = obj;
			obj = [];

			var name = src.match(/[?&]name=([^&]+)/);
			if (name)
				name = name[1];
			else
				name = null;

			var end = array_indexof(names, name);
			if (end < 0)
				end = names.length;

			var obj_add = function(x) {
				var newobj = deepcopy(baseobj);
				newobj.url = x;

				obj.push(newobj);
			};

			for (var i = 0; i < end; i++) {
				newsrc = src
					.replace(/(\.[a-z]+)\?(?:(.*)&)?format=[^&]+/, "$1?$2&")
					.replace(/&$/, "");

				newsrc = add_queries(newsrc, {name: names[i]});

				if (false) {
					newsrc = src.replace(/([?&]name=)[^&]+(&.*)?$/, "$1" + names[i] + "$2");
					if (newsrc === src)
						newsrc = src
							.replace(/(\?.*)?$/, "$1?name=" + names[i])
							.replace(/(\?.*)\?name=/, "$1&name=");
				}

				if (newsrc.match(/\.png(\?.*)?$/)) {
					obj_add(newsrc);
					obj_add(newsrc.replace(/\.png(\?.*)?$/, ".jpg$1"));

					/*if (newsrc_colon !== newsrc) {
						obj_add(newsrc_colon);
						obj_add(newsrc_colon.replace(/\.png(\?.*)?$/, ".jpg$1"));
					}*/
				} else if (newsrc.match(/\.jpg(\?.*)?$/)) {
					obj_add(newsrc.replace(/\.jpg(\?.*)?$/, ".png$1"));
					obj_add(newsrc);

					/*if (newsrc_colon !== newsrc) {
						obj_add(newsrc_colon.replace(/\.jpg(\?.*)?$/, ".png$1"));
						obj_add(newsrc_colon);
					}*/
				} else {
					obj_add(newsrc);
				}
			}

			obj_add(src);

			return obj;
		}

		if (domain === "pbs.twimg.com" &&
			string_indexof(src, "pbs.twimg.com/profile_images/") >= 0) {
			newsrc = src
				.replace(/[?#].*$/, "")
				.replace(/_bigger\.([^/_]*)$/, "\.$1")
				.replace(/_normal\.([^/_]*)$/, "\.$1")
				.replace(/_mini\.([^/_]*)$/, "\.$1")
				.replace(/_reasonably_small\.([^/_]*)$/, "\.$1")
				.replace(/_[0-9]+x[0-9]+\.([^/_]*)$/, "\.$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "pbs.twimg.com" &&
			string_indexof(src, "pbs.twimg.com/profile_banners/") >= 0) {
			newsrc = src.replace(/\/[0-9]+x[0-9]+$/, "");
			if (newsrc !== src)
				return newsrc;
		}

		if (host_domain_nosub === "twitter.com" && options && options.element) {
			var caption = common_functions.get_twitter_caption(options.element);
			if (caption) {
				return {
					url: src,
					extra: {
						caption: caption
					}
				};
			}
		}

		if (host_domain_nosub === "youtube.com" && options.element) {
			if (options.element.tagName.toLowerCase() === "svg" && options.element.parentElement.tagName.toLowerCase() === "yt-icon") {
				if (options.element.parentElement.hasAttribute("icon"))
					return {
						url: origsrc,
						bad: "mask"
					};
			}
		}

		if (domain_nowww === "youtube.com" || domain_nowww === "youtu.be") {
			match = src.match(/^[a-z]+:\/\/(?:www\.)?youtube\.com\/+watch\?(?:.*&)?v=([^&#]*)/);
			if (!match) {
				match = src.match(/^[a-z]+:\/\/(?:www\.)?youtu\.be\/+([^?&#]*)/);
			}

			if (match) {
				return "https://i.ytimg.com/vi/" + match[1] + "/mqdefault.jpg";
			}
		}

		if (domain === "ytimg.googleusercontent.com" ||
			(domain_nosub === "ytimg.com" && domain.match(/^i[0-9]*\./)) ||
			domain === "img.youtube.com") {

			obj = {
				problems: {
					possibly_different: false
				}
			};

			match = src.match(/\/(?:vi|an)(?:_webp)?\/+([^/]+)\/+[^/]+\.[^/.]+(?:[?#].*)?$/);
			id = null;
			if (match) {
				obj.extra = {page: "https://www.youtube.com/watch?v=" + match[1]};
				id = match[1];
			}

			newsrc = src.replace(/^([a-z]+:\/\/)i[0-9]+(\.ytimg\.com\/vi(?:_webp)?\/+[^/]+\/+[a-z]+\.)/, "$1i$2");
			if (newsrc !== src) {
				obj.url = newsrc;
				return obj;
			}

			regex = /^(.+\/+vi(?:_webp)?\/+[^/]*\/+)[a-z]*(default|[0-9]+)(?:_(?:live|[0-9]+s))?(\.[^/.?#]*)(?:[?#].*)?$/;
			if (regex.test(src) /*&& !problem_excluded("possibly_different")*/) {
				var sizes = [
					"maxres",
					"sd",
					"hq",
					"mq"
				];

				var urls = []
				for (var i = 0; i < sizes.length; i++) {
					var match = src.match(regex);

					var number = match[2];
					if (number.length > 1 || number === "0") {
						number = "default";
					}

					newsrc = match[1] + sizes[i] + number + match[3];

					if (newsrc !== src) {
						urls.push(newsrc);
					} else {
						break;
					}
				}

				if (urls.length > 0)
					return fillobj_urls(urls, obj);
			}

			if (id && options && options.do_request && options.cb) {
				var parse_player_response = function(player_response) {
					if (!player_response) {
						return options.cb(null);
					}

					var maxbitrate = 0;
					var maxobj = null;

					var formats = player_response.streamingData.formats;

					for (var j = 0; j < formats.length; j++) {
						var our_format = formats[j];

						if (our_format.bitrate > maxbitrate) {
							maxbitrate = our_format.bitrate;
							maxobj = our_format;
						}
					}

					obj.problems.possibly_different = false;

					if (maxobj) {
						if (maxobj.url) {
							obj.url = maxobj.url;
							obj.video = true;
							obj.is_private = true;
							obj.headers = {
								Referer: "https://www.youtube.com/"
							};

							return options.cb(obj);
						} else {
							if (maxobj.cipher) {
								console_warn("Encrypted Youtube URLs are currently not supported, sorry");
							} else {
								console_error("Unknown streamingData object", maxobj);
							}

							return options.cb(null);
						}
					} else {
						console_error("Unable to find any formats", data);
						return options.cb(null);
					}
				};

				var fetch_youtube_embed = function(id, cb) {
					var cache_key = "youtube_embed_info:" + id;
					api_cache.fetch(cache_key, cb, function(done) {
						options.do_request({
							url: "https://www.youtube.com/get_video_info?video_id=" + id,
							method: "GET",
							onload: function(resp) {
								if (resp.readyState !== 4)
									return;

								if (resp.status !== 200) {
									console_error(resp);
									return done(null, false);
								}

								var splitted = resp.responseText.split("&");
								var found_player_response = false;
								for (var i = 0; i < splitted.length; i++) {
									if (/^player_response=/.test(splitted[i])) {
										found_player_response = true;
										var data = decodeURIComponent(splitted[i].replace(/^[^=]+=/, ""));

										try {
											var json = JSON_parse(data);

											var formats = json.streamingData.formats; // just to make sure it exists
											return done(json, 5*60*60); // video URLs expire in 6 hours
										} catch (e) {
											console_error(e, resp, splitted[i], data);
										}

										break;
									}
								}

								if (!found_player_response) {
									console_error("Unable to find player_response in", splitted[i]);
								}

								done(null, false);
							}
						});
					});
				};

				var fetch_youtube_watchpage = function(id, cb) {
					var cache_key = "youtube_watchpage:" + id;
					api_cache.fetch(cache_key, cb, function(done) {
						options.do_request({
							method: "GET",
							url: "https://www.youtube.com/watch?v=" + id,
							onload: function(resp) {
								if (resp.readyState !== 4)
									return;

								if (resp.status !== 200) {
									console_error(resp);
									return done(null, false);
								}

								var match = resp.responseText.match(/ytplayer\.config\s*=\s*({.*?});/);
								if (!match) {
									console_warn("Unable to find ytplayer.config for", resp);
									return done(null, false);
								}

								try {
									var json = JSON_parse(match[1]);
									var player_response = json.args.player_response;
									var player_response_json = JSON_parse(player_response);
									done(player_response_json, 5*60*60); // video URLs expire in 6 hours
								} catch (e) {
									console_error(e, match[1]);
								}
							}
						});
					});
				};

				fetch_youtube_embed(id, function(data) {
					if (!data) {
						fetch_youtube_watchpage(id, parse_player_response);
					} else {
						parse_player_response(data);
					}
				});

				return {
					waiting: true
				};
			}
		}

		if (domain === "images.attvideo.com") {
			match = src.match(/^[a-z]+:\/\/[^/]*\/+image\/+([^/]+)\/+[^/]*(?:[?#].*)?$/);
			if (match) {
				return "https://i.ytimg.com/vi/" + match[1] + "/mqdefault.jpg";
			}
		}

		if (domain_nowww === "phim24h.vn") {
			newsrc = src.replace(/\/5yphoto\/+thumb-[0-9]+\/+/, "/5yphoto/");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/^[a-z]+:\/\/[^/]*\/+5yphoto\/+([^/]+)\/+[^/]+\.[^/.]+(?:[?#].*)?$/);
			if (match) {
				return "https://i.ytimg.com/vi/" + match[1] + "/mqdefault.jpg";
			}
		}

		if (domain === "zeroinger.herokuapp.com" ||
			domain_nowww === "you2php.me") {
			if (/^[a-z]+:\/\/[^/]*\/+thumbnail\.php\?/.test(src)) {
				var vid = url.searchParams.get("vid");
				var type = url.searchParams.get("type");
				if (!type)
					type = "mqdefault";

				return "https://i.ytimg.com/vi/" + vid + "/" + type + ".jpg";
			}
		}

		if (domain_nowww === "wapinda.in") {
			return src.replace(/.*\/vimg\.php\?(?:.*?&)?v=([^&]*).*?$/, "https://i.ytimg.com/vi/$1/mqdefault.jpg");
		}

		if (domain_nowww === "invidio.us") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+(vi\/+.*?)(?:[?#].*)?$/, "https://i.ytimg.com/$1");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+ggpht\/+(.*?)(?:[?#].*)?$/, "https://lh4.ggpht.com/$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "image.bugsm.co.kr") {
			return src.replace(/\/images\/[0-9]*\//, "/images/original/").replace(/\?.*$/, "");
		}

		if (domain_nosub === "wp.com" &&
			domain.match(/i[0-9]\.wp\.com/)) {
			newsrc = remove_queries(src, ["w", "h", "resize", "zoom", "quality", "strip"]);
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/^[a-z]+:\/\/i[0-9]*\.wp\.com\/(.*?)(?:\?.*)?$/, "$1");
			if (newsrc !== src) {
				if (src.match(/[?&]ssl=1(?:&.*)?$/))
					return "https://" + newsrc;
				else
					return "http://" + newsrc;
			}
		}


		if (domain === "img.voi.pmdstatic.net" ||

			domain === "voi.img.pmdstatic.net") {
			var base = src.replace(/.*\/fit\/([^/]*)\/.*/, "$1");
			base = base.replace(/\./g, "%");
			base = decodeURIComponent(base);
			return base;
		}

		if (domain === "dynaimage.cdn.cnn.com") {
			return decodeURIComponent(src.replace(/.*\/cnn\/[^/]*\//, ""));
		}

		if (domain === "wcmimages.ottawasun.com" ||
			domain === "wcmimages.torontosun.com" ||
			domain === "wcmimages.winnipegsun.com" ||
			domain === "wcmimages.edmontonjournal.com" ||
			src.match(/^[a-z]+:\/\/wcmimages\.[^/]*\/images\?url=http/)) {
			newsrc = decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/images.*?[?&]url=([^&]*).*/, "$1"));
			if (newsrc !== src)
				return newsrc;
		}


		if (host_domain_nosub === "yahoo.com" && string_indexof(host_domain, "search.") >= 0 && options.element) {
			if (options.element.parentElement && options.element.parentElement.tagName === "A") {
				match = options.element.parentElement.href.match(/^[a-z]+:\/\/[^/]*images\.search\.[^/]+\/+(?:search\/+images|images\/+view)(?:;.*?)\?(?:.*&)?imgurl=([^&]+)/);
				if (match) {
					newsrc = add_http(decodeURIComponent(match[1]));
					if (newsrc !== src)
						return newsrc;
				}
			}
		}

		if ((domain_nosub === "yimg.com" && domain.match(/^(?:[sl]|ct)[0-9]*\.yimg\.com/)) ||
		   domain_nosub === "yahoo.com.tw") {
			return src
				.replace(/.*\/[^/]*\/api\/res\/[^/]*\/[^/]*\/[^/]*\/(.*?)(?:\.cf\.(?:jpg|webp))?$/, "$1")
				.replace(/^([a-z]*:\/)([^/])/, "$1/$2")
				.replace(/(:\/\/[^/]*\/)x\/+r\/+[wh][0-9]+\/+i\/+/, "$1i/");
		}


		if (domain === "image.iol.co.za") {
			return decodeURIComponent(src.replace(/.*\/process\/.*\?.*source=([^&]*).*/, "$1"));
		}


		if (domain_nosub === "squarespace.com" &&
			domain.match(/^static[0-9]*\.squarespace\.com/)) {
			newsrc = src.replace(/(?:\?.*)?$/, "?format=original");
			if (newsrc !== src)
				return {
					url: newsrc,
					head_wrong_contentlength: true
				};
		}

		if (domain === "images.squarespace-cdn.com") {

			var contenttype = url.searchParams.get("content-type");
			var append = "";
			if (contenttype) {
				append = "content-type=" + encodeURIComponent(decodeURIComponent(contenttype));
			} else {
				append = "";
			}

			var aappend = append ? "&" + append : "";
			var qappend = append ? "?" + append : "";

			newsrc = src
				.replace(/\?.*/, "")
				.replace(/\/+[0-9]+w$/, "");

			return [
				newsrc + "?format=original" + aappend,
				newsrc + "?format=2500w" + aappend,
				newsrc + qappend
			];
		}

		if ((domain_nosub === "wordpress.com" && string_indexof(domain, ".files.wordpress.com") >= 0) ||

			((domain_nosub === "imgix.net" ||
			  domain === "imgix.bustle.com" ||
			  domain === "imgix.ovp.tv2.dk" ||
			  domain === "imgix.elitedaily.com" ||
			  domain === "imgix.thezoereport.com" ||
			  domain === "cdn-imgix-open.headout.com" ||
			  domain === "images.assettype.com" ||
			  domain === "images.seoghoer.dk" ||
			  (domain === "static.camp-fire.jp" && string_indexof(src, "/uploads/") >= 0) ||
			  domain === "imgix.romper.com") &&
			 !src.match(/[?&]s=[^/]*$/)) ||
			domain === "stream-cloud-uploads.imgix.net" ||

			amazon_container === "hmg-prod" ||
			domain === "blogs-images.forbes.com" ||
			domain === "images-production.global.ssl.fastly.net" ||
			domain === "images-production.freetls.fastly.net" ||
			domain_nosub === "cdnds.net" ||
			/*domain.indexOf("hbz.h-cdn.co") >= 0 ||
			  string_indexof(domain, "cos.h-cdn.co") >= 0 ||*/
			domain_nosub === "h-cdn.co" ||
			domain === "cdn.newsapi.com.au" ||
			domain === "images.indianexpress.com" ||
			domain === "images.contentful.com" ||
			domain === "imagesmtv-a.akamaihd.net" ||
			domain === "d.ibtimes.co.uk" ||
			domain === "akns-images.eonline.com" ||
			/*domain.indexOf("www.telegraph.co.uk") >= 0 ||
			domain === "subscriber.telegraph.co.uk" ||
			string_indexof(domain, "aws.telegraph.co.uk") >= 0 ||*/
			(domain_nosub === "telegraph.co.uk" && string_indexof(src, "/content/dam/") >= 0) ||
			domain === "img.buzzfeed.com" ||
			(domain_nosub === "126.net" && domain.match(/^p[0-9]*\.music\.126\.net/)) ||
			domain === "stat.profile.ameba.jp" ||
			domain === "stat.blogskin.ameba.jp" ||
			domain === "stat.ameba.jp" ||
			domain === "image.uczzd.cn" ||
			domain === "img.danawa.com" ||
			domain === "img-www.tf-cdn.com"||
			domain_nosub === "viki.io" ||
			(domain_nosub === "githubusercontent.com" && domain.match(/^avatars[0-9]*\./)) ||
			(domain_nosub === "townnews.com" && domain.match(/bloximages\..*vip\.townnews\.com/)) ||
			domain === "asset.wsj.net" ||
			domain === "images.wsj.net" ||
			domain === "steamuserimages-a.akamaihd.net" ||
			(domain_nosub === "pressassociation.io" && domain.match(/\.assets\.pressassociation\.io$/)) ||
			domain === "media.immediate.co.uk" ||
			domain === "images.immediate.co.uk" ||
			domain === "media.npr.org" ||
			domain === "assets.teamrock.com" ||
			(domain_nosub === "woopic.com" && domain.match(/^media[0-9]\.woopic\.com/)) ||
			(domain_nosub === "libe.com" && domain.match(/^md[0-9]\.libe\.com/)) ||
			domain === "medias.liberation.fr" ||
			domain === "regmedia.co.uk" ||
			domain === "imageservice.nordjyske.dk" ||
			(domain === "cms.algoafm.co.za" && string_indexof(src, "/img/") >= 0) ||
			(domain_nowww === "gosoutheast.com" && string_indexof(src, "/images/") >= 0) ||
			(domain_nosub === "nikkeibp.co.jp" && src.toLowerCase().match(/\.(?:jpg|png)/)) ||
			(domain_nosub === "best.gg" && domain.match(/s-a[0-9]*\.best\.gg/)) ||
			(domain_nosub === "walmartimages.com" && domain.match(/i[0-9]*\.walmartimages\.com/)) ||
			domain === "nails.newsela.com" ||
			domain === "ojsfile.ohmynews.com" ||
			domain === "lumiere-a.akamaihd.net" ||
			domain === "img.lum.dolimg.com" ||
			domain_nowww === "xxlmag.com" ||
			(domain_nosub === "nyt.com" && domain.match(/^static[0-9]*\.nyt\.com/)) ||
			domain === "image.insider.com" ||
			domain === "i.insider.com" ||
			domain === "image.businessinsider.com" ||
			(domain_nosub === "vice.com" && domain.match(/images\.vice\.com$/)) ||
			(domain === "i.imgur.com" && !src.match(/\?[0-9]+$/)) ||
			domain === "media.discordapp.net" ||
			domain === "images.discordapp.net" ||
			domain === "images.theconversation.com" ||
			(domain_nowww === "rspb.org.uk" && string_indexof(src, "/globalassets/") >= 0) ||
			(domain === "media.beliefnet.com" && (/\/media\//i).test(src)) ||
			(domain_nowww === "ramblers.org.uk" && (/\/media\//i).test(src)) ||
			(domain === "warnerbrosuk.azureedge.net" && (/\/media\//i).test(src)) |
			(domain_nowww === "warnerbros.co.uk" && (/\/media\//i).test(src)) |
			(domain_nosub === "vietnamnet.vn" && /^(?:[^/]*\.)?img(?:\.cdn[0-9]*|s)\./.test(domain)) ||
			domain === "i.gadgets360cdn.com" ||
			domain_nosub === "ndtvimg.com" ||
			domain === "d3lp4xedbqa8a5.cloudfront.net" ||
			(domain_nowww === "brnow.org" && string_indexof(src, "/getattachment/") >= 0) ||
			(domain_nosub === "btime.com" && domain.match(/p[0-9]*\.(?:ssl\.)?cdn\.btime\.com/)) ||
			domain === "images.twistmagazine.com" ||
			domain === "sites.google.com" ||
			domain === "images.pexels.com" ||
			domain === "images.unsplash.com" ||
			domain_nosub === "r10s.com" ||
			domain_nosub === "r10s.jp" ||
			domain === "static.netlife.vn" ||
			domain === "rs.phunuonline.com.vn" ||
			domain === "images.nbcolympics.com" ||
			(domain === "dist.joshinweb.jp" && string_indexof(src, "/img/") >= 0) ||
			domain === "compote.slate.com" ||
			domain_nosub === "gannett-cdn.com" ||
			(domain_nowww === "rdfm-radio.fr" && string_indexof(src, "/medias/") >= 0) ||
			(domain === "salleobscure.e-monsite.com" && string_indexof(src, "/medias/") >= 0) ||
			domain === "image-api.nrj.fr" ||
			domain === "api.hdwallpapers5k.com" ||
			(domain_nosub === "koreaportal.com" && domain.match(/images\.[^.]*\.koreaportal\.com/)) ||
			(domain_nowww === "officialcharts.com" && string_indexof(src, "/media/") >= 0) ||
			(domain_nowww === "citywonders.com" && string_indexof(src, "/media/") >= 0) ||
			(domain_nowww === "fee.org" && string_indexof(src, "/media/") >= 0) ||
			(domain_nosub === "agoda.net" && domain.match(/pix[0-9]*\.agoda\.net/)) ||
			domain === "images.pottermore.com" ||
			(domain_nowww === "google.com" && src.match(/\/photos\/public\/[^/]*$/)) ||
			domain === "images.streamable.com" ||
			domain === "cdn.amebaowndme.com" ||
			(domain_nowww === "kaixian.tv" && string_indexof(src, "/file/") >= 0) ||
			(domain === "sumo.cdn.tv2.no" && string_indexof(src, "/imageapi/") >= 0) ||
			(domain_nowww === "bzqzsj.com" && string_indexof(src, "/uploads/") >= 0) ||
			domain === "media.missguided.com" ||
			domain === "photo.venus.com" ||
			domain === "cdn-images.prettylittlething.com" ||
			(domain_nowww === "oxfordmail.co.uk" && string_indexof(src, "/resources/") >= 0) ||
			(domain_nowww === "popcrush.com" && string_indexof(src, "/files/") >= 0) ||
			(domain_nowww === "screencrush.com" && string_indexof(src, "/files/") >= 0) ||
			(domain_nosub === "nineentertainment.com.au" && domain.match(/assets\.[^.]*\.nineentertainment\.com\.au/)) ||
			(domain_nowww === "thenational.ae" && string_indexof(src, "/image/") >= 0) ||
			(domain_nosub === "kh1.co" && domain.match(/s[0-9]*\.kh1\.co/)) ||
			domain === "uploads.disquscdn.com" ||
			(domain_nowww === "voidu.com" && string_indexof(src, "/gallery/") >= 0) ||
			(domain === "store.playstation.com" && string_indexof(src, "/image?") >= 0) ||
			(domain_nosub === "playstation.com" && /^blog\./.test(domain)) ||
			domain === "images.interactives.dk" ||
			domain === "toyo-arhxo0vh6d1oh9i0c.stackpathdns.com" ||
			(domain_nowww === "zmonline.com" && string_indexof(src, "/media/") >= 0) ||
			domain === "img-s-msn-com.akamaized.net" ||
			(domain_nowww === "heraldscotland.com" && string_indexof(src, "/resources/images/") >= 0) ||
			(domain_nowww === "theboltonnews.co.uk" && string_indexof(src, "/resources/images/") >= 0) ||
			domain === "cdn.instructables.com" ||
			domain === "images.performgroup.com" ||
			domain === "media.playmobil.com" ||
			domain === "img.crocdn.co.uk" ||
			(domain_nowww === "calgaryherald.com" && string_indexof(src, "/cms/") >= 0) ||
			(domain_nowww === "montrealgazette.com" && string_indexof(src, "/cms/") >= 0) ||
			(domain_nosub === "ikea.com" && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "colorwallpaper.net" && string_indexof(src, "/img/") >= 0) ||
			domain === "img.cache.vevo.com" ||
			(domain === "drop.ndtv.com" && string_indexof(src, "/albums/") >= 0) ||
			(domain_nosub === "christiandaily.com" && string_indexof(domain, "images.christianitydaily.com") >= 0) ||
			domain === "cdn.christianitydaily.com" ||
			(domain_nosub === "christiandaily.co.kr" && string_indexof(domain, "images.christiandaily.co.kr") >= 0) ||
			domain === "images.christiantoday.co.kr" ||
			domain === "blogimg.goo.ne.jp" ||
			domain === "cdn.clien.net" ||
			(domain_nosub === "imgs.cc" && domain.match(/s[0-9]*\.imgs\.cc/)) ||
			(domain_nosub === "amebame.com" && string_indexof(domain, "stat.amebame.com") >= 0) ||
			domain === "i.iheart.com" ||
			domain === "cdn-hit.scadigital.io" ||
			(domain_nosub === "picsart.com" && domain.match(/cdn[0-9]*\.picsart\.com/)) ||
			domain === "cdn.ndtv.com" ||
			(domain_nosub === "kompasiana.com" && domain.match(/assets(?:-[a-z][0-9])?\.kompasiana\.com/)) ||
			domain === "images.popbuzz.com" ||
			(domain_nosub === "adis.ws" && domain.match(/i[0-9]*\.adis\.ws/)) ||
			(domain_nosub === "9c9media.com" && domain.match(/^images[0-9]*\.9c9media\.com/)) ||
			(domain_nosub === "newser.com" && domain.match(/img[0-9]*(?:-[a-z]+)?\.newser\.com/)) ||
			domain === "images.m-magazine.com" ||
			(domain_nowww === "zoom.co.uk" && src.match(/\/assets\/+images\/+/)) ||
			domain === "ctd-thechristianpost.netdna-ssl.com" ||
			domain === "img.vidible.tv" ||
			(domain_nosub === "j-14.com" && domain.match(/images\.(?:[a-z]+\.)?j-14\.com/)) ||
			domain === "cdn.abcotvs.com" ||
			(domain_nowww === "tasteofcountry.com" && string_indexof(src, "/files/") >= 0) ||
			domain === "images.thewest.com.au" ||
			(domain_nosub === "ntv.com.tr" && domain.match(/cdn[0-9]*\.ntv\.com\.tr/)) ||
			(domain_nosub === "hotnessrater.com" && domain.match(/img[0-9]*\.hotnessrater\.com/)) ||
			domain_nowww === "starcrush.com" ||
			(domain_nowww === "chrichri.dk" && string_indexof(src, "/media/") >= 0) ||
			(domain_nowww === "rightstufanime.com" && string_indexof(src, "/images/") >= 0) ||
			domain === "assets.bigcartel.com" ||
			(domain_nowww === "thenorthernecho.co.uk" && string_indexof(src, "/resources/images/") >= 0) ||
			domain === "binaryapi.ap.org" ||
			domain === "images.8tracks.com" ||
			(domain_nosub === "spoilercat.com" && domain.match(/cdn[0-9]*\.spoilercat\.com/)) ||
			(domain_nowww === "mumbailive.com" && string_indexof(src, "/images/") >= 0) ||
			domain === "static.juksy.com" ||
			domain === "img.reblog.hu" ||
			(domain_nowww === "thefw.com" && string_indexof(src, "/files/") >= 0) ||
			domain === "img.csfd.cz" ||
			(domain_nosub === "timesnownews.com" && /^img/.test(domain)) ||
			domain === "img.siksinhot.com" ||
			domain === "mp-seoul-image-production-s3.mangoplate.com" ||
			domain === "api.theweek.com" ||
			domain === "i.smalljoys.me" ||
			(domain_nosub === "huanqiu.cn" && domain.match(/t[0-9]*\.huanqiu.cn/)) ||
			(domain_nowww === "hypebeast.com" && string_indexof(src, "/image/") >= 0) ||
			(domain_nowww === "dramaguru.net" && string_indexof(src, "/images/") >= 0) ||
			domain === "sos.vfan.vlive.tv" ||
			(domain_nosub === "journalmedia.ie" && domain.match(/^img[0-9]*\./)) ||
			(domain_nosub === "thejournal.ie" && domain.match(/^img[0-9]*\./)) ||
			(domain === "external.polskieradio.pl" && string_indexof(src, "/files/") >= 0) ||
			domain === "thumb.netz.id" ||
			(domain_nosub === "tchyn.io" && string_indexof(src, "/snopes-production/uploads/") >= 0) ||
			domain === "img.diply.com" ||
			domain === "img-mdpr.freetls.fastly.net" ||
			domain === "cdn-assets.ziniopro.com" ||
			domain === "netherlands-grlk5lagedl.stackpathdns.com" ||
			(domain_nosub === "stackpathdns.com" && string_indexof(domain, "-grlk5lagedl.stackpathdns.com") >= 0) ||
			(domain_nowww === "beautycrew.com.au" && string_indexof(src, "/media/") >= 0) ||
			domain_nosub === "mtvnimages.com" ||
			(domain === "dsx.weather.com" && string_indexof(src, "/util/image/") >= 0) ||
			domain === "img.r7.com" ||
			domain === "cdn-images.rtp.pt" ||
			domain === "media.ouest-france.fr" ||
			(domain_nowww === "rbsdirect.com.br" && string_indexof(src, "/imagesrc/") >= 0) ||
			(domain_nowww === "newfoundlandlabrador.com" && string_indexof(src, "/-/media/") >= 0) ||
			domain === "images.businessoffashion.com" ||
			(domain_nosub === "pg0.cn" && domain.match(/cmsfile\.pg0\.cn/)) ||
			domain === "api.ning.com" ||
			(domain_nosub === "mpinteractiv.ro" && domain.match(/^storage[0-9]*\./) && string_indexof(src, "/media/") >= 0) ||
			(domain_nosub === "muscache.com" && src.match(/\/im\/pictures\/[-0-9a-f]+\./)) ||
			(domain_nowww === "wedd.today" && string_indexof(src, "/wallpaper/") >= 0) ||
			(domain_nowww === "looklive.at" && src.match(/\/media\/[0-9]+\//)) ||
			domain === "t.tudocdn.net" ||
			(domain === "me.phununet.com" && string_indexof(src, "/resources/img/") >= 0) ||
			(domain === "cdn.diario26.com.ar" && string_indexof(src, "/media/image/") >= 0) ||
			(domain_nowww === "discovermagazine.com" && string_indexof(src, "/media/Images/") >= 0) ||
			domain === "steamusercontent-a.akamaihd.net" ||
			(domain_nowww === "qfeast.com" && string_indexof(src, "/imret/") >= 0) ||
			(domain === "statis.gamen.vn" && string_indexof(src, "/images/upload/") >= 0) ||
			domain === "img.anikore.jp" ||
			(domain_nosub === "newsplex.pt" && domain.match(/^cdn[0-9]*\./)) ||
			(domain === "static.origos.hu" && string_indexof(src, "/img/") >= 0) ||
			(domain === "cdn.nwmgroups.hu" && string_indexof(src, "/img/") >= 0) ||
			(domain_nowww === "robertocavada.com" && string_indexof(src, "/Images/") >= 0) ||
			domain === "images.mncdn.pl" ||
			(domain_nowww === "imgsearches.com" && string_indexof(src, "/i/") >= 0) ||
			(domain_nowww === "thetimesnews.com" && (
				string_indexof(src, "/storyimage/") >= 0 ||
					string_indexof(src, "/galleryimage/") >= 0)) ||
			domain === "static.fthis.gr" ||
			(domain === "img.pixelz.com" && string_indexof(src, "/blog/") >= 0) ||
			(domain_nosub === "hellomagazine.com" && (string_indexof(src, "/imagenes/") >= 0 || string_indexof(src, "/images/") >= 0)) ||
			(domain_nosub === "1616.ro" && domain.match(/^i[0-9]*\./)) ||
			domain === "images.lifeandstylemag.com" ||
			domain === "img.freepik.com" ||
			(domain_nosub === "fonwall.ru" && domain.match(/^img[0-9]*\./)) ||
			(domain_nowww === "wonderwall.com" && string_indexof(src, "/photos/") >= 0) ||
			(domain_nowww === "thestar.com.my" && string_indexof(src, "/~/media/online/") >= 0) ||
			(domain_nowww === "vancouversun.com" && string_indexof(src, "/cms/") >= 0) ||
			domain === "imageproxy.viewbook.com" ||
			(domain === "image.lag.vn" && string_indexof(src, "/upload/") >= 0) ||
			(domain_nowww === "booktrust.org.uk" && string_indexof(src, "/globalassets/images/") >= 0) ||
			(domain_nosub === "ccio.co" && domain.match(/^ind[0-9]*\./)) ||
			(domain_nosub === "cpcache.com" && domain.match(/^i[0-9]*\./)) ||
			domain === "i.rocdn.com" ||
			(domain === "resource.globenewswire.com" && string_indexof(src, "/Resource/Download/") >= 0) ||
			(domain_nowww === "townsquare.media" && src.match(/\/+site\/+[0-9]+\/+files\/+/)) ||
			domain === "images.reference.com" ||
			domain === "assets.pcmag.com" ||
			domain === "images.jg-cdn.com" ||
			(domain_nosub === "qingstor.com" && string_indexof(src, "/images/articles/") >= 0) ||
			domain === "img.ibxk.com.br" ||
			(domain === "image.biccamera.com" && string_indexof(src, "/img/") >= 0) ||
			(domain_nosub === "mu-mo.net" && string_indexof(src, "/image/") >= 0) ||
			(domain_nowww === "g-mark.org" && string_indexof(src, "/media/") >= 0) ||
			((domain === "kubrick.htvapps.com" || amazon_container === "htv-prod-media") && string_indexof(src, "/images/") >= 0) ||
			domain === "images.france.fr" ||
			(domain === "az877327.vo.msecnd.net" && string_indexof(src, "/media/images/") >= 0) ||
			(domain_nosub === "hoteljardinlebrea.com" && string_indexof(src, "/usermedia/") >= 0) ||
			(domain_nosub === "veltra.com" && domain.match(/^cdn[0-9]*\./)) ||
			(domain === "img.letgo.com" && string_indexof(src, "/images/") >= 0) ||
			(domain === "images.musement.com" && string_indexof(src, "/cover/") >= 0) ||
			domain === "i.playground.ru" ||
			(domain_nowww === "treeoftheyear.org" && string_indexof(src, "/getmedia/") >= 0) ||
			(domain === "cdn.indicium.nu" && string_indexof(src, "/source/grazia/") >= 0) ||
			domain === "news-img.51y5.net" ||
			(domain === "st.automobilemag.com" && string_indexof(src, "/uploads/sites/") >= 0) ||
			(domain === "styles.redditmedia.com" && string_indexof(src, "/styles/") >= 0) ||
			domain === "img.webmd.com" ||
			domain === "embedwistia-a.akamaihd.net" ||
			amazon_container === "ame-prod-redonline-assets" ||
			(domain_nowww === "noted.co.nz" && string_indexof(src, "/media/") >= 0) ||
			domain === "image.cnbcfm.com" ||
			domain === "images.newrepublic.com" ||
			(domain_nowww === "madametussauds.com" && string_indexof(src, "/media/") >= 0) ||
			(domain_nowww === "tirolerin.at" && string_indexof(src, "/media/") >= 0) ||
			(domain === "media.deseretdigital.com" && string_indexof(src, "/file/") >= 0) ||
			domain === "assets.audiomack.com" ||
			domain === "pic.rutube.ru" ||
			(domain === "cdn.marketplaceimages.windowsphone.com" && string_indexof(src, "/images/") >= 0) ||
			((domain === "dsocdn.akamaized.net" ||
			 domain === "nbocdn.akamaized.net" ||
			 domain === "avecdn.akamaized.net" ||
			 domain === "static.hbvl.be" ||
			 domain === "static.standaard.be" ||
			  domain === "limnlcdn.akamaized.net") && /\/Assets\/+Images_Upload\//i.test(src)) ||
			(domain === "img.s-msn.com" && string_indexof(src, "/entityid/") >= 0) ||
			(domain === "netstorage-nur.akamaized.net" && string_indexof(src, "/images/") >= 0) ||
			domain === "images.moviebuff.com" ||
			domain === "static-ugc-media.hk01.com" ||
			(domain_nowww === "warwick.film" && string_indexof(src, "/image/") >= 0) ||
			(domain_nowww === "quizizz.com" && src.match(/\/media\/+resource\/+/)) ||
			(domain_nowww === "eroce.com" && string_indexof(src, "/img/") >= 0) ||
			(domain === "img-cdn.hipertextual.com" && string_indexof(src, "/files/") >= 0) ||
			(domain === "img.ponparemall.net" && string_indexof(src, "/imgmgr/") >= 0) ||
			(domain === "smart.usen.com" && string_indexof(src, "/data/") >= 0) ||
			domain === "img.digitaldjpool.com" ||
			(domain_nosub === "szn.cz" && domain.match(/\.sdn\./) && string_indexof(src, "_img_") >= 0) ||
			(domain_nowww === "banana1015.com" && string_indexof(src, "/files/") >= 0) ||
			(domain_nowww === "mix106radio.com" && string_indexof(src, "/files/") >= 0) ||
			(domain === "static-cdn.sr.se" && string_indexof(src, "/images/") >= 0) ||
			(domain === "cdn.kaumo.jp" && string_indexof(src, "/element/") >= 0) ||
			(domain === "d12swbtw719y4s.cloudfront.net" && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "saplinghr.com" && string_indexof(src, "/hubfs/") >= 0) ||
			(domain === "telefe-static2.akamaized.net" && string_indexof(src, "/media/") >= 0) ||
			domain_nowww === "10daily.com.au" ||
			(domain === "spark.adobe.com" && string_indexof(src, "/page/") >= 0) ||
			(domain === "s.w-x.co" && string_indexof(src, "/image/") >= 0) ||
			domain === "img.monocle.com" ||
			(domain === "bildix.mmcloud.se" && /\/bildix\/+api\/+images\/+/.test(src)) ||
			(domain === "media.cinedb.com.tr" && /\/Upload\//i.test(src)) ||
			(domain_nowww === "newidea.com.au" && string_indexof(src, "/media/") >= 0) ||
			(domain_nowww === "essexstudent.com" && /\/asset\//i.test(src)) ||
			(domain === "d.newsweek.com" && string_indexof(src, "/full/") >= 0) ||
			(domain === "images.techtimes.com" && /\/data\/+images\/+/.test(src)) ||
			domain === "oyster.ignimgs.com" ||
			(domain_nowww === "marieclaire.com.au" && string_indexof(src, "/media/") >= 0) ||
			(domain_nowww === "theargus.co.uk" && /\/resources\/+images\/+/.test(src)) ||
			(domain_nowww === "stylist.co.uk" && /\/images\/+app\/+uploads\/+/.test(src)) ||
			(domain_nowww === "britishfashioncouncil.co.uk" && string_indexof(src, "/uploads/") >= 0) ||
			(domain === "static.fptplay.net" && string_indexof(src, "/img/") >= 0) ||
			domain === "images-mega.mdstrm.com" ||
			(domain_nowww === "girlfriend.com.au" && string_indexof(src, "/media/") >= 0) ||
			domain === "imgs.ngaotruyen.com" ||
			(domain === "d1ywb8dvwodsnl.cloudfront.net" && /\/files\.fuzoku\.jp\/+img\//.test(src)) ||
			(domain_nowww === "menshealth.com.au" && string_indexof(src, "/media/") >= 0) ||
			(domain === "2sao.vietnamnetjsc.vn" && string_indexof(src, "/images/") >= 0) ||
			(domain === "images.jkn.co.kr" && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "instylemag.com.au" && string_indexof(src, "/media/") >= 0) ||
			domain === "i.eurosport.com" ||
			(domain === "blog.wirelessanalytics.com" && string_indexof(src, "/hubfs/") >= 0) ||
			(domain_nosub === "zmtcdn.com" && string_indexof(src, "/data/") >= 0) ||
			(domain === "images.mubi.com" && string_indexof(src, "/images/") >= 0) ||
			domain === "i.obozrevatel.com" ||
			domain === "1624909224.rsc.cdn77.org" ||
			(domain_nosub === "paytm.com" && /^assetscdn[0-9]*\./.test(domain)) ||
			domain === "images.perthnow.com.au" ||
			(domain === "netstorage-yen.akamaized.net" && string_indexof(src, "/images/") >= 0) ||
			(domain_nosub === "eldia.com" && /^cdn[0-9]*\./.test(domain)) ||
			(domain_nowww === "esquire.com.gr" && /\/Content\/+ImagesDatabase\//i.test(src)) ||
			domain === "akmedia.hollywoodlife.com" ||
			(domain_nowww === "refinery29.com" && string_indexof(src, "/images/") >= 0) ||
			(domain === "images.radio.com" && string_indexof(src, "/aiu-media/") >= 0) ||
			(domain_nowww === "casseycds.net" && string_indexof(src, "/uploads/") >= 0) ||
			(domain_nowww === "youmekorea.com" && /[?&]thumb=/.test(src)) ||
			(domain_nosub === "tower.jp" && /\/~\/+media\//.test(src)) ||
			domain === "images.boosty.to" ||
			domain === "media.nbcnewyork.com" ||
			domain_nosub === "sdn.cz" ||
			(domain_nosub === "unrealengine.com" && /^cdn[0-9]*\./.test(domain)) ||
			(domain === "cdn.repub.ch" && string_indexof(src, "/republik-assets/") >= 0) ||
			(domain_nowww === "smarthomebeginner.com" && string_indexof(src, "/images/") >= 0) ||
			domain === "images.ask.com" ||
			(domain === "i.vimeocdn.com" && string_indexof(src, "/video/") >= 0) ||
			src.match(/\/demandware\.static\//) ||
			src.match(/\?i10c=[^/]*$/) ||
			/^[a-z]+:\/\/[^?]*\/wp(?:-content\/+(?:uploads|blogs.dir)|\/+uploads)\//.test(src)
			/*src.indexOf("/wp-content/blogs.dir/") >= 0 ||
			string_indexof(src, "/wp-content/uploads/") >= 0 ||
			string_indexof(src, "/wp/uploads/") >= 0*/) {
			src = src.replace(/\?.*$/, "");
		}

		if (domain === "cdn.odigo.net" ||
			domain === "wangsuimg.fanshuapp.com" ||
			domain === "img.aiji66.com" ||
			domain === "imgs.aixifan.com" ||
			domain === "imgcdn.thecover.cn" ||
			domain === "img.shelive.net" ||
			domain === "img.qdaily.com" ||
			domain === "qimage.owhat.cn" ||
			domain === "cdn.ruguoapp.com" ||
			(domain === "user-assets.sxlcdn.com" && string_indexof(src, "/images/") >= 0) ||
			domain === "img.ksl.com" ||
			(domain_nosub === "tapimg.com" && domain.match(/^img[0-9]*\./)) ||
			domain === "7xka0y.com1.z0.glb.clouddn.com" ||
			(domain_nosub === "mafengwo.net" && domain.match(/^[a-z][0-9]*(?:-[a-z])?\./)) ||
			(domain_nosub === "myqcloud.com" && domain.match(/image\.myqcloud\.com/)) ||
			domain === "ci.xiaohongshu.com" ||
			domain === "pic.qianye88.com" ||
			(domain_nosub === "yohobuy.com" && domain.match(/^img[a-z]*[0-9]*\.yohobuy\.com$/)) ||
			(domain_nosub === "715083.com" && domain.match(/^i-[0-9]*-yxdown\./)) ||
			(domain_nosub === "yxdown.com" && domain.match(/^i-[0-9]*\./)) ||
			(domain === "jkcdn.pajk.com.cn" && string_indexof(src, "/image/") >= 0) ||
			(domain_nosub === "hola.com" && string_indexof(src, "/imagenes/") >= 0) ||
			(domain === "static.leiphone.com" && string_indexof(src, "/uploads/") >= 0) ||
			domain === "static-movie.a.88cdn.com" ||
			(domain === "image.qianye88.com" && string_indexof(src, "/pic/") >= 0) ||
			domain === "pc.wangpan.xycdn.n0808.com" ||
			domain === "resource.meihua.info" ||
			domain === "i.ezbuy.sg" ||
			domain === "i.guancha.cn" ||
			domain === "pic.xcarimg.com" ||
			domain === "upload-images.jianshu.io") {
			src = src.replace(/\?.*$/, "");
		}

		if (domain === "mtv.mtvnimages.com" ||
			(domain_nosub === "tradesy.com" && domain.match(/^item[0-9]*\.tradesy/) && string_indexof(src, "/images/") >= 0)) {
			return {
				url: src.replace(/\?.*$/, ""),
				can_head: false
			};
		}

		if (domain === "img-static.tradesy.com") {
			return src.replace(/:\/\/[^/]+\/+item\/+([0-9]+)\/+([^/]+)(-[0-9]+-[0-9]+)-[0-9]+-[0-9]+(\.[^/.?]+)(?:[?#].*)?$/, "://item4.tradesy.com/images/$2-$1$3$4");
		}

		if (domain === "store-images.microsoft.com" ||
			domain === "store-images.s-microsoft.com") {
			return {
				url: src.replace(/\?.*$/, ""),
				head_wrong_contenttype: true
			};
		}

		if (domain_nowww === "dailyherald.com") {
			newsrc = src.replace(/[?&].*$/, "");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nosub === "imimg.com" ||
			domain === "blogs-images.forbes.com" ||
			domain === "static.thesuperficial.com" ||
			domain === "static.celebuzz.com" ||
			domain === "img.vogue.co.kr" ||
			domain === "static.spin.com" ||
			domain_nowww === "zrockr.com" ||
			domain_nowww === "electricegg.co.uk" ||
			(domain_nowww === "iphone7wallpapers.co" && /\/media\/+uploads\//.test(src)) ||
			(domain_nowww === "androidhdwallpapers.com" && /\/media\/+uploads\//.test(src)) ||
			(domain_nosub === "hw-static.com" && domain.match(/www\.media[0-9]*\.hw-static\.com/)) ||
			(domain_nosub === "turner.com" && string_indexof(domain, ".cdn.turner.com") >= 0) ||
			domain_nowww === "k99.com" ||
			domain_nowww === "97rockonline.com" ||
			domain_nowww === "wfgr.com" ||
			domain_nowww === "wblk.com" ||
			domain_nowww === "fun107.com" ||
			domain_nowww === "965viki.com" ||
			domain_nowww === "1079ishot.com" ||
			domain === "edge.alluremedia.com.au" ||
			(amazon_container === "blogs-prod-media" && string_indexof(src, "/uploads/") >= 0) ||
			domain === "d36tnp772eyphs.cloudfront.net" ||
			domain === "img.allurekorea.com" ||
			domain_nowww === "psu.com" ||
			domain === "media.popculture.com" ||
			domain_nowww === "rap-up.com" ||
			(domain_nowww === "funweek.it" && string_indexof(src, "/uploads/") >= 0) ||
			(domain_nosub === "hankyung.com" && domain.match(/img\..*?\.hankyung\.com$/)) ||
			domain_nowww === "traveltipy.com" ||
			domain_nowww === "coveteur.com" ||
			(domain_nosub === "ehow.com" && string_indexof(domain, ".blog.ehow.com") >= 0) ||
			(domain_nosub === "theheartysoul.com" && domain.match(/cdn[0-9]*\.theheartysoul\.com/)) ||
			domain_nowww === "bandt.com.au" ||
			(domain_nosub === "theepochtimes.com" && src.match(/\/assets\/+uploads\/+/)) ||
			(domain_nosub === "gamme.com.tw" && /^images[0-9]*\./.test(domain)) ||
			domain === "images.gamme.com.cn" ||
			domain === "cdn.hoahoctro.vn" ||
			domain === "vnn-imgs-f.vgcloud.vn" ||
			domain === "static.vibe.com" ||
			(domain_nowww === "rightsinfo.org" && string_indexof(src, "/uploads/") >= 0) ||
			(domain_nowww === "spotted.tv" && string_indexof(src, "/uploads/") >= 0) ||
			domain === "img.marieclairekorea.com" ||
			(domain_nowww === "tokyopopline.com" && string_indexof(src, "/images/") >= 0) ||
			domain === "px1img.getnews.jp" ||
			domain === "media.thetab.com" ||
			domain === "assets.rockpapershotgun.com" ||
			(domain_nosub === "vietnamnet.vn" && /^(?:[^/]*\.)?img(?:\.cdn[0-9]*|s)\./.test(domain)) ||
			domain === "sloanreview.mit.edu" ||
			(domain_nosub === "inquirer.net" && string_indexof(src, "/files/") >= 0) ||
			domain === "static.thefrisky.com" ||
			domain === "hobby.dengeki.com" ||
			domain === "cdn-blog.adafruit.com" ||
			(domain_nosub === "uuhy.com" && domain.match(/s?img\.uuhy\.com/)) ||
			domain === "img.butongshe.com" ||
			(domain_nowww === "myreco.asia" && string_indexof(src, "/uploads/") >= 0) ||
			(domain === "d3p157427w54jq.cloudfront.net" && string_indexof(src, "/uploads/") >= 0) ||
			domain === "cdn.harpersbazaar.com.sg" ||
			domain === "media.harpersbazaar.com.sg" ||
			domain_nosub === "myconfinedspace.com" ||
			amazon_container === "hiphopdx-production" ||
			domain === "assets.wonderlandmagazine.com" ||
			(domain_nosub === "akamaized.net" && domain.match(/^am[0-9]*\./) && /\/tms\/+cnt\/+uploads\//.test(src)) ||
			(domain_nosub === "pressassociation.io" && string_indexof(domain, "static.pressassociation.io") >= 0) ||
			domain === "assets.vg247.com" ||
			(domain_nowww === "theblemish.com" && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "dailyxtra.com" && /\/content\/+uploads\//.test(src)) ||
			domain === "media.metrolatam.com" ||
			domain === "media.comicbook.com" ||
			domain_nowww === "grazia.it" ||
			domain === "img.kpopmap.com" ||
			domain === "s.nbst.gr" ||
			domain === "assets.metrolatam.com" ||
			domain_nosub === "mthai.com" ||
			domain === "cdn.webnoviny.sk" ||
			domain === "bloggar.expressen.se" ||
			(domain_nosub === "saostar.vn" && domain.match(/img[0-9]*\.saostar\.vn/)) ||
			(domain_nosub === "gossipcop.com" && domain.match(/s[0-9]*\.gossipcop\.com/)) ||
			(domain_nowww === "petapixel.com" && string_indexof(src, "/uploads/") >= 0) ||
			domain === "d3i6fh83elv35t.cloudfront.net" ||
			domain === "d17fnq9dkz9hgj.cloudfront.net" ||
			domain === "images.everyeye.it" ||
			googlestorage_container === "koreaboo-cdn" ||
			(domain_nowww === "behindzscene.net" && string_indexof(src, "/file/") >= 0) ||
			(domain_nowww === "hipertextual.com" && string_indexof(src, "/files/") >= 0) ||
			(domain_nowww === "geeksofdoom.com" && string_indexof(src, "/img/") >= 0) ||
			domain === "newsimages.fashionmodeldirectory.com" ||
			domain === "akm-img-a-in.tosshub.com" ||
			domain === "img.blogtamsu.vn" ||
			(domain_nosub === "starsdaily.net" && domain.match(/cdn[0-9]+\.starsdaily\.net/)) ||
			(domain_nowww === "nextnature.net" && string_indexof(src, "/uploads/") >= 0) ||
			(domain_nowww === "vooks.net" && string_indexof(src, "/img/") >= 0) ||
			domain === "i.ido.bi" ||
			domain === "cdn.techgyd.com" ||
			(domain_nowww === "4girls.co.il" && string_indexof(src, "/uploads/") >= 0) ||
			(domain_nowww === "gceleb.com" && string_indexof(src, "/photo/") >= 0) ||
			(domain_nowww === "seriouseats.com" && string_indexof(src, "/images/") >= 0) ||
			domain === "i.epochtimes.com" ||
			domain === "cdn.thammysen.vn" ||
			domain === "d1lofqbqbj927c.cloudfront.net" ||
			(domain_nowww === "180grados.com.mx" && string_indexof(src, "/img/") >= 0) ||
			domain === "photo.tin8.co" ||
			(domain_nosub === "whatculture.com" && domain.match(/^cdn[0-9]*\.whatculture\.com/) && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "electronicbeats.net" && string_indexof(src, "/uploads/") >= 0) ||
			domain === "media.iconsingapore.com" ||
			domain === "p.cosmopolitan.bg" ||
			domain === "cache.pakistantoday.com.pk" ||
			domain === "media.breitbart.com" ||
			domain === "s.rozali.com" ||
			domain === "s.sdgcdn.com" ||
			(domain === "ticket.heraldtribune.com" && string_indexof(src, "/files/") >= 0) ||
			domain === "blog.hola.com" ||
			(domain_nowww === "therussiantimes.com" && string_indexof(src, "/uploads/") >= 0) ||
			domain === "static.idolator.com" ||
			domain === "images.saloona.co.il" ||
			(domain_nowww === "bellezaenvena.com" && string_indexof(src, "/my_uploads/") >= 0) ||
			(domain_nosub === "thejournal.ie" && string_indexof(src, "/media/") >= 0) ||
			domain_nowww === "tvperson.ru" ||
			domain_nowww === "iwantpix.com" ||
			domain_nowww === "nudemodelpics.com" ||
			(domain === "images.chr.bg" && string_indexof(src, "/uploads/") >= 0) ||
			(domain_nowww === "woman.ua" && string_indexof(src, "/file/images/") >= 0) ||
			domain === "gdsit.cdn-immedia.net" ||
			domain_nowww === "cseditors.com" ||
			(domain_nowww === "imcdn.org" && string_indexof(src, "/uploads/") >= 0) ||
			domain === "t.a4vn.com" ||
			domain_nowww === "dodskypict.com" ||
			domain === "files.dals.media" ||
			domain === "cdn.crhoy.net" ||
			domain_nowww === "longwallpapers.com" ||
			domain === "files.greatermedia.com" ||
			(domain_nowww === "tsundora.com" && string_indexof(src, "/image/") >= 0) ||
			(domain_nosub === "twitch.tv" && domain.match(/^clips-media-assets[0-9]*\./) && src.match(/-preview-[0-9]+x[0-9]+\.[^/.]*$/)) ||
			(domain === "file.immo.vlan.be" && string_indexof(src, "/Image/Wordpress/") >= 0) ||
			domain === "media.trud.bg" ||
			(domain_nowww === "rotativo.com.mx" && string_indexof(src, "/assets/") >= 0) ||
			(domain_nowww === "chfi.com" && string_indexof(src, "/wp-content/") >= 0) ||
			googlestorage_container === "sin-cdn" ||
			domain === "cdn.appleigeek.com" ||
			domain === "media.cnnchile.com" ||
			(domain_nowww === "festivalteen.com.br" && string_indexof(src, "/uploads/") >= 0) ||
			(amazon_container === "rhodiesworld" && string_indexof(src, "/uploads/") >= 0) ||
			(domain_nowww === "ntdtv.kr" && string_indexof(src, "/assets/") >= 0) ||
			domain === "xdn.tf.rs" ||
			domain === "static.acg12.com" ||
			domain === "img.zone5.ru" ||
			(domain_nosub === "bssl.es" && domain.match(/^i[0-9]*\./)) ||
			domain === "images.virgula.com.br" ||
			(domain_nowww === "rotana.net" && string_indexof(src, "/assets/uploads/") >= 0) ||
			(domain === "elle.unitedinfluencers.org" && string_indexof(src, "/content/uploads/") >= 0) ||
			(domain_nowww === "notredamedeparis.fr" && string_indexof(src, "/content/uploads/") >= 0) ||
			(domain === "zenska.hudo.com" && string_indexof(src, "/files/") >= 0) ||
			(amazon_container === "seenitblog" && string_indexof(src, "/uploads/") >= 0) ||
			(domain_nowww === "marieclaire.hu" && string_indexof(src, "/uploads/") >= 0) ||
			(domain_nowww === "infinitymemories.com" && string_indexof(src, "/files/") >= 0) ||
			(domain_nowww === "tuxboard.com" && string_indexof(src, "/photos/") >= 0) ||
			(domain_nosub === "mundotkm.com" && domain.match(/^arcdn[0-9]*\./)) ||
			(domain_nosub === "game4v.com" && domain.match(/^cdn[0-9]*\./)) ||
			(domain_nosub === "cosplaytime.pl" && string_indexof(src, "/uploads/") >= 0) ||
			domain === "img.wkorea.com" ||
			(domain === "i.mdel.net" && string_indexof(src, "/newfaces/i/") >= 0) ||
			domain === "media.celebmasta.com" ||
			domain === "media.korea25.com" ||
			(domain_nowww === "sexyfandom.com" && string_indexof(src, "/images/") >= 0) ||
			domain === "i.dmarge.com" ||
			domain === "assets.boundingintocomics.com" ||
			domain === "media.profootballfocus.com" ||
			(domain_nowww === "redu.pl" && string_indexof(src, "/img/") >= 0) ||
			(domain === "multifiles.pressherald.com" && string_indexof(src, "/uploads/") >= 0) ||
			domain === "img.time2draw.com" ||
			(domain === "ffw.uol.com.br" && string_indexof(src, "/app/uploads/") >= 0) ||
			domain === "images.harianjogja.com" ||
			domain === "images.hamodia.com" ||
			(domain_nowww === "bz-berlin.de" && string_indexof(src, "/data/uploads/multimedia/") >= 0) ||
			(domain_nowww === "luxury-inside.vn" && src.match(/\/data\/+uploads\//)) ||
			domain_nowww === "worldcupgirls.net" ||
			(domain_nosub === "playstationlifestyle.net" && domain.match(/^cdn[0-9]*-www\./) && string_indexof(src, "/assets/uploads/") >= 0) ||
			domain === "assets.cdn.moviepilot.de" ||
			(domain_nosub === "aving.net" && domain.match(/^image[0-9]*\./)) ||
			domain === "ss-images.catscdn.vn" ||
			(domain === "blogs.gnome.org" && string_indexof(src, "/files/") >= 0) ||
			(domain_nosub === "zoomit.ir" && domain.match(/^cdn[0-9]*\./)) ||
			domain === "img.myfirstshow.com" ||
			(domain_nosub === "musicfeeds.com.au" && domain.match(/^cdn[0-9]*/) && string_indexof(src, "/assets/uploads/")) ||
			(domain === "d13vpcwfpcq1p8.cloudfront.net" && string_indexof(src, "/contents/") >= 0) ||
			(domain_nosub === "paperblog.fr" && domain.match(/^media[0-9]*\./)) ||
			(domain_nosub === "paperblog.com" && domain.match(/^m[0-9]*\./)) ||
			(domain === "bikini.sbgefree.org" && string_indexof(src, "/files/") >= 0) ||
			(domain_nowww === "redbust.com" && string_indexof(src, "/stuff/") >= 0) ||
			domain === "img.providr.com" ||
			domain === "static.hiphopdx.com" ||
			amazon_container === "queerty-prodweb" ||
			domain === "static.timesofisrael.com" ||
			(domain_nowww === "hairstylesweekly.com" && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "famousbirthsdeaths.com" && string_indexof(src, "/fbd-uploads/") >= 0) ||
			domain === "img.leaksx.com" ||
			(domain_nowww === "top10films.co.uk" && string_indexof(src, "/img/") >= 0) ||
			domain_nowww === "hotel-aramis.com" ||
			(domain === "dvfmubv4tjrqd.cloudfront.net" && string_indexof(src, "/uploads/") >= 0) ||
			amazon_container === "assets.whatson.cityofsydney.nsw.gov.au" ||
			domain === "static.hasselblad.com" ||
			(domain_nowww === "saywho.fr" && string_indexof(src, "/app/uploads/") >= 0) ||
			(domain_nowww === "talentandpartner.com" && string_indexof(src, "/data/uploads/") >= 0) ||
			domain === "files.vividscreen.info" ||
			(amazon_container === "lilianpacce" && string_indexof(src, "/media/") >= 0) ||
			(amazon_container === "chl-network" && string_indexof(src, "/uploads/") >= 0) ||
			(domain_nowww === "starkbros.com" && string_indexof(src, "/images/dynamic/") >= 0) ||
			domain === "m.actve.net" ||
			(domain === "d279m997dpfwgl.cloudfront.net" && string_indexof(src, "/wp/") >= 0) ||
			(domain_nowww === "telegraf.com.ua" && string_indexof(src, "/files/") >= 0) ||
			domain === "media.womanista.com" ||
			(domain_nowww === "jenny.gr" && string_indexof(src, "/storage/photos/") >= 0) ||
			(domain === "static.infomusic.ro" && string_indexof(src, "/media/") >= 0) ||
			(domain_nowww === "csiete.net" && string_indexof(src, "/contenido/imagenes/") >= 0) ||
			(domain_nowww === "wallpaper4rest.com" && string_indexof(src, "/wallpaper/") >= 0) ||
			(domain_nowww === "ichip.ru" && string_indexof(src, "/blobimgs/uploads/") >= 0) ||
			(domain_nowww === "pophaircuts.com" && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "she12.com" && string_indexof(src, "/uploads/") >= 0) ||
			(domain === "cdn.stylefrizz.com" && string_indexof(src, "/img/") >= 0) ||
			(domain_nosub === "windows7themes.net" && string_indexof(src, "/wp-content/files/") >= 0) ||
			(domain === "quotes.whyfame.com" && string_indexof(src, "/files/") >= 0) ||
			(domain_nosub === "mozilla.org" && src.match(/:\/\/[^/]*\/files\/+[0-9]{4}\/+[0-9]{2}\/+/)) ||
			(amazon_container === "pas-wordpress-media" && string_indexof(src, "/uploads/") >= 0) ||
			(domain === "news.artnet.com" && string_indexof(src, "/news-upload/") >= 0) ||
			(domain === "edmhunters-563e.kxcdn.com" && string_indexof(src, "/mediafiles/") >= 0) ||
			(amazon_container === "bkt-respaldomtonline" && string_indexof(src, "/uploads/") >= 0) ||
			domain === "img.emao.net" ||
			domain === "p.potv.bg" ||
			domain === "media.yadbegir.com" ||
			(domain_nosub === "llnwd.net" && src.match(/\/site\/+uploads\/+/)) ||
			(domain_nowww === "cinemio.it" && string_indexof(src, "/upload/") >= 0) ||
			(domain_nowww === "flavourmag.co.uk" && string_indexof(src, "/files/") >= 0) ||
			domain === "img.ananweb.jp" ||
			(domain_nowww === "filmz.dk" && string_indexof(src, "/files/") >= 0) ||
			domain_nowww === "mit24h.com" ||
			domain === "dimwhp0w2rs83.cloudfront.net" ||
			(domain === "cdn.macrumors.com" && string_indexof(src, "/article-new/") >= 0) ||
			(domain_nosub === "xcdnpro.com" && domain.match(/images\./)) ||
			(domain_nowww === "lifezette.com" && string_indexof(src, "/files/") >= 0) ||
			(domain === "cdn.junglecreations.com" && string_indexof(src, "/wp/") >= 0) ||
			(domain_nowww === "rip-youth.jp" && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "asian-sirens.net" && string_indexof(src, "/uploads/") >= 0) ||
			(domain_nosub === "comingsoon.net" && src.match(/\/assets\/+uploads\//)) ||
			domain === "cdn.zoomg.ir" ||
			(domain_nowww === "wikiofthrones.com" && src.match(/\/static\/+uploads\//)) ||
			(domain === "files.theblemish.com" && string_indexof(src, "/images/") >= 0) ||
			(domain_nosub === "esmas.com" && domain.match(/^i[0-9]*\./)) ||
			(domain_nowww === "mortalfm.es" && string_indexof(src, "/archivos/") >= 0) ||
			(domain === "d2yoo3qu6vrk5d.cloudfront.net" && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "courrier.jp" && string_indexof(src, "/media/") >= 0) ||
			(domain === "dieta.pourfemme.it" && string_indexof(src, "/img/") >= 0) ||
			(domain === "static.atmag.co.il" && string_indexof(src, "/uploads/") >= 0) ||
			(domain_nowww === "xtra.works" && string_indexof(src, "/uploads/") >= 0) ||
			(domain_nowww === "myce.com" && string_indexof(src, "/images_posts/") >= 0) ||
			(domain_nowww === "unison.org.uk" && string_indexof(src, "/content/uploads/") >= 0) ||
			(domain_nowww === "businessinsider.fr" && string_indexof(src, "/content/uploads/") >= 0) ||
			(domain === "sfo2.digitaloceanspaces.com" && string_indexof(src, "/uploads/") >= 0) ||
			domain === "m8q3v4v6.stackpathcdn.com" ||
			(domain_nowww === "linda.nl" && string_indexof(src, "/uploads/") >= 0) ||
			(domain === "ww1.sites-telechargement.com" && string_indexof(src, "/affiche/") >= 0) ||
			(domain === "d3n2u7gfnpd084.cloudfront.net" && src.match(/\/(?:[0-9a-f]{2}\/+){4}/)) ||
			(domain_nowww === "journal-farandole.com" && src.match(/\/content\/+uploads\/+/)) ||
			(domain_nowww === "dpstreaming.live" && string_indexof(src, "/upload/") >= 0) ||
			domain === "cdn.videos.rollcall.com" ||
			(domain_nowww === "rawkblog.com" && string_indexof(src, "/uploads/") >= 0) ||
			(domain === "cdn.stereo.vn" && string_indexof(src, "/uploads/") >= 0) ||
			(domain_nowww === "dailybruin.com" && string_indexof(src, "/images/") >= 0) ||
			domain_nowww === "razorpics.net" ||
			(domain_nowww === "1in.am" && string_indexof(src, "/assets/") >= 0) ||
			(domain === "cdn.entertainmentfuse.com" && string_indexof(src, "/media/") >= 0) ||
			(domain_nowww === "n3rdabl3.com" && /\/wp-content\/+images\/+uploads\/+/.test(src)) ||
			(domain_nowww === "kizsalsorunlar.com" && string_indexof(src, "/uploads/") >= 0) ||
			domain_nowww === "neskaties.lv" ||
			domain === "img.eroero-gazou.net" ||
			domain === "media.sistemampa.com.br" ||
			(domain_nosub === "looper.com" && /^img[0-9]*\./.test(domain) && string_indexof(src, "/uploads/") >= 0) ||
			(domain_nosub === "dtslb.com" && /^i[0-9]*\./.test(domain)) ||
			domain === "static.otvfoco.com.br" ||
			(domain_nosub === "diariouno.com.ar" && /^static[0-9]*\./.test(domain) && string_indexof(src, "/media/") >= 0) ||
			(domain === "static-cdn.jtvnw.net" && /\/(s3_vods|previews-ttv|ttv-boxart)\//.test(src)) ||
			(domain_nowww === "thugarmada.com" && string_indexof(src, "/ta-files/") >= 0) ||
			(domain_nowww === "flipwallpapers.com" && string_indexof(src, "/wallpapers/") >= 0) ||
			(domain_nowww === "hanamaru-photo.com" && /\/common\/+images\//.test(src)) ||
			domain === "p.elle.bg" ||
			(domain_nowww === "illustrationwest.org" && string_indexof(src, "/files/") >= 0) ||
			(domain_nowww === "myjane.ru" && /\/data\/+cache\//.test(src)) ||
			(domain_nowww === "chickradar.net" && /\/content\/+uploads\//.test(src)) ||
			(domain_nowww === "creepmachine.com" && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "theinspirationgrid.com" && /\/app\/+uploads\//.test(src)) ||
			(domain === "static-koimoi.akamaized.net" && string_indexof(src, "/new-galleries/") >= 0) ||
			(domain_nosub === "elperiodico.com" && /^img[0-9]*\./.test(domain)) ||
			domain_nosub === "cdn-immedia.net" ||
			(domain_nowww === "mmglobalmovies.com" && /\/wp-content\/+upload_folders\//.test(src)) ||
			(domain === "images.in.com" && string_indexof(src, "/uploads/") >= 0) ||
			domain === "staticuestudio.blob.core.windows.net" ||
			domain === "cdn.imvges.com" ||
			domain === "media.pamper.my" ||
			(domain_nowww === "rnbjunk.com" && string_indexof(src, "/foto/") >= 0) ||
			(domain === "cdn.xpicsxx.com" && string_indexof(src, "/uploads/") >= 0) ||
			(domain_nowww === "zokatv.net" && string_indexof(src, "/media/") >= 0) ||
			(domain_nowww === "magazin.lufthansa.com" && /\/content\/+uploads\//.test(src)) ||
			(domain_nowww === "nexofin.com" && string_indexof(src, "/archivos/") >= 0) ||
			domain === "gallery.southindianactress.in" ||
			(domain === "ffp4g1ylyit3jdyti1hqcvtb-wpengine.netdna-ssl.com" && /\/addons\/+files\//.test(src)) ||
			(domain === "medinaa.archolda.com" && string_indexof(src, "/ime/") >= 0) ||
			(domain_nowww === "londonsvenskar.com" && string_indexof(src, "/files/") >= 0) ||
			(domain === "ia.eferrit.com" && string_indexof(src, "/ia/") >= 0) ||
			(domain_nowww === "dokkaebi.tv" && string_indexof(src, "/file/") >= 0) ||
			(domain_nowww === "porncomix.one" && string_indexof(src, "/gallery/") >= 0) ||
			(domain_nowww === "porncomix.info" && string_indexof(src, "/images/") >= 0) ||
			(domain === "celebsfake.adultcase.com" && string_indexof(src, "/files/") >= 0) ||
			(domain_nowww === "theonemilano.com" && string_indexof(src, "/the-one-milano-uploads/") >= 0) ||
			domain === "public.flashingjungle.com" ||
			(domain === "static.pr.ricmais.com.br" && string_indexof(src, "/uploads/") >= 0) ||
			(domain_nowww === "smarthomebeginner.com" && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "inspirationde.com" && string_indexof(src, "/media/") >= 0) ||
			domain === "cache.escapistmagazine.com" ||
			(domain === "static.acgsoso.com" && string_indexof(src, "/uploads/") >= 0)
			) {
			src = src.replace(/-[0-9]+x[0-9]+\.([^/]*(?:[?#].*)?)$/, ".$1");
		}

		if ((domain === "store.pinseyun.com" && string_indexof(src, "/uploads/") >= 0) ||
			(domain === "media.coindesk.com" && string_indexof(src, "/uploads/") >= 0)) {
			return {
				url: src.replace(/-[0-9]*x[0-9]*\.([^/.]*)$/, ".$1"),
				can_head: false
			};
		}

		if (domain === "cdn.fashionmagazine.com") {
			return src.replace(/(\/+wp-content\/+uploads\/+[0-9]{4}\/+[0-9]{2}\/+[^/]*)-[0-9]+x[0-9]+-c-[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "cdn.heatworld.com" ||
			domain_nowww === "sohobluesgallery.com" ||
			domain === "i.vimeocdn.com" ||
			domain === "media.indiatimes.in" ||
			domain_nowww === "bangkokpost.com" ||
			(domain_nosub === "mensxp.com" && domain.match(/media[0-9]*\./) && string_indexof(src, "/media/") >= 0) ||
			domain === "221.132.38.109" ||
			(domain_nowww === "jpcoast.com" && string_indexof(src, "/img/") >= 0) ||
			domain === "pics.prcm.jp" ||
			(domain_nosub === "lprs1.fr" && domain.match(/s[0-9]*\.lprs1\.fr/)) ||
			domain === "m.wsj.net" ||
			domain === "img.lifestyler.co.kr" ||
			(domain_nosub === "eporner.com" && domain.match(/static-(?:[a-z]+-)?cdn\.eporner\.com/)) ||
			domain === "fototo.blox.pl" ||
			domain === "media.nvyouj.com" ||
			(domain === "cl.buscafs.com" && string_indexof(src, "/www.tomatazos.com/") >= 0) ||
			(domain === "tomatazos.buscafs.com" && string_indexof(src, "/uploads/images/") >= 0) ||
			(domain === "nisfeldunia.ahram.org.eg" && string_indexof(src, "/Media/") >= 0) ||
			domain === "d2t7cq5f1ua57i.cloudfront.net" ||
			domain === "socdn.smtown.com" ||
			(domain_nowww === "lecturas.com" && string_indexof(src, "/medio/") >= 0) ||
			(domain_nowww === "clara.es" && string_indexof(src, "/medio/") >= 0) ||
			domain === "images.anandtech.com" ||
			domain === "cdn.popbela.com" ||
			(domain_nosub === "lizhi.fm" && domain.match(/^cdnimg[0-9]*\./)) ||
			domain === "img.cf.47news.jp" ||
			domain === "static.filmin.es" ||
			(domain_nosub === "ropose.com" && domain.match(/^img[0-9]*\./)) ||
			domain === "resource.info.mn" ||
			(domain_nosub === "thegioitre.vn" && domain.match(/^image[0-9]*\./)) ||
			domain === "img.uduba.com" ||
			domain === "img.51ztzj.com" ||
			(domain === "mediaresources.idiva.com" && string_indexof(src, "/media/") >= 0) ||
			domain === "static.bangkokpost.com" ||
			((domain_nowww === "lepoint.fr" || domain === "static.lpnt.fr") && string_indexof(src, "/images/") >= 0) ||
			(domain_nosub === "sdpnoticias.com" && domain.match(/^i[0-9]*\./)) ||
			(domain === "s.thestreet.com" && string_indexof(src, "/files/") >= 0) ||
			(domain_nowww === "cosmopolitan.com.hk" && src.match(/\/storage\/+nodejs\/+(?:legacy\/+images|files)\//)) ||
			(domain_nosub === "bbwc.cn" && domain.match(/^s[0-9]*\.cdn\./)) ||
			(domain === "d4zcrs0v202ys.cloudfront.net" && string_indexof(src, "/images/") >= 0) ||
			(domain_nosub === "sornamag.com" && src.match(/\/files\/+images\//)) ||
			(domain === "arhiva.nacional.hr" && string_indexof(src, "/img/") >= 0) ||
			(domain === "media.cbs8.com" && string_indexof(src, "/assets/") >= 0) ||
			(domain === "img.myvideo.net.tw" && string_indexof(src, "/images/") >= 0) ||
			(domain === "image.apost.com" && string_indexof(src, "/media/") >= 0) ||
			(domain === "im.indiatimes.in" && string_indexof(src, "/content/") >= 0) ||
			(domain === "admin.alo.rs" && /\/resources\/+images\//.test(src)) ||
			(domain === "img.pcauto.com.cn" && /\/images\/+upload\//.test(src)) ||
			domain === "images.cinefil.com") {
			newsrc = src.replace(/_[0-9]+x[0-9]+(\.[^/.]*)$/, "$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "mediaonlinevn.com" ||
			(domain_nowww === "delas.pt" && string_indexof(src, "/files/") >= 0) ||
			(domain_nowww === "xda-developers.com" && string_indexof(src, "/files/") >= 0) ||
			domain_nowww === "onthemoveworld.com" ||
			/^[a-z]+:\/\/[^?]+\/wp(?:-content\/+(?:uploads|images|photos|blogs.dir)|\/+uploads)\/[^?]+-[0-9]+x[0-9]+(?:_c)?\./.test(src)
			) {
			src = src.replace(/-[0-9]+x[0-9]+(?:_c)?(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (string_indexof(src, "/wp-content/uploads/") >= 0 ||
			string_indexof(src, "/wp/uploads/") >= 0) {
			src = src.replace(/\(pp_w[0-9]+_h[0-9]+\)(\.[^/.]*)$/, "$1");
		}


		if (
			string_indexof(src, "/wp-content/uploads/") >= 0 ||
			string_indexof(src, "/wp/uploads/") >= 0) {
			src = src.replace(/__[0-9]{2,}(\.[^/.]*)$/, "$1");
		}

		if (string_indexof(src, "/wp-content/uploads/") >= 0 ||
			string_indexof(src, "/wp/uploads/") >= 0) {
			src = src.replace(/-[0-9]+x[0-9]+-c-default(\.[^/.]*)$/, "$1");
		}


		if (domain === "pictures.ozy.com" ||
			domain_nowww === "retail-jeweller.com" ||
			domain === "d1nslcd7m2225b.cloudfront.net") {
			return src.replace(/(\/[Pp]ictures\/)[0-9any]+x[0-9any]+(?:[a-z]+)?\//, "$199999999x99999999/");
		}

		if (domain === "static.gofugyourself.com" ||
			domain === "static.stereogum.com") {
			return src.replace(/-(?:[0-9]+x[0-9]+|compressed)(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain === "hips.hearstapps.com") {
			newsrc = src.replace(/.*hips\.hearstapps\.com\/([^/]+\.[^/]+)/, "http://$1");
			if (newsrc !== src)
				return newsrc;
			return src.replace(/\?[^/]*$/, "");
		}

		if (domain === "img.wennermedia.com") {
			return src.replace(/:\/\/img\.wennermedia\.com\/[^?#]*\/([^/]*)$/, "://img.wennermedia.com/$1");
		}

		if (domain_nosub === "forbesimg.com" && string_indexof(domain, "images.forbesimg.com") >= 0) {
			return {
				url: src.replace(/\/[0-9]*x[0-9]*\.([^/.?]*)(\?.*)?/, "/0x0.$1"),
				head_wrong_contentlength: true
			};
		}

		if (domain === "pixel.nymag.com") {
			return src
				.replace(/\/([^/.]*)(\.[^/]*)?\.([^/.]*)$/, "/$1.$3")
				.replace(/(\/slideshows\/.*\/[^/]*)\.[a-z](\.[^/.]*)\/[^/]*(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "assets.nydailynews.com" ||

			(domain_nosub === "nydailynews.com" && domain.match(/^static[0-9]*\./)) ||
			domain === "i.cbc.ca" ||
			domain === "cdn.newsday.com" ||
			domain_nowww === "stripes.com" ||
			domain_nowww === "irishtimes.com" ||
			domain_nowww === "ctvnews.ca" ||
			domain_nowww === "lancashirelife.co.uk" ||
			domain === "images.archant.co.uk" ||
			domain === "images.glaciermedia.ca" ||
			domain === "static.gulfnews.com" ||
			domain === "www.cp24.com" ||
			domain_nosub === "anandabazar.com" ||
			domain_nowww === "islingtongazette.co.uk" ||
			domain_nowww === "vrak.tv" ||
			domain === "images.haaretz.co.il" ||
			domain_nowww === "ltu.se" ||
			domain_nowww === "lanuovasardegna.it" ||
			domain_nowww === "huntspost.co.uk" ||
			domain_nowww === "edp24.co.uk") {
			newsrc = src
				.replace(/(\/[^/.]*\.[^_/.]*)_gen\/+derivatives\/+[^/]*\/+/, "/")
				.replace(/\/image\.[^_/]*_gen\/+derivatives\/+[^/]*\//, "/")
				.replace(/\/image\/+[^_/]*_gen\/+derivatives\/+[^/]*\//, "/image/");
			if (newsrc !== src) {
				return newsrc.replace(/\?.*/, "");
			}
		}

		if (domain === "static.gulfnews.com") {
			return {
				url: src,
				head_wrong_contenttype: true
			};
		}

		if (domain === "www.tsn.ca") {
			return src.replace(/(\/image\.[^_/]*_gen\/+derivatives\/+)[^/]*\//, "$1default/");
		}

		if (domain_nosub === "bbci.co.uk" && domain.match(/^ichef(?:-[0-9]*)?\./)) {
			newsrc = src.replace(/\/[0-9]+_[0-9]+\//, "/original/");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/(\.[^/.]*)\/[0-9]+$/, "$1/0");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/(:\/\/[^/]*)\/images\/ic\/[0-9n]+x[0-9n]+\//, "$1/images/ic/raw/");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/.*\.bbci\.co\.uk\/news\/[0-9]*\/(?:[^/]*\/)?media\//, "http://news.bbcimg.co.uk/media/");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/:\/\/[^/]*\/food\/ic\/[^/]*\//, "://food-images.files.bbci.co.uk/food/");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/\/images\/ic\/credit(\/[0-9]+x[0-9]+\/)/, "/images/ic$1");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/.*\/cpsprodpb\//, "https://c.files.bbci.co.uk/");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/news\/[0-9]+\/[^/]*\//, "https://c.files.bbci.co.uk/");
			if (newsrc !== src)
				return newsrc;

			/*var origsize = src.match(/\.bbci\.co\.uk\/[^/]*\/([0-9]*)\//);
			if (origsize && false) { // scales up
				var size = parseInt(origsize[1], 10);
				if (size < 2048) {
					return src.replace(/(\.bbci\.co\.uk\/[^/]*)\/[0-9]*\//, "$1/2048/");
				}
			}*/
		}

		if (domain === "amp.thisisinsider.com" ||
			domain === "amp.insider.com" ||
			domain === "amp.businessinsider.com") {
			return src.replace(/^[a-z]+:\/\/amp\.([^/]*)\/images\/([a-f0-9]+)-[0-9]+(?:-[0-9]+)?(\.[^/.]*)$/, "https://static2.$1/image/$2/");
		}

		if ((domain_nosub === "businessinsider.com" ||
			 domain_nosub === "insider.com" ||
			 domain_nosub === "thisisinsider.com") &&
			domain.match(/^static[0-9]*\./)) {
			return src.replace(/(\/image\/[0-9a-f]+)(?:-[^/]*|\.[^/]*)?(?:\/.*)?$/, "$1/");
		}

		if (domain === "media.nbcwashington.com" ||
			domain === "media.nbcnewyork.com" ||
			domain === "media.graytvinc.com" ||
			domain === "media.telemundochicago.com" ||
			domain === "media.nbcdfw.com" ||
			domain === "media.nbcphiladelphia.com" ||
			domain === "media.nbcsandiego.com" ||
			domain === "media.heartlandtv.com" ||
			domain === "media.nbcmiami.com" ||
			domain === "media.nbcconnecticut.com" ||
			domain === "media.nbclosangeles.com" ||
			domain === "media.nbcboston.com" ||
			domain === "media.nbcbayarea.com" ||
			domain === "media.winnipegfreepress.com" ||
			domain === "media.brandonsun.com" ||
			domain === "media.necn.com" ||
			domain === "media.telemundoareadelabahia.com" ||
			domain === "media.nbcchicago.com") {
			return src.replace(/\/images\/+[0-9]+\*[0-9]+\//, "/images/");
		}

		if (domain_nowww === "bet.com") {
			return src
				.replace(/\/(_jcr_content.*?\/[^/]*)\.custom[0-9]+fx[0-9]+fx[0-9]+xcrop\.([^/]*)\//, "/$1.custom0fx0fx0xcrop.$2/")
				.replace(/\/(_jcr_content.*?\/[^/]*)\.custom[0-9]+x[0-9]+\.([^/]*)\//, "/$1.custom0fx0fx0xcrop.$2/")
				.replace(/\/(_jcr_content.*?\/[^/]*)\.featured[0-9]+x[0-9]+\.([^/]*)\//, "/$1.custom0fx0fx0xcrop.$2/")
				.replace(/\/(_jcr_content.*?\/[^/]*)\.featuredlist\.([^/]*)\//, "/$1.custom0fx0fx0xcrop.$2/")
				.replace(/\/(_jcr_content.*?\/[^/]*)\.ampheroimage\.([^/]*)\//, "/$1.custom0fx0fx0xcrop.$2/")
				.replace(/\/(_jcr_content.*?\/[^/]*)\.feedcontainer\.([^/]*)\//, "/$1.custom0fx0fx0xcrop.$2/")
				.replace(/\/(_jcr_content.*?\/[^/]*)\.[^/.]*\.([^/]*)\//, "/$1.custom0fx0fx0xcrop.$2/")
				.replace(/\/(_jcr_content.*?\/[^/]*)\.relatedinline[0-9]+x[0-9]+\.([^/]*)\//, "/$1.custom0fx0fx0xcrop.$2/");
		}

		if ((domain_nosub === "cbsistatic.com" &&
			 (domain.match(/^cbsnews[0-9]*\./) ||
			  domain.match(/^dl[0-9]*\./) ||
			  domain.match(/zdnet[0-9]*\./))) ||
			domain === "sportshub.cbsistatic.com" ||
			domain === "cimg.tvgcdn.net") {
			newsrc = src
				.replace(/\/resize\/[0-9a-z]*x[0-9a-z]*\//, "/")
				.replace(/\/crop\/[^/]*\//, "/")
				.replace(/\/thumbnail\/[^/]*\//, "/");
			if (newsrc !== src) {
				return {
					url: newsrc,
					head_wrong_contentlength: true
				};
			}
		}

		if (domain_nosub === "cbsistatic.com" &&
			domain.match(/cnet[0-9]*\.cbsistatic\.com/)) {
			return src.replace(/\/img\/[^/]*=\/(?:fit-in\/)?(?:[0-9]+x[0-9]+:[0-9]+x[0-9]+\/)?(?:[0-9]+x[0-9]+\/)?(.*)/, "/img/$1");
		}

		if (domain_nosub === "cbsstatic.com" &&
			domain.match(/wwwimage[0-9]*(?:-secure)?\.cbsstatic\.com/)) {
			return src
				.replace(/\/thumbnails\/([^/]*)\/[-a-z0-9:]*\//, "/thumbnails/$1/files/")
				.replace("/thumbnails/photos/files/", "/base/files/");
		}

		if (domain === "api.fidji.lefigaro.fr") {
			return src.replace("://api.fidji.lefigaro.fr/", "://i.f1g.fr/");
		}

		if (domain === "i.f1g.fr") {
			newsrc = src.replace(/.*i\.f1g\.fr\/media\/ext\/[^/]*\//, "http://");
			var newdomain = newsrc.replace(/^http:\/\/([^/]*)\/.*/, "$1");
			if (newsrc !== src &&
				newdomain !== "img.tvmag.lefigaro.fr")
				return newsrc;

			return src.replace(/\/media\/([a-z]*)\/[^/]*\//, "/media/$1/orig/");
		}

		if ((domain_nosub === "h-cdn.co" ||
			 (domain_nosub === "cosmopolitan.nl" && domain.match(/h\.cdn\.cosmopolitan\./))) &&
			string_indexof(src, "/assets/") >= 0) {
			return src
				.replace(/\/[0-9]*x[0-9]*\//, "/")
				.replace(/\/(?:landscape|[a-z]+-aspect)[-_]((?:nrm_)?[0-9]{5,}[^/]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain === "imgix.ranker.com") {
			return src.replace(/\?[^/]*$/, "?fm=png");
		}

		if (domain_nosub === "rnkr-static.com" &&
			domain.match(/^img[0-9]*\./)) {
			return src.replace(/^[a-z]+:\/\/[^/]*\/+((?:user_)?node_img\/+[0-9]+\/+[0-9]+)\/+[^/]*\/+([^/]*)\.[^/.]*(?:[?#].*)?$/,
							   "https://imgix.ranker.com/$1/original/$2?fm=png");
		}

		if (domain_nosub === "imgix.net") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\.imgix\.net\/(https?%3A[^?]+).*?$/, "$1")
			if (newsrc !== src)
				return decodeURIComponent(newsrc);
		}

		if (domain === "data.whicdn.com") {
			match = src.match(/^([a-z]+:\/\/[^/]+\/+images\/+([0-9]+)\/+)[^/.]*\.([^/.]*)$/);
			if (match) {
				return {
					url: match[1] + "original." + match[3],
					extra: {
						page: "https://weheartit.com/entry/" + match[2]
					}
				};
			}
		}

		if (domain_nosub === "whicdn.com" && domain.match(/^data[0-9]*\./)) {
			return src.replace(/:\/\/data[0-9]*\./, "://data.");
		}

		if (host_domain_nowww === "weheartit.com" && options.element) {
			if (domain === "assets.whicdn.com" && /\/assets\/+whi-[^/]+\/+entry_stub\/+entry_stub_unsafe-/.test(src)) {
				if (options.element.parentElement && options.element.parentElement.tagName === "A") {
					var href = options.element.parentElement.href;
					match = href.match(/\/entry\/+([0-9]+)\/*(?:[?#].*)?$/);

					if (match) {
						return "https://data.whicdn.com/images/" + match[1] + "/original.jpg";
					}
				}
			}
		}

		if (domain === "cdn.empireonline.com") {
			return src.replace(/cdn\.empireonline\.com\/(?:jpg|png|gif)\/(?:[^/.]+\/){12}/, "cdn.empireonline.com/");
		}

		if (domain_nosub === "celebmafia.com" ||
			domain_nosub === "hawtcelebs.com") {
			return src.replace(/\/([^/]*)_thumbnail\.([^/.]*)$/, "/$1.$2");
		}

		if ((domain_nosub === "pixhost.org" ||
			 domain_nosub === "pixhost.to") &&
			domain.match(/^[a-z]*[0-9]*\./)) {
			return src
				.replace(/(:\/\/[^/]*\.)pixhost\.org\//, "$1pixhost.to/")
				.replace(/\/t([0-9]*\.pixhost\.[a-z]*)\/thumbs\//, "/img$1/images/");
		}


		if (domain_nosub === "ulximg.com" ||
			domain_nowww === "hotnewhiphop.com") {
			return src
				.replace(/\/image\/[0-9a-z]*x[0-9a-z]*\//, "/image/full/");
		}

		if (domain === "fm.cnbc.com") {
			return src.replace(/\.[0-9]+x[0-9]+\.([^/.]*)$/, ".$1");
		}

		if (domain === "images.bwwstatic.com" ||
			domain === "newimages.bwwstatic.com") {
			return src
				.replace(/\/tn-[0-9]+_([^/]*)$/, "/$1")
				.replace(/\/(?:[0-9]+)?x(?:[0-9]*)(?:x[0-9]+)?([^/]*)\.pagespeed\.[^/]*(?:[?#].*)?$/, "/$1");
		}


		if (domain === "i.pinimg.com" ||
			(domain_nosub === "pinimg.com" && domain.match(/^(?:i|(?:s-)?media-cache)-[^.]*\.pinimg/)) ||
			amazon_container === "media.pinterest.com") {
			src = src.replace(/[?#].*$/, "");

			if (src.match(/:\/\/[^/]*\/media\.pinterest\.com\//))
				newsrc = src.replace(/(:\/\/[^/]*\/media\.pinterest\.com\/)[^/]*(\/.*\/[^/]*\.[^/.]*)$/, "$1originals$2");
			else
				newsrc = src.replace(/(:\/\/[^/]*\/)[^/]*(\/.*\/[^/]*\.[^/.]*)$/, "$1originals$2");

			if (newsrc !== src) {
				return add_extensions_gif({
					url: newsrc,
					is_original: true
				});
			}
		}

		if (domain_nosub === "condecdn.net" && string_indexof(domain, "images.condecdn.net") >= 0) {
			return src.replace(/(\/image\/[^/]*\/).*/, "$1original/");
		}

		if (domain === "media.fromthegrapevine.com" ||
			(domain_nowww === "gretschpages.com" && string_indexof(src, "/media/img/") >= 0) ||
			(domain_nowww === "pornstars.me" && string_indexof(src, "/media/") >= 0) ||
			domain_nowww === "mediavillage.com") {
			return src.replace(/\/([^/.]*\.[^/.]*)\.[^/.]*\.[^/.]*$/, "/$1");
		}

		if (domain_nosub === "acsta.net" && domain.search(/img[0-9]*\.acsta\.net/) >= 0) {
			newsrc = src.replace(/acsta\.net\/[^/]*\/pictures\//, "acsta.net/pictures/");
			if (newsrc !== src)
				return newsrc;

			return src.replace(/\/[rc]_[0-9]+_[0-9]+\//, "/");
		}

		if (domain === "em.wattpad.com") {
			return src.replace(/.*\.wattpad\.com\/[a-f0-9]*\/([a-f0-9]*).*/, "$1").replace(/([0-9A-Fa-f]{2})/g, function() {
				return String.fromCharCode(parseInt(arguments[1], 16));
			});
		}

		if (domain_nosub === "nocookie.net" &&
			domain.match(/^vignette[0-9]*\.wikia\./)) {
			return src
				.replace(/\/revision\/([^/]*)\/.*?(\?.*)?$/, "/revision/$1/$2")
				.replace(/^http:\/\//, "https://");
		}

		if (domain === "static.asiachan.com") {
			return src
				.replace(/(\/[^/]*\.)[0-9]*(\.[0-9]*\.[^/.]*$)/, "$1full$2")
				.replace(/(:\/\/[^/]*\/)[0-9]+(\/[0-9]+\/[0-9]+\/[0-9]+\.[^/.]*)$/, "$1full$2");
		}

		if (domain === "pic.xiami.net" ||
			domain === "club-img.kdslife.com" ||
			(domain_nosub === "meituan.net" && /^p[0-9]*\./.test(domain)) ||
			(domain === "piccn.ihuaben.com" && string_indexof(src, "/pic/") >= 0) ||
			(domain_nosub === "lanlanlife.com" && /^oss[0-9]*\./.test(domain)) ||
			domain === "img.sdxapp.com") {
			newsrc = src.replace(/(?:@|%40)[^/]*$/, "");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "oss.tan8.com") {
			return src.replace(/(\/resource\/+attachment\/+[0-9]{4}\/+[0-9]{6}\/+[0-9a-f]{20,})_thumb(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "ultimate-guitar.com") {
			return src.replace(/(\/images\/+(?:[0-9a-f]\/+){2}[0-9a-f]+\.[^/.]*?)@[0-9]+(?:[?#].*)?$/, "$1");
		}

		if ((domain_nosub === "yinyuetai.com" && domain.match(/^img[0-9]*\.c\.yinyuetai\.com/)) ||
			(domain_nosub === "yytcdn.com" && domain.match(/^img[0-9]*\./))) {
			return src.replace(/[0-9]+x[0-9]+(\.[^/.]*)$/, "0x0$1");
		}

		if (domain.search(/mp[0-9]*\.qiyipic\.com/) >= 0 && string_indexof(src, "/passport/") < 0) {
			return src.replace(/[0-9]*_[0-9]*(\.[^/.]*)$/, "0_0$1");
		}

		if (domain_nosub === "duitang.com") {
			return src.replace(/(\/uploads\/+item\/+.*)\.thumb\.[0-9]+_[0-9]+\./, "$1.");
		}

		if (domain_nosub === "vcimg.com" &&
			domain.match(/i-[0-9]\.vcimg.com/)) {
			return src.replace(/\/(?:crop|trim)\//, "/").replace(/(\/[0-9a-f]+)(?:\(|%28)[0-9]+x(?:[0-9]+)?(?:\)|%29)\//, "$1/");
		}

		if (domain_nosub === "zhimg.com" &&
			domain.match(/pic[0-9]\.zhimg\.com/)) {
			return {
				url: src.replace(/\/((?:v[0-9]*-)?[0-9a-f]+)(?:_[^/._]*)?(\.[^/.]*)$/, "/$1_r$2"),
				headers: {
					Referer: ""
				},
				referer_ok: {
					same_domain: true
				}
			};
		}

		if (domain === "img.hb.aicdn.com" ||
			domain === "hbimg.huabanimg.com" ||
			domain_nosub === "upaiyun.com") {
			return {
				url: src.replace(/_[^/_]*$/, ""),
				headers: {
					Referer: ""
				}
			};
		}

		if (domain === "imagev2.xmcdn.com") {
			return src.replace(/![^/]*$/, "").replace(/(\/[^/]*?)(?:_[a-z_]+)?(\.[^/.]*)$/, "$1$2");
		}

		if ((domain_nosub === "bdimg.com" || domain_nosub === "baidu.com") &&
			(domain.match(/timg.*?\.bdimg\.com/) ||
			 domain.match(/timg.*?\.baidu\.com/))) {
			newsrc = decodeURIComponent(src.replace(/.*\/[^/]*[?&]src=([^&]*).*/, "$1"));
			if (newsrc !== src)
				return newsrc;
		}

		if ((domain_nosub === "bdstatic.com" ||
			 domain_nosub === "baidu.com") &&
			domain.match(/gss[0-9]*\./)) {
			if (string_indexof(src, "/timg?") >= 0) {
				return {
					url: decodeURIComponent(src.replace(/.*?\/timg.*?[?&]src=([^&]*).*/, "$1")),
					head_wrong_contenttype: true
				};
			}

			if (string_indexof(src, "/sign=") >= 0 ||
				string_indexof(src, "/pic/item/") >= 0) {
				return {
					url: src.replace(/:\/\/[^/]*\/[^/]*\//, "://imgsrc.baidu.com/"),
					head_wrong_contenttype: true
				};
			}
		}

		if (domain === "imgsrc.baidu.com" ||
			domain === "tiebapic.baidu.com" ||
			(domain_nosub === "baidu.com" && domain.match(/^(?:[a-z]\.)?hiphotos\./)) ||
			domain === "imgsa.baidu.com") {
			newsrc = decodeURIComponent(src.replace(/.*\/[^/]*[?&]src=([^&]*).*/, "$1"));
			if (newsrc !== src)
				return newsrc;

			newsrc = src
				.replace("/abpic/item/", "/pic/item/")
				.replace(/\/[^/]*(?:=|%3D)[^/]*\/sign=[^/]*\//, "/pic/item/");
			return {
				url: newsrc,
				head_wrong_contenttype: true
			};
		}

		if (domain_nosub === "baidu.com" && string_indexof(domain, "himg.baidu.com") >= 0) {
			return src.replace(/\/sys\/[^/]*\/item\//, "/sys/original/item/");
		}


		if (domain_nosub === "doubanio.com" &&
			domain.match(/^img[0-9]*\./)) {
			newsrc = src.replace(/\/img\/+trailer\/+small\/+/, "/img/trailer/medium/");
			if (newsrc !== src)
				return newsrc;

			newsrc = src
				.replace(/\/[a-z]+(\/public\/[a-f0-9]+\.[^/.]*)$/, "/raw$1")
				.replace(/\/(?:small|medium)\//, "/large/")
				.replace(/\/[a-z]pic\//, "/opic/")
				.replace(/\/+img\/+([^/]*)\/+[^/]*\/+([0-9]+[^/]*)(?:[?#].*)?$/,
						 "/pview/$1/raw/public/p$2");

			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/\/view\/+subject\/+[a-z]+\/+/, "/view/subject/raw/");
			if (newsrc !== src)
				return newsrc;

			if (src.match(/\/+view\/+([^/]*)\/+[^/]*\/+/)) {
				newsrc = src.replace(/\/+view\/+([^/]*)\/+[^/]*\/+/, "/pview/$1/raw/");

				if (newsrc !== src)
					return add_extensions(newsrc.replace(/\.webp(?:[?#].*)?$/, ".jpg"));
			}

			return {
				url: src,
				head_wrong_contentlength: true
			};
		}

		if (domain === "img.idol001.com") {
			return src
				.replace(/(:\/\/[^/]*\/+)thumbnail\/+/, "$1origin/")
				.replace(/(\/[0-9a-f]+)_watermark(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "star-img.idol001.com") {
			return src.replace(/(\/images\/+[0-9]{4}\/+(?:[0-9]{1,2}\/+){2}[^/]+\.[^/.]+)\/+[^/]*(?:[?#].*)?$/, "$1");
		}

		if (domain === "image-api.nrj.fr") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/(https?)\/([^/?&#]*).*?$/, "$1://$2");
			if (newsrc !== src)
				return decodeURIComponent(newsrc);
		}

		if (domain_nowww === "norwalkreflector.com" &&
			string_indexof(src, "/image/") >= 0) {
			return src.replace(/(\/image\/[0-9]*\/[0-9]*\/[0-9]*\/)[^/]*\/([^/]+)$/, "$1$2");
		}

		if (domain === "assets.bwbx.io") {
			return src.replace(/\/[-0-9]*x[-0-9]*(\.[^/]*)$/, "/-1x-1$1");
		}

		if (domain === "file.osen.co.kr") {
			newsrc = src.replace(/\/article_thumb\/+([0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+[0-9]+(?:_[^/._]*)?)_[0-9]+x(?:[0-9]+)?(\.[^/.]*)(?:[?#].*)?$/,
								 "/article/$1$2");
			if (newsrc !== src)
				return newsrc;

			return src
				.replace("/article_thumb/", "/article/")
				.replace(/\/article\/+([0-9]{4})\//, "/article/original/$1/")
				.replace(/_[0-9]+x(?:[0-9]+)?(\.[^/.]*)$/, "$1");
		}

		if (domain === "thumbnews.nateimg.co.kr") {
			return src.replace(/.*\/(?:view|m?news)[0-9]*\//, "");
		}

		if (domain === "thumb.pann.com") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/[^/]*\//, "");
		}

		if (domain_nosub === "nate.com" && string_indexof(domain, ".video.nate.com") >= 0) {
			return src.replace(/\/img\/thumb\/[0-9]+[^/]*\//, "/img/");
		}


		if (domain === "newsimg.sedaily.com") {
			newsrc = src.replace(/(\/[0-9]{4}\/+(?:[0-9]{2}\/+){2}[^/_]+_[0-9]+)_[a-z](\.[^/.]*)(?:[?#].*)?$/, "$1$2");
			if (newsrc !== src)
				return newsrc;

			var extra = {};
			match = src.match(/\/[0-9]{4}\/+(?:[0-9]{2}\/+){2}([^/_]+)_[0-9]+(?:_[a-z])?\.[^/.]*(?:[?#].*)?$/);
			if (match) {
				extra.page = "https://www.sedaily.com/NewsView/" + match[1];
			}

			return {
				url: src,
				extra: extra
			};
		}

		if (domain === "stat.ameba.jp" ||
			domain === "stat.profile.ameba.jp") {
			return src.replace(/\/t[0-9]*_([^/]*)$/, "/o$1");
		}

		if (domain_nosub === "blogimg.jp" ||
			domain === "image.news.livedoor.com") {
			return src.replace(/(\/[^/.]*)-[sm](\.[^/.]*)/, "$1$2");
		}

		if (domain === "image.cine21.com") {
			return src
				.replace("/resize/", "/")
				.replace(/\/(?:small|medium)(\/[^/]*)$/, "/large$1")
				.replace(/\?.*$/, "")
				.replace(/\[[WH][-0-9]*\](\.[^/.]*)$/, "$1")
				.replace(/\[[XF][0-9]+,[0-9]+\](\.[^/.]*)$/, "$1");
		}

		if (domain === "cdnimg.melon.co.kr" ||
			domain === "image.melon.co.kr" ||
			domain === "kakaoimg.melon.co.kr" ||
			domain === "cdnticket.melon.co.kr" ||
			domain === "cmtimg.melon.co.kr"/* &&
			(string_indexof(src, "/images/") >= 0 ||
			 string_indexof(src, "/image/") >= 0 ||
			 string_indexof(src, "/user_images/") >= 0)*/) {
			if (/\/resource\/+image\/+web\/+artist\/+/.test(src)) {
				return {
					url: src,
					bad: "mask"
				};
			}





			newsrc = src.replace(/(\.[a-zA-Z]+)\/melon\/.*/, "$1");
			if (newsrc !== src)
				return newsrc;

			if (string_indexof(src, "/images/main/") >= 0) {
				return src.replace(/(images\/.*\/[^/_]*)((_[^/.]*)_)?(_?[^/._]*)?(\.[^/.?]*)(?:[?/].*)?$/, "$1$3$5");
			} else {
				return src.replace(/(images\/.*\/[^/_]*)((_[^/.]*)_)?(_?[^/._]*)?(\.[^/.?]*)(?:[?/].*)?$/, "$1$3_org$5");
			}
		}

		if (domain === "cdnticket.melon.co.kr") {
			if (/\/resource\/+image\/+web\/+common\//.test(src)) {
				return {
					url: src,
					bad: "mask"
				};
			}
		}

		if (domain_nosub === "mzstatic.com" && domain.match(/is[0-9](-ssl)?\.mzstatic\.com/) &&
			string_indexof(src, "/image/thumb/") >= 0) {
			return src.replace(/\/[0-9]*x[0-9]*[a-z]*(?:-[0-9]+)?(\.[^/.]*)$/, "/999999999x0w-999$1");
		}

		if (domain === "img-tmdetail.alicdn.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+bao\/+uploaded\/+([^/]+\.[^/]+\/+)/, "$1");
			if (newsrc !== src)
				return add_http(newsrc);
		}

		if (domain_nosub === "alicdn.com" &&
			(domain.match(/[0-9]*\.alicdn\.com/) ||
			 domain === "img.alicdn.com")) {
			return src
				.replace(/_[0-9]+x[0-9]+[^/]*?$/, "")
				.replace(/\.[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1")
				.replace(/\?.*/, "");
		}

		if (domain_nosub === "1818lao.com" &&
			domain.match(/^i[0-9]*\./)) {
			return src.replace(/:\/\/[^/]*\/aliexpress[0-9]*(\/kf\/)/, "://ae01.alicdn.com$1");
		}


		if (domain === "lastfm-img2.akamaized.net" ||
			domain === "lastfm.freetls.fastly.net" ||
			domain_nosub === "lst.fm") {
			return src.replace(/\/i\/+u\/+(?:avatar)?(?:[0-9]+x[0-9]+|[0-9]+s)\//, "/i/u/");
		}

		if (domain_nosub === "myspacecdn.com" &&
			domain.match(/a[0-9](?:\...)?-images\.myspacecdn\.com/)) {
			return src.replace(/\/[^/.]*(\.[^/.]*)$/, "/full$1");
		}

		if (domain === "pics.vampirefreaks.com") {
			return src.replace(/(\/[0-9]+)_s(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "geo-media.beatport.com") {
			return src.replace(/\/image_size\/[0-9]*x[0-9]*\//, "/image_size/0x0/");
		}

		if (domain_nosub === "tumblr.com" &&
			string_indexof(domain, "media.tumblr.com") >= 0) {

			newsrc = src
				.replace(/\.pnj(\?.*)?$/, ".png$1")
				.replace(/\.gifv(\?.*)?$/, ".gif$1");
			if (newsrc !== src)
				return newsrc;

			var obj = {
				problems: {
					possibly_broken: false
				}
			};

			if (false && src.match(/_[0-9]*\.gif$/)) {
				obj.problems.possibly_broken = true;
			}

			if (src.match(/\/avatar_[0-9a-f]+_(?:64|128)\./))
				return src.replace(/_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "_512$1");

			newsrc = src.replace(/(\/(?:tumblr(?:_(?:static|inline))?_)?[0-9a-zA-Z]+(?:_og)?(?:_r[0-9]*)?)_[0-9]+(?:sq)?(\.[^/.]*)$/, "$1_1280$2");
			if (newsrc !== src) {
				obj.url = newsrc;
				return obj;
			}
		}

		if (domain_nosub === "tumblr.com" && /media\.tumblr\.com$/.test(domain) && options && options.element &&
			options.do_request && options.cb) {

			var apikey = null;
			if (options.rule_specific && options.rule_specific.tumblr_api_key)
				apikey = options.rule_specific.tumblr_api_key;

			var get_initialstate_from_text = function(text) {
				var match = text.match(/window\['___INITIAL_STATE___'\]\s*=\s*({.*?"tumblelog":.*?})\s*;\s*(?:<\/script>[\s\S]*)?$/);
				if (match) {
					var json = match[1].replace(/(\"\s*:)\s*\/.*?\/\s*([,}])/g, "$1null$2");

					try {
						var parsed = JSON_parse(json);
						return parsed;
					} catch (e) {
						console_error(e);
					}

					return null;
				}

				return null;
			};

			var get_initialstate = function() {
				var scripts = document.getElementsByTagName("script");
				for (var i = 0; i < scripts.length; i++) {
					var text = scripts[i].innerText;

					if (text.length < 100)
						continue;

					var initialstate = get_initialstate_from_text(text);
					if (initialstate)
						return initialstate;
				}

				return null;
			};

			var get_apikey_from_initialstate = function(initialstate) {
				if (initialstate && initialstate.apiFetchStore && "API_TOKEN" in initialstate.apiFetchStore) {
					return initialstate.apiFetchStore.API_TOKEN;
				}

				return null;
			};

			var initialstate = get_initialstate();
			var initialstate_api_key = get_apikey_from_initialstate(initialstate);

			var get_beta_api_key = function(cb) {
				api_cache.fetch("tumblr_api_key", cb, function(done) {
					if (initialstate_api_key) {
						return done(initialstate_api_key, 24*60*60);
					}

					options.do_request({
						method: "GET",
						url: "https://support.tumblr.com/archive",
						headers: {
							Referer: ""
						},
						onload: function(resp) {
							if (resp.readyState !== 4)
								return;

							if (resp.status !== 200) {
								console_error(resp);
								return done(null, false);
							}

							var initialstate = get_initialstate_from_text(resp.responseText);
							if (initialstate) {
								var apikey = get_apikey_from_initialstate(initialstate);
								if (apikey) {
									return done(apikey, 24*60*60);
								}
							}

							return done(null, false);
						}
					});
				});
			};

			var do_tumblr_api_call = function(path, params, cb) {

				var request_obj = {
					url: "https://api.tumblr.com/v2" + path + "?api_key=" + apikey + "&" + params,
					method: "GET",
					onload: function(resp) {
						if (resp.readyState !== 4)
							return;

						if (resp.status !== 200) {
							console_error(resp);
							return cb(null);
						}

						return cb(resp.responseText);
					}
				};

				if (!apikey) {
					get_beta_api_key(function(apikey) {
						request_obj.headers = {
							"Authorization": "Bearer " + apikey,
							"X-Version": "redpop/3/0//redpop/"
						};

						request_obj.url = request_obj.url.replace(/\?api_key=.*?&/, "?");
						options.do_request(request_obj);
					});
				} else {
					options.do_request(request_obj);
				}
			};

			var get_post_api = function(blogname, postid, cb) {
				var cache_key = "tumblr_api_blog:" + blogname + ":post:" + postid;
				api_cache.fetch(cache_key, cb, function(done) {
					do_tumblr_api_call("/blog/" + blogname + "/posts", "id=" + postid + "&npf=true", function(data) {
						if (!data) {
							return done(null, false);
						}

						try {
							var json = JSON_parse(data);
							var post = json.response.posts[0];

							var content = post.content;

							if (post.trail) {
								for (var i = 0; i < post.trail.length; i++) {
									[].push.apply(content, post.trail[i].content);
								}
							}

							return done(content, 6*60*60);
						} catch (e) {
							console_error(e, data);
							return done(null, false);
						}
					});
				});
			};

			var get_post_http = function(blogname, postid, cb) {
				var cache_key = "tumblr_blog_image:" + blogname + ":post:" + postid;
				api_cache.fetch(cache_key, cb, function(done) {
					options.do_request({
						url: "https://" + blogname + ".tumblr.com/image/" + postid + "/",
						method: "GET",
						onload: function(resp) {
							if (resp.readyState !== 4)
								return;

							if (resp.status !== 200) {
								console_error(resp);
								return done(null, false);
							}

							var match = resp.responseText.match(/window\['___INITIAL_STATE___'\]\s*=\s*({.*?});\s*<\/script>/);
							if (!match) {
								console_warn("Unable to find match", resp);
								return done(null, false);
							}

							var jsontxt = match[1].replace(/,\"[^"]+?\":\/.*?,\"/, ",\"");
							try {
								var json = JSON_parse(jsontxt);
								var imageresponse = json.ImagePage.photo.imageResponse;

								var firstimage = {media: []};
								for (var i = 0; i < imageresponse.length; i++) {
									imageresponse[i].media_key = imageresponse[i].mediaKey;
									imageresponse[i].has_original_dimensions = imageresponse[i].hasOriginalDimensions;
									firstimage.media.push(imageresponse[i]);
								}

								return done([firstimage], 6*60*60);
							} catch(e) {
								console_error(e);
								return done(null, false);
							}
						}
					});
				});
			}

			var get_post = function(blogname, postid, cb) {
				if (apikey || initialstate_api_key) {
					return get_post_api(blogname, postid, cb);
				} else {
					get_beta_api_key(function (apikey) {
						if (apikey) {
							return get_post_api(blogname, postid, cb);
						} else {
							return get_post_http(blogname, postid, cb);
						}
					});
				}
			};

			var find_largest_from_media = function(media) {
				var pixels = 0;
				var obj = null;
				for (var i = 0; i < media.length; i++) {
					var ourobj = media[i];
					var ourpixels = ourobj.width * ourobj.height;

					if (ourpixels > pixels) {
						obj = ourobj;
						pixels = ourpixels;
					}
				}

				return obj;
			};

			var find_mediakey_from_url = function(url) {
				var match = url.match(/^[a-z]+:\/\/[^/]+\/+([0-9a-f]{32})\/+([0-9a-f]{16}-[0-9a-f]{2})\/+/);
				if (match) {
					return match[1] + ":" + match[2];
				}

				return null;
			};

			var find_media_from_post_mediakey = function(post, mediakey) {
				for (var i = 0; i < post.length; i++) {
					if (!post[i].media)
						continue;

					if (post[i].media[0].media_key === mediakey) {
						return post[i].media;
					}
				}

				return null;
			};

			var find_blogname_from_url = function(url) {
				match = url.match(/^[a-z]+:\/\/([^/.]+)\.tumblr\.com\//);
				if (match) {
					if (match[1] !== "www" && match[1] !== "support")
						return match[1];
				}

				return null;
			};

			var find_blogname_id_from_url = function(url) {
				match = url.match(/^[a-z]+:\/\/([^/.]+)\.tumblr\.com\/+post\/+([0-9]+)\//);
				if (match) {
					return {
						blogname: match[1],
						postid: match[2]
					};
				}

				return null;
			};

			var find_blogname_from_head = function() {
				if (!options.document)
					return null;

				var links = options.document.getElementsByTagName("link");
				for (var i = 0; i < links.length; i++) {
					var href = links[i].href;
					if (!href)
						continue;

					var match = href.match(/^android-app:\/\/com\.tumblr\/tumblr\/x-callback-url\/blog\?blogName=(.*?)(?:[&%].*)?$/);
					if (match) {
						return match[1];
					}
				}

				return null;
			};

			var find_blogname_from_initialstate = function() {
				if (initialstate && "tumblelog" in initialstate && "name" in initialstate.tumblelog) {
					return initialstate.tumblelog.name;
				}

				return null;
			};

			var find_postid_from_el = function(el) {
				var currentel = el;

				while ((currentel = currentel.parentElement)) {
					if (currentel.tagName === "ARTICLE" && currentel.hasAttribute("data-id")) {
						var id = currentel.getAttribute("data-id");
						if (/^[0-9]+$/.test(id)) {
							return id;
						}
					}

					var id = currentel.getAttribute("id");

					if (id) {
						var match = id.match(/^photoset_([0-9]+)$/);
						if (match) {
							return match[1];
						}

						if (currentel.tagName === "ARTICLE") {
							return id;
						}
					}

					if (currentel.tagName === "A") {
						var match = currentel.href.match(/:\/\/[^/]+\/+image\/+([0-9]{8,})\/*(?:[?#].*)?$/);
						if (match) {
							return match[1];
						}
					}

					if (currentel.tagName === "FIGURE") {
						try {
							var as = currentel.parentElement.parentElement.getElementsByTagName("a");
							for (var i = 0; i < as.length; i++) {
								var match = as[i].href.match(/:\/\/[^/]+\/+(?:image|post)\/+([0-9]{8,})\/*(?:[?#].*)?$/);
								if (match) {
									return match[1];
								}
							}
						} catch (e) {
							console_error(e);
						}
					}
				}

				return null;
			};

			var el_has_mediakey = function(el, mediakey) {
				if (el.tagName === "IMG") {
					return find_mediakey_from_url(el.src) === mediakey;
				} else if (el.tagName === "IFRAME" && /\/post\/+[0-9]+\/+/.test(el.src)) {
					try {
						var images = el.contentDocument.querySelectorAll("img");
						for (var i = 0; i < images.length; i++) {
							if (el_has_mediakey(images[i], mediakey))
								return true;
						}
					} catch (e) {
						console_error(e, el);
						return false;
					}
				}

				return false;
			};

			var try_finding_postid_from_url = function(url) {
				if (!options.document)
					return null;

				var mediakey = find_mediakey_from_url(url);
				if (!mediakey)
					return null;

				var cache_key = "tumblr_url_postid:" + mediakey;
				if (api_cache.has(cache_key)) {
					return api_cache.get(cache_key);
				}

				var els = options.document.querySelectorAll("article img, article iframe");
				for (var i = 0; i < els.length; i++) {
					if (el_has_mediakey(els[i], mediakey)) {
						var postid = find_postid_from_el(els[i]);
						if (postid) {
							api_cache.set(cache_key, postid);
						}

						return postid;
					}
				}

				return null;
			}

			var try_finding_info = function() {
				var info = find_blogname_id_from_url(options.host_url);
				if (info)
					return info;

				var blogname = find_blogname_from_url(options.host_url);
				if (!blogname) {
					blogname = find_blogname_from_head();
				}

				if (!blogname) {
					blogname = find_blogname_from_initialstate();
				}

				if (blogname) {
					var obj = {
						blogname: blogname
					};

					var postid = find_postid_from_el(options.element);
					if (!postid) {
						postid = try_finding_postid_from_url(src);
					}

					if (postid) {
						obj.postid = postid;
						return obj;
					}
				} else if (host_domain_nowww === "tumblr.com") {
					var currentel = options.element;
					while ((currentel = currentel.parentElement)) {
						if ((currentel.tagName === "DIV" || currentel.tagName === "ARTICLE") && currentel.getAttribute("data-json")) {
							var jsondata = currentel.getAttribute("data-json");
							try {
								var json = JSON_parse(jsondata);
								return {
									blogname: json.tumblelog,
									postid: json["post-id"]
								};
							} catch(e) {
								console_error(e);
							}

							return null;
						}
					}
				}

				return null;
			};

			var mediakey = find_mediakey_from_url(src);
			if (mediakey) {
				var info = try_finding_info();
				if (info) {
					get_post(info.blogname, info.postid, function(data) {
						if (!data) {
							console_warn("Unable to get post data from API", info);
							return options.cb(null);
						}

						var media = find_media_from_post_mediakey(data, mediakey);
						if (!media) {
							console_warn("Unable to find media from post and mediakey", data, mediakey);
							return options.cb(null);
						}

						var largest = find_largest_from_media(media);

						var obj = {url: largest.url};
						if (largest.has_original_dimensions)
							obj.is_original = true;

						return options.cb(obj);
					});

					return {
						waiting: true
					};
				}
			}
		}

		if (domain === "wc-ahba9see.c.sakurastorage.jp" ||
			domain === "cdn.worldcosplay.net") {
			newsrc = src;
			if (string_indexof(src, "/max-1200/") >= 0) {
				newsrc = src.replace(/-[0-9a-z]+(\.[^/.]*)$/, "-1200$1");
			} else {
				newsrc = src.replace(/-[0-9]+(?:x[0-9]+)?(\.[^/.]*)$/, "-3000$1");
			}

			return {
				url: newsrc,
				can_head: false,
				headers: {
					Referer: ""
				},
				referer_ok: {
					same_domain: true
				}
			};
		}

		if (domain_nowww === "nautiljon.com" && src.match(/\/images[a-z]*\//)) {
			return src
				.replace(/\/imagesmin\//, "/images/")
				.replace(/\/images\/[0-9]+x[0-9]+\//, "/images/")
				.replace(/\/mini\/([^/]*)$/, "/$1")
				.replace(/(\/[0-9]+\/[0-9]+\/)[a-z]+\/([^/]*)$/, "$1$2");
		}


		if (domain === "art.wsj.net") {
			if (string_indexof(src, "/api/photos/gams-files/") >= 0) {
				return src.replace(/\/gams-files\/[^-_/.]*-[^-_/.]*_([^/_.]*)_.*$/, "/gams-id:$1");
			}

			if (string_indexof(src, "/api/photos/gams-id:") >= 0) {
				return src.replace(/(\/gams-id:[^/]*)\/.*$/, "$1");
			}
		}

		if (domain_nosub === "fanpop.com" &&
			domain.match(/images[0-9]*\.fanpop\.com/)) {
			return src
				.replace(/([0-9]+)-[0-9]+-[0-9]+(\.[^/.]*)$/, "$1$2")
				.replace(/([0-9]+)_[0-9]+_[0-9]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "image.jimcdn.com") {
			return src.replace(/(\/app\/cms\/image\/transf\/)[^/]*\//, "$1none/");
		}


		if ((domain_nosub === "ladmedia.fr" ||
			 domain_nosub === "lanmedia.fr") &&
			domain.match(/(?:resize|cdn)[0-9]*-[a-z]*\./)) {
			return {
				url: src
					.replace(/\/(?:r|crop|rcrop)\/[^/]*\//, "/")
					.replace(/:\/\/resize[0-9]*-([a-z]+[^/]*?)\/+img\/var\//, "://cdn-$1/var/")
					.replace(/_[a-z0-9_]+(\.[^/.]*)$/, "$1"),
				can_head: true
			};
		}

		if (domain_nosub === "imgbox.com" &&
			(domain.match(/^thumbs[0-9]*\./) ||
			 domain.match(/images[0-9]*\./))) {
			newsrc = src
				.replace(/\/thumbs([0-9]*)\.imgbox\.com\//, "/images$1.imgbox.com/")
				.replace(/_[a-z](\.[^/.]*)/, "_o$1");
			if (newsrc !== src)
				return newsrc;

			if (/\/(?:[0-9a-f]{2}\/+){2}[^/_.]+_o\./.test(src)) {
				return {
					url: src,
					is_original: true
				};
			}
		}

		if ((domain_nosub === "steamstatic.com" && domain.match(/cdn\.[^.]*\.steamstatic\.com/)) ||
			(domain_nosub === "akamaihd.net" && domain.match(/steamcdn(?:-[a-z]*)?\.akamaihd\.net/))) {
			newsrc = src.replace(/(\/steam\/+apps\/+[0-9]+\/+movie)[0-9]+(\.[^/.]+)(?:[?#].*)?$/, "$1_max$2");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/(\/steam\/+apps\/+[0-9]+\/+movie)\.(?:jpg|JPG|jpeg|JPEG|png|PNG)(?:[?#].*)?$/, "$1_max.webm");
			if (newsrc !== src)
				return {
					url: newsrc,
					video: true
				};

			newsrc = src.replace(/(\/steamcommunity\/+public\/+images\/+clans\/+[0-9]+\/+[0-9a-f]{20,})_[0-9]+x[0-9]+(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
			if (newsrc !== src)
				return newsrc;

			if (string_indexof(src, "/public/images/avatars/") >= 0) {
				src = src.replace(/(?:_[^/.]*)?(\.[^/.]*)$/, "_full$1");
			}
			return src.replace(/\.[0-9]+x[0-9]+(\.[^/]*)$/, "$1");
		}

		if (domain === "steamstore-a.akamaihd.net") {
			if (/\/public\/+images\/+v[0-9]*\/+(?:app\/+game_page|(?:ico_)?game_).*/.test(src))
				return {
					url: src,
					bad: "mask"
				};
		}

		if (domain_nosub === "nutaku.com" && /^cdn[0-9]*-images\./.test(domain)) {
			return src.replace(/(\/images\/.*)-thumb(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "medium.com" &&
			(domain.match(/cdn-images-[0-9]*\.medium\.com/) ||
			 domain === "miro.medium.com")) {
			return src.replace(/(:\/\/[^/]*\/).*?\/([^/]*)$/, "$1$2");
		}

		if ((domain_nosub === "zimbio.com" ||
			 domain_nosub === "stylebistro.com" ||
			 domain_nosub === "livingly.com") &&
			domain.match(/www[0-9]*\.pictures\.(.*\.)?[a-z]+\.com/)) {
			return src.replace(/[a-z](\.[^/.]*)$/, "x$1");
		}

		if (domain_nowww === "theplace2.ru" ||
			domain_nowww === "theplace.ru" ||
			domain === "image.chaojimote.com" ||
			domain === "moviepic.manmankan.com" ||
			domain === "img.star.iecity.com") {
			newsrc = src.replace(/_[a-z](\.[^/.]*)$/, "$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "theplace2.ru" ||
			domain_nowww === "theplace.ru") {
			return src.replace(/(:\/\/[^/]*\/)cache\/(.*?)-g[^/.]*(\.[^/.]*)/, "$1$2$3");
		}

		if ((domain_nosub === "craveonline.com" && domain.match(/cdn[0-9]*-www\.craveonline\.com/)) ||
			domain_nowww === "legswiki.com" ||
			(domain_nosub === "mandatory.com" && domain.match(/^cdn[0-9]*-www\./)) ||
			domain_nowww === "indiancinemagallery.com" ||
			(domain_nowww === "allwomensites.com" && string_indexof(src, "/gallery/") >= 0) ||
			domain_nowww === "zemanceleblegs.com" ||
			domain === "blog.chron.com" ||
			(domain_nosub === "comingsoon.net" && domain.match(/cdn[0-9]*-www\./)) ||
			(domain_nosub === "dogtime.com" && domain.match(/cdn[0-9]*-www\./))) {
			newsrc = src.replace("/thumbs/thumbs_", "/");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/-[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "static.tvgcdn.net") {
			return src
				.replace("/smallcrops/", "/")
				.replace("/thumbs/", "/")
				.replace(/sm(\.[^/.]*)$/, "$1")
				.replace(/_[0-9]+x[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain_nosub === "blogcdn.com") {
			return src
				.replace(/(\/S[0-9]+\/)[a-z](\.[^/.]*)$/, "$1l$2")
				.replace(/\/slug\/[a-z]\//, "/slug/l/");
		}

		if (domain === "photos.imageevent.com") {
			return src.replace(/\/(?:small|large|huge|giant|icons)\/([^/]*)$/, "/$1");
		}

		if (domain === "image.ajunews.com") {
			return src.replace(/_[0-9]*_[0-9]*(\.[^/.]*)$/, "$1");
		}

		if (domain_nosub === "telegraph.co.uk" &&
			(domain_nowww === "telegraph.co.uk" ||
			 string_indexof(domain, "aws.telegraph.co.uk") >= 0 ||
			 domain === "subscriber.telegraph.co.uk")) {
			return src.replace(/-(?:x*(?:large|medium|small))(_[^/]*)?(\.[^/.]*$)$/, "$1$2");
		}


		if (domain === "image.munhwa.com") {
			return src.replace("/gen_thumb/", "/gen_news/").replace(/_[^/._]*(\.[^/.]*$)/, "_b$1");
		}

		if (domain_nosub === "dspmedia.co.kr") {
			return src.replace(/(\/file\/[^/]*\/)thumb_[0-9]*x[0-9]*[^/]*\//, "$1");
		}

		if (domain_nosub === "wixmp.com" &&
			options && options.do_request && options.cb && (!options._internal_info || !options._internal_info.deviantart_page)) {
			match = src.match(/^[a-z]+:\/\/(?:images-)?wixmp-[0-9a-f]+\.wixmp\.com\/+(?:intermediary\/+)?[^/]*\/+[-0-9a-f]+\/+([0-9a-z]+)-[-0-9a-f]+\.[^/.]+(?:\/+v[0-9]*\/.*?)?(?:[?#].*)?$/);
			if (match) {
				id = match[1];

				common_functions.deviantart_page_from_id(options.do_request, api_cache, id, function(result) {
					options._internal_info.deviantart_page = (result && result.finalUrl) || true;

					if (!result) {
						return options.cb({
							url: null,
							waiting: false
						});
					}

					common_functions.deviantart_fullimage(options, api_cache, src, id, function(obj) {
						if (!obj) {
							obj = {
								url: src,
								extra: {
									page: result.finalUrl
								}
							};
						}

						obj.forcerecurse = true;

						if (!obj.extra) {
							obj.extra = {
								page: result.finalUrl
							};
						}

						return options.cb(obj);
					});
				});

				return {
					waiting: true
				};
			}
		}

		if (domain === "static.wixstatic.com" ||
			domain_nosub === "wixmp.com") {

			if (/^[a-z]+:\/\/[^/]+\/+media\/+0da768_361994b0f154464682a0aaf9724471cc\./.test(src)) {
				return {
					url: src,
					bad: "mask"
				};
			}

			obj = {};

			if (options._internal_info && options._internal_info.deviantart_page && typeof options._internal_info.deviantart_page === "string") {
				obj.extra = {page: options._internal_info.deviantart_page};
			} else {
				obj._copy_old_props = ["extra"];
			}

			newsrc = src.replace(/(:\/\/[^/]*\/)(f\/+[-0-9a-f]{36}\/+[0-9a-z]+-[-0-9a-f]{20,}(?:\.[^/.]*)?\/+v1\/+fill\/+w_[0-9]+,h_[0-9]+)(?:,[^/]+)?(\/+.*[?&]token=.*)$/, "$1$2,q_100$3");
			if (newsrc !== src) {
				obj.url = newsrc;
				return obj;
			}

			newsrc = src.replace(/(:\/\/[^/]*\/)(f\/+[-0-9a-f]{36}\/+.*?)[?&]token=.*$/, "$1intermediary/$2");
			if (newsrc !== src) {
				obj.likely_broken = true;
				obj.url = newsrc;
				return obj;
			}

			if (!src.match(/[?&]token=.{30,}/)) {
				newsrc = src
					.replace(/(\.[^/.]*)\/v1\/.*/, "$1")
					.replace(/(\/[^/.]*\.[^/.]*?)_[_0-9.a-z]*$/, "$1");

				if (newsrc !== src) {
					obj.url = newsrc;
					return obj;
				}
			}
		}

		if (domain_nosub === "kukinews.com" ||
			domain_nowww === "inews365.com" ||
			domain_nowww === "newsinstar.com" ||
			domain_nowww === "artkoreatv.com" ||
			domain_nowww === "cnbnews.com" ||
			domain_nowww === "newsgg.net" ||
			domain_nowww === "newsam.co.kr" ||
			domain_nowww === "fetv.co.kr" ||
			domain_nowww === "ddaily.co.kr") {
			return src
				.replace(/\/data\/+cache\/+public\//, "/data/")
				.replace(/_[0-9]+x[0-9]+(?:_c[0-9]*)?\.([^/.]*)/, ".$1");
		}

		if (domain === "cdn.emetro.co.kr") {
			return src
				.replace(/\/image_view\.php.*?[?&]f=([^&]*).*/, "/image_view_maxw.php?f=$1&x=9999999999&ds=9999999999")
				.replace(/\/imagebank\/[0-9]*\/[0-9]*\/[0-9]*\/[0-9]*\/([^/]*)$/, "/html/image_view_maxw.php?f=$1&x=9999999999&ds=9999999999")
				.replace(/\/image_view_maxw.php.*?[?&]f=([^&]*).*/, "/image_view_maxw.php?f=$1&x=9999999999&ds=9999999999");
			/*origsize = src.match(/\/([0-9]*)\/[^/]*$/);
			if (origsize) {
				size = parseInt(origsize[1], 10);
				if (size < 1024) {
					return src.replace(/\/[0-9]*(\/[^/]*)$/, "/1024$1");
				}
			}*/
		}

		if (domain === "50.7.164.242:8182" ||
			(domain_nosub === "imgspice.com" && domain.match(/^img[0-9]*\./)) ||
			(domain_nosub === "imageporter.com" && domain.match(/^img[0-9]*\./)) ||
			(domain_nosub === "pixroute.com" && domain.match(/img[0-9]*\./))) {
			newsrc = src.replace(/(\/i\/.*\/[^/.]*)_t(\.[^/.]*)$/, "$1$2");

			obj = {
				url: src,
				headers: {
					Referer: ""
				},
				bad_if: [
					{
						headers: {
							"content-length": "40275",
							"content-type": "image/jpeg",
							"last-modified": "Sat, 09 Mar 2019 01:29:42 GMT"
						}
					}
				]
			};

			if (newsrc !== src) {
				return fillobj_urls(add_extensions_jpeg(newsrc), obj);
			} else {
				return obj;
			}
		}

		if (domain_nosub === "imagetwist.com" && src.match(/:\/\/[^/]*\/+error\.[^/.]*(?:[?#].*)?$/)) {
			return {
				url: src,
				bad: true
			};
		}

		if ((domain_nosub === "imagetwist.com" ||
			 domain_nosub === "imagexport.com" ||
			 domain_nosub === "imagenpic.com" ||
			 domain_nosub === "picshick.com") &&
			domain.match(/i(?:mg)?[0-9]*\./)) {

			id = src.replace(/^([a-z]+:\/\/)(?:[^/.]*\.)?([^/.]+\.[^/.]+\/+)th\/+[0-9]+\/+([0-9a-z]+)(\.[^/.]*)?$/,
							 "$1$2$3/$4");
			if (id !== src && options && options.cb && options.do_request) {
				options.do_request({
					method: "GET",
					url: id,
					onload: function(resp) {
						if (resp.readyState === 4) {
							var match = resp.responseText.match(/<a\s+href=["'](https?:\/\/i(?:mg)?[0-9]*\.[^/]+\/+.*?)["'][^>]*\sdownload>/);
							if (match) {
								options.cb({
									url: match[1],
									is_original: true
								});
							} else {
								options.cb(null);
							}
						}
					}
				});

				return {
					waiting: true
				};
			}

			return {
				url: src.replace(/\/th\//, "/i/"),
				headers: {
					Referer: src.replace(/:\/\/(?:[^/]*\.)?([^/.]*\.[^/.]*)\/.*/, "://$1/"),
					Origin: src.replace(/:\/\/(?:[^/]*\.)?([^/.]*\.[^/.]*)\/.*/, "://$1/"),
					"Sec-Metadata": "destination=image, site=same-site"
				}
			};
		}

		if (domain_nosub === "fappic.com") {
			id = src.replace(/^([a-z]+:\/\/)(?:[^/.]*\.)?([^/.]+\.[^/.]+\/+)(?:th|i)\/+[0-9]+\/+([0-9a-z]+)(?:_t)?(\.[^/.]*)?$/,
							 "$1$2$3/$4");
			if (id !== src && options && options.cb && options.do_request) {
				options.do_request({
					method: "GET",
					url: id,
					onload: function(resp) {
						if (resp.readyState === 4) {
							var match = resp.responseText.match(/<img\s+src=["'](https?:\/\/[^/]+\/+img\/+.*?)["'][^>]*\s+class=['"]pic["']/);
							if (match) {
								options.cb({
									url: match[1],
									is_original: true
								});
							} else {
								options.cb(null);
							}
						}
					}
				});

				return {
					waiting: true
				};
			}

			return {
				url: src,
				headers: {
					Referer: src.replace(/:\/\/(?:[^/]*\.)?([^/.]*\.[^/.]*)\/.*/, "://$1/"),
					Origin: src.replace(/:\/\/(?:[^/]*\.)?([^/.]*\.[^/.]*)\/.*/, "://$1/"),
					"Sec-Metadata": "destination=image, site=same-site"
				}
			};
		}

		if (domain_nosub === "imgflare.com" ||
			domain_nosub === "imgbabes.com") {
			id = src.replace(/^([a-z]+:\/\/)(?:[^/.]*\.)?([^/.]+\.[^/.]+\/+)i\/+[0-9]+\/+([0-9a-z]+)(?:_t)?(\.[^/.]*)?$/,
							 "$1www.$2$3/$4");
			if (id !== src && options && options.cb && options.do_request) {
				common_functions.get_testcookie_cookie(options, api_cache, "http://www." + domain_nosub + "/", function (cookie) {
					if (!cookie) {
						return options.cb(null);
					}

					options.do_request({
						method: "GET",
						headers: {
							Referer: "",
							Cookie: cookie
						},
						url: id + ".html",
						onload: function (resp) {
							if (resp.readyState === 4) {
								var match = resp.responseText.match(/<img\s+src=["'](https?:\/\/[^/]+\/+files\/+.*?)["'][^>]*\s+class=['"]pic["']/);
								if (match) {
									options.cb({
										url: match[1],
										is_original: true
									});
								} else {
									options.cb(null);
								}
							}
						}
					});
				});

				return {
					waiting: true
				};
			}

			return {
				url: src,
				headers: {
					Referer: src.replace(/:\/\/(?:[^/]*\.)?([^/.]*\.[^/.]*)\/.*/, "://$1/"),
					Origin: src.replace(/:\/\/(?:[^/]*\.)?([^/.]*\.[^/.]*)\/.*/, "://$1/"),
					"Sec-Metadata": "destination=image, site=same-site"
				}
			};
		}

		if (domain === "www.theactuary.com") {
			return src.replace(/getresource\.axd\?.*(AssetID=[0-9]*).*/, "getresource.axd?$1");
		}

		if (domain === "static.new-magazine.co.uk" ||
			amazon_container === "star-magazine.co.uk") {
			return src.replace(/(\/prod\/media\/images\/)[^/]*\//, "$1original/");
		}

		if (domain_nowww === "irishexaminer.com" ||
			domain_nowww === "breakingnews.ie" ||
			domain === "ip.index.hr" ||
			domain === "images.radiotimes.com" ||
			domain_nowww === "mffashion.com" ||
			domain === "download.iask.ca" ||
			domain === "ip.trueachievements.com") {
			newsrc = src.replace(/.*:\/\/[^/]*\/remote\/([^?]*).*/, "$1");
			if (newsrc !== src)
				return "http://" + decodeURIComponent(newsrc);
		}

		if (domain === "images.radiotimes.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/namedimage\/[^?]*.*?[?&]url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return urljoin(src, newsrc, true);
		}

		if (domain === "images.radiotimes.com" ||
			domain_nowww === "radiotimes.com") {
			return src.replace(/(\/uploads\/+images\/+[a-zA-Z]+\/+[0-9]+(?:\.[-0-9a-f]+)?\.[^/.?]*)(?:[?#].*)?$/, "$1");
		}

		if ((domain === "tellymix-spykawebgroup.netdna-ssl.com" ||
			 domain_nosub === "tellymixcdn.com") &&
			src.match(/:\/\/[^/]*\/+ts\//)) {
			return src.replace(/^[a-z]+:\/\/[^/]*\/+ts\/+[0-9]*\/+[0-9]*\/+/, "http://");
		}

		if (domain === "assets.goodhousekeeping.co.uk") {
			return src.replace(/(\/(?:embedded|galleries)\/+(?:[0-9]*\/+)[^/]*)__[^/.]*(\.[^/]*)$/, "$1$2");
		}


		if (domain === "img.buzzfeed.com") {
			return src
				.replace(/_big(\.[^/.]*)$/, "_dblbig$1")
				.replace(/_wide(\.[^/.]*)$/, "_dblbig$1")
				.replace(/_dblwide(\.[^/.]*)$/, "_dblbig$1");
		}

		if (domain_nowww === "thegenealogist.co.uk") {
			return src.replace(/\/images\/+featuredarticles\/+header_sm\//, "/images/featuredarticles/header_lg/");
		}

		if (domain === "251d2191a60056d6ba74-1671eccf3a0275494885881efb0852a4.ssl.cf1.rackcdn.com" ||
			domain === "8583b52b4a309671f69d-b436b898353c7dc300b5887446a26466.ssl.cf1.rackcdn.com" ||
			domain === "2e0a24317f4a9294563f-26c3b154822345d9dde0204930c49e9c.ssl.cf1.rackcdn.com" ||
			domain === "7f9c61237bd6e732e57e-5fa18836a2ae6b5e7c49abcc89b20237.ssl.cf1.rackcdn.com" ||
			domain === "b6c18f286245704fe3e9-05e2055f4cd9122af02914269431c9f6.ssl.cf1.rackcdn.com" ||
			domain === "41dcdfcd4dea0e5aba20-931851ca4d0d7cdafe33022cf8264a37.ssl.cf1.rackcdn.com" ||
			domain === "575717b777ff8d928c6b-704c46a8034042e4fc898baf7b3e75d9.ssl.cf1.rackcdn.com" ||
			domain === "598d5fcf392acad97538-395e64798090ee0a3a571e8c148d44f2.ssl.cf1.rackcdn.com") {
			return src.replace(/(\/[^/.]*)_[a-z](\.[^/.?]*)(?:\?[^/]*)?$/, "$1$2");
		}

		if (domain === "be35832fa5168a30acd6-5c7e0f2623ae37b4a933167fe83d71b5.ssl.cf3.rackcdn.com") {
			return src.replace(/__hero(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain === "7e7781e2f767494a7128-d0458b7f69ca26f2871554faadb4939e.ssl.cf5.rackcdn.com") {
			if (/\/wp-content\/+themes\/+photolux\/+images\//.test(src))
				return {
					url: src,
					bad: "mask"
				};
		}

		if (domain_nosub === "shopify.com" && /^cdn[0-9]*\./.test(domain)) {
			return src.replace(/_(?:large|medium|small|grande|compact|[0-9]+x(?:[0-9]+)?)(?:@[0-9]+x)?(?:_crop_[a-z]+)?(?:\.progressive)?(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain === "cdn.itv.com") {
			return src.replace(/\/[a-z]*_([^/_]*)$/, "/$1");
		}

		if (domain === "thumbnailer.mixcloud.com") {
			return src.replace(/\/unsafe\/[0-9]+x[0-9]+\//, "/unsafe/0x0/");
		}

		if (domain === "d3mkh5naggjddw.cloudfront.net" ||
			domain === "img.blvds.com" ||
			domain === "resizer.mundotkm.com" ||
			domain === "d2isyty7gbnm74.cloudfront.net" ||
			domain === "thumb.connect360.vn" ||
			domain === "img.movietimes.com" ||
			domain === "t.t2online.co.in" ||
			domain === "thum.buzzni.com" ||
			domain === "resizer.mundotkm.com" ||
			domain === "quicksilver.scoopwhoop.com" ||
			domain_nosub === "telegraphindia.com" ||
			domain === "mediacdn.libertatea.ro" ||
			(domain_nosub === "genius.com" && domain.match(/t[0-9]*\.genius\.com/))) {
			return add_http(decodeURIComponent(src
											   .replace(/.*\/unsafe\/(?:fit-in\/)?(?:[0-9]+x[0-9]+\/)?smart\/(?:filters:[^/]*\/)?/, "")
											   .replace(/.*\/unsafe\/fit-in\/smart\//, "")
											   .replace(/.*\/unsafe\/(?:fit-in\/)?(?:[0-9]*x[0-9]*\/)?(?:center\/)?(?:smart\/)?/, "")));
		}

		if (domain === "cdn.uwants.com" ||
			domain === "cdn.discuss.com.hk") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/+t\/+[0-9a-f]+\/+f\/+[0-9]+x[0-9]+\/+(?:filters:[^/]*\/+)?(https?:)/, "$1");
		}

		if (domain === "elsewhere.scdn3.secure.raxcdn.com") {
			return src.replace(/\/images\/[sv][0-9]+\/articles\//, "/images/downloads/articles/");
		}

		if (domain === "japantoday-asset.scdn3.secure.raxcdn.com") {
			return src.replace(/(\/+img\/+store\/+[0-9a-f]{2}\/+[0-9a-f]{2}\/+[0-9a-f]+\/+[^/]*)(?:\/+_[wh][0-9]+)?(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "static01.nyt.com" ||
			(domain_nosub === "nytimes.com" && domain.match(/^graphics[0-9]*\./)) ||
			domain === "cdn1.nyt.com") {
			var matched = src.match(/-([^-_/.]*?)(?:-v[0-9]*)?\.[^/.]*$/);
			if (matched) {
				if (matched[1] === "jumbo" ||
					matched[1] === "thumbStandard" ||
					matched[1] === "facebookJumbo" ||
					matched[1] === "articleInline" ||
					matched[1] === "articleLarge" ||
					matched[1] === "sfSpan" ||
					matched[1] === "videoLarge" ||
					matched[1] === "thumbLarge" ||
					matched[1].match(/^videoSixteenByNine(?:Jumbo)?(?:[0-9]{0,3}|[01][0-9]{3}|20[0-4][0-9])?$/) ||
					matched[1].match(/^mediumThreeByTwo[0-9]*/) ||
					matched[1].match(/^threeByTwoSmall/) ||
					matched[1].match(/^watch[0-9]*$/) ||
					matched[1].slice(0, 6) === "master" ||
					matched[1].slice(0, 6) === "square" ||
					matched[1].slice(0, 4) === "blog") {
					newsrc = src.replace(/-[^-_/.]*(-v[0-9]*)?(\.[^/.]*)$/, "-superJumbo$1$2");
					if (newsrc !== src) {
						if (newsrc.match(/-v[0-9]*\.[^/.]*$/)) {
							return [
								newsrc,
								newsrc.replace(/-v[0-9]*(\.[^/.]*)$/, "$1")
							];
						} else {
							return [
								newsrc,
								newsrc.replace(/-superJumbo(\.[^/.]*)$/, "-jumbo$1")
							];
						}
					}
				}
			}
		}

		if (domain === "render.fineartamerica.com") {
			return src.replace(/render\.fineartamerica\.com\/images\/rendered\/search\/print\/[^/]*(-[0-9]*)\/([^/]*)$/, "images.fineartamerica.com/images-medium-large$1/$2");
		}

		if (domain === "media.npr.org") {
			return {
				url: src
					.replace(/(\/[^/]*)-[sc][0-9]*(?:-[sc][0-9]*)?(\.[^/.]*)/, "$1$2")
					.replace(/(_custom)?(?:_[a-z]+)?-([a-f0-9]{30,})(\.[^/.]*)$/, "$1-$2$3"),
				head_wrong_contentlength: true
			};
		}

		if (domain_nosub === "pbsrc.com" && domain.match(/rs[0-9]*\.pbsrc\.com/)) {
			return src
				.replace(/rs([0-9]*)\.pbsrc\.com/, "i$1.photobucket.com")
				.replace(/\?.*/, "")
				.replace(/(?:~[^/.]*)?$/, "~original");
		}

		if (domain_nosub === "photobucket.com" && /^i(?:[0-9]+|mg)\./.test(domain)) {
			newsrc = src.replace(/\?.*/, "");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/(?:~[^/.]*)?(?:[?#].*)?$/, "~original");
			if (newsrc !== src)
				return newsrc;

			return src.replace(/:\/\/i((?:[0-9]+|mg)\..*?)(?:~original)?(?:[?#].*)?$/, "://oi$1");
		}

		if (domain_nosub === "photobucket.com" && /^o?i(?:[0-9]*|mg)\./.test(domain)) {
			match = src.match(/^[a-z]+:\/\/[a-z]*([0-9]+)?\.[^/]*\/+albums\/+[a-z]{1,2}[0-9]+\/+([^/]+)\/+([^/]+\.[^/.~]*?)(?:~[^/.]*)?(?:[?#].*)?$/);
			if (match) {
				var domain = "www";
				if (match[1])
					domain = "s" + match[1];

				var page = "http://" + domain + ".photobucket.com/user/" + match[2] + "/media/" + match[3] + ".html";
				return {
					url: src,
					headers: {
						Referer: page
					},
					extra: {
						page: page
					}
				};
			}
		}

		if (domain === "www.welt.de") {
			return src.replace(/-w[0-9]*(\/[^/]*)$/, "-w0$1");
		}

		if (domain === "cdn.baeblemusic.com" && string_indexof(src, "/images/") >= 0) {
			return src.replace(/-[0-9]*(\.[^/.]*)$/, "$1");
		}

		if (domain_nosub === "deviantart.net" && domain.match(/^t[0-9]*\./)) {
			return src.replace(/:\/\/.*?\.deviantart\.net\/.*?\/[0-9]*x[0-9]*\/[^/]*\/([^/]*)\/(.*)/, "://$1\.deviantart\.net/$2");
		}

		if (domain_nosub === "grazia.fr" && domain.match(/img[0-9]*\.grazia\.fr/) ||
			domain_nowww === "glamour.de" ||
			domain === "asia.nikkei.com" ||
			domain_nosub === "elle.co.jp" ||
			domain_nowww === "harpersbazaar.jp" ||
			domain_nowww === "fotogramas.es" ||
			domain_nosub === "ellegirl.jp" ||
			domain_nosub === "esquire.jp" ||
			domain === "img.25ans.jp" ||
			domain_nowww === "cosmopolitan.com.hk" ||
			domain_nowww === "elle.com.hk" ||
			domain_nosub === "vogue.de" ||
			(domain_nosub === "telestar.fr" && domain.match(/img[0-9]*\.telestar\.fr/)) ||
			domain_nowww === "haz.de" ||
			domain_nowww === "connexionfrance.com" ||
			domain_nowww === "novilist.hr" ||
			domain_nowww === "ville-wasquehal.fr" ||
			domain_nowww === "gq-magazin.de" ||
			domain_nowww === "esa.int" ||
			domain_nowww === "quejadore.com" ||
			(domain_nosub === "grazia.fr" && /^file[0-9]*\./.test(domain)) ||
			domain_nosub === "esquirehk.com" ||
			domain === "gfx.elle.ci" ||
			(domain_nosub === "closermag.fr" && domain.match(/(?:img|file)[0-9]*\.closermag\.fr/))) {
			return src
				.replace(/(:\/\/[^/]*\/var\/+(?:[^/]*\/+)?storage\/+images\/+.*\/[^/]+?)(?:_[a-z][^-/._]*(?:_[0-9]+){0,})?(\.[^/.?]*)(?:[?#].*)?$/, "$1$2")
				.replace(/\/storage\/+images\/+_aliases\/+[^/]*\/+/, "/storage/images/");
		}


		if ((domain_nosub === "purepeople.com" ||
			 domain_nosub === "purepeople.com.br" ||
			 domain_nosub === "purebreak.com" ||
			 domain_nosub === "purebreak.com.br" ||
			 domain_nosub === "get-the-look.ca" ||
			 domain_nosub === "hairstyle.com" ||
			 domain_nosub === "puretrend.com") &&
			domain.match(/^static[0-9]*\./)) {
			return src.replace(/([-_])[0-9]+(?:x[0-9]+)?(-[0-9]\.[^/.]*)/, "$1999999999x0$2");
		}

		if (domain_nosub === "belezaextraordinaria.com.br" && domain.match(/^static[0-9]*\./)) {
			return src.replace(/([-_])(?:[0-9]+(?:x[0-9]+)?|(?:article|opengraph)_[^-/.]+)(-[0-9]\.[^/.]*)/, "$1article_news$2");
		}

		if (domain === "medias.unifrance.org") {
			return src.replace("/format_web/", "/format_page/");
		}

		if (domain_nosub === "lisimg.com" ||
			domain_nosub === "listal.com") {
			return src
				.replace(/:\/\/[^\./]*\.lisimg\.com\//, "://ilarge.lisimg.com/")
				.replace(/\/([^/]*)\.jpg$/, "/99999999999full.jpg");
		}

		if (domain_nosub === "lesinrocks.com") {
			return src.replace(/\/width-[0-9]*-height-[0-9]*/, "/width-0-height-0");
		}

		if (domain === "media.senscritique.com") {
			return src.replace(/(\/media\/+[0-9]{8,}\/+)[0-9]+(?:[_x][0-9]+)?\/+([^/]*\.[^/.]*)(?:[?#].*)?$/, "$10/$2");
		}

		if (domain_nowww === "franceinter.fr" ||
			domain === "cdn.radiofrance.fr") {
			return src.replace(/\/[0-9]+(?:x[0-9]+)?_([^/]*\.[^/.]+)(?:[?#].*)?$/, "/$1");
		}

		if (domain === "1645110239.rsc.cdn77.org") {
			return src
				.replace(/\/image\/[a-z][0-9]+\//, "/image/") // to be repeated
				.replace(/\/image\/x[0-9]+x[0-9]+\//, "/image/")
				.replace(/\/([^/.]*)\.[0-9]+(\.[^/.]*)$/, "/$1$2");
		}

		if (domain_nowww === "diymag.com" && string_indexof(src, "/media/img") >= 0) {
			return src.replace(/(\/media\/img\/.*\/)_[^/]*\/([^/]*)$/, "$1$2");
		}

		if (domain_nowww === "semainedelacritique.com" && string_indexof(src, "/ttimg-rsz") >= 0) {
			newsrc = src.replace(/.*\/ttimg-rsz\?(?:.*&)?src=([^&]*).*/, "$1");
			if (newsrc !== src)
				return urljoin(src, newsrc, true);
		}

		if (domain_nosub === "pmdstatic.net" && domain.match(/img\..*?pmdstatic\.net$/)) {
			return decodeURIComponent(src.replace(/.*?\.pmdstatic\.net\/fit\/([^/]*).*/, "$1").replace(/\./g, "%"));
		}

		if (domain === "photo.gala.fr") {
			return {
				url: src,
				head_wrong_contentlength: true
			};
		}

		if (domain === "cdn.cnn.com" ||
			(domain_nosub === "turner.com" && domain.match(/(?:i[0-9]*\.)?cdn\.turner\.com/))) {
			return {
				url: src.replace(/-(?:small|medium|large|exlarge|super|full|overlay|alt|tease|story-top|horizontal)(?:-(?:small|medium|large|exlarge|super|full|overlay|alt|tease))?(?:-(?:[0-9]+|gallery))?(\.[^/.]*)$/, "$1"),
				can_head: false
			};
		}

		if (domain === "ugc.kn3.net"/* &&
			string_indexof(src, "/i/origin/") >= 0*/) {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/i\/(?:c_)?[0-9a-z]+\//, "");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "media.shoko.fr" ||
			domain === "media.fan2.fr" ||
			domain === "media.melty.it" ||
			domain === "media.virginradio.fr" ||
			domain === "media.melty.mx" ||
			domain === "media.melty.fr") {
			return src
				.replace(/(:\/\/[^/]*)\/([^/]*?)-[^-/]*((?:-f[^/]*)?\.[^/.]*)$/, "$1/$2-redim$3")
				.replace(/(:\/\/[^/]*)\/([^/-]*?-[0-9]*-)[^/-]*(-f[^/]*)?\//, "$1/$2redim$3/");
		}

		if (domain_nosub === "vogue.fr" ||
			domain_nowww === "glamourparis.com" ||
			domain_nowww === "gqmagazine.fr") {
			src = src.replace(/\/images\/+thumbs\//, "/images/");
			newsrc = src.replace(/(\.[^/._]*)_[^/]*$/, "$1");
			if (newsrc !== src)
				return newsrc;
			return src.replace(/_north_[0-9]*x(?:[0-9]+)?_(?:white|transparent)(\.[^/.]*)$/, "$1");
		}

		if (domain === "img.culturacolectiva.com") {
			newsrc = src.replace(/-(?:high|medium|low)(\.[^/.]*)$/, "$1");
			if (newsrc !== src)
				return newsrc;
		}

		if ((domain_nosub === "reveliststatic.com" ||
			 domain_nosub === "cafemom.com" ||
			 domain_nosub === "cafemomstatic.com") &&
			domain.match(/^(?:cdn-)?ugc(?:-[0-9]*)?\./)) {

			return src.replace(/\/gen\/(?:constrain|crop|resize|min)\/[0-9]*\/[0-9]*\/[0-9]*\//, "/gen/full/");
		}

		if (domain === "images.shazam.com") {
			return src.replace(/_s[0-9]+(\.[^/.]*)$/, "_s0$1");
		}

		if (domain_nosub === "ebayimg.com") {

			obj = {
				url: src,
				can_head: false, // fixme: does it hang here too?
				bad_if: [{
					headers: {
						"x-ebay-c-extension": "responsecode=404,responsemessage=Not Found"
					}
				}]
			};

			newsrc = src.replace(/\/t\/.*?(\/[0-9]+\/s\/)/, "$1");
			if (newsrc !== src) {
				return fillobj_urls([newsrc, src], obj);
			}

			newsrc = src.replace(/\/[0-9]+\/[a-z]+\/[^/]*\/[a-z]+\/([^/]+)\/[^/.]*(\.[^/.]*)$/, "/images/g/$1/s-l9999$2");
			if (newsrc !== src) {
				newsrc = newsrc.replace(/(.*\.)[^/.]*$/, "$1") + newsrc.replace(/.*\.([^/.]*)$/, "$1").toLowerCase();
				return fillobj_urls([newsrc, src], obj);
			}

			newsrc = src
				.replace(/\/thumbs\/images\//, "/images/")
				.replace(/-l[0-9]+(\.[^/.]*)$/, "-l9999$1");
			if (newsrc !== src) {
				return fillobj_urls([newsrc, src], obj);
			}

			return obj;
		}

		if ((domain_nosub === "ebaystatic.com" && domain.match(/thumbs[0-9]*\.ebaystatic\.com/)) ||
			domain === "securethumbs.ebay.com") {
			newsrc = src
				.replace(/^[a-z]*:\/\/[^/]*\/(.*?)\/[0-9]+(\.[^/.]*)$/, "https://ssli.ebayimg.com/images/$1/s-l9999$2")
				.replace(/^[a-z]*:\/\/[^/]*\/d\/l[0-9]+\/([a-z]\/[^/]*)(\.[^/.]*)$/, "https://ssli.ebayimg.com/images/$1/s-l9999$2");
			if (newsrc !== src) {
				return newsrc;
			}

			newsrc = src.replace(/\/d\/[a-z][0-9]+\/pict\//, "/d/l9999/pict/");
			if (newsrc !== src) {
				return newsrc;
			}

			return {
				url: src,
				can_head: false, // it just hangs
				bad_if: [{
					headers: {
						"x-ebay-c-extension": "responsecode=404,responsemessage=Not Found"
					}
				}]
			};
		}

		if (domain_nowww === "picclickimg.com") {
			return src
				.replace(/:\/\/www.picclickimg.com\/d\//, "://thumbs.ebaystatic.com/d/")
				.replace(/:\/\/www.picclickimg.com(\/[0-9]+\/s\/)/, "://i.ebayimg.com$1");
		}

		if (domain_nowww === "ezcorporateembroidery.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/packs\/images[a-z]+\/(https?:\/\/)/, "$1");
			if (newsrc !== src)
				return newsrc;
			return src.replace(/^[a-z]+:\/\/[^/]*\/images\/scale\/([a-z][0-9]+\/)/, "https://securethumbs.ebay.com/d/$1");
		}

		if (domain_nowww === "dealsanimg.com") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/+d\//, "https://securethumbs.ebay.com/d/");
		}

		if (domain === "i.slkimg.com") {
			return src
				.replace(/\/fill\/[0-9]+,[0-9]+\//, "/")
				.replace(/[0-9]+(\.[^/.]*)$/, "999999999$1");
		}

		if (domain === "i.vimeocdn.com" && string_indexof(src, "/filter/overlay") >= 0) {
			return decodeURIComponent(src.replace(/.*\/overlay\?.*?src0=([^&]*).*/, "$1"));
		}

		if (domain === "imagelab.nownews.com") {
			return decodeURIComponent(src.replace(/.*[/?&]src=(.*)$/, "$1"));
		}

		if (domain === "cdn.discordapp.com") {
			return src.replace(/\?size=[0-9]*$/, "?size=4096");
		}

		if (domain_nosub === "discordapp.net" && domain.match(/images-ext-[0-9]*\.discordapp\.net/)) {
			return decodeURIComponent(src.replace(/.*\/external\/[^/]*\/(?:([^/]*)\/)?(https?)\/(.*?)(?:\?[^/]*)?$/, "$2://$3$1"));
		}

		if (domain_nowww === "hot-korea.net") {
			return src.replace(/\/uploads\/+([^/]*\/+)thumbs\//, "/uploads/$1");
		}

		if (domain_nosub === "sndcdn.com" && domain.match(/i[0-9]*\.sndcdn\.com/)) {
			return src.replace(/-[^-/.]*(\.[^/.]*)$/, "-original$1");
		}

		if (domain === "media.licdn.com") {
			return src.replace(/\/shrinknp_[0-9]+_[0-9]+\//, "/");
		}

		if ((domain_nosub === "townnews.com" && domain.match(/bloximages\..*vip\.townnews\.com/)) ||
			amazon_container === "syd.cdn.coreweb.com.au") {
			return src.replace(/^(.*?:\/\/)[^/]*\//, "http://");
		}

		if (domain_nosub === "psbin.com" && domain.match(/cdn[^.]*\.psbin\.com/)) {
			return {
				url: src.replace(/\/img\/[^/]*=[^/]*\//, "/img/"), // repeated
				head_wrong_contentlength: true
			};
		}

		if (domain === "wac.450f.edgecastcdn.net") {
			return src.replace(/^(.*?:\/\/)[^/]*\/80450F\/(.*?)$/, "http://$2");
		}

		if (domain === "gp1.wac.edgecastcdn.net") {
			return src.replace(/(\/images\/[0-9]*\/[^/]*\/)[^/]*:[^/]*\//, "$1"); // repeated
		}

		if (domain === "www.century21.com") {
			return src.replace(/.*?\/photo\/[0-9a-z]*x[0-9a-z]*\//, "http://");
		}

		if (domain === "cdn.instructables.com") {
			return src.replace(/(:\/\/[^/]*\/)(.*)\.[^/.]*(\.[^/.]*)$/, "$1ORIG/$2$3");
		}

		if (domain_nosub === "pressreader.com" && domain.match(/cdn[0-9]*-img\.pressreader\.com/)) {
			return src.replace(/getimage\.aspx[^/]*[?&](regionKey=[^&]*).*$/, "getimage.aspx?$1");
		}

		if (domain_nowww === "layfielddesign.com") {
			return src.replace(/\/uploads\/+([^/]*)\/+_[^/]*\//, "/uploads/$1/");
		}

		if (googlestorage_container === "mediaslide-europe") {
			return src.replace(/\/[a-z]*-([^/]*)$/, "/$1");
		}

		if (domain_nosub === "netinfo.bg" && domain.match(/m[^.]*\.netinfo\.bg/)) {
			return src.replace(/([/=]media\/images\/[0-9]*\/[0-9]*\/)(?:r-)?[^-/.]*-[^-/.]*/, "$1orig-orig");
		}

		if (domain === "i.imgur.com") {

			if (src.match(/\/removed\.[a-zA-Z]+(?:[?#].*)?$/))
				return {
					url: src,
					bad: true
				};

			var baseobj = {
				url: src.replace(/\?.*/, ""),
				headers: {
					Referer: null
				},
				referer_ok: {
					same_domain_nosub: true
				}
			};

			var idregex = /^[a-z]+:\/\/[^/]+\/+([a-zA-Z0-9]{7})\./;

			newsrc = src.replace(/\/([a-zA-Z0-9]{7})(?:[hrlgmtbs]|_d)(\.[^/.?]*)$/, "/$1$2");
			if (newsrc !== src) {
				baseobj.url = newsrc;

				match = newsrc.match(idregex);
				if (match) {
					baseobj.extra = {page: "https://imgur.com/" + match[1]};
				}

				return baseobj;
			}

			match = src.match(idregex);
			var idhash;
			if (match) {
				idhash = match[1];
				baseobj.extra = {page: "https://imgur.com/" + idhash};
			}

			if (options && options.cb && options.do_request && baseobj.extra) {
				var parse_imageinfo = function(baseobj, json) {
					var retobj = [];

					try {
						if (json.description || json.title) {
							baseobj.extra.caption = json.description || json.title;
						}

						var realfilename = null;
						if (json.hash && json.ext) {
							realfilename = json.hash + json.ext;

							var obj = deepcopy(baseobj);
							obj.url = "https://i.imgur.com/" + realfilename;

							if (/^video\//.test(json.mimetype)) {
								obj.video = true;
								retobj.push(obj);

								var obj = deepcopy(baseobj);
								obj.url = "https://i.imgur.com/" + json.hash + ".jpg"
								retobj.push(obj);
							} else {
								if (json.animated && json.prefer_video) {
									var newobj = deepcopy(baseobj);
									newobj.url = "https://i.imgur.com/" + json.hash + ".mp4";
									newobj.video = true;

									retobj.push(newobj);
								}

								retobj.push(obj);
							}
						}

						if (json.source && /^https?:\/\//.test(json.source)) {
							if (!("rule_specific" in options) || !("imgur_source" in options.rule_specific) || options.rule_specific.imgur_source === true) {
								var newobj = {url: json.source};
								retobj.unshift(newobj);
							}
						}
					} catch (e) {
						console_error(e);
						console_log(match);

						retobj = [];
					}

					return retobj;
				};

				var imageinfo = api_cache.get("imgur_imageinfo:" + idhash);
				if (imageinfo) {
					var retobj = parse_imageinfo(baseobj, imageinfo);

					if (retobj.length > 0) {
						return retobj;
					}
				}

				var nsfw_headers = undefined;
				if (options.rule_specific && options.rule_specific.imgur_nsfw_headers) {
					nsfw_headers = options.rule_specific.imgur_nsfw_headers;
				}

				common_functions.fetch_imgur_webpage(options.do_request, api_cache, nsfw_headers, baseobj.extra.page, function(data) {
					if (!data) {
						return options.cb(obj);
					}

					if (data.bad) {
						var obj = deepcopy(baseobj);
						obj.bad = true;
						return options.cb(obj);
					}

					var retobj = [];

					if (data.ogvideo) {
						var obj = deepcopy(baseobj);
						obj.url = data.ogvideo;
						obj.video = true;

						retobj.push(obj);
					}

					if (data.ogimage) {
						var obj = deepcopy(baseobj);
						obj.url = data.ogimage;

						retobj.push(obj);
					}

					if (data.imageinfo) {
						var newretobj = parse_imageinfo(baseobj, data.imageinfo);

						if (newretobj.length > 0) {
							retobj = newretobj;
						}

						return options.cb(retobj);
					} else {
						return options.cb(retobj);
					}
				});

				return {
					waiting: true
				};
			}

			return baseobj;
		}

		if (domain === "imgur.dcard.tw" ||
			domain === "i.wgirlz.us") {
			return src.replace(/:\/\/[^/]*\//, "://i.imgur.com/");
		}

		if (domain_nowww === "siamzone.com") {
			return src
				.replace(/^[a-z]+:\/\/[^/]*\/board\/+(?:imgur\.php\?|imgur\/+.\/+)([^/]*)$/, "https://i.imgur.com/$1");
		}

		if (domain_nowww === "vidble.com" ||
			domain === "i.vidble.com") {
			return src.replace(/_[^/._]*(\.[^/.]*)$/, "$1");
		}

		if (domain === "itpro.nikkeibp.co.jp") {
			return src.replace(/\/thumb_[0-9]+_([^/]*)$/, "/$1");
		}

		if (domain === "media-cdn.tripadvisor.com") {
			return src.replace(/\/media\/photo-[a-z]\//, "/media/photo-o/");
		}

		if (domain_nowww === "traveller.com.au" ||
			domain === "resources.stuff.co.nz" ||
			domain_nowww === "fairfaxstatic.com.au" ||
			domain_nowww === "smh.com.au" ||
			domain_nowww === "dailylife.com.au" ||
			domain_nowww === "essentialbaby.com.au") {
			return src.replace(/(\/images\/+(?:[0-9a-z]\/+){4,}image\.).*$/, "$1");
		}

		if (domain_nowww === "getwallpapers.com" ||
			domain_nowww === "hintergrundbild.org" ||
			domain_nowww === "wallpapertag.com") {
			return src.replace(/\/wallpaper\/+[^/]*\/+((?:[0-9a-f]\/+){3})/, "/wallpaper/full/$1");
		}

		if (domain === "ideascdn.lego.com") {
			return src
				.replace(/-thumbnail[^/.]*(\.[^/.]*)$/, "$1")
				.replace(/-square[^/.]*(\.[^/.]*)$/, "$1");
		}

		if (domain === "images.newindianexpress.com" ||
			domain === "images.indulgexpress.com") {
			return src.replace(/\/[wh][0-9]+X[0-9]*\//, "/original/");
		}


		if (domain_nosub === "blastingcdn.com" && domain.match(/staticr[0-9]*\.blastingcdn\.com/)) {
			return src
				.replace(/\/b_[auto0-9]+x[auto0-9]+\/([^/]*)$/, "/$1")
				.replace(/\/[0-9]+x[0-9]+\/([^/]*)$/, "/main/$1");
		}

		if (domain_nowww === "gjdream.com") {
			return src.replace(/_tmb(\.[^/.]*)$/, "$1");
		}

		if (domain === "www.imaeil.com") {
			return src.replace(/\/m_wiz\/imgsrc[0-9]\.php.*?[?&]src=([^&]*).*/, "/$1");
		}

		if (domain === "www.kwnews.co.kr") {
			return src.replace(/\/kwnews_view\.asp?.*?kwurl=([0-9]{4})([0-9]{2})[0-9]*-([0-9]*)-[0-9]*(\.[^/.]*)$/, "/newsphoto/$1/$2/$3$4");
		}

		if (domain_nowww === "yeongnam.com" &&
			string_indexof(src, "/Photo/") >= 0) {
			return src.replace(/\/[A-Z]([^/]*)$/, "/R$1");
		}

		if (domain_nowww === "yeongnam.com" &&
			string_indexof(src, "/news/screennews/") >= 0) {
			return src
				.replace(/\/news\/screennews\/[0-9]+_[0-9]+_[A-Z]([0-9]{4})([0-9]{2})([0-9]{2})_([0-9]+)(\.[^/.]*)/,
						 "/Photo/$1/$2/$3/R$1$2$3.$4.jpeg");
		}

		if (domain === "db.kookje.co.kr") {
			match = src.match(/\/[A-Z]?([0-9]{8}\.[0-9]+)i[0-9]+\.[^/.]*(?:[?#].*)?$/);
			var extra = {};
			if (match) {
				extra.page = "http://www.kookje.co.kr/news2011/asp/newsbody.asp?key=" + match[1];
			}

			return {
				url: src.replace(/\/[A-Z]([0-9]{8}\.[0-9]+i[0-9]+\.[^/.]*)(?:[?#].*)?$/, "/L$1"),
				extra: extra
			};
		}

		if (domain_nowww === "kookje.co.kr") {
			return src
				.replace("/thumb/", "/")
				.replace(/.*\/[0-9]+_[0-9]+_([0-9]{4})([0-9]{4})([^/]*)$/, "http://db.kookje.co.kr/news2000/photo/$1/$2/L$1$2$3");
		}

		if (domain_nowww === "joongdo.co.kr" ||
			domain_nowww === "viva100.com") {
			return src.replace(/\/webdata\/+content\/+([0-9]{4}y\/+[0-9]{2}m\/+[0-9]{2}d\/+)crop([0-9]+\.[^/.]+)(?:[?#].*)?$/, "/file/$1$2");
		}



		if (domain === "img.asiatoday.co.kr") {
			match = src.match(/\/[0-9]{4}y\/+[0-9]{2}m\/+[0-9]{2}d\/+([0-9]{15,})_[0-9]+(?:_[0-9]+)?\./);
			if (match) {
				return {
					url: src,
					extra: {
						page: "http://www.asiatoday.co.kr/view.php?key=" + match[1]
					}
				};
			}
		}

		if (domain === "jmagazine.joins.com" ||
			(domain === "www.urbanbug.net" && string_indexof(src, "/uploads/") >= 0) ||
			(domain_nowww === "shobiddak.com" && /\/uploads\/+picture\//.test(src)) ||
			(domain_nowww === "celebs-news.ru" && string_indexof(src, "/images/") >= 0)) {
			return src.replace(/\/thumb_([^/]*)$/, "/$1");
		}

		if (domain === "cdn.ppomppu.co.kr" ||
			domain_nowww === "popco.net") {
			return src.replace(/(\/zboard\/+data[0-9]*\/+(?:[^/]*\/+)?[0-9]{4}\/+[0-9]{2}\/*[0-9]{2}\/+)(?:thumb|m)_/, "$1");
		}

		if (domain === "media.codeweavers.com") {
			return src.replace(/\/thumb_([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain === "img.tvreport.co.kr" ||
			domain === "img.tvreportcdn.de") {
			return src.replace(/_[0-9]*(\.[^/.]*)$/, "_0$1");
		}

		if (domain === "ojsfile.ohmynews.com") {
			newsrc = src.replace(/\/STD(_IMG_FILE\/+.*?)_STD(\.[^/.]*)(?:[?#].*)?$/, "/PHT$1_PHT$2");
			if (newsrc !== src)
				return newsrc;

			return src
				.replace(/\/CT_T_IMG\/(.*?)\/([^/]*)_[A-Z]+(\.[^/.]*?)(?:\?.*)?$/, "/ORG_IMG_FILE/$1/$2_ORG$3")
				.replace(/\/[A-Z]*_IMG_FILE\/(.*?)\/([^/]*)_[A-Z]*(\.[^/.]*)(?:\?.*)?$/, "/ORG_IMG_FILE/$1/$2_ORG$3");
		}

		if (domain === "cmsimg.mnet.com" ||
			domain === "cmsimg.global.mnet.com") {
			regex = /(\/clipimage\/.*?[^0-9]\/)[0-9]+\/([0-9]+\/[0-9]+\/[0-9]+\.[^/.]*)$/;
			return [
				src.replace(regex, "$1$2"),
				src.replace(regex, "$11024/$2")
			];
		}

		if (domain === "image.cloud.sbs.co.kr") {
			return src.replace(/_[0-9]*(\.[^/.]*)$/, "$1");
		}

		if (domain_nosub === "st-hatena.com" &&
			(domain.match(/cdn-ak-scissors\.[a-z]\.st-hatena\.com/) ||
			 domain === "cdn.image.st-hatena.com")) {
			newsrc = src.replace(/.*?\/image\/(?:scale|square)\/[^/]*\/[^/]*\/(.*)$/, "$1");
			if (newsrc !== src)
				return decodeURIComponent(newsrc);
		}

		if (domain_nosub === "st-hatena.com" && domain.match(/cdn(?:-[a-z]+)\.[a-z]\.st-hatena\.com/)) {
			return src.replace(/_(?:[0-9]+|m)(\.[^/.]*)$/, "$1");
		}

		if (domain === "nimage.newsway.kr") {
			if (src.match(/[?&]simg=[%/]/)) {
				return decodeURIComponent(src.replace(/\/phpwas\/restmb_idxmake\.php.*?simg=([^&]*).*?$/, "$1"));
			}

		}

		if (domain === "imgsrv.piclick.me") {
			newsrc = src.replace(/\/publish\.php\?(?:.*&)?pid=([0-9]+)(?:[?#].*)?$/, "/cimg/$1.jpg");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/\/cimg\/+[0-9]+x[0-9]+x/, "/cimg/");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/\/cimg\/+N_/, "/cimg/");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "slate.com") {
			return src.replace(/(\/[^/.]*\.[^/.]*)\.[^/]*$/, "$1");
		}

		if (domain === "img.cinemablend.com" ||
			domain === "img.minq.com") {
			return src.replace(/(:\/\/[^/]*\/)filter:[^/]*\/(.*?)(?:\?[^/]*)?$/, "$1$2");
		}

		if (((domain_nosub === "abcimg.es" && domain.match(/r[0-9]*\.abcimg\.es/)) ||
			 domain === "resizer.lasprovincias.es" ||
			 domain === "resizer.abc.es" ||
			 domain === "resizer.elcorreo.com") &&
			string_indexof(src, "/resizer.php") >= 0) {
			newsrc = src.replace(/.*\/resizer\.php.*?[?&]imagen=([^&]*).*$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "resizer.elnortedecastilla.es" ||
			domain === "resizer.ideal.es" ||
			domain === "resizer.nortecastilla.es") {

			if (false) {
				return {
					url: src.replace(/\/+resizer\.php.*?[?&](imagen=[^&]*).*/, "/resizer.php?$1"),
					head_wrong_contentlength: true
				};
			} else {
				return src
					.replace(/:\/\/[^/]*\/+resizer\/+resizer\.php.*?[?&]imagen=\/+deliverty\/+[^/]*\/resources\/+([^&]*).*/, "://foto-cache.elnortedecastilla.es/$1")
					.replace(/^[a-z]+:\/\/[^/]*\/+resizer\/+resizer\.php.*?[?&]imagen=(http[^&]*).*/, "$1");
			}
		}

		if (domain === "vz.cnwimg.com") {
			return src.replace(/\/thumb[a-z]*-[0-9]+x[0-9]+\//, "/");
		}

		if (domain_nowww === "coleman-rayner.com" &&
			string_indexof(src, "/watermark/insertwm.php?") >= 0) {
			return {
				can_head: false,
				url: decodeURIComponent(src.replace(/.*\/watermark\/insertwm\.php.*?[?&]src=([^&]*).*$/, "$1"))
			};
		}

		if (domain === "media.guestofaguest.com") {
			return src.replace(/(:\/\/[^/]*\/)[^/]*\/(wp-content|gofg-media)\//, "$1$2/");
		}

		if (domain_nosub === "heartyhosting.com" &&
			domain.match(/i[0-9]*\.heartyhosting\.com/)) {
			return src.replace(/.*?:\/\/[^/]*\//, "http://");
		}

		if (domain === "images.contactmusic.com") {
			newsrc = src.replace(/-cm(\.[^/.]*)$/, "$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "contactmusic.com") {
			var retarray = [];

			newsrc = src.replace(/\/pics\/[a-z]([a-z]?\/)/,"/pics/s$1");
			if (newsrc !== src)
				retarray.push({
					url: newsrc,
					problems: {
						smaller: true
					}
				});

			newsrc = src.replace(/\/pics\/[a-z]([a-z]?\/)/,"/pics/l$1");
			if (newsrc !== src)
				retarray.push({
					url: newsrc,
					problems: {
						watermark: true
					}
				});

			if (retarray.length > 0)
				return retarray;
		}

		if (domain === "d15mj6e6qmt1na.cloudfront.net") {
			return src.replace(/(\/i\/[0-9]*)\/.*/, "$1");
		}

		if ((domain_nosub === "bing.net" && domain.match(/tse[0-9]*\.(?:mm|explicit)\.bing\.net/)) ||
			domain_nosub === "bing.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/th[^/]*[?&]rurl=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeURIComponent(newsrc);

			newsrc = src.replace(/(:\/\/[^/]*)\/th[^/]*[?&]id=([^&]*)&[^/]*$/, "$1/th?id=$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "cdn.4archive.org") {
			return src.replace(/(\/img\/[^/.]{7})m(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "i.4cdn.org" ||
			domain === "i.4pcdn.org") {
			return fillobj_urls(add_extensions(src.replace(/(\/[0-9]*)s(\.[^/.]*)$/, "$1$2")), {
				headers: {
					Referer: ""
				},
				referer_ok: {
					same_domain_nosub: true
				}
			});
		}

		if (domain_nowww === "thebarchive.com" ||
			domain_nosub === "desu-usergeneratedcontent.xyz" ||
			domain_nowww === "archiveofsins.com") {
			return add_extensions(src.replace(/(\/)thumb(\/.*\/[0-9]+)s(\.[^/.]*)$/, "$1image$2$3"));
		}

		if (domain === "arch.b4k.co") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+files\/+([^/]+)\/+thumb\/+[0-9]+\/+[0-9]+\/+([0-9]+s\.[^/.]*)$/, "https://i.4cdn.org/$1/$2");
			if (newsrc !== src)
				return newsrc
		}

		if (domain === "ii.yakuji.moe" ||
			domain_nowww === "iichan.hk" ||
			domain_nosub === "2chan.net" ||
			domain === "aqua.komica.org") {
			newsrc = src.replace(/\/thumb\/([0-9]+)s(\.[^/.]*)$/, "/src/$1$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "file.tinnhac.com" ||
			domain === "file.tintuckpop.net" ||
			domain === "media.bongda.com.vn" ||
			domain_nosub === "404content.com" ||
			domain === "image.vtcns.com" ||
			domain === "media.cungcau.vn" ||
			domain === "image.vtc.vn") {
			return src
				.replace(/\/crop\/+[-0-9]+x[-0-9]+\//, "/")
				.replace(/\/resize\/+[-0-9]+x[-0-9]+\//, "/");
		}

		if (domain_nosub === "zadn.vn" && domain.match(/^photo-resize-/)) {
			newsrc = src.replace(/(:\/\/[^/]*\/+)w[0-9]+h[0-9]+(?:_[a-z]+)?\/+/, "$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "image.mp3.zdn.vn") {
			return src.replace(/\/thumb\/+[0-9]+_[0-9]+\/+/, "/");
		}

		if (domain_nosub === "zadn.vn" ||
			(domain_nosub === "vov.vn" && domain.match(/images?\.(?:[^/.]*\.)?vov\.vn/)) ||
			domain === "ss-images.catscdn.vn" ||
			domain_nosub === "zdn.vn" ||
			domain === "media.stylenews.vn" ||
			domain === "img.vietnamplus.vn" ||
			domain === "cdnimg.vietnamplus.vn" ||
			(domain_nosub === "baonghean.vn" && domain.match(/^image[0-9]*\./)) ||
			domain === "image.baophapluat.vn" ||
			domain === "image.giaoducthoidai.vn" ||
			domain === "image.laodong.com.vn" ||
			domain === "cdn-images.saostar.vn" ||
			(domain_nosub === "tienphong.vn" && domain.match(/image[0-9]*\.tienphong\.vn/)) ||
			(domain_nosub === "infonet.vn" && /^img[0-9]*/.test(domain)) ||
			domain === "img.giaoduc.net.vn" ||
			domain === "media.laodong.vn") {
			newsrc = src
				.replace(/(:\/\/[^/]*)\/+[wht]p?m?[0-9]+x?(?:_[^/]*)?(?:[0-9]+)?\//, "$1/")
				.replace(/(:\/\/[^/]*)\/+[-0-9]+x[-0-9]+\//, "$1/")
				.replace(/(?:\.ashx)?\?.*$/, "");

			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "cdn.tuoitre.vn" ||
			domain_nowww === "dantricdn.com" ||
			domain_nowww === "afamilycdn.com" ||
			domain_nosub === "cafebizcdn.vn" ||
			domain_nosub === "sohacdn.com" ||
			domain_nosub === "kenh14cdn.com" ||
			domain === "cafebiz.cafebizcdn.vn" ||
			domain_nosub === "vcmedia.vn" ||
			domain_nosub === "genkcdn.vn" ||
			domain === "icdn.dantri.com.vn" ||
			domain_nosub === "mediacdn.vn") {
			return src
				.replace(/\/zoom\/[^/]*\//, "/")
				.replace(/-[0-9]+-[0-9]+-[0-9]+-[0-9]+-crop-[0-9]+(\.[^/.]*)$/, "$1")
				.replace(/-crop-[0-9]{13,}(\.[^/.]*)$/, "$1")
				.replace(/(:\/\/[^/]*)\/thumb_[a-z]\/[0-9]+\//, "$1/");
		}

		if (domain_nosub === "24hstatic.com" ||
			domain_nosub === "24h.com.vn" ||
			(domain_nosub === "danviet.vn" && domain.match(/^streaming[0-9]*\./)) ||
			(domain_nosub === "eva.vn" && domain.match(/^image(?:-[a-z]+)?\./)) ||
			domain === "anh.eva.vn") {
			if (src.match(/\/upload\/+[0-9]+-[0-9]{4}\/+images\//)) {
				return src
					.replace(/(\/images\/.*)_[0-9]+_[0-9]+(\.[^/.]*)$/, "$1$2")
					.replace(/-watermark(\.[^/.]*)$/, "$1")
					.replace(/-auto-crop-[0-9]+x[0-9]+(\.[^/.]*)$/, "$1")
					.replace(/(\/images\/[0-9]*-[0-9]*-[0-9]*\/)[^/]*\/([^/]*)$/, "$1$2");
			}
		}

		if ((domain_nosub === "yan.vn" && domain.match(/static[0-9]*\.yan\.vn/)) ||
			(domain_nowww === "myzutv.ro" && string_indexof(src, "/uploads/") >= 0) ||
			(domain_nowww === "rainews.it" && string_indexof(src, "/img/") >= 0) ||
			(domain === "a.jeu.cc" && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "soyuz.ru" && /\/uploads\/+files\//.test(src)) ||
			(domain === "www.mjuznews.com" && string_indexof(src, "/photos/") >= 0)) {
			return src.replace(/\/[0-9]+x[0-9]+_([^/]*)$/, "/$1");
		}

		if (domain_nosub === "autoimg.cn") {
			return src.replace(/\/(?:[0-9]+x[0-9]+_(?:[0-9]+_)?|[0-9]+_)([^/]+)(?:[?#].*)?$/, "/$1");
		}

		if (domain === "image.thanhnien.vn") {
			return {
				url: src
					.replace(/\?.*/, "")
					.replace(/(:\/\/[^/]*)\/+(?:[0-9]+|c[0-9]+x[0-9]+\/+[0-9]+(?:,[0-9]+){3})\/+(uploaded\/)/i, "$1/$2"),
				can_head: false // hangs?
			};
		}

		if (domain === "media-local.phunu365.net" ||
			domain === "media-local.mywow.vn") {
			return src.replace(/.*?\/api[0-9]+x[0-9]+\/res\/ext\/[0-9]+x[0-9]+\/[^/]*\//, "http://");
		}

		if ((domain_nosub === "nguoiduatin.vn" && domain.match(/.*media[0-9]*\.nguoiduatin\.vn/)) ||
			domain === "media.doisongphapluat.com" ||
			domain === "media.vietq.vn" ||
			domain_nowww === "baouc.com" ||
			domain === "media.hay.tv" ||
			domain === "media.hotbirthdays.com") {
			return src.replace(/(:\/\/[^/]*)\/[^/]*x[0-9]+x(?:[0-9]+)?\//, "$1/");
		}

		if (domain === "www.wowkorea.live" ||
			(domain_nowww === "fotofap.net" && string_indexof(src, "/img/") >= 0) ||
			domain === "images.vfl.ru") {
			return src.replace(/(\/[0-9]*)_[a-z](\.[^/.]*)$/, "$1$2");
		}

		if (domain === "content.erooups.com") {
			return src.replace(/(\/img[0-9]*\/+[0-9]{8}\/+[0-9]+\/+[^/]*)_m(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "seoul.co.kr" && domain.match(/img[^.]*\.seoul\.co\.kr/)) {
			return src.replace(/_[A-Z](?:[0-9]){0,2}(\.[^/.]*)$/, "$1");
		}

		if (domain === "img.yonhapnews.co.kr" ||
			domain === "vmg.yonhapnews.co.kr" ||
			(domain_nosub === "yna.co.kr" && domain.match(/^img[0-9]*\./))) {
			var extra = {};

			match = src.match(/\/(PYH[0-9]{17})[0-9]{2}[_.][^/]*(?:[?#].*)?$/);
			if (match) {
				extra.page = "https://www.yna.co.kr/view/" + match[1];
			}

			return {
				url: src
					.replace(/(\/PYH[^/_.]*)_[^/.]*(\.[^/.]*)$/, "$1_P4$2")
					.replace(/(\/A(?:KR|JP|CK|EN)[^/_.]*_[0-9]+_i)_[^/.]*(\.[^/.]*)$/, "$1$2"),
				can_head: false, // 400
				extra: extra
			};
		}

		if (domain_nosub === "yonhapnews.co.kr" && domain.match(/big[0-9]*\.yonhapnews\.co\.kr/)) {
			return src.replace(/.*:\/\/[^/]*\/gate\/[^/]*\//, "http://");
		}

		if (domain_nosub === "bunjang.net") {
			return {
				url: src.replace(/_[wh][0-9]*(\.[^/.]*)$/, "$1"),
				can_head: false // 404
			};
		}

		if (domain === "betanews.heraldcorp.com" ||
			domain_nowww === "betanews.net") {
			return src
				.replace(/(\/imagedb\/(:?[^/]*\/)?)(?:first|thumb)\//, "$1orig/");
		}

		if (domain === "img.smlounge.co.kr") {
			return src.replace(/\/thumb\/([^/.]*)-sample[^/.-]*(\.[^/.]*)$/, "/$1$2");
		}


		if (domain === "img.etoday.co.kr") {
			return src.replace(/(\/pto_db\/+[0-9]{4}\/+[0-9]{2}\/+)[0-9]+\/+([0-9]+_[0-9]+[^/]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "image.tving.com" &&
			string_indexof(src, "tving.com/resize.php") >= 0) {
			return src.replace(/.*:\/\/[^/]*\/resize\.php.*?[?&]u=([^&]*).*/, "$1");
		}

		if (domain === "cdn.pastemagazine.com") {
			return src
				.replace(/(\/[^/]*\/)assets_[^/]*\/[0-9]*\/[0-9]*\/([^-]*)-.*(\.[^/.]*)$/, "$1$2$3")
				.replace(/(\/www\/+system\/+images\/+.*\/)[a-z]+\/+([^/]*)(?:[?#].*)?$/, "$1original/$2");
		}

		if (domain_nowww === "beauty-co.jp") {
			return src.replace(/\/news\/+assets_[^/]*\/+[0-9]{4}\/+[0-9]{2}\/+([^-]*)-.*(\.[^/.]*)$/, "/news/img/$1$2");
		}

		if (domain_nowww === "seichimap.jp") {
			newsrc = src.replace(/(\/[^/]*\/)assets_[^/]*\/+[0-9]{4}\/+[0-9]{2}\/+([^-]*)-.*(\.[^/.]*)$/, "$1pht/$2$3");
			if (newsrc !== src) {
				return add_extensions_upper(newsrc);
			}
		}

		if (domain === "www.agencyteo.com") {
			return src
				.replace(/-[0-9]*[wh]*(\.[^/.]*)$/, "$1")
				.replace(/(\/download\/[0-9]*\/)[wh]\/[0-9]*\//, "$1");
		}

		if (domain_nosub === "riotpixels.net" && domain.match(/s[0-9]*\.riotpixels\.net/)) {
			return src.replace(/(\/data\/[a-f0-9]*\/[a-f0-9]*\/[^./]*\.[^/.]*)[./].*$/, "$1");
		}

		if ((domain_nosub === "ignimgs.com" ||
			 domain_nosub === "ign.com") &&
			(domain.match(/^assets[0-9]*\./) ||
			 domain.match(/^[^.]*media\./))) {
			newsrc = src.replace(/[?#].*$/, "");
			if (newsrc !== src)
				return newsrc;

			return src.replace(/_(?:[0-9]+[hw]|medium|grande)(\.[^/.]*)$/, "$1");
		}

		if (domain === "i.neoseeker.com" &&
			src.match(/\/size\/[0-9]+x[0-9]+\//)) {
			return src.replace(/\/size\/[0-9]+x[0-9]+\//, "/size/0x0/");
		}

		if (domain === "i.neoseeker.com" &&
			src.match(/\/p\/[0-9]*\/[0-9]*\//)) {
			return src.replace(/_thumb_([^/]*$)/, "_image_$1");
		}

		if (domain === "resource.supercheats.com") {
			return src.replace(/\/library\/(?:(?:[0-9]*[wh])|thumbs)\//, "/library/");
		}

		if (amazon_container === "intergi-phoenix") {
			return src.replace(/\/images\/thumb_large_([^/]*)$/, "/images/$1");
		}

		if (domain === "www.primagames.com" ||
			domain_nowww === "rockpapershotgun.com" ||
			domain === "images.eurogamer.net" ||
			domain === "assets.rockpapershotgun.com") {
			return src.replace(/(\/[^/]*\.[^/.]*)\/[A-Z0-9]+\/(?:resize|format|crop|quality)?\/.*$/, "$1");
		}

		if ((domain_nosub === "ixquick.com" ||
			 domain_nosub === "startpage.com") &&
			domain.match(/s[0-9]*-[^.]*\./)) {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+cgi-bin\/+serveimage.*?[?&]url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeURIComponent(newsrc);
		}

		if (host_domain_nowww === "startpage.com" && /\/sp\/+search/.test(options.host_url) && options.element) {
			if (options.element.parentElement && options.element.parentElement.tagName === "DIV") {
				if (options.element.parentElement.classList.contains("image-container")) {
					var imgurl = options.element.parentElement.getAttribute("data-img-click-url");
					if (imgurl !== src)
						return imgurl;
				}
			}
		}

		if (domain === "beta.ems.ladbiblegroup.com") {
			return src.replace(/\/s3\/content\/[0-9]+x[0-9]+\//, "/s3/content/");
		}

		if (domain === "mtv-intl.mtvnimages.com") {
			return src.replace(/(\?ep=[^&]*).*/, "$1");
		}

		if (domain === "gaia.adage.com") {
			return src.replace(/\/images\/bin\/image\/[^/]*\//, "/images/bin/image/");
		}

		if (domain === "t-eska.cdn.smcloud.net") {
			return src.replace(/\/[^/]*?n-([^/]*)$/, "/n-$1");
		}

		if (domain === "cdn.wegow.com") {
			return src.replace(/\.[-0-9]*x[-0-9]*(\.[^/.]*)$/, "$1");
		}

		if (domain === "www.ecestaticos.com") {
			return src.replace(/\/clipping\/[0-9]*\//, "/clipping/0/");
		}

		if (domain_nowww === "sonarreykjavik.com" ||
			domain_nowww === "gamer.ru" ||
			domain_nosub === "linode.com" ||
			domain === "img.noritter.com" ||
			domain_nowww === "esportivo.net" ||
			domain_nowww === "pvgroup.ru" ||
			domain_nowww === "leit.is" ||
			domain_nowww === "rovina-project.eu" ||
			domain_nowww === "mmorpg.org.pl" ||
			domain_nowww === "girlspolish.jp" ||
			domain_nowww === "joah-girls.com" ||
			domain === "listas.eleconomista.es" ||
			domain_nowww === "gonintendo.com" ||
			(domain_nowww === "gozzip.id" && /\/BlogBody\/+photos\//.test(src)) ||
			domain_nowww === "greasyfork.org" ||
			domain === "static.themezy.com" ||
			domain_nowww === "hairhapi.com" ||
			domain_nowww === "babyfashion.me" ||
			domain_nowww === "sonar.es") {
			return src
				.replace(/(\/attached_images\/+[0-9]+\/+)[a-z]+\/+/, "$1original/")
				.replace(/(\/(?:system|images)\/+(?:(?:attached|item)_images|App\/+BlogBody|post_pictures|events|posts|items|articles|file_uploads|screenshots|resources|pictures)\/+(?:(?:images|photos|files|assets|pictures|posters|uploads|screenshots|previews|imgs)\/+)?(?:[0-9]{3}\/+){3})[a-z_0-9]+\/+/, "$1original/");
		}

		if (domain === "pgw.udn.com.tw") {
			return src.replace(/.*\/photo\.php.*?[?&]u=([^&]*).*/, "$1");
		}

		if (domain === "uc.udn.com.tw") {
			return {
				url: src,
				can_head: false
			};
		}

		if ((domain_nosub === "hdslb.com" && domain.match(/i[0-9]*\.hdslb\.com/)) ||
			domain === "img.xiaohongshu.com" ||
			(domain_nosub === "xiaoka.tv" && domain.match(/alcdn\.img\.xiaoka\.tv/))) {
			return src
				.replace(/(:\/\/[^/]*\/)[0-9]+_[0-9]+\//, "$1")
				.replace(/(\.[^/.]*)@[_0-9a-z]*(?:\.[^/.]*)?$/, "$1");
		}

		if (domain === "d.ifengimg.com") {

			return src.replace(/.*?\/[a-z]+[0-9]*(?:_[whq][0-9]*)?\//, "http://");
		}

		if (domain === "www.nationalgeographic.com") {
			return src.replace(/\.[^/]*(\.[^/.]*)$/, "$1");
		}

		if (domain === "media.mnn.com" ||
			domain_nowww === "alljapantours.com" ||
			domain === "shows.gqimg.com.cn" ||
			domain === "shows.vogueimg.com.cn" ||
			(domain_nosub === "webcollage.net" && string_indexof(domain, "media.webcollage.net") >= 0) ||
			domain_nowww === "metronews.ca" ||
			domain === "images.meredith.com" ||
			domain === "img-cdn.jg.jugem.jp" ||
			domain_nowww === "anime-thai.net" ||
			domain_nowww === "mymypic.net" ||
			domain_nowww === "styleyen.com" ||
			domain_nowww === "nbstr.org" ||
			(domain_nosub === "forbiddenplanet.com" && domain.match(/^dyn[0-9]*\.media\.forbiddenplanet\.com/)) ||
			domain === "media.treehugger.com" ||
			(domain_nowww === "attitude.co.uk" && string_indexof(src, "/media/images/") >= 0) ||
			domain === "media.allyou.net" ||
			domain === "image.pbs.org" ||
			(domain_nowww === "plasticsurgerystar.com" && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "peoplite.com" && string_indexof(src, "/public/album_photo/") >= 0) ||
			(domain === "cdn.admireme.vip" && string_indexof(src, "/media/") >= 0) ||
			(domain_nosub === "globo.com" && string_indexof(src, "/fotos/") >= 0) ||
			domain === "d26oc3sg82pgk3.cloudfront.net" ||
			(domain_nowww === "memberme.net" && string_indexof(src, "/media/") >= 0) ||
			(domain_nowww === "funasia.net" && string_indexof(src, "/site_media/") >= 0) ||
			domain === "image.photocnc.com" ||
			domain === "d53l9d6fqlxs2.cloudfront.net") {
			newsrc = src.replace(/(\/[^/.]*\.[^/.]*)\.[^/]*$/, "$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "media.contentapi.ea.com") {
			newsrc = src.replace(/(\/[^/.]*\.[^/.]*)\.[^/]*$/, "$1");
			if (newsrc !== src)
				return {
					url: newsrc,
					can_head: false
				};
		}


		if (domain === "img.bleacherreport.net") {
			return src
				.replace(/\/crop_exact_([^/]*)$/, "/$1")
				.replace(/\?.*$/, "?w=999999999999&h=999999999999");
		}

		if (domain === "images.gr-assets.com") {
			return src
				.replace(/(\/(?:authors|users)\/[0-9]*p)[0-9]\//, "$18/")
				.replace(/(\/books\/[0-9]*)[a-z]\//, "$1l/");
		}

		if (domain === "dynamic.indigoimages.ca") {
			return src.replace(/(\?.*)?$/, "?width=999999999");
		}

		if (domain === "cdn.mos.cms.futurecdn.net") {
			return src.replace(/-[0-9]+-[0-9]+(\.[^/.]*(?:\.webp)?)(?:[?#].*)?$/, "$1");
		}

		if (domain === "www.allkpop.com") {
			return src.replace(/\/af\/([0-9]*\/[^/]*)$/, "/af_org/$1");
		}

		if (domain === "cwcontent.asiae.co.kr") {
			return src
				.replace(/^(.*?:\/\/).*\/[^/]*resize\/[0-9]*\/([^/]*)$/, "$1cphoto.asiae.co.kr/listimglink/4/$2");
		}

		if (domain === "cphoto.asiae.co.kr") {
			return src
				.replace(/\/listimglink\/[0-9]*\//, "/listimglink/4/")
				.replace(/\/resizeimglink\/[0-9]*\//, "/listimglink/4/")
				.replace(/\/listimg_link\.php.*?[?&]no=([^&]*).*?$/, "/listimg_link.php?idx=4&no=$1");
		}

		if (domain === "thumbs-prod.si-cdn.com") {
			return src.replace(/.*\/(https?:\/\/)/, "$1");
		}

		if (domain === "assets.atlasobscura.com") {
			return src.replace(/\/article_images\/[0-9]*x\//, "/article_images/");
		}

		if (domain_nowww === "wonderopolis.org" && string_indexof(src, "/_img") >= 0) {
			newsrc = src.replace(/\/_img\?(?:.*&)?(img=[^&]*).*/, "/_img?$1");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+_img\?img=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return urljoin(src, newsrc, true);
		}

		if (domain_nowww === "thehindu.com" ||
			domain_nowww === "gloria.hr" ||
			((domain_nosub === "mirror.co.uk" ||
			  domain_nosub === "birminghammail.co.uk" ||
			  domain_nosub === "dailypost.co.uk" ||
			  domain_nosub === "bristolpost.co.uk" ||
			  domain_nosub === "irishmirror.ie" ||
			  domain_nosub === "coventrytelegraph.net" ||
			  domain_nosub === "dublinlive.ie") &&
			 domain.match(/i[0-9]*(?:-prod)?\./)) ||
			domain_nowww === "mirror.co.uk" ||
			domain === "beta.images.theglobeandmail.com" ||
			domain_nowww === "globalblue.com" ||
			(domain_nosub === "belfasttelegraph.co.uk" && domain.match(/^(?:cdn(-[0-9]+)?|www)\./)) ||
			domain_nowww === "ladylike.gr" ||
			domain_nowww === "24horas.cl" ||
			domain_nowww === "jyllands-posten.dk" ||
			domain_nowww === "svtstatic.se" ||
			domain_nowww === "adressa.no" ||
			domain_nowww === "miamiherald.com" ||
			domain_nowww === "filmweb.no" ||
			domain_nowww === "kansascity.com" ||
			domain_nowww === "sanluisobispo.com" ||
			domain_nowww === "star-telegram.com" ||
			domain_nosub === "jutarnji.hr" ||
			domain_nowww === "oneman.gr" ||
			domain_nowww === "th.thgim.com" ||
			domain_nowww === "thehindubusinessline.com" ||
			domain === "d30fl32nd2baj9.cloudfront.net" ||
			domain_nowww === "elsoldepuebla.com.mx" ||
			domain_nowww === "svensktnaringsliv.se" ||
			src.match(/\/(?:article[0-9]+\.(?:ece|svt)|[0-9a-z]+\/+picture[0-9]{4,})\/+(?:[^/]*\/)?alternates\//i) ||
			src.match(/:\/\/i[0-9]*(?:-prod)?\..*\/article[^/]*\.(?:ece|svt)\//i) ||
			domain_nosub === "independent.ie") {
			newsrc = src.replace(/(?:alternates|autocrop|binary)\/+[^/]*\/+([^/]*)$/i, "BINARY/$1");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/\/representations\/+[^/]*\/+([^/]*)(?:[?#].*)?$/i, "/BINARY/$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "ekstrabladet.dk") {
			return src.replace(/\/IMAGE_ALTERNATES\/+[^/]*\/+/, "/IMAGE_BINARY/original/");
		}

		if (domain === "images.fandango.com" ||
			domain_nowww === "statf.com") {
			newsrc = src.replace(/\/ImageRenderer\/.*?\/images\//, "/images/");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "images.fandango.com") {
			return src.replace(/_[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (amazon_container === "assets.forward.com") {
			return src.replace(/.*:\/\/[^/]*\//, "http://");
		}

		if (domain === "assets.forward.com" ||
			domain === "assets.forwardcdn.com") {
			return src.replace(/\/images\/cropped\//, "/images/");
		}

		if (domain_nowww === "thejewelleryeditor.com") {
			return src.replace(/\/images_thumbnails\/+[^/]*_thumbnails\/+([^/]*\/+[0-9]*\/+[^/.]*\.[^_/.]*)__[^/]*(?:[?#].*)?$/, "/images/$1");
		}

		if (domain_nowww === "sass.com.ua" ||
			domain === "bento.cdn.pbs.org") {
			return src.replace(/\/(?:filer_)?public_thumbnails\/+(.*\/[^/]*?\.[a-z]+)__[^/]*(?:[?#].*)?$/, "/$1");
		}

		if (domain === "files.sharenator.com") {
			return src.replace(/(:\/\/[^/]*\/)[^/.]+-s[0-9]+x[0-9]+-([0-9]{4,})(?:-[0-9]*)?(\.[^/.]*)$/, "$1$2$3");
		}

		if (domain === "cdn.jolie.de" ||
			domain === "cdn.maedchen.de") {
			return src.replace(/\/image[0-9]*[wh]\//, "/original/");
		}

		if (domain === "img.mp.itc.cn" ||
			(domain_nowww === "wallpapercraze.com" && string_indexof(src, "/images/wallpapers/") >= 0) ||
			domain === "img.mp.sohu.com") {
			return src.replace(/_th(\.[^/.]*)$/, "$1");
		}

		if (domain_nosub === "qpic.cn") {
			if (src.match(/\/mblogpic\//))
				return {
					url: src.replace(/\/[0-9]*(?:\.[^/.]*)?(?:\?.*)?$/, "/2000"),
					headers: {
						Referer: "" // if not, it redirects to t100.qpic.cn, which doesn't work
					}
				};

			return src.replace(/\/[0-9]*(?:\.[^/.]*)?(?:\?.*)?$/, "/0");
		}

		if ((domain_nosub === "49qmz.com" ||
			 domain_nosub === "ii77.com" ||
			 domain_nosub === "vhaor.com")
			&& domain.match(/^img[0-9]*\./)) {
			return src.replace(/:\/\/[^/]*\/mmbiz_jpg\//, "://mmbiz.qpic.cn/mmbiz_jpg/");
		}

		if (domain_nowww === "vod.lu" && /\/media\/+cache\//.test(src)) {

			return src.replace(/\/media\/+cache\/+(?:resolve\/+)?[0-9]+x[0-9]+\/+/, "/media/");
		}

		if (domain_nowww === "vogue.ua") {
			return src
				.replace(/\/media\/+cache\/+resolve\//, "/cache/")
				.replace(/\/cache\/+[^/]+\/+(uploads?)\//, "/$1/");
		}

		if (domain === "bt.bmcdn.dk") {
			return src.replace(/\/image_[0-9]+(?:x[0-9]+)?\/+image\//, "/image/image/");
		}

		if (domain_nosub === "blogspot.es") {
			return src.replace(/(?:\/media)?\/+cache\/+(?:resolve\/+)?media\/+files\//, "/files/");
		}

		if (domain === "csn.naekranie.pl") {
			return src.replace(/\/media\/+cache\/+(?:resolve\/+)?[^/]*\/+([0-9]{4}\/+[0-9]{2}\/+)/,
							   "/wp-content/uploads/$1");
		}

		if (domain === "imagesvc.timeincuk.net" ||
			domain === "imagesvc.meredithcorp.io") {
			return decodeuri_ifneeded(src.replace(/^[a-z]+:\/\/[^/]*\/v3\/+[a-z]+\/+image.*?[?&]url=([^&]*).*/, "$1"));
		}

		if (domain === "img.timeinc.net") {
			return src.replace(/\/[0-9]+_([^/_]+_[0-9]+(?:_[^/]*)?\.[^/.]*)$/, "/$1");
		}

		if (domain === "i.ksd-i.com") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/+s\/+[0-9]+_[0-9]+_[0-9a-f]+\/+/, "http://");
		}

		if (domain === "a.ksd-i.com") {
			return src.replace(/.*:\/\/[^/]*\/s\/[^/]*\//, "http://");
		}

		if (domain === "static.koreastardaily.com") {
			return src.replace(/.*:\/\/[^/]*\/([0-9]+-[0-9]+-[0-9]+\/[0-9]+-[0-9]+\.[^/.]*)$/, "https://a.ksd-i.com/a/$1");
		}

		if (domain === "pic.pimg.tw") {
			return src
				.replace(/\/[a-z]+_([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/, "/$1")
				.replace(/_w?[a-z](\.[^/]*)$/, "$1");
		}

		if (domain === "imageproxy.pimg.tw") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+(?:zoomcrop|resize)\?(?:.*?&)?url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "helloidol.com" &&
			string_indexof(src, "/script/get_pic.php") >= 0) {
			return src.replace(/.*\/script\/get_pic\.php.*?[?&]src=([^&]*).*?$/, "$1");
		}

		if (domain === "yams.akamaized.net" &&
			string_indexof(src, "/Assets/") >= 0) {
			return src.replace(/\/(?:[^/._]*_)?([^/_]*)$/, "/l_$1");
		}

		if (domain_nosub === "pixpo.net" && domain.match(/img[0-9]\.pixpo\.net/)) {
			return src.replace(/_t[0-9]+x[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain === "www.mirrormedia.mg" &&
			string_indexof(src, "/assets/images/") >= 0) {
			return src.replace(/-desktop(\.[^/.]*)$/, "$1");
		}

		if (domain_nosub === "kknews.cc" && domain.match(/i[0-9]*\.kknews\.cc/)) {
			return src.replace(/_[a-z]*(\.[^/.]*)$/, "$1");
		}

		if (domain === "resource.holyshare.com.tw") {
			return src.replace(/\/article\/[0-9]*x[0-9]*\//, "/article/");
		}

		if (domain_nowww === "kyeongin.com") {
			var extra = {};
			match = src.match(/^[a-z]+:\/\/[^/]+\/+mnt\/+[^/]+\/+[0-9]{6}\/+([0-9]{17})(?:_[0-9]+)?\.[^/.]+(?:[?#].*)?$/);
			if (match) {
				extra.page = "http://www.kyeongin.com/main/view.php?key=" + match[1];
			}

			regex = /\/mnt\/+(?:thum|file(?:_[a-z]+)?)\/+/;
			if (regex.test(src)) {
				return [
					{
						url: src.replace(regex, "/mnt/org/"),
						extra: deepcopy(extra)
					},
					{
						url: src.replace(regex, "/mnt/file/"),
						extra: deepcopy(extra)
					}
				];
			} else {
				return {
					url: src,
					extra: extra
				};
			}
		}

		if (domain_nowww === "wallpaperup.com") {
			return src.replace(/-[0-9]+(\.[^/.]*)$/, "$1");
		}

		if ((domain_nosub === "wallhere.com" ||
			 domain_nosub === "pxhere.com") &&
			domain.match(/^[a-z]*\./)) {
			return src
				.replace(/[a-z]*\.wallhere\.com/, "get.wallhere.com")
				.replace(/[a-z]*\.pxhere\.com/, "get.pxhere.com")
				.replace(/\/(?:photos|images)\/[0-9a-f]*\/[0-9a-f]*\/([^/.]*\.[^/.!]*).*?$/, "/photo/$1")
				.replace(/\/[0-9]+x[0-9]+-px-([^/]*)$/, "/$1")
				.replace(/-[0-9]+x[0-9]+-px-([0-9]+\.[^/.]*)$/, "-$1");
		}

		if (domain === "img.grouponcdn.com") {
			return src.replace(/\/v[0-9]+\/[^/]*$/, "");
		}

		if ((domain_nosub === "goodfon.com" ||
			 domain_nosub === "goodfon.ru" ||
			domain_nosub === "badfon.ru") &&
			domain.match(/^img[0-9]*\./)) {
			return src.replace(/(:\/\/[^/]*\/)[^/]*\/[^/]*\//, "$1wallpaper/original/");
		}

		if (domain_nosub === "greatfon.com" && domain.match(/^cdn[0-9]*\./)) {
			return src.replace(/(\/uploads\/+picture\/.*\/)thumbs_([^/]*)(?:[?#].*)?$/, "$1$2");
		}

		if ((domain_nosub === "yimg.jp" && string_indexof(domain, ".c.yimg.jp") >= 0) &&
			src.match(/:\/\/[^/]*\/im_/)) {
			return src.replace(/(:\/\/[^/]*\/)im_[^/]*\//, "$1");
		}

		if ((domain_nosub === "yimg.jp" && string_indexof(domain, ".c.yimg.jp") >= 0) &&
			src.match(/:\/\/[^/]*\/sim\?/)) {
			return src.replace(/.*:\/\/[^/]*\/sim.*?[?&]furl=([^&]*).*/, "http://$1");
		}

		if ((domain_nosub === "yimg.jp" && string_indexof(domain, ".c.yimg.jp") >= 0) &&
			src.match(/\/Images\/(?:[a-z]_)?[0-9]+(\.[^/]*)$/)) {
			return src.replace(/\/Images\/(?:[a-z]_)?([0-9]+\.[^/.]*)$/, "/Images/f_$1");
		}

		if (domain_nosub === "yimg.jp" && string_indexof(domain, ".c.yimg.jp") >= 0 &&
			src.match(/:\/\/[^/]*\/yjimage\?/)) {
			return src.replace(/^([a-z]+:\/\/[^/]*\/yjimage).*$/, "$1") + "?q=" + url.searchParams.get("q") + "&sig=" + url.searchParams.get("sig");
		}

		if (domain === "av.watch.impress.co.jp") {
			return src.replace(/(\/[0-9]+)_s(\.[^/.]*)$/, "$1_o$2");
		}

		if (domain === "internet.watch.impress.co.jp") {
			return src.replace(/(\/[0-9]+)_s(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "media.image.infoseek.co.jp") {
			return src.replace(/-[a-z]*(\.[^/.]*)$/, "$1");
		}

		if (domain === "bundles.sfd.pl") {
			return src.replace(/(\/SFD\/+[0-9]{4}\/+[0-9]+\/+[0-9]+\/+[0-9a-f]{10,})-small(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "s.eximg.jp") {
			return src.replace(/(_[0-9]+)_s(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "imgc.eximg.jp") {
			newsrc = src.replace(/.*?\/i=([^,&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
			newsrc = src.replace(/.*?\/cv\/+(?:resize|trim)\?i=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "image.itmedia.co.jp" &&
			!src.match(/\/l_[^/]*$/) &&
			!src.match(/\/[0-9]+_[^/]*$/)) {
			return src.replace(/\/([^/]*)$/, "/l_$1");
		}

		if (domain_nosub === "bigcommerce.com" && domain.match(/cdn[0-9]*\.bigcommerce\.com/)) {
			newsrc = src.replace(/\/images\/+stencil\/+[0-9]+x[0-9]+\//, "/images/stencil/original/");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/\/products\/+([0-9]+)\/+images\/+([0-9]+)\/+(.*\.[0-9]{8,})(?:\.[0-9]+\.[0-9]+)?(\.[^/.]*)(?:[?#].*)?$/, "/images/stencil/original/products/$1/$2/$3$4");
			if (newsrc !== src)
				return newsrc;

			return {
				url: src,
				head_wrong_contentlength: true
			};
		}

		if (domain_nosub === "behance.net") {
			return [
				{
					url: src.replace(/(\/project(?:_modules|s)\/+)[^/]*\//, "$1source/"),
					is_original: true
				},
				src.replace(/(\/project(?:_modules|s)\/+)[^/]*\//, "$1fs/")
			];
		}

		if (domain === "www.worldatlas.com") {
			return src.replace(/(:\/\/[^/]*\/)r\/[^/]*\/(upload\/)/, "$1$2");
		}

		if (domain_nosub === "thrillist.com" && domain.match(/assets[0-9]*\.thrillist\.com/)) {
			return src.replace(/\/size\/[^/]*$/, "");
		}

		if (domain_nowww === "vacationidea.com" &&
			string_indexof(src, "/pix/") >= 0) {
			return src.replace(/_mobi(\.[^/.]*)$/, "$1");
		}

		if (domain === "qph.fs.quoracdn.net") {
			return src.replace(/-[a-z]$/, "");
		}


		if (domain_nosub === "haibao.cn" && domain.match(/c[0-9]*\.haibao\.cn/)) {
			newsrc = src.replace(/:\/\/[^/]*\/(.*)\/+imagecut\/+[0-9]+_[0-9]+\/+/, "://c3.haibao.cn/$1/");
			if (newsrc !== src) {
				return {
					url: newsrc,
					can_head: false
				};
			}
		}

		if (domain.match(/c[0-9]*\.haibao\.cn/)) {
			newsrc = src.replace(/:\/\/[^/]*\/img\/+[0-9]+(?:_[0-9]+){3}\//, "://c4.haibao.cn/img/0_0_0_0/");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nosub === "hbimg.cn" && domain.match(/cdn[0-9]*\.hbimg\.cn/)) {
			return {
				url: src.replace(/\/(?:thumbs|tuku|snsthumbs)\/[0-9]+(?:_[0-9]+)\//, "/wm/"),
				problems: {
					watermark: true
				}
			};
		}

		if (domain_nowww === "wallpaperset.com") {
			return src.replace(/(:\/\/[^/]*\/w\/)[^/]*\//, "$1full/");
		}

		if (domain_nosub === "wallpapermania.eu") {
			newsrc = src
				.replace("://static.wallpapermania.eu/", "://www.wallpapermania.eu/")
				.replace(/\/images\/[a-z]?thumbs\//, "/images/data/")
				.replace(/\/download\/([^/]*)\/([0-9]*)\/([^/.]*)_[0-9]+x[0-9]+(\.[^/.]*)$/, "/images/data/$1/$2_$3$4");
			if (newsrc !== src) {
				var referer = newsrc.replace(/.*\/[0-9]+_([^/.]*)(?:_[0-9]+x[0-9]+)?\.[^/.]*$/,
											 "http://www.wallpapermania.eu/wallpaper/$1");
				return {
					url: newsrc,
					headers: {
						"Referer": referer
					}
				};
			}
		}

		if (domain === "img-aws.ehowcdn.com" ||
			domain === "img.aws.ehowcdn.com" ||
			domain === "img.aws.livestrongcdn.com" ||
			domain === "img-aws.livestrongcdn.com") {
			/*newsrc = src.replace(/.*?:\/\/[^/]*\/[0-9]+x[0-9]+p\//, "http://")
			if (newsrc !== src) {
				return newsrc;
			}*/
			newsrc = src
				.replace(/(:\/\/[^/]*\/)[^/]*\//, "$1default/");
			if (newsrc !== src) {
				return newsrc;
			}

			newsrc = src.replace(/.*?:\/\/[^/]*\/.*?\/(photos\.demandstudios\.com\/)/, "http://$1");
			if (newsrc !== src) {
				return newsrc;
			}

			newsrc = src.replace(/.*?:\/\/[^/]*\/[^/]*\/ehow-([^-/.]*)-blog-([^/.]*)\//, "http://$1-ehow-com.blog.ehow.com/");
			if (newsrc !== src) {
				return newsrc;
			}

			newsrc = src.replace(/.*?:\/\/[^/]*\/.*?\/www_ehow_com\/([^/.]*\.[^/]*)\//, "http://$1/");
			if (newsrc !== src) {
				return newsrc;
			}

			newsrc = src.replace(/.*?:\/\/[^/]*\/[^/]*\/s3\.amazonaws\.com\//, "https://s3.amazonaws.com/");
			if (newsrc !== src) {
				return newsrc;
			}
		}

		if (amazon_container === "cme_public_images") {
			newsrc = src.replace(/.*?:\/\/[^/]*\/.*?\/(photos\.demandstudios\.com\/)/, "http://$1");
			if (newsrc !== src) {
				return newsrc;
			}
		}

		if (domain === "imageproxy.themaven.net") {
			return decodeURIComponent(src.replace(/^.*?:\/\/[^/]*\//, "").replace(/\?.*/, ""));
		}

		if ((domain_nosub === "demandstudios.com" && domain.match(/photos[0-9]*\.demandstudios\.com/)) &&
			string_indexof(src, "/dm-resize/") >= 0) {
			return decodeURIComponent(src.replace(/.*?\/dm-resize\/([^/?]*).*/, "http://$1"));
		}

		if (domain_nosub === "demandstudios.com" && domain.match(/photos[0-9]*\.demandstudios\.com/)) {
			return src.replace(/(\/[0-9]+)_XS(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nosub === "abcnews.com") {
			return src.replace(/_[0-9]+x[0-9]+[a-z]?_[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "nationmultimedia.com") {
			return src.replace(/-[^/.]*(\.[^/.]*)$/, "$1");
		}

		if (domain_nosub === "indiatimes.com" ||
			domain === "img.etimg.com" ||
			domain === "etimg.etb2bimg.com" ||
			domain === "telugu.samayam.com" ||
			domain === "static.langimg.com" ||
			domain_nosub === "timesofindia.com" ||
			domain_nosub === "vijaykarnataka.com" ||
			domain === "static.toiimg.com") {
			newsrc = src.replace(/(:\/\/[^/]*\/)thumb\//, "$1photo/");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/\/(?:thumb|photo)\/[^/]*msid-([0-9]*)[,/].*$/, "/photo/$1.cms");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/\/(?:thumb|photo)\/(?:[^/]*\/)?([0-9]*)\.[^/.]*$/, "/photo/$1.cms");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "opt.toiimg.com") {
			return src.replace(/\/recuperator\/+img\/+toi\/+m-([0-9]+)(?:,[^/]*)\/+.*/,
							   "/recuperator/img/toi/$1.cms");
		}

		if (domain_nowww === "jawapos.com") {
			return src.replace(/\/thumbs\/+[^/]*\//, "/uploads/");
		}

		if (domain_nowww === "tnnthailand.com") {
			return src.replace(/\/media\/+[^/]+\/+([^/]+)(?:[?#].*)?$/, "/media/$1");
		}

		if ((domain_nosub === "ibtimes.co.in" ||
			 domain_nosub === "ibtimes.sg") &&
			domain.match(/data[0-9]*\.ibtimes\./)) {
			return src
				.replace(/(:\/\/[^/]*\/)cache-img-[0-9]*-[0-9]*(?:-photo)?\//, "$1")
				.replace(/\/[a-z]*(\/[0-9]+\/[^/]*)$/, "/full$1")
				.replace(/\?.*$/, "");
		}

		if (amazon_container === "astro-image-resizer" ||
			domain_nosub === "astro.com.my") {
			return src
				.replace(/:\/\/(?:astro-image-resizer\.([^.]*\.)?amazonaws\.com|static[0-9]*\.astro\.com\.my)\/+/, "://astrokentico.s3.amazonaws.com/")
				.replace(/(:\/\/s3[^/.]*\.amazonaws\.com\/+)astro-image-resizer\//, "://astrokentico.s3.amazonaws.com/")
				.replace(/(:\/\/[^/]*)\/[0-9]*\/+resize\//, "$1/")
				.replace(/\/[0-9]+x[0-9]+_/, "/");
		}

		if ((amazon_container && string_indexof(amazon_container, "nxs-wkrgtv-media") >= 0) ||
			(domain_nosub === "win4000.com" && domain.match(/^pic[0-9]*\.win4000\.com/)) ||
			domain === "media.fox29.com" ||
			domain === "media.foxla.com" ||
			domain === "sharedmedia.grahamdigital.com" ||
			(domain === "pic.sucaibar.com" && src.match(/\/pic\/+[0-9]{6}\/+/)) ||
			domain === "mediaassets.wxyz.com") {
			newsrc = src.replace(/_[0-9]+_[0-9]+(\.[^/.]*)$/, "$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nosub === "win4000.com" && domain.match(/^pic[0-9]*\./)) {
			return src.replace(/\?down$/, ""); // removes forced downloading
		}

		if (domain === "bobcat.grahamdigital.com") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/image\/+upload\/+view.*?[?&]url=([^&]*).*?$/, "$1");
		}

		if ((domain === "image.photohito.k-img.com" ||
			 domain === "photohito.k-img.com") &&
			string_indexof(src, "/uploads/") >= 0) {
			return src.replace(/_[a-z]*(\.[^/]*)/, "_o$1");
		}

		if (domain === "eiga.k-img.com") {
			return src
				.replace(/(\/images\/+[a-z]+\/+(?:[a-z]+\/+)?[0-9]+\/)[0-9]+x(?:[0-9]+)?(\.[^/.]*)$/, "$1original$2")
				.replace(/(\/images\/+[a-z]+\/+(?:[a-z]+\/+)?[0-9]+\/(?:[a-z]+\/)?[0-9a-f]+)\/[0-9]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "akiba-souken.k-img.com" ||
			domain === "image.akiba-souken.k-img.com") {
			return src
				.replace(/\/thumbnail\/+images\/+/, "/assets/images/")
				.replace(/(\/images\/+[^/]+\/+(?:(?:[0-9]{3}\/+){2}|[0-9a-f]{2}\/+))(?:t[0-9]+_)?([^/]+?)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "image.yes24.com") {
			return src.replace(/(:\/\/[^/]*\/goods\/[0-9]+)(?:\/S)?$/i, "$1/L");
		}

		if (domain === "img.danawa.com" ||
			domain_nowww === "wallpoper.com" ||
			domain === "cdn.maximsfinest.com" ||
			domain_nowww === "phileweb.com" ||
			(domain_nowww === "imgpic.org" && string_indexof(src, "/upload/images/") >= 0) ||
			(domain_nosub === "imgdino.com" && domain.match(/img[0-9]*\.imgdino\.com/) && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "foto-pic.net" && /\/images\/+[0-9]{10,}_thumb\./.test(src)) ||
			(domain === "storage.cobak.co" && string_indexof(src, "/uploads/") >= 0) ||
			(domain_nowww === "mycotopia.net" && string_indexof(src, "/uploads/") >= 0) ||
			(domain === "static.becomegorgeous.com" && string_indexof(src, "/img/") >= 0) ||
			domain === "i.imagepow.com" ||
			(domain_nosub === "techadvisor.co.uk" && domain.match(/^cdn[0-9]*\./)) ||
			(domain_nowww === "funtasticecards.com" && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "hockey-live.sk" && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "khaandaniha.ir" && src.match(/\/Upload\/+Public\/+Content\/+Images\/+/)) ||
			(domain_nowww === "imagecurl.com" && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "mediavida.com" && string_indexof(src, "/imagenes/") >= 0) ||
			(domain === "img.twitrer.com" && string_indexof(src, "/upload/") >= 0) ||
			(domain === "img.beevar.com" && string_indexof(src, "/upload/") >= 0) ||
			(domain === "adobe-abid.waphall.com" && string_indexof(src, "/images/") >= 0) ||
			(domain_nosub === "cdnme.se" && string_indexof(src, "/images/") >= 0) ||
			(domain_nosub === "zadn.vn" && src.match(/\/uploaded\//i)) ||
			domain_nowww === "bellazon.com") {
			newsrc = src.replace(/_thumb(\.[^/.]*)(?:[?#].*)?$/, "$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nosub === "pcgames.com.cn" && domain.match(/^img[0-9]*\./)) {
			return src.replace(/_(?:thumb|small|medium)(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "mosaicon.hu" && string_indexof(src, "/wallpapers/") >= 0) {
			return src.replace(/_(?:large)?thumb(\.[^/.]*)$/, "$1");
		}

		if (domain === "item.ssgcdn.com") {
			return src.replace(/(\/item\/[^/]*)_[0-9]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nosub === "coupangcdn.com" && domain.match(/thumbnail[0-9]*\.coupangcdn\.com/)) {
			return src
				.replace(/thumbnail([0-9]*\.coupangcdn\.com)/, "image$1")
				.replace(/\/thumbnails\/remote\/[^/]*\//, "/");
		}

		if (domain === "image.notepet.co.kr") {
			return src.replace(/\/resize\/[^/]*\//, "/");
		}

		if (domain_nowww === "koreamg.com" ||
			domain_nowww === "yufit.co.kr" ||
			domain_nowww === "lovelanc.com" ||
			domain_nowww === "mimibabi.com" ||
			domain === "akamai.poxo.com" ||
			domain_nowww === "getbarrel.com" ||
			domain_nowww === "sexypet.co.kr" ||
			domain_nowww === "withdrama.net" ||
			domain_nowww === "fncstore.com") {
			newsrc = src.replace(/\/web\/+product\/+(?:tiny|small|medium)\//, "/web/product/big/");
			if (newsrc !== src) {
				return [newsrc, newsrc.replace(/0(\.[^/.]*)(?:[?#].*)?$/, "00$1")];
			}
		}

		if (domain_nosub === "mynavi.jp") {
			return src
				.replace(/\/index_images\/[^/]*(?:\/[^/]*)?$/, "/images/001l.jpg")
				.replace(/\/images\/([0-9]+)(\.[^/.]*)$/, "/images/$1l$2");
		}

		if (domain === "cdn.deview.co.jp") {
			return src.replace(/\/imgs\/news_image\.img\.php.*?am_file=([^&])([^&])([^&])([^&]*).*/, "/imgs/news/$1/$2/$3/$1$2$3$4");
		}

		if (domain === "imgcache.dealmoon.com") {
			return src.replace(/.*?:\/\/[^/]*\/(.*?)(\.[^/._]*)_[^/]*?$/, "http://$1$2");
		}

		if (domain === "www.usmall.us" ||
			domain === "www.sofiehouse.co" ||
			domain === "www.thecelebritydresses.com" ||
			domain === "www.celebredcarpetdresses.com" ||
			domain === "www.minimal.co.id" ||
			domain === "www.bridesmaidca.ca" || // doesn't work
			domain === "www.sisley-paris.com" ||
			domain === "d2ovdo5ynwfl3w.cloudfront.net" ||
			domain === "d1cizyvjjqnss7.cloudfront.net" ||
			domain_nowww === "executiveponies.com" ||
			domain_nowww === "trendygowns.com" ||
			domain === "www.lizandliz.com" ||
			(domain_nosub === "cdp.pl" && domain.match(/^cdn-[0-9]+\./)) ||
			src.match(/(?:\/media)?\/catalog\/product\/cache\/(?:[0-9]*\/[^/]*\/)?(?:[0-9]+x(?:[0-9]+)?\/)?[0-9a-f]{32}\//)) {
			/*return src
				.replace(/(\/cache\/[0-9]*\/)small_image\//, "$1/image/")
				.replace(/\/(thumbnail|image)\/[0-9]+x[0-9]+\//, "/$1/");*/
			newsrc = src.replace(/\/+cache\/+(?:[0-9]*\/+[^/]*\/+)?(?:[0-9]*x[0-9]*\/+)?[0-9a-f]{32}\/+((?:.\/+.\/+)|(?:[^/]*\/+))([^/]*)$/, "/$1$2");
			if (newsrc !== src)
				return newsrc;
			/*newsrc = src.replace(/(\/+cache\/+[0-9]+\/+)thumbnail\/+[0-9]+x[0-9]+\/+/, "$1image/");
			if (newsrc !== src)
				return newsrc;*/
		}

		if (domain === "cdn.okdress.co.nz" ||
			(domain_nowww === "promshopau.com" && false)) {
			return src.replace(/(\/media\/catalog\/product\/)cache\/[0-9]*\/[^/]*\/[0-9]+x[0-9]+\//, "$1");
		}

		if (domain === "img.nextmag.com.tw") {
			return src.replace(/\/[0-9]+x(?:[0-9]+)?_([^/]*)$/, "/$1");
		}

		if (domain_nosub === "meitudata.com") {
			return src.replace(/![^/]*$/, "");
		}

		if (domain === "www.shogakukan.co.jp") {
			return src.replace(/\/thumbnail\/books\//, "/thumbnail/snsbooks/");
		}

		if (domain === "images.sysapi.mtg.now.com") {
			return src.replace(/\/[a-z]\/([^/]*)_[a-z](\.[^/.]*)$/, "/o/$1_o$2");
		}

		if (domain_nosub === "lst.fm" && domain.match(/img[0-9]*[^.]*\.lst\.fm/)) {
			return src.replace(/(\/i\/[a-z]\/)(?:avatar)?[0-9]+s\//, "$1");
		}

		if (domain === "www.hdwallpapers.in" ||
			domain_nowww === "bhmpics.com" ||
			domain_nowww === "freshwallpapers.in") {
			return src
				.replace(/\/(?:download|thumbs)\//, "/walls/")
				.replace(/-[^-_/.]*(\.[^/.]*)$/, "-wide$1");
		}

		if (domain_nowww === "superwallpapers.in") {
			return src.replace(/(\/hdwallpapers\/+[^/]+)_[0-9]+_[0-9]+_[0-9]+_[0-9a-f]{2}(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}


		if (domain === "cdn.wallpaper.com") {
			regex = /\/main\/styles\/[^/]*\/[^/]*\/(.*\/)?(?:l-)?([^/]*)/;
			return [src.replace(regex, "/main/$2"), src.replace(regex, "/main/$1$2")];
		}

		if (domain === "cdn.wallpaper.com") {
			return src.replace(/\/main\/styles\/[^/]*\/[^/]*\//, "/");
		}

		if (domain === "static.warthunder.com") {
			return src.replace(/\/_thumbs\/[0-9]+x(?:[0-9]+)?\//, "/");
		}

		if (domain === "cdn.wallpaperdirect.com") {
			return src.replace(/(\/[0-9]*)_[^/.]*(\.[^/.]*)$/, "$1orig$2");
		}

		if (domain_nosub === "bamcontent.com") {
			return src.replace(/(\/images\/photos\/[0-9]*\/)[0-9]+x[0-9]+\/[^/.]*(\.[^/.]*)$/, "$1raw$2");
		}

		if (domain_nosub === "mail.ru" &&
			domain.match(/filed.*\.mail\.ru$/) &&
			src.match(/:\/\/[^/]*\/pic/)) {
			return decodeURIComponent(src.replace(/.*\/pic.*?[?&]url=([^&]*).*?$/, "$1"));
		}

		if (domain_nosub === "mail.ru" && /^content-[0-9]+\.foto\.my\./.test(domain)) {
			return {
				url: src.replace(/(\/community\/+[^/]+\/+_groupsphoto\/+)i(-[0-9]+\.[^/.?#]+)(?:[?#].*)?$/, "$1b$2"),
				can_head: false // 400
			};
		}

		if (domain_nosub === "mail.ru" &&
			domain.match(/^avt.*\.foto\.mail\.ru/)) {
			return src.replace(/\/_avatar[0-9]*(?:[?#].*)?$/, "/_avatarbig");
		}

		if (domain === "games.mail.ru") {
			return src.replace(/\/pre_(?:[0-9]+x[0-9]+|[a-z]+)_resize\/+(pic|hotbox)\/+/, "/$1/");
		}

		if (domain === "mg.soupingguo.com") {
			return src.replace(/(\/attchment[^/]*\/[^/]*Img\/)[0-9]+x[0-9]+\//, "$10x0/");
		}

		if (domain === "img.yaplog.jp") {
			return src.replace(/(\/img\/[^/]*\/)mo\//, "$1pc/");
		}

		if (domain === "www.hochi.co.jp") {
			return src.replace(/-[A-Z](\.[^/.]*)$/, "-L$1");
		}

		if (domain_nosub === "wikispaces.com") {
			return src.replace(/(\/view\/[^/]*\.[^/]*)\/.*?$/, "$1");
		}


		if (domain_nowww === "sponichi.co.jp") {
			return {
				url: src.replace(/_thum(\.[^/.]*)$/, "_view$1"),
				can_head: false
			};
		}

		if (domain === "thumbnail.image.rakuten.co.jp") {
			return src.replace(/.*?:\/\/[^/]*\/@[^/]*\/([^?]*).*?$/, "http://shop.r10s.jp/$1");
		}

		if (domain === "tshop.r10s.com") {
			return src.replace(/:\/\/[^/]*\/(.*?)(?:[?#].*)?$/, "://shop.r10s.com/$1");
		}

		if (domain_nowww === "billboard-japan.com" ||
			domain === "fotos.jornaldacidadeonline.com.br") {
			return src.replace(/\/(?:[0-9]+)?x(?:[0-9]+)?_([^/]*)$/, "/$1");
		}

		if (domain_nosub === "tsite.jp" &&
			string_indexof(domain, "top.tsite.jp") >= 0 &&
			string_indexof(src, "/contents_image/") >= 0) {
			return src.replace(/(\/[0-9]+)_[0-9]+(?:_[^/.]+)?(\.[^/.]*)$/, "$1_0$2");
		}

		if (domain === "www.sanspo.com") {
			return src.replace(/(\/images\/[0-9]*\/[^/]*-)[a-z]([0-9]+\.[^/.]*)$/, "$1p$2");
		}

		if (domain === "www.sonymusicshop.jp") {
			return src.replace(/__[0-9]+_[0-9]+_[0-9]+_([a-z]+\.[a-z]*)(?:\?.*)?$/, "_$1");
		}

		if (domain_nowww === "prtimes.jp") {
			if (/\/api\/file\.php\?/.test(src)) {
				var c = url.searchParams.get("c");
				var r = url.searchParams.get("r");
				var f = url.searchParams.get("f");
				return "https://prtimes.jp/i/" + c + "/" + r + "/original/" + f;
			}

			return src
				.replace(/\/thumb\/+(?:[0-9]+x[0-9]+\/+)?(d[0-9]+-)/, "/original/$1")
				.replace(/\/resize\/+([^/]*)$/, "/original/$1");
		}

		if (domain_nosub === "vietbao.vn") {
			return src
				.replace(/:\/\/img\.vietbao\.vn\/images\/[0-9]+\//, "://a9.vietbao.vn/images/");
		}

		if (domain === "media.tinnong.net.vn") {
			return src.replace(/\/Images\/[^/]*\//, "/Images/Original/");
		}

		if (domain === "images.kienthuc.net.vn" ||
			domain === "images.khoeplus24h.vn") {
			return src.replace(/\/zoom[a-z]\/[0-9]*\//, "/");
		}

		if (domain === "rez.cdn.kul.vn" ||
			domain_nowww === "glamour.pl" ||
			domain_nowww === "al3arabi.com" ||
			domain_nowww === "starwars-holocron.net" ||
			domain_nowww === "elle.pl" ||
			domain_nowww === "gala.pl" ||
			domain === "i.moveek.com" ||
			domain_nowww === "mainstyles.ru" ||
			domain_nowww === "longtake.it" ||
			src.match(/^[a-z]+:\/\/[^/]*\/+media\/+cache\/+[^/]*\/+uploads\/+media\//)) {
			return src
				.replace(/\/media\/+cache\/+.*\/+uploads\/+/, "/uploads/")
				.replace(/\/media\/+cache\/+[^/]*\//, "/");
		}

		if (domain_nowww === "themebeta.com") {
			return src.replace(/\/media\/+cache\/+[0-9]+x[0-9]+\/+files\//, "/files/");
		}

		if (domain === "static.kstyle.com") {
			return src.replace(/\/r\.[0-9]+x[0-9]+$/, "");
		}

		if (amazon_container === "lifesite-cache" ||
			domain_nowww === "golocalprov.com" ||
			(domain === "d1lfxha3ugu3d4.cloudfront.net" && /\/assets\/+system-images\/+remote\/+http/.test(src)) ||
			(domain_nosub === "thelineofbestfit.com" && domain.match(/^cdn[0-9]*\.thelineofbestfit\.com/))) {
			return src
				.replace(/.*\/(?:system-)?images\/+remote\/+(https?)_(.*?)(?:(?:_[0-9]+)?_[0-9]+_[0-9]+(?:_[a-z]_[a-z][0-9])?)?(\.[^/.]*)$/, "$1://$2$3");
		}

		if (domain === "d1lfxha3ugu3d4.cloudfront.net") {
			return src.replace(/(\/exhibitions\/+images\/+[^/]+)_[0-9]+w(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "fap.to" ||
			(domain_nosub === "imagefap.com" && /^(?:www|images)\./.test(domain))) {
			return src
				.replace(/:\/\/x[0-9]*\./, "://x.") // temporary error?
				.replace(/(:\/\/[^/]*\/)[a-z0-9.]+\/images\//, "$1images/")
				.replace(/\/images\/[a-z]*\//, "/images/full/");
		}

		if (domain === "cdn.imagefap.com") {
			return src.replace(/:\/\/cdn\.(.*?)(?:[?#].*)?$/, "://images.$1");
		}

		if (domain === "www.gannett-cdn.com" &&
			string_indexof(src, "/-ip-/") >= 0) {
			return src.replace(/.*?\/-ip-\//, "");
		}

		if (domain === "cdn.mainichi.jp") {
			return src.replace(/\/[0-9]+(\.[^/.]*)$/, "/9$1");
		}

		if (domain === "img.evbuc.com") {
			return decodeURIComponent(src.replace(/.*:\/\/[^/]*\/([^?]*).*/, "$1"));
		}

		if (domain === "img.cdandlp.com") {
			return src.replace("/imgS/", "/imgL/").replace("/imgM/", "/imgL/");
		}

		if (domain === "walter.trakt.tv") {
			return src.replace(/\/thumb\/([^/]*)$/, "/full/$1");
		}

		if (domain === "cdn.apk-cloud.com" ||
			domain_nowww === "apk-dl.com") {
			return src
				.replace(/(?:=[a-z][0-9]*)?(\.[^/.]*)$/, "=h0$1");
		}

		if (domain_nosub === "polyvoreimg.com") {
			var cginame = src.replace(/.*\/cgi\/([^/]*)\/.*/, "$1");
			var paramsbase = src.replace(/.*\/cgi\/[^/]*/, "");
			var params = paramsbase.replace(/\/([^/]*)\/([^/]*)/g, "$1=$2&");
			params = params
				.replace(/(.*)\.([^/.&]*)&$/, ".out=$2&$1");
			return "https://www.polyvore.com/cgi/" + cginame + "?" + params;
		}

		if (domain === "www.polyvore.com" &&
			string_indexof(src, "/cgi/") >= 0) {
			return src
				.replace(/\/img-set(.*?)&size=[^&]*/, "/img-set$1&size=c99999x99999");
		}

		if (domain === "aliyun-cdn.hypebeast.cn" &&
			string_indexof(src, "/hypebeast.com/") >= 0) {
			return src.replace(/.*:\/\/[^/]*\//, "http://");
		}

		if (domain_nowww === "girlscene.nl" ||
			domain_nowww === "trendalert.nl") {
			return src.replace(/\/thumb\/+[0-9]+x[0-9]+\/+ckfinder\/+/, "/ckfinder/");
		}

		if (domain_nowww === "hdfullfilmizle.com" ||
			domain_nowww === "sinemangoo.org") {
			return src.replace(/\/thumb\/+[0-9]+x[0-9]+\/+uploads\//, "/uploads/");
		}

		if (domain_nowww === "hairstyleinsider.com" ||
			domain === "sites.eveyo.com" ||
			domain_nowww === "24sata.info" ||
			domain_nowww === "alankabout.com" ||
			domain_nowww === "cosmopolitan.rs" ||
			domain_nowww === "elle.rs") {
			return src.replace(/\/(?:thumbnail|files).php.*?file=([^&]*).*/, "/files/$1");
		}

		if (domain_nowww === "zkpm.net" ||
			domain_nowww === "zyzpes.com" ||
			domain_nowww === "yoptel.ru" ||
			domain === "img.viyuedu.com") {
			return src.replace(/.*\/img\.php.*?url=(.*)/, "$1");
		}

		if (domain_nowww === "aidu360.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+img\.php\?(?:.*?&)?u=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "img.xiaohuazu.com") {
			return src.replace(/.*?[?&]url=(.*)/, "$1").replace(/z-z/g, ".").replace(/^/, "http://");
		}

		if (domain === "www.viewsofia.com") {
			return src.replace("/fck_editor_thumb/", "/fck_editor/");
		}

		if (domain_nosub === "condenast.ru" && string_indexof(domain, ".static.media.condenast.ru") >= 0) {
			return src.replace(/\/[a-z][0-9]*(?:x[0-9]+)?$/, "/w99999999");
		}

		if (domain === "www.vogue.co.jp") {
			return src.replace(/-[0-9]+-[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain_nosub === "vogue.mx" ||
			domain_nosub === "glamour.mx" ||
			domain_nosub === "glamour.es" ||
			domain === "cdn.gq.com.mx" ||
			domain === "cdn.revistavanityfair.es" ||
			domain_nosub === "vogue.es") {
			newsrc = src
				.replace(/\/uploads\/+images\/+thumbs(?:(\/+(?:mx|es)\/+(?:vog|glam|gq|vf)\/+)[0-9]*\/+([a-z]))?(\/+(?:[0-9]{4}\/+[0-9]+|[0-9]{4,})\/+[^/]*)_[0-9]+x(?:[0-9]+)?(\.[^/.]*)(?:[?#].*)?$/, "/uploads/images$1$2$3$4");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nosub === "ykt.ru" ||
			domain_nosub === "ykt2.ru") {
			return src.replace(/\/thumb\/([^/]*)$/, "/$1");
		}

		if (domain === "i.guim.co.uk") {
			return src.replace(/:\/\/[^/]*\/img\/([^/]*)\/([^?]*).*?$/, "://$1.guim.co.uk/$2");
		}

		if (domain === "media.guim.co.uk") {
			return src
				.replace(/\/((?:[0-9]*_){3}[0-9]*)\/[^/]*\/([0-9]*\.[^/.]*)$/, "/$1/$2")
				.replace(/(\/[0-9]+_[0-9]+_([0-9]+)_[0-9]+\/)[0-9]+(\.[^/.]*)$/, "$1$2$3");
		}

		if (domain_nowww === "g2d.co.in") {
			return src.replace(/(\/img\/+Photo\/+[0-9]+\/+)(?:sml|med)_([^/]*\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "vogue.gjstatic.nl" ||
			domain === "glamour.gjstatic.nl") {
			return src
				.replace(/\/teaserFileUpload\//, "/fileUpload/") // doesn't work on all
				.replace(/\/fileUpload\/[^/]*\//, "/fileUpload/big/");
		}

		if (domain_nosub === "favim.com" && domain.match(/^s[0-9]*\./)) {
			return src.replace(/(:\/\/[^/]*\/)[^/]*\//, "$1orig/");
		}

		if (domain_nowww === "derpicdn.net") {
			newsrc = src.replace(/\/(?:thumb|large)(\.[^/.]*)$/, "/full$1");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/(\/img\/+)([0-9]{4}\/+[0-9]{1,2}\/+[0-9]{1,2}\/+[0-9]+)\/+[^/.]+(\.[^/.]+)(?:[?#].*)?$/, "$1view/$2$3");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/\/img\/+(?:view\/+)?[0-9]{4}\/+(?:[0-9]{1,2}\/+){2}([0-9]+)[./]/);
			if (match) {
				return {
					url: src,
					extra: {page: "https://derpibooru.org/" + match[1]}
				};
			}
		}

		if (domain_nosub === "iimg.me") {
			newsrc = src.replace(/.*\/[a-z]*\.php.*?[?&]url=([^&]*).*?$/, "$1");
			if (newsrc !== src && string_indexof(newsrc, "http") === 0) {
				return decodeURIComponent(newsrc);
			}
		}

		if (domain === "pix.avaxnews.com" ||
			domain === "pix.avax.news" ||
			domain_nowww === "pixhost.icu" ||
			domain_nowww === "pxhst.co") {
			return src.replace(/_[^/]*(\.[^/.]*)$/, "$1");
		}

		if (domain_nosub === "hurimg.com" ||
			domain === "img.posta.com.tr" ||
			domain === "icdn.pstimg.com" ||
			domain === "img.fanatik.com.tr" ||
			domain === "assets.dogannet.tv" ||
			domain_nosub === "fanatik.com.tr" ||
			((domain_nosub === "pstimaj.com" ||
			  domain_nosub === "milimaj.com" ||
			  domain_nosub === "cnnturk.com") && /^i[0-9]*\./.test(domain))) {

			return src.replace(/\/[0-9]+\/[0-9]+x[0-9]+\/([0-9a-f]+(?:\.[^/.]*)?)([?#].*)?$/, "/100/0x0/$1");
		}

		if (domain === "www.kurtkomaromi.com" ||
			domain_nosub === "typepad.com" ||
			domain === "blogs.elpais.com" ||
			domain === "www.summerofdan.net" ||
			domain_nowww === "thequeenofstyle.com" ||
			domain_nowww === "thequestforit.com" ||
			domain_nosub === "blogs.com" ||
			domain === "latimesblogs.latimes.com" ||
			domain === "www.weeklystorybook.com") {
			return src.replace(/^([a-z]+:\/\/[^/]+\/+\.a\/+[0-9a-f]{10,})-[^/]*(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "bathingnews.com") {
			return {
				can_head: false,
				url: src.replace(/(\/\.a\/[^-/]*)-[^/]*$/, "$1")
			};
		}

		if (domain_nowww === "fmyokohama.jp") {
			return src.replace(/\/thumbnail\/+([^/]*)-[0-9]+wi(\.[^/.]*)(?:[?#].*)?$/, "/$1$2");
		}

		if (domain === "gd.image-qoo10.jp") {
			return src.replace(/\.[^/.]*(\.[^/.]*)$/, "$1");
		}

		if (domain === "img.jakpost.net") {
			return [
				src.replace(/\.[^/.]*(\.[^/.]*)$/, "$1"),
				src.replace(/\._[a-z]+(\.[^/.]*)$/, "._large$1")
			];
		}

		if (domain === "r.ddmcdn.com") {
			return src.replace(/:\/\/[^/]*\/(?:[^/_]*_[^/_]*\/)*/, "://static.ddmcdn.com/");
		}

		if (domain_nosub === "newegg.com" && domain.match(/images[0-9]*\.newegg\.com/)) {
			return src.replace(/(:\/\/[^/]*\/)(?:NeweggImage\/)?(?:ProductImage|productimage)[^/]*\//, "$1ProductImageOriginal/");
		}

		if (domain === "images.costco-static.com" ||
			domain === "images.costcobusinesscentre.ca") {
			return src
				.replace(/([?&])recipeName=[^&]*/, "$1")
				.replace(/&$/, "");
		}

		if (amazon_container === "emerge-tech") {
			return src.replace(/\/[a-z]*\/([0-9]*_[a-z]*_)[a-z]*(\.[^/.]*)$/, "/full/$1full$2");
		}

		if (domain_nosub === "wfcdn.com" && domain.match(/img[0-9]*(?:-[^.]*)?\.wfcdn\.com/)) {
			return src.replace(/(\/im\/[0-9]+\/)[^/]*\//, "$1compr-r85/");
		}

		if (domain_nosub === "hdnux.com") {
			return src.replace(/\/[0-9]+x[0-9]+(\.[^/.]*)$/, "/rawImage$1");
		}

		if (domain_nosub === "busan.com" && domain.match(/^news[0-9]*\./)) {
			return src.replace(/(\/content\/+image\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9]+)_t(\.[^/.]*)$/, "$1_0$2");
		}

		if (domain_nowww === "busan.com") {
			return src.replace(/(\/wcms\/+wcms_data\/+photos\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9]+)_[sm](\.[^/.]*)(?:[?#].*)?$/, "$1_l$2");
		}

		if (amazon_container === "bucket.scribblelive.com" ||
			domain === "images.scribblelive.com") {
			return src.replace(/_[0-9]+(\.[^/.]*)$/, "$1");
		}


		if (domain === "www.dhresource.com" ||
			domain === "image.dhgate.com") {
			return src
				.replace(/(:\/\/[^/]*\/albu_[0-9]+_[0-9]+[-/].*?)[0-9]+x[0-9]+/, "$10x0")
				.replace(/(:\/\/[^/]*\/)[0-9]+x[0-9]+/, "$10x0")
				.replace(/(:\/\/[^/]*\/)[^/]*\/[^/]*\/[0-9]+x[0-9]+\//, "$10x0/");
		}

		if (domain_nowww === "storify.com" && /\/services\/+proxy\//.test(src)) {
			return src.replace(/.*\/services\/proxy\/[0-9]+\/[^/]*\/([a-z]+)\/(.*)$/, "$1://$2");
		}

		if (domain_nowww === "mbcsportsplus.com" ||
			domain === "m.mbcsportsplus.com") {

			var outdir = null;
			if (src.match(/[?&]type=m[^a-z0-9A-Z]/))
				outdir = "/data/home/data/msplMain";
			else if (src.match(/[?&]type=a[^a-z0-9A-Z]/))
				outdir = "/data/board/attach";

			if (outdir)
				return src.replace(/\/images\/+img\.php\?(?:.*&)?src=([^&]*).*/, outdir + "$1");
		}

		if (domain_nosub === "joomag.com" && domain.match(/s[0-9]*cdn\.joomag\.com/)) {
			return src.replace(/(\/mobile\/.*\/[0-9]+_)[0-9]+(-[0-9]*\.[^/]*)$/, "$10$2");
		}

		if (domain === "i.pximg.net" ||
			(domain_nosub === "techorus-cdn.com" && /^tc-pximg[0-9]*\./.test(domain))) {
			newsrc = src
				.replace(/(\/user-profile\/+img\/.*\/[0-9]+_[0-9a-f]{20,})_[0-9]+(\.[^/.]+)(?:[?#].*)?$/, "$1$2")
				.replace(/\/c\/[0-9]+x[0-9]+(?:_[0-9]+)?(?:_[a-z]+[0-9]+)?\//, "/")
				.replace(/\/img-master\//, "/img-original/")
				.replace(/(\/[0-9]+_p[0-9]+)_[^/]*(\.[^/.]*)$/, "$1$2");

			var referer_url = "https://www.pixiv.net/";

			var illust_id = src.replace(/.*\/([0-9]+)_[^/]*$/, "$1");
			if (illust_id === src) {
				illust_id = null;
			} else {
				referer_url = "https://www.pixiv.net/artworks/" + illust_id;
			}

			obj = {
				headers: {
					Referer: referer_url
				},
				extra: {
					page: referer_url
				}
			};

			var urls = [src];
			if (newsrc !== src) {
				urls = add_extensions(newsrc);
			}

			var retobj = fillobj_urls(urls, obj);

			if (illust_id && options && options.force_page && options.cb && options.do_request) {
				options.do_request({
					url: referer_url,
					method: "GET",
					headers: {
						Referer: ""
					},
					onload: function(resp) {
						if (resp.readyState !== 4)
							return;

						if (resp.status !== 200) {
							return options.cb(retobj);
						}

						var match = resp.responseText.match(/<meta\s+name=\"preload-data\"\s+id=\"meta-preload-data\"\s+content='(.*?)'>/);
						if (!match) {
							console_error("Unable to find match", resp);
							return options.cb(retobj);
						}

						try {
							var json = JSON_parse(decode_entities(match[1]));

							obj.extra.caption = json.illust[illust_id].illustTitle;
							return options.cb(fillobj_urls(urls, obj));
						} catch(e) {
							console_error(e);
							return options.cb(retobj);
						}
					}
				});

				return {
					waiting: true
				};
			}

			return retobj;
		}

		if (domain_nosub === "booth.pm" && domain.match(/s[0-9]*\.booth\.pm/)) {
			newsrc = src
				.replace(/\/c\/[a-z]_[0-9]+\//, "/")
				.replace(/_c_[0-9]+x[0-9]+(\.[^/.]*)$/, "$1");
			if (newsrc !== src) {
				if (newsrc.match(/\.jpg$/)) {
					return [newsrc, newsrc.replace(/\.jpg$/, ".JPG")];
				}
				return newsrc;
			}

		}

		if (domain === "booth.pximg.net") {
			newsrc = src.replace(/(:\/\/[^/]*\/)c\/[0-9]+x[0-9]+(?:_[^/]*)?\//, "$1");
			if (newsrc !== src)
				return add_extensions(newsrc);

			return add_extensions(src.replace(/(\/[-0-9a-f]+)_[^/.]*(\.[^/.]*)$/, "$1$2"));
		}

		if (domain === "cache-graphicslib.viator.com") {
			return {
				url: src.replace(/([-_][0-9]+)-[^-_/.]*(\.[^/.]*)$/, "$1-raw$2"),
				head_wrong_contentlength: true
			};
		}

		if (domain_nosub === "4sqi.net") {
			return src.replace(/\/img\/+general\/+[0-9]+x[0-9]+\//, "/img/general/original/");
		}

		if (domain === "static.panoramio.com" ||
			googlestorage_container === "static.panoramio.com") {
			return src.replace(/\/photos\/[^/]*\//, "/photos/original/");
		}

		if (domain_nosub === "myportfolio.com" && domain.match(/.*cdn.*\.myportfolio\.com$/)) {
			return src.replace(/_(?:rw_[0-9]+|rwc_[0-9]+(?:x[0-9]+){4})(\.[^/.?]*)(?:\?.*)?$/, "$1");
		}

		if (domain === "i.dell.com") {
			return src.replace(/(\/(?:[0-9a-f]+-){4}[0-9a-f]+\/[0-9]+\/)LargePNG/, "$1originalpng");
		}

		if (domain === "thumb.zumst.com") {
			return src.replace(/.*:\/\/[^/]*\/[0-9]+[^/]*\//, "");
		}

		if (domain === "imgp.golos.io") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/+[0-9]+x[0-9]+\/+(https?:)/, "$1");
		}

		if (domain === "i.postimg.cc" && options && options.do_request && options.cb) {
			match = src.match(/:\/\/[^/]*\/+([^/]+)\/+/);
			if (match) {
				options.do_request({
					url: "https://postimg.cc/" + match[1],
					method: "GET",
					headers: {
						Referer: null
					},
					onload: function(result) {
						if (result.readyState !== 4)
							return;

						if (result.status !== 200) {
							console_error(result);
							return options.cb(null);
						}

						var match = result.responseText.match(/<meta\s+property=["']og:image["']\s+content=["'](.*?)["']/);
						if (match) {
							return options.cb({
								url: decode_entities(match[1]),
								headers: {
									Referer: result.finalUrl
								},
								extra: {
									page: result.finalUrl
								}
							});
						} else {
							console_error("Unable to find og:image", result);
							return options.cb(null);
						}
					}
				});

				return {
					waiting: true
				};
			}
		}

		if ((domain_nosub === "zumst.com" ||
			 domain_nosub === "zum.com") &&
			domain.match(/^static\./)) {
			return src
				.replace(/\/images\/+thumb\/+/, "/images/")
				.replace(/\/([0-9a-f]+)_[0-9]+x[0-9]+[a-z]?(\.[^/.]*)(?:[?#].*)?$/, "/$1$2");
		}

		if (domain === "file.mk.co.kr" ||
			domain_nosub === "mediapundit.net") {
			newsrc = src.replace(/\.thumb(?:[?#].*)?$/, "");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "kobis.or.kr") {
			return src.replace("/thumb/thn_", "/");
		}

		if (domain_nowww === "breaknews.com") {
			return src.replace(/\/data\/([^/]*)\/mainimages\/[0-9]*\/([0-9]{6})/, "/imgdata/$1/$2/$2");
		}

		if (domain_nowww === "ilyo.co.kr" ||
			domain_nowww === "beatsports.net" ||
			domain_nowww === "ohfun.net") {
			return src.replace(/\/thm[0-9]+_/, "/");
		}

		if (domain_nowww === "joseilbo.com") {
			return src
				.replace(/\.thumbnail(\.[^/.]*)$/, "$1")
				.replace(/\/gisa_img\/([0-9]+_[^/._]+)(\.[^/.]*)$/, "/gisa_img_origin/$1_origin$2");
		}

		if (domain_nosub === "phncdn.com" ||
			domain_nosub === "pornhub.com") {
			newsrc = src.replace(/\/(?:\([a-z]+=[^/)]*\))*([^/]*)$/, "/$1");
			if (newsrc !== src) {
				obj = add_full_extensions(newsrc, ["mp4", "gif", "png", "jpg"], true);
				return obj;
			}

			newsrc = src.replace(/(:\/\/[^/]*\/+)[a-z]=[^/]*\/+/, "$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (host_domain_nosub === "pornhub.com" && options.element && options.do_request && options.cb) {
			var get_pornhub_videodata = function(key, cb) {
				var cache_key = "pornhub_video:" + key;
				api_cache.fetch(cache_key, cb, function(done) {
					options.do_request({
						url: "https://www.pornhub.com/view_video.php?viewkey=" + key,
						method: "GET",
						onload: function(resp) {
							if (resp.readyState !== 4) {
								return;
							}

							if (resp.status !== 200) {
								console_error(resp);
								return done(null, false);
							}

							var match = resp.responseText.match(/<script[^>]*>\s*var (flashvars_[0-9]+) = ({.*});\s*var player_mp4_seek\s*=\s*\S+;\s*(var ra[0-9a-z]{10,}\s*=.*;)/);
							if (!match) {
								console_error("Unable to find flashvars match", resp);
								return done(null, false);
							}

							var vartable = {};

							try {
								vartable[match[1]] = JSON_parse(match[2]);
							} catch (e) {
								console_error(e, resp);
								return done(null, false);
							}

							var main_name = match[1];
							var obfuscated = match[3];
							var tokens = [];
							var current_token = "";
							var in_comment = false;
							var escaping = false;
							var in_string = false;

							var commit_token = function() {
								if (current_token.length > 0) {
									tokens.push(current_token);
									current_token = "";
								}
							};

							for (var i = 0; i < obfuscated.length; i++) {
								var c = obfuscated[i];

								if (in_comment) {
									if (c === "/" && obfuscated[i - 1] === "*") {
										in_comment = false;
										continue;
									} else {
										continue;
									}
								}

								if (escaping) {
									current_token += c;
									escaping = false;
									continue;
								}

								if (in_string) {
									current_token += c;

									if (c === in_string) {
										in_string = false;
										commit_token();
									}

									continue;
								}

								if (/\s/.test(c)) {
									commit_token();
									continue;
								}

								if (/["']/.test(c)) {
									commit_token();

									current_token += c;
									in_string = c;
									continue;
								}

								if (/[\[\]=;+:,{}]/.test(c)) {
									commit_token();

									current_token = c;
									commit_token();
									continue;
								}

								if (c === "\\") {
									escaping = true;
									continue;
								}

								if (c === "/" && obfuscated[i + 1] === "*") {
									in_comment = true;
									i++;
									continue;
								}

								current_token += c;
							}

							var eval_varvalue = function(varvalue) {
								var newvalues = [];

								for (var i = 0; i < varvalue.length; i++) {
									var currentvalue = varvalue[i];

									currentvalue = currentvalue.replace(/^'(.*)'$/, '"$1"'); // TODO: handle "

									if (currentvalue === "+")
										continue;

									if (varvalue[i] in vartable) {
										currentvalue = vartable[varvalue[i]];

										var currentname = null;
										var startbracket = -1;
										var had_startbracket = false;
										for (; (i + 1) < varvalue.length; i++) {
											if (varvalue[i] === "[") {
												startbracket = i;
												had_startbracket = true;
												continue;
											} else if (!had_startbracket) {
												break;
											}

											if (varvalue[i] === "]") {
												had_startbracket = false;

												currentname = varvalue.slice(startbracket + 1, i);
												currentvalue = currentvalue[eval_varvalue(currentname)];
												continue;
											}
										}

										currentvalue = JSON_stringify(currentvalue);
									}

									if (i > 1 && varvalue[i-1] === "+") {
										var lastvalue = JSON_parse(newvalues[newvalues.length-1]);
										newvalues[newvalues.length-1] = JSON_stringify(lastvalue + JSON_parse(currentvalue));
									} else {
										newvalues.push(currentvalue);
									}
								}

								return JSON_parse(newvalues.join(" "));
							};

							for (var i = 0; i < tokens.length; i++) {
								var token = tokens[i];
								if (token === "var") {
									var varname = tokens[i + 1];
									i += 3; // =
									var begin = i;
									for (; i < tokens.length; i++) {
										if (tokens[i] === ";")
											break;
									}

									var varvalue = tokens.slice(begin, i);
									vartable[varname] = eval_varvalue(varvalue);
								} else if (token in vartable) {
									var parent_table = vartable;
									var currentname = token;
									var startbracket = -1;
									var had_startbracket = false;

									i++;
									for (; (i + 1) < tokens.length; i++) {
										if (tokens[i] === "[") {
											startbracket = i;
											had_startbracket = true;
											continue;
										} else if (!had_startbracket) {
											break;
										}

										if (tokens[i] === "]") {
											had_startbracket = false;

											parent_table = parent_table[currentname];
											currentname = tokens.slice(startbracket + 1, i);
											currentname = eval_varvalue(currentname);
											continue;
										}
									}

									i++; // =

									var begin = i;
									for (; i < tokens.length; i++) {
										if (tokens[i] === ";")
											break;
									}

									var varvalue = tokens.slice(begin, i);
									parent_table[currentname] = eval_varvalue(varvalue);
								}
							}

							done(vartable[main_name], 3*60*60);
						}
					});
				});
			};

			var get_max_pornhub_video = function(data) {
				var obj = {
					url: null,
					extra: {}
				};

				if (data.link_url) {
					obj.extra.page = data.link_url;
				}

				if (data.video_title) {
					obj.extra.caption = data.video_title;
				}

				if (!data.mediaDefinitions) {
					return obj;
				}

				var maxquality = null;
				var maxobj = null;
				for (var i = 0; i < data.mediaDefinitions.length; i++) {
					if (data.mediaDefinitions[i].format === "hls")
						continue;

					var ourquality = data.mediaDefinitions[i].quality;
					if (!ourquality) {
						console_warn("Unable to find quality for ", data.mediaDefinitions[i]);
						continue;
					}
					ourquality = parseInt(ourquality);

					if (!data.mediaDefinitions[i].videoUrl) {
						continue;
					}

					if (!maxquality || ourquality > maxquality) {
						maxquality = ourquality;
						maxobj = data.mediaDefinitions[i];
					}
				}

				if (maxobj && maxobj.videoUrl) {
					obj.url = maxobj.videoUrl;
					return obj;
				} else {
					obj.url = null;
					return obj;
				}
			};

			if (options.element.parentElement && options.element.parentElement.tagName === "A") {
				var match = options.element.parentElement.href.match(/\/view_video\.php\?viewkey=([^&]+)/);
				if (match) {
					get_pornhub_videodata(match[1], function(data) {

						if (!data) {
							return options.cb(null);
						}

						return options.cb(get_max_pornhub_video(data));
					});

					return {
						waiting: true
					};
				}
			}
		}

		if (domain_nosub === "t8cdn.com" ||
			domain_nosub === "ypncdn.com") {
			newsrc = src
				.replace(/\/[0-9]+x[0-9]+\/+([0-9]+)(?:[(][a-z]+=[^/)]*\)){1,}([^/]*)$/, "/originals/$1$2")
				.replace(/(\/originals?\/+[0-9]+(?:\/+[^/]*?)?)(?:[(][a-z]+=[^/)]*\)){1,}([^/]*)$/, "$1$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nosub === "rdtcdn.com" && /^[a-z]i\./.test(domain)) {
			return src
				.replace(/\/m=[^/]*\/+_thumbs\/+/, "/m=/_thumbs/")
				.replace(/\/m=[^/]*\/+(_thumbs\/+gallery\/.*_[0-9]+_)\/+[^/]+-([0-9]+\.[^/.]+)(?:[?#].*)?$/, "/m=/$1$2");
		}

		if ((domain_nosub === "t8cdn.com" || domain_nosub === "ypncdn.com") && options && options.do_request && options.cb) {
			id = src.match(/:\/\/[^/]+\/+[0-9]{6}\/+[0-9]{1,2}\/+([0-9]+)\/+(?:vl_|[0-9]+[pP]_[0-9]+[kK]_|(?:[0-9]+x[0-9]+|originals?)\/+[0-9]+(?:[(][a-z]+=[^/)]*\)){0,}\.)/);
			if (id) {
				id = id[1];

				var fetch_tube8 = function(done) {
					options.do_request({
						url: "https://www.tube8.com/a/a/" + id + "/",
						method: "GET",
						onload: function(resp) {
							if (resp.readyState < 4)
								return;

							if (resp.status !== 200) {
								console_error(resp);
								return done(null, false);
							}

							var match = resp.responseText.match(/var\s+flashvars\s*=\s*({.*?})\s*; *\n/);
							if (!match) {
								console_error("Unable to find match", resp);
								return done(null, false);
							}

							try {
								var json = JSON_parse(match[1]);
								done(json, 6*60*60);
							} catch (e) {
								console_error(e);
								return done(null, false);
							}
						}
					});
				};

				var fetch_ypn = function(done) {
					options.do_request({
						url: "https://www.youporn.com/watch/" + id + "/",
						method: "GET",
						onload: function(resp) {
							if (resp.readyState < 4)
								return;

							if (resp.status !== 200) {
								console_error(resp);
								return done(null, false);
							}

							var match = resp.responseText.match(/page_params.video.mediaDefinition\s*=\s*(\[{.*?}\])\s*; *\n/);
							if (!match) {
								console_error("Unable to find match", resp);
								return done(null, false);
							}

							try {
								var mediaDefinition = JSON_parse(match[1]);
								var json = {
									link_url: resp.finalUrl,
									mediaDefinition: mediaDefinition
								};
								done(json, 6*60*60);
							} catch (e) {
								console_error(e);
								return done(null, false);
							}
						}
					});
				};

				var cache_key = id;
				var fetch = null;

				if (domain_nosub === "t8cdn.com") {
					cache_key = "tube8:" + id;
					fetch = fetch_tube8;
				} else if (domain_nosub === "ypncdn.com") {
					cache_key = "ypn:" + id;
					fetch = fetch_ypn;
				}

				api_cache.fetch(cache_key, function(data) {
					if (!data) {
						return options.cb(null);
					}

					try {
						var maxdef = 0;
						var maxobj = null;
						for (var i = 0; i < data.mediaDefinition.length; i++) {
							if (!data.mediaDefinition[i].videoUrl)
								continue;

							if (data.mediaDefinition[i].quality > maxdef) {
								maxdef = data.mediaDefinition[i].quality;
								maxobj = data.mediaDefinition[i];
							}
						}

						var newsrc = maxobj.videoUrl;
						var noq = src.replace(/[?#].*$/, "").replace(/^[a-z]+:\/\/[^/]+\/+/, "");
						var newnoq = newsrc.replace(/[?#].*$/, "").replace(/^[a-z]+:\/\/[^/]+\/+/, "");

						if (noq === newnoq)
							newsrc = src;

						return options.cb({
							url: newsrc,
							extra: {
								page: data.link_url
							},
							video: true,
							is_private: true // linked to IP
						});
					} catch(e) {
						console_error(e);
					}

					options.cb(null);
				}, fetch);

				return {
					waiting: true
				};
			}
		}

		if (domain === "cci.xnxx.fan") {
			return src.replace(/(\/original\/)(?:\([a-z]+=[^/)]*\))*([^/]*)$/, "$1$2");
		}

		if (domain === "img.fril.jp") {
			return src.replace(/\/[a-z]\/([^/]*)$/, "/l/$1");
		}

		if (domain === "img.cinematoday.jp") {
			return src
				.replace(/-[a-z0-9]*x[a-z0-9]*(\.[^/.]*)$/, "$1")
				.replace(/\/_size_[^/]*\//, "/");
		}

		if (domain_nosub === "seesaa.net" ||
			(domain_nowww === "fanblogs.jp" && string_indexof(src, "/file/") >= 0)) {
			return src.replace(/-thumbnail[0-9]*(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}


		if (domain_nosub === "ledevoir.com" && domain.match(/media[0-9]*\.ledevoir\.com/)) {
			return src.replace(/\/images_galerie\/(?:[^-/._]*_)?([0-9]+_[0-9]+)\//, "/images_galerie/nwdp_$1/");
		}

		if (domain_nowww === "infotel.ca") {
			return src.replace(/\/medialibrary\/image\/[^-_/.]*-/, "/medialibrary/image/orig-");
		}

		if ((domain_nowww === "eleconomista.com.mx" && string_indexof(src, "/img/") >= 0) ||
			(domain_nowww === "informador.mx" && string_indexof(src, "/img/") >= 0) ||
			(domain_nowww === "eldeber.com.bo" && string_indexof(src, "/img/") >= 0) ||
			(domain_nowww === "chispa.tv" && string_indexof(src, "/img/") >= 0) ||
			(domain_nowww === "debate.com.mx" && string_indexof(src, "/img/") >= 0) ||
			(domain_nowww === "diarioshow.com" && string_indexof(src, "/img/") >= 0) ||
			(domain === "wp.eldeber.com.bo" && string_indexof(src, "/img/") >= 0) ||
			(domain_nowww === "tribuna.com.mx" && string_indexof(src, "/img/") >= 0) ||
			(domain_nowww === "nacionrex.com" && string_indexof(src, "/img/") >= 0) ||
			(domain_nowww === "enpareja.com" && string_indexof(src, "/img/") >= 0) ||
			(domain_nowww === "filo.news" && string_indexof(src, "/img/") >= 0) ||
			(domain_nowww === "laverdadnoticias.com" && string_indexof(src, "/img/") >= 0) ||
			(domain_nowww === "rosario3.com" && string_indexof(src, "/img/") >= 0) ||
			(domain_nowww === "soycarmin.com" && string_indexof(src, "/img/") >= 0) ||
			src.match(/^[a-z]+:\/\/[^/]*\/(?:__)?export\/+(?:[0-9]{10,}\/+)?sites\/+[^/]+\/+img\/+(?:[^/]*\/+)?[0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+[^/]*\.[^/._]*_[0-9]{5,}\.[^/.]*(?:_[0-9]{5,}\.[^/.]*)?(?:[?#].*)?$/)) {
			return src.replace(/(\/[^/.]*\.[^/]*)_[0-9]+\.[^/._]*$/, "$1");
		}

		if (domain === "gcm-v2.omerlocdn.com") {
			return src.replace(/_[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain === "www.dailystar.com.lb" ||
			domain === "dailystar.com.lb") {
			return src.replace(/([0-9]+)(?:_[^0-9][^/.]*)*(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "images.dailykos.com") {
			return src.replace(/(\/(?:images|avatars)\/[0-9]+\/)[^/]*\/([^/]*)$/, "$1original/$2");
		}

		if (domain_nowww === "imgmax.com" ||
			domain_nowww === "404store.com" ||
			domain_nowww === "imgpost.co.uk" ||
			domain === "ap.imagensbrasil.org" ||
			domain_nowww === "picture-post.com" ||
			domain_nowww === "stakimages.me" ||
			domain_nowww === "imgsnap.com" ||
			domain === "img.imgmax.com" ||
			domain_nowww === "imgpile.com" ||
			domain === "i.img.ie" ||
			domain === "i.lensdump.com" ||
			domain === "img.faploads.com" ||
			(domain_nowww === "celebact.org" && string_indexof(src, "/images/") >= 0) ||
			domain_nosub === "imghost.io" ||
			domain_nosub === "img26.com" ||
			(domain_nowww === "cheapesthosting.xyz" && src.match(/\/i\/+images\//)) ||
			domain_nowww === "s4tu.com" ||
			domain_nowww === "jiopic.com" ||
			domain_nowww === "beautifulmodels.xyz" ||
			domain === "images.superimg.com" ||
			(domain_nowww === "ultraimg.com" && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "celebfeetpics.com" && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "lolzilla.net" && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "extraimage.net" && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "celebphotos.club" && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "image.farm" && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "girlspic.net" && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "tgfimage.rocks" && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "imgfy.net" && string_indexof(src, "/images/") >= 0) ||
			(domain_nosub === "picho.st" && /^s[0-9]*\./.test(domain)) ||
			domain_nowww === "imagenup.com" ||
			(domain_nowww === "imgz.pw" && /\/ch\/+images\//.test(src)) ||
			(domain_nowww === "xxximg.art" && string_indexof(src, "/images/") >= 0) ||
			(domain_nosub === "tag-fox.com" && /^data[0-9]*\./.test(domain)) ||
			(domain_nosub === "gifyu.com" && string_indexof(src, "/images/") >= 0) ||
			domain_nowww === "image-bugs.com") {
			return src.replace(/\.(?:th|md)(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "cheapesthosting.xyz") {
			return src.replace(/\/upload\/[a-z]+(\/[0-9]+\/[0-9]+\/[0-9]+\/)/, "/upload/big$1");
		}

		if (domain === "static.maxmodels.pl") {
			if (string_indexof(src, "/photos/") >= 0 || string_indexof(src, "/article/") >= 0) {
				return src.replace(/_thumb(\.[^/.]*)$/, "$1");
			}

			if (string_indexof(src, "/profile/") >= 0) {
				return src.replace(/_[a-z]+(\.[^/.]*)$/, "_profile$1");
			}
		}

		if (domain_nosub === "img.yt") {
			return src.replace("/small/", "/big/");
		}

		if (domain === "cdn.wallpaperjam.com") {
			return src.replace(/\/static\/images\/.*?\/([a-f0-9]+)(\.[^/.]*)$/, "/$1/image$2");
		}

		if (domain === "cdn.oboi7.com") {
			return src.replace(/\/static\/images\/[a-z]\//, "/content/images/");
		}

		if (domain === "media.tabloidbintang.com") {
			return src.replace(/\/thumb\/([^/]*\.[^/.]*)(?:\/.*)/, "/$1");
		}

		if (domain === "media.teen.co.id") {
			return src.replace(/\/thumb\/([^/?]*\.[^/.?]*).*?[?&](p=[^&]*).*/, "/view/$1?$2");
		}

		if ((domain_nosub === "hudong.com" && string_indexof(domain, ".att.hudong.com") >= 0) ||
			(domain_nowww === "eiga-board.com" && string_indexof(src, "/assets/medias/") >= 0) ||
			(domain_nowww === "kaigai-drama-board.com" && string_indexof(src, "/assets/medias/") >= 0)) {
			newsrc = src.replace(/_s(\.[^/.]*)$/, "$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nosub === "hudong.com" && string_indexof(domain, ".att.hudong.com") >= 0) {
			return src.replace(/_[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "ihanyu.com") {
			return src.replace(/\/cache\/([^/]*)\/([^/]*\/[0-9]+\/[0-9]+\/)[0-9]+\/([0-9]+\.[^/.]*)$/, "/uploadfile/$1/$2$3");
		}

		if (domain === "imgsh.jpnxcn.com" ||
			domain_nowww === "gongl8.com" ||
			domain === "upload.mcchina.com" ||
			domain_nowww === "suanning.com" ||
			domain === "upload.site.cnhubei.com" ||
			domain_nowww === "artsbj.com" ||
			domain === "img.94hnr.com" ||
			domain === "upload.taihainet.com" ||
			domain === "img.sexbeautygirl.com" ||
			domain === "news.yule.com.cn" ||
			domain === "upload.sanqin.com" ||
			domain === "upload.art.ifeng.com" ||
			domain === "file.acgxmanga.com" ||
			(domain_nosub === "ablwang.com" && domain.match(/m[0-9]*\.ablwang\.com/))) {
			return src.replace(/\/thumb_[0-9]+_[0-9]+_/, "/");
		}

		if (domain_nowww === "gaobei.com" &&
			string_indexof(src, "/upload/") >= 0) {
			return src.replace(/_[a-z](\.[^/.]*)$/, "_b$1");
		}

		if (host_domain_nosub === "tiktok.com" && options.element) {
			if (options.element.tagName === "DIV" && options.element.classList.contains("play-button")) {
				return {
					url: origsrc,
					bad: "mask"
				};
			}
		}

		if (host_domain_nosub === "tiktok.com" && options && options.cb && options.do_request && options.element) {
			var query_tiktok = function(url, cb) {
				var normalized_url = url
					.replace(/^[a-z]+:\/\//, "https://")
					.replace(/:\/\/[^/]*tiktok\.com\/+/, "://www.tiktok.com/")
					.replace(/\/*(?:[?#].*)?$/, "");
				var cache_key = "tiktok:" + normalized_url;
				api_cache.fetch(cache_key, cb, function(done) {
					options.do_request({
						url: url,
						method: "GET",
						onload: function(resp) {
							if (resp.readyState < 4)
								return;

							if (resp.status !== 200) {
								console_error(resp);
								return done(null, false);
							}

							var match = resp.responseText.match(/<script[^>]+id="__NEXT_DATA__"[^>]*>({.*?})\s*<\/script>/);
							if (!match) {
								console_error("Unable to find match", resp);
								return done(null, false);
							}

							try {
								var json = JSON_parse(match[1]);
								return done(json, 60*60);
							} catch (e) {
								console_error(e);
								return done(null, false);
							}
						}
					});
				});
			};

			var tiktok_finalcb = function(obj) {
				var videourl = obj.url;
				common_functions.set_tiktok_vid_filename(obj)

				if (options.rule_specific && options.rule_specific.tiktok_no_watermarks) {
					common_functions.get_best_tiktok_url(api_cache, do_request, videourl, function(newurl) {
						if (newurl) {
							videourl = newurl;
							obj.url = videourl;
						}

						if (!common_functions.set_tiktok_vid_filename(obj)) {
							delete obj.filename;
						}

						return options.cb(obj);
					});
				} else {
					return options.cb(obj);
				}
			};

			var current = options.element;
			while (current) {
				if (current.tagName === "A" && /\/@[^/]+\/+video\/+[0-9]+\/*(?:[?#].*)?$/.test(current.href)) {
					query_tiktok(current.href, function(data) {
						if (!data) {
							var video_query = current.querySelector("video");
							if (video_query && video_query.src && /muscdn\.com\//.test(video_query.src)) {
								var obj = {
									url: video_query.src,
									video: true
								};

								return tiktok_finalcb(obj);
							}

							return options.cb(null);
						}

						try {
							var item = data.props.pageProps.videoData.itemInfos;
							var videourl = item.video.urls[0];
							var caption = item.text;

							var obj = {
								url: videourl,
								extra: {
									caption: caption
								},
								video: true
							};

							return tiktok_finalcb(obj);
						} catch (e) {
							console_error(e);
							return options.cb(null);
						}
					});

					return {
						waiting: true
					};
				}

				current = current.parentElement;
			}
		}

		if (domain_nosub === "muscdn.com" && /^v[0-9]+\./.test(domain)) {
			var baseobj = {
				url: src,
				video: true
			};

			common_functions.set_tiktok_vid_filename(baseobj);

			if (options.do_request && options.cb && options.rule_specific && options.rule_specific.tiktok_no_watermarks) {
				common_functions.get_best_tiktok_url(api_cache, options.do_request, src, function(newurl) {
					if (newurl) {
						baseobj.url = newurl;
					}
					common_functions.set_tiktok_vid_filename(baseobj);

					options.cb(baseobj);
				});

				return {
					waiting: true
				};
			} else {
				return baseobj;
			}
		}

		if ((domain_nosub === "pstatp.com" ||
			 domain_nosub === "ipstatp.com" ||
			 domain_nosub === "sgpstatp.com")
			&& domain.match(/[pi][a-z]?[0-9]*\./) ||
			domain === "p16-tiktokcdn-com.akamaized.net" ||
			domain === "p16.tiktokcdn.com" ||
			domain === "p16-hypstarcdn-com.akamaized.net" ||
			domain === "p16.hypstarcdn.com" ||
			domain_nosub === "bytecdn.cn" ||
			domain_nosub === "byteimg.com" ||
			domain_nosub === "muscdn.com") {
			obj = {
				url: src
					.replace(/(:\/\/[^/]*\/+)(?:medium|large|obj|aweme|list)\/+(?:[0-9]+x[0-9]+\/+)?/, "$1origin/")
					.replace(/(:\/\/[^/]+\/+origin\/+.*?)\.image(?:[?$].*)?$/, "$1.jpg")
					.replace(/\/img\/+(.+\/+[0-9a-f]{10,}(?:\/+[^/]+|\.[^/.~?#]+)?)~noop(\.[^/.]*)(?:[?#].*)?$/, "/origin/$1$2")
					.replace(/(\/origin\/+.+\/+[0-9a-f]{10,}\/+[^/.]+\.[^/.]+)\.[^/.]+$/, "$1")
					.replace(/(\/img\/+.+\/+[0-9a-f]{10,}(?:\/+[^/]+|\.[^/.~?#]+)?~)[^/.]*?(\.[^/.]*)(?:[?#].*)?$/, "$1noop$2")
					.replace(/\?imageView2.*/, ""),
				can_head: false // 404
			};

			if (/^v[0-9]*\./.test(domain) && string_indexof(src, "/video/") >= 0) {
				obj.video = true;
			}

			return obj;
		}

		if (false) {
			newsrc = src
				.replace(/(\/img\/+[^/]*\/+[0-9a-f]+~)[^/.]*?(\.[^/.]*)(?:[?#].*)?$/, "$1noop$2")
				.replace(/\/large(\/+[^/]*\/+[0-9a-f]+)(?:\.[^/.]*)?(?:[?#].*)?$/, "/img$1~noop.jpeg")
				.replace(/\/(?:aweme|list)\/+[0-9]+x[0-9]+\/+/, "/obj/")
				.replace(/\?imageView2.*/, "");
			if (newsrc !== src)
				return newsrc;

			return {
				url: src,
				can_head: false // 404
			};
		}


		if (domain === "img-bcy-qn.pstatp.com") {
			return src
				.replace(/(\/post\/[a-z0-9]+\/[a-z0-9]+\.[^/.]*)\/[^/]*(?:[?#].*)?$/, "$1")
				.replace(/(\/avatar\/[0-9]+\/[a-f0-9]+\/[^/]*\.[^/.]*)\/[^/]*(?:[?#].*)?$/, "$1");
		}

		if (domain === "tuchong.pstatp.com") {
			return add_full_extensions(src.replace(/(\/[0-9]+\/+)[a-z](?:t[0-9]+)?(\/+[0-9]+\.[^/.]*)(?:[?#].*)?$/,
												   "$1f$2"));
		}

		if (domain === "p16-sg-default.akamaized.net") {
			return src.replace(/^[a-z]+:\/\/[^/]+\/+china-img\/+/, "https://p0.pstatp.com/");
		}

		if (domain_nosub === "tiktokcdn.com" && /^v[0-9]*\./.test(domain) && string_indexof(src, "/video/") >= 0) {
			return {
				url: src,
				video: true
			};
		}

		if (domain === "img.jizy.cn") {
			return src.replace(/\/img\/[a-z]\//, "/img/l/");
		}

		if (domain === "v.img.pplive.cn") {
			return src.replace(/(:\/\/[^/]*\/)sp[0-9]+\//, "$1");
		}

		if (domain === "uploadfile.bizhizu.cn") {
			return src.replace(/(\/[0-9a-f]*\.)([^/.]*)(?:\.[0-9]+\.[0-9]+\.[^/.]*)?$/, "$1$2.source.$2");
		}

		if (domain_nosub === "tgbusdata.cn" ||
			domain_nosub === "tuwandata.com") {
			return src.replace(/.*\/thumb\/[^/]*\/[^/]*\/u\//, "http://");
		}

		if (domain_nowww === "dajiazhao.com" ||
			domain_nowww === "hscbw.com" ||
			domain_nowww === "xgkoushi.com" ||
			domain_nowww === "youdew88yl.com" ||
			domain_nowww === "nvshenmen.com" ||
			domain_nowww === "sfyry.com" ||
			domain_nowww === "ynjzsf.com" ||
			src.match(/:\/\/[^/]*\/+(?:uploads|upfile)\/+allimg\/+[0-9]{6,8}\/+[0-9]+-[0-9A-Z]+-lp\.[^/.]*(?:[?#].*)?$/)) {
			return src.replace(/(\/(?:uploads|upfile)\/+allimg\/+[0-9]+\/+[0-9A-Z]+(?:-[0-9A-Z]+)?)(?:-lp)(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "taopic.com" && domain.match(/^pic[0-9]*\./)) {
			return src.replace(/:\/\/pic([0-9]*\.[^/]*\/[0-9]+\/[0-9]+-[A-Z0-9]+)-[a-z]+(\.[^/.]*)$/,
							   "://img$1$2");
		}

		if (domain_nowww === "renwenjun.com") {
			return src.replace(/_lit(\.[^/.]*)$/, "_0$1");
		}

		if (domain_nowww === "9x2.net" ||
			(domain_nosub === "izaoxing.com" && /^img[0-9]*\./.test(domain))) {
			return src.replace(/_lit(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "lrfczp.com") {
			return src.replace(/_[0-9]+_[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain === "img-toutiao.mia.com") {
			return src.replace(/\&.*/, ""); // removing @ works, but forces download, and very big images have imgScale so it's probably fine
		}

		if (domain_nosub === "xeeok.com" && string_indexof(domain, "pic.xeeok.com") >= 0) {
			return src.replace(/_s[0-9]*(\.[^/.]*)$/, "$1");
		}

		if (domain === "img.laonanren.com" ||
			domain === "cf.whl4u.jp") {
			return src.replace(/t(\.[^/.]*)$/, "$1");
		}

		if (domain_nosub === "xuite.net" && string_indexof(domain, ".share.photo.xuite.net") >= 0) {
			return src.replace(/_[a-zA-Z](\.[^/.]*)$/, "_y$1");
		}

		if (domain === "img.sportsv.net") {
			return src.replace(/\/[a-z]+-([^-/]*)-[0-9a-z]*x[0-9a-z]*(\.[^/.]*)$/, "/$1$2");
		}

		if (domain_nosub === "espncdn.com") {
			newsrc = decodeURIComponent(src.replace(/\/combiner\/i\?img=([^&]*).*/, "$1"));
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/(\/r[0-9]+)(?:_[^/.]*)(\.[^/.]*)$/, "$1$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (host_domain_nosub === "fc2.com" && options.element) {
			if (options.element.tagName === "svg" && options.element.parentElement && options.element.parentElement.tagName === "DIV") {
				if (string_indexof(options.element.parentElement.getAttribute("class"), "c-videoThumbOver") >= 0)
					return {
						url: origsrc,
						bad: "mask"
					};
			}
		}

		if (domain_nosub === "fc2.com" && domain.match(/blog-imgs-[0-9]*(?:-[^.]*)?.*\.fc2\.com/)) {
			newsrc = src.replace(/:\/\/(blog-imgs-[0-9]*)\./, "://$1-origin.");
			if (newsrc !== src)
				return newsrc;

			return src.replace(/s(\.[^/.]*)$/, "$1");
		}

		if (domain_nosub === "fc2.com" && /^(?:vip-)?video[0-9]*-thumbnail[0-9]*\./.test(domain)) {
			obj = {
				url: src
			};

			id = null;
			match = src.match(/\/up\/+thumb[0-9]*\/+[0-9]{6}\/+[0-9]{2}\/+.\/+([0-9]{8}[a-zA-z0-9]+)\./);
			if (match) {
				id = match[1];

				obj.extra = {page: "https://video.fc2.com/content/" + id};
			}

			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+(?:[wh][0-9]+){1,2}\/+([^/]+\.fc2\.com\/)/, "http://$1");
			if (newsrc !== src)
				return fillobj_urls([newsrc, src], obj);

			newsrc = src.replace(/(\/member\/+[0-9]+\/+[0-9]+\/+mb_pict_)icon_/, "$1");
			if (newsrc !== src)
				return fillobj_urls([newsrc, src], obj);

			newsrc = src.replace(/\/up\/+thumb\/+([0-9]{6}\/+)/, "/up/thumb2/$1");
			if (newsrc !== src)
				return fillobj_urls([newsrc, src], obj);

			if (id && options.do_request && options.cb) {
				var cache_key = "fc2_video:" + id;
				api_cache.fetch(cache_key, function(data) {
					options.cb(data);
				}, function(done) {
					options.do_request({
						method: "GET",
						url: obj.extra.page,
						headers: {
							Referer: ""
						},
						onload: function(resp) {
							if (resp.readyState !== 4)
								return;

							if (resp.status !== 200) {
								console_error(resp);
								return done(null, false);
							}

							var match = resp.responseText.match(/window\.FC2VideoObject\.push\(\['..',\s*'([0-9a-f]{10,})'/);
							if (!match) {
								console_error("Unable to find match", resp);
								return done(null, false);
							}

							obj.extra.page = resp.finalUrl;

							options.do_request({
								method: "GET",
								url: "https://video.fc2.com/api/v3/videoplaylist/" + id + "?sh=1&fs=0",
								headers: {
									Referer: resp.finalUrl,
									"X-FC2-Video-Access-Token": match[1],
									"Sec-Fetch-Site": "same-origin",
									"Sec-Fetch-Dest": "empty",
									"Sec-Fetch-Mode": "cors"
								},
								onload: function(resp) {
									if (resp.readyState !== 4)
										return;

									if (resp.status !== 200) {
										console_error(resp);
										return done(null, false);
									}

									try {
										var json = JSON_parse(resp.responseText);
										var playlist = json.playlist;

										var newurl = playlist.nq || playlist.sample;

										if (!url) {
											console_warn("Unable to find playlist URL", json);
											return done(obj, false);
										} else {
											obj.url = urljoin(obj.extra.page, newurl, true);
											obj.headers = {
												Referer: obj.extra.page
											};
											obj.referer_ok = {same_domain_nosub: true};
											obj.video = true;
											return done(obj, 6*60*60);
										}
									} catch(e) {
										console_error(e);
										return done(null, false);
									}
								}
							});
						}
					});
				});

				return {
					waiting: true
				};
			}

			return obj;
		}

		if (domain === "photos.hancinema.net" ||
			domain_nowww === "hancinema.net") {
			return src.replace(/\/photos\/[a-z]*(photo[0-9]*\.[^/.]*)$/, "/photos/fullsize$1");
		}

		if (domain_nowww === "inimura.com" ||
			domain_nowww === "celebritydresses.shop" ||
			domain_nowww === "thecelebritydresses.us" ||
			domain_nowww === "customcelebritydresses.com" ||
			domain_nowww === "realistgold.com" ||
			domain_nowww === "supersoccershop.com" ||
			domain_nowww === "cheap-promdresses.com" ||
			domain_nowww === "weandestudio.com" ||
			domain_nowww === "black-leatherjacket.com" ||
			domain_nowww === "kpopultra.net" ||
			domain_nowww === "myhaircare.com.au" ||
			domain_nowww === "ultimateapparels.com" ||
			domain_nowww === "cajalwinterconference.es" ||
			domain_nowww === "justfashionnow.com" ||
			domain === "mall.3785tv.com" ||
			domain_nowww === "layla-lingerie.com" ||
			domain_nowww === "fnac-andorra.com" ||
			domain_nowww === "keymailrecords.com" ||
			domain_nowww === "bdmaster.net" ||
			domain_nowww === "prettyzone.net" ||
			domain_nowww === "filmstarjackets.com" ||
			domain_nowww === "moviesjacket.com" ||
			domain_nowww === "af-hobby.com" ||
			domain_nowww === "sky-seller.com" ||
			domain_nowww === "i-aurai.com" ||
			domain_nowww === "onesieponatime.com" ||
			domain_nowww === "ucanstarjob.com" ||
			domain === "beniko.ninethemes.net" ||
			domain_nowww === "duvardamoda.com" ||
			domain_nowww === "fresh38.ru" ||
			domain_nowww === "malaysiadropship.com" ||
			domain_nowww === "yukite.ru" ||
			domain_nowww === "huzzk.com" ||
			domain_nowww === "gasookpopgalore.net" ||
			domain_nowww === "honeydear.my" ||
			src.match(/^[a-z]+:\/\/[^/]+\/+image\/+cache\/+data\/.*-[0-9]+x[0-9]+\.[^/.]+(?:[?#].*)?$/)) {
			return src
				.replace(/\/image_cache\/+resize\/+[0-9]+x[0-9]+\/+image\/+/, "/image/")
				.replace(/\/cache\/(.*)-[0-9]+x[0-9]*(?:-[a-z_]+)?(?:_[0-9]+|[wh])?(?:\.[a-z_]+)?(\.[^/.]*(?:[?#].*)?)$/, "/$1$2");
		}

		if (domain_nowww === "outlookweekly.net" && string_indexof(src, "/images/") >= 0) {
			return src.replace(/.*:\/\/[^/]*\/images\//, "http://");
		}

		if (domain === "www.cdn.tv2.no" ||
			domain === "image.side3.no" ||
			domain_nowww === "dbstatic.no") {
			newsrc = src
				.replace(/(:\/\/[^/]*\/[0-9]+\.[^/.?#]*)(?:[?#].*)?$/, "$1?width=-1&height=-1")
				.replace(/\/images.*?[?&]imageId=([0-9]+).*/, "/images?imageId=$1&height=-1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "media-spiceee.net") {
			return src.replace(/\/(?:large|small|thumb_lg|thumb)_([^/]*)$/, "/$1");
		}

		if (domain_nowww === "vettri.net") {
			return src.replace(/\/thumb\/([^/]*)_resize(\.[^/.]*)$/, "/$1$2");
		}

		if (domain_nowww === "tiarashop.eu" ||
			domain_nowww === "streetstylestore.com" ||
			domain_nowww === "asog-labs.com" ||
			domain_nowww === "dreamline.com" ||
			domain_nowww === "techno-import.fr" ||
			domain === "cdn.poplook.com" ||
			domain_nowww === "directgardening.com" ||
			domain_nowww === "kanedashop.com" ||
			domain_nowww === "myamericanmarket.com" ||
			domain_nowww === "lapakfiguremiina.com" ||
			domain_nowww === "donnashape.com" ||
			(domain_nosub === "nin-nin-game.com" && domain.match(/^media[0-9]*\.nin/)) ||
			domain_nowww === "adintime.com" ||
			domain === "shop.marymary.gr" ||
			domain_nowww === "evawigs.com" ||
			domain_nowww === "tehrankbs.pw" ||
			domain_nowww === "ticketshow.ma" ||
			domain_nowww === "asiaworldmusic.fr" ||
			domain_nowww === "kpopmart.com" ||
			domain_nowww === "planetongames.com" ||
			domain_nowww === "baba.es" ||
			domain_nowww === "teamworldshop.it" ||
			domain_nowww === "flyhighstore.pl") {
			return src
				.replace(/(\/img\/+p\/+(?:[0-9]\/+){1,}[0-9]+)[-_][^/.]*(\.[^/.]*)$/, "$1$2")
				.replace(/(:\/\/[^/]*\/+[0-9]+(?:-[0-9]+)?)(?:[-_][^/]*?)?(\/[^/]*)$/, "$1$2");
		}

		if (domain === "skinzwearphotography.com") {
			return src.replace(/\/prod[A-Z][a-z]*\//, "/prodImages/");
		}

		if (domain_nowww === "dubiobikinis.com") {
			return src.replace(/(\/prodimages\/+[^/]*(?:-[A-Z]|_[0-9]+))t(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "shelot.com") {
			return src.replace(/\/upload\/thumbnails\/[0-9]+x[0-9]+[^/]*\//, "/upload/");
		}

		if (domain === "xo.lulus.com") {
			return src.replace(/(\/images\/[^/]*\/)[^/]*\/([^/]*)$/, "$1w_1.0/$2");
		}

		if (domain_nowww === "in-tense.se") {
			return src
				.replace(/\/thumbnails\/[^/]*\//, "/")
				.replace(/(\.[^/._])*_[^/]*$/, "$1");
		}

		if (domain === "image.brazilianbikinishop.com") {
			return src.replace(/\/cache_images\/([^/_]*)_[^/.]*(\.[^/.]*)$/, "/$1$2");
		}

		if (domain === "i.embed.ly" ||
			domain === "i-cdn.embed.ly") {
			newsrc = src.replace(/.*\/(?:display|image).*?[?&]url=([^&]*).*/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "i.genius.com") {
			return decodeURIComponent(src.replace(/.*\/[0-9a-f]+.*?[?&]url=([^&]*).*/, "$1"));
		}

		if (domain_nowww === "hdqwalls.com") {

			return src
				.replace(/\/wallpapers\/[a-z]?thumb\//, "/wallpapers/");
		}

		if (domain_nowww === "wallpaperclicker.com") {
			if (src.match(/\/storage\/+Thumb\/+/)) {
				return [
					src.replace(/\/storage\/+Thumb\/+/, "/storage/wallpaper/"),
					src.replace(/\/storage\/+Thumb\/+/, "/storage/image/")
				];
			}

			return src
				.replace(/\/download\/+Image\.aspx.*?[?&]Imagefilename=([^&]*).*?$/, "/storage/Thumb/$1")
				.replace(/\/wallpaper\/+Download\.aspx.*?[?&]wallfilename=([^&]*).*?$/, "/storage/Thumb/$1");
		}

		if (domain_nosub === "prothomalo.com") {
			return src.replace(/\/cache\/images\/[0-9]+x[0-9]+(?:x[0-9]+)\//, "/");
		}

		if (domain === "c.tribune.com.pk") {
			return src.replace(/-[0-9]+-[0-9]+x[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain_nosub === "reutersmedia.net") {
			var querystr = src.replace(/.*\/r\/\?/, "&");
			var d = querystr.replace(/.*&d=([^&]*).*/, "$1");
			var t = "2";//querystr.replace(/.*&t=([^&]*).*/, "$1");
			i = querystr.replace(/.*&i=([^&]*).*/, "$1");
			return src.replace(/\/r\/\?.*/, "/r/?d=" + d + "&t=" + t + "&i=" + i);
		}

		if (domain === "r.fod4.com") {
			return src.replace(/^[a-z]*:\/\/[^/]*\/.*\/([a-z]*:\/\/)/, "$1");
		}

		if (domain === "p.fod4.com") {
			return src.replace(/(\/media\/[0-9a-f]+\/)[a-z]=[a-z0-9]+\//, "$1");
		}

		if (domain_nowww === "xdressy.com" ||
			domain_nowww === "starcelebritydresses.com") {
			return src.replace(/-thumb(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "wallpaperflare.com" && src.match(/\/static\//)) {
			return src.replace(/-(?:thumb|preview)(\.[^/.]*)$/, "$1");
		}

		if (domain === "cdn.store-assets.com") {
			return src.replace(/_[0-9]+x(?:[0-9]+)?(\.[^/.]*)$/, "$1");
		}

		if (domain === "img.shopperboard.com") {
			return src.replace(/-[a-z]+(\.[^/.]*)$/, "$1");
		}

		if ((domain_nosub === "gog.com" ||
			domain_nosub === "gog-statics.com") && domain.match(/images.*\./)) {
			newsrc = src.replace(/(\/[0-9a-f]{10,}_bs_logo)_small(\.[^/.]+)(?:[?#].*)?$/, "$1_big$2");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/(\/[0-9a-f]{10,}_bs_background)_(?:500|800)(\.[^/.]+)(?:[?#].*)?$/, "$1_1275$2");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/(\/[0-9a-f]{10,}_product_tile_(?:349|398))(\.[^/.]+)(?:[?#].*)?$/, "$1_2x$2");
			if (newsrc !== src)
				return newsrc;

			return src.replace(/(\/[0-9a-f]*)_[^/.]*(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "items.gog.com") {
			if (/\/mp4\/+[^/]+\.mp4(?:[?#].*)?$/.test(src)) {
				return {
					url: src,
					video: true
				};
			}
		}

		if (domain === "www.destructoid.com") {
			return src.replace(/-(?:t|noscale)(\.[^/.]*)$/, "$1");
		}

		if (domain_nosub === "xboxlive.com" &&
			domain.match(/images.*\.xboxlive\.com/) &&
			src.match(/\/image\?/)) {
			newsrc = src.replace(/\/image[^/]?[?&]url=([^&]*).*/, "/image?url=$1");
			if (newsrc !== src)
				return newsrc;

			return {
				url: src,
				head_wrong_contenttype: true
			};
		}

		if (domain === "musicimage.xboxlive.com") {
			return {
				url: src.replace(/\/image\?.*/, "/image?locale=en-US"),
				can_head: false // GET, OPTIONS
			};
		}

		if (domain === "media.withtank.com") {
			return src.replace(/_[0-9]+_wide(\.[^/.]*)$/, "$1");
		}

		if (domain === "images.cdn.realviewdigital.com") {
			var type = src.match(/[?&]type=([^&]*)/);
			return src.replace(/\?.*/, "?type=" + type[1]);
		}

		if (domain === "proxy.duckduckgo.com" ||
			domain === "external-content.duckduckgo.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+iur?\/\?(?:.*&)?u=([^&]*).*/, "$1");
			if (newsrc !== src) {
				return decodeuri_ifneeded(newsrc);
			}
		}

		if (host_domain_nowww === "duckduckgo.com" && options.element && options.do_request && options.cb) {
			var get_query_params = function(url, cb) {
				var query = url.replace(/.*\?(?:.*&)?q=([^&]+).*?$/, "$1");

				var cache_key = "duckduckgo_imagequeryparams:" + query;
				api_cache.fetch(cache_key, cb, function(done) {
					options.do_request({
						url: url,
						method: "GET",
						onload: function(resp) {
							if (resp.readyState !== 4)
								return;

							if (resp.status !== 200) {
								console_error(resp);
								return done(null, false);
							}

							var respobj = {
								query: query,
								start: 0
							};

							var match = resp.responseText.match(/'\/[td]\.js\?(?:.*?&)?l=([^&]+)/);
							if (!match) {
								console_error("Unable to find language", resp);
								return done(null, false);
							}

							respobj.lang = match[1];

							match = resp.responseText.match(/;\s*vqd\s*=\s*["']([-0-9]{20,})["']/);
							if (!match) {
								console_error("Unable to find vqd", resp);
								return done(null, false);
							}

							respobj.vqd = match[1];

							return done(respobj, 60*60);
						}
					});
				});
			};

			var get_images = function(querydata, cb) {
				var cache_key = "duckduckgo_imageresults:" + querydata.query + "," + querydata.start;
				api_cache.fetch(cache_key, cb, function(done) {
					var start_query = "";
					if (querydata.start) {
						start_query = "&s=" + querydata.start;
					}

					options.do_request({
						method: "GET",
						url: "https://duckduckgo.com/i.js?l=" + querydata.lang + "&o=json&q=" + querydata.query + "&vqd=" + querydata.vqd + "&p=-1" + start_query + "&v7exp=a",
						onload: function(resp) {
							if (resp.readyState !== 4)
								return;

							if (resp.status !== 200) {
								console_error(resp);
								return done(null, false);
							}

							try {
								var json = JSON_parse(resp.responseText);
								return done(json, 60*60);
							} catch (e) {
								console_error(e);
								return done(null, false);
							}
						}
					});
				});
			};

			var get_bing_imageid = function(url) {
				var newurl = url.replace(/.*[?&]id=([^&]+).*/, "$1");
				if (newurl !== url)
					return newurl;

				return null;
			};

			var get_result_from_image = function(querydata, image, cb) {
				get_images(querydata, function(data) {
					if (!data) {
						return cb(null);
					}

					var image_src = decodeURIComponent(image.src);
					var image_id = get_bing_imageid(image_src);
					if (!image_id) {
						console_warn("Unable to get bing image id for" + image.src);
						return cb(null);
					}

					for (var i = 0; i < data.results.length; i++) {
						var thumbnail_id = get_bing_imageid(data.results[i].thumbnail);
						if (!thumbnail_id) {
							console_warn("Unable to get bing image id for" + data.results[i].thumbnail);
							continue;
						}

						if (image_id === thumbnail_id) {
							return cb(data.results[i].image);
						}
					}

					if (data.next) {
						var start = data.next.replace(/.*&s=([0-9]+).*$/, "$1");
						if (start !== data.next) {
							querydata.start = start;
						}

						return get_result_from_image(querydata, image, cb);
					}

					return cb(null);
				});
			}

			if (!options._internal_info || !options._internal_info.duckduckgo_host) {
				if (options.element.tagName === "IMG") {
					if (options.element.classList.contains("tile--img__img") ||
						options.element.classList.contains("module--images__thumbnails__image")) {
						options._internal_info.duckduckgo_host = true;

						get_query_params(document.location.href, function(data) {
							if (!data) {
								return options.cb(null);
							}

							get_result_from_image(data, options.element, function(url) {
								return options.cb(url);
							});
						});

						return {
							waiting: true
						};
					}
				}
			}
		}

		if (domain === "static.scientificamerican.com") {
			return src.replace(/(?:_[^/.]*)?(\.[^/.?]*)(?:\?.*)?$/, "_source$1");
		}

		if (domain_nosub === "qwant.com" && domain.match(/s[0-9]*\.qwant\.com/) &&
			string_indexof(src, "/thumbr/") >= 0) {
			return decodeURIComponent(src.replace(/.*[?&]u=([^&]*).*/, "$1"));
		}

		if (domain === "b.fssta.com") {
			return src.replace(/(\/[^/.]*)\.[^/]*(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nosub === "espn.com" && domain.match(/media\..*\.espn\.com$/)) {
			return src.replace(/(\/[0-9]+\/[0-9]+\/)([^/]*)\/[^/]*(\.[^/.]*)$/, "$1$2\/$2$3");
		}

		if (domain === "img.skysports.com" ||
			domain_nosub === "365dm.com") {
			return src.replace(/(:\/\/[^/]*\/[0-9]*\/[0-9]*\/)[^/]*\/(?:[0-9]*\/)?([^/]*)$/, "$1master/$2");
		}

		if (domain_nosub === "bcbits.com" &&
			domain.match(/f[0-9]*\.bcbits\.com/) &&
			string_indexof(src, "/img/") >= 0) {
			return src.replace(/_[0-9]+(\.[^/.]*)$/, "_0$1");
		}

		if (domain_nosub === "motherlessmedia.com") {
			return src
				.replace(/(:\/\/cdn[0-9]*\.)thumbs(\.motherlessmedia\.com\/)/, "$1images$2")
				.replace(/\/thumbs\//, "/images/")
				.replace(/-[a-z]*(\.[^/.?]*)(?:\?.*)?$/, "$1");
		}

		if (domain_nosub === "kiev.ua" &&
			string_indexof(domain, "shram.kiev.ua") >= 0 &&
			string_indexof(src, "/img/") >= 0) {
			return src
				.replace(/-small(\.[^/.]*)$/, "-big$1")
				.replace(/-w[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (googlestorage_container === "cr-resource" &&
			string_indexof(src, "/image/") >= 0) {
			return src.replace(/\/[0-9]+(\/[0-9a-f]*\.[^/.]*)$/, "$1");
		}

		if (domain_nosub === "iol.pt" &&
			string_indexof(src, "/multimedia/") >= 0) {
			return src.replace(/(\/id\/[0-9a-f]+)\/[0-9]+(?:\.[0-9a-f]+)?(?:\.[^/.]*)?$/, "$1");
		}

		if (domain === "img.purch.com") {
			return src
				.replace(/\/[a-z]+\/[0-9]+(?:x[0-9]+)?\//, "/o/")
				.replace(/\/[a-z0-9]+-[0-9]+x[0-9]+(?:-[^/]*)?\/o\//, "/o/");
		}

		if ((domain_nosub === "taobaocdn.com" ||
			 domain_nosub === "tbcdn.cn") &&
			domain.match(/img[0-9]*\./)) {
			return src.replace(/(\.[^/._]*)_[^/.]*\.[^/.]*$/, "$1");
		}

		if (domain === "www.musictory.com" &&
			src.match(/\/pictures\//)) {
			return src.replace(/\/pictures\/[a-z]*\//, "/pictures/originali/");
		}

		if (domain_nosub === "santabanta.com" && domain.match(/media[0-9]*\.santabanta\.com/)) {
			newsrc = src
				.replace(/_th(\.[^/.]*)$/, "$1")
				.replace(/:\/\/media\.santabanta\.com\/medium[0-9]*\//, "://media1.santabanta.com/full1/");
			if (newsrc !== src)
				return newsrc;

			if (src.match(/\/full[0-9]*\//)) {
				return [
					src.replace(/\/full[0-9]*\//, "/full8/"),
					src.replace(/\/full[0-9]*\//, "/full7/"),
					src.replace(/\/full[0-9]*\//, "/full6/"),
					src.replace(/\/full[0-9]*\//, "/full5/")
				];
			}
		}

		if (domain === "www.movieinsider.com" &&
			string_indexof(src, "/images/p/") >= 0) {
			return src.replace(/\/images\/p\/[0-9]+\//, "/images/p/");
		}

		if (domain_nosub === "kastden.org") {
			return src
				.replace(/\/thumbs\/+([^/]+)_th(\.[^/]+)(?:[?#].*)?$/, "/$1$2")
				.replace(/\/thumb\//, "/original/");
		}

		if (domain === "img.hani.co.kr") {
			return src
				.replace(/\/(?:thumbnail|resize)\//, "/original/")
				.replace(/\/[0-9]+_[0-9]+_([0-9]+_[0-9]+\.[^/.]*)$/, "/$1");
		}

		if (domain === "flexible.img.hani.co.kr") {
			return src.replace(/:\/\/[^/]*\/flexible\/.*?\/imgdb\//, "://img.hani.co.kr/imgdb/");
		}

		if (domain === "static.ok.co.uk" ||
			amazon_container === "static.ok.co.uk" ||
			domain === "images.24ur.com" ||
			domain === "image.dnevnik.hr" ||
			domain === "img.bg.sof.cmestatic.com" ||
			domain === "static.altchar.com" ||
			domain === "img.cz.prg.cmestatic.com" ||
			domain === "image.stirileprotv.ro" ||
			amazon_container === "ns.hitcreative.com") {
			return src
				.replace(/\?.*/, "")
				.replace(/\/media\/images\/[^/]*\//, "/media/images/original/");
		}

		if (domain_nosub === "galaxypub.vn" && domain.match(/rs[0-9]*\.galaxypub\.vn/)) {
			return src.replace(/:\/\/[^/]*(\/.*?)(?:\?.*)?$/, "://st.galaxypub.vn$1");
		}

		if (domain_nowww === "implanetcorp.com" &&
			string_indexof(src, "/upload/ctoon/") >= 0) {
			return src
				.replace(/\/[a-z]*\/([0-9]+\.[^/.]*)$/, "/original/$1");
		}

		if (domain_nowww === "doramakun.ru") {
			return src.replace(/\/thumbs\/(.*)-[0-9]+(\.[^/.]*)$/, "/$1$2");
		}

		if (domain === "1gr.cz") {
			return src.replace(/(\/[0-9]+\/[0-9]+\/)[^/]*\/([^/]*)$/, "$1org/$2");
		}

		if (domain_nosub === "ancensored.com" &&
			src.match(/\/files\/images\//)) {
			return src.replace(/(\/[0-9a-f]*)(?:_[a-z]*)?(\.[^/.]*)$/, "$1_full$2");
		}

		if (domain_nowww === "desktopbackground.org" ||
			domain_nowww === "desktop-background.com") {
			return src
				.replace(/\/download\/[^/]*\//, "/download/o/")
				.replace(/(:\/\/[^/]*)\/[pt]\//, "$1/download/o/");
		}

		if (domain === "image.mlive.com" ||
			domain === "image.nj.com" ||
			domain === "image.cleveland.com") {
			return src.replace(/:\/\/image\.([^/]*)\/home\/[a-z]+-media\/[^/]*\/img\//, "://media.$1/");
		}

		if (domain === "s9v7j7a4.ssl.hwcdn.net") {
			return src.replace(/\/galleries\/[^/]*\//, "/galleries/full/");
		}

		if (domain === "k5x5n5g8.ssl.hwcdn.net" ||
			domain === "k4b8k3x5.ssl.hwcdn.net" ||
			domain === "f6j6u6m9.ssl.hwcdn.net" ||
			domain === "cdn.yavtube.com" ||
			domain === "img.yavtube.com" ||
			domain === "a2h6m3w6.ssl.hwcdn.net" ||
			domain === "c9w8v3m5.ssl.hwcdn.net") {
			return src.replace(/(\/content\/+[0-9]+\/+[^/]+-[0-9]+)_[wh][0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "hwcdn.funbags.com" ||
			domain === "hwcdn.voyeurweb.com") {
			regex = /(\/albums\/+[0-9]+\/+)(?:thumb|large)\/+/;

			obj = [];

			newsrc = src.replace(regex, "$1orig/");
			if (newsrc !== src)
				obj.push(newsrc);

			newsrc = src.replace(regex, "$1large/");
			if (newsrc !== src)
				obj.push(newsrc);

			if (obj.length > 0)
				return obj;
		}

		if (domain === "aws-foto.amateri.com") {
			return src.replace(/\/[0-9]+x[^/.]*(\.[^/.]*)$/, "/x$1");
		}

		if (domain === "cdn.tobi.com") {
			return src
				.replace(/\/product_images\/[a-z]+\//, "/product_images/lg/")
				.replace(/(\/[^/.@]*)(\.[^/.]*)$/, "$1@2x$2");
		}

		if (domain === "www.rfa.org") {
			return src.replace(/\/@@images\/.*/, "");
		}

		if (domain === "resize.blogsys.jp") {
			return src.replace(/^[a-z]*:\/\/.*\/([a-z]*:\/\/.*)$/, "$1");
		}

		if (amazon_container === "bw-1651cf0d2f737d7adeab84d339dbabd3-gallery") {
			return src.replace(/_[a-z]+(\.[^/.]*)$/, "_original$1");
		}

		if (domain_nosub === "rl0.ru" && domain.match(/img[0-9]*\.rl0\.ru/)) {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/[^/]*\/+c?[-0-9]+x[-0-9]+(?:q[0-9]+)?i?\/+([^/]*\.[^/.]*\/)/, "$1");
			if (newsrc !== src)
				return add_http(newsrc);
		}

		if (domain_nosub === "wanelo.com" && domain.match(/cdn-img-[0-9]+\.wanelo\.com/)) {
			return src.replace(/\/[^/.]*(\.[^/.]*)$/, "/full_size$1");
		}

		if (domain === "static.kvraudio.com") {
			return src.replace(/(:\/\/[^/]*\/i\/)[a-z]\//, "$1b/");
		}

		if (domain === "contents.dt.co.kr") {
			return src.replace(/\/thum\/[0-9]+\/([0-9]{6})([0-9]+)_[^/.]*(\.[^/.]*)/, "/images/$1/$1$2$3");
		}

		if (domain === "eng.dt.co.kr") {
			return src.replace(/\/images\/thum\/([0-9]+)/, "/images/oriimg/$1[0]");
		}

		if (domain === "image.ytn.co.kr" ||
			domain === "img.sciencetv.kr") {
			return src
				.replace(/(:\/\/[^/]*\/[^/]*\/jpg\/[^/]*\/[^/]*\/[0-9]+_)[a-z](\.[^/.]*)$/, "$1d$2")
				.replace(/^[a-z]+:\/\/[^/]*\/osen\/+([0-9]{4}\/+[0-9]{2}\/+)([0-9]{6})([0-9]{2})([^/]*)(?:[?#].*)?$/,
						 "http://file.osen.co.kr/article/$1$3/$2$3$4");
		}

		if (domain === "photo.kmib.co.kr") {
			return src.replace(/\/thumb_([0-9]+\.[^/.]*)$/, "/$1");
		}

		if (domain === "image.kmib.co.kr") {
			var extra = {};
			match = src.match(/\/online_image\/+[0-9]{4}\/+[0-9]{4}\/+[0-9]*([0-9]{10})(?:_[0-9]+)?\.[^/.]*(?:[?#].*)?$/);
			if (match) {
				extra.page = "http://news.kmib.co.kr/article/view.asp?arcid=" + match[1];
			}

			return {
				url: src,
				extra: extra
			};
		}

		if (domain === "r.kelkoo.com") {
			newsrc = src.replace(/^[a-z]*:\/\/(?:[^/]*\/){7}(http[^/]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);

			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+resize\.php\?(?:.*?&)?image=([^&]+).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (((domain_nosub === "assetsadobe2.com" ||
			 domain_nosub === "legocdn.com" ||
			 domain === "store.storeimages.cdn-apple.com" ||
			 domain === "media.playstation.com" ||
			 domain === "images.samsung.com" ||
			 domain === "images.anthropologie.com" ||
			 domain_nosub === "louisvuitton.com" ||
			 domain === "images.ulta.com" ||
			 domain === "image.uniqlo.com" ||
			 domain_nowww === "staples-3p.com" ||
			 domain === "s.shld.net" ||
			 domain === "assets.ray-ban.com" ||
			 domain === "images.menswearhouse.com" ||
			 domain === "images.shaneco.com"// ||
			 /*domain_nosub === "scene7.com"*/
			 ) && string_indexof(src, "/is/image/") >= 0) ||
			domain === "c.shld.net") {
			match = src.match(/\/i\/s\/.*\?(?:.*&)?src=(https?%3A.*?)(?:&.*)?$/);
			if (match) {
				return decodeURIComponent(match[1]);
			}

			match = src.match(/\/is\/image\/+.*\?(?:.*?&)?src=is{(.*?)}/);
			if (match) {
				return src.replace(/\/is\/image\/.*/, "/is/image/" + decodeURIComponent(match[1]));
			}

			var srcadd = "";

			if (src.match(/\.(?:jpe?g|JPE?G)(?:\?.*)?$/))
				srcadd = "scl=1";
			else
				srcadd = "scl=1&fmt=png-alpha";

			newsrc = src.replace(/(\/is\/+image\/+.*)\?(?:.*?&)?src=([^&]*).*?$/, "$1?src=$2&" + srcadd);
			if (newsrc !== src)
				return newsrc;

			if (!(/[?&]src=/.test(src)))
				return src.replace(/(?:\?.*)?$/, "?" + srcadd);
		}

		if (domain === "sm.ign.com") {
			return src.replace(/\.[0-9]+\.([^/.]*)$/, ".999999999999999.$1");
		}

		if ((domain === "shop.unitedcycle.com" ||
			 domain_nowww === "unicorncards.co.uk" ||
			 domain_nowww === "4my3boyz.com") &&
			string_indexof(src, "/images/thumbs/") >= 0) {
			return src.replace(/_[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "imgr.es") {
			return src.replace(/\/thumb(?:[?#].*)?$/, "");
		}

		if (domain_nosub === "frgimages.com" ||
			domain === "images.footballfanatics.com") {
			newsrc = decodeURIComponent(src.replace(/\/FFImage\/thumb.aspx.*?[?&]i=([^&]*).*/, "$1"));
			if (newsrc !== src)
				return newsrc;

			return src
				.replace(/_[a-z]+(\.[^/.?&]*)$/, "_full$1");
		}

		if (domain === "myanimelist.cdn-dena.com" ||
			domain === "cdn.myanimelist.net" ||
			domain_nowww === "dizigol.net") {
			return src.replace(/(:\/\/[^/]*\/)r\/+[0-9]+x[0-9]+\/+/, "$1");
		}

		if (domain === "picture-cdn.wheretoget.it") {
			return src.replace(/(\/[a-z0-9]*-[a-z])[^/.]*(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "sl.sbs.com.au") {
			return src.replace(/(\/public\/image\/file\/[-a-f0-9]*)\/.*/, "$1");
		}

		if ((domain_nosub === "prezly.com" && string_indexof(domain, ".assets.prezly.com") >= 0) ||
			domain === "i.shgcdn.com" ||
			domain_nowww === "ucarecdn.com" ||
			domain === "cdn.vyper.io" ||
			domain === "assets.myzeki.com" ||
			domain === "thumb.tildacdn.com" ||
			domain === "cdn.slant.co") {
			return src.replace(/(:\/\/[^/]*\/+(?:[a-z]+)?[-a-f0-9]{30,}(?:~[0-9a-f]+)?\/+(?:nth\/+[0-9]+\/+)?).*?(?:\/([^/.]+\.[^/.]+))?(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "slant.co") {
			if (/:\/\/[^/]+\/+images\/+play\.png(?:[?#].*)?$/.test(src)) {
				return {
					url: src,
					bad: "mask"
				};
			}
		}

		if (domain === "leonardo.osnova.io") {
			return src.replace(/(:\/\/[^/]*\/[-a-f0-9]*\/)-\/.*/, "$1");
		}

		if (domain === "cdn.iview.abc.net.au") {
			return src.replace(/\/thumbs\/[0-9]+\//, "/thumbs/i/");
		}

		if (domain_nosub === "ipstatic.net") {
			newsrc = decodeURIComponent(src.replace(/.*\/img.*?[?&]url=([^&]*).*?$/, "$1"));
			if (newsrc !== src)
				return newsrc;

			return src.replace(/\/thumbs\/[0-9]+x[0-9]+\//, "/photos/");
		}

		if (domain === "image.fnnews.com") {
			return src
				.replace(/(\/[a-z]?[0-9]+)_[a-z](\.[^/.]*)/, "$1$2");
		}

		if (domain_nosub === "kym-cdn.com") {
			return add_extensions({
				url: src.replace(/(:\/\/[^/]*\/[^/]*\/(?:images|icons)\/)[^/]*\//, "$1original/"),
				is_original: true
			});
		}

		if (domain === "cmeimg-a.akamaihd.net" ||
			domain === "leafimg-a.akamaihd.net") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/(?:[0-9]+(?:x[0-9]+)?|x[0-9]+|cute-article-rcp)\/([^/.]*\.[^/]*\/.*)/, "http://$1");
			if (newsrc !== src)
				return newsrc;

			return src.replace(/^([a-z]+:\/\/[^/]*\/)(?:[0-9]+(?:x[0-9]+)?|x[0-9]+)\//, "$1default/");
		}

		if ((domain_nosub === "luxnet.ua" && domain.match(/imagecdn[0-9]*\.luxnet\.ua/)) ||
			domain_nowww === "beztabu.net" ||
			domain_nowww === "34.ua" ||
			domain_nosub === "24tv.ua" ||
			domain_nowww === "football24.ua" ||
			domain_nowww === "maximum.fm" ||
			domain_nowww === "lux.fm") {
			return src.replace(/\/[0-9]+(?:x[0-9]+|[wh])?_DIR\//, "/");
		}

		if (domain_nowww === "userstyles.org") {
			return src.replace("/style_screenshot_thumbnails/", "/style_screenshots/");
		}

		if (domain_nosub === "narvii.com" &&
			domain.match(/^[a-z]+[0-9]*\./)) {
			regex = /(\/[0-9a-z]+(?:-[-0-9a-z]+)?_)[^/.]*(\.[^/.]*)/;
			return [
				src.replace(regex, "$1uhq$2"),
				src.replace(regex, "$1hq$2")
			];
		}

		if (domain === "img.oastatic.com") {
			return src
				.replace(/\/img\/[0-9]+\/[0-9]+(?:\/fit)?\/([0-9]+)\/([^/]*)$/, "/img/$1/$2")
				.replace(/\/img\/([0-9]+)\/([^/]*)$/, "/img2/$1/full/$2")
				.replace(/\/imgmax\/([0-9]+)\/([^/]*)$/, "/img2/$1/full/$2")
				.replace(/(\/img2\/[0-9]+\/)[^/]*\/([^/]*)$/, "$1full/$2")
				.replace(/\/img2\/([0-9]+)\/full\/([^/]*)$/, "/imgsrc/$1/$2");
		}

		if (domain === "img.valais.ch" ||
			domain === "images.weserv.nl" ||
			(domain_nosub === "qiuzhi5.com" && domain.match(/^i[0-9]*\./)) ||
			(domain_nosub === "baaz.com" && domain.match(/^proxy[0-9]*\./)) ||
			(domain_nosub === "powr-media.com" && domain.match(/^p[0-9]*-ia\./)) ||
			domain === "nimg.ws.126.net") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/.*?[?&]url=([^&]*).*/, "$1");
			if (newsrc !== src)
				return add_http(decodeURIComponent(newsrc));
		}

		if (domain_nosub === "etsystatic.com" &&
			(domain.match(/img[0-9]*\.etsystatic\.com/) ||
			 domain === "i.etsystatic.com")) {
			return src.replace(/(\/[a-z]+_)[0-9a-zA-Z]+x[0-9a-zA-Z]+\./, "$1fullxfull.");
		}

		if (domain_nosub === "twnmm.com") {
			return urljoin(src, src.replace(/^[a-z]+:\/\/[^/]*\/thumb.*?[?&]src=([^?&]*).*/, "$1"), true);
		}

		if (domain_nowww === "vuecinemas.nl") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+thumb\?(?:.*&)?src=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return urljoin(src, decodeURIComponent(newsrc), true);
		}

		if (domain === "www.findx.com" &&
			string_indexof(src, "/api/images/assets/") >= 0) {
			return src.replace(/^[a-z]+:\/\/[^/]*\/api\/images\/assets\/[^/]*\//, "");
		}

		if (domain_nowww === "konsolinet.fi") {
			return src.replace(/(:\/\/[^/]*\/[^/]*\/)[0-9]+x[0-9]+\//, "$1");
		}

		if ((domain_nosub === "vigbo.com" ||
			 domain_nosub === "gophotoweb.com") &&
			domain.match(/^static[0-9]*\./)) {
			return src.replace(/\/[0-9]+-((?:[^/]*-)?[a-f0-9]{20,}\.[^/.]*)$/, "/2000-$1");
		}

		if (domain_nosub === "feelway.com" && domain.match(/img[0-9]*.feelway\.com/)) {
			return src.replace(/(\/[0-9]+\/)small([^/]*)$/, "$1$2");
		}

		if (domain_nosub === "pichunter.com") {
			newsrc = src.replace(/(:\/\/[^/]*\/+(?:[0-9]+\/+[0-9]+\/+[0-9]+\/+)?[0-9]+_[0-9]+)(?:_[a-z])?(\.[^/.]*)(?:[?#].*)?$/, "$1_o$2");
			if (newsrc !== src) {
				return {
					url: newsrc,
					headers: {
						Referer: "https://www.pichunter.com/"
					},
					referer_ok: {
						same_domain: true
					}
				};
			}
		}

		if (domain === "appdb.winehq.org") {
			return src.replace(/(\/appimage\.php.*?)([?&])bThumbnail=[^&]*/, "$1$2").replace(/&$/, "");
		}

		if (domain_nosub === "ikea.com" &&
			string_indexof(src, "/images/") >= 0) {
			return src.replace(/([-_][sS])[0-9](\.[^/.]*)/, "$15$2");
		}

		if (domain === "img.onestore.co.kr") {
			return src.replace(/\/[0-9]+_[0-9]+_(?:F[0-9]+_)?[0-9]+\//, "/0_0_100/");
		}

		if (domain_nosub === "ismcdn.jp" ||
			domain === "mikiki.tokyo.jp" ||
			domain === "wedge.ismedia.jp" ||
			domain === "ure.pia.co.jp" ||
			domain_nowww === "afpbb.com") {
			return src.replace(/(\/mwimgs\/+(?:[0-9a-f]\/+){2})[0-9]+(?:[xmhw][wh]?[0-9]*)?(\/+img_[0-9a-f]{20,}\.[^/.]*)(?:[?#].*)?$/, "$1-$2");
		}

		if (domain_nosub === "yomiuri.co.jp" && string_indexof(src, "/photo/") >= 0) {
			return src.replace(/-[A-Z0-9](\.[^/.]*)$/, "-L$1");
		}

		if (domain_nosub === "newswitch.jp" && domain.match(/c[0-9]*\.newswitch\.jp/)) {
			return decodeURIComponent(src.replace(/^[a-z]*:\/\/[^/]*\/cover.*?[?&]url=([^&]*).*?$/, "$1"));
		}

		if (domain_nowww === "fashion-headline.com") {
			return src.replace(/\/api\/+image\/+(?:width|height)\/+[0-9]+\//, "/");
		}

		if (domain_nosub === "stream.ne.jp" && string_indexof(domain, "cdnext.stream.ne.jp") >= 0) {
			return src.replace(/(\/[0-9a-f]+)_[0-9]*_[0-9]*(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nosub === "kn3.net") {
			return src.replace(/\/(?:c_)?[0-9]+x[0-9]+_([0-9A-Z]*\.[^/.]*)$/, "/$1");
		}

		if (domain_nosub === "ptcdn.info") {
			return src.replace(/([-_])[a-z](\.[^/.]*)$/, "$1o$2");
		}

		if (domain_nosub === "pikabu.ru") {
			return src
				.replace(/\/post_img\/([0-9]+)\//, "/post_img/big/$1/")
				.replace(/\/images\/+previews_comm\/+/, "/images/big_size_comm/");
		}

		if (domain_nosub === "podium.life") {
			return src.replace(/\/content\/r\/[wh]?[0-9]*(?:x[0-9]+)?\//, "/content/");
		}

		if (domain_nowww === "lelulove.com") {
			return src.replace(/(\/content\/+photo\/+[0-9a-z]+\/+)thumbs\/+/, "$1full/");
		}

		if (domain_nosub === "filesor.com") {
			return {
				url: src.replace(/_(?:[a-z]|[0-9])(\.[^/.]*)/, "$1"),
				bad_if: [{
					headers: {
						"content-length": "9408",
						"content-type": "image/gif"
					}
				}]
			};
		}

		if (domain_nowww === "namooactors.com") {
			return src.replace(/\/thumb(?:_[^/]*)?\/[a-z]+_[0-9]+(?:px|X[0-9]+)_(.*?\.[^/.]*)\.[^/.]*$/, "/$1");
		}

		if (domain_nowww === "ftopx.com" ||
			domain_nowww === "goodwp.com" ||
			domain_nowww === "kartinkijane.ru" ||
			domain_nowww === "nastol.com.ua" ||
			domain_nowww === "fonstola.ru" ||
			domain_nowww === "luxfon.com" ||
			domain_nowww === "artleo.com") {
			var prefix;

			if (domain_nowww === "ftopx.com") {
				var timestamp = parseInt(src.replace(/.*[/_]([a-f0-9]{10,})\.[^/.]*$/, "$1"), 16);
				prefix = ["ftop.ru", "ftopx.com"];
				if (timestamp > 1579484587755500)
					prefix = ["ftopx.com", "ftop.ru"];
			} else {
				prefix = domain_nosub;
			}

			regex = /\/(?:(?:(?:mini?|large)|pic\/+[0-9]+x[0-9]+)\/+([0-9]+)|pic\/+([0-9]{6})\/+[0-9]+x[0-9]+)\/+(?:[^/]*?\.[a-z]+[-_])?((?:[0-9a-f]+|[0-9]+)\.[^/.]*)(?:[?#].*)?$/;

			if (is_array(prefix)) {
				var urls = [];
				for (i = 0; i < prefix.length; i++) {
					urls.push(src.replace(regex, "/images/$1$2/" + prefix[i] + "_$3"));
				}
				return urls;
			} else {
				return src.replace(regex, "/images/$1$2/" + prefix + "_$3");
			}
		}

		if (domain_nowww === "wykop.pl" && string_indexof(src, "/cdn/") >= 0) {
			return src.replace(/,[^/.]*(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "phoronix.net") {
			return src.replace(/(\/image\.php.*?[?&]image=[^&]*)_med/, "$1");
		}

		if (domain === "image.diyidan.net") {
			return {
				url: src.replace(/!.*/, ""),
				headers: {
					Referer: "https://www.diyidan.com/"
				}
			};
		}

		if (domain === "www.hrkgame.com" &&
			string_indexof(src, "/.thumbnails/") >= 0) {
			return src.replace(/\/\.thumbnails\/([^/]*)\/.*/, "/$1");
		}

		if (domain_nowww === "dlcompare.com" &&
			string_indexof(src, "/upload/cache/") >= 0) {
			return src.replace(/\/upload\/cache\/[^/]*\//, "/");
			/*newsrc = src.replace(/\/upload\/cache\/[^/]*\/upload\//, "/upload/");
			if (newsrc !== src)
				return newsrc;
			return src.replace(/\/upload\/cache\/[^/]*\//, "/upload/cache/slider/");*/
		}

		if (domain_nosub === "alphacoders.com" &&
			(domain.match(/images[0-9]*\.alphacoders\.com/) ||
			 domain === "artfiles.alphacoders.com" ||
			 domain === "picfiles.alphacoders.com" ||
			 domain === "avatarfiles.alphacoders.com" ||
			 domain === "coverfiles.alphacoders.com" ||
			 domain === "photofiles.alphacoders.com")) {
			return src.replace(/\/thumb(?:-[0-9]*)?-([0-9]*\.[^/.]*)$/, "/$1");
		}

		if (domain === "mfiles.alphacoders.com" &&
			options && options.cb && options.do_request) {
			id = src.replace(/.*\/(?:thumb(?:-[0-9]*)?-)?([0-9]+)\.[^/.]*$/, "$1");
			if (id !== src) {
				options.do_request({
					url: "https://mobile.alphacoders.com/wallpapers/view/" + id + "/",
					method: "GET",
					onload: function(resp) {
						if (resp.readyState === 4) {
							var page = resp.finalUrl;
							var match = resp.responseText.match(/"([a-z]+:\/\/wall\.alphacoders\.com\/big\.php\?i=[0-9]+)"/);
							if (match) {
								options.do_request({
									url: match[1],
									method: "GET",
									onload: function(resp) {
										if (resp.readyState === 4) {
											var match = resp.responseText.match(/<meta *property="og:image" *content="([^"]*)"/);
											if (match) {
												options.cb({
													url: match[1],
													is_original: true,
													extra: {
														page: page
													}
												});
											} else {
												options.cb(null);
											}
										}
									}
								});
							} else {
								options.cb(null);
							}
						}
					}
				});

				return {
					waiting: true
				};
			}
		}

		if (domain_nowww === "gpstatic.com") {
			return src.replace(/\/(s[0-9]*)_thumb(-[a-f0-9]*\.[^/.]*)$/, "/$1$2");
		}

		if (domain_nosub === "walldevil.com") {
			return src.replace(/\/(?:thumb|preview)\//, "/");
		}

		if (domain === "www.peency.com" &&
			string_indexof(src, "/images/") >= 0) {
			return src.replace(/_[^/._]*(\.[^/.]*)$/, "$1");
		}

		if (domain === "www.wallpaperbetter.com" &&
			string_indexof(src, "/wallpaper/") >= 0) {
			return src.replace(/-(?:thumb|middle-size)(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "thiswallpaper.com" && string_indexof(src, "/cdn/") >= 0) {
			return src.replace("/cdn/thumb/", "/cdn/hdwallpapers/");
		}

		if (domain_nowww === "superwallpapers.in") {
			return src.replace(/_[0-9]+_[0-9]+_[0-9]+_cf(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "booklikes.com") {
			newsrc = src.replace(/.*\/www\/+thumb\.php\?(?:.*&)?img=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return "http://" + domain + "/" + decodeURIComponent(newsrc);

			return src.replace(/\/photo\/+max\/+[0-9]+\/+[0-9]+\/upload\/+/, "/upload/");
		}

		if ((domain_nowww === "hdwallsource.com" ||
			 domain_nowww === "pickywallpapers.com") &&
			string_indexof(src, "/img/") >= 0) {
			return src.replace(/\/thumb\/+([^/.]*)-thumb(\.[^/.]*)(?:[?#].*)?$/, "/$1$2");
		}

		if ((domain_nowww === "customity.com" && string_indexof(src, "/storage/public/") >= 0) ||
			(domain_nowww === "wisebread.com" && string_indexof(src, "/files/") >= 0)) {
			return src.replace(/\/imagecache\/+[0-9]+x[0-9]+\//, "/");
		}

		if ((domain_nowww === "desktopimages.org" ||
			 domain_nowww === "hdbilder.eu" ||
			 domain_nowww === "hdfondos.eu" ||
			 domain_nowww === "banktapet.pl" ||
			 domain_nowww === "fondsecran.eu")) {
			if (string_indexof(src, "/pictures/") >= 0) {
				newsrc = src.replace(/\/[^/]*[-_]([0-9]+\.[^/.]*)$/, "/orig_$1");
				if (newsrc !== src)
					return newsrc;
			}

			if (options && options.cb && options.do_request &&
				src.match(/\/(?:p|preview)\/+get_photo\/+[0-9]+\/+[0-9]+/)) {
				var querysrc = src.replace(/\/(?:p|preview)\/+get_photo\/+([0-9]+\/+[0-9]+)(?:\/.*)?$/, "/p/$1/0/o");
				options.do_request({
					url: querysrc,
					method: "GET",
					onload: function(resp) {
						if (resp.readyState === 4) {
							var match = resp.responseText.match(/<img *src="([^">]*\/orig_[^/">]*)"/);
							if (match) {
								options.cb(match[1]);
							} else {
								options.cb(null);
							}
						}
					}
				});

				return {
					waiting: true
				};
			}
		}

		if (domain_nowww === "bikerpunks.com" && string_indexof(src, "/media/") >= 0) {
			return src.replace(/\/media\/[^/]*\/([0-9a-f]*\.[^/.]*)$/, "/media/$1");
		}

		if (domain_nosub === "livejournal.com" && domain.match(/i[a-z]*\.pics\.livejournal\.com/)) {
			return src
				.replace(/(\/pic\/[0-9a-z]+\/)s[0-9]+x[0-9]+(?:[?#].*)?$/, "$1")
				.replace(/_[0-9]*(\.[^/.]*)$/, "_original$1")
				.replace(/(\/[0-9]+\/)[0-9]+(\.[^/.]*)$/, "$1original$2");
		}

		if (domain === "pics.livejournal.com") {
			return src.replace(/\/s[0-9]+x[0-9]+(?:[?#].*)?$/, "/original");
		}

		if (domain_nowww === "yastatic.net") {
			if (/\/s3\/+web4static\/+_\/+v2\/+/.test(src))
				return {
					url: src,
					bad: "mask"
				};
		}

		if ((host_domain_nowww === "yandex.ru" ||
			 host_domain_nowww === "yandex.com")
			 && options && options.element) {
			var current = options.element;
			while ((current = current.parentElement)) {
				if (current.tagName === "A") {
					match = current.href.match(/^[a-z]+:\/\/[^/]+\/+images\/+search\?(?:.*&)?img_url=([^&]+)/);
					if (match) {
						newsrc = decodeuri_ifneeded(match[1]);
						if (newsrc !== src)
							return newsrc;
					}

					break;
				}
			}
		}

		if (domain === "img-fotki.yandex.ru") {
			return src.replace(/_[^-/._]*(\.[^/.]*)?$/, "_orig$1");
		}

		if (domain_nosub === "yandex.ru" && /downloader.disk.yandex.ru/.test(domain)) {
			return src.replace(/(\/preview\/+[0-9a-f]{30,}\/+.*?\?(?:.*&)?size=)[0-9]+x[0-9]+(&.*)?$/, "$10x0$2");
		}

		if (domain_nosub === "steemitimages.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/[0-9]+x[0-9]+\//, "");
			if (newsrc !== src)
				return newsrc;
			return src.replace(/\?.*/, "");
		}

		if (amazon_container &&
			amazon_container.match(/^steemit-production-imageproxy-[^-.]*/)) {
			return src
				.replace(/-imageproxy-thumbnail((?:\.[^/]*)?\/[A-Za-z0-9]*)_[0-9]+x[0-9]+$/, "-imageproxy-upload$1");
		}

		if (domain_nowww === "newsprom.ru") {
			return src.replace(/\/([0-9]+_[a-f0-9]+\.[^/.]*)$/, "/tn_$1");
		}

		if (domain_nosub === "fotocdn.net" && domain.match(/i[0-9]*\.fotocdn\.net/)) {
			regex = /_(?:[a-z]+|[0-9]+c)(\/+[0-9]+\/+[0-9]+\.[^/.]*)$/;
			return [
				src.replace(regex, "_xl$1"),
				src.replace(regex, "_l$1")
			];
		}

		if (domain_nowww === "news-people.fr" && string_indexof(src, "/galerie/") >= 0) {
			return src.replace(/\/([0-9]*)(\.[^/.]*)$/, "/$1_hd$2");
		}

		if (domain_nosub === "cdn107.com") {
			return src.replace(/_[a-z]*(\.[^/.]*)$/, "$1");
		}

		if (domain === "hairstyles.thehairstyler.com") {
			return src.replace(/(\/[0-9]*\/)[^/]*\/([^/]*)$/, "$1original/$2");
		}

		if (domain_nowww === "abload.de" ||
			domain_nowww === "wallpaperpulse.com") {
			return src.replace(/\/thumb\//, "/img/");
		}

		if (domain === "assets.capitalfm.com" ||
			domain === "assets.gcstatic.com") {
			return src.replace(/(\/[^/]*-[0-9]{8,})-[^/]*(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "leathercelebrities.com") {
			return decodeURIComponent(src
									  .replace(/^[a-z]+:\/\/[^/]*\/download\.php.*?[?&]file=([^&]*).*?$/, "$1")
									  .replace(/(\/uploads\/[0-9]*\/[^/]*)__thumb(\.[^/.]*)$/, "$1$2"));
		}

		if (domain === "img.cache.vevo.com" ||
			domain === "scache.vevo.com") {
			newsrc = src.replace(/[?#].*$/, "");
			if (newsrc !== src)
				return newsrc;

			return src.replace(/(\/thumb\/[^/]*\/[^/]*)\/[0-9]+x[0-9]+(\.[^/.]*?)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "superiorpics.com" ||
			domain === "i.bollywoodmantra.com") {
			return src.replace(/\/thumb[0-9]+\//, "/");
		}

		if (domain_nowww === "celebjihad.com" ||
			domain === "image.dek-d.com") {
			return src.replace(/\/t_([^/]*)$/, "/$1");
		}


		if (domain === "cdn.teamcococdn.com") {
			return src.replace(/\/image\/[^/]*\//, "/file/");
		}

		if (domain === "live.staticflickr.com") {
			newsrc = src.replace(/(\/buddyicons\/+[0-9]+@N[0-9]+)(?:_[slm])?(\.[^/.]*)(?:[?#].*)?$/, "$1_r$2");
			if (newsrc !== src)
				return newsrc;
		}

		if ((domain_nosub === "staticflickr.com" ||
			 (domain_nosub === "flickr.com" && string_indexof(domain, ".static.flickr.com") >= 0)) &&
			src.match(/\/[0-9]+_[0-9a-f]+(?:_[a-z0-9]*)?\.[a-z]+.*$/) &&
			options && options.do_request && options.cb) {

			function do_flickr_request(url, cb) {
				var headers = {
					"Origin": "https://www.flickr.com",
					"Referer": "https://www.flickr.com/",
					"Sec-Fetch-Site": "same-site",
					"Cookie": ""
				};

				return options.do_request({
					url: url,
					method: "GET",
					headers: headers,
					onload: cb
				});
			}

			function find_api_info(cb) {
				api_cache.fetch("flickr_api_info", cb, function (done) {
					do_flickr_request("https://www.flickr.com/", function (resp) {
						if (resp.readyState === 4) {
							var regex = /root\.YUI_config\.flickr\.api\.site_key *= *['"]([0-9a-f]+)['"] *; */;
							var matchobj = resp.responseText.match(regex);
							if (!matchobj) {
								console_error("Unable to find Flickr API key");
								return done(null, false);
							}

							var key = matchobj[1];
							if (!key || typeof key !== "string") {
								console_error("Unable to find Flickr API key");
								return done(null, false);
							}

							regex = /root\.auth\s*=\s*({.*?});/;
							matchobj = resp.responseText.match(regex);
							var auth = {};
							if (!matchobj) {
								console_error("Unable to find Flickr auth info");
							} else {
								try {
									auth = JSON_parse(matchobj[1]);
								} catch (e) {
									auth = {};
									console_error("Unable to find Flickr auth info");
								}
							}

							var reqid = "";
							matchobj = resp.responseText.match(/root\.reqId\s*=\s*["']([0-9a-f]+)["']\s*;/);
							if (matchobj) {
								reqid = matchobj[1];
							}

							if ("user" in auth && "nsid" in auth.user) {
								auth.nsid = auth.user.nsid;
							}

							var info = {
								key: key,
								nsid: auth.nsid,
								csrf: auth.csrf,
								reqId: reqid
							};

							done(info, 60 * 60);
						}
					});
				});
			}

			function do_flickrapi_request(info, params, cb) {
				var url = "https://api.flickr.com/services/rest?csrf=";

				if (info.csrf) {
					url += info.csrf;
				}

				url += "&api_key=" + info.key + "&format=json&hermes=1&hermesClient=1";

				if (info.nsid) {
					url += "&viewerNSID=" + info.nsid;
				} else {
					url += "&viewerNSID=";
				}

				if (info.reqId) {
					url += "&reqId=" + info.reqId;
				}

				url += "&format=json&nojsoncallback=1&" + params;

				do_flickr_request(url, cb);
			}

			function find_image_info(info, src, cb) {
				var photoid = src.replace(/.*\/([0-9]+)_[^/]*$/, "$1");
				var photosecret = src.replace(/.*\/[0-9]+_([0-9a-z]+)_[^/]*$/, "$1");

				var cache_key = "flickr_image_info:" + photoid + "_" + photosecret;

				api_cache.fetch(cache_key, cb, function(done) {
					var apirequest = "method=flickr.photos.getInfo&photo_id=" + photoid;

					if (photosecret !== "face")
						apirequest += "&secret=" + photosecret;

					do_flickrapi_request(info, apirequest, function (resp) {
						try {
							var out = JSON_parse(resp.responseText);

							if (out.stat === "fail") {
								console_error(out.message);
							}

							var url = out.photo.urls.url[0]._content;
							if (!(/^https?:\/\//.test(url))) {
								url = null;
							}

							var caption = out.photo.title._content;
							if (!caption)
								caption = null;

							var info = {
								page: url,
								caption: caption
							};

							if (url) {
								done(info, 6 * 60 * 60);
							} else {
								done(info, false);
							}
						} catch (e) {
							console_error(e);
							done(null, false);
						}
					});
				});
			}

			newsrc = src.replace(/(:\/\/[^/]*\/(?:[0-9]+\/)?[0-9]+\/[0-9]+_[0-9a-f]+(?:_[a-z])?\.[a-zA-Z0-9]*).*$/, "$1");
			if (newsrc.match(/\/[0-9]+_[0-9a-f]+_o\.[a-z]+.*$/)) {
				var obj = {
					url: src,
					is_original: true
				};

				if (options.force_page) {
					find_api_info(function(info) {
						if (!info) {
							return options.cb(obj);
						}

						find_image_info(info, newsrc, function(info) {
							if (info) {
								obj.extra = info;
							}

							options.cb(obj);
						});
					});

					return {
						waiting: true
					};
				} else {
					return obj;
				}
			}

			function find_larger_image(info, src, cb) {
				var photoid = src.replace(/.*\/([0-9]+)_[^/]*$/, "$1");

				var cache_key = "flickr_larger:" + photoid;
				api_cache.fetch(cache_key, cb, function(done) {
					do_flickrapi_request(info, "method=flickr.photos.getSizes&photo_id=" + photoid, function (resp) {
						if (resp.readyState !== 4)
							return;

						if (resp.status !== 200) {
							console_error(cache_key, resp);
							return done(null, false);
						}

						try {
							var out = JSON_parse(resp.responseText);
							if (_nir_debug_)
								console_log(cache_key, deepcopy(out));

							var largesturl = null;
							var largestsize = 0;
							out.sizes.size.forEach(function (size) {
								var currentsize = parseInt(size.width) * parseInt(size.height);
								if (currentsize > largestsize || size.label === "Original") {
									largestsize = currentsize;
									largesturl = size.source;
								}
							});

							if (typeof largesturl === "string" && /^https?:\/\//.test(largesturl)) {
								done(largesturl, 10 * 60);
							} else {
								done(null, false);
							}
						} catch (e) {
							console_error(cache_key, e);
							return done(null, false);
						}
					});
				});
			}

			find_api_info(function(info) {
				if (!info) {
					return options.cb(null);
				}

				find_larger_image(info, src, function(largesturl) {
					if (!largesturl) {
						return options.cb(null);
					}

					if (options.force_page) {
						find_image_info(info, newsrc, function(info) {
							if (info) {
								var obj = {
									url: largesturl,
									extra: info
								};

								options.cb(obj);
							} else {
								options.cb(largesturl);
							}
						});
					} else {
						options.cb(largesturl);
					}
				});
			});

			return {
				waiting: true
			};
		}

		if (domain === "flimg.flickr.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+\?(?:.*&)?url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "www.imagozone.com" ||
			domain_nowww === "imagozone.ro" ||
			domain_nowww === "ohsohumorous.com" ||
			domain_nowww === "travelseyahat.com" ||
			domain_nowww === "carphotoshow.com" ||
			domain_nowww === "crnobelo.com" ||
			domain_nowww === "myhentaicomics.com" ||
			domain_nowww === "celebritiestown.com") {
			if (/\/modules\/+hover_navigation\/+images\//.test(src))
				return {
					url: src,
					bad: "mask"
				};

			if (!src.match(/\/\.album\.[^/.]*$/)) {
				newsrc = src.replace(/(\/var\/+thumbs\/)/, "/var/resizes/");
				if (newsrc !== src)
					return newsrc;

				return src.replace(/\/var\/(?:resizes|thumbs)\//, "/var/albums/");
			}
		}

		if (domain_nowww === "bjwinslow.com") {
			return src.replace(/(\/albums\/.*)\.sized(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "tunes.zone") {
			var basic = src.replace(/\/[0-9]+x[0-9]+[a-z]?\//, "/full/");
			if (basic !== src) {
				var other = basic.replace(/-[a-z]+-[a-z]+(-[0-9]+)?(\.[^/.]*)$/, "$1$2");
				var other1 = basic.replace(/(\/[0-9]+-+)[^/]*-([0-9]+\.[^/.]*)$/, "$1$2");
				if (basic.match(/\/[0-9]+--/))
					return [other1, basic, other];
				else if (basic.match(/-[0-9]+\.[^/.]*$/))
					return [other, basic, other1];
				else
					return [basic, other, other1];
			}
		}

		if (domain === "sf.co.ua") {
			return src.replace(/\/tn-([0-9]*\.[^/.]*)/, "/wallpaper-$1");
		}

		if (domain_nosub === "zerochan.net" && domain.match(/^s[^.]*.zerochan.net/)) {
			return src
				.replace(/:\/\/s[^.]*.zerochan.net\//, "://static.zerochan.net/")
				.replace(/(:\/\/[^/]*\/[^/]*\.)[0-9]+(\.[0-9]+\.[^/.]*)$/, "$1full$2");
		}

		if (domain === "cdn.donmai.us") {
			return src.replace(/^[a-z]+:\/\/[^/]+\/+(preview|sample)\/+(?:[0-9a-f]{2}\/+){2}([0-9a-f]{10,}\.[^/.]+)(?:[?#].*)?$/, "https://danbooru.donmai.us/data/$1/$2");
		}

		if (domain_nosub === "donmai.us") {
			newsrc = src
				.replace(/\/data\/sample\/([^/]*__)?sample-([0-9a-f]*\.[^/.]*)$/, "/data/$1$2")
				.replace(/:\/\/danbooru\.donmai\.us\/data\/preview\//, "://hijiribe.donmai.us/data/")
				.replace(/\/data\/preview\/([^/]*)$/, "/data/__original__$1");

			if (newsrc !== src)
				return add_extensions(newsrc);
		}

		if ((domain_nosub === "e621.net" &&
			 domain.match(/static[0-9]*\.e621\.net/)) ||
			domain_nowww === "behoimi.org") {
			newsrc = src.replace(/(:\/\/[^/]*\/)data\/(?:preview|sample)\//, "$1data/");
			if (newsrc !== src) {
				return add_extensions(newsrc);
			}
		}

		if (domain_nowww === "hypnohub.net") {
			newsrc = src.replace(/\/data\/(?:preview|sample)\/([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/, "/data/image/$1");
			if (newsrc !== src) {
				return add_extensions(newsrc);
			}
		}

		if (domain === "files.yande.re") {
			return add_extensions(src
								  .replace(/\/(?:sample|jpeg)\/+([0-9a-f]+\/)/, "/image/$1"));
		}

		if (domain === "assets.yande.re") {
			return add_extensions(src.replace(/:\/\/assets.yande.re\/data\/preview\/[0-9a-f]+\/[0-9a-f]+\//, "://files.yande.re/image/"));
		}


		if (domain_nowww === "konachan.com" ||
			domain_nowww === "konachan.net") {
			newsrc = src.replace(/(?:\/data\/preview\/[0-9a-f]+\/[0-9a-f]+\/|\/(?:sample|jpeg)\/)([0-9a-f]+)(\/[^/]*)?(\.[^/.?]*)(?:[?#]*)?$/, "/image/$1$3");
			if (newsrc !== src)
				return add_full_extensions(newsrc);

			return {
				url: src,
				headers: {
					Referer: "https://konachan.com/post"
				}
			};
		}

		if (domain_nowww === "gelbooru.com") {
			return src.replace(/:\/\/[^/]*\/thumbnails\//, "://img2.gelbooru.com/thumbnails/");
		}

		if (domain_nosub === "gelbooru.com" &&
			domain.match(/^s?img[0-9]*\.gelbooru\.com/)) {
			obj = {
				url: src,
				headers: {
					Referer: ""
				},
				referer_ok: {
					same_domain_nosub: true
				}
			};

			newsrc = src.replace(/\/+thumbnails\/([0-9a-f]+\/[0-9a-f]+\/)thumbnail_/, "/images/$1")
						.replace(/\/+samples\/([0-9a-f]+\/[0-9a-f]+\/)sample_/, "/images/$1");
			if (newsrc !== src) {
				return fillobj_urls(add_full_extensions(newsrc), obj);
			}
		}

		if (domain_nowww === "safebooru.org" ||
			domain_nowww === "tbib.org" ||
			domain === "img.xbooru.com" ||
			domain === "img.booru.org" ||
			domain_nosub === "rule34.xxx" ||
			domain_nowww === "realbooru.com") {
			newsrc = src.replace(/\/(?:thumbnails|samples)(\/+[0-9]+\/+)(?:thumbnail|sample)_([0-9a-f]+\.[^/.]*)$/, "/images$1$2");
			if (newsrc !== src)
				return add_full_extensions(newsrc);
		}

		if (domain === "thumbs.booru.org") {
			return src.replace(/:\/\/[^/]*\//, "://img.booru.org/");
		}

		if (domain_nosub === "booru.org" && domain !== "img.booru.org") {
			return src.replace(/:\/\/([^/]*)\.booru\.org\/thumbnails\/+/, "://img.booru.org/$1/thumbnails/");
		}

		if (domain === "cdn.vor.us") {
			return src.replace(/\/thumbs[a-z]?\//, "/og/");
		}

		if (domain === "wallpapers.wallhaven.cc" ||
			domain === "alpha.wallhaven.cc") {
			return src.replace(/\/thumb\/[^/]*\/th-/, "/full/wallhaven-");
		}

		if (domain === "cdn.animenewsnetwork.com" ||
			domain_nowww === "animenewsnetwork.com") {
			return src.replace(/\/thumbnails\/[^/]*\/cms\//, "/images/cms/");
		}

		if ((domain_nowww === "digitalart.io" && string_indexof(src, "/storage/") >= 0) ||
			domain_nowww === "fashion-press.net" && string_indexof(src, "/img/") >= 0) {
			return src.replace(/(\/[0-9]*\/)[wh][0-9]+_([^/]*)$/, "$1$2");
		}

		if (domain_nowww === "4everstatic.com" ||
			domain === "pictures.4ever.eu") {

			if (!src.match(/:\/\/[^/]*\/data\/+download\//)) {
				return [
					src.replace(/:\/\/[^/]*\/[a-z]+\/(?:[0-9X]+x[0-9X]+\/)?([^?]*).*?$/, "://pictures.4ever.eu/data/download/$1?no-logo"),

					src.replace(/(:\/\/[^/]*\/[a-z]+\/)[0-9X]+x[0-9X]+\//, "$1")
				];
			}
		}

		if (domain_nowww === "tapeciarnia.pl") {
			return src.replace(/\/tapety\/[^/]*\//, "/tapety/normalne/");
		}

		if (domain_nowww === "zastavki.com") {
			return src.replace(/\/pictures\/[0-9]+x[0-9]+\/(.*_[0-9]+)_[0-9]+(\.[^/.]*)$/, "/pictures/originals/$1_$2");
		}

		if (domain_nosub === "wallls.com" && domain.match(/w[0-9]*\.wallls.com/)) {
			return src.replace(/\/uploads\/[^/]*\/([0-9]*\/[0-9]*\/)([0-9]*\.[^/.]*)$/, "/uploads/original/$1wallls.com_$2");
		}

		if (domain === "content.hardtunes.com") {
			return src.replace(/\/[0-9]+x[0-9]+(\.[^/.]*)$/, "/original$1");
		}

		if (domain_nowww === "4kw.in" &&
			string_indexof(src, "/Wallpapers/") >= 0) {
			return src.replace(/1(\.[^/.]*)$/, "$1");
		}

		if (domain === "imgs-art-dragoart-386112.c.cdn77.org") {
			return src.replace(/_[0-9]*(\.[^/.]*)$/, "_1$1");
		}

		if (domain_nosub === "nyafuu.org" && domain.match(/archive-media-[0-9]*\.nyafuu\.org/)) {
			return src
				.replace(/:\/\/archive-media-[0-9]*\./, "://archive-media-0.")
				.replace(/\/thumb\//, "/image/")
				.replace(/(\/[0-9]*)[a-z](\.[^/.]*)$/, "$1$2");
		}

		if (domain_nosub === "hulkshare.com" && domain.match(/s[0-9]*\.hulkshare\.com/)) {
			return src.replace(/\/[0-9]*\/([0-9a-f]\/[0-9a-f]\/[0-9a-f]\/[0-9a-f]*\.[^/.]*)$/, "/original/$1");
		}

		if (domain_nowww === "wikihow.com" &&
			string_indexof(src, "/images/") >= 0) {
			return src.replace(/\/thumb\/(.*?\.[^/.]*)(?:\/.*)/, "/$1");
		}

		if (domain_nosub === "kakaocdn.net" ||
			domain_nosub === "kakao.co.kr") {
			return src
				.replace(/(\/img(?:_[a-z]+)?\.[^/.?#]*)(?:[?#].*)?$/, "$1")
				.replace(/\/img_[a-z]+(\.[^/.]*)$/, "/img$1");
		}

		if (domain === "obs.line-scdn.net") {
			return {
				can_head: false,
				url: src.replace(/(:\/\/[^/]*\/[-_0-9A-Za-z]*)\/[a-z0-9]*$/, "$1")
			};
		}

		if (domain === "scdn.line-apps.com") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/(?:obs\/)?([-_0-9A-Za-z]*)(?:\/[a-z0-9]*)?$/, "https://obs.line-scdn.net/$1");
		}

		if (domain === "cdn-obs.line-apps.com") {
			return src.replace(/(\/[-0-9A-F]+\.[0-9a-z]+)\/[^/]*$/, "$1");
		}

		if (domain === "resize-image.lineblog.me") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/[0-9a-f]*\/.*?\/([a-z]+:\/\/.*)/, "$1");
		}

		if (domain === "stickershop.line-scdn.net") {
			return src.replace(/;[a-z]+=[^/]*(?:[?#].*)?$/, "");
		}

		if (domain === "d.line-scdn.net") {
			return src.replace(/[?].*/, "");
		}

		if (domain_nosub === "viki.io" && domain.match(/^[0-9]*\.viki\.io/)) {
			return {
				can_head: false,
				url: src.replace(/\?.*/, "")
			};
		}

		if (domain_nosub === "crunchyroll.com" && domain.match(/img[0-9]*\.[^/.]*\.crunchyroll\.com/)) {
			return src.replace(/(\/[0-9a-f]+)_[a-z]*(\.[^/.]*)$/, "$1_full$2");
		}

		if (domain === "d3ieicw58ybon5.cloudfront.net") {
			return {
				url: src
					.replace(/\/resize\/[0-9]+\//, "/full/")
					.replace(/\/ex\/[0-9]+\.[0-9]+\/(?:(?:[0-9]+\.){3}[0-9]+\/)?/, "/full/"),
				can_head: false
			};
		}

		if (domain === "az616578.vo.msecnd.net") {
			return src.replace(/\/files\/responsive\/[^/]*\/[^/]*\/[^/]*\//, "/files/");
		}

		if ((domain_nosub === "dreamwiz.net" && domain.match(/img[0-9]*\.dreamwiz\.net/)) ||
			domain === "kep.cdn.index.hu" ||
			domain === "kep.index.hu" ||
			domain === "kep.cdn.indexvas.hu") {
			return src.replace(/_(?:[a-z]|wm)(\.[^/.]*)$/, "_o$1");
		}

		if ((domain_nosub === "sportsworldi.com" ||
			 domain_nosub === "segye.com") &&
			string_indexof(src, "/content/image/") >= 0) {


			return src.replace(/(\/content\/+image\/+[0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+)[0-9]+\/+([0-9]+(?:_[^/.]*)?\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nowww === "maximkorea.net") {
			return src.replace(/(\/[0-9]+_[0-9]+(?:_[0-9]+)?_img)_[0-9]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "blogimgc.eximg.jp") {
			return decodeURIComponent(decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/.*?i=([^,]*).*?$/, "$1")));
		}

		if (domain === "i.gzn.jp" ||
			(domain_nosub === "uludagsozluk.com" && domain.match(/^galeri[0-9]*\./))) {
			return src.replace(/_[a-z](\.[^/.]*)$/, "$1");
		}

		if (domain_nosub === "okmusic.jp" ||
			domain === "newsimg.music-book.jp") {
			return src.replace(/(\/images\/[0-9]*\/)[^/.]*(\.[^/.]*)$/, "$1original$2");
		}

		if (domain_nowww === "kai-you.net") {
			return src
				.replace(/\/r\/img\/[a-z]\/[a-z]?[0-9]+x(?:[0-9]+)?\//, "/press/img/")
				.replace(/(\/images\/.*)\/[a-z]?[0-9]+x(?:[0-9]+)?\/([^/]*)$/, "$1/$2");
		}

		if (domain === "cdn.fortune-girl.com") {
			return src.replace(/(:\/\/[^/]*\/)[a-z]*\/([-0-9a-f]*\.[^/.]*)$/, "$1original/$2");
		}

		if (domain === "storage.withnews.jp" ||
			amazon_container === "storage.withnews.jp") {
			return src.replace(/-[a-z]+(\.[^/.]*)$/, "$1");
		}

		if (domain_nosub === "wowma.net" && domain.match(/ic[0-9]*-a\.wowma\.net/)) {
			return src.replace(/^[a-z]*:\/\/[^/]*\/mis?\/.*?\/([^/]*\.[^/]*\/.*)/, "http://$1");
		}

		if (domain_nowww === "sokuup.net") {
			return src.replace(/\/img[a-z]\//, "/img/");
		}

		if (domain_nosub === "cosp.jp" && domain.match(/image[0-9]\.cosp\.jp/)) {
			return src
				.replace(/\/thumb\/(.*?\/[0-9]+)[a-z]\.(?:gif|jpg)$/, "/images/$1.jpg")
				.replace(/(\/images\/.*\/[0-9]+)_[0-9]+(\.[^/.]*)$/, "$1$2")
				.replace(/(\/[0-9]+)[a-z](\.[^/.]*)/, "$1$2");
		}

		if (domain_nowww === "gahag.net") {
			return {
				url: src.replace(/:\/\/[^/]*\/img\/([^/]*)\/([0-9]+)[a-z]\/([^-/]+-[0-9]+)-[0-9](\.[^/.]*)/,
								 "://img01.gahag.net/$1/$2o/$3$4"),
				headers: {
					Referer: "http://gahag.net/",
				}
			};
		}

		if (domain === "img-cdn.jg.jugem.jp") {
			return src.replace(/_[a-z](\.[^/.]*)$/, "$1");
		}

		if (domain === "public.muragon.com") {
			return src.replace(/\/(?:crop|resize)\/[0-9]+x[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain === "base-ec2if.akamaized.net") {
			return src.replace(/(:\/\/[^/]*\/)[^/]*[a-z]=[0-9][^/]*\//, "$1");
		}

		if (domain_nosub === "imageflux.jp") {
			return src.replace(/\/c!\/[^/]*[a-z]=[0-9][^/]*\//, "/");
		}

		if (domain_nosub === "bloguru.com" &&
			string_indexof(src, "/userdata/") >= 0) {
			return src.replace(/\/([^/_]*)$/, "/orig_$1");
		}

		if (domain === "www.atpress.ne.jp") {
			return src.replace(/(\/[0-9]+\/)[a-z]+_(img_[0-9]+[^/]*\.[^/.]*)$/, "$1$2");
		}

		if (domain === "img-proxy.blog-video.jp" ||
			domain === "api.pddataservices.com") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/images.*?[?&]url=([^&]*).*?$/, "$1"));
		}

		if (domain === "cdn.otamart.com") {
			return src.replace(/-thumbnail(\.[^/.]*)$/, "$1");
		}

		if (domain === "www.pakutaso.com") {
			return src.replace(/(\/img\/thumb\/[^/_.]*)_[^/.]*(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "cdn.atwiki.jp" ||
			(domain === "kinohron.mskcentrum.sk" && string_indexof(src, "/data-files/") >= 0) ||
			(domain_nowww === "neosite.pl" && string_indexof(src, "/upload/user/"))) {
			return src.replace(/\/small_([^/]*)$/, "/$1");
		}

		if (domain === "getfile.fmkorea.com") {
			return decodeURIComponent(src.replace(/.*?\/getfile\.php.*?[?&]file=([^&]*).*/, "$1"));
		}

		if (domain === "img.ruliweb.com" ||
			(domain_nosub === "dtiblog.com" && domain.match(/[0-9]*\.dtiblog\.com/)) ||
			domain === "img.korewaeroi.com" ||
			domain === "abc.imgxyqpdrs.xyz" ||
			(domain_nowww === "koreanartist.jp" && string_indexof(src, "/img/") >= 0 && src.match(/\/[0-9]+s\./)) ||
			domain === "image-bankingf25.com") {
			return src.replace(/s(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain === "img2.ruliweb.com") {
			return src.replace(/(\/mypi\/gup\/a\/[0-9]+\/[0-9]+\/)[a-z]+(\/[0-9]+\.[^/.]*)$/, "$1o$2");
		}

		if (domain === "file.thisisgame.com") {
			return src.replace(/\/s_([0-9]+_[0-9]+\.[^/.]*)$/, "/$1");
		}

		if (domain === "www.op.gg") {
			return src.replace(/^.*?\/forum\/outImage\/(http.*)$/, "$1");
		}

		if (domain === "ssproxy.ucloudbiz.olleh.com" ||
			domain === "s.gae9.com") {
			return src.replace(/\.[a-z]*(?:[?#].*)?$/, ".orig");
		}

		if (domain_nosub === "artstation.com" && domain.match(/cdn[a-z]*\.artstation\.com/)) {
			regex = /(\/assets\/+images\/+images\/+[0-9]{3}\/+[0-9]{3}\/+[0-9]{3}\/+)(?:[0-9]+\/+)?(?:small(?:er)?|micro|medium|large)(?:_square)?\/([^/]*)$/;
			return [
				src.replace(regex, "$1original/$2"),
				src.replace(regex, "$1large/$2")
			];
		}

		if (domain === "static.cosplay-it.com") {
			return src.replace(/(\/[-0-9a-f]*)_[a-z]+(\.[^/.]*)/, "$1$2");
		}

		if (domain === "a.fsdn.com") {
			return src.replace(/(\/screenshots\/[^/]*)\/[0-9]+\/.*/, "$1");
		}

		if (domain === "media.moddb.com" ||
			domain === "cdn.dbolical.com" ||
			domain === "media.indiedb.com") {
			newsrc = src.replace(/\/cache\/+images\/(.*?)\/[a-z]+_[0-9]+[^/]*(\/[^/]*\.[^/.]*)$/, "/images/$1$2");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/\/images\/+(.*?)\.mp4\.jpg(?:[?#].*)?$/, "/videos/$1.mp4");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "static.gamefront.com" ||
			domain_nowww === "wallpaperawesome.com" ||
			(domain_nowww === "ukstockingsex.com" && string_indexof(src, "/galleries/") >= 0) ||
			domain === "cg.adultwork.com") {
			return src.replace("/thumbnails/", "/");
		}

		if (domain === "thumb.test.mod.io" ||
			domain === "thumb.mod.io") {
			return src.replace(/:\/\/thumb\.([^/]*\..*?\/)[a-z]+_[0-9]+[^/]*\/([^/]*)$/, "://image.$1$2");
		}

		if (domain_nosub === "scirra.net" && domain.match(/static[0-9]*\.scirra\.net/)) {
			return src.replace(/\/avatars\/[0-9]+\//, "/avatars/256/");
		}

		if (domain === "cdn.gamer-network.net" ||
			domain === "d2skuhm0vrry40.cloudfront.net") {
			return src.replace(/(\.[^/.]*)\/EG[0-9]+\/.*/, "$1");
		}

		if (domain_nosub === "wikiart.org" && domain.match(/uploads[0-9]*\.wikiart\.org/)) {
			return src.replace(/![^/]*$/, "");
		}

		if ((domain_nosub === "fdncms.com" ||
			 domain_nosub === "miaminewtimes.com" ||
			 domain_nosub === "houstonpress.com" ||
			 domain_nosub === "laweekly.com" ||
			 domain_nosub === "phoenixnewtimes.com") &&
			domain.match(/^(?:images|media)[0-9]*\./)) {
			newsrc = src.replace(/\/imager\/u\/[^/]*\/([0-9]+\/[^/]*)(?:\/.*)?/, "/imager/u/original/$1");
			if (newsrc !== src)
				return newsrc;

			if (src.match(/\/imager\/u\/[^/]*\/[0-9]+\/https?_3[aA]/)) {
				newsrc = src
					.replace(/.*?\/imager\/u\/[^/]*\/[0-9]+\/(https?_3[aA].*\.[^/._]*)(?:_[^/]*)?$/, "$1")
					.replace(/_([0-9a-fA-F][0-9a-fA-F])/g, "%$1");
				newsrc = decodeURIComponent(newsrc);
				return newsrc;
			}
		}

		if (domain === "images.techhive.com" ||
			domain === "images.idgesg.net") {
			return src.replace(/-[a-z]+(?:\.[^/.]*)?(\.[^/.]*)$/, "-orig$1");
		}

		if (domain_nosub === "rimg.com.tw" ||
			domain_nosub === "rimg.tw" ||
			domain === "photo.roodo.com" ||
			domain === "img.ruten.com.tw") {

			if (src.match(/(\/photos\/[0-9]+_[0-9a-z]+_)[a-z](\.[^/.]*)$/)) {
				return src.replace(/(\/photos\/[0-9]+_[0-9a-z]+_)[a-z](\.[^/.]*)$/, "$1o$2");
			}

			return {
				url: src.replace(/_[a-z](\.[^/.]*)$/, "$1"),
				headers: {
					Referer: "https://goods.ruten.com.tw/item/show"
				},
				referer_ok: {
					same_domain: true
				}
			};
		}

		if (domain === "buy.line-scdn.net") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/[a-f0-9]+\/s\//, "https://s.yimg.com/");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nosub === "hc360.cn" && domain.match(/img[0-9]*\.hc360\.cn/)) {
			return src.replace(/(\.[^/.]*)\.[^/]*$/, "$1");
		}

		if (domain_nosub === "ganref.jp" && domain.match(/photo[0-9]*\.ganref\.jp/)) {
			return src.replace(/(\/[0-9a-f]+\/)thumb[0-9]*(\.[^/.]*)$/, "$1original$2");
		}

		if (domain === "img.cdn.nimg.jp") {
			return src.replace(/:\/\/img\.cdn\.nimg\.jp\/s\/(.*?\/images\/+[0-9]+\/+(?:[0-9a-f]+|[^/.]*\.[^/.]*))\/.*/,
							   "://dcdn.cdn.nimg.jp/$1");
		}

		if (domain_nosub === "photozou.jp") {
			return src
				.replace(/(\/photo\/[0-9]+\/org\.bin)\?.*$/, "$1")
				.replace(/(\/photo\/[0-9]+)_[a-z0-9]+(\.[^/]*)$/, "$1_org$2");
		}

		if (domain === "desktop.sakura.ne.jp" ||
			domain === "kysiamesecat.sakura.ne.jp") {
			return src.replace(/-thumbnail[0-9]*(\.[^/.]*)$/, "$1");
		}

		if ((domain === "www.ya.sakura.ne.jp" || domain === "mabi4751.orz.hm") &&
			string_indexof(src, "/~mabi/") >= 0) {
			return src.replace(/s(\.jpg|\.JPG)$/, "$1");
		}

		if (domain === "waganeko.sakura.ne.jp") {
			return src.replace(/\/\.thumbnail\/([^/]*\.[^/.]*)\.[^/.]*$/, "/$1");
		}

		if (domain_nosub === "storage-yahoo.jp") {
			return src.replace(/_(?:m|thumb)(\?.*)?$/, "$1");
		}

		if (domain === "static-mercari-jp-imgtr2.akamaized.net" ||
			domain === "mercari-images.global.ssl.fastly.net") {
			return src
				.replace(/\/thumb\/+(photos|members)\/+/, "/$1/")
				.replace(/(\/(?:photos|members)\/.*?)(?:[?#].*)$/, "$1");
		}



		if (domain === "waichi.sakura.ne.jp") {
			return src.replace(/(\/[^/_.]*)(\.(?:jpg|JPG|jpeg|png|PNG))$/, "$1_L$2");
		}

		if (domain === "news.merumo.ne.jp") {
			return src
				.replace(/\/imgs\/+rect[0-9]*\//, "/imgs/src/")
				.replace(/\/(?:article)?img\/+([0-9]+)(?:\/+[0-9]+)?(?:[?#].*)?$/, "/imgs/src/$1");
		}

		if ((domain_nosub === "shikimori.org" ||
			domain_nosub === "shikimori.one") &&
			string_indexof(src, "/system/") >= 0) {
			return src.replace(/\/[a-z]+\/+((?:[0-9]+\/+)?[0-9]+\.[^/.]*)$/, "/original/$1");
		}

		if (domain === "assets.survivalinternational.org") {
			return src.replace(/_[0-9a-z_]+(\.[^/.]*)$/, "_original$1");
		}

		if (domain === "alioss.g-cores.com" &&
			string_indexof(src, "/uploads/image/") >= 0) {
			return src.replace(/_watermark(\.[^/.]*)$/, "$1");
		}

		if (domain === "img.nyato.com" ||
			domain === "img.ifun01.com" ||
			domain === "img.itw01.com" ||
			domain === "img.mttmp.com" ||
			domain === "img.maoduoer.com" ||
			domain === "img.sycs.net" ||
			domain === "img.6jyx.com" ||
			domain === "dn-kdt-img.qbox.me" ||
			domain === "pic-cdn.35pic.com" ||
			domain === "images.sharerails.com" ||
			domain === "oi5mmhyk8.qnssl.com" ||
			domain === "pic.cr173.com" ||
			(domain_nosub === "5ceimg.com" && /^i[0-9]*\./.test(domain)) ||
			domain === "movieboo.oss-cn-hangzhou.aliyuncs.com" ||
			domain === "cover.read.duokan.com" ||
			domain === "img.mgpyh.com") {
			newsrc = src.replace(/!.*/, "");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nosub === "90sjimg.com" ||
			domain === "bpic.588ku.com" ||
			domain === "588ku.qiao88.com" ||
			domain === "static.pptstore.net" ||
			domain === "img.lovepik.com") {
			newsrc = src.replace(/(?:!|%21)\/.*/, "");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nosub === "woyaogexing.com" && domain.match(/^img[0-9]*\./)) {
			return src.replace(/(\/[0-9a-z]+)![^/]*(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "inews.gtimg.com" ||
			domain === "pnewsapp.tc.qq.com") {
			return {
				url: src.replace(/\/newsapp_[a-z]+\/+([0-9]+\/+[0-9]+\/+)[0-9]+(?:\/+[0-9]+)?(?:[?#].*)?$/, "/newsapp_match/$10"),
				headers: {
					"Origin": "",
					"Referer": ""
				}
			};
		}

		if ((domain === "acg.ms" ||
			 domain === "m.acg.ms") &&
			string_indexof(src, "/photo/") >= 0) {
			return {
				redirects: true,
				url: src.replace(/(\/[0-9]+)_[0-9]+_[0-9]+(\.[^/.]*)$/, "$1_0_9999999$2")
			};
		}

		if (domain === "sl.news.livedoor.com") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/[a-f0-9]+\/[^/]*\//, "");
		}

		if (domain === "photo.tuchong.com") {
			return src.replace(/\/[a-z]\/([0-9]+\.[^/.]*)/, "/f/$1");
		}

		if ((domain === "arine.akamaized.net" ||
			 domain === "media-assets.aumo.jp") &&
			string_indexof(src, "/uploads/photo/") >= 0) {
			return src.replace(/(\/[0-9]+\/)[a-z]+_([-0-9a-z]+\.[^/.]*)$/, "$1$2");
		}

		if (domain === "newsimg.glossom.jp") {
			return src.replace(/(\/[0-9]+_)[0-9]+(_[0-9]+\.[^/.]*)$/, "$1org$2");
		}

		if (domain === "www.vtianxia.cn") {
			return src.replace(/(\/uploadfile\/[a-z]+\/)[a-z]+\//, "$1big/");
		}

		if (domain === "img.over-blog-kiwi.com" ||
			domain === "img.over-blog.com" ||
			(domain_nosub === "over-blog.com" && string_indexof(domain, ".idata.over-blog.com") >= 0)) {
			return src.replace(/(:\/\/[^/]*\/)[0-9]+x[0-9]+(?:-[a-z]+)?\//, "$1");
		}

		if (domain === "resize.over-blog.com") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/[0-9]+x[0-9]+(?:-[a-z]+)?\.[^/.?]*\?/, "");
		}

		if (domain_nosub === "eastday.com" && domain.match(/[0-9]*\.?imgmini\.eastday\.com/)) {
			return src
				.replace(/(\/[0-9]+_[0-9a-f]+_[0-9]+)_[^/.]*(\.[^/.]*)(?:[?#].*)?$/, "$1$2")
				.replace(/_wmk(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain_nosub === "so-net.ne.jp") {
			return src.replace(/\/(?:m|S[0-9]+)_([^/]*)$/, "/$1");
		}

		if (amazon_container === "rejob-v2-images-production" ||
			(amazon_container === "mosaia" && src.match(/\/images\/[0-9]+\/[^/]*$/))) {
			return src.replace(/\/[^/.]*(\.[^/.]*)$/, "/original$1");
		}

		if (domain === "lohas.nicoseiga.jp") {
			newsrc = src.replace(/(\/thumb\/+[0-9]+)[a-z][a-z]?(\?.*)?$/, "$1l$2");
			if (newsrc !== src)
				return newsrc;

			var get_nicovideo_obj = function(id) {
				return {
					url: src,
					extra: {page: "https://seiga.nicovideo.jp/seiga/im" + id}
				};
			};

			obj = null;
			match = src.match(/\/thumb\/+([0-9]+)(?:[a-z]+)(?:[?#].*)?$/);
			if (match) {
				id = match[1];
				obj = get_nicovideo_obj(id);
			}

			if (match && options.do_request && options.cb) {
				var cache_key = "nicoseiga:" + id;
				api_cache.fetch(cache_key, options.cb, function(done) {
					options.do_request({
						url: "https://seiga.nicovideo.jp/image/source/" + id,
						method: "GET",
						onload: function(resp) {
							if (resp.readyState !== 4)
								return;

							if (resp.status !== 200) {
								console_error(resp)
								return done(null, false);
							}

							var headers = headers_list_to_dict(parse_headers(resp.responseHeaders));
							if ("content-type" in headers) {
								if (/^image\//.test(headers["content-type"])) {
									return done(resp.finalUrl, 6*60*60);
								}
							}

							var match = resp.responseText.match(/<div\s+class=\"illust_view_big\"[^>]+data-src=\"([^"]+)\"/);
							if (!match) {
								console_error("Unable to find match (are you logged in to nicovideo?)", resp);
								return done(null, false);
							}

							var url = decode_entities(match[1]);
							obj.url = urljoin(resp.finalUrl, url, true);
							done(obj, 6*60*60);
						}
					});
				});

				return {
					waiting: true
				};
			}

			if (!match) {
				match = src.match(/\/priv\/+[0-9a-f]{10,}\/+[0-9]+\/+([0-9]+)(?:[?#].*)?$/);
				if (match) {
					id = match[1];
					obj = get_nicovideo_obj(id);
				}
			}

			if (obj) {
				return obj;
			}
		}

		if (domain === "appdata.hungryapp.co.kr") {
			return src
				.replace(/\/data_img_[a-z]\//, "/data_img/")
				.replace(/(\/[0-9]{6}\/[0-9]{2}\/[^/]*\.[^/.]*)\/hungryapp\/.*/, "$1");
		}

		if (domain === "media.funradio.fr") {
			return src.replace(/\/cache\/[0-9a-zA-Z]+\/[0-9a-z]+-[0-9]+\//, "/");
		}

		if (domain === "image.afcdn.com" ||
			domain === "assets.afcdn.com" ||
			domain === "imalbum.aufeminin.com" ||
			domain === "imworld.aufeminin.com") {
			return {
				url: src
					.replace(/(\/acc[0-9]+_(?:[a-z]+_)?[0-9]+\/+[a-z0-9]+)_[a-z0-9]+(\.[^/.]*)/, "$1$2")
					.replace(/(-[0-9]+_[A-Z0-9]+_)[A-Z](?:_[a-z[0-9]+)?(\.[^/.]*)/, "$1L$2")
					.replace(/([-/](?:[a-z]+)?[0-9]+)_[a-z0-9]+(\.[^/.]*)/, "$1$2"),
				headers: {
					Origin: "https://www.sofeminine.co.uk/",
					Referer: "https://www.sofeminine.co.uk/"
				}
			};
		}

		if ((domain_nosub === "greatsong.net" && domain.match(/static[0-9]*\.greatsong\.net/)) ||
			(domain_nosub === "china.com" && domain.match(/img[0-9]*\.(?:[a-z]+\.)?china\.com/))) {
			return src.replace(/\/[0-9]+x[0-9]+\//, "/original/");
		}

		if (domain === "ipravda.sk") {
			return src.replace(/\/thumbs\/([^/]*)-(?:stvorec|nestandard[0-9]*|galeria|clanokW?|strednaW?|malaW?)(\.[^/.]*)$/, "/$1$2");
		}

		if (domain === "d919ce141ef35c47fc40-b9166a60eccf0f83d2d9c63fa65b9129.ssl.cf5.rackcdn.com" ||
			domain === "media.phillyvoice.com") {
			return src.replace(/(\/images\/[^/.]+\.)[^/]*(\.[^/.]*)$/, "$1original$2");
		}

		if (domain === "nation.com.pk") {
			return src.replace(/\/print_images\/[a-z]+\//, "/print_images/large/");
		}

		if (domain === "dazedimg.dazedgroup.netdna-cdn.com" ||
			domain === "dazedimg-dazedgroup.netdna-ssl.com") {
			return src.replace(/(:\/\/[^/]*\/)[0-9]+\/(?:[0-9]+-[0-9]+-[0-9]+-[0-9]+\/)?/, "$1");
		}

		if (domain === "www.fuse.tv") {
			return src.replace(/(\/image\/[0-9a-f]+\/)[0-9]+\/[0-9]+\/([^/]*)$/, "$1$2");
		}

		if (domain === "assets.capitalxtra.com") {
			return src.replace(/(\/[^/.]*-[0-9]{8,})-[^/.]*(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "ukmix.org") {
			match = src.match(/\/proxy\.php\?(?:.*&)?url=([^&]+)/);
			if (match) {
				return base64_decode(decodeURIComponent(match[1]));
			}
		}

		if (domain === "www.washingtonpost.com" &&
			string_indexof(src, "/pbox.php?") >= 0) {
			newsrc = src.replace(/.*?\/pbox\.php.*?[?&]url=([^&]*).*$/, "$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "m.static.newsvine.com" &&
			string_indexof(src, "/servista/imagesizer?") >= 0) {
			return src.replace(/(\/servista\/imagesizer).*?[?&](file=[^&]*).*/, "$1?$2");
		}

		if (domain_nosub === "cosplaywon.com" && domain.match(/theprecious[0-9]*\.cosplaywon\.com/)) {
			return src.replace(/(\/photo\/[0-9]+\/)[a-z]+_([-0-9a-f]+\.[^/.]*)$/, "$1original_$2");
		}

		if (domain_nowww === "otaku.com") {
			return src.replace(/(\/files\/images\/)[^/]*\/(?:[A-Z]+_)?([^/]*)$/, "$1fullsize/$2");
		}

		if ((domain_nosub === "joyreactor.com" ||
			 domain_nosub === "reactor.cc" ||
			 domain_nosub === "joyreactor.cc") &&
			domain.match(/^img[0-9]*\./)) {
			newsrc = src.replace(/(\/pics\/post\/)([^/]*)$/, "$1full/$2");
			if (newsrc !== src)
				return newsrc;

			id = src.replace(/.*\/pics\/thumbnail\/post-([0-9]+)\.[^/.]*$/, "$1");
			if (id !== src && options && options.cb && options.do_request) {
				options.do_request({
					url: src.replace(/:\/\/img[0-9]*\.([^/]*\/).*/, "://$1post/" + id),
					method: "GET",
					onload: function(resp) {
						if (resp.readyState === 4) {
							var match = resp.responseText.match(/<div class="image">\s*(?:<a[^>]*>)?\s*<img[^>]*src="([^"]*)"/);
							if (match) {
								options.cb(match[1]);
							} else {
								options.cb(null);
							}
						}
					}
				});

				return {
					waiting: true
				};
			}
		}

		if (domain_nowww === "coolwallpaperz.info") {
			return src.replace(/(\/uploads\/wall\/)thumb(\/[0-9]+\/[^/]*)$/, "$1o$2");
		}

		if (domain_nowww === "besthqwallpapers.com" ||
			domain_nowww === "wallpapers4screen.com" ||
			domain_nowww === "brightwallpapers.com.ua") {
			return src.replace(/(\/[Uu]ploads\/+(?:[0-9]{1,2}-){2}[0-9]{4}\/+(?:[0-9]+|[-0-9a-f]+)\/+)thumb[0-9]*-([^/]*)$/, "$1$2");
		}

		if (domain_nowww === "avatanplus.com") {
			if (string_indexof(src, "/resize.php?") >= 0) {
				var type = url.searchParams.get("type");
				var file = url.searchParams.get("file");
				return "https://avatanplus.com/files/" + type + "/original/" + file;
			}
			return src
				.replace(/(\/files\/[a-z]+)\/[a-z]+\/([0-9a-f]+\.[^/.]*)$/, "$1/original/$2");
		}

		if (domain === "data.japanese.kpopstarz.com" ||
			domain === "image.kpopstarz.com" ||
			domain === "images.kpopstarz.com" ||
			domain === "images.kstars.kr" ||
			domain === "images.enstarz.com" ||
			domain === "cdn.breathecast.com" ||
			(domain_nosub === "yibada.com" && /^image\./.test(domain)) ||
			(domain_nosub === "stackpathdns.com" && domain.match(/1tmxd3aba43noa\./))) {
			newsrc = src
				.replace(/\/data\/thumbs\/full\/([0-9]+)\/[0-9]+\/[0-9]+\/[0-9]+\/[0-9]+\/([^/]*)$/,
						 "/data/images/full/$1/$2")
				.replace(/[?#].*$/, "");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "studiosol-a.akamaihd.net") {
			return src
				.replace(/(:\/\/[^/]*\/)([a-z]+)\/[0-9]+x[0-9]+\//, "$1uploadfile/$2/")
				.replace(/-tb_[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "letradamusica.net" && string_indexof(src, "/fotos/") >= 0) {
			return src.replace(/-tb(?:_[0-9]+)?(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (amazon_container === "quietus_production") {
			return src.replace(/_crop_[0-9]+x[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "bmi.com" &&
			string_indexof(src, "/images/") >= 0) {
			return src.replace(/\/cache\/([^/]*)_[0-9]+_[0-9]+_[0-9]+(?:_[a-z]+)?(\.[^/.]*)$/, "/$1$2");
		}

		if (domain === "i.warosu.org") {
			return src.replace(/\/thumb\/([0-9]+\/[0-9]+\/[0-9]+)s?(\.[^/.]*)$/, "/img/$1$2");
		}

		if (domain === "media.rbl.ms") {
			if (string_indexof(src, "/image?") >= 0) {
				var u = decodeURIComponent(url.searchParams.get("u")).replace(/^([^/])/, "/$1");
				var ho = decodeURIComponent(url.searchParams.get("ho")).replace(/\/$/, "");
				return ho + u;
			}
		}

		if (domain_nowww === "filepicker.io" ||
			domain === "d1t35hkz8sx2bl.cloudfront.net") {
			return {
				url: src.replace(/\/convert\?.*/, ""),
				head_wrong_contenttype: true
			};
		}

		if (domain === "cdn.filestackcontent.com") {
			return src.replace(/(:\/\/[^/]*\/)(?:(?:[a-z]+=[^/]*\/+)+|api\/+file\/+)?([^/?]*)(?:\/+(?:convert.*)?|\/*\?.*)?$/, "$1$2");
		}

		if (domain === "cdn.teenidols4you.com" ||
			domain === "www.teenidols4you.com") {
			return src.replace(/:\/\/[^/]*\/thumb\/(.*?)\/(?:[0-9]+)?([^/]*)$/, "://www.teenidols4you.com/blink/$1/$2");
		}

		if (domain === "cdn.pixabay.com") {
			newsrc = src.replace(/.*?\/photo\/.*\/([^/]*-[0-9]+)_+[0-9]+[^/]*(\.[^/.]*)$/,
								 "https://pixabay.com/en/photos/download/$1$2");

			var obj = {
				url: src,
				redirects: true,
				headers: {
					Referer: "https://pixabay.com/"
				}
			};

			var ret = [obj];

			if (newsrc !== src) {
				var newobj = deepcopy(obj);
				newobj.url = newsrc;
				ret.unshift(newobj);
			}

			return ret;
		}

		if (domain_nosub === "meetupstatic.com") {
			return src
				.replace(/\/photo_api\/([^/]*)\/.*\/([0-9]+\.[^/.]*)$/, "/photos/$1/0/0/0/highres_$2")
				.replace(/\/(?:thumb|[0-9]+)_([0-9]+\.[^/.]*)$/, "/highres_$1");
		}

		if (domain === "d38c5dutwb1t0j.cloudfront.net") {
			return src.replace(/\/Pictures\/[0-9a-z]+x[0-9a-z]+\//, "/Pictures/9999999xany/");
		}

		if (domain === "img.drillspin.com") {
			return src.replace(/(:\/\/[^/]*\/[a-z]+\/)[0-9]+\//, "$1orig/");
		}

		if (domain === "pics.drillspin.com") {
			return src.replace(/\/[0-9]+(\.[^/.]*)$/, "/orig$1");
		}

		if ((domain_nosub === "bestie.vn" && domain.match(/^static[0-9]*\./)) ||
			domain === "img.highloadtest.com" ||
			domain === "cdn.sinemia.com") {
			return src.replace(/(:\/\/[^/]*\/)[0-9]+x[0-9]+\//, "$1");
		}

		if (domain === "cdn.britannica.com") {
			return src.replace(/(:\/\/[^/]*\/)(?:s:)?[0-9]+x[0-9]+(?:,[^/]+)?\//, "$1");
		}

		if (domain === "cdn.awsli.com.br") {
			return src.replace(/(:\/\/[^/]+\/+)[0-9]+x[0-9]+\/+([0-9]+\/+[0-9]+\/+)/, "$1$2");
		}

		if (domain_nowww === "pets4homes.co.uk") {
			return src.replace(/(\/images\/[^/]+\/[0-9]+\/)[a-z]+\/([^/]*)$/, "$1original/$2");
		}

		if (domain === "r.hswstatic.com") {
			return src.replace(/:\/\/[^/]*\/[a-z]_[0-9]+\//, "://s.hswstatic.com/");
		}

		if (domain === "images.woovly.com") {
			return src.replace(/(:\/\/[^/]+\/+)[wh]_[0-9]+\/+/, "$1");
		}

		if (domain_nowww === "candb.com") {
			return src.replace(/(\/candb\/)cache\/([^/]*\/)[0-9]+\/([^/]*)_[0-9]+x[0-9]+(?:[^/]*)?(\.[^/.]*)$/, "$1images/$2$3$4");
		}

		if (domain_nowww === "ottawalife.com") {
			return src.replace(/\/cms\/images\/[a-z]+\//, "/cms/images/large/");
		}

		if (domain === "assets.rpgsite.net") {
			return src.replace(/(\/images\/[0-9]+\/[0-9]+\/[0-9]+\/)[a-z]+\/([^/]+)$/, "$1original/$2");
		}

		if (domain_nowww === "nusabali.com") {
			return src.replace(/(\/article_images\/[0-9]+\/[^/]*)-(?:thumb|800)(-[0-9]+-[0-9]+-[0-9]+-[0-9]+_[0-9]+\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "wowkeren.com") {
			return src
				.replace(/\/display\/images\/[0-9]+x[0-9]+\//, "/display/images/photo/")
				.replace(/\/images\/events\/[^/]*\//, "/images/events/ori/");
		}

		if (domain === "m.aceshowbiz.com" ||
			domain_nowww === "aceshowbiz.com") {
			return src
				.replace(/(\/display\/images\/)[0-9]+x[0-9]+\//, "$1/photo/")
				.replace(/(\/images\/[^/]*\/)preview\//, "$1");
		}

		if (domain === "asset.kompas.com" ||
			(domain_nosub === "grid.id" && domain.match(/^asset(?:-[a-z]*)?\./))) {
			return src
				.replace(/\/crop\/[0-9x:]+\/[0-9]*x[0-9]*\//, "/")
				.replace(/\/crops\/+.*\/+(data\/+photo\/+)/, "/$1");
		}

		if (domain === "img.4plebs.org") {
			return src.replace(/\/thumb(\/[0-9]+\/[0-9]+\/[0-9]+)[a-z](\.[^/.]*)$/, "/image$1$2");
		}

		if (domain === "cdn.thinglink.me") {
			return src.replace(/(\/api\/image\/[0-9a-zA-Z]+)\/.*$/, "$1");
		}

		if (domain === "images.sk-static.com") {
			return src.replace(/(\/[0-9]+\/)[a-z_]+$/, "$1original");
		}

		if (domain === "www.stalkcelebs.com" && string_indexof(src, "/img-folder/") >= 0) {
			return src.replace(/_t(_[0-9]+\.[^/.]*)$/, "$1");
		}

		if (domain === "www.picsofcelebrities.com") {
			return src.replace(/\/media(\/.*\/pictures\/)[a-z]+(\/[^/]*)$/, "$1large$2");
		}

		if (domain === "dxglax8otc2dg.cloudfront.net") {
			return src.replace(/\/media\/cache\/(.*)[-_]thumb\.[a-f0-9]+(\.[^/.]*)$/, "/media/$1$2");
		}

		if (domain_nosub === "smugmug.com" ||
			domain === "photos.smugmug.com") {
			regex  =/(\/i-[A-Za-z0-9]+\/+[0-9]+\/+(?:[a-f0-9]+\/+)?)(?:[A-Z0-9x]+|Ti)(\/+[^/]*)(?:\?.*)?$/;
			newsrc = src.replace(regex, "$1O$2");
			var obj = {
				url: src
			};

			if (newsrc !== src) {
				var obj = {
					url: newsrc,
					redirects: true
				};
			}

			if (options.force_page && options.do_request && options.cb && (newsrc !== src || regex.test(src))) {
				var get_original_page = function(imageid, cb) {
					api_cache.fetch("smugmug_original_page:" + imageid, cb, function(done) {
						options.do_request({
							method: "GET",
							url: "https://washingtonlife.smugmug.com/A/" + imageid + "/A",
							onload: function (result) {
								if (result.readyState !== 4)
									return;

								var match = result.responseText.match(/document\.location\.href\s*=\s*["'](.*?)["']/);
								if (match) {
									return done(match[1], 6*60*60);
								}

								done(null, false);
							}
						});
					});
				};

				match = src.match(/\/(i-[A-Za-z0-9]+)\/+[0-9]+\/+/);
				if (match) {
					get_original_page(match[1], function(originalpage) {
						if (originalpage)
							obj.extra = { page: originalpage };

						options.cb(obj);
					});

					return {
						waiting: true
					};
				}
			}

			return obj;
		}

		if (domain_nosub === "lithium.com" && string_indexof(domain, ".i.lithium.com") >= 0 ||
			domain === "answers.ea.com" ||
			domain === "nasz.orange.pl") {
			var v = src.replace(/.*[?&](v=[^&]*).*/, "$1");
			if (v === src)
				v = "";
			else
				v = "?" + v;

			return {
				url: src.replace(/(\/image-id\/+[0-9]+i[0-9A-F]+)\/+image-size\/.*(?:\?.*)?$/, "$1" + v),
				head_wrong_contentlength: true
			};
		}

		if (domain === "www.favepeople.com") {
			return src.replace(/(\/photos\/[0-9a-z]+\/[0-9a-z]+\/[0-9a-z]+)(?:_[a-z]+)?(\/[^/]*)?(\.[^/.]*)$/, "$1_b$2$3");
		}

		if (domain === "img.anews.com") {
			newsrc = src.replace(/.*\?url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeURIComponent(newsrc);

			return src.replace(/\/r\/[0-9]+x[0-9]+\//, "/media/posts/images/");
		}

		if (domain_nosub === "5ce.com" && /^img(?:-[0-9]+)\.proxy\./.test(domain)) {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+\?(?:.*?&)?url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "immagini.quotidiano.net" ||
			domain === "safeimage.vusercontent.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+\?url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "realitytvworld.com" ||
			domain === "cdn.realitytvworld.com") {
			newsrc = src.replace(/(\/images\/+gallery\/+[0-9]+)-[a-z](\.[^/.]*)(?:[?#].*)?$/, "$1-o$2");
			if (newsrc !== src)
				return newsrc;

			var regex = /(\/heads\/gen\/embedded\/[0-9]+)-[a-z](\.[^/.]*)$/;
			return [
				src.replace(regex, "$1-a$2"),
				src.replace(regex, "$1-l$2")
			];
		}

		if (domain === "video.newsserve.net" ||
			domain === "cdn.newsserve.net") {
			return src.replace(/(:\/\/[^/]*\/)[0-9]+\//, "$1");
		}

		if (domain_nowww === "laughspark.info") {
			return src.replace(/\/thumbfiles\/[0-9]+X[0-9]+\//, "/uploadfiles/");
		}

		if (domain_nosub === "giphy.com" && domain.match(/^(?:media[0-9]*|i)\./)) {

			var baseobj = {
				url: src,
				headers: {
					Referer: "https://giphy.com/"
				}
			};
			obj = baseobj;

			match = src.match(/\/media\/+([^/]{10,})\/+/);
			if (match) {
				var get_giphy_page = function(id) {
					return "https://giphy.com/gifs/" + id;
				};

				id = match[1];
				var page = get_giphy_page(id);
				baseobj.extra = {page: page};

				var query_giphy = function(id, cb) {
					var cache_key = "giphy:" + id;
					api_cache.fetch(cache_key, cb, function(done) {
						options.do_request({
							url: get_giphy_page(id),
							method: "GET",
							headers: {
								Referer: ""
							},
							onload: function(result) {
								if (result.readyState !== 4)
									return;

								if (result.status !== 200) {
									console_error(result);
									return done(null, false);
								}

								var match = result.responseText.match(/<script>\s*Giphy\.renderDesktop\(.*?{\s*gif:\s*({.+}),/);
								if (match) {
									try {
										var json = JSON_parse(match[1]);
										return done(json, 6*60*60);
									} catch (e) {
										console_error(e);
									}
								} else {
									console_warn("No match found", result);
								}

								done(null, false);
							}
						});
					});
				};

				if (options && options.do_request && options.cb) {
					query_giphy(id, function(data) {
						if (!data) {
							return options.cb(null);
						}

						var images = data.images;

						var get_image = function(obj) {
							var ourobj = deepcopy(baseobj);
							ourobj.url = obj.url;
							return ourobj;
						}

						if (images.source) {
							return options.cb(get_image(images.source));
						} else {
							return options.cb(null);
						}
					});

					return {
						waiting: true
					};
				}
			}

			if (false) {
				newsrc = src.replace(/:\/\/media[0-9]*\./, "://i.");
				if (newsrc !== src) {
					obj.url = newsrc;
					return obj;
				}

				newsrc = src.replace(/\/(?:giphy|[0-9]+[whs_]*)\.(?:gif|webp|mp4)/, "/source.mp4");
				if (newsrc !== src) {
					obj.url = newsrc;
					return obj;
				}
			}

			return obj;
		}

		if (domain === "pics.dmm.com" ||
			domain === "pics.dmm.co.jp") {
			return src.replace(/s(\.[^/.]*)$/, "l$1");
		}

		if (domain === "www.showgle.co.kr" && string_indexof(src, "/uploads/") >= 0) {
			return src.replace(/(\/[0-9a-f]+)_[a-z](\.[^/.]*)$/, "$1$2");
		}

		if (domain === "photos.modelmayhem.com") {
			return src
				.replace(/(\/photos\/[0-9]+\/[0-9a-f]+\/[0-9a-f]+)_[a-z](\.[^/.]*)$/, "$1$2")
				.replace(/(\/potd\/entrants\/[0-9]+\/[^/]*)-[a-z]+(\.[^/.]*)$/, "$1-big$2");
		}

		if (domain === "static.artbible.info") {
			return src.replace(/(:\/\/[^/]*\/)[a-z]+\//, "$1large/");
		}

		if (domain_nosub === "oclc.org" && domain.match(/\.contentdm\./) ||
			domain === "digital.denverlibrary.org") {
			newsrc = src
				.replace(/\/+digital\/+api\/+singleitem\/+image\/+([^/]*\/+[0-9]+)\/+.*(\.[^/.?#]*)(?:[?#].*)?$/,
						 "/digital/iiif/$1/full/full/0/default$2")
				.replace(/\/+digital\/+download\/+collection\/+([^/]*)\/(?:.*\/)?id\/+([0-9]+).*?$/,
						 "/digital/api/singleitem/image/$1/$2/default.jpg");

			if (newsrc !== src)
				return newsrc;

			if (src.match(/:\/\/[^/]*\/+utils\/+ajaxhelper\/+/)) {
				var folder = url.searchParams.get("CISOROOT");
				var id = url.searchParams.get("CISOPTR");

				return src.replace(/(:\/\/[^/]*\/).*$/, "$1digital/api/singleitem/image/" + folder + "/" + id + "/default.jpg");
			}
		}

		if (domain === "tools.wmflabs.org") {
			newsrc = src.replace(/(\/zoomviewer\/+iipsrv\.fcgi\/+\?(?:.*&)?iiif=[^/]+\/+[^/]+\/+)[^/]+\/[^/]+\/[^/]+\/([^/]+\.[^/.]*)$/, "$1full/full/0/$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "koodtv.com" ||
			domain_nowww === "hanimusic.com") {
			return src.replace(/\/thumb-([^/.]*)_[0-9]+x[0-9]+(\.[^/.]*)$/, "/$1$2");
		}

		if (domain === "file.bodnara.co.kr") {
			return decodeURIComponent(src.replace(/.*?\/insidelogo\.php.*?[?&]image=([^&]*).*$/, "$1")).replace(/^\/*/, "");
		}

		if (domain === "passport.mobilenations.com" ||
			(domain_nowww === "timpul.md" && string_indexof(src, "/uploads/") >= 0)) {
			return src.replace(/\/[0-9]+x[0-9]+_([^/]*\.[^/.]*)$/, "/$1");
		}

		if (domain === "www.dollargeneral.com" &&
			string_indexof(src, "/media/") >= 0) {
			return src.replace(/\/cache\/(?:image|thumbnail)\/[0-9]+x[0-9]+\//, "/cache/image/");
		}

		if (domain === "www.dollartree.com" &&
			string_indexof(src, "/assets/") >= 0) {
			return src.replace(/\/styles\/[^/]*\/([^/]*)$/, "/styles/jumbo/$1");
		}

		if (domain === "d2192bm55jmxp1.cloudfront.net") {
			return src.replace(/\/resize\/[a-z]+\//, "/origin/");
		}

		if (domain === "i.marieclaire.com.tw") {
			newsrc = src.replace(/\/[0-9]+X[0-9]+\/([0-9A-F]+\.[^/.]*)$/, "/$1");
			if (newsrc !== src) {
				return [newsrc.replace(/\.jpg$/, ".jpeg"), newsrc];
			}
		}

		if (domain === "ir.marieclaire.com.tw") {
			return src.replace(/:\/\/[^/]*\/(?:[a-z][0-9]+){1,}\/+assets\/+/, "://i.marieclaire.com.tw/assets/");
		}

		if (domain === "img.vogue.com.tw" ||
			domain === "dw6vrgax4fzym.cloudfront.net" ||
			domain === "img.gq.com.tw") {
			return src
				.replace(/\/_rs\/[0-9]+\//, "/")
				.replace(/\/userfiles\/(?:sm|thumbnail)\/sm[0-9]+_images/, "/userfiles/images");
		}

		if (domain === "niuerdata.donews.com" ||
			domain === "niuerdata.g.com.cn") {
			return src
				.replace(/\/new_thumb\//, "/big_media_img/")
				.replace(/\/thumb_([0-9a-f]+\.[^/.]*)$/, "/$1");
		}

		if (domain === "pictures.icpress.cn") {
			return src.replace(/s(\.[^/.]*)$/, "$1");
		}

		if ((domain === "interview365.mk.co.kr" &&
			 string_indexof(src, "/images/") >= 0)) {
			return src.replace(/(\/[0-9]+_)[A-Z](_[0-9]+\.[^/.]*)$/, "$1L$2");
		}

		if (domain === "draw.acharts.net") {
			return src.replace(/-[a-z](\.[^/.]*)$/, "-l$1");
		}

		if (domain === "gqhotstuff.gq.com.mx") {
			return src.replace(/\/api\/photos\/[a-z]+\//, "/api/photos/original/");
		}

		if (domain === "images.apester.com") {
			return src.replace(/(:\/\/[^/]*\/[^/]*\.[^/.]*)\/[/a-z0-9]+$/, "$1");
		}

		if (domain === "twt-thumbs.washtimes.com") {
			return src.replace(/(:\/\/twt-)thumbs(\.[^/]*\/media\/(?:image|img)\/.*?)_[cs][0-9][-_0-9a-z]+(\.[^/.?]*)(?:\?.*)?$/, "$1media$2$3");
		}

		if (domain === "dynamicmedia.zuza.com") {
			return src
				.replace(/\/zz\/m\/[^/]*\//, "/zz/m/original_/")
				.replace(/(\.[0-9]*(?:-[0-9]+)?)_[^-./]*(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "images.commeaucinema.com") {
			return src.replace(/\/galerie\/([^/]*)$/, "/galerie/big/$1");
		}

		if (domain === "static.screenweek.it" ||
			amazon_container === "static.screenweek.it") {
			return src.replace(/_[a-z]+(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "goodblud.com") {
			return src.replace(/(\/media\/+mppicture\/+[0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+[0-9]+_[-0-9a-f]{30,})_mid(\.[^/.]*)(?:[?#].*)?$/, "$1_big$2");
		}

		if (domain === "da4pli3l5vc0d.cloudfront.net") {
			return src.replace(/(:\/\/[^/]*\/(?:(?:[0-9a-f]{2})\/){2}[0-9a-f]+)\/.*$/, "$1");
		}

		if (domain_nosub === "hollywood.com" && domain.match(/photo\.media[0-9]*\.hollywood\.com/)) {
			return src.replace(/(:\/\/[^/]*\/)[^/]*\//, "$1full/");
		}

		if (((domain === "www.doodoo.ru" ||
			  domain === "www.youloveit.ru" ||
			  domain_nowww === "take.az" ||
			  domain_nowww === "starmodels.ru" ||
			  domain_nowww === "nevseoboi.com.ua" ||
			  domain_nowww === "photoofnude.com" ||
			  domain_nowww === "hronika.info" ||
			  domain_nowww === "qadin.net" ||
			  domain_nowww === "allday2.com" ||
			  domain_nowww === "tonshuul.mn" ||
			  domain_nowww === "animechan.ru" ||
			  domain_nowww === "sexs-foto.com" ||
			  domain_nowww === "sexs-foto.info" ||
			  domain_nowww === "informaplus.ru" ||
			  domain_nowww === "modnaya.org" ||
			  domain_nowww === "negani.com" ||
			  domain_nowww === "chukcha.net" ||
			  domain_nowww === "opa.kg" ||
			  domain_nowww === "pirojok.net" ||
			  domain_nowww === "femanes.ru" ||
			  domain_nowww === "vistanews.ru" ||
			  domain_nowww === "tour-rest.ru" ||
			  domain_nowww === "znamenitka.ru" ||
			  domain_nowww === "all-stars.su" ||
			  domain_nowww === "kinoukr.com" ||
			  domain === "ww22.zone-telechargement.lol" ||
			  domain_nowww === "hdxa.me" ||
			  domain_nowww === "razdam.net" ||
			  domain_nowww === "bugaga.ru") &&
			 string_indexof(src, "/uploads/") >= 0) ||
			domain === "cdn.prognozist.ru" ||
			src.match(/^[a-z]+:\/\/[^/]*\/uploads\/posts\/[0-9]{4}-[0-9]{2}\/(?:thumbs|medium)\/[0-9]+(?:_[^/]*)?\.[^/.]*$/) ||
			src.match(/^[a-z]+:\/\/[^/]*\/uploads\/[a-z]+\/(?:[0-9]+x[0-9]+\/)?[0-9]{4}-[0-9]{2}\/thumbs\/[0-9]+x[0-9]+_crop_[0-9]+_[^/]*$/)) {
			newsrc = src.replace(/\/uploads\/[a-z]+\/(?:[0-9]+x[0-9]+\/)?([0-9]+-[0-9]+\/)(?:(?:thumbs|medium)\/)?(?:[0-9]+x[0-9]+_crop_)?([^/]*)$/,
								 "/uploads/posts/$1$2");
			if (newsrc !== src)
				return newsrc;
			newsrc = src.replace(/(\/+posts\/+[0-9]{4}-[0-9]{2}\/+)thumbs\/+/, "$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "henchan.me") {
			return src.replace(/(:\/\/[^/]*\/)[^/]*\/+(uploads\/+posts\/+)/, "$1$2");
		}

		if (domain_nowww === "goldwallpapers.com") {
			return src.replace(/(\/uploads\/posts\/[^/]*\/)thumb\/([^/]*)$/, "$1$2");
		}

		if (domain_nowww === "tooob.com" &&
			string_indexof(src, "/api/") >= 0) {
			return src.replace(/(\/[0-9]+\/)__[^/]*$/, "$1");
		}

		if (domain_nosub === "ask.fm" && domain.match(/akphoto[0-9]*\.ask\.fm/) ||
			domain === "dok7xy59qfw9h.cloudfront.net") {
			return src.replace(/\/[a-z]+\/((?:[0-9]+|.+)\.[^/.]*)$/, "/large/$1");
		}

		if (domain_nosub === "ask.fm" && domain.match(/^akimg[0-9]*\.ask\.fm/) ||
			domain === "d2hhj3gz5jljkm.cloudfront.net" ||
			domain === "d15eldcwi10xcl.cloudfront.net") {
			return src.replace(/\/thumb(?:_big)?\/+([^/]+)(?:[?#].*)?$/, "/normal/$1");
		}

		if (domain === "d2p4y1juxwnww4.cloudfront.net") {
			return src.replace(/(\/(?:[0-9a-f]+\/+){6})thumb\/+/, "$1original/");
		}

		if (domain_nowww === "alchetron.com") {
			return src.replace(/-resize-[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain === "img.ltn.com.tw") {
			return src
				.replace(/(\/Upload\/+(?:[a-z]+\/+page|news)\/+)[0-9]+[A-Z]?\/+([0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+)/, "$1orig/$2")
				.replace(/\/Upload\/+style\/+bphoto\/+[^/]+\/+/, "/Upload/style/bphoto/original/");
		}

		if (domain_nosub === "lystit.com" && domain.match(/cdn[a-z]?\.lystit\.com/)) {
			return src.replace(/(:\/\/[^/]*\/)[0-9]+\/[0-9]+\/[0-9a-z]+\/photos\//, "$1photos/");
		}

		if (domain === "www.newshub.co.nz") {
			return {
				url: src.replace(/(\/image(?:[_.][0-9]+)?\.dynimg\.)[^/]*(\.q[0-9]+\.[^/.]*)(\/.*)?$/, "$1full$2$3"),
				can_head: false
			};
		}

		if (amazon_container === "s3.931wolfcountry.com" ||
			domain === "assets.instyle.co.uk" ||
			domain === "d2nzqyyfd6k6c7.cloudfront.net" ||
			(amazon_container && amazon_container.match(/^s3(?:\.[^/]*)?\.radio\.com$/)) ||
			domain === "qtxasset.com" ||
			amazon_container === "wpr-public" ||
			amazon_container === "prod-media.gameinformer.com" ||
			domain === "community-content-assets.minecraft.net" ||
			domain === "cdn-s3.si.com" ||
			(domain_nosub === "hercampus.com" && domain.match(/^cdn[0-9]*\./)) ||
			amazon_container === "southbank-ipcmedia-com" ||
			domain === "cdn.newsbusters.org" ||
			amazon_container === "radioimg" ||
			amazon_container === "b2bm" ||
			amazon_container === "looppacificassets" ||
			amazon_container === "kpopimg") {
			newsrc = src.replace(/\/styles\/+[^/]+\/+s3(?:fs)?\//, "/");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nosub === "s-nbcnews.com" && domain.match(/media[0-9]*\.s-nbcnews\.com/)) {
			return src.replace(/(:\/\/[^/]*\/)j(\/.*\/[^/.]*)[^/]*(\.[^/.]*)$/, "$1i$2$3");
		}

		if (domain === "imgcp.aacdn.jp") {
			return src.replace(/\/img-a\/[0-9a-z]+\/[0-9a-z]+\//, "/img-a/auto/auto/");
		}

		if (domain_nowww === "kpopselca.com") {
			return src.replace(/\/selca\/thumb\//, "/selca/");
		}

		if (domain_nowww === "koogle.tv") {
			return src.replace(/\/\.thumbnails\/([^/]*)-[0-9]+x[0-9]+(\.[^/.]*)$/, "/$1$2");
		}

		if (domain_nowww === "fanaru.com" ||
			domain_nowww === "stuffpoint.com") {
			return src.replace(/\/image\/thumb\//, "/image/");
		}

		if (domain === "d9nvuahg4xykp.cloudfront.net" ||
			domain === "d1w8cc2yygc27j.cloudfront.net") {
			return src.replace(/_thumbnail(\.[^/.]*)$/, "$1");
		}

		if (domain === "i.lihkg.com" ||
			domain_nowww === "f3img.gq") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/[0-9]+\/http/, "http");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "f3img.gq") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/[0-9]*.*?[?&]u=(http[^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "www.nintendoworldreport.com" &&
			string_indexof(src, "/media/") >= 0) {
			return src.replace(/\/gallery\/([0-9]+\.[^/.]*)$/, "/$1");
		}

		if (domain_nowww === "nintendoeverything.com" &&
			string_indexof(src, "/gallery/") >= 0) {
			return src.replace(/\/thumbs\/thumbs_([^/]*)$/, "/$1");
		}

		if (domain === "images.nintendolife.com") {
			return src
				.replace(/(\/attachment\/[^/]*\/)[^/]+(\.[^/.]*)$/, "$1original$2")
				.replace(/(:\/\/[^/]+\/+[0-9a-f]{5,}\/+(?:[^/]+\.)?)[0-9]*x[0-9]*(\.[^/.]+)(?:[?#].*)?$/, "$1original$2");
		}

		if (domain === "cdn.igromania.ru") {
			return src.replace(/(\/[0-9a-f]+)(?:_[^-_/.]*)?(\.[^/.]*)$/, "$1_original$2");
		}

		if (domain === "i.blogs.es") {
			return src.replace(/\/[0-9]+_[0-9]+(\.[^/.]*)$/, "/original$1");
		}


		if (domain_nosub === "rbl.ms") {

			newsrc = src.replace(/.*\/image.*?[?&]source=([^&]*).*/, "$1");
			if (newsrc !== src) {
				return decodeURIComponent(newsrc);
			}

			newsrc = src.replace(/.*?\/simage\/([^/]*)\/.*/, "$1");
			if (newsrc !== src) {
				return decodeURIComponent(newsrc);
			}
		}

		if (domain === "resize-rbl-ms.cdn.ampproject.org" ||
			(domain_nosub === "ampproject.org" && domain.match(/\.cdn\.ampproject/))) {
			return src.replace(/^[a-z]+:\/\/[^/]*\/ii\/[wh][0-9]+\/s\//, "http://");
		}

		if (domain === "assets.rbl.ms") {
			return src.replace(/(:\/\/[^/]*\/[0-9]+\/)[^/.]*(\.[^/.]*)$/, "$1origin$2");
		}

		if (domain === "i.paigeeworld.com" ||
			domain === "d395qfwg4461au.cloudfront.net") {
			return src.replace(/(\/user-(?:uploads|media\/[0-9]+)\/[0-9a-f]+_[0-9a-f]+_)[^/.]*(\.[^/.]*)$/, "$1rz$2");
		}

		if (domain === "1d31c772ec21a65b0a71-0707aae3004193da193e1ad4a942592d.ssl.cf2.rackcdn.com") {
			return src.replace(/(:\/\/[^/]*\/[0-9]+\/[^/.]*)__[^_/.]*(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "i.nextmedia.com.au") {
			return decodeURIComponent(src.replace(/.*\/Utils\/ImageResizer\.ashx.*?[?&]n=([^&]*).*?$/, "$1"));
		}

		if (domain_nosub === "heritagestatic.com" && domain.match(/dyn[0-9]*\.heritagestatic\.com/)) {
			var set = url.searchParams.get("set");
			if (!set)
				return src;

			set = decodeURIComponent(set)
				.replace(/,sizedata\[[^,]*\]/, "")
				.replace(/,$/, "");

			return src.replace(/(.*[?&]set=)[^&]*(.*?)$/, "$1" + encodeURIComponent(set) + "$2");
		}

		if (domain_nowww === "natedsanders.com" && string_indexof(src, "/ItemImages/") >= 0) {
			return src.replace(/(\/[0-9a-z]+)_[a-z]+(\.[^/.]*)$/, "$1_lg$2");
		}

		if (domain === "d1x0dndjbjw02n.cloudfront.net" ||
			amazon_container === "static.lovely-media.jp" ||
			amazon_container === "static.cdn.xtreeem.com" ||
			amazon_container === "cdn.bibi-star.jp" ||
			amazon_container === "cdn.panda-gossips.com") {
			return src.replace(/(\/(?:[0-9]{3}\/+){3})[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1original$2");
		}

		if (domain === "summary-sv.fc2.com") {
			return decodeURIComponent(src.replace(/.*?\/api\/resize_img\.php.*?[?&]src=([^&]*).*?$/, "$1"));
		}

		if (domain === "c.fc2.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+imgs\/+(http[^/]*)\/*$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "image.tmdb.org") {
			return src.replace(/\/[wh][0-9]+(?:_and[^/]*)?\/([0-9a-zA-Z]+\.[^/.]*)$/, "/original/$1");
		}


		if (domain === "embedly.massrelevance.com") {
			return decodeURIComponent(src.replace(/.*\/image.*?[?&]url=([^&]*).*?$/, "$1"));
		}

		if (domain === "media.lolusercontent.com") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/api\/embedly\/1\/image\/resize.*?[?&]url=([^&]*).*?$/, "$1"));
		}

		if (domain === "img.day.az" ||
			domain === "img.milli.az") {
			newsrc = src.replace(/\/(?:(?:[0-9]+x[0-9]+[a-z]?)|thumb)(\/[^/]*)$/, "$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "publika.az") {
			return src.replace(/(\/storage\/[0-9]{4}\/[0-9]{2}\/[0-9]{2}\/)[0-9]+x[0-9]+[a-z]?\/([^/]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "fotos.caras.uol.com.br") {
			return src.replace(/\/media\/images\/[^/]*\//, "/media/images/original/");
		}

		if (domain === "www.wmj.ru" ||
			domain === "sih.avn.com" ||
			domain_nowww === "ferra.ru" ||
			domain_nowww === "motor.ru" ||
			domain_nowww === "passion.ru") {
			return src.replace(/(:\/\/[^/]*\/)(?:thumb\/)?[0-9]+x[0-9]+\/(?:top\/)?filters:[^/]*\//, "$1");
		}

		if (domain_nosub === "centerblog.net" && string_indexof(domain, "pic.centerblog.net") >= 0) {
			return src.replace(/(:\/\/[^/]*\/)(?:[a-z]\/)?([^/]*)$/, "$1o/$2");
		}

		if (domain === "d2n4wb9orp1vta.cloudfront.net") {
			return src.replace(/;.*/, "");
		}

		if (domain === "gd.image-gmkt.com") {
			return src.replace(/\.[^/.]*(\.[^/.]*)$/, "$1");
		}

		if (domain_nosub === "toonpool.com") {
			return {
				url: src
					.replace(/(\/user\/[0-9]+\/)thumbs\/([^/.]+_[0-9]+)(\.[^/.]*)$/, "$1files/$29$3")
					.replace(/(\/user\/[0-9]+\/)files\/([^/.]+_[0-9]+)[0-9](\.[^/.]*)$/, "$1files/$29$3"),
				problems: {
					watermark: true
				}
			};
		}

		if (domain === "www.cartoonmovement.com") {
			return src.replace(/\/cartoon_thumbnails\//, "/cartoons/");
		}

		if (domain_nosub === "sndimg.com") {
			return src.replace(/(\/[^/.]+\.[^/.]+)\.[^/]*(?:\/[^/]*)?$/, "$1");
		}

		if (domain === "st.hzcdn.com") {
			return src
				.replace(/\/fimgs\/([0-9a-f]+)_([0-9]+)-[^/]*(\.[^/.]*)$/, "/simgs/$1_14-$2$3")
				.replace(/(\/simgs\/[0-9a-f]+)_[0-9]+(-[0-9]+[./])/, "$1_14$2");
		}

		if ((domain_nowww === "axisanimation.com" ||
			 domain_nowww === "sexo18.net") &&
			string_indexof(src, "/assets/") >= 0) {
			return src.replace(/\.[0-9]+x[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain === "media.baselineresearch.com") {
			return src.replace(/(\/[0-9]+_)[a-z]+(\.[^/.]*)$/, "$1full$2");
		}

		if (domain_nowww === "mireportz.com" ||
			domain_nowww === "gameworld.gr" ||
			domain_nowww === "pakistani.pk") {
			return src.replace(/\/photos\/+thumbnail\/+[0-9]+x[0-9]+[a-z]*\//, "/photos/original/");
		}

		if (domain_nosub === "gazeta.pl" && domain.match(/bis?\.gazeta\.pl/)) {
			return src.replace(/(\/z[0-9]+)[A-Z]+(,[^/.]*)?(\.[^/.]*)$/, "$1O$2$3");
		}

		if (domain_nosub === "static6.com" && domain.match(/cdn[0-9]*(-[a-z]+)?\.production\.[^./]*\.static6\.com$/)) {
			newsrc = src.replace(/(:\/\/cdn[0-9]*(?:-[a-z]+)?\.production\.)[^./]*(\.static6\.com)\/.*?\/([^/.-]+)-media-production\/(medias\/)/,
								"$1$3$2/$4");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "lajt.co.uk") {
			return src.replace(/\/media\/cache\/[0-9]+x[0-9]+\//, "/media/cache/original/");
		}

		if (domain_nowww === "ellearabia.com" ||
			domain_nowww === "national-geographic.pl" ||
			domain_nowww === "ecranlarge.com" ||
			domain === "estaticos.marie-claire.es") {
			return src.replace(/\/+media\/+cache\/+[^/]*\/+/, "/");
		}

		if (domain === "static.t13.cl") {
			match = src.match(/\/images\/+sizes\/+[^/]*\/+([0-9]{8,})-/);
			if (match) {
				var date = new Date(match[1] * 1000);
				return src.replace(/\/images\/+sizes\/+[^/]*\/+/, "/images/original/" + date.getUTCFullYear() + "/" + zpadnum(date.getUTCMonth() + 1, 2) + "/");
			}

			return src.replace(/__[0-9]+x(?:[0-9]+)?(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "cosmo.com.ua") {
			return src.replace(/(\/photos_publication\/[0-9]+\/)[0-9]+_[0-9]+\//, "$1");
		}

		if (domain_nosub === "trrsf.com" && domain.match(/p[0-9]*\.trrsf\.com/)) {
			return src.replace(/^[a-z]+:\/\/[^/]*\/image\/fget\/[^/]*\/(?:(?:[0-9]+\/){4})?(?:fit-in\/+)?[0-9]+\/[0-9]+\//, "http://");
		}

		if (domain === "c.igte.ch" ||
			domain === "img.digitalag.ro") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/.*?[?&]u=([^&]*).*/, "$1"));
		}

		if (domain_nowww === "trendus.com" ||
			domain === "trendus.igte.ch" ||
			domain_nowww === "boxerdergisi.com.tr") {
			return src.replace(/\/PhotoGallery\/+size[0-9]*\//, "/PhotoGallery/original/");
		}

		if (domain === "mediacdn.grabone.co.nz") {
			return src.replace(/(\/asset\/[-_=a-zA-Z0-9]+)\/[a-z]+=.*/, "$1");
		}

		if (domain_nosub === "woman.ru" && domain.match(/i[0-9]*(?:-cdn)?\.woman\.ru/)) {
			return src.replace(/_[0-9]+_[0-9]+x[0-9]+(\.[^/.]*)$/, "$1");
		}


		if (domain_nosub === "20minutos.es" && domain.match(/st-[^/.]*\.20minutos\.es/)) {
			return src.replace(/(\/[0-9a-z]+)_[0-9]+px(\.[^/.]*)$/, "$1_original$2");
		}

		if (domain_nosub === "thehunt.com" && domain.match(/cdn[0-9]*\.thehunt\.com/)) {
			return src.replace(/(\/[0-9]+\/)[^/]*\/([0-9a-f]+\.[^/.]*)$/, "$1original/$2");
		}

		if (domain === "hosted.thehun.net") {
			return src.replace(/(\/\.content\/+[0-9a-f]{3}\/+[0-9a-f]{5,}\/+)thumb\./, "$1");
		}

		if (domain_nosub === "devote.se" && domain.match(/static[0-9]*\.devote\.se/)) {
			return src.replace(/\/gallery\/[^/]+\//, "/gallery/big/");
		}

		if (domain_nowww === "vev.ru" &&
			string_indexof(src, "/uploads/images/") >= 0) {
			return src.replace(/(?:_[a-z]+)?(\.[^/.]*)$/, "_original$1");
		}

		if (domain === "s.glbimg.com") {
			return src.replace(/\/[0-9]+x[0-9]+\//, "/original/");
		}

		if (domain === "image.xahoi.com.vn" ||
			domain === "media.tinmoi.vn" ||
			domain === "media.ngoisao.vn") {
			return src.replace(/(:\/\/[^/]*\/)resize_[0-9]+(?:x[0-9]+)?\//, "$1");
		}

		if (domain_nosub === "intermoda.ru" && domain.match(/s[0-9]*\.intermoda\.ru/)) {
			return src.replace(/\/p\/(?:max|h|w)[0-9]*\//, "/p/original/");
		}

		if (domain === "static.life.ru") {
			return src.replace(/__[0-9]+x(?:[0-9]+)?(\.[^/.]*)$/, "$1");
		}

		if (domain === "a.ltrbxd.com") {
			return src.replace(/\/resized\/(.*)-(?:[0-9]+-){4}crop(?:-[0-9]+)?(\.[^/.]*)$/, "/$1$2");
		}

		if (domain === "images.goodsmile.info") {
			return src.replace(/\/[a-z]+(\/[0-9a-f]+\.[^/.]*)$/, "/original$1");
		}

		if (domain === "dzt1km7tv28ex.cloudfront.net") {
			return src.replace(/_[a-z](\.[^/.]*)$/, "_o$1");
		}

		if (domain_nowww === "celebs-place.com") {
			return src.replace(/(\/(?:gallery|news\/+pics)\/+[^/]*\/+[^/]*)_[a-z](\.[^/.]*)$/, "$1$2");
		}

		if (domain === "static.vfiles.com") {
			return {
				url: src.replace(/(\/image\/media\/[0-9]+\/)[a-z]+(?:\?.*)?$/, "$1original"),
				can_head: false
			};
		}

		if (domain === "img.mediacentrum.sk") {
			return src.replace(/\/gallery\/(?:.*?\/)?[0-9]+(?:_[a-z])?\/([0-9]+\.[^/.]*)$/, "/gallery/original/$1");
		}

		if (domain === "images.says.com") {
			return src.replace(/(\/[0-9]+\/)[a-z_]+_([0-9]+\.[^/.]*)$/, "$1$2");
		}

		if (domain_nosub === "skyrock.net" && /^(?:[0-9]+\.)?wir\./.test(domain)) {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/.*[?&]im=([^&]*).*$/, "$1");
			if (newsrc !== src)
				return urljoin("https://i.skyrock.net/", decodeuri_ifneeded(newsrc), true);
		}

		if (domain === "i.skyrock.net") {
			return src
				.replace(/(\/pics\/(?:photo_)?[0-9]+)_[a-z]+((?:_[0-9]+)?\.[^/.]*)$/, "$1$2")
				.replace(/(\/pics\/[0-9]+)_[0-9]_([0-9]+_[A-Za-z0-9]+\.[^/.]*)$/, "$1_1_$2");
		}

		if (domain === "mgl.skyrock.net") {
			return src.replace(/(\/art\/+[A-Z]+\.[0-9]+\.[0-9]+)\.0(\.[^/.]*)(?:[?#].*)?$/, "$1.1$2");
		}

		if (domain_nosub === "xhcdn.com" &&
			(domain.match(/thumb-p[0-9]*\.xhcdn\.com/) ||
			 domain === "upt.xhcdn.com" ||
			 domain === "ept.xhcdn.com")) {
			newsrc = src.replace(/(:\/\/[^/]*\/)a\/+[-_a-zA-Z0-9]{10,}\/+([0-9]+\/)/, "$1$2");
			if (newsrc !== src)
				return newsrc;

			return src.replace(/(\/[0-9]+_)[0-9]+(\.[^/.]*)$/, "$11000$2");
		}

		if (domain_nosub === "netease.com" && domain.match(/img[0-9]*\.cache\.netease\.com/)) {
			return src
				.replace(/[?#].*$/, "")
				.replace(/\/(?:[a-z]|[0-9]+x[0-9]+)_([^/]*)$/, "/$1")
				.replace(/_[0-9]+(\.[^/.]*)$/, "$1")
				.replace(/\.[0-9]+x\.[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain === "img.lengding.cn") {
			return src.replace(/.*?\/getImage\.php.*?[?&]url=(.*)$/, "$1");
		}

		if (domain === "cdn.sabay.com") {
			return src.replace(/:\/\/[^/]+\/cdn\/([a-z]+\.[a-z]+\.[a-z]+\/)/, "://$1");
		}

		if (domain === "media.sabay.com") {
			return src.replace(/_[a-z]+(\.[^/.]*)$/, "$1");
		}

		if (domain === "s.olj.me" ||
			(domain === "static.addiyar.com" && string_indexof(src, "/storage/attachments/") >= 0) ||
			(domain_nosub === "annahar.com" && domain.match(/^static[0-9]*\.annahar/) && string_indexof(src, "/storage/attachments/"))) {
			return src.replace(/(\/storage\/+attachments\/+[0-9]+\/+[^/]*_[0-9]+)(?:_(?:large|thumbnail))?(\.[^/.]*)(?:\/+r\/+[0-9]+)?(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "binaryapi.ap.org") {
			return {
				url: src,
				can_head: false
			};
		}

		if (domain_nosub === "ekladata.com") {
			return src.replace(/@[0-9]+x(?:[0-9]+)?(\.[^/.]*)$/, "$1");
		}

		if (domain === "img.geinoueroch.com" ||
			domain === "image.hoopchina.com" ||
			(domain === "san.imatin.net" && string_indexof(src, "/images/") >= 0) ||
			(domain_nowww === "kandamori.com" && src.match(/\/hamblog[0-9]+\/+[0-9]+/)) ||
			domain === "img.momon-ga.com") {
			return src.replace(/-s(\.[^/.]*)$/, "$1");
		}

		if (domain === "note.taable.com" &&
			string_indexof(src, "/photo/") >= 0) {
			return decodeURIComponent(src.replace(/:\/\/[^/]*\/photo\//, "://"));
		}

		if (domain_nosub === "adult-gazou.me" && domain.match(/img-[^.]*\.adult-gazou\.me/)) {
			return src.replace(/\/[a-z](\/[0-9]*\.[^/.]*)$/, "/l$1");
		}

		if (domain_nosub === "fukugan.com" &&
			string_indexof(src, "/rssimg/") >= 0) {
			return decodeURIComponent(decodeURIComponent(src.replace(/.*\/(https?[%:].*)/, "$1"))).replace(/\.#.*/, "");
		}

		if ((domain_nosub === "lostbird.vn" ||
			 domain_nosub === "nginx.page") &&
			domain.match(/img[0-9]*\./)) {
			return src.replace(/(:\/\/[^/]*\/)[-0-9]+x[-0-9]+\//, "$1");
		}

		if (domain === "img.biggo.com.tw") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/[^/]*\//, "");
		}

		if (((domain_nosub === "book.com.tw" && domain.match(/im[0-9]*\.book\.com\.tw/)) ||
			 domain === "www.books.com.tw" ||
			 domain === "buy.line-scdn.net") &&
			string_indexof(src, "/getImage") >= 0) {
			return decodeURIComponent(src.replace(/.*\/getImage.*?[?&]i=([^&]*).*?$/, "$1"));
		}

		if (domain === "img.feebee.com.tw") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/ip\/[0-9]+\/[^/]+=\//, "");
		}

		if (domain === "img.fireden.net") {
			return add_full_extensions(src.replace(/\/thumb(\/.*?)s(\.[^/.]*)$/, "/image$1$2"));
		}

		if (domain === "www.consolefun.fr" ||
			domain_nowww === "nookthem.com" ||
			domain_nowww === "nude-gals.com") {
			return src.replace(/\/thumbs\/+th_/, "/");
		}

		if (domain === "ex.f3img.gq") {
			return decodeURIComponent(src.replace(/.*?\/api\/image.*?[?&]url=([^&]*).*?$/, "$1"));
		}

		if (domain === "images.everyeye.it") {
			return src.replace(/(-v[0-9]+-[0-9]+)-[0-9]+(\.[&/.]*)/, "$1$2");
		}

		if (domain_nowww === "bobx.com") {
			var get_obj = function(url) {
				var normurl = url
					.replace(/\/thumbnail\/+(.*?)(?:-preview)?(-[0-9]+)(?:\.t)?(\.[^/.]*)$/, "/$1-$2$3")
					.replace(/--([0-9]+\.[^/.]*)$/, "-$1");

				var obj = {
					url: url,
					headers: {
						Referer: normurl.replace(/\/([^/]+)\.[^/.]*$/, "/$1.html")
					},
					bad_if: [{
						headers: {
							"content-length": "281567",
							"last-modified": function(value) {
								if (/^Tue, 19 Nov 2019/i.test(value)) {
									return true;
								}

								return false;
							}
						}
					}]
				};

				if (url.match(/\/[^/]+[^-]-[0-9]+\.[^/.]+(?:[?#].*)?$/)) {
					obj.headers.Referer = normurl.replace(/\/([^/]+)\.[^/.]*$/, "/large-$1.html");
					obj.is_original = true;
				}

				obj.extra = {page: obj.headers.Referer};

				return obj;
			};

			newsrc = src.replace(/\/thumbnail\/+(.*?)(?:-preview)?(-[0-9]+)(?:\.t)?(\.[^/.]*)$/, "/$1-$2$3");
			if (newsrc !== src)
				return get_obj(newsrc);

			newsrc = src.replace(/--([0-9]+\.[^/.]*)$/, "-$1");
			if (newsrc !== src)
				return get_obj(newsrc);

			return get_obj(src);
		}

		if ((domain === "www.altcine.com" && string_indexof(src, "/photo/") >= 0) ||
			domain === "gallery-cdn.tiscali.it" ||
			(domain === "img.amur.info" && string_indexof(src, "/res/") >= 0) ||
			(domain === "thumbs.vaultsex.com" && string_indexof(src, "/content/") >= 0) ||
			(domain_nowww === "dspdaily.com" && string_indexof(src, "/data/") >= 0)) {
			return src.replace(/\/[0-9]+x[0-9]+\/([^/]*)$/, "/$1");
		}

		if (domain === "www.spacetelescope.org" &&
			string_indexof(src, "/static/") >= 0) {
			return src.replace(/\/images\/[^/]*\//, "/images/large/");
		}

		if (domain === "static.qobuz.com") {
			return src.replace(/_[0-9]+(\.[^/.]*)$/, "_org$1");
		}

		if (domain_nowww === "sarajevo.travel" &&
			string_indexof(src, "/assets/photos/") >= 0) {
			return src.replace(/\/[a-z]+\/([^/]*)$/, "/original/$1");
		}

		if (domain === "avatars.mds.yandex.net" ||
			domain === "avatars.yandex.net") {
			obj = {
				url: src.replace(/\/(?:[a-z]+_|[a-zA-Z])?[0-9]+(?:x[0-9]+)?(?:[A-Za-z_0-9]+)([?&].*)?$/, "/orig$1")
			};

			match = src.match(/\/get-[^/]+\/+([0-9]+)\/+[^/]+\/+[^/]+(?:[?#].*)?$/);
			if (match)
				obj.filename = match[1];

			return obj;
		}

		if (domain_nowww === "t2online.com") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/unsafe\//, "http://");
		}

		if (domain_nowww === "filmnegah.com") {
			return decodeURIComponent(src.replace(/(:\/\/[^/]*)\/Image\/(?:Resize|Thumbnail).*?[?&](?:url|path)=~([^&]*).*?$/, "$1$2"));
		}

		if (domain_nowww === "niagara.sk" && string_indexof(src, "/images/") >= 0) {
			return src.replace(/\/tmb-[0-9]+(?:-[0-9]+)\//, "/big/");
		}

		if (domain_nowww === "cdn-cinenode.com") {
			return src.replace(/(\/[0-9]+\/)([^/]*)-[0-9]+-[0-9]+(\.[^/.]*)$/, "$1full/$2$3");
		}

		if (domain_nosub === "canalblog.com" && domain.match(/\.storage\.canalblog\.com$/)) {
			return src
				.replace(/\.[^/]*(\.[^/.]*)$/, "$1")
				.replace(/(\/[0-9]+)(?:_[a-z])?(\.[^/.]*)$/, "$1_o$2");
		}

		if (domain_nowww === "film-like.com") {
			return src.replace(/\/thumb\//, "/full/");
		}

		if (domain_nowww === "24smi.org" ||
			domain_nowww === "24celebs.com") {

			return src
				.replace(/\/img\/+[0-9]+_[0-9]+\//, "/img/999999999_999999999/")
				.replace(/\/public\/+media\/+(?:resize\/+)?[-0-9]+x[-0-9]+\//, "/public/media/");
		}

		if (domain === "cdn.metrotvnews.com" ||
			domain === "cdn.medcom.id") {
			return src.replace(/\?.*/, "?w=99999999999");
		}

		if (domain_nowww === "znqnet.com" && string_indexof(src, "/fileupload/") >= 0) {
			return src.replace(/\/fileupload\/thumb\//, "/fileupload/image/");
		}

		if (domain_nosub === "rpp-noticias.io" && domain.match(/[a-z]\.rpp-noticias\.io/)) {
			return src.replace(/:\/\/e\.rpp-noticias\.io\/[a-z]+\//, "://f.rpp-noticias.io/");
		}

		if (domain_nosub === "diarioinformacion.com" && domain.match(/fotos[0-9]*\.diarioinformacion\.com/)) {
			return src.replace(/\/[0-9]+x[0-9]+\//, "/");
		}

		if (domain === "imagenes-cdn.diarioinformacion.com" && string_indexof(src, "/multimedia/fotos/")) {
			return src.replace(/_[a-z](\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "digi-film.ro") {
			return src.replace(/\/onedb\/picture\([^/]*\//, "/onedb/picture/");
		}

		if (domain === "s.iw.ro") {
			newsrc = src.replace(/\.thumb(\.[^/.]+)([?#].*)?$/, "$1$2");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/\/gateway\/+g\/+(([^/.?]{32}\/+){1,}[^/.]*\.[^/.?#]*)/);
			if (match) {
				var splittedhex = match[1].replace(/\/+/g, "/").replace(/\..*/, "").split("/");
				var endstr = "";
				for (var i = 0; i < splittedhex.length; i++) {
					endstr += base64_decode(splittedhex[i]);
				}

				newsrc = endstr.replace(/^(?:.*&)?fileSource=([^&]*).*?$/, "$1");
				if (newsrc !== src)
					return decodeURIComponent(newsrc);
			}
		}

		if (domain_nosub === "jiemian.com" && domain.match(/img[0-9]*\.jiemian\.com/)) {
			return src.replace(/\/([0-9]+)_[a-zA-Z0-9]+x[a-zA-Z0-9]+(\.[^/.]*)$/, "/$1$2");
		}

		if (domain === "cdn.highdefdigest.com") {
			return src.replace(/(\/uploads\/[0-9]+\/[0-9]+\/[0-9]+\/)[0-9]+\/([^/]*)$/, "$1$2");
		}

		if (domain_nowww === "ojosdecafe.com") {
			return src.replace(/\/thumbs\/[0-9]+x[0-9]+_/, "/");
		}

		if (domain === "img.gestion.pe") {
			return src.replace(/\/files\/[^/]*\/uploads\//, "/uploads/");
		}

		if (domain === "thumb.guucdn.net") {
			return src.replace(/:\/\/[^/]*\/[0-9]+x[0-9]+\//, "://");
		}

		if ((domain_nowww === "game4v.com" && string_indexof(src, "/thumb/thumb.php?") >= 0) ||
			domain === "static.pinwallpapers.com") {
			newsrc = src.replace(/.*?\/thumb\.php.*?[?&]src=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "newpal.ps" ||
			domain_nosub === "iravunk.com" ||
			(domain_nowww === "mrcapetown.co.za" && src.match(/\/wp-content\/.*\/functions\/+thumb\.php/)) ||
			domain_nowww === "vtimes.com.au" ||
			domain_nowww === "pardisgame.net") {
			newsrc = decodeURIComponent(src.replace(/.*?\/thumb\.php.*?[?&]src=([^&]*).*?$/, "$1"));
			if (newsrc !== src) {
				if (newsrc.match(/^[a-z]+:\/\//))
					return newsrc;
				else
					return urljoin(src, ("/" + newsrc).replace(/^\/\//, "/"), true);
			}
		}

		if (domain === "static.santruyen.com") {
			return src.replace(/(\/[0-9]+)_[a-z]+(\.[^/.]*)$/, "$1$2");
		}

		if (amazon_container === "boligsurf-production" &&
			string_indexof(src, "/assets/images/") >= 0) {
			return src.replace(/\/fixed_[0-9]+_[0-9]+\//, "/original/");
		}

		if (domain_nowww === "sonara.net") {
			return decodeURIComponent(src.replace(/:\/\/[^/]*\/cro\.php.*?[?&]image=([^&]*).*?$/, "://images.sonara.net$1"));
		}

		if (domain === "d2u7zfhzkfu65k.cloudfront.net" &&
			string_indexof(src, "/resize/wp-content/") >= 0) {
			return src.replace(/:\/\/[^/]*\/resize\//, "://d3kszy5ca3yqvh.cloudfront.net/");
		}

		if (domain === "imgbp.hotp.jp") {
			return src.replace(/_[0-9]+-[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain === "review.zafcdn.com") {
			return src.replace(/(\/upload\/+zaful\/+review\/+[0-9]{8}\/+)(?:thumb\/+)?([0-9A-F]{10,})_(?:[0-9]+-[0-9]+|thumb)(\.[^/.]*)(?:[?#].*)?$/,
							"$1$2$3");
		}

		if (amazon_container === "ro69-bucket") {
			return src.replace(/(\/image\/[0-9]+\/)[^/]*\/resize_image/, "$1default/resize_image");
		}

		if (domain === "aimg-pictpix.akamaized.net") {
			return src.replace(/(:\/\/[^/]*\/)ts[0-9]+x[0-9]+\/img\//, "$1img/");
		}

		if (amazon_container === "lattepic") {
			return src.replace(/(\/[a-z]+_)[a-z]+(\.[^/.?]*)$/, "$1org$2");
		}

		if (domain === "coconala.akamaized.net") {
			return src.replace(/\/service_images\/[0-9]+x[0-9]+\//, "/service_images/original/");
		}

		if (domain === "dplhqivlpbfks.cloudfront.net") {
			return src.replace(/.*(\/[0-9a-f]+-[0-9]+\.[^/.]*)$/, "https://coconala.akamaized.net/coconala-public-files/service_images/original$1");
		}

		if (domain === "www.ysbnow.com") {
			return src.replace(/(:\/\/[^/]*\/dam).*?[?&](media-id=[^&]*).*?$/, "$1?$2");
		}

		if (domain === "www.quizz.biz" &&
			string_indexof(src, "/uploads/") >= 0) {
			return src.replace(/(\/[0-9]+\/)[a-z]+(\/[0-9]+)(?:_[0-9]+)?(\.[^/.]*)$/,"$1orig$2$3");
		}

		if (domain_nosub === "flipagramcdn.com" && domain.match(/c[0-9]*\.flipagramcdn\.com/)) {
			return src.replace(/-[a-z]+(?:\?.*)?$/, "");
		}

		if (domain === "images-cdn.9gag.com" ||
			domain === "img-9gag-fun.9cache.com" ||
			domain === "d24w6bsrhbeh9d.cloudfront.net" ||
			domain === "miscmedia-9gag-fun.9cache.com") {
			return src
				.replace(/\/thumbnail-facebook\/([^/]*)_[0-9]+x[0-9]+(\.[^/.]*)$/, "/thumbnail-facebook/$1_n$2")
				.replace(/_460swp(\.[^/.]*)$/, "_700bwp$1")
				.replace(/_460s(_v1)?(\.[^/.]*)$/, "_700b$1$2");
		}

		if (domain_nowww === "samironsheadshots.com" &&
			string_indexof(src, "/images/") >= 0) {
			return src.replace(/\/small\/([^/]*)$/, "/large/$1");
		}

		if (domain === "cps-static.rovicorp.com") {
			return src
				.replace(/\/_derived_[^/]*(\/[^/]*)$/, "$1")
				.replace(/\/JPG_[0-9]+\//, "/JPG_SRC/");
		}

		if (domain === "rovimusic.rovicorp.com") {
			if (/[?&]c=[^&]{10,}/.test(src)) {
				var queries = get_queries(src);
				newsrc = src.replace(/\?.*/, "?c=" + queries.c + "&f=0");
				if (newsrc !== src)
					return newsrc;
			}
		}

		if (domain_nosub === "netflixmovies.com" && domain.match(/i[0-9]*\.netflixmovies\.com/)) {
			return src.replace(/\/image\/upload\/[wh]_[0-9]+\//, "/image/upload/");
		}

		if (domain === "www.shakespearesglobe.com") {
			return src.replace(/(\/images\/[0-9]+)\/[a-z]+(?:\?.*)?$/, "$1");
		}

		if (domain_nosub === "rsc.org.uk" && domain.match(/cdn[0-9]*\.rsc\.org\.uk/)) {
			return src.replace(/\.tmb-img-[0-9]*\./, ".");
		}

		if (domain === "images.fashionmodeldirectory.com") {
			return src.replace(/-(?:square(?:small|medium)|single|fit|modelprofile[A-Za-z]+|profile|alternative)(\.[^/.]*)$/, "-fullsize$1");
		}

		if (domain === "cdn.public.hegre.com" ||
			domain === "p.hegre.com") {
			newsrc = src.replace(/-image-[0-9]+x(?:_2x)?(\.[^/.]*)(?:[?#].*)?$/, "-image-fullsize$1");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/(\/films\/+[^/]+\/+[0-9]+\/+[^/]+)-[0-9]*x[0-9]*\./, "$1.");
			if (newsrc !== src)
				return newsrc;

			var sizes = [
				"2160p",
				"1080p",
				"720p",
				"480p"
			];

			match = src.match(/\/films\/+[^/]+\/+[^/]+-([0-9]+p)\.mp4/);
			if (match && array_indexof(sizes, match[1]) > 0) {
				sizes = sizes.slice(0, array_indexof(sizes, match[1]));

				var urls = [];
				for (var i = 0; i < sizes.length; i++) {
					urls.push({
						url: src.replace(/(\/films\/+[^/]+\/+[^/]+-)[0-9]+p\./, "$1" + sizes[i] + "."),
						video: true
					});
				}

				return urls;
			}
		}

		if (domain === "storage.hegre.com") {
			return src.replace(/(\/img\/+gallery-[0-9]+-)thumb(-[0-9]+\.[^/.]+)(?:[?#].*)?$/, "$1image$2");
		}

		if (domain_nowww === "crystal.cafe" ||
			domain === "cdn.syn-ch.com" ||
			domain_nowww === "alphachan.org" ||
			(domain_nowww === "zonadelta.net" && string_indexof(src, "/deltachan/") >= 0) ||
			domain_nowww === "lolcow.farm") {
			return add_extensions(src.replace(/\/thumb\//, "/src/"));
		}

		if (domain_nowww === "neochan.ru") {
			newsrc = src.replace(/:\/\/(?:www\.)?([^/]+\/+[^/]+\/+)thumb\/+/, ":\/\/s.$1");
			if (newsrc !== src) {
				obj = add_extensions_gif(newsrc);

				[].push.apply(obj, add_extensions_gif(src.replace(/\/thumb\//, "/src/")));
				return obj;
			}
		}

		if ((domain_nowww === "skinnygossip.com" ||
			 domain_nowww === "alternatehistory.com" ||
			 domain_nowww === "gtplanet.net" ||
			 domain_nowww === "burbuja.info" ||
			 domain_nowww === "kadinlarkulubu.com" ||
			 domain_nowww === "arrse.co.uk") &&
			string_indexof(src, "/proxy.php?") >= 0) {
			return decodeURIComponent(src.replace(/.*?\/proxy\.php.*?[?&]image=([^&]*).*?$/, "$1"));
		}

		if (domain === "usa-grlk5lagedl.stackpathdns.com") {
			return src.replace(/(\/images\/[^/]*)\?.*$/, "$1?fm=pjpg");
		}

		if (domain === "static.sify.com") {
			return src.replace(/(\/cms\/image\/[^/]*)_[a-z]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "wikifeet.com") {
			return src.replace(/(\/profiles\/+[0-9]+)(\.[^/.]+)(?:[?#].*)?$/, "$1_fs$2");
		}

		if (domain === "thumbs.wikifeet.com") {
			return src.replace(/:\/\/[^/]*\//, "://pics.wikifeet.com/");
		}

		if (domain === "my.wikifeet.com") {
			obj = {
				url: src.replace(/\/(photos|avatars)\/+t([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/$1/$2"),
				bad_if: [{
					headers: {
						"content-length": "169698",
						"last-modified": "Thu, 01 Aug 2019 12:02:01 GMT"
					}
				}]
			};

			match = obj.url.match(/:\/\/[^/]+\/+avatars\/+([0-9]+)\./);
			if (match) {
				obj.extra = {page: "https://my.wikifeet.com/models/" + match[1]};
			}

			return obj;
		}

		if (domain === "static.spotboye.com") {
			return src.replace(/(_[0-9a-f]+)(?:_[a-z]+)?(\.[^/.]*)$/, "$1_original$2");
		}

		if (domain === "img.news.goo.ne.jp") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/cpimg\/([^/]*\.[^/]*)/, "http://$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "img.news.goo.ne.jp") {
			return src.replace(/\/picture\/+([a-z]+)\/+s((?:_\1-[0-9]{8}wow[0-9]+|_\1-[0-9]{10,}|[0-9]{10,})\.[^/.]*)(?:[?#].*)?$/, "/picture/$1/m$2");
		}

		if (domain_nowww === "redditstatic.com") {
			if (/^[a-z]+:\/\/[^/]+\/+sprite-[^/]+\./.test(src))
				return {
					url: src,
					bad: "mask"
				};
		}

		if (domain === "preview.redd.it") {
			return src.replace(/:\/\/preview\.redd\.it\/([^/.]*\.[^/.?]*)\?.*$/, "://i.redd.it/$1");
		}

		if (domain === "i.redd.it" && src.match(/^[a-z]+:\/\/[^/]*\/[0-9a-z]+\.[^-/._?#]*$/)) {
			return {
				url: src,
				is_original: true
			};
		}

		if (domain === "i.reddituploads.com") {
			newsrc = src.replace(/(:\/\/[^/]*\/[0-9a-f]+)\?.*$/, "$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (host_domain_nosub === "reddit.com" && options.element) {
			if (src === "" && options.element.tagName === "A" && options.element.classList.contains("image")) {
				return {
					url: origsrc,
					bad: "mask"
				};
			}
		}

		if ((domain_nosub === "redditmedia.com" ||
			 domain_nosub === "redd.it" ||
			 (domain_nosub === "reddit.com" && /:\/\/[^/]+\/+[ru]\/+/.test(src)) ||
			 string_indexof(origsrc, "blob:") === 0) &&
			host_domain_nosub === "reddit.com" &&
			options.element && options.do_request && options.cb) {
			newsrc = (function() {
				function checkimage(url) {
					url = urljoin(options.host_url, url, true);
					if (url.match(/^(?:[a-z]+)?:\/\/[^/]*\.redditmedia\.com\//) ||
						url.match(/^(?:[a-z]+)?:\/\/(?:i|(?:external-)?preview)\.redd\.it\//))
						return url;

					var opts = {
						fill_object: false,
						iterations: 3,
						use_cache: "read",
						use_api_cache: false,
						null_if_no_change: true,
						exclude_problems: [],
						do_request: function(){},
						cb: function(){}
					};

					opts = get_bigimage_extoptions(opts);
					if (bigimage_recursive(url, opts) !== null) {
						return url;
					}
				}

				var request_reddit = function(id, cb) {
					api_cache.fetch("reddit:" + id, cb, function(done) {
						options.do_request({
							method: "GET",
							url: "https://www.reddit.com/api/info.json?id=" + id,
							onload: function(result) {
								if (result.status !== 200) {
									console_error(result);
									return done(null, false);
								}

								try {
									var json = JSON_parse(result.responseText);
									var item = json.data.children[0].data;
									done(item, 60*60);
								} catch (e) {
									console_log(id);
									console_log(result);
									console_error(e);
								}
							}
						});
					});
				};

				function request(url) {
					var id;

					if (url.match(/^t3_/)) {
						id = url;
					} else {
						id = url.replace(/.*\/comments\/([^/]*)\/[^/]*(?:\/(?:\?.*)?)?$/, "$1");
						if (id === url) {
							return;
						}

						id = "t3_" + id;
					}

					request_reddit(id, function(item) {
						if (item.crosspost_parent) {
							return request(item.crosspost_parent);
						}

						var image = item.url;

						var obj = [];

						var preview_image;
						if (item.preview.images[0].variants.gif)
							preview_image = item.preview.images[0].variants.gif.source.url;
						else
							preview_image = item.preview.images[0].source.url;

						preview_image = preview_image.replace(/&amp;/g, "&");
						obj.push(preview_image);

						if (item.secure_media && item.secure_media.reddit_video) {
							var rv = item.secure_media.reddit_video;

							if (rv.fallback_url) {
								obj.unshift({
									url: rv.fallback_url,
									video: true
								});
							}

							if (rv.hls_url) {
								obj.unshift({
									url: rv.hls_url,
									video: "hls"
								});
							}

							if (rv.dash_url) {
								obj.unshift({
									url: rv.dash_url,
									video: "dash"
								});
							}
						}

						var checked = checkimage(image);
						if (checked) {
							image = checked;
							obj.unshift(image);
						}

						return options.cb(obj);
					});

					return {
						waiting: true
					};
				}

				if (options.element.parentElement && options.element.parentElement.parentElement) {
					if (options.element.tagName === "VIDEO") {
						var link_url = options.element.parentElement.getAttribute("data-link-url");
						if (link_url) {
							newsrc = request(link_url);
							if (newsrc)
								return newsrc;
						}
					}

					var doubleparent = options.element.parentElement.parentElement;
					newsrc = doubleparent.getAttribute("data-url");

					if (newsrc) {
						var checked = checkimage(newsrc);
						if (checked)
							return checked;
						else {
							var id = doubleparent.getAttribute("data-fullname");
							if (id) {
								newsrc = request(id);
								if (newsrc)
									return newsrc;
							}
						}
					}

					if (options.element.classList.contains("title") &&
						options.element.parentElement.classList.contains("title")) {
						try {
							var current = doubleparent.parentElement.parentElement;
							var id = current.getAttribute("data-fullname");
							if (id) {
								newsrc = request(id);
								if (newsrc)
									return newsrc;
							}
						} catch (e) {}
					}

					if (doubleparent.parentElement) {
						if (doubleparent.parentElement.tagName === "A" && options.do_request && options.cb) {
							newsrc = request(doubleparent.parentElement.href);
							if (newsrc)
								return newsrc;
						}

						if (options.element.parentElement.tagName === "A" &&
							(options.element.parentElement.getAttribute("target") === "_blank" ||
							 options.element.parentElement.classList.contains("PostThumbnail"))) {

							newsrc = options.element.parentElement.href;

							var checked = checkimage(newsrc);
							if (checked)
								return checked;
						}

						var current = options.element;
						var found = false;
						while ((current = current.parentElement)) {
							if (current.classList.contains("scrollerItem") ||
								current.getAttribute("data-test-id") === "post-content" ||
								current.classList.contains("Post__top")) {
								found = true;
								break;
							}
						}

						if (found) {
							var elements = current.getElementsByTagName("a");
							for (var i = 0; i < elements.length; i++) {
								var element = elements[i];
								if (element.getAttribute("data-click-id") === "body" ||
									element.getAttribute("data-click-id") === "timestamp" ||
									element.classList.contains("Post__absoluteLink")) {
									newsrc = request(element.href);
									if (newsrc)
										return newsrc;
									else
										return;
								}
							}
						}
					}
				}
			})();

			if (newsrc !== undefined && newsrc !== src)
				return newsrc;
		}

		if ((domain_nosub === "gstatic.com" ||
			 domain === "") &&
			host_domain_nosub.match(/^google\./) && options.element) {
			newsrc = (function() {
				var current = options.element;
				while ((current = current.parentElement)) {
					if (current.tagName !== "A")
						continue;

					if (!current.href.match(/\/imgres\?/))
						continue;

					newsrc = current.href.replace(/.*?\/imgres.*?[?&]imgurl=([^&]*).*?$/, "$1");
					if (newsrc !== src)
						return decodeURIComponent(newsrc);
				}

				current = options.element;
				if (current.tagName === "IMG") {
					while ((current = current.parentElement)) {
						if (current.tagName !== "DIV")
							continue;

						var tbnid = current.getAttribute("data-tbnid");
						if (!tbnid)
							continue;

						var regex = new RegExp("\\[[0-9]+\\s*,\\s*\"" + tbnid + "\"\\s*,\\s*\\[[^\\]]*\\]\\s*,\\s*\\[(\"[^\"]+\")\\s*,");
						var match = document.documentElement.innerHTML.match(regex); // TODO: optimize
						if (match) {
							return JSON_parse(match[1]);
						}
					}
				}

				current = options.element;
				while ((current = current.parentElement)) {
					if (current.tagName !== "A")
						continue;

					if (!current.href.match(/\/search\?/))
						continue;

					var parent = current.parentElement;
					if (parent.tagName !== "DIV")
						continue;

					var notranslate = parent.getElementsByClassName("notranslate");
					if (!notranslate.length) {
						parent = parent.parentElement;
						if (!parent || parent.tagName !== "DIV")
							break;

						notranslate = parent.getElementsByClassName("notranslate");
						if (!notranslate.length)
							continue;
					}

					for (var i = 0; i < notranslate.length; i++) {
						var el = notranslate[i];
						if (!el.innerHTML.match(/^ *{/))
							continue;

						var json = JSON_parse(el.innerHTML);

						if (json.ou.match(/^[a-z]+:/))
							return json.ou;

						break;
					}
				}
			})();
			if (newsrc !== undefined)
				return newsrc;
		}

		if (domain_nosub === "fbcdn.net" &&
			host_domain_nosub === "facebook.com" && options.element) {
			var element = options.element;

			if (element.tagName === "IMG") {
				while ((element = element.parentElement)) {
					if (element.tagName === "A") {
						var ploi = element.getAttribute("data-ploi");
						if (ploi && ploi.match(/^https?:\/\/[^/]*fbcdn\.net\//) && ploi !== src) {
							return ploi;
						}
					}
				}
			}
		}

		if (domain === "thumbnail.named.com" ||
			domain === "thumb.named.com") {
			return [
				src.replace(/:\/\/[^/]*\/.*?(\/+file\/+photo\/+.*)/, "://thumb.named.com/normal/resize/origin$1")
			];
		}

		if (host_domain_nosub === "bing.com" &&
			(domain === "" || domain_nosub === "bing.com") && options.element) {
			var current = options.element;
			while ((current = current.parentElement)) {
				if (current.tagName !== "A")
					continue;

				if (!current.href.match(/\/images\/search\?.*mediaurl=/))
					continue;

				newsrc = current.href.replace(/.*?\/search.*?[?&]mediaurl=([^&]*).*?$/, "$1");
				if (newsrc !== current.href && newsrc !== src) {
					newsrc = decodeuri_ifneeded(newsrc);
					return newsrc;
				}
			}
		}

		if (domain_nosub === "deviantart.net" &&
			(domain.match(/^pre[0-9]*\.deviantart\.net/) ||
			 domain.match(/^img[0-9]*\.deviantart\.net/)) &&
			src.match(/\/[a-z0-9]\/[0-9]+\/[0-9a-z]+\/[0-9a-z]+\/[0-9a-z]+\/[^/]*-[0-9a-z]+\.[^/.]*$/) &&
			options && options.do_request && options.cb) {

			newsrc = (function() {
				var id = src.replace(/.*-([0-9a-z]+)\.[^/.]*$/, "$1");

				return common_functions.deviantart_fullimage(options, api_cache, src, id, options.cb);
			})();

			return {
				"waiting": true
			};
		}

		if (domain_nowww === "huangdi5000.com") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/p\.php.*?[?&]p=(.*)$/, "$1"));
		}

		if (domain_nowww === "popcornfor2.com" && string_indexof(src, "/upload/") >= 0) {
			return src.replace(/\/news-[a-z]+-([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/news-full-$1");
		}

		if (domain === "img.daily.co.kr") {
			return src.replace("/content_watermark/", "/content/");
		}

		if (domain === "file.gamedonga.co.kr") {
			return src.replace(/(\/files\/[0-9]+\/[0-9]+\/[0-9]+\/)[a-z]\/([^/]*)$/, "$1$2");
		}

		if (domain === "jrimage.dongascience.com") {
			return src.replace(/(\/[0-9]+)_thumb(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "cloud.tcusercontent.net") {
			return src.replace(/(-[0-9]+-[0-9a-f]+)-[0-9]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "image.entertainment-topics.jp" ||
			domain === "image.code-file.jp" ||
			domain === "image.make-book.jp") {
			return src.replace(/(:\/\/[^/]*\/)(item\/image\/|article\/)[a-z]+(\/[^/]*)$/, "$1$2original$3");
		}

		if ((domain_nosub === "styapokupayu.ru" &&
			 string_indexof(src, "/images/") >= 0) ||
			(domain_nosub === "yapokupayu.ru" &&
			 string_indexof(src, "/system/images/") >= 0)) {
			return src.replace(/(\/[0-9]+)_[a-z]+(\.[^/.]*)$/, "$1_original$2");
		}

		if (domain_nowww === "thai.ac") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/news\/tn\/.*?[?&]fn=([^&]*).*?/, "$1"));
		}

		if (domain_nosub === "styleshare.kr" &&
			domain.match(/usercontents(?:-[a-z])?\.styleshare\.kr/)) {
			return src.replace(/(\/images\/[0-9]+\/)[-0-9]+x[-0-9]+(?:\?.*)?$/, "$1original");
		}

		if (domain === "images.vingle.net") {
			return src.replace(/(:\/\/[^/]*\/upload\/)t_[^/]*\/([^/]*)$/, "$1$2");
		}

		if (domain === "media.vingle.net") {
			newsrc = src.replace(/(\/images\/+(?:ca|us))_s\//, "$1_l/");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/(\/images\/+(?:ca|us))_l\//, "$1_xl/");
			if (newsrc !== src)
				return newsrc;

			return {
				url: src,
				headers: {
					Referer: ""
				}
			};
		}

		if (domain === "www.anewsa.com") {
			return src.replace(/(_images\/[0-9]+\/[0-9]+\/[0-9]+\/)mark(\/[^/]*)$/, "$1original$2");
		}

		if (domain === "img.newspim.com") {
			return src
				.replace(/(\/[0-9]+)_(?:t[a-z0-9]|w)(\.[^/.]*)$/, "$1$2")
				.replace(/(\/magazine\/+[0-9]{4}\/+[0-9]{2}\/+[0-9]+_[0-9a-f]+)_thumb(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "www.insectidentification.org") {
			return src.replace(/(\/imgs\/[a-z]+\/)thumbnails\//, "$1");
		}

		if (domain_nowww === "a-z-animals.com") {
			return src.replace(/\/images\/[0-9]+x[0-9]+\//, "/images/original/");
		}

		if (domain_nosub === "nicematin.com" &&
			domain.match(/cdn\.static[0-9]*\.nicematin\.com/)) {
			return src.replace(/(:\/\/[^/]*\/media\/[^/]*\/)[^/]*(\/[0-9]+\/[0-9]+\/[^/]*)$/, "$1original$2");
		}

		if (domain_nosub === "fbcdn.net" || domain_nosub === "cdninstagram.com") {

			newsrc = remove_queries(src, ["se"]);
			if (newsrc !== src)
				return newsrc;

			newsrc = remove_queries(src, ["_nc_cat", "_nc_rid", "efg", "ig_cache_key"]);
			if (newsrc !== src)
				return newsrc;
		}

		if (((domain_nosub === "fbcdn.net" && domain.match(/^instagram\./)) ||
			 domain_nosub === "cdninstagram.com") &&
			host_domain_nosub === "instagram.com" && options.element &&
			options.do_request && options.cb) {
			newsrc = (function() {
				var info = common_functions.instagram_find_el_info(document, options.element, options.host_url);
				return common_functions.instagram_parse_el_info(api_cache, options.do_request, options.rule_specific.instagram_use_app_api, info, options.cb);
			})();
			if (newsrc !== undefined)
				return newsrc;
		}

		if (domain_nowww === "instagram.com") {
			if (/\/static\/+bundles\/+[^/]+\/+sprite_/.test(src)) {
				return {
					url: src,
					bad: "mask"
				};
			}
		}

		if (domain_nosub === "instagram.com" &&
			src.match(/:\/\/[^/]*\/+p\/+[^/]*\/+media\/*[?]size=[a-z](?:[&#].*)?$/)) {
			return src.replace(/\/media.*?[?&]size=[a-z].*?$/, "/media?size=l");
		}

		if (domain_nosub === "instagram.com" &&
			!(domain_nowww === "instagram.com" &&
			  (src.match(/:\/\/[^/]*\/p\/[^/]*/) ||
			   src.match(/:\/\/[^/]*\/[-a-zA-Z0-9_.]+(?:\?.*)?$/)))) {
			return {
				url: src,
				headers: {
					"Referer": "https://www.instagram.com"
				}
			};
		}

		if (host_domain_nowww === "discordapp.com" && (domain_nosub === "fbcdn.net" || domain_nosub === "cdninstagram.com") && options.element && options.cb && options.do_request) {
			if (options.element.tagName === "IMG" && options.element.parentElement && options.element.parentElement.tagName === "A") {
				var aparent = options.element.parentElement;
				var aparent_class = aparent.getAttribute("class");
				if (string_indexof(aparent_class, "embedMedia") >= 0) {
					var divparent = aparent.parentElement;
					if (divparent) {
						var divs = divparent.getElementsByTagName("div");
						for (var i = 0; i < divs.length; i++) {
							var ourdiv_class = divs[i].getAttribute("class");
							if (string_indexof(ourdiv_class, "embedTitle") >= 0) {
								var our_as = divs[i].getElementsByTagName("a");

								if (our_as.length === 1 && /instagram\.com\//.test(our_as[0].href)) {
									var info = [{
										type: "post",
										subtype: "link",
										url: our_as[0].href,
										image: src,
										element: options.element
									}];

									return common_functions.instagram_parse_el_info(api_cache, options.do_request, options.rule_specific.instagram_use_app_api, info, options.cb);
								}

								break;
							}
						}
					}
				}
			}
		}

		if (domain === "cdns.klimg.com" ||
			domain === "cdn.klimg.com") {
			return src
				.replace(/(:\/\/[^/]*\/+[^/]*\.[^/]*\/+)resized\/+[0-9]+x(?:[0-9]+)?\/+/, "$1")
				.replace(/\/resized\/[0-9]+x(?:[0-9]+)?\//, "/kapanlagi.com/")
				.replace(/(\/g\/.\/.\/[^/]*\/)t\/([^/]*)$/, "$1$2")
				.replace(/(\/p\/+[^/]*\/+)[0-9]+x[0-9]+\/+([^/]*$)/, "$1$2")
				.replace(/\/t\/+([^/]*\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nosub === "nintendo-europe.com" &&
			domain.match(/^cdn[0-9]*\./)) {
			return src.replace(/_image[0-9]*[wh](\.[^/.]*)$/, "$1");
		}

		if (domain_nosub === "15min.lt" &&
			domain.match(/s[0-9]*\.15min\.lt/) &&
			string_indexof(src, "/images/") >= 0) {
			return src.replace(/\/[a-z]+(\/[^/]*)$/, "/original$1");
		}


		if (domain_nosub === "lazygirls.info" &&
			domain.match(/^img[0-9]*\.lazygirls\.info/)) {
			return src.replace(/:\/\/.*\/([^/.]*)\.[^/]*$/, "://lzimages.lazygirls.info/$1");
		}

		if (domain === "lzimages.lazygirls.info") {
			return src.replace(/\.(?:thumb|sized)$/, "");
		}

		if (domain === "image.way2enjoy.com") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/misc\/lazyg\/.*?\/([^/]*)\.[^/.]*\.[^/.]*$/,
							   "http://lzimages.lazygirls.info/$1");
		}

		if (domain === "www.mobtada.com") {
			return src.replace(/(:\/\/[^/]*\/)resize.*?[?&]src=([^&]*).*?$/, "$1$2");
		}

		if (domain_nowww === "art-dept.com") {
			return src.replace(/\/cache\/([^/]*)_[0-9]+(\.[^/.]*)$/, "/$1$2");
		}

		if (domain === "static.stylosophy.it" ||
			domain === "static.ellahoy.es" ||
			domain === "image.nanopress.it") {
			newsrc = src.replace(/(:\/\/[^/]*\/)(r\/+|[^/]+\/+fotogallery\/+)[0-9]+[xX][0-9]+\//, "$1$2999999999999X0/");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/r\/+[0-9]+[xX][0-9]+\/+([^/]*\.[^/]*\/)/, "http://$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "static.stylosophy.it" ||
			domain === "static.ellahoy.es") {
			return src.replace(/(:\/\/[^/]*\/)[0-9]+X[0-9]+\/+www\/+/, "$1999999999999X0/www/");
		}

		if (amazon_container === "nikeinc" &&
			string_indexof(src, "/assets/") >= 0) {
			return src.replace(/_hd_[0-9]+(\.[^/.]*)$/, "_original$1");
		}

		if ((domain_nowww === "mitazamagica.com" ||
			 domain === "mengqi.chenggong.it") &&
			string_indexof(src, "/Shops/") >= 0) {
			return src.replace(/_(?:s|m|ml)(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "imgbase.info" &&
			string_indexof(src, "/images/safe-wallpapers/") >= 0) {
			newsrc = src.replace(/\/[a-z]+_([0-9]+[^/]*)$/, "/$1");
			return {
				url: newsrc,
				headers: {
					Referer: "https://www.wallpapervortex.com/wallpaper-" + src.replace(/\/[a-z]+_([0-9]+[^/]*)\.[^/.]*$/, "$1") + ".html"
				}
			};
		}

		if (domain_nowww === "paperlief.com" ||
			domain_nowww === "celebritywc.com" ||
			domain_nowww === "ya-webdesign.com" ||
			domain_nowww === "quotationof.com" ||
			domain_nowww === "7desktopthemes.com" ||
			domain_nowww === "eskipaper.com") {
			return src.replace(/\/images[0-9]+_([0-9]+)?\//, "/images/");
		}

		if (domain_nosub === "anidb.net" &&
			domain.match(/img[0-9]*(?:-[a-z]+)?\.anidb\.net/)) {
			return src.replace(/\/thumbs\/[0-9]+x[0-9]+(\/[0-9]+\.[^/.-]*)-thumb\.[^/.]*$/, "$1");
		}

		if (domain === "web-ace.jp") {
			return src.replace(/\/rp\/[0-9_]+\/[0-9]+_[0-9]+\/img\//, "/img/");
		}

		if (domain_nowww === "classy-girls.com") {
			return src.replace(/\/images\/+thumbnails\//, "/images/");
		}

		if (domain === "staticdelivery.nexusmods.com") {
			return src.replace(/\/images\/+thumbnails\//, "/images/");
		}

		if (domain === "forums.nexusmods.com") {
			return src.replace(/\/uploads\/profile\/photo-thumb-([0-9]+)/, "/uploads/profile/photo-$1");
		}

		if ((domain_nowww === "kb4images.com" ||
			 domain_nowww === "digitalimagemakerworld.com" ||
			 domain_nowww === "99desktopwallpapers.com") &&
			string_indexof(src, "/images/") >= 0) {
			return src.replace(/-small(\.[^/.]*)$/, "$1");
		}

		if (domain === "pics.me.me") {
			return src.replace(/\/thumb_([^/]*-[0-9]+\.[^/.]*)$/, "/$1");
		}

		if (domain === "image.winudf.com") {
			return src
				.replace(/\?[wh]=[0-9]+&/, "?")
				.replace(/&[wh]=[0-9]+/, "");
		}

		if (domain === "i.supload.com") {
			return src.replace(/\/[0-9]+x[0-9]+\//, "/");
		}

		if (domain_nosub === "yaplakal.com" &&
			domain.match(/s[0-9]*\.yaplakal\.com/)) {
			return src.replace(/\/pics\/pics_[a-z]+\//, "/pics/pics_original/");
		}

		if (domain === "static.comicvine.com" ||
			domain === "static.giantbomb.com" ||
			domain === "static.gamespot.com" ||
			domain_nosub === "cbsistatic.com") {
			newsrc = src.replace(/(:\/\/[^/]+\/+)uploads\/+[^/]+\/+((?:mig\/+(?:[0-9]\/+){4}|[0-9]+\/+[0-9]+\/+)[0-9]+(?:-[^/]+)?\.)/, "$1uploads/original/$2");
			if (newsrc !== src)
				return add_extensions(newsrc);
		}


		if ((domain === "i.ibb.co" ||
			domain === "image.ibb.co" ||
			domain === "thumb.ibb.co" ||
			domain === "preview.ibb.co") &&
			options && options.cb && options.do_request) {
			match = src.match(/^[a-z]+:\/\/[^/]*\/+([-_A-Za-z0-9=]+)\/+([^?#]*?)(\.[^/.]*)?(?:[?#].*)?$/);
			if (match) {
				options.do_request({
					method: "GET",
					url: "https://ibb.co/" + match[1],
					onload: function(result) {
						if (result.readyState === 4) {
							var tmatch = result.responseText.match(/CHV\.obj\.image_viewer\.image\s*=\s*{.*?[^a-zA-Z0-9_]url:["'](.*?)["']/);
							if (tmatch) {
								return options.cb({
									url: tmatch[1],
									is_original: true,
									extra: {
										page: urljoin(result.finalUrl + "/", tmatch[1].replace(/.*\/([^/.]*).*?$/, "$1"), true)
									}
								});
							}

							options.cb(null);
						}
					}
				});

				return {
					waiting: true
				};
			}
		}


		if (domain === "cdn.imagepush.to") {
			return src.replace(/(:\/\/[^/]*\/)(?:in|out)\/[0-9]+x[0-9]+\//, "$1");
		}

		if (domain_nosub === "luscious.net" && /^w[0-9]+\./.test(domain)) {
			return src.replace(/^[a-z]+:\/\/[^/]+\/+/, "https://cdnio.luscious.net/");
		}

		if (domain_nosub === "luscious.net" &&
			domain.match(/cdn[a-z]*\.luscious\.net/)) {
			newsrc = src
				.replace(/\.[0-9]+x[0-9]+(\.[^/]*)$/, "$1")
				.replace(/\/resized\/[0-9]+\//, "/");
			if (newsrc !== src) {
				return add_full_extensions(newsrc);
			}
		}

		if (domain === "d2pqhom6oey9wx.cloudfront.net") {
			return src.replace(/\/img_resize\//, "/img_original/");
		}

		if (domain === "i.gyazo.com") {
			return src.replace(/\/thumb\/[0-9]+\/([0-9a-f]+)-([a-z]+)\.[^/.]*$/, "/$1.$2");
		}

		if (domain === "cdn.theatlantic.com") {
			newsrc = src.replace(/\/thumbor\/.*\/(?:assets\/+)?media\/+/, "/assets/media/");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "cdn.theatlantic.com" &&
			src.match(/\/assets\/+media\/+(?:[^/]*\/)?img\//)) {
			return src.replace(/\/[^/.]*(\.[^/.]*)$/, "/original$1");
		}

		if (domain === "media.tag24.de") {
			return src.replace(/(:\/\/[^/]*\/)[0-9]+\//, "$10/");
		}

		if ((amazon_container && amazon_container.match(/^customink-iotw/)) ||
			domain === "d2fzf9bbqh0om5.cloudfront.net") {
			return src.replace(/(\/images\/[0-9]+\/)[^/]*\//, "$1original/");
		}

		if (domain === "www.mypokecard.com") {
			return src.replace(/\/galery\/thumbs\//, "/galery/");
		}

		if (domain === "media.women.com" &&
			string_indexof(src, "/images/images/") >= 0) {
			return src.replace(/\/[a-z]+\/([^/]*)$/, "/original/$1");
		}

		if (domain_nowww === "alternathistory.com") {
			return src.replace(/\/files\/+resize(\/+.*\/+[^/]*)-[0-9]+x[0-9]+(\.[^/.]*)$/, "/files$1$2");
		}

		if (domain === "img.velvet.by") {
			return src.replace(/\/files\/+resize\/+(userfiles\/+[0-9]+\/+[^/]*)-[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "/files/$1$2");
		}

		if (domain_nowww === "mtvwe.com") {
			return src
				.replace(/\/vpic\.php.*?[?&]f=([^&]*).*?$/, "/$1")
				.replace(/(:\/\/[^/]*\/)\/*/, "$1");
		}

		if (domain === "yahoo.tw.weibo.com") {
			return src.replace(/(:\/\/[^/]+\/)[a-z]+\//, "$1large/");
		}

		if (domain === "mobile.pic.people.com.cn") {
			return src.replace(/\/thumbs\/[0-9]+\/[0-9]+\//, "/");
		}

		if (domain === "images.asianstar.cz") {
			return src.replace(/\/[a-z]+_([^/]*)$/, "/$1");
		}

		if (domain === "images-cache.asianstar.cz") {
			return src.replace(/:\/\/[^/]*\//, "://images.asianstar.cz/");
		}

		if (domain === "picservice.qimai.cn") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/get\//, "");
		}

		if (domain_nowww === "wxzixun.com" ||
			domain_nowww === "zhihutai.com") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/p\//, "");
		}

		if (domain === "mediaday.co.kr" ||
			domain_nowww === "secretvt.com" ||
			domain === "file.candlemystar.com" ||
			domain_nowww === "5rs-1.com") {
			return src.replace(/\/cache\/+([_a-z]+\/+[0-9]{4}\/+[0-9]{2}\/+)thumb-([0-9a-f]+(?:_[0-9]+)?)_[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "/$1$2$3");
		}

		if (domain === "up.kpop.re") {
			return src.replace(/(:\/\/[^/]*\/)[a-z]+(\/[0-9a-f]{2}\/[0-9a-f]+\.[^/.]*)(?:\/[^/]*)?$/, "$1src$2");
		}

		if (domain === "st.kp.yandex.net" &&
			string_indexof(src, "/images/") >= 0) {
			return src
				.replace(/_iphone\/+iphone[0-9]+_([0-9]+\.[^/.]*)(?:[?#].*)?$/, "_original/$1")
				.replace(/\/[a-z]+_([0-9]+(?:_[^/]*)?\.[^/.]*)$/, "/$1");
		}

		if (domain_nowww === "startfilm.ru" && string_indexof(src, "/images/") >= 0) {
			return src.replace(/\/sm_([0-9]+_[^/]*)$/, "/$1");
		}

		if (domain_nowww === "publicfeet.com") {
			return src.replace(/\/img\/t([0-9]+\.[^/.]*)$/, "/img/$1");
		}

		if (domain_nowww === "twatis.com") {
			return src.replace(/\/thumb(\.[^/.]*)$/, "/photo$1");
		}

		if (domain === "images.starlets.photos") {
			return src.replace(/(\/[0-9]+-)[a-z]+(\.[^/.]*)$/, "$1original$2");
		}

		if (domain === "s.smutty.com") {
			return src.replace(/\/[a-z]\/+([^/]*)(?:[?#].*)?$/, "/b/$1");
		}

		if (domain_nowww === "welovesexyfeet.com" ||
			domain_nowww === "podolatria.net" ||
			domain === "img.ibxk.com.br") {
			return src.replace(/-t[0-9]+x[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "hotflick.net" ||
			amazon_container === "hotflicknet") {
			return src
				.replace(/(:\/\/[^/]*\/.*\/[0-9]+\/)tn\/[0-9]+_[a-z]+\/([^/]*)$/, "$1$2")
				.replace(/\/+(?:tn[0-9]+|Thumb)\/+([^/]*)(?:[?#].*)?$/, "/$1")
				.replace(/:\/\/(?:[^/.]*-)?s3\.amazonaws\.com\/hotflicknet\/(.*\/)tn\/([^/]*)$/, "://hotflicknet.s3.amazonaws.com/$1$2")
				.replace(/:\/\/[^/]*\/(.*\/)tn\/([^/]*)$/, "://hotflicknet.s3.amazonaws.com/$1$2");
		}

		if (domain_nowww === "seniorclassaward.com" ||
			domain_nowww === "zocalo.com.mx") {
			return src.replace(/\/images\/sized(\/images\/.*)-[0-9]+x[0-9]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "hentairing.com") {
			return src
				.replace(/\/(?:width|height)-[0-9]+\/([^/]*)(?:[?#].*)?$/, "/$1")
				.replace(/\?.*/, "");
		}

		if (domain_nosub === "enterdesk.com" &&
			domain.match(/^up\./)) {
			return src.replace(/\/edpic_[0-9]+(?:_[0-9]+)?\//, "/edpic_source/");
		}

		if (domain_nosub === "gtimg.com" &&
			domain.match(/img[0-9]*\.gtimg\.com/)) {
			return src.replace(/_[0-9]+x[0-9]+(?:_[0-9]+)?(\.[^/.]*)$/, "$1");
		}

		if (domain_nosub === "zol-img.com.cn") {
			return src
				.replace(/(:\/\/[^/]*\/)t_[0-9a-z]+\//, "$1")
				.replace(/(\/article\/[0-9]+)_[0-9]+x[0-9]+\//, "$1/");
		}

		if (domain === "img.faxing11.com" ||
			domain === "img.pipimi.com" ) {
			return src.replace(/^[a-z]+:\/\/[^/]*\/img\//, "http://");
		}

		if (domain === "static.congnghe.vn" ||
			domain_nowww === "congnghe.vn") {
			newsrc = src.replace(/\/srv_thumb\.ashx.*?[?&]f=([^&]*).*/, "/$1");
			if (newsrc !== src)
				return newsrc.replace(/\\/g, "/");

			return src.replace(/\/[^/]*-([0-9]+\.[^/.]*)\.[0-9]+\.[0-9]+\.[^/.]*$/, "/$1");
		}

		if (domain === "images.foody.vn") {
			return src.replace(/\/s[0-9]+x[0-9]+\/([^/]*)$/, "/s/$1");
		}

		if (domain === "programma.sorrisi.com" ||
			domain_nowww === "shamra.sy") {
			return src.replace(/\/uploads\/+media\/+cache\/+[^/]*\/+uploads\//, "/uploads/");
		}

		if (domain_nowww === "vidofa.com") {
			newsrc = src.replace(/.*?\/module\/+thumb\/+sr\.php\?(?:.*&)?src=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "free4kwallpapers.com") {
			return src
				.replace(/\/uploads\/+wallpaper\/+([0-9]{2}\/.*)-[0-9]+x[0-9]+-wallpaper(\.[^/.]*)$/, "/uploads/originals/20$1$2")
				.replace(/\/uploads\/+originals\//, "/no-watermarks/originals/");
		}

		if (domain_nosub === "webnode.com.br" ||
			domain_nosub === "webnode.cz" ||
			domain_nosub === "webnode.sk" ||
			domain_nosub === "webnode.com") {
			return src.replace(/(:\/\/(?:files\.[^/]*\.[^/.]*\.[^/]*\/+|[^/]*\/+_files\/+))(?:[^/]*_)?([0-9]+-[0-9a-f]+(?:-[a-z]+)?\/)+(?:[0-9]+\/+)?([^/]*)$/, "$1$2$3");
		}

		if (amazon_container === "cdn.roosterteeth.com") {
			return src.replace(/(\/images\/[-0-9a-f]+\/)[a-z]+(\/[^/]*)$/, "$1original$2");
		}

		if (domain_nowww === "wallpaperk.com") {
			return src.replace(/\/resoluciones\/[0-9]+\/([^/]*)_[0-9]+x[0-9]+_([0-9]+\.[^/.]*)$/, "/$1-$2");
		}

		if (domain === "pic.58pic.com" ||
			domain_nosub === "90sjimg.com" ||
			domain === "pic.qiantucdn.com") {
			return src.replace(/!.*/, "");
		}


		if (domain === "img.wallpapersafari.com" ||
			domain === "cdn.wallpapersafari.com") {
			newsrc = src
				.replace(/\/img[0-9]+\//, "/")
				.replace(/\/desktop\/+[0-9]+\/+[0-9]+\//, "/");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/^[a-z]+:\/\/[^/]+\/+[0-9]+\/+[0-9]+\/+([^/.]+)\.[^/.]+$/);
			if (match) {
				return {
					url: src,
					extra: {
						page: "https://www.wallpapersafari.com/w/" + match[1]
					}
				};
			}
		}

		if (domain === "mcdn.wallpapersafari.com") {
			return src.replace(/:\/\/mcdn(\.[^/]*\/)(?:medium|small)\/+/, "://cdn$1");
		}

		if (domain_nosub === "fjcdn.com" &&
			domain.match(/static[0-9]*\.fjcdn\.com/)) {
			return src
				.replace(/\/thumbnails\/comments\//, "/comments/")
				.replace(/\/pictures\/[^/]*_([0-9a-f]{2})([0-9a-f]{2})([0-9a-f]+_[0-9]+\.[^/.]*)$/, "/large/pictures/$1/$2/$1$2$3");
		}

		if (googlestorage_container === "kendam-148811.appspot.com") {
			return src.replace(/\/thumb[0-9]+\/tn[0-9]+_/, "/temp/");
		}

		if (domain === "image-cdn.hypb.st") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/([^&]*).*/, "$1"));
		}

		if (domain_nowww === "productionparadise.com") {
			return src.replace(/(\/photos\/[0-9]+\/)[a-z]+(\/[^/]*)$/, "$1original$2");
		}

		if (domain_nosub === "cargocollective.com" &&
			domain.match(/payload[0-9]*\.cargocollective\.com/)) {
			return src.replace(/_[0-9]{3,4}(\.[^/.]*)$/, "$1");
		}

		if (domain_nosub === "imagebam.com" &&
			(domain.match(/^thumbs[0-9]*\.imagebam\.com/) ||
			 domain.match(/^thumbnails[0-9]*\.imagebam\.com/)) &&
			options && options.cb && options.do_request) {
			id = src.replace(/.*\/([0-9a-f]+)\.[^/.]*$/, "$1");
			if (id !== src) {
				page = "http://www.imagebam.com/image/" + id;
				options.do_request({
					url: page,
					method: "GET",
					onload: function(resp) {
						if (resp.readyState === 4) {
							var match = resp.responseText.match(/<meta *property="og:image" *content="([^"]*)"/);
							if (match) {
								options.cb({
									url: match[1],
									is_original: true,
									extra: {
										page: page
									}
								});
							} else {
								options.cb(null);
							}
						}
					}
				});

				return {
					waiting: true
				};
			}
		}

		if (domain_nosub === "imagebam.com") {
			if (/\/nohotlinking\.jpg(?:[?#].*)?$/.test(src)) {
				return {
					url: src,
					bad: true
				};
			}
		}

		if (domain_nosub === "imagevenue.com" &&
			domain.match(/^img[0-9]+\.imagevenue\.com/) &&
			src.match(/\/th_([^/]*)$/) &&
			options && options.cb && options.do_request) {
			id = src.replace(/.*\/th_([^/]*?)(?:[?#].*)?$/, "$1");
			if (id !== src) {
				var requrl = src.replace(/(:\/\/[^/]*\/).*/, "$1img.php?image=" + id);

				var do_req = function (requrl, first) {
					var headers = {
						Referer: requrl
					};

					options.do_request({
						url: requrl,
						method: "GET",
						headers: headers,
						onload: function (resp) {
							if (resp.readyState !== 4) {
								return;
							}

							if (resp.status !== 200) {
								console_error("Bad status", resp);
								return options.cb(null);
							}

							var match = resp.responseText.match(/<img *id=['"]thepic['"][^>]* (?:src|SRC)=['"]([^"']*)['"]/);
							if (match) {
								options.cb({
									url: urljoin(resp.finalUrl, match[1], true),
									is_original: true,
									extra: {
										page: requrl
									}
								});
							} else {
								console_error("Unable to find match", resp);

								if (first) {
									return do_req(resp.finalUrl, false);
								} else {
									options.cb(null);
								}
							}
						}
					});
				};

				do_req(requrl, true);

				return {
					waiting: true
				};
			}
		}

		if (domain === "static.cinemagia.ro") {
			return src.replace(/\/img\/resize\/db\/(.*\/(?:[^/]*?-)?[0-9]+[a-z])-[^/.]*?(\.[^/.]*)$/, "/img/db/$1$2");
		}

		if (domain === "static.kinokopilka.pro") {
			return src.replace(/(\/[0-9]+)_[a-z]+(\.[^/.]*)$/, "$1_original$2");
		}

		if (domain === "d1uzk9o9cg136f.cloudfront.net") {
			return src.replace(/(\/[a-f0-9]+)_[a-z]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "static.azteca.com") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/crop\/crop\.php.*?[?&]img=([^&]*).*?$/, "$1"));
		}

		if (domain === "www.tasoeur.biz") {
			return src.replace(/\/thumbs\/([^/]*)\.thumb(\.[^/.]*)$/, "/$1$2");
		}

		if (domain_nowww === "taipeitimes.com") {
			return src.replace(/(\/images\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2})thumbs\/+/, "$1");
		}

		if (domain === "imageproxy.ifcdn.com") {
			return src.replace(/:\/\/[^/]*\/[^/]*(\/images\/[^/]*)$/, "://img.ifcdn.com$1");
		}

		if (domain === "resource.mingweekly.com") {
			return src.replace(/\/userfiles\/sm\/sm[0-9]+_images/, "/userfiles/images");
		}

		if (domain_nosub === "allegroimg.com") {
			return src.replace(/(:\/\/[^/]*\/)s[0-9]+\//, "$1original/");
		}

		if (domain === "mmc.tirto.id") {
			return src.replace(/\/image\/otf\/[0-9]+x[0-9]+\//, "/image/");
		}

		if (domain === "ia.tmgrup.com.tr" ||
			domain === "iaahbr.tmgrup.com.tr") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/[^/]*\/(?:[0-9]+\/){5}[0-9]+\/?.*?[?&]u=([^&]*).*?$/, "$1"));
		}

		if (domain === "img.erogazou.co") {
			return src.replace(/\/thumb\//, "/photo/");
		}

		if (domain === "tracks.content.hardstyle.com") {
			return src.replace(/\/thumbs\/[0-9]+x[0-9]+\//, "/original/");
		}

		if ((domain_nosub === "thedjlist.com" && domain.match(/i[0-9]*\./)) ||
			domain === "static.inaturalist.org") {
			return src.replace(/(\/photos\/[0-9]+\/)[a-z]+(\.[^/.]*)$/, "$1original$2");
		}

		if (domain === "artist-assets.hubbardradio.com") {
			return src.replace(/_v[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain === "cdn.ebaumsworld.com" ||
			domain === "images.ebaumsworld.com") {
			return src.replace(/\/thumbs\/(?:picture|gallery)\//, "/mediaFiles/picture/");
		}

		if (domain_nosub === "kapook.com") {
			return src
				.replace(/\/rf\/[0-9]+\/[0-9]+\//, "/o/")
				.replace(/\/rq\/+[0-9]+\/+auto\/+[0-9]+\/+pagebuilder\/+/, "/pagebuilder/");
		}

		if (domain_nosub === "amcn.in" &&
			domain.match(/^cdn[0-9]*\.amcn\.in/)) {
			return src.replace(/^[a-z]+:\/\/[^/]*\/a\/([^/.]*\.[^/]*\/)/, "http://$1");
		}

		if (domain_nosub === "litlepups.net" &&
			domain.match(/^cdn[0-9]*\.litlepups\.net/)) {
			return src.replace(/:\/\/[^/]*\/resize\/(.*)\/[a-z]+-([^/]*)$/, "://cdn.litlepups.net/$1/$2");
		}

		if (domain_nowww === "wede-mail.com" ||
			domain_nowww === "warnerchappell.com") {
			return src.replace(/\/slir\/(?:[whc][0-9:]+(?:-[^/]*)|orig)\//, "/");
		}

		if (domain_nowww === "photorator.com" ||
			domain_nowww === "contrastspace.ru" ||
			domain_nowww === "galleon-realty.ru" ||
			domain_nowww === "teatrvmeste.ru" ||
			domain_nowww === "pornopics.co") {
			newsrc = src.replace(/\/photos\/(?:thumbs|small)\//, "/photos/images/");

			page = src.replace(/(\/photo)s\/+[^/]+\/+[^/]+-([0-9]+)\.[^/.]+(?:[?#].*)?$/, "$1/$2");
			if (page !== src) {
				return {
					url: newsrc,
					extra: {
						page: page
					}
				};
			} else {
				return newsrc;
			}
		}

		if (domain_nosub === "cdn-expressen.se") {
			return src
				.replace(/\/[0-9]+(?:@[0-9]+)?(\.[^/.]*)$/, "/original$1")
				.replace(/(\/[0-9a-f]+\/)[0-9]+x[0-9]+(\/[^/]*)$/, "$1annan$2");
		}

		if (domain === "images.csmonitor.com") {
			return src.replace(/(?:\?.*)?$/, "?alias=original");
		}

		if (domain === "www.incimages.com" ||
			domain_nowww === "firenewsfeed.com") {
			return src.replace(/\/image\/[0-9]+x[0-9]+\//, "/image/");
		}

		if (domain === "cached.imagescaler.hbpl.co.uk") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/resize\/[^/]*\/[0-9]+\/([^/]*\.[^/]*\/)/, "http://$1");
		}

		if (domain === "i.tuenlinea.com") {
			return src.replace(/(\/[^/]*(\.[^/.]*))\.img[wh]\.[0-9]+\.[0-9]+\.[^/.]*$/, "$1.imgo$2");
		}

		if (domain === "cdn-photos.extratv.com") {
			return src.replace(/_thumb(\.[^/.]*)$/, "_full$1");
		}

		if (domain === "media.extratv.com") {
			return src.replace(/-(?:[0-9]+x[0-9]+|[0-9]+[wh])(?:-[0-9]+)?(\.[^/.]*)$/, "$1");
		}

		if (domain === "i.wpimg.pl") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/(?:[A-Z]\/)?[0-9]+x(?:[0-9]+)?\//, "http://");
		}

		if (domain === "static.keptelenseg.hu") {
			return src.replace(/\/thumbs\/thumb\/p\//, "/p/");
		}

		if (domain_nosub === "wdfiles.com") {
			return src.replace(/\/local--resized-images\/(.*\.[^/.]*)\/[a-z]+\.[^/.]*$/, "/local--files/$1");
		}

		if (domain_nowww === "astrofactor.com") {
			return src.replace(/(\/imgs\/[^/]*\/)[0-9]+(\/[^/]*)$/, "$1full$2");
		}

		if (domain === "makeup.nuovogennarino.org") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/static\/[0-9]+\/img\/([^/.]+\.[^/]+)/, "http://$1");
		}

		if ((domain_nosub === "newsdog.today" && domain.match(/^img[0-9]*\./)) ||
			amazon_container === "img.newsdog.today") {
			return src.replace(/(\.newsdog\.today\/)[a-z_]+_([a-f0-9]+)$/, "$1origin_$2");
		}

		if (domain === "d1qikntta4cp8k.cloudfront.net") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/.*?[?&]url=([^&]*)$/, "$1"));
		}

		if (domain_nosub === "img.com.ua") {
			newsrc = src.replace(/\/[0-9]+x[0-9]+(\/[0-9a-f]\/[0-9a-f]{2}\/[0-9a-f]+\.[^/.]*)$/, "/orig$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "rs.img.com.ua") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/crop.*?[?&]url=([^&]*).*?$/, "$1");
			if (newsrc !== src) {
				return urljoin(src, decodeURIComponent(newsrc), true);
			}
		}

		if (domain === "img-hw.xvideos.com") {
			return src.replace(/(\/pic_[0-9]+)_[a-z]+(\.[^/.]*)$/, "$1_big$2");
		}

		if ((host_domain_nowww === "xvideos.com" || host_domain_nowww === "xnxx.com") && options.element && options.do_request && options.cb) {
			var our_host = host_domain_nosub;

			var full_videourl = function(videourl) {
				if (our_host === "xvideos.com") {
					return urljoin("https://www.xvideos.com/", videourl, true);
				} else if (our_host === "xnxx.com") {
					return urljoin("https://www.xnxx.com/", videourl, true);
				}
			};

			var parse_xvideos_page = function(cache_key, resp) {
				var matches = resp.responseText.match(/html5player\.set(.*)/g);
				if (!matches) {
					console_error(cache_key, "Unable to find matches", resp);
					return null;
				}

				var table = {};
				for (var i = 0; i < matches.length; i++) {
					var match = matches[i].match(/html5player\.set([a-zA-Z0-9]+)\('([^']+)'\);$/);
					if (!match) {
						continue;
					}

					table[match[1]] = decode_entities(match[2]);
				}

				return table;
			};

			var objify_xvideos_table = function(table) {
				var obj = {extra: {}};

				if (table.VideoTitle) {
					obj.extra.caption = table.VideoTitle;
				}

				if (table.VideoURL) {
					obj.extra.page = full_videourl(table.VideoURL);
				}

				var urls = [];
				if (table.VideoHLS) {
					urls.push({
						url: table.VideoHLS,
						video: "hls"
					});
				}

				if (table.VideoUrlHigh) {
					urls.push(table.VideoUrlHigh);
				}

				if (table.VideoUrlLow) {
					urls.push(table.VideoUrlLow);
				}

				return fillobj_urls(urls, obj);
			};

			var videoid_to_videourl = function(videoid) {
				if (our_host === "xvideos.com") {
					return "https://www.xvideos.com/video" + videoid + "/";
				} else if (our_host === "xnxx.com") {
					return "https://www.xnxx.com/video-" + videoid + "/";
				}
			};

			var query_xvideos_page = function(videoid, cb) {
				var cache_key = "xvideos:" + videoid;
				if (our_host === "xnxx.com")
					cache_key = "xnxx:" + videoid;

				api_cache.fetch(cache_key, cb, function(done) {
					options.do_request({
						url: videoid_to_videourl(videoid),
						method: "GET",
						onload: function(resp) {
							if (resp.readyState !== 4)
								return;

							if (resp.status !== 200) {
								console_error(cache_key, resp);
								return done(null, false);
							}

							var parsed_table = parse_xvideos_page(cache_key, resp);
							if (!parsed_table) {
								return done(null, false);
							}

							var obj = objify_xvideos_table(parsed_table);
							if (!obj) {
								console_error(cache_key, "Unable to get URLs from", parsed_table, resp);
								return done(null, false);
							}

							return done(obj, 3*60*60);
						}
					});
				});
			};

			var get_videoid_from_url = function(url) {
				var regex = null;

				if (our_host === "xvideos.com") {
					regex = /:\/\/[^/]+\/+video([0-9]+)\//;
				} else if (our_host === "xnxx.com") {
					regex = /:\/\/[^/]+\/+video-([^-/._]+)\//;
				}

				if (!regex)
					return null;

				var match = url.match(regex);
				if (!match)
					return null;

				return match[1];
			}

			var get_xvideos_vidid_from_el = function(el) {
				var current = el;

				for (; current; current = current.parentElement) {
					if (current.tagName === "A") {
						var id = get_videoid_from_url(current.href);
						if (id) {
							return id;
						}
					}
				}
			};

			var vidid = get_xvideos_vidid_from_el(options.element);
			if (vidid) {
				query_xvideos_page(vidid, options.cb);

				return {
					waiting: true
				};
			}
		}

		if (domain === "camo.derpicdn.net") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/[0-9a-f]+\?url=([^&]*).*?$/, "$1"));
		}

		if (domain === "live.store.cmcm.com") {
			return src.replace(/(:\/\/[^/]*\/)[a-z]+\/(liveme\/)/, "$1$2");
		}

		if (domain === "media.timeout.com") {
			return src.replace(/(\/images\/[0-9]+\/)[0-9]+\/[0-9]+\/([^/]*)$/, "$1$2");
		}

		if (domain_nowww === "tokkoro.com") {
			return src.replace(/\/thumbs\//, "/picsup/");
		}

		if (domain === "www.wallpapermaiden.com") {
			newsrc = src.replace(/(\/image\/[0-9]+\/[0-9]+\/[0-9]+\/[^/]*)-resized(\.[^/.]*)$/, "$1$2");
			if (newsrc !== src)
				return newsrc;

			id = src.replace(/(\/wallpaper\/[0-9]+\/)download\/[0-9]+x[0-9]+\/([^/]*)\.[^/.]*(?:[?#].*)?$/,
							 "$1$2");
			if (id !== src &&
				options && options.cb && options.do_request) {
				options.do_request({
					url: id,
					method: "GET",
					onload: function(resp) {
						if (resp.readyState === 4) {
							var match = resp.responseText.match(/<meta *property="og:image" *content="([^"]*)"/);
							if (match) {
								options.cb(match[1]);
							} else {
								options.cb(null);
							}
						}
					}
				});
			}
		}

		if (domain_nowww === "2ch.hk" ||
			domain_nowww === "2ch.pm" ||
			domain_nowww === "shisharc.com") {
			return add_extensions(src.replace(/\/thumb\/([0-9]+\/[0-9]+)s(\.[^/.]*)$/, "/src/$1$2"));
		}

		if (domain === "s.heavenlynudes.net" ||
			domain === "s.nufap.com" ||
			domain === "s.fapsex.com" ||
			domain === "s.clickmyboobs.com" ||
			domain === "s.papajizz.com") {
			return src.replace(/\/thumbs\/([0-9a-f]+\.[^/.]*)$/, "/$1");
		}

		if (domain_nosub === "bbend.net" ||
			domain === "cdn.cnngreece.gr" ||
			domain_nowww === "funtime.gr" ||
			domain === "i.mommd.gr" ||
			domain === "i.rtpmd.gr" ||
			domain === "i.gosmd.gr" ||
			domain === "i.astmd.gr" ||
			domain === "i.onmmd.gr" ||
			domain === "i.lfmd.gr" ||
			domain === "i.qumd.gr") {
			newsrc = src
				.replace(/(\/media\/+.*\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9]+\/+)(?:facebook\/+facebook|(?:thumb|snapshot|facebook)\/+)/, "$1main/")
				.replace(/(\/media\/+.*\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9]+\/+photos\/+)[a-z]+\/+/, "$1full/");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "i.gosmd.gr") {
			if (/\/templates\/+gossip\/+images\/+placeholder\.jpg/.test(src))
				return {
					url: src,
					bad: "mask"
				};
		}

		if (domain === "static.fustany.com") {
			return {
				url: src.replace(/\/photo\/[a-z]+_([^/]*)$/, "/photo/$1"),
				headers: {
					Referer: "http://fustany.com/en/fashion/celebrity-style/front-row-celebrities-at-new-york-fashion-week-spring-2015"
				}
			};
		}

		if (domain_nosub === "protv.md" &&
			domain.match(/^assets[0-9]*\.protv/)) {
			return src
				.replace(/:\/\/[^/]*\/articles\/files\/thumbs\/[0-9]+x(?:[0-9]+)?(\/[0-9]+\.[^/.]*)$/,
						 "://retete.perfecte.md/assets/articles/images/original$1")
				.replace(/\/articles\/+files\/+thumbs\/+[0-9]+x(?:[0-9]+)?\/+/, "/articles/images/original/");
		}

		if (domain_nowww === "bgol.us" ||
			domain_nowww === "defenceforumindia.com") {
			return decodeURIComponent(src.replace(/.*?\/forum\/proxy\.php.*?[?&]image=([^&]*).*?$/, "$1"));
		}

		if (domain === "resources.tidal.com") {
			return src.replace(/\/[0-9]+x[0-9]+(\.[^/.]*)$/, "/origin$1");
		}

		if (domain === "i.kfs.io") {
			return src.replace(/\/(?:fit|cropresize)\/+[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "/original$1");
		}

		if (domain_nosub === "fuskator.com") {
			return {
				url: src
					.replace(/(:\/\/[^/]*\/)small\//, "$1large/"),
				can_head: false
			};
		}

		if (domain === "ssref.net") {
			return decodeURIComponent(src.replace(/.*?\/image_resize\.cgi.*?[?&]url=([^&]*).*?$/, "$1"));
		}

		if ((domain_nosub === "m1905.cn" && domain.match(/image[0-9]*\.m1905\.cn/)) ||
			domain_nowww === "ozanyerli.com" ||
			domain_nowww === "entline.cn" ||
			domain === "img.ixiumei.com" ||
			domain_nowww === "69876.com") {
			newsrc = src.replace(/\/(?:thumb_[0-9]+_[0-9]+_(?:[0-9]+_)?)?([0-9]+)(?:_watermark)?(\.[^/.]*)$/, "/$1$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nosub === "intermedia.ge"/* &&
			domain.match(/server[0-9]*\.intermedia\.ge/)*/) {
			return src
				.replace(/(\/article_images[0-9]*\/)[a-z]+\//, "$1large/")
				.replace(/\/pictures\/[a-z]+\//, "/pictures/original/");
		}

		if (domain_nowww === "sqshi.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/img\.php.*?[?&]imgurl=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "i.annihil.us") {
			return src.replace(/(\/[0-9a-f]+\/)detail(\.[^/.]*)$/, "$1clean$2");
		}

		if (domain === "pimg.mycdn.me" ||
			(domain_nosub === "sywcdn.net" && domain.match(/^s[0-9]*\./))) {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/getImage.*?[?&]url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "dissidia-france.com") {
			return src.replace(/\/small\/([^/]*)$/, "/big/$1");
		}

		if (domain === "cdn.asiatatler.com" ||
			domain_nowww === "indonesiatatler.com") {
			return src.replace(/_(?:cropped|resized)_[0-9]+x[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain_nosub === "stylowi.pl" &&
			domain.match(/^img[0-9]*\.stylowi\.pl/)) {
			return src.replace(/\/images\/items\/[a-z]\//, "/images/items/o/");
		}

		if (domain === "files.dals.media") {
			return src.replace(/__[a-z]+__(\.[^/.]*)$/, "__original__$1");
		}

		if (domain === "media.linkonlineworld.com") {
			return src.replace(/\/img\/large\//, "/img/original/");
		}

		if (amazon_container === "marieclairebucket") {
			return src.replace(/000[0-9]+?x[0-9]+(\.[^/.]*)$/, "000$1");
		}

		if (domain_nosub === "nh.ee" ||
			domain_nosub === "delphi.lv" ||
			domain_nosub === "delfi.ee" ||
			domain_nosub === "dcdn.lt") {
			return src.replace(/\/images\/pix\/[0-9]+x[0-9]+\//, "/images/pix/");
		}

		if (domain === "read.html5.qq.com" ||
			domain === "cdn.read.html5.qq.com") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/image.*?[?&]imageUrl=([^&]*).*?$/, "$1"));
		}

		if (domain === "img.chuansong.me" ||
			domain === "img.weiduba.net" ||
			(domain_nosub === "zx590.com" && domain.match(/^img[0-9]*\./)) ||
			(domain_nosub === "jinciwei.cn" && domain.match(/img[0-9]*\./))) {
			return src.replace(/:\/\/[^/]*\/(mmbiz(?:_[a-z]+)?)\//, "://mmbiz.qpic.cn/$1/");
		}

		if (domain === "cdn.iguang.co") {
			return src.replace(/:\/\/[^/]*\/[0-9a-f]+\/(mmbiz(?:_[a-z]+)?)\//, "://mmbiz.qpic.cn/$1/");
		}

		if (domain === "pic.kuaizhan.com") {
			return src.replace(/\/imageView\/v[0-9]*\/.*/, "");
		}

		if ((domain_nosub === "popcornnews.ru" && domain.match(/v[0-9]*\.popcornnews\.ru/)) ||
			domain === "static.kinoafisha.info") {
			return src
				.replace(/(:\/\/[^/]*\/+)k\/+[a-z_]+\/+[0-9]+(?:x[0-9]+)?\/+upload\//, "$1upload/")
				.replace(/(:\/\/[^/]*\/+upload\/+)_[0-9]+_[0-9]+_[0-9]+_([^/_.]*\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "amorq.com" ||
			domain_nowww === "prikolno.cc" ||
			domain_nowww === "interesno.cc" ||
			domain_nowww === "obaldenno.com") {
			return src.replace(/\/uploads\/tumb(\/[a-z]+\/[0-9]+\/[^/.]+)_tumb_[0-9]+(\.[^/.]*)$/, "/uploads$1$2");
		}

		if (domain_nosub === "pinme.ru" &&
			domain.match(/^cdn/)) {
			return src.replace(/\/+tumb\/+[0-9]+\/+photo\//, "/photo/");
		}

		if (domain_nosub === "kakprosto.ru") {
			return src.replace(/\/tumb\/+[0-9]+\/+images\//, "/images/");
		}

		if (domain_nosub === "postila.ru" &&
			domain.match(/^img[0-9]*\.postila\.ru/)) {
			return urljoin(src, decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/resize.*?[?&]src=([^&]*).*?$/, "$1")), true);
		}

		if (domain_nowww === "enolivier.com" ||
			domain_nowww === "fotoventasdigital.com") {
			return src.replace(/\/_thumb\/([^/]*)_[0-9]+x[0-9]+(\.[^/.]*)$/, "/_fullsize/$1$2");
		}

		if (domain === "ss.metronews.ru" ||
			domain === "ss.sport-express.ru") {
			return src.replace(/(\/userfiles\/materials\/[0-9]+\/[0-9]+\/)(?:[0-9]+x[0-9]+|[a-z]+)(\.[^/.]*)$/, "$1origin$2");
		}

		if (domain_nosub === "mtvnimages.com") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/uri\/mgid:file:(https?):shared:([^/:]*\/)/, "$1://$2");
		}

		if (domain_nowww === "vh1.com") {
			return {
				url: src,
				can_head: false
			};
		}

		if (domain === "user-uploads.aznude.com") {
			return src.replace(/(:\/\/[^/]*\/data\/)thumbs\//, "$1azncdn/");
		}

		if (domain_nowww === "bstars.eu" ||
			domain_nowww === "wstars.net" ||
			domain_nowww === "bstars.ru") {
			return src
				.replace(/(\/media\/+djcatalog2\/+images\/+item\/+[0-9]+\/+[^/]+\.[0-9]+)_[a-z](\.[^/.]*)$/, "$1$2")
				.replace(/(\/media\/+djcatalog2\/+images\/+[^/]+)_[a-z](\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "media.toofab.com" ||
			domain === "toofab.akamaized.net") { // doesn't work for all urls
			src = src.replace(/-[0-9]+w\.([^/.]*)/, ".$1");

			src = src.replace(/-[0-9]+x[0-9]+\.([^/.]*)/, ".$1");

			return src;
		}

		if (domain === "thumb.tvpalace.org") {
			return src.replace(/\/[wh][0-9]+_([^/]*\.[^/.]*)$/, "/$1");
		}

		if (domain_nowww === "mafab.hu") {
			return src.replace(/\/static\/thumb\/[wh][0-9]+\//, "/static/");
		}

		if (domain === "film.kinootziv.com" ||
			(domain_nosub === "moviesfan.org" && domain.match(/^static/)) ||
			domain_nowww === "kinokach.com") {
			return src.replace(/(:\/\/[^/]*\/source\/files\/[^/]*)_thumbs\//, "$1/");
		}

		if (amazon_container === "harmony-assets-live") {
			return {
				src: url,
				can_head: false
			};
		}

		if (domain === "cdn.kme.si") {
			return src.replace(/\/public\/images-cache\/[0-9X]+x[0-9X]+\/([0-9]+\/[0-9]+\/[0-9]+\/)[0-9a-f]+\/[0-9a-f]+\/([0-9a-f]+\.[^/.]*)$/,
							   "/public/images/$1$2");
		}

		if (domain_nosub === "slidely.com" &&
			domain.match(/cdn\.slidely\.com$/)) {
			return src.replace(/^[a-z]+:\/\/[^/]*\/img-proxy\//, "http://");
		}

		if (domain_nosub === "hochu.ua") {
			return src.replace(/\/thumbnails\/articles\/crop[a-z]*_[0-9]+x[0-9]+\//, "/images/articles/");
		}

		if (domain === "media.professionali.ru") {
			return src.replace(/(\/processor\/[a-z]+\/)[^/]*\//, "$1original/");
		}

		if (domain_nowww === "stars-naked.ru") {
			return src.replace(/(:\/\/[^/]*\/pictures\/[^/]*\/)[^/]*\/[a-z]+_([^/]*)$/, "$1$2");
		}

		if (domain_nowww === "avsforum.com" ||
			domain_nowww === "sitcomsonline.com") {
			return src.replace(/(\/photopost\/data\/[0-9]+\/)(?:thumbs|medium)\//, "$1");
		}

		if (domain_nowww === "pgmscreen.fr") {
			newsrc = decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/(?:protect|watermark)\.php.*?[?&]src=([^&]*).*?$/, "$1"));
			if (newsrc !== src)
				return newsrc;
			return src.replace(/\/galerie\/[a-z]+\/[a-z]+-/, "/galerie/grande/max-");
		}

		if (domain === "celeb.gate.cc") {
			return src.replace(/(:\/\/[^/]*\/media\/cache\/)[a-z]+\/upload\//, "$1original/upload/");
		}

		if (domain_nowww === "modagid.ru") {
			return src.replace(/(\/files\/photos\/imgs\/[0-9]+\/[0-9]+\/)[a-z]+_/, "$1original_");
		}

		if (domain_nowww === "zdf.de" &&
			string_indexof(src, "/assets/") >= 0) {
			return src.replace(/~[0-9]+x[0-9]+([?#].*)?$/, "~original$1");
		}

		if (domain_nowww === "arcinfo.ch" ||
			domain_nowww === "lacote.ch") {
			return src.replace(/(\/media\/image\/[0-9]*\/)[^/]*\/([0-9]+-[0-9]+\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "gulf365.co" ||
			domain_nosub === "mogaznews.com" ||
			domain_nowww === "ksa-press.com" ||
			domain_nowww === "news-sinaa.com" ||
			domain_nowww === "akherkhabrtoday.com" ||
			domain_nowww === "arabyoum.com" ||
			domain_nowww === "uk-arabicnews.com" ||
			domain_nowww === "alsharqtimes.com" ||
			src.match(/^[a-z]+:\/\/[^/]*\/temp\/+(?:thumb|resized)\/+(?:small|medium|[0-9]+x[0-9]+)_[0-9]{4}-[0-9]{2}-[0-9]{2}-[0-9a-f]{10}\.[^/._]+(?:[?#].*)?$/) ||
			src.match(/^[a-z]+:\/\/[^/]*\/temp\/+(?:thumb|resized)\/+(?:small|medium|[0-9]+x[0-9]+)_uploads,[0-9]{4},[0-9]{2},[0-9]{2},[0-9a-f]{10}\.[^/._]+(?:[?#].*)?$/)) {
			return src
				.replace(/(:\/\/[^/]*\/)temp\/(?:thumb|resized)\/[^-/_.]+_uploads,([0-9]+),([0-9]+),([0-9]+),([0-9a-f]+\.[^/.]*)$/,
						 "$1content/uploads/$2/$3/$4/$5")
				.replace(/(:\/\/[^/]*\/)temp\/(?:thumb|resized)\/[a-z]+_([0-9]+)-([0-9]+)-([0-9]+)-([0-9a-f]+\.[^/.]*)$/,
						 "$1content/uploads/$2/$3/$4/$5");
		}

		if (domain_nowww === "aljazeera.com") {
			return src.replace(/\/mritems\/imagecache\/[^/]*\/mritems\//, "/mritems/");
		}

		if (domain === "media.publika.md" ||
			domain === "media.dcbusiness.ro" ||
			domain === "media.realitatea.net" ||
			domain === "media.feminis.ro") {
			return src.replace(/(\/image\/+[0-9]{6}\/)+w[0-9]+(?:h[0-9]+)?\/+/, "$1full/");
		}

		if (domain === "www1.wdr.de") {
			return src.replace(/~_v-[a-z0-9]+(\.[^/.]*)$/, "~_v-original$1");
		}

		if (domain_nowww === "xrimaonline.gr" ||
			domain_nowww === "athensmagazine.gr" ||
			domain_nowww === "digalakis.com" ||
			domain_nowww === "youweekly.gr" ||
			src.match(/^[a-z]+:\/\/[^/]+\/+photos\/+[wc]_[0-9]+px(?:_[0-9]+px)?\/+(?:products|articles)\//)) {
			return src.replace(/\/photos\/+[a-z]_[0-9]+px[^/]*\/+([^/]+)\//, "/photos/master/$1/");
		}

		if (domain_nosub === "avisen.dk" &&
			domain.match(/^media[0-9]*\.avisen\.dk/)) {
			return src.replace(/([?&]sizeid=)[0-9]+/, "$1255");
		}

		if (domain_nowww === "elimparcial.es" &&
			string_indexof(src, "/fotos/") >= 0) {
			return src.replace(/_thumb_[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain_nosub === "agora.md" &&
			domain.match(/^st[0-9]*\.agora\.md/)) {
			return src.replace(/(:\/\/[^/]*\/news\/)[a-z]+\//, "$1big/");
		}

		if (domain_nowww === "demokrathaber.org") {
			return src.replace(/\/images\/resize\/[0-9]+\/[0-9]+x[0-9]+\//, "/images/");
		}

		if (domain === "imgl.krone.at") {
			return src.replace(/(\/scaled\/[0-9]+\/[0-9a-z]+\/)[0-9]+x[0-9]+([?#].*)?$/, "$1full$2");
		}

		if (domain_nowww === "elfinanciero.com.mx" &&
			string_indexof(src, "/uploads/") >= 0) {
			return src.replace(/(\/[0-9a-f]+)_[a-z_]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nosub === "redcdn.pl" &&
			string_indexof(domain, "dcs.redcdn.pl") >= 0) {
			return src.replace(/(:\/\/[^/]*\/)scale(\/.*?)(?:[?#].*)?$/, "$1dcs/$2");
		}

		if (domain === "media.nu.nl") {
			return src.replace(/(:\/\/[^/]*\/[a-z]\/[0-9a-z]+)_[0-9a-z]+(\.[^/.]*)(?:\/[^/]*\.[^/.]*)?$/, "$1$2");
		}

		if (domain === "cloudia.hnonline.sk" ||
			domain === "img.joj.sk") {
			return src.replace(/(:\/\/[^/]*\/)[a-z]?[0-9]*x[0-9]*[a-z]?\/([0-9a-f]+(?:\.[^/.]*?)?)(?:\?.*)?$/, "$1$2");
		}

		if (googlestorage_container === "nana10img") {
			return src.replace(/\/crop\/(?:_[a-z]-[0-9]+){1,}\/images\//, "/images/");
		}

		if ((domain_nowww === "webnews.bg" ||
			 domain === "static.dir.bg") &&
			string_indexof(src, "/uploads/images/") >= 0) {
			return src.replace(/\/[0-9]+x(?:[0-9]+)?(\.[^/.]*)$/, "/orig$1");
		}

		if (domain === "static.ffx.io") {
			return {
				url: src.replace(/\/images\/\$[^/]*\/.*\/([0-9a-f]+)([?#].*)?$/, "/images/$1$2"),
				can_head: false // 404 for some images
			};
		}

		if (domain === "images.stv.tv") {
			return src.replace(/\/articles\/[wh][0-9]+(?:xh[0-9]+)?(?:xm[^/]*)?\/([^/]*)$/, "/articles/master/$1");
		}

		if (domain === "cdn.cretalive.gr") {
			return src.replace(/(:\/\/[^/]*\/)_[a-z]+Image\//, "$1");
		}

		if ((domain_nosub === "ellingtoncms.com" &&
			 domain.match(/\.media\.clients\.ellingtoncms\.com$/)) ||
			domain === "media.lasvegasweekly.com" ||
			domain === "media.spokesman.com") {
			return src.replace(/_t[0-9]*(?:x[0-9]+)?(\.[^/.]*)$/, "$1");
		}

		if (domain === "ajo.prod.reuters.tv") {
			return src.replace(/(\/img\/[0-9a-f]+-[0-9]+).*?[?&](location=[^&]*).*?$/, "$1?$2");
		}

		if (domain_nowww === "nos.nl") {
			return src.replace(/(\/data\/image\/[0-9]+\/[0-9]+\/[0-9]+\/[0-9]+\/)[a-z]+(\.[^/.]*)$/, "$1original$2");
		}

		if (domain_nowww === "braunschweiger-zeitung.de" ||
			domain === "img.derwesten.de" ||
			domain_nowww === "thueringen24.de" ||
			domain_nowww === "helmstedter-nachrichten.de") {
			return src.replace(/\/img\/(?:panorama|welt)\/crop([0-9]+)\/.*(\.[^/.]*)$/, "/bin/src-$1$2");
		}

		if (domain === "iprx.ten.com.au") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/ImageHandler\.ashx.*?[?&]u=([^&]*).*?$/, "$1"));
		}

		if (domain === "images.sudouest.fr") {
			return src.replace(/(\/[0-9a-f]+\/)[a-z]+\/[0-9]+x[0-9]+\/([^/]*)$/, "$1$2");
		}

		if (domain === "gdb.rferl.org" ||
			domain === "gdb.voanews.com" ||
			domain === "gdb.alhurra.eu" ||
			domain === "gdb.radiosawa.us") {
			return src.replace(/(:\/\/[^/]*\/+[-0-9A-F]+)(?:_c?[a-z][0-9]*){1,}(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "img.nzz.ch") {
			return src.replace(/^[a-z]+:\/\/[^/]+\/+(?:[A-Z]=[^/]+\/+){1,}(https?:\/\/)/, "$1");
		}

		if (domain === "images-cdn.impresa.pt" ||
			domain === "images.impresa.pt") {
			newsrc = src
				.replace(/\/(?:[0-9]+x[0-9]+|original)\/+m[wh]-[0-9]+(?:\?.*)?$/, "/original")
				.replace(/\?.*/, "");
			if (newsrc !== src)
				return newsrc;

			obj = {
				url: src,
				head_wrong_contentlength: true
			};

			match = src.match(/^[a-z]+:\/\/[^/]+\/+[a-z]+\/+([^/]+\.[^/.]*?)(?:-[0-9])?\/original/);
			if (match) {
				obj.filename = match[1];
			}

			return obj;
		}

		if (domain === "awsimages.detik.net.id" ||
			domain === "akcdn.detik.net.id") {
			if (src.match(/\/customthumb\//)) {
				return src.replace(/\?.*/, "");
			}

			newsrc = src.replace(/(\/community\/+media\/+visual\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[-0-9a-f]{20,})_[0-9]+\./, "$1.");
			if (newsrc !== src)
				return newsrc;

			return src.replace(/(?:\?.*)?$/, "?a=1");
		}

		if (domain_nowww === "sexhd.pics" ||
			domain_nowww === "sexphotos.pw" ||
			domain_nosub === "yespornpics.com" ||
			domain_nowww === "babe.today" ||
			domain_nowww === "xxxporn.pics" ||
			domain_nosub === "jjgirls.com") {
			newsrc = src
				.replace(/(:\/\/[^/]*\/)photo(\/.*\/)hd-([^/]*)$/, "$1gallery$2$3")
				.replace(/(:\/\/[^/]*\/)image(\/.*\/)hd-([^/]*)$/, "$1xxx$2$3")
				.replace(/(:\/\/[^/]*\/)(?:thumbs|pictures)(\/.*\/)hd-([^/]*)$/, "$1pictures$2$3")
				.replace(/(:\/\/[^/]*\/)thumb(\/.*\/)hd-([^/]*)$/, "$1media$2$3")
				.replace(/(:\/\/[^/]*\/)pic(\/.*\/)hd-([^/]*)$/, "$1pics$2$3");

			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "nakedwench.com") {
			return src.replace(/(\/galleries\/+[0-9]{4}\/+[^/]*\/+)thumbnail([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1middle$2");
		}

		if (domain === "pics.tubetubetube.com" ||
			domain === "pics.javbtc.com") {
			return src.replace(/(:\/\/[^/]*\/)thumbs(\/.*\/)hd-([^/]*)$/, "$1pornpics$2$3")
		}

		if (domain_nowww === "javhd.pics" ||
			domain_nowww === "javpornpics.com") {
			return src.replace(/(:\/\/[^/]*\/)media(\/.*\/)(?:u?hd|pin)-([^/]*)$/, "$1photos$2$3");
		}

		if (domain_nowww === "javpics.com") {
			return src.replace(/(:\/\/[^/]*\/)xxx(\/.*\/)(?:u?hd|pin)-([^/]*)$/, "$1images$2$3");
		}

		if (domain_nowww === "purejapanese.com" ||
			domain_nowww === "jjgirls.com" ||
			domain_nowww === "1pondo.com" ||
			domain_nowww === "asiauncensored.com" ||
			domain_nowww === "69dv.com" ||
			domain_nowww === "japanesethumbs.com" ||
			domain_nowww === "javtube.com") {
			newsrc = src.replace(/(:\/\/[^/]*\/)(pic|tokyo(?:pic|sex)|uncensored|javmodel|japansex|heydouga|jav|japanese(?:girl)?)(\/.*\/)cute-([^/]*)$/, "$1$2$3$4");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "jjgirls.com") {
			return src
				.replace(/(\/photo\/+[^/]*\/+[^/]*\/+[^/]*\/+[0-9]+)t(\.[^/.]*)(?:[?#].*)?$/, "$1$2")
				.replace(/(\/photo\/+[^/]*\/+[^/]*\/+[^/]*\/+[^/]*)_small(\.[^/.]*)(?:[?#].*)?$/, "$1$2")
				.replace(/(\/photo\/+[^/]+\/+[^/]+\/+[0-9]+\/+)tn([0-9]+\.[^/.]+)(?:[?#].*)?$/, "$1$2")
				.replace(/(\/photo\/+watch4beauty\/+.*\/+)thumbnail([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1middle$2");;
		}

		if (domain_nowww === "javbtc.com") {
			return src.replace(/(:\/\/[^/]*\/)media(\/.*\/)hd-([^/]*)$/, "$1photos$2$3");
		}

		if (domain_nowww === "japanesebeauties.net") {
			return src.replace(/\/media\/+(.*\/)hd-([^/]*)(?:[?#].*)?$/, "/$1$2");
		}

		if (domain_nowww === "tubetubetube.com") {
			return src.replace(/(:\/\/[^/]*\/)media(\/.*\/)(?:u?hd|pin)-([^/]*)$/, "$1pics$2$3");
		}

		if (domain_nowww === "nuceleb.ru") {
			return src.replace(/(\/assets\/images\/resources\/[0-9]+\/)[0-9]+x[0-9]+\/([^/]*)$/, "$1$2");
		}

		if (domain_nosub === "imgserve.net") {
			return {
				url: src.replace(/(:\/\/[^/]*\/)images\/[a-z]+\//, "$1images/big/"),
				headers: {
					Referer: src
				}
			};
		}

		if (domain === "image.celebrityrave.com") {
			return src.replace(/(\/[0-9a-f]*)-[0-9a-z]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "photo.addyoursex.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/remote_control\.php.*?[?&]file=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return urljoin(src, decodeURIComponent(newsrc), true);
		}

		if (domain === "statics.cdntrex.com" ||
			domain_nowww === "anon-v.com" ||
			domain_nowww === "pornrewind.com") {
			id = null;
			match = src.match(/\/videos_screenshots\/+[0-9]+\/+([0-9]+)\/+(?:[0-9]+x[0-9]+\/+|preview\.)/);
			if (match) {
				id = match[1];
			}

			var basedomain = "https://www.porntrex.com/";
			var videos_component = "video";
			var addslash = "";
			var cache_host = "porntrex";
			if (domain_nosub === "pornrewind.com") {
				basedomain = "https://www.pornrewind.com/";
				videos_component = "videos";
				addslash = "/";
				cache_host = "pornrewind";
			} else if (domain_nosub === "anon-v.com") {
				basedomain = "https://www.anon-v.com/";
				videos_component = "videos";
				addslash = "/";
				cache_host = "anon-v";
			}

			if (id && options.do_request && options.cb) {
				var cache_key = cache_host + ":" + id;
				api_cache.fetch(cache_key, function(data) {
					if (!data) {
						return options.cb(null);
					}

					var maxurl = null;
					var maxsize = null;

					for (var key in data) {
						if (/^video_.*url[0-9]*$/.test(key)) {
							var oururl = data[key];
							var oursize = oururl.replace(/.*_([0-9]+)p\.[^/.]+(?:\/*[^/]*)?(?:[?#].*)?$/, "$1");
							if (oursize === oururl) {
								if (oururl.match(/\/([0-9]+)\/+\1\.[^/.]+(?:\/*[^/]*)?(?:[?#].*)?$/)) {
									oursize = 480;
								} else {
									console_warn("Unable to detect size from URL: " + oururl);
									continue;
								}
							}
							oursize = parseInt(oursize);

							if (!maxsize || oursize > maxsize) {
								maxsize = oursize;
								maxurl = oururl;
							}
						}
					}

					if (maxurl) {
						return options.cb({
							url: maxurl,
							headers: {
								Referer: basedomain
							}
						});
					} else {
						return options.cb(null);
					}
				}, function(done) {
					options.do_request({
						url: basedomain + videos_component + "/" + id + "/a" + addslash,
						method: "GET",
						onload: function(resp) {
							if (resp.readyState !== 4)
								return;

							if (resp.status !== 200) {
								console_error(resp);
								return done(null, false);
							}

							var match = resp.responseText.match(/var\s+flashvars\s*=\s*({[\s\S]+?});/);
							if (!match) {
								console_error("Unable to find match", resp);
								return done(null, false);
							}

							try {
								var jsoned = match[1]
									.replace(/([{,])\s*([\S]+)\s*:/g, "$1 \"$2\":")
									.replace(/(\"[\S]+\":)\s*'([^']*)'/g, "$1 \"$2\"");
								var json = JSON_parse(jsoned);
								done(json, 6*60*60);
							} catch (e) {
								console_error(e);
								return done(null, false);
							}
						}
					});
				});

				return {
					waiting: true
				};
			}
		}

		if (domain === "cora.porntrex.com") {
			return {
				url: src,
				headers: {
					Referer: "https://www.porntrex.com/"
				}
			};
		}

		if (domain_nowww === "zceleb.com" ||
			domain === "cdn.faponix.com" ||
			domain_nosub === "fapality.com" ||
			domain === "alb-xb.hellcdn.net" ||
			domain_nowww === "vjav.com" ||
			domain === "i.tubsexer.com" ||
			domain === "i.pornsexer.com" ||
			domain_nowww === "pervertedmilfs.com" ||
			domain_nowww === "xcadr.com" ||
			domain_nowww === "anon-v.com" ||
			domain_nosub === "okporn.com" ||
			(domain_nosub === "cdntrex.com" && /^album[0-9]*\./.test(domain)) ||
			domain_nosub === "p7cdn.com" ||
			domain_nosub === "pornoreino.com" ||
			domain_nowww === "mylust.com" ||
			domain_nosub === "smutr.com" ||
			domain_nowww === "megaporn.co" ||
			domain === "i.tubsexer.com" ||
			domain_nowww === "break.ie" ||
			(domain_nosub === "watchmygf.me" && /^cdn[0-9]*\./.test(domain)) ||
			domain === "watchmyexgf.good-cdn.com" ||
			domain_nowww === "fitting-room.com" ||
			(domain_nosub === "tubepornclassic.com" && /^static[0-9]*\./.test(domain)) ||
			domain_nowww === "iloveporn.xxx" ||
			domain_nowww === "zzgays.com" ||
			domain_nowww === "pornhat.com" ||
			domain === "i.bobs-tube.com" ||
			domain_nowww === "xbabe.com" ||
			domain_nowww === "pornzee.com" ||
			domain === "media.thebestshemalevideos.com" ||
			domain_nowww === "themilf.net" ||
			domain_nowww === "pornstarsadvice.com" ||
			domain_nowww === "porntb.com" ||
			domain_nowww === "fetishburg.com" ||
			domain_nosub === "zbporn.tv" ||
			domain === "cdn.pornstill.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+remote_control\.php\?(?:.*?&)?file=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return urljoin(src, decodeURIComponent(newsrc), true);

			newsrc = src.replace(/\/get_image\/[0-9a-f]\/[0-9a-f]+\/main\/(.*?)\/+(?:[?#].*)?$/, "/contents/albums/main/$1");
			if (newsrc !== src)
				return newsrc;

			var max = null;
			if (domain_nowww === "porntb.com") {
				max = "1920x1920";
			} else if (domain_nowww === "fetishburg.com") {
				max = "1920x1200";
			} else if (domain_nosub === "zbporn.tv") {
				max = "9998x9998";
			}

			match = src.match(/\/main\/+([0-9]+x[0-9]+)\/+/);
			if (max && match) {
				var get_pixels = function(x) {
					var match = x.match(/^([0-9]+)x([0-9]+)$/);
					if (!match)
						return 0;

					return parseInt(match[1]) * parseInt(match[2]);
				};

				if (get_pixels(max) > get_pixels(match[1])) {
					newsrc = src.replace(/\/main\/+[0-9]+x[0-9]+\/+/, "/main/" + max + "/");
				}
			} else if (!max) {
				newsrc = src.replace(/\/main\/+[0-9]+x[0-9]+\/+/, "/sources/");
			}

			obj = {
				url: newsrc
			};

			var referer = "http://" + domain_nosub + "/";
			if (domain_nosub === "p7cdn.com") {
				referer = "https://www.sex-hd.xxx/";
			} else {
				obj.referer_ok = {same_domain: true};
			}

			obj.headers = {Referer: referer};

			return obj;
		}

		if (domain === "image.jeuxvideo.com") {
			return {
				url: src.replace(/(:\/\/[^/]*\/medias)-[a-z]+(\/[0-9]+\/)/, "$1$2"),
				can_head: false
			};
		}

		if (domain === "images.pushsquare.com") {
			return src
				.replace(/((?:\/news\/+[0-9]+\/+[0-9]+\/+[^/]+\/+)|(?:\/screenshots\/+[0-9]+\/+)|(?:\/reviews\/+[^/]*\/+[^/]*\/+))(?:[0-9]+x(?:[0-9]+)?|[a-z]+)(\.[^/.]*)$/, "$1original$2")
				.replace(/(\/games\/.*\/cover)(?:_[a-z]+)?(\.[^/.]*)$/, "$1_original$2");
		}

		if (domain_nowww === "pcgames.de") {
			return src.replace(/\/screenshots\/+[0-9]*x[0-9]*\//, "/screenshots/original/");
		}

		if (domain_nowww === "collinsdictionary.com") {
			return src.replace(/\/images\/thumb\/([^/]*)_[0-9]+(\.[^/.]*)(\?.*)?$/, "/images/full/$1$2$3");
		}

		if (domain_nowww === "abc.net.au") {
			return src
				.replace(/(\/pluck-cache\/images\/[-0-9a-f]+\.)[A-Z][a-z]+(\.[^/.]*)$/, "$1Full$2")
				.replace(/(\/cm\/rimage\/[0-9]+-[0-9]+x[0-9]+-)[a-z]+(\.[^/.]*)$/, "$1large$2");
		}

		if (domain_nowww === "1x.com") {
			return src
				.replace(/(:\/\/[^/]*\/)img\/[^/]*?[?&]id=([0-9a-f]+).*$/, "$1images/user/$2-hd4.jpg")
				.replace(/(\/images\/user\/[0-9a-f]+)-[a-z]+(?:[0-9]+)?(\.[^/.]*)$/, "$1-hd4$2");
		}

		if (domain === "fotografroku.ifotovideo.cz") {
			return src.replace(/(:\/\/[^/]*\/image\.php).*?[?&]id=([0-9]+).*?$/, "$1?id=$2&size=2");
		}

		if (domain === "img.kutikomiya.jp") {
			return src.replace(/\/thumbnail(\/[^/]*\/)W[0-9]+(?:xH[0-9]+)?\//, "/album$1");
		}

		if (domain === "attach.setn.com") {
			regex = /(\/newsimages\/+[0-9]+\/+[0-9]+\/+[0-9]+\/+[0-9]+)(?:-[A-Z]+)?(\.[^/.]*)$/;
			if (regex.test(src)) {
				return [
					src.replace(regex, "$1-SOURCE$2"),
					src.replace(regex, "$1$2")
				];
			}
		}

		if (domain_nowww === "heyzo.com") {
			return src.replace(/\/gallery\/thumbnail_([0-9]+\.[^/.]*)$/, "/gallery/$1");
		}

		if (domain === "img.picabcd.com") {
			return src.replace(/\/thumbnail\/images\//, "/images/");
		}

		if (domain === "mozishop.cdn.shoprenter.hu") {
			return src.replace(/\/mozishop\/image\/cache\/(?:[a-z]{1,}[0-9]+){1,}\//, "/mozishop/image/data/");
		}

		if (domain === "terrigen-cdn-dev.marvel.com") {
			return src.replace(/(\/content\/[a-z]+\/)1x\//, "$12x/");
		}

		if (domain_nowww === "czmodels.cz" ||
			domain_nowww === "phmodels.cz") {
			return src.replace(/(\/image\/itemid-([0-9]+)\/).*/, "$1q-100/$2.jpg");
		}

		if (domain_nosub === "amazonaws.com" &&
			domain.match(/^[a-z0-9]+\.execute-api\./)) {
			return src.replace(/(:\/\/[^/]*\/image\/)fit-in\/[0-9]+x[0-9]+\//, "$1");
		}

		if (domain === "cdn.pudra.com") {
			return src.replace(/(:\/\/[^/]*\/[0-9]+\/)fit[a-z]?[0-9]+x[0-9]+\//, "$1");
		}

		if (domain === "arhiva.dalje.com") {
			return src.replace(/(\/slike_[0-9]*\/)r[0-9]*(\/g[0-9]+\/)/, "$1r1$2");
		}

		if (domain === "media.super.cz") {
			return src.replace(/(:\/\/[^/]*\/images\/+)[^/]+\/(.*?)(?:[?#].*)?$/, "$1original/$2");
		}

		if (domain_nowww === "d1g.com") {
			return src.replace(/(\/photos\/[0-9]+\/[0-9]+\/[0-9]+)_[a-z]+(\.[^/.]*)$/, "$1_max$2");
		}

		if (domain_nowww === "e-shuushuu.net") {
			return add_extensions_jpeg(src.replace(/(:\/\/[^/]*\/images\/)thumbs\//, "$1"));
		}

		if (domain === "cdn.anime-pictures.net" ||
			domain_nosub === "anime-pictures.net") {
			return src.replace(/(\/[0-9a-f]+)_[a-z]+(\.[^/.]*)(?:\.webp)?(\?.*)?$/, "$1$2$3");
		}

		if (domain === "images.treccani.it") {
			return src.replace(/\/enc\/media\/share\/images\/[a-z]+\/system\//, "/enc/media/share/images/orig/system/");
		}

		if (domain === "image-store.slidesharecdn.com") {
			return src.replace(/(:\/\/[^/]*\/[-0-9a-f]+)-large(\.[^/.]*)$/, "$1-original$2");
		}

		if (domain_nosub === "stgy.ovh" &&
			domain.match(/citynews-(?:rimini)?today\.stgy\.ovh/)) {
			return src.replace(/\/~media\/[-a-z]+(\/[0-9]+\/)/, "/~media/$1");
		}

		if (domain === "media.raccweb.com") {
			return src.replace(/(:\/\/[^/]*\/)__sized__\/(.*)-thumbnail-[0-9]+x[0-9]+-[0-9]+(\.[^/.]*)$/, "$1$2$3");
		}

		if (domain_nowww === "mormonnewsroom.org") {
			return src.replace(/\/media\/[0-9]+x[0-9]+\//, "/media/original/");
		}

		if (domain === "contents.mediadecathlon.com") {
			return src.replace(/(\/p[0-9]+\/)(?:[0-9]+x[0-9]+\/)?(?:[a-z]+\/)?([^/]*)$/, "$1$2");
		}

		if (domain_nowww === "overcast.fm") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/art.*?[?&]u=([^&]*).*?$/, "$1"));
		}

		if (domain === "spottappeu01prd.blob.core.windows.net") {
			return src.replace(/(\/[0-9a-f]+\/)[0-9]+(?:_[0-9]+)(\.[^/.]*)$/, "$10$2");
		}

		if (domain === "medias.spotern.com") {
			return src.replace(/\/[wh][0-9]+\/([0-9]+(?:_[0-9a-f]+)?(?:-[0-9]+)?\.[^/.]*)$/, "/original/$1");
		}

		if (domain_nosub === "ounousa.com") {
			return src.replace(/\/Content\/ResizedImages\/[0-9]+\/[0-9]+\/[a-z]+(\/[0-9]+\.[^/.]*)$/, "/content/uploads/Article$1");
		}

		if (domain_nowww === "divahair.ro") {
			return src.replace(/(\/articole_imagini\/[^/]*\/[0-9]+\.[0-9]+\.[0-9]+\/)[0-9]+x[0-9]+\/([^/]*)_[0-9]+_[0-9]+(\.[^/.]*)$/, "$1$2$3");
		}

		if (domain_nowww === "wall.hr") {
			return src.replace(/\/cdn\/uploads\/[0-9]+x\//, "/cdn/uploads/");
		}

		if (domain_nosub === "tialoto.bg") {
			return src.replace(/(\/media\/tialoto\/files\/[a-z]+\/)[0-9]+x[0-9]+(?:[a-z]+)?(\/[0-9a-f]+\.[^/.]*)$/, "$1orig$2");
		}

		if (domain === "bizweb.dktcdn.net") {
			return src.replace(/(:\/\/[^/]*\/)thumb\/[a-z]+\//, "$1");
		}

		if (domain_nowww === "socialbliss.com") {
			return src.replace(/\/img\/[0-9]+x[0-9]+\/assets\//, "/assets/");
		}

		if (domain_nowww === "forumfr.com") {
			return "http://" + src.replace(/.*\/applications\/core\/interface\/imageproxy\/ximageproxy\.php.*?,qimg=,h(.*?)(?:,[^_].*)?$/, "$1").replace(/,_/g, "/");
		}

		if (domain_nowww === "pinkomatic.com") {
			return src.replace(/(\/images\/[0-9]+\/)[a-z]+\//, "$1full/");
		}

		if (domain_nowww === "khmeread.com" ||
			(domain_nosub === "techbang.com" && domain.match(/^cdn[0-9]*(?:-[a-z]+)?\./))) {
			return src.replace(/(\/images\/[0-9]+\/)[a-z]+\//, "$1original/");
		}

		if (domain === "cfshopeetw-a.akamaihd.net" ||
			domain === "cf.shopee.tw" ||
			domain === "cf.shopee.co.id" ||
			domain === "cf.shopee.ph" ||
			domain === "cf.shopee.com.my" ||
			domain === "cf.shopee.sg" ||
			domain === "cf.shopee.co.th" ||
			domain === "cf.shopee.vn") {
			return src.replace(/(\/file\/[0-9a-f]+)_tn(?:\?.*)?$/, "$1");
		}

		if (domain_nowww === "theredlist.com") {
			return src.replace(/\/media\/\.cache\/database\/(.*\/)[0-9]+-([^/]*)$/, "/media/database/$1$2");
		}

		if ((domain_nosub === "hentai-cosplay.com" ||
			 domain_nosub === "hentai-image.com" ||
			 domain_nosub === "porn-image-xxx.com" ||
			 domain_nosub === "porn-movie-xxx.com" ||
			 domain_nosub === "hentai-animes.com" ||
			 domain_nosub === "hentai-comic.com" ||
			 domain_nosub === "aniimg.com") &&
			domain.match(/^static[0-9]*\./) &&
			string_indexof(src, "/upload/") >= 0) {
			newsrc = src
				.replace(/^[a-z]+:\/\/[^/]*\/upload\/cache\/thumbnail\/create\.php.*?[?&]image=([^&]*).*?$/, "$1");
			if (newsrc !== src) {
				return newsrc;
			}

			return src
				.replace(/(\/+[0-9]+\/+)(?:[a-z]+=[^/]*\/+){1,}([0-9]+\.[^/.]*)$/, "$1$2")
				.replace(/(\/+[0-9A-Z]+\/+)(?:[a-z]+=[^/]*\/+){1,}([0-9A-Za-z]+\.[^/.]*)$/, "$1$2");
		}

		if (domain === "d1in1v57myx5v.cloudfront.net" ||
			domain === "mpics.mgronline.com") {
			return src.replace(/\/pics\/+Thumbnails\//, "/pics/Images/");
		}

		if (domain_nowww === "artistic-nude-images.com") {
			return src.replace(/\/thumbnail\/tn-([^/]*)$/, "/$1");
		}

		if (domain_nowww === "babefilter.net") {
			return {
				url: src.replace(/(\/i\/[0-9a-f]+\/[0-9a-f]+)_tn(\.[^/.]*)$/, "$1$2"),
				headers: {
					Referer: src
				}
			};
		}

		if ((domain_nosub === "dagospia.com" && string_indexof(src, "/img/") >= 0) ||
			(domain_nowww === "sutki.net" && string_indexof(src, "/img/") >= 0) ||
			(domain_nowww === "screensonic.net" && string_indexof(src, "/images/") >= 0)) {
			return src.replace(/_tn(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "fabhairypussy.com") {
			return {
				url: src.replace(/(\/+galleries\/+[^/]*\/+(?:[0-9]{4}\/+[0-9]{2}\/+)?[^/]+\/+[0-9]+)(?:_[a-z]+){1,}(\.[^/.]*)(?:[?#].*)?$/, "$1$2"),
				headers: {
					Referer: ""
				},
				referer_ok: {
					same_domain: true
				}
			};
		}

		if (domain_nowww === "redcarpetnudes.com" ||
			domain_nowww === "shamelesscelebrities.com" ||
			domain === "cdn.onedala.tv") {
			return src.replace(/-tn(\.[^/.]*)$/, "$1");
		}

		if (domain === "pat.primecdn.net") {
			return src.replace(/\/pics\/[a-z]+(\/[0-9a-f]+\/[0-9a-f]+\.[^/.]*)$/, "/pics/original$1");
		}

		if (domain_nosub === "imagearn.com") {
			return src.replace(/(:\/\/[^/]*\/)([0-9]+\/[0-9]+\.[^/.]*)$/, "$1imags/$2");
		}

		if (domain === "img.static-smb.be" ||
			domain === "img.static-rmg.be") {
			return src.replace(/\/view\/q[0-9]*\/w[0-9]*\/h[0-9]*\//, "/view/q100/w/h/");
		}

		if (domain_nosub === "buro247.mn" ||
			domain_nosub === "buro247.sg" ||
			domain_nosub === "buro247.kz" ||
			domain_nosub === "buro247.mx" ||
			domain_nosub === "buro247.com.au" ||
			domain_nosub === "buro247.ua" ||
			domain_nosub === "buro247.mn" ||
			domain_nosub === "buro247.ru") {
			return src
				.replace(/(\/thumb\/[^/]*\/)galleries\//, "$1local/images/buro/galleries/")
				.replace(/\/thumb\/[0-9]+x[0-9]+(?:_[0-9]+)?((?:\/local)?\/(?:images|thumb)\/)/, "$1");
		}

		if (domain_nosub === "hellomagazine.com") {
			return src.replace(/\/thumb\/+[0-9]+x[0-9]+(?:_[0-9]+)?\/+(images|gallery\/+)/, "/$1");
		}

		if (domain === "images.vogue.it") {
			return src
				.replace(/(\/gallery\/[0-9]+\/)[A-Za-z]+(\/[-0-9a-f]+\.[^/.]*)$/, "$1Original$2")
				.replace(/(\/Photovogue\/+[-0-9a-f]{10,})_small(\.[^/.]+)(?:[?#].*)?$/, "$1_large$2");
		}

		if (domain === "blogs.glamour.de") {
			return src.replace(/\/thumbs\/[^/]*\//, "/files/");
		}

		if (domain === "img.beatles.ru") {
			return src.replace(/(\/[0-9]+)(\/[0-9]+\.[^/.]*)$/, "$1b$2");
		}

		if (domain_nowww === "boutique.az") {
			return src.replace(/(\/images\/[0-9]+)_[a-z](\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "izum.ua") {
			return src.replace(/(_[0-9]+)_sml[0-9]*(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nosub === "senatus.net" &&
			domain.match(/^cache[0-9]*\./)) {
			return src.replace(/\/files\/albums\/resized(?:[0-9]+x)?\//, "/files/albums/");
		}

		if (domain_nowww === "max-pix.com" ||
			domain_nowww === "kollywoodzone.com" ||
			domain_nowww === "celebwallpaper.org" ||
			domain_nowww === "carlyslayjepsen.com" ||
			domain_nosub === "moregirls.org") {
			return src.replace(/\/data\/thumbnails(\/[0-9]+\/)/, "/data/media$1");
		}

		if (domain_nowww === "tapeteos.pl") {
			return src
				.replace(/\/data\/thumbnails(\/[0-9]+\/)/, "/data/media$1big/")
				.replace(/(\/data\/media\/[0-9]+\/)([^/]*)$/, "$1big/$2")
				.replace(/__resized_[0-9]+x[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain === "media.8ch.net" ||
			domain === "media.8kun.top") {
			return src.replace(/\/file_store\/thumb\//, "/file_store/");
		}

		if (domain_nowww === "elles-se-mettent-nues-pour-nous.fr") {
			return src.replace(/(\/photos\/[^/]+\/)thumb\//, "$1");
		}

		if (domain_nosub === "fashionnetwork.com" &&
			domain.match(/^(?:[a-z]+\.)?media\./)) {
			return src.replace(/(\/[a-f0-9]+\/+)[0-9]+x[0-9]+(?:\.[0-9]+)?\/+([a-f0-9]+\.[^/.]*)$/, "$1$2");
		}

		if (domain === "media.filmz.ru") {
			return src.replace(/\/photos\/+[a-z]+\/+([^/_]{2,}_)?(?:[a-z]_)?([0-9]+\.[^/.]*)$/, "/photos/full/$1f_$2");
		}

		if (domain_nowww === "kinofilms.ua") {
			return src.replace(/\/images\/photos\/[a-z0-9]+\/([0-9]+\.[^/.]*)$/, "/images/photos/hd/$1");
		}

		if (domain_nowww === "grandmagazine.gr") {
			return src.replace(/\/cache\/uploaded_images\/(.*)\/[0-9]+_[0-9]+_([^/]*)$/, "/uploaded_images/$1/$2");
		}

		if (domain === "a69.g.akamai.net") {
			return add_http(src.replace(/^[a-z]+:\/\/[^/]*\/[a-z]+\/[0-9]+\/[0-9]+\/v[0-9]+\//, ""));
		}

		if (domain === "images.allocine.fr") {
			return src.replace(/\/[rc]_[0-9]+_[0-9]+\/+pictures\/+/, "/pictures/");
		}

		if (domain === "cdn-media.rtl.fr") {
			return src.replace(/\/cache\/[-a-zA-Z0-9_]+\/[0-9]+v[0-9]+(?:-[0-9]+)?\/online\//, "/online/");
		}

		if (domain === "bcdn.newshunt.com" ||
			(domain_nosub === "dailyhunt.in" && domain.match(/bcdn[-.]/))) {
			return src.replace(/\/cmd\/[a-z]+\/[^/]*\/(fetchdata[0-9]*\/images\/)/, "/rx/$1");
		}

		if (domain === "media.search.lt") {
			return src.replace(/\/GetFile\.php.*?[?&](OID=[0-9]+).*?$/, "/GetFile.php?$1&filetype=4");
		}

		if (domain_nowww === "wallofcelebrities.com") {
			return {
				url: src.replace(/\/pictures\/[a-z]+\/([^/]*_[0-9]+\.[^/.]*)$/, "/pictures/original/$1"),
				headers: {
					Referer: "https://www.wallofcelebrities.com/"
				}
			};
		}

		if (domain_nosub === "lmstube.com") {
			return src.replace(/\/[0-9]+x[0-9]+\/([^/]*)$/, "/$1");
		}

		if (domain === "assets.mubi.com") {
			return src.replace(/(\/images?)-[a-z0-9]+(\.[^/.]*)$/, "$1-original$2");
		}

		if (domain_nowww === "fundir.org") {
			return src.replace(/(:\/\/[^/]*\/)th\//, "$1img/");
		}

		if (domain_nowww === "spletnik.ru") {
			return src.replace(/\/thumb\/[0-9]+x[0-9]+(?:_[0-9]+)?\/img\//, "/img/");
		}

		if (domain === "r.mtdata.ru") {
			return src.replace(/:\/\/[^/]*\/[a-z][-0-9]+x[-0-9]+\//, "://mtdata.ru/");
		}

		if (domain_nowww === "mtdata.ru" &&
			src.match(/:\/\/[^/]*\/u[0-9]+\/photo/)) {
			return src.replace(/\/(?:huge|big)(\.[^/.]*)$/, "/original$1");
		}

		if (domain === "img.uduba.com") {
			newsrc = src.replace(/:\/\/[^/]*\/mtdata\.ru\//, "://mtdata.ru/");
			if (newsrc !== src) {
				return newsrc.replace(/\/[a-z]+_[0-9]+x[0-9]+(\.[^/.]*)$/, "/original$1");
			}
		}

		if (domain_nowww === "russianpulse.ru") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/img\/([a-z]+:\/\/.*)$/, "$1");
		}

		if (domain === "img.noobzone.ru" ||
			domain === "m.btdx8.com" ||
			domain_nowww === "guihuayun.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+(?:uploads\/+)?getimg\.php\?(?:.*&)?(?:url|src)=([^&]+).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "media.aintitcool.com") {
			return src.replace(/_(?:large|big|huge|medium)(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "lichnosti.net") {
			return src
				.replace(/\/rs\/+thumbs\/+([0-9]+)\/+set_([^/]*)_([a-z]+)_[0-9]+_[0-9]+_[0-9]+\.[^/.]*$/, "/photos/$1/sets/$2.$3")
				.replace(/\/rs\/+thumbs\/+([0-9]+)\/+([^/]*)_([a-z]+)_[0-9]+_[0-9]+_[0-9]+\.[^/.]*$/, "/photos/$1/$2.$3");
		}

		if (domain_nosub === "zmones.lt" &&
			domain.match(/^s[0-9]*\.zmones\.lt/)) {
			return src.replace(/(\/images\/photos\/[0-9]+\/[0-9]+\/[0-9]+\/)[a-z]+(\/[^/]*\.[^/.]*)$/, "$1original$2");
		}

		if (domain_nosub === "hsmedia.ru" ||
			domain_nosub === "elle.ru") {
			return src.replace(/(:\/\/[^/]*\/[0-9a-f]{2}\/[0-9a-f]{2}\/[0-9a-f]{2}\/[0-9a-f]+\/)[0-9]+x[0-9]+_[^/]*@([0-9]+x[0-9]+_[^/]*)$/, "$1$2");
		}

		if (domain_nowww === "starbeat.ru" &&
			string_indexof(src, "/gallery/") >= 0) {
			return src.replace(/\/(?:small|medium)\/([^/]*)$/, "/$1");
		}

		if ((domain === "image.kilimall.com" ||
			 domain === "d2lpfujvrf17tu.cloudfront.net") &&
			string_indexof(src, "/shop/store/") >= 0) {
			return src.replace(/(\/[0-9]+_[0-9]+)_[0-9]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "zevenmart.com") {
			return src.replace(/\/resized\/([^/]*)_[0-9]+x[0-9]+(\.[^/.]*)$/, "/$1$2");
		}

		if (domain_nowww === "superherotv.net" ||
			domain_nowww === "starsplanet.ru") {
			return src.replace(/\/photo\/images_small\//, "/photo/images_large/");
		}

		if (domain_nowww === "xxx-photo.com") {
			return src.replace(/\/photo\/t\/(.*?)(?:_t)?(\.[^/.]*)$/, "/photo/i/$1$2");
		}

		if (domain_nowww === "div.bg" ||
			domain_nowww === "jenite.bg") {
			return src.replace(/(\/pictures\/[0-9]+)_[0-9]*(?:_[0-9]*)?(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "cdn.hqsluts.com" ||
			domain_nowww === "hqbabes.com" ||
			domain === "c.xme.net") {
			return src.replace(/(:\/\/[^/]*\/(?:t\/)?[0-9]+)[a-z]([0-9]+(?:\.[^/.]*)?)$/, "$1c$2");
		}

		if ((domain_nowww === "sensualgirls.org" ||
			 domain_nowww === "deliciousbabes.org" ||
			 domain_nowww === "girlsofdesire.org") &&
			src.match(/\/media\/pictures(?:_new)?\//)) {
			return {
				url: src.replace(/(\/[^/.]*)_thumb(\.[^/.]*)$/, "$1$2"),
				headers: {
					Origin: "https://" + domain,
					Referer: "https://" + domain + "/"
				}
			};
		}

		if ((domain_nosub === "mainbabes.com" && domain.match(/^content[0-9a-z]*\./)) ||
			(domain_nosub === "livejasminbabes.net" && domain.match(/^content[0-9a-z]*\./)) ||
			(domain_nosub === "babesandgirls.com" && domain.match(/^content[0-9a-z]*\./)) ||
			(domain_nosub === "exgirlfriendmarket.com" && domain.match(/^content[0-9a-z]*\./)) ||
			(domain_nosub === "rossoporn.com" && domain.match(/^content[0-9a-z]*\./)) ||
			(domain_nosub === "novostrong.com" && domain.match(/^content[0-9a-z]*\./)) ||
			(domain_nosub === "novoporn.com" && domain.match(/^content[0-9a-z]*\./)) ||
			(domain && domain.match(/^content[0-9a-z]*\./) && src.match(/^[a-z]+:\/\/[^/]*\/[^/.]*\.[^/.]*\/[0-9]{4,}\/(?:[^/]*_)?tn_[0-9]{2}\.[^/.]*(?:[?#].*)?$/)) ||
			(domain_nosub === "thousandbabes.com" && domain.match(/^content[0-9a-z]*\./)) ||
			(domain_nosub === "morazzia.com" && domain.match(/^content[0-9a-z]*\./)) ||
			(domain_nosub === "babesmachine.com" && domain.match(/^content[0-9a-z]*\./)) ||
			(domain_nosub === "glam0ur.com" && domain.match(/^content[0-9a-z]*\./)) ||
			domain_nosub === "avidolz.com" ||
			domain_nosub === "pbabes.com" ||
			(domain === "static.thenude.eu" && string_indexof(src, "/galleries/") >= 0) ||
			(domain_nowww === "eros-and-grace.net" && string_indexof(src, "/galleries/") >= 0) ||
			domain === "gals.sextronix.com" ||
			domain_nosub === "pornodontstop.com") {
			newsrc = src.replace(/(\/(?:[^/]*_)?)tn_([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nosub === "livejasminbabes.net" && domain.match(/^content[0-9]*\./)) {
			return src.replace(/\/upload\/main\/tn\//, "/upload/main/");
		}

		if (domain === "static.thenude.eu") {
			return src.replace(/(\/admin\/+covers\/+.*\/+)(?:thumbs|medheads)\/+/, "$1");
		}

		if ((domain_nowww === "captaingoodlink.com" && string_indexof(src, "/celebrities/") >= 0) ||
			domain_nowww === "nudecelebs-a-z.com" ||
			domain_nowww === "poseposter.com" ||
			(domain_nosub === "tacamateurs.com" && domain.match(/^cdn(?:-[^/]*)?\./) && src.match(/\/tn_pic[0-9]+\./))) {
			return src.replace(/\/tn_([^/]*\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nosub === "pornodontstop.com" ||
			domain === "hosted.amourangels.com" ||
			domain_nowww === "hornyteen.pro" ||
			domain_nowww === "theasianpics.com" ||
			domain_nowww === "teenporngallery.net" ||
			domain_nowww === "asiangalleries.net" ||
			domain_nowww === "chinesepornpics.com" ||
			domain_nowww === "asianteenpictures.com" ||
			domain_nowww === "japanesegirlspictures.com" ||
			domain_nowww === "koreanpornpictures.com" ||
			domain_nowww === "sexyasians.net" ||
			domain_nowww === "maturexxxpics.net" ||
			domain_nowww === "japxxx.pro" ||
			domain_nowww === "hornybabepics.com" ||
			domain_nowww === "teengalleries.mobi") {
			return src.replace(/\/th_([0-9]+\.[^/.]*)$/, "/$1");
		}

		if (domain === "i.girlstop.info" ||
			domain_nosub === "girlstop-extra.info" ||
			domain_nowww === "girlstop.info") {
			return src.replace(/\/thumbs\/+[0-9]+px_([^/]*)$/, "/$1");
		}

		if (domain === "img.freeones.com") {
			newsrc = src.replace(/\/pinned_pictures\/pin\//, "/pinned_pictures/original/");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "photos.freeones.com") {
			return src.replace(/(:\/\/[^/]*\/[^/]*_[^/]*\/+[^/]*\/+[^/]*\/+)([^/]*)(?:[?#].*)?$/, "$1images/$2");
		}

		if (domain_nosub === "freeones.com" && /img\./.test(domain)) {
			return src.replace(/(\/photos\/+[0-9]{3}\/+[0-9a-z]{2}\/+[0-9a-z]{2}\/+[^/]{10,}\/+)preview\/+/, "$1big/");
		}

		if (domain_nowww === "starspics.ru" ||
			domain_nowww === "otherstars.ru") {
			return src.replace(/(:\/\/[^/]*\/)thumbs\//, "$1img/");
		}

		if (domain === "images.apina.biz") {
			return src.replace(/(:\/\/[^/]*\/)[a-z]+\/[a-z]+_([0-9]+\.[^/.]*)$/, "$1full/$2");
		}

		if (domain_nowww === "pluska.sk" ||
			domain_nosub === "casopiszdravie.sk") {
			return src.replace(/\/thumb\/images\/(.*?)(?:[?#].*)?$/, "/images/$1");
		}

		if (domain_nowww === "mad-movies.com") {
			return src.replace(/(:\/\/[^/]*\/)[0-9]+-[0-9]+-[0-9]+-images\//, "$1images/");
		}

		if (domain === "assets.mycast.io") {
			return src.replace(/(\/[0-9]+)_[a-z]+(\.[^/.]*)$/, "$1_original$2");
		}

		if (domain === "cdn.cinemur.fr") {
			return src.replace(/(?:\/cache\/[0-9]+x[0-9]+_|\/blur\/)([0-9]+\.[^/.]*)$/, "/original/$1");
		}

		if (domain_nowww === "farfarawaysite.com") {
			return src.replace(/\/hires\/t\//, "/hires/");
		}

		if (domain === "movieplayer.net-cdn.it" ||
			domain === "d17vsf20mehj1i.cloudfront.net" ||
			domain === "multiplayer.net-cdn.it") {
			return src.replace(/(?:\/(?:thumbs|t))?(\/+images\/.*?|:\/\/[^/]*\/[^/]*)_([a-z]+)_(?:[0-9]+x[0-9]+_)?(?:crop_)?(?:upscale_)?(?:q[0-9]+)?(\.[^/.]*)$/, "$1.$2");
		}

		if (domain === "u.livelib.ru") {
			return src
				.replace(/\/[a-z](\/[a-z0-9]+\/)[^/]*(\.[^/.]*)$/, "/o$1o-o$2")
				.replace(/\.jpg(\?.*)?$/, ".jpeg$1");
		}

		if (domain_nosub === "celebsnetworth.org") {
			return src.replace(/(:\/\/[^/]*\/main\/)thumbs\//, "$1images/");
		}

		if (domain_nowww === "multimedios.com") {
			return src.replace(/(:\/\/[^/]*\/)files\/[^/]*\/uploads\//, "$1uploads/");
		}

		if (domain === "file.hstatic.net") {
			return src.replace(/(\/file\/[^/]*-[0-9]+)_[a-z]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "sm.askmen.com") {
			return src.replace(/(:\/\/[^/]*\/)t\/(.*\/[^/]*)\.[0-9]+(\.[^/.]*)$/, "$1$2$3");
		}

		if (domain_nosub === "elsiglodetorreon.com.mx" && /^media[0-9]*\./.test(domain)) {
			newsrc = src.replace(/:\/\/[^/]*\/+(n?[a-z]\/+)/, "://www.elsiglodetorreon.com.mx/m/$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "elsiglodetorreon.com.mx" ||
			domain_nowww === "siglo.mx") {
			newsrc = src
				.replace(/(:\/\/[^/]*\/+(?:[^/]*\/+)?m\/+)ni\/+(.*\/[0-9]+)\.[0-9]+(\.[^/.]*)$/,
						 "$1i/$2$3");
			if (newsrc !== src)
				return newsrc
					.replace(/\.jpg(\?.*)?$/, ".jpeg$1");
		}

		if (domain_nowww === "elsiglo.mx") {
			return src
				.replace(/(:\/\/[^/]*\/m\/)n([a-z]\/.*\/[0-9]+)\.[0-9]+(\.[^/.]*)$/,
						 "$1$2$3")
				.replace(/(\/[0-9]+)_[a-z](\.[^/.]*)$/, "$1$2")
				.replace(/\.jpg(\?.*)?$/, ".jpeg$1");
		}

		if (domain === "img.richestcelebrities.org" ||
			domain_nowww === "celebritypictures.wiki" ||
			domain === "img.networthpost.org" ||
			domain_nowww === "thefappeningpics.com") {
			return src.replace(/(:\/\/[^/]*\/)thumbs\//, "$1images/");
		}

		if (domain_nowww === "fashionscene.nl") {
			return src.replace(/_thumb_[0-9]+x[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "monitor.bg") {
			return src.replace(/\/gallery\/thumb_[0-9]+x[0-9]+_([^/]*)$/, "/gallery/$1");
		}

		if (domain_nowww === "textilwirtschaft.de" ||
			domain_nowww === "fashionmagazine.it") {
			return src.replace(/(\/gallery\/media\/[0-9]+\/[0-9]+)-[a-z]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "horizont.net" && string_indexof(src, "/media/") >= 0) {
			return src.replace(/(-[0-9]+)-[a-z]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "twproxy.stiletto.fr") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/[0-9a-z]+\/[wh][0-9]+\//, "http://");
		}

		if (domain_nowww === "we123.com") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/img\/p\.php.*?[?&]p=([^&]*).*?$/, "$1"));
		}

		if (domain_nosub === "vagalume.com" && /^s[0-9]*\./.test(domain)) {
			return src.replace(/(\/images\/g?[0-9]+)[wh][0-9]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nosub === "glanacion.com" && domain.match(/^bucket[0-9]*\./)) {
			return src.replace(/(\/fotos\/+[0-9]+\/+[0-9]+)[wh][0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "storage.gra1.cloud.ovh.net" &&
			src.match(/\/Futurenet[^/]*Images\//)) {
			return src.replace(/_[0-9]+x[0-9]+(\?.*)?$/, "$1");
		}

		if (domain === "iasbh.tmgrup.com.tr") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/[0-9a-f]+\/(?:[0-9]+\/){5}[0-9]+.*?[?&]u=([^&]*).*?$/,"$1"));
		}

		if (domain === "i.sabah.com.tr") {
			return src.replace(/(\/sb\/+[^/]*\/+magazin\/+[^/]*\/+[^/]*)_[a-z](\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "bideew.com") {
			return src.replace(/(\/photos\/thumbnail\/[0-9]+\/)[a-z]+(\/*)(\?.*)?$/, "$1master$2$3");
		}

		if (domain === "media.filfan.com") {
			return src.replace(/(\/NewsPics\/[^/]*\/)[a-z]+(\/[^/]*)$/, "$1original$2");
		}

		if (domain_nowww === "ana.rs") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/forum\/thumbs\/img\.php.*?[?&]src=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "squa.re") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/wp-content\/+themes\/+[^/]*\/+img\.php.*?[?&]src=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return urljoin(src, newsrc, true);
		}

		if (domain_nosub === "itvnet.lv" &&
			src.match(/\/upload[0-9]*\/articles\//)) {
			return src
				.replace(/\/thumbs\/[0-9]+([^/]*)$/, "/images/$1")
				.replace(/\/images\/([^_][^/]*)$/, "/images/_origin_$1");
		}

		if (domain_nowww === "connectgalaxy.com") {
			return src.replace(/\/gallery\/icon\/([0-9]+)\/[a-z0-9]+(\?.*)?$/, "/gallery/download/$1");
		}

		if (domain === "nst.sky.it" ||
			domain === "sydney.edu.au" ||
			domain_nowww === "hbo.com") {
			return src.replace(/(\/[^/]*\.[^/.]*)\/_jcr_content\/renditions\/.*/, "$1");
		}

		if (domain === "cdnimage.terbitsport.com") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/image\.php.*?[?&]image=([^&]*).*?$/, "$1"));
		}

		if (domain_nowww === "beimg.com") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/fangimg\.php.*?[?&]url=([^&]*).*?$/, "$1"));
		}

		if (domain_nowww === "que.es") {
			return src.replace(/-[0-9X]+x[0-9X]+(?:x[0-9]+)?(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "independent.com.mt") {
			return src.replace(/(:\/\/[^/]*\/file\.aspx).*?[?&](f=[0-9]+).*?$/, "$1?$2");
		}

		if (domain === "img.selenka.pl") {
			return src.replace(/(\/(?:fb)?walls\/[0-9a-f]+\/)[a-z](\.[^/.]*)$/, "$1o$2");
		}

		if (domain === "ki.ill.in.ua") {
			return src.replace(/\/[0-9]+x[0-9]+(\/[0-9]+\.[^/.]*)$/, "/0x0$1");
		}

		if (domain === "stg.pcg.space") {
			return src.replace(/\/[a-z]+(\.[^/.]*)$/, "/org$1");
		}

		if (domain === "static.noticiasaominuto.com" ||
			domain === "static.noticiasaominuto.com.br") {
			return src.replace(/(\/stockimages\/)(?:gallery\/)?[^/]*(\/[^/]*)$/, "$1hires_original$2");
		}

		if (domain === "media-manager.noticiasaominuto.com") {
			return src
				.replace(/\?.*/, "")
				.replace(/(:\/\/[^/]*\/)[0-9]+\/+[0-9]+\/+(naom_)/, "$1$2");
		}

		if (domain_nowww === "harry-potter.net.pl") {
			return src.replace(/(\/images\/photoalbum\/[^/]*\/[^/]*)_t[0-9](\.[^/.]*)$/, "$1$2");
		}

		if ((domain_nosub === "kanobu.ru" ||
			 domain_nosub === "kanobu.net") &&
			domain.match(/^i[0-9]*\./)) {
			return src.replace(/:\/\/[^/]*\/r\/[0-9a-f]+\/[-0-9]+x[-0-9]+\//, "://");
		}

		if (domain_nowww === "tv3.lt" ||
			domain_nosub === "inspektorius.lt") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+usi\/+[0-9]+x[0-9]+\/+.*?[?&]f=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return urljoin(src, decodeURIComponent(newsrc), true);
		}

		if (domain_nosub === "gallery.ru" &&
			domain.match(/^data[0-9]*\.(?:i\.)?gallery\.ru/)) {
			return src.replace(/(\/albums\/gallery\/[^/]*-[0-9]{5,}-)[whmc]?[0-9]+(?:x[0-9]+)?(?:-[0-9a-z]+)?(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "fotosdefamosas.tk") {
			return src.replace(/\/miniaturas\/TN_([^/]*)$/, "/$1");
		}

		if (domain_nowww === "starwiki.org") {
			return {
				url: src.replace(/\/thumbs\/photos\//, "/photos/"),
				head_wrong_contenttype: true
			};
		}

		if (domain_nowww === "postervdom.ru" && string_indexof(src, "/upload/") >= 0) {
			return src.replace(/(\/item_[0-9]+\/)[a-z]+_(item_[0-9]+[^/]*)$/, "$1$2");
		}

		if (domain_nosub === "fastpic.ru" &&
			domain.match(/^i[0-9]*\.fastpic\.ru/)) {
			return src.replace(/(:\/\/[^/]*\/)thumb(\/.*\.)jpeg/, "$1big$2jpg?noht=1");
		}

		if (domain === "i.nahraj.to") {
			return src.replace(/(:\/\/[^/]*\/)[a-z](\/[^/]+\.[^/.]*)$/, "$1f$2");
		}

		if (domain === "skwww.plusden.sk") {
			return src.replace(/\/thumb(\/images\/gallery\/.*?)(?:\?.*)?$/, "$1");
		}

		if (domain_nowww === "admags.xyz" ||
			domain_nowww === "hotmags.xyz") {
			return src.replace(/_tb(\/[^/]*)$/, "$1");
		}


		if (domain === "img.topky.sk") {
			return src.replace(/(\/dc\/+[0-9]+\/+)thumb\//, "$1");
		}

		if (domain_nowww === "kungahuset.org") {
			return src.replace(/\/uploads\/thumb\/[0-9]+X[0-9]+_/, "/uploads/");
		}

		if (domain_nosub === "e-monsite.com") {
			return src.replace(/\/resize_[0-9]+_[0-9]+\//, "/");
		}

		if (domain === "static.guide.supereva.it" ||
			domain === "media.gossipblog.it" ||
			domain === "media.fashionblog.it") {
			return src.replace(/\/thn_([^/]*)$/, "/$1");
		}

		if (domain_nowww === "apachan.net" ||
			domain_nowww === "prokote.info") {
			return src.replace(/(:\/\/[^/]*\/)(?:thumbs|previews)\//, "$1images/");
		}

		if (domain === "imggaleri.hurriyet.com.tr" ||
			domain === "imgkelebek.hurriyet.com.tr") {
			return src.replace(/(\/+LiveImages\/+[^/]*\/+[0-9]+\/+[^/]*)(?:\/+ThumbFolder)?\/+([^/]*)$/, "$1/LargeFolder/$2");
		}

		if (domain_nowww === "army.mil") {
			return src.replace(/(\/e2\/c\/images\/)(.*\/)size[0-9]*(\.[^/.]*)$/, "$1$2original$3");
		}

		if (domain_nosub === "disquscdn.com") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/get.*?[?&]url=([^&]*).*?$/, "$1"));
		}

		if (domain_nowww === "smore.com") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/external_image.*?[?&]url=([^&]*).*?$/, "$1"));
		}

		if (domain === "images.ctfassets.net") {
			return src.replace(/\?.*$/, "?h=4000");
		}

		if (domain_nosub === "okccdn.com") {
			return src.replace(/(\/php\/load_okc_image\.php\/images\/)(?:[0-9]+(?:x[0-9]+)?\/){1,}([0-9]+\.[^/.]*)$/, "$1$2");
		}

		if (domain_nosub === "metapix.net") {
			newsrc = src.replace(/\/files\/[a-z]+\/([0-9]+\/[^/]*)$/, "/files/full/$1");
			if (newsrc !== src)
				return newsrc;

			return add_extensions(src.replace(/\/thumbnails\/[a-z]*\/([0-9]+\/[0-9]+[^/]*)_noncustom(\.[^/.]*)$/, "/files/full/$1$2"));
		}

		if (domain === "cdn.furiffic.com") {
			return add_extensions(src.replace(/(:\/\/[^/]*\/)[a-z]+(\/[0-9]+\.[^/.]*)$/, "$1originals$2"));
		}

		if (domain === "d3gz42uwgl1r1y.cloudfront.net") {
			return add_extensions(src.replace(/(\/submission\/[0-9]+\/[0-9]+\/[0-9a-f]+)\/[0-9]+x[0-9]+(\.[^/.]*)$/, "$1$2"));
		}

		if (domain_nosub === "route50.net") {
			return src.replace(/(\/data\/gallery\/submissions\/[0-9a-f]+_)[a-z]+(\.[^/.]*)$/, "$1full$2");
		}

		if (domain === "img.luxusbenz.com" ||
			domain === "img.bmwcase.com" ||
			domain === "img.benzspirit.com") {
			return src.replace(/(:\/\/[^/]*\/)[a-z]+(\/[a-f0-9]+(?:-[^/]*)?\.[^/.]*)$/, "$1full$2");
		}

		if (domain === "dot.asahi.com") {
			return src.replace(/\/upload\/[0-9]+_([0-9]{10,}_[0-9]+\.[^/.]*)$/, "/upload/$1");
		}

		if (domain === "news-img.dwango.jp" ||
			amazon_container === "green-img-news-dwango-jp-prod") {
			return src.replace(/(\/uploads\/[a-z]+\/file\/.*\/)(?:md|sm|lg)_([^/]*)$/, "$1$2");
		}

		if (domain_nowww === "getnews.jp") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/thumb2\/ext\/([a-z]+:\/\/.*)$/, "$1");
		}

		if (domain_nowww === "amuro.fr" ||
			domain_nowww === "freshfappening.com" ||
			domain_nowww === "pornoplaatjes.nl" ||
			domain_nowww === "thefappening.sexy" ||
			domain_nowww === "thefappening.rocks" ||
			domain === "gallery.cyclegarden.com" ||
			src.match(/\/_data\/+i\/+upload\/+/)) {
			newsrc = src.replace(/(\/_data\/+i\/+(?:upload|galleries)\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9]{10,}-[0-9a-f]+)-th\./, "$1-me.");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/\/i\.php\?\/upload\/+/, "/_data/i/upload/");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/\/_data\/+i(\/+(?:upload|galleries)\/+(?:[0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9]{10,}-[0-9a-f]+|[^/]+\/+[^/]+\/+[^/]+))-(?:[^/.]*|cu_s[0-9]+x[0-9]+)(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nosub === "cdn-news30.it") {
			return src.replace(/\/blobs\/variants\/((?:[a-f0-9]\/){4}[-0-9a-f]+)_[a-z]+(\.[^/.]*)$/, "/blobs/full/$1$2");
		}

		if (domain === "imagebox.cz.osobnosti.cz" ||
			domain === "img.osobnosti.cz") {
			return src.replace(/\/[A-Z]([0-9]+)(?:-[0-9]+x[0-9]+)?(-[0-9a-z]+)?(\.[^/.]*)(?:[?#].*)?$/, "/O$1$2$3");
		}

		if (domain === "spicy.southdreamz.com") {
			return src.replace(/\/cache\/(.*?)(?:_[0-9]+|(?:(?:_[a-z][0-9]+){1,}(?:_thumb)?))(\.[^/.]*)$/, "/spicysource/$1$2");
		}

		if (domain === "img.miumag.pl") {
			return src.replace(/\/[a-z]([0-9]+\.[^/.]*)$/, "/c$1");
		}

		if (domain_nowww === "hotnews.bg") {
			return src.replace(/(\/uploads\/news\/[^/]*\/)thumb(\/[0-9]+\.[^/.]*)$/, "$1images$2");
		}

		if (domain === "c6oxm85c.cloudimg.io" ||
			domain === "mediaproxy.salon.com" ||
			domain === "ce8737e8c.cloudimg.io") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/(?:cdn[a-z]\/n\/)?(?:(?:width|height)\/[0-9]+\/)*(?:q[0-9]*\/)?([a-z]*:\/\/)/, "$1");
		}

		if (domain_nowww === "kinataka.ru") {
			return src.replace(/\/photos\/[0-9]+\/([^/]*)$/, "/photos/$1");
		}

		if (domain === "easttouch.my-magazine.me") {
			return src.replace(/(\/upload\/photoalbum\/)thumbnail(\/[0-9a-f]+\/[0-9a-f]+\.[^/.]*)$/, "$1original$2");
		}

		if (domain === "tvdaily.asiae.co.kr") {
			return src.replace(/\/upimages\/+photoda\//, "/upimages/gisaimg/");
		}

		if (domain === "img.thefactjp.com" ||
			domain === "img.thefact.tv") {
			return src.replace(/(\/service\/[a-z]+\/[0-9]+\/[0-9]+\/[0-9]+\/)(?:[0-9]+|thumb)\/([0-9a-f]+\.[^/.]*)$/, "$1$2");
		}

		if (domain === "rlv.zcache.co.nz" ||
			domain === "rlv.zcache.ca" ||
			domain === "rlv.zcache.com.br") {
			return src.replace(/(-r[0-9a-f]+(?:_[_0-9a-z]+)?)_[0-9]+(\.[^/.?]*)(?:\?.*?)?$/, "$1_999999999$2?rvtype=content");
		}

		if (domain_nowww === "empireposter.de") {
			return src.replace(/\/bilder\/bilder_[a-z]\//, "/bilder/bilder_l/");
		}

		if (domain === "dg31sz3gwrwan.cloudfront.net") {
			return src.replace(/(\/[0-9]+\/[0-9]+)_[a-z]+(?:-optimized-[0-9]*)?(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "img.mach-shiko.net") {
			return src.replace(/(\/images\/[0-9]+\/[0-9]+\/)[a-z]\/([0-9]+\.[^/.]*)$/, "$1$2");
		}

		if (domain === "img.onvshen.com") {
			return src.replace(/(\/gallery\/+[0-9]+\/+[0-9]+\/+)[a-z]\/+([0-9]+\.[^/.]*)$/, "$1$2");
		}

		if (domain === "img.geinou-img.com") {
			return src.replace(/(\/[_0-9]+\/)[a-z]\/([0-9]+\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "showbiz.cz") {
			return src.replace(/(?:\/public)?(\/+files\/+gallery\/+)thumb\/+([0-9a-f]+\/[0-9a-f]+)(?:_[a-z]+)?(\.[^/.]*)$/, "$1$2$3");
		}

		if (domain === "images.nudereviews.com") {
			return src.replace(/(\/data\/media\/image\/)[a-z]+\//, "$1original/");
		}

		if (domain_nowww === "thefamouspeople.com") {
			return src.replace(/\/profiles\/thumbs\//, "/profiles/images/");
		}

		if (domain === "i.trust.ua") {
			return src.replace(/\/files\/photo\/[0-9]+\//, "/files/photo/source/");
		}

		if (domain === "media.beritagar.id") {
			return src.replace(/\/(?:lme-)?([0-9]{4}-[0-9]{2}\/)(?:[a-z]+_[0-9]+\/)?(?:wd\/+)?((?:[0-9a-f]+|[^/]*_[0-9]{5,}))(?:_imresized)?(\.[^/.]*)(?:[?#].*)?$/,
							   "/$1$2$3");
		}

		if (domain === "imgs.ckcdn.com" ||
			domain === "i.imgscc.com") {
			return {
				url: src.replace(/\?.*$/, ""),//src.replace(/\?.*$/, "?_w=9999999999999"),
				headers: {
					Origin: null,
					Referer: null
				},
				can_head: false
			};
		}

		if (domain === "im.mtv.fi" ||
			domain_nowww === "mz-web.de" ||
			domain_nowww === "swissinfo.ch") {
			return src.replace(/\/image(\/[0-9]+\/)(?:(?:portrait|landscape[^/]*|3x2|max)\/[0-9]+\/[0-9]+\/)?([0-9a-f]+\/)[a-zA-Z0-9]{2}\/([^/.]+)(\.[^/.]*)$/,
							   "/blob$1$2$3-data$4");
		}

		if (domain_nowww === "imageweb.ws" ||
			domain_nowww === "pezporn.com" ||
			domain_nowww === "hardcoreluv.com" ||
			domain_nosub === "pussyspot.net") {
			newsrc = src.replace(/(:\/\/)cdn[0-9]*\.([^/]*\/+media\/+images)/, "$1www.$2");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/\/media\/+images_[0-9]+\//, "/media/images/");
			if (newsrc !== src)
				return newsrc;
		}

		if ((domain === "t.imageweb.ws" ||
			 domain_nowww === "pezporn.com" ||
			 (domain_nosub === "pussyspot.net" && domain.match(/^cdn[0-9]*\./))
			) &&
			string_indexof(src, "/media/") && options && options.cb && options.do_request) {
			match = src.match(/\/media\/+[^/]*\/+[0-9]+\/+[0-9]+\/+([0-9]+)\.[^/.]*/);
			if (match) {
				id = match[1];

				var queryurl = "http://www." + domain_nosub + "/a/" + id + ".html";
				options.do_request({
					url: queryurl,
					method: "GET",
					onload: function(resp) {
						if (resp.readyState === 4) {
							var match = resp.responseText.match(/href="(\/media\/images\/[^"]*)"/);
							if (match) {
								options.cb({
									url: urljoin("http://www." + domain_nosub + "/", match[1], true).replace(/\?.*/, "")
								});
							} else {
								options.cb(null);
							}
						}
					}
				});

				return {
					waiting: true
				};
			}
		}

		if ((domain_nosub === "pictoa.com" ||
			 domain === "s.lilitube.com" ||
			 domain_nosub === "babaporn.com")
			&& domain.match(/^s[0-9]*\./)) {
			return src.replace(/(\/[0-9a-f]{3}\/+[0-9a-f]{3}\/+[0-9a-f]+\/+)thumbs\/+([^/]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "vcg.com" &&
			domain.match(/goss[0-9]*\.vcg/)) {
			return src.replace(/\/editorial\/vcg\/[0-9]+\//, "/editorial/vcg/nowarter800/");
		}

		if (domain_nosub === "renault-dacia.com.ua") {
			newsrc = urljoin("http://www.renault-dacia.com.ua/", decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/im\.php.*?[?&]image=([^&]*).*?$/, "$1")), true);
			if (newsrc !== src)
				return newsrc;

			return src.replace(/\/pictgen\/[0-9]+\//, "/topicsfoto/");
		}

		if (domain_nowww === "numberoneturk.com.tr" ||
			domain_nowww === "fashionone.com.tr") {
			return src.replace(/(\/plog-content\/+)thumbs(\/+.*\/+)[a-z]+\/+[0-9]+-([^/]*)$/, "$1images$2$3");
		}

		if (domain === "prsize.allviki.com") {
			return src.replace(/:\/\/[^/]*\/resize_[0-9]+(?:x[0-9]+)?\//, "://pic.allviki.com/");
		}

		if (domain_nowww === "celebritygalls.com") {
			return src.replace(/\/cache\/(.*)_[0-9]+_cw[0-9]+_ch[0-9]+_thumb(\.[^/.]*)$/, "/albums/$1$2");
		}

		if (domain_nowww === "instantfap.com") {
			return src.replace(/:\/\/[^/]*\/image\/([^/]*)$/, "://i.imgur.com/$1");
		}

		if (domain_nowww === "agensite.com" ||
			domain === "cdn.agensite.online" ||
			domain === "cdn.conexaohost.com.br") {
			return {
				url: src
					.replace(/\/img\/[0-9]+x[0-9]+\//, "/img/original/")
					.replace(/\/temp\/img\/([0-9a-z]+)_([0-9]+)_([0-9a-z]+)_([0-9a-z]+)_([0-9]+)_([a-z]+)_[^/]*$/,
							 "/img/original/0/0/png/$1/$2/$3/$4/$5.$6")
					.replace(/\/imagizer_export\.php\?([^,]*).*/, "/img/original/0/0/png/$1"),
				redirects: true
			};
		}

		if (domain === "images.mtvnn.com") {
			return src.replace(/(:\/\/[^/]*\/[0-9a-f]+\/)[0-9]+(?:x[0-9]+)?_?(\.[^/.?]*)?(?:[?#].*)?$/, "$1original$2");
		}


		if (domain_nowww === "hipernoticias.com.br" ||
			domain_nowww === "noticiamax.com.br") {
			return src.replace(/(\/storage\/+webdisco\/+[0-9]+\/+[0-9]+\/+[0-9]+\/).*(\/[0-9a-f]+\.[^/.]*)$/, "$1original$2");
		}

		if (domain_nowww === "bd-journal.com") {
			match = src.match(/\/image-contents\/[0-9]+x[0-9]+x[0-9]+\/news-photos\/((?:[0-9]+\/){2,})([-0-9a-zA-Z_=+]+)(?:[?#].*)?$/);
			if (match) {
				return src.replace(/(:\/\/[^/]*\/).*/, "$1") + "assets/news_photos/" + match[1] + base64_decode(match[2]);
			}
		}

		if (domain_nowww === "myidol.com.vn") {
			return src.replace(/\/pictures\/pic[a-z]+(\/[0-9]+\/[0-9]+\/[0-9]+\/)[0-9]+\/([a-z]+[0-9]+\.[^/.]*)$/,
							   "/pictures/picfullsizes/$1$2");
		}

		if (domain_nowww === "dw.com") {
			return src.replace(/(:\/\/[^/]*\/image\/[0-9]+_)[0-9]+(\.[^/.]*)$/, "$17$2");
		}

		if (domain_nowww === "finds.ir") {
			return src.replace(/\/img\/tu\//, "/img/");
		}

		if ((domain_nosub === "xuk.mobi" ||
			 domain_nosub === "xuk.one" ||
			 domain_nosub === "xuk.life" ||
			 domain_nosub === "xuk.ooo") &&
			domain.match(/^img[0-9]*\./) &&
			string_indexof(src, "/images/photos/") >= 0) {
			return src.replace(/\/(?:thumb|big|mini)\/+([0-9a-f]+\.[^/.]*)$/, "/origin/$1");
		}

		if (domain === "s.filmsextv.com") {
			return src.replace(/(\/[0-9a-f]+\/)thumbs\/([0-9a-f]+\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "lanita.ru" &&
			string_indexof(src, "/images/offer/") >= 0) {
			return src.replace(/(\/[0-9]+)-thumb(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "backalleypics.net") {
			return src.replace(/(\/images\/(?:[0-9]+-[0-9]+-[0-9]+|photoshoots)\/)thumbs\/([^/]*)$/, "$1$2");
		}

		if ((domain_nosub === "turboimg.net" ||
			 domain_nosub === "turboimagehost.com") &&
			options && options.cb && options.do_request) {
			id = src.replace(/.*\/([0-9]+)_([^/]*)$/, "$1/$2");
			if (id !== src) {
				var nid = id.split("/")[0];
				var pid = id.split("/")[1];
				var queryurl = "http://www.turboimagehost.com/p/" + nid + "/" + pid + ".html";
				options.do_request({
					url: queryurl,
					method: "GET",
					onload: function(resp) {
						if (resp.readyState === 4) {
							var match = resp.responseText.match(/<meta *property="og:image" *content="([^"]*)"/);
							if (match) {
								options.cb({
									url: match[1],
									is_original: true,
									extra: {
										page: resp.finalUrl
									}
								});
							} else {
								options.cb(null);
							}
						}
					}
				});

				return {
					waiting: true
				};
			}
		}

		if (domain_nowww === "arhivach.cf" ||
			domain_nowww === "arhivach.ng") {
			return src.replace(/(\/storage[0-9]*\/)t\/([0-9a-f])([0-9a-f]{2})([0-9a-f]+\.[^/.]*)$/, "$1$2/$3/$2$3$4");
		}

		if (domain === "content.wafflegirl.com" ||
			domain_nosub === "youx.xxx" ||
			domain_nosub === "xxxwaffle.com" ||
			domain === "cdn.pornpictureshq.com") {
			return src.replace(/\/gthumb\/(.*)(?:__x[0-9]+|_[0-9]+x(?:[0-9]+)?_?)(\.[^/.]*)$/, "/content/$1$2");
		}

		if (domain_nowww === "anawalls.com") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/repic\/image\.php.*?[?&]src=([^&]*).*?$/, "$1"));
		}

		if (domain === "d3ofq03apmfb8c.cloudfront.net" ||
			domain_nowww === "lipstickalley.com" ||
			(domain_nosub === "mandatory.com" && domain.match(/^cdn[0-9]*-hfboards\./)) ||
			domain === "forum.purseblog.com" ||
			(domain_nowww === "behindbigbrother.com" && string_indexof(src, "/forums/data/") >= 0) ||
			domain_nowww === "simsettlements.com" ||
			src.match(/^[a-z]+:\/\/[^/]*\/+(?:community\/+)?data\/+avatars\/+[sml]\/+[0-9]+\/+[0-9]+\.[a-z]+(?:\?[0-9]+)?$/)) {
			var regex = /\/data\/+avatars\/+[a-z]\/+([0-9]+\/+[0-9]+\.[^/.]*)$/;
			return [
				src.replace(regex, "/data/avatars/o/$1"),
				src.replace(regex, "/data/avatars/l/$1")
			];
		}

		if (domain === "img-cache.cdn.gaiaonline.com") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/[0-9a-f]+\//, "");
		}

		if (domain === "y.zdmimg.com") {
			return src.replace(/(\/[0-9a-f]+\.[^/._]*)_[a-z][0-9]+\.[^/.]*$/, "$1");
		}

		if (domain_nosub === "oboi.ws") {
			return src.replace(/\/wallpapers\/(?:[0-9]+|[a-z])_([0-9]+_[^/]*)(?:_[0-9]+x[0-9]+)?(\.[^/.]*)$/, "/originals/original_$1$2");
		}

		if (domain === "images.wallpaperscraft.com" &&
		   options && options.cb && options.do_request) {
			id = src.replace(/.*\/([^/]*)_[0-9]+x[0-9]+(\.[^/.]*)$/, "$1/$2");
			if (id !== src) {
				var pid = id.split("/")[0];
				var ext = id.split("/")[1];
				options.do_request({
					url: "https://wallpaperscraft.com/wallpaper/" + pid,
					method: "GET",
					onload: function(resp) {
						if (resp.readyState !== 4) {
							return;
						}

						var match = resp.responseText.match(/<a[^>]*\/download\/[^>]*>\s*Original Resolution: *([0-9]+x[0-9]+)/);
						if (match) {
							options.cb("https://images.wallpaperscraft.com/image/" + pid + "_" + match[1] + ext);
						} else {
							options.cb(null);
						}
					}
				});
			}

			return {
				waiting: true
			};
		}

		if (domain === "cdn.eso.org" ||
			domain_nowww === "eso.org") {
			return src.replace(/(\/images\/|\/archives\/postcards\/)[^/]*\/([^/.]*\.[^/.]*)$/, "$1large/$2");
		}

		if (domain === "www.galex.caltech.edu" ||
			domain === "galex.caltech.edu") {
			return src
				.replace(/(_img[0-9]*)_(?:Sm|small)(\.[^/.]*)$/, "$1$2")
				.replace(/(_vid[0-9]*)_tn(\.[^/.]*)$/, "$1_shot$2");
		}

		if (domain_nowww === "astro-austral.cl") {
			return src.replace(/(\/imagenes\/galaxies\/[^/]*\/)[a-z]+(\.[^/.]*)$/, "$1max$2");
		}

		if (domain === "cdn.spacetelescope.org") {
			return src.replace(/(\/archives\/images\/)[^/]*(\/[^/.]*\.[^/.]*)$/, "$1large$2");
		}

		if (domain_nowww === "buddhistdoor.net") {
			return src.replace(/(\/upload\/file\/[0-9]+\/[0-9]+\/[0-9a-f]+)_[0-9]+(?:__[0-9]+)?(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "agraroldal.hu") {
			return add_extensions(src.replace(/\/[^/]*\/[0-9]+x[0-9]+\/([0-9]+\.[^/.]*)$/, "/upload/upimages/$1"));
		}

		if (domain === "media.lamsao.com") {
			return src.replace(/\/Thumbnail\/Cache\/*Data\/(.*)_[0-9]+_[0-9]+(\.[^/.]*)$/, "/Data/$1$2");
		}

		if (domain === "tupian.aladd.net") {
			return src.replace(/(\.[^-/.]*)-[0-9]+(?:[?#]*)?$/, "$1");
		}

		if (domain === "uploadfile.huiyi8.com" ||
			domain === "i.hexuexiao.cn" ||
			(domain_nowww === "popwindshop.com" && string_indexof(src, "/pics/") >= 0)) {
			return src.replace(/(\.[^/.]*)\.[0-9]+\.[^/.]*$/, "$1");
		}

		if ((domain_nosub === "redocn.com" && domain.match(/^img[0-9]*\./)) ||
			domain === "image.tupian114.com") {
			return {
				url: src.replace(/(\/[^/]*\.[^/.]*)\.[0-9]+\.[^/.]*$/, "$1"),
				problems: {
					watermark: true
				}
			};
		}

		if (domain === "media.alienwarearena.com") {
			return src.replace(/\/thumb(?:nail)?_[0-9]+x[0-9]+\/+([0-9a-f]{10,}\.[^/.]*)$/, "/media/$1");
		}

		if (domain_nowww === "6asian.com") {
			return src.replace(/(\/imglink\/[a-z]+)-thumb[0-9]+x[0-9]+\//, "$1/");
		}

		if (domain === "music.fetnet.net") {
			return src.replace(/(\/img\/album\/[0-9]+)-[0-9]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "wallpaperscave.ru") {
			return src.replace(/\/images\/thumbs\/[^/]*\/[0-9]+x[0-9]+\//, "/images/original/");
		}

		if (domain_nowww === "lilit.lv") {
			return src.replace(/\/[0-9]+px_([0-9]+_[0-9a-f]+\.[^/.]*)$/, "/full_$1");
		}

		if (domain === "img.kurocore.com") {
			return src.replace(/(:\/\/[^/]*\/)thumbnail\/p\//, "$1p/");
		}

		if (domain_nowww === "mocah.org") {
			return src.replace(/(:\/\/[^/]*\/)thumbs\/([0-9]+-[^/]*)$/, "$1uploads/posts/$2");
		}

		if (domain_nowww === "anime-zone.ru") {
			return src.replace(/(\/inc\/goods_[a-z]+\/[^/]*\/)[a-z]+\/([^/]*)$/, "$1$2");
		}

		if (domain_nowww === "99px.ru") {
			return src.replace(/:\/\/[^/]*\/sstorage\/[0-9]+\/[0-9]+\/[0-9]+\/[a-z]+_([0-9]+)_[0-9]+\.[^/.]*$/,
							   "://wallpapers.99px.ru/cms/mhost.php?tid=53&act=getimage&id=$1");
		}

		if (domain_nowww === "wall2born.com") {
			return src.replace(/\/file\/download\/[0-9]+x[0-9]+\/([0-9]+\/[^/]*)-[0-9]+x[0-9]+(\.[^/.]*)$/,
							   "/data/out/$1$2");
		}

		if (domain_nowww === "wallpapersexpert.com") {
			return src.replace(/\/images\/+file\/+([0-9]+\/+)[0-9]+x[0-9]+-([0-9]+-)/, "/data/out/$1$2");
		}

		if (amazon_container === "desktop-backgrounds-org") {
			return src.replace(/\/(?:[^-/]*-)?[0-9]+x[0-9]+\/([^/]*\.[^/.]*)$/, "/$1");
		}

		if (domain_nowww === "oboi.cc" ||
			domain_nowww === "oboik.ru") {
			return src.replace(/\/(?:[-0-9]+)?uploads(?:_full)?(\/[0-9]+_[0-9]+_[0-9]+\/)[a-z]+(\/[0-9]+\/[^/]*)$/,
							   "/uploads$1view$2");
		}

		if (domain === "img.getbg.net") {
			return src.replace(/\/upload\/[a-z]+\/([0-9]*\/)(?:thumbnail_)?([^/]*)$/,
							   "/upload/full/$1$2");
		}

		if ((domain_nosub === "fichub.com" ||
			 domain === "scontent.ccdn.cloud")
			&& string_indexof(src, "/image/") >= 0) {
			return src.replace(/(\/image\/+[^/]+\/+[-0-9a-f]{30,}\/+[^/]+?)(?:-(?:maxw-[0-9]+|[0-9]+x[0-9]+))?(?:\/+[0-9]+x[0-9]+)?(\.[^/.]*)$/,
							   "$1$2");
		}

		if (domain_nowww === "nishinippon.co.jp" && string_indexof(src, "/import/") >= 0) {
			return src.replace(/(\/[0-9]+\/[0-9]+_[0-9]+)_s(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "nishinippon.co.jp") {
			return src.replace(/(\/uploads\/+image\/+[0-9]+\/+)[a-z]+_([0-9a-f]{6,}(?:_org)?\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "resource.shuud.mn") {
			return src.replace(/(\/[0-9]+)_t(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "anime-fresh.ru") {
			return src.replace(/(\/upload\/[a-z]+\/)thumbs\/([^/]*)$/, "$1$2");
		}

		if (domain === "imgfiles.plaync.com") {
			return src.replace(/\/download_thumbnail\/([^/]*)$/, "/download/$1");
		}

		if (domain_nowww === "jjdao.com") {
			return src.replace(/(\/wiki\/uploads\/[0-9]+\/[0-9+][0-9a-zA-Z]+)_[a-z](\.[^/.]*)$/, "$1$2");
		}

		if (domain === "t.huv.kr") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/thumb_crop_resize\.php.*?[?&]url=([^&?]*).*?$/, "$1"));
		}

		if (domain === "down.humoruniv.org") {
			return {
				url: src,
				headers: {
					Referer: ""
				}
			};
		}

		if (domain_nosub === "pocoimg.cn" &&
			domain.match(/^img[0-9]*\./)) {
			return src.replace(/_[WH][0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "qcmt.bid") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/upload\/.*?[?&]src=([^&]*).*?$/, "$1"));
		}

		if (domain === "pic.xiao4j.com") {
			return src.replace(/(\/[0-9]+\.[^/._]*)_[0-9]+_[0-9]+\.[^/.]*$/, "$1");
		}

		if (domain === "img.mm29.com") {
			return src.replace(/(\/[0-9]+\.[^/.]*)\/[0-9]+\.[^/.]*$/, "$1");
		}

		if (domain_nosub === "2chmatome2.jp" &&
			domain.match(/^image[0-9]*\./)) {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/v2\/thumb\/app\/[0-9]+\/[0-9]+\/.*?[?&]url=([^&]*).*?$/, "$1"));
		}

		if (domain === "file.2chmatome2.jp") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/(http)/, "$1");
			if (newsrc !== src) {
				return decodeURIComponent(decodeURIComponent(newsrc));
			}
		}

		if (domain_nowww === "wnacg.net") {
			return src.replace(/\/data\/thumb\//, "/data/");
		}

		if (domain === "rs.n1info.com") {
			return src.replace(/\/Thumbnail(\/[0-9]+\/)/, "/Picture$1");
		}

		if (domain_nowww === "adevarul.ro" && string_indexof(src, "/MRImage/") >= 0) {
			return src.replace(/\/[0-9]+x[0-9]+(\.[^/.]*)$/, "/orig$1");
		}

		if (domain_nowww === "eva.bg") {
			return src.replace(/\/media_cache\/[^/]*\/media\//, "/media/");
		}

		if (domain === "data.kontrakty.ua") {
			return src.replace(/(\/cache\/[a-z]+\/)[0-9]+\//, "$1orig/");
		}

		if (domain === "d3t543lkaz1xy.cloudfront.net") {
			return src.replace(/(\/photo\/[0-9a-f]+)_[a-z](?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "merokhabar.net") {
			return src.replace(/\/thumbnail\/thumb([^/]*)$/, "/original/ori$1");
		}

		if (domain_nosub === "ali213.net" &&
			domain.match(/^img[0-9]*\./)) {
			return src.replace(/\/[0-9]+_([0-9]+\.[^/.]*)$/, "/$1");
		}

		if (domain === "reseuro.magzter.com" ||
			domain === "rse.magzter.com") {
			return src
				.replace(/:\/\/[^/]*\/[0-9]+x[0-9]+\/articles\//, "://magarticles.magzter.com/articles/")
				.replace(/:\/\/rse\.[^/]*\/[0-9]+x[0-9]+\//, "://cdn.magzter.com/");
		}

		if (domain === "dev.magzter.com") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/dynimage\/thumb_news\.php.*?[?&]src=([^&]*).*$/, "$1"));
		}

		if (domain === "img.bulawayo24.com") {
			return src.replace(/\/articles\/thumbs\/[0-9]+x[0-9]+\//, "/articles/");
		}

		if (domain === "static.eva.ro") {
			return src.replace(/\/img\/auto_resized\/db\/(.*)-[0-9]+x[0-9]+(?:-.-[0-9a-f]+)?(\.[^/.]*)$/, "/img/db/$1$2");
		}

		if (domain === "s-image.hnol.net") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/x\/[0-9]+x[0-9]+\/auto\/(https?):\/\/?/, "$1://");
		}

		if (domain === "cdn.tower.jp") {
			return src.replace(/(:\/\/[^/]*\/za\/)[a-z]\//, "$1o/");
		}

		if (domain_nowww === "gamers-onlineshop.jp" ||
			domain_nowww === "animate-onlineshop.jp" ||
			domain_nowww === "acoop-onlineshop.jp" ||
			domain === "cdn.melonbooks.co.jp" ||
			domain_nowww === "melonbooks.co.jp" ||
			domain === "jpstore.dwango.jp" ||
			domain === "melonbooks.akamaized.net") {
			return src.replace(/(:\/\/[^/]*\/)(?:user_data\/packages\/)?resize_image\.php.*?[?&]image=([^&]*).*$/, "$1upload/save_image/$2");
		}

		if (domain_nowww === "hmv.com.hk") {
			return src.replace(/\/data\/upload\/[0-9]+\//, "/data/upload/");
		}

		if (domain_nowww === "buysmartjapan.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/images\/[0-9a-f]+.*?[?&]original=([^&]*).*$/, "$1");
			if (newsrc !== src)
				return decodeURIComponent(newsrc);
		}

		if (domain_nowww === "sonymusic.co.jp") {
			return src.replace(/(\/adm_image\/common\/.*)__[0-9]+_[0-9]+_[0-9]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nosub === "imageban.ru" &&
			domain.match(/^i[0-9]*\./)) {
			return src.replace(/(:\/\/[^/]*\/)thumbs\/([0-9]+)\.([0-9]+)\.([0-9]+)\/([0-9a-f]+\.[^/.]*)$/, "$1out/$2/$3/$4/$5");
		}

		if (domain === "image.hackadoll.com") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/thumbs.*?[?&]u=([^&]*).*$/, "$1"));
		}

		if (domain === "images.niooz.fr") {
			return urljoin(src, decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/safe_image\.php.*?[?&]i=([^&]*).*$/, "$1")), true);
		}

		if (domain_nowww === "mengchongzhi.com") {
			return src.replace(/\/uploadfile\/thumb\//, "/uploadfile/");
		}

		if (domain_nowww === "playground33.com") {
			return src.replace(/(\/uploads\/(?:[0-9]\/){4}[0-9]+\/[0-9]+)(\.[^/.]*)$/, "$1_orig$2");
		}

		if (domain_nosub === "trends.com.cn" &&
			domain.match(/^cdn[0-9]*\./)) {
			return src.replace(/-tweb\.[a-z]+(?:\.[^/.]*)?(?:[?#].*)?$/, "");
		}

		if (domain_nowww === "la-soubrette.fr") {
			return src
				.replace(/\/photo-ps\/+([^/]*)-([0-9]+)\./, "/photo-ps-grande/$2-$1.")
				.replace(/\/thumbs-photo-nue\//, "/photo-ps-nue/");
		}

		if (domain === "static.feber.se" && string_indexof(src, "/article_images/") >= 0) {
			return src.replace(/(\/[0-9]+)_[0-9]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "previiew.com" ||
			domain_nowww === "lfi-online.de" ||
			domain_nowww === "s-magazine.photography" ||
			domain_nowww === "m-magazine.photography" ||
			domain_nowww === "l-mount.com" ||
			domain_nowww === "bettinaschoenbach.com" ||
			domain_nowww === "portrix-ls.de" ||
			domain_nowww === "glocon.eu" ||
			domain_nosub === "olympus.eu") {
			return {
				url: src.replace(/(\/webfile\/+img\/+[0-9]+)(?:\/+[xy]=[0-9]+)*(?:\/+[a-zA-Z0-9=]{10,})?(\/[^/]+\.[^/.?]+)?(?:[?#].*)?$/, "$1$2"),
				can_head: false
			};
		}

		if (domain === "d2e7nuz2r6mjca.cloudfront.net") {
			return src.replace(/-[0-9]+[wh](\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "vplate.ru" ||
			domain_nowww === "wlooks.ru") {
			return src.replace(/\/images\/article\/cropped\/[0-9]+-[0-9]+\//, "/images/article/orig/");
		}

		if (domain === "media.vogue.com") {
			return src.replace(/(:\/\/[^/]*\/)r\/[a-z]_[0-9]+(?:,[a-z]_[0-9]+)*\//, "$1r/original/");
		}

		if (domain_nosub === "efu.com.cn" &&
			domain.match(/^img[0-9]*\./)) {
			return src.replace(/(\/upfile\/[^/]*\/photo\/)[a-z]+\/([0-9]+\/[0-9]+\.[^/.]*)$/, "$1$2");
		}

		if (domain === "images.stopgame.ru") {
			return src
				.replace(/(\/articles\/[0-9]+\/[0-9]+\/[0-9]+\/)c[0-9]+x[0-9]+\/[0-9a-zA-Z]+\/([^/]*)$/, "$1$2")
				.replace(/(\/articles\/[0-9]+\/[0-9]+\/[0-9]+\/[^/]*-[0-9]+)-s(\.[^/.]*)$/, "$1$2")
				.replace(/(\/uploads\/images\/[0-9]+\/[a-z]+\/[0-9]+\/[0-9]+\/[0-9]+\/)[a-z]+_([0-9]+\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "pure-nude-celebs.com" ||
			domain_nowww === "celebsandstarsnude.com" ||
			domain_nowww === "celebsking.com") {
			return src.replace(/\/pic-([^/]*)$/, "/$1");
		}

		if (domain_nowww === "pincelebs.net") {
			return src.replace(/\/thumbs(\/(?:[0-9a-f]\/){3}[0-9a-f]+\.[^/.]*)$/, "/images$1");
		}

		if (domain === "img.new2005.com") {
			return src.replace(/(\/[0-9a-f]+\.[^/._]+)_old(?:[?#].*)?$/, "$1");
		}

		if (domain === "s.libertaddigital.com") {
			return src
				.replace(/(:\/\/[^/]*\/(?:(?:fotos\/+(?:galerias\/+[^/]*|[a-z]+))|(?:[0-9]{4}\/+[0-9]{2}\/+[0-9]{2}))\/+)[0-9]+\/+[0-9]+\/+(?:fit\/+)?([^/]*)(?:[?#].*)?$/,
						 "$1$2");
		}

		if (domain_nowww === "imgstudio.org" ||
			domain_nowww === "imgadult.com" ||
			domain_nowww === "imageboom.net" ||
			domain_nowww === "acidimg.cc" ||
			domain_nowww === "imagedecode.com" ||
			domain_nowww === "damimage.com" ||
			domain_nosub === "imgcredit.xyz") {
			newsrc = src.replace(/\/upload\/+[-a-z]+\//, "/upload/big/");
			if (newsrc != src)
				return newsrc;

			return {
				url: src,
				headers: {
					Referer: "http://" + domain_nosub + "/",
					Cookie: "showing=3"
				}
			};
		}

		if (domain === "i.acidimg.cc") {
			return src.replace(/(:\/\/[^/]*\/)[a-z]+\//, "$1big/");
		}

		if (domain_nowww === "imgwallet.com" ||
			domain_nowww === "imgdrive.net" ||
			domain_nowww === "imgtaxi.com") {
			return src.replace(/\/images\/[-a-z]+\//, "/images/big/");
		}

		if (domain === "img.depo.ua") {
			return src.replace(/(:\/\/[^/]*\/)[0-9X]+x[0-9X]+\//, "$1original/");
		}

		if ((domain_nowww === "elfagr.com" ||
			 domain_nowww === "arabmubasher.com" ||
			 domain_nowww === "elbalad.news" ||
			 domain_nowww === "elmwatin.com" ||
			 domain_nowww === "dostor.org" ||
			 domain_nowww === "studio-dostor.org" ||
			 domain_nowww === "albawabhnews.com") &&
			/\/upload\/+photo\//.test(src)) {
			return src.replace(/\/+[0-9]+x[0-9]+[a-z]?\/+([^/]*?)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nowww === "cdnimpuls.com") {
			return src.replace(/\/-[0-9]+-[0-9]+-([0-9a-f]+(?:_[^/]*)?\.[^/.]*)$/, "/$1");
		}

		if (domain_nowww === "mshreqnews.net" ||
			(domain_nowww === "mykentfamily.co.uk" && string_indexof(src, "/_media/") >= 0) ||
			domain_nowww === "pdfkul.com") {
			newsrc = src.replace(/\/img\/+[0-9]+x[0-9]+\/+([^/]*)(?:[?#].*)?$/, "/img/$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "choisirunfilm.fr") {
			return src.replace(/\/persons\/small\/([0-9]+\.[^/.]*)$/, "/persons/$1");
		}

		if (domain_nosub === "gazetaexpress.com" ||
			domain === "old.fishmedia.info") {
			return src.replace(/(\/public\/+uploads\/+image\/+[0-9]+\/+[0-9]+\/+)[0-9]+x[0-9]+\/+([^/]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "kosovapress.com") {
			return src.replace(/(\/public\/+uploads\/+image\/+)[0-9]+x[0-9]+\/+/, "$1");
		}

		if (domain_nowww === "haynews.am") {
			return src.replace(/\/thumbs\/[0-9]+x(?:[0-9]+)?\//, "/images/");
		}

		if (domain_nowww === "emlakkulisi.com") {
			return src.replace(/\/resim\/[a-z]+\//, "/resim/orjinal/");
		}

		if (domain_nowww === "aquarellefm.md" && string_indexof(src, "/storage/") >= 0) {
			return src.replace(/_Fit_[0-9]+x[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "joy.hu" ||
			domain === "static.marquardmedia.hu") {
			return src.replace(/(\/data\/[^/]*\/[0-9]+\/[0-9]+)(?:-[^/.]*)?\.(?:[0-9]+(?:x[0-9]+)?|layer|w)(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "ivoirematin.com") {
			return src.replace(/(\/images\/[0-9]+-[0-9]+\/)fb_([0-9a-f]+\.[^/.]*)$/, "$1$2");
		}

		if (domain === "img.theqoo.net") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/proxy\.php.*?[?&]url=([^&]*).*?$/, "$1");

			if (newsrc === src) {
				newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/proxy\/(.*)$/, "$1");
			}

			if (newsrc !== src) {
				if (!newsrc.match(/^[a-z]+:\/\//))
					newsrc = "http://" + newsrc;
				return newsrc;
			}
		}

		if (domain_nowww === "gpslifetime.com.br" && string_indexof(src, "/uploads/") >= 0) {
			return src.replace(/\/image\/thumbs\//, "/image/");
		}

		if (domain_nowww === "lasillarota.com" ||
			domain === "lasillarotarm.blob.core.windows.net.optimalcdn.com" ||
			domain === "info7rm.blob.core.windows.net.optimalcdn.com") {
			return src
				.replace(/:\/\/[^/]*lasillarota\.com\/images\/tnfocus\/(?:[0-9]+\/){4}([0-9]+\/[0-9]+\/[0-9]+\/[^/]*)$/, "://lasillarotarm.blob.core.windows.net.optimalcdn.com/images/$1")
				.replace(/(\/images\/[0-9]+\/[0-9]+\/[0-9]+\/[^/]*)-focus(?:-[0-9]+){4}(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "znaj.ua" ||
			domain_nowww === "akcenty.com.ua") {
			return src.replace(/\/crops\/+[a-z0-9]+\/+[0-9]+x[0-9]+\/+[0-9]+\/+[0-9]+\/+([0-9]+\/+[0-9]+\/+[0-9]+\/+[^/]*)$/, "/images/$1");
		}

		if (domain_nowww === "raragente.com.br") {
			return src.replace(/\/imagem\/noticia\/[0-9]+\/[0-9]+\//, "/images/materias/");
		}

		if (domain_nowww === "pastefs.com") {
			return src.replace(/\/resource\/[a-z]+\//, "/resource/files/");
		}

		if (domain_nowww === "sigmalive.com") {
			return src.replace(/\/application\/cache\/[^/]*\/(images\/[^/]*\/)[0-9]+x[0-9]+\//, "/uploads/$1");
		}

		if (domain_nowww === "luna.ovh") {
			return src.replace(/\/imgw\/[0-9]+px\//, "/imgw/");
		}

		if (domain_nowww === "shalala.ru") {
			return src.replace(/(\/upload\/artists\/[0-9]+\/)[a-z]+\/([0-9]+_[0-9]+\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "fay3.com") {
			return src.replace(/\/assets\/uploads\/(?:images_)?thumbs\//, "/assets/uploads/images/");
		}

		if (domain_nowww === "vijesti.ba" ||
			domain_nosub === "novi.ba") {
			return src.replace(/(\/storage\/[0-9]+\/[0-9]+\/[0-9]+\/)thumbs\/([^/]*)-(?:preview(?:Org)?|[0-9]+x[0-9]+)(\.[^/.]*)$/, "$1$2$3");
		}

		if (domain === "m.smedata.sk") {
			return src.replace(/(\/api-media\/media\/image\/.*)_[0-9]+x(\.[^/.]*)$/, "$1$2");
		}

		if ((domain_nosub === "diariolibre.com" && domain.match(/estatico[0-9]*\.diariolibre\.com/)) ||
			domain_nowww === "expreso.ec" ||
			domain_nowww === "vanguardia.com" ||
			domain_nowww === "thesundaily.my" ||
			domain_nowww === "canarias7.es") {
			return src.replace(/\/binrepository\/[0-9]+x[0-9]+\/.*(\/[^/]*)(?:[?#].*)?$/, "/binrepository$1");
		}

		if (domain_nowww === "modelisto.com") {
			return src.replace(/(-[0-9]+)@[0-9]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "media.kg-portal.ru") {
			return src.replace(/(_[0-9]+)[st](\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "desktopwallpaperhd.net") {
			return src.replace(/(:\/\/[^/]*\/)thumbs\//, "$1wallpapers/");
		}

		if (domain_nowww === "wallpaperplay.com") {
			return src.replace(/\/walls\/[^/]*\//, "/walls/full/");
		}

		if (domain_nosub === "pics.vc" &&
			domain.match(/^s[0-9]*\./)) {
			return src.replace(/(:\/\/[^/]*\/pics\/)[a-z]\//, "$1o/");
		}

		if ((domain_nosub === "imageshack.us" ||
			 domain_nosub === "imageshack.com") &&
			domain.match(/^imagizer/)) {
			return src.replace(/(:\/\/[^/]*\/v2\/)[^/]*\//, "$1x/");
		}

		if (domain_nowww === "girlofthehour.com") {
			return src.replace(/\/IMG\/poster_resized\//, "/IMG/poster/");
		}

		if (domain === "hwcdn.ddstatic.com") {
			regex = /(-gal-)[0-9]+(-[a-z]+\/[^/]*)$/;
			return [
				src.replace(regex, "$11600$2"),
				src.replace(regex, "$11024$2")
			];
		}

		if (domain === "img.sexpornpages.com") {
			return src.replace(/\/converted\/+[a-z]+\/+([a-z]+[0-9]*\/)/, "/$1");
		}

		if (domain === "static.hdw.eweb4.com" &&
			options && options.cb && options.do_request) {
			id = src.replace(/.*\/media\/thumbs\/.*\/([0-9]+)\.[^/.]*$/, "$1");
			if (id !== src) {
				options.do_request({
					url: "http://hdw.eweb4.com/out/" + id + ".html",
					method: "GET",
					onload: function(resp) {
						if (resp.readyState === 4) {
							var match = resp.responseText.match(/<meta *property="og:image" *content="([^"]*)"/);
							if (match) {
								options.cb(match[1]);
							} else {
								options.cb(null);
							}
						}
					}
				});

				return {
					waiting: true
				};
			}
		}

		if (domain_nowww === "wallpaperbackgrounds.com") {
			return src.replace(/(\/Content\/wallpapers\/.*\/)thumb-([0-9]+-[0-9]+\.[^/.]*)$/, "$1$2");
		}

		if (domain_nosub === "barelist.com" && /^img[0-9]*\./.test(domain)) {
			return src
				.replace(/(\/images\/+hosted\/+[^/]*\/+[^/]*\/+)thumbs\//, "$1pics/")
				.replace(/\/images\/+gallery_[0-9]+_[0-9]+\//, "/images/gallery/");
		}

		if (domain === "proxy.imgsmail.ru") {
			return add_http(decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/.*?[?&]url[0-9]*=([^&]*).*?$/, "$1")));
		}

		if (domain_nosub === "imgsmail.ru" &&
			domain.match(/^filapp[0-9]*\./)) {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+pic.*?[?^]url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nosub === "imgsmail.ru" && /^my[0-9]*\./.test(domain)) {
			if (/\/mail\/+ru\/+images\/+my\/+compass\/+static\//.test(src)) {
				return {
					url: src,
					bad: "mask"
				};
			}
		}

		if (domain_nowww === "pornoonline.com.pl") {
			return src.replace(/(\/images\/galerie\/[0-9]+\/)[a-z]+\/[a-z]+_([0-9a-f]+\.[^/.]*)$/, "$1$2");
		}

		if (domain === "img.cdn.pornzone.com") {
			return src.replace(/(\/galerie\/+[0-9]+\/+[0-9]+\/+)small_([0-9a-f]{20,}\.)/, "$1big_$2");
		}

		if (domain_nowww === "photo-erotique.org") {
			return src.replace(/(\/girls\/[0-9]+\/)thumb_([0-9]+\.[^/.]*)$/, "$1$2");
		}

		if (domain === "s.mediasole.ru") {
			return src.replace(/:\/\/[^/]*\/cache\/preview\/data\//, "://mediasole.ru/data/");
		}

		if ((domain_nosub === "godsartnudes.com" ||
			 domain_nosub === "godsartglam.com") &&
			domain.match(/^pics[0-9]*\./)) {
			return src.replace(/(-[0-9]+_[0-9]+)(?:_thumb)?(\.[^/.]*)$/, "$1_big$2");
		}

		if (domain_nowww === "gotgalleries.com") {
			return src.replace(/th([0-9]*\.[^/.]*)$/, "$1");
		}

		if ((domain_nowww === "perfectnaked.com" ||
			 domain === "cdn.erocurves.com")
			&& string_indexof(src, "/galleries/") >= 0) {
			return src.replace(/(\/[0-9]+)_tn(\.[^/.]*)$/, "$1_big$2");
		}

		if (domain_nowww === "facefuckingporn.com") {
			return src.replace(/(\/galleries\/+[^/]+\/+[^/]+)_tn(\.[^/.]+)(?:[?#].*)?$/, "$1_big$2");
		}

		if (domain === "cdn.eroticbeauties.net" ||
			domain === "cdn.hometownnudes.com" ||
			domain === "media.pinkworld.com") {
			newsrc = src.replace(/(\/content\/+[^/]*\/+)(?:[^/]*\/+[^/]*\/+)?(?:tn@[^/]*|[0-9]+)\/+([0-9]+\.[^/.]*)$/, "$1full/$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "media.pinkworld.com") {
			return {
				url: src,
				headers: {
					Referer: "http://pinkworld.com/"
				}
			};
		}

		if (domain_nowww === "barahla.net") {
			return src.replace(/(\/images\/photo\/[0-9]+\/[0-9]+\/[0-9]+\/)(?:[a-z]+\/)?([0-9]+)(?:_[a-z]+)?(\.[^/.]*)$/, "$1big/$2_big$3");
		}

		if (domain === "i.pipec.info") {
			return src.replace(/\/[0-9]+px\/([^/]*)$/, "/$1");
		}

		if (domain_nosub === "hothag.com" && domain.match(/^cdn[0-9]*\./)) {
			newsrc = src.replace(/(\/media\/+(?:galleries|images))_[0-9]+\//, "$1/");
			if (newsrc !== src)
				return add_full_extensions(newsrc);

			return src.replace(/(\/media\/+gifs)_[0-9]+(\/+[0-9/]+)\.[^/.]*(?:[?#].*)?$/, "$1/$2.gif");
		}

		if ((domain_nosub === "1zoom.ru" ||
			 domain_nosub === "1zoom.me") &&
			domain.match(/^s[0-9]*\./)) {
			return src.replace(/(:\/\/[^/]*\/)big[0-9]*\//, "$1big3/");
		}

		if (domain === "st-gdefon.gallery.world" ||
			domain === "st.gde-fon.com") {
			return src.replace(/\/wallpapers_[a-z]+(\/[0-9]+_[^/]*)$/, "/wallpapers_original$1");
		}

		if (domain_nowww === "boorp.com") {
			return src.replace(/\/miniature(\/[^/]*)$/, "$1");
		}

		if (domain === "d2pptc4exyus09.cloudfront.net" ||
			domain === "static.puzzlefactory.pl") {
			return src.replace(/(\/puzzle\/[0-9]+\/[0-9]+\/)[a-z]+(\.[^/.]*)$/, "$1original$2");
		}

		if (domain_nowww === "sfondo.info") {
			return src.replace(/\/i\/[a-z]+\//, "/i/original/");
		}

		if (domain_nosub === "yiihuu.com" &&
			domain.match(/^img[0-9]*\./)) {
			return src
				.replace(/\/[0-9]+X[0-9]+\/upimg\//, "/upimg/")
				.replace(/\?.*/, "");
		}

		if (domain_nowww === "superwall.us" ||
			domain_nowww === "rex-fox.com") {
			return src.replace(/(:\/\/[^/]*\/)thumbnail\//, "$1wallpaper/");
		}

		if (domain_nosub === "mangadrawing.net") {
			return src.replace(/\/imagecache\/display\/image\//, "/image/");
		}

		if (domain_nosub === "ucoz.ru" ||
			domain_nowww === "intoclassics.net" ||
			domain_nowww === "zonemod.com" ||
			domain_nosub === "ucoz.net") {
			newsrc = src
				.replace(/(\/_ph\/+[0-9]+\/+)[0-9]+\/+([0-9]+\.[^/.]*)$/, "$1$2")
				.replace(/(\/_(?:nw|pu)\/+[0-9]+\/+)s([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
			if (newsrc !== src)
				return add_extensions_jpeg(newsrc);
		}

		if (domain_nowww === "wallpapersdig.com") {
			return src.replace(/\/images\/[a-z]+(?:\/.*)?(\/[^/]*-[0-9]+\.[^/.]*)$/, "/images/original$1");
		}

		if (domain === "img.hebus.com") {
			return src.replace(/\/[a-z]+\/[a-z]?([0-9]+_[0-9]+\.[^/.]*)$/, "/$1");
		}

		if (domain === "content.eroo.pl") {
			return src.replace(/(\/[0-9]+)_[a-z]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "timdir.com") {
			return src.replace(/\.tn(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "live-stats.me") {
			return src.replace(/\/pics\/+thumbs\//, "/pics/images/");
		}

		if (domain_nowww === "plekoflady.ga") {
			return src.replace(/\/media\/+thumbs\/+/, "/media/images/");
		}

		if (domain_nowww === "celebcafe.net") {
			newsrc = src.replace(/\/blog\/thumbs\//, "/blog/pics/");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "z-celebs.com" ||
			domain_nowww === "celebcafe.net" ||
			domain_nowww === "neocelebs.com" ||
			domain_nowww === "starsmaster.com" ||
			domain_nowww === "starzhunter.com" ||
			domain_nowww === "easycelebritys.com") {
			return src.replace(/\/pics\/+thumbs\/+/, "/pics/");
		}

		if (domain === "cdn.xpics.me" ||
			domain === "cdn.pornpic.xxx" ||
			domain === "cdn.pornpictures.today" ||
			domain === "cdn.xnxxpics.top" ||
			domain === "cdn.hdpornpics.net" ||
			domain === "cdn.pussy-porn-pics.com") {
			return src.replace(/(:\/\/[^/]*\/)[0-9]\//, "$10/");
		}

		if (domain_nowww === "nehuha.net" && string_indexof(src, "/data/photo/") >= 0) {
			return src.replace(/\/thumb\.([^/]*)$/, "/$1");
		}

		if (domain_nowww === "pornrice.com" ||
			domain_nowww === "monocdn.com" ||
			domain === "media.jpegworld.com" ||
			domain_nowww === "tokyoteenies.com" ||
			domain === "cdn.smutie.com" ||
			domain_nowww === "sexthread.com" ||
			domain_nowww === "sugarnips.com" ||
			domain_nowww === "galleryportal.com") {
			newsrc = src.replace(/(:\/\/[^/]*|\/media)\/+thumbs\/+((?:[0-9a-f]\/){5}[0-9a-f]+\/)[0-9]+x[0-9]+\//, "$1/galleries/$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "tokyoteenies.com") {
			return src.replace(/(\/content\/+[^/]*\/+)thumbs_([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1image_$2");
		}

		if (domain === "wallpapers.jami.lt") {
			return src.replace(/_big(\.[^/.]*)$/, "$1");
		}

		if (domain === "y.yarn.co") {
			if (/\/site\/+assets\/+images\/+black\./.test(src))
				return {
					url: src,
					bad: "mask"
				};

			obj = {
				url: src
			};

			match = src.match(/:\/\/[^/]*\/([-0-9a-f]+)(?:_.*?)(?:[?#].*)?$/);
			if (match) {
				obj.extra = {page: "https://yarn.co/yarn-clip/" + match[1]};
			}

			newsrc = src.replace(/(:\/\/[^/]*\/[-0-9a-f]+)_thumb(\.[^/.]*)$/, "$1_screenshot$2");
			if (newsrc !== src) {
				obj.url = newsrc;
				return obj;
			}

			newsrc = src.replace(/(:\/\/[^/]*\/[-0-9a-f]+)_[0-9]+_[wh]_[0-9]+\.gif(?:[?#].*)?$/, "$1.mp4");
			if (newsrc !== src) {
				obj.url = newsrc;
				obj.video = true;

				return obj;
			}
		}

		if (domain === "images.genius.com") {
			return src.replace(/\/avatars\/[a-z]+\/([0-9a-f]+)(?:[?#].*)?$/, "/avatars/original/$1");
		}

		if (domain_nosub === "smcloud.net" &&
			domain.match(/^cdn[0-9]*\.thumbs\.common\./)) {
			return src.replace(/:\/\/[^/]*\/common\/.\/.\/s\/([^/]*\.[^/.]*)\/.*/, "://static.common.smcloud.net/s/$1");
		}

		if (domain_nowww === "recordeli.com") {
			return src.replace(/(\/items\/[0-9]+\/image_)[a-z]+(?:[?#].*)?$/, "$1big");
		}

		if (domain_nosub === "liveinternet.ru" &&
			domain.match(/^img[0-9]*\./)) {
			return src.replace(/(\/[0-9]+)_[a-z]+(_+[0-9A-Za-z]+\.[^/.]*)$/, "$1$2");
		}

		if (domain === "assets.heart.co.uk" ||
			domain === "assets.popbuzz.com") {
			return src.replace(/(-[0-9]+)-(?:view|herowidev[0-9]*)-[0-9]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "kms.yawmiyati.com") {
			return src.replace(/\/Images\/[0-9io]+x[0-9io]+x[0-9io]+\//, "/content/uploads/Article/");
		}

		if (domain === "pic.chinasspp.com") {
			return src.replace(/_s_[0-9]+x[0-9]+(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "eveningecho.ie") {
			return src.replace(/(\/portalsuite\/image\/+[-0-9a-f]+)\/+[^/]*(\.[^/.]*)$/,
							   "$1/mainMediaSize=FILE__image$2");
		}

		if (domain_nowww === "drunkenstepfather.com") {
			return src.replace(/\/cms\/ul\/t-/, "/cms/ul/");
		}

		if (domain_nowww === "fraufluger.ru") {
			return src.replace(/(\/files\/images\/html_gallerys\/[^/]*\/[0-9a-f]+)_[0-9]+(\.[^/.]*)$/, "$1_origin$2");
		}

		if (domain_nowww === "alfacdn.com") {
			return src.replace(/(\/[0-9]+)h?(\.[^/.]*)/, "$1m$2");
		}

		if (domain === "s.img.mix.sina.com.cn") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/auto\/resize.*?[?&]img=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeURIComponent(newsrc);
		}

		if (domain === "albums.timg.co.il") {
			return src.replace(/(\/userFolders\/[0-9]+\/[0-9]+\/)[a-z]+\/([0-9]+\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "aljanh.net") {
			return src.replace(/(\/data\/archive\/)[a-z]+\/(img\/[0-9]+\.[^/.]*)$/, "$1$2");
		}

		if (amazon_container === "files.d20.io") {
			return src.replace(/(\/images\/[0-9]+\/[^/]*\/)[a-z]+(\.[^/.?]*)(?:[?#].*)?$/, "$1max$2");
		}

		if (domain_nowww === "wallpaperdx.com") {
			return src.replace(/\/photo(\/[^/]*)-[0-9]+-[0-9]+(\.[^/.]*)$/, "/images$1$2");
		}

		if (domain_nowww === "patrasevents.gr") {
			return src.replace(/(\/imgsrv\/+.\/+)[0-9]+x(?:[0-9]+)?\/+/, "$1full/");
		}

		if (domain_nowww === "bestfon.info") {
			return src.replace(/\/images\/+joomgallery\/+[a-z]+\//, "/images/joomgallery/originals/");
		}

		if (domain_nowww === "alazmenah.com" ||
			domain_nowww === "syria-in.com") {
			return src.replace(/\/thumb_photo\//, "/photo/");
		}

		if (domain === "images.alwatanvoice.com") {
			return src.replace(/\/news\/thumbs\//, "/news/large/");
		}

		if (domain_nowww === "ammanalyoum.com") {
			return {
				url: src.replace(/(\/images\/upload\/[^/]*\.[^/.]*)\/[0-9]+\/[0-9]+\/[0-9]+(?:[?#].*)?$/, "$1"),
				can_head: false // 403
			};
		}

		if (domain_nowww === "shitpostbot.com") {
			newsrc = src.replace(/(:\/\/[^/]*\/)resize\/[0-9]+\/[0-9]+.*?[?&]img=([^&]*).*?$/, "$1$2");
			if (newsrc !== src) {
				return decodeURIComponent(newsrc).replace(/(:\/\/[^/]*)\/+/, "$1/");
			}
		}

		if (domain_nowww === "extremnews.com") {
			return src.replace(/\/images\/[a-z_]+-([0-9a-f]+\.[^/.]*)$/, "/images/full-$1");
		}

		if (domain === "contestimg.wish.com") {
			return {
				url: src.replace(/(\/api\/webimage\/[0-9a-f]+(?:-[0-9]+)?)-[a-z]+(\.[^/.?]*)(?:[?#].*)?$/, "$1-original$2"),
				head_wrong_contenttype: true
			};
		}

		if (domain === "image.phimmoi.net") {
			return src.replace(/(\/profile\/[0-9]+\/)[a-z]+(\.[^/.]*)$/, "$1full$2");
		}

		if (domain_nosub === "static-bluray.com" &&
			domain.match(/^images[0-9]*\./)) {
			return src.replace(/(\/[0-9]+_[0-9]+_)[a-z]+(\.[^/.]*)$/, "$1original$2");
		}

		if (domain_nowww === "titus.kz" ||
			domain_nowww === "shymkent.kz") {
			return src.replace(/(\/load_theme\/files\/[0-9a-f]+)\.[^/]*(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "mfoxes.net") {
			return src
				.replace(/\/[a-z],[0-9]+,[^/]*\/gal\//, "/gal/")
				.replace(/\/castimagethumb-([0-9]+\.[^/.]*)$/, "/castimage-$1");
		}

		if (domain === "i.ucrazy.ru") {
			return src.replace(/(\/files\/i\/[0-9]+\.[0-9]+\.[0-9]+\/)thumbs\//, "$1");
		}

		if (domain_nowww === "athinorama.gr") {
			return src.replace(/(\/articles\/[0-9]+\/[^/.]*\.[^/.]*)\.ashx\?.*$/, "$1");
		}

		if (domain_nosub === "spiiky.com" && domain.match(/^img[0-9]*\./)) {
			return src.replace(/\.ashx\?.*/, "");
		}

		if (domain_nowww === "murekkephaber.com") {
			return src.replace(/(\/images\/[^/]*\/[0-9]+\/[0-9]+\/[^/]*)_t(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "egehaber.com") {
			return src.replace(/\/images\/+haberler\/+thumbs\/+/, "/images/haberler/");
		}

		if (domain_nowww === "dienvienvietnam.vn" &&
			string_indexof(src, "/assets/img/") >= 0) {
			return src.replace(/(_[0-9]+)_[a-z]+(\.[^/.]*)$/, "$1_grande$2");
		}

		if (amazon_container === "s3bucketvn") {
			return src.replace(/\/images\/thumbs\/[a-z]+\/([0-9a-f]+\.[^/.]*)$/, "/images/$1");
		}

		if (domain_nosub === "gazeta.ua" &&
			domain.match(/^static[0-9]*\./)) {
			return src.replace(/(\/img2?\/+)[a-z]+\/+([a-z]+\/+[0-9]+\/+[0-9]+(?:_[0-9]+)?)_[^/]*(\.[^/.]*)$/, "$1original/$2$3");
		}

		if (domain_nowww === "gazzettadiparma.it" ||
			domain === "img.liberoquotidiano.it" ||
			domain_nowww === "dundalkdemocrat.ie" ||
			domain_nowww === "donegaldemocrat.ie" ||
			domain_nowww === "liberoquotidiano.it") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/resizer_ext\/[-0-9]+\/[-0-9]+\/[a-z]+\/(http)/, "$1");
			if (newsrc !== src)
				return decodeURIComponent(newsrc).replace(/--(\.[^/.]*)$/, "");

			newsrc = src.replace(/\/resizer\/[-0-9]+\/[-0-9]+\/[a-z]+\/([0-9]+(?:_[0-9]+)?\.[^-/.]+)--.*/, "/upload/$1");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/\/resizer\/+[-0-9]+\/+[-0-9]+\/+[a-z]+\//, "/resizer/-1/-1/true/");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "multimidia.correiodopovo.com.br") {
			return src.replace(/(:\/\/[^/]*\/)thumb\.aspx.*?Caminho=([^&]*).*?$/, "$1$2");
		}

		if (domain_nosub === "diziler.com" &&
			domain.match(/^s[0-9]*\./)) {
			return src.replace(/(:\/\/[^/]*\/)img\/[0-9]+x[0-9]+\//, "$1original/");
		}

		if (domain === "g.denik.cz") {
			return src.replace(/_sip-[0-9]+up(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "imperiodefamosas.com") {
			return src.replace(/(:\/\/[^/]*\/)image_cache\.php.*?[?&]file=([^&]*).*?$/, "$1$2");
		}

		if (domain_nowww === "lostfilm.info") {
			return src.replace(/\/images\/+[0-9]+(photo|poster)/, "/images/$1");
		}

		if (domain_nowww === "hinhnenso1.com") {
			return src.replace(/(\/images\/blogs\/[0-9]+\/[0-9]+\/)[a-z]+\//, "$1original/");
		}

		if (domain_nowww === "avatarko.ru") {
			return src.replace(/(:\/\/[^/]*\/img\/)[a-z]+\//, "$1kartinka/");
		}

		if (domain_nowww === "ppe.pl") {
			return src.replace(/(\/upload\/news\/(?:[0-9]+\/){3})[a-z]+_([0-9]+\.[^/.]*)$/, "$1$2");
		}

		if (domain === "storge.pic2.me") {
			return src.replace(/\/cm?\/[0-9]+x[0-9]+\//, "/upload/");
		}

		if (domain_nowww === "serey.io") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/imageproxy\/[0-9]+x[0-9]+\/(https?:\/\/)/, "$1");
		}

		if (domain === "static.pjmedia.com") {
			return src.replace(/\.sized-[0-9t]+x[0-9t]+x[0-9t]+(\.[^/.]*)$/, "$1");
		}

		if (domain === "file.nxing.cn" ||
			domain === "upload.nxing.cn") {
			return {
				url: src.replace(/(\/uploads\/uploads\/[0-9]+\/[0-9]+\/[0-9]+\/[0-9]+\/[0-9a-f]+)-size[0-9]+x[0-9]+(?:_[^/]*)?(\.[^/.]*)$/, "$1$2"),
				headers: {
					Referer: ""
				}
			};
		}

		if (amazon_container === "kateryan" ||
			(domain_nosub === "lookbookspro.com" && domain.match(/assets\.lookbookspro\.com$/))) {
			return src.replace(/\/[a-z]+_/, "/gxxl_");
		}

		if (domain_nowww === "havepussy.com") {
			return src.replace(/\/uploads\/photos\/previews\/([0-9]+_[^/]*)$/, "/0-0/$1");
		}

		if (domain === "news.walkerplus.com") {
			return {
				url: src.replace(/(\/article\/[0-9]+\/[0-9]+)_[0-9]+(\.[^/.]*)$/, "$1$2"),
				can_head: false // 404
			};
		}

		if (domain === "movie.walkerplus.com") {
			newsrc = src.replace(/(\/api\/+resizeimage\/.*?)(?:[?#].*)?$/, "$1?w=");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/:\/\/[^/]+\/+api\/+resizeimage\/+news\/+(.*?)(?:[?#].*)?$/, "://news.walkerplus.com/$1.jpg");
			if (newsrc !== src)
				return add_extensions(newsrc);
		}

		if (domain === "img.jj20.com") {
			return src.replace(/(\/[0-9A-Z]+-[0-9]+)-[a-z]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "i.iplsc.com") {
			return src.replace(/(\/[0-9A-Z]+)-C[0-9]+(?:-F[0-9]+)?(\.[^/.]*)$/, "$1-C0$2");
		}

		if (domain === "s3.viva.pl") {
			return src.replace(/-GALLERY_[0-9]+(\.[^/.]*)$/, "-GALLERY_BIG$1");
		}

		if ((domain_nosub === "vogue.com.tr" && domain.match(/^cdn[0-9]*\./)) ||
			(domain_nosub === "gq.com.tr" && domain.match(/^cdn[0-9]*\./)) ||
			domain === "wr3mii5n.rocketcdn.com" ||
			domain === "voguecdn.blob.core.windows.net") {
			return src.replace(/\/files\/img\/[^/]*\//, "/files/original/");
		}

		if (domain_nosub === "clients-cdnnow.ru" ||
			domain === "s.properm.ru") {
			return src
				.replace(/(\/[a-z]+Storage\/(?:(?:post|news)\/)?(?:[0-9a-f]{2}\/){1,}[0-9a-f]+)_resizedScaled_[0-9]+to[0-9]+(\.[^/.]*)$/, "$1$2")
				.replace(/(:\/\/[^/]*\/)[a-z]+Storage\/+/, "$1originalStorage/");
		}

		if (domain_nowww === "hollywoodtuna.com") {
			return src.replace(/\/images([0-9])\/([^/]*)$/, "/images$1/bigimages$1/$2");
		}

		if (domain_nosub === "soupcdn.com" &&
			domain.match(/^asset-.\./)) {
			return src.replace(/(\/asset\/[0-9]+\/[0-9]+_[0-9a-z]+)_[0-9]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nosub === "qqmofasi.com" &&
			domain.match(/^pic[0-9]*\./)) {
			return src.replace(/_crop(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "download-rooz.com" ||
			domain_nowww === "day-download.com") {
			return src.replace(/\/img\/+[0-9]+-[0-9]+\//, "/img/");
		}

		if (domain_nowww === "know.cf") {
			return src.replace(/\/imgw\/[0-9]+px\/([^/]*)$/, "/imgw/$1");
		}

		if (amazon_container === "timely-api-public") {
			return src.replace(/(\/timely-api-public[^/]*\/[0-9]+_[0-9a-zA-Z]+)_[a-z]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "image-share.com") {
			return src.replace(/(\/upload\/[0-9]+\/[0-9]+)m(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "amando.it") {
			return src.replace(/\/imagesdyn\/gallery_plus\/(?:[0-9]+x[0-9]+|orig_[0-9]+)\//,
							   "/imagesdyn/gallery_plus/orig/");
		}

		if (domain_nowww === "textureking.com") {
			return add_extensions_upper(src.replace(/\/content\/img\/stock\/([^/]*)$/, "/content/img/stock/big/$1"));
		}

		if (domain_nowww === "textures.com") {
			if (src.match(/\/system\/+gallery\/+photos\/+[^/]*\/+(?:[^/]*\/+)?[0-9]+\/+[^/]+(?:[?#].*)?$/)) {
				return {
					url: src.replace(/(\/[0-9]+\/+)([^/]*_)(?:download)?[0-9]+(\.[^/.]*)$/, "$1hotlink-ok/$2shared$3"),
					problems: {
						watermark: true
					}
				};
			}
		}

		if ((domain_nosub === "gmbox.ru" ||
			 domain_nosub === "vestifinance.ru") &&
			domain.match(/^[a-z][0-9]*\./)) {
			return src.replace(/(\/c\/[0-9]+)\.[0-9]+x[0-9p]+(?:c[0-9]+)?(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nosub === "game2day.ru") {
			return src.replace(/\/images\/made\/[0-9a-f]+\/([0-9]+-[0-9]+)_[0-9]+_[0-9]+_(?:s_)?c[0-9]+(\.[^/.]*)$/,
							   "/uploads/userfiles/images/$1$2");
		}

		if (domain_nowww === "topcrochetpatterns.com") {
			return src.replace(/\/images\/+made\/+(images\/+uploads\/+blog\/+[^/]*?)_[0-9]+_[0-9]+_[0-9]+_c[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "/$1$2");
		}

		if (domain_nowww === "dangerousminds.net") {
			return src.replace(/\/content\/+uploads\/+images\/+made\/+(content\/+uploads\/+images\/+[^/]+)_[0-9]+_[0-9]+_int(\.[^/.]*)(?:[?#].*)?$/, "/$1$2");
		}

		if (domain_nosub === "nupics.pro" ||
			domain_nowww === "epicsoid.com" ||
			domain_nowww === "xwetpics.com" ||
			domain_nowww === "xsexpics.com" ||
			domain_nowww === "picsninja.club" ||
			domain_nowww === "hotnupics.com" ||
			domain_nowww === "ehotpics.com" ||
			domain_nowww === "picsegg.com" ||
			domain_nowww === "upicsz.com" ||
			domain_nowww === "xpicse.com" ||
			domain_nowww === "hdpicsx.com" ||
			domain_nowww === "picsninja.com" ||
			domain_nowww === "repicsx.com") {
			return src.replace(/(\/pics\/+[0-9]+\/+)_([^/]*)$/, "$1$2");
		}

		if (domain_nowww === "pornstarapex.com") {
			return src.replace(/(\/pics\/+[0-9a-f]+\/+)th_([0-9]+\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "cloud.it" &&
			domain.match(/^ldm[0-9]*\.r1-it\.storage\./)) {
			return src
				.replace(/(\/img\/.*\/)big\/([0-9]+_[0-9a-f]+\.[0-9]+\.[^/.]*)$/, "$1$2")
				.replace(/(\/(?:av|logo)\/.*\/)thumbs\/([0-9a-f]+\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "m2ch.hk") {
			return src.replace(/\/big\/thumb(\/[0-9]+\/[0-9]+)s(\.[^/.]*)$/, "/src/$1$2");
		}

		if (domain_nosub === "cfimg.com" &&
			domain.match(/^i[0-9]*\./)) {
			return src.replace(/(:\/\/[^/]*\/[0-9]+\/[0-9a-f]+)s(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "i.chzbgr.com") {
			return src.replace(/(:\/\/[^/]*\/+)(?:thumb[0-9]*|full)\/+/, "$1original/");
		}

		if (domain === "img.memecdn.com") {
			return add_full_extensions(src.replace(/(:\/\/[^/]*\/[^/]*_)[a-z]+(_[0-9]+\.[^/.]*)(?:[?#].*)?$/,
											  "$1o$2"));
		}

		if (domain === "p.memecdn.com") {
			return src.replace(/\/avatars\/+[a-z]+_([0-9]+_[0-9a-f]+\.[^/.]*)(?:[?#].*)?$/, "/avatars/$1");
		}

		if (domain_nosub === "123rf.com") {
			if (!src.match(/:\/\/[a-z]+cdn\./) &&
				src.match(/:\/\/[^/]*\/(?:images|[0-9]+wm)\//)) {
				var full = src.replace(/:\/\/[^/]*\/[^/]*\//, "://previews.123rf.com/images/");
				var small = src.replace(/:\/\/[^/]*\/[^/]*\//, "://us.123rf.com/450wm/");

				return [
					{
						url: full,
						problems: {
							watermark: true
						}
					},
					{
						url: small,
						problems: {
							smaller: true
						}
					}
				];
			}
		}

		if (domain_nowww === "chenderroad.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/image\.php.*?[?&]src=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeURIComponent(newsrc);
		}

		if (domain_nosub === "estranky.cz" ||
			domain_nosub === "estranky.sk") {
			if (domain !== "s3a.estranky.sk") {
				return src
					.replace(/(:\/\/[^/]*\/img\/)[a-z]+\//, "$1original/")
					.replace(/(:\/\/[^/]*\/img\/original\/[0-9]+)(\.[^/.]*)/, "$1/image$2");
			}
		}

		if (domain_nowww === "stagelook.ru") {
			return src.replace(/\/images\/tumbs\//, "/images/");
		}

		if (domain === "media.taaze.tw") {
			return src.replace(/\/showProdImageByPK\.html.*?[?&]pid=([0-9]+).*?$/, "/showTakeLook/$1.jpg");
		}

		if (domain === "st.cdjapan.co.jp") {
			return src.replace(/\/pictures\/s\//, "/pictures/l/");
		}

		if (domain === "image.blozoo.info") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/v2\/thumb\/[0-9]+\/[0-9]+\/.*?[?&]url=(http[^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeURIComponent(newsrc);
		}

		if (domain === "img.suilengea.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/img\.php.*?[?&]url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return add_http(decodeURIComponent(newsrc));
		}

		if (domain === "files.sirclocdn.xyz") {
			return src.replace(/(\/products\/_[0-9]+_[^/._]*)(?:_[a-z]+)?(\.[^/.]*)$/, "$1_ori$2");
		}

		if (domain === "s.tvp.pl") {
			return src.replace(/(\/images2?\/(?:[0-9a-f]\/){3})[-_a-zA-Z0-9]*(uid_[0-9a-f]+)[^/.]*?(\.[^/.]*)$/,"$1$2_width_9999999999999$3");
		}

		if (domain === "sf.be.com") {
			return src.replace(/(\/photo\/[0-9]+\/[0-9a-z]+\/[^/]*-)[a-z][0-9]+(\.[^/.]*)$/, "$1img$2");
		}

		if (domain_nowww === "fitwell.bg") {
			return src.replace(/(\/pictures\/[0-9]+)_[0-9]+_*(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "bilder.bild.de") {
			return src.replace(/\/fotos-skaliert(\/[^/]*\/)([0-9]+),[^/.]*(\.bild\.[^/.]*)$/, "/fotos$1Bild/$2$3");
		}

		if (domain === "galeria.cdn.divany.hu") {
			return src.replace(/(\/[0-9]+_[0-9a-f]+)_[a-z](\.[^/.]*)$/, "$1$2");
		}

		if (domain === "img.cncenter.cz") {
			return src.replace(/(\/img\/[0-9]+\/)[a-z]+(\/[^/]*)$/, "$1full$2");
		}

		if (domain_nowww === "cosmopolitan.de") {
			return src.replace(/\/bilder\/[0-9]+(?:x[0-9]+)?\//, "/assets/");
		}

		if (domain === "d1o73ibskzeirn.cloudfront.net") {
			return src.replace(/(:\/\/[^/]*\/)[0-9]+x[0-9]+\/content\//, "$10x0/content/");
		}

		if (domain_nowww === "elle.co.za" ||
			domain_nowww === "xgn.nl" ||
			domain === "cache.wizardworld.com") {
			return src.replace(/\/+_[0-9AUTO]+x[0-9AUTO]+_(?:crop|fit)_[^/]+\/+/, "/");
		}

		if (domain_nowww === "kartinnay-galerey.ru") {
			return src.replace(/\/uploads\/posts\/news_thumb\//, "/uploads/posts/");
		}

		if (domain_nowww === "imhomir.com") {
			return src.replace(/(\/uploads\/[a-z]+\/preview\/.*)_[0-9]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "mumsnet.com") {
			return src.replace(/(\/uploads\/talk\/[0-9]+\/)[a-z]+-([0-9]+-[^/]*)$/, "$1$2");
		}

		if (domain === "static.stylemagazin.hu") {
			return src.replace(/(\/medias\/[0-9]+\/)[0-9]+x[0-9]+\//, "$1");
		}

		if (domain === "img.thoibao.today") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/home\/fetch.*?[?&]u=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeURIComponent(newsrc);
		}

		if (amazon_container === "photo.elcinema.com") {
			return src.replace(/\/uploads\/_[0-9]+x(?:[0-9]+)?_([0-9a-f]+\.[^/.]*)$/, "/uploads/$1");
		}

		if (domain_nowww === "colors.life") {
			return src.replace(/(\/upload\/blogs\/[0-9a-f]{2}\/[0-9a-f]{2}\/[0-9a-f]+)_RSZ_[0-9]+(\.[^/.]*)$/, "$1$2");
		}


		if (domain_nosub === "tnews.ir" && domain.match(/^i[0-9]*\./)) {
			return src.replace(/(\/[0-9]+\/[0-9]+\/[0-9]+\/)Thumbnail\//, "$1");
		}

		if (domain === "multimedia.mmc.com.do") {
			return src.replace(/(\/multimedia\/[0-9]+\/[0-9]+\/[0-9]+\/[0-9a-f]+)_max.?_[0-9]+x[0-9]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "img.europapress.es") {
			return src.replace(/(\/fotoweb\/[a-z]+_(?:[0-9]+-)?[0-9]+)(?:_[0-9]+)?(\.[^/.]*)$/, "$1_9999$2");
		}

		if (domain === "img.estadao.com.br") {
			return src.replace(/\/thumbs\/[0-9]+\/resources\//, "/resources/");
		}

		if (domain_nosub === "asianews.cc" &&
			domain.match(/^s[0-9]*\./)) {
			return src.replace(/(\/[0-9]+\/[0-9a-f]+)_[a-z]+(\.[^/.]*)$/, "$1_orgn$2");
		}

		if (domain_nosub === "wallpapermix.club" ||
			domain_nosub === "wallpaper-planet.com") {
			return src.replace(/(\/doc\/wallpaper\/img\/)[a-z]\//, "$1l/");
		}

		if (domain_nosub === "utaten.com" && string_indexof(src, "/uploads/images/") >= 0) {
			return src
				.replace(/\/thumbnail\/+[0-9]+x[0-9]+\/+/, "/")
				.replace(/_(?:[a-z]|[0-9]+x[0-9]+)(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "frostsnow.com") {
			return add_extensions_with_jpeg(src
											.replace(/\/x([^/.]*\.[^/.]*)\.pagespeed\.[^/]*$/, "/$1")
											.replace(/\/[0-9]+x[0-9]+\/([^/]*)$/, "/$1"));
		}

		if (domain_nowww === "mewch.net") {
			return src.replace(/\/\.media\/t_([0-9a-f]+-imagejpeg)(?:[?#].*)?$/, "/.media/$1.jpg");
		}

		if (domain === "media.mehrnews.com") {
			return src.replace(/\/old\/[^/]*\/([0-9]+\/)/, "/old/Original/$1");
		}

		if (domain_nowww === "barking-moonbat.com") {
			return src.replace(/(\/images\/uploads\/[^/]*)_thumb(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "deseretnews.com") {
			return src.replace(/\/images\/article\/[a-z]+res\//, "/images/article/hires/");
		}

		if (domain_nowww === "orsm.net") {
			return src.replace(/(\/i\/galleries\/[^/]*\/)thumbnails\//, "$1");
		}

		if (domain_nowww === "niklife.com.ua") {
			return src.replace(/(:\/\/[^/]*\/)[0-9]+x[0-9]+[A-Z]?\/images\//, "$1images/");
		}

		if (domain === "img.ef43.com.cn") {
			return src.replace(/(\/newsImages\/[0-9]+\/[0-9]+\/[0-9]+)small(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "st.clopotel.t1.ro") {
			return src.replace(/(\/_files\/datafiles\/.*\/)thumbs\/([0-9]+\.[^/.]*)$/, "$1$2");
		}

		if (domain_nosub === "akamaized.net" &&
			domain.match(/^cdn[0-9]*-production-images-kly\./)) {
			return src.replace(/:\/\/[^/]*\/.*?\/kly-media-production\/([a-z]+\/[0-9]+\/)/, "://cdn-production-assets-kly.akamaized.net/$1");
		}

		if ((domain_nosub === "akamaized.net" && domain.match(/^cdn[0-9]*-production-assets-kly\./)) ||
			domain_nosub === "static6.com") {
			return src.replace(/(\/medias\/+[0-9]+\/+)[^/]*\/+/, "$1original/");
		}

		if (domain === "cdn.xn--cumpleaosdefamosos-t0b.com") {
			return src.replace(/(\/gallery\/[0-9]+\/[0-9]+\/[^/]*\/[^/]*)_thumb(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "up.zhuoku.org" ||
			domain === "up.padtu.com" ||
			domain === "up.deskbus.com" ||
			domain === "up.bizhitupian.com" ||
			domain === "up.8desk.com") {
			return src.replace(/\/pic_[0-9]+\//, "/pic/");
		}

		if (domain_nowww === "apherald.com" && string_indexof(src, "/ImageStore/images/") >= 0) {
			return src.replace(/\/tn-([^/]*)$/, "/$1");
		}

		if (domain_nowww === "luxo.co.za") {
			return src.replace(/\/system-files\/[a-z]+\//, "/system-files/");
		}

		if (domain_nowww === "longroom.com") {
			return src.replace(/\/uploads\/stock\/[a-z]+_([0-9]+\.[^/.]*)$/, "/uploads/stock/$1");
		}

		if ((domain_nosub === "cennoticias.com" ||
			 domain_nosub === "baca.co.id") &&
			domain.match(/^img\.cdn\./)) {
			return src.replace(/:\/\/img\.([^/]*)\/([-0-9a-f]+)(?:_thumbnail)?(?:[?#].*)?$/, "://raw.$1/$2");
		}

		if (domain === "cdn-tehran.wisgoon.com") {
			return src.replace(/\/dlir-s3\/([0-9]+)?[0-9]{3}x[0-9]{3}_([0-9]+\.[^/.]*)$/, "/dlir-s3/$1$2");
		}

		if (domain === "s.stylemode.com") {
			return src.replace(/(\/uploads\/(?:(?:article|blog|album|recommend)(?:_desc|detail)?)\/)(?:[0-9]+x[0-9]+|deal)\//, "$1origin/");
		}

		if (domain_nosub === "akairan.com" && domain.match(/^cdn[0-9]*\./) &&
			string_indexof(src, "/files/images/") >= 0) {
			return src.replace(/waterM(\.[^/.]*)$/, "$1");
		}

		if (domain === "cdn.idntimes.com" && string_indexof(src, "/content-images/") >= 0) {
			return src.replace(/_[0-9auto]+x[0-9auto]+(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "comicsblog.fr") {
			return src.replace(/\/images\/galerie\/[a-z]+image\/(?:small_)?/, "/images/galerie/bigimage/");
		}

		if (domain === "ent.sina.com.cn" ||
			domain === "auto.sina.com.cn") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/([^/]*\.sinaimg\.cn\/)/, "http://$1");
		}

		if (domain_nowww === "analisadaily.com") {
			return src.replace(/\/assets\/image\/news\/small\//, "/assets/image/news/big/");
		}

		if (domain === "fi.realself.com") {
			return src.replace(/(:\/\/[^/]*\/)[a-z]+(\/[0-9a-f]+\/)/, "$1full$2");
		}

		if (domain_nosub === "babeimpact.com" && domain.match(/^content[0-9]*\./)) {
			return src.replace(/_tn(_[0-9]+\.[^/.]*)$/, "$1");
		}

		if (domain === "i.widelec.org") {
			return src.replace(/_[a-z](\.[^/.]*)$/, "$1");
		}

		if (domain === "image.kurier.at") {
			return src.replace(/\/images\/[^/]*(\/[0-9]+\/)/, "/images/original$1");
		}

		if (domain === "media.game8.vn") {
			newsrc = src.replace(/\.[0-9]+\.(?:[0-9]+\.)?cache(?:[?#].*)?$/, "");
			if (newsrc !== src)
				return newsrc;

			if (string_indexof(src, "/srv_thumb.ashx") >= 0) {
				return urljoin("http://media.game8.vn/", url.searchParams.get("f"), true) + "." + url.searchParams.get("w");
			}
		}

		if (domain_nosub === "xiongyan.tv" &&
			domain.match(/^img[0-9]*\./)) {
			return src.replace(/\/[a-z]+_([0-9a-f]+[0-9A-Za-z]+\.[^/.]*)$/, "/origin_$1");
		}

		if (domain_nowww === "coffelt.me") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/image.*?[?&]q=([^&]*)$/, "$1");
			if (newsrc !== src)
				return decodeURIComponent(newsrc);
		}

		if (domain === "kiev.segodnya.ua") {
			return src.replace(/(\/img\/gallery\/[0-9]+\/[0-9]+\/[0-9]+)_[a-z]+(\.[^/.]*)$/, "$1_main$2");
		}

		if (domain === "image.jjang0u.service.concdn.com" ||
			domain === "pic.doctorteen.com" ||
			domain === "pic.freeporn.hu" ||
			(domain_nowww === "hotxxxgirls.org" && /\/images\/+galleries\/+/.test(src)) ||
			domain === "content.brazz-girls.com" ||
			(domain_nowww === "metartdb.com" && string_indexof(src, "/images/galleries/") >= 0)) {
			newsrc = src.replace(/\/t_([0-9]+\.[^/.]*)$/, "/$1");
			if (newsrc !== src) {
				return newsrc;
			}
		}

		if (domain === "image.jjang0u.service.concdn.com") {
			return {
				url: src,
				headers: {
					Referer: "http://fun.jjang0u.com/"
				}
			};
		}

		if (domain_nowww === "fotoshoots.be") {
			return src.replace(/(\/images\/fotografen\/[0-9]+\/)thumbs\//, "$1");
		}

		if (domain_nosub === "tapatalk.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/groups\/[^/]*\/imageproxy\.php.*?[?&]url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeURIComponent(newsrc);
		}

		if (domain === "dl.backbook.me") {
			return src.replace(/(:\/\/[^/]*\/)[a-z_]+(\/[0-9a-f]+\.[^/.]*)$/, "$1full$2")

		}

		if (domain === "d.backbook.me") {
			return src
				.replace(/(\/file\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9]+\/+)[a-z_]+_([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/,
						 "$1full_$2");
		}

		if (domain === "nthumb.cyworld.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/thumb.*?[?&]url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeURIComponent(newsrc);
		}

		if (domain === "e.porosenka.net") {
			return src.replace(/(\/uploads\/[0-9a-f]\/[0-9a-f]+\/[0-9a-f]+)_[a-z]+(\.[^/.]*)$/, "$1$2");
		}

		if ((domain_nosub === "qhimg.com" ||
			 domain_nosub === "qhmsg.com") &&
			domain.match(/^p[0-9]*\./)) {
			return src.replace(/(:\/\/[^/]*\/+)[a-z]+\/+[0-9]+_[0-9]+_\/+/, "$1");
		}

		if (domain_nosub === "gamersky.com" && domain.match(/^img[0-9]*\./)) {
			return {
				url: src
					.replace(/(\/upimg\/pic\/[0-9]{4}\/[0-9]{2}\/[0-9]{2}\/[0-9]+)_[a-z]+(\.[^/.]*)$/, "$1$2")
					.replace(/(\/upimg\/users\/[0-9]{4}\/[0-9]{2}\/[0-9]{2}\/)[a-z]+_([^/]*)$/, "$1origin_$2")
					.replace(/(\/image[0-9]{4}\/+[0-9]{2}\/+[0-9]{8}_[a-z]{2}_[0-9]+_[0-9]+\/+[^/_.]*)_[A-Z](\.[^/.]*)(?:[?#].*)?$/, "$1$2"),
				headers: {
					Referer: ""
				},
				referer_ok: {
					same_domain_nosub: true
				}
			};
		}

		if (domain_nosub === "game234.com" && domain.match(/^webimg[0-9]*\./) ||
			domain_nowww === "reho.st") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+((?:[a-z]+:\/\/)?[^/]+\.[^/.]+\/+)/, "$1");
			if (newsrc !== src)
				return add_http(newsrc);
		}

		if (domain === "img.izismile.com" ||
			domain_nowww === "izismile.com") {
			return src
				.replace(/(\/img[0-9]*\/+[0-9]{8}\/+)640\/+([^/]*_)640_(?:high_)?([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$11000/$2$3")
				.replace(/(\/img[0-9]*\/+[0-9]{8}\/+[^/]*)_l(_[0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "cdn.snsimg.carview.co.jp") {
			return src.replace(/(\/blog\/(?:[0-9]+\/){5}[^/]*)[sm](\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "wallpapersontheweb.net") {
			regex = /\/wallpapers\/[a-z]\/(?:(?:[0-9]+x[0-9]+)?\/)?(?:(?:[a-z]|[0-9]+x[0-9]+)-)?([^/]*)-([0-9]+)(\.[^/.]*)$/;
			return {
				url: src.replace(regex, "/wallpapers/l/$1-$2$3"),
				headers: {
					Referer: src.replace(regex, "/$2-$1/")
				}
			};
		}

		if (domain_nosub === "funon.cc" && domain.match(/^s[0-9]*\./)) {
			return src.replace(/(\/img\/)[a-z]+(\/[0-9]+\/)/, "$1orig$2");
		}

		if (domain === "img.joemonster.org") {
			return src.replace(/(\/upload\/[^/]*\/)[a-z]_([0-9a-f]+[0-9a-z]+\.[^/.]*)$/, "$1$2");
		}

		if (domain === "mobimg.b-cdn.net") {
			return src.replace(/\/pic\/v2\/gallery\/[^/]*\//, "/pic/v2/gallery/real/");
		}

		if (domain === "img.wanduorou.com") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/imgs\/([^/.]*\.[^/]*\/)/, "http://$1");
		}

		if (domain === "i.gbc.tw") {
			return src.replace(/(\/gb_img\/[0-9]+)[a-z](\.[^/.]*)$/, "$1$2");
		}

		if ((domain_nosub === "xuehuaimg.com" && domain.match(/^pic[0-9]*\./)) ||
			(domain_nosub === "11street.my" && domain.match(/^cdn[0-9]*\./))) {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/proxy\/+(?:[a-z]+\/+)?([a-z]+:\/\/)/, "$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "img.tgchengzi.com") {
			return src.replace(/(\/Uploads\/ueditor\/php\/upload\/image\/[0-9]+\/[0-9]+)_[a-z](\.[^/.]*)$/, "$1$2");
		}

		if (domain_nosub === "doyo.cn" &&
			domain.match(/^s[0-9]*\./)) {
			return src.replace(/(\/img\/(?:[0-9a-f]{2}\/){2}[0-9a-f]+\.[^/._]*)_[a-z]+(?:[?#].*)?$/, "$1");
		}

		if (domain === "newsimg.hankookilbo.com" ||
			domain === "image.hankookilbo.com" ||
			domain === "betaimage.hankookilbo.com") {
			var extra = {};
			match = src.match(/^[a-z]+:\/\/[^/]*\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}([0-9]{8,})_[0-9]+(?:_[a-z])?\.[^/.]*(?:[?#].*)?$/);
			if (match) {
				extra.page = "http://star.hankookilbo.com/News/Read/" + match[1];
			}

			return {
				url: src.replace(/(\/[0-9]{4}\/[0-9]{2}\/[0-9]{2}\/(?:[0-9]+_[0-9]+|[-0-9a-f]+))_[a-z](\.[^/.]*)$/, "$1$2"),
				extra: extra
			};
		}

		if (domain === "dispatch.cdnser.be") {
			return src.replace(/(\/[0-9]+[^-/_.]*_)T[0-9]*(_[0-9]+\.[^/.]*)$/, "$1T5$2");
		}

		if (domain === "cms.artandarchaeology.princeton.edu") {
			return src.replace(/\/media\/thumbnails\/([^/?]*)\?.*$/, "/media/files/$1");
		}

		if (domain_nowww === "pronto.com.ar") {
			return src.replace(/\/asset\/thumbnail[,%][^/]*\/media\//, "/media/");
		}

		if (domain === "cdn.thebest.gr") {
			return src.replace(/\/media\/images\/[^/]*\//, "/media/images/original/");
		}

		if (domain_nowww === "glow.gr" ||
			domain === "omegalive-sf.cdn.edgeport.net" ||
			domain_nowww === "omegalive.com.cy") {
			return src.replace(/(:\/\/[^/]*\/)(?:image|[a-z]+-img)\/+[^/]+(\/+[0-9]+\/+[^/]*\.[^/.?#]*)(?:[?#].*)?$/,
							   "$1image/original$2");
		}

		if (domain_nowww === "scifi-forum.de" ||
			domain_nowww === "phica.net") {
			return src.replace(/\/filedata\/fetch.*?[?&](id=[0-9]+).*?$/, "/filedata/fetch?$1");
		}

		if (domain === "cache.net-a-porter.com") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/content\/images\/(story-[^/]*\.[^/.]*)\/[^/]*$/, "https://assets.ynap-content.com/$1");
		}

		if (domain_nowww === "celluloidportraits.com" && string_indexof(src, "/img/") >= 0) {
			return src.replace(/(_[0-9]+_)[A-Z](\.[^/.]*)$/, "$1L$2");
		}

		if (domain_nowww === "nowtoronto.com") {
			return src.replace(/(\/downloads\/[0-9]+\/download\/[^/?#]*)(?:.*?[?&](cb=[0-9a-f]+).*?)?$/, "$1?$2");
		}

		if (domain === "cde.peru.com") {
			return src.replace(/(\/ima\/(?:[0-9]\/){5}(?:[0-9]+\/)?)(?:[0-9]+x[0-9]+|thumb)\/([^/]*)$/, "$1$2");
		}

		if (domain === "cd.cinescape.com.pe") {
			match = src.match(/^[a-z]+:\/\/[^/]*\/cinescape-[0-9]+x[0-9]+-([0-9]+)(\.[^/.]*)(?:[?#].*)?$/);
			if (match) {
				var digits = match[1].replace(/[0-9]{3}$/, "");
				var length = digits.length;
				for (i = length; i < 5; i++) {
					digits = "0" + digits;
				}
				return "http://e.cinescape.americadigital.pe/ima/"
					+ digits[0] + "/" + digits[1] + "/" + digits[2] + "/" + digits[3] + "/" + digits[4] + "/" + match[1] + match[2];
			}
		}

		if (domain_nowww === "celebritytalent.net") {
			return src.replace(/\/photos\/[a-z]+\/([0-9]+\.[^/.]*)$/, "/photos/lg/$1");
		}

		if (domain === "img-ovh-cloud.zszywka.pl") {
			return src.replace(/\/thb_([0-9]+\.[^/.]*)$/, "/$1");
		}

		if (domain_nosub === "clickthecity.com" && domain.match(/^cdn[0-9]*\./)) {
			return src.replace(/\/profiles\/[0-9]+(\/[0-9]+(?:_[0-9]+)?\.[^/.]*)$/, "/profiles$1");
		}

		if (domain_nowww === "lafactoriadelshow.com") {
			return src.replace(/\/[a-z]+_([0-9a-f]+_[0-9]+\.[^/.]*)$/, "/full_$1");
		}

		if (domain_nowww === "elintra.com.ar" ||
			domain_nowww === "diariorepublica.com.ar" ||
			domain_nosub === "minutoneuquen.com" ||
			domain_nowww === "abcdiario.com.ar" ||
			domain_nowww === "gamesbras.com" ||
			domain_nowww === "paginasiete.bo" ||
			domain_nowww === "diariolaprovinciasj.com" ||
			domain_nowww === "elintransigente.com" ||
			/^[a-z]+:\/\/[^/]+\/+u\/+fotografias\/+m\/[0-9]{4}\/+[0-9]{1,2}\/+[0-9]{1,2}\/+f[0-9]+x[0-9]+-[0-9]+_/.test(src)) {
			return src.replace(/\/fotografias\/m\/([0-9]{4}\/[0-9]+\/[0-9]+\/)f[0-9]+x[0-9]+-([0-9]+)_[0-9]+[^/]*(\.[^/.]*)$/,
							   "/fotografias/fotosnoticias/$1$2$3");
		}

		if (domain === "f.aukro.cz") {
			return src.replace(/(\/images\/[^/]*\/)[0-9]+x[0-9]+\/([-0-9a-f]+)(?:[?#].*)?$/, "$1$2");
		}


		if (domain === "envivoblog.estrellatv.com" ||
			domain_nowww === "gamegrin.com" ||
			domain === "cdn.okjeok.hr") {
			return src.replace(/\/_resampled\/(?:Set(?:Width|Height)|resizedimage)[0-9]+-([^/]*)$/, "/$1");
		}

		if (domain === "images.sex.com") {
			return {
				url: src.replace(/(\/images\/pinporn\/[0-9]{4}\/[0-9]{2}\/[0-9]{2}\/)(?:300|460|126x126)\//, "$1620/"),
				headers: {
					Referer: "https://www.sex.com/" // no referer doesn't work
				},
				referer_ok: {
					same_domain_nosub: true
				}
			};
		}

		if (domain === "fotografias.antena3.com" ||
			domain === "image.europafm.com" ||
			src.match(/^[a-z]+:\/\/[^/]*\/clipping\/cmsimages[0-9]*\/[0-9]{4}\/[0-9]{2}\/[0-9]{2}\/[-0-9A-F]+\/[^/]*\.[^/.]*$/)) {
			newsrc = src.replace(/(\/clipping\/cmsimages[0-9]*\/[0-9]{4}\/[0-9]{2}\/[0-9]{2}\/[-0-9A-F]+\/)[^/]*(\.[^/.]*)$/,
								 "$1default$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nosub === "nickiswift.com" && domain.match(/^img[0-9]*\./)) {
			return src.replace(/-[0-9]+x[0-9]+(?:_rev[0-9]*)?(\.[^/.]*)$/, "$1");
		}


		if (amazon_container === "sphm-female-site-production") {
			return src.replace(/-[0-9]+px(\.[^/.]*)$/, "$1");
		}

		if (domain_nowww === "maxpapendieck.com") {
			return src.replace(/(\/app\/webroot\/upload\/[0-9]+\/[^/]*)-(?:large|medium|thumb)(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "i.mdel.net") {
			return src.replace(/(\/i\/db\/[0-9]+\/[0-9]+\/[0-9]+\/[0-9]+)-[0-9]+[a-z]n?(\.[^/.]*)$/, "$1-orig$2");
		}

		if (domain === "cdn-img.fimfiction.net") {
			return src
				.replace(/(\/story\/[^-/]*-[0-9]+-[0-9]+-)[a-z]+([?#].*)?$/, "$1full$2")
				.replace(/(\/user\/[^-/]*-[0-9]+-[0-9]+-)[0-9]+([?#].*)?$/, "$1512$2");
		}

		if (domain_nosub === "tv-happening.com" && domain.match(/^img(?:-[a-z]+)?[0-9]*\./)) {
			return src.replace(/\/[a-z]\/([0-9]+\.[^/.]*)$/, "/b/$1");
		}

		if (domain_nowww === "kbbs.jp" ||
			domain_nowww === "ibbs.info") {
			return src.replace(/\/fitimg\/cache\/([^/]*)\/[0-9]+\/[^/_.]*_([^/]*)(?:[?#].*)?$/, "/data/$1/img/$2");
		}

		if (domain_nowww === "pzy.be") {
			return src.replace(/(:\/\/[^/]*\/)t(\/[0-9]+\/[^/]*)$/, "$1i$2");
		}

		if ((domain_nosub === "blick.ch" && domain.match(/^f[0-9]*\./)) ||
			domain === "static.pulse.ng" ||
			domain === "static.pulselive.co.ke" ||
			(domain_nosub === "focus.de" && domain.match(/^p[0-9]*\./)) ||
			domain === "static.pulse.com.gh") {
			return src.replace(/\/img\/[a-z]+\/(?:origs|crop)([0-9]*)\/[0-9]+(?:-[^/]*)?\/([^/?#]*\.[^/.?#]*)(?:[?#].*)?$/,
							   "/media/$1/$2");
		}

		if (domain === "img.blick.ch") {
			return src.replace(/\?.*/, "?ratio=FREE&x=0&y=0");
		}

		if (domain_nowww === "mixnews.lv") {
			return src.replace(/(\/uploads\/media\/image\/[0-9]{4}\/[0-9]{2}\/[0-9]{2}\/[^/]*)_medium\.png(?:[?#].*)?$/,
							   "$1.jpg");
		}

		if (domain_nosub === "promiflash.de" &&
			domain.match(/^content[0-9]*\./)) {
			return src.replace(/\/article-images\/+gallery1024\//, "/article-images/gallery2048/");
		}

		if (domain_nowww === "mycharm.ru") {
			return src.replace(/(\/data\/cache\/[0-9]{4}[a-z]+\/.*)thumb[0-9]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "images.worthpoint.com") {
			return src.replace(/(\/files\/[^/]*\/)tn\//, "$1");
		}

		if (domain_nowww === "benishop.co" ||
			domain === "cdn.benishop.co" ||
			domain_nowww === "awanshop.co") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/imgcdn\/[0-9]+\/([a-z]+)\/(.*)_\.[^/.]*$/, "$1://$2");
		}

		if (domain === "img.auctiva.com") {
			return src.replace(/(\/[0-9]+_)[a-z]+(\.[^/.]*)$/, "$1o$2");
		}

		if (domain === "img.networthpost.com") {
			return src.replace(/\/thumbs\/([0-9]+_)/, "/images/$1");
		}

		if (domain === "img.static.butygirls.com") {
			return src.replace(/\/resize\/[0-9]+\/[0-9]+\/[0-9]+(\/[0-9a-f]+\.[^/.]*)$/, "$1");
		}

		if (domain === "coubsecure-s.akamaihd.net") {
			return src.replace(/\/[a-z]+_([0-9]+_image\.[^/.]*)$/, "/$1");
		}

		if (domain_nowww === "wallpapersmug.com" ||
			domain_nowww === "picstatio.com") {
			return src.replace(/\/(?:download\/+[0-9]+x[0-9]+|(?:thumb|large))\/+([-0-9a-z]+\/+[^/]*)(?:[?#].*)?$/, "/u/$1");
		}

		if (domain === "cp12.nevsepic.com.ua") {
			return src.replace(/(\/[0-9]+\/)thumbs\/([0-9]+-[0-9]+-[^/]*)$/, "$1$2");
		}

		if (domain === "itn.dmarge.com") {
			return src.replace(/:\/\/[^/]*\/[0-9]+x[0-9]+\//, "://i.dmarge.com/");
		}

		if (domain_nosub === "idnes.cz" && domain.match(/^img[0-9]*\./)) {
			return src.replace(/\/thumb\//, "/images/");
		}

		if (domain_nowww === "funsubstance.com") {
			return src.replace(/\/uploads\/[a-z]+\//, "/uploads/original/");
		}

		if (domain === "global.unitednations.entermediadb.net") {
			return src.replace(/\/image[0-9]+x[0-9]+(?:cropped)?(\.[^/.]*)$/, "/image$1");
		}

		if (domain === "rimg.bookwalker.jp") {
			return src.replace(/:\/\/[^/]*\/([0-9]+)\/[0-9a-zA-Z_]+(\.[^/.]*)$/, "://c.bookwalker.jp/$1/t_700x780$2");
		}

		if (domain === "c.bookwalker.jp") {
			match = src.match(/:\/\/[^/]*\/([0-9]+)\/[^/]*(?:[?#].*)?$/);
			if (match) {
				var number = match[1];
				var reversed_number = parseInt(reverse_str(number)) - 1;
				return "https://c.bookwalker.jp/coverImage_" + reversed_number + src.replace(/.*(\.[^/.?#]*)(?:[?#].*)?$/, "$1");
			}
		}

		if (domain === "images.cdn.circlesix.co") {
			return src.replace(/\/image\/(?:[0-9]+\/){3}uploads\//, "/image/uploads/");
		}

		if (domain === "static.japanhdv.com") {
			return src.replace(/(:\/\/[^/]*\/)cache\/(.*)\.[0-9]+x[0-9]+(\.[^/.]*)$/, "$1$2$3");
		}

		if (domain === "mosaic.tnaflix.com") {
			return src.replace(/(\/[0-9a-z]+):[0-9a-z]+(\/|\.[^/.]*)/, "$1$2");
		}

		if (domain_nosub === "tnastatic.com" && domain.match(/^img[0-9]*\./)) {
			return src.replace(/(:\/\/[^/]*\/)[^/]*\/(pics|thumbs)\//, "$1q100/$2/");
		}

		if (domain === "image.gala.de") {
			return src
				.replace(/(:\/\/[^/]*\/v[0-9]*\/cms\/[^/]*\/[^/]*_[0-9]+-)[a-z]+[-_][^/]*(\.[^/.]*)$/, "$1original-lightbox$2")
				.replace(/:\/\/[^/]*\/+([0-9]+)\/+(?:[a-z]+)?(?:uncropped|[0-9]+x[0-9]+)-[0-9]+-[0-9]+\/([0-9a-f]{20,})\/+[^/]*\/+([^/]*)(\.[^/.]*)(?:[?#].*)?$/,
						 "://www.gala.de/resource/blob/$1/$2/$3-data$4");
		}

		if (domain_nosub === "fashionwelike.com" &&
			domain.match(/^static[0-9]*\./)) {
			return src.replace(/\/photos\/thumbnail\//, "/photos/original/");
		}

		if (domain === "images.metmuseum.org") {
			return src.replace(/(\/CRDImages\/+[^/]*\/+)[^/]*\/+/, "$1original/");
		}

		if (domain === "images-assets.nasa.gov") {
			return src.replace(/~[a-z]+(\.[^/.]*)$/, "~orig$1");
		}

		if (domain === "www.jpl.nasa.gov") {
			return src.replace(/:\/\/[^/]+\/+spaceimages\/+images\/+[^/]+\/+(PIA[0-9]+)(?:_[^/]+|-[0-9]+(?:x[0-9]+|[wh]))(\.(?:jpg|jpeg|JPG|JPEG))(?:[?#].*)?$/,
				               "://photojournal.jpl.nasa.gov/jpeg/$1$2");
		}

		if (domain === "imagecache.jpl.nasa.gov") {
			match = src.match(/:\/\/[^/]+\/+images\/+[0-9]+x[0-9]+\/+(?:pia|PIA)([0-9]+)-[0-9]+-[0-9]+x[0-9]+\./);
			if (match) {
				return "https://photojournal.jpl.nasa.gov/jpeg/PIA" + match[1] + ".jpg";
			}
		}

		if (domain === "solarsystem.nasa.gov") {
			return src.replace(/:\/\/[^/]+\/+system\/+(?:resources|news_items)\/+(?:detail_files|(?:list|main)_images)\/+[0-9]+_(?:pia|PIA)([0-9]+)(?:_[a-z]+)?(?:\.[^/.]+_[^/]+|[-_][^/]+)?\.[^/.]+(?:[?#].*)?$/,
								"://photojournal.jpl.nasa.gov/jpeg/PIA$1.jpg");
		}

		if (domain === "photojournal.jpl.nasa.gov") {
			obj = {url: src};
			id = src.replace(/.*\/(PIA[0-9]+)(?:_[^/]+)?\.[^/.]+(?:[?#].*)?$/, "$1");

			if (id !== src) {
				obj.extra = {
					page: "https://www.jpl.nasa.gov/spaceimages/details.php?id=" + id
				};
			}

			newsrc = src.replace(/(:\/\/[^/]+\/+)jpegMod\/+([^/]+)_modest\./, "$1jpeg/$2.");
			if (newsrc !== src) {
				obj.url = newsrc;
				return obj;
			}

			newsrc = src.replace(/(:\/\/[^/]+\/+)(?:browse|thumb)\/+/, "$1jpeg/");
			if (newsrc !== src) {
				obj.url = newsrc;
				return obj;
			}

			return obj;
		}

		if (domain === "nssdc.gsfc.nasa.gov") {
			obj = {url: src};

			id = src.match(/\/imgcat\/+[^/]+res\/+(vg[0-9]*_p[0-9]+)\./);
			if (id) {
				obj.extra = {page: "https://nssdc.gsfc.nasa.gov/imgcat/html/object_page/" + id[1] + ".html"};
			}

			newsrc = src.replace(/(\/imgcat\/+)midres\/+/, "$1hires/");
			if (newsrc !== src) {
				obj.url = newsrc;
				return obj;
			}

			return obj;
		}

		if (amazon_container === "attachments.readmedia.com") {
			return src.replace(/(\/+files\/+[0-9]+\/+)[a-z]+\/+([^/]*)(?:[?#].*)?$/, "$1original/$2");
		}

		if (domain === "files.sexyandfunny.com") {
			return src.replace(/\/(?:img|gallery)_[a-z]+\//, "/img_orig/");
		}

		if (domain === "mb.cision.com") {
			return src.replace(/(\/Public\/+[0-9]+\/+[0-9]+\/+[0-9a-f]+)_[^/_]*(\.[^/.]*)$/, "$1_org$2");
		}

		if (domain_nowww === "ftvmagic.com") {
			return src
				.replace(/(\/+grand-media\/+image\/+)thumb\//, "$1")
				.replace(/-[0-9]+x[0-9]+(\.[^/.]*)$/, "$1")
				.replace(/(\/post_pics\/+[0-9]+\/+[0-9]+\/+[0-9]+)_thumb(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "wdb.space" ||
			domain_nowww === "onlythere.com") {
			return src.replace(/(\/media\/[^/]*\/[^/]*)_scale_[0-9]+x[0-9]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "iphone8wallpapers.com" && string_indexof(src, "/media/uploads/") >= 0) {
			return {
				url: src.replace(/-[0-9]+x[0-9]+(\.[^/.]*)$/, "$1"),
				headers: {
					Referer: src.replace(/(:\/\/[^/]*\/).*/, "$1") // works without, but adds a "click here full resolution, no hotlinking" watermark
				}
			};
		}

		if (domain === "upload.sanqin.com" ||
			domain === "upload.mnw.cn") {
			return src.replace(/\/thumb_[0-9]+__([^/]*)(?:[?#]*.*)?$/, "/$1");
		}

		if (domain_nosub === "best-wallpaper.net" && string_indexof(src, "/wallpaper/") >= 0 &&
			options && options.do_request && options.cb) {
			match = src.match(/\/wallpaper\/+[^/]+\/[0-9]+\/([^/]*)_[^-_/.]+\.[^/.]*$/);
			if (match) {
				options.do_request({
					url: "https://best-wallpaper.net/" + match[1] + "_wallpapers.html",
					method: "GET",
					onload: function(resp) {
						if (resp.readyState === 4) {
							if (resp.status !== 200) {
								console_log(result);
								options.cb(null);
								return;
							}

							var match = resp.responseText.match(/<img ID="viewImg"[^>]*data-src="([^">]*)"/);
							if (match) {
								options.cb(urljoin(src, match[1], true));
							} else {
								options.cb(null);
							}
						}
					}
				});

				return {
					waiting: true
				};
			}
		}

		if (domain_nosub === "trueart.com" &&
			domain.match(/^s[0-9]*\./)) {
			return src.replace(/(\/[0-9]+)_[0-9]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "atkfan.com") {
			return src.replace(/\/thumbs(\/[^/]*)_[0-9]+(\.[^/.]*)$/, "/set$1$2");
		}

		if (domain === "imbbsfile.imbc.com") {
			return src.replace(/\/thnb_([^/]*\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nowww === "hdwall.us") {
			return {
				url: src.replace(/(:\/\/[^/]*\/+)(?:wallpaper|thumbnail)[^/]*\/+([^/]*\.[^/.]*)(?:[?#].*)?$/, "$1wallpaper/$2"),
				problems: {
					watermark: true
				}
			};
		}

		if (domain === "thmbs.baklol.com") {
			return src.replace(/:\/\/thmbs\./, "://images.");
		}

		if (domain_nowww === "svtstatic.se") {
			return {
				url: src.replace(/\/image\/+[^/]*\/+[^/]*\/+([0-9]+\/+[0-9]+)(?:\.[^/?#]*)?(?:[?#].*)?$/,
								 "/image/original/unscaled/$1.png"),
				head_wrong_contentlength: true
			};
		}

		if (domain === "thmb.inkfrog.com" ||
			amazon_container === "thmb.inkfrog.com") {
			return src
				.replace(/^[a-z]+:\/\/.*\/+thumb[^/]*\/+([^/]*\/+[^/]*\.[^/.=]*)(?:=[0-9]+)?(?:[?#].*)?$/,
						 "https://imgs.inkfrog.com/pix/$1")
				.replace(/^[a-z]+:\/\/.*\/+pix\/+([^/]*\/+[^/]*\.[^/.]*)\/+[0-9]+\/+[0-9]+(?:[?#].*)?$/,
						 "https://imgs.inkfrog.com/pix/$1");
		}

		if (domain_nosub === "xnostars.com" &&
			domain.match(/^img[0-9]*\./)) {
			return src.replace(/(\/fotos\/+[^/]*)\/+thumbs\/+/, "$1/");
		}

		if (domain_nowww === "mobilmusic.ru") {
			return src.replace(/(\/mfile\/+[0-9a-f]{2}\/+[0-9a-f]{2}\/+[0-9a-f]{2}\/+[0-9]+)-[0-9]+(\.[^/.]*)$/,
							   "$1$2");
		}

		if (domain === "pics.definebabe.com") {
			return src.replace(/\/thumbs\/+thumb_([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nosub === "definebabe.com" &&
			domain.match(/^cdn-i[0-9]+\./)) {
			return src.replace(/(\/+[0-9a-f]+\/+)[a-z]?[0-9]+\/+((?:[^/]*-)?[0-9]+\.[^/.]*)$/, "$1$2");
		}

		if (amazon_container === "everipedia-storage") {
			return add_extensions_upper(src.replace(/(\/NewlinkFiles\/+[0-9]+\/+(?:[^/]*\/+)?[^/]+)_[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2"));
		}

		if (domain_nowww === "w-dog.net") {
			return src.replace(/(\/wallpapers\/+[0-9]+\/+[0-9]+\/+)[a-z]+(\/[0-9]+\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nosub === "wsimg.com" &&
			domain.match(/^img[0-9]*\.wsimg\.com/)) {
			return src.replace(/(\/+isteam\/+ip\/+[-0-9a-f]+\/+[^/]*\.[^/.]*)(?:\/+:\/+.*)?(?:[?#].*)?$/, "$1");
		}

		if (domain === "img.lovpho.com") {
			return src.replace(/\/+anh\/+(?:width|height)[0-9]+\//, "/anh/");
		}

		if (domain_nowww === "divnil.com") {
			return src.replace(/(\/wallpaper\/.*_[0-9a-f]+_)[a-z]+(\.[^/.]*)$/, "$1raw$2");
		}

		if (domain === "images.dbnaked.com") {
			return src.replace(/\/thumb_(?:[0-9]+x[0-9]+_)?([0-9]+\.[^/.]*)$/, "/$1");
		}

		if (domain === "img.iseephoto.com") {
			return src.replace(/(\/+files\/+[^/]*\/+)thumes\/+/, "$1");
		}

		if (domain === "media.elle.gr") {

			return src.replace(/.*\/engine\/[^/]*?_([0-9]+)_([0-9]+)(?:_type[0-9]+)?\.[^/.]*(?:[?#].*)?$/,
							   "http://engine.numatek.netuse.gr/?imgid=$2&srcid=$1&type=2");
		}

		if (domain === "engine.numatek.netuse.gr") {
			var imgid = url.searchParams.get("imgid");
			var srcid = url.searchParams.get("srcid");
			if (imgid && srcid) {
				return "http://engine.numatek.netuse.gr/?imgid=" + imgid + "&srcid=" + srcid + "&type=2";
			}
		}

		if (domain === "actualite.benchmark.fr") {
			return src.replace(/_diaporama_[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "showstudio.com") {
			return src.replace(/(\/+img\/+images\/+[0-9]+-[0-9]+\/+[0-9]+)_[^/]*(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "static.trendme.net") {
			newsrc = src.replace(/\/showThumb\.php.*?[?&]src=([^&]*).*?$/, "/showThumb.php?src=$1&h=99999999&height=99999999&zc=3");
			if (newsrc !== src)
				return newsrc;

			return src
				.replace(/\/+temp\/+thumbs\/+[0-9]+-[0-9]+-[0-9]+-[0-9]+\/+/, "/pictures/items/")
				.replace(/\/showThumb\.php.*?[?&]src=([^&]*).*?$/, "/pictures/items/$1");
		}

		if (domain_nowww === "ifairer.com") {
			return src.replace(/\/+article_image\/+[a-z]+[0-9]+\/+/, "/article_image/");
		}

		if (amazon_container === "khaskhabar") {
			return src.replace(/\/+khaskhabarimages\/+[a-z]+img\/+img[0-9]+\/+/, "/khaskhabarimages/img500/");
		}

		if (domain === "i.infospesial.net") {
			return src.replace(/:\/\/[^/]*\/+[0-9]+x(?:[0-9]+)?\/+p\/+/, "://media.infospesial.net/image/p/");
		}

		if (domain === "nik.bot.nu") {
			return src.replace(/(:\/\/[^/]*\/+)[a-z]([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1o$2");
		}

		if (domain_nosub === "eyeem.com" &&
			domain.match(/^cdn[0-9]*\./)) {
			return {
				url: src.replace(/(\/+thumb\/+[0-9a-f]+-[0-9]+\/+).*$/, "$1full"),
				problems: {
					watermark: true
				}
			};
		}

		if (domain === "3c1703fe8d.site.internapcdn.net" ||
			domain_nowww === "phys.org") {
			newsrc = src.replace(/\/newman\/+csz\/+([^/]*)\/+[^/]*\/+([0-9]{4}\/+[^/]*)(?:[?#].*)?$/, "/newman/gfx/$1/$2");
			if (newsrc !== src)
				return newsrc;

			return newsrc.replace(/\/+newman\/+gfx\/+([^/]*)\/+([0-9]{4}\/+[^/]*)(?:[?#].*)?$/,
								  "/newman/gfx/$1/hires/$2");
		}

		if (domain_nowww === "sott.net") {
			return src.replace(/(\/+image\/+s[0-9]+\/+[0-9]+\/+)[a-z]+\//, "$1full/");
		}

		if (domain_nowww === "fimgs.net") {
			return src.replace(/\/(?:images|mdimg)\/+([^/]*)\/+[^/.]*\.([0-9]+\.[^/.]*)(?:[?#].*)?$/,
							   "/images/$1/o.$2");
		}

		if ((domain_nosub === "ndsstatic.com" ||
			 domain_nosub === "gentside.co.uk") &&
			domain.match(/^img[0-9]*\./)) {
			return src.replace(/(:\/\/[^/]*\/+)article\/+[0-9]+\/+/, "$1article/");
		}

		if (domain_nowww === "stickpng.com") {
			return src.replace(/\/+assets\/+[a-z]+\/+([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/,
							   "/assets/images/$1");
		}

		if (domain === "media.bizj.us") {
			return src.replace(/(\/view\/img\/[0-9]+\/[^/*]*)\*[^/.]*(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "assets.becomegorgeous.com") {
			return src.replace(/:\/\/[^/]*\/+assets\/+(static\.becomegorgeous\.com\/)/, "://$1");
		}

		if (domain === "static.becomegorgeous.com") {
			return src.replace(/\/+gallery\/+thumbs\/+thumb_/, "/gallery/pictures/");
		}

		if (domain_nowww === "styleslum.com") {
			return src.replace(/\/+main_image_thumbs\/+([^/]*\.[^/.]*)\.thumb_[^/.]*\.[^/.]*(?:[?#].*)?$/, "/main_image/$1");
		}

		if (domain_nowww === "sideshowtoy.com") {
			return src.replace(/(\/+assets\/+products\/+[0-9]+-[^/]*\/+)[a-z]+\/+/, "$1lg/");
		}

		if (domain === "cdn.suwalls.com" &&
			options && options.cb && options.do_request) {
			newsrc = src.replace(/\/+_media\/+users_[0-9]+x[0-9]+\/+/, "/_media/users/");
			if (newsrc !== src)
				return newsrc;

			id = src.replace(/.*\/+wallpapers\/+([^/]+\/+[^/]*-[0-9]+)-[0-9]+x[0-9]+\.[^/.]*(?:[?#].*)?$/, "$1");
			if (id !== src) {
				options.do_request({
					url: "https://suwalls.com/" + id + "/",
					method: "GET",
					onload: function(resp) {
						if (resp.readyState === 4) {
							var match = resp.responseText.match(/<a[^>]* href="(https?:\/\/cdn\.suwalls\.com\/+wallpapers\/+[^">]*)"[^>]*>\s*<img/);
							if (match) {
								options.cb(match[1]);
							} else {
								options.cb(null);
							}
						}
					}
				});

				return {
					waiting: true
				};
			}
		}

		if (domain_nowww === "japanator.com") {
			return src.replace(/(\/+ul\/+[0-9]+-\/+[0-9]+)-[0-9]+x(?:[0-9]+)?(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "cs.mg.co.za") {
			return src.replace(/\/+crop\/+(content\/+images\/+[0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+[^/]*\.[^/.]*)\/+[0-9]+x[0-9]+\/*(?:[?#].*)?$/,
							   "/$1");
		}

		if (domain_nowww === "drytickets.com.au") {
			return src.replace(/\/+assets\/+upload\/+[0-9]+\/+[0-9]+\/+[0-9]+\/+/, "/assets/upload/");
		}

		if (domain_nosub === "filmibeat.com") {
			newsrc = src
				.replace(/\/ph-[^/]*\/+/, "/ph-big/")
				.replace(/\/+img[hm]\/+[0-9]+x[0-9]+\/+/, "/ph-big/")
				.replace(/\/img\/+[0-9]+x[0-9]+(?:x[0-9]*)?\//, "/img/");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "g.ahan.in") {
			return src.replace(/\/+thumbnails\/+([^/]*\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nowww === "behindwoods.com") {
			newsrc = src.replace(/\/+thumbnails\/+([^/]*\.[^/.]*)(?:[?#].*)?$/, "/$1");
			if (newsrc !== src) {
				return {
					url: newsrc,
					problems: {
						watermark: true
					}
				};
			}
		}

		if (domain === "gallery.123telugu.com") {
			return src.replace(/\/+thumbs\/+tn_([^/]*)(?:[?#].*)?$/, "/images/$1");
		}

		if (domain === "images.gludy.com") {
			return src.replace(/(\/+photos\/+[0-9]+\/+[^/]*)_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "static.az-cdn.ch") {
			return src.replace(/\/n-[^/]*(?:[?#].*)?$/, "/teaser-goldbach");
		}

		if (domain_nowww === "starchive.ru") {
			return src.replace(/\/+thumbnails_foto\/+thumb_/, "/foto/");
		}

		if (domain === "pre.aichi.jp") {
			return src.replace(/\/+archive\/+storage\/+thumb\/+/, "/archive/storage/");
		}

		if (domain === "img.kaikai.ch") {
			return src.replace(/\/thumb(?:_[a-z]+)\//, "/img/");
		}

		if (domain_nowww === "4fzone.com") {
			return src.replace(/(\/+uploads\/+images\/+clip_[0-9]+_[0-9]+)_[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1_poster$2");
		}

		if ((domain_nosub === "s1sf.com" ||
			 domain_nosub === "isanook.com") &&
			 domain.match(/^(?:s|p[0-9]+)\./)) {
			return src.replace(/\/+rp\/+r\/+[wh][0-9]+\/+/, "/rp/r/w9999999/");
		}

		if (domain_nosub === "caping.co.id" &&
			domain.match(/^image[0-9]*\./)) {
			return src.replace(/(\.[^/._]+)_[0-9A-Z]+(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "zozchat.com") {
			return src.replace(/\/+thumb-news\/+[0-9]+\/+[0-9]+\/+/, "/images/uploads/images/");
		}

		if (domain === "media.tintuc.vn") {
			return src.replace(/(\/+uploads\/+medias\/+[0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+)[0-9]+x[0-9]+\//, "$1");
		}

		if (domain_nosub === "gigacircle.com") {
			return src.replace(/\/+media\/+[0-9]+x[0-9]+_/, "/media/");
		}

		if (domain_nowww === "ckywf.com") {
			return {
				url: src.replace(/(\/+data\/+[^/]*\/+[0-9]{4}\/+[0-9]{2}\/[0-9a-f]+)_thumb(\.[^/.]*)$/, "$1$2"),
				head_wrong_contenttype: true
			};
		}

		if (domain_nowww === "hao.news") {
			return src.replace(/(\/+Uploads\/+Picture\/+[0-9]{4}-[0-9]{2}-[0-9]{2}\/+[0-9a-f]+)_[0-9]+_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "img.joinfo.ua") {
			return src.replace(/(\/+g\/+[0-9]{4}\/+[0-9]{2}\/+)[0-9]+x[0-9]+\/+/, "$1");
		}

		if (domain_nosub === "hebeilong.com" &&
			domain.match(/^img/)) {
			return src.replace(/^[a-z]+:\/\/[^/]*\/+img\.php\?(http.*)$/, "$1");
		}

		if (domain === "cdn.blogimage2.crooz.jp") {
			return src.replace(/\/smp_thum_([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nosub === "lockerdome.com" && domain.match(/^cdn[0-9]*\./)) {
			return src.replace(/(\/+uploads\/+[0-9a-f]+)_:?[_a-z]+(?:[?#].*)?$/, "$1_:original");
		}

		if (domain === "images.headlines.pw") {
			return src.replace(/(\/[0-9a-f]+)_[0-9]+_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "harianbernas.com") {
			return src.replace(/\/image_news_[0-9]+\//, "/image_news/");
		}

		if (domain === "static.weloveshopping.com") {
			return src.replace(/(\/[^/_.]*)_[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "heykorean.com" &&
			domain.match(/^store[0-9]*\./)) {
			return src.replace(/(\/+board\/+[0-9]+\/+[0-9]+\/+)thumb\//, "$1");
		}

		if (domain_nowww === "searx.info") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+image_proxy.*?[?&]url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeURIComponent(newsrc);
		}

		if (domain_nosub === "nocutnews.co.kr" && domain.match(/^file[0-9]*\./)) {
			return src.replace(/(\/[0-9]+)_preview(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "flixcart.com") {
			return src.replace(/(:\/\/[^/]*\/+image)\/+[0-9]+\/+[0-9]+\/+([0-9a-z]+\/+(?:[^/]*\/+)?.\/+.\/+.\/+[^/]*\.[^/.?]*)(?:[?#].*)?$/, "$1/$2");
		}

		if (domain === "conteudo.imguol.com.br") {
			return src.replace(/(\/[^/]*\.[^/.?#]*)x(?:[?#].*)?$/, "$1");
		}

		if (domain === "images.gog.com") {
			return src.replace(/(:\/\/[^/]*\/+[0-9a-f]+)_[^/]*(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "imx.to") {
			newsrc = src
				.replace(/(:\/\/[^/]+\/+u\/+)t\/+/, "$1i/")
				.replace(/(:\/\/[^/]+\/+)upload\/+[a-z]+\/+/, "$1u/i/");
			if (newsrc !== src)
				return newsrc;

			return src
				.replace(/:\/\/[^/]*\/u\/t\//, "://imx.to/t/")
				.replace(/:\/\/[^/]*\/upload\/+[a-z]+\/+/, "://x001.imx.to/i/");
		}

		if (domain === "t.imx.to") {
			return src.replace(/:\/\/[^/]*\/t\//, "://i.imx.to/i/");
		}

		if (domain_nosub === "imx.to" && domain.match(/^[xi][0-9]+\./)) {
			return src.replace(/(:\/\/[^/]*\/)t\//, "$1i/");
		}

		if (domain_nowww === "sisajb.com" ||
			domain === "image.kbsm.net") {
			newsrc = src.replace(/\/data\/newsThumb\/([0-9]+(?:&&)?).*?$/, "/data/newsData/$1.jpg");
			if (newsrc !== src)
				return add_extensions_upper(newsrc);
		}

		if (domain === "assets.cdn.moviepilot.de") {
			return src.replace(/(\/+files\/+[0-9a-f]+)\/+(?:limit|fill)\/+[0-9]+\/+[0-9]+\/+/, "$1/");
		}

		if (domain === "img.xuehi.cn") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+(?:[0-9]\/+){3}([^/]*%2F.*?\.[^%]+?)[.!]([0-9]+\.)?[0-9]+x[0-9]+\.auto\.[^/.]+(?:[?#].*)?$/, "$1");
			if (newsrc !== src)
				return add_http(decodeURIComponent(newsrc));
		}

		if (domain_nowww === "infectedbyart.com" && string_indexof(src, "/Images/") >= 0) {
			return {
				url: src.replace(/\/+thumbs\/+/, "/"),
				problems: {
					watermark: true
				}
			};
		}

		if (domain === "static.doramatv.me") {
			return src.replace(/(\/uploads\/+pics\/+.*\/[0-9]+)(\.[^/.]*)(?:[?#].*)?$/, "$1_o$2");
		}

		if (domain === "chi.gomtv.com") {
			return src.replace(/\/imgview\.cgi.*?[?&]nid=([0-9]+).*?$/, "/imgview.cgi?nid=$1&type=0");
		}

		if (domain === "static-thechristianpost.netdna-ssl.com" ||
			domain === "cdn.christianpost.com") {
			return src
				.replace(/\/files\/+cache\/+image\/+([0-9]{1,2}\/+[0-9]{1,2}\/+[0-9]+)_(?:[wh]_[0-9]+(?:_[0-9]+)?|(?:a(?:_[0-9]+){4}))(\.[^/.]*)(?:[?#].*)?$/,
						 "/files/original/image/$1$2")
				.replace(/(\/image\/+[^/]*\.[^/.?#]+)(?:[?#].*)?$/, "$1");
		}

		if (domain === "prnewswire2-a.akamaihd.net") {
			return src.replace(/(\/+entry_id\/+[0-9a-z]+_[0-9a-z]+\/+).*$/, "$1def_height/0/def_width/0");
		}

		if (domain_nosub === "eldarioncloud.com" &&
			domain.match(/^radiocms-images\./)) {
			return src.replace(/^[a-z]+:\/\/[^/]*\/+resize\/+[0-9]+\/+(https?:)/, "$1");
		}

		if (domain === "media.ntslive.co.uk") {
			return src.replace(/\/(?:crop|resize)\/+[0-9]+x[0-9]+\/+/, "/images/");
		}

		if (domain_nosub === "tedsby.com" && domain.match(/^cdn[0-9]*\./)) {
			return src.replace(/\/tb\/+[a-z]+\/+storage\//, "/storage/");
		}

		if (domain_nosub === "cumicumi.com") {
			return src.replace(/(\/uploads\/+public\/+(?:[0-9a-f]+\/+){3})th_[0-9]+x[0-9]+_([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "kanal247.com") {
			return src.replace(/\/images\/+media\/+[0-9]+x[0-9]+\/+/, "/images/media/photo/");
		}

		if (domain_nowww === "sportuvai.bg") {
			return src.replace(/(\/pictures\/+[0-9]+_{1,2})[0-9]+(_+(?:[0-9]+)?)?(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2$3");
		}

		if (domain_nosub === "thetimes.co.uk" && string_indexof(src, "/imageserver/image/") >= 0) {
			return {
				url: src.replace(/(?:\?.*)?$/, "?resize=999999999"),
				head_wrong_contentlength: true
			};
		}

		if (domain === "cdn.celebyolo.com") {
			return src.replace(/\/+(?:thumbnails\/+)?([^/]*-[0-9]+)-[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "/$1$2");
		}

		if (domain_nowww === "voice.fi" ||
			domain_nowww === "radionova.fi") {
			return src.replace(/(\/+files\/+media\/+image\/+(?:[0-9]{4}\/+[0-9]{2}\/+[0-9]{4}_[0-9]{2}_[0-9]{2}_[0-9a-f]+|[0-9a-z]{4}\/+[0-9a-z]{2}\/+[0-9a-z]+|[0-9]+\/+_r\/+[0-9]+_retinaarticle))_[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nowww === "funtime.ge") {
			return src
				.replace(/\/+img\/+[0-9]+\/+uploaded\/+/, "/uploaded/")
				.replace(/(\/+uploaded\/+[a-z]+\/+[0-9]{4}-[0-9]{2}\/+[0-9]+)_[a-z]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain === "t.nhentai.net" ||
			(domain_nosub === "hentaifox.com" && /^i[0-9]*\./.test(domain)) ||
			domain === "images.asmhentai.com" ||
			domain === "t.nyahentai.net") {
			newsrc = src.replace(/(:\/\/[^/]+\/+(?:galleries|[0-9]+)\/+[0-9]+\/+[0-9]+)t(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
			if (newsrc !== src)
				return newsrc.replace(/:\/\/t\./, "://i.");

			return src.replace(/(:\/\/[^/]+\/+(?:galleries|[0-9]+)\/+[0-9]+\/)thumb(\.[^/.]*)(?:[?#].*)?$/, "$1cover$2");
		}

		if (domain_nosub === "hitomi.la") {
			newsrc = src.replace(/:\/\/([a-z])tn\.hitomi\.la\/+(?:small|big)tn\/+([0-9a-f]\/+[0-9a-f]{2}\/+[0-9a-f]{20,})\.[^/.]+(?:[?#].*)?$/, "://$1a.hitomi.la/webp/$2.webp");
			if (newsrc !== src)
				return newsrc;

			var regex = /:\/\/[a-z]?tn\.hitomi\.la\/+[a-z]+\/+([0-9]+\/+p?[0-9]+\.[^/.]*)\.[^/.]*(?:[?#].*)?$/;

			if (src.match(regex)) {
				return [
					src.replace(regex, "://aa.hitomi.la/galleries/$1"),
					src.replace(regex, "://ba.hitomi.la/galleries/$1"),
				];
			} else {
				var pageid = src.replace(/^[a-z]+:\/\/[^/]*\/+[a-z]+\/+([0-9]+)\/+.*/, "$1");
				if (src !== pageid) {
					var page = "https://hitomi.la/galleries/" + pageid + ".html";

					return {
						url: src,
						extra: { page: page }
					};
				}
			}
		}

		if (domain === "img.insight.co.kr") {
			return src.replace(/(\/static\/+[0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+)[0-9]+\/+([0-9a-z]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "express.de") {
			return src.replace(/\/+image\/+([0-9]+)\/+[0-9]+x[0-9]+\/+[0-9]+\/+[0-9]+\/+([0-9a-f]+)\/+..\/+([^/.]*)(\.[^/]*)(?:[?#].*)?$/,
							   "/blob/$1/$2/$3-data$4");
		}

		if (domain === "media.news.de") {
			return src.replace(/:\/\/[^/]*\/(?:images\/+[0-9]+|resources)\/+images\/+([0-9a-f]{2}\/+[0-9a-f]{2}\/+)([0-9a-f]+)\/+.*(\.[^/.]*)(?:[?#].*)?$/,
							   "://media.news.de/resources/images/$1$2$3");
		}

		if (domain === "store.live.ksmobile.net") {
			return src.replace(/(:\/\/[^/]*\/)[a-z]+\/+liveme\/+/, "$1liveme/");
		}

		if (domain === "c.fantia.jp") {
			return src.replace(/(\/uploads\/.*\/)[a-z]+_((?:[0-9a-f]+-){4}[0-9a-f]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (host_domain_nowww === "e-hentai.org" &&
			options.host_url.match(/:\/\/[^/]*\/s\//) &&
			options && options.element && options.document && options.cb && options.do_request) {
			if (options.element.tagName === "IMG" && options.element.id === "img" &&
				options.element.src === src) {
				var els = options.document.getElementsByTagName("a");
				for (var i = 0; i < els.length; i++) {
					var el = els[i];
					if (!el.href.match(/https?:\/\/(?:www\.)?e-hentai\.org\/fullimg\.php/))
						continue;

					options.do_request({
						url: el.href,
						method: "HEAD",
						headers: {
							Cookie: document.cookie
						},
						onload: function(resp) {
							if (resp.readyState === 4) {
								if (resp.status === 200) {
									options.cb(resp.finalUrl.replace(/[?#].*$/, ""));
								}
							}
						}
					});

					return {
						waiting: true
					};
				}
			}
		}

		if (host_domain_nowww === "e-hentai.org" &&
			options.host_url.match(/:\/\/[^/]*\/g\//) &&
			domain_nowww === "ehgt.org" &&
			options && options.element && options.document && options.cb && options.do_request) {
			var el = options.element.children[0];
			if (el && el.tagName === "A") {
				options.do_request({
					url: el.href,
					method: "GET",
					onload: function(resp) {
						if (resp.readyState === 4) {
							if (resp.status !== 200) {
								options.cb(null);
								return;
							}

							var mainurl = null;
							var match = resp.responseText.match(/<img *id=["']img["'] *src=["'](.*?)["']/);
							if (match) {
								mainurl = match[1];
							} else {
								return options.cb(null);
							}

							match = resp.responseText.match(/href=["'](https?:\/\/(?:www\.)?e-hentai\.org\/fullimg\.php.*?)["']/);
							if (match) {
								var url = match[1]
									.replace(/&amp;/g, "&");

								options.do_request({
									url: url,
									method: "HEAD",
									headers: {
										Cookie: document.cookie
									},
									onload: function(resp) {
										if (resp.readyState === 4) {
											if (resp.status === 200) {
												options.cb(resp.finalUrl.replace(/[?#].*$/, ""));
											} else {
												options.cb(mainurl);
											}
										}
									}
								});
							} else {
								options.cb(mainurl);
							}
						}
					}
				});

				return {
					waiting: true
				};
			}
		}

		if ((domain === "cdn.zenfolio.net" ||
			domain_nosub === "zenfolio.com") && /:\/\/[^/]+\/+cdn[0-9]*\/+pub\//.test(src)) {
			return src.replace(/:\/\/[^/]*\/+cdn[0-9]*\/+pub\/+(?:[^?#]*?)(\/(?:s[0-9]*\/+)?[^/]*\/+p[0-9]+-[0-9]+\.[^/.?#]*)(?:[?#].*)?$/,
							   "://www.zenfolio.com/img$1");
		}

		if (domain_nosub === "zenfolio.com") {


			var regex = /(\/img\/+(?:s[0-9]*\/+)?v-?[0-9]+\/+p[0-9]+)-[0-9]+(\.[^/.?#]*)(?:[?#].*)?$/;
			return fillobj_urls([
				src.replace(regex, "$1$2"),
				src.replace(regex, "$1-7$2"),
				src.replace(regex, "$1-6$2"),
				src.replace(regex, "$1-5$2"),
				src.replace(regex, "$1-4$2")
			], {
				bad_if: [{
					headers: {
						"content-length": "3446",
						"content-type": "image/png"
					}
				}],
				can_head: false // head only works sometimes, and returns odd last-modified
			});
		}

		if (domain === "gallery.greatandhra.com") {
			return src.replace(/(\/upload\/+[0-9]+\/+)[a-z]+\/+/, "$1images/");
		}

		if (domain_nowww === "fortstore.net" ||
			domain_nowww === "fiyar.live") {
			newsrc = src.replace(/(\/data_server_[0-9]+\/+[0-9]+\/+)[a-z]+\/+[a-z]+_([^/]*\.[^/.]*)(?:[?#].*)?$/,
								   "$1big/$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "static.jpg.pl") {
			return src.replace(/(\/static\/+photos\/+[0-9a-f]+\/+[0-9a-f]+\/+[0-9a-f]+)_[a-z]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1_original$2");
		}

		if ((domain_nosub === "nevsepic.com.ua" ||
			 domain_nosub === "nevseoboi.com.ua") &&
			domain.match(/^c[po][0-9]*\./)) {
			return src.replace(/(\/[0-9]{3}\/+[0-9]+\/+)thumbs\//, "$1");
		}

		if (domain === "static.pulsk.com" ||
			domain_nowww === "pulsk.com") {
			return src.replace(/(\/images\/+[0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+)thumb_[0-9]+_([^/]*\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nowww === "pornchampion.com") {
			return src.replace(/\/images\/+[^/]*\/+([0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/)/, "/images/big_images/$1");
		}

		if (domain === "cdn.vthumbs.com") {
			return src.replace(/:\/\/[^/]*\/thumbs\/[0-9]+px\/content\//, "://content.pornpics.com/");
		}

		if (domain === "content.pornpics.com" ||
			domain === "img.pornpics.com") {
			var folder = "pics";
			if (domain === "img.pornpics.com")
				folder = "pics1";
			return src.replace(/:\/\/[^/]*\/+(?:[0-9]+\/+)?([0-9]{4}-[0-9]{2}-[0-9]{2}\/+[0-9]+_[0-9]+)(\.[^/.]*)(?:[?#].*)?$/,
							   "://cdn.pornpics.com/" + folder + "/$1big$2");
		}

		if (domain === "images.pornpics.com") {
			return src.replace(/(:\/\/[^/]*\/)(?:300|460)\/+/, "$11280/");
		}

		if (domain_nowww === "vipissy.com" ||
			domain === "media.fistertwister.com" ||
			domain === "media.puffynetwork.com") {
			return src.replace(/(\/fhg\/+[0-9a-f]+\/+)thumbs\//, "$1files/");
		}

		if (domain === "images.porninspector.com") {
			return src.replace(/\/tmb_([^/]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nosub === "zoznam.sk") {
			newsrc = src.replace(/:\/\/([^/.]*)\.[^/]*\/cacheImg\/[^/]*\/([0-9]+px)\/(?:[^/.]*-)?([0-9]+[^/]*)(?:[?#].*)?$/, "://static.$1.zoznam.sk/$2/$3");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nosub === "zoznam.sk") {
			return src.replace(/(:\/\/[^/]*\/)[0-9]+px\//, "$1original/");
		}

		if (domain === "gamesite.zoznam.sk") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/images\/+thumb\.php.*?[?&]src=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);

			return {
				url: src,
				headers: {
					Referer: ""
				}
			};
		}

		if (domain_nowww === "wallhalla.com" &&
			options && options.cb && options.do_request) {
			id = src.replace(/.*\/thumbs\/.*\/([^/.]*)[^/]*(?:[?#].*)?$/, "$1");
			if (id !== src) {
				options.do_request({
					url: "https://wallhalla.com/wallpaper/" + id,
					method: "GET",
					onload: function(resp) {
						if (resp.readyState === 4) {
							var match = resp.responseText.match(/ data-wallurl=["']([^"']*)["']/);
							if (match) {
								options.cb(urljoin(src, match[1], true));
							} else {
								options.cb(null);
							}
						}
					}
				});

				return {
					waiting: true
				};
			}
		}

		if (domain === "i.artfile.ru" && options && options.cb && options.do_request) {
			id = src.replace(/^[a-z]+:\/\/[^/]*\/[0-9]+x[0-9]+_([0-9]+)_.*$/, "$1");
			if (id === src) {
				id = src.replace(/^[a-z]+:\/\/[^/]*\/s\/+([0-9]+)_[0-9]+_[^/]*(?:[?#].*)?$/, "$1");
			}
			if (id !== src) {
				options.do_request({
					url: "http://www.artfile.ru/i.php?i=" + id,
					method: "GET",
					onload: function(resp) {
						if (resp.readyState === 4) {
							var match = resp.responseText.match(/top.location.href *= *["'](https?:\/\/i\.artfile\.ru\/[^"']*)["']/);
							if (match) {
								options.cb({
									url: urljoin(src, match[1], true),
									extra: {
										page: resp.finalUrl
									}
								});
							} else {
								options.cb(null);
							}
						}
					}
				});

				return {
					waiting: true
				};
			}
		}

		if (domain_nowww === "relook.ru" ||
			domain === "st.relook.ru") {
			return src.replace(/(\/data\/+cache\/+.*)(?:nothumb[0-9]+|-[0-9]+x[0-9]+)(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nowww === "izhevsk.ru") {
			return src.replace(/(\/forum_pictures\/+[^/]*\/+)thm\//, "$1");
		}

		if (domain === "img.navodayatimes.in") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/default\.aspx.*?[?&]img=([^&]*).*?$/,
							   "$1");
		}

		if (domain_nowww === "wallmachine.pl") {
			return src.replace(/(\/media\/+ecommerce\/+products\/+product[0-9]+\/+)[0-9]+x[0-9]+_(?:true|false)_/, "$1");
		}

		if (domain === "ds393qgzrxwzn.cloudfront.net") {
			return src.replace(/\/resize\/m[0-9]+x[0-9]+\//, "/");
		}

		if (domain_nowww === "religionpeace.ru" ||
			domain_nowww === "eurikacosmetics.ru") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/[0-9]+\/+[0-9]+\/+(https?)\/(.*)$/, "$1://$2");
		}

		if (domain_nowww === "fb.ru") {
			return src
				.replace(/\/misc\/+i\/+thumb\/+[^/]*\/+/, "/media/i/")
				.replace(/(\/media\/+i\/+(?:[0-9]+\/+){1,}i\/+[0-9]+)_[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "specialone.co.kr") {
			return src.replace(/(\/alldata\/+[^/]*\/+)thum_([^/]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "static.mellbimbo.eu") {
			return {
				url: src.replace(/(:\/\/[^/]*\/+)thumb\//, "$1files/"),
				headers: {
					Origin: "http://mellbimbo.eu",
					Referer: "http://mellbimbo.eu"
				}
			};
		}

		if (domain_nowww === "8xxx.net" &&
			options && options.do_request && options.cb) {
			id = src.replace(/^[a-z]+:\/\/[^/]*\/preview\/+[0-9]\/+[0-9]+\/+([0-9]+)\.[^/.]*(?:[?#].*)?$/,
							 "$1");
			if (id !== src) {
				options.do_request({
					url: "http://8xxx.net/pictures/" + id,
					method: "GET",
					onload: function(resp) {
						if (resp.readyState === 4) {
							var match = resp.responseText.match(/<p><a href=["'](.*?)["']>https?:\/\//);
							if (match) {
								options.cb(match[1]);
							} else {
								options.cb(null);
							}
						}
					}
				});

				return {
					waiting: true
				};
			}
		}

		if (domain === "i.frg.im") {
			return src.replace(/_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "thehshq.com") {
			regex = /\/d\/+([0-9]+)(-[0-9]+\/+[^/]*)(?:[?#].*)?$/;
			match = src.match(regex);
			if (match) {
				var parsed = parseInt(match[1]);
				return [
					{
						url: src.replace(regex, "/d/" + (parsed - 2) + "$2"),
						norecurse: true
					},
					{
						url: src.replace(regex, "/d/" + (parsed - 1) + "$2"),
						norecurse: true
					}
				];
			}
		}

		if (domain_nowww === "wfmynews2.com" ||
			domain_nowww === "wusa9.com" ||
			domain_nowww === "11alive.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+img\/+resize\/+([^/]*\.[^/]*\/.*?)(?:[?#].*)?$/, "$1");
			if (newsrc !== src)
				return add_http(newsrc);
		}

		if (domain_nowww === "celeb6free.com" ||
			domain_nowww === "celeb-for-free.com") {
			newsrc = src.replace(/\/pics\/+tn_([^/]*)(?:[?#].*)?$/, "/pics/$1");
			if (newsrc !== src) {
				return {
					url: newsrc,
					extra: {
						page: newsrc.replace(/\/pics\/[^/]*$/, "/")
					}
				};
			}
		}

		if (domain === "images.poms.omroep.nl") {
			return src.replace(/\/image\/+(?:s[0-9]+(?:x[0-9]+)?|c[0-9]+(?:x[0-9]+)?)\/+/, "/image/");
		}

		if (domain_nosub === "tvbuzer.com" &&
			domain.match(/^static[0-9]*\./)) {
			return src.replace(/(\/images\/+[a-z]+\/+)([0-9a-f]+\/+[0-9a-f]+-[0-9]+)-[0-9]+-[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1sources/$2$3");
		}

		if (domain_nosub === "mrskincdn.com" &&
			domain.match(/^assets[0-9]*\./)) {
			return src.replace(/(\/[^/]*-[0-9a-f]+)_[a-z_]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "spankbang.com" &&
			domain.match(/^cdnthumb[0-9]*\./)) {
			return src.replace(/(:\/\/[^/]*\/)[0-9]+\/+([0-9]\/+[0-9]\/+[0-9]+-t)/, "$10/$2");
		}

		if (domain === "static.spankbang.com") {
			return src.replace(/(\/pornstarimg\/+f\/+[0-9]+)-[0-9]+(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "sb-cd.com" && /^tb[0-9]*\./.test(domain)) {
			newsrc = src
				.replace(/(\/[0-9]+-t[0-9]+)-(?:s|enh)\/+/, "$1/")
				.replace(/(:\/\/[^/]+\/+)w:[0-9]+\/+/, "$1w:0/");
			if (newsrc !== src)
				return newsrc;
		}

		if ((domain_nosub === "sb-cd.com" && /^tbv?[0-9]*\./.test(domain)) ||
			(domain_nosub === "spankbang.com" && /^vcdn-[^-]+(?:-[0-9]+)?\./.test(domain))) {
			match = src.match(/:\/\/[^/]+\/+(?:w:[0-9]+\/+)?[0-9]\/+[0-9]\/+([0-9]+)-/);
			var vidid_numeric, vidid;
			if (match) {
				vidid_numeric = parseInt(match[1]);
				vidid = vidid_numeric.toString(36);

				obj = {
					url: src,
					extra: {page: "https://spankbang.com/" + vidid + "/video/"}
				};
			}

			if (options.cb && options.do_request && match) {
				var get_streamkey_for_vidid = function(vidid, cb) {
					var cache_key = "spankbang_streamkey:" + vidid;
					api_cache.fetch(cache_key, cb, function(done) {
						options.do_request({
							method: "GET",
							url: "https://spankbang.com/" + vidid + "/video/",
							onload: function(resp) {
								if (resp.readyState !== 4)
									return;

								if (resp.status !== 200) {
									console_error(resp);
									return done(null, false);
								}

								match = resp.responseText.match(/data-videoid=["'][0-9]+["']\s+data-streamkey=["']([^"']+)["']/);
								if (match) {
									return done(match[1], 24*60*60);
								} else {
									console_warn("Unable to find match", resp);
									return done(null, false);
								}
							}
						});
					});
				};

				var get_streamdata = function(streamkey, cb) {
					var cache_key = "spankbang_streamdata:" + streamkey;
					api_cache.fetch(cache_key, cb, function(done) {
						options.do_request({
							method: "POST",
							url: "https://spankbang.com/api/videos/stream",
							data: "id=" + streamkey + "&data=0",
							headers: {
								"Origin": "https://spankbang.com",
								"Referer": "https://spankbang.com/", // add /.../video?
								"Content-Type": "application/x-www-form-urlencoded; charset=UTF-8",
								"X-Requested-With": "XMLHttpRequest"
							},
							onload: function(resp) {
								if (resp.readyState !== 4)
									return;

								if (resp.status !== 200) {
									console_error(resp);
									return done(null, false);
								}

								try {
									var response = JSON_parse(resp.responseText);
									return done(response, 6*60*60);
								} catch (e) {
									console_error(e, resp);
									return done(null, false);
								}
							}
						});
					});
				};

				var find_largest_from_streamdata = function(streamdata) {
					var sizes = [
						"4k",
						"1080p",
						"720p",
						"480p",
						"360p",
						"320p",
						"240p",
						"144p"
					];

					var largestsize = 100;

					for (var key in streamdata) {
						if (!streamdata[key] || streamdata[key].length === 0)
							continue;

						var ourindex = array_indexof(sizes, key);
						if (ourindex >= 0 && ourindex < largestsize)
							largestsize = ourindex;
					}

					if (largestsize < sizes.length) {
						return streamdata[sizes[largestsize]][0];
					}

					return null;
				};

				var get_norm_sb_url = function(url) {
					return url.replace(/.*\/([0-9]+-[0-9]+[a-z]+\.[^/.?]+)(?:[?#].*)?$/, "$1");
				};

				get_streamkey_for_vidid(vidid, function(streamkey) {
					if (!streamkey)
						return options.cb(obj);

					get_streamdata(streamkey, function(streamdata) {
						if (!streamdata)
							return options.cb(obj);

						var largest = find_largest_from_streamdata(streamdata);

						if (get_norm_sb_url(largest) !== get_norm_sb_url(src)) {
							obj.url = largest;
						}

						options.cb(obj);
					});
				});

				return {
					waiting: true
				};
			} else if (match) {
				return options.cb(obj);
			}
		}

		if (domain_nowww === "kosova-sot.info" ||
			domain_nowww === "botasot.info" ||
			domain_nowww === "syri.net") {
			return src.replace(/(\/(?:uploads|media)\/+(?:[^/]+\/+)?images\/+[0-9]{4}\/+[A-Z][a-z]+\/+[0-9]+\/+)(?:auto|thumb|thumbauto|[0-9]+x[0-9]+)_([^/]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "img.mako.co.il") {
			return src.replace(/(:\/\/[^/]*\/[0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+[^/]*)_(?:[a-z]|reduced)?(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nosub === "okeinfo.net" &&
			domain.match(/^img(?:-[^.]*)?\./)) {
			return src.replace(/(:\/\/[^/]*\/)okz\/+[0-9]+\/+([a-z]+\/)/, "$1$2");
		}

		if (domain_nowww === "freexcafe.com" ||
			domain_nowww === "foxyporn.com") {
			return src
				.replace(/\/img\/([0-9]+)(\.[^/.]*)(?:[?#].*)?$/, "/pics/pics$1$2")
				.replace(/\/img\/([0-9a-f]+)-thumb[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "/pics/$1$2");
		}

		if (domain === "cdn-webimages.wimages.net") {
			return src.replace(/(:\/\/[^/]*\/[0-9a-f]+)(?:-(?:[a-z]+|v[0-9]+)){1,}(\.[^/.]*)$/,
							   "$1$2");
		}

		if (domain === "portal-images.azureedge.net") {
			return src.replace(/\/images\/([-0-9a-f]+\.[^/.]*?)(?:[?#].*)?$/, "/images/$1");
		}

		if (domain === "img.goldlive.co.kr") {
			return src.replace(/\/resize_[0-9]+x[0-9]+\//, "/");
		}

		if (domain_nowww === "nicegirl.io" ||
			domain_nowww === "sexypic.org") {
			return src.replace(/(\/album_[0-9]+\/+)t_(i_[^/]*)(?:[?#].*)?$/, "$1$2");
		}

		if ((domain_nosub === "35photo.ru" ||
			 domain_nosub === "35photo.pro") &&
			src.match(/:\/\/[^/]*\/photos_/)) {
			newsrc = src
				.replace(/:\/\/m[0-9]*\.35photo[^/]*\//, "://35photo.pro/")
				.replace(/\/photos_(?:col|temp)\/+(?:r[0-9]|sizes)\/+([0-9]+\/[0-9]+)_[^/.]*(\.[^/.]*)(?:[?#].*)?$/,
							   "/photos_main/$1$2");

			if (newsrc !== src)
				return newsrc;

			match = src.match(/:\/\/[^/]+\/+photos_main\/+[0-9]+\/+([0-9]+)\./);
			if (match) {
				return {
					url: src,
					extra: {page: "https://35photo.pro/photo_" + match[1] + "/"}
				};
			}
		}

		if ((domain === "pw.artfile.me" ||
			 domain === "i.artfile.me") &&
			options && options.cb && options.do_request) {
			id = src.replace(/.*\/wallpaper\/+[0-9]+-[0-9]+-[0-9]{4}\/+[0-9]+x[0-9]+\/+([^/.?#]*).*?$/, "$1");
			if (id !== src) {
				options.do_request({
					url: "https://artfile.me/wallpaper/" + id + ".html",
					method: "GET",
					onload: function(resp) {
						if (resp.readyState === 4) {
							var match = resp.responseText.match(/<meta *property="og:description" *content="([^"]*)"/);
							if (match) {
								var size = match[1].replace(/.*?размер: ([0-9]+x[0-9]+).*?$/, "$1");
								if (size !== match[1]) {
									newsrc = src.replace(/\/[0-9]+x[0-9]+\//, "/" + size + "/")
												.replace(/:\/\/pw\.artfile/, "://i.artfile");

									options.cb({
										url: newsrc,
										extra: {
											page: resp.finalUrl
										}
									});
								}
							}

							options.cb(null);
						}
					}
				});

				return {
					waiting: true
				};
			}
		}

		if (domain_nosub === "fotokto.ru" && domain.match(/^s[0-9]*\./)) {
			return src.replace(/\/photo\/+[a-z]+\//, "/photo/full/");
		}

		if (host_domain_nosub === "500px.com" && options && options.element) {
			if (string_indexof(src, "://drscdn.500px.org/") < 0) {
				if (options.element.tagName === "A" && options.element.parentElement && options.element.parentElement.classList.contains("nsfw_placeholder")) {
					var img = options.element.querySelector("img");
					return img.src;
				}
			}
		}

		if (domain === "drscdn.500px.org" &&
			options && options.cb && options.do_request) {
			/*
			PHOTO_GRID_IMAGE_SIZES: ["1", "2", "32", "31", "33", "34", "35", "36", "2048", "4", "14"],
			PHOTO_SIZES: {
				S3_IMG_SIZE_70: 1,
				S3_IMG_SIZE_140: 2,
				S3_IMG_GRID_300: 32,
				S3_IMG_GRID_450: 31,
				S3_IMG_GRID_600: 33,
				S3_IMG_WEB_1000: 34,
				S3_IMG_WEB_1500: 35,
				S3_IMG_WEB_2000: 36,
				S3_IMG_SIZE_XLARGE: 2048,
				S3_IMG_STORE_900: 14
			},
			*/

			id = src.replace(/^[a-z]+:\/\/[^/]*\/photo\/+([0-9]+)\/.*$/, "$1");
			var page = "https://500px.com/photo/" + id + "/";
			if (id !== src) {
				options.do_request({
					url: "https://api.500px.com/v1/photos?image_size%5B%5D=1&image_size%5B%5D=2&image_size%5B%5D=32&image_size%5B%5D=31&image_size%5B%5D=33&image_size%5B%5D=34&image_size%5B%5D=35&image_size%5B%5D=36&image_size%5B%5D=2048&image_size%5B%5D=4&image_size%5B%5D=14&expanded_user_info=true&include_tags=true&include_geo=true&include_equipment_info=true&vendor_photos=true&include_licensing=true&include_releases=true&liked_by=1&following_sample=100&ids=" + id,
					method: "GET",
					onload: function(resp) {
						if (resp.readyState !== 4)
							return;

						if (resp.status !== 200) {
							console_error(resp);
							return options.cb(null);
						}

						var obj = {
							url: src,
							extra: {
								page: page
							}
						};

						var sizelist = [0, 1, 2, 32, 31, 33, 14, 34, 35, 36, 2048];
						var comparesizes = function(a, b) {
							var a_index = array_indexof(sizelist, a);
							if (a_index < 0)
								return false;

							var b_index = array_indexof(sizelist, b);
							if (b_index < 0)
								return false;

							return b_index > a_index;
						};

						try {
							var data = JSON_parse(resp.responseText);
							var images = data.photos[id].images;
							page = urljoin(page, data.photos[id].url, true);
							obj.extra.page = page;
							obj.extra.caption = data.photos[id].name || data.photos[id].description;
							var largestsize = 0;
							var largesturl = null;

							for (var i = 0; i < images.length; i++) {
								if (comparesizes(largestsize, images[i].size)) {
									largestsize = images[i].size;
									largesturl = images[i].https_url;
								}
							}

							if (largesturl !== null)
								obj.url = largesturl;
						} catch (e) {
							console_error(e);
							return options.cb(null);
						}

						return options.cb(obj);
					}
				});

				return {
					waiting: true
				};
			}
		}

		if (domain_nowww === "xportsnews.com" ||
			domain === "image.xportsnews.com") {
			return src.replace(/(\/contents\/+images\/+upload\/+.*\/)thm_/, "$1");
		}

		if (domain === "contents.innolife.net") {
			return src.replace(/(\/mobile\/img\/item\/[0-9]+_)[a-z]+(\.[^/.]*)$/, "$1l$2");
		}

		if (domain === "mnews.imaeil.com" ||
			domain === "news.imaeil.com") {
			return src.replace(/(\/photos\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9]{5,})_[sm](\.[^/.]*)(?:[?#].*)?$/,
							   "$1_l$2");
		}

		if (domain_nowww === "blognews.am") {
			return src.replace(/\/static\/+news\/+[a-z]\/+/, "/static/news/b/");
		}

		if (domain_nowww === "sasisa.ru") {
			return src.replace(/(\/blog\/+content\/+[0-9]+\/+[0-9a-f]+)_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "i.photographers.ua" ||
			domain_nowww === "photographers.ua") {
			return src.replace(/\/thumbnails\/+((?:pictures|users)\/+[0-9]+)\/+[0-9]+x([^/.]*\.[^/.]*)(?:[?#].*)?$/,
							   "/images/$1/$2");
		}

		if (domain_nowww === "copia-di-arte.com" &&
			string_indexof(src, "/kunst/") >= 0) {
			return src.replace(/_lo(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain_nosub === "photosight.ru" &&
			domain.match(/^prv-[0-9]{4}-[0-9]+\./)) {
			return src.replace(/:\/\/prv(-.*\/)pv_([0-9]+\.[^/.]*)(?:[?#].*)?$/,
							   "://img$1$2");
		}

		if (domain === "cdn.photosight.ru") {
			newsrc = src.replace(/(\/img\/+[0-9a-f]\/+[0-9a-f]{3}\/+[0-9]+)_(?:thumb|large|mini)(\.[^/.]+)(?:[?#].*)?$/, "$1_xlarge$2");

			match = src.match(/\/img\/+[0-9a-f]\/+[0-9a-f]{3}\/+([0-9]+)(?:_[a-z]+)?\./);
			if (match) {
				return {
					url: newsrc,
					extra: {page: "https://photosight.ru/photos/" + match[1] + "/"}
				};
			} else {
				return newsrc;
			}
		}

		if (domain === "rs.kantie.org") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/it\/(https?:)/, "$1");
		}

		if (domain_nowww === "zmut.com") {
			return src.replace(/\/uploads\/cache\/(pins\/+[0-9]{4}\/+[0-9]{2}\/+[^/]*)-[0-9]+x(\.[^/.]*)(?:[?#].*)?$/,
							   "/uploads/$1$2");
		}

		if (domain_nosub === "2photo.ru" &&
			domain.match(/^i[0-9]*\./)) {
			return src.replace(/(:\/\/[^/]*)\/[a-z]+\/+(.\/+.\/+[0-9]+\.)/, "$1/$2");
		}

		if (domain === "m.salon24.pl") {
			return src.replace(/(:\/\/[^/]*\/[0-9a-f]+)(?:,[0-9]+){4}(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "cdn.fishki.net" ||
			domain === "cdn-tn.fishki.net") {
			return src
				.replace(/(\/upload\/+post\/+[0-9]{6}\/+[0-9]{2}\/+[0-9]+\/+)tn\/+/, "$1")
				.replace(/(\/upload\/+post\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9]+\/+(?:gallery\/+)?)tn\/+/, "$1")
				.replace(/:\/\/cdn-tn\.([^/]*\/)(?:[0-9]+\/+)?upload\//, "://cdn.$1upload/");
		}

		if (domain_nowww === "ravshaniya.com") {
			return src.replace(/\/uploads\/+photos\/+thumbs\/+[0-9]+\/+/, "/uploads/photos/");
		}

		if (domain_nowww === "onlyhdwallpapers.com" ||
			domain_nowww === "hdwallpapers.cat") {
			var cookie = null;
			var watermark = true;
			if (options.document && options.document.cookie) {
				cookie = options.document.cookie;
				watermark = false;
			}

			return {
				url: src.replace(/(:\/\/[^/]*\/)thumbnail(?:_[a-z]+)?\/+/, "$1wallpaper/"),
				headers: {
					Origin: "https://" + domain,
					Referer: src,
					Cookie: cookie
				},
				problems: {
					watermark: true
				}
			};
		}

		if (domain === "cdn.slushe.com") {
			newsrc = src.replace(/\/thumbs\/+((?:[0-9a-f]\/+){5}[0-9a-f]+\.mp4)\/+[^/]+(?:[?#].*)?$/, "/videos/$1");
			if (newsrc !== src) {
				return newsrc.replace(/:\/\/cdn\./, "://vcdn.");
			}
		}

		if (domain_nowww === "bestpornbabes.com" ||
			domain === "cdn.shesfreaky.com" ||
			domain === "media.ruleporn.com" ||
			domain_nowww === "teenplanet.org" ||
			domain_nowww === "goodsexporn.org" ||
			domain === "cdn.slushe.com" ||
			domain === "media.babesource.com") {
			return src.replace(/(\/+galleries\/+[0-9a-f]+\/+)thumbs\/+(?:[0-9]+x[0-9]+\/+)?/, "$1");
		}

		if (domain === "staticpopopics.popopics.com") {
			return src.replace(/\/uploads\/(?:thumb|display)\//, "/uploads/original/");
		}

		if (domain_nowww === "stopga.me") {
			return src.replace(/(\/images\/+(?:uploads\/+images|news)\/+[0-9]+\/+.*\/)normal_([^/]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "foap.com" && domain.match(/^cdn[0-9]*\./)) {
			return src.replace(/(\/images\/+[-0-9a-f]+\/+)[wh][0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1original$2");
		}

		if (domain_nosub === "foap.com" && domain.match(/^images[0-9]*\./)) {
			return src.replace(/:\/\/images[0-9]*(\.foap\.com\/images\/+[-0-9a-f]+\/+)[^/.]*(\.[^/.?#]*)(?:[?#].*)?$/,
							   "://cdn2$1original$2");
		}

		if (domain_nowww === "themepack.me") {
			return src.replace(/\/i\/+c\/+[0-9]+x[0-9]+\/+media\/+/, "/media/");
		}

		if (domain === "static.mycuteasian.com") {
			return src.replace(/\/cache\/+(content\/+pictures\/+.*)\.[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "/$1$2");
		}

		if (domain_nowww === "jiji.com") {
			return src.replace(/(\/v[0-9]*_photos\/.*)_[sm](\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "storage.mantan-web.jp") {
			return src.replace(/(\/[^/]*)_(?:size[0-9]*|thumb[0-9]*)(\.[^/.]*)(?:[?#].*)?$/, "$1_size10$2");
		}

		if (domain_nowww === "cinra.net") {
			return src.replace(/(\/uploads\/+img\/+.*)_[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1_full$2");
		}

		if ((domain_nosub === "uecdn.es" && /^e00-[a-z]+\./.test(domain)) ||
			domain === "estaticos.telva.com") {
			return src.replace(/_(?:inc|movil|m|[0-9]+x[0-9]+)(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain === "fs.kinomania.ru") {
			return src.replace(/\/image(\/+file\/.*\/[0-9a-f]+)\.[0-9]+\.[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "notigape.com") {
			return src.replace(/\/thumburl\/+thumbnail\/+[0-9]+x[0-9]+\/+outbound\/+uploads\/+/, "/uploads/");
		}

		if (domain_nowww === "prdelinky.cz") {
			return src.replace(/\/uploaded\/+gallery\/+thumb_/, "/uploaded/gallery/");
		}

		if (domain_nosub === "selfimg.com.cn" &&
			domain.match(/^img[0-9]*\./)) {
			return src
				.replace(/(:\/\/[^/]*\/)gq[0-9]+[^/]*\/+/i, "$1uedgqcms/")
				.replace(/(:\/\/[^/]*\/)vogue[^/]*\//i, "$1uedvoguecms/");
		}

		if (domain === "hp.funrahi.com") {
			return src.replace(/(:\/\/[^/]*\/[0-9]{4}\/+[0-9]{2}\/+[^/]*\/+[^/]*)_(?:t|[0-9]+)(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "img.007shoes.com") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/\?img_url=(https?:)/, "$1");
		}

		if (domain_nowww === "cineol.net") {
			return src.replace(/\/fotos\/+thumb[0-9]+_/, "/fotos/");
		}

		if (domain_nowww === "nosolocine.es") {
			return src.replace(/\/images\/+galeria\/+thumbs\/+/, "/images/galeria/");
		}

		if (domain === "pics.filmaffinity.com") {
			return src.replace(/-s[0-9]*(\.[^/.]*)(?:[?#].*)?$/, "-large$1");
		}

		if (domain === "fitsnews.files.wordpress.com") {
			newsrc = src.replace(/\.thumbnail(\.[^/.]*)(?:[?#].*)?$/, "$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "data.okinny.heypo.net") {
			return {
				url: src.replace(/\/image\/+thumb\/+([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/, "/image/large/$1"),
				headers: {
					Referer: "https://okinny.heypo.net/image/12345"
				}
			};
		}

		if (domain_nosub === "mgstage.com" &&
			domain.match(/^(?:spimg[0-9]*|image)\./)) {
			return src.replace(/(\/[^/_]*_)[^/_]*(_[^/]*)(?:[?#].*)?$/, "$1e$2");
		}

		if (domain_nosub === "gettextbooks.com" &&
			domain.match(/^img[0-9]*\./)) {
			return src.replace(/(\/pi\/+[0-9]+\/+)[-0-9]+\/+[-0-9]+(?:\/+)?(?:[?#].*)?$/, "$1999999999/999999999");
		}

		if (domain_nosub === "y3600.cn" &&
			domain.match(/^img[0-9]*\./)) {
			return src.replace(/\/z(?:resize|crop)?\/+[0-9]+\/+[0-9]+\//, "/zresize/999999999999/999999999999/");
		}

		if (domain_nowww === "life.tw" ||
			domain === "amazon.life.com.tw") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/proxy\.php\?url=([^&]*).*?$/, "$1");
			if (newsrc !== src) {
				return decodeURIComponent(newsrc);
			}
		}

		if (host_domain_nosub === "onlyfans.com" &&
		     (domain_nosub === "onlyfans.com" ||
			  domain === "media.onlyfans.com" ||
			  amazon_container === "of2media") && options && options.element && options.do_request && options.cb) {

			var get_appkey = function(cb) {
				return cb(base64_decode("Lj0zM2Q1N2FkZThjMDJkYmM1YTMzM2RiOTlmZjlhZTI2YQ==").substr(2));
			};

			var onlyfans_api_call = function(url, querystring, referer, cb) {
				get_appkey(function(appkey) {
					if (!appkey)
						return cb(null);

					url = urljoin("https://onlyfans.com/api2/v2", url, false);

					if (querystring) {
						url += "?" + querystring + "&app-token=" + appkey;
					} else {
						url += "?app-token=" + appkey;
					}

					options.do_request({
						url: url,
						method: "GET",
						headers: {
							Referer: referer,
							"Sec-Fetch-Site": "same-origin"
						},
						onload: function(result) {
							if (result.readyState !== 4)
								return;

							if (result.status !== 200) {
								console_error(result);
								return cb(null);
							}

							try {
								var json = JSON_parse(result.responseText);
								cb(json);
							} catch (e) {
								console_error(e, result);
								cb(null);
							}
						}
					});
				});
			};

			var get_user = function(username, cb) {
				api_cache.fetch("onlyfans_user:" + username, cb, function(done) {
					onlyfans_api_call("/users/" + username, null, "https://onlyfans.com/" + username, function(data) {
						if (data) {
							return done(data, 1*60*60);
						} else {
							return done(null, false);
						}
					});
				});
			};

			var get_avatar = function(username, cb) {
				get_user(username, function(data) {
					if (data) {
						return cb(data.avatar);
					} else {
						return cb(null);
					}
				});
			};

			var get_avatar_from_a = function(el, cb) {
				var match = el.href.match(/^https?:\/\/[^/]+\/+([^/]+)\/*(?:[?#].*)?$/);
				if (match) {
					get_avatar(match[1], cb);
					return true;
				}

				return false;
			};

			if (options.element.parentElement && options.element.parentElement.classList.contains("b-avatar")) {
				var dparent = options.element.parentElement.parentElement;
				if (dparent.tagName === "A") {
					if (get_avatar_from_a(dparent, options.cb)) {
						return {
							waiting: true
						};
					}
				} else if (dparent.tagName === "DIV" && dparent.classList.contains("b-profile__user")) {
					var link = dparent.querySelector("a.g-user-realname__wrapper");
					if (link && get_avatar_from_a(link, options.cb)) {
						return {
							waiting: true
						};
					}
				}
			}
		}

		if (domain === "img.mfcimg.com") {
			return src.replace(/(\/photos2\/+[0-9]+\/+[0-9]+\/+[-0-9]+)\.[0-9x]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "webnewtype.com") {
			return {
				url: src.replace(/(\/rsz\/+S[0-9]*\/+[0-9]+\/+[^/.]*\.[^/.?#]*)(?:\/[wh][0-9]+.*)?(?:[?#].*)?$/,
								 "$1"),
				can_head: false // returns 404
			};
		}

		if (domain === "cdn.themis-media.com") {
			return src.replace(/\/media\/+global\/+images\/+galleries\/+[a-z]+\/+/,
							   "/media/global/images/galleries/full/");
		}

		if (domain === "tn.nozomi.la") {
			newsrc = src.replace(/:\/\/tn\.([^/]*\/[0-9a-f]+\/+[0-9a-f]+\/+[0-9a-f]+)(\.[^/.]*)(?:\.[^/.]*)(?:[?#].*)?$/,
								 "://i.$1$2");
			if (newsrc !== src) {
				return {
					url: newsrc,
					headers: {
						Referer: "https://nozomi.la/"
					}
				};
			}
		}

		if (domain === "pic.baike.soso.com") {
			return src.replace(/(\/baikepic[0-9]*\/+[^/]*\/+[^/]*\/+)[0-9]+(?:[?#].*)?$/, "$10");
		}

		if (domain_nosub === "jpopasia.com" && domain.match(/^i[0-9]*\./)) {
			return add_extensions(src.replace(/(\/(?:assets|news|albums)\/+[0-9]+\/+[^/]*)-t(\.[^/.]*)(?:[?#].*)?$/, "$1$2"));
		}

		if (domain_nowww === "jpgravure.com" ||
			domain_nowww === "tiedvirgins.com" ||
			domain_nowww === "hqjpporn.com" ||
			domain_nowww === "thethaigirls.com") {
			return src.replace(/(\/galleries\/.*\/)tn([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "highasianporn.com") {
			return src.replace(/(\/g\/.*\/)tn([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "asianxxxpics.com" ||
			domain_nowww === "japanfetishpics.com" ||
			domain_nowww === "asianfuckpics.com" ||
			domain_nowww === "asianteenpics.com" ||
			domain_nowww === "nudejapangirls.com" ||
			domain_nowww === "japaneseidols.org" ||
			domain === "promo.spunkyangels.com") {
			return src.replace(/\/tn([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nowww === "vintage-flash.com") {
			return {
				url: src.replace(/(\/vfg\/+[^/]*\/+)tn([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$10$2"),
				headers: {
					Referer: ""
				}
			};
		}

		if (domain_nowww === "1pondo.tv") {
			return src
				.replace(/(\/assets\/+sample\/+[0-9]+_[0-9]+\/+[^/]*)_s(\.[^/.]*)(?:[?#].*)?$/, "$1$2")
				.replace(/(\/assets\/+sample\/+[0-9]+_[0-9]+\/+)thum_[0-9]+(\/+[0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1popu$2");
		}

		if (domain_nosub === "javhd.com" ||
			domain_nosub === "lingerieav.com") {
			return src.replace(/(\/[0-9]+\/+[0-9]+)t(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "galleries.allgravure.com") {
			return src.replace(/(\/[0-9]+\/+[^/]*\/+[0-9]+)t(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "bravoerotica.com") {
			return src.replace(/(\/[0-9]+)t(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "japanesepussyclub.com" ||
			domain_nowww === "kabukicho-girls.com") {
			return {
				url: src.replace(/(\/+[^/]*\/+)thumbnails\/+([^/]*)_tn(\.[^/.]*)(?:[?#].*)?$/, "$1images/$2$3"),
				headers: {
					Referer: "http://" + domain
				}
			};
		}

		if (domain_nowww === "asianthumbs.org") {
			return src.replace(/(\/gallery\/+.*\/+)tn\/+([^/]*\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "images.anilos.com" ||
			domain === "images.nubiles.net") {
			return src.replace(/\/tn\/+([^/]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain === "cdn.handjobjapan.com") {
			return src.replace(/(\/preview\/+[0-9a-z]+\/+[0-9]+)s(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "cre.ma" && domain.match(/^assets[0-9]*\./)) {
			return src.replace(/(\/image[0-9]*\/+)thumbnail_([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "cdn.megacountry.livenation.com") {
			return src.replace(/(\/production\/+[^/]*-photo\/+[0-9]+\/+)[a-z]+_([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "annangelxxx.com") {
			return src.replace(/(\/tgp\/+[^/]*\/+[0-9]+\/+)thumbnails\/+tn([0-9]+\.[^/.]*)(?:[?#].*)?$/,
							   "$1originalimages/$2");
		}

		if (domain_nosub === "coedcherry.com" &&
			domain.match(/^content[0-9]*\./)) {
			return src.replace(/\/th[0-9]+x[0-9]+(?:_[a-z])?_([^/]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nowww === "nicsgalleries.com") {
			return src.replace(/(\/g\/.*\/[0-9a-f]+\/+)thumbs\/+thumb_([^/]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "classypussy.com" ||
			domain_nosub === "skankypussy.com") {
			return src.replace(/\/thumbs\/+thumbs_/, "/");
		}

		if (domain_nowww === "hiqqu.xxx" ||
			domain_nowww === "hiqqu.com") {
			return src.replace(/\/files\/+[0-9]+x[0-9]+\/+/, "/files/");
		}

		if (domain === "img.chan4chan.com") {
			return {
				url: src.replace(/(\/img\/+[0-9]{4}-[0-9]{2}-[0-9]{2}\/+)tn_([^/]*)(?:[?#].*)?$/, "$1$2"),
				can_head: false // 404
			};
		}

		if (domain_nowww === "haxprofiles.com") {
			return src.replace(/(\/gallery\/+[^/]+\/+)tn([^/]*)_[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2$3");
		}

		if (domain_nowww === "drago99.com") {
			return src.replace(/\/tn([^/]*\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain === "img.indexxx.com") {
			return {
				url: src.replace(/\/images\/+thumbs\/+[0-9]+x[0-9]+\/+/, "/images/"),
				headers: {
					Referer: "https://www.indexxx.com/"
				}
			};
		}

		if (domain_nosub === "ero.today" ||
			domain_nosub === "oldax.com") {
			newsrc = src.replace(/\/p\/+[0-9]+x[0-9]+_([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/m$1");
			if (newsrc !== src) {
				return {
					url: newsrc,
					headers: {
						Referer: "http://www." + domain_nosub + "/"
					}
				};
			}
		}

		if (domain_nosub === "dreamercdn.com") {
			return src.replace(/(\/media\/+[0-9]+\/+[0-9]+\/+)[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1original$2");
		}

		if (amazon_container === "images.charitybuzz.com" ||
			domain === "images.charitybuzz.com") {
			return src.replace(/(\/images\/+[0-9]+\/+)[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1original$2");
		}

		if (domain === "media.movieassets.com") {
			return src.replace(/(\/static\/+images\/+.*\/)[0-9]+\/+[0-9]+\/+(?:[^/]*-)?([0-9a-f]{20,}\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2")
		}

		if (domain === "movieassetsdigital.sgp1.cdn.digitaloceanspaces.com") {
			return src.replace(/\/thumb\/+([0-9a-f]{20,})(?:[?#].*)?$/, "/original/$1");
		}

		if (domain_nosub === "happyhair.sk") {
			return src.replace(/\/celebrity_img\/+thumbs[^/]*\/+/, "/celebrity_img/");
		}

		if (domain_nosub === "wn.com" &&
			domain.match(/^cdn[0-9]*\./)) {
			return src.replace(/(\/[0-9a-f]{20,}[-_])small(\.[^/.]*)(?:[?#].*)?$/, "$1large$2");
		}

		if (domain_nowww === "atoananet.com.br") {
			return src.replace(/(\/links\/+[0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+[^/]*)_[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "img.ifcdn.com") {
			regex = /(\/images\/+[0-9a-f]+_)[0-9]+(\.[^/.]*)(?:[?#].*)?$/;
			return [
				src.replace(regex, "$13$2"),
				{
					url: src.replace(regex, "$11$2"),
					problems: {
						watermark: true
					}
				}
			];
		}

		if (domain_nowww === "babeprofiles.com") {
			return src.replace(/(\/media\/+models\/.*\/)[0-9]+x[0-9]+\/+([^/]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nosub === "meetmecdna.com") {
			return src.replace(/\/thumb_userimages(\/+.*\/)thm_/, "/userimages$1");
		}

		if (domain === "images.spicyadulttools.com") {
			return src.replace(/\/images\/+thumb\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/,
							   "/images/full/$1");
		}

		if (domain_nosub === "trafficdeposit.com" &&
			domain.match(/^s[0-9]*\./)) {
			return src.replace(/(\/vid\/+[0-9a-f]+\/+[0-9a-f]+\/+)[a-z]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1full$2");
		}

		if (domain_nowww === "babepedia.com") {
			return src.replace(/\/galleries-thumbs\/+/, "/galleries/");
		}

		if (domain_nowww === "yourdailygirls.com" ||
			domain_nowww === "girlsfordays.com" ||
			domain_nowww === "girlznation.com") {
			return src.replace(/(\/galleries\/.*\/[^/]*)-tn(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "static.ftvgirls.com" ||
			domain === "promo.ftvgirls.com") {
			return src
				.replace(/(\/galleries\/.*\/[0-9]+)-thumb(\.[^/.]*)(?:[?#].*)?$/, "$1$2")
				.replace(/\/thumbnails\/+[0-9]+x[0-9]+_/, "/pictures/content_");
		}

		if (domain === "image.istyle24.com") {
			return src.replace(/(\/upload\/+ProductImage\/+[0-9]+\/+[0-9]+\/+[0-9]+_(?:[0-9]+_)?)[a-zA-Z]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1L$2");
		}

		if (domain === "zine.istyle24.com") {
			if (/\/App_Themes\/.*\/images\/+common\//.test(src))
				return {
					url: src,
					bad: "mask"
				};

			return src.replace(/(\/FileUpload\/+Content\/+[0-9]{10,})_[0-9]+(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "bbstar.kr") {
			return src.replace(/(\/model\/+[^/]*[0-9]+)s(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nosub === "asn.im" && domain.match(/^i[0-9]*\./)) {
			return src.replace(/(:\/\/[^/]*\/[^/]*)-[a-z](\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nowww === "bebzol.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/thumbs.*?[?&]i=([^&]*).*?$/, "$1");
			if (newsrc !== src) {
				return urljoin(src, "/" + decodeURIComponent(newsrc), true);
			}
		}

		if (domain_nowww === "thecandidforum.com") {
			return src.replace(/(\/attachments\/+[0-9a-f]+)t(-[^/]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "hottystop.com" ||
			domain === "d3m3u5a3.ssl.hwcdn.net" ||
			domain_nowww === "nextdoortease.com" ||
			domain_nowww === "foxhq.com" ||
			domain === "a2w8r2x2.ssl.hwcdn.net") {
			newsrc = src.replace(/\/smallimage([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "img.highviral.news") {
			return src.replace(/(:\/\/[^/]*\/)[0-9]+x[0-9]+\/+([0-9]+_[0-9a-f]+\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "mondrian.mashable.com") {
			newsrc = src.replace(/(:\/\/[^/]*\/[^/?]*%252F[^/?]*)\?.*$/, "$1");
			if (newsrc !== src)
				return decodeURIComponent(decodeURIComponent(newsrc));

			newsrc = src.replace(/:\/\/[^/]*\/(wp-content\/+.*?\/[^/]*\.[^/.]*)\/[^/]*\.[^/.]*(?:[?#].*)?$/, "://admin.mashable.com/$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "image.wisetrail.com") {
			return src.replace(/(\/bookmark[0-9]+\/+[^/]*)_small(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "internationallovescout.com") {
			return src
				.replace(/(\/entries\/+[0-9]+\/+)thumb_/, "$1")
				.replace(/(\/images\/+uploads\/.*)-[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain.match(/^images\.locanto\./)) {
			return src.replace(/(:\/\/[^/]*\/[^/]*\/+)[a-z]+_([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "dailytravelphotos.com") {
			return src.replace(/(\/images\/+[0-9]+)t(\/[^/]*)_th(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "galleries.grooby.com") {
			return src.replace(/\/tn-([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if ((domain_nosub === "adult-empire.com" ||
			domain_nosub === "cozyxxx.com") &&
			domain.match(/^pbs(?:-[0-9]+)?\./)) {
			newsrc = src
				.replace(/\/tns\/+tn([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/$1")
				.replace(/\/thumb\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/pics/$1")
				.replace(/\/pic\/+t(?:n[0-9]*\/+)?([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/pic/$1")
				.replace(/\/(?:thumbnails|tm)\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/$1")
				.replace(/\/thumbs\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/pics/$1")
				.replace(/\/t\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/p/$1")
				.replace(/\/tumb\/+tumb([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/pics/$1")
				.replace(/\/pics\/+thumbs\/+p([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/pics/p$1")
				.replace(/(\/[0-9]+)xxx(\.[^/.]*)(?:[?#].*)?$/, "$1$2")
				.replace(/\/th\/+thumbnails\/+tn([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/$1")
				.replace(/(\/[0-9]+)_resize(\.[^/.]*)(?:[?#].*)?$/, "$1$2")
				.replace(/\/tn(chicksandbeasts\.com-[0-9]+\.[^/.]*)(?:[?#].*)?$/, "/www.$1")
				.replace(/\/th([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/$1")
				.replace(/(\/[0-9]+)a(\.[^/.]*)(?:[?#].*)?$/, "$1$2")
				.replace(/\/thumbs\/+thumb([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/pics/pic$1")
				.replace(/(\/[0-9]+\/+[0-9]+\/+p[0-9]+\/+)tn_([^/]*)(?:[?#].*)?$/, "$1$2")
				.replace(/(\/[0-9]+\/+[0-9]+\/+)thumbnails\/+tn([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1images/$2")
				.replace(/(\/[0-9]+\/+[0-9]+\/+cg_[0-9]+_[0-9]+\/+)thumbs\/+tn_([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2")
				.replace(/(:\/\/[^/]*\/[^/]*\/+[^/]*\/+[^/]*\/+)tn([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2")
				.replace(/(:\/\/[^/]*\/+[0-9]{3}\/+[0-9]+\/+[0-9]+\/+)t([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1i$2");
			if (newsrc !== src)
				return newsrc;

			regex = /\/thumbnails\/+tn([0-9]+\.[^/.]*)(?:[?#].*)?$/;
			if (src.match(regex)) {
				return [
					src.replace(regex, "/images/$1"),
					src.replace(regex, "/$1")
				];
			}
		}

		if (domain_nowww === "exl.io") {
			return src.replace(/(\/[0-9a-f]{20,}\/+)tmd([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "idols69.net") {
			return src.replace(/(\/pictures\/+[^/]*\/+)t([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "fhg.mycuteasian.com" ||
			domain === "fhg.avidolz.com") {
			return src.replace(/(\/picture\/+[^/._]*_)t([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "teenfilipina.net") {
			return src
				.replace(/\/thumbs\//, "/")
				.replace(/(-[0-9]+)thumb(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "rexxx.co") {
			return src.replace(/(:\/\/[^/]*\/[0-9]+\/[0-9]+)(\.[^/.]*)(?:[?#].*)?$/,
							   "$1f$2");
		}

		if (domain_nosub === "mobile9.com") {
			return src.replace(/\/download\/+thumb\/+([0-9]+)\/+[0-9]+\/+([^/]*\.[^/.]*)(?:[?#].*)?$/,
							   "/download/media/$1/$2");
		}

		if (domain === "s.fap5.com" ||
			domain === "s.mycosplayclub.com") {
			return src.replace(/(\/[0-9a-f]{15,}\/+)thumbs\/+([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nowww === "coolspotters.com") {
			return src.replace(/(\/files\/+photos\/+[0-9]+\/+[^/]*)-(?:small|large|medium|micro|popover|gallery)(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "celebsdump.com" ||
			domain_nowww === "nakedcelebs.top") {
			regex = /(\/posts\/+[0-9]{4}\/+[0-9]{2}\/+[0-9]+\/+)(?:thumbs|featured|images)\/+/;
			return [
				src.replace(regex, "$1originals/"),
				src.replace(regex, "$1images/")
			];
		}

		if (domain === "photos.laineygossip.com") {
			return src.replace(/\/thumbs\/+tmb_[0-9]+x[0-9]+_/, "/articles/");
		}

		if (domain_nowww === "laineygossip.com") {
			return urljoin(src, src
						   .replace(/^[a-z]+:\/\/[^/]*\/Thumbnail\.axd.*?[?&]p=([^&]*).*?$/, "$1")
						   .replace(/[+]/g, "%20"), true);
		}

		if (domain === "images.successstory.com") {
			return src.replace(/(\/img_[a-z]+\/+[^/]*\/+)[0-9]+X[0-9]+\/+/, "$1");
		}

		if (domain_nosub === "flixster.com" &&
			domain.match(/^content[0-9]*\./)) {
			return src.replace(/(\/photo\/+[0-9]+\/+[0-9]+\/+[0-9]+\/+[0-9]+_)[a-z]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1ori$2");
		}

		if (amazon_container === "focusmicrosites") {
			return src.replace(/(\/assets\/+uploads\/+)_tmp\/+([^/]*)-[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2$3");
		}

		if (domain === "i.digiguide.tv") {
			return src.replace(/(\/p\/+[0-9]+\/+)tn-/, "$1");
		}

		if (domain_nowww === "azquotes.com") {
			return src.replace(/(\/public\/+pictures\/+.*\/)c_([0-9a-f]+[^/]*)_thumb(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2$3");
		}

		if (domain_nowww === "newsarama.com" ||
			domain === "i.newsarama.com") {
			return src.replace(/(\/images\/+i\/+[0-9]+\/+[0-9]+\/+[0-9]+\/+)i[0-9]+\/+/,"$1original/");
		}

		if (domain_nowww === "moly.hu") {
			newsrc = src.replace(/(\/system\/+authors\/+[0-9]+\/+image_)[a-z]+\./,
								 "$1original.");
			if (newsrc !== src)
				return newsrc;

			regex = /(\/system\/+[^/]*\/+)[a-z]+(\/+[a-z]+_[0-9]+\.)/;
			newsrc = src.replace(regex, "$1original$2");
			if (newsrc !== src)
				return [newsrc, src.replace(regex, "$1big$2")];
		}

		if (domain_nowww === "arasale.com") {
			return src.replace(/(\/photo[0-9]*\/.*)_small[0-9]*(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "imgth.com") {
			return src.replace(/\/thumbs\/+/, "/images/");
		}

		if (domain_nowww === "intimatecelebs.com") {
			return src.replace(/(\.[^/.]*)\.icthumb\.[^/.]*(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "celebriot.com") {
			return src.replace(/(\.[^/.]*)\.thumb_[0-9]+x[0-9]+\.[^/.]*(?:[?#].*)?$/, "$1");
		}

		if (domain_nosub === "paheal.net" ||
			domain_nowww === "rule34hentai.net") {
			return src.replace(/:\/\/[^/]*\/_thumbs\//, "://peach.paheal.net/_images/");
		}

		if (domain_nosub === "lovethispic.com") {
			return src.replace(/\/uploaded_images\/+thumbs\/+/, "/uploaded_images/");
		}

		if (domain_nowww === "zwz.cz" &&
			options && options.do_request && options.cb) {
			id = src.replace(/^[a-z]+:\/\/[^/]*\/o[^/]*\.zwz\/+([A-Z0-9]+)\.[^/.]*(?:[?#].*)?$/, "$1");
			if (id !== src) {
				options.do_request({
					url: "http://zwz.cz/f/" + id,
					method: "GET",
					headers: {
						Referer: "http://zwz.cz/",
						Cookie: "agree=yes"
					},
					onload: function(result) {
						if (result.readyState === 4) {
							if (result.status !== 200) {
								console_log(result);
								options.cb(null);
								return;
							}

							var match = result.responseText.match(/<img *id=['"]?pic["']? *class[^>]*?src=["']([^'"]*)['"]/);
							if (match) {
								options.cb({
									url: urljoin(src, match[1], true),
									headers: {
										Referer: "http://zwz.cz/",
										Cookie: "agree=yes"
									}
								});
							} else {
								options.cb(null);
							}
						}
					}
				});

				return {
					waiting: true
				};
			}
		}

		if (domain_nowww === "hockeygods.com") {
			return src.replace(/(\/system\/+gallery_images\/+[0-9]+\/+)[a-z]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1original$2");
		}

		if (domain === "galleries.cosmid.net") {
			return src.replace(/(:\/\/[^/]*\/[0-9]+)\/+thumbs\/+/, "$1/");
		}

		if (domain_nowww === "porn-star.com" ||
			(domain_nosub === "ptclassic.com" && /^galleries[0-9]*\./.test(domain))) {
			newsrc = src.replace(/\/thumbs\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "suburbanamateurs.com") {
			return {
				url: src.replace(/(\/galleries\/+[^/]*\/+)thumbs\/+/, "$1highres/"),
				headers: {
					Referer: "http://www.suburbanamateurs.com/"
				}
			};
		}

		if (domain === "video.soloteengirls.net") {
			return src.replace(/\/materials\/+thumbs\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/,
							   "/materials/$1");
		}

		if (domain_nosub === "fantasti.cc" &&
			domain.match(/^cdn(?:-[a-z]+)?\./)) {
			return src.replace(/\/thumb\/+([^/]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain === "images.larepubliquedespyrenees.fr") {
			return src.replace(/(\/[0-9a-f]{20,}\/+)golden\/+[0-9]+x[0-9]+\/+/, "$1");
		}

		if (domain === "img.aws.la-croix.com") {
			return src.replace(/_[0-9]+_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "getty.edu") {
			return src.replace(/\/media\/+images\/+web\/+[^/]*\/+/, "/media/images/web/download/");
		}

		if (domain_nosub === "yelpcdn.com") {
			return src.replace(/(\/b?photo\/+[^/]*\/+)(?:[0-9]+s|[a-z]+)(\.[^/.]*)(?:[?#].*)?$/,
							   "$1o$2");
		}

		if (domain_nosub === "rtbf.be" &&
			domain.match(/^(?:[^/]*\.)?static\./)) {
			return src.replace(/\/article\/+image\/+[0-9]+x[0-9]+\/+/, "/article/image/original/");
		}

		if (domain === "d5xydlzdo08s0.cloudfront.net") {
			return src.replace(/(\/media\/+[^/]*\/+[0-9]+\/+(?:[^/]*\/)?[^/]*)(?:__[A-Z]|_[0-9]+)(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "photos.geni.com") {
			return src.replace(/_[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "_original$1");
		}


		if (domain === "cdn.hipwallpaper.com") {
			return src.replace(/(:\/\/[^/]*\/)m\/+/, "$1i/");
		}

		if (domain === "photos.bandsintown.com" ||
			amazon_container === "bit-photos") {
			return src.replace(/\/thumb\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/large/$1");
		}

		if (domain === "d1dojdv0rym6r6.cloudfront.net") {
			return src.replace(/\/imgs\/+[a-z]+\/+([0-9]+_pic[0-9a-f]+\.[^/.]*)(?:[?#].*)?$/, "/imgs/large/$1")
		}

		if (domain_nowww === "mynetsit.ir") {
			return src.replace(/(:\/\/[^/]*\/).*?[?&]dt=([^/&]*).*?$/, "$1?di=$2");
		}

		if (domain === "cf.kizlarsoruyor.com" ||
			domain === "cf.girlsaskguys.com") {
			return src.replace(/(\/[a-z][0-9]+\/+[0-9a-f]+-[-0-9a-f]+)-m(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "static.framar.bg") {
			return src.replace(/\/thumbs\/+[0-9]+\/+lifestyle\/+/,
							   "/snimki/lstyle/");
		}

		if (domain === "life.tert.am" ||
			domain_nowww === "tert.am" ||
			domain_nowww === "xochuanime.online") {
			return add_full_extensions(src.replace(/(:\/\/[^/]*\/)(?:[^/]*\/+)?cache_image\/+(.*)[-_][0-9]+x[0-9]+(?:_[0-9]+[a-z]*)?(\.[^/.]*)(?:[?#].*)?$/,
									   "$1$2$3"));
		}

		if (domain_nowww === "woman.at") {
			return src
				.replace(/\/storage\/+[^/]*\/+file\/+/, "/storage/master/file/") // this doesn't seem to do any difference
				.replace(/(\/file\/+[0-9]+\/+)download\/+/, "$1"); // removes force-downloading
		}

		if (domain === "files.elfann.com") {
			return src.replace(/\/imagine\/+pictures_[0-9]+(?:_[0-9]+)?\/+/, "/pictures/");
		}

		if (domain === "cdn.axar.az") {
			return src.replace(/(:\/\/[^/]*\/[0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+)[0-9]+\/+([^/]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "media.milovana.com") {
			return src.replace(/(\/timg\/+(?:[0-9]+\/+[0-9]+\/+)?)tb_[a-z]+\/+/, "$1tb_l/");
		}

		if (domain === "uploads.spiritfanfiction.com") {
			return src.replace(/\/fanfics\/+thumbs\/+/, "/fanfics/historias/");
		}

		if (domain === "pic.homepornbay.com") {
			return src.replace(/(\/[0-9]+)s(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "cdn.funpic.us") {
			return src.replace(/(-[0-9]+)-[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "themanwiththehat.de") {
			return src.replace(/\/files\/+attachments\/+[0-9]+\/+/, "/files/attachments/");
		}

		if (domain_nosub === "motorsport.com" &&
			domain.match(/^cdn(?:-[0-9]*)?\./)) {
			return src.replace(/(\/images\/+[^/]*\/+[^/]*\/+)s[5897]\/+/, "$1s10/");
		}

		if (domain_nowww === "epicwallpapers.com" ||
			domain_nowww === "freshwallpapers.net" ||
			domain_nowww === "alliswall.com") {
			return src
				.replace(/\/imagecache\/+thumbnails\/+([0-9]+)\/+[0-9]+x[0-9]+\.[^/.]*(?:[?#].*)?$/,
						 "/file/$1/0x0/crop/")
				.replace(/(\/file\/+[0-9]+)\/+[0-9]+x[0-9]+\/+(?:crop|[0-9]+:[0-9]+)\/+/, "$1/0x0/crop/");
		}

		if (domain === "kookbang.dema.mil.kr") {
			return src.replace(/(\/upload\/+[0-9]{8}\/+)thumb[0-9]*\/+/, "$1");
		}

		if (domain === "dyaaf063c1cms.cloudfront.net") {
			return src.replace(/(:\/\/[^/]*\/[0-9a-f]+\/)(?:sq[0-9]+|medium)_/, "$1large_");
		}

		if (domain_nowww === "specialfruit.com") {
			return src
				.replace(/\/thumbnail\/+product[^/]*\/+/, "/thumbnail/productFull/")
				.replace(/\/thumbnail\/+inline\/+/, "/thumbnail/full/");
		}

		if (domain === "static.planetminecraft.com") {
			return src.replace(/(\/files\/.*_[0-9]+)_[a-z]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "games.renpy.org") {
			return src.replace(/\/media\/+screenshot\/+small\/+/, "/media/screenshot/");
		}

		if (domain_nowww === "700016.xyz") {
			return src.replace(/(\/files\/.*)_thumb(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "mytinyphone.com") {
			return src.replace(/(\/uploads\/.*\/)sm[0-9]*\/([0-9]+\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nowww === "wallcoo.net") {
			return src.replace(/\/s\/+([^/]*_[0-9]+)s(\.[^/.]*)(?:[?#].*)?$/, "/images/$1$2");
		}

		if (domain_nowww === "absfreepic.com") {
			return src.replace(/\/[a-z]+_photos\/+([^/]*)(?:[?#].*)?$/, "/original_photos/$1");
		}

		if (domain_nowww === "gif-favicon.com") {
			return src.replace(/\/images\/+download\.php.*?[?&]image=([^&]*).*?$/,
							   "/images/$1");
		}

		if (domain_nowww === "ametart.com") {
			return src.replace(/\/thumbs\/+([0-9]+\/+[0-9]+\/+)[0-9]+x_([^/]*)(?:[?#].*)?$/,
							   "/photos/$1$2");
		}

		if (domain_nowww === "nicepornphotos.com" ||
			domain_nowww === "suzisporn.com" ||
			domain_nowww === "youngporno.com" ||
			domain_nowww === "hoodtube.com" ||
			domain_nosub === "swapsmut.com" ||
			domain_nosub === "asianpornmovies.com") {
			return src.replace(/(\/galleries\/+[0-9]+\/+[0-9]+\/+[0-9a-f]{10,})-t(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nowww === "hostave2.net") {
			return src.replace(/(\/photo\/+[^/]*\/+[^/]*\/+)th\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "i.penisbot.com") {
			return src.replace(/\/tmb_([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nowww === "erowall.com") {
			newsrc = src
				.replace(/\/download_img\.php.*?[?&]dimg=([0-9]+).*?$/, "/wallpapers/original/$1.jpg")
				.replace(/\/wallpapers\/+[a-z]+\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/wallpapers/original/$1");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/\/wallpapers\/+[a-z]+\/+([0-9]+)\.[^/.]+(?:[?#].*)?$/);
			if (match)
				return {
					url: src,
					extra: {
						page: "https://erowall.com/w/" + match[1]
					}
				};
		}

		if (domain === "promo.mattsmodels.com" ||
			domain_nowww === "nextdoortease.com" ||
			domain_nowww === "amateurindex.com") {
			return src.replace(/(\/galleries\/+[^/]*\/+[^/]*\/+)thumbs\/+/, "$1");
		}

		if (domain === "ww3.aziani.com" ||
			domain_nowww === "clubglamour.net") {
			return src.replace(/(\/galleries\/+[^/]+\/+)thumbs\/+/, "$1");
		}

		if (domain === "galleries.allover30.com") {
			return src.replace(/\/Z[0-9]+\/+([^/]*)(?:[?#].*)?$/,
							   "/$1");
		}

		if (domain === "images.sxx.com" ||
			domain === "images.blazingmovies.com") {
			return src.replace(/\/thumbs\/+([^/]*)_tb(\.[^/.]*)(?:[?#].*)?$/,
							   "/$1$2");
		}

		if (domain === "ww2.aziani.com") {
			return src.replace(/\/t_([^/]*-[0-9]+\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain === "fhg.karupsow.com" ||
			domain === "fhg.karupsha.com") {
			return src.replace(/\/images\/+thumbs(-[^/]*\/)/, "/images/pics$1");
		}

		if (domain_nowww === "stockingvideos.com") {
			return src.replace(/\/images\/thumbs\/thumb([0-9]+\.[^/.]*)(?:[?#].*)?$/,
							   "/images/full/full$1");
		}

		if (domain === "media.virbcdn.com") {
			return src.replace(/\/cdn_images\/+resize_[0-9]+x[0-9]+\/+/, "/images/");
		}

		if (domain === "images.yuku.com") {
			return src.replace(/(\/image\/+[^/]*\/+[0-9a-f]+(?:\.[0-9]+)?)_[^/.]*(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (host_domain_nosub === "kpopping.com" && domain_nowww === "kpopping.com" && options.element) {
			if (/\/uploads\/+documents\//.test(src)) {
				if (options.element.tagName === "IMG" && options.element.classList.contains("img-fluid")) {
					if (options.element.parentElement.tagName === "A" && options.element.parentElement.href && /\/uploads\/+documents\//.test(options.element.parentElement.href)) {
						return options.element.parentElement.href;
					}
				}
			}
		}

		if (domain_nowww === "kpopping.com") {
			if (/\/bundles\/+app\/+img\//.test(src))
				return {
					url: src,
					bad: "mask"
				};


			/*
			  seems like they changed something, again.
			  every time i update this script, it seems like a few days later, something changes in the way their site works, breaking the fix i did for this site.
			  they're aware of this script (they linked to it at one point), so it could very well be intentional on their part.
			*/
			if (false) {
				return src
					.replace(/\/uploads\/+documents\/+[^/]+\/+(?:[0-9]*x){0,2}([^/]+?)(?:,[0-9]+){0,}(\.[^/]+?)(?:\.(?:keep|crop)\..*)?(?:[?#].*)?$/, "/uploads/documents/$1$2");
			}

			return src
					.replace(/\/uploads\/+documents\/+[^/]+\/+(?:[0-9]*x){0,2}([^/]+?)(\.[^/]+?)(?:\.(?:keep|crop)\..*)?(?:[?#].*)?$/, "/uploads/documents/$1$2");
		}

		if (domain_nosub === "tin247.com" &&
			domain.match(/^image[0-9]*\./)) {
			return src.replace(/\/picsmall\/+([0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+)[0-9]+\/+/,
							   "/$1");
		}

		if (domain === "assets.blogr.xxx" ||
			domain === "assets-old.blogr.xxx") {
			return src.replace(/\/posts\/+[^/]*\/+([0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+)/, "/posts/large/$1");
		}

		if (domain_nowww === "internationalcelebrityfeet.com") {
			return src.replace(/\/thumbs\/([^/]*)thumb(\.[^/.]*)(?:[?#].*)?$/, "/pics/$1$2");
		}

		if (domain === "content.gulte.com") {
			return src.replace(/(\/content\/+[0-9]{4}\/+[0-9]{2}\/+photos\/+events\/+[^/]*\/+)thumb\/+([^/]*)(?:[?#].*)?$/,
							   "$1normal/$2");
		}

		if (domain_nowww === "moda.ru") {
			return src.replace(/(\/files\/+user\/+(?:[0-9]{2}\/+){3}content\/+[0-9]+\/+)[0-9]+x[0-9]+x[0-9]+_([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nosub === "sodonsolution.org" &&
			domain.match(/^resource[0-9]*\./)) {
			return src.replace(/(\/images\/[0-9]{4}\/+[0-9]+\/+[0-9a-f]+\/+[0-9a-z]+)_[a-z](\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "youmediafanpage.akamaized.net") {
			return src.replace(/(\/gallery\/+[0-9a-f]+_[0-9a-f]+_)p[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1o$2");
		}

		if (domain_nosub === "hayatapp.com" &&
			domain.match(/^cdn-media-[0-9]+\./)) {
			return src.replace(/(\/uploads\/+backend_wysiwyg_asset\/+asset\/+)[a-z]+_([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "static.klix.ba") {
			return src.replace(/(\/media\/+images\/+vijesti\/+[0-9]+\.[0-9]+)(?:_[a-z]+)?(\.[^/.]*)(?:[?#].*)?$/,
							   "$1_mn$2");
		}

		if (domain === "img.nga.178.com") {
			return {
				url: src.replace(/(\/attachments\/+[^/]*_[0-9]{6}\/+[0-9]+\/+[^/]*)\.(?:thumb|medium)(?:_[a-z])?\.[^/.]*(?:[?#].*)?$/, "$1"),
				headers: {
					Referer: null
				},
				referer_ok: {
					same_domain_nosub: true
				}
			};
		}

		if (domain_nowww === "livesport.ru" ||
			domain === "cdn.livesport.ru") {
			return src.replace(/\/picture--[0-9]+(\.[^/.]*)(?:[?#].*)$/, "/picture$1");
		}

		if (domain_nowww === "vseprosport.ru") {
			return src.replace(/(\/images\/+uploads[0-9]+\/+)thumbs\/+[0-9]+x[0-9]+\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nowww === "evrimagaci.org") {
			return src.replace(/\/public\/+content_media\/+[0-9]+\/+/, "/public/content_media/");
		}

		if (domain_nowww === "jagranimages.com") {
			return src.replace(/(\/images\/+[0-9]{2}_[0-9]{2}_[0-9]{4}-[^-/_]+)_[a-z](\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "cdn.doctailieu.com") {
			return src.replace(/(\/images\/+[0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+[^/]*)-rs[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nosub === "cdn-redfin.com") {
			return src.replace(/(\/photo\/+[0-9]+\/+)[^/]*\/+([0-9]+\/+)[^/.]*\.((?:[0-9A-Z]+-)?[0-9A-Z]+_[0-9A-Z]+(?:_[0-9A-Z]+)?\.[^/.]*)(?:[?#].*)?$/,
							   "$1bigphoto/$2$3");
		}

		if (domain === "images.estately.net") {
			return src.replace(/_[0-9]+x[0-9]+[a-z]?(\.[^/.]*)(?:[?#].*)?$/,
							   "$1");
		}

		if (domain_nowww === "mjolbybillack.se") {
			return src.replace(/\/img\/+media\/+[sl]_/, "/img/media/");
		}

		if (domain_nowww === "mathworks.com") {
			return src.replace(/\/responsive_image\/+(?:[0-9]+\/+){5}cache\/+/, "/");
		}

		if (domain === "images.kz.prom.st") {
			return src.replace(/(:\/\/[^/]*\/[0-9]+)(?:_[wh][0-9]+){1,}(_[^/]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "treesale.sccd.org") {
			newsrc = urljoin(src, src.replace(/^[a-z]+:\/\/[^/]*\/thumbnail\.asp.*?[?&]file=([^&]*).*?$/,
											  "$1"), true);
			if (newsrc !== src)
				return newsrc;

			return src.replace(/(\/assets\/+images\/+[^/]+)_thumbnail(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nowww === "zive.cz") {
			return src.replace(/\/GetThumbNail\.aspx.*?[?&](id_file=[0-9]+).*?$/,
							   "/GetFile.aspx?$1");
		}

		if (domain === "hasbrouck.asu.edu") {
			regex = /(\/imglib\/+.*)_[a-z]+(\.[^/.]*)(?:[?#].*)?$/;
			return [
				src.replace(regex, "$1_lg$2"),
				src.replace(regex, "$1_web$2")
			];
		}

		if (domain_nowww === "parsstock.ir") {
			return {
				url: src.replace(/(:\/\/[^/]*\/)thumbs\/+([0-9]+\/)/, "$1photos/$2"),
				problems: {
					watermark: true
				}
			};
		}

		if (domain === "media.istockphoto.com") {
			return {
				url: src.replace(/(\/photos\/+[^/]*-id[0-9]+)\?s=[0-9]+x[0-9]+$/, "$1"),
				problems: {
					watermark: true
				}
			};
		}

		if (domain === "img.pixers.pics") {
			return src
				.replace(/:\/\/[^/]*\/pho_wat[(]s3:([^,)]*).*/, "://s3.pixers.pics/pixers/$1")
				.replace(/:\/\/[^/]*\/pho_wat[(]cms:([^,)]*).*/, "://s3.pixers.pics/cms/$1");
		}

		if (domain === "d7hftxdivxxvm.cloudfront.net") {
			return decodeURIComponent(src.replace(/^[a-z]+:\/\/[^/]*\/.*?[?&]src=([^&]*).*?$/, "$1"));
		}

		if (domain === "d32dm0rphc51dk.cloudfront.net") {
			return src.replace(/\/large(\.[^/.]*)(?:[?#].*)?$/, "/larger$1");
		}

		if (domain_nowww === "royalparks.org.uk") {
			return src.replace(/(\/_gallery\/+[^/]*\.[^/.]*)\/+[^/]*\.[^/.]*(?:[?#].*)?$/, "$1");
		}

		if (domain === "cdn.downtoearth.org.in") {
			return src.replace(/\/library\/+[a-z]+\/+([0-9]{4}-[0-9]{2}-[0-9]{2}\/)/, "/library/original/$1");
		}

		if (domain_nowww === "arborday.org") {
			return src.replace(/(\/images\/+hero\/+)[a-z]+\/+/, "$1");
		}

		if (domain === "assets.publishing.service.gov.uk") {
			return src.replace(/(\/image_data\/+file\/+[0-9]+\/+)s[0-9]+_([^/]*)(?:[?#].*)?$/, "$1$2");
		}


		if (domain === "media.swncdn.com") {
			return src.replace(/\.[0-9]+[wh]\.tn(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain === "media.wnyc.org") {
			return src.replace(/(:\/\/[^/]*\/i)\/+[0-9]+\/+[0-9]+\/+(?:[cl]\/[0-9]+\/)?/, "$1/raw/");
		}

		if (domain_nowww === "tagbrand.com") {
			return src.replace(/(\/uploads\/+[0-9]+\/+[0-9a-f]+)_[a-z](\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nosub === "7sur7.be" &&
			domain.match(/^static[0-9]*\./)) {
			return src.replace(/\/media_[a-z]+_([0-9]+\.[^/.]*)(?:[?#].*)?$/,
							   "/crop_$1");
		}

		if (domain_nowww === "suryaa.com") {
			return src.replace(/\/thumbnail[0-9]*\/+([^/]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain === "phapluatdansinh.phapluatxahoi.vn") {
			return src.replace(/\/upload\/+zoom\/+[0-9]+_[0-9]+\/+/, "/upload/");
		}

		if (domain === "assets.sport.ro") {
			return src
				.replace(/\/t_size[0-9]*\/+thumb_/, "/")
				.replace(/_size[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "diarioarapiraca.com.br" ||
			domain_nowww === "todosegundo.com.br") {
			return urljoin(src, decodeuri_ifneeded(src.replace(/^[a-z]+:\/\/[^/]*\/thumbs\.php.*?[?&]imagem=([^&]*).*?$/, "$1")), true);
		}

		if (domain === "static.ilike.com.vn") {
			return src.replace(/\/thumb\/+s[0-9]+\//, "/thumb/origin/");
		}

		if (domain === "b.njus.me") {
			return src.replace(/\/[a-z]_([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nowww === "wallpaperstream.com") {
			return src.replace(/\/thumbnails\/(.*)_thumb(\.[^/.]*)(?:[?#].*)?$/,
							   "/full/$1$2");
		}

		if (domain_nowww === "wallpapersite.com" &&
			options && options.cb && options.do_request) {
			id = src.replace(/.*\/images\/+pages\/+[^/]*\/+([0-9]+)\.[^/.]*(?:[?#].*)?$/, "$1");
			if (id !== src) {
				options.do_request({
					url: "https://wallpapersite.com/anime/-" + id + ".html",
					method: "GET",
					onload: function(resp) {
						if (resp.readyState === 4) {
							var match = resp.responseText.match(/<a\s+href=["'](\/images\/wallpapers\/[^'"]*)["']\s+class=["']?original/);
							if (match) {
								options.cb(urljoin(src, match[1], true));
							} else {
								options.cb(null);
							}
						}
					}
				});

				return {
					waiting: true
				};
			}
		}

		if (domain_nosub === "record.pt" ||
			domain_nosub === "flash.pt" ||
			domain_nosub === "cmjornal.pt" ||
			domain_nosub === "destakjornal.com.br" ||
			domain_nosub === "maxima.pt" ||
			src.match(/^[a-z]+:\/\/cdn[0-9]*\.[^/]*\/+images\/+[0-9]{4}-[0-9]{2}\/+img_[0-9]+x[0-9]+\$[0-9]{4}(?:_[0-9]{2}){5}_[0-9]+\.[^/.]*(?:[?#].*)?$/)) {
			return src.replace(/(\/images\/+[0-9]{4}-[0-9]{2}\/+)img_[0-9]+x[0-9]+[$]/, "$1OriginalSize$");
		}

		if (domain_nowww === "latestwall.com") {
			newsrc = src
				.replace(/\/imagecache\/+thumb[a-z]+\/+/, "/images/wallpaper/")
				.replace(/\/main-image\/+cache\/+([^/]*)\/+[0-9]+(?:\/+[0-9]+)(?:[?#].*)?$/, "/images/wallpaper/$1");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/\/download\/+[0-9]+~[0-9]+~([^/]*)(?:[?#].*)?$/, "/images/wallpaper/$1.jpg");
			if (newsrc !== src)
				return add_full_extensions(newsrc);
		}

		if (domain_nosub === "hdxwallpaper.com" && domain.match(/^orig[0-9]*\./)) {
			return {
				url: src.replace(/-[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1"),
				headers: {
					Referer: "https://hdxwallpaper.com"
				}
			};
		}

		if (domain === "cacheimg.bonsante.fr") {
			return decodeuri_ifneeded(src.replace(/^[a-z]+:\/\/[^/]*\/i\.php.*?[?&]img=([^&]*).*?$/, "$1"));
		}

		if (domain_nowww === "generations.fr") {
			return src.replace(/\/thumb\/+[0-9]+x[0-9]+_([^/]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain === "images.gsp.ro") {
			return src.replace(/\/thumbs\/+thumb_[0-9]+_x_[0-9]+\//, "/imagini/");
		}

		if (domain === "cacheimg.gsp.ro") {
			return src.replace(/(\/autocrop\/+.*?)(?:[?#].*)?$/, "$1?width=0&height=0");
		}

		if (domain === "tt.inf.ua") {
			return src.replace(/(\/files\/+upload\/+(?:[0-9a-f]{2}\/+){3}[0-9a-f]+)\.[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nosub === "ckm.pl" && domain.match(/^static[0-9]*\./)) {
			return src.replace(/\/\.thumbnails\/+[0-9]+x[0-9]+\/+/, "/");
		}

		if (domain_nowww === "zurnal.mk" ||
			domain_nowww === "harpersbazaar.rs" ||
			domain_nowww === "moviegoers.me") {
			return urljoin(src, decodeuri_ifneeded(src.replace(/^[a-z]+:\/\/[^/]*\/(?:images\/+thumbs\/+)?image\.php\/+[^/]*\?(?:.*?&)?image=([^&]*).*?$/, "$1")), true);
		}

		if (domain === "cde.laprensa.e3.pe") {
			return src.replace(/(\/ima\/+(?:[0-9]\/+){5}[0-9]+)\/+[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "myclip.vn" && domain.match(/^static[0-9]*\./)) {
			return src.replace(/(\/image[0-9]*\/+[0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+[0-9]{2}\/+[0-9a-f]+\/+[-0-9a-f]+)_[0-9]+_[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nowww === "elsalvadortimes.com") {
			return src.replace(/\/asset\/+[^/]*\/+media\/+/, "/media/");
		}

		if (domain === "assets.media-platform.com") {
			return src.replace(/(\/images\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9]{8}_[^/]*)-[wh][0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nowww === "noticiastudoaqui.com") {
			return decodeuri_ifneeded(src.replace(/^[a-z]+:\/\/[^/]*\/tim\.php.*?[?&]src=([^&]*).*?$/, "$1"));
		}

		if (domain === "images.buzzerie.com") {
			return src.replace(/_[0-9]+x[0-9wh]+(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "cocacolabrasil.com.br") {
			return src.replace(/\.rendition\.[0-9]+\.[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain === "img.uodoo.com") {
			return src.replace(/;,.*/, "");
		}

		if (domain_nowww === "bumm.sk") {
			return src.replace(/(\/uploads\/+news\/+[0-9]+\/+[0-9]+\/+)[0-9]+x[0-9]+\/+/, "$1");
		}

		if (domain === "multimedia.larepublica.pe" ||
			domain === "prod.media.larepublica.pe" ||
			domain === "prod.presets.larepublica.pe") {
			return src.replace(/(:\/\/[^/]*\/)(?:[0-9]+)?x(?:[0-9]+)?\//, "$1x/");
		}

		if (domain_nowww === "wallpaperfm.com") {
			return src.replace(/\/img\/+[a-z]+\/+/, "/img/original/");
		}

		if (domain === "dux7id0k7hacn.cloudfront.net") {
			return src.replace(/(\/cmi\/+(?:[0-9]\/+){4}[0-9]+\/+[^/]*\.[^/.]*)\/.*$/, "$1");
		}

		if (domain_nowww === "printel.biz" ||
			domain_nowww === "steklozerkalo116.ru" ||
			domain_nowww === "studio-14.ru") {
			return src.replace(/\/thumb\/.*(\/d\/+[^/]*\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "10wallpaper.com" &&
			options && options.cb && options.do_request) {
			var id = src.replace(/.*\/wallpaper\/+[^/]*\/+[0-9]+\/+([^/]*)_[^./_]+\.[^/.]*(?:[?#].*)?$/, "$1");
			if (id !== src) {
				options.do_request({
					url: "https://www.10wallpaper.com/view/" + id + ".html",
					method: "GET",
					onload: function(resp) {
						if (resp.readyState === 4) {
							var match = resp.responseText.match(/>Original Resolution: <a\s+href=["'](\/wallpaper\/[^'"]+)/);
							if (match) {
								options.cb(urljoin(src, match[1], true));
							} else {
								options.cb(null);
							}
						}
					}
				});

				return {
					waiting: true
				};
			}
		}

		if (domain_nosub === "wallpapersok.com" && domain.match(/^cdn[0-9]*\./)) {
			return src.replace(/\/(?:crop\/+)?(uploads\/+picture\/+[0-9]{3}\/+[0-9]{3}\/+[0-9]+\/+)(?:thumbs_)?([^/?]*)(?:[?#].*)?$/,
							   "/$1$2");
		}

		if (domain_nowww === "woweiquan.com") {
			return decodeuri_ifneeded(src.replace(/^[a-z]+:\/\/[^/]*\/Service\/+Img\/+wx.*?[?&]url=([^&].*)?.*?$/, "$1"));
		}

		if (domain === "s.kaskus.id") {
			return {
				url: src.replace(/(:\/\/[^/]*\/+)[rc][0-9]+x[0-9]+\/+/, "$1"),
				headers: {
					Referer: "https://www.kaskus.co.id"
				}
			};
		}

		if (domain === "dl.kaskus.id") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+([^/]*\.[^/]*\/)/, "$1");
			if (newsrc !== src)
				return add_http(newsrc);

			return {
				url: src,
				headers: {
					Referer: ""
				}
			}
		}

		if (domain_nowww === "shofona.net") {
			return src.replace(/\/cached_uploads\/+resize\/+[0-9]+\/+[0-9]+\/+/, "/cached_uploads/full/");
		}

		if (domain_nowww === "bayanbox.ir") {
			return src.replace(/\/preview\/+([0-9]+)\/+/, "/view/$1/");
		}

		if (domain_nowww === "guiadasemana.com.br") {
			return src
				.replace(/(\/contentFiles\/+image\/+)opt_[^/]*\/+/, "$1")
				.replace(/(\/contentFiles\/+system\/+pictures\/+[0-9]{4}\/+[0-9]+\/+[0-9]+\/+)[a-z]+\/+/, "$1original/");
		}

		if (domain === "img.arabstoday.net") {
			return src.replace(/(\/[0-9]{4}\/+[0-9]{2}\/+)[a-z]+\/+/, "$1normal/");
		}

		if (domain_nowww === "nodud.com") {
			return src.replace(/\/sized\/+(source\/+.*\/[0-9a-z]+)-[^/]*(\.[^/.]*)(?:[?#].*)?$/, "/$1$2");
		}

		if (domain === "g.acdn.no") {
			newsrc = src.replace(/.*?\/API\/+dynamic\/+r1\/+external\/+[^/]*\/+[^/]*\/+(https?.*?)(?:[?#].*)?$/, "$1");
			if (newsrc !== src)
				return decodeURIComponent(newsrc);
		}

		if (domain_nowww === "pensorosa.com") {
			return src.replace(/\/upload\/+[a-z]+\/+/, "/upload/images/");
		}

		if (domain === "smlycdn.akamaized.net") {
			return src.replace(/\/products\/+[^/]*\/+[0-9a-f]+\/+([0-9a-f]+)(\.[^/.]*)(?:[?#].*)?$/,
							   "/data/product2/2/$1_l$2");
		}

		if (domain === "cdn.aizel.ru") {
			return src.replace(/\/i\/+[0-9]+x[0-9]+\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/i/$1");
		}

		if (domain_nowww === "trelisecooperonline.com") {
			return src.replace(/\/user\/+images\/+([0-9]+)_[0-9]+_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "/user/images/$1$2");
		}

		if (domain_nowww === "thairomances.com") {
			return src.replace(/(\/userfiles\/+[^/]*\/+[0-9]+\/+[0-9]+-)[0-9]+x[0-9]+\./, "$1full_size.");
		}

		if (domain === "cn.opendesktop.org" ||
			domain === "cdn.pling.com") {
			return src.replace(/\/cache\/+[0-9]+x[0-9]+(?:-[a-z0-9]+)?\/+img\/+/, "/img/");
		}

		if (domain_nowww === "3dnews.ru") {
			return src
				.replace(/(\/assets\/.*\/)[a-z]+\.([0-9A-Z]+)\.[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2$3")
				.replace(/\/z\/.*?\/([0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+[0-9a-f]+\/+[0-9a-fA-Z]+\.[^/.]*)(?:\/+[0-9]+)?(?:[?#].*)?$/,
						 "/assets/external/galleries/$1");
		}

		if (domain === "mix.tn.kz") {
			return src.replace(/\/thumb_[a-z]+\/+(photo_[0-9]+\.[^/.]*)(?:[?#].*)?$/,
							   "/$1");
		}

		if (domain === "st.overclockers.ru") {
			return src.replace(/\/c\/+[0-9]+\/+[0-9]+\/+images\//, "/images/");
		}

		if (domain_nosub === "kpcdn.net" &&
			domain.match(/^s[0-9]*\.stc\.all\./)) {

			return src.replace(/^[a-z]+:\/\/[^/]*\/(share\/+i\/+[0-9]+\/+[0-9]+)\/+(?:[^/]*\.[^/.]*)?(?:[?#].*)?$/, "https://kp.ru/$1");
		}

		if (domain_nowww === "kp.ru") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/(share\/+i\/+[0-9]+\/+[0-9]+)\/+(?:[^/]*\.[^/.]*)?(?:[?#].*)?$/, "https://kp.ru/$1");
		}

		if (domain_nowww === "oplace.ru") {
			return src.replace(/\/images\/+photos\/+[a-z]+\/+([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/, "/images/photos/$1");
		}

		if (domain_nosub === "rightinthebox.com" && domain.match(/^li/)) {
			newsrc = src.replace(/(:\/\/[^/]*\/images\/+)[0-9]+x[0-9]+\//, "$1v/");
			if (newsrc !== src) {
				return {
					url: newsrc,
					problems: {
						watermark: true
					}
				};
			}

			return {
				url: src,
				headers: {
					Origin: "https://www.lightinthebox.com",
					Referer: "https://www.lightinthebox.com/"
				}
			};
		}

		if (domain === "img.esportnews.gg") {
			return src.replace(/(\/pictures\/+content\/+[0-9]+\/+[0-9]+)_[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "img.tyt.by") {
			return src.replace(/(:\/\/[^/]*\/)[0-9]+x[0-9]+[a-z]\/+/, "$1");
		}

		if (domain === "vi.ill.in.ua") {
			return src.replace(/(:\/\/[^/]*\/[a-z]\/+)[0-9]+x[0-9]+\/+/, "$10x0/");
		}

		if (domain_nosub === "vsco.co" && domain.match(/^image(?:-aws)?.*/) ||
			domain === "im.vsco.co") {
			newsrc = src.replace(/(\/[0-9a-f]{20,}\/+)[0-9]+x[0-9]+\/+(?:[0-9a-f]+\/+)?(vsco_?[0-9a-f]+\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/^[a-z]+:\/\/[^/]+\/+(?:(?:aws-[^/]+\/+)?[0-9a-f]{6}\/+[0-9]+|[0-9]+\/+[0-9a-f]{10,})\/+([0-9a-f]{20,})\/+/);
			id = match[1];
			if (match && options && options.force_page && options.cb && options.do_request) {
				options.do_request({
					url: "https://vsco.co/vsco/media/" + id,
					method: "GET",
					onload: function(resp) {
						if (resp.readyState !== 4)
							return;

						if (resp.status !== 200)
							return options.cb(null);

						var obj = {
							url: src,
							extra: {
								page: resp.finalUrl
							}
						};

						if (false) {
							var match = resp.responseText.match(/<meta\s+(?:data-rh=\S+\s+)?property=["']og:description["']\s+content=["'](.*?)["']/);
							if (match) {
								obj.extra.caption = decode_entities(match[1]);
							} else {
								console_error("Unable to find caption", resp);
							}
						} else {
							try {
								var match = resp.responseText.match(/window\.__PRELOADED_STATE__\s*=\s*({.*?});?\s*<\/script/);
								if (match) {
									var json = JSON_parse(match[1]);
									obj.extra.caption = json.medias.byId[id].media.description;
								}
							} catch (e) {
								console_error(e);
							}
						}

						return options.cb(obj);
					}
				});

				return {
					waiting: true
				};
			}
		}

		if (domain_nowww === "screenbeauty.com") {
			return src.replace(/\/image\/+compress\/+/, "/image/wallpapers/");
		}

		if (domain === "e.radio-studio92.io") {
			return src.replace(/(:\/\/[^/]*\/)[a-z]+\/+([0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+[0-9]+(?:_[0-9]+)?\.[^/.]*)(?:[?#].*)?$/,
							   "$1xlarge/$2");
		}

		if (amazon_container === "topdesk") {
			return src.replace(/\/((?:[0-9]{3}\/+){3})[a-z]+\/+([^/]*)(?:[?#].*)?$/, "/$1original/$2");
		}

		if (domain === "static.hdw.eweb4.com") {
			return src.replace(/\/media\/+wp_[0-9]+\/+/, "/media/wallpapers/");
		}

		if (domain_nowww === "sitioandino.com.ar") {
			return src.replace(/(\/files\/+image\/+[0-9]+\/+[0-9]+\/+[0-9a-f]+)_[0-9]+_[0-9]+!(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "demo.brainymore.com") {
			return src.replace(/\/thumbs\/+([0-9]+)_[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "/$1$2");
		}

		if (domain_nosub === "1freewallpapers.com" && domain.match(/^data[0-9]*\./)) {
			return src
				.replace(/(:\/\/[^/]*\/)download\/+([^/]+)-[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1original/$2$3")
				.replace(/(:\/\/[^/]*\/)[a-z]+\/+([^/]*\.[^/.]*)(?:[?#].*)?$/, "$1original/$2");
		}

		if (domain_nowww === "downloadwallpapers.info") {
			return src.replace(/(:\/\/[^/]*\/)(?:thumbs|preview|dl\/+[^/]*)\/+([0-9]{4})\/+/, "$1dl/o/$2/");
		}

		if (domain_nowww === "miyanali.com") {
			return src.replace(/(:\/\/[^/]*\/)patch2image\.php.*?[?&]patch=(usr\/[^&]*).*?$/, "$1$2");
		}

		if (domain_nowww === "all4desktop.com") {
			return src.replace(/\/data_images\/+[0-9]+(?:%20|\s)*x(?:%20|\s)*[0-9]+\/+/, "/data_images/original/");
		}

		if (amazon_container === "hsdreams1") {
			return src.replace(/(\/[0-9]{4}\/+[0-9]{2}\/+)[a-z]+(\/+[0-9a-f]+\.[^/.]*)(?:[?#].*)?$/, "$1big$2");
		}

		if (domain === "cdn.quotesgram.com") {
			return src.replace(/(:\/\/[^/]*\/)[a-z]+\/+/, "$1img/");
		}

		if (domain_nowww === "razdachi.net" ||
			domain_nowww === "wallpaperfuel.com") {
			return src.replace(/(:\/\/[^/]*\/)thumbnail\/+/, "$1wallpaper/");
		}

		if (domain_nowww === "zazzybabes.com") {
			return src.replace(/\/(girls)\/+([^/]*\/+)thumb-([^/]*\.[^/.]*)(?:[?#].*)?$/, "/$1/$2$3");
		}

		if (domain_nosub === "mtime.cn" && domain.match(/^img[0-9]*\./)) {
			return {
				url: src.replace(/(\.[0-9]+)(?:_[0-9]+(?:X[0-9]+){0,2})?(\.[^/.]*)(?:[?#].*)?$/, "$1_0$2"),
				head_wrong_contenttype: true
			};
		}

		if (domain === "images.gutefrage.net") {
			return src.replace(/(\/bilder\/+[^/]*\/+[0-9]+_)[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1original$2");
		}

		if (domain === "v.angel-porns.com" ||
			domain_nowww === "sexgirlspics.com") {
			return src.replace(/\/tnx?(sn[0-9]+\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain === "cdn15764270.ahacdn.me") {
			return src.replace(/(\/galleries\/+[^/]*\/+[0-9a-f]+\/+)[0-9]+x[0-9]+(\/+[^/]*\.[^/.]*)(?:[?#].*)?$/,
							   "$1origin$2");
		}

		if (domain === "images.nubilefilms.com") {
			return src.replace(/\/photos\/+tn\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/photos/$1");
		}

		if (domain === "g.bnrslks.com") {
			return src.replace(/\/tn\/+([^/]*\.[^/.]*)(?:[?#].*)?$/, "/full/$1");
		}

		if (domain_nosub === "joymii.com" && domain.match(/^n[0-9]*\./)) {
			return src.replace(/\/thumbs-[0-9]+x[0-9]+\//, "/fhg/");
		}

		if (domain_nowww === "famechain.com") {
			return src.replace(/\/images\/+resized-image\/+[^/]*\/+([0-9]+)(?:[?#].*)?$/, "/images/image/$1");
		}

		if (domain_nosub === "autoevolution.com" &&
			domain.match(/^s[0-9]*\.cdn\./)) {
			return src.replace(/\/images\/+news-gallery-[0-9]+x\/+([^/]*)-thumbnail(_[0-9]+\.[^/.]*)(?:[?#].*)?$/,
							   "/images/news/gallery/$1$2");
		}

		if (domain === "pics.haircutshairstyles.com") {
			return src.replace(/\/img\/+photos\/+thumbs\/+([0-9]{4}-[0-9]{2})\/+thumb_/,
							   "/img/photos/full/$1/");
		}

		if (domain === "clzmovies.r.sizr.io") {
			return src.replace(/(:\/\/[^/]*\/core\/+[a-z]+\/+)[a-z]+(\/+[0-9a-f]{2}\/+[0-9a-f]{2}_[^/]*)(?:[?#].*)?$/,
							   "$1original$2");
		}

		if (domain === "images.firstpost.com") {
			return src.replace(/\/fpimages\/+[0-9]+x[0-9]+\/+[a-z]+\/+[a-z]+\/+([0-9]{4}\/+[0-9]{2}\/+)/,
							   "/wp-content/uploads/$1");
		}

		if ((domain_nowww === "hustlebunny.com" && string_indexof(src, "/content/") >= 0) ||
			(domain === "24.p3k.hu" && string_indexof(src, "/uploads/") >= 0) ||
			(domain === "static.soltana.ma" && string_indexof(src, "/uploads/") >= 0) ||
			(domain_nowww === "cosmopolitan.hu" && src.match(/\/app\/+uploads\//)) ||
			(domain_nowww === "pedestrian.tv" && string_indexof(src, "/content/") >= 0)) {
			return src
				.replace(/_wm(\.[^/.]*)(?:[?#].*)?$/, "$1")
				.replace(/-[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1")
				.replace(/-e[0-9]{8,}(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "vanishingtattoo.com") {
			return src.replace(/_thumbnails\/+([^/]*)_th(_[0-9]+\.[^/.]*)(?:[?#].*)?$/, "_large/$1$2");
		}

		if (domain_nowww === "wallpaperama.com") {
			return src.replace(/(\/post-images\/+wallpapers\/+[a-z]+\/+[^/]*)-t[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nowww === "hashtaglegend.com") {
			return src.replace(/(\/storage\/+app\/+media\/+)cropped-images\/+([^/]*)(?:-[0-9]+){4}-[0-9]{8,}(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2$3");
		}

		if (domain_nowww === "anime-planet.com") {
			return src.replace(/(\/images\/+[a-z]+\/+(?:[a-z]+\/+)?)thumbs\/+/, "$1");
		}

		if (domain === "di2ponv0v5otw.cloudfront.net" ||
			domain === "dtpmhvbsmffsz.cloudfront.net") {
			return src.replace(/(\/[a-z]+\/+[0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+[0-9a-f]+\/+)[a-z]_([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if ((domain_nosub === "freeasianpics.net" && domain.match(/^cdn[0-9]*\./)) ||
			domain === "th.sexhotpictures.net") {
			return src.replace(/\/(?:_[a-z]_|n)([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (amazon_container === "nudiez-production" ||
			domain === "d2c4tvdc1w38dj.cloudfront.net") {
			return src.replace(/(\/photos\/(?:[-0-9a-f]{3}\/+){3})[a-z]+\/+([^/?]+)(?:[?#].*)?$/, "$1original/$2");
		}

		if (domain_nosub === "avdbs.com" && /^i[0-9]*\./.test(domain)) {
			return src.replace(/^[a-z]+:\/\/[^/]*\//, "https://s3.ap-northeast-2.amazonaws.com/img.avdbs.com/");
		}

		if (amazon_container === "img.avdbs.com" ||
			domain === "img.avdbs.com" ||
			(domain_nosub === "avdbs.com" && /^i[0-9]*\./.test(domain))) {
			regex = /(\/[0-9]+_[0-9]+_)[sr](\.[^/.]*)(?:[?#].*)?$/;
			return [
				src.replace(regex, "$1o$2"),
				src.replace(regex, "$1r$2")
			];
		}

		if (domain === "cdn.lengmenjun.com") {
			return src.replace(/!lengmenjun-[0-9]+(?:[?#].*)?$/, "!lengmenjun");
		}

		if (amazon_container === "gallerist") {
			return src.replace(/(\/products\/+[0-9]+\/+)[a-z]+\/+([^/]*)(?:[?#].*)?$/, "$1original/$2");
		}

		if (domain === "dingo.care2.com") {
			return src.replace(/(\/pictures\/+.*\/[0-9]+-[0-9]+-)[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1orig$2");
		}

		if (domain === "newsinteractives.cbc.ca") {
			return src.replace(/(\/images\/+[^/]*\/+)_[a-z]+\/+/, "$1");
		}

		if (domain_nowww === "clipartmax.com") {
			return src.replace(/(:\/\/[^/]*\/(?:png|jpg|jpeg|gif)\/+)[a-z]+\/+/, "$1full/");
		}

		if (domain_nowww === "createphotocalendars.com") {
			return src.replace(/(\/studio\/+generated[a-zA-Z]+\/+[^/]*\/+)thumbs\/+/i, "$1/");
		}

		if (domain_nowww === "larutadelsorigens.cat") {
			return src.replace(/\/filelook\/+[a-z]+\/+/, "/filelook/full/");
		}

		if (domain === "wwcdn.weddingwire.com") {
			return src.replace(/(\/wedding\/+[0-9]+_[0-9]+\/+[0-9]+\/+)thumbnails\/+[0-9]+x[0-9]+_/, "$1");
		}

		if (domain_nowww === "weddingandpartynetwork.com") {
			return src.replace(/\/gallery\/+photos\/+thumb\/+([0-9]+)(?:[?#].*)?$/, "/gallery/photos/$1");
		}

		if (domain_nowww === "sandinyoureye.co.uk") {
			return src.replace(/\/files\/+gallery\/+_[0-9]+x[0-9]+_[^/]*\/+/, "/files/gallery/");
		}

		if (domain_nowww === "foto4ka.com") {
			return src.replace(/(:\/\/[^/]*\/)thumbnails\/+([0-9]+\/+)/, "$1wallpapers/$2");
		}

		if (domain_nowww === "showwallpaper.com") {
			return src.replace(/(\/wallpaper\/+[0-9]+\/+)tn_([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "cduniverse.ws" && domain.match(/^c[0-9]*\./)) {
			return src.replace(/(\/resized[a-z]*\/+)[0-9]+x[0-9]+\/+/, "$19000x9000/");
		}

		if (domain === "img.mandarake.co.jp") {
			return src.replace(/(\/webshopimg\/+[0-9]+\/+[0-9]+\/+[0-9]+\/+[0-9]+\/+)[a-z]_([0-9]+\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "d2adpaynhf6x63.cloudfront.net") {
			return src.replace(/\/image\/+thumbs\/+thumb[0-9]+_/, "/image/");
		}

		if (domain === "static-cdn.jtvnw.net") {
			return src.replace(/(\/emoticons\/+v1\/+[0-9]+\/+)[012][0-9.]*(?:[?#].*)?$/, "$13.0");
		}

		if (domain_nowww === "patch.com") {
			newsrc = src.replace(/(\/img\/+cdn[0-9]*\/+.*\/+(?:[0-9]{4}\/+[0-9]{2}|[0-9]{8}\/+[0-9]+)\/+)(?:T[0-9]+x[0-9]+\/+)?((?:styles\/+[^/]+\/+public\/+[^/]+\/+)?[^/]*?)(?:[?#].*)?$/,
								 "$1$2");
			if (newsrc !== src)
				return newsrc;

			return src.replace(/(\/img\/+cdn\/+users\/+[0-9]+\/+[0-9]{4}\/+[0-9]{2}\/+)([^/]*)(?:[?#].*)?$/, "$1raw/$2");
		}

		if (domain_nowww === "wsws.org") {
			return src.replace(/(\/asset\/+[-0-9a-f]+I?\/+[^/]*?)(?:[?#].*)?$/, "$1");
		}

		if (domain === "media.apnarm.net.au") {
			return src.replace(/(\/media\/+images\/+[0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+[^/]*)_f?c?t[0-9]+(?:x[0-9]+)?(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "images.forwardcdn.com") {
			return src.replace(/\/image\/+[0-9]+x(?:[0-9]+)?\/+[a-z]+\/+(?:[a-z]+\/+)?images\/+(?:cropped\/+)?/, "/image/images/");
		}

		if (domain_nowww === "goodnet.org") {
			return src.replace(/\/photos\/+[0-9]+x[0-9]+\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/photos/$1");
		}

		if (domain_nowww === "pinknews.co.uk") {
			return src.replace(/(\/images\/+[0-9]{4}\/+[0-9]{2}\/+[^/]*)[-_][0-9]+x[0-9]+(?:_[^/]*?)?(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "static-sothebys-production-2.gtsstatic.net") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/images\/+v_[0-9_]+\/+localimagereader\.ashx.*?[?&]imageurl=([^&]*).*?$/,
								 "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nosub === "boltdns.net" && domain.match(/^cf-images\./)) {
			return {
				url: src,//.replace(/\/[-0-9]+x[-0-9]+\/+match\/+/, "/-1x-1/match/"),
				can_head: false // 405, method not allowed
			};
		}

		if (domain === "expo.advance.net") {
			return src.replace(/(\/img\/[0-9a-f]+\/+)(?:(?:height|width)[0-9]+\/+)?([a-z0-9]{3}_[^/.]+\.[^/.]*)(?:[?#].*)?$/, "$1orig/$2");
		}

		if (domain === "media.resources.festicket.com") {
			return src.replace(/\/image\/+.*?\/+www\/+/, "/image/www/");
		}

		if (domain === "static.getindiebill.com") {
			return src.replace(/(:\/\/[^/]*\/)[^/]*=\/[0-9]+x[0-9]+\/+(?:smart\/+)?([-0-9a-f]{10,}\/+)/, "$1$2");
		}

		if (domain_nowww === "seatgeek.com") {
			return src.replace(/(\/images\/+performers-landscape\/+[^/]*-[0-9a-f]+\/+[0-9]+\/+)[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$10x0$2");
		}

		if (domain === "vid.alarabiya.net") {
			return src.replace(/(\/images\/+[0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+[-0-9a-f]+\/+[-0-9a-f]+)_[^/]*_[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if ((domain === "cdn.onebauer.media" ||
			 domain === "media.planetradio.co.uk" ||
			 domain === "cdn.graziadaily.co.uk" ||
			 domain === "cdn.closeronline.co.uk" ||
			 domain === "cdn.planetradio.co.uk" ||
			 domain === "cdn.heatworld.com") &&
			src.match(/^[a-z]+:\/\/[^/]*\/+one\//)) {
			return src.replace(/(\/[^/?]*)(?:\?.*)?$/, "$1?resize=atrophy");
		}

		if (domain === "assets.planetradio.co.uk") {
			return src.replace(/\/track\/+[0-9]+x[0-9]+\/+/, "/track/");
		}

		if (domain_nowww === "realestate.com.au") {
			return src.replace(/\/blog\/+images\/+[^/]*\/+([0-9]{4})\/+/, "/blog/wp-content/uploads/$1/");
		}

		if (domain_nowww === "kanke365.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/[^/]*\/+imgBridge\.php.*?[?&]url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "i.obozrevatel.com") {
			return src.replace(/(:\/\/[^/]*\/)([0-9]{4}\/+[0-9]{1,2}\/+[0-9]{1,2}\/+[^/?]*)(?:[?#].*)?$/,
							   "$1gallery/$2");
		}

		if (domain_nowww === "stuttgarter-nachrichten.de") {
			return src.replace(/(:\/\/[^/]*\/media\.media\.[-0-9a-f]+\.)(?:[^/]*\.)?([^/.]*)(?:[?#].*)?$/,
							   "$1original.$2");
		}

		if (domain_nosub === "360buyimg.com" &&
			domain.match(/^(?:img[0-9]*|m)\./)) {
			return src
				.replace(/(:\/\/[^/]*\/)(?:[a-z0-9]+\/+s[0-9]+x[0-9]+_jfs|popWaterMark\/+jfs)\//, "$1imgzone/jfs/")
				.replace(/![^/]*(?:[?#].*)?$/, "");
		}

		if (domain === "c.actve.net") {
			return src.replace(/(:\/\/[^/]*\/)([0-9])([0-9])([0-9])([0-9]+)(?:\/[^/]*)?(?:[?#].*)?$/,
							   "$1original/$2/$3/$4/$2$3$4$5.jpg");
		}

		if (domain === "resources.wimpmusic.com") {
			return src.replace(/(\/images\/.*\/)[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1origin$2");
		}

		if (domain === "improxy.starmakerstudios.com") {
			return src.replace(/\/tools\/+im\/+[0-9]+\/+/, "/tools/im/0/");
		}

		if (domain === "pl.scdn.co") {
			return src.replace(/:\/\/[^/]*\/images\/+pl\/+default\/+([0-9a-f]+)(?:[?#].*)?$/,
							   "://i.scdn.co/image/$1");
		}

		if (domain === "s.mxmcdn.net") {
			return src.replace(/(\/images-storage\/.*\/[0-9]+_)[0-7][0-9][0-9]_[0-7][0-9][0-9](\.[^/.]*)(?:[?#].*)?$/,
							   "$1800_800$2");
		}

		if (domain_nowww === "radiopotok.ru") {
			return src.replace(/\/article\/+s_([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/article/$1");
		}

		if (domain_nowww === "becteroradio.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/resizeimage\/+index\.php.*?[?&]url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "electronicamx.net") {
			return src.replace(/(\/images\/+[0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+[^/]*)_medium(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nowww === "tophit.ru") {
			return src.replace(/\/i\/+[0-9]+x[0-9]+\/+/, "/i/");
		}

		if (domain_nosub === "zvooq.com" && domain.match(/^cdn[0-9]*\./) &&
			src.match(/:\/\/[^/]*\/pic\?/)) {
			return src.replace(/([?&]size=)[0-9]+x[0-9]+/, "$10x0");
		}

		if (domain === "d3us2i0tqwa7m7.cloudfront.net") {
			return src.replace(/(\/[0-9]+_res_)[^/._]*(\.[^/.]*)(?:[?#].*)?$/, "$1original$2");
		}

		if (domain_nowww === "theaudiodb.com" && string_indexof(src, "/images/") >= 0) {
			return src.replace(/(\/[a-z]+[0-9]+\.[^/.]+)(?:\/[a-z]+)(?:[?#].*)?$/, "$1");
		}

		if (domain_nosub === "gaoshouyou.com" && domain.match(/^rs\.[0-9]*\./)) {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/gsy\.php.*?[?&]img=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "mirrormedia.com.tw") {
			return src.replace(/(\/assets\/+images\/+[0-9]+-[0-9a-f]+)-mobile(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "twpriceget.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/img-[^/]*.*?[?&]url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "bestchinanews.com" ||
			domain_nowww === "zhaizou.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/url\.php.*?[?&]p=(http[^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "img.com.link") {
			return src.replace(/(\/[0-9a-f]+_)[0-9]+[a-z]?(\.[^/.]*)(?:[?#].*)?$/, "$1l$2");
		}

		if (domain_nowww === "europaplus.ru") {
			return src.replace(/\/upload\/+thumb\/+[^/]*\/+images\/+/, "/images/");
		}

		if (domain_nosub === "ozone.ru" && domain.match(/^cdn[0-9]*\./)) {
			return src.replace(/\/multimedia\/+c[0-9]+\/+/, "/multimedia/");
		}

		if (domain === "img.tsn.ua") {
			return src.replace(/(\/cached\/+[0-9]+\/+tsn-[0-9a-f]+\/+thumbs\/+)[0-9X]*x[0-9X]*\/+/, "$1x/");
		}

		if (domain_nowww === "ivi.ru") {
			return src.replace(/(\/uploads\/+[0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+[0-9a-f]+\.[^/.]+)\/+[0-9]+x[0-9]+(?:[?#].*)?$/,
							   "$1");
		}

		if (domain === "aisvip-a.akamaihd.net") {
			return src.replace(/(\/[0-9]+\/+)[0-9]+x[0-9]+\/+([^/]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "my-hit.org") {
			return src.replace(/(\/storage\/+[0-9]+_)[0-9]+x[0-9]+x[0-9]+(?:x[0-9]+)?(\.[^/.]*)(?:[?#].*)?$/,
							   "$199999x99999x99999$2");
		}

		if (domain_nosub === "censor.net.ua" && domain.match(/^storage[0-9]*\./)) {
			return {
				url: src.replace(/(\/images\/+(?:[0-9a-f]\/+){4}[0-9a-f]+\/+)[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
								 "$1original$2"),
				can_head: false // 503
			};
		}

		if (domain_nowww === "vokrug.tv") {
			return src.replace(/(\/pic\/+[^/]*\/+(?:.\/+){4})[a-z]+_([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nowww === "mucche.mu") {
			return src.replace(/(:\/\/[^/]*\/)t\/+([^/.]*)(?:[?#].*)?$/, "$1i/$2");
		}

		if (domain_nowww === "nudography.com") {
			return src.replace(/(\/photos\/+[^/]*\/+[0-9]{4}_[0-9]{1,2}\/+)[a-z]+\/+/, "$1original/");
		}

		if (domain_nowww === "celebs-porno.com") {
			return src.replace(/(:\/\/[^/]*\/[^/]*\/+)[a-z]{2}_([^/]*\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}


		if (domain_nowww === "forumkinopoisk.ru" ||
			domain === "forums.drom.ru" ||
			domain_nowww === "mexat.com") {
			return src.replace(/\/attachment\.php.*?[?&](attachmentid=[0-9]+).*?$/, "/attachment.php?$1");
		}

		if (domain_nowww === "chat1102.com") {
			return src.replace(/\/cache-img\/+(?:[0-9]+\/+){3}/, "/images/uploads/images/");
		}

		if (domain_nowww === "sobaka.ru") {
			return src.replace(/(\/images\/+(?:image|post)\/+(?:[0-9]{2}\/+){4})_(?:normal|huge|origin)(\.[^/.]*)(?:[?#].*)?$/, "$1_zoom$2");
		}

		if (domain_nowww === "ziaruldeiasi.ro") {
			return src.replace(/(\/gethumb(?:\.details)?\.php).*?[?&](id=[0-9]+).*?$/, "$1?$2&w=99999&h=99999");
		}

		if (domain_nowww === "jyii.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/showyixia\.php.*?[?&]url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "ishow.gr") {
			return src.replace(/(^[a-z]+:\/\/[^/]*\/)Thumbnail\.ashx.*?[?&]url=([^&]*).*?$/, "$1$2");
		}

		if (domain_nowww === "scienceleadership.org") {
			return src.replace(/\/thumbnail\/+([0-9]+)\/+[0-9]+x[0-9]+(?:[?#].*)?$/, "/media/open/$1");
		}

		if (domain === "cdn.toomar.net") {
			return src.replace(/\/thumbs\/+([0-9]+)_x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "/$1$2");
		}

		if (domain_nowww === "looti.net") {
			return src.replace(/\/shared_files\/+uploaded\/+([0-9]+)\/+([0-9]+)_([0-9]+)\.[^/.]*(?:[?#].*)?$/,
							   "/index.php?action=downloadfile&topic=$1&upfile=$2&sub=$3");
		}

		if (domain === "i.fokzine.net") {
			return src.replace(/(\/upload\/+(?:[0-9]{2}\/+){2}[^/]*)_[0-9]+_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "tr-osdcp.qunarzz.com") {
			return src.replace(/(\/img\/+[0-9a-f]+\.[^/._]*)(?:_r)?_[0-9]+x[0-9]+x[0-9]+_[0-9a-f]+\.[^/.]*(?:[?#].*)?$/, "$1");
		}

		if (domain === "file.newswire.co.kr") {
			return src.replace(/(\/data\/+datafile[0-9]+\/+)thumb_[0-9]+\/+/, "$1data/");
		}

		if (domain === "m.gadzetomania.pl") {
			return src.replace(/(-[0-9a-f]+)(?:,[0-9]+){4}(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "mklr.pl") {
			return src.replace(/(\/uimages\/+services\/+.*\/+[0-9]{6}\/+[^/]*)_[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nowww === "bifaxing.com") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/gpPic\/+[0-9]+\/+[0-9]+\/+([^/]*\.[^/]*)/, "http://$1");
		}

		if (domain_nosub === "filmpro.ru" && domain.match(/^b[0-9]*\./)) {
			return src.replace(/(\/c\/[0-9]+)\.[0-9]+x[0-9a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "static.leiphone.com") {
			return src.replace(/\/uploads\/+new\/+article\/+[0-9]+_[0-9]+\/+/, "/uploads/new/article/pic/");
		}

		if (domain_nowww === "tolknews.ru") {
			return src.replace(/(\/uploads\/+galleries_images\/+(?:[0-9]+\/+){2})[a-z]+(\.[^/.]*)(?:{?#].*)?$/,
							   "$1original$2");
		}

		if (domain === "s3.friday-magazine.ch") {
			return src.replace(/(:\/\/[^/]*\/[^/]*\/+)_[^/]+\/+([^/]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "m.tuniucdn.com") {
			return src.replace(/(\/[^/_.]*)(?:_[a-z][0-9]+){1,}(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "stat.mozi24.hu") {
			return src.replace(/(\/images\/+.*\/+[0-9]+_)[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$10x0$2");
		}

		if (domain_nosub === "autokopen.nl" && domain.match(/^media[0-9]*\./)) {
			return src.replace(/(\/[0-9]+)-[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1-original$2");
		}

		if (domain === "vbmspic.video.friday.tw") {
			return src.replace(/(\/[0-9]+_[0-9]+)_[A-Z](\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "hardware.fi" && domain.match(/^cdn[0-9]+[a-z]?\./)) {
			return src.replace(/\/storage\/+pictures\/+[0-9]+\/+/, "/storage/pictures/");
		}

		if (domain_nowww === "kinofuxy.tv") {
			return src.replace(/\/thumbs\/+(uploads\/+posts\/+[0-9]{4}-[0-9]{2}\/+)[0-9]+_(?:no|y)_(?:no|y)_/, "/$1");
		}

		if (domain_nosub === "web.de" && domain.match(/^i[0-9]*\./)) {
			return src.replace(/(\/image\/+[0-9]+\/+[0-9]+)[^/]*?(,pd=[0-9]+)[^/]*\/+/, "$1$2/");
		}

		if (domain === "img.culturacolectiva.com") {
			return src.replace(/\/cdn-cgi\/+image\/+[^/]*\/+/, "/");
		}

		if (domain_nowww === "movierj.com") {
			return src
				.replace(/\/cdn-cgi\/+image\/+(?:.*?,)?f=[^/]*\/+(upmovies\/+[^,]*).*?$/, "/$1")
				.replace(/(\/upmovies\/+upload\/+[^/]*)-min(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "wegotthiscovered.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/wp-content\/+themes\/+wgtc_v2\/+resizer\/+resizer\.php.*?[?&]file=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return urljoin(src, decodeuri_ifneeded(newsrc).replace(/^uploads\//, "/wp-content/uploads/"), true);
		}

		if (domain_nowww === "extra.cz") {
			return src.replace(/\/images\/+thumbs\/+((?:[0-9a-f]{2}\/+){2}[^/]+)-[0-9]+x[0-9]+-(?:shrink|fit)(\.[^/.]*)(?:[?#].*)?$/, "/images/original/$1$2");
		}

		if (domain === "media.services.cinergy.ch") {
			return src.replace(/\/media\/+[^/]*\/+/, "/media/raw/");
		}

		if (domain === "m.s1ar.cc") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/+file\/+(https?:\/\/)/, "$1");
		}

		if (domain === "i.joylada.net") {
			return src.replace(/(\.[^/.]+)\/+thumbnail(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "etoland.co.kr") {
			return src.replace(/(\/data\/+daumeditor[0-9]*\/+[0-9]+\/+)thumbnail[0-9]*\/+/, "$1");
		}

		if (domain_nowww === "bomb01.com") {
			return src.replace(/(\/upload\/+news(?:_cover)?\/+)[0-9]+x[0-9]+\/+/, "$1original/");
		}

		if (domain_nosub === "jfcdns.com") {
			return src.replace(/\/thumb\/+((?:up\/+)?[0-9]{4}-[0-9]{2}\/+[0-9]+)_[0-9]+_[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "/$1$2");
		}

		if (domain_nosub === "mysnap.top" && domain.match(/^sv[0-9]*\./)) {
			return src.replace(/:\/\/sv[0-9]*\.([^/]*\/)thumb\.php.*?[?&]src=([^&]*).*?$/, "://sv2.$1thumb.php?src=$2");
		}

		if (domain_nowww === "gttourkorea.com") {
			return src.replace(/(\/data\/+goods\/+[0-9]\/+[0-9]+\/+[^/]*)\.list(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "media.karousell.com") {
			return src.replace(/(\/media\/+photos\/+products\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[^/]*?)(?:_progressive)?_thumbnail(\.[^/.]*)?(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "media.matamata.com") {
			return src.replace(/\/thumbs\/+([0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9]+-[^/]*)\/+[^/]*(\.[^/.]*)(?:[?#].*)?$/,
							   "/images/$1$2");
		}

		if (domain === "img.haihanxeng.com" ||
			domain === "img.gurugamer.com") {
			return src.replace(/(:\/\/[^/]*\/)resize\/+[-0-9]+x[-0-9]+\/+/, "$1");
		}

		if (domain === "static.appledaily.hk") {
			return src.replace(/(\/images\/+[^/]*\/+[0-9]{8}\/+)[a-z]+\/+/, "$1large/");
		}

		if (domain_nowww === "jk6.cc") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/api\.php.*?[?&]url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "meduza.io") {
			return src.replace(/(\/images\/+(?:[0-9]{3}\/+){3})[a-z]+\/+/, "$1original/");
		}

		if (domain_nowww === "popgun.ru") {
			return src.replace(/(\/files\/+[^/]*\/+[0-9]+\/+)[a-z]+\/+/, "$1orig/");
		}

		if (domain_nowww === "rosphoto.com") {
			return src.replace(/\/rimg\/+[0-9]+-[0-9]+(?:-[a-z])?\/+images\/+/, "/images/");
		}

		if ((domain_nosub === "popmeh.ru"/* ||
			 domain_nosub === "bazaar.ru"*/)
			&& domain.match(/^images[0-9]+\./)) {
			return src.replace(/(\/upload\/+img_cache\/+[0-9a-f]{3}\/+[0-9a-f]+(?:_[^/_]+_[0-9]+x[0-9]+x[0-9]+x[0-9]+)?)_(?:cropped|fitted)_[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nowww === "decoder.ru") {
			return src.replace(/\/media\/+pic_[a-z]+\/+/, "/media/pic_full/");
		}

		if (domain_nowww === "fotogora.ru") {
			return src.replace(/\/img\/+blog\/+[a-z]+\/+/, "/img/blog/or/");
		}

		if (domain_nowww === "photosynthesis.bg") {
			return src.replace(/\/images\/+imagecache\/+[0-9]+_[0-9]+_([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/,
							   "/uploads/article_icons/$1");
		}

		if (amazon_container === "hobbydb-production") {
			return src.replace(/(\/image\/+[0-9]+\/+[0-9]+-[0-9]+-[0-9]+\/+[^/]*)_large(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "d1kls9wq53whe1.cloudfront.net") {
			return src.replace(/(\/articles\/+[0-9]+\/+)[0-9]+x[0-9]+\/+/, "$1ORG/");
		}

		if (domain === "images.vrt.be") {
			return src.replace(/(:\/\/[^/]*\/)(?:width|height)[0-9]+\/+/, "$1orig/");
		}

		if (domain_nowww === "gramerkagoj.com") {
			return src.replace(/(\/[0-9]+)_th(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "img.bisnis.com") {
			return src.replace(/\/thumb(\/+posts\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9]+\/+[^/?#]*).*?$/, "$1");
		}

		if (domain_nosub === "news18.com" && domain.match(/^images\./)) {
			newsrc = src.replace(/(:\/\/[^/]*\/+[^/]*\/+)uploads\/+[0-9]+x[0-9]+\/+[^/]*\/+([0-9]{4}\/+[0-9]{2}\/+)/,
							   "$1uploads/$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nosub === "dainikbhaskar.com" &&
			domain.match(/^i[0-9]*\./)) {
			return src.replace(/(:\/\/[^/]*\/)thumbnails\/+[0-9]+x[0-9]+\/+/, "$1");
		}

		if (domain_nowww === "baocantho.com.vn") {
			return src.replace(/(\/image\/+news\/+[0-9]{4}\/+[0-9]{8}\/+)thumbnail\/+[0-9]+x[0-9]+\/+/,
							   "$1fckimage/");
		}

		if (domain_nowww === "akhbarus.com") {
			return src.replace(/\/thumbnaile\/+[a-z]+\/+[0-9]+\/+[0-9]+\/+news\/+/, "/uploads/files/news/");
		}

		if (domain === "cdn-stylehaus-jp.akamaized.net") {
			return src.replace(/(\/article(?:s|_[a-z]+)\/+[0-9]+\/+[0-9]+_)[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1original$2");
		}

		if (domain_nowww === "primetime.ge") {
			return src.replace(/(\/uploads\/+[^/]+\/+[0-9]{2}-[0-9]{4}\/+[0-9a-f]{10,})_thumb(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nosub === "bnt.bg" && domain.match(/^nws[0-9]*\./)) {
			return src.replace(/-[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "-original$1");
		}

		if (domain === "storage.face.ba" ||
			domain === "storage.bljesak.info" ||
			domain === "storage.radiosarajevo.ba") {
			return {
				url: src.replace(/(\/(?:article|image)\/+[0-9]+\/+)[0-9]+x[0-9]+\/+/, "$1original/"),
				headers: {
					Referer: src
				}
			};
		}

		if (domain === "ii.cdn.tf") {
			return src.replace(/\/r[0-9]+x[0-9]+\//, "/original/");
		}

		if (domain_nowww === "blaxup.in") {
			return src.replace(/(\/upload\/+img\/+[0-9]{4}\/+[0-9]{2}\/+[^/]*)(?:_[WH][0-9]+){1,}(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nowww === "harryweb.net") {
			return src.replace(/(\/plupload\/+uploads\/+[0-9]+\/+)[a-z]+_([0-9a-z]+\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "m.baishixi.com") {
			return add_http(src.replace(/^[a-z]+:\/\/[^/]*\/p\/+([^/]*\.[^/]*\/)/, "$1"));
		}

		if (amazon_container === "otw-ao3-icons") {
			return src.replace(/(\/icons\/+[0-9]+\/+)[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1original$2");
		}

		if (domain === "media.outnow.ch") {
			return src.replace(/(\/Bilder\/+[0-9]{4}\/+[^/]*\/+(?:[^/]*\/+)?[0-9]+)_[a-z]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "pics.kinokadr.ru") {
			return src.replace(/\/gallery\/+thumb\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/gallery/$1");
		}

		if (domain_nosub === "creativecirclemedia.com" && domain.match(/^cdn[0-9]*\./)) {
			return src.replace(/\/[a-z]+\/+([0-9]{8}-[0-9]+-[0-9]+\.[^/.]*)(?:[?#].*)?$/,
							   "/original/$1");
		}

		if (domain_nowww === "middleburgfilm.org") {
			return src.replace(/\/img\/+upload\/+[a-z]+\/+/, "/img/upload/original/");
		}

		if (domain_nowww === "gonet.cz") {
			return src.replace(/\/[0-9]+(\/+[0-9a-f]\/+[0-9a-f]{10,}\.[^/.]*)(?:[?#].*)?$/,
							   "/full$1");
		}

		if (domain === "media.port.hu") {
			return src.replace(/(\/images\/+(?:[0-9]{3}\/+){2})[0-9]+x[0-9]+\/+/, "$1");
		}

		if (domain_nowww === "edb.co.il") {
			return src.replace(/(\/photos\/+[0-9]+(?:_[^/.]*?)?)(?:\.[^/.]*)?(\.[^/.]*)(?:[?#].*)?$/,
							   "$1.full$2");
		}

		if (domain_nowww === "silenzioinsala.com" ||
			domain_nowww === "olharalerta.com.br" ||
			domain_nowww === "wikiquicky.com" ||
			domain_nowww === "dmbtattoo.com" ||
			domain_nowww === "joinsmediacanada.com") {
			newsrc = src.replace(/.*\/+phpThumb\.php\?(?:.*?&)?src=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return urljoin(src.replace(/\/+phpThumb\.php.*/, "/"), decodeURIComponent(newsrc), true);
		}

		if (domain === "img.fdb.cz") {
			return src.replace(/(\/galerie)_[a-z]+(\/+[0-9a-f]\/+[0-9a-f]{10,}\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "wuaki.tv" && domain.match(/^images(?:-[0-9]*)\./)) {
			return src.replace(/(?:-(?:width|height|quality)[0-9]+){1,}(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain_nosub === "bebee.com" && domain.match(/^std[0-9]*\./)) {
			return src.replace(/\/br(\/+pb\/+[0-9]+\/+[0-9a-f]+)\/+[0-9]+(?:[?#].*)?$/,
							   "/bg$1");
		}

		if (domain_nowww === "mosaically.com") {
			return src.replace(/\/photo\/+[0-9]+\/+([0-9]{4}\/+)/, "/photo/full/$1");
		}

		if (domain === "img.ecartelera.com") {
			return src.replace(/(\/[0-9]+)(?:-[a-z0-9]+)?(\.[^/.]*)(?:[?#].*)?$/, "$1-m$2");
		}

		if (domain === "cdn.releases.com") {
			return src.replace(/(\/img\/+postattachments\/+[0-9]+)\/+[0-9]+(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "4words.ru") {
			return src.replace(/\/gallery\/+thumbs\/+[0-9]+\/+/, "/gallery/");
		}

		if (domain_nosub === "myheimat.de" && domain.match(/^media[0-9]*\./)) {
			return src.replace(/(\/[0-9]+)_[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1_orig$2");
		}

		if (domain_nowww === "cumonprintedpics.com") {
			return src.replace(/\/download\/+file\.php.*?[?&](id=[0-9]+).*?$/, "/download/file.php?$1");
		}

		if (domain === "aws.revistavanityfair.es" ||
			domain === "aws.glamour.mx") {
			return src.replace(/\/assets\/+[0-9]+x[0-9]+\/+/, "/assets/original/");
		}

		if (domain === "sticker.bahuma.org") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/thumbnail\/+[0-9]+\/+gallery\/+[-A-Za-z0-9]+:(https?:\/\/.*?)\.[^/.]*?$/,
							   "$1");
		}

		if (domain_nowww === "quizizz.com") {
			return src.replace(/(\/_media\/+[^/]*\/+[-0-9a-f]+)_[0-9]+_[0-9]+(?:[?#].*)?$/, "$1");
		}

		if (domain === "news.cube-soft.jp") {
			return src
				.replace(/\/thumb\/+([0-9]+)(\.[^/.]*)(?:[?#].*)?$/, "/image/$1_0$2")
				.replace(/\/thumb\/+(JI[0-9]{10,}\.[^/.]*)(?:[?#].*)?$/, "/image/$1");
		}

		if (domain_nowww === "boutreview.com") {
			return src.replace(/\/media\/+thumbnail\/+([a-z]+)_([0-9]{6})/, "/media/$1/$2");
		}

		if (domain_nowww === "bg.gg" ||
			domain_nowww === "bgm.gg") {
			return {
				url: src.replace(/(:\/\/[^/]*\/i\/[0-9a-f]+)\/+resize(?:_[a-z]+)?(?:[?#].*)?$/, "$1"),
				headers: {
					Referer: "https://te31.com/bggg.php"
				}
			};
		}

		if (domain === "my.evilmilk.com") {
			return src.replace(/(:\/\/[^/]*\/p)\/+.\/+.\/+([^/]*)_t[0-9]*(\.[^/.]*)(?:[?#].*)?$/,
							   "$1/$2$3");
		}

		if (domain_nowww === "bonnassesworld.com") {
			return src.replace(/\/[a-z]+_([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/full_$1");
		}

		if (domain === "img.oppaisan.com" ||
			domain_nowww === "megamich.com" ||
			(domain_nosub === "fightingirl.com" && /^img[0-9]*\./.test(domain))) {
			newsrc = src.replace(/(\/lib\/+(?:bbs_)?thumb[0-9]+\.php).*?[?&](p=[^&]*).*?$/, "$1?$2&w=-1");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/^[a-z]+:\/\/[^/]*\/+(?:[^/]+\/+)?lib\/+(?:bbs_)?thumb[0-9]+\.php\?(?:.*&)?p=([^&]+)/);
			if (match) {
				var path = base64_decode(match[1]);
				var newpath = path.replace(/.*\/public_html\/+_[^/]+\/+/, "/");
				if (newpath !== path) {
					return urljoin(src, newpath, true);
				}
			}
		}

		if (domain_nowww === "517japan.com") {
			return src.replace(/\/rimg_[0-9]+x[0-9]+\/+attachments\/+/, "/attachments/");
		}

		if (domain === "mpic.haiwainet.cn") {
			newsrc = src.replace(/:\/\/[^/]*\/thumb\/+.\/+uploadfile\/+([0-9]{4}\/*[0-9]{4}\/+[0-9]+)(?:,[wh]_[0-9]+)?(\.[^/.]*)(?:[?#].*)?$/,
								 "://images.haiwainet.cn/$1$2");
			return {
				url: newsrc,
				headers: {
					Referer: "http://m.haiwainet.cn/"
				}
			};
		}

		if (domain === "images.haiwainet.cn") {
			return {
				url: src,
				headers: {
					Referer: "http://news.haiwainet.cn/"
				}
			};
		}

		if (domain_nowww === "fotostate.ru") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/redirect\.php.*?[?&]url=(https?:\/\/[^&]*).*?$/, "$1");
		}

		if (domain === "static.apple.nextmedia.com") {
			return src.replace(/(\/[0-9]{8}\/+)small\/+/, "$1large/");
		}

		if (domain_nowww === "share-ero.pics" ||
			domain === "45.33.110.172" ||
			domain_nowww === "panchira-news.net" ||
			domain_nowww === "idol-gazoum.net") {
			return src.replace(/(\/uploads\/+[^/]*\/+image\/+(?:[0-9]+\/+)?[0-9]+\/+)[a-z]+_(?:resize|thumb)_/, "$1");
		}

		if (domain === "att2.citysbs.com") {
			return src.replace(/(\/image[0-9]*\/+[0-9]{4}\/+[0-9]{2}\/+[0-9]+-[0-9]+\/+)[a-z]+_([0-9]{8}_[0-9a-f]+[^/]*\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "d2l930y2yx77uc.cloudfront.net") {
			return src.replace(/(\/production\/+uploads\/+images\/+[0-9]+\/+)[^/]*_([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "d1jo5b1m9v3ic.cloudfront.net") {
			return src.replace(/(\/(?:item|profile)\/+[a-z][0-9]+\/+[a-z][0-9a-f]+)-[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1-large$2");
		}

		if (domain === "oshiete.xgoo.jp") {
			return src.replace(/(\/images\/+[^/]*\/+[0-9a-z]+\/+[0-9]+_[0-9a-f]+\/+)[A-Z]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1ORG$2");
		}

		if (domain === "p.mamastar.jp") {
			return src.replace(/(:\/\/[^/]*\/)[a-z]\/+([0-9]{4}-[0-9]{2}\/+)/, "$1$2");
		}

		if (domain === "image.gamechosun.co.kr") {
			return src.replace(/(\/dataroom\/+[^/]*\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2})thumb\/+/, "$1");
		}

		if (domain_nowww === "ascii.jp") {
			return src.replace(/(_[0-9]+x[0-9]+)_[0-9]+x(?:[0-9]+)?(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nowww === "yscore.co.kr" ||
			domain === "data.bepick.net") {
			return src.replace(/(\/bbs\/+[0-9]{4}\/+[0-9]{2}\/+)thumb\/+/, "$1");
		}

		if (domain === "img.tf.co.kr") {
			return src.replace(/\/thumb(\/+(?:[0-9]+_)?[0-9a-f]{5,})_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "my.cdn.tokyo-hot.com") {
			return src
				.replace(/(\/media\/+(?:[-0-9a-z]+|[A-Z]+-[0-9]+)\/+(?:[sv]cap|list_image)\/+[^/]*)\/+[^/]*(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "tvdeav.com") {
			return src
				.replace(/(\/media\/+banner\/+top\/+[0-9]{10,})\/+[^/]*(\.[^/.]*)(?:[?#].*)?$/, "$1$2")
				.replace(/(\/media\/+product\/+(?:[a-z]?[0-9]+|[A-Z]+-[0-9]+)\/+(?:[sv]cap|list_image)\/+[^/]*)\/+[^/]*(\.[^/.]*)(?:[?#].*)?$/, "$1$2")
				.replace(/(\/media\/+product\/+[a-z]?[0-9]+\/+[sv]cap\/+[0-9]+\.)[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1origi$2");
		}

		if (domain_nosub === "bakusai.com" && domain.match(/^img[0-9]*\./)) {
			return src.replace(/(\/imagebbs\/+_nosync\/+[0-9]+\/+[0-9]+\/+[0-9]+\/+)[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1org$2");
		}

		if (domain === "storage.oxii.vn") {
			return src
				.replace(/\/Cache\/+(Sites\/+.*?\/)[0-9]+x[0-9]+\/+/, "/$1")
				.replace(/\/thumbnail\/([^-/]+)-[0-9]+(?:-[0-9]+)?-([0-9]{4})-([0-9]{1,2})-([0-9]{1,2})\/+/,
						 "/Sites/$1/Storage/Images/$2/$3/$4/");
		}

		if (domain_nowww === "edna.cz") {
			return src.replace(/\/runtime\/+cache\/+images\/+[^/]*\/+/, "/runtime/userfiles/");
		}

		if (domain_nowww === "dramaqueen.com.tw") {
			return src.replace(/\/images\/+news\/+[a-z]+\/+/, "/images/news/");
		}

		if (domain_nowww === "smobserved.com") {
			return src.replace(/\/\.TEMP\/+([^/]*?)TEMP[0-9]+x[0-9]+-[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nowww === "naftemporiki.gr" ||
			domain === "ctrl.naftemporiki.gr") {
			newsrc = src.replace(/\/fu\/+t\/+([0-9]+)\/+/, "/fu/p/$1/");
			if (false && newsrc !== src)
				return newsrc;

			return {
				url: src.replace(/(\/fu\/+p\/+[0-9]+\/+)(?:[0-9]+|original)\/+(?:[0-9]+|original)\/+/, "$1original/original/"),
				headers: {
					Referer: ""
				}
			};
		}


		if (domain === "cdn.moar.exchange" ||
			domain === "cdn.viral.porn" ||
			domain === "cdn.moar.network" ||
			domain === "cdn.nsfw.exchange") {
			return add_full_extensions(src.replace(/(\/media\/+[0-9]+-[0-9]+\/+)conversions\/+([0-9a-zA-Z]+)-[^-/.]*(\.[^/.]*)(?:[?#].*)?$/,
												   "$1$2$3"));
		}

		if (domain_nowww === "echosrecordbar.co.za") {
			return src.replace(/(\/storage\/+[0-9]+\/+)conversions\/+([0-9]+)-[a-z]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2$3");
		}

		if (domain === "m.blog.hu") {
			return src.replace(/(\/image\/+[^/]*\/+[^/]*\/+)(?:smnxs-([^/]*)|([0-9]+)sm)(\.[^/.]*)(?:[?#].*)?$/, "$1$2$3$4");
		}

		if (domain_nowww === "gosee.de") {
			return src
				.replace(/\/prev\/+[0-9]+x(?:[0-9]+)?\/+images\/+/, "/images/")
				.replace(/\/images\/+preview\/+[0-9a-f]+\/+[0-9]+x(?:[0-9]+)?-/, "/images/content2/");
		}

		if (domain_nowww === "uploadbeta.com") {
			return src.replace(/(\/_s\/+upload\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9a-f]+\.[a-zA-Z]+)[0-9]+\.[^/.]*(?:[?#].*)?$/,
							   "$1");
		}

		if (domain_nowww === "onlyasianpictures.com" ||
			domain_nowww === "avidolpics.com") {
			return src.replace(/(\/galleries\/+[0-9]{4}\/+[0-9]{2}\/+[^/]*\/+)thumbs\/+/, "$1");
		}

		if (domain_nowww === "caribbeancompr.com") {
			return {
				url: src.replace(/\/images\/+s\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/images/l/$1"),
				headers: {
					Referer: "https://www.caribbeancompr.com/"
				}
			};
		}


		if (domain_nowww === "ai-av.com") {
			return src.replace(/(\/[0-9]{4}\/+[0-9]+-[0-9]+\/+[0-9]+)s(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "blovcdn.com" && domain.match(/^cdn[0-9]*\./)) {
			return src.replace(/([?&]format=)[^&]*/, "$1s");
		}

		if (domain === "img.nimo.tv" ||
			domain === "img-service.nimo.tv") {
			return src.replace(/(:\/\/[^/]*\/t\/+[^/]*.[^/.]*)\/+[^/]*\/+([^/]*\.[^/.]*)(?:[?#].*)?$/,
							   "$1/full/$2");
		}

		if (domain === "wspicsa.vod.nimo.tv") {
			return src.replace(/\/live\/+[0-9]+[*][0-9]+\/+/, "/live/");
		}

		if (domain === "cdn.lesilla.filoblu.com") {
			return src.replace(/\/([a-z]+)\/+content\/+resized\/+[0-9]+x(?:[0-9]+)?\/+\1\/+/, "/$1/");
		}

		if (domain === "mimg.koreatimes.com") {
			return src.replace(/:\/\/[^/]*\/[0-9]+\/+[0-9]+\/+article\/+/, "://image.koreatimes.com/article/");
		}

		if (domain === "d2360iq24ihnyn.cloudfront.net") {
			return src.replace(/(\/image_[0-9a-f]{10,})_small(\.[^/.]*)(?:[?#].*)?$/,
							   "$1_large$2");
		}

		if (domain === "dingyue.ws.126.net") {
			return src.replace(/compressflag(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain === "times.hinet.net") {
			return src.replace(/\/s_([0-9a-f]+(?:-[0-9a-f]+){3,}\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nowww === "kpopdaily.co.kr" ||
			domain_nowww === "ujnews.co.kr" ||
			domain_nosub === "segyelocalnews.com" ||
			domain_nowww === "upinews.kr" ||
			domain_nowww === "thedrive.co.kr" ||
			domain_nowww === "newswiz.kr") {
			regex = /(\/news\/+data\/+[0-9]{8}\/+[a-z]*[0-9]+_[0-9]+)_(?:h[0-9]*|thum)(\.[^/.]*)(?:[?#].*)?$/;

			var extra = {};
			match = src.match(/\/news\/+data\/+[0-9]+\/+p([0-9]{10,})_/);
			if (match) {
				extra = {
					page: "http://www." + domain_nosub + "/news/newsview.php?ncode=" + match[1]
				};
			}

			if (regex.test(src)) {
				return [
					{
						url: src.replace(regex, "$1$2"),
						extra: extra
					},
					{
						url: src.replace(regex, "$1_h3$2"),
						extra: extra
					},
					{
						url: src.replace(regex, "$1_thum$2"),
						extra: extra
					},
					{
						url: src.replace(regex, "$1_h2$2"),
						extra: extra
					}
				];
			} else if (extra) {
				return {
					url: src,
					extra: extra
				};
			}

			/*newsrc = src.replace(/(\/news\/+data\/+[0-9]{8}\/+p[0-9]+_[0-9]+)_(?:thum|h2?)(\.[^/.]*)(?:[?#].*)?$/, "$1_h3$2");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/\/news\/+data\/+[0-9]+\/+p([0-9]{10,})_/);
			if (match) {
				return {
					url: src,
					extra: {
						page: "https://www.upinews.kr/news/newsview.php?ncode=" + match[1]
					}
				};
			}*/
		}

		if (domain === "image.newstomato.com") {
			return src.replace(/\/thum(?:_[a-z])?_([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if ((domain_nosub === "nijie.info" ||
			 domain_nosub === "nijie.net") && domain.match(/^pic[0-9]*\./)) {
			return src.replace(/\/__rs_[a-z]*[0-9]+x[0-9]+\/+/, "/");
		}

		if (domain === "cdn-img.jamendo.com") {
			return src.replace(/(\/[0-9]+\.)[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$10$2");
		}

		if (domain_nowww === "cdbaby.name") {
			return src.replace(/(\/[0-9a-z]+)(?:_[a-z]+)?(\.[^/.]*)(?:[?#].*)?$/, "$1_large$2");
		}

		if (domain_nowww === "livingelectro.com") {
			return src.replace(/(\/files\/+images\/+[^/]*\/+[0-9]+\/+[^/]*)_small(\.[^/.]*)(?:[?#].*)?$/, "$1_large$2");
		}

		if (domain_nosub === "archive.org" && domain.match(/^ia[0-9]*\./)) {
			newsrc = src.replace(/(\/items\/+mbid-[-0-9a-f]+\/+mbid-[-0-9a-f]+)_(?:thumb[0-9]+|itemimage)(\.[^/.]*)(?:[?#].*)?$/, "$1$2");

			if (newsrc !== src) {
				return add_extensions({
					url: newsrc,
					can_head: false
				});
			}
		}

		if (domain_nowww === "coverartarchive.org") {
			return src.replace(/(\/[0-9]+)-[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "45cat.com") {
			return src.replace(/:\/\/[^/]*\/image\/+[0-9]+\/+thumb\/+([^/]*)-t(\.[^/.]*)(?:[?#].*)?$/,
							   "://images.45cat.com/$1$2");
		}

		if (domain === "images.45cat.com") {
			return src.replace(/-s(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain === "d16klsh1z1xre7.cloudfront.net") {
			return src.replace(/-size(?:-[a-z]+)?-[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "-size-original$1");
		}

		if (domain_nosub === "magnatune.com") {
			return src.replace(/(\/cover)_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "fast.ilicensemusic.com") {
			return src.replace(/:\/\/[^/]*\/music\//, "://magnatune.com/music/");
		}

		if (domain === "img.1ting.com") {
			return {
				url: src.replace(/\/s[0-9]+_([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/, "/$1"),
				headers: {
					Referer: ""
				}
			};
		}

		if (domain === "omusic.friday.tw") {
			return src.replace(/(\/img\/+album\/+[0-9]+)-[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "rocklyric.jp") {
			return src.replace(/_thum(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain === "spice.eplus.jp") {
			return src.replace(/(\/images\/+[a-zA-Z0-9]{10,})(?:\/+[a-z]+)?(?:[?#].*)?$/, "$1/original");
		}

		if (domain_nowww === "earone.it") {
			return src.replace(/(\.[^/._]+)___th_[0-9]+_[0-9]+\.[^/.]*(?:[?#].*)?$/, "$1");
		}

		if (domain === "n11scdn.akamaized.net") {
			return src.replace(/(\/a1\/+)[0-9]+\/+/, "$1org/");
		}

		if (domain_nowww === "wmg.jp") {
			return src.replace(/\/images\/+(?:pi_[0-9]+x[0-9]+_)?([a-z]+[0-9]+)_LL?\./, "/images/$1_LLL.");
		}

		if (domain_nowww === "bontonland.cz") {
			return src.replace(/\/image\.php.*?[?&]image=([^&]*).*?$/, "/image.php?image=$1");
		}

		if (domain === "s12emagst.akamaized.net") {
			return src.replace(/(\/images\/+res_[0-9a-f]{10,})_[0-9]+x[0-9]+(?:_[a-z]+)?(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (amazon_container === "img.iluria.com") {
			return src.replace(/\/[0-9N]+x[0-9N]+(\.[^/.]*)(?:[?#].*)?$/, "/original$1");
		}

		if (domain_nosub === "m24.ru") {
			return src.replace(/(\/[0-9]+)\.[0-9p]+x[0-9p]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "amoeba.com") {
			return src.replace(/\/sized-images\/+[a-z]+\/+(?:[0-9]+\/+){2}uploads\/+/, "/admin/uploads/");
		}

		if (amazon_container === "groundctrl") {
			return src.replace(/(\/images\/+assets\/+)[a-z]+\.([0-9a-zA-Z_]+\.[^/.]*)(?:[?#].*)?$/, "$1original.$2");
		}

		if (domain === "img.manoramaonline.com") {
			return src.replace(/\.image\.[0-9]+\.[0-9]+\.[^/.]*(?:[?#].*)?$/, "");
		}

		if (domain_nowww === "washwasha.org") {
			return src.replace(/(\/upload\/+photo\/+gallery\/+[0-9]+\/+[0-9]+\/+)[0-9]+x[0-9]+\/+/, "$1");
		}

		if (domain === "images.mimint.co.kr") {
			return src.replace(/(\/[0-9]{4}\/+(?:[0-9]{2}\/+){2}S[0-9]{10,})[a-z](\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nosub === "mimint.co.kr") {
			newsrc = src.replace(/.*\/article\/+thumbnail\.asp\?(?:.*?&)?thumb=([^&]+).*?$/, "$1");
			if (newsrc !== src) {
				return decodeuri_ifneeded(newsrc);
			}
		}

		if (domain_nowww === "hrising.com") {
			return src
				.replace(/\/img\/.*?[?&]p=([^&]*).*?$/, "/data/$1")
				.replace(/(\/data\/+[^/]*\/+)thumbnail\/+attach\/+/, "$1attach/");
		}

		if (domain === "freight.cargo.site") {
			return src.replace(/(?:\/(?:[wh]\/+[0-9]+|t\/+[a-z]+))?\/+i\/+/, "/t/original/i/");
		}

		if (domain === "i.inswave.net" ||
			domain === "f.xza.co.kr") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/(?:img_s|f_img)\.php\?(?:.*?&)?[up]=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "daejonilbo.com") {
			return src.replace(/\/news_photo\/+[a-z]Img\/+/, "/news_photo/oImg/");
		}

		if (domain_nowww === "staraz.co.kr") {
			return src.replace(/(\/Files\/+[0-9]+\/+Images\/+[0-9]{6}\/+[0-9]+(?:_[0-9]+){2})_[a-z](\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "dy6g3i6a1660s.cloudfront.net") {
			return src
				.replace(/(:\/\/[^/]*\/[^/]{15,}\/+)[_a-z0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1orig$2")
				.replace(/(:\/\/[^/]*\/[^/]{15,}\/+)([_a-z0-9]+)(?:-[^/]*)?\/.*(\.[^/.]*)$/, "$1$2$3");
		}

		if (domain === "d3pxmkp5ez9u31.cloudfront.net") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/sticker\/+(http)/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "dontlinkthis.net") {
			return src.replace(/\/thumbnails\//, "/images/");
		}

		if (domain_nowww === "reho.st") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/thumb\/([^/]*\.[^/]*)/, "$1");
			if (newsrc !== src)
				return add_http(newsrc);
		}

		if (domain_nowww === "funny-anekdot.ru") {
			return src.replace(/\/photos\/+thumb_(p[0-9]+\.[^/.]*)(?:[?#].*)?$/, "/photos/$1");
		}

		if (domain === "imagenes.montevideo.com.uy") {
			return src.replace(/\/_[WH][0-9]+\//, "/");
		}

		if (domain_nowww === "stars-photos.com") {
			return src.replace(/\/resize\.php.*?[?&](id=[0-9]+).*?$/, "/image.php?$1");
		}

		if (domain_nosub === "at.ua") {
			regex = /(\/_ph\/+[0-9]\/+)[0-9]+\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/;
			if (regex.test(src)) {
				return [
					src.replace(regex, "$1$2"),
					src.replace(regex, "$12/$2")
				];
			}
		}

		if (domain_nowww === "missero.ru") {
			return src.replace(/\/images\/+/, "/photo_full/");
		}

		if (domain_nosub === "oneniceapp.com" && domain.match(/^img[0-9]*\./)) {
			return src.replace(/\.half(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "expofashionmagazine.com") {
			return src.replace(/(\/uploads\/+[^/]*\/+(?:[0-9]{4}\/+[0-9]{2}\/+)?)[wh][0-9]+px_/, "$1");
		}

		if (domain_nosub === "sinclairstoryline.com" && domain.match(/^static-[0-9]*\./)) {
			return src.replace(/(\/resources\/+media\/+[-0-9a-f]+)-[a-z]+Scale_(AP[0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1-$2");
		}

		if (domain_nowww === "maminka.cz") {
			return src.replace(/\/getthumbnail\.aspx.*?[?&](id_file=[0-9]+).*?$/, "/getthumbnail.aspx?w=100000&h=100000&q=100&$1");
		}

		if (domain_nosub === "digitaltrends.com" && domain.match(/^icdn[0-9]*\./)) {
			return src.replace(/-[0-9]+x[0-9]+(?:-c-ar[0-9.]+)?(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain === "d26oc3sg82pgk3.cloudfront.net") {
			return src.replace(/(\/image\/+[0-9]+\/+)[^/.]*(\.[^/.]*)(?:[?#].*)?$/, "$1original$2")
		}

		if (domain_nowww === "glow.pl") {
			return src.replace(/(\/site_media\/+[^/]*\/+)cache\/+([^/]*)_[a-z]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2$3");
		}

		if (domain_nosub === "qhimgs4.com" && domain.match(/^p[0-9]*\./)) {
			return src.replace(/\/bdr\/+[0-9]+__\/+/, "/");
		}

		if (domain === "vip.tietu.la") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/getpic\.php.*?[?&]imgsrc=(http[^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nosub === "publicbroadcasting.net") {
			return {
				url: src.replace(/\/styles\/+[^/]*\/+[^/]*\/+([0-9]{6})\/+/, "/$1/"),
				head_wrong_contentlength: true
			};
		}

		if (domain_nowww === "rabstol.net" ||
			domain_nowww === "vgtimes.ru" ||
			domain_nowww === "youloveit.ru") {
			return src.replace(/(\/uploads\/+gallery\/+)[a-z]+(\/+[0-9]+\/+)/, "$1main$2");
		}

		if (amazon_container === "reacho") {
			return src.replace(/(\/httpimages\/+[0-9]+\/+)[0-9]+\/+(img_[0-9]+[0-9a-f]+\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nowww === "famavip.com") {
			return src.replace(/(:\/\/[^/]*\/)(?:n|lg)\/+/, "$1xl/");
		}

		if (domain_nowww === "notifeed.net") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/thumb.*?[?&]u=(http[^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "atmag-static-timeout.netdna-ssl.com") {
			return src.replace(/(\/media\/+[0-9]{4}\/+[0-9]{2}\/+)sizes\/+([^/]*?)_wo_[0-9]+_[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2$3");
		}

		if (domain_nowww === "bliasak.bg") {
			return src.replace(/-[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "-original$1");
		}

		if (domain === "news.data.bg") {
			return src.replace(/(\/storage\/+news\/+[0-9]{4}(?:-[0-9]{2}){2}\/+[0-9]+)_[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nosub === "blogcu.com" && domain.match(/^img[0-9]*\./)) {
			return src.replace(/\/v2\/+images\/+[a-z]+\/+/, "/v2/images/orj/");
		}

		if (domain === "img.a9vg.com") {
			return src.replace(/\/s_([0-9a-f]{20,}\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nosub === "waikeung.info" && string_indexof(src, "id=ror_grab_ttkzm")) {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/.*?[?&]url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "tn.pooh.pw") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/\?(http)/, "$1");
		}

		if (domain_nosub === "animeanime.jp") {
			return src.replace(/\/imgs\/+(?:[^/]*|p\/+(?:jtKDOVlKAvjRrNw8SXAVejagI61Nrq|ypfYP8UGHHv1ocFz1cgmQGihmaytrq|qC3cX52_w9YJ28j7VPltKcSgI60Yrq|GAauWYP2lbxlzgJRQBgLncSmE68prq|F0wY7sTvXbM-zUyY3Ct0qjajM6ytrq|rQC-FVStleeQ4uCDm4cdn6GgF60oy6)_oqaqr)\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/imgs/p/sQnw7oxGTTu4umdJ9vPIDqGgI6ytrq_oqaqr/$1");
		}

		if (domain_nosub === "response.jp" ||
			domain_nosub === "inside-games.jp" ||
			domain_nosub === "gamebusiness.jp" ||
			domain_nosub === "rbbtoday.com" ||
			domain_nosub === "cyclestyle.net" ||
			domain_nosub === "resemom.jp" ||
			domain_nosub === "cinemacafe.net" ||
			domain_nosub === "gamespark.jp" ||
			domain_nosub === "spyder7.com") {

			var orig = null;

			if (domain_nosub === "response.jp")
				orig = "zkztXxppQllf4-RUXYhKtU1Mz0BBQkNERUZH";
			else if (domain_nosub === "spyder7.com")
				orig = "X4UiqBKWADm8lT-ssAf7B9LTUN-e3dzb2tnY";
			else if (domain_nosub === "inside-games.jp")
				orig = "1UNSdyJKsYSbpw0t-zkIAAoLiAcGBQQDAgEA";
			else if (domain_nosub === "gamebusiness.jp")
				orig = "bo91TFR6ywNB2t2bceYnWYiJCoWEh4aBgIOC";
			else if (domain_nosub === "rbbtoday.com")
				orig = "O24ii8l5s579T1Z71xNOb0xNzkFAQ0JFREdG";
			else if (domain_nosub === "cyclestyle.net")
				orig = "SG6XwypJr-L-0cc0vCoMDigpqiUkJyYhICMi";
			else if (domain_nosub === "resemom.jp")
				orig = "iznZhtjPh4xcpFmdtjU2rkRFxklIS0pNTE9O";
			else if (domain_nosub === "cinemacafe.net")
				orig = "JgD9AR1Ir8Z47f6Bz3cIbgIDgA8ODQwLCgkI";
			else if (domain_nosub === "gamespark.jp")
				orig = "PseAHU_gq3GEwRLM5htwMwoLvAaDYAQDAgEA";

			return src.replace(/\/imgs\/+(?:[^/]*|p\/+[^/]{30,})\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/imgs/p/" + orig + "/$1");
		}

		if (domain_nowww === "pixsell.hr") {
			newsrc = src.replace(/\/scripts\/+get_image\.php.*?[?&](image_id=[0-9]+).*?$/, "/scripts/get_image.php?$1");
			if (newsrc !== src)
				return newsrc;

			if (src.match(/\/scripts\/+get_image\.php\?image_id=[0-9]+$/)) {
				return {
					url: src,
					filename: src.replace(/.*image_id=([0-9]+)$/, "$1")
				};
			}
		}

		if (domain_nowww === "softline.com.bd" ||
			domain_nowww === "softline.uz") {
			return src.replace(/\/uploads\/+resizer\/+i\/+((?:[0-9a-f]{2}\/+){8,}origin)_([a-z]+)-[^/]*(?:[?#].*)?$/,
							   "/uploads/i/$1.$2");
		}

		if (googlestorage_container === "wzukusers") {
			return src.replace(/(\/images\/+[0-9a-zA-Z]+\/+[^/]*)_d[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "mindmaps.ai-pharma.dka.global") {
			return src.replace(/(\/storage\/+[^/]*)_[0-9]+px\/+/, "$1/");
		}

		if (domain_nowww === "ibusiness.de") {
			return src.replace(/\/cgi-bin\/+resize\/+(upload\/+bilder\/+[^/]*?)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nowww === "picadilist.fr") {
			return src.replace(/\/mini\/+img_[0-9]+_[0-9]+\/+/, "/images/upload_gen/");
		}

		if (domain_nowww === "mirrorlessreports.com" ||
			domain_nowww === "canonnews.com") {
			return src.replace(/\/[^-/.]*?(img-[^/]*\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain === "d9j5qtehtodpj.cloudfront.net") {
			return src.replace(/\/thumbnail\/.*?[?&]image=([0-9a-f]+).*?$/, "/media/$1/image");
		}

		if (domain === "cdn.46graus.com") {
			return src.replace(/(\/files\/+portfolio\/+[^/]*\/+[-0-9a-f]+\/+)[^/]*_([-0-9a-f]+\.[^/.]*)(?:[?#].*)?$/,
							   "$1original_$2");
		}

		if (domain_nowww === "spirig-schulungscenter.ch") {
			return src.replace(/(\/uploads\/+[^/]*\/+)[0-9]+x[0-9]+_[0-9]+x[0-9]+\/+/, "$1");
		}

		if (domain === "static.caravan.kz") {
			return src.replace(/\/image\/+[0-9]+\/+[0-9]+\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/image/$1");
		}

		if (domain_nosub === "crhoy.net" && domain.match(/^icdn[0-9]*\./)) {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/(?:[whq]\/+[0-9]+\/+){1,}[a-z]+\/+[0-9]+\/+c\/+[0-9]+\/+s\/+([^/]*\.[^/]*\/+)/,
								 "$1");
			if (newsrc !== src)
				return add_http(newsrc);
		}

		if (domain_nosub === "ppstatic.pl") {
			return src.replace(/(\/[^/.,]*)(?:,(?:size|[whq]),[^/,.]*){1,}(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "uzone.id" && domain.match(/^cdn[0-9]*\./)) {
			return src.replace(/(\/assets\/+uploads\/+.*?)\/+[0-9]+(?:[?#].*)?$/, "$1");
		}

		if (amazon_container === "fuckin.news") {
			return src.replace(/(\/posts\/+images\/+(?:[0-9]{3}\/+){3})[a-z]+\/+/, "$1original/");
		}

		if (domain_nowww === "dengekionline.com") {
			return src.replace(/(\/images\/+.*?)_main(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "hypnoweb.net") {
			return src.replace(/(\/photo\/+[0-9]+\/+[^/]*\/+)mini\/+/, "$1ok/");
		}

		if (domain_nowww === "allbox.tv") {
			return src.replace(/(\/[0-9a-f]+)_small(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "api.superguidatv.it") {
			return {
				url: src.replace(/\?.*/, ""),
				redirects: true
			}
		}

		if (domain === "cloud.filmfed.com") {
			return src.replace(/\/[a-z]_([0-9a-f]+(?:-[0-9a-f]+){2,}\.[^/.]*)(?:[?#].*)?$/, "/o_$1");
		}

		if (domain === "images.hi67.cn") {
			return {
				url: src,
				headers: {
					Referer: "",
					Origin: ""
				}
			};
		}

		if (domain === "midiastm.gazetaonline.com.br" ||
			domain === "static.gazetaonline.com.br") {
			return src.replace(/\/[0-9]+x[0-9]+\/+[0-9]+_([^/]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nosub === "gifposter.com" && string_indexof(src, "/images/") >= 0) {
			return src.replace(/\/[a-z](_[0-9]+\.[^/.]*?)(?:_[a-z]+)?(?:[?#].*)?$/, "/p$1");
		}

		if (domain_nowww === "xiuren.org") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+cover\.php\?(?:.*?&)?src=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);

			return src.replace(/\/Thum\/+Thum-/, "/");
		}

		if (domain_nowww === "haipic.com") {
			return src.replace(/(\/icon\/+)[a-z]_[0-9]{3}([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "photo.l99.com") {
			return src.replace(/(:\/\/[^/]*\/)[a-z]+\/+/, "$1source/");
		}

		if (domain_nowww === "cinemapassion.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/miniature2\.php.*?[?&]pic=(http[^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (amazon_container === "abandomedia") {
			return src.replace(/(\/db\/+[a-z]+)_thumb\/+/, "$1/");
		}

		if (domain_nowww === "arlingtonhotelgroup.com") {
			return src.replace(/\/[0-9]+@[0-9]+\./, "/@.");
		}

		if (domain_nowww === "cine-vox.com" ||
			domain_nowww === "cinemorvan.fr") {
			return src.replace(/(:\/\/[^/]*\/+[^/]*\/+)[0-9]+x[0-9]+\/+([^/]*\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "cdnscreenshot.xyz") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/retail\.php.*?[?&]src=(http[^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "comic.systems") {
			return src.replace(/(\/images\/.*?)-thumb(?:[?#].*)?$/, "$1");
		}

		if (domain_nosub === "gittigidiyor.net") {
			return src.replace(/(\/[0-9]+\/+)tn[0-9]+\/+([0-9]+)_tn[0-9]+(_[0-9]+\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2$3");
		}

		if (domain_nowww === "impawards.com") {
			return {
				url: src
					.replace(/\/med_([^/]*)(?:[?#].*)?$/, "/$1")
					.replace(/(?:_xx?lg)?(\.[^/.]*)(?:[?#].*)?$/, "_xxlg$1"),
				headers: {
					Referer: "",
					Origin: ""
				}
			};
		}

		if (domain_nowww === "lescinemasaixois.com") {
			return src.replace(/(\/affiche\/+affiche_[0-9]+\.[^/.?&#]*).*?$/, "$1");
		}

		if (domain_nowww === "sortiesdvd.com") {
			return src.replace(/\/+affichethumb\/+/, "/affiche/");
		}

		if (domain_nowww === "cineswellington.com") {
			return src.replace(/\/zoom\/+zoom_/, "/");
		}

		if (domain === "img.seriebox.com") {
			return src.replace(/\/_thumbs\/+[0-9]+_[0-9]+\/+/, "/");
		}

		if (domain === "d3ewd3ysu1dfsj.cloudfront.net") {
			return src.replace(/(\/images\/+stories\/+)[a-z]+\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1original/$2");
		}

		if (domain_nowww === "mairiedesamoens.fr") {
			return src.replace(/\/iso_upload\/+([0-9]+)\/+t_[0-9]+_[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "/imageProvider.aspx?resource=$1&fn=$1$2");
		}

		if (domain_nowww === "citizenkid.com") {
			return src.replace(/\/uploads\/+medias\/+cache\/+[^/]*\/+[^/]*\/+[^/]*\/+uploads\/+medias\/+/, "/uploads/medias/");
		}

		if (amazon_container === "weclap-prodcluster") {
			return src.replace(/(\/images\/+[0-9]+\/+)[a-z]+\/+([0-9a-f]+\/+)/, "$1original/$2");
		}

		if (domain_nosub === "sogou.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/net\/+.\/+[0-9]+\/+link.*?[?&]url=(http[^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nosub === "sogoucdn.com" && /^img[0-9]*\./.test(domain)) {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+v2\/+thumb\/+.*?\?(?:.*&)?url=([^&]+).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "cinemapalace.ch") {
			return src.replace(/\/uploads\/+[0-9]+_[0-9]+_/, "/uploads/");
		}

		if (domain === "cdn.simplesite.com") {
			return {
				url: src.replace(/\._sz(?:[wh][0-9]+){1,}_(\.[^/.]*)(?:[?#].*)?$/, "$1"),
				can_head: false // 404
			}
		}

		if (domain_nowww === "inews.bg") {
			return src
				.replace(/(\/pictures\/+[0-9]+)_[0-9]*_[0-9]*(?:_[0-9]+x[0-9]+)?(\.[^/.]*)(?:[?#].*)?$/, "$1$2")
				.replace(/(\/pictures\/+[0-9]+)_[0-9]*_[0-9]*_([0-9]+)(\.[^/.]*)(?:[?#].*)?$/, "$1___$2$3");
		}

		if (domain_nosub === "starofservice.com" && domain.match(/^cdn-uploads[0-9]*\./)) {
			return src.replace(/(\/uploads\/+pj\/+)thumbs(?:-[a-z]+)?\/+/, "$1");
		}

		if (domain_nowww === "hayabusa.io") {
			return src
				.replace(/(\/[^/.]*)(?:\.[-a-z]+[0-9]*)*(\.[^/.]*)(?:[?#].*)?$/, "$1$2")
				.replace(/\?[whq]=[0-9]+(?:&(.*))?$/, "?$1")
				.replace(/&[whq]=[0-9]+(&.*)?$/, "$1")
				.replace(/\?(?:width|height|quality)=[0-9]+(?:&(.*))?$/, "?$1")
				.replace(/&(?:width|height|quality)=[0-9]+(&.*)?$/, "$1")
				.replace(/\?$/, "")
				.replace(/(\.(?:jpe?g|png|webp|gif))(?:\?.*)?/, "$1?q=100&quality=100");
		}

		if (domain === "s.dou.ua") {
			return src.replace(/(\/img\/+avatars\/+)[0-9]+x[0-9]+_/, "$1");
		}

		if (domain === "d3kq2xhl2rew87.cloudfront.net" ||
			amazon_container === "gfinity-img") {
			return src.replace(/(\/user\/+image\/+(?:[0-9]\/+){3})[0-9]+x[0-9]+(?:-[a-z]+)?\/+/, "$1");
		}

		if (domain_nosub === "tokopedia.net") {
			return src.replace(/\/img\/+cache\/+[0-9]+(?:x[0-9]+)?(?:-[a-z]+)?\/+(.*?)(?:(\.[^/.]+)?\.webp)?(?:[?#].*)?$/, "/img/$1$2");
		}

		if (domain === "icdn.2cda.pl") {
			return src.replace(/\/thumbs\/+([0-9a-f]+(?:-[0-9]+)?\.[^/._]*)(?:_[^/]*)?(?:[?#].*)?$/, "/oryginalne/$1");
		}

		if (domain === "i.imagesup.co") {
			return src.replace(/\/sz\/+[-0-9]+x[-0-9]+\/+images\/+/, "/images/");
		}

		if (domain_nosub === "kgimg.com") {
			return src.replace(/(\/kugouicon\/+)[0-9]+\/+([0-9]{8}\/+[0-9]+\.[^/.]*)(?:[?#].*)?$/, "/$1$2");
		}

		if (domain === "s.starladder.com") {
			return src.replace(/(\/uploads\/.*\/)thumb_[0-9]+_([0-9a-f]{10,}\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "cilacila.com") {
			return src.replace(/\/uploads\/+thumb(\/+[0-9]{8}\/+[0-9]+\/+[^/]*?)(?:_[0-9]+){3}(\.[^/.]*)(?:[?#].*)?$/,
							   "/uploads/$1$2");
		}

		if (domain === "vip.img.cdn.keeng.vn" ||
			domain === "image.keeng.com.kh") {
			regex = /\/images\/+images_thumb\/+(?:[^/]+\/+)?([^/]+\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[^/]+)_[0-9]+_[0-9]+(\.[^/.]*)(?:[?#].*)?$/;
			return [
				src.replace(regex, "/$1$2"),
				src.replace(regex, "/images/$1$2")
			];
		}

		if (domain_nowww === "highresaudio.com") {
			return src.replace(/(\/imgcache\/+[0-9a-f]{20,}\/+[^/]*?)_[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "d1wtzzt4oxg683.cloudfront.net") {
			return src.replace(/\/images\/+covers\/+[a-z]+\/+/, "/images/covers/");
		}

		if (domain_nowww === "nnmassets.cf") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/forum\/+image\.php\?(?:.*?&)?link=(http[^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeURIComponent(newsrc);
		}

		if (domain_nosub === "jpc.de" && domain.match(/^media[0-9]*\./)) {
			return src.replace(/\/image\/+[wh][0-9]+\/+(?:[a-z]+\/+)?/, "/image/w999999/");
		}

		if (domain_nowww === "mall.cz" ||
			domain === "i.cdn.nrholding.net") {
			return src.replace(/(:\/\/[^/]*\/(?:i\/+)?[0-9]+)\/+[0-9]+(?:\/+[0-9]+)?\/*(?:[?#].*)?$/, "$1");
		}

		if (domain === "eshop.fayaque.com.tw") {
			return src.replace(/\/img\/+[0-9]+x[0-9]+\/+/, "/img/original/");
		}

		if (domain === "img.shoplineapp.com") {
			return src.replace(/(\/media\/+[^/]*\/+[0-9a-f]{8,}\/+)[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1original$2");
		}

		if (domain === "media.blogto.com") {
			return src.replace(/\?.*/, "?w=-1");
		}

		if (domain === "hbb.afl.rakuten.co.jp") {
			if (src.match(/^[a-z]+:\/\/[^/]*\/hgp\/+\?/)) {
				newsrc = url.searchParams.get("pc") || url.searchParams.get("m");
				if (newsrc)
					return decodeuri_ifneeded(newsrc);
			}
		}

		if (amazon_container === "broadtime_thumbnails") {
			return src.replace(/(:\/\/broadtime_|\.amazonaws\.com\/broadtime_)thumbnails([^/]*\.amazonaws\.com)?\/[0-9]+\/+([0-9]+)\/+[0-9]+(?:%3A|:)[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1photo$2/$3");
		}

		if (domain === "knihy.abz.cz") {
			return src.replace(/(\/imgs\/+[^/]*\/+img_[0-9]+_)[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1orig$2");
		}

		if (domain_nowww === "readdork.com") {
			return src.replace(/\/images\/+_crop[0-9]+x[0-9]+\/+/, "/images/");
		}

		if (domain === "media-cdn.sueddeutsche.de") {
			return src.replace(/(\/image\/+sz\.[0-9]+\.[0-9]+)\/+.*$/, "$1");
		}

		if (domain_nowww === "swensonhomesolutions.com"
			) {
			return src.replace(/\/images\/+covers\/+/, "/images/realsize/");
		}

		if (domain === "media.arkansasonline.com") {
			return src.replace(/(\/img\/+photos\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[^/]*)__t[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nosub === "jintiankansha.me" && domain.match(/^img[0-9]*\./)) {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/get\?(?:.*?&)?src=(http[^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "cdjournal.com") {
			return src.replace(/\/image\/+jacket\/+[0-9]+\/+/, "/image/jacket/large/");
		}

		if (domain_nowww === "redwallpapers.com") {
			return src.replace(/\/public\/+redwallpapers-[^/]*\//, "/download/original/");
		}

		if (domain_nowww === "show-biz.by") {
			return src.replace(/(\/(?:(?:gallery|blog)_image|upimg_file)\/+[0-9]+\/+)[^/]+(\/+_v=[0-9a-f]+)?(?:[?#].*)?$/, "$1original$2");
		}

		if (domain_nowww === "songtexte.co") {
			return src.replace(/(\/Images\/+[^/]+\/+)Thumbs\/+/, "$1");
		}

		if (domain === "d2h1pu99sxkfvn.cloudfront.net") {
			newsrc = src.replace(/\/([PU])[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "/$10$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (host_domain_nowww === "depop.com") {
			return {
				url: src,
				need_blob: true
			};
		}

		if (domain === "img.joomcdn.net") {
			return src.replace(/(\/[0-9a-f]{20,})_[0-9]+_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1_original$2");
		}

		if (domain_nowww === "fashionbay.gr") {
			return src.replace(/\/thumber\.php\?(?:.*?&)?img=([^&]*).*?$/, "/$1");
		}

		if (domain_nowww === "ft.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/__origami\/+service\/+image\/+v2\/+images\/+[^/]*\/+(https?.*?)(?:[?#].*)?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "cdnimg.rg.ru") {
			return src.replace(/(\/img\/+content\/+.*)_t_[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "files.der-farang.com") {
			return src.replace(/\/cache\/+[0-9]+x[0-9]+\/+[0-9]+\/+[0-9]+\/+files\/+/, "/files/");
		}

		if (domain_nosub === "cdnvideo.ru" && domain.match(/^phototass[0-9]*\./)) {
			return src.replace(/\/(?:width|height)\/+[0-9]+_[0-9a-f]+\/+tass\/+/, "/tass/");
		}

		if (domain_nowww === "duonao.tv") {
			return src.replace(/(\/upload\/+[^/]*\/+[0-9]{8,})s(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "independent.bbvms.com") {
			return src.replace(/\/pthumbnail\/+[0-9]+\/+[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "/pthumbnail/0/0$1");
		}

		if (domain === "media-manager.starsinsider.com") {
			return src.replace(/\/gallery\/+[0-9]+\/+/, "/gallery/0/");
		}

		if (domain === "images.madame.de") {
			if (src.match(/^[a-z]+:\/\/[^/]*\/[^/.,]*(?:,[a-z]+=(?:[^,]+|[0-9.,]+)){1,}(?:\.[^/.]*)?(?:[?#].*)?$/)) {
				var filename = src.replace(/^[a-z]+:\/\/[^/]*\/+([^/.,]*).*?$/, "$1");
				var id = src.replace(/.*?,(id=[0-9a-z]+).*?$/, "$1");
				var brand = src.replace(/.*?,(b=[0-9a-z]+).*?$/, "$1");
				var ext = src.replace(/.*?(\.[^/.]*)(?:[?#].*)?$/, "$1");
				if (ext === src)
					ext = "";

				if (id !== src && brand !== src) {
					return {
						url: "https://images.madame.de/" + filename + "," + id + "," + brand + ",rm=sk" + ext,
						filename: filename + ext
					};
				}
			}
		}

		if (domain_nowww === "tportal.hr") {
			return src.replace(/\/media\/+thumbnail\/+[0-9]+x[0-9]+\/+/, "/media/thumbnail/w1000/");
		}

		if (domain_nowww === "nudecelebs.world" ||
			domain_nowww === "nudevideovixens.com" ||
			domain_nowww === "hollywoodnude.club" ||
			domain_nowww === "nakedebonycelebs.com" ||
			domain_nowww === "famousandnude.com" ||
			domain_nowww === "famousandnude.net" ||
			domain_nowww === "selenagomeznude.club" ||
			domain_nowww === "nude-scene.net" ||
			domain_nowww === "nudecelebrities.mobi" ||
			domain_nowww === "nudeasiancelebs.com" ||
			domain_nowww === "bigtitscelebrities.com" ||
			domain_nowww === "famousbombshells.com" ||
			domain_nowww === "latinacelebrities.com" ||
			domain_nowww === "selenagomeznude.club" ||
			domain_nowww === "xxxcelebscenes.com" ||
			domain_nowww === "nicolekidmannudes.com") {
			return src.replace(/(\/(?:g|gals|xhc)\/[0-9]{5,}-[^/]*\/[^/]*-[0-9]+)-(?:[0-9][0-9]+|tn)(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "kino.search.ch") {
			return src.replace(/(\/images\/+..\/+[^/]*)_m(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "teluguone.com") {
			return src.replace(/(\/uploadsExt\/+uploads\/+.*)_small(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "blog-news.it") {
			return src.replace(/(\/images\/+[^/]*\/+)tmb\/+/, "$1big/");
		}

		if (domain_nowww === "polecamfilm.pl") {
			return src.replace(/\/min(\/+[0-9]+_[a-z][0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain_nosub === "ltmcdn.com" && domain.match(/^t[0-9]*\.salir\./)) {
			return src.replace(/_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "_orig$1");
		}

		if (domain === "images.curved.de") {
			return src.replace(/\/article_teaser_md\/+/, "/article_detail_xl/");
		}

		if (domain === "ocio.diariodeibiza.es") {
			return src.replace(/__[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain === "img.movist.com") {
			newsrc = src.replace(/(:\/\/[^/]*\/)\?(?:.*?&)?img=([^&]*).*?$/, "$1data/imgroot$2");
			if (newsrc !== src)
				return decodeURIComponent(newsrc);
		}

		if (domain_nosub === "c-ctrip.com" && domain.match(/^youimg[0-9]*\./)) {
			return src.replace(/(\/target\/+[0-9a-z]+)(?:_[A-Z0-9]+){1,}(\.[^/.?#]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "china.com.cn") {
			return src.replace(/((?:\/img[0-9]*\/+[0-9]{4}(?:_[0-9]{2}){2}|nimg\/+[0-9]{8})\/+[0-9]+\/+(?:img_)?[0-9a-f]{20,})(?:_[0-9]+){2}(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "assets.jalantikus.com") {
			return src.replace(/\/assets\/+cache\/+[0-9]+\/+[0-9]+\/+userfiles\/+/, "/assets/cache/0/0/userfiles/");
		}

		if (domain === "res.idollook.cn") {
			return src.replace(/(\/[0-9]+\/+)(?:thumb_)?([0-9a-f]{20,}\.[^/.!?#]*)(?:[?#!].*)?$/, "$1$2");
		}

		if (domain_nosub === "altervista.org" ||
			(domain_nowww === "3xcelebs.com" && string_indexof(src, "/cache/") >= 0)) {
			return add_full_extensions(src.replace(/\/cache\/+(.*?)_[0-9]+(?:_c[wh][0-9]+)*(?:_thumb)?(\.[^/.]*)(?:[?#].*)?$/, "/albums/$1$2"));
		}

		if (domain_nowww === "haberkonseyi.com") {
			return src.replace(/\/news_t\/+/, "/news/");
		}

		if (domain_nowww === "snimai.com") {
			return src
				.replace(/(\/photos\/+[0-9]{8}\/+[0-9a-z]+\/+)thumbs\/+/, "$1")
				.replace(/(\/photos\/+[0-9]{8}\/+[0-9a-f]+)_thumb(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "fem-img.herokuapp.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/[0-9a-f]{20,}\/\?(?:.*&)?url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nosub === "qunliao.info" && domain.match(/^img[0-9]*\./)) {
			return src.replace(/\/[-0-9]+x[-0-9]+\/+/, "/-x-/");
		}

		if (domain_nosub === "phnx.pics") {
			return src.replace(/-[a-z_]+_[0-9]+(?:X[a-z_]+_[0-9]+)*(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain === "condenast-media.gcdn.co") {
			return src.replace(/\/[wht][-0-9]+(?:x[-0-9]+)?(?:[?#].*)?$/, "/w9999999999");
		}

		if (domain_nowww === "quvnoq.com") {
			return src.replace(/\/post\/+[a-z]+\/+/, "/post/original/");
		}

		if (domain_nosub === "kontraband.com" && domain.match(/^cdn[0-9]*\./)) {
			return src.replace(/(\/uploads\/+image\/+[0-9]{4}\/+(?:[0-9]{1,2}\/+){2})[a-z]+_([0-9a-z]+\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nosub === "52shijing.com" && domain.match(/^pic[0-9]*\./)) {
			return src.replace(/(\/file\/+[0-9]{8}\/+[0-9a-f]+)(?:_[0-9]+){2}(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "qmde.de" && domain.match(/^pix[0-9]*\./)) {
			return src.replace(/(\/pics\/+gallery\/+[^/]*)__w[0-9]+xh[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "nujhrcqkiwag1408085.cdn.ntruss.com") {
			return src.replace(/(\/static\/+upload\/+[^/]*\/+)[0-9]+x[0-9]+\/+/, "$1");
		}

		if (domain_nowww === "chisholm-poster.com") {
			return src.replace(/(:\/\/[^/]*\/)small\/+/, "$1large/");
		}

		if (domain_nowww === "doublage.qc.ca") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/pi\.php\?(?:.*?&)?img=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return urljoin(urljoin(src, "/files", true), decodeURIComponent(newsrc));
		}

		if (domain === "s3.bukalapak.com") {
			return src.replace(/(\/img\/+[0-9]+\/+)[wh]-[0-9]+\/+/, "$1original/");
		}

		if (amazon_container === "mz-prod") {
			return src.replace(/(\/uploads\/+photo\/+file\/+[0-9]+\/+)[a-z]+_([0-9a-f]{20,}[^/]*\.[^/.]*)(?:{?#].*)?$/, "$1$2");
		}

		if (domain === "capebreton.lokol.me") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/remote\.jpg\.ashx\?(?:.*?&)?urlb64=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return base64_decode(newsrc);
		}

		if (domain === "i.moveek.com") {
			return src.replace(/\/media\/+resized\/+[^/]*\/+/, "/media/");
		}

		if (domain === "i.ryt9.com" ||
			domain === "imageproxy.haaartland.com") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/[0-9]*x[0-9]*\/+http/, "http");
		}

		if (domain === "cinemas.nos.pt") {
			return src.replace(/\/RenderImage\.ashx\?(?:.*?&)?file=([^&]*).*?$/, "/RenderImage.ashx?file=$1");
		}

		if (domain === "img.pooq.co.kr") {
			return {
				url: src.replace(/(\/movieImg\/+.*?)_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2"),
				can_head: false // 400
			};
		}

		if (domain === "i.toynewsi.com") {
			return src.replace(/\/g\/+generated\/+(.*?)__scaled_[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "/g/albums/$1$2");
		}

		if (domain_nowww === "piletimaailm.com") {
			return src.replace(/(\/pictures\/+[0-9]+)\/+[a-z]+(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "meowg.com") {
			return src.replace(/(\.[a-z]+)-[0-9]+\.[^/.]*(?:[?#].*)?$/, "$1");
		}

		if (domain === "photos-epg.media-press.tv") {
			return src.replace(/(:\/\/[^/]*\/)(?:width|height)[0-9]+\/+/, "$1original/");
		}

		if (domain === "blog.jinbo.net") {
			return src.replace(/\/thumbnail\/+([0-9]+\/+[^/]*)\.w[0-9]+-h[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "/attach/$1$2");
		}

		if (domain === "cdnpuc.sinchew.com.my") {
			return src.replace(/\/t[0-9]*_[(][0-9X]+[)]([-0-9a-f]{30,}\.[^/.]*)(?:[?#].*)?$/,
							   "/$1");
		}

		if (domain_nowww === "lebanonfiles.com") {
			return src.replace(/\/files\/+thumbs\/+/, "/files/images/");
		}

		if (domain_nowww === "tahrirnews.com" ||
			domain === "img.eltahrer.com") {
			return src.replace(/(\/Content\/+Upload\/+)(?:med|slider)\/+/i, "$1large/");
		}

		if (domain === "images.f2fcdn.net") {
			return src.replace(/\/thumb\/+(.*?)(?:[?#].*)?$/, "/files/$1");
		}

		if (domain === "f.i.uol.com.br") {
			return src.replace(/(\/fotografia\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9a-f]+_[0-9]+_[0-9]+x[0-9]+)_xs(\.[^/.]*)(?:[?#].*)?$/,
							   "$1_rt$2");
		}

		if (domain === "image.thmeythmey.com") {
			return src.replace(/(\/pictures\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2})thumb[0-9]*\/+/, "$1");
		}

		if (domain_nowww === "galdump.com") {
			return {
				url: src.replace(/(\/images\/+websites\/+[0-9]+\/+[0-9]+\/+)thumb[0-9]*\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2"),
				headers: {
					Referer: "http://galdump.com/"
				}
			};
		}

		if (domain === "drp-images.nettavisen.no") {
			return src.replace(/(\/images\/+article\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9]+\/+[0-9]+\/)[^/]*\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/,
							   "$1/$2");
		}

		if (domain === "media.malaymail.com") {
			return src.replace(/\/resize_cache\/+(uploads\/+.*)-(?:small|seo)(\.[^/.]*)(?:[?#].*)?$/, "/$1$2");
		}

		if (domain_nowww === "alaraby.co.uk" ||
			domain_nowww === "almodon.com") {
			return {
				url: src.replace(/\/File\/+GetImageCustom\/+([-0-9a-f]{20,})\/+[0-9]+(?:\/+[0-9]+)?(?:[?#].*)?$/i,
								 "/File/Get/$1"),
				can_head: false // 403
			};
		}

		if (domain === "media.doisonghonnhan.vn") {
			return src.replace(/\/resize_[0-9]*x[0-9]*x[0-9]*\/+files\/+/, "/files/");
		}

		if (domain_nowww === "from-ua.com") {
			return src.replace(/(\/upload\/+articles\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2})[a-z]+\/+/, "$1original/");
		}

		if (domain_nosub === "onedio.com" && /^img/.test(domain)) {
			return src
				.replace(/:\/\/[^/]*\/id-([0-9a-f]+)\/.*?(\.[^/.]*)(?:[?#].*)?$/, "://img-3.onedio.com/img/$1$2")
				.replace(/\/img\/+(?:[0-9]+\/+bound\/+)?[0-9]*r[0-9]\/+/, "/img/");
		}

		if (domain_nowww === "sn.at") {
			return src.replace(/\/[0-9]*x[0-9]*(\/+[0-9]+\.[0-9]+\.[0-9]+)(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "meninafm.com.br") {
			return src.replace(/\/thumb_indice_noticias\.php\?(?:.*&)?img=([^&]*).*?$/, "/$1");
		}

		if (domain === "img.lemde.fr") {
			return src.replace(/(:\/\/[^/]*\/[0-9]{4}\/+(?:[0-9]{2}\/+){2})(?:[0-9]+\/+){8}([0-9a-f]+_[0-9]+-[0-9]+-[0-9]+\.[^/.]*)(?:[?#].*)?$/,
							   "$10/0/0/0/0/0/0/0/$2");
		}

		if (domain_nowww === "christian-dogma.com") {
			return src.replace(/\/im[0-9]+photos\/+([0-9]{8}\/+[0-9a-f]+\.[^/.?&#]*).*?$/, "/photos/$1");
		}

		if (domain_nosub === "feedme.id" && domain.match(/^cdn[0-9]*\./)) {
			return src.replace(/\/media\/+post\/+[a-z]+\/+/, "/media/post/");
		}

		if (domain_nowww === "businessinsider.in") {
			return src.replace(/\/thumb\/+([0-9]+\/+[^/?#]*)(?:[?#].*)?$/, "/photo/$1");
		}

		if (domain_nowww === "ucatx.cat") {
			return src.replace(/\/wallpic\/+[a-z]+\/+/, "/wallpic/full/");
		}

		if (domain === "img.sosanhgia.com") {
			return src.replace(/\/images\/+[0-9]+x[0-9]+\/+/, "/images/");
		}

		if (domain_nosub === "slatic.net") {
			return src.replace(/(\/p\/+[0-9a-f]+\.[^/._]*)_[^/]*(?:[?#].*)?$/, "$1");
		}

		if (domain === "nrt.ysc123.com.cn") {
			return src.replace(/(\/uploadImg\/+[0-9]{4}\/+[0-9]+\/+[0-9]+)_[0-9]+_[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nowww === "kpopnews.vn") {
			return src.replace(/(\/images\/+[0-9]{4}\/+(?:[0-9]{1,2}\/+){2}[0-9a-f]{15,}\/+)[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1default$2");
		}

		if (domain === "t.ylilauta.org") {
			return src.replace(/:\/\/t(\.[^/]*\/)/, "://i$1");
		}

		if (domain === "static.esea.net") {
			return src.replace(/(\/images\/+[^/]*\/+[0-9]+\.[0-9]+)_t(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "cdn.shazoo.ru") {
			return src.replace(/(:\/\/[^/]*\/)c[0-9]+x[0-9]+\/+/, "$1");
		}

		if (domain_nowww === "adultnode.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/thumb\.php\?(?:.*?&)?src=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "en.luxuretv.com") {
			return src.replace(/(\/media\/+[^/]*\/+)thumb-([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "image.bada.tv") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/files\/+crawling\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}imgur\/+([^/?#^]*)(?:[?#].*)?$/, "$1");
			if (newsrc !== src)
				return "https://i.imgur.com/" + newsrc;
		}

		if (domain_nowww === "fappenist.com") {
			return src.replace(/(\/Uploads\/+Media\/+[^/]*\/+[^/]*\/+[0-9]+\/+)[a-z]_([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nowww === "hitek.fr") {
			return src.replace(/(\/img\/+products\/+.*?\/+photos\/+)w_([^/]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "cdn.dribbble.com") {
			newsrc = src.replace(/(\/users\/+[0-9]+\/+screenshots\/+[0-9]+\/+[^/]+)_(?:[0-9]+x|teaser)(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
			obj = {
				url: newsrc
			};

			match = src.match(/\/screenshots\/+([0-9]+)\/+/);
			if (match) {
				obj.extra = {page: "https://dribbble.com/shots/" + match[1]};
			}

			return obj;
		}

		if (domain === "s.tmimgcdn.com") {
			newsrc = src.replace(/[?#].*/, "");
			if (newsrc !== src)
				return newsrc;

			return src
				.replace(/(\/scr\/+[0-9]+\/+(?:[^/]*_)?[0-9]+)-big(\.[^/.]*)$/, "$1-original$2")
				.replace(/(\/scr\/+[0-9]+\/+[0-9]+-)1300(_[0-9]+\.[^/.]*)(?:[?#].*)?$/, "$12560$2");
		}

		if (domain === "images.all-free-download.com") {
			newsrc = src.replace(/\/images\/+((?:templates|footage)_|graphic)(?:medium|large|thumb?)\/+/, "/images/$1original/");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/images\/+wallpapers_[a-z]+\/+/, "http://files.all-free-download.com/downloadfiles/wallpapers/original/");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "files.all-free-download.com") {
			return {
				url: src.replace(/\/downloadfiles\/+wallpapers\/+[^/]*\/+/, "/downloadfiles/wallpapers/original/"),
				headers: {
					Referer: "https://all-free-download.com/"
				}
			}
		}

		if (domain === "images.themevault.net") {
			return add_extensions(src.replace(/\/images\/+[a-z]+(-[0-9]+\.[^/.]*)(?:[?#].*)?$/, "/images/full$1"));
		}

		if (domain_nowww === "templatemo.com") {
			newsrc = src.replace(/\/thumbnails-[0-9]+\/+tm-([0-9]+-[^/]*)(?:[?#].*)?$/, "/screenshots/templatemo_$1");
			if (newsrc !== src)
				return newsrc.replace(/-/g, "_");
		}

		if (domain === "cdn.pocket-lint.com") {
			return src.replace(/\/r\/+s\/+[0-9]*x[0-9]*\/+assets\/+/, "/assets/");
		}

		if (domain === "image.isu.pub") {
			return src.replace(/(:\/\/[^/]*\/[0-9]+-[0-9a-f]{20,}\/+[^/]*\/+page_[0-9]+)_[_a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "static.techspot.com") {
			return src.replace(/(\/articles-info\/+[0-9]+\/+images\/+)T_([^/]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "mweb.co.za") {
			if (/\/MediaHandler\.ashx\?/.test(src)) {
				var portalid = url.searchParams.get("portalid");
				var moduleid = url.searchParams.get("moduleid");
				var mediaid = url.searchParams.get("mediaid");

				if (portalid && moduleid && mediaid) {
					return src.replace(/\?.*/, "?portalid=" + portalid + "&moduleid=" + moduleid + "&mediaid=" + mediaid);
				}
			}
		}

		if (domain_nowww === "gaminglives.com") {
			return src.replace(/\/wp-content\/+plugins\/+dynpicwatermark\/+DynPicWaterMark_ImageViewer\.php\?(?:.*?&)?path=([^&]*).*?$/,
							   "/wp-content/uploads/$1");
		}

		if (domain === "addons.cdn.mozilla.net") {
			return src.replace(/(\/user-media\/+previews\/+)[a-z]+\/+/, "$1full/");
		}

		if (domain_nosub === "zopix.net") {
			return src.replace(/\/image_upload\/+thumb\/+(?:[0-9]+[wh]_)?/, "/image_upload/");
		}

		if (domain_nowww === "jooinn.com") {
			return src.replace(/\/images[0-9]*_[0-9]*\/+/, "/images/");
		}

		if (domain === "prt.iza.ne.jp") {
			return src.replace(/(\/images\/+[0-9]{6}\/+ent[0-9]{10,}-)[a-z]([0-9]\.[^/.]*)(?:[?#].*)?$/,
							   "$1m$2");
		}

		if (domain_nosub === "plus613.net") {
			newsrc = src.replace(/\/images\/+thumbs\/+/, "/images/");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/\/page\/+hotlink\?(?:.*?&)?action=([^&]*).*?$/, "/images/$1");
			if (newsrc !== src)
				return newsrc;

			return {
				url: src,
				headers: {
					Referer: "http://www.plus613.net/"
				}
			};
		}

		if (domain === "img.milanuncios.com") {
			return src.replace(/(:\/\/[^/]*\/)fp\/+/, "$1fg/");
		}

		if (domain === "ssmscdn.yp.ca") {
			return src.replace(/\/image\/+resize\/+([-0-9a-f]{20,})\/+[^/]*\/+([^/]*)(?:[?#].*)?$/, "/image/original/$1/$2");
		}

		if (amazon_container === "oblok") {
			return src.replace(/(\/gallery_items\/+[0-9]+\/+)[^/]*\/+/, "$1original/");
		}

		if (domain_nowww === "salonfloris.ru") {
			return src.replace(/\/thumb\/+[^/]*\/+[0-9]+r[0-9]+\/+([0-9]+\/+)/, "/d/$1d/");
		}

		if (domain_nowww === "kmarket43.ru") {
			return src
				.replace(/\/media\/+cache\/+[0-9]+x[0-9]+\/+offers\/+/, "/media/offers/")
				.replace(/(\.[^/.]+)\.[^/.]+\.[^/.]+(?:[?#].*)?$/, "$1");
		}

		if (domain === "assets.perfecte.ro") {
			return src
				.replace(/(\/image_galleries\/+[0-9]+\/+[^/]*)_size[0-9]*(\.[^/.]*)(?:[?#].*)?$/, "$1$2")
				.replace(/(\/image_galleries\/+[0-9]+\/+)t_size[0-9]*\/+thumb_/, "$1");
		}

		if ((domain_nosub === "cuore.es" ||
			domain_nosub === "stilo.es" ||
			domain_nosub === "woman.es")
			&& domain.match(/^img[0-9]*\./)) {
			return src.replace(/(\/(?:[0-9a-f]{2}\/+){3}[^/]*)-[0-9]+(?:x[0-9]+)?(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "100posto.hr") {
			return src.replace(/\/datastore\/+imagestore\/+[0-9]+x[0-9]+\/+[0-9]+x[0-9]+_/, "/datastore/imagestore/original/");
		}

		if (domain_nowww === "hot-celebs.co") {
			return src.replace(/(\/image\/+[0-9]+)\/+[^/]*(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "findnudecelebs.com" ||
			domain_nowww === "searchcelebrityhd.com" ||
			domain_nowww === "pluscelebs.com" ||
			domain_nowww === "imgtrend.com" ||
			domain_nowww === "chopris.com" ||
			domain_nowww === "leadceleb.com" ||
			domain_nowww === "ultraceleb.com" ||
			domain_nowww === "freshfappening.com" ||
			domain_nowww === "allcelebs.club") {
			newsrc = src.replace(/\/(small)\/+\1-([^/]+\.[^/.]*)(?:[?#].*)?$/, "/$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "celebritying.com") {
			return src
				.replace(/(\/pictures\/+[^/]*\/+[^/]*)-tn(-[0-9]+)?(\.[^/.]*)(?:[?#].*)?$/, "$1$2$3")
				.replace(/(\/images\/+[^/]*)-tn(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "daily-celebvideos.com") {
			return src.replace(/00([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "nitrovideo.com") {
			return src.replace(/\/thumbs\/+tn_/, "/pictures/");
		}

		if (domain_nowww === "isecretarylegs.com" ||
			domain_nowww === "pussynclit.com" ||
			domain_nowww === "ulingerie.com" ||
			domain_nowww === "vaginavulva.com") {
			return src.replace(/\/thumbs\/+tn_([^/]+)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nowww === "celebact.com") {
			return src.replace(/\/celebrity-thump-/, "/naked-celebrity-");
		}

		if (domain_nowww === "nudecelebs-a-z.com") {
			return src.replace(/(:\/\/[^/]*\/[^/]*\/+)tn_/, "$1");
		}

		if (domain_nowww === "nudecelebspics.net") {
			return src.replace(/\/preview\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nowww === "thephuketnews.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/image-[.0-9]+\/+image\.php\?(?:.*?&)?image=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nosub === "naharnet.com" && domain.match(/^images[0-9]*\./)) {
			return src.replace(/(\/images\/+[0-9]+\/+)[wh][0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1original$2");
		}

		if (domain === "img.beritasatu.com") {
			return src.replace(/\/cache\/+([^/]*)\/+[0-9]+x[0-9]+-[0-9]+\/+/, "/$1/");
		}

		if (domain === "nnimgt-a.akamaihd.net") {
			return src.replace(/\/transform\/+v1\/+[a-z]+\/+(frm\/+[^/]*\/+[^/]*\.[^/.]*)\/+[^/]*?(\.[^/.]*)?(?:[?#].*)?$/,
							   "/transform/v1/resize/$1/w0_h0_fscale$2");
		}

		if (domain_nosub === "productserve.com" && domain.match(/^images[0-9]*\./)) {
			match = src.match(/^[a-z]+:\/\/[^/]*\/\?(?:.*?&)url=([^&]*).*?$/);
			if (match) {
				newsrc = match[1];
				match = newsrc.match(/^ssl(?::|%3[Aa])(.*)/)
				if (match) {
					return "https://" + decodeURIComponent(match[1]);
				}
			}
		}

		if (domain_nowww === "sunhome.ru" ||
			domain === "i.sunhome.ru") {
			return src.replace(/(\/wallpapers\/+[0-9]+\/+[^/]*)\.(?:[0-9]+x[0-9]+|[a-z])(\.[^/.]*)(?:[?#].*)?$/,
							   "$1.orig$2");
		}

		if (domain_nowww === "i-gency.ru") {
			return src.replace(/(\/[0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9]+\/+)first_([0-9a-f]{15,}\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "photos.stories-porno.net") {
			return src.replace(/(:\/\/[^/]*\/)preview\/+/, "$1photos/");
		}

		if (domain_nowww === "bazos.sk") {
			return src.replace(/(\/img\/+[0-9]+)t\/+/, "$1/");
		}

		if (domain_nowww === "pegitboard.com") {
			return src.replace(/(\/pics\/+)(?:[^/]*\/+)?(?:[a-z]-)?([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "media.tio.ch") {
			return src.replace(/\/images\/+thumb\/+/, "/images/");
		}

		if (domain === "cdn-images.oovvuu.com") {
			return src.replace(/(:\/\/[^/]*\/)[0-9]+x[0-9]+\/+/, "$10x0/");
		}

		if (domain_nowww === "noihirek.hu") {
			return src
				.replace(/(:\/\/[^/]*\/)kep\.php\?(?:.*?&)?kep=([^&]*).*?$/, "$1$2")
				.replace(/(:\/\/[^/]*\/)thumb_pictures\/+/, "$1pictures/");
		}

		if (domain_nosub === "worldsex.com" && domain.match(/^cdn[0-9]*/)) {
			return src.replace(/_[0-9]+x(?:@[0-9]+x)?(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "famitsu.com") {
			regex = /(\/images\/+(?:[0-9]{3}\/+){3})[tly]_([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/;
			if (regex.test(src)) {
				return [
					src.replace(regex, "$1z_$2"),
					src.replace(regex, "$1y_$2"),
					src.replace(regex, "$1l_$2")
				];
			}

			return src.replace(/(\/images\/+s[0-9]{6,})s(\.[^/.]*)(?:[?#].*)?$/, "$1o$2");
		}

		if (domain_nowww === "sexiestpicture.com") {
			return src.replace(/(\/upload\/+Pictures\/+)thumbs\/+([^/]+)_[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1storage/$2$3");
		}

		if (domain_nowww === "woman-project.com") {
			return src.replace(/(\/uploads\/+[0-9]+\/+)thumb\/+/, "$1");
		}

		if (domain_nowww === "hamaraphotos.com") {
			return src.replace(/(\/albums300\/+wpw-[0-9]+\/+)thumb_/, "$1fullsize_");
		}

		if (domain_nosub === "vetton.ru" && domain.match(/^img[0-9]*\./)) {
			return src.replace(/(\/upl\/+[0-9]+\/+[0-9]+\/+)[0-9]+\/+/, "$1");
		}

		if (domain === "edu.glogster.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/proxy\?(?:.*?&)?url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "imagez.tmz.com") {
			return src.replace(/(\/[0-9a-f]{30,})_[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "offnews.bg" && domain.match(/^i[0-9]*\./)) {
			return src.replace(/(\/[^/._]+)_[*0-9]+x[*0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "slender-model.com") {
			return src.replace(/(\/images\/+[^/]*\/+[^/]*[0-9]+)s(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "24eropixel.net") {
			return src.replace(/(\/uploads\/+images\/+(?:[0-9]+\/+){3,})(?:[0-9]+x[0-9]+|desktop_thumbs)\/+/,
							   "$1original/");
		}

		if (domain === "cdn.fashionsnap.com") {
			return src.replace(/:\/\/[^/]*\/collection\/+assets_c\/+([0-9]{4}\/+[0-9]{2}\/+[^/]*)-thumb-[auto0-9]+x[auto0-9]+-[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "://res.fashionsnap.com/image/upload/media2/$1$2");
		}

		if (domain_nowww === "xoxodigital.com") {
			return src.replace(/(\/content\/+post\/+[0-9]+\/+)thumbs\/+/, "$1");
		}

		if (domain_nosub === "xdn.vn" && /^[0-9]+\.mm\.baomoi\./.test(domain)) {
			return src.replace(/(:\/\/[^/]*\/)[wh][0-9]+_r[0-9]*\/+/, "$1");
		}

		if (domain_nosub === "meinbezirk.at" && /^media[0-9]*\./.test(domain)) {
			return src.replace(/(\/article\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9]+\/+[0-9]+)_[A-Z]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1_NATIVE$2");
		}

		if (domain_nowww === "elporvenir.mx") {
			newsrc = src.replace(/(\/imagenes\/+noticias\/+)thumb\/+((?:[0-9]{2}\/+){3})[a-z]+_([0-9]{6}_)/,
								 "$1original/$2$3");
			if (newsrc !== src)
				return newsrc;

			return src.replace(/\/imagenes\/+noticias\/+portada\/+/, "/imagenes/noticias/original/");
		}

		if (domain_nowww === "newsbeast.gr") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/resizeme\.php\?(?:.*?&)?src=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "s.nbst.gr") {
			return src.replace(/(\/files\/.*)\.thumbnail(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "image.newsis.com") {
			return src.replace(/(:\/\/[^/]*\/[0-9]{4}\/+(?:[0-9]{2}\/+){2}NISI[0-9]+_[0-9]+_)thm(\.[^/.]*)(?:[?#].*)?$/,
							   "$1web$2");
		}

		if (domain_nowww === "techpowerup.com") {
			return src.replace(/(\/img\/+[^/]+)_thm(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "plaza.jp.rakuten-static.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/api\/+Proxy\.php\?(?:.*?&)?a=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "5true.net") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/img-prx\.php\?(?:.*?&)?u=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return base64_decode(newsrc);
		}

		if (domain === "imb.veooz.com") {
			return src.replace(/-small(\.[^/.]*)(?:[?#].*)?$/, "-large$1");
		}

		if (domain_nowww === "criturk.com") {
			return src.replace(/(\/image\/+upload\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[a-z]+_[0-9a-f]{20,})-[0-9]+X[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "cdn.20m.es") {
			return src.replace(/\/recortes\/+([0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9]+)(?:-[0-9]+){1,2}(\.[^/.]*)(?:[?#].*)?$/,
							   "/$1$2");
		}

		if (domain === "tgp.inthecrack.com") {
			return {
				url: src.replace(/\/assets\/+images\/+hosted\/+galleries\/+([-0-9a-f]{20,})/, "/assets/images/hosted/galleries/full/$1"),
			};
		}

		if (domain === "content.pornstarplatinum.com") {
			return src.replace(/\/images\/+thumb([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/images/pic$1");
		}

		if (domain === "t.auntmia.com" ||
			domain === "t.sexywhitepussy.com") {
			return src.replace(/(\/[0-9]+_[0-9]+)(\.[^/.]*)(?:[?#].*)?$/, "$1b$2");
		}

		if (domain_nowww === "giveawayoftheday.com") {
			return src.replace(/(\/wp-content\/+uploads\/+[0-9]{4}\/+[0-9]{2}\/+[0-9a-f]{20,})_[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nowww === "nato.int") {
			return src.replace(/(\/assets\/+pictures\/+[^/]*\/+[^/]*)_rdax_[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain_nosub === "pbase.com" && /^a[0-9]*\./.test(domain) &&
			options && options.cb && options.do_request) {
			var id = src.replace(/^[a-z]+:\/\/[^/]*\/[a-np-z][0-9]+\/+.*\/+([0-9]+)\.[^/.]+\.(?:[^/.]*\.)?[^/.]*(?:[?#].*)?$/, "$1");
			if (id !== src) {
				options.do_request({
					method: "GET",
					url: "https://pbase.com/image/" + id + "/original",
					onload: function(resp) {
						if (resp.readyState === 4) {
							var match = resp.responseText.match(/<(?:img|IMG)\s+class=["']display\s*["']\s+src=["'](.*?)["']/)
							if (match) {
								options.cb(match[1]);
							} else {
								options.cb(null);
							}
						}
					}
				});

				return {
					waiting: true
				};
			}
		}

		if (amazon_container === "photo-net-production-images" ||
			domain === "d6d2h4gfvy8t8.cloudfront.net") {
			return src.replace(/(\/[0-9]+)-[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1-orig$2");
		}

		if (domain_nowww === "thoimoi.vn" ||
			domain_nowww === "vir.com.vn" ||
			domain_nowww === "petrotimes.vn") {
			return src.replace(/(\/stores\/+news_dataimages\/+[^/]*\/+[0-9]{6}\/+[0-9]+\/+[0-9]+\/+)[a-z_]+\/+/, "$1");
		}

		if (domain_nosub === "energyfm.ru" && domain.match(/^(?:cdn[0-9]*|www)\./)) {
			if (/\/proxy\/+vardata\//.test(src)) {
				return src.replace(/:\/\/[^/]*\/proxy\/+vardata\/+(.*?)(?:[?#].*)?$/, "://www.energyfm.ru/vardata/$1");
			}

			return src.replace(/(\/files\/+[0-9]+\/+[0-9]+\/+[^/]*?)\?.*/, "$1?w=-");
		}

		if (domain_nowww === "chacuncherchesonfilm.fr") {
			return src.replace(/(\/storage\/+[^/]*\/+[0-9a-f]{2}\/+[0-9a-f]+)-[0-9_]+x[0-9_]+(\.[^/.]*(?:[?#].*)?)$/,
							   "$1$2");
		}

		if (domain === "structurecms-staging-psyclone.netdna-ssl.com") {
			return src.replace(/(\/media\/+picture\/+(?:[0-9a-f]{4}\/+){6})[a-z]+_/, "$1original_");
		}

		if (domain_nowww === "reforma.com" ||
			domain_nowww === "mural.com" ||
			domain_nowww === "cancha.com" ||
			domain_nowww === "elnorte.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/aplicacioneslibre\/+compartir\/+ImageTransformer.aspx\?(?:.*?&)?img=(http[^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "madtv.com.cy") {
			return src.replace(/(\/assets\/+modules\/+wnp\/+articles\/+[0-9]{6}\/+[0-9]+\/+images\/+)[ms]_([^/]*)(?:[?#].*)?$/,
							   "$1b_$2");
		}

		if (domain === "d15hng3vemx011.cloudfront.net") {
			if (src.match(/\/attachment\/+[^/.]*\.[a-z]+/)) {
				return [
					src.replace(/(\/attachment\/+[^/.]*)\.[a-z]+/, "$1"),
					src.replace(/(\/attachment\/+[^/.]*)\.[a-z]+/, "$1.large"),
				];
			}
		}

		if (domain === "static.flaixfm.cat") {
			return src.replace(/([./])(?:pw-(?:640|480)|cwh-(?:320-320|160-160))\1/, "$1pwh-2560-1440$1");
		}

		if (domain_nowww === "music.hu") {
			return src.replace(/(\/pictures\/+[0-9]+\/+)resized\/+([^/]*?)(?:_[wh][0-9]+)+(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2$3");
		}

		if (domain === "dspvdh6gst59b.cloudfront.net") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/(http.*?)(?:[?#].*)?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "factnews.mn") {
			return src.replace(/\/images\/+news\/+crop[0-9]+\/+/, "/images/news/");
		}

		if (domain_nowww === "videos-de-celebrites.com") {
			return src.replace(/\/tn\/+tn_/, "/");
		}

		if (domain_nowww === "friends.kz") {
			return src.replace(/\/uploads\/+posts\/+thumbs\/+/, "/uploads/posts/");
		}

		if (domain_nowww === "sama-sama-sama.ru") {
			return src.replace(/(\/starimp\/+[^/]*\/+)p(s[0-9]+\.[^/.]*)/, "$1$2");
		}

		if (domain === "img.index.hu") {
			return src.replace(/(\/imgfrm\/+(?:.\/+){4})(?:THM|BIG|MED)_/, "$1IMG_");
		}

		if (domain_nowww === "eventpixx.org") {
			return src.replace(/(\/image\/+[0-9a-f]+)_[0-9]*x[0-9]*(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "images.hgmsites.net") {
			return src.replace(/\/sml\/+([^/]*)_s(\.[^/.]*)(?:[?#].*)?$/, "/hug/$1_h$2");
		}

		if (domain === "cdn.lifebuzz.com") {
			return src.replace(/(\/images\/+[0-9]+\/+lifebuzz-[0-9a-f]{20,})-[a-z]+_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1-original$2");
		}

		if (domain === "jurnalul.antena3.ro") {
			return src.replace(/\/thumbs\/+[^/]*\/+([0-9]{4}\/+(?:[0-9]{2}\/+){2})[^/]*-([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/pictures/$1$2");
		}

		if (domain_nowww === "anime1.com") {
			return src.replace(/(\/img\/+content\/+[^/]*\/+[^/]*)-[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "animetoon.org") {
			return src.replace(/(\/images\/+[^/]*\/+)small\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1big/$2");
		}

		if (domain_nowww === "anime-tourism.jp") {
			return src.replace(/\/title_th\/+/, "/title_image/");
		}

		if ((domain_nosub === "filmweb.pl" && /^(?:ssl-)?gfx\./.test(domain)) ||
			domain_nosub === "fwcdn.pl") {
			return src.replace(/(\/[0-9]+(?:_[0-9]+)?\.)[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$$$2");

		}

		if (domain_nowww === "fantlab.ru") {
			return src.replace(/(\/blogfiles\/+b[0-9]+\/+img\/+[0-9]+)_sm(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "animeyt.org") {
			return src.replace(/(\/uploads\/+[^/]*\/+)optimized\/+(?:thumb\/+)?/, "$1");
		}

		if (domain_nowww === "otakotaku.com") {
			return src.replace(/(\/asset\/+img\/+[^/]*\/+)thumb\/+[0-9]+x[0-9]+\/+/, "$1");
		}

		if (domain === "img.topddl.net") {
			return src.replace(/(\/images\/+[0-9a-f]{10,})\.th(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "monoschinos.com") {
			return src.replace(/\/image\/+imagen\/+[0-9]+\/+[0-9]+\/+/, "/assets/img/serie/imagen/");
		}

		if (domain === "a.atcdn.org") {
			return src.replace(/\/c\/+[a-z]+_([^/.]*\.[^/.]*)(?:[?#].*)?$/, "/c/$1");
		}

		if (domain_nowww === "you-anime.ru") {
			return src.replace(/\/anime-images\/+posters\/+preview\/+/, "/anime-images/posters/");
		}

		if (domain_nowww === "tapcuoi.com") {
			return src.replace(/(\/wp-content\/+anime\/+cover\/+)tbn\/+([0-9]+_[0-9]+)_[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2$3");
		}

		if (domain_nowww === "komyulife.com") {
			return src.replace(/(\/media\/+[^/]*\/+[^/]*\/+)[0-9]+x[0-9]+\/+/, "$1");
		}

		if (domain_nowww === "animeon.pl") {
			return src.replace(/(\/img\/+[^/]*\/+[0-9]+\/+)thumbs\/+/, "$1");
		}

		if (domain_nowww === "4icdn.com") {
			newsrc = src.replace(/^([a-z]+:\/\/[^/]*\/)\?(?:.*&)?src=([^&]*).*?$/, "$2");
			if (newsrc !== src)
				return urljoin(src, decodeuri_ifneeded(newsrc), true);
		}

		if (domain === "cdn.legsjapan.com") {
			return src.replace(/(\/photo\/+data\/+[^/]*\/+[^/]*-[0-9]+)s(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "explicithd.com" && /^static-cdn/.test(domain)) {
			return src.replace(/(\/content\/+[^/]*\/+[0-9a-f]{20,}\/+)thumbs\/+/, "$1files/");
		}

		if (domain_nowww === "lbfmaddiction.com") {
			return src.replace(/(\/fhg\/+[^/]*\/+[^/]*\/+[^/]*)_small(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "thaicuties.net") {
			return {
				url: src.replace(/\/images\/+tn_([^/]*)_([a-zA-Z]+)\.[^/.]*(?:[?#].*)?$/, "/images/$1.$2"),
				head_wrong_contenttype: true
			};
		}

		if (domain === "fhg.propertysex.com") {
			return src.replace(/(:\/\/[^/]*\/+[^/]*\/+)t([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "manga-news.com") {
			return src.replace(/(\/public\/+images\/+events\/+)\.([^/]*?)_[a-z](\.[^/.]*)(?:[?#].*)?$/, "$1$2$3");
		}

		if (domain_nowww === "filmsomniac.com") {
			return src.replace(/(\/images\/+covers\/+films-new\/+)[0-9]+\/+([0-9]+\/+[0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "film-download.club") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+uploads\/+(https?-dk-.*?)\.[^/]*(?:[?#].*)?$/, "$1");
			if (newsrc !== src) {
				return newsrc
					.replace(/-dk-/g, ":")
					.replace(/-sl-/g, "/")
					.replace(/-kr-/g, ".");
			}
		}

		if (domain_nowww === "thaishop.in.th") {
			return src.replace(/(:\/\/[^/]*\/+)thumb\.php\?(?:.*?&)?(id=[0-9]+).*?$/, "$1image.php?$2");
		}

		if (domain_nowww === "animes-portal.info") {
			return src.replace(/\/thumbs\/+[0-9]+\/+/, "/resources/");
		}

		if (domain === "cdn.staticaly.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+img\/+([^/]*\.[^/]*\/+)/, "$1");
			if (newsrc !== src)
				return add_http(newsrc);
		}

		if (domain === "assets.fireside.fm") {
			return src.replace(/(\/images\/+[0-9a-f]\/+[-0-9a-f]{20,}\/+(?:[^/]*\/+[0-9a-f]\/+[-0-9a-f]{20,}\/+)?[^/]*)_(?:thumb|small|medium)(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "disk.mediaindonesia.com") {
			return src.replace(/\/thumbs\/+[-0-9]+x[-0-9]+\/+/, "/files/");
		}

		if (domain === "images.lindependant.fr") {
			return src.replace(/(\/api\/+v[0-9]+\/+images\/+view\/+[0-9a-f]{15,}\/+)[^/]*\/+/, "$1");
		}

		if (domain_nowww === "kurir.rs") {
			return src.replace(/(\/data\/+images\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9]+\/+[^/]*)_[a-z]{2}(?:-[a-z]+)?(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "reunion.orange.fr") {
			return src.replace(/\/images\/+[0-9]+x[0-9]+\/+actu\/+/, "/IMG/");
		}

		if (domain === "cdn.sahafahn.net" ||
			domain === "cdn.pressbee.net" ||
			domain === "cdn.24press.net") {
			match = src.match(/^[a-z]+:\/\/[^/]*\/img(?:-|\/+)[0-9]+-[0-9]+\/([^/]{10,})\.[^/.]*(?:[?#].*)?$/);
			if (match) {
				var str = base64_decode(match[1]);
				newsrc = str.replace(/.*?\"(https?:[^"]*).*?$/, "$1");
				if (newsrc !== str) {
					return newsrc;
				}
			}
		}

		if (domain_nosub === "keponews.com" && domain.match(/^scdn/)) {
			match = src.match(/^[a-z]+:\/\/[^/]*\/scontent\/+([^/.]{15,})(?:[?#].*)?$/);
			if (match) {
				var str = base64_decode(match[1]);
				newsrc = str.replace(/^[0-9]+\/+(https?:.*)$/, "$1");
				if (newsrc !== str) {
					return newsrc;
				}
			}
		}

		if (domain === "media.kleinezeitung.at" ||
			domain === "media.diepresse.com") {
			return src.replace(/\/images\/+uploads_[a-z]?[0-9]+\/+/, "/images/uploads/");
		}

		if (domain === "republic-imagekit.azureedge.net") {
			return src.replace(/(\/republic-prod\/+stories\/+[^/]*\/+)[^/]*\/+tr:[^/]*\/+/, "$1");
		}

		if (domain_nowww === "newsstandhub.com") {
			return src.replace(/(\/files\/+[^/]*\/+images\/+[^/]*\/+)hdpi\/+/, "$1");
		}

		if (domain_nowww === "urbanhit.fr" ||
			domain_nowww === "xplicitkontemp.com") {
			return src.replace(/(\/upload\/+[^/]*\/+)(?:normal|mini|main)\/+/, "$1original/");
		}

		if (domain_nowww === "choualbox.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+mini\.php\?(?:.*?&)?src=([^&]*).*?$/, "$1");
			if (newsrc !== src) {
				return urljoin(src, decodeURIComponent(newsrc), true);
			}
		}

		if (domain === "im.milliyet.com.tr") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+[0-9]+\/+[0-9]+\/+([^/]*)\.[^/.]*(?:[?#].*)?$/, "$1");
			if (newsrc !== src) {
				return add_http(newsrc.replace(/-\.-/g, "/"));
			}
		}

		if (domain_nosub === "indiatvnews.com" && /^resize[0-9]*\./.test(domain)) {
			return src.replace(/\/resize\/+oldbucket\/+[-0-9]+_[-0-9]+\/+/, "/oldbucket/");
		}

		if (domain === "girabsas.demo.vincolo.com") {
			newsrc = src.replace(/.*\/tools\/+image\.php\?(?:.*?&)?p=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return urljoin(src, decodeURIComponent(newsrc), true);

				return src.replace(/(\/files\/+image\/+[0-9a-f]\/+[0-9a-f]+\/+[^/.]*)_[0-9]+_[0-9]+!(\.[^/.?#]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "seffafgazete.com") {
			return src.replace(/\/uploads\/+sized\/+([0-9]{4}\/+[0-9]{2}\/+[0-9a-f]+)-[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "/uploads/o/$1$2");
		}

		if (domain === "cdn.posh24.se") {
			return src.replace(/\/images\/+:[a-z]+\/+/, "/images/:original/");
		}

		if (domain_nowww === "kinoafisha.ua") {
			match = src.match(/(\/upload\/+[0-9]{4}\/+[0-9]{2}\/+[^/]*\/+[0-9]+\/+fotos\/+)/);
			if (match) {
				newsrc = src.replace(/(\/+fotos\/+)[a-z]+\/+([^/]*)(?:[?#].*)?$/, "$1$2");
				if (newsrc !== src)
					return newsrc;
				if (!/\/src_[^/]*(?:[?#].*)?$/.test(src))
					return src.replace(/(\/fotos\/+)([^/]*)(?:[?#].*)?$/, "$1src_$2");
			}
		}

		if ((domain_nosub === "akamaized.net" && /^cache[0-9]*-ebookjapan\./.test(domain)) ||
			domain === "haishin.ebookjapan.jp") {
			return src.replace(/\/contents\/+thumb\/+[ms]\/+/, "/contents/thumb/l/");
		}

		if (domain === "aws-cf.imdoc.fr" ||
			domain === "aws-cf.caradisiac.com") {
			return src.replace(/(\/prod\/+photos\/+(?:[0-9]+\/+){5})(?:tns[0-9]*|img)-/, "$1big-");
		}

		if (domain === "asobokey.x.fc2.com") {
			return src.replace(/(\/image\/+[0-9]{4,})_thumb(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "hbox.jp") {
			return src.replace(/(\/image[0-9]*\/+content\/+[0-9]+\/+)sp_thumbnail_/, "$1");
		}

		if (domain === "storage.ws.pho.to") {
			return src.replace(/(\/[0-9a-f]{20,})_[a-z](\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "freakolla.com") {
			return src.replace(/\/THG\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/IMG/$1");
		}

		if (domain_nosub === "breath-takers.com" ||
			domain === "galleries.sophiassexylegwear.com" ||
			domain_nosub === "girlfolio.com") {
			return src.replace(/(\/assets\/+images\/+.*\/)(?:small|medium|large)\/+/, "$1orig/");
		}

		if (domain_nosub === "thepluginz.com" && /^img\./.test(domain)) {
			return src.replace(/(\/photos\/+[^/]*\/+)tn_/, "$1");
		}

		if (domain === "galleries.gloryhole-initiations.com") {
			return src.replace(/\/pics\/+tn\/+/, "/pics/pic/");
		}

		if (domain === "g.misslk.com") {
			return src.replace(/\/tn\/+/, "/full/");
		}

		if (domain_nosub === "radgalleries.com") {
			return src.replace(/\/thumbnails\/+t([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain === "img.maximummedia.ie" ||
			domain === "img.resized.co") {
			match = src.match(/^[a-z]+:\/\/[^/]*\/[^/]*\/+([^/]{50,})(?:\/+[^/]*)?(?:[?#].*)?$/);
			if (match) {
				var json = JSON_parse(base64_decode(match[1]));
				if (json.data) {
					var data = JSON_parse(json.data);
					if (data.url)
						return data.url;
				}
			}
		}

		if (domain === "imagevars.gulfnews.com") {
			return src.replace(/(_[0-9a-f]{6,})_[-a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1_original-ratio$2");
		}

		if (domain === "az743702.vo.msecnd.net" ||
			domain === "cdn.ko-fi.com") {
			newsrc = src.replace(/(\/useruploads\/+)((?:[a-z_]+_)?[-0-9a-f]{20,}(?:[^/.]+)?)_post(\.[^/.]*)(?:[?#].*)?$/, "$1$2_display$3");
			if (newsrc !== src)
				return newsrc;

			return src.replace(/(\/useruploads\/+)(?:[a-z]+_)?([-0-9a-f]{20,}(?:[a-z]+)?)(?:_[a-z]+)?(\.[^/.]*)(?:[?#].*)?$/, "$1$2$3");
		}

		if (domain_nowww === "silviomessina.pw" ||
			domain_nowww === "bevilacqua.me" ||
			domain_nowww === "geodomehome.org" ||
			domain_nowww === "soamasterclass.com" ||
			domain_nowww === "royal-city.com" ||
			domain_nowww === "ulsterscotsacademy.org" ||
			domain_nowww === "newkunci.co" ||
			domain_nowww === "xianweizhu.com" ||
			domain === "www.edu.kustantiblogs.web.id") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+images\/+[^/]*__([^-/._]{30,})\.[^/.]*(?:[?#].*)?$/, "$1");
			if (newsrc !== src)
				return base64_decode(newsrc);
		}

		if (domain_nowww === "xxxsexy.xyz" ||
			domain_nowww === "pornxxxbest.xyz") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+images\/+([^.]{50,})\.[^/.]*(?:[?#].*)?$/, "$1");
			if (newsrc !== src) {
				var splitted = newsrc.split("/");
				newsrc = base64_decode(splitted[0]);
				if (splitted.length > 1)
					newsrc += "?" + base64_decode(splitted[1]);
				return newsrc;
			}
		}

		if (domain_nosub === "soumeiwang.com" ||
			domain_nosub === "shianwang.com" ||
			domain_nosub === "baobaoyuer.com" ||
			domain_nosub === "daguaw.com" ||
			domain_nosub === "ditiw.com" ||
			domain_nosub === "chudaowang.com" ||
			domain_nosub === "desangw.com" ||
			domain_nosub === "dalangw.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+img\/+([^-/._]{30,})\.[^/.]*(?:[?#].*)?$/, "$1");
			if (newsrc !== src)
				return base64_decode(newsrc);
		}

		if (domain_nowww === "kabelversand.info" ||
			domain_nowww === "teamupforkansaskids.com" ||
			domain_nowww === "gbracingpix.com" ||
			domain_nowww === "kickin.info" ||
			domain_nowww === "catmario.info" ||
			domain_nowww === "greateasternwallgallery.com" ||
			domain_nowww === "stepanakert.info" ||
			domain_nowww === "modellini.info" ||
			domain_nowww === "carltonstaffing.info") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+pics\/+([^-/._]{30,})(?:\.[^/.]*)?(?:[?#].*)?$/, "$1");
			if (newsrc !== src)
				return base64_decode(newsrc);
		}

		if (domain_nosub === "gaanacdn.com" && /^a[0-9]*\./.test(domain)) {
			return src.replace(/(\/artists\/+[^/]*\/+[^/]*\/+size_)[sm]_/, "$1l_");
		}

		if (domain_nosub === "baidu-img.cn" && /^cdn[0-9]*\./.test(domain)) {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/timg\?(?:.*?&)?src=(http[^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "pic.yupoo.com") {
			return src.replace(/(\/[0-9a-f]{6,}\/+)(?:small|medium)(\.[^/.]*)(?:[?#].*)?$/, "$1large$2");
		}

		if (domain === "img.popnroll.tv" ||
			domain === "images.dappei.com") {
			return src.replace(/(\/uploads\/+[^/]*\/+image\/+[0-9]+\/+)(?:thumb|medium|large)_/, "$1");
		}

		if (domain_nosub === "adulttrade.net") {
			return src.replace(/(_pics_galleries\/.*\/)tn_([^/]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "thepaysites.com") {
			return src.replace(/(\/[0-9]+\/+[^/]+\/+)thumb_([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "i.myass.ru") {
			return src.replace(/(\/[0-9]{6}\/+[0-9]+\/+)[a-z]\/+([0-9]+[0-9a-f]+\.[^/.]*)(?:[?#].*)?$/, "$1f/$2");
		}

		if (domain === "img.cactusboy.com") {
			return src.replace(/(\/[0-9a-f]\/+[0-9a-f]{2}\/+[0-9a-f]{10,}\/+)tn_([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "images.eastbabes.com" ||
			domain === "images.ebonygirls.sexy" ||
			domain === "images.nakedblackbabes.pics") {
			return src.replace(/(\/p\/+[^/]*\/+)th\/+/, "$1");
		}

		if (domain_nowww === "simdunyasi.com" && src.match(/[?&]qa_blobid=[0-9]+/)) {
			return src.replace(/\?(?:.*&)?(qa_blobid=[0-9]+).*?$/, "?qa=image&$1");
		}

		if (domain === "image.tophinhanh.me" ||
			domain === "image.buonchoai.club") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+rx\/+[0-9]*x[0-9]*\/+([^/]*\.[^/]*\/)/, "$1");
			if (newsrc !== src)
				return add_http(newsrc);
		}

		if (domain_nowww === "artspace712.com" ||
			domain_nowww === "dientumaytinh.com" ||
			domain_nowww === "aihara-company.com") {
			match = src.match(/^[a-z]+:\/\/[^/]*\/+uploads\/+[^/]*\/+([^/.]{60,})\/+.*(?:[?#].*)?$/);
			if (match) {
				return base64_decode(base64_decode(base64_decode(match[1])));
			}
		}

		if (domain_nosub === "lozi.vn") {
			return src.replace(/(\/v1\/+[^/]*\/+)resized\/+([^/]*?)(?:[?#].*)?$/, "$1original/$2");
		}

		if (domain_nosub === "ilikewallpaper.net") {
			return src.replace(/(\/pic\/+[0-9]{6}\/+[^/]*?)[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "fatakat.com" && /^files[0-9]*\./.test(domain)) {
			return src.replace(/(\/[0-9]{4}\/+[0-9]{1,2}\/+(?:[^/]*\/+)?)small\/+/, "$1");
		}

		if (domain_nowww === "oir.mobi") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+src\.php\?(?:.*&)?src=(http[^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "tn.smilevideo.jp") {
			return {
				url: src.replace(/^([a-z]+:\/\/[^/]*\/+smile\?)(?:.*&)?(i=[0-9]+)(?:\.M)?(?:&.*)?$/, "$1$2.L"),
				can_head: false // 403
			};
		}

		if (domain_nowww === "wallpaperlayer.com") {
			return src.replace(/(\/img\/+[0-9]{4}\/+[0-9]{2}\/+)thumb\/+([^/]*)-thumb(\.[^/.]*)(?:[?#].*)?$/, "$1$2$3");
		}

		if (domain === "str.your-pictionary.com") {
			return src.replace(/\.md(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "rin-city.com") {
			return src.replace(/(\/images\/+previews\/+[^/]*\/+)thumb_/, "$1full_");
		}

		if (domain_nowww === "mypornstarbook.net") {
			return src.replace(/\/thumbnails\/+tn([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/images/$1");
		}

		if (domain_nowww === "picgym.net") {
			match = src.match(/^[a-z]+:\/\/[^/]*\/+img\/+[^/?]*\?([^-/._]{30,})(?:#.*)?$/);
			if (match) {
				return base64_decode(match[1]);
			}
		}

		if (domain === "acgn-cdn.sunteorum.com") {
			return src.replace(/(\.[A-Za-z0-9]+)_[0-9]+[wh]\.[^/.]*(?:[?#].*)?$/, "$1");
		}

		if (domain === "pratilipi.s.llnwi.net" && /\/pratilipi\/+(?:content\/+image|cover)\?/.test(src)) {
			id = url.searchParams.get("pratilipiId");
			var name = url.searchParams.get("name");
			var version = url.searchParams.get("version");
			if (id) {
				var newquery = src.replace(/\?.*/, "?pratilipiId=" + id + "&");

				if (name) {
					return newquery + "name=" + name;
				} else if (version) {
					return newquery + "version=" + version;
				}
			}
		}

		if (domain_nowww === "forumhentai.net") {
			return src.replace(/\/imagehosting\/+thum_/, "/imagehosting/");
		}

		if (domain_nowww === "mfqqx.com") {
			return src.replace(/(\/d\/+file\/+[0-9]{4}\/+[0-9]{2}\/+)small([0-9a-f]{32})[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2$3");
		}

		if (domain_nowww === "between-legs.com") {
			return src.replace(/(\/content\/+galleries\/+[0-9]+\/+[0-9]+\/+)thumbs\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1full/$2")
		}

		if (domain_nowww === "babeunion.com") {
			return src.replace(/(\/gallery\/+[^/]*\/+)thumbs\/+/, "$1");
		}

		if (domain_nowww === "bustyparade.com") {
			return src.replace(/\/thumbs\/+galls\//, "/galls/");
		}

		if (domain === "assets.marthastewart.com") {
			return src.replace(/(:\/\/[^/]*\/+)styles\/+[^/]*\/+/, "$1");
		}

		if (domain === "images.plurk.com") {
			return src.replace(/(^[a-z]+:\/\/[^/]*\/+)mx_([^/.]*\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "imgs.plurk.com") {
			return src.replace(/(:\/\/[^/]*\/+[^/]{3}\/+[^/]{3}\/+[a-zA-Z0-9]{10,})_tn(\.[^/.]*)(?:[?#].*)?$/, "$1_lg$2");
		}

		if (domain === "avatars.plurk.com") {
			newsrc = src.replace(/(\/[0-9]+-)(?:medium|small)([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1big$2");
			if (newsrc !== src)
				return add_full_extensions(newsrc);
		}

		if (domain === "cdnph.upi.com") {
			return src.replace(/(:\/\/[^/]*\/+)(?:topic\/+ph\/+[0-9]+\/+upi\/+|upi\/+)/, "$1pv/upi/");
		}

		if (domain_nowww === "glamour.bg") {
			return src.replace(/\/pictures\/+pic_small\/+/, "/pictures/pic_big/");
		}

		if (domain === "adminhonna.elwatannews.com" ||
			domain === "honnaimg.elwatannews.com") {
			return src.replace(/\/image_archive\/+(?:original_lower_quality|[0-9]+x[0-9]+)\/+/, "/image_archive/original/");
		}

		if (domain_nowww === "yeswedding.com.br") {
			return src.replace(/(\/[0-9]+)_big(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}


		if (domain_nosub === "adultempire.com" && /^imgs[0-9]*cdn\./.test(domain)) {
			return src.replace(/(\/galleries\/+[0-9]+\/+[0-9]{5,})_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1_9999$2");
		}

		if (domain_nowww === "mixmag.com.tr") {
			return src.replace(/(\/assets\/+uploads\/+images\/+)_facebook\/+/, "$1");
		}

		if (domain_nowww === "harpersbazaar.co.id") {
			return src.replace(/(\/lkgallery\/+[^/]*\/+)resize\/+[0-9]+x[0-9]+\/+/, "$1");
		}

		if (domain_nosub === "mobify.com" && /^ir[0-9]*\./.test(domain)) {
			return src.replace(/^[a-z]+:\/\/[^/]*\/+[0-9]+\/+http/, "http");
		}

		if (domain_nowww === "kanal24.az") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+photo\.php\?(?:.*&)?src=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return urljoin(src, decodeuri_ifneeded(newsrc), true);
		}

		if (domain_nowww === "aquarelle.md") {
			return src.replace(/\/uploads\/+cache\/+(news\/+[0-9]+\/+)[0-9]+X[0-9]+\/+/, "/uploads/$1");
		}

		if (domain === "img.meta.kz") {
			return src.replace(/(:\/\/[^/]*\/+\?)[0-9]+x/, "$1");
		}

		if (domain === "img.rtvslo.si") {
			return src.replace(/(\/upload\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9]+)_[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "i.stpl.gr") {
			match = src.match(/^[a-z]+:\/\/[^/]*\/+image\.ashx\?(?:.*&)?f=([^&]*).*?$/);
			if (match) {
				return urljoin(src, base64_decode(decodeURIComponent(match[1])), true);
			}
		}

		if (domain_nowww === "marecipe.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+image\.ashx\?(?:.*&)?u=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "cdn.akurat.co") {
			return src.replace(/(\/images\/+uploads\/+images\/+)[a-z]+\/+([^/]*\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "static.vesti.rs") {
			return src.replace(/(\/slike-[0-9]+\/+)tmb_/, "$1");
		}

		if (domain_nowww === "al-mlab.com") {
			return src.replace(/\/Thumbnails\/+([0-9]+\/+[0-9]+\/+)[0-9]+\/+([^/]*\.[^/.]*)(?:[?#].*)?$/, "/ArticleImages/$1$2");
		}

		if (domain_nowww === "govastileto.gr") {
			return src.replace(/\/assets\/+images\/+[0-9]*x[0-9]*\/+/, "/assets/images/");
		}

		if (amazon_container === "dotemirates-media") {
			return src.replace(/(\/[0-9]{4}\/+[0-9]{2}\/+)[a-z]+-([0-9a-f]{20,}\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "grazia.rs") {
			return src.replace(/\/uploads\/+imagecache\/+[0-9]+x[0-9]+\/+storage\/+/, "/storage/");
		}

		if (domain_nowww === "asian-amatuer.com") {
			return src.replace(/\/wp-content\/+vms_cache\/+images\/+(content\/+[^/]*\/+[^/]*)\.[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "/$1$2");
		}

		if (domain_nowww === "xxx69.net") {
			return src.replace(/\/tn_[0-9]+_[0-9]+_([^/.]*\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain === "cdn.milffox.com") {
			return src.replace(/\/thumb([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/pic$1");
		}

		if ((domain_nosub === "footprint.net" && /^cdni-rt\./.test(domain)) ||
			domain === "cdni.rbth.com") {
			return src.replace(/(\/images\/+[0-9]{4}\.[0-9]{2}\/+)[a-z]+\/+/, "$1original/");
		}

		if (domain_nowww === "tapchimypham.com.vn") {
			return src.replace(/^([a-z]+:\/\/[^/]*\/+ImageHandler\.ashx\?)(?:.*?&)?(ThumbnailID=[0-9]+).*$/, "$1$2");
		}

		if (domain === "cdn.lipscosme.com") {
			return src.replace(/(\/user\/+user[0-9]+-[0-9]+)-(?:small|thumb)(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "yesky.com" && /^[a-z]-pic-image\./.test(domain)) {
			return src.replace(/^([a-z]+:\/\/)[^/]*\/[-0-9]+x[-0-9]+\/+uploadImages\/+/, "$1pic-image.yesky.com/uploadImages/");
		}

		if (domain === "img.gifmagazine.net") {
			return src.replace(/(\/images\/+[0-9]+\/+)[^/]*(\.gif)(?:[?#].*)?$/, "$1original$2");
		}

		if (domain === "mediaprocessor.websimages.com") {
			return src.replace(/^([a-z]+:\/\/[^/]*\/+)(?:(?:width|height|crop)\/+[^/]*\/+){0,}([^/]*\.[^/]*\/+)/, "$1$2");
		}

		if (domain_nowww === "x18.xxx" ||
			domain_nosub === "drtst.com") {
			return src.replace(/\/media\/+photos\/+tmb(?:_new)?\/+/, "/media/photos/");
		}

		if (domain_nowww === "axtar.org") {
			match = src.match(/^[a-z]+:\/\/[^/]+\/+thumb\.php\?(?:.*&)?src=([^&]{50,}).*?$/, "$1");
			if (match) {
				var splitted = match[1].split("/");
				newsrc = base64_decode(splitted[0]);
				if (splitted[1]) {
					newsrc += "?" + base64_decode(splitted[1]);
				}

				return newsrc;
			}
		}

		if (domain_nosub === "popo8.com") {
			return src
				.replace(/(\/data\/+cimages\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}c_pic_[0-9]{8,}\.[^/._]*)_[a-z]\.[^/.]*(?:[?#].*)?$/, "$1")
				.replace(/(\/[0-9]{6}\/+[0-9]{2}\/+[0-9a-f]\/+(?:[0-9a-f]+|p[0-9]+_[0-9]+)\.[^/._]+)_[a-z]\.[^/.]*(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "navicdn.com") {
			return src.replace(/\/nhacpro\/+[0-9]+x[0-9]+\/+images\/+/, "/nhacpro/images/");
		}

		if (domain_nowww === "needradio.fr") {
			return src.replace(/(\/upload\/+videos\/+)[a-z]+\/+/, "$1original/");
		}

		if (domain === "assets.rebelmouse.io") {
			match = src.match(/^[a-z]+:\/\/[^/]*\/+[^-_/.]{20,}\.([^-_/.]{30,})\./);
			if (match) {
				var json = JSON_parse(base64_decode(match[1]));
				return json.image;
			}
		}

		if (domain_nowww === "issuedaily.com" ||
			domain === "img.issuedaily.com") {
			return src.replace(/\/thumb_image\/+(?:(?:new_)?main|thumb)_/, "/news_image/");
		}


		if (domain_nowww === "newsshare.co.kr") {
			return src.replace(/\/data\/+([^/]+)\/+mainimages\/+([0-9]{6})\/+[0-9]{6}_/, "/imgdata/$1/$2/");
		}

		if (domain_nowww === "dosugbook.nl") {
			return src.replace(/(\/upfiles\/+photos\/+persons\/+[0-9]+\/+)thumbs[0-9]+\/+/, "$1");
		}

		if (domain_nowww === "sexbahrain.club") {
			return {
				url: src.replace(/\/photos\/+[0-9]+_[0-9]+\/+/, "/photos/"),
				problems: {
					watermark: true
				}
			};
		}

		if (domain === "img.vphotos.cn") {
			newsrc = src.replace(/(:\/\/[^/]*\/+)download\/+([0-9A-F]{20,}\/+)/, "$1$2");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/\/(postp|selected)\/+logo(large|double|small|1920|thumb)\/+/, "/$1/$2/");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/\/(postp|selected)\/+thumbs\/+([^/]*)_thumb(\.[^/.]+)(?:[?#].*)?$/, "/$1/1920/$2_small$3");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/\/(postp|selected)\/+(?:logo)?(?:large|double|small|1920|thumb)\/+/, "/$1/large/");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/(\/[0-9A-F]{20,}\/+)selected\/+/, "$1postp/");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "pauza.sk") {
			return src.replace(/(\/web_object\/+[0-9]+)_[a-z](\.[^/.]*)(?:[?#].*)?$/, "$1_b$2");
		}

		if (domain === "d26672rfe314dk.cloudfront.net") {
			return src.replace(/(:\/\/[^/]*\/+[0-9]+\/+)[0-9]+\/+([^/]*\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "roleplayhaven.net") {
			return src.replace(/\/gallery\/+thumb_([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/gallery/$1");
		}

		if (domain === "images.ladbible.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+thumbnail\?(?:.*&)?url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "file.starnnews.com") {
			regex = /(\/[0-9]{8,}_[0-9a-f]+_)(?:10|1)(\.[^/.]*)(?:[?#].*)?$/;
			if (regex.test(src))
				return [
					src.replace(regex, "$12$2"),
					src.replace(regex, "$11$2")
				];
		}

		if (domain_nowww === "starjn.com") {
			newsrc = src.replace(/\/data\/+starjn_com\/+facebook\/+[0-9]+_([0-9]{6})([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/data/starjn_com/mainimages/$1/$1$2");
			if (newsrc !== src)
				return newsrc;

			return src.replace(/\/data\/+starjn_com\/+mainimages\/+/, "/imgdata/starjn_com/");
		}

		if (domain === "ipx.image-gmkt.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+imageproxy\/+\?(?:.*&)?src=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "i.011st.com") {
			return src.replace(/\/ex_t\/+R\/+[0-9]+x[0-9]+\/+[0-9]+\/+[0-9]+\/+[0-9]+\/+src\/+/, "/ex_t/R/0x0/0/100/0/src/");
		}

		if (domain === "cdn.mydaily.co.kr") {
			match = src.match(/\/FILES\/+[0-9]{6}\/+([0-9]+)_[0-9]+\.[^/.]*(?:[?#].*)?$/);

			var extra = {};
			if (match) {
				extra.page = "http://www.mydaily.co.kr/new_yk/html/read.php?newsid=" + match[1];
			}

			return {
				url: src,
				extra: extra
			};
		}

		if (domain_nosub === "iqiyipic.com") {
			return src.replace(/(\/legopi_[0-9a-f]{20,}_)[0-9]+_x(\.[^/.]*)(?:[?#].*)?$/, "$1xxx$2");
		}

		if (domain_nowww === "stevens-media.com" ||
			domain_nowww === "stecamsrl.it") {
			match = src.match(/\/(?:wp-content\/+wverrors\.php|page_id\.php)\?(?:.*&)?getimage=([^&]*).*?$/);
			if (match) {
				return base64_decode(match[1]);
			}
		}

		if (domain_nosub === "famtimes.co.kr" && /^img[0-9]*\./.test(domain)) {
			return src.replace(/(\/resources\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2})[sl]_([^/]*\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "fiyar.live") {
			regex = /\/themes\/+latest\/+ssd\/+small\/+([0-9]+)\/+small-/;

			if (regex.test(src)) {
				return [
					src.replace(regex, "/themes/latest/uploads4/pixsense/big/$1/"),
					src.replace(regex, "/themes/latest/uploads3/pixsense/big/$1/"),
					src.replace(regex, "/themes/latest/uploads2/pixsense/big/$1/")
				];
			}
		}


		if (domain === "hosted.foxes.com" ||
			domain === "promo.foxes.com") {
			return src.replace(/(\/fhgs?\/+[^/]*\/+)t(?:_[0-9]+x[0-9]+)?\/+/, "$1o/");
		}

		if (domain_nosub === "justjared.com" && /^cdn[0-9]*\.cdn\./.test(domain)) {
			return src.replace(/(\/wp-content\/+uploads\/+[0-9]{4}\/+[0-9]{2}\/+[^/]*\/+)thumbs\/+/, "$1");
		}

		if (domain_nosub === "thesupermodelsgallery.com" && /^cdn[0-9]*\./.test(domain)) {
			return src.replace(/(\/[0-9]{4}\/+[0-9]{2}\/+[^/]*\/+)tn_([^/]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "celebritymoviezone.com") {
			return src.replace(/\/thumbs\/+([^/]*\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nowww === "supermodels-online.com") {
			return src.replace(/(\/photos\/+[^/]*\/+[^/]*)-sm(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "thumbs.hentai-foundry.com" && options && options.do_request && options.cb) {
			match = src.match(/^[a-z]+:\/\/[^/]*\/+thumb\.php\?(?:.*&)?pid=([0-9]+)/);
			if (match) {
				var page = "https://www.hentai-foundry.com/pictures/" + match[1] + "/";
				options.do_request({
					method: "GET",
					url: page + "?enterAgree=1",
					onload: function(result) {
						if (result.readyState !== 4)
							return;

						var match = result.responseText.match(/<img[^>]*src=["']((?:https?:)?\/\/pictures\.hentai-foundry\.com\/[^'"]+)/);
						if (match) {
							return options.cb({
								url: urljoin(src, match[1], true),
								extra: {
									page: result.finalUrl
								}
							});
						} else {
							return options.cb(null);
						}
					}
				});

				return {
					waiting: true
				};
			}
		}

		if (domain_nowww === "boutiquefonts.com" ||
			domain_nowww === "animuk.co.uk") {
			return src.replace(/\/images\/+thumbnails\/+[0-9]+\/+[0-9]+\/+/, "/images/");
		}

		if (domain_nosub === "userapi.com") {
			newsrc = src.replace(/(:\/\/[^/]+\/+)impg\/+(.*?)(?:[?#].*)?$/, "$1$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "vk.com") {
			if (/^[a-z]+:\/\/[^/]+\/+images\/+(?:icons\/+)?[a-z_]+\.png(?:[?#].*)?$/.test(src))
				return {
					url: src,
					bad: "mask"
				};
		}

		if (domain_nosub === "userapi.com" &&
			host_domain_nosub === "vk.com" && options.element &&
			options.do_request && options.cb) {
			newsrc = (function() {
				var request_video = function(id, list, cb) {
					var cache_key = "vk_video:" + id;
					api_cache.fetch(cache_key, cb, function(done) {
						var listkey = "";
						if (list)
							listkey = "&list=" + list;
						options.do_request({
							method: "POST",
							url: "https://vk.com/al_video.php?act=show",
							data: "act=show&al=1&al_ad=0&video=" + id + listkey,
							headers: {
								"Content-Type": "application/x-www-form-urlencoded",
								"X-Requested-With": "XMLHttpRequest",
								"Referer": "https://vk.com/",
								"Origin": "https://vk.com"
							},
							onload: function(result) {
								if (result.readyState !== 4)
									return;

								try {
									var json = JSON_parse(result.responseText);

									var params = json.payload[1][4].player;
									return done(params, 3*60*60);
								} catch (e) {
									console_error("vk_video", e);
								}

								return done(null, false);
							}
						});
					});
				};

				function request_photo(id, list, cb) {
					var cache_key = "vk_photo:" + id;
					api_cache.fetch(cache_key, cb, function (done) {
						var listkey = "";
						if (list)
							listkey = "&list=" + list;
						options.do_request({
							method: "POST",
							url: "https://vk.com/al_photos.php",
							data: "act=show&al=1&photo=" + id + listkey,
							headers: {
								"Content-Type": "application/x-www-form-urlencoded",
								"X-Requested-With": "XMLHttpRequest"
							},
							onload: function (result) {
								if (result.readyState !== 4)
									return;

								try {
									var json = JSON_parse(result.responseText);

									var jsonarray = json.payload[1][3];
									var jsonobj = null;


									for (var i = 0; i < jsonarray.length; i++) {
										if (!jsonarray[i].id)
											continue;

										api_cache.set("vk_photo:" + jsonarray[i].id, jsonarray[i], 3*60*60);

										if (jsonarray[i].id !== id)
											continue;

										jsonobj = jsonarray[i];
										break;
									}

									if (jsonobj !== null)
										done(jsonobj, 3*60*60);
									else
										done(null, false);
								} catch(e) {
									console_log("vk_photo", result);
									console_error("vk_photo", e);
									return done(null, false);
								}

								if (false) {
								var match = result.responseText.match(/<!json>(\[.*?\])<!>/);
								if (!match) {
									return done(null, false);
								}

								try {
									var jsonarray = JSON_parse(match[1]);
									var jsonobj = null;


									for (var i = 0; i < jsonarray.length; i++) {
										if (!jsonarray[i].id)
											continue;

										api_cache.set("vk_photo:" + jsonarray[i].id, jsonarray[i], 3*60*60);

										if (jsonarray[i].id !== id)
											continue;

										jsonobj = jsonarray[i];
										break;
									}

									if (jsonobj !== null)
										done(jsonobj, 3*60*60);
									else
										done(null, false);
								} catch (e) {
									console_log("vk_photo", result);
									console_error("vk_photo", e);
									done(null, false);
								}
								}
							}
						});
					});
				}

				function process_photo(jsonobj) {

					var maxsize = 0;
					var maxurl = null;
					for (var key in jsonobj) {
						if (/^[a-z]_$/.test(key) && (key + "src") in jsonobj &&
							is_array(jsonobj[key]) && jsonobj[key].length === 3) {
							var size = jsonobj[key][1] * jsonobj[key][2];
							if (size > maxsize) {
								maxsize = size;
								maxurl = jsonobj[key + "src"];
							}
						}
					}

					if (!maxurl) {
						console_log("Unable to find photo for", deepcopy(jsonobj));
					}

					return maxurl;
				}

				function process_video(player) {
					var params = player.params[0];

					var maxsize = 0;
					for (var key in params) {
						var match = key.match(/^url([0-9]+)$/);
						if (match) {
							var size = parseInt(match[1]);
							if (size > maxsize)
								maxsize = size;
						}
					}

					if (maxsize !== 0) {
						return params["url" + maxsize];
					}

					if (player.type === "youtube") {
						if (params.extra_data) {
							return "https://i.ytimg.com/vi/" + params.extra_data + "/mqdefault.jpg";
						}
					}

					console_log("Unable to find video for", deepcopy(params));

					return null;
				}

				function process_el(current) {
					if (current.tagName === "A") {
						var is_photo = true;

						var match = current.href.match(/^[a-z]+:\/\/(?:[^/]+\.)?vk\.com\/+photo(-?[0-9]+_[0-9]+)\/*(?:[?#].*)?$/);
						if (!match) {
							var onclick = current.getAttribute("onclick");
							if (onclick) {
								match = onclick.match(/^return\s+showPhoto\(["'](-?[0-9]+_[0-9]+)["']\s*,\s*["']([^'"]+)["']/);
								if (!match) {
									match = onclick.match(/^return\s+showVideo\(["'](-?[0-9]+_[0-9]+)["']\s*,\s*["']([^'"]+)["']/);
									if (match) {
										is_photo = false;
									}
								}
							}
						}

						if (match) {
							if (is_photo) {
								request_photo(match[1], match[2], function(obj) {
									if (!obj) {
										options.cb(null);
									} else {
										options.cb(process_photo(obj));
									}
								});
							} else {
								request_video(match[1], match[2], function(obj) {
									if (!obj) {
										options.cb(null);
									} else {
										options.cb(process_video(obj));
									}
								});
							}

							return true;
						}
					}

					return false;
				}

				var current = options.element;
				if (current.children && current.children.length === 1) {
					if (process_el(current.children[0])) {
						return {
							waiting: true
						};
					}
				}

				while (current) {
					if (process_el(current)) {
						return {
							waiting: true
						};
					}

					current = current.parentElement;
				}

				return src;
			})();

			if (newsrc && newsrc !== src)
				return newsrc;
		}

		if (domain_nosub === "vkuservideo.net") {
			return {
				url: src,
				can_head: false,
				headers: {
					Origin: "https://vk.com",
					Referer: "https://vk.com",
					"Sec-Fetch-Site": "none",
					"Sec-Fetch-Dest": "document",
					"Sec-Fetch-Mode": "navigate",
					"Sec-Fetch-User": "?1",
					"Sec-Origin-Policy": "0"
				}
			};
		}

		if (domain_nowww === "free2music.com") {
			return src.replace(/\/images\/+thumb\/+(.*\.[^/.,]+),[0-9]+(?:,[0-9]+)?\.[^/.]*(?:[?#].*)?$/, "/images/$1");
		}

		if (domain === "cdn.muabannhanh.com") {
			return {
				url: src.replace(/\/img\/+gallery\/+thumbnail\/+/, "/img/gallery/"),
				problems: {
					watermark: true
				}
			};
		}

		if (domain_nosub === "eski.mobi") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+mobile\/+[^/]*\/+cache\/+img\/+q[0-9]+\/+[0-9]+\/+(https?%)/, "$1");
			if (newsrc !== src)
				return decodeURIComponent(newsrc);
		}

		if (domain_nosub === "soccer365.ru" && /^st[0-9]*\./.test(domain)) {
			return src
				.replace(/(\/players\/+[0-9]+_[0-9]+_[0-9]+\/+[0-9]+)_thumb(\.[^/.]*)(?:[?#].*)?$/, "$1$2")
				.replace(/(\/images\/+[0-9]+_[0-9]+_[0-9]+\/+img_[^/._]+)_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "static.grouple.co") {
			return src.replace(/(\/uploads\/+pics\/+[0-9]{2}\/+[0-9]{2}\/+[0-9]+)_p(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "vkpeople.ru") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+_\/+[^/?]+\?([^/]+\.userapi\.com\/+)/, "$1");
			if (newsrc !== src)
				return add_http(newsrc);
		}

		if (domain_nowww === "edchomes.com") {
			return src.replace(/\/CropUp\/+[-0-9]+x[-0-9]+\/+media\/+/i, "/media/");
		}

		if (domain === "turntable.kagiso.io") {
			return src.replace(/(\/images\/+[^/]+)\.(?:width|height)-[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1.original$2");
		}

		if (domain === "t.facdn.net" && options.cb && options.do_request) {
			match = src.match(/^[a-z]+:\/\/[^/]*\/+([0-9]+)@[0-9]+-[0-9]+\.[^/.]*(?:[?#].*)?$/);
			if (match) {
				var page = "https://www.furaffinity.net/full/" + match[1] + "/";
				options.do_request({
					method: "GET",
					url: page,
					headers: {
						"Referer": "",
						"Origin": ""
					},
					onload: function(result) {
						if (result.readyState !== 4)
							return;

						if (result.status !== 200)
							return options.cb(null);

						var match = result.responseText.match(/<a\s+href=["'](.*?)["']\s*>Download<\/a>/);
						if (match) {
							return options.cb({
								url: urljoin(src, match[1], true),
								extra: {
									page: page
								}
							});
						} else {
							return options.cb(null);
						}
					}
				});

				return {
					waiting: true
				};
			}
		}

		if (domain === "static.aryion.com") {
			newsrc = src.replace(/\/thumb\/+([0-9]+-[0-9]+)-thumb(?:\.[^/]*)?(\.[^/.]*)(\/+[^/]*)?(?:[?#].*)?$/, "/data/$1$2");
			if (newsrc !== src)
				return add_full_extensions(newsrc);

			match = src.match(/^[a-z]+:\/\/[^/]*\/+([^/]*)\/+(?:thumb|data\/+)([0-9]+)-/);
			if (match) {
				var obj = {
					url: src,
					extra: {
						page: "https://aryion.com/" + match[1] + "/view/" + match[2]
					}
				};

				return obj;
			}
		}

		if (domain === "ano.lolcathost.org") {
			return src.replace(/(:\/\/[^/]*\/+)thumbs\/+/, "$1pics/");
		}

		if (domain_nowww === "okkisokuho.com") {
			return src.replace(/(\/wp-content\/+picture\/+[0-9]{4}\/+[0-9]{6}\/+[^/]*\/+[^/]*_as[0-9]+)tn(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "pimg.togetter.com") {
			newsrc = src.replace(/(?:\?.*)?$/, "?w=o&h=o");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "simg.userboard.org" ||
			domain === "pimg.togetter.com") {
			match = src.match(/^[a-z]+:\/\/[^/]*\/+[0-9a-f]{20,}\/+([0-9a-f]{16,})/);
			if (match) {
				return hex_to_ascii(match[1]);
			}
		}

		if (domain_nowww === "hot.ag") {
			newsrc = src.replace(/\/([^/]+)\/+\1(?:%20| )Thumbnails\/+/, "/$1/");
			if (newsrc !== src)
				return add_full_extensions(newsrc);
		}

		if (domain_nowww === "finn-neo-admin.info" ||
			domain_nowww === "finn-neo.com") {
			return src.replace(/(\/newsimage\/+[0-9]+)_pc_[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "freeaddon.com") {
			return src.replace(/\/thumb[0-9]+x[0-9]+-/, "/bg-");
		}

		if (amazon_container === "classconnection") {
			return src.replace(/-thumb[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}



		if (domain === "pictures.topspeed.com") {
			return src.replace(/(\/IMG\/+)crop\/+([0-9]{6}\/+[^/]+)_[0-9]+x[0-9]+[wh](\.[^/.]+)(?:[?#].*)?$/, "$1jpg/$2$3");
		}


		if (domain === "monsite.woopic.com") {
			return src.replace(/(:\/\/[^/]*\/+[0-9]+\/+)f\/+[0-9]*x[0-9]*\/+p\/+/, "$1p/");
		}

		if (domain_nosub === "erome.com" && /^s[0-9]*\./.test(domain)) {
			return src.replace(/\/thumbs\/+([^/]*\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nosub === "bdsmlr.com") {
			return src.replace(/(\/uploads\/+photos\/+[0-9]{4}\/+[0-9]{2}\/+[0-9]+\/+[^/]*-[0-9]+-[a-zA-Z0-9]+)(\.[^/.]*)(?:[?#].*)?$/, "$1-og$2");
		}

		if (domain_nowww === "sofurryfiles.com") {
			newsrc = src.replace(/\/std\/+(?:thumb|preview)\?/, "/std/content?");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/\/std\/+[a-z]+\?(?:.*&)?page=([0-9]+).*?$/);
			if (match) {
				var obj = {
					url: src,
					extra: {
						page: "https://www.sofurry.com/view/" + match[1]
					}
				};

				return obj;
			}
		}

		if (amazon_container === "bw-1651cf0d2f737d7adeab84d339dbabd3-bcs") {
			return src.replace(/(\/product_[0-9]+\/+)Thumb([0-9]+_[0-9a-f]+\.[^/.]*)(?:[?#].*)?$/, "$1Full$2");
		}

		if (domain === "static.myfigurecollection.net") {
			newsrc = src.replace(/\/pics\/+figure\/+(?:big\/+)?([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/pics/figure/large/$1");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/\/pics\/+figure\/+(?:[a-z]+\/+)?([0-9]+)\.[^/.]+/);
			if (match) {
				return {
					url: src,
					extra: {
						page: "https://myfigurecollection.net/item/" + match[1]
					}
				};
			}

			newsrc = src.replace(/(\/pics\/+(?:encyclopedia|group)\/+)[0-9]+\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/\/pics\/+(encyclopedia|group)\/+(?:[0-9]+\/+)?([0-9]+)\.[^/.]+/);
			if (match) {
				var middle = "entry";
				if (match[1] === "group")
					middle = "club";

				return {
					url: src,
					extra: {
						page: "https://myfigurecollection.net/" + middle + "/" + match[2]
					}
				};
			}

			return src.replace(/\/image\/+thumbnails\/+/, "/image/");
		}

		if (domain_nowww === "8muses.com") {
			return src.replace(/(:\/\/[^/]*\/+)image\/+th\/+/, "$1image/fl/");
		}

		if (domain === "stored-edge.slickpic.com") {
			return {
				url: src.replace(/\/[0-9]+\/+([^/]*\.[^/.]*)(?:[?#].*)?$/, "/-/$1"),
				head_wrong_contentlength: true
			};
		}

		if (domain === "cdn.pururin.io") {
			newsrc = src.replace(/(\/assets\/+images\/+data\/+[0-9]+\/+[0-9]+)t(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/\/assets\/+images\/+data\/+([0-9]+)\/+([0-9]+|cover)t?\./);
			if (match) {
				var pageurl = "https://pururin.io/read/" + match[1];

				if (match[2] === "cover") {
					pageurl += "/a";
				} else {
					pageurl += "/" + zpadnum(match[2], 2) + "/a";
				}

				return {
					url: src,
					extra: {
						page: pageurl
					}
				};
			}
		}

		if (domain === "image.newdaily.co.kr") {
			newsrc = src.replace(/(\/site\/+data\/+)thumb(\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9]{10,}_[0-9]+)_thumb\./, "$1img$2.")
			if (newsrc !== src)
				return newsrc;

			match = src.match(/\/site\/+data\/+img\/+([0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9]{10,})_[0-9]+\./);
			if (match) {
				return {
					url: src,
					extra: {
						page: "http://www.newdaily.co.kr/site/data/html/" + match[1] + ".html"
					}
				};
			}
		}

		if (domain === "img.stoo.com") {
			match = src.match(/\/upimages\/+gisaimg\/+[0-9]{6}\/+(?:[0-9]{2}_)?([0-9]+)_([0-9]+)\./);
			if (match) {
				return {
					url: src,
					extra: {
						page: "http://stoo.asiae.co.kr/article.php?aid=" + match[1] + match[2]
					}
				};
			}
		}

		if (domain_nowww === "freeseximages.com") {
			return src.replace(/\/static\/+thumbnail\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/static/source/$1");
		}

		if (domain_nosub === "ravelrycache.com" && /^images[0-9]*/.test(domain)) {
			newsrc = src.replace(/(\/[0-9]+\/+)webp\/+([^/]*)\.webp([?#].*)?$/, "$1$2.jpg");
			if (newsrc !== src)
				return add_full_extensions(newsrc);

			newsrc = src.replace(/(\/flickr\/+(?:[0-9]\/+){3}[0-9]+\/+[0-9]+)(?:_n)?(\.[^/.]*)(?:[?#].*)?$/, "$1_b$2");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/\/flickr\/+(?:[0-9]\/+){3}[0-9]+\/+([0-9]+)(?:_[a-z])?\.[^/.]*(?:[?#].*)?$/);
			if (match) {
				return {
					url: "https://live.staticflickr.com/0000/" + match[1] + "_face_k.jpg",
					fake: true
				};
			}

			return src.replace(/(\/uploads\/+[^/]*\/+[0-9]+\/+[^/]*)_(?:small|medium|square)[0-9]*(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "fotorgia.com") {
			return src.replace(/(\/wp-content\/+plugins\/+wp-o-matic\/+cache\/+[0-9a-f]+_[^/]+)-[0-9]-[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "bonjourlesmadames.fr") {
			return src.replace(/(\/static\/+files\/+[0-9]{4}\/+[0-9]{4}-[0-9]{2}-[0-9]{2}\/+)[0-9]+_([^/]*\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "img.blesk.cz") {
			return src.replace(/(\/img\/+[0-9]+(?:\/+|-))[^/]*((?:\/+|-)[0-9]+[^0-9])/, "$1origin$2");
		}

		if (domain === "img.ngfiles.com") {
			if (/\/icons\/+hovers\/+[a-z]+\.png/.test(src))
				return {
					url: src,
					bad: "mask"
				};
		}

		if (domain === "uimg.ngfiles.com") {
			return src.replace(/(\/icons\/+[0-9]+\/+[0-9]+_)(?:small|medium)\./, "$1large.")
		}

		if (domain === "art.ngfiles.com") {

			newsrc = src.replace(/(\/thumbnails\/+[0-9]+\/+[0-9]+)\./, "$1_full.");
			if (newsrc !== src)
				return newsrc;

			var get_origpage_for_ngart = function(id, cb) {
				var cache_key = "ngart_page:" + id;
				api_cache.fetch(cache_key, cb, function(done) {
					options.do_request({
						url: "https://www.newgrounds.com/content/share/" + id + "/4", // 4 = art?, 3 = audio
						method: "GET",
						onload: function(resp) {
							if (resp.readyState !== 4)
								return;

							if (resp.status !== 200) {
								console_error(cache_key, resp);
								return done(null, false);
							}

							var match = resp.responseText.match(/<div\s+class=["']embed-link["'][\s\S]{10,100}<input\s+type="text"\s+value=["']([^"']+)/);
							if (!match) {
								console_error(cache_key, "Unable to find match for", resp);
								return done(null, false);
							}

							return done(decode_entities(match[1]), 12*60*60);
						}
					});
				});
			};

			var get_orig_from_ngartpage = function(pageurl, cb) {
				var cache_key = "ngart_orig:" + pageurl;
				api_cache.fetch(cache_key, cb, function(done) {
					options.do_request({
						url: pageurl,
						method: "GET",
						onload: function(resp) {
							if (resp.readyState !== 4)
								return;

							if (resp.status !== 200) {
								console_error(cache_key, resp);
								return done(null, false);
							}

							var match = resp.responseText.match(/PHP\.merge\(({.*?"full_image_text":.*?})\);/);
							if (!match) {
								console_error(cache_key, "Unable to find match for", resp);
								return done(null, false);
							}

							try {
								var json = JSON_parse(match[1]);
								var img = json.full_image_text;

								match = img.match(/src="([^"]+)"/);
								if (!match) {
									console_error(cache_key, "Unable to find img match for", img);
								} else {
									return done(decode_entities(match[1]), 12*60*60);
								}
							} catch (e) {
								console_error(cache_key, e);
							}

							return done(null, false);
						}
					});
				});
			};

			id = src.match(/\/(?:thumbnails|images)\/+[0-9]+\/+([0-9]+)/);
			if (id && options.do_request && options.cb) {
				id = id[1];
				get_origpage_for_ngart(id, function(page) {
					var obj = {
						url: src
					};

					if (page) {
						obj.extra = {page: page};

						if (src.indexOf("/thumbnails/") >= 0) {
							return get_orig_from_ngartpage(page, function(url) {
								if (url) {
									obj.url = url;
								}

								return options.cb(obj);
							});
						}
					}

					return options.cb(obj);
				});

				return {
					waiting: true
				};
			}
		}

		if (domain_nowww === "pinimg.icu") {
			return src.replace(/\/wall\/+[0-9]+x[0-9]+\/+/, "/wall/0x0/");
		}

		if ((domain_nosub === "netnews.vn" && /\.media\./.test(domain)) ||
			domain === "mediaold.tiin.vn") {
			return src.replace(/(_[0-9]+)wap_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "d343xr8tqpm2fx.cloudfront.net") {
			return src.replace(/(\/userfiles\/+[0-9]{4}\/+[0-9]{2}\/+[0-9]{10,})s([0-9]*\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}


		if (domain_nosub === "minkch.com" && /^imgs/.test(domain)) {
			return src.replace(/(\/imgs\/+[^/]+\/+i_[0-9]+)s(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "img.8kmm.com") {
			return src.replace(/(\/[0-9]{4}\/+[0-9]{1,2}\/+[0-9]{8,})_S(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "kpopis.com" && /^cdn[0-9]*\./.test(domain)) {
			return src.replace(/(\/data\/+file\/+lib2\/+[0-9]+\/+[0-9]+_[0-9]+_[0-9]+\.[^/._]+)_SM\.[^/.]*(?:[?$].*)?$/, "$1");
		}

		if (domain_nosub === "vnecdn.net" && domain.match(/^(?:(?:vcdn|i)-[^.]*|(?:.*\.)?img)\./)) {
			return src.replace(/(?:_m)?_[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain === "pocket.playxp.com") {
			return {
				url: src.replace(/(:\/\/[^/]+\/+)(?:thumbs\/+[0-9]*x[0-9]*|preview)\/+/, "$1download/"),
				head_wrong_contenttype: true,
				headers: {
					Referer: "https://www.playxp.com/community/funny/view.php?article_id=12345"
				}
			};
		}

		if (domain_nowww === "flash24.co.kr") {
			if (/\/g[0-9]*\/+m\/+thumb\.php\?/.test(src)) {
				var bo_table = url.searchParams.get("bo_table");
				var img = url.searchParams.get("img");

				if (bo_table && img) {
					return src.replace(/\/thumb\.php\?.*/, "/thumb.php?bo_table=" + bo_table + "&img=" + img);
				}
			}
		}

		if ((domain_nowww === "imgbaron.com" ||
			 domain_nowww === "picbaron.com")
			&& options.do_request && options.cb) {
			match = src.match(/\/i\/+[0-9]+\/+([0-9a-z]+)_t\./);
			if (match) {
				var pageurl = "https://" + domain_nosub + "/" + match[1] + "/.html";
				options.do_request({
					method: "POST",
					url: pageurl,
					data: "op=view&id=" + match[1] + "&pre=1&next=Continue+to+image...",
					headers: {
						Origin: "https://" + domain_nosub,
						Referer: pageurl,
						"Content-Type": "application/x-www-form-urlencoded"
					},
					onload: function(result) {
						if (result.readyState !== 4)
							return;

						if (result.status !== 200) {
							return options.cb(null);
						}

						var match = result.responseText.match(/<img\s+src=["']((?:https?:)?\/\/[^/]+\/+img\/+[0-9a-z]{10,}\/+.*?)['"]\s+class=["']pic["']/);
						if (match) {
							return options.cb({
								url: urljoin(url, match[1], true),
								is_original: true
							});
						} else {
							return options.cb(null);
						}
					}
				});

				return {
					waiting: true
				};
			}
		}

		if ((domain_nosub === "imgtown.net" ||
			 domain_nosub === "imgtown.pw") &&
			options && options.cb && options.do_request) {
			match = src.match(/\/i\/+[0-9]+\/+([0-9a-z]+)_t\./);
			if (match) {
				id = match[1];
				var pageurl = "https://" + domain_nosub + "/" + id + "/.html";
				options.do_request({
					method: "GET",
					url: pageurl,
					headers: {
						Referer: ""
					},
					onload: function(result) {
						if (result.readyState !== 4)
							return;

						if (result.status !== 200) {
							console_error("Bad response code for " + pageurl, result);
							return options.cb(null);
						}

						var match = result.responseText.match(/((?:var _0x[0-9a-f]+=(?:"[-_A-Za-z0-9=]*"|(?:_0x[0-9a-f]+\+){1,}_0x[0-9a-f]+);){1,}var _0x[0-9a-f]+=atob\((?:_0x[0-9a-f]+\+){1,}_0x[0-9a-f]+\))/);
						if (!match) {
							console_error("Unable to find encrypted URL for " + pageurl, result);
							return options.cb(null);
						}

						var final = "";
						function parse_vartable(varstring) {
							var splitted = varstring.split(";");

							var vartable = {};
							for (var i = 0; i < splitted.length; i++) {
								splitted[i] = splitted[i].replace(/^\s*|\s*$/, "");
								if (splitted[i].length === 0)
									continue;

								var smatch = splitted[i].match(/^var (_0x[0-9a-f]+)=(.*)/);

								var varname = smatch[1];
								var varval = smatch[2];

								var vmatch = varval.match(/^["'](.*)["']/);
								if (vmatch) {
									vartable[varname] = vmatch[1];
								} else if ((vmatch = varval.match(/^_0x[0-9a-f]+\+/))) {
									var vsplitted = varval.split("+");
									varval = "";

									for (var j = 0; j < vsplitted.length; j++) {
										varval += vartable[vsplitted[j]];
									}

									vartable[varname] = varval;
								} else if ((vmatch = varval.match(/^atob/))) {
									var vsplitted = varval.replace(/^atob\((.*)\)$/, "$1").split("+");
									varval = "";

									for (var j = 0; j < vsplitted.length; j++) {
										varval += vartable[vsplitted[j]];
									}

									vartable[varname] = base64_decode(varval);
									final = vartable[varname];
								} else {
									console_log("Unknown varval type:" + splitted[i]);
								}
							}

							return vartable;
						}

						parse_vartable(match[1]);

						if (!final) {
							console_error("Unable to create vartable", result);
							return options.cb(null);
						}

						options.do_request({
							method: "GET",
							url: final,
							headers: {
								Referer: result.finalUrl
							},
							onload: function(result) {
								if (result.readyState !== 4)
									return;

								if (result.status !== 200) {
									return options.cb(null);
								}

								var varmatch = result.responseText.match(/{(?:var _0x[0-9a-f]+='[0-9a-z]*';){4,}/g);
								if (varmatch.length > 1 || varmatch.length === 0) {
									console_error("varmatches.length != 1", result);
									return options.cb(null);
								}

								var varsstr = varmatch[0].substr(1); // remove {
								var vartable = parse_vartable(varsstr);

								var combinedmatch = result.responseText.match(/,((?:_0x[0-9a-f]+\+){4,}_0x[0-9a-f]+)\)/);
								if (!combinedmatch) {
									console_error("Unable to find variable combiner", result);
									return options.cb(null);
								}

								var combiner = combinedmatch[1].split("+");
								var hash = "";
								for (var i = 0; i < combiner.length; i++) {
									if (!(combiner[i] in vartable)) {
										console_error("Unable to find variable " + combiner[i] + " in vartable", vartable, result);
										return options.cb(null);
									}

									hash += vartable[combiner[i]];
								}

								options.do_request({
									method: "POST",
									url: result.finalUrl,
									headers: {
										Origin: result.finalUrl.replace(/^([a-z]+:\/\/[^/]+\/+).*/, "$1"),
										Referer: result.finalUrl,
										"Content-Type": "application/x-www-form-urlencoded"
									},
									data: "op=view&id=" + id + "&pre=1&hito=tm&" + hash + "=1",
									onload: function (result) {
										if (result.readyState !== 4) {
											return options.cb(null);
										}

										if (result.status !== 200) {
											return options.cb(null);
										}

										var match = result.responseText.match(/<img\s+src=["']((?:https?:)?\/\/[^/]+\/+img\/+[0-9a-z]{10,}\/+.*?)['"]\s+class=['"]picview["']/);
										if (match) {
											return options.cb({
												url: urljoin(src, match[1], true),
												is_original: true
											});
										} else {
											return options.cb(null);
										}
									}
								});
							}
						});
					}
				});

				return {
					waiting: true
				};
			}
		}

		if (domain === "m.wdqyan.com") {
			return src.replace(/\/static\/+upload\/+thumbs\/+/, "/static/upload/");
		}

		if (domain === "kan.china.com") {
			return src.replace(/(\/utuku\/+img[0-9]*\/+)[0-9]+x[0-9]+\/+/, "$10x0/");
		}

		if (domain === "storage.enuri.info") {
			match = src.match(/\/pic_upload\/+knowbox_rss\/+([^/.?#]+)(?:[?#].*)?$/);
			if (match) {
				return base64_decode(match[1]);
			}
		}

		if (domain === "img.mediapen.com") {
			match = src.match(/\/news\/+[0-9]{6}\/+news_([0-9]+)_[0-9]+_[a-z]\./);
			if (match) {
				return {
					url: src,
					extra: {
						page: "http://www.mediapen.com/news/view/" + match[1]
					}
				};
			}
		}

		if (domain === "cdnimage.dailian.co.kr") {
			newsrc = src.replace(/\/news\/+icon\/+([0-9]{6}\/+news_[0-9]+_[0-9]+)_c(\.[^/.]*)(?:[?#].*)?$/, "/news/$1_m_1$2");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/\/news\/+(?:[a-z]+\/+)?[0-9]{6}\/+news_[0-9]+_([0-9]+)_[a-z](?:_[0-9]+)?\./);
			if (match) {
				return {
					url: src,
					extra: {
						page: "http://www.dailian.co.kr/news/view/" + match[1]
					}
				};
			}
		}

		if (domain_nowww === "teengirl-pics.com") {
			match = src.match(/^[a-z]+:\/\/[^/]+\/+([^/.]{50,})\.[^/.]*(?:[?#].*)?$/);
			if (match) {
				return base64_decode(decodeURIComponent(match[1]));
			}
		}

		if (domain === "static-fhg.metart.com" ||
			domain === "static-fhg.alsscan.com") {
			newsrc = src.replace(/(\/media\/+[0-9A-F]{10,}\/+)wt_([0-9A-F]{10,}\.[^/.]+)(?:[?#].*)?$/, "$1w_$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "fhg.asiansexdiary.com") {
			return src.replace(/(\/wp-content\/+uploads\/+[^/]*\/+[^/]+_)th_([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "lesarchive.com") {
			return src.replace(/(\/pictures\/+[0-9]+\/+[^/]+\/+)thumbs\/+/, "$1pics/");
		}

		if (domain === "images.hqseek.com") {
			return src.replace(/(\/pictures\/+[^/]+\/+)thumbs\/+/, "$1");
		}

		if (domain_nosub === "scoreuniverse.com") {
			return src.replace(/(\/images_content\/+[^/]*\/+[0-9]+)_tn(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "sexgrace.com") {
			return {
				url: src.replace(/\/tr\/+/, "/pics/"),
				headers: {
					Referer: "http://www.sexgrace.com/"
				}
			};
		}

		if (domain_nowww === "idealbabes.net" ||
			domain_nowww === "purescans.com") {
			return src.replace(/_tr(\/+[^/]*\.[^/.]*)(?:[?#].*)?$/, "_pics$1");
		}

		if (domain === "imganuncios.mitula.net") {
			return src.replace(/(:\/\/[^/]*\/+)[a-z]+\/+([^/]+_[0-9]{10,}\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "recbook.com") {
			return src.replace(/\/storage\/+thumbs\/+/, "/storage/uploads/");
		}

		if (domain === "apollo-frankfurt.akamaized.net" ||
			domain === "apollo-virginia.akamaized.net" ||
			domain === "apollo-singapore.akamaized.net") {
			return src.replace(/(\/files\/+[^/]*\/+image)(?:%3B|;)[^/]*(?:[?#].*)?$/, "$1");
		}

		if (domain_nosub === "jdmagicbox.com" && /^content[0-9]*\./.test(domain)) {
			return src.replace(/(\/catalogue\/+[^/?]*-[a-z0-9]{10,})(?:-[0-9]+)?(\.[^/.]*?)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "image.aladin.co.kr") {
			return src.replace(/(\/product\/+[0-9]+\/+[0-9]+\/+)cover(?:sum|150)\/+/, "$1cover500/");
		}

		if (domain === "book.interpark.com" ||
			domain === "bimage.interpark.com") {
			return src.replace(/(\/goods_image\/+(?:[0-9]\/+){4}[0-9]+)[hoi](\.[^/.]*)(?:[?#].*)?$/, "$1g$2");
		}

		if (domain === "mobile.kyobobook.co.kr") {
			newsrc = src.replace(/.*\/common\/+image\/+resize\?(?:.*&)?url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "image.kyobobook.co.kr") {
			return src.replace(/\/images\/+book\/+large\/+([0-9]{3}\/+)l/, "/images/book/xlarge/$1x");
		}

		if (domain === "cdn.011st.com") {
			return src.replace(/\/+11dims\/.*?\/11src\/+/, "/");
		}

		if (domain_nowww === "comiczone.co.kr") {
			return src.replace(/\/data\/+goods\/+t\/+/, "/data/goods/");
		}

		if (domain === "static.mangahub.ru") {
			return src.replace(/\/uploads\/+media\/+manga_cover\/+thumbnail\/+small\/+/, "/uploads/media/manga_cover/thumbnail/big/");
		}


		if (domain_nosub === "poringa.net" && /^img-[0-9]*\./.test(domain)) {
			return src.replace(/(\/posts\/+(?:[^/]\/+){6}[^/]+\/+)[0-9]+x[0-9]+_([^/]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "warashi-asian-pornstars.fr") {
			return src.replace(/(\/[^/]+-img\/+[^/]*\/+[0-9]+\/+[0-9]+\/+)mini\/+/, "$1large/");
		}

		if (domain_nowww === "kinghost.com") {
			return {
				url: src.replace(/(\/+[0-9]+\/+)s([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2"),
				headers: {
					Referer: "http://www.kinghost.com/asian/"
				},
				referer_ok: {
					same_domain: true
				}
			};
		}

		if (domain_nowww === "japanesegirl-4you.com") {
			return src.replace(/(\/[0-9]{4}\/+[0-9]{4}\/+)thumbs\/+/, "$1");
		}

		if (domain_nowww === "nsgalleries.com") {
			return src.replace(/(\/hosted2\/+[^/]+\/+hrpics\/+[^/]+\/+)thumbs\/+([0-9]+\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "sexygirlcity.com") {
			return src.replace(/(\/gal\/+[0-9]+\/+[0-9]+)_tumb[a-z]?(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "image.pornyl.com") {
			return src.replace(/(\/content\/+[0-9]+-[0-9]+\/+)tn\/+/, "$1");
		}

		if (domain === "media.gettyimages.com") {
			if (src.match(/\/photos\/+[^/?]*-id[0-9]+(?:#.*)?$/)) {
				return src.replace(/(-id[0-9]+)$/, "$1?s=2048x2048");
			}

			obj = [];
			id = src.replace(/^[a-z]+:\/\/[^/]*\/+photos\/+[^/?#]+-id([0-9]+)(?:[?#].*)?$/, "$1");
			if (id !== src && !problem_excluded("watermark")) {
				obj.push({
					url: src.replace(/(-id[0-9]+)(?:[?#].*)?$/, "$1?s=2048x2048"),
					problems: {
						watermark: true
					}
				});
			}

			if (id !== src && options && options.do_request && options.cb && (options.force_page || !problem_excluded("smaller"))) {
				options.do_request({
					method: "GET",
					url: "https://www.gettyimages.com/detail/a/a/" + id,
					headers: {
						Referer: ""
					},
					onload: function(result) {
						if (result.readyState !== 4)
							return;

						if (result.status !== 200)
							return options.cb(obj || null);

						var extra = {page: result.finalUrl.replace(/(:\/\/www\.gettyimages\.)[^/]+\//, "://www.gettyimages.com/")};

						var caption = result.responseText.match(/<meta\s+content=["']([^'"]+?)["']\s+property=["']og:description["']/);
						if (caption) {
							extra.caption = decode_entities(caption[1]);
						} else {
							console_warn("Unable to find caption", result);
						}

						var smallerurl = result.responseText.match(/"thumb612Url":\s*(["']https?:\/\/media\.gettyimages\.com\/+photos\/+[^/]*-id[0-9]+\?k=6.*?["'])/);
						if (smallerurl && !problem_excluded("smaller")) {
							obj.push({
								url: JSON_parse(smallerurl[1]),
								problems: {
									smaller: true
								}
							});
						}

						if (obj.length > 0) {
							for (var i = 0; i < obj.length; i++) {
								obj[i].extra = extra;
							}
						} else {
							obj = {
								url: src,
								extra: extra
							};
						}

						options.cb(obj);
					}
				});

				return {
					waiting: true
				};
			}

			if (obj.length > 0)
				return obj;
		}

		if (domain_nowww === "distrowatch.com") {
			return src.replace(/(\/images\/+(?:screenshots|slinks)\/+[^/]*)-small(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "tengritravel.kz") {
			return src.replace(/(\/userdata\/+images\/+u[0-9]+\/+)resized\/+/, "$1");
		}

		if (domain === "tb.ziareromania.ro") {
			return src.replace(/:\/\/tb\.([^/]*)\/+[^/]*\/+([0-9a-f]{15,})\/.*/, "://s1.$1/?mmid=$2");
		}

		if (domain_nowww === "povarenok.ru") {
			return src.replace(/(\/images\/+[^/]*\/+[0-9]{2}_[0-9]{2}_[0-9]{2}\/+)[a-z]+\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "i.servimg.com") {
			return src.replace(/\/tm\/+([^/]*\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nowww === "ammonnews.net") {
			return src.replace(/\/image\.php\?(?:.*?&)?token=([0-9a-f]+).*?$/, "/image.php?token=$1");
		}

		if (domain === "f.mahmure.com") {
			return src.replace(/(\/galeri2\/+galeri_[0-9]+\/+)thumbnail\/+[0-9]+x[0-9]+_([^/]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "img.eronrg.com" ||
			domain_nowww === "eronrg.com") {
			return src.replace(/(\/gallery\/+[0-9]+\/+)small\/+([^/]*_)thumb(_[0-9]+)(\.[^/.]*)(?:[?#].*)?$/, "$1big/$2img$3$4");
		}

		if (domain_nowww === "boobz.dk") {
			return src.replace(/\/th\/+([0-9]+)_[a-z]+\./, "/$1.");
		}

		if (domain_nowww === "redbloodedthing.com") {
			return src.replace(/\/galleries\/+thumbs\/+(oldgallery[0-9a-f]{20,}\/+)[0-9]+(?:-[a-z]+)?-([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/galleries/images/$1600-$2");
		}

		if (domain_nowww === "b92.net") {
			return src.replace(/(\/pics\/+[0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+[0-9]+[0-9a-f]+[0-9]+)_[wh][0-9]+\./, "$1_orig.");
		}

		if (domain_nosub === "albumoftheyear.org" && /^cdn[0-9]*\./.test(domain)) {
			return src.replace(/(?:\/[0-9]*x[0-9]*\/+album\/+|\/album\/+thumbs\/+)/, "/album/");
		}

		if (domain_nowww === "rap4ever.org") {
			return src.replace(/(\/uploads\/+albums\/+[^/]*\/+[^/]*?)[0-9]+X[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "musicmeter.nl") {
			return src.replace(/(\/images\/+cover\/+[0-9]+\/+[0-9]+\.)[0-9]+\.([^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "ecp.yusercontent.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+mail\?(?:.*?&)?url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "foto.sondakika.com") {
			return src.replace(/(\/haber\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[^/]*-[0-9]+_[0-9]+)_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1_osd$2");
		}

		if (domain_nowww === "segre.to") {
			return src.replace(/(\/shouts\/+[^/]*\/+[^/]*)_hq(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "img.fotodom.ru") {
			obj = [];

			regex = /(:\/\/[^/]*\/+[^/?]*)(?:\?size=[sml])?(?:\#.*)?$/;
			newsrc = src.replace(regex, "$1?size=l");
			if (newsrc !== src)
				obj.push({
					url: newsrc,
					problems: {
						watermark: true
					}
				});

			newsrc = src.replace(regex, "$1?size=m");
			if (newsrc !== src)
				obj.push({
					url: newsrc,
					problems: {
						smaller: true
					}
				});

			return obj;
		}

		if (domain === "d3m2o2bzrl719s.cloudfront.net") {
			obj = [];

			regex = /(:\/\/[^/]*\/+photos\/+[0-9]{4}\/+[0-9]{2}\/+[^/]+\/+)(?:gallery|lightbox)_([-0-9a-f]{20,}\.)/;
			newsrc = src.replace(regex, "$1lightbox_$2");
			if (false && newsrc !== src) // disabling because too many images get smaller
				obj.push({
					url: newsrc,
					problems: {
						watermark: true
					}
				});

			newsrc = src.replace(regex, "$1gallery_$2");
			if (newsrc !== src)
				obj.push({
					url: newsrc,
					problems: {
						smaller: true
					}
				});

			return obj
		}

		if (domain_nowww === "secureservercdn.net") {
			return src.replace(/^[a-z]+:\/\/[^/]*\/+(?:[0-9]+\.){3}[0-9]+\/+([^/]*\.[^/]*\/)/, "http://$1");
		}


		if (domain === "cdn-yotpo-images-production.yotpo.com") {
			newsrc = src.replace(/\/low_resolution(\.[^/.]*)(?:[?#].*)?$/, "/standard_resolution$1");
			if (newsrc !== src)
				return newsrc;

			return src.replace(/(\/(?:Product|Review|Account)\/+[0-9]+\/+[0-9]+\/+)[a-z_]+(\.[^/.]*)(?:[?#].*)?$/, "$1original$2");
		}

		if (domain_nowww === "maquillagecreation.ch") {
			return src.replace(/(\/img\/+content\/+[^/]*\/+[^/]*\/+[^/]*?)(?:_m|[0-9]+x[0-9]+)(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "image.gamer.ne.jp") {
			return src.replace(/(\/news\/+[0-9]{4}\/+[0-9]{8}\/+[0-9a-f]{20,}\/+)[a-z]\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1o/$2");
		}

		if (domain_nowww === "moviemagik.in") {
			return src.replace(/(\/files\/+[^/]*\/+[0-9]+\/+)thumb\/+/, "$1");
		}

		if (domain_nowww === "itedou.com") {
			return src.replace(/(\/Uploads\/+[^/]*\/+[0-9]+\/+[0-9]+)_s(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "img.cnread.news") {
			return src.replace(/(\/uploads\/+[0-9]+\/+[0-9A-F]{2}\/+[0-9A-F]+w[0-9]+h[0-9]+)(?:_[0-9]+x[0-9]+)?(?:_s[0-9]*)?(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "filmitena.com") {
			return src
				.replace(/(\/img\/+Actor\/+)[a-zA-Z]+\/+([0-9]+)_[a-z]+(\.[^/.]*)(?:[?#].*)?$/i, "$1Original/$2_or$3")
				.replace(/(\/img\/+)Image(?:Tmb)?\/+([0-9]+)_[a-zA-Z]+(\.[^/.]*)(?:[?#].*)?$/i, "$1big/$2_bg$3");
		}

		if (domain_nowww === "fandimefilmu.cz") {
			return src.replace(/(\/files\/+images\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2})[a-z_]+_([a-z0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "fetcher-cdn.nullmu.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+social(?:_preview)?\/+[0-9a-f]{20,}\.[^/.?]*\?(?:.*&)?u=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "bharatstudent.com") {
			return src.replace(/\/gallery\/+thumb\/+events\/+/, "/gallery/normal/events/");
		}

		if (domain === "mimgs.sulekha.com") {
			newsrc = src
				.replace(/\/(?:images\/+event-photos|events)\/+([^/]*\/+)spot[0-9]*\/+/, "/images/event-photos/$1")
				.replace(/\/images\/+stills\/+thumbnail\/+/, "/images/stills/");
			if (newsrc !== src)
				return {
					url: newsrc,
					problems: {
						watermark: true
					}
				};
		}

		if (domain_nowww === "i.indiglamour.com") {
			return src.replace(/(\/photogallery\/+.*\/normal\/+[^/]*_[0-9]+)(?:thmb|rs)(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "mom50.com") {
			return {
				url: src.replace(/(\/galleries\/+[^/]*\/+[0-9]{4}\/+[0-9]{2}\/+[^/]*\/+[0-9]+)(?:_[a-z]+){1,}(\.[^/.]*)(?:[?#].*)?$/, "$1$2"),
				headers: {
					Referer: ""
				}
			};
		}

		if (domain_nowww === "fastpics.eu") {
			return src.replace(/\/tn_([^/]*\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nowww === "forbidden-dreams.com" ||
			domain_nowww === "porn-service.us") {
			return src.replace(/\/([0-9]+)_ci(\.[^/.]*)(?:[?#].*)?$/, "/ci_$1$2");
		}

		if (domain_nowww === "iie8.com" ||
			domain_nowww === "vq50.com" ||
			domain_nowww === "e1nn.com") {
			return src.replace(/(\/cdn\/+i\/+[0-9a-f]{10,}\/+)th-[0-9]*x[0-9]*\/+/, "$1");
		}

		if (domain_nowww === "lustfulmodels.com") {
			return src.replace(/\/s_([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nowww === "tannerbucks.com") {
			return src.replace(/\/index_files\/+tn([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain === "galleries.imctrck.com") {
			return src.replace(/\/tn\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/images/$1");
		}

		if (domain_nowww === "boobgoddess.com") {
			return src.replace(/\/thumbs([0-9]*)\/+tn_/, "/pics$1/");
		}

		if (domain_nowww === "nextpicturez.com") {
			return src.replace(/(\/gallery\/.*\/)thumbs\/+/, "$1");
		}

		if (domain_nosub === "porngals4.com") {
			newsrc = src.replace(/\/media\/+galleries_[0-9]+x[0-9]+\/+/, "/media/galleries/");
			if (newsrc !== src)
				return newsrc;

			id = src.match(/^[a-z]+:\/\/[^/]*\/+media\/+gals_[0-9]+x[0-9]+\/+[0-9]+\/+[0-9]+\/+([0-9]+)-[0-9]+\/+/);
			if (id && options && options.do_request && options.cb) {
				console_log(id);
				api_cache.fetch("porngals4:" + id[1],
					function (text) {
						if (!text)
							return options.cb(null);

						options.cb(src.replace(/\/media\/+gals_[0-9]+x[0-9]+\/+([0-9]+\/+[0-9]+\/+[0-9]+-[0-9]+\/+)([0-9]+-[0-9]+\.[^/.]*)(?:[?#].*)?$/,
						           "/media/galleries/$1" + text + "-$2"));
					},
					function (done) {
						options.do_request({
							method: "GET",
							url: "https://www.porngals4.com/" + id[1] + "/",
							headers: {
								Referer: "",
							},
							onload: function (resp) {
								if (resp.readyState !== 4)
									return;

								if (resp.status !== 200)
									return done(null, false);

								var match = resp.finalUrl.match(/^[a-z]+:\/\/[^/]*\/+([^/]*)-[0-9]+\/+/);
								if (match) {
									return done(match[1], 6*60);
								} else {
									return done(null, false);
								}
							}
						});
					}
				);

				return {
					waiting: true
				};
			}
		}

		if (domain_nowww === "archivegalleries.net") {
			return {
				url: src.replace(/(\/collection\/.*\/[0-9]+)_[a-z]+size(\.[^/.]*)(?:[?#].*)?$/, "$1$2"),
				headers: {
					Referer: ""
				}
			};
		}

		if (domain_nosub === "alsscan.com" && /^s[0-9]*\./.test(domain)) {
			return src.replace(/\/(sample[0-9]+)s(\.[^/.]*)(?:[?#].*)?$/, "/$1$2");
		}

		if (domain_nowww === "mnvg.com") {
			return src.replace(/(\/gallery\/+[0-9]+\/+[0-9]+)(\.[^/.]*)(?:[?#].*)?$/, "$1-full$2");
		}

		if (domain_nowww === "pornpaw.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+showthumb\.php\?(?:.*?&)?thumblink=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nosub === "newstracklive.com") {
			return src.replace(/(\/uploads\/+.*\/+[0-9]{1,2}\/+)[a-z]+_thumb\/+/, "$1big_thumb/");
		}

		if (domain_nowww === "ap7am.com") {
			return src.replace(/(\/backimages\/+telugu-news\/+)thumbs\/+(tnews-)/, "$1bigimages/$2");
		}

		if (domain === "api.parkseason.ru") {
			return src.replace(/\/images\/+styles\/+[^/]*\/+/, "/images/");
		}

		if (domain_nowww === "vgamuseum.info") {
			return src.replace(/\/smart_thumbs\/+([^/]*?)_thumb_other[0-9]+_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "/$1$2");
		}

		if (domain_nowww === "metager.org") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+meta\/+picture\?(?:.*&)?url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "i.swisscows.ch") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+\?link=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "cdn.swisscows.ch") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+(https?:)/, "$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "mithaaprilia.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+image\?(?:.*?&)?q=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "proprofs.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+games\/+puzzle\/+sliding\/+resize_image\.php\?(?:.*?&)?image=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "sweetgirl.org") {
			return src.replace(/(\/files\/+sets\/+[0-9]+\/+[^/]*\/+)thumb\/+/, "$1");
		}

		if (domain === "img.lady.ru") {
			newsrc = src.replace(/(\/data\/+aphoto\/.*\/)small\/+/, "$1large/");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/(\/data\/+users\/.*\/)small(\.[^/.]*)(?:[?#].*)?$/, "$1large$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "cfake.com" ||
			domain_nowww === "krishna24.ru" ||
			domain_nowww === "lovedat.ru") {
			return src.replace(/\/medias\/+thumbs\/+/, "/medias/photos/");
		}

		if (domain_nowww === "sks-52.ru" ||
			domain === "gals.kindgirls.com") {
			return src.replace(/(\/d[0-9]+\/+[^/]*\/+)m[0-9]+\/+/, "$1");
		}

		if (domain === "abc.2008php.com") {
			return src.replace(/(\/[0-9]+_)smallimg\/+/, "$1Website_appreciate/");
		}

		if (domain === "pro.imgcdn2.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+\?(?:.*&)?url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return add_http(decodeuri_ifneeded(newsrc.replace(/z-z/g, ".")));
		}

		if (domain_nowww === "girlswithmuscle.com") {
			newsrc = src.replace(/\/images\/+thumbs\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/images/full/$1");
			if (newsrc !== src) {
				return add_full_extensions(newsrc, ["mp4", "jpg"], true);
			}
		}

		if (domain === "img.jiahes.com") {
			return src.replace(/(\/upload\/+image\/+[^/]*\/+[0-9]+\/+)c_[0-9]{3}([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "img.peta2.jp") {
			return src.replace(/(\/img\/+upload\/+th\/+[0-9]+-[0-9]+\/+[0-9]+\/+[^/]*_[0-9]+_[0-9]+)_(?:thumb)?[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1_org$2");
		}

		if (domain_nowww === "fakty.ua") {
			return src.replace(/(\/photos\/+articles\/+[0-9]{2}\/+[0-9]\/+[0-9]+)(?:[a-z][0-9]*){1,}(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "metrosport.gr") {
			return src.replace(/\/images\/+thumbs_[a-z]+\/+/, "/images/");
		}

		if (domain_nowww === "boobieblog.com") {
			return src.replace(/(\/img[0-9]*\/+[^/]*)_small(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "cdn.wearehairy.com") {
			return src.replace(/\/th\/+([^/]*\.[^/.]*)(?:[?#].*)?$/, "/img/$1");
		}

		if (domain === "cdn.picson.mobi") {
			return src.replace(/(:\/\/[^/]*\/+)[0-9]+\/+([0-9]+\/+[0-9]+\/+[0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1original/$2");
		}

		if (domain === "s.deviant.life" ||
			domain === "s.pornpics.today" ||
			domain === "s.hardcore.pink" ||
			domain === "s.brunette.site" ||
			domain === "s.nudepornpics.xyz" ||
			domain === "s.picsofporn.club" ||
			domain === "s.xpics.info" ||
			domain === "s.sexoic.com" ||
			domain === "s.pornphotos.space" ||
			domain === "s.pornpictures.space" ||
			domain === "s.teenpornpics.today" ||
			domain === "s.pornphotos.pw" ||
			domain === "s.pics.red" ||
			domain === "s.softcore.life") {
			return src.replace(/(\/img\/+(?:[0-9a-f]{2}\/+){4})[wh][0-9]+px\/+/, "$1");
		}

		if (domain === "d17c3xisf00oox.cloudfront.net") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+api\/+image-loader\.php\?(?:.*&)?url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "darkmovie.info") {
			return src.replace(/(\/upload\/+cast\/+nm[0-9]+)_[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "shutterstock.com" && /^editorial[0-9]*\./.test(domain)) {
			id = src.replace(/^[a-z]+:\/\/[^/]*\/+[^/]*\/+([0-9]+[a-z])\/+.*/, "$1");
			if (id !== src && options.do_request && options.cb) {
				api_cache.fetch("shutterstock_editorial:" + id, function(data) {
					options.cb(data);
				}, function (done) {
					page = "https://shutterstock.com/editorial/image-editorial/" + id;

					options.do_request({
						method: "GET",
						url: page,
						headers: {
							Referer: ""
						},
						onload: function (result) {
							if (result.readyState !== 4)
								return;

							if (result.status !== 200)
								return done(null, false);

							var base_obj = {
								url: src,
								extra: {
									page: result.finalUrl
								}
							};

							var orig_obj = base_obj;

							var obj = [];

							var match = result.responseText.match(/<meta\s+property=["']og:image["']\s+content=["'](.*?)["']/);
							if (match) {
								base_obj = deepcopy(base_obj);
								base_obj.url = decode_entities(match[1]);
								base_obj.problems = { watermark: true };
								obj.push(base_obj);
							}

							var match = result.responseText.match(/<script\s+type=["']application\/ld\+json["']>\s*(\[.*?\])\s*<\/script>/);
							if (match) {
								var json_obj = JSON_parse(match[1]);
								var url = json_obj[1].thumbnail;
								base_obj = deepcopy(base_obj);
								base_obj.url = url;
								base_obj.problems = { smaller: true };
								obj.push(base_obj);
							}

							obj.push(orig_obj);

							if (obj.length > 0)
								return done(obj, 6*60*60);
							else
								return done(null, false);
						}
					});
				});

				return {
					waiting: true
				};
			}
		}

		if (domain === "images.vinted.net") {
			return src.replace(/\/thumbs\/+(?:[0-9]+x[0-9]+|f[0-9]+)\/+/, "/thumbs/");
		}

		if (domain_nowww === "celebnakedness.com") {
			return src.replace(/(\/gallery[0-9]*\/+tourgallery\/+)thumbnails\/+/, "$1");
		}

		if (domain === "userimg.tellonym.me") {
			return src.replace(/:\/\/[^/]*\/+[a-z]+\/+/, "://img.tellonym.me/");
		}

		if (domain === "img.omni7.jp") {
			return src.replace(/(\/product\/+[0-9]+\/+[0-9]+\/+image\/+[0-9]+)_main_[sl](\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "d2jtsb989t238a.cloudfront.net") {
			return src.replace(/(\/p\/+[0-9]+\/+[0-9]+\/+)small(?:[?#].*)?$/, "$1normal");
		}

		if (domain === "cdnx.natalie.mu") {
			return src.replace(/(?:_fixw_(?:120|234)|_fit_120x120|_fixw_(?:640|730)_hq)\./, "_fixw_750_lt.");
		}

		if (domain_nowww === "gohong01.com") {
			return src.replace(/(\/img\/+[0-9]+\/+)mini(\.[^/.]*)(?:[?#].*)?$/, "$1t$2");
		}

		if (domain === "static.mercdn.net") {
			return src.replace(/\/thumb\/+([a-z]+)\/+/, "/item/detail/orig/$1/");
		}

		if ((domain_nosub === "readthis.one" && /^s[0-9]*\./.test(domain))) {
			return src.replace(/(\/yimg\/+[^/]{50,}\.[^/._]+)_[0-9]+\.[^/.]*(?:[?#].*)?$/, "$1");
		}

		if (domain === "photos.fashiongroup.com") {
			newsrc = src.replace(/(\/[0-9]{4}-[0-9]{2}-[0-9]{2}\/+[0-9]+\/+)thumbs_([^/]*\.[^/.]*)(?:[?#].*)?$/, "$1$2");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/(\/[0-9]{4}-[0-9]{2}-[0-9]{2}\/+[0-9]+\/+)([^/]*\.[^/.]*)(?:[?#].*)?$/, "$12048/$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "napiszar.com") {
			return src.replace(/(\/uploads\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9a-f]{30,})_thumb(\.[^/.]*)(?:[?#].*)?$/, "$1_origi$2");
		}

		if (domain_nowww === "incontri-passionali.com" ||
			domain_nowww === "incontri-in-italia.com" ||
			domain_nowww === "incontri-bakeka.com" ||
			domain_nowww === "sesso-escort.com") {
			return src.replace(/(\/images\/+[^/]+\/+)med-([0-9]+-[0-9]+\.)/, "$1max-$2");
		}

		if (domain_nowww === "filmyerotyczne.tv") {
			return src.replace(/(\/gallery\/+[^/]*\/+[0-9]+_[0-9]+\/+)medium\/+/, "$1large/");
		}

		if (domain_nowww === "idposter.com") {
			return src.replace(/\/img\/+smalls\/+([0-9]+\/+)s_id/, "/img/bigs/$1id");
		}

		if (domain_nowww === "fappeningbook.com") {
			return src.replace(/(\/photos\/+.\/+.\/+[^/]*\/+[0-9]+\/+[0-9]+)t(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "pics.hotsexygirls.eu") {
			newsrc = src.replace(/\/cache\/+([^/]*\.[^/.]*)\.thumb(\.[^/.]+)(?:[?#].*)?$/, "/cache/$1.small$2");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/\/cache\/+([^/_]+)_([^/]*\.[^/.]*)\.[a-z]+\.[^/.]*(?:[?#].*)?$/);
			if (match) {
				return urljoin(src, "/image.php?twg_album=" + decodeURIComponent(match[1]) + "&twg_show=" + decodeURIComponent(match[2]), true);
			}

			match = src.match(/\/image\.php\?(.*?&)?twg_album=/);
			if (match) {
				return {
					url: urljoin(src, "/image.php?twg_album=" + encodeURIComponent(url.searchParams.get("twg_album")) + "&twg_show=" + encodeURIComponent(url.searchParams.get("twg_show")), true),
					extra: {
						page: "http://pics.hotsexygirls.eu/?twg_album=" + encodeURIComponent(url.searchParams.get("twg_album")) + "&twg_show=" + encodeURIComponent(url.searchParams.get("twg_show"))
					}
				};
			}
		}

		if (domain === "mygalleries.allpornmodels.com") {
			return src.replace(/(:\/\/[^/]*\/+[^/]*\/+)small\/+/, "$1");
		}

		if (domain === "forum.donanimhaber.com") {
			newsrc = src.replace(/.*?\/cache\/+ImageThumbnail2\.aspx\?(?:.*&)?path=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nosub === "newtumbl.com") {
			obj = {
				url: src,
				headers: {
					Referer: ""
				},
				bad_if: [{
					headers: {
						"content-type": "image/jpg",
						"content-length": "688362"
					}
				}],
				referer_ok: {
					same_domain_nosub: true
				},
				can_head: false // completely wrong output
			};

			newsrc = src.replace(/(\/img\/+[0-9]+\/+[0-9]+\/+[0-9]+\/+[0-9]+\/+nT_[0-9a-z]{10,})_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
			if (newsrc !== src) {
				obj.url = newsrc;
				return obj;
			}

			regex = /(\/img\/+[0-9]+\/+[0-9]+\/+[0-9]+\/+[0-9]+\/+nT_[0-9a-z]{10,})\.[^/.]+(?:[?#].*)?$/;

			if (regex.test(src) && options && options.do_request && options.cb) {
				var cachelink = src.replace(regex, "$1");
				api_cache.fetch("newtumbl:" + cachelink, function(url) {
					if (url) {
						obj.url = url;
					}

					options.cb(obj);
				}, function(done) {
					var req;

					var do_test = function(ext) {
						loaded = false;
						loading_ext = ext;
						var loadfunc = function(resp) {
							onload(resp, ext);
						};

						req = options.do_request({
							url: src.replace(regex, "$1." + ext),
							method: "GET",
							headers: {
								Referer: ""
							},
							onprogress: loadfunc,
							onload: loadfunc
						});
					};

					var loaded = false;
					var loading_ext;
					var onload = function(resp, current_ext) {
						if (loading_ext !== current_ext || loaded) {
							req.abort();
							return;
						}

						if (resp.readyState === 4 || resp.responseHeaders) {
							loaded = true;
							req.abort();
						}

						if (!loaded)
							return;

						if (resp.status !== 200) {
							return done(null, false);
						}

						var headers_list = parse_headers(resp.responseHeaders);
						var headers = headers_list_to_dict(headers_list);

						var splitted = headers["content-type"].split("/");
						var mediatype = splitted[0];
						var ext = splitted[1];

						var newsrc = src.replace(regex, "$1." + ext);

						if (current_ext === "mp4") {
							if (mediatype === "video") {
								return done(newsrc, 24*60*60);
							} else {
								do_test("gif");
							}
						} else {
							return done(newsrc, 24*60*60);
						}
					};

					do_test("mp4");
				});

				return {
					waiting: true
				};
			}

			newsrc = src.replace(/(\/img\/+[0-9]+\/+[0-9]+\/+[0-9]+\/+[0-9]+\/+nT_[0-9a-z]{10,})\.(?:jpg|JPG|jpeg|JPEG|png|PNG|webp|WEBP)(?:[?#].*)?$/, "$1.mp4");
			if (newsrc !== src) {
				obj.url = newsrc;
				return obj;
			}
		}

		if (domain === "img.ohpolly.com") {
			return src.replace(/:\/\/[^/]*\/+[^/]*=\/+(?:.*?\/)?(catalog\/+product\/)/, "://www.ohpolly.com/media/$1");
		}

		if (domain_nowww === "brightvibes.com") {
			newsrc = src.replace(/(\/file\/+[^/]*)\/+[0-9]+x[0-9]+(?:[?#].*)?$/, "$1");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/\/image-cache\/+[0-9]+x[0-9]+-([^/]*\.[^/.]*)(?:[?#].*)?$/, "/images/$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "cdn.webshopapp.com" ||
			domain === "cdn.shoplightspeed.com") {
			return src.replace(/(\/files\/+[0-9]+\/+)[0-9]+x[0-9]+(?:x[0-9])?\/+/, "$1");
		}

		if (domain_nowww === "hdnicewallpapers.com") {
			return src.replace(/(:\/\/[^/]*\/+)walls\/+[a-z]+\/+/i, "$1Walls/Big/");
		}

		if (domain === "img.theculturetrip.com") {
			return src.replace(/\/[0-9]*x[0-9]*\/+wp-content\/+/, "/wp-content/");
		}

		if (domain_nosub === "ning.com" && /^(?:st[0-9]*|storage)\./.test(domain)) {
			return src.replace(/(\/file\/+get\/+[0-9]+)\?.*$/, "$1?profile=original");
		}

		if (domain_nowww === "comedywildlifephoto.com") {
			return src.replace(/(\/images\/+gallery\/+[0-9]+\/+[0-9]+)_t(\.[^/.]*)(?:[?#].*)?$/, "$1_p$2");
		}

		if (domain_nowww === "islam.kz") {
			return src.replace(/(\/uploads\/+images\/+(?:[^/]{3}\/+){2}[^/.]*)-(?:sm|md)(\.[^/.]*)(?:[?#].*)?$/, "$1-lg$2");
		}

		if (domain_nowww === "todofondos.com" && options && options.cb && options.do_request) {
			id = src.replace(/.*\/bin\/+fondos\/+((?:[0-9]{2}\/+){2}[0-9]+)[a-z]\.[^/.]*(?:[?#].*)?$/, "$1");
			if (id !== src) {
				options.do_request({
					url: "http://todofondos.com/f/" + id.replace(/\/+/g, "").replace(/^0*/, ""),
					method: "GET",
					headers: {
						Referer: ""
					},
					onload: function(result) {
						if (result.readyState !== 4)
							return;

						if (result.status !== 200)
							return options.cb(null);

						var match = result.responseText.match(/<button[^>]*onclick="\s*javascript:location.href='(https?:\/\/todofondos\.com\/+f\/+[0-9]+\/+[0-9]+x[0-9]+)'"[^>]*>\s*<span[^>]*>\s*<\/span>\s*resoluci.n\s+original/);
						if (!match)
							return options.cb(null);

						options.do_request({
							url: match[1],
							method: "GET",
							headers: {
								Referer: ""
							},
							onload: function(result) {
								if (result.readyState !== 4)
									return;

								if (result.status !== 200)
									return options.cb(null);

								var match = result.responseText.match(/<a href="([^"']*\/bin\/descargas\/.*?)"/);
								if (!match)
									return options.cb(null);

								return options.cb({
									url: match[1],
									extra: {
										page: result.finalUrl
									}
								});
							}
						});
					}
				});

				return {
					waiting: true
				};
			}
		}

		if (domain === "d2w9rnfcy7mm78.cloudfront.net") {
			return src.replace(/(\/[0-9]+\/+)[a-z]+_([0-9a-f]{20,}\.[^/.]*)(?:[?#].*)?$/, "$1original_$2");
		}

		if (domain === "birkrdnawa.com") {
			return src.replace(/(\/content\/+siteimages\/+[a-z]+\/+)thumb\/+/i, "$1");
		}

		if (domain === "media.bernardinai.lt") {
			return src.replace(/(\/o\/+[0-9a-f]{20,})_[_a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "econet.ru") {
			return src.replace(/(\/media\/+video_covers\/+[0-9]+\/+)[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1original$2");
		}

		if (domain_nowww === "displate.com") {
			return src.replace(/\/displates\/+(?:[0-9]+\/+[a-z]+\/+)?([0-9]{4}-[0-9]{2}-[0-9]{2}\/+[^/]*?)(?:[?#].*)?$/, "/displates/$1");
		}

		if (domain_nowww === "lens2print.co.uk") {
			return src.replace(/(\/cms\/+upload_area\/+images\/+[0-9]+_processed\/+)(?:medium|crop)\/+/, "$1large/");
		}

		if (domain_nowww === "iqna.ir" ||
			domain_nowww === "parsine.com") {
			return src.replace(/(\/files\/+[^/]*\/+news_albums\/+[0-9]+\/+[0-9]+\/+)thumbnails\/+thm_/, "$1resized/resized_");
		}

		if (domain_nowww === "az-deteto.bg") {
			return src.replace(/(\/pic\/+article\/+)logo\/+/, "$1picture/");
		}

		if (domain_nosub === "efreenews.com") {
			return src.replace(/(\/storage\/+posts\/+[A-Za-z]+[0-9]{4}\/+[0-9a-f]{20,})-thumb(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "iziva.com") {
			return {
				url: src.replace(/\/media\/+jact\/+[^/]*\/+images\/+/, "/images/"),
				headers: {
					Referer: ""
				},
				referer_ok: {
					same_domain: true
				}
			};
		}

		if ((domain_nosub === "viewbug.com" && /^cdnpt[0-9]*\./.test(domain)) ||
			amazon_container === "photo-viewbug") {
			return src.replace(/(\/media\/+mediafiles\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9]+)_widepreview400(\.[^/.]*)(?:[?#].*)?$/, "$1_large1300$2");
		}

		if (domain === "bilder.t-online.de") {
			return src.replace(/(\/id_[0-9]+\/+)(?:[0-9]+(?:_[0-9]+)?[wh]?\/+)?(?:c_[^/]*\/+)?tid_da\/+/, "$1c_raw/tid_da/");
		}

		if (domain_nowww === "khabarban.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+NewsImage\/+[0-9]+\/+([^/.]{50,})(?:[?#].*)?$/, "$1");
			if (newsrc !== src) {
				return base64_decode(newsrc);
			}
		}

		if ((domain_nosub === "imagecollect.com" && /^static[0-9]*\./.test(domain)) ||
			amazon_container === "live-imagecollect") {
			newsrc = src.replace(/\/thumbnail\/+([0-9]+\/+[0-9a-f]+)(?:[?#].*)?$/, "/preview/$1");
			if (newsrc !== src)
				return {
					url: newsrc,
					problems: {
						watermark: true
					}
				};
		}

		if (domain_nosub === "corriereobjects.it" && /^images[0-9]*\./.test(domain)) {
			return src.replace(/_MGZOOM(\.[^/.]*)(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "mojeek.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+image\?(?:.*&)?img=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "pic.templetons.com") {
			return src.replace(/\/thumb\/+(img_[0-9]+\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain === "assets.yandycdn.com") {
			return src.replace(/\/(?:t[0-9]*(?:-compressed)?|Products)\/+/, "/HiRez/");
		}

		if (domain_nosub === "facciabuco.com" && /^cdn-img-[a-z]\./.test(domain)) {
			return src.replace(/_[ac](\.[^/.]*)(?:[?#].*)?$/, "_b$1");
		}

		if (domain_nowww === "uskinfo.ba") {
			return src.replace(/(\/files\/+[^/]*\/+)thumbs\/+/, "$1");
		}

		if (domain_nowww === "drago99.com") {
			return src.replace(/(\/Drago99\/+[^/]*\/+)tn([^/]*\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "galleries.justebonysex.com") {
			return src.replace(/(\/[0-9]+\/+)t(p[0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "intim-photos.ru") {
			return src.replace(/(\/photos\/+[0-9]+\/+[0-9]+\/+)thumb\/+/, "$1");
		}

		if (domain === "hwnds.ddfstatic.com") {
			return src.replace(/(\/content\/.*\/)thu\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1fulm/$2");
		}

		if (domain_nosub === "mlo.me") {
			newsrc = src.replace(/(:\/\/[^/]*\/+upen\/+)[sm]\/+/, "$1l/");
			if (newsrc !== src) {
				return newsrc;
			}

			newsrc = src.replace(/(:\/\/[^/]+\/+upen\/+)l\/+/, "$1v/");
			if (newsrc !== src) {
				return {
					url: newsrc,
					problems: {
						watermark: true
					}
				};
			}
		}

		if (domain_nowww === "lilykoh.com") {
			return src.replace(/(\/content\/+[^/]*\/+)[0-9]+tn([0-9]+\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "utporn.com" && /^pics[0-9]*\./.test(domain)) {
			return src.replace(/(\/albums\/+[0-9]+\/+[0-9]+-[^/]+\/+)[0-9]+(?:x[0-9]+)?\/+/, "$1");
		}

		if (domain_nowww === "girlsilove.net" ||
			domain_nowww === "tight-clothed.com" ||
			domain_nowww === "teen-angels.org" ||
			domain_nowww === "nude-beauty.net" ||
			domain_nowww === "rgsex.com") {
			return src.replace(/(\/galleries\/+[^/]*\/+[^/]*\/+(?:[^/]*\/+)?|\/archive\/+[^/]*\/+.\/+[^/]*\/+[0-9]+\/+)p-([^/]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "easyhotpics.com") {
			return src
				.replace(/(\/tgp\/+[^/]+\/+[0-9]+\/+)t([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1i$2")
				.replace(/(\/tgp\/+[^/]*\/+[^/]*\/+)t([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "cdn.pax6.com") {
			return src.replace(/(\/galleries\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9a-f]{10,}\/+images\/+)t([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1i$2");
		}

		if (domain === "promo.hollyrandall.com") {
			return src.replace(/\/img\/+tn-([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/img/img-$1");
		}

		if (domain_nowww === "bikinidream.net") {
			return src.replace(/(\/gals\/+[^/]+\/+)th_([0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "cdnberry.com") {
			return src.replace(/(\/posts\/+[^/]+\/+)th_([^/]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "nakedebonybabes.com") {
			return src.replace(/(\/[0-9a-f]+\/+)th_([0-9]+\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "picua.org") {
			return src.replace(/(:\/\/[^/]*\/+)thumbs\/+([0-9]{4}-[0-9]{2}\/+)/, "$1img/$2");
		}

		if (domain_nowww === "mariuszlewandowski.pl" ||
			domain_nowww === "voiceshop.pl") {
			return src.replace(/\/environment\/+cache\/+images\/+[0-9]+_[0-9]+_productGfx_/, "/userdata/gfx/");
		}

		if (domain_nowww === "s5o.ru" ||
			domain === "s5o.sports.ru") {
			return src.replace(/(\/storage\/+simple\/.*\/[0-9a-z]+)\.[0-9]+\.[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "trunkarchive.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+A\.aspx\?(?:.*?&)?url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return urljoin(src, decodeuri_ifneeded(newsrc), true);
		}

		if (domain === "img.mbjuan.com") {
			return src.replace(/(\/content\/+media\/+thumbs\/+p[0-9]+)_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "ua.today") {
			return src.replace(/\/resize_img\/+[0-9]+x[0-9]+\/+images\/+/, "/images/");
		}

		if (domain_nowww === "nashaamerica.com") {
			return src.replace(/(\/files\/+news\/+[0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+)[^/]*\/+([0-9a-f]{10,}\.[^/.]*)(?:[?#].*)?$/, "$1original/$2");
		}

		if (domain_nowww === "bootysource.com") {
			return src.replace(/(\/img2\/+[^/]*)_small(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "img.freshpopmusic.co") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+hsi\/+([^/.]{50,})\/+.*/, "$1");
			if (newsrc !== src)
				return base64_decode(newsrc);
		}

		if (domain_nosub === "lifo.gr") {
			return src.replace(/\/icache\/+[0-9]+\/+[0-9]+\/+[0-9]\/+([0-9]+)_/, "/uploads/image/$1/");
		}

		if (domain_nowww === "epicenter.bg") {
			return src.replace(/(\/images\/+news\/+[0-9]+\/+)thumbs\/+[a-z]+_/, "$1pics/");
		}

		if (domain_nosub === "yokacdn.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+a\.jpg\?(?:.*&)?imageurl=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (amazon_container === "stezor-img-res") {
			return src.replace(/\/[0-9]+x[0-9]+\/+([-0-9a-f]{20,})(?:[?#].*)?$/, "/$1");
		}

		if (domain === "cdn.letrasdemusicas.fm") {
			return src.replace(/(\/production\/+images\/+[0-9]{4}\/+[0-9]{2}\/+)cover_([^/]*\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "img.erogazou.gallery" ||
			domain_nosub === "erogazounosuke.com") {
			return src.replace(/(\/[a-z]+[0-9]*\/+[0-9]+\/+)s(\/+[0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1b$2");
		}

		if (domain === "screens.latestscreens.com") {
			return src.replace(/(\/screenshots\/+[^/]+\/+)thumbs\/+/, "$1");
		}

		if (domain_nowww === "perfect-girls-net.com") {
			return src.replace(/\/thumbs\/+([^/]+\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nowww === "mobygames.com") {
			return src.replace(/(\/images\/+shots\/+)[st]\/+/, "$1l/");
		}

		if (domain === "img.bhs4.com") {
			return src.replace(/(\/[0-9a-f]{2}\/+[0-9a-f]\/+[0-9a-f]{20,})_[a-z]+(\.[^/.]*)(?:[?#].*)?$/, "$1_orig$2");
		}

		if (domain === "pisces.bbystatic.com") {
			return src.replace(/(\/images\/+products\/+[0-9]+\/+[0-9]+(?:_[^/.]*)\.[^/.;]+);.*/, "$1");
		}

		if (domain_nowww === "celebs101.com") {
			return src.replace(/(\/gallery\/+[^/]*\/+[0-9]+\/+)tn_([^/]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "nnconnect.com") {
			newsrc = src.replace(/(:\/\/[^/]+\/[^/]+\/+[^/]+\/+)tn_([^/]+-[0-9]+\.[^/.]+)(?:[?#].*)?$/, "$1$2");
			if (newsrc !== src)
				return {
					url: newsrc,
					extra: {
						page: src.replace(/(:\/\/[^/]+\/+[^/]+\/+[^/]+)\/.*$/, "$1.html")
					}
				};
		}

		if (domain_nowww === "hq-celebrity.com") {
			newsrc = src.replace(/(\/photos\/+[^/]*\/+)tn_([^/]*)(?:[?#].*)?$/, "$1$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "hq-celebrity.com") {
			newsrc = src.replace(/\/wp-content\/+plugins\/+watermark-hotlinked-images\/+watermark\.php\?(?:.*&)?img=([^&]*).*?$/, "/$1");
			if (newsrc !== src)
				return newsrc;

			return {
				url: src,
				headers: {
					Referer: ""
				}
			};
		}

		if (domain === "d3bhdfps5qyllw.cloudfront.net") {
			return src.replace(/(:\/\/[^/]*\/+)(?:sqr|thm)\/+/, "$1org/");
		}

		if (domain_nowww === "imageocd.com") {
			return src
				.replace(/(\/images\/+[^/]+\/+[^/]+\/+)main\/+([^/]*?\.[^/.]*)\.thumb_[0-9]+_[a-z]+\.[^/.]*(?:[?#].*)?$/, "$1$2")
				.replace(/(\/u\/+[0-9]+\/+)m_[a-z]\/+([0-9]+\/+)[^/]*\/+([^/]*?\.[^/.]*)\.thumb_[0-9]+_[a-z]+\.[^/.]*(?:[?#].*)?$/, "$1i/$2$3");
		}

		if (domain_nowww === "kickasspays.com") {
			return src.replace(/(\/galleries\/+[^/]*\/+[^/]*)_t(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "xnudehack.com") {
			return src.replace(/\/imgwp\/+t_/, "/imgcache/");
		}

		if (domain === "media.karups.com" ||
			domain === "media.boyfun.com") {
			newsrc = src.replace(/(\/lowres\/+)thumbs\/+([^/]*)\.thumb(\.[^/.]*)(?:[?#].*)?$/, "$1$2$3");
			if (newsrc !== src)
				return newsrc;

			return src.replace(/\/lowres\/+([^/]*)_lowres(_[0-9]+\.[^/.]*)(?:[?#].*)?$/, "/highres/$1_highres$2");
		}

		if (domain_nowww === "freefeet4u.com") {
			return src.replace(/(_[0-9]+)_t(\.[^/.]*)(?:[?#].*)?$/, "$1_b$2");
		}

		if (domain === "content.atkingdom.com") {
			return src.replace(/(\/models\/+[^/]*\/+[0-9]+\/+[0-9]+\/+[^/]*)_tn(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "atkingdom-network.com" && /^(?:cdn[0-9]*|content)\./.test(domain)) {
			return src.replace(/(\/secure\/+content\/+.\/+[^/]+\/+[0-9]+\/+)(?:thumbs\/+)?([^/?]+)(?:[?#].*)?$/, "$13000/$2");
		}

		if (domain === "bl.definefetish.com") {
			return src.replace(/\/thumb\/+([^/]+\.[^/.]*)(?:[?#].*)?$/, "/$1");
		}

		if (domain === "fileup.calobye.com") {
			return src.replace(/\/m_[0-9]+\/+upload\/+/, "/upload/");
		}

		if (domain_nowww === "chyoa.com") {
			return src.replace(/\/data\/+covers\/+[sm]\/+/, "/data/covers/l/");
		}

		if (domain_nowww === "m1sv.net") {
			return src.replace(/(\/[0-9]+\/+)Resize\/+/, "$1");
		}

		if (domain === "image.i-bbs.sijex.net") {
			return {
				url: src.replace(/(\/[0-9]{5,})[sm](\.[^/.]*)(?:[?#].*)?$/, "$1o$2"),
				headers: {
					Referer: ""
				}
			};
		}

		if (domain_nowww === "pic-b.com") {
			return src.replace(/(\/photo\/+[0-9]+\/+[0-9]+\/+[-0-9]{20,})(\.[^/.]*)(?:[?#].*)?$/, "$1-pc$2");
		}

		if (domain_nosub === "momogaki.com") {
			return src.replace(/\/img2\/+([0-9]+\.[^/.]*)(?:[?#].*)?$/, "/img/$1");
		}

		if (domain === "images.contentexchange.me") {
			newsrc = src.replace(/.*?(?:.*&)?url=(https?%[^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "thumbs.uloz.to") {
			return src.replace(/(\/[0-9a-f]\/+[0-9a-f]\/+[0-9a-f]\/+[0-9a-f]{20,})\.[0-9]+x[0-9]+(\.[^/.?]*)(?:[?#].*)?$/, "$1.9999999999999x9999999999999$2");
		}

		if (domain_nosub === "patreonusercontent.com" && options && options.do_request && options.cb) {
			var postid_regex = /\/([^/]*)\/+patreon-media\/+p\/+post\/+([0-9]+)\/([0-9a-f]{20,})\/+/;
			var postid = src.match(postid_regex);
			if (postid) {
				var modifiers = postid[1];
				var posthash = postid[3];
				postid = postid[2];

				var are_modifiers_original = function(modifiers) {
					try {
						var obj = JSON_parse(base64_decode(decodeURIComponent(modifiers)));
						var obj_string = JSON_stringify(obj);

						return obj_string === JSON_stringify({}) || obj_string === JSON_stringify({"p":1});
					} catch (e) {
						console_log(modifiers);
						console_error(e);
						return false;
					}
				};

				var find_id_in_included = function(included, id) {
					for (var i = 0; i < included.length; i++) {
						if (included[i].id === id)
							return included[i];
					}

					return null;
				}

				var page = "https://www.patreon.com/posts/" + postid;

				if (are_modifiers_original(modifiers)) {
					return {
						url: src,
						is_original: true,
						extra: {
							page: page
						}
					};
				}

				var do_website_request = function(page, cb) {
					var cache_key = "patreon_website:" + page;
					api_cache.fetch(cache_key, cb, function (done) {
						options.do_request({
							url: page,
							method: "GET",
							onload: function (result) {
								if (result.readyState !== 4)
									return;

								if (result.status !== 200) {
									console_error(cache_key, result);
									return done(null, false);
								}

								var match = result.responseText.match(/Object\.assign\s*\(\s*window\.patreon\.bootstrap\s*,\s*({[\s\S]*?})\);\s*Object\./);
								if (match) {
									try {
										var json = JSON_parse(match[1]);
										return done(json, 60*60);
									} catch (e) {
										console_error(cache_key, e);
									}
								}

								return done(null, false);
							}
						});
					});
				};

				var get_image_from_postdata = function(postdata, imageurl) {
					var image_ids = postdata.post.data.attributes.post_metadata.image_order;
					var included = postdata.post.included;
					var posthash = imageurl.match(postid_regex)[3];

					for (var i = 0; i < image_ids.length; i++) {
						var image_data = find_id_in_included(included, image_ids[i]).attributes;

						var image_url;

						if (image_data.image_urls.original) {
							image_url = image_data.image_urls.original;
						} else if (image_data.download_url) {
							image_url = image_data.image_urls.download_url;
						} else if (image_data.image_urls.default) {
							image_url = image_data.image_urls.default;
						}

						if (!image_url)
							continue;

						var image_match = image_url.match(postid_regex);
						if (image_match[3] !== posthash)
							continue;

						return image_url;
					}

					return null;
				};

				var find_larger_postimage = function(imageurl, cb) {
					var postmatch = imageurl.match(postid_regex);
					var cache_key = "patreon_postimage:" + postmatch[2] + "/" + postmatch[3];

					api_cache.fetch(cache_key, function(result) {
						obj = {
							url: result,
							extra: {
								page: page
							}
						};

						if (!result)
							return cb(null);

						var match = result.match(postid_regex);
						if (match && are_modifiers_original(match[1])) {
							obj.is_original = true;
						}

						cb(obj);
					}, function(done) {
						do_website_request(page, function(result) {
							if (!result) {
								return done(null, false);
							}

							try {
								imageurl = get_image_from_postdata(result, imageurl);
								return done(imageurl, 60*60); // FIXME: figure out when these URLs expire
							} catch(e) {
								console_error(cache_key, e);
							}

							done(null, false);
						});
					});
				};

				find_larger_postimage(src, options.cb);

				return {
					waiting: true
				};
			}
		}

		if (domain === "s3.dexerto.com") {
			return src.replace(/\/thumbnails\/+_thumbnailSmall\/+/, "/thumbnails/_thumbnailLarge/");
		}

		if (domain === "scp.indiegames.us") {
			return src.replace(/(\/scn\/+[^/]*\/+[^/]+)_tn(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "content.ray-web.jp") {
			if (/\/photos\/+pictures\/+[0-9]+\/+original\/+[0-9a-f]{20,}\./.test(src))
				return {
					url: src,
					is_original: true
				};
		}

		if (domain === "content.ray-web.jp" && options && options.cb && options.do_request && options.element &&
			host_domain_nowww === "ray-web.jp") {

			regex = /\/photos\/+pictures\/+([0-9]+)\/+([a-z]+)\/+[0-9a-f]{20,}\./;
			match = src.match(regex);

			if (match) {
				var get_og_image = function(data) {
					var url = data.match(/<meta\s+property=["']og:image["']\s+content=["'](https?:.*?)["']/);
					if (url)
						return decode_entities(url[1]);
					return null;
				};

				var fetch_image_page = function(albumid, pictureid, cb) {
					var cache_key = "ray-web:" + albumid + "-" + pictureid;
					api_cache.fetch(cache_key, cb, function(done) {
						options.do_request({
							url: "https://ray-web.jp/" + albumid + "/photos/" + pictureid,
							method: "GET",
							onload: function(result) {
								if (result.readyState !== 4)
									return;

								if (result.status !== 200) {
									console_error(cache_key, result);
									return done(null, false);
								}

								return done(result.responseText, 6*60*60);
							}
						});
					});
				};

				var rayweb_url_to_parts = function(url) {
					var match = url.match(/:\/\/[^/]*\/+([0-9]+)\/+photos\/+([0-9]+)(?:[?#].*)?$/);
					if (match) {
						return [match[1], match[2]];
					} else {
						return null;
					}
				};

				var parts = null;
				if (options.element.parentElement.tagName === "A") {
					parts = rayweb_url_to_parts(options.element.parentElement.href);
				} else {
					parts = rayweb_url_to_parts(options.host_url);
					if (!parts || match[1] !== parts[1])
						parts = null;
				}

				if (parts) {
					fetch_image_page(parts[0], parts[1], function (data) {
						options.cb(get_og_image(data));
					});

					return {
						waiting: true
					};
				}
			}
		}

		if (domain_nosub === "icdn.ru") {
			regex = /:\/\/s[0-9]+\.[^/]+\/+.\/+([^/]+)\/+[0-9]+\/+imgsrc\.ru_([0-9]+)...(\.[^/.]*)(?:[?#].*)?$/;

			match = src.match(regex);
			if (match) {
				obj = {
					url: src,
					extra: {
						page: "https://imgsrc.ru/" + match[1] + "/" + match[2] + ".html"
					}
				};

				if (options && options.do_request && options.cb) {
					var cache_key = "imgsrc.ru:" + match[1] + "/" + match[2];
					api_cache.fetch(cache_key, function(url) {
						if (!url)
							return options.cb(null);
						obj.url = url;
						options.cb(obj);
					}, function(done) {
						options.do_request({
							url: obj.extra.page,
							method: "GET",
							headers: {
								Referer: ""
							},
							onload: function(result) {
								if (result.readyState !== 4)
									return;

								if (result.status !== 200) {
									console_error(cache_key, "Bad status (" + result.status + ":", result);
									return done(null, false);
								}

								var match = result.responseText.match(/<script(?:\s+type=["']text\/javascript["'])?>[\s\S]*?var\s+(\S+)\s*=\s*["'](.*?)["'][\s\S]*?["']\+((?:\1(?:\s*\.\s*charAt\s*\(\s*|\s*\[\s*)([0-9]+)\s*(?:\)|\])\s*\+\s*){1,})\s*['"][\s\S]*?<\/script>/);
								if (!match) {
									console_error(cache_key, "Invalid match", result);
									return done(null, false);
								}

								var splitted = match[3].split("+");
								var chars = "";
								for (var i = 0; i < splitted.length; i++) {
									var charid = splitted[i].replace(/.*(?:\[|\()\s*([0-9]+).*?$/, "$1");
									if (charid !== splitted[i])
										chars += match[2].charAt(charid);
								}

								if (chars.length !== 3) {
									console_error(cache_key, "Invalid chars:", chars, result);
									return done(null, false);
								}

								done(src.replace(/:\/\/s([0-9]+.*_[0-9]+)...(\.[^/.]*)(?:[?#].*)?$/, "://b$1" + chars + "$2"), 6*60*60);
							}
						});
					});

					return {
						waiting: true
					};
				}
			}
		}

		if (domain === "rs.fairywigs.com") {
			return src.replace(/(\/images\/.*\/[0-9]+\/+[0-9]+_[0-9]+)_s(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "epio.app" && /^cdn[0-9]*-images\./.test(domain)) {
			newsrc = src.replace(/.*?\/image\/+download\/+([^./]{40,})(?:\/+[0-9]+X[0-9]+)?(?:[?#].*)?$/, "$1");
			if (newsrc !== src)
				return base64_decode(newsrc);
		}

		if (domain === "giga-images-makeshop-jp.akamaized.net") {
			return src.replace(/(\/shopimages\/+[0-9]+\/+[0-9]+\/+)s([0-9]+_[0-9]+\.[^/.]*)(?:[?#].&)?$/, "$1$2");
		}

		if (domain === "cdn-uploads.gameblog.fr") {
			return src.replace(/(\/images\/+[^/]+\/+)tn\/+/, "$1");
		}

		if (domain_nowww === "guru3d.com") {
			return src.replace(/(\/index\.php\?(?:.*?&)?action=)thumb(&.*)?$/, "$1file$2");
		}

		if (domain === "medialazy.vandalimg.com") {
			return src.replace(/:\/\/[^/]+\/+/, "://media.vandal.net/");
		}

		if (domain === "mediamaster.vandal.net" ||
			domain === "media.vandalsports.com" ||
			domain === "media.vandal.net") {
			return src.replace(/(:\/\/[^/]+\/+)(?:m|i\/+[0-9]+x[0-9]+)\/+/, "$1master/");
		}

		if (domain_nosub === "cgames.de" && /^[0-9]*images\./.test(domain)) {
			return src.replace(/(\/images\/+gamestar\/+)[0-9]+\/+/, "$14/");
		}

		if (domain === "assets.folhavitoria.com.br") {
			return src.replace(/(\/images\/+[-0-9a-f]{30,})--(?:(?:[a-z][^-_/.]+_)*[a-z][^-_/.]+)(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "dnh0aphdpud22.cloudfront.net") {
			return src.replace(/(\/sized\/+[0-9a-f]+-[A-Z]+[0-9]+\/+)med_thumbnail(\.[^/.]*)(?:[?#].*)?$/, "$1high_res$2");
		}

		if (domain_nowww === "iansphoto.in") {
			return src.replace(/(\/photoimages_new\/+)[0-9]+\/+([0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+)/, "$1$2");
		}

		if (domain_nowww === "mamby.com") {
			return src.replace(/(\/images\/+[^/]*)_mediumres(\.[^/.]*)(?:[?#].*)?$/, "$1_maxres$2");
		}

		if (domain_nowww === "pixelup.net") {
			return src.replace(/(\/images\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9a-f]{20,}\/+)p(\.[^/.]*)(?:[?#].*)?$/, "$1r$2");
		}

		if (domain_nowww === "tattoodaze.com") {
			return src.replace(/(:\/\/[^/]+\/+)images\/+/, "$1tattoo-images/");
		}

		if (domain_nowww === "ximg.pro") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+\?(?:.*&)?url=([^&/]{20,}).*?$/, "$1");
			if (newsrc !== src)
				return base64_decode(newsrc);
		}

		if (domain === "cdnimage.ebn.co.kr" ||
			domain === "file.ebn.co.kr") {
			newsrc = src.replace(/(\/news\/+[0-9]{6}\/+news_[0-9]+_[0-9]+)_thumbs(\.[^/.]*)(?:[?#].*)?$/, "$1_main1$2");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/\/news\/+[0-9]{6}\/+(?:news_[0-9]+|[0-9]{1,3})_([0-9]+)_/);
			if (match) {
				return {
					url: src,
					extra: {
						page: "https://www.ebn.co.kr/news/view/" + match[1]
					}
				};
			}
		}

		if (domain === "img.kbs.co.kr") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+kbs\/+[0-9]+\/+([^/]+\.[^/]+\/+)/, "$1");
			if (newsrc !== src)
				return add_http(newsrc);
		}

		if (domain_nowww === "kpop-today.com") {
			return src.replace(/\/img-(?:small|large)\/+medias\/+/, "/medias/");
		}

		if (domain_nowww === "popnable.com") {
			newsrc = src.replace(/(\/images\/+[a-z]+\/+)small\/+/, "$1large/");
			if (newsrc !== src)
				return newsrc;

			return src.replace(/(\/images\/+[a-z]+\/+)(?:small|large)\/+/, "$1original/");
		}

		if (domain === "media.distractify.com") {
			return src.replace(/(\/brand-img\/+[^/]+\/+)[0-9]+x[0-9]+\/+/, "$10x0/");
		}

		if (domain_nosub === "unblocked-sites.pro" ||
			domain_nosub === "unblockweb.cc" ||
			domain_nosub === "unblock-sites.info" ||
			domain_nosub === "unblock.men" ||
			domain_nosub === "unblock.loan" ||
			domain_nosub === "unblock.science" ||
			domain_nosub === "unblock-sites.xyz" ||
			domain_nosub === "unblock-proxy.xyz" ||
			domain_nosub === "unblock.cricket" ||
			domain_nosub === "unblock.racing" ||
			domain_nosub === "unblocked.nl") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+\?(?:.*?&)?cdURL=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "cdn.btportail.0pb.org") {
			return src.replace(/(\/[0-9a-f]{20,}\/+[0-9]+\/+)[0-9]+\/+[0-9]+\/+([^/]+\.[^/.]*)(?:[?#].*)?$/, "$10/0/$2");
		}

		if (domain_nowww === "cosmopolitan.hr" ||
			domain_nowww === "sensa.hr" ||
			domain_nowww === "lovesensa.rs" ||
			/^[a-z]+:\/\/[^/]+\/+repository\/+images\/+_variations\/+[0-9a-f]\/+[0-9a-f]+\/+[0-9a-f]{20,}(?:-[^/]*?)?_[^/]+(\.[^/.]*)(?:[?#].*)?$/.test(src)) {
			return src.replace(/(\/repository\/+images\/+)_variations\/+([0-9a-f]\/+[0-9a-f]\/+[0-9a-f]{20,}(?:-[^/]*?)?)_[^/]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2$3");
		}

		if (amazon_container === "blogadda") {
			return src.replace(/(\/images\/+posts\/+[^/]+-[0-9]+)-medium(\.[^/.]*)(?:[?#].*)?$/, "$1-large$2");
		}

		if (domain_nosub === "muzmob.org") {
			return src.replace(/(\/news\/+i\/+[0-9]+)_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "static.zone-adulte.com") {
			return src.replace(/(\/img\/+[0-9]+\/+)[0-9]+x[0-9]+\/+/, "$1");
		}

		if (domain_nosub === "ujarani.com" && /^i[0-9]*\./.test(domain)) {
			return src.replace(/(:\/\/[^/]+\/+)4\//, "$13/");
		}

		if (domain === "media.loupak.fun") {
			return src.replace(/\/obrazky_[tn]\//, "/obrazky/");
		}

		if (domain_nowww === "megalyrics.ru") {
			return src.replace(/(\/uploads\/+blog\/+cover\/+[0-9a-f]{2}\/+[0-9a-f]{10,}\/+)cs_crop_main_([0-9a-f]{20,}\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}


		if (domain === "uploads.bemparana.com.br") {
			return src.replace(/(\/upload\/+image\/+noticia\/+)tb[0-9]*\/+/, "$1");
		}

		if (domain_nowww === "beicon.ru") {
			return src.replace(/(\/uploads\/+(?:[0-9a-f]{20,}|[0-9]{10,}))_(?:16_9_[0-9]+|[0-9]+_[0-9]+_[0-9]+_exact)(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "bigmenu.com.br") {
			newsrc = src.replace(/\/temp\/+img\/+(arquivos)_([0-9]+)_(conteudo)_(posts)_([0-9]+)_([a-z]+)_[0-9]+_[0-9]+_[0-9]+_[0-9]+__[a-z]+\.[^/.]*(?:[?#].*)?$/, "/$1/$2/$3/$4/$5.$6");
			if (newsrc !== src)
				return newsrc;

			return src.replace(/\/img\/+[0-9]+x[0-9]+\/+[0-90]+\/+[0-9]+\/+[a-z]+\/+arquivos\/+/, "/arquivos/");
		}

		if (domain_nowww === "pixaap.com") {
			return src.replace(/\/images\/+items_thumb\/+M-/, "/images/items/");
		}

		if (amazon_container === "theshonet-assets") {
			return src.replace(/\/filemanager\/+shared_[0-9]+\/+/, "/filemanager/shared/");
		}

		if (domain === "media.lelombrik.net") {
			return src.replace(/\/t\/+([0-9a-f]{20,})\/+s\/+01(\.[^/.]*)(?:[?#].*)?$/, "/t/$1/f/$1$2");
		}

		if (domain_nowww === "funnymasti.com") {
			return src.replace(/\/files\/+resized\/+[^/]+\/+files\/+/, "/files/");
		}

		if (domain === "proxy.zagon.pe") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+proxify\.php\?(?:.*?&)?q=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return base64_decode(newsrc);
		}

		if (domain_nowww === "wikizeroo.org") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+index\.php\?(?:.*&)?q=([^&]{20,}).*?$/, "$1");
			if (newsrc !== src)
				return base64_decode(newsrc);
		}

		if (domain === "cachedimages.podchaser.com") {
			match = src.match(/:\/\/[^/]+\/+[0-9]+x[0-9]+\/+([^/.]{30,})\/+/);
			if (match) {
				return base64_decode(decodeURIComponent(match[1]));
			}
		}

		if (domain === "ca.slack-edge.com") {
			return src.replace(/^([a-z]+:\/\/[^/]+\/+[^/]+-[0-9a-f]{8,})-\d+(?:[?#].*)?$/, "$1-999999999999999");
		}

		if (domain === "files.slack.com") {
			return src.replace(/\/files-tmb\/+([A-Z0-9]+-[A-Z0-9]+)-[0-9a-f]+\/+([^/]+)_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "/files-pri/$1/$2$3");
		}

		if (domain_nowww === "chathour.com" ||
			domain_nowww === "chathourmobile.com") {
			return src.replace(/\/photos\/+thumb\//, "/photos/full/");
		}

		if (domain === "nimage.globaleconomic.co.kr") {
			newsrc = src.replace(/\/imagesphp\/+[0-9]+\/+([^/]+\.[^/.]*?)(?:[?#].*)?$/, "/phpwas/restmb_allidxmake.php?idx=999&simg=$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "photoresources.wtatennis.com") {
			return src.replace(/\/photo-resources\/+([0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+[-0-9a-f]{20,}\/+[^/]+\.[^/.?]*)(?:[?#].*)?$/, "/wta/photo/$1");
		}

		if (domain_nowww === "brdteengal.com") {
			return src.replace(/\/galleries\/+thumbs\/+((?:[0-9a-f]\/+){5}[0-9a-f]{10,}\/+)[0-9]+x[0-9]+\/+/, "/galleries/$1");
		}

		if (domain_nowww === "destacamos.com") {
			return src.replace(/(\/images\/+listings\/+[0-9]{4}-[0-9]{2}\/+)(?:bigThmb|thmb)\/+/, "$1");
		}

		if (domain === "statici.behindthevoiceactors.com") {
			return {
				url: src.replace(/(\/_img\/+[^/]+\/+)thumbs\/+([^/]+)_thumb(\.[^/.]*)(?:[?#].*)?$/, "$1$2$3"),
				headers: {
					Referer: "https://www.behindthevoiceactors.com/"
				}
			};
		}

		if (domain_nowww === "schokkendnieuws.nl") {
			return src.replace(/\/images\/+multithumb_thumbs\/+b_[0-9]+_[0-9]+_[0-9]+_[0-9]+_images_stories_com_form2content_(p[0-9]+)_(f[0-9]+)_(?:thumbs_)?([^/]*)(?:[?#].*)?$/,
								"/images/stories/com_form2content/$1/$2/$3");
		}

		if (domain_nowww === "biosagenda.nl") {
			return src.replace(/(\/poster\/+[^/]+_[0-9]+)_[0-9]+_[0-9]+_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "primewire.li" ||
			domain === "primewire.unblocked.nu") {
			return src.replace(/\/poster\/+small\/+/, "/poster/medium/");
		}

		if (domain_nowww === "moviemistakes.com") {
			return src.replace(/(\/images\/+titles\/+)[a-z]+\/+/, "$1full/");
		}

		if (domain === "simg.chomikuj.pl") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+[0-9a-f]{20,}\?(?:.*&)?url=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nosub === "ccnxs.cn" && /^img[0-9]*\./.test(domain)) {
			return src.replace(/(\/uploadfile\/+.*\/[^/]*\.[^/.]+)_\/+fillcrop\/+[0-9]+x[0-9]+(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "puschkino.de") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+cms\/+thumbs\.php\?(?:.*&)?src=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return urljoin(src, decodeuri_ifneeded(newsrc), true);
		}

		if (domain === "d3d8y6yhucfd29.cloudfront.net") {
			return src.replace(/(\/1-t[0-9]+-)[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "prothinspo.com") {
			return src.replace(/\/sitebuilder\/+(images\/+[^/]+)-[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "/$1$2");
		}

		if (domain === "assets.yesstud.io") {
			return src.replace(/\/cache\/+([a-z]+)-([0-9]+)(?:-[whqrb][a-z]?[0-9]*){1,}(\.[^/.]*)(?:[?#].*)?$/, "/image/$1_$2$3");
		}

		if (domain === "light.sunphoto.ro") {
			newsrc = src.replace(/\/photos\/+thumb(?:_[a-z]+)?\/+/, "/photos/normal/");
			if (newsrc !== src)
				return newsrc;

			return {
				url: src,
				headers: {
					Referer: "https://sunphoto.ro/"
				},
				referer_ok: {
					same_domain_nosub: true
				}
			};
		}

		if (domain_nowww === "thesimsresource.com") {
			return src.replace(/(\/scaled\/+[0-9]+\/+)[wh]-[0-9]+[hw]-[0-9]+-([0-9]+\.)/, "$1$2");
		}

		if (domain_nowww === "the-numbers.com") {
			return src.replace(/(\/images\/+movies\/+.*)-Thumbnail(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "cinefish.bg") {
			return src.replace(/(\/data\/+movies_images\/+[0-9]+\/+p_[0-9]+)_thumb(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "oneringtrailers.com") {
			return src.replace(/(\/images\/+[a-z]+\/+)thumb_/, "$1");
		}

		if (domain_nowww === "boxofficeturkiye.com") {
			return src.replace(/(\/images\/+haberler\/+)[0-9]+x[0-9]+\/+([^/]+)_c_[0-9]+(\.[^/]*)(?:[?#].*)?$/, "$1buyuk/$2$3");
		}

		if (amazon_container === "trello-avatars") {
			return src.replace(/(\/[0-9a-f]{20,}\/+)[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1original$2");
		}

		if (domain === "s.ebiografia.com") {
			return src.replace(/(\/assets\/+img\/+authors\/+..\/+..\/+[^/]+)-[ml](\.[^/.]*)(?:[?#].*)?$/, "$1-o$2");
		}

		if (domain === "rrdsoa.appspot.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+l\/+([^/.?&]{30,})(?:[?#].*)?$/, "$1");
			if (newsrc !== src)
				return base64_decode(newsrc);
		}

		if (domain_nowww === "pinkfineart.com") {
			return src.replace(/(\/galleries\/+.*\/)pthumbs\/+/, "$1full/");
		}

		if (domain_nosub === "x69.biz") {
			return src.replace(/(:\/\/[^/]+\/+)thumb(\/+fb\/+v\/+[0-9]+\.[^/.]+)(?:[?#].*)?$/, "$1full$2")
		}

		if (domain_nowww === "dudepins.com") {
			return src.replace(/(\/images\/+socialpinboard\/+)thumbnails\/+/, "$1");
		}

		if (domain_nosub === "s3c.es") {
			return src.replace(/(\/imag\/+_v[0-9]+\/+[0-9]+x[0-9]+\/+(?:[0-9]\/+){3})[0-9]+x_/, "$1");
		}

		if (domain_nowww === "playboyrussia.com") {
			return src.replace(/(\/data\/+galleries\/+[0-9]+\/+[0-9]+)s(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "animaldiversity.org") {
			newsrc = src.replace(/(\/collections\/+contributors\/+(?:[^/]+\/+){2,4})(?:small|medium)(\.[^/.]*)(?:[?#].*)?$/, "$1large$2");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/\/collections\/+contributors\/+(?:[^/]+\/+){2,4}(?:small|medium|large)\.[^/.]*(?:[?#].*)?$/);
			if (match)
				return {
					url: src,
					extra: {
						page: src.replace(/\/[^/]+(?:[?#].*)?$/, "/")
					}
				};
		}

		if (domain === "api.creativecommons.engineering") {
			return src.replace(/^[a-z]+:\/\/[^/]+\/+t\/+(https?:)/, "$1");
		}

		if (domain === "capl.washjeff.edu") {
			newsrc = src.replace(/^([a-z]+:\/\/[^/]+\/+[0-9]+\/+)[sm](\/+[0-9]+\.[^/.]*)(?:[?#].*)?$/, "$1l$2");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/^[a-z]+:\/\/[^/]+\/+([0-9]+)\/+([sml])\/+([0-9]+)\.[^/.]*(?:[?#].*)?$/);
			if (match) {
				return {
					url: src,
					extra: {
						page: "http://capl.washjeff.edu/browseresults.php?langID=" + match[1] + "&photoID=" + match[3] + "&size=" + match[2]
					}
				};
			}
		}

		if (domain === "openaccess-cdn.clevelandart.org") {
			newsrc = src.replace(/(\/[0-9.a-z]+)_web(\.[^/.]*)(?:[?#].*)?$/, "$1_print$2");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/^[a-z]+:\/\/[^/]+\/+([0-9.a-z]+)\/+/);
			if (match) {
				return {
					url: src,
					extra: {
						page: "https://www.clevelandart.org/art/" + match[1]
					}
				};
			}
		}

		if (domain_nowww === "emovieposter.com") {
			return src.replace(/(\/images\/+[^/]+\/+[A-Z0-9]+\/+)[0-9]+\/+([^/]+\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "dimu.org") {
			return src.replace(/^([a-z]+:\/\/[^/]+\/+image\/+[^/]+?)(?:[?#].*)?$/, "$1?dimension=max");
		}

		if (domain_nowww === "flora-on.pt") {
			newsrc = src.replace(/(:\/\/[^/]+\/+[^/]+?)(?:_(?:low|ori))?(_[a-zA-Z0-9]+)(\.jpg)(?:down)?(?:[?#].*)?$/, "$1_ori$2$3");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/:\/\/[^/]+\/+[^/]+?(?:_(?:low|ori))?_([a-zA-Z0-9]+)\.jpg(?:down)?(?:[?#].*)?$/);
			if (match) {
				return {
					url: src,
					extra: {
						page: "https://flora-on.pt/#/h" + match[1]
					}
				};
			}
		}

		if (domain_nosub === "geograph.org.uk" && /^s[0-9]*\./.test(domain)) {
			match = src.match(/^[a-z]+:\/\/[^/]+\/+geophotos\/+(?:[0-9]{2}\/+){3}([0-9]+)_([0-9a-f]+)\./);
			if (match) {
				return {
					url: "https://www.geograph.org.uk/reuse.php?id=" + match[1] + "&download=" + match[2],
					extra: {
						page: "https://www.geograph.org.uk/photo/" + match[1]
					}
				};
			}
		}

		if (domain === "collections.museumvictoria.com.au") {
			return src.replace(/(\/content\/+media\/+[^/]+\/+[0-9]+-)(?:thumbnail|medium)(\.[^/.]*)(?:[?#].*)?$/, "$1large$2");
		}

		if (domain_nowww === "phylopic.org") {
			newsrc = src.replace(/(\/assets\/+images\/+submissions\/+[-0-9a-f]{20,})\.[0-9]+\.png(?:[?#].*)?$/, "$1.svg");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/\/assets\/+images\/+submissions\/+([-0-9a-f]{20,})\./);
			if (match) {
				return {
					url: src,
					extra: {
						page: "http://phylopic.org/image/" + match[1]
					}
				};
			}
		}

		if (domain_nowww === "svgsilh.com") {
			newsrc = src.replace(/\/png\/+([0-9]+)-([0-9a-f]+)\.png(?:[?#].*)?$/, "/svg/$1-$2.svg");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/:\/\/[^/]+\/+[a-z]+\/+([0-9]+)-([0-9a-f]+)\.[a-z]+(?:[?#].*)?$/);
			if (match) {
				return {
					url: src,
					extra: {
						page: "https://svgsilh.com/" + match[2] + "/image/" + match[1] + ".html"
					}
				};
			}
		}

		if (domain_nowww === "walldevil.co") {
			return src.replace(/(\/wallpapers\/+w[0-9]+\/+)thumb\/+/, "$1");
		}

		if (domain === "cdni.rt.com") {
			return src.replace(/(\/files\/+[0-9]{4}\.[0-9]{2}\/+)[a-z]+\/+/, "$1original/");
		}

		if (domain === "api.97bike.com" ||
			domain === "pic.qmglive.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+pic\.php\?(?:.*?&)?url=([^&]+).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "hxyy56.com" ||
			domain_nosub === "meijuxia.com" ||
			domain === "zy.itono.cn") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+tu\.php\?(?:.*&)?(?:url|pic)=([^&]+).*?$/, "$1");
			if (newsrc !== src)
				return add_http(decodeuri_ifneeded(newsrc));
		}

		if (domain_nowww === "process-safety-lab.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+imges\.php\?(?:.*&)?src=([^&]+).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "bvla.com") {
			return src.replace(/(\/media\/+inventory\/+product\/+)alt(\/+[0-9]+_[0-9]+\.[^/.]+)(?:[?#].*)?$/, "$1full$2");
		}

		if (domain === "vangogh.teespring.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+static\.jpg\?(?:.*&)?image_url=([^&]+).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);

			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+opengraph\.jpg\?(?:.*&)?composition=([^&]+).*?$/, "$1");
			if (newsrc !== src) {
				try {
					var json = JSON_parse(decodeURIComponent(newsrc));
					return json.artwork.url;
				} catch (e) {
					console_error(e);
					return src;
				}
			}

			return src
				.replace(/(\/v3\/+image\/+[^/]{10,}\/+)[0-9]+\/+[0-9]+(\.[^/.]+)(?:[?#].*)?$/, "$1999999/999999$2")
				.replace(/(\/shirt_pic\/+(?:[0-9]+\/+){4})[0-9]+x[0-9]+\/+([^/]+\.[^/.]+)(?:[?#].*)?$/, "$1999999x999999/$2");
		}

		if (domain_nosub === "lvme.me" && /^[a-z]\./.test(domain)) {
			newsrc = src.replace(/(:\/\/[^/]+\/+[a-z0-9]+)_[0-9]+(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/^[a-z]+:\/\/[^/]+\/+([a-z0-9]+)\.[^/.]+(?:[?#].*)?$/);
			if (match)
				return {
					url: src,
					extra: {
						page: "https://www.livememe.com/" + match[1] + "/"
					}
				};
		}

		if (domain_nowww === "jmusicitalia.com" ||
			domain === "static.jmusicitalia.com") {
			return src.replace(/(\/artisti\/+[^/]+\/+album\/+[^/]+?)(?:-big)?(\.[^/.]+)(?:[?#].*)?$/, "$1-big$2");
		}

		if (domain_nowww === "cooltrack.co.kr") {
			return src.replace(/(\/shopimages\/+cooltrack\/+[0-9]{12})[0-9](\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "rakuten.com" && /^images\./.test(domain)) {
			return src.replace(/(\/photo\/+[0-9]+)_[SML](\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "suruga-ya.jp") {
			return src.replace(/\/pics\/+boxart_[a-z]\/+([0-9]+)[a-z](\.[^/.]*)(?:[?#].*)?$/, "/database/pics/game/$1$2");
		}

		if (domain_nowww === "itsfun.com.tw") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+images\/+(?:[0-9a-f]{2}\/+){2}([^/.]{30,})\.[^/.]+(?:[?#].*)?$/, "$1");
			if (newsrc !== src) {
				return base64_decode(reverse_str(newsrc));
			}
		}

		if (domain_nowww === "hot-sex-photos.com") {
			return src.replace(/\.thumb(\.[^/.]+)(?:[?#].*)?$/, "$1");
		}

		if (domain === "s.io.ua") {
			return {
				url: src.replace(/:\/\/s(\.[^/]+\/+img_aa\/+)small(\/+[0-9]+\/+[0-9]+\/+[0-9]+)_0(\.[^/.]+)(?:[?#].*)?$/, "://g$1large$2$3"),
				headers: {
					Referer: ""
				},
				referer_ok: {
					same_domain_nosub: true
				}
			};
		}

		if (domain_nosub === "io.ua") {
			obj = {
				url: src,
				headers: {
					Referer: ""
				},
				referer_ok: {
					same_domain_nosub: true
				}
			};

			match = src.match(/\/img_aa\/+[a-z]+\/+[0-9]+\/+[0-9]+\/+([0-9]+)(?:_[0-9]+)?\./);
			if (match) {
				obj.extra = { page: "https://io.ua/" + match[1] };
			}

			return obj;
		}

		if (domain_nowww === "sexylabia.net") {
			return {
				url: src.replace(/\/uploads\/+thumbnails\/+FileUpload\/+/, "/uploads/FileUpload/"),
				headers: {
					Referer: ""
				},
				referer_ok: {
					same_domain_nosub: true
				}
			};
		}

		if (domain_nowww === "labsporno.com") {
			return src.replace(/(\/images\/+[0-9]\/+[0-9]+)-t(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "astro.cz") {
			return src.replace(/(\/apod_data\/+[0-9]{4}\/+[0-9]{2}\/+[^/]+)_detail(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "img.poliigon.com") {
			return src.replace(/\/assets\/+small\/+([^/]+\.[^/.]+)(?:[?#].*)?$/, "/assets/large/$1");
		}

		if (domain_nosub === "cgtrader.com" && /^img/.test(domain)) {
			return src.replace(/(\/items\/+[0-9]+\/+[0-9a-f]+\/+)thumb\/+/, "$1");
		}

		if (domain_nowww === "sketchuptextureclub.com") {
			if (/\/imgs\/+q\.png(?:[?#].*)?$/.test(src)) {
				return {
					url: src,
					bad: "mask"
				};
			}

			newsrc = src.replace(/\/public\/+texture_m\/+([0-9]+-)/, "/public/texture/$1");
			if (newsrc !== src) {
				return {
					url: newsrc,
					problems: {
						watermark: true
					}
				};
			}
		}

		if (domain_nowww === "gamingonlinux.com") {
			newsrc = src.replace(/(\/uploads\/+articles\/+article_media\/+)thumbs\/+/, "$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (host_domain_nowww === "gamingonlinux.com" && options.element) {
			if (options.element.tagName === "IMG" && options.element.parentElement && options.element.parentElement.tagName === "DIV") {
				var videoid = options.element.parentElement.getAttribute("data-video-id");
				if (videoid) {
					return "https://i.ytimg.com/vi/" + videoid + "/mqdefault.jpg";
				}
			}
		}

		if (domain === "img.karaoketexty.sk") {
			return src.replace(/(\/img\/+[^/]+\/+[0-9]+\/+(?:[0-9]+\/+)?)thumb\/+thumb-/, "$1");
		}

		if (domain === "img.buymeacoffee.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+api\/+\?(?:.*?&)?url=([^&]{20,}).*?$/, "$1");
			if (newsrc !== src)
				return base64_decode(newsrc);
		}

		if (domain === "thumbs.gfycat.com" ||
			domain === "zippy.gfycat.com" ||
			domain === "giant.gfycat.com") {
			newsrc = src
				.replace(/:\/\/zippy\.([^/]+\/+[^/]+)(?:-[^/.]+)?\.[^/.]+(?:[?#].*)?$/, "://giant.$1.mp4")
				.replace(/:\/\/thumbs\.([^/]+\/+[^/]+)-(?:size_restricted\.gif|mobile\.(?:webm|mp4)|(?:mobile|poster)\.jpg)(?:[?#].*)?$/, "://zippy.$1.mp4");
			obj = {
				url: newsrc
			};

			if (newsrc !== src) {
				obj.video = true;
			}

			match = src.match(/^[a-z]+:\/\/[^/]+\/+([^-./]+)[-.]/);
			if (match) {
				obj.extra = {page: "https://gfycat.com/" + match[1]};
			}

			return obj;
		}

		if (amazon_container === "blurbprod1000") {
			return src.replace(/(\/[0-9a-f]{20,}\/+[-0-9a-f]{20,}\/+)[A-Z](\.[^/.]*)(?:[?#].*)?$/, "$1O$2");
		}

		if (domain === "bookshow.blurb.com") {
			return src.replace(/(\/bookshow\/+cache\/+[^/]+\/+)lw\/+/, "$1md/");
		}

		if (domain === "contents.sixshop.com") {
			return src.replace(/\/thumbnails\/+(uploadedFiles\/+[0-9]+\/+[^/]+\/+[^/]+)_[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "/$1$2");
		}

		if (domain_nosub === "gumroad.com" && /^static(?:-[0-9]+)\./.test(domain)) {
			return src.replace(/(\/res\/+gumroad\/+[0-9]+\/+asset_previews\/+[0-9a-f]{10,}\/+)[a-z]+\/+/, "$1original/");
		}

		if (domain === "cdn.sharesome.com" ||
			domain === "cache.sharesome.com") {
			regex = /(\/uploads\/+user-images\/+u[0-9]+\/+[^-/.]+-[^-/.]+)(?:-[smlp]|-(?:lg|xl))?(\.[^/.]+)(?:[?#].*)?$/;

			if (regex.test(src)) {
				return [
					src.replace(regex, "$1-o$2"),
					src.replace(regex, "$1-lg$2")
				];
			}
		}

		if (domain === "media.sharesome.com") {
			newsrc = src.replace(/\/thumb-((?:[0-9]+-[0-9]+-[0-9]+|[^-/.]+)\.(?:mp4|webm))\.[^/.]+(?:[?#].*)?$/, "/$1");
			if (newsrc !== src)
				return {
					url: newsrc,
					video: true
				};
		}

		if (domain_nowww === "nsfwalbum.com" && /\/pic\/+loading\.gif(?:[?#].*)?$/.test(src)) {
			return {
				url: src,
				bad: true
			};
		}

		if (domain_nowww === "luciecline.com") {
			return src.replace(/(\/cdn-content\/+[0-9]+\/+[^/]+\/+[0-9]+\/+)thumbs\/+img/, "$1img");
		}

		if (domain_nosub === "wankerson.com" && /^(?:thumbs|pics)\./.test(domain)) {
			newsrc = src.replace(/:\/\/thumbs\./, "://pics.");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/^[a-z]+:\/\/[^/]+\/+(free-porn\/+.+-[0-9a-f]{10,}-[0-9]+\.)/);
			if (match) {
				return {
					url: src,
					extra: {
						page: "http://www.wankerson.com/porn/" + match[1] + "html"
					}
				};
			}
		}

		if (domain_nowww === "meituyuan.com") {
			return src.replace(/(\/data\/+share\/+[0-9]{4}\/+(?:[0-9]{2}\/+){2}[0-9a-f]{10,})_[smb](\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "dto9r5vaiz7bu.cloudfront.net") {
			return src.replace(/^([a-z]+:\/\/[^/]+\/+[0-9a-z]{10,}\/+)[_a-z]+(\.[^/.]+)(?:[?#].*)?$/, "$1source$2");
		}

		if (domain_nowww === "robertsspaceindustries.com") {
			if (/^[a-z]+:\/\/[^/]+\/+rsi\/+static\/+images\/+/.test(src)) {
				return {
					url: src,
					bad: "mask"
				};
			}

			return src.replace(/(\/media\/+[0-9a-z]{10,}\/+)[_a-z]+\/+/, "$1source/");
		}

		if (domain === "d111vui60acwyt.cloudfront.net") {
			newsrc = src.replace(/(\/stores\/+avatars\/+[0-9]+\/+)[a-z]+\/+/, "$1original/");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nosub === "fireworktv.com" && /^cdn[0-9]*\./.test(domain)) {
			return src.replace(/(\/medias\/+[0-9]{4}\/+(?:[0-9]{1,2}\/+){2}[^/]+\/+)[0-9]+_[0-9]+\/+/, "$1original/");
		}

		if (domain_nosub === "highwebmedia.com" && /^(?:ssl-)?ccstatic\./.test(domain)) {
			if (/\/tsdefaultassets\/+locked_rectangle[0-9]*\./.test(src))
				return {
					url: src,
					bad: "mask"
				};
		}

		if (domain_nowww === "extension.zone") {
			return src.replace(/(\/wp-content\/+uploads\/+.*)-[0-9]+x[0-9]+@[0-9]+x(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "infcdn.net" && /^art/.test(domain)) {
			return src.replace(/(\/articles_uploads\/+[0-9]\/+[0-9]+\/+)thumb\/+([^/.]+)-[0-9]*x[0-9]*(\.[^/.]+)(?:[?#].*)?$/, "$1$2$3");
		}

		if (domain === "d2bq4ntp986cty.cloudfront.net") {
			return src.replace(/(\/[0-9]{4}\/+[0-9]+\/+[^/]+\/+[^/]+\.(?:im|fb))\.(?:lg|ms)(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "freehostedpics.com") {
			return src.replace(/\/thumbs\/+tnpics([0-9]+\.)/, "/pics$1");
		}

		if (domain_nowww === "erotopper.nl") {
			newsrc = src.replace(/(\/images\/+[0-9]{4}\/+[0-9]+\/+)th-/, "$1");

			match = src.match(/:\/\/[^/]+\/+([^/]+)\/+images\/+[0-9]{4}\/+([0-9]+)\/+[^/]+\.[^/.]+(?:[?#].*)?$/);
			if (match) {
				return {
					url: newsrc,
					extra: {
						page: "http://erotopper.nl/" + match[1] + "/gallery.php?id=" + match[2]
					}
				};
			} else {
				return newsrc;
			}
		}

		if (domain_nosub === "litmind.com") {
			return src
				.replace(/(\/data\/+photos\/+(?:[0-9]\/+){3}[0-9]+)\.(?:thumbnail|profile(_non_retina)?|big|list_premium)\./, "$1.huge.")
				.replace(/(\/data\/+logos\/+(?:[0-9]\/+){3}[0-9]+)\.profile_non_retina\./, "$1.profile.");
		}

		if (domain_nowww === "kabe-uchiroom.com") {
			return src
				.replace(/\/upfile_image\/+([0-9]+)([0-9])(\/+[^/?]+\.[^/.?]+)$/, "/accounts/upfile/$2/$1$2/$3")
				.replace(/(\/upfile_image\/.*?)(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "mangadex.cc") {
			newsrc = src.replace(/(\/images\/+manga\/+[0-9]+)\.thumb(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
			if (newsrc !== src)
				return add_extensions_jpeg(newsrc);

			match = src.match(/\/images\/+manga\/+([0-9]+)\./);
			if (match) {
				return {
					url: src,
					extra: {
						page: "https://mangadex.cc/title/" + match[1] + "/"
					}
				};
			}
		}

		if (domain_nowww === "ipetgroup.com") {
			return src.replace(/(\/photo\/+[0-9]+_[0-9]+)_[0-9]+(\.[^/.]+)(?:[?#].*)?$/, "$1_99999999$2");
		}

		if (domain === "cdn.aarp.net") {
			return src.replace(/\.imgcache\.rev[0-9a-f]{20,}\.web(?:\.[0-9]+\.[0-9]+)?(\.[^/.]+)(?:[?#].*)?$/, "$1");
		}

		if (domain === "img.particlenews.com") {
			if (/\/image\.php\?/.test(src))
				return remove_queries(src, ["type"]);
		}

		if (domain_nowww === "joy.pl") {
			return src.replace(/\/u\/+ic\/+T[0-9]+\/+u\/+/, "/u/");
		}

		if (domain_nowww === "dynasty-scans.com") {
			if (/\/assets\/+(?:prev|next|clear)-[0-9a-f]{20,}\.(?:png|gif)/.test(src))
				return {
					url: src,
					bad: "mask"
				};
		}

		if (domain === "reader.kireicake.com" ||
			domain === "kobato.hologfx.com" ||
			domain_nowww === "sensescans.com" ||
			(domain_nosub === "world-three.org" && string_indexof(domain, "slide.") >= 0) ||
			domain_nowww === "jaiminisbox.com") {
			return src.replace(/(\/content\/+.*_[0-9a-f]{10,}\/)thumb_([^/]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "piczel.tv") {
			newsrc = src.replace(/(\/static\/+uploads\/+gallery_image\/+[0-9]+\/+image\/+[0-9]+\/+)thumb_/, "$1");

			obj = {
				url: newsrc
			};

			match = src.match(/\/gallery_image\/+[0-9]+\/image\/+([0-9]+)\/+[^/]+(?:[?#].*)?$/);
			if (match) {
				obj.extra = {page: "https://piczel.tv/gallery/image/" + match[1]};
			}

			return obj;
		}

		if (domain === "cdn.9hentai.com") {
			newsrc = src
				.replace(/(\/images\/+[0-9]+\/+)cover-small\./, "$1cover.")
				.replace(/(\/images\/+[0-9]+\/+)preview\/+([0-9]+)t(\.[^/.]+)(?:[?#].*)?$/, "$1$2$3");

			obj = {
				url: newsrc
			};

			match = src.match(/\/images\/+([0-9]+)\/+(?:cover|(?:preview\/+)?([0-9]+)t?\.)/);
			if (match) {
				obj.extra = {page: "https://9hentai.com/g/" + match[1] + "/" + (match[2] ? match[2] + "/" : "")};
			}

			return obj;
		}

		if (domain_nowww === "hentaishark.com") {
			newsrc = src.replace(/(\/uploads\/+manga\/+[0-9]+\/+chapters\/+[0-9]+\/+)thumb_/, "$1");

			obj = {
				url: newsrc
			};

			match = src.match(/\/uploads\/+(manga)\/+([0-9]+)\/+chapters\/+([0-9]+)\/+(?:thumb_)?([0-9]+)\./);
			if (match) {
				var type = match[1];
				var typeid = match[2];
				var chapter = match[3];
				var page = parseInt(match[4]);

				obj.extra = {page: "https://www.hentaishark.com/" + type + "/" + typeid + "/" + chapter + "/" + page};
			}

			return obj;
		}

		if (domain === "i.hentaihand.com") {
			return src.replace(/(\/images\/+)([0-9]+)t(\.[^/.]+)(?:[?#].*)?$/, "$1full/$2$3");
		}

		if (domain_nosub === "sh-cdn.com") {
			return src.replace(/(\/images\/+(?:[0-9a-f]\/+){4}[0-9]+\/+)(?:giant_)?thumb_([0-9a-f]+\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "static.showit.co") {
			return src.replace(/(:\/\/[^/]+\/+)[0-9]+(\/+[^/]+\/+[0-9]+\/+)/, "$199999999$2");
		}

		if (domain === "d33veqcui7lu1w.cloudfront.net") {
			regex = /(\/files\/+[^/]+\/+)[a-z_]+\/+/;
			if (regex.test(src)) {
				return [
					src.replace(regex, "$1original/"),
					src.replace(regex, "$1large/")
				];
			}
		}

		if (domain_nosub === "ymcart.com" && string_indexof(domain, "imgcdn.") >= 0) {
			return src.replace(/\/([0-9]+)\/+thumb\/+[0-9]+x[0-9]+\/+/, "/$1/");
		}

		if (domain === "cdn.neow.in") {
			return src.replace(/(\/images\/+uploaded\/+[0-9]{4}\/+[0-9]{2}\/+[^/]+)_(?:story|small)(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "gamereactor.no") {
			return src.replace(/(\/media\/+[0-9]+\/+[^/]*_[0-9]+)(?:t|_[0-9]+x[0-9]+)(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "fullyimportant.ru") {
			return src.replace(/\/thumbs\/+([0-9]+)_([0-9]+)(\.[^/.]+)(?:[?#].*)?$/, "/cdn/$2/$1$3");
		}

		if (domain === "az879543.vo.msecnd.net") {


			newsrc = src.replace(/(:\/\/[^/]+\/+[a-z]+\/+[^/]+)_s(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/(:\/\/[^/]+\/+[a-z]+\/+)(?:%ED%81%AC%EA%B8%B0%EB%B3%80%ED%99%98_|크기변환_)?([^/]+)_(?:\(1\))?(\.[^/.]+)(?:[?#].*)?$/, "$1$2$3");
			if (newsrc !== src)
				return add_extensions_upper(newsrc);
		}

		if (domain === "static-buyma-com.akamaized.net") {
			return src.replace(/(\/imgdata\/+item\/+[0-9]+\/+[0-9]+\/+[0-9]+\/+)[0-9]+(\.[^/.]+)(?:[?#].*)?$/, "$1org$2");
		}

		if (domain === "images.spoilertv.com") {
			return src.replace(/(\/cache\/+.*\/[^/]+)_[0-9]+_cw[0-9]+_ch[0-9]+_thumb\./, "$1_FULL.");
		}

		if (domain_nosub === "itc.cn" && string_indexof(domain, ".biz.") >= 0) {
			return src.replace(/(\/pic\/+[^/]+\/+)[sn](\/+(?:[0-9]{2}\/+){2}Img[0-9]+)_[sn](\.[^/.]+)(?:[?#].*)?$/, "$1f$2_f$3");
		}

		if (domain === "photo.isportskorea.com") {
			newsrc = src.replace(/(\/photo\/+)thumbnail(\/+[0-9]{4}\/+[0-9]{2}\/+[0-9]+\.[^/.]+)(?:[?#].*)?$/, "$1images$2");
			obj = {
				url: newsrc
			};

			match = src.match(/\/photo\/+[^/]+\/+[0-9]{4}\/+[0-9]{2}\/+([0-9]+)\./);
			if (match) {
				obj.extra = {page: "http://www.isportskorea.com/mstory/?mode=view&no=" + match[1]};
			}

			return obj;
		}

		if (domain === "edc2.healthtap.com") {
			return src
				.replace(/(\/user_answer\/+[^/]+\/+[0-9]+\/+)topic_large(\/+[^/]+)\.jpeg(?:[?#].*)?$/, "$1original$2.jpg")
				.replace(/(\/user_answer\/+[^/]+\/+[0-9]+\/+)(?:(?:topic|xx?)_)?(?:medium|normal|large)(\/+[^/]+\.[^/.]+)(?:[?#].*)?$/, "$1topic_large$2");
		}

		if (domain_nowww === "bomz.org") {
			return src.replace(/(\/i\/+[^/]+\/+)thumb_([^/]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "gtainside.com") {
			if (/\/includes\/+lightbox2\/+img\//.test(src))
				return {
					url: src,
					bad: "mask"
				};

			return src.replace(/(\/downloads\/+picr\/+[0-9]{4}-[0-9]{2}\/+)thb_/, "$1");
		}

		if (domain_nowww === "gta.cz") {
			if (/\/download\/+image\.php\?/.test(src)) {
				return remove_queries(src, ["thumb"]);
			}
		}

		if (domain_nowww === "feralinteractive.com") {
			if (/\/images\/+games\/+bottom-frame\./.test(src))
				return {
					url: src,
					bad: "mask"
				};

			return src.replace(/(\/screenshots\/+)thumbs\/+thumb([0-9]+\.[^/.]+)(?:[?#].*)?$/, "$1screen$2");
		}

		if (domain_nosub === "nsimg.net") {
			return src.replace(/(\/biopic\/+)(?:160x120|320x240|200x150|240x180)\/+/, "$1original4x3/");
		}

		if (domain === "photo.naiadmmm.com") {
			return src.replace(/(\/fast_photo\.php\?)(.*&)?type=thumbnail/, "$1$2type=photo");
		}

		if (domain === "prodimage.images-bn.com") {
			return {
				url: src,
				headers: {
					Referer: ""
				},
				can_head: false // doesn't return errors
			}
		}

		if (domain === "dn3pm25xmtlyu.cloudfront.net") {
			match = src.match(/\/photos\/+[a-z]+\/+([0-9]+)\./);
			if (match) {
				var get_twitpic_page_from_id = function(id) {
					return "https://twitpic.com/" + parseInt(id).toString(36);
				};

				id = match[1];

				obj = {
					url: src,
					extra: { page: get_twitpic_page_from_id(id) },
				};

				var query_twitpic = function(id, cb) {
					var cache_key = "twitpic:" + id;

					api_cache.fetch(cache_key, cb, function(done) {
						options.do_request({
							url: get_twitpic_page_from_id(id),
							method: "GET",
							onload: function(result) {
								if (result.readyState !== 4)
									return;

								if (result.status !== 200) {
									console_error(e);
									return done(null, false);
								}

								var match = result.responseText.match(/<meta\s+name="twitter:image"\s+value="(https:\/\/dn3pm25xmtlyu\.[^"']+)"\s*\/>/);
								if (!match) {
									console_error("Unable to find match", result);
									return done(null, false);
								}

								return done(decode_entities(match[1]), 60*60);
							}
						});
					});
				};

				if (options && options.cb && options.do_request) {
					query_twitpic(id, function(url) {
						if (url) {
							obj.url = url;
						}

						return options.cb(obj);
					});

					return {
						waiting: true
					};
				}

				return obj;
			}
		}

		if (domain === "media-ima002.globaltalentsystems.com") {
			return src.replace(/^[a-z]+:\/\/([^/]+)\/+/, "https://s3.amazonaws.com/$1/");
		}

		if (amazon_container === "media-ima002.globaltalentsystems.com") {
			return [
				src.replace(/.*\/media-ima002\.[^/]+\/+([0-9]+)\/+[0-9]+\/+([0-9]+_[^/]+\.[^/.]+)(?:[?#].*)?$/, "https://www.globaltalentsystems.com/main/$1/$2"),
				src.replace(/\/media-ima002\.([^/]+)\/+([0-9]+)\/+[0-9]+\/+([0-9]+_[^/]+\.[^/.]+)(?:[?#].*)?$/, "/media-joined000.$1/2/$2/$3")
			];
		}

		if (domain_nosub === "rbk.ru" && /^s[0-9]*\./.test(domain)) {
			return src.replace(/\/resized\/+[0-9W]+x[0-9H]+(?:_crop)?\/+media\/+/, "/media/");
		}

		if (domain_nowww === "kazanfirst.ru") {
			return src.replace(/(\/storage\/+post\/+[^/]+\/+[^/]+)-watermark(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "vgtour.ro") {
			newsrc = src.replace(/(\/media\/+poze\/+[^/]+\/+[^/]+\/+)thumbs\/+/, "$1");
			if (newsrc !== src)
				return newsrc;

			if (/\/watermark\.php\?/.test(src)) {
				var source = url.searchParams.get("source");
				id = url.searchParams.get("id");
				var nume = url.searchParams.get("nume");

				return src.replace(/\/watermark\.php.*/, "/media/poze/" + source + "/" + id + "/" + nume);
			}
		}

		if (domain_nowww === "maiolifruttiantichi.it") {
			return src.replace(/(\/e-commerce\/+prodotti\/+img\/+)watermark\/+/, "$1");
		}

		if (domain === "photo.jpnn.com") {
			return src.replace(/\/arsip\/+watermark\/+/, "/arsip/normal/");
		}

		if (domain_nowww === "posterswholesale.com") {
			return src.replace(/\/resize\/+(Shared\/+.*?)(?:[?#].*)?$/i, "/$1");
		}

		if (domain === "images.summitmedia-digital.com") {
			return src.replace(/\/resize\/+[^/]+\/+(images\/+)/, "/$1");
		}

		if ((domain_nosub === "hiclipart.com" ||
			 domain_nosub === "pngwave.com" ||
			 domain_nosub === "pngguru.com" ||
			 domain_nosub === "pngfuel.com")
			&& /^[a-z][0-9]*\./.test(domain)) {
			return src.replace(/((?:\/preview\/+(?:[0-9]+\/+){3}|\/png\/+(?:[0-9]+\/+){2,3})[^/]+)-thumbnail(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "aerisdies.com") {
			newsrc = src.replace(/\/thumbnails(\/+categories\/+[0-9]+\/+[0-9]+\.)gif(?:[?#].*)?$/, "/images$1png");
			if (newsrc !== src)
				return add_extensions_gif(newsrc);
		}

		if (domain_nowww === "erofus.com") {
			return src.replace(/\/thumb(\/+[0-9]+\/+[0-9a-f]+\.)/, "/medium$1");
		}

		if (domain_nosub === "hentaicdn.com") {
			return src
				.replace(/\/thumbnails\/+(hcdn[0-9]+)tmb(\.[^/.]+)(?:[?#].*)?$/, "/$1$2")
				.replace(/\/cover\/+[0-9]+\/+_S/, "/cover/_S");
		}

		if (domain_nowww === "hentaidude.com") {
			if (/\/wp-content\/+themes\/+awp\/+images\//.test(src))
				return {
					url: src,
					bad: "mask"
				};
		}

		if (domain_nowww === "theonemilano.com") {
			if (/\/wp-content\/+themes\/+[^/]+\/+images\/+/.test(src))
				return {
					url: src,
					bad: "mask"
				};
		}

		if (domain_nowww === "jabarchives.com") {
			obj = {
				url: src
			};

			match = src.match(/\/main\/+media\/+.*\/([A-Z0-9]{10,})1(?:_[a-z]+)\./);
			if (match)
				obj.extra = {page: "https://jabarchives.com/main/post/" + match[1]};

			newsrc = src.replace(/(\/main\/+media\/+.*)_(?:thumb|preview)\./, "$1_large.");
			if (newsrc !== src)
				obj.url = newsrc;

			return obj;
		}

		if (domain === "images.myhentaigrid.com") {
			return src.replace(/(\/imagesgallery\/+images\/+[^/]+\/+)thumbnail\/+/, "$1original/");
		}

		if (domain === "content.tsumino.com") {
			match = src.match(/\/thumbs\/+([0-9]+)\/+([0-9]+)(?:[?#].*)?$/);
			if (match && options && options.cb && options.do_request) {
				page = "https://www.tsumino.com/Read/Index/" + match[1] + "?page=" + match[2];

				var cache_key = "tsumino:" + match[1] + ":" + match[2];
				var book_page = match[2];
				api_cache.fetch(cache_key, options.cb, function(done) {
					options.do_request({
						url: page,
						method: "GET",
						onload: function(resp) {
							if (resp.readyState !== 4)
								return;

							if (resp.status !== 200) {
								return done(null, false);
							}

							var match = resp.responseText.match(/<div[^>]+data-cdn=["'](https:\/\/content\.tsumino[^'"]+\/\[PAGE\][^'"]+)["']/);
							if (!match) {
								console_warn("Unable to find match for", resp);
								return done(null, false);
							}

							options.cb({
								url: decode_entities(match[1]).replace("[PAGE]", book_page),
								headers: {
									Referer: resp.finalUrl
								},
								extra: {
									page: resp.finalUrl
								}
							}, 6*60*60);
						}
					});
				});

				return {
					waiting: true
				};
			}
		}

		if (domain_nowww === "2dgalleries.com") {
			return src.replace(/\/planches\/+[0-9]+[WH]\/+/, "/planches/");
		}

		if (domain === "img.tamindir.com") {
			return src.replace(/\/resize\/+[0-9]+x[0-9]+\/+([0-9]{4}\/+)/, "/$1");
		}

		if (domain === "data.yiff.party") {
			newsrc = src.replace(/(\/avatars\/+[^/]+\/+)thumb\/+/, "$1full/");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/\/patreon_inline\/+thumb\/+/, "/patreon_inline/");
			if (newsrc !== src)
				return add_extensions_gif(newsrc);

			if (/\/patreon_data\/+[0-9]+\/+[0-9]+\/+thumb\./.test(src) && options && options.element) {
				var current = options.element;
				if (current.classList.contains("lazyloaded")) {
					var link = current.parentElement.parentElement.querySelector(".card-action > a");
					if (link && string_indexof(link.href, "/patreon_data/") >= 0) {
						return link.href;
					}
				}
			}
		}

		if (domain_nosub === "dlivecdn.com" && /^images-sihv[0-9]*\./.test(domain)) {
			return src.replace(/:\/\/images-[^/.]+\.prd\.dlivecdn\.com\/+fit-in\/+[0-9]+x[0-9]+\/+filters:[^/]+\/+([-a-z]+\/+[^/]+)(?:[?#].*)?$/, "://images.prd.dlivecdn.com/$1");
		}

		if (domain_nosub === "bitchute.com" && /^static/.test(domain)) {
			newsrc = src.replace(/(\/live\/+channel_images\/+.*)_(?:small|medium)(\.[^/.]+)(?:[?#].*)?$/, "$1_large$2");
			if (newsrc !== src)
				return newsrc;

			regex = /(\/live\/+cover_images\/+.*_)([0-9]+x[0-9]+)(\.[^/.]+)(?:[?#].*)?$/;
			match = src.match(regex);
			if (match) {
				var baseobj = {url:src};
				var idmatch = src.match(/\/live\/+cover_images\/+[^/]+\/+([^/_]{5,15})_/);
				if (idmatch) {
					baseobj.extra = {page: "https://www.bitchute.com/video/" + idmatch[1] + "/"};
				}

				var sizes = [
					"1280x720",
					"640x360",
					"320x180"
				];

				var sizetoint = function(size) {
					var match = size.match(/^([0-9]+)x([0-9]+)$/);
					return parseInt(match[1]) * parseInt(match[2]);
				};

				var urlsize = sizetoint(match[2]);
				var newurls = [];
				for (var i = 0; i < sizes.length; i++) {
					if (urlsize < sizetoint(sizes[i])) {
						var obj = deepcopy(baseobj);
						obj.url = src.replace(regex, "$1" + sizes[i] + "$3");

						newurls.push(obj);
					}
				}

				if (newurls.length > 0) {
					return newurls;
				}

				var videourl = src.replace(/^[a-z]+:\/\/static-([0-9])\.bitchute\.com\/+live\/+cover_images\/+([^/]+\/+[^/_]{5,15})_[0-9]+x[0-9]+\..*/, "https://seed$100.bitchute.com/$2.mp4");
				if (videourl !== src) {
					baseobj.url = videourl;
					return baseobj;
				}
			}
		}

		if (domain_nosub === "bitchute.com" && /^seed[0-9]*\./.test(domain)) {
			match = src.match(/^[a-z]+:\/\/[^/]+\/+[^/]+\/+([^/.]+)\./);
			if (match) {
				return {
					url: src,
					extra: {page: "https://www.bitchute.com/video/" + match[1] + "/"}
				};
			}
		}

		if (domain_nowww === "bitchute.com") {
			if (/\/static\/+v[0-9]+\/+images\/+/.test(src))
				return {
					url: src,
					bad: "mask"
				};
		}

		if (domain_nowww === "platinum-celebs.com") {
			return src.replace(/(\/news_pic\/+[0-9]+\/+)tn_/, "$1big_");
		}

		if (domain === "video.like.video") {
			newsrc = src.replace(/(\/[0-9A-Za-z]+)_[0-9]+(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "videosnap.like.video") {
			newsrc = src.replace(/(\/[0-9A-Za-z]+)_[12](\.[^/.]+)(?:[?#].*)?$/, "$1_4$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (host_domain_nowww === "likee.com" && options && options.element && options.do_request && options.cb && !options.exclude_videos) {
			if (options.element.tagName === "A" && /:\/\/[^/]+\/+s\/+[0-9A-Za-z]+$/.test(options.element.href)) {
				var cache_key = "likee:" + options.element.href.replace(/.*\/([^/]+)$/, "$1");
				api_cache.fetch(cache_key, options.cb, function(done) {
					options.do_request({
						url: options.element.href,
						method: "GET",
						onload: function(resp) {
							if (resp.readyState < 4)
								return;

							if (resp.status !== 200) {
								return done(null, false);
							}

							var match = resp.responseText.match(/<script>\s*window\.data\s*=\s*({.*?})\s*;\s*<\/script/);
							if (!match) {
								console_error("Unable to find match", resp);
								return done(null, false);
							}

							try {
								var json = JSON_parse(match[1]);
								return done(json.video_url, 12*60*60);
							} catch (e) {
								console_error(e);
								return done(null, false);
							}
						}
					});
				});

				return {
					waiting: true
				};
			}
		}

		if (domain_nowww === "roleplay.me") {
			return src.replace(/(\/photos\/+(?:[0-9]+\/+){2})[ms]_([0-9]+\.)/, "$1$2");
		}

		if (domain === "cdn.21buttons.com") {
			return src
				.replace(/(\/users\/+[0-9a-f]{20,})\.(?:small|medium|large)\./, "$1.")
				.replace(/\/posts\/+[0-9]+x[0-9]+\/+/, "/posts/");
		}

		if (domain === "cwh-cdn.21buttons.com") {
			return src.replace(/(\/[0-9a-f]{20,}\.)(?:small|medium)\./, "$1smedium.");
		}

		if (domain_nosub === "porngo.com" && /^img[0-9]*\./.test(domain)) {
			return src.replace(/\/(?:small|medium)(?:@2x)?\/+([0-9]+\.[^/.]+)(?:[?#].*)?$/, "/medium@2x/$1");
		}

		if (domain === "assets-jpcust.jwpsrv.com") {
			return src.replace(/(\/thumbnails\/+[^-./]+)-[0-9]+(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "keep2share.cc" && /^prx-[0-9]+\./.test(domain)) {
			match = src.match(/\/video\/+[0-9a-f]+\/+[0-9a-f]+\/+alternative_resolution_[0-9a-f]+_.*\?(?:.*&)?uf=([0-9a-f]+)(?:&.*)?$/);
			if (match && options && options.do_request && options.cb) {
				var fileid = match[1];
				var cache_key = "keep2share:" + fileid;
				api_cache.fetch(cache_key, options.cb, function(done) {
					options.do_request({
						url: "https://api.k2s.cc/v1/files/" + fileid + "?referer=",
						method: "GET",
						headers: {
							Referer: "https://k2s.cc/file/" + fileid + "/",
							Origin: "https://k2s.cc",
							"Accept": "application/json, text/plain, */*",
							"Sec-Fetch-Site": "same-site"
						},
						onload: function(result) {
							if (result.readyState < 4)
								return;

							if (result.status !== 200) {
								console_error(result);
								return done(null, false);
							}

							try {
								var json = JSON_parse(result.responseText);
								return done({
									url: json.videoPreview.video,
									extra: {
										page: "https://k2s.cc/file/" + fileid + "/" + json.name,
									},
									video: true,
									filename: json.name
								}, 3*60*60);
							} catch (e) {
								console_error(e);
								return done(null, false);
							}
						}
					});
				});

				return {
					waiting: true
				};
			}
		}

		if (domain_nowww === "notion.so") {
			if (/^[a-z]+:\/\/[^/]+\/+image\/+https?.*\?/) {
				newsrc = remove_queries(src, ["width", "height"]);
				if (newsrc !== src)
					return newsrc;
			}

			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+image\/+(https?%3.*?)(?:[?#].*)?$/, "$1");
			if (false && newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "cdn.futura-sciences.com") {
			return src.replace(/\/buildsv6\/+images\/+(?:(?:small|medium|midi)original|wide(?:640|1920|1440)|smallsquare|smallrectangle)\/+/, "/buildsv6/images/largeoriginal/");
		}

		if (domain === "6d4be195623157e28848-7697ece4918e0a73861de0eb37d08968.ssl.cf1.rackcdn.com") {
			return src.replace(/(:\/\/[^/]+\/+[0-9]+)_[0-9]+[wh](\.[^/.]+)(?:[?#].*)?$/, "$1_original$2");
		}

		if (domain_nosub === "editmysite.com" && /^cdn[0-9]*\./.test(domain)) {
			if (/:\/\/[^/]+\/+images\/+blank\.gif(?:[?#].*)?$/.test(src)) {
				return {
					url: src,
					bad: "mask"
				};
			}
		}

		if (domain === "d1rw89lz12ur5s.cloudfront.net") {
			return src.replace(/(\/photo\/+[^/]+\/+file\/+[0-9a-f]{20,}\/+)(?:thumb|medium|large)\/+/, "$1");
		}

		if (domain_nowww === "howtogeek.com") {
			return src.replace(/\/thumbcache\/+[0-9]+\/+[0-9]+\/+[0-9a-f]{20,}\/+/, "/");
		}

		if (domain === "cdnu.porndoe.com") {
			return src.replace(/(\/movie\/+(?:[0-9]\/+){5,}[^/]+-)(?:preview|240p-400|480p-1000)\./, "$1720p-hd-2500.");
		}

		if (domain === "images.newsnowgr.com" ||
			domain === "images.newsnowgreece.com") {
			newsrc = src.replace(/:\/\/[^/]+\/+([0-9]{3}\/+[0-9]+\/+)[^/]+-([0-9]+)-[0-9]+x[0-9]+(\.[^/.]+)(?:[?#].*)?$/, "://images.newsnowgreece.com/$1$2$3");
			obj = {
				url: newsrc
			};

			match = src.match(/:\/\/[^/]+\/+[0-9]{3}\/+([0-9]+)\/+(?:[^/]+-)?([0-9]+)(?:-[0-9]+x[0-9]+)?\.[^/.]+(?:[?#].*)?$/);
			if (match) {
				obj.extra = {page: "http://www.newsnowgr.com/photo/" + match[1] + "/" + match[2] + "/"};
			}

			return obj;
		}

		if (domain_nowww === "wallpapers.net") {
			newsrc = src.replace(/(\/web\/+wallpapers\/+[^/]+\/+thumbnail\/+)(?:xs|sm|md)\./, "$1lg.");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/(:\/\/[^/]+\/+)([^/]+\/+)download\/+/, "$1web/wallpapers/$2");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/\/web\/+wallpapers\/+([^/]+)\/+(?:[0-9]+x[0-9]+|thumbnail)/);
			if (match && options && options.do_request && options.cb) {
				var page = "https://wallpapers.net/" + match[1];

				cache_key = "wallpapers.net:" + page;
				api_cache.fetch(cache_key, options.cb, function(done) {
					options.do_request({
						url: page,
						method: "GET",
						headers: {
							"Accept-Language": "en;q=0.9"
						},
						onload: function(resp) {
							if (resp.readyState < 4)
								return;

							if (resp.status !== 200) {
								return done(null, false);
							}

							var match = resp.responseText.match(/<p class="small">Resolution:\s*<a href="\/[^"]+"\s*>([0-9]+x[0-9]+)<\/a>/);
							if (!match) {
								console_error("Unable to find match", resp);
								return done(null, false);
							}

							return done({
								url: src.replace(/(\/web\/+wallpapers\/+[^/]+\/+).*?(\.[^/.]+)(?:[?#].*)?$/, "$1" + match[1] + "$2"),
								extra: {
									page: resp.finalUrl
								}
							}, 12*60*60);
						}
					});
				});

				return {
					waiting: true
				};
			}
		}

		if (domain_nowww === "youngjump.jp") {
			return src.replace(/(\/images\/+)tn([0-9]+\.[^/.]+)(?:[?#].*)?$/, "$1photo$2");
		}

		if (domain === "forums.auscelebs.net") {
			return src.replace(/(\/[0-9]+\/+)thumbs\/+([^/]+\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "dyncdn2.com") {
			return src.replace(/(\/mimages\/+[0-9]+\/+)over_opt\./, "$1poster_opt.");
		}

		if (domain_nosub === "1z2x1z.com") {
			if (/\/wp-content\/+plugins\/+use-your-drive\/+css\/+images\//.test(src))
				return {
					url: src,
					bad: "mask"
				};
		}

		if (domain_nosub === "vercity.ru") {
			return src.replace(/(\/gallery\/+img\/+.*\/)[0-9]+x[0-9]*\/+([^/]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "auto.sk") {
			return src.replace(/(\/static\/+gallery\/+[0-9]{4}\/+[0-9]{2}\/+[0-9a-f]{5,}\/+[0-9a-f]{5,}\/+[0-9]+)-thumb(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "imgs.firmenwagen.co.at") {
			return src.replace(/(:\/\/[^/]+\/+m\/+[0-9]+_[0-9]+)_[0-9]+-[0-9]+-[0-9]+_(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "s.car.info") {
			return src.replace(/(\/image_files\/+)[0-9]+\/+/, "$1orig/");
		}

		if (domain_nowww === "cryengine.com") {
			return src.replace(/(\/files\/+[a-z]+\/+)[0-9]+\/+([0-9a-f]{10,}\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "cdn-l-cyberpunk.cdprojektred.com") {
			return src.replace(/\/gallery\/+1080p\/+/, "/gallery/2160p/");
		}

		if (domain_nowww === "romsmode.com" ||
			domain_nowww === "romsmania.cc") {
			return src.replace(/(\/statics\/+assets\/+covers\/+.*_[0-9]+)_mini(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "myabandonware.com") {
			newsrc = src.replace(/(\/media\/+screenshots\/.*\/)thumbs\/+/, "$1");
			if (newsrc !== src)
				return add_extensions(newsrc);
		}

		if (domain_nowww === "d-gram.co.kr") {
			return src.replace(/(\/data\/+dgram\/.*\/)small\/+([^/]+)(?:[?#].*)?$/, "$1big/$2");
		}

		if (domain_nosub === "sz-search.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+t.html\?(?:.*&)?ph=(http[^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "sanet.pics") {
			return src.replace(/(\/storage-[0-9]+\/+[0-9]+\/+)th_([^/]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "uploads.cdn.triller.co") {
			newsrc = src.replace(/(\/v1\/+uploads\/+[0-9]+\/+[0-9]+)_thumbnail\.[^/.?#]+(?:[?#].*)?$/, "$1_video.mp4");
			if (newsrc !== src) {
				return {
					url: newsrc,
					video: true
				};
			}
		}

		if (domain_nowww === "kinolift.ru") {
			return src.replace(/(\/media\/+users\/+[0-9]+\/+[0-9]+)_[sn](\.[^/.]+)(?:[?#].*)?$/, "$1_l$2");
		}


		if ((domain_nosub === "dropbox.com" || domain_nosub === "dropboxusercontent.com") && /previews\.dropbox/.test(domain)) {
			if (/\/p\/+thumb\/+[^/]{50,}\/+[^/]+(?:[?#].*)?$/.test(src)) {
				return src.replace(/(\/[^?#]+)(?:[?#].*)?$/, "$1?fv_content=true&size_mode=5");
			}
		}

		if (domain_nosub === "ddimg.cn") {
			newsrc = src
				.replace(/(\/[0-9]{2}\/+[0-9]{2}\/+[0-9]+-[0-9]+)_[a-z]_([0-9]+\.[^/.]+)(?:[?#].*)?$/, "$1_o_$2")
				.replace(/(:\/\/[^/]+\/+[0-9]+)_[a-z](\.[^/.]+)(?:[?#].*)?$/, "$1$2");

			return {
				url: newsrc,
				can_head: false
			};
		}

		if (domain === "image.bitautoimg.com") {
			return src.replace(/\/appimage-[^/]+\/+mapi\/+media\/+/, "/mapi/media/");
		}

		if (amazon_container === "miscellaneous-content") {
			regex = /(\/uploads\/+bfa\/+[0-9]+\/+([0-9]+)\/+)(?:thumb|large|wl|preview|original)_([0-9]+_[0-9]+\.[^/.]+)(?:[?#].*)?$/;
			match = src.match(regex);

			if (match) {
				page = "https://bfa.com/home/photo/" + match[2];

				obj = {
					url: src,
					extra: {
						page: page
					},
					headers: {
						Referer: page,
						"Sec-Fetch-Dest": "image",
						"Sec-Fetch-Mode": "same-origin",
						"Sec-Fetch-Site": "same-site"
					}
				};

				if (string_indexof(src, "/thumb_") >= 0 || string_indexof(src, "/large_") >= 0) {
					return fillobj_urls([
						src.replace(regex, "$1original_$3"),
						{
							url: src.replace(regex, "$1wl_$3"),
							problems: {
								watermark: true
							}
						},
						src.replace(regex, "$1large_$3")
					], obj);
				}

				if (string_indexof(src, "/wl_") >= 0 || string_indexof(src, "preview_") >= 0) {
					return fillobj_urls([
						src.replace(regex, "$1original_$3"),
						{
							url: src.replace(regex, "$1large_$3"),
							problems: {
								smaller: true
							}
						}
					], obj);
				}

				return obj;
			}
		}

		if (googlestorage_container === "inpock-link-media-storage") {
			return src.replace(/(\/media\/+images\/+[0-9]{4}\/+[0-9]+\/+[0-9]+\/+)thumb@[0-9]+_/, "$1");
		}

		if (domain_nowww === "24cos.com" ||
			domain === "imgs.diercun.com") {
			var referer = "https://" + domain_nosub + "/";
			if (domain_nosub === "diercun.com") {
				referer = "https://www.24mntp.com/";
			}

			return {
				url: src
					.replace(/:\/\/imgs(\.[^/]+\/+[^/]+\/+[0-9]{4}\/+[0-9]{4}\/+[0-9]+\/+)m(24[a-z]+_[0-9]+\.[^/.]+)(?:[?#].*)?$/, "://bimg$1$2")
					.replace(/(\/+[0-9]{4}\/+[0-9]{4}\/+[0-9]+\/+)m(24[a-z]+_[0-9]+\.[^/.]+)(?:[?#].*)?$/, "$1$2"),
				headers: {
					Referer: referer
				}
			};
		}

		if (domain === "img.xda-cdn.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+[^/]{10,}\/+(https?.*)/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "img.itch.zone" && host_domain_nosub === "itch.io" && options && options.element) {
			if (options.element.classList.contains("screenshot")) {
				if (options.element.parentElement.tagName === "A" && string_indexof(options.element.parentElement.href, "://img.itch.zone/") >= 0) {
					return options.element.parentElement.href;
				}
			}
		}

		if (domain === "im.camjaxx.com") {
			return src.replace(/:\/\/[^/]+\/+([0-9a-f]{10,})\/+preview\/+.*/, "://camjaxx.com/stream/$1");
		}

		if (domain_nowww === "camjaxx.com") {
			if (/:\/\/[^/]+\/+stream\/+[0-9a-f]+\/*(?:[?#].*)?$/.test(src)) {
				return {
					url: src,
					video: true
				};
			}
		}

		if (domain === "ars.els-cdn.com") {
			newsrc = src.replace(/(\/content\/+image\/+[^/]+)\.sml(?:[?#].*)?$/, "$1.jpg");
			if (newsrc !== src)
				return add_extensions_gif(newsrc);
		}

		if (domain === "static.gearslutz.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+board\/+imgext\.php\?(?:.*&)?u=([^&]*).*?$/, "$1");
			if (newsrc !== src) {
				return decodeuri_ifneeded(newsrc);
			}
		}

		if ((domain_nosub === "hstatic.dk" ||
			 domain_nosub === "smartweb-static.com") && string_indexof(src, "/upload_dir/") >= 0) {
			return src.replace(/(\/[^/]+?)(?:\.(?:[wh][0-9]+|fill)){1,}(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (amazon_container === "planetary") {
			return src.replace(/(\/assets\/+images\/+.*)_f[0-9]+(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "pastebin.com") {
			if (/^[a-z]+:\/\/[^/]+\/+i\/+t\.gif(?:[?#].*)?$/.test(src)) {
				return {
					url: src,
					bad: "mask"
				};
			}
		}

		if (domain_nowww === "davidrevoy.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+plugins\/+vignette\/+plxthumbnailer\.php\?(?:.*&)?src=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "d2xkkdgjnsfvb0.cloudfront.net") {
			return src.replace(/\/Vault\/+Thumb\?(?:.*&)?VaultID=([0-9]+).*$/, "/Vault/VaultOutput?ID=$1");
		}

		if (domain_nowww === "albumartexchange.com") {
			return src.replace(/(\/coverart\/+)_tn\/+/, "$1gallery/");
		}

		if (amazon_container === "inteng-storage") {
			return src.replace(/\/sizes\/+([^/]+)_resize_[^/.]+(\.[^/.]+)(?:[?#].*)?$/, "/$1$2");
		}

		if (domain === "covers.vitalbook.com") {
			return src.replace(/(\/vbid\/+[0-9]+)(?:\/+(?:height|width)\/+[0-9]+\/*)?(?:[?#].*)?$/, "$1");
		}

		if (domain_nosub === "subirimagenes.com" && /^s[0-9]*\./.test(domain)) {
			return src.replace(/\/previo\/+thump_/, "/");
		}

		if (host_domain_nowww === "justfor.fans" && options && options.element) {
			if (options.element.parentElement && options.element.parentElement.classList.contains("video-thumbnail")) {
				var link = options.element.parentElement.parentElement;
				if (link.tagName === "A") {
					var match = link.href.match(/^javascript:MakeMovieHD\("[^"]+"\s*,\s*({.*?})\s*,/);
					if (match) {
						try {
							var json = JSON_parse(match[1]);
							var maxres = 0;
							var maxobj = null;

							for (var resolution in json) {
								var our_resolution = parseInt(resolution);
								if (our_resolution > maxres) {
									maxres = our_resolution;
									maxobj = json[resolution];
								}
							}

							if (maxobj) {
								return {
									url: maxobj,
									video: true,
									is_private: true // linked to ip
								};
							}
						} catch (e) {
							console_error(e);
						}
					}
				}
			}
		}

		if (domain === "d3373orltkomw3.cloudfront.net") {
			if (/\/[0-9]+\/+[0-9a-z]+\.mp4/.test(src)) {
				return {
					url: src,
					video: true
				};
			}
		}

		if (domain_nowww === "apclips.com") {
			return src.replace(/(\/img\/+model\/+[^/]+\/+photo\/+[0-9a-f]{10,})_th(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "iwantclips.com") {
			return src.replace(/(\/uploads\/+aboutme_previews\/+[0-9]+\/+)[0-9]+_/, "$1");
		}

		if (domain === "s.bellesa.co") {
			newsrc = src.replace(/(\/v\/+[0-9a-f]{10,}\/+)(?:preview_[0-9]+)(\.[^/.]+)(?:[?#].*)?$/, "$1360$2");
			if (newsrc !== src)
				return newsrc;

			var sizes = [2160, 1080, 720, 480, 360];

			regex = /(\/v\/+[0-9a-f]{10,}\/+)([0-9]+)(\.[^/.]+)(?:[?#].*)?$/;
			match = src.match(regex);
			if (match) {
				var obj = [];
				for (var i = 0; i < sizes.length; i++) {
					if (sizes[i] <= match[1])
						break;

					obj.push(src.replace(regex, "$1" + sizes[i] + "$3"));
				}

				return obj;
			}
		}

		if (domain_nowww === "censored.tv") {
			if (/\/images\/+censored-bug\.png(?:[?#].*)?$/.test(src)) {
				return {
					url: src,
					bad: "mask"
				};
			}
		}

		if (domain === "imgs.ototoy.jp") {
			return src.replace(/(\/imgs\/+jacket\/+[0-9]+\/+[0-9]+\.[0-9]+\.[0-9]+)_[0-9]+(\.[^/.]+)(?:[?#].*)?$/, "$1orig$2");
		}

		if (domain === "gl.weburg.net") {
			return src.replace(/(\/albums\/+[0-9]+\/+[0-9]+\/+)[0-9]+x[0-9]+\/+/, "$1original/");
		}

		if (domain_nowww === "vgmdownloads.com") {
			return src.replace(/\/thumbs\/+([^/]+\.[^/.]+)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nowww === "linuxgamepublishing.com") {
			return src
				.replace(/(\/images\/+gamegraphics\/+[0-9]+\/+image)[0-9]+(\.[^/.]+)(?:[?#].*)?$/, "$1$2")
				.replace(/(\/images\/+gamegraphics\/+[0-9]+\/+)thumbnail([0-9]+\.[^/.]+)(?:[?#].*)?$/, "$1screenshot$2")
		}

		if (domain_nowww === "kino-teatr.ua") {
			return src.replace(/(\/public\/+main\/+[^/]+\/+)x[0-9]*_(photo_)/, "$1$2");
		}

		if (domain === "images.findagrave.com") {
			obj = {
				url: src.replace(/\/photo(?:Thumbnails|s[0-9]+)\/+photos\/+/, "/photos/"),
				head_wrong_contentlength: true
			};

			match = src.match(/\/photos\/+[0-9]{4}\/+[0-9]+\/+([0-9]+)_[0-9]+\./);
			if (match) {
				obj.extra = {page: "https://www.findagrave.com/memorial/" + match[1]};
			}

			return obj;
		}

		if (domain_nosub === "watson.de") {
			return src.replace(/(\/imgdb\/+[0-9a-f]+\/+)[^/]+,[^/]+\/+([0-9]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "img.zvuqa.net") {
			return src.replace(/\/artist[0-9]+x[0-9]+\/+/, "/artist-min/");
		}

		if (domain_nowww === "xboxplay.games") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+imagenes\/+redimensionar2\.php\?(?:.*&)?imagen=([^&]+).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "owo.lewd.ninja") {
			return src.replace(/(\/images\/+games\/+)(?:[a-z]_)?([0-9]+_[0-9a-f]{10,})(?:_thumb)?(\.[^/.]+)(?:[?#].*)?$/, "$1$2$3");
		}

		if (domain_nowww === "kinogo.by") {
			match = src.match(/\/uploads\/+cache\/+(?:[0-9a-f]\/+){9}([0-9]+-[^/]+)-[0-9]+x[0-9]+(\.[^/.]+)(?:[?#].*)?$/);
			if (match) {
				obj = [];
				obj.push(src.replace(/\/uploads\/+.*/, "/uploads/posters/" + match[1] + match[2]));

				var match1 = match[1].match(/^([0-9]{10,})-[0-9]+-/);
				if (match1) {
					var date = new Date(parseInt(match1[1]) * 1000);
					obj.push(src.replace(/\/uploads\/.*/, "/uploads/posts/" + date.getUTCFullYear() + "-" + (date.getUTCMonth() + 1) + "/" + match[1] + match[2]));
				}

				return obj;
			}
		}

		if (domain === "ip.bitcointalk.org") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+\?(?:.*&)?u=([^&]+).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "carbonmade-media.accelerator.net" ||
			domain === "carbon-media.accelerator.net") {
			return src.replace(/(:\/\/[^/]+\/+(?:[0-9]+\/+[^/.]+|[0-9]+));[0-9]*x[0-9]*(?:\/+[^/.]+)?(\.[^/.]+)(?:[?#].*)?$/, "$1;original$2");
		}

		if (domain_nowww === "streamja.com") {
			if (/\/img\/+play\.png(?:[?#].*)?$/.test(src)) {
				return {
					url: src,
					bad: "mask"
				};
			}
		}

		if (domain === "img.thedonald.win" && host_domain_nowww === "thedonald.win" && options && options.element) {
			if (options.element.parentElement && options.element.parentElement.tagName === "A") {
				var parent = options.element.parentElement;
				var dparent = parent.parentElement;
				if (dparent && dparent.tagName === "DIV" && dparent.classList.contains("thumb")) {
					return parent.href;
				}
			}
		}

		if (host_domain_nowww === "derstandard.at" && options.element) {
			if (options.element.tagName === "BUTTON" && options.element.classList.contains("figure-fullscreen-trigger")) {
				return {
					url: origsrc,
					bad: "mask"
				};
			}

			if (options.element.parentElement && options.element.parentElement.tagName === "PICTURE") {
				var fullscreen_img = options.element.parentElement.querySelector("img[data-fullscreen-src]");
				if (fullscreen_img) {
					return urljoin(src, fullscreen_img.getAttribute("data-fullscreen-src"), true);
				}
			}
		}

		if (domain === "cl.fame10.com") {
			return src
				.replace(/(\/image\/+upload\/+f10\/+.*)-[0-9]+x[0-9]+(\.[^/.]+)(?:[?#].*)?$/, "$1$2")
				.replace(/\/image\/+upload\/+[^/]+\/+(f10\/+.*)$/, "/image/upload/$1");
		}

		if (domain_nowww === "photo-forum.net") {
			return src
				.replace(/\/imgs_(?:thumbs_(?:[0-9]+|[a-z]+)|medium)\/+/, "/imgs/")
				.replace(/(\/static\/+monthlytheme\/+id[0-9]+)_small(\.[^/.]+)(?:[?#].*)?$/, "$1$2")
				.replace(/(\/static\/+media\/+[0-9]{4}\/+[^/]+)-[0-9]+px(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "photoforum.ru") {
			return src.replace(/(\/f\/+(?:photo|portr))\.([^/]+)(\/+[0-9]{3}\/+[0-9]{3}\/+[0-9]+_[0-9]+)\.\2(\.[^/.]+)(?:[?#].*)?$/, "$1$3$4");
		}

		if (domain_nowww === "clippituser.tv") {
			if (/\/static\/+[^/]+\/+images\/+small_play_btn\./.test(src)) {
				return {
					url: src,
					bad: "mask"
				};
			}
		}

		if (domain === "clips.clippit.tv") {
			obj = {url: src};
			id = null;

			match = src.match(/:\/\/[^/]+\/+([^/]+)\/+/);
			if (match) {
				id = match[1];
				obj.extra = {page: "https://www.clippituser.tv/c/" + id};
			}

			if (/\/360\.mp4(?:[?#].*)?$/.test(src)) {
				obj.url = src.replace(/\/[0-9]+(\.[^/.]+)(?:[?#].*)?$/, "/720$1");
				return obj;
			}

			if (/\/thumbnail\.jpg(?:[?#].*)?$/.test(src)) {
				obj.url = src.replace(/\/thumbnail\.jpg(?:[?#].*)?$/, "/360.mp4");
				return obj;
			}

			return obj;
		}

		if (amazon_container === "clippit-prod") {
			return src.replace(/(\/media\/+profiles\/+[0-9]+\/+[0-9]+\.[^/.]+)(?:\.[^/]+\.[^/.]+)(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "porntrex.com" ||
			domain_nowww === "fapster.xxx" ||
			domain_nowww === "fapality.com" ||
			domain_nowww === "okporn.com" ||
			domain === "storage.hegre.com" ||
			domain_nowww === "pornrewind.com") {
			if (/\/skin\/+img\/+play_/.test(src))
				return {
					url: src,
					bad: "mask"
				};
		}

		if (domain === "s.sc-cdn.net") {
			newsrc = src.replace(/\/default\/+preview\.jpg(?:[?#].*)?$/, "/default/media.mp4");
			if (newsrc !== src) {
				var full = add_full_extensions(newsrc, ["mp4", "jpg"], true);

				if (host_domain_nosub === "snapchat.com")
					full[0].need_blob = true;

				return full;
			}

			if (false && /\/default\/+overlay\.jpg(?:[?#].*)?$/.test(src)) {
				return {
					url: src,
					bad: "mask"
				};
			}

			match = src.match(/:\/\/[^/]+\/+[0-9a-f]{2}\/+([^/]{10,})\//);
			if (false && match) {
				return {
					url: src,
					filename: match[1]
				};
			}
		}

		if (host_domain_nosub === "snapchat.com" && options.do_request && options.cb && options.element) {
			var info = common_functions.get_snapchat_info_from_el(options.element);
			if (info) {
				common_functions.get_obj_from_snap_info(api_cache, options.do_request, info, options.cb);

				return {
					waiting: true
				};
			}

			if (src && /\.mp4/.test(src)) {
				return {
					url: src,
					need_blob: true
				};
			} else {
				return null;
			}
		}

		if (domain === "soup.onerpm.com") {
			return src.replace(/(\/web\/+user\/+(?:images|content)\/+.\/+..\/+[^/]+\.)[0-9]+(\.[^/.]+)(?:[?#].*)?$/, "$1orig$2");
		}

		if (domain_nosub === "crank-in.net") {
			newsrc = src.replace(/(\/img\/+db\/+[0-9]+)_(?:[0-9]{3}|1[01][0-9]{2})(\.[^/.]+)(?:[?#].*)?$/, "$1_1200$2");
			if (newsrc !== src) {
				return {
					url: newsrc,
					problems: {
						possibly_upscaled: true
					}
				};
			}
		}

		if (domain === "hs.mediadelivery.fi" ||
			domain === "is.mediadelivery.fi") {
			newsrc = src.replace(/(\/img\/+(?:square\/+)?)(?:[0-9]{3}|1[0-8][0-9]{2})\//, "$11920/");
			if (newsrc !== src) {
				return {
					url: newsrc,
					problems: {
						possibly_upscaled: true
					}
				};
			}
		}

		if (domain === "cdn.cdkeys.com") {
			return src.replace(/(:\/\/[^/]+\/+)[0-9]+x[0-9]+\/+media\//, "$1media/");
		}

		if (domain === "d1466nnw0ex81e.cloudfront.net") {
			return src.replace(/\/n_ii\/+[0-9]+\/+/, "/n_ii/originalimage/");
		}

		if (domain === "assets.upflix.pl") {
			return src.replace(/(\/media\/.*)__[0-9]+_[0-9]+(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "enfilme.com") {
			return src.replace(/(\/img\/+content\/+[^/]+_[0-9a-f]+)_[0-9]+_[0-9]+(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "koha.net") {
			return src.replace(/(\/uploads\/+images\/+[0-9]{4}\/+[A-Za-z]+\/+[0-9]+\/+)[0-9]+x[0-9]+_/, "$1");
		}

		if (domain_nowww === "nrwkino.de" ||
			domain_nowww === "biograph.de") {
			newsrc = src.replace(/.*\/img\.php\?(?:.*&)?src=([^&]+).*?$/, "$1");
			if (newsrc !== src) {
				return urljoin(src, decodeURIComponent(newsrc.replace(/\+/g, "%20")), true);
			}

			return src.replace(/\/img\/+[wh][0-9]+\/+upload\/+/, "/upload/");
		}

		if (domain === "img.lap.recochoku.jp") {
			newsrc = decodeURIComponent(src);
			if (/common\/+store(?:_[^/]+)?\/+[^/]+\.(?:gif|png)(?:[?#].*)?$/.test(src))
				return {
					url: src,
					bad: "mask"
				};


			if (/\/p1\/+imgkp\?/.test(src)) {
				var queries = get_queries(src);
				for (var query in queries) {
					if (query !== "f" && query !== "FFh" && query !== "FFw" && query !== "p") {
						delete queries[query];
						continue;
					}

					if (query === "FFh" || query === "FFw") {
						queries[query] = "999999999";
					}
				}

				if (queries) {
					newsrc = src.replace(/\?.*/, "") + "?" + stringify_queries(queries);
					if (newsrc !== src)
						return newsrc;
				}
			}
		}

		if (domain === "s.akb48.co.jp") {
			return src.replace(/(:\/\/[^/]+\/+)sns[0-9]+\/+/, "$1sns/");
		}

		if (domain_nowww === "16bit.pl") {
			newsrc = src.replace(/(\/download\/+games\/+screens\/+[^/]+\/+)t([0-9]+\.[^/.]+)(?:[?#].*)?$/, "$1$2");
			if (newsrc !== src)
				return add_extensions(newsrc);
		}

		if (domain === "vp.beeg.com") {
			id = null;
			match = src.match(/:\/\/[^/]+\/+vp\/+[0-9]{2}\/+([0-9]+)_[0-9]+_[0-9]+\.mp4/);
			if (match) {
				id = match[1];
			}

			if (id && options.do_request && options.cb) {
				api_cache.fetch("beeg:" + id, options.cb, function(done) {
					options.do_request({
						url: "https://beeg.com/api/v6/" + Date.now() + "/video/" + id + "?v=2",
						method: "GET",
						headers: {
							Referer: "https://beeg.com/1989513890",
							"X-Requested-With": "XMLHttpRequest"
						},
						onload: function(resp) {
							if (resp.readyState !== 4)
								return;

							if (resp.status !== 200) {
								console_error(resp);
								return done(null, false);
							}

							try {
								var json = JSON_parse(resp.responseText);
								var maxp = 0;
								var maxurl = null;
								for (var key in json) {
									var match = key.match(/^([0-9]+)p/);
									if (!match || !json[key])
										continue;

									var our_p = parseInt(match[1]);
									if (string_indexof(json[key], "//video.beeg.com/") < 0)
										continue;

									if (our_p > maxp) {
										maxp = our_p;
										maxurl = json[key];
									}
								}

								if (maxurl) {
									maxurl = maxurl.replace(/{DATA_MARKERS}/, "data=pc_US__" + Date.now() + "_");
									maxurl = urljoin("https://beeg.com/", maxurl, true);
									return done({
										url: maxurl,
										private: true,
										video: true
									}, 2*60*60); // expires in 3 hours
								} else {
									return done(null, false);
								}
							} catch (e) {
								console_error(e, resp);
								return done(null, false);
							}
						}
					});
				});

				return {
					waiting: true
				};
			}
		}

		if (host_domain_nowww === "dogpile.com" && domain_nosub === "bing.net" && /^ts[^.]+\.mm\./.test(domain) && options && options.element) {
			if (options.element.tagName === "IMG" && options.element.parentElement && options.element.parentElement.tagName === "A") {
				if (options.element.parentElement.classList.contains("link")) {
					var parent = options.element.parentElement;
					if (parent.parentElement && parent.parentElement.classList.contains("image")) {
						if (parent.href !== src)
							return parent.href;
					}
				}
			}
		}

		if (host_domain_nowww === "yippy.com" && domain_nosub === "bing.net" && /^ts[^.]+\.mm\./.test(domain) && options && options.element) {
			if (options.element.tagName === "IMG" && options.element.parentElement && options.element.parentElement.tagName === "DIV") {
				var parent = options.element.parentElement;
				if (parent.classList.contains("media-box-thumbnail-container") && parent.parentElement) {
					var dparent = parent.parentElement;
					newsrc = dparent.getAttribute("data-mfp-src");
					if (newsrc && newsrc !== src) {
						return newsrc;
					}
				}
			}
		}

		if (host_domain_nowww === "gigablast.com" && options && options.element) {
			if (options.element.tagName === "IMG" && options.element.parentElement && options.element.parentElement.tagName === "A") {
				var parent = options.element.parentElement;
				var current = parent;
				while ((current = current.parentElement)) {
					if (current.tagName === "TABLE" && current.classList.contains("image")) {
						if (parent.href !== origsrc) {
							return parent.href;
						} else {
							break;
						}
					}
				}
			}
		}

		if (host_domain_nowww === "ecosia.org" && options && options.element) {
			if (options.element.tagName === "IMG" && options.element.parentElement && options.element.parentElement.tagName === "A") {
				var parent = options.element.parentElement;
				if (parent.classList.contains("image-result")) {
					if (parent.href !== src) {
						return parent.href;
					}
				}
			}
		}

		if (domain === "spapi.searchencrypt.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+image\?(?:.*&)?path=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "boardreader.com") {
			if (/\/img\/+placeholder\.gif/.test(src))
				return {
					url: src,
					bad: "mask"
				};
		}

		if (domain === "imgproxy.geocaching.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+[0-9a-f]{10,}\?(?:.*?&)?url=([^&]+).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "wolframcdn.com") {
			return src.replace(/(\/waimage\/.*_v[0-9]+)s(\.[^/.]+)(?:[?#].*)?$/, "$1ms$2");
		}

		if (domain_nowww === "allthetests.com") {
			return src.replace(/\/picture_thumb\/+pic_/, "/picture/pic_");
		}

		if (domain_nowww === "dicelacancion.com") {
			return src.replace(/(\/static\/+img\/+[^/]+\/+.\/+[^/]+\/+[^/]+)_[lbm](\.[^/.]+)(?:[?#].*)?$/, "$1_o$2");
		}

		if (domain === "static.pressfrom.info") {
			return src.replace(/\/upload\/+images\/+small\/+/, "/upload/images/real/");
		}

		if (domain_nowww === "shadestation.co.uk") {
			return src.replace(/\/media\/+thumbs\/+[0-9]+x[0-9]+\/+(media\/+product_images\/+[^/]+)fw[0-9]+fh[0-9]+(\.[^/.]+)(?:[?#].*)?$/, "/$1$2");
		}

		if (domain === "image.ceneostatic.pl") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+offercandidates\/+([^/?#]{30,})(?:[?#].*)?$/, "$1");
			if (newsrc != src) {
				var decoded = base64_decode(decodeURIComponent(newsrc));
				newsrc = decoded.replace(/.*(https?:\/\/)/, "$1");
				if (newsrc.match(/^https?:\/\//)) {
					return newsrc;
				}
			}
		}

		if (domain === "img-cloud.megaksiazki.pl") {
			return src.replace(/(:\/\/[^/]+\/+[0-9]+-)[a-z]+\/+([0-9a-f]{10,}\/+)/, "$1original/$2");
		}

		if (domain_nowww === "e-onkyo.com") {
			return src.replace(/\/image\/+jacket\/+s[1-5]\/+/, "/image/jacket/s0/");
		}

		if (domain_nowww === "ubuy.qa") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+productimg\/+\?(?:.*&)?image=([^&.]+)(?:\.[^/.&]+)?.*?$/, "$1");
			if (newsrc !== src)
				return atob(decodeURIComponent(newsrc));
		}

		if (domain === "web.rds.it") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+i\/+[^/]+\/+\?(?:.*&)?url=([^&]+).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "media.loot.co.za") {
			return src.replace(/(:\/\/[^/]+\/+images\/+)x(?:40|80|120|200)\/+/, "$1x400/");
		}

		if (domain === "img.ibs.it") {
			return src.replace(/(:\/\/[^/]+\/+images\/+[0-9]{5,})(?:_[0-9]+){4}(\.[^/.]+)(?:[?#].*)?$/, "$1_0_0_0_0$2");
		}

		if (domain_nowww === "melodycenter.hu") {
			return src.replace(/(\/img\/+[0-9]+\/+[^/]+\/+)[0-9]+x[0-9]+(?:,[^/]+)?\/+/, "$1");
		}

		if (domain === "images.music-story.com") {
			return src.replace(/(\/img\/+artiste_[A-Z])_[0-9]+\/+/, "$1/");
		}

		if (domain === "naslehiphop.com") {
			return src.replace(/(\/files\/+uploads\/+[0-9]{4}\/+[0-9]+\/+[0-9]+)_thumbnail(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "pic.eslite.com") {
			return src.replace(/(\/Upload\/+Product\/+[0-9]{6}\/+)[a-z]\/+/, "$1o/");
		}

		if (domain_nosub === "vibbidi-vid.com") {
			newsrc = src.replace(/(\/vibbidi-[a-z]+\/+(?:albums|audio|videos)\/+(?:(?:img|audio|video)_[A-F0-9]{10,}|[0-9]+)(?:\.(?:[0-9]+|mp3)){0,})\.(?:[0-9]*x[0-9]*|small)(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/(\/videos\/+video_[0-9A-F]{10,}\.mp4)(?:\..*)?(?:[?#].*)?$/, "$1");
			if (newsrc !== src)
				return {
					url: newsrc,
					video: true
				};

			if (/\/videos\/+video_[0-9A-F]{10,}\.mp4$/.test(src)) {
				return {
					url: src,
					video: true
				};
			}
		}

		if (domain_nowww === "vibbidi.net") {
			if (/\/static\/+img\/+no_[0-9]+x[0-9]+\./.test(src))
				return {
					url: src,
					bad: "mask"
				};
		}

		if (host_domain_nowww === "vibbidi.net" && options.element) {
			if (options.element.tagName.toLowerCase() === "svg") {
				if (options.element.classList.contains("IcoPlayMedium")) {
					return {
						url: origsrc,
						bad: "mask"
					};
				}
			}
		}

		if (domain_nowww === "puregrainaudio.com") {
			return src.replace(/(\/images\/+blogimages\/+)[0-9]+s\/+/, "$1");
		}

		if (domain_nowww === "mfmm.ru") {
			return src.replace(/(\/_bd\/+[0-9]+\/+)s([0-9]+\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "mlstatic.com") {
			return src.replace(/(:\/\/[^/]+\/+(?:[^/]+-)?D_[^/.]+)-[A-Z](\.[^/.]+)(?:-[A-Z]\.[^/.]+)?(?:[?#].*)?$/, "$1-F$2");
		}

		if (domain === "angartwork.akamaized.net") {
			obj = {
				url: src,
				head_wrong_contentlength: true
			};

			var queries = get_queries(src);
			if ("id" in queries) {
				return fillobj_urls([src.replace(/\?.*/, "?id=" + queries.id)], obj);
			}

			return obj;
		}

		if (domain === "pimg.awa.io") {
			return {
				url: src.replace(/(\/v2\/+[^/]+\/+[0-9a-f]{10,})(?:\.(?:[whv][0-9]+|fitcrop)){1,}(\.[^/.]+)(?:[?#].*)?$/, "$1.w9999999999$2"),
				can_head: false // 404
			};
		}

		if (domain_nosub === "hudobny.sk") {
			return src.replace(/\/images\/+[a-z]+\/+([0-9]+\.[^/.]+)(?:[?#].*)?$/, "/images/original/$1");
		}

		if (domain === "cdnimages01.azureedge.net") {
			return src.replace(/(\/megahits\/+[^/]+[0-9a-f]{8})_square[a-z]*(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain === "media.becteroradio.com") {
			return src.replace(/(\/pix\/+[^/]+\/+[0-9]+)_[a-z]+(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}


		if (domain_nowww === "saga-rian.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+common\/+image2\.php\?(?:.*&)?f=([^&]+).*?$/, "$1");
			if (newsrc !== src) {
				return decodeuri_ifneeded(newsrc);
			}
		}

		if (domain_nowww === "wigtypes.com") {
			return src.replace(/(\/product_description\/+[^/]+\/+[^/]+)_175(\.[^/.]+)(?:[?#].*)?$/, "$1_600$2");
		}

		if (domain_nowww === "beautyhill.com") {
			return src.replace(/(\/img\/+[^/]+\/+[0-9]{4}\/+.*)_thumb(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "she12.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+wp-content\/+plugins\/+seo-alrp\/+php\/+thumb\.php\?(?:.*&)?src=([^&]+).*?$/, "$1");
			if (newsrc !== src) {
				return urljoin(src, decodeuri_ifneeded(newsrc), true);
			}
		}

		if (domain_nowww === "soemo.co.uk") {
			return src.replace(/(:\/\/[^/]+\/+images\/+[^/]+)x(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "tasior.com") {
			return src.replace(/(\/upload\/+foto\/+[0-9]+\/+)th([0-9]+\.)/, "$1$2");
		}

		if (domain_nosub === "blogerka.cz") {
			return src.replace(/(\/obrazky\/+[^/]+\/+[^/]+)\.tn\.[^/.]+(?:[?#].*)?$/, "$1");
		}

		if (domain_nowww === "worldsmostunique.com") {
			return src.replace(/\/images\/+thumbnails\/+([^/]+)_[a-z]+(\.[^/.]+)(?:[?#].*)?$/, "/images/images/$1$2");
		}

		if (domain_nowww === "zarzarmodels.com") {
			return src.replace(/(\/images\/+models\/+[0-9]+\/+)thumb\/+/i, "$1");
		}

		if (domain === "img.budgettravel.com") {
			return src.replace(/(:\/\/[^/]+\/+)_[a-zA-Z]+\/+/, "$1");
		}

		if (domain === "d3ts7pb9ldoin4.cloudfront.net" ||
			amazon_container === "star-uploads") {
			return src.replace(/(\/uploads\/+users\/+[0-9]+\/+[^/]+\/+[0-9a-f]{8}(?:-[0-9a-f]{4}){3}-[0-9a-f]{12})-[0-9]+x[0-9]+(?:_[0-9]+x[0-9]+){2}(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (amazon_container === "static.e-junkie.com") {
			return src.replace(/\/products\/+[a-z]+-images\/+/, "/products/images/");
		}

		if (domain === "api.europeana.eu") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+api\/+v2\/+thumbnail-by-url\.json\?(?:.*&)?uri=([^&]+).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "fanparty.ru") {
			return src.replace(/(\/gallery\/+[0-9]+_[^/]+)_medium(\.[^/.]+)(?:[?#].*)?$/, "$1_pic$2");
		}

		if (domain_nowww === "metlifestadium.com") {
			return src.replace(/(\/images\/+.*)\.tmb-(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nosub === "fxinteractive.com") {
			return src.replace(/(\/images\/+games\/+[^/]+\/+screenshots\/+[^/]+)_thumbnail_([0-9]+\.[^/.]+)(?:[?#].*)?$/, "$1_screenshot_$2");
		}

		if (domain_nowww === "kinogallery.com") {
			return src.replace(/(\/pimages\/+[0-9]+\/+[^-/]+-)pr-/, "$1");
		}

		if (domain === "image.mux.com") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+([^/.]{30,})\/+thumbnail\.[^/.]+(?:[?#].*)?$/, "https://stream.mux.com/$1.m3u8");
			if (newsrc !== src) {
				return {
					url: newsrc,
					video: "hls"
				};
			}
		}

		if (domain === "thumbnails.pcgamingwiki.com") {
			return src.replace(/:\/\/[^/]+\/+(.)\/+(..)\/+([^/]*)\/+[0-9]+px-.*?$/, "://images.pcgamingwiki.com/$1/$2/$3");
		}

		if (domain_nowww === "filmix.co") {
			return src.replace(/(\/uploads\/+frames\/+[0-9]+\/+[^/]+)_small(\.[^/.]+)(?:[?#].*)?$/, "$1_original$2");
		}

		if (domain === "thumbs.filmix.co") {
			return src.replace(/\/posters\/+thumbs\/+[wh][0-9]+\/+/, "/posters/orig/");
		}

		if (domain_nowww === "5tv5.ru") {
			return src
				.replace(/\/smallposters\/+/, "/posters/")
				.replace(/(\/frames\/+[0-9]+\/+)s([0-9]+\.)/, "$1f$2");
		}

		if (domain === "thumbs.dfs.ivi.ru") {
			return src.replace(/(\/contents\/+[0-9a-f]\/+[0-9a-f]\/+[0-9a-f]{10,}\.[^/.]+)(?:\/+[0-9]+x[0-9]+\/*)(?:[?#].*)?$/, "$1");
		}

		if (domain === "i.pixxxels.cc") {
			var get_pixxxels_id = function(url) {
				id = url.match(/^[a-z]+:\/\/[^/]+\/+([^-_/.]+)\/+[^/]+(?:[?#].*)?$/);
				if (id)
					return id[1];

				return null;
			};

			var get_pixxxels_referer = function(url) {
				var id = get_pixxxels_id(url);

				if (id) {
					return "https://pixxxels.cc/" + id;
				} else {
					return "https://pixxxels.cc/";
				}
			};

			id = get_pixxxels_id(src);
			if (id && options.do_request && options.cb) {
				var fetch_pixxxels_page = function(id, cb) {
					var cache_key = "pixxxels:" + id;
					api_cache.fetch(cache_key, cb, function(done) {
						options.do_request({
							url: "https://pixxxels.cc/" + id,
							method: "GET",
							headers: {
								Referer: ""
							},
							onload: function(resp) {
								if (resp.readyState !== 4)
									return;

								if (resp.status !== 200) {
									console_error(cache_key, resp);
									return done(null, false);
								}

								var match = resp.responseText.match(/<a[^>]+href=["'](https?:\/\/i\.pixxxels\.cc\/+[^"']+\?dl=1)["']/);
								if (!match) {
									console_error(cache_key, "Unable to find download link for", resp);
									return done(null, false);
								}

								var dllink = decode_entities(match[1]).replace(/[?#].*$/, "");
								var dllink_id = get_pixxxels_id(dllink);
								var obj = {
									url: dllink,
									headers: {
										Referer: get_pixxxels_referer(dllink)
									},
								};

								if (dllink_id) {
									obj.extra = {page: "https://pixxxels.cc/" + dllink_id};
								}

								return done(obj, 60*60);
							}
						});
					});
				};

				fetch_pixxxels_page(id, options.cb);
				return {
					waiting: true
				};
			} else if (id) {
				return {
					url: dllink,
					headers: {
						Referer: get_pixxxels_referer(dllink)
					}
				};
			}
		}

		if (domain === "t.meadd.net") {
			return src.replace(/:\/\/[^/]+\/+photos\/+[0-9]+\/+([0-9]+\.[^/.]+)(?:[?#].*)?$/, "://cdn.meadd.net/photos/full/$1");
		}

		if (domain === "s.uicdn.com") {
			return {
				url: src,
				bad: "mask"
			};
		}

		if (domain_nosub === "x-art.com" && /^(?:hosted|content[0-9]*)\./.test(domain)) {
			newsrc = src.replace(/(\/models\/+[^/]+\/+[^/]+)-tn(\.[^/.]+)(?:[/#].*)?$/, "$1$2");
			if (newsrc !== src)
				return newsrc;

			match = src.match(/^([a-z]+:\/\/[^/]+\/+galleries\/+[^/]+\/+[^/]+)-(l?tn|sml|med)(\.[^/.]+)(?:[?#].*)?$/);
			if (match) {
				var sizes = [
					"lrg",
					"med",
					"sml",
					"ltn",
					"tn"
				];

				var sizes_index = sizes.indexOf(match[2]);
				var urls = [];
				if (sizes_index >= 0) {
					for (var i = 0; i < sizes_index; i++) {
						urls.push(match[1] + "-" + sizes[i] + match[3]);
					}

					return urls;
				}
			}
		}

		if (domain_nowww === "vkanketa.ru") {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+file\.php\?(?:.*&)?img=(http[^&]+)(?:[#].*)?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain === "blob.freent.de") {
			return src.replace(/\/image\/+([0-9]+)\/+[0-9]+x[0-9]+\/+[0-9]+\/+[0-9]+\/+[0-9a-f]{2}\/+[0-9a-f]{10,}\/+[^/]+\/+([^/]+)(?:[?#].*)?$/,
								"/contentblob/$1/original/$2");
		}

		if (domain_nosub === "cdnsw.com" && /^mfs[0-9]*\./.test(domain)) {
			return src.replace(/\/fs\/+Root\/+[a-z]+\/+/, "/fs/Root/");
		}

		if (domain_nowww === "kupidon-rf.ru") {
			return src.replace(/(\/uploads\/+[^/]+\/+[0-9]{4}\/+[0-9]{2}\/+[0-9]{2}\/+[0-9]+\/+)[a-z]+-([0-9a-f]{5,}\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}

		if (domain_nowww === "fondecranmagique.com") {
			return src.replace(/\/min\/+([0-9]+\.[^/.]+)(?:[?#].*)?$/, "/$1");
		}

		if (domain_nowww === "photohost.ru") {
			return src.replace(/\/t\/+[0-9]+\/+[0-9]+\/+/, "/pictures/");
		}

		if (domain_nosub === "sat1.de" && /^i[0-9]*-img\./.test(domain)) {
			return {
				url: src.replace(/\/profile:[^/]+(?:[?#].*)?$/, "/profile:original?source"),
				can_head: false // 403
			};
		}

		if (domain === "m.gjcdn.net") {
			return src
				.replace(/\/screenshot-thumbnail\/+[0-9]+x[0-9]+\/+/, "/screenshot-thumbnail/999999999x999999999/")
				.replace(/\/(game-(?:header|thumbnail))\/+[0-9]+\/+([0-9]+-)(?:crop[0-9]+(?:_[0-9]+){3}-)?/, "/$1/999999999/$2");
		}

		if (domain_nowww === "crosti.ru") {
			return src.replace(/(\/patterns\/+(?:[0-9a-f]{2}\/+){3}[0-9a-f]{5,}\/+)thumbnail(\.[^/.]+)(?:[?#].*)?$/, "$1picture$2");
		}

		if (domain === "cdn.carbuzz.com") {
			return src.replace(/(\/(?:car-thumbnails|gallery-images)\/+)[0-9]+(?:x[0-9]+)?\/+/, "$1original/");
		}

		if (domain_nosub === "slideserve.com" && /^(?:image|cdn)[0-9]*\./.test(domain)) {
			return src.replace(/(\/[0-9]+\/+[^/]+)-t(\.[^/.]+)(?:[?#].*)?$/, "$1-n$2");
		}

		if (domain === "resizer.mujerhoy.com") {
			if (/\/resizer\/+resizer\.php\?/.test(src)) {
				return remove_queries(src, ["nuevoancho", "nuevoalto", "crop"]);
			}
		}

		if (domain_nowww === "lahiguera.net") {
			return src.replace(/(\/fotos\/+[0-9]+\/+[^/]+)-p(\.[^/.]+)(?:[?#].*)?$/, "$1$2");
		}














































































































































































		if (src.match(/\/ImageGen\.ashx\?/)) {
			return urljoin(src, src.replace(/.*\/ImageGen\.ashx.*?image=([^&]*).*/, "$1"));
		}

		if (domain === "i.pinger.pl" ||
			src.search(/(?::\/\/[^/]*\/(gallery|photos|photogallery)|:\/\/gallery\.[^/]*)?\/albums\/+[^/]*(?:\/+.*)?\/+(normal|thumb|userpics)_[^/.]*\.[^/.]*$/) >= 0) {
			newsrc = src.replace(/\/[a-z]*_([^/.]*\.[^/.]*)$/, "/$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain_nowww === "torrenthaja.com" ||
			src.match(/\/data\/[^/]*\/[^/]*\/(?:_thumb\/[^/]*\/|thumbs\/)?[^/]*$/)) {
			newsrc = src
				.replace(/\/thumb-([^/]*)_[0-9]+x[0-9]+(\.[^/.]*)(?:[?#].*)?$/, "/$1$2")
				.replace(/(\/data\/[^/]*\/[0-9]+\/)_thumb\/[0-9]+x[0-9]_[0-9]+\//, "$1")
				.replace(/(\/data\/[^/]*\/[0-9]+\/)thumbs\//, "$1");

			if (newsrc !== src)
				return add_full_extensions(newsrc);
		}

		if (src.search(/\/phpwas\/restmb_[a-z]*make\.php\?/) >= 0) {
			if (domain === "cgeimage.commutil.kr" ||
				domain === "cliimage.commutil.kr") {
				src = src.replace(/\/phpwas\/restmb_[a-z]*make\.php/, "/phpwas/restmb_allidxmake.php");
			}

			if (string_indexof(domain, "nimage.") === 0) {
				newsrc = src.replace(/\/phpwas\/restmb_idxmake\.php.*?simg=([0-9]{4})([0-9]{2})([0-9]{2})([^&]*).*?$/, "/photo/$1/$2/$3/$1$2$3$4");
				if (newsrc !== src)
					return newsrc;
			}

			if (domain === "res.heraldm.com" &&
				decodeURIComponent(src.match(/simg=([^&]*)/)[1])[0] === "/") {
				return urljoin(src, decodeURIComponent(src.match(/simg=([^&]*)/)[1]), true);
			}

			return src.replace(/(\/phpwas\/restmb_[a-z]*make\.php)\?.*(simg=[^&]*)/, "$1?idx=999&$2");
		}

		if (src.match(/.*?\/timthumb(?:\/index)?\.php\?(?:.*&)?src=/)) {
			newsrc = src.replace(/.*\/timthumb(?:\/index)?\.php\?(?:.*&)?src=([^&]+).*$/, "$1");
			if (newsrc !== src) {
				return urljoin(src, decodeURIComponent(newsrc), true);
			}
		}

		if (src.match(/\/fotogallery\/[0-9]+X[0-9]+\//)) {
			return src.replace(/\/fotogallery\/[0-9]+X[0-9]+\//, "/fotogallery/9999999999X0/");
		}

		if (string_indexof(src, "/redim_recadre_photo.php") >= 0) {
			return src.replace(/.*\/redim_recadre_photo\.php\?.*?path_url=([^&]*).*/, "$1");
		}

		if (string_indexof(src, "/wp-apps/imrs.php?") >= 0) {
			return src.replace(/.*\/wp-apps\/imrs\.php\?[^/]*src=([^&]*).*/, "$1");
		}

		if (src.match(/\/dynimage\/[^/]*\/[0-9]*\/[^/]*$/)) {
			return src.replace(/\/dynimage\/[^/]*\//, "/dynimage/original/"); // can be anything
		}

		if (src.match(/\/phocagallery\/.*\/thumbs\/[^/]*$/)) {
			return src.replace(/\/thumbs\/phoca_thumb_[^/._]*_/, "/");
		}

		if (src.match(/\/sfc\/servlet\.shepherd\/version\/renditionDownload/)) {
			return src.replace(/\/renditionDownload.*?[?&]versionId=([^&]*).*/, "/download/$1");
		}

		if (domain === "cdn.ome.lt") {
			return src.replace(/(:\/\/[^/]*\/)[-_A-Za-z0-9]+=\/(?:(?:full-)?fit-in\/)?(?:[0-9x:]+\/)?(?:[0-9x:]+\/)?(?:(?:top|center|middle)\/)?(?:(?:smart|top|center|middle)\/)?(?:filters:[^/]*\/)?/, "$1");
		}

		if ((domain_nosub === "vox-cdn.com" && string_indexof(domain, "cdn.vox-cdn.com") >= 0) ||
			domain === "thumbnails.trvl-media.com" ||
			domain === "thumbor-static.factorymedia.com" ||
			domain === "cdnrockol-rockolcomsrl.netdna-ssl.com" ||
			domain === "thumbor-titelmediaug.netdna-ssl.com" ||
			domain_nowww === "infobae.com" ||
			(domain_nosub === "glbimg.com" && domain.match(/s[0-9]*\.glbimg\.com/)) ||
			domain === "i.amz.mshcdn.com" ||
			domain === "img.ilcdn.fi" ||
			domain === "img.fstatic.com" ||
			domain === "img.mthcdn.com" ||
			domain === "images.thestar.com" ||
			(domain_nosub === "holidaypirates.com" && domain.match(/thumb[0-9]*\.holidaypirates\.com/)) ||
			domain === "img.yasmina.com" ||
			domain === "resize.abcradio.net.au" ||
			domain === "th.pinkblog.it" ||
			domain === "th.blogosfere.it" ||
			domain === "th.fashionblog.it" ||
			domain === "images.jovempanfm.uol.com.br" ||
			domain === "img.atyabtabkha.com" ||
			domain === "imageresizer.static9.net.au" ||
			domain === "img-ha.mthcdn.com" ||
			domain === "imagenes.milenio.com" ||
			domain_nowww === "dyn.media.titanbooks.com" ||
			domain === "i.osvigaristas.net.br" ||
			domain === "dynamic.zacdn.com" ||
			domain === "svirtus.cdnvideo.ru" ||
			domain === "t.jwwb.nl" ||
			domain === "thumbs.videogamer.com" ||
			domain === "th.gossipblog.it" ||
			domain === "th.cineblog.it" ||
			domain === "images.news18.com" ||
			domain === "static.telugu.news18.com" ||
			domain === "th.clickblog.it" ||
			domain === "thumbor.forbes.com" ||
			domain === "thumbor.f3.cool" ||
			domain === "dw8stlw9qt0iz.cloudfront.net" ||
			domain === "img.nrj.fr" ||
			domain === "img.wynk.in" ||
			domain === "images.adrise.tv" ||
			domain_nowww === "nydailynews.com" ||
			src.match(/:\/\/[^/]*\/thumbor\/[^/]*=\//) ||
			src.match(/:\/\/[^/]*\/resizer\/[^/]*=\/(?:fit-in\/+)?[0-9]+x[0-9]+(?::[^/]*\/[0-9]+x[0-9]+)?\/(?:filters:[^/]*\/)?/)) {
			newsrc = src.replace(/.*\/(?:thumb(?:or)?|(?:new-)?resizer)\/.*?\/(?:filters:[^/]*\/)?([a-z]*:\/\/.*)/, "$1");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/.*\/(?:thumb(?:or)?|(?:new-)?resizer)\/.*?\/(?:filters:[^/]*\/)?([^%/]*\..*)/, "http://$1");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/[-_A-Za-z0-9]{64}\/([^%/]*\.[^/]*\/)/, "http://$1");
			if (newsrc !== src)
				return newsrc;

			newsrc = src.replace(/^[a-z]+:\/\/[^/]+\/+thumbor\/+[0-9]+x[0-9]+\/+http/, "http");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);

			newsrc = src.replace(/.*\/(?:[-_A-Za-z0-9]+=|unsafe)\/(?:(?:full-)?fit-in\/)?(?:[0-9x:]+\/)?(?:[0-9x:]+\/)?(?:(?:smart|top|center|middle)\/)?(?:(?:smart|top|center|middle)\/)?(?:filters:[^/]*\/)?((?:https?(?::\/\/|%3[aA]%2[fF]%2[fF]))?[^/%]*\..*)/, "$1");
			if (newsrc.match(/^[^/]*%2/))
				newsrc = decodeURIComponent(newsrc);

			if (string_indexof(newsrc, "http") !== 0) {
				newsrc = "http://" + newsrc;
			}

			if (newsrc.match(/^[a-z]*%3/) && false)
				newsrc = decodeURIComponent(newsrc);
			return newsrc;
		}

		if (src.match(/:\/\/[^/]*\/astronaut\/uploads\/[a-z]_[^/]*$/)) {
			return src.replace(/\/[a-z]_([^/]*)$/, "/$1");
		}

		if (src.match(/\/spree\/images\/attachments\/[0-9]+\/[0-9]+\/[0-9]+\//) ||
			domain === "d3on60wtl1ot7i.cloudfront.net") {
			return src.replace(/(\/[0-9]+\/[0-9]+\/[0-9]+\/)[^/]*\/([^/?]*)[^/]*?$/, "$1original/$2");
		}

		if (src.match(/\/applications\/core\/interface\/imageproxy\/imageproxy\.php/)) {
			return decodeURIComponent(src.replace(/.*\/imageproxy\/imageproxy\.php.*?[&?]img=([^&]*).*?$/, "$1"));
		}

		if (src.match(/\/dims[0-9]*\/.*?\/(?:(?:(?:thumbnail|resize)\/[0-9>%A-F]+[xX][0-9>%A-F]+[^/]*\/)|(?:crop\/[0-9]+[xX][0-9]+)).*?(?:\/https?:\/\/|\?url=https?%3A)/)) {
			newsrc = src.replace(/.*\/(?:thumbnail|crop|resize)\/.*?\/(https?:\/\/.*)/, "$1");
			if (newsrc !== src)
				return newsrc;
			newsrc = src.replace(/.*\/(?:thumbnail|crop|resize)\/.*?\/\?url=(https?.*)/, "$1");
			if (newsrc !== src)
				return decodeURIComponent(newsrc);
		}

		if (src.match(/\/dims\/CSFF\/[0-9]+\/[-0-9]+\/[-0-9]+\/[-0-9]+\/https?:\/\//)) {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/dims\/CSFF\/[0-9]+\/[-0-9]+\/[-0-9]+\/[-0-9]+\/(https?:\/\/)/, "$1");
			if (newsrc !== src)
				return newsrc;
		}

		if (src.match(/\/media\/k2\/items\/cache\/[^/]*_[^/]*\.[^/.]*$/)) {
			return src.replace(/\/cache\/([^/]*)_[^/._]*?(\.[^/.]*)$/, "/src/$1$2");
		}

		if (src.match(/^[a-z]+:\/\/[^/]*\/index\.php\?.*=com_joomgallery/) ||
			src.match(/^[a-z]+:\/\/[^/]*\/component\/+joomgallery\/+image\./)) {


			if (src.match(/:\/\/[^/]*\/index\.php\?/)) {
				if (url.searchParams.get("option") === "com_joomgallery" &&
					url.searchParams.get("view") === "image" &&
					url.searchParams.get("id")) {
					return src.replace(/(:\/\/[^/]*\/).*/, "$1component/joomgallery/image.raw?view=image&type=orig&format=raw&option=com_joomgallery&id=" + url.searchParams.get("id"));
				}
			}

			return src.replace(/\/component\/joomgallery\/image\..*?[?&]id=([0-9]+).*?$/, "/component/joomgallery/image.raw?view=image&type=orig&format=raw&option=com_joomgallery&id=$1");
		}

		if (src.match(/:\/\/[^/]*\/+(?:(?:vb|forum)\/+)?proxy\.php\?(?:.*?&)?image=http/)) {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+(?:[a-z]+\/+)?proxy\.php\?(?:.*?&)?image=(http[^&]*).*?$/, "$1")
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (src.match(/\/media\/photologue\/photos\/cache\//)) {
			return src.replace(/\/cache\/([^/]*)_[a-z]+(\.[^/.]*)$/, "/$1$2");
		}

		if (src.match(/\/\.evocache\/([^/]*\.[^/]*)\/fit-[0-9]+x[0-9]+\.[^/.]*$/)) {
			return src.replace(/\/\.evocache\/([^/]*\.[^/]*)\/[^/]*$/, "/$1");
		}

		if (src.match(/:\/\/[^/]*\/yahoo_site_admin[0-9]*\/assets\/images\//)) {
			return src.replace(/(\.[0-9]+)_[a-z]+(\.[^/.]*)$/, "$1$2");
		}

		if (domain_nowww === "dvdprime.com" ||
			domain_nowww === "mania.kr" ||
			src.match(/\/data\/+cheditor[0-9]*\/+[0-9]+\/+view_thumbnail\/+[^/]+(?:[?#].*)?$/)) {
			return src.replace(/\/view_thumbnail\/([^/]*)$/, "/$1");
		}

		if (src.match(/\/images\/easyblog_articles\/[0-9]+\/b2ap3_[^/]*$/)) {
			return src.replace(/\/b2ap3_[a-z]+_([^/]*)$/, "/$1");
		}

		if (src.match(/\/wp-content\/plugins\/BdSGallery\//)) {
			return src.replace(/(\/BdSGallery\/BdSGaleria\/[0-9]+)_[a-z]+(\.[^/.]*)$/, "$1$2");
		}

		if ((((domain_nosub === "fbcdn.net" && domain.match(/^external[-.].*\.fbcdn\.net/)) ||
			  domain === "img.globuya.com" ||
			  domain === "img.gluseum.com" ||
			  domain === "img.govserv.org" ||
			  domain === "img.gleauty.com") || (
				  src.match(/^[a-z]+:\/\/[^/]*\/2\/safe_image\.php.*?[?&]url=http/)
			  )) && string_indexof(src, "/safe_image.php?") >= 0) {
			newsrc = src.replace(/.*\/safe_image\.php\?(?:.*&)?url=([^&]+).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (src.match(/^[a-z]+:\/\/[^/]*\/product_photos\/[0-9]+\/[^/.]*_[^/.]*\.[^/.]*$/)) {
			return src.replace(/(\/product_photos\/[0-9]+\/[^/]*)_[^/._]*(\.[^/.]*)$/, "$1_original$2");
		}

		if (domain_nowww === "beckygworld.com" ||
			domain_nowww === "mariahsworld.com" ||
			domain_nowww === "nferaclub.com" ||
			domain_nowww === "tdjakes.com" ||
			src.match(/^[a-z]+:\/\/[^/]*\/public\/img\/posts\/photos\/[0-9]+_[0-9a-f]+_[a-z]\.[^/.]*$/)) {
			return src.replace(/(\/img\/posts\/photos\/[0-9]+_[0-9a-f]+)_[a-z](\.[^/.]*)$/, "$1$2");
		}

		if (src.match(/\/wp-content\/+(?:uploads\/(?:.*?\/)?nggallery|[gG]allery)\/(?:.*?\/)?(?:cache|dynamic)\/+[^/]+\.[^-_/.]*-nggid[0-9]+-ngg0dyn-[^/]*$/)) {
			return src.replace(/\/(?:cache|dynamic)(\/+[^/]+\.[^-_/.]+)-[^/]*$/, "$1");
		}

		if (src.match(/\/+wp-content\/+uploads\/+cache\/+[0-9]{4}\/+[0-9]{2}\/+[^/]*\/+[0-9]+\.[^/.]*(?:[?#].*)?$/)) {
			return src.replace(/\/+wp-content\/+uploads\/+cache\/+([0-9]{4}\/+[0-9]{2}\/+[^/]*)\/+[0-9]+(\.[^/.]*)([?#].*)?$/,
							   "/wp-content/uploads/$1$2$3");
		}

		if (domain_nowww === "mywhere.it" ||
			domain_nowww === "radiowebitalia.it" ||
			src.match(/\/+wp-content\/+gallery\/+[^/]*\/+thumbs\/+thumbs_([^/]*\.[^/.]*)(?:[?#].*)?$/)) {
			return src.replace(/(\/+wp-content\/+[^/]*\/+[^/]*\/+)(?:thumbs\/+thumbs_|webview\/+)([^/]*\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (src.match(/\/+wp-content\/+plugins\/+doptg\/+uploads\/+thumbs\/+/)) {
			return src.replace(/\/+wp-content\/+plugins\/+doptg\/+uploads\/+thumbs\/+/, "/wp-content/plugins/doptg/uploads/");
		}

		if (src.match(/\/wp-content\/+plugins\/+phastpress\/+phast\.php\/+/)) {
			newsrc = src.replace(/.*\/wp-content\/+plugins\/+phastpress\/+phast\.php\/+([^/]*)\/.*?$/, "$1");
			if (newsrc !== src) {
				return newsrc.replace(/(-[0-9A-F][0-9A-F])/g, function(x){return decodeURIComponent(x.replace(/^-/, "%"));});
			}
		}

		if (domain_nowww === "krauzer.ru" ||
			domain === "kino.tricolor.tv" ||
			domain_nosub === "prostitutki.today" ||
			domain_nowww === "menslife.com" ||
			domain_nowww === "sncmedia.ru" ||
			domain_nowww === "fashiontime.ru" ||
			domain_nowww === "hi-fi.ru" ||
			domain_nowww === "super.ru" ||
			src.match(/:\/\/[^/]*\/upload\/+resize_cache\/+(?:[^/]*\/+)?iblock\/+[0-9a-f]{3}\/+[0-9]+_[0-9]+_/) ||
			src.match(/:\/\/[^/]*\/(?:upload\/+)?resize_cache_imm\/+iblock\/+[0-9a-f]{3}\/+[0-9a-f]{4}\/+[0-9]+x[0-9]+_Quality[0-9]+_[0-9a-f]+\./)) {
			newsrc = src
				.replace(/\/resize_cache(?:\/+[^/]*)?(\/+[^/]*\/+...\/+)[0-9]+_[0-9]+_[0-9]+\/([0-9a-f]+(?:-[0-9]+x[0-9]+)?\.[^/.]*)$/, "$1$2")
				.replace(/\/resize_cache(?:\/+[^/]*)?(\/+[^/]*\/+...\/+)[0-9]+_[0-9]+_[0-9a-f]+\/([^\.]+(?:-[0-9]+x[0-9]+)?\.[^/.]*)$/, "$1$2")
				.replace(/\/resize_cache_imm\/+(iblock\/+[0-9a-f]{3}\/+)[0-9a-f]{4}\/+[0-9]+x[0-9]+_Quality[0-9]+_([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/, "/$1$2");
			if (newsrc !== src)
				return newsrc;

			newsrc = src
				.replace(/(\/iblock\/+[0-9a-f]{3}\/+[0-9a-f]+)(?:_[a-z][0-9]+){1,}(?:_[a-z]+)(\.[^/.]*)(?:[?#].*)?$/, "$1$2")
				.replace(/(\/iblock\/+[0-9a-f]{3}\/+[0-9a-f]+)[wh][0-9]+(\.[^/.]*)(?:[?#].*)?$/, "$1$2");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "storage.journaldemontreal.com" ||
			domain === "storage.torontosun.com" ||
			domain === "storage.ottawasun.com" ||
			domain === "storage.chathamthisweek.com" ||
			domain === "storage.journaldequebec.com" ||
			domain === "storage.journaldemontreal.com" ||
			domain === "storage.canoe.com" ||
			domain === "storage-cube.quebecormedia.com" ||
			domain_nowww === "stylosophy.it" ||
			domain_nosub === "nanopress.it" ||
			src.match(/^[a-z]+:\/\/storage\.[^/]*\/+v[0-9]*\/+dynamic_resize\/+sws_path\//)) {
			newsrc = src.replace(/^[a-z]+:\/\/[^/]*\/+v[0-9]*\/+dynamic_resize\/*.*?[?&]src=([^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeURIComponent(newsrc);

			newsrc = src
				.replace(/\/dynamic_resize\/+sws_path\//, "/")
				.replace(/_(?:JDX-[0-9]+x[0-9]+_[A-Z]+|LARGE_BOX)(\.[^/.]*)$/, "_ORIGINAL$1")
				.replace(/\?.*$/, "");
			if (newsrc !== src)
				return {
					url: newsrc,
					head_wrong_contenttype: true
				};

			return {
				url: src,
				head_wrong_contenttype: true
			};
		}

		if (domain === "upload.wikimedia.org" ||
			domain_nowww === "generasia.com" ||
			domain === "cdn.wikimg.net" ||
			domain_nowww === "liquipedia.net" ||
			domain === "i.know.cf" ||
			src.match(/\/thumb\/+[0-9a-f]\/+[0-9a-f]{2}\/+[^/]*\.[^/]*\/+[0-9]+px-[^/]*(?:[?#].*)?$/)) {
			newsrc = src.replace(/\/(?:thumb\/+(archive\/+)?)?(.)\/+(..)\/+([^/]*)\/+[0-9]+px-.*?$/, "/$1$2/$3/$4");
			if (newsrc !== src)
				return newsrc;
		}

		if (domain === "upload.wikimedia.org") {
			match = src.match(/\/commons\/+[0-9a-f]\/+[0-9a-f]{2}\/+([^/]+\.[^/.]+)(?:[?#].*)?$/);
			if (match) {
				return {
					url: src,
					is_original: true,
					extra: {
						page: "https://commons.wikimedia.org/wiki/File:" + match[1]
					}
				};
			}
		}

		if (domain_nowww === "wikitoes.com" ||
			src.match(/\/lib\/+exe\/+fetch\.php.*?[?&]media=[^&]*.*?$/)) {
			return src.replace(/\/lib\/+exe\/+fetch\.php.*?[?&](media=[^&]*).*?$/,
							   "/lib/exe/fetch.php?$1");
		}

		if (domain === "cdn.himalaya.com" ||
			domain === "i.linkeddb.com" ||
			domain === "img.heypik.com" ||
			domain === "club-img.kdslife.com" ||
			(domain_nosub === "ofashion.com.cn" && domain.match(/^img[0-9]*\.ofashion.com.cn/)) ||
			domain === "kanfaimage.oss-cn-beijing.aliyuncs.com" ||
			(domain_nosub === "mioreport.com" && domain.match(/^s[0-9]*\./)) ||
			(domain_nosub === "aliyuncs.com" && domain.match(/[./]oss-cn-/)) ||
			domain === "rmrbcmsonline.peopleapp.com" ||
			domain === "img.fxe0898.com" ||
			domain === "img.xiumi.us" ||
			domain === "s.oneroof.co.nz" ||
			(domain_nosub === "hoopchina.com.cn" && domain.match(/[ci][0-9]*\.hoopchina\.com\.cn/)) ||
			src.match(/[?&]x-oss-process=(?:image|style)\//)) {
			var authkey = src.replace(/.*[?&]auth_key=([-0-9a-f]+).*?$/, "$1");
			var replacement = "";
			if (authkey !== src)
				replacement = "?auth_key=" + authkey;

			if (src.match(/[?&]x-oss-process=/)) {
				newsrc = src.replace(/[?#].*/, replacement);
				if (newsrc !== src)
					return newsrc;
			}
		}

		if (domain === "tfk.thefreekick.com" ||
			domain_nowww === "discourse-cdn-sjc1.com" ||
			domain_nowww === "swagup.co.kr" ||
			amazon_container === "pixls-discuss" ||
			domain === "forums.lutris.net" ||
			domain === "forum.combustionpunks.co.uk" ||
			domain_nowww === "discourse-cdn-aws1.com" ||
			domain_nosub === "discourse-cdn.com" ||
			src.match(/^[a-z]+:\/\/[^/]*\/(?:(?:forum\/+)?uploads\/+[a-z]+\/+)?(?:optimized|original)\/+[0-9]X\/+(?:[0-9a-f]\/+)*[0-9a-f]{10,}_[0-9]+_[0-9]+x[0-9]+\.[^/.]*(?:[?#].*)?$/)) {
			return src.replace(/(:\/\/[^/]*\/|\/uploads\/[a-z]+\/)?optimized(\/.*\/[0-9a-f]+)_[0-9]+_[0-9]+x[0-9]+(\.[^/.]*)$/,
							   "$1original$2$3");
		}

		if (domain_nowww === "myproana.com" ||
			domain === "static.nulled.to" ||
			domain_nowww === "tesall.ru" ||
			src.match(/^[a-z]+:\/\/[^/]+\/+uploads\/+profile\/+photo-thumb-[0-9]+\.[^/.]*(?:[?#].*)?$/) ||
			src.match(/^[a-z]+:\/\/[^/]+\/+uploads\/+monthly_[0-9]{2}_[0-9]{4}\/+post-[0-9]+-[0-9]+(?:-[0-9]+-[0-9]+)?(?:_thumb)\.[^/]*(?:[?#].*)?/) ||
			src.match(/^[a-z]+:\/\/[^/]+\/+uploads\/+gallery\/+(?:album|category)_[0-9]+\/+[a-z]+_(?:gallery_[0-9]+_[0-9]+_[0-9]+)\.[^/.]*(?:[?#].*)?$/)) {
			return src
				.replace(/(\/uploads\/+gallery\/+(?:album|category)_[0-9]+\/)[a-z]+_(gallery_[0-9]+[^/]*\.[^/.]*)$/, "$1$2")
				.replace(/(\/uploads\/+profile\/+photo-)thumb-([0-9]+\.[^/.]*)$/, "$1$2")
				.replace(/(\/uploads\/+monthly_[0-9]{2}_[0-9]{4}\/+post-[0-9]+-[0-9]+(?:-[0-9]+-[0-9]+)?)_thumb(\.[^/.]*)(?:[?#].*)?$/,
							   "$1$2");
		}

		if (domain === "files.mastodon.social" ||
			domain === "cf.mastohost.com" ||
			src.match(/\/media_attachments\/+files\/+(?:[0-9]{3}\/+){3}[a-z]+\/+[0-9a-f]{16}\.[^/.]*(?:[?#].*)?$/)) {
			return src.replace(/\/[a-z]+\/([0-9a-f]+\.[^/.]*)(?:[?#].*)?$/, "/original/$1");
		}

		if (domain_nowww === "diasp.org" ||
			domain_nowww === "diasp.eu" ||
			/^[a-z]+:\/\/[^/]+\/+uploads\/+images\/+(?:thumb|scaled)_(?:large|small|full)_[0-9a-f]{20}\.[^/.]+(?:[?#].*)?$/.test(src)) {
			return src.replace(/\/uploads\/+images\/+[a-z_]+_([0-9a-f]{10,}\.[^/.]*)(?:[?#].*)?$/, "/uploads/images/$1");
		}

		if (/\/wp-content\/+themes\/+[^/]+\/+functions\/+resize_ext\.php\?(?:.*&)?image=http/.test(src)) {
			newsrc = src.replace(/.*\/wp-content\/+themes\/+[^/]+\/+functions\/+resize_ext\.php\?(?:.*&)?image=(http[^&]*).*?$/, "$1");
			if (newsrc !== src)
				return decodeuri_ifneeded(newsrc);
		}

		if (domain_nowww === "12andus.com" ||
			domain === "vln.school.nz" ||
			domain_nowww === "ict-21.ch" ||
			/\/mod\/+file\/+thumbnail\.php\?(?:.*&)?file_guid=[0-9]+(?:&.*)?$/.test(src)) {
			return src.replace(/(\/mod\/+file\/+thumbnail\.php\?)(?:.*&)?(file_guid=[0-9]+).*$/, "$1$2&size=original");
		}

		if (/\/monthly_[0-9]{4}_[0-9]{2}\/[^/]+\./.test(src) && options && options.element) {
			if (options.element.tagName === "SPAN" && options.element.hasAttribute("data-ipslightbox") && options.element.getAttribute("data-fullurl")) {
				return options.element.getAttribute("data-fullurl");
			}

			var get_gallery_image = function(url, cb) {
				var cache_key = "invision_gallery_image:" + url.replace(/^[a-z]+:\/\//, "://").replace(/[?#].*/, "");
				api_cache.fetch(cache_key, cb, function(done) {
					options.do_request({
						url: url,
						method: "GET",
						onload: function(resp) {
							if (resp.readyState < 4)
								return;

							if (resp.status !== 200)
								return done(null, false);

							var match = resp.responseText.match(/<meta\s+property=["']og:image["']\s+content=["']([^'"]+)["']/);
							if (match) {
								return done(decode_entities(match[1], 12*60*60));
							} else {
								console_warn("Unable to find match", resp);
								return done(null, false);
							}
						}
					});
				});
			};

			if ((options.element.tagName === "IMG" || options.element.tagName === "LI") && options.do_request && options.cb) {
				var ael = options.element.parentElement;
				if (ael.tagName !== "A" && options.element.children.length === 1) {
					ael = options.element.children[0];
				}

				if (ael.tagName === "A" && ael.hasAttribute("data-imagelightbox")) {
					get_gallery_image(ael.href, function(url) {
						if (url) {
							options.cb(url);
						} else {
							options.cb(null);
						}
					});

					return {
						waiting: true
					};
				}
			}
		}

		if (domain === "lakeimagesweb.artic.edu" ||
			(domain_nosub === "oclc.org" && domain.match(/\.contentdm\./)) ||
			domain === "stor.artstor.org" ||
			domain === "ids.lib.harvard.edu" ||
			domain === "digitalcollections.lib.washington.edu" ||
			domain_nowww === "mtmemory.org" ||
			domain === "ids.si.edu" ||
			domain === "stacks.stanford.edu" ||
			domain === "images.britishart.yale.edu" ||
			domain === "iiif.archivelab.org" ||
			domain === "cplorg.cdmhost.com" ||
			domain === "iiif.lib.ncsu.edu" ||
			domain === "iiif.eluxemburgensia.lu" ||
			domain === "archives.norwich.edu" ||
			domain === "iiif.library.ubc.ca" ||
			domain === "iiif.library.utoronto.ca" ||
			domain === "iiif.bodleian.ox.ac.uk" ||
			domain === "iiif.lib.utexas.edu" ||
			domain === "digital.library.unt.edu" ||
			domain === "texashistory.unt.edu" ||
			domain === "media.ng-london.org.uk" ||
			domain === "iiif.ucd.ie" ||
			(domain_nosub === "unifr.ch" && string_indexof(src, "/loris/") >= 0) ||
			domain === "iiif.irht.cnrs.fr" ||
			domain === "gallica.bnf.fr" ||
			/\/(?:iiif|loris)\/+(?:.*\/)?[^/]+\/+(?:full|square|(?:pct:)?[0-9.]+(?:,[0-9.]+){3})\/+(?:full|max|[0-9.]+,|,[0-9.]+|!?[0-9.]+,[0-9.]+|pct:[0-9.]+)\/+!?[0-9.]+\/+(?:color|gray|bitonal|default|native)\.(?:jpg|tif|png|gif|jp2|pdf|webp)(?:[?#].*)?$/.test(src)) {

			obj = {
				url: src
			};

			newsrc = src.replace(/(\/ark:\/+[0-9]+\/+[^/]+(?:\/+f[0-9])?)\.(?:thumbnail|lowres)(?:[?#].*)?$/, "$1.highres");
			if (newsrc !== src)
				return newsrc;

			var prefix = "(?:iiif|loris)";
			if (domain === "iiif.library.ubc.ca" || domain === "iiif.library.utoronto.ca") {
				obj.head_wrong_contenttype = true; // text/html
				prefix = "image";
			}

			regex = new RegExp("(/" + prefix + "/.*?/)[^/]+/[^/]+/[^/]+/([^/]+\\.[^/.]*)$");
			newsrc = src.replace(regex, "$1full/full/0/$2");
			if (newsrc !== src)
				return fillobj_urls([newsrc], obj);
		}



















		if (domain === "blogs-images.forbes.com" ||
			domain === "images-origin.playboy.com" ||
			domain === "images.kpopstarz.com" ||
			domain === "etimg.etb2bimg.com" ||
			domain_nosub === "thetimes.co.uk" ||
			domain === "mediaresources.idiva.com" ||
			domain === "telugu.samayam.com" ||
			(domain_nosub === "cpcache.com" && domain.match(/^i[0-9]*\./)) ||
			domain_nosub === "filmibeat.com" ||
			domain === "www.cdn.tv2.no" ||
			domain === "photo.voici.fr" ||
			domain_nowww === "essexstudent.com" ||
			domain === "th.thgim.com" ||
			domain_nosub === "timesofindia.com" ||
			domain === "static.langimg.com" ||
			domain_nosub === "indiatimes.com" ||
			domain_nosub === "thehindu.com" ||
			domain === "images.contentful.com") {
			return {
				url: src,
				head_wrong_contentlength: true
			};
		}

		if (domain === "cdn.marketplaceimages.windowsphone.com") {
			return {
				url: src,
				head_wrong_contentlength: true,
				head_wrong_contenttype: true
			};
		}

		if ((domain_nosub === "appspot.com" && domain.match(/^wixmp-[0-9a-f]+\./)) ||
			(domain_nosub === "wixmp.com" && domain.match(/^api-da(?:.*)?\./) && string_indexof(src, "/download/file") >= 0)) {
			return {
				url: src,
				head_wrong_contentlength: true,
				head_wrong_contenttype: true,
				forces_download: true,
				is_private: true
			};
		}

		if (domain_nowww === "ozanyerli.com" ||
			domain === "gallica.bnf.fr" ||
			domain === "dieta.pourfemme.it" ||
			domain_nowww === "ellahoy.es" ||
			domain_nowww === "allgirlannihilation.net" ||
			domain === "cdn.akb48.co.jp") {
			return {
				url: src,
				head_wrong_contenttype: true
			};
		}

		if ((domain === "images.thestar.com" && string_indexof(src, "/content/dam/") >= 0) ||
			(domain_nowww === "thestar.com" && string_indexof(src, "/content/dam/") >= 0) ||
			(domain_nosub === "akamaihd.net" && domain.match(/^steamuserimages-[a-z]\./)) ||
			domain_nowww === "kansascity.com" ||
			domain_nowww === "sanluisobispo.com" ||
			domain_nowww === "iichan.hk" || // 403
			domain_nowww === "pdfkul.com" ||
			domain_nosub === "strikinglycdn.com" || // 403
			domain === "images.boosty.to" || // 400
			(domain_nosub === "irishmirror.ie" && domain.match(/i[0-9]*(?:-prod)?\./))) {
			return {
				url: src,
				can_head: false
			};
		}

		if ((domain_nosub === "radikal.ru" && domain.match(/^s[0-9]*\./)) ||
			domain_nowww === "atkmodels.com" ||
			domain_nowww === "akibatan.com" ||
			domain_nowww === "n3rdabl3.com") {
			return {
				url: src,
				headers: {
					Referer: ""
				}
			};
		}

		if (domain_nowww === "image.fomos.kr" ||
			domain_nowww === "onlifezone.com" ||
			domain === "write.dcinside.com") {
			return {
				url: src,
				can_head: false,
				headers: {
					Referer: ""
				}
			};
		}

		if (domain_nowww === "1in.am" ||
			(domain_nosub === "bobaedream.co.kr" && /^file[-0-9]+\./.test(domain)) ||
			domain_nowww === "dogdrip.net") {
			return {
				url: src,
				headers: {
					Referer: src
				}
			};
		}











		if (options.null_if_no_change) {
			if (src !== origsrc)
				return src;
			return null;
		}

		return src;
	}
	// -- end bigimage --

	function get_helpers(options) {
		var host_domain = "";
		var host_domain_nowww = "";
		var host_domain_nosub = "";
		if (options.host_url) {
			host_domain = options.host_url.replace(/^[a-z]+:\/\/([^/]*)(?:\/.*)?$/,"$1");

			host_domain_nowww = host_domain.replace(/^www\./, "");
			host_domain_nosub = host_domain.replace(/^.*\.([^.]*\.[^.]*)$/, "$1");
			if (host_domain_nosub.match(/^co\.[a-z]{2}$/)) {
				host_domain_nosub = host_domain.replace(/^.*\.([^.]*\.[^.]*\.[^.]*)$/, "$1");
			}
		}

		try {
			if (options.document)
				document = options.document;
			if (options.window)
				window = options.window;
		} catch (e) {
			//console_warn("Failed to set document/window", e);
		}


		var new_image = function(src) {
			var img = document.createElement("img");
			img.src = src;
			return img;
		};

		var new_video = function(src) {
			var video = document.createElement("video");
			video.src = src;
			return video;
		};

		var new_media = function(src, is_video) {
			if (is_video)
				return new_video(src);
			else
				return new_image(src);
		};


		if (host_domain_nosub === "imgur.com" && host_domain !== "i.imgur.com") {
			return {
				gallery: function(el, nextprev) {
					if (!el)
						return "default";

					var find_from_el = function() {
						if (options.host_url.match(/\/a\/+[^/]+\/+embed/)) {
							return "default";
						}

						var current = el;
						while ((current = current.parentElement)) {
							if (current.tagName === "DIV" &&
								current.classList && current.classList.contains("post-image-container")) {
								while (current) {
									var next = current.nextElementSibling;
									if (!nextprev)
										next = current.previousElementSibling;

									current = next;
									if (!current)
										return "default";

									var img = current.querySelector("img");
									if (img) {
										return img;
									}
								}
							}
						}

						return "default";
					};

					// "default" isn't used here because it's only used as find_from_api ||
					var find_from_api = function(images) {
						var big = bigimage_recursive(el.src, {
							fill_object: true,
							use_cache: false,
							rule_specific: {imgur_source: false}
						});

						var current_hash = big[0].url.replace(/.*\/([^/._]+?)(?:_[^/]+)?\.[^/.]*?(?:[?#].*)?$/, "$1");
						if (current_hash !== el.src) {
							for (var i = 0; i < images.length; i++) {
								if (images[i].hash === current_hash) {
									var image_id;
									if (nextprev) {
										if (i + 1 >= images.length)
											return null;

										image_id = i + 1;
									} else {
										if (i - 1 < 0)
											return null;

										image_id = i - 1;
									}

									var ext = images[image_id].ext;
									if (options.exclude_videos && (ext === ".mp4" || ext === ".webm"))
										ext = ".jpg";

									var newel = document.createElement("img");
									newel.src = "https://i.imgur.com/" + images[image_id].hash + ext;
									return newel;
								}
							}
						}

						return null;
					};

					var find_from_both = function(images) {
						return find_from_api(images) || find_from_el();
					};

					if (!window.runSlots && options.do_request) {
						common_functions.fetch_imgur_webpage(options.do_request, real_api_cache, undefined, options.host_url, function(data) {
							if (!data || !data.imageinfo || data.bad) {
								return options.cb(find_from_el());
							}

							try {
								return options.cb(find_from_both(data.imageinfo.album_images.images));
							} catch (e) {
								console_error(e);
								return options.cb(find_from_el());
							}
						});

						return "waiting";
					} else {
						try {
							return find_from_both(window.runSlots.item.album_images.images);
						} catch (e) {
							console_error(e);
							return find_from_el();
						}
					}
				}
			};
		}

		if (host_domain_nosub === "instagram.com") {
			return {
				gallery: function(el, nextprev) {
					if (!el)
						return "default";

					if (!options.do_request)
						return "default";

					var query_el = el;
					if (!el.parentElement) { // check if it's a fake element returned by this function
						query_el = options.element;
					}

					var info = common_functions.instagram_find_el_info(document, query_el, options.host_url);
					var can_apply = false;
					var use_default_after = false;
					for (var i = 0; i < info.length; i++) {
						if ((info[i].type === "post" && (info[i].subtype === "popup" || info[i].subtype === "page" || info[i].subtype === "home" || (info[i].subtype === "link" && options.rule_specific.instagram_gallery_postlink && !options.is_counting))) ||
						     info[i].type === "story") {
							info[i].all = true;
							can_apply = true;

							if (info[i].type === "post" && info[i].subtype === "link")
								use_default_after = true;
						}
					}

					if (can_apply) {
						var imageid_el = common_functions.instagram_get_el_for_imageid(el);
						var imageid_el_src = common_functions.instagram_get_image_src_from_el(imageid_el);
						var our_imageid = common_functions.instagram_get_imageid(imageid_el_src);
						var add = nextprev ? 1 : -1;
						common_functions.instagram_parse_el_info(real_api_cache, options.do_request, options.rule_specific.instagram_use_app_api, info, function(data) {
							for (var i = nextprev ? 0 : 1; i < data.length - (nextprev ? 1 : 0); i++) {
								var current_imageid = common_functions.instagram_get_imageid(data[i].src);
								var current_videoid = "null";
								if (data[i].video) {
									current_videoid = common_functions.instagram_get_imageid(data[i].video);
								}

								if (our_imageid === current_imageid || our_imageid === current_videoid) {
									return options.cb(new_media(data[i + add].video || data[i + add].src, data[i + add].video));
								}
							}

							if (!use_default_after)
								return options.cb(null);
							else
								return options.cb(get_next_in_gallery(options.element, nextprev));
						});

						return "waiting";
					}

					return "default";
				}
			};
		}

		if (host_domain_nosub === "tiktok.com") {
			return {
				gallery: function(el, nextprev) {
					if (el.tagName === "VIDEO" && el.parentElement && el.parentElement.parentElement) {
						if (el.parentElement.classList.contains("video-card") &&
							el.parentElement.parentElement.classList.contains("image-card")) {
							return get_next_in_gallery(el.parentElement.parentElement, nextprev);
						}
					}

					return "default";
				}
			};
		}

		if (host_domain_nowww === "twitter.com") {
			return {
				gallery: function(el, nextprev) {
					var is_photo_a = function(el) {
						return el.tagName === "A" && el.href && /\/status\/+[0-9]+\/+photo\/+/.test(el.href);
					};

					var get_img_from_photo_a = function(el) {
						var imgel = el.querySelector("img");
						if (imgel) {
							// don't return the <img> element because opacity: 0
							var prev = imgel.previousElementSibling;
							if (prev && prev.tagName === "DIV" && prev.style.backgroundImage)
								return prev;
						}

						return imgel;
					}

					var get_nextprev = function(el) {
						if (nextprev) {
							return el.nextElementSibling;
						} else {
							return el.previousElementSibling;
						}
					};

					var get_photoel_from_photo_container = function(nextel) {
						if (nextel.tagName === "A") {
							return get_img_from_photo_a(nextel);
						} else if (nextel.tagName === "DIV") {
							var childid = nextprev ? 0 : (nextel.children.length - 1);

							if (nextel.children.length > 0 && is_photo_a(nextel.children[childid])) {
								return get_img_from_photo_a(nextel.children[childid]);
							}
						} else {
							return "default";
						}
					};

					// tweet albums: https://twitter.com/phoronix/status/1229117085432926209
					var current = el;
					while ((current = current.parentElement)) {
						if (is_photo_a(current)) {
							var nextel = get_nextprev(current);

							if (nextel) {
								return get_photoel_from_photo_container(nextel);
							} else {
								var parent = current.parentElement;
								var sibling = get_nextprev(parent);

								if (sibling) {
									return get_photoel_from_photo_container(sibling);
								}
							}

							return null;
						}
					}

					return "default";
				},
				element_ok: function(el) {
					var tweet = common_functions.get_twitter_video_tweet(el, window);
					// disable for now as this method needs to be implemented
					if (tweet && false) {
						return true;
					}

					return "default";
				}
			};
		}

		if (host_domain_nowww === "500px.com") {
			return {
				element_ok: function(el) {
					if (el.tagName === "A" && el.classList.contains("photo_link") && el.querySelector("div.nsfw_placeholder_content")) {
						return true;
					}

					return "default";
				}
			};
		}

		if (host_domain_nosub === "snapchat.com") {
			return {
				gallery: function(el, nextprev) {
					if (el.tagName !== "VIDEO" && el.tagName !== "IMG")
						return "default";

					var info = common_functions.get_snapchat_info_from_el(el);
					if (info) {
						common_functions.get_snapchat_story(real_api_cache, options.do_request, info.username, function(data) {
							if (!data) {
								return options.cb(null);
							}

							if (!("pos" in info)) {
								for (var i = 0; i < data.snapList.length; i++) {
									if (data.snapList[i].snapUrls.mediaUrl === info.url) {
										info.pos = i;
										break;
									}
								}
							}

							if (!("pos" in info)) {
								return options.cb(null);
							}

							var diff = nextprev ? 1 : -1;
							var newpos = info.pos + diff;

							if (newpos < 0 || newpos >= data.snapList.length)
								return options.cb(null);

							var url = data.snapList[newpos].snapUrls.mediaUrl;
							var mediael = new_media(url, /\.mp4(?:[?#].*)?$/.test(url) && false);
							mediael.setAttribute("data-username", info.username);
							mediael.setAttribute("data-pos", newpos);
							mediael.setAttribute("data-url", url);
							mediael.setAttribute("data-imu", "true");

							return options.cb(mediael);
						});

						return "waiting";
					}

					return "default";
				},
				element_ok: function(el) {
					if (el.tagName === "VIDEO") {
						return true;
					}

					return "default";
				}
			};
		}

		if (host_domain_nosub === "allmusic.com") {
			return {
				gallery: function(el, nextprev) {
					// thanks to nimuxoha on github: https://github.com/qsniyg/maxurl/issues/279
					// https://www.allmusic.com/album/release/mr0002329530
					// https://www.allmusic.com/album/release/mr0002781134 -- no gallery
					// https://www.allmusic.com/album/release/mr0000570239
					// https://www.allmusic.com/album/release/mr0001144566 -- no gallery

					if (el.tagName !== "IMG" || !el.parentElement)
						return "default";

					if (!el.classList.contains("gallery-main-image"))
						return "default";

					var carousel_img = el.parentElement.querySelector("#carousel > ul > li.thumb-img > img.highlight-on");
					if (!carousel_img) {
						console_warn("Unable to find carousel thumbnail image");
						return "default";
					}

					var valid_or_null = function(el) {
						if (!el.classList.contains("thumb-img"))
							return null;
						return el.querySelector("img");
					}

					var li = carousel_img.parentElement;
					if (nextprev) {
						return valid_or_null(li.nextElementSibling);
					} else {
						return valid_or_null(li.previousElementSibling);
					}
				}
			};
		}

		if (host_domain_nosub === "reddit.com") {
			return {
				element_ok: function(el) {
					if (el.tagName === "VIDEO") {
						return true;
					}

					return "default";
				}
			};
		}

		return null;
	}

	var get_next_in_gallery = null;

	var fullurl_obj = function(currenturl, obj) {
		if (!obj)
			return obj;

		if (!is_array(obj)) {
			obj = [obj];
		}

		var newobj = [];
		obj.forEach(function (url) {
			if (typeof(url) === "string") {
				newobj.push(fullurl(currenturl, url));
			} else {
				if (url.url) {
					if (is_array(url.url)) {
						for (var i = 0; i < url.url.length; i++) {
							url.url[i] = fullurl(currenturl, url.url[i]);
						}
					} else {
						url.url = fullurl(currenturl, url.url);
					}
				}
				newobj.push(url);
			}
		});

		return newobj;
	};

	var fillobj = function(obj, baseobj) {
		//if (typeof obj === "undefined")
		if (!obj) {
			//return [];
			obj = {};
		}

		if (!is_array(obj)) {
			obj = [obj];
		}

		if (!baseobj)
			baseobj = {};

		if (is_array(baseobj))
			baseobj = baseobj[0];

		for (var i = 0; i < obj.length; i++) {
			if (typeof(obj[i]) === "undefined") {
				continue;
			}

			if (typeof(obj[i]) === "string") {
				obj[i] = {url: obj[i]};
			}

			var item;
			// Only copy from baseobj if the urls are the same (or n/a)
			if (!obj[i].url || !baseobj.url || baseobj.url === obj[i].url)  {
				for (item in baseobj) {
					if (!(item in obj[i])) {
						obj[i][item] = baseobj[item];
					}
				}
			}

			for (item in default_object) {
				if (!(item in obj[i])) {
					obj[i][item] = default_object[item];
				}
			}
		}

		return obj;
	};

	var same_url = function(url, obj) {
		obj = fillobj(obj);

		if (obj[0] && obj[0].url === url)
			return true;

		return false;
	};

	var get_bigimage_extoptions_first = function(options) {
		if (!("allow_thirdparty" in options)) {
			options.allow_thirdparty = (settings["allow_thirdparty"] + "") === "true";
		 }

		 if (!("allow_apicalls" in options)) {
			 options.allow_apicalls = (settings["allow_apicalls"] + "") === "true";
		 }

		 if (!("allow_thirdparty_libs" in options)) {
			options.allow_thirdparty_libs = (settings["allow_thirdparty_libs"] + "") === "true";
		}

		return options;
	};

	var get_bigimage_extoptions = function(options) {
		if ("exclude_problems" in options) {
			for (var option in settings) {
				if (option in option_to_problems) {
					var problem = option_to_problems[option];
					var index = array_indexof(options.exclude_problems, problem);

					if (settings[option]) {
						if (index >= 0)
							options.exclude_problems.splice(index, 1);
					} else {
						if (index < 0)
							options.exclude_problems.push(problem);
					}
				}
			}
		}

		if (!options.allow_apicalls) {
			options.do_request = null;
		}

		if ("rule_specific" in options) {
			options.rule_specific.deviantart_prefer_size = settings.deviantart_prefer_size;
			options.rule_specific.imgur_source = settings.imgur_source;
			options.rule_specific.instagram_use_app_api = settings.instagram_use_app_api;
			options.rule_specific.instagram_gallery_postlink = settings.instagram_gallery_postlink;
			options.rule_specific.tiktok_no_watermarks = settings.tiktok_no_watermarks;
			options.rule_specific.tumblr_api_key = settings.tumblr_api_key;
		}

		// Doing this here breaks things like Imgur, which will redirect to an image if a video was opened in a new tab
		if (false && !settings.allow_video) {
			options.exclude_videos = true;
		} else {
			options.exclude_videos = false;
		}

		return options;
	};

	var bigimage_recursive = function(url, options) {
		if (!url)
			return url;

		if (!options)
			options = {};

		if (is_userscript || is_extension) {
			get_bigimage_extoptions_first(options);
		}

		for (var option in bigimage_recursive.default_options) {
			if (!(option in options)) {
				options[option] = deepcopy(bigimage_recursive.default_options[option]);
				continue;
			}

			if (is_iterable_object(options[option])) {
				for (var rsoption in bigimage_recursive.default_options[option]) {
					if (!(rsoption in options[option])) {
						options[option][rsoption] = deepcopy(bigimage_recursive.default_options[option][rsoption]);
					}
				}
			}
		}

		if (is_userscript || is_extension) {
			get_bigimage_extoptions(options);
		}

		var waiting = false;
		var forcerecurse = false;

		var url_is_data = false;
		var origurl = url;
		if (typeof url === "string" && /^(?:data|blob):/.test(url)) {
			url_is_data = true;
		}

		var newhref = url;
		var endhref;
		var currenthref = url;
		var pasthrefs = [url];
		//var lastobj = fillobj(newhref);
		//var lastobj = newhref;
		var pastobjs = [];
		var currentobj = null;
		var used_cache = false;
		var loop_i = 0;

		var do_cache = function() {
			if (_nir_debug_)
				console_log("do_cache (endhref, currentobj):", deepcopy(endhref), deepcopy(currentobj));

			if (!endhref)
				return;

			if (!get_currenthref(endhref))
				return;

			var cache_endhref = fillobj(endhref, currentobj);
			currenthref = get_currenthref(cache_endhref);
			if (!currenthref)
				return;

			if (!used_cache && (options.use_cache === true) && !waiting) {
				for (var i = 0; i < pasthrefs.length; i++) {
					var href = pasthrefs[i];

					if (href) {
						if (_nir_debug_) {
							console_log("do_cache:", href, "=", deepcopy(cache_endhref));
						}

						url_cache.set(href, deepcopy(cache_endhref), options.urlcache_time);
					}
				}
			}
		};

		var get_currenthref = function(objified) {
			if (!objified) {
				return objified;
			}

			if (is_array(objified)) {
				objified = objified[0];
			}

			if (!objified) {
				return objified;
			}

			if (is_array(objified.url))
				currenthref = objified.url[0];
			else
				currenthref = objified.url;
			return currenthref;
		};

		var parse_bigimage = function(big) {
			if (_nir_debug_) {
				console_log("parse_bigimage (big)", deepcopy(big));
			}

			if (!big) {
				if (newhref === url && options.null_if_no_change)
					newhref = big;
				return false;
			}

			var newhref1 = fullurl_obj(currenthref, big);
			if (_nir_debug_) {
				console_log("parse_bigimage (newhref1)", deepcopy(newhref1));
			}

			if (!newhref1) {
				return false;
			}

			// Copy important old properties
			var important_properties = {};
			if (pastobjs.length > 0) {
				if (pastobjs[0].likely_broken)
					important_properties.likely_broken = pastobjs[0].likely_broken;
				if (pastobjs[0].fake)
					important_properties.fake = pastobjs[0].fake;
			}

			var objified = fillobj(deepcopy(newhref1), important_properties);
			if (_nir_debug_) {
				console_log("parse_bigimage (objified)", deepcopy(objified));
			}

			for (var i = 0; i < objified.length; i++) {
				var obj = objified[i];

				var remove_obj = function() {
					objified.splice(i, 1);
					if (is_array(newhref1)) {
						newhref1.splice(i, 1);
					}

					i--;
				};

				// Remove null URLs
				if (obj.url === null && !obj.waiting) {
					remove_obj();
					continue;
				}

				if (obj.url === "" && url_is_data) {
					obj.url = origurl;
				}

				// Remove problems in exclude_problems
				for (var problem in obj.problems) {
					if (obj.problems[problem] &&
						array_indexof(options.exclude_problems, problem) >= 0) {
						remove_obj();
						continue;
					}
				}

				if (options.exclude_videos && obj.video) {
					remove_obj();
					continue;
				}

				if (obj._copy_old_props && pastobjs[0]) {
					for (var j = 0; j < obj._copy_old_props.length; j++) {
						var prop = obj._copy_old_props[j];
						obj[prop] = deepcopy(pastobjs[0][prop]);
					}
				}
			}

			if (_nir_debug_) {
				console_log("parse_bigimage (objified, processed)", deepcopy(objified));
			}

			if (objified.length === 0) {
				if (_nir_debug_)
					console_log("parse_bigimage: objified.length == 0");

				return false;
			}

			waiting = false;
			forcerecurse = false;
			var temp_newhref1 = newhref1;
			if (is_array(newhref1))
				temp_newhref1 = newhref1[0];
			if (typeof(temp_newhref1) === "object") {
				currentobj = newhref1;
				if (temp_newhref1.waiting) {
					waiting = true;
					if (!temp_newhref1.url) {
						newhref = newhref1;
						return false;
					}
				}

				if (temp_newhref1.forcerecurse) {
					forcerecurse = true;
				}
			} else {
				currentobj = null;
			}

			if (same_url(currenthref, objified) && !forcerecurse) {
				if (_nir_debug_)
					console_log("parse_bigimage: sameurl(currenthref, objified) == true", deepcopy(currenthref), deepcopy(objified));

				// FIXME: this is a terrible hack
				try {
					if (!options.fill_object || (newhref[0].waiting === true && !objified[0].waiting))
						newhref = objified;
				} catch (e) {}

				return false;
			} else {
				if (!forcerecurse) {
					for (var i = 0; i < pasthrefs.length; i++) {
						if (same_url(pasthrefs[i], objified)) {
							if (_nir_debug_)
								console_log("parse_bigimage: sameurl(pasthrefs[" + i + "], objified) == true", deepcopy(pasthrefs[i]), deepcopy(objified));
							return false;
						}
					}
				}

				if (_nir_debug_)
					console_log("parse_bigimage: setting currenthref and newhref");
				currenthref = get_currenthref(objified);
				newhref = newhref1;
			}

			pasthrefs.push(currenthref);

			// Prepend objified to pastobjs
			var current_pastobjs = [];
			for (var i = 0; i < objified.length; i++) {
				current_pastobjs.push(objified[i]);
			}

			for (var i = 0; i < pastobjs.length; i++) {
				current_pastobjs.push(pastobjs[i]);
			}

			pastobjs = current_pastobjs;

			if (false && !waiting) {
				// lastobj isn't used
				lastobj = newhref;
			}

			if (objified[0].norecurse)
				return false;

			if (_nir_debug_ && _nir_debug_.no_recurse) {
				return false;
			}

			return true;
		};

		var do_bigimage = function() {
			if (_nir_debug_)
				console_log("do_bigimage", currenthref, deepcopy(options));

			if (options.use_cache && url_cache.has(currenthref) && !forcerecurse) {
				if (_nir_debug_) {
					console_log("do_bigimage: newhref = url_cache[" + currenthref + "]", deepcopy(url_cache.get(currenthref)));
				}

				newhref = url_cache.get(currenthref);
				used_cache = true;
				return false;
			}

			if (options.filter) {
				if (!options.filter(currenthref)) {
					console_log("Blacklisted: " + currenthref);
					return false;
				}
			}

			var big;

			if (options.catch_errors) {
				try {
					big = bigimage(currenthref, options);
				} catch(e) {
					console_error(e);
					console_error(e.stack);
				}
			} else {
				big = bigimage(currenthref, options);
			}

			return parse_bigimage(big);
		};

		var finalize = function() {
			if (options.fill_object) {
				if (_nir_debug_)
					console_log("finalize (fillobj(newhref, currentobj))", deepcopy(newhref), deepcopy(currentobj));

				if (used_cache && newhref === null) {
					endhref = deepcopy(currentobj);
				} else {
					// the reason for using fillobj(..., currentobj) is for objects that add to the last object (e.g. {url: src, head_wrong_contentlength: true})
					endhref = fillobj(deepcopy(newhref), currentobj);
				}

				if (options.include_pastobjs) {
					for (var i = 0; i < pastobjs.length; i++) {
						if (obj_indexOf(endhref, pastobjs[i].url) < 0 && !pastobjs[i].fake)
							endhref.push(deepcopy(pastobjs[i]));
					}
				}
			} else {
				if (_nir_debug_)
					console_log("finalize (newhref)", deepcopy(newhref));
				endhref = deepcopy(newhref);
			}

			if (_nir_debug_)
				console_log("endhref =", deepcopy(endhref));
		};

		var cb = null;
		if (options.cb) {
			var orig_cb = options.cb;
			options.cb = function(x) {
				// Is this needed?
				if (false) {
					for (var i = 0; i < pastobjs.length; i++) {
						if (pastobjs[i].url === null && pastobjs[i].waiting) {
							pastobjs.splice(i, 1);
							i--;
						}
					};
				}

				if (_nir_debug_)
					console_log("options.cb", deepcopy(x));

				var do_end = function() {
					if (_nir_debug_)
						console_log("do_end");

					finalize();
					do_cache();

					var blankurl = null;
					if (!options.null_if_no_change)
						blankurl = pasthrefs[pasthrefs.length - 1];

					if (!endhref || (is_array(endhref) && !endhref[0])) {
						endhref = blankurl;
					} else if (typeof endhref === "string") {
						endhref = blankurl;
					} else if (is_array(endhref) && typeof endhref[0] === "string") {
						endhref[0] = blankurl;
					} else if (is_array(endhref) && endhref[0] && !endhref[0].url) {
						endhref[0].url = blankurl;
					}

					orig_cb(endhref);
				};

				var parseresult = parse_bigimage(x);
				if ((!parseresult || (loop_i + 1) >= options.iterations) && !forcerecurse) {
					do_end();
				} else {
					for (; loop_i < options.iterations; loop_i++) {
						if (!do_bigimage()) {
							break;
						}
					}

					if (!waiting) {
						do_end();
					}
				}
			};
		}

		options._internal_info = {};

		for (loop_i = 0; loop_i < options.iterations; loop_i++) {
			if (!do_bigimage())
				break;
		}

		if (_nir_debug_)
			console_log("return finalize");

		finalize();
		do_cache();

		newhref = null;

		if (options.cb && !waiting) {
			options.cb(endhref);
		}

		return deepcopy(endhref);
	};
	bigimage_recursive.default_options = default_options;

	function is_internet_url(url) {
		if (!url || typeof url !== "string")
			return false;

		if (!/^https?:\/\//.test(url))
			return false;

		// local addresses (IPv4)
		if (/^[a-z]+:\/\/(127\.0\.0\.1|192\.168\.[0-9]+\.[0-9]+|10\.[0-9]+\.[0-9]+\.[0-9]+|172\.(?:1[6-9]|2[0-9]|3[01])\.[0-9]+\.[0-9]+|localhost|[^/.]+)\//.test(url))
			return false;

		// IPv6 (TODO: implement)
		if (/^[a-z]+:\/\/(?:[0-9a-f]*\:){1,}\//.test(url))
			return false;

		return true;
	}
	bigimage_recursive.is_internet_url = is_internet_url;

	function clear_all_caches() {
		url_cache.clear();
		real_api_cache.clear();
		cookie_cache.clear();
	}
	bigimage_recursive.clear_caches = clear_all_caches;

	var obj_to_simplelist = function(obj) {
		var out = [];
		for (var i = 0; i < obj.length; i++) {
			out.push(obj[i].url);
		}
		return out;
	};

	var obj_indexOf = function(obj, url) {
		return array_indexof(obj_to_simplelist(obj), url);
	};

	var obj_merge = function(newobj, oldobj) {
		var newobj_simple = obj_to_simplelist(newobj);

		for (var i = 0; i < oldobj.length; i++) {
			if (array_indexof(newobj_simple, oldobj[i].url) >= 0)
				continue;
			newobj.push(oldobj[i]);
		}

		return newobj;
	}

	var bigimage_recursive_loop = function(url, options, query, fine_urls, tried_urls, oldobj) {
		var newoptions = {};
		if (!fine_urls) {
			fine_urls = [];
		}

		if (!tried_urls) {
			tried_urls = [];
		}

		if (!oldobj) {
			oldobj = [];
		}

		for (var option in options) {
			if (option === "cb") {
				newoptions.cb = function(obj) {
					if (_nir_debug_) {
						console_log("bigimage_recursive_loop's cb: obj:", deepcopy(obj));
						console_log("bigimage_recursive_loop's cb: oldobj:", deepcopy(oldobj));
					}

					obj = obj_merge(obj, oldobj);
					var images = obj_to_simplelist(obj);

					for (var i = 0; i < obj.length; i++) {
						// TODO: also remove bad_if
						if (obj[i].bad) {
							var obj_url = obj[i].url;
							var orig_url = null;

							obj.splice(i, 1);
							images.splice(i, 1);
							i--;

							for (var j = 0; j < tried_urls.length; j++) {
								if (tried_urls[j][2] === obj_url) {
									var orig_url = tried_urls[j][3].url;
									var index = array_indexof(images, orig_url);
									tried_urls[j][4] = true;

									if (index >= 0) {
										obj.splice(index, 1);
										images.splice(index, 1);

										if (index < i)
											i -= 2;
										else if (index === i)
											i--;
									}

									break;
								}
							}
						}
					}

					if (_nir_debug_) {
						console_log("bigimage_recursive_loop's cb: obj after:", deepcopy(obj));
						console_log("bigimage_recursive_loop's cb: images after:", deepcopy(images));
						console_log("bigimage_recursive_loop's cb: fine_urls:", deepcopy(fine_urls));
						console_log("bigimage_recursive_loop's cb: tried_urls:", deepcopy(tried_urls));
					}

					for (var i = 0; i < fine_urls.length; i++) {
						var index = array_indexof(images, fine_urls[i][0]);
						if (index >= 0) {
							obj = [obj[index]];
							return options.cb(obj, fine_urls[i][1]);
						}
					}

					var try_any = false;
					for (var i = 0; i < tried_urls.length; i++) {
						if (tried_urls[i][0] === url || try_any) {
							if (tried_urls[i][4] === true) {
								try_any = true;
								continue;
							}

							var index = array_indexof(images, tried_urls[i][2]);
							if (index >= 0) {
								obj = [obj[index]];
								return options.cb(obj, tried_urls[i][1]);
							} else {
								return options.cb(null, tried_urls[i][1]);
							}
						}
					}

					query(obj, function (newurl, newobj, data) {
						if (!newurl) {
							return options.cb(null, data);
						}

						fine_urls.push([newurl, data]);
						tried_urls.push([url, data, newurl, deepcopy(newobj), false]);

						//if (array_indexof(images, newurl) < 0 && newurl !== url || true) {
						var newurl_index = array_indexof(images, newurl);
						if (newurl_index < 0 || !obj[newurl_index].norecurse) {
							bigimage_recursive_loop(newurl, options, query, fine_urls, tried_urls, obj);
						} else {
							//obj = obj.slice(array_indexof(images, newurl));
							obj = [obj[newurl_index]];
							options.cb(obj, data);
						}
					});
				};
			} else {
				newoptions[option] = options[option];
			}
		}

		if (_nir_debug_)
			console_log("bigimage_recursive_loop", url);

		return bigimage_recursive(url, newoptions);
	};
	bigimage_recursive.loop = bigimage_recursive_loop;

	var send_redirect = function(obj, cb) {
		if (is_extension) {
			extension_send_message({
				type: "redirect",
				data: obj
			}, function() {
				cb();
			});
		} else {
			cb();
		}
	};

	var redirect = function(url, obj) {
		if (_nir_debug_) {
			console_log("redirect", url, obj);
		}

		if (_nir_debug_ && _nir_debug_.no_redirect)
			return;

		if (url === window.location.href)
			return;

		// wrap in try/catch due to nano defender
		try {
			// avoid downloading more before redirecting
			window.stop();
		} catch (e) {
		}

		send_redirect(obj, function() {
			if (settings.redirect_history) {
				window.location.assign(url);
			} else {
				window.location.replace(url);
			}
		});
	};

	// these functions can run before the document has loaded
	var cursor_wait = function() {
		if (document.documentElement)
			document.documentElement.style.cursor = "wait";
	};

	var cursor_default = function() {
		if (document.documentElement)
			document.documentElement.style.cursor = "default";
	};


	var infobox_timer = null;
	var show_image_infobox = function(text) {
		var div = document.createElement("div");
		div.style.backgroundColor = "#fffabb";
		div.style.position = "absolute";
		div.style.top = "0px";
		div.style.left = "0px";
		div.style.padding = ".4em .8em";
		div.style.boxShadow = "0px 0px 20px rgba(0,0,0,.6)";
		div.style.margin = ".8em";

		div.innerHTML = text;

		div.onclick = function() {
			document.body.removeChild(div);

			if (infobox_timer) {
				clearTimeout(infobox_timer);
				infobox_timer = null;
			}
		};

		document.body.appendChild(div);

		var do_timeout = function() {
			if (infobox_timer)
				return;

			infobox_timer = setTimeout(function() {
				document.body.removeChild(div);
			}, 7000);
		};

		if (document.hasFocus()) {
			do_timeout();
		} else {
			document.onfocus = function() {
				do_timeout();
				document.onfocus = null;
			};

			window.onfocus = function() {
				do_timeout();
				window.onfocus = null;
			};
		}
	};

	var check_ok_error = function(ok_errors, error) {
		if (ok_errors && is_array(ok_errors)) {
			for (var i = 0; i < ok_errors.length; i++) {
				if (error.toString() === ok_errors[i].toString()) {
					return true;
				}
			}

			return false;
		}

		return null;
	};

	var get_single_trigger_key_text = function(list) {
		list = list.sort(function(a, b) {
			if (a === b)
				return 0;

			if (a === "ctrl")
				return -1;
			if (b === "ctrl")
				return 1;
			if (a === "shift")
				return -1;
			if (b === "shift")
				return 1;
			if (a === "super")
				return -1;
			if (b === "super")
				return 1;
			if (a === "alt")
				return -1;
			if (b === "alt")
				return 1;
			if (a < b)
				return -1;
			if (b > a)
				return 1;
		});

		var newlist = [];
		for (var i = 0; i < list.length; i++) {
			var capitalized = list[i].charAt(0).toUpperCase() + list[i].slice(1);
			if (list.length === 1 && (capitalized === "Left" || capitalized === "Right" || capitalized === "Up" || capitalized === "Down")) {
				capitalized += " Arrow";
			}

			newlist.push(_(capitalized));
		}

		return newlist.join("+");
	};

	var get_trigger_key_texts = function(list) {
		if (!is_array(list[0])) {
			list = [list];
		}

		var result = [];

		for (var i = 0; i < list.length; i++) {
			result.push(get_single_trigger_key_text(list[i]));
		}

		return result;
	};

	var get_trigger_key_text = function(list) {
		return get_trigger_key_texts(list).join(" / ");
	};

	var check_image = function(obj, err_cb, ok_cb) {
		if (_nir_debug_)
			console_log("check_image", deepcopy(obj));

		if (is_array(obj)) {
			obj = obj[0];
		}

		if (!obj || !obj.url) {
			ok_cb(obj);
			return;
		}

		var print_orig = function() {
			if (obj && obj.extra) {
				if (obj.extra.page) {
					console_log("Original page: " + obj.extra.page);
				}

				if (obj.extra.caption) {
					console_log("Caption: " + obj.extra.caption);
				}
			}
		};

		var url = obj.url;
		var err_txt;

		if (url === window.location.href) {
			print_orig();
			ok_cb(url);
		} else  {
			var headers = obj.headers;
			console_log(obj.url);

			print_orig();

			if (obj) {
				if (obj.bad) {
					err_txt = "Bad image";
				} else if (obj.video && obj.video !== true) {
					err_txt = "Can't redirect to streaming video type " + obj.video;
				}

				if (err_txt) {
					if (err_cb) {
						err_cb(err_txt);
					} else {
						console_error(err_txt);
					}
					return;
				}
			}

			var mouseover_text = function(reason) {
				var mouseover;
				if (!settings.mouseover) {
					mouseover = "disabled";
				} else if (settings.mouseover_trigger_behavior === "keyboard") {
					mouseover = get_trigger_key_text(settings.mouseover_trigger_key);
				} else if (settings.mouseover_trigger_behavior === "mouse") {
					mouseover = "delay " + settings.mouseover_trigger_delay + "s";
				}

				// TODO: another option could be to allow it whenever the image can be imu'd
				imagetab_ok_override = true;

				var trigger_options_link = "<a style='color:blue; font-weight:bold' href='" + options_page + "' target='_blank' rel='noreferrer'>" + mouseover + "</a>";
				show_image_infobox(_("Mouseover popup (%%1) is needed to display the original version", trigger_options_link) + " (" + _(reason) + ")");
			};

			if (!_nir_debug_ || !_nir_debug_.no_request) {
				cursor_wait();

				var url_domain = url.replace(/^([a-z]+:\/\/[^/]*).*?$/, "$1");

				var origheaders = deepcopy(headers);

				var customheaders = true;
				if (!headers || Object.keys(headers).length === 0) {
					customheaders = false;
					headers = {
						// Origin is not often added by the browser, and doesn't work for some sites
						//"Origin": url_domain,
						"Referer": window.location.href
					};
				} else if (!headers.Origin && !headers.origin) {
					//headers.Origin = url_domain;
				}

				if (customheaders && Object.keys(headers).length === 1 && ("Referer" in headers)) {
					var domain = window.location.href;
					domain = domain.replace(/^[a-z]+:\/\/([^/]*).*?$/, "$1");
					var url_domain = url.replace(/^[a-z]+:\/\/([^/]*).*?$/, "$1");

					if (obj.referer_ok.same_domain && domain === url_domain) {
						customheaders = false;
					} else if (obj.referer_ok.same_domain_nosub && get_domain_nosub(domain) === get_domain_nosub(url_domain)) {
						customheaders = false;
					}
				}

				if (customheaders && !is_extension) {
					document.documentElement.style.cursor = "default";
					console_log("Custom headers needed, currently unhandled");

					mouseover_text("custom headers");
					return;
				}

				if (_nir_debug_)
					console_log("(check_image) headers", headers);

				if (obj.always_ok ||
					(!obj.can_head && !settings.canhead_get))
					return ok_cb(url);

				var method = "HEAD";
				if (!obj.can_head && settings.canhead_get) {
					if (_nir_debug_)
						console_log("Trying GET");
					method = "GET";
				}

				var handled = false;

				var onload_cb = function(resp) {
					if (handled)
						return;

					handled = true;

					if (_nir_debug_)
						console_log("(check_image) resp", resp);

					// FireMonkey returns null when tracking protection blocks a URL
					if (is_userscript && userscript_manager === "FireMonkey" && !resp) {
						err_txt = "Error: resp == null (tracking protection blocked, FireMonkey bug)";
						if (err_cb) {
							err_cb(err_txt);
						} else {
							console_error(err_txt);
						}

						return;
					}

					// nano defender removes this.DONE
					if (resp.readyState < 2) {
						return;
					}

					if (is_userscript && !resp.status && resp.readyState < 4) {
						// Tampermonkey and Greasemonkey have a bug where status isn't set for onprogress events
						// Tampermonkey issue: https://github.com/Tampermonkey/tampermonkey/issues/857
						// Greasemonkey issue: https://github.com/greasemonkey/greasemonkey/issues/3068
						handled = false;
						return;
					}

					if (resp.readyState < 4) {
						if (req && req.abort)
							req.abort();
					}

					if (resp.status === 0 ||
						check_tracking_blocked(resp)) {
						// error loading image (IP doesn't exist, etc.), ignore
						err_txt = "Error: status == 0";
						if (err_cb) {
							err_cb(err_txt);
						} else {
							console_error(err_txt);
						}

						return;
					}

					cursor_default();

					if (resp.finalUrl === window.location.href) {
						console_log(resp.finalUrl);
						console_log("Same URL");
						return;
					}

					var headers_list = parse_headers(resp.responseHeaders);
					var headers = headers_list_to_dict(headers_list);

					if (_nir_debug_)
						console_log("(check_image) resp headers", headers);


					var digit = resp.status.toString()[0];

					var ok_error = check_ok_error(obj.head_ok_errors, resp.status);

					if (((digit === "4" || digit === "5") &&
						resp.status !== 405) &&
						ok_error !== true) {
						err_txt = "Error: " + resp.status;
						if (err_cb) {
							err_cb(err_txt);
						} else {
							console_error(err_txt);
						}

						return;
					}

					var content_type = headers["content-type"];
					if (!content_type)
						content_type = "";
					content_type = content_type.toLowerCase();

					if (content_type.match(/text\/html/) && !obj.head_wrong_contenttype &&
						ok_error !== true) {
						var err_txt = "Error: Not an image: " + content_type;
						if (err_cb) {
							err_cb(err_txt);
						} else {
							console_error(err_txt);
						}

						return;
					}

					if (!is_extension || settings.redirect_disable_for_responseheader) {
						if (obj.forces_download || (
							(content_type.match(/(?:binary|application|multipart)\//) ||
								// such as [image/png] (server bug)
								content_type.match(/^ *\[/)) && !obj.head_wrong_contenttype) ||
							(headers["content-disposition"] &&
								headers["content-disposition"].toLowerCase().match(/^ *attachment/))) {
							console_error("Forces download");
							mouseover_text("forces download");
							return;
						}
					}

					if (headers["content-length"] && headers["content-length"] == "0" && !obj.head_wrong_contentlength) {
						console_error("Zero-length image");
						return;
					}

					if (check_bad_if(obj.bad_if, resp)) {
						console_error("Bad image (bad_if)", obj.bad_if, resp);
						return err_cb("bad image");
					}

					if (!customheaders || is_extension)
						ok_cb(url);
					else
						console_log("Custom headers needed, currently unhandled");
				};

				var req = do_request({
					method: method,
					url: url,
					headers: headers,
					trackingprotection_failsafe: true,
					onprogress: function(resp) {
						// 2 = HEADERS_RECEIVED
						if (resp.readyState >= 2 && resp.responseHeaders) {
							onload_cb(resp);
						}
					},
					onload: onload_cb
				});
			}
		}
	};

	function do_export() {
		$$IMU_EXPORT$$ = bigimage_recursive;

		if (is_node) {
			module.exports = bigimage_recursive;
		} else if (is_scripttag) {
			imu_variable = bigimage_recursive;
		}
	}

	function currenttab_is_image() {
		return !document.contentType.match(/^text\//);
	}

	function do_redirect() {
		if (!currenttab_is_image()) {
			return;
		}

		cursor_wait();

		var force_page = false;
		if ((settings["redirect_force_page"] + "") === "true")
			force_page = true;

		bigimage_recursive_loop(window.location.href, {
			fill_object: true,
			force_page: force_page,
			document: document,
			window: get_window(),
			cb: function(newhref) {
				if (_nir_debug_) {
					console_log("do_redirect (final)", newhref);
				}

				cursor_default();

				if (!newhref) {
					return;
				}

				if (settings.print_imu_obj)
					console_log(newhref);

				var newurl = newhref[0].url;

				if (false && newhref[0].extra && newhref[0].extra.page) {
					console_log("Original page: " + newhref[0].extra.page);
				}

				if (newurl === window.location.href)
					return;

				if (!newurl)
					return;

				if (_nir_debug_)
					console_log("redirect (recursive loop)", newhref);

				redirect(newurl, newhref);
			}
		}, function(newhref, real_finalcb) {
			if (_nir_debug_) {
				console_log("do_redirect", newhref);
			}

			var finalcb = function(newurl, data, newobj) {
				real_finalcb(newurl, newobj, data);
			};

			if (false && (!newhref[0].can_head || newhref[0].always_ok)) {
				var newurl = newhref[0].url;

				if (newurl === window.location.href) {
					cursor_default();
					return;
				}

				if (_nir_debug_) {
					console_log("Not checking due to can_head == false || always_ok == true");
				}

				finalcb(newurl);
				return;
			}

			var new_newhref = newhref;

			if (!settings.allow_video) {
				var new_newhref = [];
				for (var i = 0; i < newhref.length; i++) {
					if (!newhref[i].video) {
						new_newhref.push(newhref[i]);
					}
				}
			}

			var index = 0;
			var cb = function(err_txt) {
				index++;
				if (index >= new_newhref.length) {
					cursor_default();
					console_error(err_txt);
					return;
				}
				check_image(new_newhref[index], cb, finalcb);
			};
			check_image(new_newhref[0], cb, finalcb);
		});
	}

	function onload(cb) {
		if (document.readyState === "complete" ||
			document.readyState === "interactive") {
			cb();
		} else {
			var state_cb = function() {
				if (document.readyState === "complete" ||
					document.readyState === "interactive") {
					cb();

					our_removeEventListener(document, "readystatechange", state_cb);
				}
			};

			our_addEventListener(document, "readystatechange", state_cb);
		}
	}

	function get_keystrs_map(event, value) {
		var keys = {};

		if (event.ctrlKey) {
			keys.ctrl = true;
		} else {
			keys.ctrl = false;
		}

		/*if (event.metaKey) {
			keys["super"] = true;
		} else {
			keys["super"] = false;
		}*/

		if (event.altKey) {
			keys.alt = true;
		} else {
			keys.alt = false;
		}

		if (event.shiftKey) {
			keys.shift = true;
		} else {
			keys.shift = false;
		}

		if (event.buttons !== undefined) {
			var buttonnames = ["button1", "button2", "button3", "button4", "button5"];
			var buttons = event.buttons;
			while (buttonnames.length > 0) {
				if (buttons & 1) {
					keys[buttonnames[0]] = true;
				} else {
					keys[buttonnames[0]] = false;
				}

				buttons >>= 1;
				buttonnames.shift();
			}
		}

		if (event.type === "wheel") {
			if (event.deltaY < 0) {
				keys.wheelUp = true;
			} else if (event.deltaY > 0) {
				keys.wheelDown = true;
			}

			if (event.deltaX < 0) {
				keys.wheelLeft = true;
			} else if (event.deltaX > 0) {
				keys.wheelRight = true;
			}
		}

		var str = keycode_to_str(event);
		if (str === undefined) {
			return keys;
		}

		keys[str] = value;
		return keys;
	}

	var keystr_is_wheel = function(keystr) {
		return /^wheel/.test(keystr);
	};

	var keystr_is_button12 = function(keystr) {
		return keystr === "button1" || keystr === "button2";
	}

	var chord_is_only_wheel = function(chord) {
		for (var i = 0; i < chord.length; i++) {
			if (!keystr_is_wheel(chord[i]) && !keystr_is_button12(chord[i])) {
				return false;
			}
		}

		return true;
	};

	var keysequence_bad = function(keyseq) {
		if (chord_is_only_wheel(keyseq))
			return true;

		if (keyseq.length !== 1)
			return false;

		return keystr_is_button12(keyseq[0]);
	};

	var keysequence_valid = function(keyseq) {
		if (keyseq.length === 0)
			return false;

		if (keysequence_bad(keyseq))
			return false;

		if (keyseq.length > 1)
			return true;

		return true;
	};

	var prefers_dark_mode = function() {
		try {
			return window.matchMedia("(prefers-color-scheme: dark)").matches;
		} catch (e) {
			return false;
		}
	};

	function update_dark_mode() {
		if (!is_maxurl_website && !is_options_page) {
			return;
		}

		if (prefers_dark_mode()) {
			set_default_value("dark_mode", true);
		}

		if (settings.dark_mode) {
			document.documentElement.classList.add("dark");
		} else {
			document.documentElement.classList.remove("dark");
		}
	}

	var request_permission = function(permission, cb) {
		if (!is_extension)
			return cb(false);

		// This has to be done in the content script under firefox: https://github.com/qsniyg/maxurl/issues/254
		if (true) {
			try {
				chrome.permissions.request({
					permissions: [permission]
				}, function(granted) {
					cb(granted);
				});
			} catch(e) {
				console_error(e);
				cb(false);
			}
		} else {
			extension_send_message({
				type: "permission",
				data: {
					permission: permission
				}
			}, function(result) {
				cb(result.data.granted);
			});
		}
	};

	var current_options_tab = "general";
	function do_options() {
		update_dark_mode();

		var recording_keys = false;
		var options_chord = [];
		var current_options_chord = [];

		function update_options_chord(event, value) {
			if (!recording_keys)
				return;

			var map = get_keystrs_map(event, value);

			if ((keycode_to_str(event) || event.type === "mousedown") &&
				current_options_chord.length === 0) {
				// Don't clear the options chord for either left or right mouse buttons
				if (event.button !== 0 && event.button !== 2)
					options_chord = [];
			}

			var old_options_chord = deepcopy(options_chord);
			for (var key in map) {
				update_options_chord_sub(key, map[key]);
			}

			if (keysequence_bad(options_chord))
				options_chord = old_options_chord;

			recording_keys();
		}

		function update_options_chord_sub(str, value) {
			if (value) {
				if (array_indexof(options_chord, str) < 0) {
					options_chord.push(str);
				}

				if (array_indexof(current_options_chord, str) < 0) {
					current_options_chord.push(str);
				}
			} else {
				if (array_indexof(current_options_chord, str) >= 0) {
					current_options_chord.splice(array_indexof(current_options_chord, str), 1);
				}
			}
		}

		document.addEventListener('keydown', function(event) {
			update_options_chord(event, true);

			if (recording_keys) {
				event.preventDefault();
				return false;
			}
		});

		document.addEventListener('mousedown', function(event) {
			update_options_chord(event, true);

			if (recording_keys) {
				event.preventDefault();
				//event.stopImmediatePropagation();
				return false;
			}
		});

		document.addEventListener('wheel', function(event) {
			update_options_chord(event, true);

			if (recording_keys) {
				event.preventDefault();
				event.stopImmediatePropagation();
				event.stopPropagation();
				return false;
			}
		}, {
			capture: true,
			passive: false
		});

		document.addEventListener("contextmenu", function(event) {
			if (recording_keys) {
				event.preventDefault();
				event.stopImmediatePropagation();
				return false;
			}
		});

		document.addEventListener('keyup', function(event) {
			update_options_chord(event, false);

			if (recording_keys) {
				event.preventDefault();
				return false;
			}
		});

		document.addEventListener('mouseup', function(event) {
			if (event.button === 1)
				return;

			update_options_chord(event, false);

			if (recording_keys) {
				event.preventDefault();
				event.stopImmediatePropagation();
				return false;
			}
		});

		var options_el = document.getElementById("options");

		if (!is_extension_options_page)
			options_el.innerHTML = "<h1>" + _("options_header") + "</h1>";
		else
			options_el.innerHTML = "";

		var saved_el = document.getElementById("saved");
		if (!saved_el) {
			saved_el = document.createElement("div");
			saved_el.style.visibility = "hidden";
			saved_el.id = "saved";
			saved_el.classList.add("topsaved");
		}

		var get_default_saved_text = function() {
			var text = "saved_refresh_target";
			if (is_extension || typeof GM_addValueChangeListener !== "undefined") {
				text = "saved_no_refresh";
			}

			return text;
		};

		var set_saved_text = function(id) {
			saved_el.innerHTML = "<p>" + _(id) + "</p>";
		};

		set_saved_text(get_default_saved_text());
		//saved_el.style.pointer_events = "none";
		//saved_el.style.textAlign = "center";
		//saved_el.style.paddingTop = "1em";
		//saved_el.style.fontStyle = "italic";
		//saved_el.style.color = "#0af";
		var saved_timeout = null;

		var create_update_available = function() {
			var update_available_el = document.createElement("div");
			update_available_el.classList.add("update-available");
			update_available_el.innerHTML = "Update available: v" + current_version + " -&gt; ";

			var link = settings.last_update_url;
			if (!link) {
				if (is_firefox_webextension) {
					link = firefox_addon_page;
				} else if (is_userscript) {
					link = userscript_update_url;
				} else {
					link = null;
				}
			}

			if (link) {
				update_available_el.innerHTML += "<a href=\"" + link + "\" target=\"_blank\" rel=\"noreferer\">v" + settings.last_update_version + "</a>";
			} else {
				update_available_el.innerHTML += "v" + settings.last_update_version;
			}

			return update_available_el;
		};

		if (settings.check_updates && version_compare(current_version, settings.last_update_version) === 1) {
			options_el.appendChild(create_update_available());
		}

		var topbtns_holder = document.createElement("div");
		topbtns_holder.id = "topbtns";
		options_el.appendChild(topbtns_holder);

		var importexport_ocontainer = document.createElement("div");
		importexport_ocontainer.id = "importexport";
		importexport_ocontainer.classList.add("center-outer");
		importexport_ocontainer.style.display = "none";
		options_el.appendChild(importexport_ocontainer);

		var importexport_container = document.createElement("div");
		importexport_container.classList.add("center-inner");
		importexport_ocontainer.appendChild(importexport_container);

		var importexport_text = document.createElement("textarea");
		importexport_container.appendChild(importexport_text);

		var importexport_btn = document.createElement("button");
		importexport_btn.innerText = _("Import");
		importexport_btn.onclick = function() {
			var value;

			try {
				value = JSON_parse(importexport_text.value);
			} catch (e) {
				console_error(e);
				importexport_text.value = "Error!";
				return;
			}

			var changed = false;

			var current_settings = deepcopy(settings);
			var new_settings = deepcopy(orig_settings);
			var settings_version;
			for (var key in value) {
				if (!(key in settings)) {
					if (key === "settings_version") {
						settings_version = value[key];
					} else {
						console_log("Unknown key in imported settings: ", key);
					}

					continue;
				}

				new_settings[key] = value[key];
			}

			for (var key in new_settings) {
				if (JSON_stringify(new_settings[key]) !== JSON_stringify(current_settings[key])) {
					settings[key] = new_settings[key];
					set_value(key, new_settings[key]);
					changed = true;
				}
			}

			if (settings_version === undefined) {
				settings_version = 1;
			}

			upgrade_settings_with_version(settings_version, new_settings, function(changed) {
				if (changed) {
					console_log("Settings imported");
					do_options();
				} else {
					console_log("No settings changed");
				}

				show_importexport(false);
			});
		};
		importexport_container.appendChild(importexport_btn);

		var show_importexport = function(show, btn) {
			if (show) {
				importexport_ocontainer.style.display = "block";
			} else {
				importexport_state = null;

				importexport_ocontainer.style.display = "none";
			}

			if (btn) {
				if (show)
					importexport_state = "import";

				importexport_btn.style.display = "inline-block";
			} else {
				if (show)
					importexport_state = "export";

				importexport_btn.style.display = "none";
			}
		};

		var importexport_state = null;

		var import_btn = document.createElement("button");
		import_btn.id = "importbtn";
		import_btn.innerText = _("Import");
		import_btn.title = _("Import settings");
		import_btn.onclick = function() {
			if (importexport_state === "import")
				return show_importexport(false);

			importexport_text.value = "";

			show_importexport(true, true);
		};
		topbtns_holder.appendChild(import_btn);

		var export_btn = document.createElement("button");
		export_btn.id = "exportbtn";
		export_btn.innerText = _("Export");
		export_btn.title = _("Export settings");
		export_btn.onclick = function() {
			if (importexport_state === "export")
				return show_importexport(false);

			var newsettings = deepcopy(settings);
			get_value("settings_version", function(value) {
				if (value !== undefined)
					newsettings.settings_version = value;
				importexport_text.value = JSON_stringify(newsettings);

				show_importexport(true, false);
			});
		};
		topbtns_holder.appendChild(export_btn);

		var enabled_map = {};
		var reason_map = {};

		var check_sub_option = function(meta, reason) {
			if (typeof reason === "undefined") {
				reason = {};
			}

			var enabled = true;

			var prepare_array = function(value) {
				var result = deepcopy(value);
				if (!result) {
					return null;
				}

				if (!is_array(result)) {
					result = [result];
				}

				return result;
			}

			var requires = prepare_array(meta.requires);
			var disabled_if = prepare_array(meta.disabled_if);

			if (!meta.imu_enabled_exempt) {
				if (!settings.imu_enabled) {
					enabled = false;
				}
			}

			reason.good = [];
			reason.bad = [];
			if (enabled && requires) {
				enabled = check_validity(requires, reason);
				reason.good = [];
			}

			if (enabled && disabled_if) {
				reason.good = [];
				reason.bad = [];

				enabled = !check_validity(disabled_if, reason);
				reason.bad = [];
			}

			return enabled;
		};

		var check_option = function(setting) {
			var meta = settings_meta[setting];

			enabled_map[setting] = "processing";
			reason_map[setting] = {};
			var enabled = check_sub_option(meta, reason_map[setting]);
			enabled_map[setting] = enabled;

			return enabled;
		};

		var check_validity = function(array, reason) {
			for (var i = 0; i < array.length; i++) {
				var current = array[i];
				var current_valid = true;

				var good_reason, bad_reason;

				if (typeof reason !== "undefined") {
					good_reason = [];
					bad_reason = [];
				}

				for (var required_setting in current) {
					if (required_setting[0] === "_")
						continue;

					var required_value = current[required_setting];

					var value = settings[required_setting];
					if (is_array(value) && !is_array(required_value))
						value = value[0];

					if (!(required_setting in enabled_map)) {
						check_option(required_setting);
					}

					if (enabled_map[required_setting] === "processing") {
						console_error("Dependency cycle detected for: " + setting + ", " + required_setting);
						return;
					}

					var current_reason = {
						setting: required_setting,
						required_value: required_value,
						current_value: value,
						enabled: enabled_map[required_setting]
					};

					if (enabled_map[required_setting] && value === required_value) {
						//current_valid = true;
						if (typeof good_reason !== "undefined") {
							good_reason.push(current_reason);
						}
					} else {
						current_valid = false;

						if (typeof bad_reason !== "undefined") {
							bad_reason.push(current_reason);
						}
					}

					if (!current_valid && typeof reason === "undefined") {
						break;
					}
				}

				if (typeof reason !== "undefined") {
					reason.good.push(good_reason);
					reason.bad.push(bad_reason);
				}

				if (current_valid) {
					return true;
				}
			}

			return false;
		};

		var get_option_from_options = function(options, option) {
			if (option in options)
				return options[option];

			for (var option_name in options) {
				if (/^_group/.test(option_name)) {
					return get_option_from_options(options[option_name], option);
				}
			}

			return null;
		};

		var is_nonempty_reason = function(goodbad) {
			for (var i = 0; i < goodbad.length; i++) {
				if (goodbad[i].length !== 0)
					return true;
			}

			return false;
		};

		var is_reason_goodbad = function(reason) {
			if (is_nonempty_reason(reason.good))
				return "good";

			if (is_nonempty_reason(reason.bad))
				return "bad";

			return null;
		};

		var fill_requirements = function(reason, div) {
			div.innerHTML = "";

			if (!settings.settings_show_requirements)
				return;

			var goodbad = is_reason_goodbad(reason);
			if (!goodbad)
				return;

			var requires_p = document.createElement("p");
			requires_p.innerText = _("Requires:");
			div.appendChild(requires_p);

			var els = [];

			var array = reason[goodbad];
			for (var i = 0; i < array.length; i++) {
				if (array[i].length === 0)
					continue;

				var ul = document.createElement("ul");

				for (var j = 0; j < array[i].length; j++) {
					var single_reason = array[i][j];

					var option_el = document.getElementById("option_" + single_reason.setting);
					var option_name = option_el.getElementsByClassName("name_td")[0].innerText;

					var wanted_value_el = document.querySelector("label[for=\"input_" + single_reason.setting + "_" + single_reason.required_value + "\"]");
					var wanted_value = single_reason.required_value;
					if (wanted_value_el) {
						wanted_value = wanted_value_el.innerText;
					}

					var equals = "=";
					if (goodbad === "good")
						equals = "!=";

					var li = document.createElement("li");
					li.innerText = option_name + " " + equals + " " + wanted_value;
					ul.appendChild(li);
				}

				els.push(ul);
			}

			var newels = [];
			for (var i = 0; i < els.length - 1; i++) {
				newels.push(els[i]);

				// FIXME: this should be 'and' for disabled_if
				var or_p = document.createElement("p");
				or_p.innerText = _("Or:");

				newels.push(or_p);
			}

			newels.push(els[els.length-1]);

			for (var i = 0; i < newels.length; i++) {
				div.appendChild(newels[i]);
			}
		};

		function check_disabled_options() {
			var options = options_el.querySelectorAll("div.option");

			enabled_map = {};

			for (var i = 0; i < options.length; i++) {
				var setting = options[i].id.replace(/^option_/, "");

				var enabled = check_option(setting);

				if (enabled) {
					options[i].classList.remove("disabled");
					options[i].classList.remove("disabled-hidden");

					options[i].getElementsByClassName("requirements")[0].style.display = "none";

					var meta = settings_meta[setting];
					var meta_options = meta.options;
					var regexp = new RegExp("^input_" + setting + "_");

					var els = options[i].querySelectorAll("input, textarea, button, select");
					for (var j = 0; j < els.length; j++) {
						var input = els[j];

						input.disabled = false;

						if (meta_options) {
							var option_name = input.id.replace(regexp, "");
							if (option_name !== input.id) {
								var option_value = get_option_from_options(meta_options, option_name);
								if (option_value) {
									if (!check_sub_option(option_value)) {
										input.disabled = true;
									}
								}
							}
						}
					}
				} else {
					options[i].classList.add("disabled");

					if (!settings.settings_show_disabled)
						options[i].classList.add("disabled-hidden");

					var els = options[i].querySelectorAll("input, textarea, button, select");
					for (var j = 0; j < els.length; j++) {
						var input = els[j];
						input.disabled = true;
					}

					var requirements_div = options[i].getElementsByClassName("requirements")[0];
					requirements_div.style.display = "block";
					fill_requirements(reason_map[setting], requirements_div);
				}
			}
		}

		function show_warnings() {
			var options = options_el.querySelectorAll("div.option");
			for (var i = 0; i < options.length; i++) {
				var setting = options[i].id.replace(/^option_/, "");

				var meta = settings_meta[setting];
				if (meta.warning) {
					var warning = meta.warning[settings[setting] + ""];
					var el = options[i].querySelector(".warning");
					if (!el)
						continue;

					if (warning) {
						el.innerHTML = warning;
						el.style.display = "block";
					} else {
						el.style.display = "none";
					}
				}
			}
		}

		function show_saved_message(meta) {
			if (meta.needrefresh) {
				set_saved_text("saved_refresh_target");
			} else {
				set_saved_text(get_default_saved_text());
			}

			saved_el.setAttribute("style", "");
			saved_el.classList.remove("fadeout");

			if (saved_timeout)
				clearTimeout(saved_timeout);

			saved_timeout = setTimeout(function() {
				saved_el.classList.add("fadeout");
			}, 2000);
		}

		function md_to_html(parent, text) {
			var current_el = null;
			var current_text = "";
			var current_tag = null;

			var apply_tag = function() {
				if (current_text.length === 0)
					return;

				if (current_tag === "`") {
					current_el = document.createElement("code");
				} else {
					current_el = document.createElement("span");
				}

				current_el.innerText = current_text;
				current_text = "";
				parent.appendChild(current_el);
			}

			for (var i = 0; i < text.length; i++) {
				if (text[i] === current_tag) {
					apply_tag();
					current_tag = null;
					continue;
				}

				if (text[i] === "`") {
					apply_tag();
					current_tag = text[i];
					continue;
				}

				current_text += text[i];
			}

			apply_tag();
		}

		var tabscontainer;
		if (settings.settings_tabs) {
			tabscontainer = document.createElement("div");
			tabscontainer.id = "tabs";
			options_el.appendChild(tabscontainer);
		}

		var category_els = {};
		var subcategory_els = {};

		for (var category in categories) {
			var catname = _(categories[category]);

			var div = document.createElement("div");
			div.id = "cat_" + category;
			div.classList.add("category");

			category_els[category] = [div];

			if (settings.settings_tabs) {
				div.classList.add("tabbed");

				var tab = document.createElement("span");
				tab.classList.add("tab");
				//tab.href = "#cat_" + category;
				tab.id = "tab_cat_" + category;
				tab.innerText = catname;

				(function(category) {
					tab.onclick = function() {
						current_options_tab = category;
						do_options();
					};
				})(category);

				if (category === current_options_tab) {
					tab.classList.add("active");
				} else {
					div.style.display = "none";
				}

				category_els[category].push(tab);

				tabscontainer.appendChild(tab);
			} else {
				var h2 = document.createElement("h2");
				h2.innerText = catname;
				div.appendChild(h2);
			}

			var subdiv = document.createElement("div");
			subdiv.id = "subcat_" + category;
			subdiv.classList.add("subcat");
			subdiv.classList.add("frame");
			div.appendChild(subdiv);
			subcategory_els[category] = subdiv;

			if (category in subcategories) {
				for (var subcat in subcategories[category]) {
					var newsubdiv = document.createElement("div");
					newsubdiv.id = "subcat_" + subcat;
					newsubdiv.classList.add("subcat");
					newsubdiv.classList.add("frame");

					var h3 = document.createElement("h3");
					h3.innerText = _(subcategories[category][subcat]);
					newsubdiv.appendChild(h3);

					div.appendChild(newsubdiv);
					subcategory_els[subcat] = newsubdiv;
				}
			}

			options_el.appendChild(div);
		}

		var show_advanced = settings.advanced_options;

		var normalize_value = function(value) {
			if (is_array(value) && value.length === 1) {
				return JSON.stringify(value[0]);
			}

			return JSON.stringify(value);
		};

		var category_settings = {};

		for (var setting in settings) {
			(function(setting) {
				var meta = settings_meta[setting];
				if (!meta) {
					return;
				}

				if (!(setting in orig_settings))
					return;

				var value = settings[setting];
				var orig_value = orig_settings[setting];

				if (meta.hidden)
					return;

				if (meta.userscript_only && !is_userscript)
					return;

				if (meta.extension_only && !is_extension)
					return;

				if (meta.advanced && !show_advanced)
					return;

				category_settings[meta.category] = true;
				if (settings.settings_tabs && meta.category !== current_options_tab)
					return;

				var option = document.createElement("div");
				option.classList.add("option");
				option.id = "option_" + setting;

				var table = document.createElement("table");
				table.style.border = "0";

				var tr = document.createElement("tr");
				table.appendChild(tr);

				var name = document.createElement("strong");
				md_to_html(name, _(meta.name));

				var description = _(meta.description);
				if (meta.description_userscript && is_userscript)
					description = _(meta.description_userscript);

				name.title = description;

				var name_td = document.createElement("td");
				name_td.style.verticalAlign = "middle";
				name_td.classList.add("name_td");

				var check_value_orig_different = function(value) {
					var value_norm = normalize_value(value);
					var orig_norm = normalize_value(orig_value);

					if (meta.type === "keysequence") {
						value_norm = JSON_stringify(normalize_keychord(value));
						orig_norm = JSON_stringify(normalize_keychord(orig_value));
					}

					var value_orig_different = value_norm !== orig_norm;
					if (value_orig_different) {
						name_td.classList.add("value_modified");
					} else {
						name_td.classList.remove("value_modified");
					}
				};
				check_value_orig_different(value);

				var do_update_setting_real = function(setting, new_value, meta) {
					update_setting(setting, new_value);

					run_soon(function() {
						check_value_orig_different(new_value);
					});

					show_saved_message(meta);
				};

				var do_update_setting = function(setting, new_value, meta) {
					if (is_extension && meta.required_permission) {
						request_permission(meta.required_permission, function(granted) {
							if (granted) {
								do_update_setting_real(setting, new_value, meta);
							} else {
								do_options();
							}
						});
					} else {
						do_update_setting_real(setting, new_value, meta);
					}
				};

				name_td.appendChild(name);
				tr.appendChild(name_td);

				var value_td = document.createElement("td");
				value_td.classList.add("value_td");

				var type = "options";
				var option_list = {};

				if (typeof orig_value === "boolean") {
					type = "options";
					option_list["true"] = {name: _("yes")};
					option_list["false"] = {name: _("no")};
					if (value)
						option_list["true"].checked = true;
					else
						option_list["false"].checked = true;
				} else if (meta.options) {
					if (meta.options._type === "combo") {
						type = "combo";
					} else {
						type = "options";
						option_list = deepcopy(meta.options);

						var check_optionlist = function (val, list) {
							if (val in list) {
								list[val].checked = true;
							} else {
								for (var item in list) {
									if (item.match(/^_group/)) {
										check_optionlist(val, list[item]);
									}
								}
							}
						};

						if (is_array(value)) {
							value.forEach(function (val) {
								check_optionlist(val, option_list);
							});
						} else {
							check_optionlist(value, option_list);
						}
					}
				} else if (meta.type) {
					if (meta.type === "textarea" ||
						meta.type === "keysequence" ||
						meta.type === "number" ||
						meta.type === "lineedit")
						type = meta.type
				}

				if (type === "options") {
					var option_type = option_list._type;
					if (!option_type)
						option_type = "or";

					var add_setting = function(parent, op, val, option_type, group, group_type) {
						var id = "input_" + setting + "_" + op;
						var name = setting;
						if (group && group_type === "and")
							name += "_" + group;

						var input = document.createElement("input");
						if (option_type === "or" && false)
							input.setAttribute("type", "radio");
						else if (option_type === "and" || true)
							input.setAttribute("type", "checkbox");
						input.name = name;
						input.value = op;
						input.id = id;
						if (val.checked)
							input.setAttribute("checked", "true");

						input.addEventListener("change", function(event) {
							var value = this.value;

							if (value === "true") {
								value = true;
							}

							if (value === "false")
								value = false;

							if (this.checked) {
								if (group && group_type === "or") {
									for (var child_i = 0; child_i < value_td.children.length; child_i++) {
										var child = value_td.children[child_i];
										if (child.id !== (setting + group)) {
											for (var subchild_i = 0; subchild_i < child.children.length; subchild_i++) {
												var subchild = child.children[subchild_i];
												if (subchild.tagName === "INPUT") {
													subchild.checked = false;
												}
											}
										}
									}
								}

								if (option_type === "or") {
									for (var child_i = 0; child_i < parent.children.length; child_i++) {
										var child = parent.children[child_i];
										if (child.tagName === "INPUT" && child.id != input.id) {
											child.checked = false;
										}
									}
								}
							} else {
								var onechecked = false;
								for (var child_i = 0; child_i < value_td.children.length; child_i++) {
									var child = value_td.children[child_i];
									for (var subchild_i = 0; subchild_i < child.children.length; subchild_i++) {
										var subchild = child.children[subchild_i];
										if (subchild.tagName === "INPUT") {
											if (subchild.checked) {
												onechecked = true;
												break;
											}
										}
									}
									if (onechecked)
										break;
								}

								if (!onechecked) {
									this.checked = true;
								}
							}

							var new_value = value;
							if (group || option_type !== "or") {
								var out_value = [];

								var inputs = value_td.getElementsByTagName("input");
								for (var child_i = 0; child_i < inputs.length; child_i++) {
									var child = inputs[child_i];
									if (child.checked) {
										out_value.push(child.value);
									}
								}

								new_value = out_value;
								do_update_setting(setting, new_value, meta);
							} else {
								do_update_setting(setting, value, meta);
							}

							//settings[setting] = new_value;
							check_disabled_options();
							show_warnings();
						});

						parent.appendChild(input);

						var label = document.createElement("label");
						label.setAttribute("for", id);
						label.innerText = _(val.name);

						if (val.description) {
							label.title = _(val.description);
						}

						parent.appendChild(label);
					};

					for (var op in option_list) {
						if (option_list[op].extension_only && !is_extension)
							continue;

						if (op.match(/^_group/)) {
							var option_type1 = option_list[op]._type;
							if (!option_type1)
								option_type1 = "or";

							var sub = document.createElement("div");
							sub.classList.add("group");
							sub.id = setting + op;
							for (var op1 in option_list[op]) {
								if (!op1.match(/^_/))
									add_setting(sub, op1, option_list[op][op1], option_type1, op, option_type);
							}
							value_td.appendChild(sub);
						} else if (!op.match(/^_/)) {
							add_setting(value_td, op, option_list[op], option_type);
						}
					}
				} else if (type === "textarea") {
					var sub = document.createElement("table");
					var sub_tr = document.createElement("tr");
					var sub_ta_td = document.createElement("td");
					sub_ta_td.style.verticalAlign = "middle";
					//sub_ta_td.style.height = "1px";
					var sub_button_tr = document.createElement("tr");
					var sub_button_td = document.createElement("td");
					sub_button_td.style.textAlign = "center";
					//sub_button_td.style.verticalAlign = "middle";
					//sub_button_td.style.height = "1px";
					var textarea = document.createElement("textarea");
					textarea.style.height = "5em";
					textarea.style.width = "20em";
					if (value)
						textarea.value = value;
					var savebutton = document.createElement("button");
					savebutton.innerText = _("save");
					savebutton.onclick = function() {
						do_update_setting(setting, textarea.value, meta);

						// Background CSS style unlocks Background fade
						check_disabled_options();
						//settings[setting] = textarea.value;
					};

					sub_ta_td.appendChild(textarea);
					sub_button_td.appendChild(savebutton);
					sub_button_tr.appendChild(sub_button_td);
					sub_tr.appendChild(sub_ta_td);
					sub.appendChild(sub_tr);
					sub.appendChild(sub_button_tr);

					value_td.appendChild(sub);
				} else if (type === "number" || type === "lineedit") {
					var sub = document.createElement("table");
					var sub_tr = document.createElement("tr");
					var sub_in_td = document.createElement("td");
					sub_in_td.style = "display:inline";
					var input = document.createElement("input");

					if (false && type === "number") {
						// doesn't work properly on Waterfox, most of the functionality is implemented here anyways
						// thanks to decembre on github for reporting: https://github.com/qsniyg/maxurl/issues/14#issuecomment-531080061
						input.type = "number";
					} else {
						input.type = "text";
					}

					input.setAttribute("spellcheck", false);

					if (type === "number") {
						input.style = "text-align:right";
						if (meta.number_max !== undefined)
							input.setAttribute("max", meta.number_max.toString());
						if (meta.number_min !== undefined)
							input.setAttribute("min", meta.number_min.toString());
						if (meta.number_int)
							input.setAttribute("step", "1");
					}

					if (value !== undefined)
						input.value = value;
					input.oninput = function(x) {
						var value = input.value.toString();

						if (type === "number") {
							var value = parseFloat(value);
							var orig_value = value;

							if (isNaN(value)) {
								return;
							}

							if (meta.number_int) {
								value = parseInt(value);
							}

							if (meta.number_max !== undefined)
								value = Math.min(value, meta.number_max);
							if (meta.number_min !== undefined)
								value = Math.max(value, meta.number_min);

							if (isNaN(value)) {
								console_error("Error: number is NaN after min/max");
								return;
							}

							if (meta.number_int || value !== orig_value)
								input.value = value;

							value = parseFloat(value);
						}

						do_update_setting(setting, value, meta);
					}

					var sub_units_td = document.createElement("td");
					//sub_units_td.style = "display:inline";
					sub_units_td.classList.add("number_units");
					if (meta.number_unit)
						sub_units_td.innerText = _(meta.number_unit);

					sub_tr.appendChild(input);
					sub_tr.appendChild(sub_units_td);
					sub.appendChild(sub_tr);
					value_td.appendChild(sub);
				} else if (type === "keysequence") {
					var sub = document.createElement("table");

					var values = deepcopy(value);

					if (values.length > 0 && !is_array(values[0]))
						values = [values];

					var indices = [];
					for (var i = 0; i < values.length; i++) {
						indices.push(i);
					}

					var is_only_keyseq = function() {
						var active_indices = 0;
						for (var i = 0; i < indices.length; i++) {
							if (indices[i] >= 0)
								active_indices++;
						}

						return active_indices < 2;
					};

					var update_keyseq_setting = function() {
						var result = [];

						for (var i = 0; i < indices.length; i++) {
							if (indices[i] >= 0 && values[i].length > 0) {
								result.push(values[i]);
							}
						}

						do_update_setting(setting, result, meta);
					};

					var recalculate_removebtns = function() {
						var do_remove = !meta.keyseq_allow_none && is_only_keyseq();

						for (var i = 0; i < sub.children.length; i++) {
							var child = sub.children[i];

							var removebtns = child.getElementsByClassName("removebtn");
							if (removebtns.length > 0) {
								if (do_remove) {
									removebtns[0].style.display = "none";
								} else {
									removebtns[0].style.display = "initial";
								}
							}
						}
					};

					var add_keyseq_tr = function(index, start_recording) {
						var sub_tr = document.createElement("tr");
						sub_tr.classList.add("keyseq");

						var sub_key_td = document.createElement("td");
						//sub_key_td.style = "display:inline;font-family:monospace";
						sub_key_td.classList.add("record_keybinding");
						if (value) {
							sub_key_td.innerText = get_trigger_key_texts(values)[index];
						}

						var sub_record_td = document.createElement("td");
						sub_record_td.style = "display:inline";

						var sub_record_btn = document.createElement("button");
						sub_record_btn.innerText = _("record");

						var do_record = function() {
							if (recording_keys) {
								if (keysequence_valid(options_chord)) {
									values[index] = options_chord;
									update_keyseq_setting();
									//do_update_setting(setting, options_chord, meta);
									//settings[setting] = options_chord;

									do_cancel();
								}
							} else {
								options_chord = [];
								current_options_chord = [];
								recording_keys = function() {
									var our_chord = options_chord;
									if (our_chord.length === 0)
										our_chord = values[index];

									sub_key_td.innerText = get_trigger_key_texts(our_chord);

									if (keysequence_valid(options_chord)) {
										sub_record_btn.classList.remove("disabled");
									} else {
										sub_record_btn.classList.add("disabled");
									}
								};
								sub_record_btn.innerText = _("save");
								sub_cancel_btn.style = "display:inline-block";
							}
						};
						sub_record_btn.onmousedown = do_record;

						var sub_cancel_btn = document.createElement("button");
						sub_cancel_btn.innerText = _("cancel");
						sub_cancel_btn.style = "display:none";

						var do_cancel = function() {
							recording_keys = false;
							sub_record_btn.innerText = _("Record");
							sub_record_btn.classList.remove("disabled");
							sub_cancel_btn.style = "display:none";
							sub_key_td.innerText = get_trigger_key_texts(values)[index];
						};
						sub_cancel_btn.onmousedown = do_cancel;

						var sub_remove_btn = document.createElement("button");
						//sub_remove_btn.innerText = "—";
						sub_remove_btn.innerText = "\xD7";
						sub_remove_btn.title = _("Remove");
						sub_remove_btn.classList.add("removebtn");
						sub_remove_btn.classList.add("small");

						if (!meta.keyseq_allow_none && is_only_keyseq()) {
							sub_remove_btn.style = "display:none";
						}

						sub_remove_btn.onclick = function() {
							if (!meta.keyseq_allow_none && is_only_keyseq())
								return;

							recording_keys = false;

							indices[index] = -1;
							for (var i = index + 1; i < indices.length; i++) {
								indices[i]--;
							}

							sub_tr.parentElement.removeChild(sub_tr);
							recalculate_removebtns();

							update_keyseq_setting();
						};

						sub_tr.appendChild(sub_key_td);
						sub_record_td.appendChild(sub_record_btn);
						sub_record_td.appendChild(sub_cancel_btn);
						sub_record_td.appendChild(sub_remove_btn);
						sub_tr.appendChild(sub_record_td);

						if (start_recording)
							do_record();

						return sub_tr;
					};

					for (var i = 0; i < indices.length; i++) {
						sub.appendChild(add_keyseq_tr(i));
					}

					var sub_add_tr = document.createElement("tr");
					var sub_add_td = document.createElement("td");
					var sub_add_btn = document.createElement("button");
					sub_add_btn.innerText = "+";
					sub_add_btn.title = _("Add keybinding");
					sub_add_btn.classList.add("small");
					sub_add_btn.onclick = function() {
						var last_index = -1;

						for (var i = indices.length - 1; i >= 0; i--) {
							if (indices[i] >= 0) {
								last_index = indices[i];
								break;
							}
						}

						indices.push(last_index + 1);
						values.push([]);

						sub.insertBefore(add_keyseq_tr(indices.length - 1, true), sub_add_tr);
						recalculate_removebtns();
					};
					sub_add_td.appendChild(sub_add_btn);
					sub_add_tr.appendChild(sub_add_td);

					sub.appendChild(sub_add_tr);
					value_td.appendChild(sub);
				} else if (type === "combo") {
					var sub = document.createElement("select");

					for (var coption in meta.options) {
						if (!coption || coption[0] === '_')
							continue;

						var optionel = document.createElement("option");
						optionel.innerText = _(meta.options[coption].name);
						optionel.value = coption;

						sub.appendChild(optionel);
					}

					sub.value = settings[setting];

					sub.onchange = function() {
						do_update_setting(setting, sub.value, meta);
					};

					value_td.appendChild(sub);
				}

				tr.appendChild(value_td);

				option.appendChild(table);

				if (settings.settings_visible_description) {
					var description_el = document.createElement("p");
					md_to_html(description_el, description);
					//description_el.innerText = description;
					description_el.classList.add("description");

					option.appendChild(description_el);
				}

				if (meta.warning) {
					var warning = document.createElement("p");
					warning.style.display = "none";
					warning.classList.add("warning");

					option.appendChild(warning);
				}

				var requirements = document.createElement("div");
				requirements.style.display = "none";
				requirements.classList.add("requirements");
				option.appendChild(requirements);

				if (meta.example_websites) {
					var examples = document.createElement("ul");
					examples.classList.add("examples");
					for (var example_i = 0; example_i < meta.example_websites.length; example_i++) {
						var example_text = meta.example_websites[example_i];
						var example_el = document.createElement("li");
						example_el.innerText = _(example_text);
						examples.appendChild(example_el);
					}

					option.appendChild(examples);
				}

				if (meta.documentation) {
					var get_title = function(expanded) {
						var arrow = "⯈";
						if (expanded) {
							arrow = "⯆";
						}

						return arrow + " " + _(meta.documentation.title);
					};

					var text = get_title(false);

					var spoiler_title = document.createElement("span");
					spoiler_title.classList.add("spoiler-title");
					spoiler_title.innerText = text;

					var expanded = false;
					spoiler_title.onclick = function() {
						expanded = !expanded;

						if (expanded) {
							spoiler_contents.style.display = "block";
						} else {
							spoiler_contents.style.display = "none";
						}

						spoiler_title.innerText = get_title(expanded);
					};

					var spoiler_contents = document.createElement("div");
					spoiler_contents.classList.add("spoiler-contents");
					spoiler_contents.style.display = "none";
					spoiler_contents.innerHTML = meta.documentation.value;

					option.appendChild(spoiler_title);
					option.appendChild(spoiler_contents);
				}

				var errordiv = document.createElement("div");
				errordiv.classList.add("error");
				option.appendChild(errordiv);

				if (meta.category) {
					var subcat = meta.category;
					if (meta.subcategory)
						subcat = meta.subcategory;

					subcategory_els[subcat].appendChild(option);
				} else {
					options_el.appendChild(option);
				}
			})(setting);
		}

		check_disabled_options();
		show_warnings();

		for (var category in category_els) {
			var our_els = category_els[category]

			if (!(category in category_settings)) {
				for (var i = 0; i < our_els.length; i++) {
					our_els[i].parentNode.removeChild(our_els[i])
				}
			}
		}

		for (var subcategory in subcategory_els) {
			var our_el = subcategory_els[subcategory];

			if (our_el.querySelectorAll(".option").length === 0) {
				our_el.parentNode.removeChild(our_el);
			}
		}

		document.body.appendChild(saved_el);
	}

	function get_single_setting_raw(value) {
		if (is_array(value))
			return value[0];
		return value;
	}

	function get_single_setting(setting) {
		return get_single_setting_raw(settings[setting]);
	}

	function parse_value(value) {
		try {
			return JSON_parse(value);
		} catch (e) {
			return value;
		}
	}

	function serialize_value(value) {
		return JSON_stringify(value);
	}

	function get_value(key, cb) {
		if (is_extension) {
			extension_send_message({
				type: "getvalue",
				data: [key]
			}, function(response) {
				response = response.data;
				cb(parse_value(response[key]));
			});
		} else if (typeof GM_getValue !== "undefined" &&
				   // Unfortunately FireMonkey currently implements GM_getValue as a mapping to GM.getValue for some reason
				   //   https://github.com/erosman/support/issues/98
				   // Until this is fixed, we cannot use GM_getValue for FireMonkey
				   userscript_manager !== "FireMonkey") {
			return cb(parse_value(GM_getValue(key, undefined)));
		} else if (typeof GM !== "undefined" && GM.getValue) {
			GM.getValue(key, undefined).then(function (value) {
				cb(parse_value(value));
			});
		}
	}

	var updating_options = 0;
	function set_value(key, value, cb) {
		if (key in settings_meta && settings_meta[key].onedit) {
			settings_meta[key].onedit(value);
		}

		value = serialize_value(value);
		//console_log("Setting " + key + " = " + value);

		if (is_extension) {
			var kv = {};
			kv[key] = value;
			//chrome.storage.sync.set(kv, function() {});
			updating_options++;
			extension_send_message({
				type: "setvalue",
				data: kv
			}, function() {
				updating_options--;

				cb && cb();
			});
		} else if (typeof GM_setValue !== "undefined") {
			GM_setValue(key, value);

			cb && cb();
		} else if (typeof GM !== "undefined" && GM.getValue) {
			GM.setValue(key, value).then(function() {
				cb && cb();
			});
		}
	}

	function update_setting(key, value) {
		if (value === settings[key])
			return false;

		value = deepcopy(value);
		settings[key] = value;

		if (is_extension) {
			if (!(key in settings_history))
				settings_history[key] = [];

			settings_history[key].push(value);
		}

		set_value(key, value);
		return true;
	}

	function set_default_value(key, value) {
		if (!(key in user_defined_settings)) {
			settings[key] = value;
		}
	}

	function settings_updated_cb(changes) {
		if (!settings.allow_live_settings_reload)
			return;

		//console_log(message);
		var changed = false;

		for (var key in changes) {
			if (changes[key].newValue === undefined)
				continue;

			//console_log("Setting " + key + " = " + changes[key].newValue);
			var newvalue = JSON_parse(changes[key].newValue);
			if (key in settings_history) {
				var index = array_indexof(settings_history[key], newvalue);

				var pass = false
				if (index >= 0 && index < settings_history[key].length - 1) {
					pass = true;
				}

				settings_history[key].splice(index, 1);

				if (pass)
					continue;
			}

			var setting_updated = update_setting_from_host(key, newvalue);
			changed = setting_updated || changed;

			if (setting_updated && key in settings_meta && "onupdate" in settings_meta[key]) {
				settings_meta[key].onupdate();
			}
		}

		if (changed && updating_options <= 0 && is_options_page) {
			//console_log("Refreshing options");
			do_options();
		}
	};

	function upgrade_settings_with_version(version, new_settings, cb) {
		if (!version) {
			version = 0;
		} else if (typeof version !== "number") {
			version = parseInt(version);
			if (isNaN(version))
				version = 0;
		}

		if (new_settings === undefined)
			new_settings = settings;

		var changed = false;

		if (version === 0) {
			if (new_settings.mouseover_trigger) {
				var trigger_keys = [];
				for (var i = 0; i < new_settings.mouseover_trigger.length; i++) {
					var trigger = new_settings.mouseover_trigger[i];
					if (trigger.match(/^delay_[0-9]+/)) {
						var delay = parseInt(new_settings.mouseover_trigger[i].replace(/^delay_([0-9]+).*?$/, "$1"));
						if (delay <= 0 || isNaN(delay))
							delay = false;
						if (typeof delay === "number" && delay >= 10)
							delay = 10;
						update_setting("mouseover_trigger_delay", delay);
						continue;
					}

					trigger_keys.push(trigger);
				}

				if (trigger_keys.length === 0) {
					update_setting("mouseover_trigger_key", orig_settings["mouseover_trigger_key"]);
					update_setting("mouseover_trigger_behavior", "mouse");
				} else {
					update_setting("mouseover_trigger_key", trigger_keys);
					update_setting("mouseover_trigger_behavior", "keyboard");
				}
			}

			update_setting("settings_version", 1);
			changed = true;

			version = 1;
		}

		if (version === 1) {
			var partial_setting = "none";
			var partial_setting_set = new_settings.mouseover_use_fully_loaded_video !== undefined ||
									  new_settings.mouseover_use_fully_loaded_image !== undefined;

			if (partial_setting_set) {
				if (new_settings.mouseover_use_fully_loaded_video === false ||
					new_settings.mouseover_use_fully_loaded_video === undefined) {
					partial_setting = "video";
				}

				if (new_settings.mouseover_use_fully_loaded_image === undefined) {
					if (!orig_settings.mouseover_use_fully_loaded_image) {
						partial_setting = "media";
					}
				} else if (new_settings.mouseover_use_fully_loaded_image === false) {
					partial_setting = "media";
				}

				update_setting("mouseover_allow_partial", partial_setting);
			}

			update_setting("settings_version", 2);
			changed = true;

			version = 2;
		}

		if (version === 2) {
			if ("mouseover_close_on_leave_el" in new_settings) {
				var policy;

				if (new_settings.mouseover_close_on_leave_el) {
					policy = "both";
				} else {
					policy = "popup";
				}

				update_setting("mouseover_close_el_policy", policy);
			}

			update_setting("settings_version", 3);
			changed = true;

			version = 3;
		}

		if (version === 3) {
			if ("mouseover_scroll_behavior" in new_settings) {
				if (get_single_setting_raw(new_settings.mouseover_scroll_behavior) !== "zoom") {
					update_setting("mouseover_scrollx_behavior", new_settings.mouseover_scroll_behavior);
				}

				update_setting("mouseover_scrolly_behavior", new_settings.mouseover_scroll_behavior);
			}

			update_setting("settings_version", 4);
			changed = true;

			version = 4;
		}

		if (version === 4) {
			if ("mouseover_mask_styles" in new_settings && new_settings.mouseover_mask_styles) {
				update_setting("mouseover_mask_styles2", new_settings.mouseover_mask_styles);
				update_setting("mouseover_enable_mask_styles", true);
			}

			update_setting("settings_version", 5);
			changed = true;

			version = 5;
		}

		cb(changed);
	}

	function upgrade_settings(cb) {
		try {
			create_blacklist_regexes();
		} catch(e) {
			console_error(e);
		}

		if (!settings.last_update_check) {
			update_setting("last_update_check", Date.now());
		}

		check_updates_if_needed();

		// TODO: merge this get_value in do_config for performance
		get_value("settings_version", function(version) {
			upgrade_settings_with_version(version, settings, cb);
		});
	}

	function update_setting_from_host(setting, value) {
		if (value !== undefined) {
			if (typeof settings[setting] === "number") {
				value = parseFloat(value);
			}

			user_defined_settings[setting] = value;

			if (value !== settings[setting]) {
				settings[setting] = value;
				return true;
			}
		}

		return false;
	}

	function do_config() {
		if (is_userscript || is_extension) {
			var settings_done = 0;
			var total_settings = Object.keys(settings).length + old_settings_keys.length;

			var add_value_change_listeners = function(cb) {
				if (typeof GM_addValueChangeListener === "undefined") {
					return cb();
				}

				for (var setting in settings) {
					GM_addValueChangeListener(setting, function(name, oldValue, newValue, remote) {
						if (remote === false)
							return;

						var updated = {};
						updated[name] = {newValue: newValue};
						settings_updated_cb(updated);
					});
				}

				cb();
			};

			var process_setting = function(setting) {
				get_value(setting, function(value) {
					settings_done++;
					update_setting_from_host(setting, value);

					if (settings_done >= total_settings)
						upgrade_settings(function(value) {
							add_value_change_listeners(function() {
								start(value);
							});
						});
				});
			};

			for (var setting in settings) {
				process_setting(setting);
			}

			for (var i = 0; i < old_settings_keys.length; i++) {
				process_setting(old_settings_keys[i]);
			}
		} else {
			start();
		}
	}

	function set_el_all_initial(el) {
		el.style.all = "initial";

		// Under Waterfox, if offsetInlineStart is set to anything (even unset), it'll set the left to 0
		// Thanks to decembre on github for reporting this: https://github.com/qsniyg/maxurl/issues/14#issuecomment-531080061
		el.style.removeProperty("offset-inline-start");
	}

	function check_bad_if(badif, resp) {
		if (_nir_debug_)
			console_log("check_bad_if", badif, resp);

		if (!badif || !is_array(badif) || badif.length === 0) {
			if (_nir_debug_)
				console_log("check_bad_if (!badif)");
			return false;
		}

		var headers = parse_headers(resp.responseHeaders);

		var check_single_badif = function(badif) {
			if (badif.headers) {
				for (var header in badif.headers) {
					var header_lower = header.toLowerCase();

					var found = false;
					for (var i = 0; i < headers.length; i++) {
						if (headers[i].name.toLowerCase() === header_lower) {
							if (typeof (badif.headers[header]) === "function") {
								found = badif.headers[header](headers[i].value);
							} else if (typeof (badif.headers[header]) === "string") {
								found = headers[i].value === badif.headers[header];
							}

							if (found) {
								break;
							} else {
								return false;
							}
						}
					}

					if (!found)
						return false;
				}
			}

			return true;
		};

		for (var j = 0; j < badif.length; j++) {
			if (check_single_badif(badif[j]))
				return true;
		}

		return false;
	}
	bigimage_recursive.check_bad_if = check_bad_if;

	function is_probably_video(obj) {
		if (obj.video)
			return true;

		if (/\.(?:mp4|webm|mkv|mpg|ogv|wmv)/i.test(obj.url))
			return true;

		return false;
	}

	function is_video_contenttype(contenttype) {
		if (/^\s*\[?video\//.test(contenttype))
			return true;

		// https://upload.wikimedia.org/wikipedia/commons/7/74/Leucochloridium.ogv
		if (/^application\/og[gv]$/.test(contenttype))
			return true;

		return false;
	}

	var is_video_type_supported = function(videotype) {
		if (videotype === true || videotype === "direct") {
			return true;
		}

		if (videotype === "dash") {
			return settings.allow_thirdparty_libs && settings.allow_dash_video;
		}

		if (videotype === "hls") {
			return settings.allow_thirdparty_libs && settings.allow_hls_video;
		}

		return false;
	};

	function get_event_error(e) {
		// https://stackoverflow.com/a/46064096
		var error = e;

		if (e.path && e.path[0]) {
			error = e.path[0].error;
		}

		if (e.originalTarget) {
			error = e.originalTarget.error;
		}

		return error;
	}

	var trigger_gallery;

	function serialize_img(img) {
		var obj = {
			tag: img.tagName.toLowerCase(),
			src: img.src,
			autoplay: img.getAttribute("autoplay"),
			controls: img.getAttribute("controls"),
			loop: img.getAttribute("loop"),
			muted: img.muted,
			volume: img.volume
		};

		return obj;
	}

	function deserialize_img(obj, cb) {
		var el = document.createElement(obj.tag);
		if (obj.tag === "video") {
			if (obj.autoplay)
				video.setAttribute("autoplay", obj.autoplay);
			if (obj.controls)
				video.setAttribute("controls", obj.controls);
			if (obj.loop)
				video.setAttribute("loop", obj.loop)
			if (obj.muted)
				video.muted = obj.muted;
			if (obj.volume !== undefined)
				video.volume = obj.volume;

			video.onloadedmetadata = function() {
				cb(el);
			};
		}

		el.src = obj.src;

		if (obj.tag !== "video") {
			cb(el);
		}
	}

	var get_window_url = function() {
		return window.URL || window.webkitURL;
	};

	var create_dataurl = function(blob, cb) {
		var a = new FileReader();
		a.onload = function(e) {
			try {
				cb(e.target.result);
			} catch (e) {
				console_error(e);
				console_error(e.stack);
				cb(null);
			}
		};
		a.readAsDataURL(blob);
	};

	var create_objecturl = function(blob) {
		return get_window_url().createObjectURL(blob);
	};

	var revoke_objecturl = function(objecturl) {
		if (is_element(objecturl))
			objecturl = objecturl.src;

		return get_window_url().revokeObjectURL(objecturl);
	};

	var is_video_el = function(el) {
		if (el.tagName === "VIDEO")
			return true;

		if (el.tagName !== "SOURCE")
			return false;

		if (el.parentElement && el.parentElement.tagName === "VIDEO")
			return true;
		return false;
	}

	function check_image_get(obj, cb, processing) {
		if (_nir_debug_) {
			console_log("check_image_get", obj, cb, processing);
		}

		if (!obj || !obj[0]) {
			return cb(null);
		}

		if (!processing.running) {
			return cb(null);
		}

		var method = "GET";
		var responseType = "blob";
		var last_objecturl = null;

		var incomplete_request = false;
		if (processing.incomplete_image || (is_probably_video(obj[0]) && processing.incomplete_video))
			incomplete_request = true;

		if (obj[0].need_blob)
			incomplete_request = false;

		if (processing.head || incomplete_request) {
			if (incomplete_request && !obj.can_head) {
				method = "GET";
			} else {
				method = "HEAD";
			}

			responseType = undefined;
		}

		if (obj[0].url.match(/^data:/)) {
			var img = document.createElement("img");
			img.src = obj[0].url;
			img.onload = function() {
				cb(img, obj[0].url, obj[0]);
			};
			img.onerror = function(e) {
				console_log("Error loading image", e);
				err_cb();
			};
			return;
		}

		function err_cb() {
			revoke_objecturl(last_objecturl);
			obj.shift();

			if (_nir_debug_) {
				console_log("check_image_get(err_cb):", obj, processing);
			}

			return check_image_get(obj, cb, processing);
		}

		var url = obj[0].url;

		console_log("Trying " + url);

		if (obj[0] && obj[0].video && !settings.allow_video) {
			console_log("Video, skipping due to user setting");
			return err_cb();
		}

		if (obj[0] && obj[0].bad) {
			console_log("Bad image");
			return err_cb();
		}

		var url_domain = url.replace(/^([a-z]+:\/\/[^/]*).*?$/, "$1");

		var headers = obj[0].headers;

		if (!headers || Object.keys(headers).length === 0) {
			headers = {
				//"Origin": url_domain,
				"Referer": window.location.href
			};
		} else if (!headers.Origin && !headers.origin) {
			//headers.Origin = url_domain;
		}

		var handled = false;
		var onload_cb = function(resp) {
			if (handled) {
				return;
			}

			handled = true;

			if (!processing.running) {
				return cb(null);
			}

			if (resp.readyState == 4 || true) {
				if (_nir_debug_) {
					console_log("check_image_get(onload)", resp, resp.readyState);
				}

				var digit = resp.status.toString()[0];

				var ok_error = check_ok_error(obj[0].head_ok_errors, resp.status);

				if (((digit === "4" || digit === "5") &&
					 resp.status !== 405) && obj[0].can_head &&
					ok_error !== true) {
					if (err_cb) {
						console_log("Bad status: " + resp.status + " ( " + url + " )");
						err_cb();
					} else {
						console_error("Error: " + resp.status);
					}

					return;
				}

				if (check_bad_if(obj[0].bad_if, resp)) {
					console_log("Bad image (bad_if)", resp, obj[0].bad_if);
					return err_cb();
				}

				if (processing.head) {
					cb(resp, obj[0]);
					return;
				}

				var parsed_headers = headers_list_to_dict(parse_headers(resp.responseHeaders));
				var is_video = false;
				var video_type = "direct";

				// TODO: improve
				if (obj[0].video || parsed_headers["content-type"] && is_video_contenttype(parsed_headers["content-type"])) {
					is_video = true;

					if (obj[0].video && obj[0].video !== true) {
						video_type = obj[0].video;
					}

					if (!is_video_type_supported(video_type)) {
						console_warn("Video type", video_type, "is not supported");
						return err_cb();
					}
				}

				if (is_video && !settings.allow_video) {
					console_log("Video, skipping due to user setting");
					return err_cb();
				}

				if (settings.mouseover_matching_media_types && processing && processing.source && processing.source.el) {
					if (is_video_el(processing.source.el)) {
						if (!is_video) {
							console_log("!video, source was video, skipping");
							return err_cb();
						}
					} else {
						if (is_video) {
							console_log("video, source was image, skipping");
							return err_cb();
						}
					}
				}

				var good_cb = function(img) {
					if (_nir_debug_) {
						console_log("check_image_get(good_cb):", img, resp.finalUrl, obj[0], resp);
						console_trace();
					}

					cb(img, resp.finalUrl, obj[0], resp);
				};

				var create_video_el = function() {
					var video = document.createElement("video");

					video.setAttribute("autoplay", "autoplay");

					if (settings.mouseover_video_controls)
						video.setAttribute("controls", "controls");

					if (settings.mouseover_video_loop && !settings.mouseover_gallery_move_after_video)
						video.setAttribute("loop", true);

					if (settings.mouseover_video_muted)
						video.muted = true;
					else {
						var volume = parseInt(settings.mouseover_video_volume);
						volume = Math.max(Math.min(volume, 100), 0);
						video.volume = volume / 100.;
					}

					var remove_loaded_metadata_listener = function() {
						video.onloadedmetadata = null;
						video.removeEventListener("loadedmetadata", loaded_metadata_listener);
					};

					var errorhandler = function(e) {
						console_error("Error loading video", get_event_error(e));

						remove_loaded_metadata_listener();
						err_cb();
					};

					var ran_loadedmetadata_listener = false;
					var loaded_metadata_listener = function() {
						if (!ran_loadedmetadata_listener) {
							ran_loadedmetadata_listener = true;
						} else {
							return;
						}

						video.removeEventListener("error", errorhandler, true);
						remove_loaded_metadata_listener();

						if (settings.mouseover_video_resume_from_source && processing.source && processing.source.el) {
							var sourceel = processing.source.el;
							if (sourceel.tagName === "SOURCE") {
								sourceel = sourceel.parentElement;
							}

							if (sourceel.tagName === "VIDEO" && sourceel.currentTime) {
								video.currentTime = sourceel.currentTime;
							}
						}

						run_soon(function() {
							good_cb(video)
						});
					};

					video.onloadedmetadata = loaded_metadata_listener;
					video.addEventListener("loadedmetadata", loaded_metadata_listener);

					video.onended = function() {
						if (settings.mouseover_enable_gallery && settings.mouseover_gallery_move_after_video) {
							trigger_gallery(1);
						}
					};

					video.addEventListener("error", errorhandler, true);

					return video;
				};

				var set_video_src = function(video, src) {
					if (video_type === "direct") {
						video.src = src;
					} else {
						if (video_type === "dash") {
							get_library("dash", settings, do_request, function(dashjs) {
								if (!dashjs) {
									video.src = src;
									return;
								}

								var player = dashjs.MediaPlayer().create();
								player.updateSettings({
									streaming: {
										abr: {
											initialBitrate: {
												audio: Number.MAX_SAFE_INTEGER,
												video: Number.MAX_SAFE_INTEGER
											},
											autoSwitchBitrate: {
												audio: false,
												video: false
											}
										}
									}
								});
								player.initialize(video, src, true);
							});
						} else if (video_type === "hls") {
							get_library("hls", settings, do_request, function(hls_wrap) {
								if (!hls_wrap || !hls_wrap.Hls || !hls_wrap.Hls.isSupported()) {
									// this will work if (video.canPlayType('application/vnd.apple.mpegurl'))
									// if not, it will fail, then go to the next URL
									video.src = src;
									return;
								}

								var Hls = hls_wrap.Hls;
								var hls = new Hls();

								hls.loadSource(src);
								hls.attachMedia(video);
								hls.on(Hls.Events.MANIFEST_PARSED, function() {
									hls.startLoad(-1);
									video.play();
								});

								hls.on(Hls.Events.ERROR, function(e) {
									console_error(e);
									// TODO: run error handler
								});
							});
						}
					}
				};

				if (incomplete_request) {
					var load_image;

					if (!is_video) {
						load_image = function () {
							var img = document.createElement("img");
							img.src = resp.finalUrl;

							var end_cbs = function () {
								clearInterval(height_interval);
								img.onload = null;
								img.onerror = null;
							};

							img.onload = function () {
								end_cbs();

								if (img.naturalWidth === 0 || img.naturalHeight === 0) {
									if (_nir_debug_)
										console_log("naturalWidth or naturalHeight == 0", img);

									return err_cb();
								}

								good_cb(img);
							};

							img.onerror = function (e) {
								if (_nir_debug_)
									console_log("Error loading image", img, e);

								end_cbs();
								err_cb();
							};

							var height_interval = setInterval(function () {
								if (img.naturalWidth !== 0 && img.naturalHeight !== 0) {
									end_cbs();
									good_cb(img);
								}
							}, 15);
						};
					} else {
						load_image = function () {
							var video = create_video_el();
							set_video_src(video, resp.finalUrl);
						};
					}

					if (is_extension) {
						extension_send_message({
							type: "override_next_headers",
							data: {
								url: resp.finalUrl,
								method: "GET",
								headers: headers
							}
						}, function() {
							load_image();
						});
					} else {
						load_image();
					}

					return;
				}

				if (!resp.response) {
					err_cb();
					return;
				}

				var loadcb = function(urldata) {
					if (_nir_debug_) {
						console_log("check_object_get's loadcb", urldata, is_video);
					}

					last_objecturl = urldata;

					if (!urldata) {
						return err_cb();
					}

					if (!is_video) {
						var img = document.createElement("img");
						img.src = urldata;
						img.onload = function() {
							// Firefox thinks SVGs have an empty naturalWidth/naturalHeight
							if (img.naturalWidth === 0 || img.naturalHeight === 0) {
								return err_cb();
							}

							good_cb(img);
						};
						img.onerror = function() {
							err_cb();
						};
					} else {
						var video = create_video_el();
						set_video_src(video, urldata);
					}
				};

				if (!settings.mouseover_use_blob_over_data && !obj[0].need_blob) {
					create_dataurl(resp.response, loadcb);
				} else {
					var objecturl = create_objecturl(resp.response);
					loadcb(objecturl);
				}
			}
		};

		var req = do_request({
			method: method,
			url: url,
			responseType: responseType,
			headers: headers,
			trackingprotection_failsafe: true,
			need_blob_response: method == "GET",
			onprogress: function(resp) {
				var do_abort = function() {
					if (!req || !req.abort) {
						console_warn("Unable to abort request");
						return;
					}

					req.abort();
				};

				if (!processing.running) {
					do_abort();
				}

				if (incomplete_request && resp.readyState >= 2 && resp.responseHeaders) {
					do_abort();
					onload_cb(resp);
				}
			},
			onload: onload_cb
		});
	}

	var str_to_keycode_table = {
		backspace: 8,
		enter: 13,
		shift: 16,
		ctrl: 17,
		alt: 19,
		space: 32,
		left: 37,
		up: 38,
		right: 39,
		down: 40,
		//"super": 91,
		";": 186,
		"=": 187,
		",": 188,
		"-": 189,
		".": 190,
		"/": 191,
		"`": 192,
		"[": 219,
		"\\": 220,
		"]": 221,
		"'": 222
	};

	var keycode_to_str_table = {
		8: "backspace",
		13: "enter",
		16: "shift",
		17: "ctrl",
		18: "alt",
		32: "space",

		37: "left",
		38: "up",
		39: "right",
		40: "down",

		//91: "super",

		// numpad
		97: "1",
		98: "2",
		99: "3",
		100: "4",
		101: "5",
		102: "6",
		103: "7",
		104: "8",
		105: "9",
		111: "/",
		106: "*",
		107: "+",
		109: "-",
		110: ".",

		186: ";",
		187: "=",
		188: ",",
		189: "-",
		190: ".",
		191: "/",
		192: "`",
		219: "[",
		220: "\\",
		221: "]",
		222: "'"
	};

	//var maxzindex = 2147483647;
	// some sites have z-index: 99999999999999 (http://www.topstarnews.net/)
	// this gets scaled down to 2147483647 in the elements panel, but it gets seen as higher than 9999* by the browser
	// Number.MAX_VALUE doesn't work at all (z-index doesn't get set)
	var maxzindex = Number.MAX_SAFE_INTEGER;
	// sites like topstarnews under Firefox somehow change sans-serif as the default font
	var sans_serif_font = '"Noto Sans", Arial, Helvetica, sans-serif';

	function keycode_to_str(event) {
		var x = event.which;

		if (event.code) {
			// when pressing Shift
			var match = event.code.match(/^Numpad([0-9]+)$/);
			if (match) {
				return match[1];
			};
		}

		if (x in keycode_to_str_table) {
			return keycode_to_str_table[x];
		}

		if (!((x >= 65 && x <= 90) ||
			  // numbers
			  (x >= 48 && x <= 57))) {
			return;
		}

		return String.fromCharCode(x).toLowerCase();
	}

	function str_to_keycode(x) {
		if (x in str_to_keycode_table) {
			return str_to_keycode_table[x];
		}
		return x.toUpperCase().charCodeAt(0);
	}

	function normalize_keychord(keychord) {
		if (keychord.length === 0)
			return [[]];

		if (!is_array(keychord[0]))
			return [keychord];

		return keychord;
	}

	function do_mouseover() {
		var mouseover_enabled = function() {
			var base_enabled = settings.imu_enabled && settings.mouseover;

			if (settings.apply_blacklist_host && !bigimage_filter(window.location.href))
				return false;

			return base_enabled;
		};

		var mouseover_mouse_enabled = function() {
			return mouseover_enabled() && delay !== false && typeof delay === "number" && delay_mouseonly;
		};

		var mousepos_initialized = false;

		var mouseX = 0;
		var mouseY = 0;
		var mouseAbsX = 0;
		var mouseAbsY = 0;
		var mouse_frame_id = "top";

		var mouseContextX = 0;
		var mouseContextY = 0;
		var mouseAbsContextX = 0;
		var mouseAbsContextY = 0;

		var mouse_in_image_yet = false;
		var mouseDelayX = 0;
		var mouseDelayY = 0;

		var lastX = 0;
		var lastY = 0;

		var processing_list = [];
		var popups = [];
		var mask_el = null;
		var popup_obj = null;
		var popup_objecturl = null;
		var popup_el = null;
		var popup_el_is_video = false;
		var popup_orig_url = null;
		var resetpopup_timeout = null;
		var real_popup_el = null;
		var next_popup_el = null;
		var last_popup_el = null;
		var popup_orig_el = null;
		var popup_el_automatic = false;
		var popup_el_remote = false;
		var popups_active = false;
		var popup_trigger_reason = null;
		var can_close_popup = [false, false];
		var popup_hold = false;
		var popup_zoom_func = nullfunc;
		var popup_wheel_cb = null;
		var popup_update_pos_func = null;
		var popup_hidecursor_func = nullfunc;
		var popup_hidecursor_timer = null;
		var popup_client_rect_cache = null;
		var last_popup_client_rect_cache = 0;
		var dragstart = false;
		var dragstartX = null;
		var dragstartY = null;
		var dragoffsetX = null;
		var dragoffsetY = null;
		var popupOpenX = null;
		var popupOpenY = null;
		var popupOpenLastX = null;
		var popupOpenLastY = null;
		var dragged = false;
		var controlPressed = false;
		var waiting = false;

		var waitingel = null;
		var waitingstyleel = null;
		var waitingel_cursor = null;
		var elwaitingstyleclass = null;
		var elwaitingstyleel = null;
		var waitingsize = 200;

		var current_chord = [];
		var current_chord_timeout = {};
		var release_ignore = [];
		var editing_text = false;

		function resetifout(e) {
			// doesn't work, as e doesn't contain ctrlKey etc.
			if (!trigger_complete(settings.mouseover_trigger_key)) {
				//current_chord = [];
				stop_waiting();
				resetpopups();
			}
		}

		// runs on every focusout, not just window
		/*document.addEventListener("focusout", resetifout);
		  document.addEventListener("blur", resetifout);
		  unsafeWindow.addEventListener("focusout", resetifout);
		  unsafeWindow.addEventListener("blur", resetifout);*/

		if (false) {
			var disable_click = false;
			document.addEventListener("click", function(e) {
				if (disable_click && popups_active && false) {
					e.stopPropagation();
					e.stopImmediatePropagation();

					return true;
					//return false;
				}
			}, true);
		}

		var delay = false;
		var delay_handle = null;
		var delay_handle_triggering = false;
		var delay_mouseonly = true;
		var delay_el = null;

		var get_nonwaitingel = function(x, y) {
			var els = document.elementsFromPoint(x, y);

			for (var i = 0; i < els.length; i++) {
				if (els[i] === waitingel)
					continue;
				return els[i];
			}

			return document.body;
		};

		function update_waiting() {
			if (!waitingel)
				return;

			var x = mouseX;//mouseAbsX;
			var y = mouseY;//mouseAbsY;
			waitingel.style.left = (x - (waitingsize / 2)) + "px";
			waitingel.style.top = (y - (waitingsize / 2)) + "px";
		}

		function start_waiting(el, cursor) {
			if (!cursor)
				cursor = "wait";

			waitingel_cursor = cursor;

			waiting = true;

			if (!settings.mouseover_wait_use_el) {
				if (waitingstyleel) {
					stop_waiting();
				}

				waitingstyleel = document.createElement("style");
				waitingstyleel.innerText = "* {cursor: " + cursor + "!important}";
				document.documentElement.appendChild(waitingstyleel);
				return;
			}

			if (!waitingel) {
				waitingel = document.createElement("div");
				set_el_all_initial(waitingel);
				waitingel.style.zIndex = maxzindex;
				waitingel.style.cursor = cursor;
				waitingel.style.width = waitingsize + "px";
				waitingel.style.height = waitingsize + "px";
				//waitingel.style.pointerEvents = "none"; // works, but defeats the purpose, because the cursor isn't changed
				waitingel.style.position = "fixed";//"absolute";

				var simevent = function(e, eventtype) {
					waitingel.style.display = "none";
					document.elementFromPoint(e.clientX, e.clientY).dispatchEvent(new MouseEvent(eventtype, e));
					waitingel.style.display = "block";
				};

				our_addEventListener(waitingel, "click", function(e) {
					return simevent(e, "click");
				});

				our_addEventListener(waitingel, "contextmenu", function(e) {
					return simevent(e, "contextmenu");
				});

				document.documentElement.appendChild(waitingel);
			}

			waitingel.style.cursor = cursor;
			waitingel.style.display = "block";

			update_waiting();
		}

		function start_progress(el) {
			start_waiting(el, "progress");
		}

		function stop_waiting() {
			waiting = false;

			if (!settings.mouseover_wait_use_el) {
				if (waitingstyleel) {
					waitingstyleel.parentElement.removeChild(waitingstyleel);
					waitingstyleel = null;
				}
			}

			if (waitingel)
				waitingel.style.display = "none";
		}

		var not_allowed_timer = null;
		function cursor_not_allowed() {
			if (!settings.mouseover_enable_notallowed)
				return;

			start_waiting(undefined, "not-allowed");

			if (not_allowed_timer) {
				clearTimeout(not_allowed_timer);
			}

			not_allowed_timer = setTimeout(function() {
				not_allowed_timer = null;

				if (waitingel_cursor === "not-allowed")
					stop_waiting();
			}, settings.mouseover_notallowed_duration);
		};

		function in_clientrect(mouseX, mouseY, rect, border) {
			if (isNaN(border) || border === undefined)
				border = 0;

			if (mouseX >= (rect.left - border) && mouseX <= (rect.right + border) &&
				mouseY >= (rect.top - border) && mouseY <= (rect.bottom + border)) {
				return true;
			} else {
				return false;
			}
		}

		function stop_processing() {
			for (var i = 0; i < processing_list.length; i++) {
				processing_list[i].running = false;
			}
		}

		var clear_resetpopup_timeout = function() {
			if (resetpopup_timeout) {
				clearTimeout(resetpopup_timeout);
				resetpopup_timeout = null;
			}
		};

		var add_resetpopup_timeout = function() {
			if (!settings.mouseover_auto_close_popup || !settings.mouseover_auto_close_popup_time)
				return;

			clear_resetpopup_timeout();

			resetpopup_timeout = setTimeout(resetpopups, settings.mouseover_auto_close_popup_time * 1000);
		};

		var removepopups_timer = null;
		function removepopups() {
			popups.forEach(function (popup) {
				var els = popup.querySelectorAll("img, video");
				for (var i = 0; i < els.length; i++) {
					// TODO: maybe check to make sure it's a blob? according to the spec, this will silently fail, but browsers may print an error
					revoke_objecturl(els[i].src);
				}

				if (popup.parentNode)
					popup.parentNode.removeChild(popup);

				var index = array_indexof(popups, popup);
				if (index > -1) {
					popups.splice(index, 1);
				}
			});

			if (removepopups_timer) {
				clearTimeout(removepopups_timer);
				removepopups_timer = null;
			}
		}

		var remove_mask = function() {
			if (mask_el) {
				if (mask_el.parentElement)
					mask_el.parentElement.removeChild(mask_el);
				mask_el = null;
			}

			if (removemask_timer) {
				clearTimeout(removemask_timer);
				removemask_timer = null;
			}
		};

		var removemask_timer = null;
		function resetpopups(from_remote) {
			popups.forEach(function (popup) {
				if (settings.mouseover_fade_time > 0) {
					popup.style.opacity = 0;

					if (!removepopups_timer) {
						removepopups_timer = setTimeout(removepopups, settings.mouseover_fade_time);
					}
				} else {
					// FIXME: this is called for each popup
					removepopups();
				}
			});

			if (mask_el) {
				set_important_style(mask_el, "pointer-events", "none");

				if (settings.mouseover_mask_fade_time > 0) {
					set_important_style(mask_el, "opacity", 0);

					if (!removemask_timer) {
						removemask_timer = setTimeout(remove_mask, settings.mouseover_mask_fade_time);
					}
				} else {
					remove_mask();
				}
			}

			if (!from_remote && can_use_remote()) {
				if (is_in_iframe) {
					remote_send_message("top", {type: "resetpopups"});
				} else if (popup_el_remote) {
					remote_send_message(popup_el_remote, {type: "resetpopups"});
				}
			}

			disable_click = false;
			popups_active = false;
			delay_handle_triggering = false;

			clear_resetpopup_timeout();

			next_popup_el = null;
			if (popup_el)
				last_popup_el = popup_el;
			popup_el = null;
			real_popup_el = null;
			popup_el_automatic = false;
			popup_el_remote = false;
			popup_el_is_video = false;
			popup_orig_url = null;
			popup_zoom_func = nullfunc;
			popup_wheel_cb = null;
			popup_update_pos_func = null;
			popup_client_rect_cache = null;
			last_popup_client_rect_cache = 0;

			stop_processing();
			stop_waiting();

			if (!delay_mouseonly && delay_handle) {
				clearTimeout(delay_handle);
				delay_handle = null;
			}
		}

		function get_viewport() {
			if (window.visualViewport) {
				return [
					window.visualViewport.width,
					window.visualViewport.height
				];
			} else {
				return [
					window.innerWidth || document.documentElement.clientWidth || document.body.clientWidth,
					window.innerHeight || document.documentElement.clientHeight || document.body.clientHeight
				];
			}
		}

		function get_processed_styles(str) {
			if (!str || typeof str !== "string" || !strip_whitespace(str))
				return;

			var styles = {};
			var splitted = str.split(/[;\n]/);
			for (var i = 0; i < splitted.length; i++) {
				var current = strip_whitespace(splitted[i]);
				if (!current)
					continue;

				if (string_indexof(current, ":") < 0)
					continue;

				if (/^\s*\/\//.test(current))
					continue;

				var property = strip_whitespace(current.replace(/^(.*?)\s*:.*/, "$1"));
				var value = strip_whitespace(current.replace(/^.*?:\s*(.*)$/, "$1"));

				var important = false;
				if (value.match(/!important$/)) {
					important = true;
					value = strip_whitespace(value.replace(/!important$/, ""));
				}

				styles[property] = {value: value, important: important};
			}

			return styles;
		}

		function get_styletag_styles(str) {
			var styles = get_processed_styles(str);
			if (!styles)
				return;

			var styles_array = [];
			for (var property in styles) {
				var current = property + ": " + styles[property].value;
				if (styles[property].important || true) {
					current += " !important"
				}

				styles_array.push(current);
			}

			return styles_array.join("; ");
		}

		function apply_styles(el, str, force_important, variables) {
			var styles = get_processed_styles(str);
			if (!styles)
				return;

			var oldstyle = el.getAttribute("style");
			if (oldstyle) {
				el.setAttribute("data-imu-oldstyle", oldstyle);
			}

			for (var property in styles) {
				var obj = styles[property];
				var value = obj.value;

				// todo: maybe handle escape sequences with JSON_parse?
				if (value.match(/^['"].*['"]$/)) {
					value = value.replace(/^["'](.*)["']$/, "$1");
				}

				if (variables) {
					for (var variable in variables) {
						value = value.split(variable).join(variables[variable]);
					}
				}

				if (obj.important || force_important) {
					el.style.setProperty(property, value, "important");
				} else {
					el.style.setProperty(property, value);
				}

				el.setAttribute("data-imu-newstyle", true);
			}
		}

		function revert_styles(el) {
			var oldstyle = el.getAttribute("data-imu-oldstyle");

			if (oldstyle) {
				el.setAttribute("style", oldstyle);
				el.removeAttribute("data-imu-oldstyle");
			} else if (el.getAttribute("style") && el.getAttribute("data-imu-newstyle")) {
				el.removeAttribute("style");
			}

			el.removeAttribute("data-imu-newstyle");
		}

		function set_important_style(el, property, value) {
			el.style.setProperty(property, value, "important");
		}

		function get_caption(obj, el) {
			if (obj && obj.extra && obj.extra.caption) {
				return strip_whitespace(obj.extra.caption);
			}

			if (el) {
				do {
					// don't use el.title/el.alt because if the element is <form>, it refers to form > input[name="title"]
					var el_title = el.getAttribute("title");
					var el_alt = el.getAttribute("alt");

					if (el_title || el_alt) {
						var caption = el_title || el_alt;

						// When opening an image in a new tab in Firefox, alt is set to the src
						if (caption === el.src)
							return null;

						return strip_whitespace(caption);
					}
				} while ((el = el.parentElement));
			}

			return null;
		}

		var get_tagname = function(el) {
			return el.tagName.toUpperCase();
		};

		function get_el_dimensions(el) {
			if (get_tagname(el) === "VIDEO") {
				return [
					el.videoWidth,
					el.videoHeight
				];
			} else if (get_tagname(el) === "CANVAS") {
				return [
					el.width,
					el.height
				];
			} else if (get_tagname(el) === "SVG") {
				return [
					el.width.animVal.value,
					el.height.animVal.value
				];
			} else {
				return [
					el.naturalWidth,
					el.naturalHeight
				]
			}
		}

		function add_link_to_history(link) {
			if (is_extension) {
				extension_send_message({
					type: "add_to_history",
					data: {
						url: link
					}
				});
			}
		}

		function makePopup(obj, orig_url, processing, data) {
			if (_nir_debug_) {
				console_log("makePopup", obj, orig_url, processing, data);
			}

			if (settings.mouseover_add_to_history) {
				add_link_to_history(data.data.obj.url);
			}

			var openb = get_single_setting("mouseover_open_behavior");
			if (openb === "newtab") {
				stop_waiting();

				var theobj = data.data.obj;
				theobj.url = data.data.resp.finalUrl;

				open_in_tab_imu(theobj);
				return;
			}

			//var x = mouseX;//mouseAbsX;
			//var y = mouseY;//mouseAbsY;
			var x = data.x;
			var y = data.y;

			if (x === null || x === undefined) {
				x = lastX;
			}

			if (y === null || y === undefined) {
				y = lastY;
			}

			dragged = false;
			dragstart = false;
			var seekstart = false;

			function cb(img, url) {
				if (!controlPressed && false) {
					if (processing.running)
						stop_waiting();
					return;
				}

				if (!img) {
					delay_handle_triggering = false;

					if (processing.running)
						stop_waiting();
					return;
				}

				var is_video = img.tagName === "VIDEO";

				var newobj = data.data.obj;

				if (!newobj)
					newobj = {};

				var estop = function(e) {
					e.stopPropagation();
					e.stopImmediatePropagation();
					return true;
				};

				var estop_pd = function(e) {
					e.preventDefault();
					estop(e);
					return true;
				};

				lastX = x;
				lastY = y;

				popupOpenX = x;
				popupOpenY = y;
				popupOpenLastX = x;
				popupOpenLastY = y;

				var initial_zoom_behavior = get_single_setting("mouseover_zoom_behavior");

				//img.onclick = estop;
				//img.onmousedown = estop;
				//img.addEventListener("click", estop, true);
				//img.addEventListener("mousedown", estop, true);

				var bgcolor = "#333";
				var fgcolor = "#fff";
				var textcolor = "#fff";
				var shadowcolor = "rgba(0,0,0,.5)";

				var setup_mask_el = function(mask) {
					set_el_all_initial(mask);

					set_important_style(mask, "opacity", 1);
					if (settings.mouseover_enable_mask_styles)
						apply_styles(mask, settings.mouseover_mask_styles2, true);

					if (!settings.mouseover_close_click_outside) {
						set_important_style(mask, "pointer-events", "none");
					}

					set_important_style(mask, "position", "fixed");
					set_important_style(mask, "z-index", maxzindex - 3);
					set_important_style(mask, "width", "100%");
					set_important_style(mask, "height", "100%");
					set_important_style(mask, "left", "0px");
					set_important_style(mask, "top", "0px");

					if (settings.mouseover_mask_fade_time > 0) {
						set_important_style(mask, "transition", "opacity " + (settings.mouseover_mask_fade_time / 1000.) + "s");

						// this allows us to respect a custom opacity for mouseover_mask_styles
						var old_opacity = mask.style.opacity;

						if (!popup_el_automatic) {
							set_important_style(mask, "opacity", 0);
							// this is needed in order to make the transition happen
							setTimeout(function() {
								set_important_style(mask, "opacity", old_opacity);
							}, 1);
						} else {
							set_important_style(mask, "opacity", old_opacity);
						}
					}

					our_addEventListener(mask, "click", function() {
						if (!settings.mouseover_close_click_outside)
							return;

						set_important_style(mask, "pointer-events", "none");
						resetpopups();
					}, true);

					return mask;
				};

				remove_mask();

				if (settings.mouseover_close_click_outside || settings.mouseover_enable_mask_styles) {
					mask_el = document.createElement("div");
					setup_mask_el(mask_el);
				}

				var outerdiv = document.createElement("div");
				set_el_all_initial(outerdiv);
				set_important_style(outerdiv, "position", "fixed");
				set_important_style(outerdiv, "z-index", maxzindex - 2);

				if (settings.mouseover_fade_time > 0) {
					set_important_style(outerdiv, "transition", "opacity " + (settings.mouseover_fade_time / 1000.) + "s");

					if (!popup_el_automatic) {
						set_important_style(outerdiv, "opacity", 0);

						// this is needed in order to make the transition happen
						setTimeout(function() {
							set_important_style(outerdiv, "opacity", 1);
						}, 1);
					} else {
						set_important_style(outerdiv, "opacity", 1);
					}
				}

				var div = document.createElement("div");
				var popupshown = false;
				set_el_all_initial(div);
				set_important_style(div, "box-shadow", "0 0 15px " + shadowcolor);
				set_important_style(div, "border", "3px solid " + fgcolor);
				set_important_style(div, "position", "relative");
				set_important_style(div, "top", "0px");
				set_important_style(div, "left", "0px");
				set_important_style(div, "display", "block");
				set_important_style(div, "background-color", "rgba(255,255,255,.5)");

				/*var styles = settings.mouseover_styles.replace("\n", ";");
				div.setAttribute("style", styles);
				if (!styles.match(/^\s*box-shadow\s*:/) &&
					!styles.match(/;\s*box-shadow\s*:/)) {
					div.style.boxShadow = "0 0 15px rgba(0,0,0,.5)";
				}
				if (!styles.match(/^\s*border(?:-[a-z]+)?\s*:/) &&
					!styles.match(/;\s*border(?:-[a-z]+)?\s*:/)) {
					div.style.border = "3px solid white";
					}*/

				// http://png-pixel.com/
				var transparent_gif = "data:image/gif;base64,R0lGODlhAQABAIAAAP///wAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==";
				var styles_variables = {};
				if (popup_orig_url && !popup_el_is_video) {
					styles_variables["%thumburl%"] = encodeuri_ifneeded(popup_orig_url);
				} else {
					styles_variables["%thumburl%"] = transparent_gif;
				}

				if (!is_video) {
					styles_variables["%fullurl%"] = encodeuri_ifneeded(get_img_src(img));
				} else {
					styles_variables["%fullurl%"] = transparent_gif;
				}

				apply_styles(div, settings.mouseover_styles, true, styles_variables);
				outerdiv.appendChild(div);

				//div.style.position = "fixed"; // instagram has top: -...px
				//div.style.zIndex = maxzindex - 2;


				//div.onclick = estop;
				//div.onmousedown = estop;
				//div.addEventListener("click", estop, true);
				//div.addEventListener("mousedown", estop, true);
				// useful for instagram
				//disable_click = true;


				var outer_thresh = 16;
				var border_thresh = 20;
				var top_thresh = 30;
				var top_mb = top_thresh - border_thresh;
				var viewport;
				var vw;
				var vh;

				var v_mx = Math.max(x - border_thresh, 0);
				var v_my = Math.max(y - top_thresh, 0);

				/*if (window.visualViewport) {
					vw = window.visualViewport.width;
					vh = window.visualViewport.height;
				} else {
					vw = window.innerWidth || document.documentElement.clientWidth || document.body.clientWidth;
					vh = window.innerHeight || document.documentElement.clientHeight || document.body.clientHeight;
					}*/

				var update_vwh = function(x, y) {
					viewport = get_viewport();
					vw = viewport[0];
					vh = viewport[1];

					vw -= border_thresh * 2;
					vh -= border_thresh + top_thresh;

					if (typeof x !== "undefined") {
						v_mx = Math.min(vw, Math.max(x - border_thresh, 0));
						v_my = Math.min(vh, Math.max(y - top_thresh, 0));
					}
				};

				var set_top = function(x) {
					//outerdiv.style.top = (x + top_thresh) + "px";
					outerdiv.style.top = x + "px";
				};

				var set_left = function(x) {
					//outerdiv.style.left = (x + border_thresh) + "px";
					outerdiv.style.left = x + "px";
				};

				var set_lefttop = function(xy) {
					set_top(xy[1]);
					set_left(xy[0]);
				};

				// https://stackoverflow.com/a/23270007
				function get_lefttopouter() {
					var style = outerdiv.currentStyle || window.getComputedStyle(outerdiv);
					return [style.marginLeft + style.borderLeftWidth,
							style.marginTop + style.borderTopWidth];
				}

				update_vwh(x, y);

				var el_dimensions = get_el_dimensions(img);
				set_el_all_initial(img);

				var add_link = false;
				if (!is_video && settings.mouseover_add_link) {
					add_link = true;
				} else if (is_video && settings.mouseover_add_video_link) {
					add_link = true;
				}

				if (add_link) {
					img.style.cursor = "pointer";
				}
				// https://stackoverflow.com/questions/7774814/remove-white-space-below-image
				img.style.verticalAlign = "bottom";
				img.style.setProperty("display", "block", "important");

				var img_naturalHeight, img_naturalWidth;

				img_naturalWidth = el_dimensions[0];
				img_naturalHeight = el_dimensions[1];

				var imgh = img_naturalHeight;
				var imgw = img_naturalWidth;

				if (initial_zoom_behavior === "fit") {
					img.style.maxWidth = vw + "px";
					img.style.maxHeight = vh + "px";
				} else if (initial_zoom_behavior === "custom") {
					var zoom_percent = settings.mouseover_zoom_custom_percent / 100;
					imgw = Math.max(imgw * zoom_percent, 20);
					imgh = Math.max(imgh * zoom_percent, 20);
					img.style.maxWidth = imgw + "px";
					img.style.width = img.style.maxWidth;
					img.style.maxHeight = imgh + "px";
					img.style.height = img.style.maxHeight;
				}

				if (imgh < 20 || imgw < 20) {
					// FIXME: This will stop "custom" percentages with low percentages for small images
					stop_waiting();
					console_error("Image too small to popup (" + imgw + "x" + imgh + ")");
					return;
				}

				var get_imghw_for_fit = function(width, height) {
					if (width === undefined)
						width = vw;

					if (height === undefined)
						height = vh;

					//height -= border_thresh * 2;
					//width  -= border_thresh * 2;

					var our_imgh = imgh;
					var our_imgw = imgw;

					if (imgh > height || imgw > width) {
						var ratio;
						if (imgh / height >
							imgw / width) {
							ratio = imgh / height;
						} else {
							ratio = imgw / width;
						}

						our_imgh /= ratio;
						our_imgw /= ratio;
					}

					return [our_imgw, our_imgh];
				};

				function calc_imghw_for_fit(width, height) {
					var new_imghw = get_imghw_for_fit(width, height);

					imgw = new_imghw[0];
					imgh = new_imghw[1];
				}

				if (initial_zoom_behavior === "fit") {
					calc_imghw_for_fit();
				}

				popup_update_pos_func = function(x, y, resize) {
					var popup_left, popup_top;

					update_vwh(x, y);

					var sct = scrollTop();
					var scl = scrollLeft();
					sct = scl = 0;

					var mouseover_position = get_single_setting("mouseover_position");
					if (popup_hold && settings.mouseover_hold_position_center)
						mouseover_position = "center";

					if (mouseover_position === "cursor") {
						popup_top =  sct + Math.min(Math.max(v_my - (imgh / 2), 0), Math.max(vh - imgh, 0));
						popup_left = scl + Math.min(Math.max(v_mx - (imgw / 2), 0), Math.max(vw - imgw, 0));
					} else if (mouseover_position === "center") {
						popup_top =  sct + Math.min(Math.max((vh / 2) - (imgh / 2), 0), Math.max(vh - imgh, 0));
						popup_left = scl + Math.min(Math.max((vw / 2) - (imgw / 2), 0), Math.max(vw - imgw, 0));
					} else if (mouseover_position === "beside_cursor") {
						var update_imghw;
						if (resize) {
							update_imghw = function(w, h) {
								calc_imghw_for_fit(w, h);
							};
						} else {
							update_imghw = nullfunc;
						}

						var calc_imgrect = function(w, h) {
							var new_imghw = [imgw, imgh];

							if (resize) {
								new_imghw = get_imghw_for_fit(w, h);
							}

							if (new_imghw[0] > w || new_imghw[1] > h)
								return null;

							return new_imghw;
						};

						var cursor_thresh = border_thresh;
						var ovw = vw - cursor_thresh;
						var ovh = vh - cursor_thresh;

						var calc_imgposd = function(lefttop, info, popupd) {
							var moused = lefttop ? v_mx : v_my;
							var vd = lefttop ? ovw : ovh;

							switch (info) {
								case -1:
									return Math.min(vd - popupd, Math.max(0, moused - (popupd / 2)));
								case 0:
									return Math.max(0, moused - popupd - cursor_thresh);
								case 1:
									return Math.min(vd - popupd, moused + cursor_thresh);
							}
						};

						var all_rects = [
							// top
							[-1, 0, ovw, v_my - cursor_thresh],
							// right
							[1, -1, ovw - v_mx - cursor_thresh, ovh],
							// bottom
							[-1, 1, ovw, ovh - v_my - cursor_thresh],
							// left
							[0, -1, v_mx - cursor_thresh, ovh]
						];

						var rects = [];

						// TODO: move the current popup position to the top

						if (x > viewport[0] / 2) {
							rects.push(all_rects[3]);
						} else {
							rects.push(all_rects[1]);
						}

						if (y > viewport[1] / 2) {
							rects.push(all_rects[0]);
						} else {
							rects.push(all_rects[2]);
						}

						for (var i = 0; i < all_rects.length; i++) {
							if (rects.indexOf(all_rects[i]) < 0) {
								rects.push(all_rects[i]);
							}
						}

						var largest_rectsize = -1;
						var largest_rect = null;
						var largest_origrect = null;

						for (var i = 0; i < rects.length; i++) {
							var our_rect = calc_imgrect(rects[i][2], rects[i][3]);
							if (!our_rect)
								continue;

							var our_rectsize = our_rect[0] * our_rect[1];
							if (our_rectsize > largest_rectsize) {
								largest_rectsize = our_rectsize;
								largest_rect = our_rect;
								largest_origrect = rects[i];
							}
						}

						if (!largest_origrect) {
							largest_rectsize = -1;
							for (var i = 0; i < rects.length; i++) {
								var rectsize = rects[i][2] * rects[i][3];
								if (rectsize > largest_rectsize) {
									largest_origrect = rects[i];
									largest_rectsize = rectsize;
								}
							}
						}

						if (largest_origrect) {
							update_imghw(largest_origrect[2], largest_origrect[3]);

							popup_top = calc_imgposd(false, largest_origrect[1], imgh);
							popup_left = calc_imgposd(true, largest_origrect[0], imgw);
						} else {
							// ???
						}
					} else if (mouseover_position === "beside_cursor_old") {
						// TODO: maybe improve this to be more interpolated?

						var popupx;
						var popupy;

						var cursor_thresh = border_thresh;

						var ovw = vw - cursor_thresh;
						var ovh = vh - cursor_thresh;

						var update_imghw;
						if (resize) {
							update_imghw = function(w, h) {
								calc_imghw_for_fit(w, h);
							};
						} else {
							update_imghw = nullfunc;
						}

						update_imghw(ovw, ovh);

						for (var loop_i = 0; loop_i < (resize ? 16 : 1); loop_i++) {
							if (y > viewport[1] / 2) {
								popupy = v_my - imgh - cursor_thresh;
							} else if (popupy === undefined) {
								popupy = v_my + cursor_thresh;
							}

							if (x > viewport[0] / 2) {
								popupx = v_mx - imgw - cursor_thresh;
							} else if (popupx === undefined) {
								popupx = v_mx + cursor_thresh;
							}

							if (popupy < 0) {
								popupy = 0;

								if (settings.mouseover_prevent_cursor_overlap) {
									update_imghw(ovw, v_my);
									//continue;
								}
							}

							if (popupx < 0) {
								popupx = 0;

								if (settings.mouseover_prevent_cursor_overlap) {
									update_imghw(v_mx, ovh);
									//continue;
								}
							}

							if ((popupy + imgh) > vh) {
								if (settings.mouseover_prevent_cursor_overlap) {
									update_imghw(ovw, ovh - v_my);
									//continue;
								} else {
									popupy = Math.max(vh - imgh - cursor_thresh, 0);
								}
							}

							if ((popupx + imgw) > vw) {
								if (settings.mouseover_prevent_cursor_overlap) {
									update_imghw(ovw - v_mx, ovh);
									//continue;
								} else {
									popupx = Math.max(vw - imgw - cursor_thresh, 0);
								}
							}

							//break;
						}

						popup_top =  popupy;
						popup_left = popupx;
					}

					return [
						popup_left + border_thresh,
						popup_top + top_thresh
					];
				};

				set_lefttop(popup_update_pos_func(x, y, true));

				var set_popup_size_helper = function(size, maxsize, widthheight) {
					if (maxsize === undefined)
						maxsize = size;

					if (typeof size === "number")
						size = size + "px";
					if (typeof maxsize === "number")
						maxsize = maxsize + "px";

					if (widthheight) {
						img.style.width = size;
						img.style.maxWidth = maxsize;
					} else {
						img.style.height = size;
						img.style.maxHeight = maxsize;
					}
				};

				var set_popup_width = function(width, maxwidth) {
					set_popup_size_helper(width, maxwidth, true);
				};

				var set_popup_height = function(height, maxheight) {
					set_popup_size_helper(height, maxheight, false);
				};

				set_popup_width(imgw, "initial");
				set_popup_height(imgh, "initial");
				/*console_log(x - (imgw / 2));
				  console_log(vw);
				  console_log(imgw);
				  console_log(vw - imgw);*/

				function get_defaultopacity() {
					var defaultopacity = (settings.mouseover_ui_opacity / 100);
					if (isNaN(defaultopacity))
						defaultopacity = 1;
					if (defaultopacity > 1)
						defaultopacity = 1;
					if (defaultopacity < 0)
						defaultopacity = 0;

					return defaultopacity;
				}

				var defaultopacity = get_defaultopacity();

				function opacity_hover(el, targetel, action) {
					if (!targetel)
						targetel = el;

					our_addEventListener(el, "mouseover", function(e) {
						targetel.style.opacity = "1.0";

						if (action)
							targetel.style.boxShadow = "0px 0px 5px 1px white";
					}, true);
					our_addEventListener(el, "mouseout", function(e) {
						targetel.style.opacity = get_defaultopacity();

						if (action)
							targetel.style.boxShadow = "none";
					}, true);
				}

				var btndown = false;
				function addbtn(text, title, action, istop) {
					var tagname = "span";
					if (typeof action === "string")
						tagname = "a";

					var btn = document.createElement(tagname);

					if (action) {
						var do_action = function() {
							return !btn.hasAttribute("data-btn-noaction");
						};

						if (typeof action === "function") {
							our_addEventListener(btn, "click", function(e) {
								if (!do_action())
									return;

								//console_log(e);
								e.stopPropagation();
								e.stopImmediatePropagation();
								e.preventDefault();
								action();
								return false;
							}, true);
						} else if (typeof action === "string") {
							btn.href = action;
							btn.target = "_blank";
							btn.setAttribute("rel", "noreferrer");

							our_addEventListener(btn, "click", function(e) {
								e.stopPropagation();
								e.stopImmediatePropagation();
							}, true);
						}

						our_addEventListener(btn, "mouseover", function(e) {
							set_important_style(btn, "box-shadow", "0px 0px 5px 1px white");
						}, true);

						our_addEventListener(btn, "mouseout", function(e) {
							set_important_style(btn, "box-shadow", "none");
						}, true);
					}

					our_addEventListener(btn, "mousedown", function(e) {
						btndown = true;
					}, true);
					our_addEventListener(btn, "mouseup", function(e) {
						btndown = false;
					}, true);
					if (!istop) {
						opacity_hover(btn, undefined, true);
					} else if (typeof text === "object" && text.truncated !== text.full) {
						our_addEventListener(btn, "mouseover", function(e) {
							var computed_style = get_computed_style(btn);
							set_important_style(btn, "width", computed_style.width || (btn.clientWidth + "px"));

							btn.innerText = text.full;

						}, true);

						our_addEventListener(btn, "mouseout", function(e) {
							btn.innerText = text.truncated;
							btn.style.width = "initial";
						}, true);
					}

					set_el_all_initial(btn);
					if (action) {
						set_important_style(btn, "cursor", "pointer");
					}

					set_important_style(btn, "background", bgcolor);
					set_important_style(btn, "border", "3px solid " + fgcolor);
					set_important_style(btn, "border-radius", "10px");
					// test: https://www.yeshiva.org.il/ (topbarel sets this to ltr, so it must be set to the initial value in order to respect the direction)
					set_important_style(btn, "direction", text_direction);

					// workaround for emojis: https://stackoverflow.com/a/39776303
					if (typeof text === "string" && text.length === 1 && text.charCodeAt(0) > 256) {
						set_important_style(btn, "color", "transparent");
						set_important_style(btn, "text-shadow", "0 0 0 " + textcolor);
					} else {
						set_important_style(btn, "color", textcolor);
					}

					set_important_style(btn, "padding", "4px");
					set_important_style(btn, "line-height", "1em");
					//btn.style.whiteSpace = "nowrap";
					set_important_style(btn, "font-size", "14px");
					set_important_style(btn, "font-family", sans_serif_font);
					apply_styles(btn, settings.mouseover_ui_styles, true);

					set_important_style(btn, "z-index", maxzindex - 1);
					if (!istop) {
						set_important_style(btn, "position", "absolute");
						set_important_style(btn, "opacity", defaultopacity);
					} else {
						set_important_style(btn, "position", "relative");
						set_important_style(btn, "margin-right", "4px");
					}
					set_important_style(btn, "vertical-align", "top");
					//btn.style.maxWidth = "50%";
					set_important_style(btn, "white-space", "pre-wrap");
					set_important_style(btn, "display", "inline-block");
					if (action) {
						set_important_style(btn, "user-select", "none");
					}

					if (typeof text === "string") {
						btn.innerText = text;
					} else {
						btn.innerText = text.truncated;
					}

					if (title)
						btn.title = title;

					return btn;
				}

				var ui_els = [];

				var text_direction = "initial";

				var popup_el_style;
				if (popup_el) {
					popup_el_style = get_computed_style(popup_el);
				} else {
					popup_el_style = get_computed_style(document.body);
				}

				if (popup_el_style && popup_el_style.direction === "rtl") {
					text_direction = "rtl";
				}

				var cached_previmages = 0;
				var cached_nextimages = 0;

				function lraction(isright) {
					trigger_gallery(isright ? 1 : -1, function(changed) {
						if (!changed) {
							create_ui();
						}
					});
				}

				function create_topbarel() {
					var topbarel = document.createElement("div");
					set_el_all_initial(topbarel);
					set_important_style(topbarel, "position", "absolute");
					set_important_style(topbarel, "opacity", defaultopacity);
					set_important_style(topbarel, "z-index", maxzindex - 1);
					set_important_style(topbarel, "white-space", "nowrap");

					// test: https://www.yeshiva.org.il/
					// otherwise, the buttons are in the wrong order
					set_important_style(topbarel, "direction", "ltr");
					return topbarel;
				}

				function create_ui(use_cached_gallery) {
					for (var el_i = 0; el_i < ui_els.length; el_i++) {
						var ui_el = ui_els[el_i];
						ui_el.parentNode.removeChild(ui_el);
					}

					ui_els = [];

					var emi = 14;
					var em1 = emi + "px"
					var emhalf = (emi / 2) + "px";
					var gallerycount_fontsize = "13px";
					var galleryinput_fontsize = "12px";

					var topbarel = create_topbarel();
					topbarel.style.left = "-" + em1;
					topbarel.style.top = "-" + em1;

					if (!settings.mouseover_ui) {
						// Not sure why this is needed, but without it, clicking and dragging images doesn't work under Firefox (#78)
						outerdiv.appendChild(topbarel);
						return;
					}

					opacity_hover(topbarel);

					if (settings.mouseover_ui_closebtn) {
						// \xD7 = ×
						var closebtn = addbtn("\xD7", _("Close") + " (" + _("ESC") + ")", function() {
							resetpopups();
						}, true);
						topbarel.appendChild(closebtn);
					}

					outerdiv.appendChild(topbarel);
					ui_els.push(topbarel);

					var get_img_height = function() {
						var rect = img.getBoundingClientRect();
						// This is needed if img isn't displayed yet
						var rect_height = rect.height || imgh;

						if (isNaN(rect_height))
							rect_height = img_naturalHeight;

						return rect_height;
					};


					var prev_images = 0;
					var next_images = 0;

					function get_imagesizezoom_text() {
						var text = "";

						var rect_height = get_img_height();

						var zoom_percent = rect_height / img_naturalHeight;
						var currentzoom = parseInt(zoom_percent * 100);

						if (settings.mouseover_ui_imagesize) {
							text = img_naturalWidth + "x" + img_naturalHeight;

							if (settings.mouseover_ui_zoomlevel && currentzoom !== 100) {
								text += " (" + currentzoom + "%)"
							}
						} else {
							text = currentzoom + "%";
						}

						return text;
					}

					if (settings.mouseover_ui_imagesize || settings.mouseover_ui_zoomlevel) {
						var imagesize = addbtn(get_imagesizezoom_text(100), "", null, true);
						set_important_style(imagesize, "font-size", gallerycount_fontsize);
						topbarel.appendChild(imagesize);
					}

					var get_imagestotal_text = function() {
						if (prev_images + next_images > settings.mouseover_ui_gallerymax) {
							return settings.mouseover_ui_gallerymax + "+";
						} else {
							return (prev_images + 1) + " / " + (prev_images + next_images + 1);
						}
					};

					var update_imagestotal = function() {
						if (images_total_input_active)
							return;

						if (prev_images + next_images > 0) {
							set_important_style(images_total, "display", "inline-block");
							images_total.innerText = get_imagestotal_text();
						} else {
							set_important_style(images_total, "display", "none");
						}
					};

					var imagestotal_input_enable = function() {
						images_total_input_active = true;
						editing_text = true;
						images_total.innerText = "";

						set_important_style(images_total_input, "display", "initial");
						images_total_input.value = prev_images + 1;
						images_total.setAttribute("data-btn-noaction", true);
						images_total.appendChild(images_total_input);

						// https://stackoverflow.com/a/19498477
						setTimeout(function() {
							images_total_input.select();
							images_total_input.setSelectionRange(0, images_total_input.value.length);
						}, 100);
					};

					var imagestotal_input_disable = function() {
						editing_text = false;
						if (!images_total_input_active)
							return;

						set_important_style(images_total_input, "display", "none");
						images_total.removeChild(images_total_input);
						images_total.removeAttribute("data-btn-noaction");
						images_total_input_active = false;

						update_imagestotal();
					};

					var popup_width = (popupshown && outerdiv.clientWidth) || imgw;

					if (settings.mouseover_enable_gallery && settings.mouseover_ui_gallerycounter) {
						var images_total = addbtn(get_imagestotal_text(), "", imagestotal_input_enable, true);
						set_important_style(images_total, "font-size", gallerycount_fontsize);
						set_important_style(images_total, "display", "none");

						var images_total_input = document.createElement("input");
						var images_total_input_active = false;
						set_el_all_initial(images_total_input);
						set_important_style(images_total_input, "display", "none");
						set_important_style(images_total_input, "background-color", "white");
						set_important_style(images_total_input, "font-family", sans_serif_font);
						set_important_style(images_total_input, "font-size", galleryinput_fontsize);
						set_important_style(images_total_input, "padding", "1px");
						set_important_style(images_total_input, "padding-left", "2px");
						set_important_style(images_total_input, "width", "5em");
						our_addEventListener(images_total_input, "mouseout", imagestotal_input_disable);
						our_addEventListener(images_total_input, "keydown", function(e) {
							if (e.which === 13) { // enter
								var parsednum = images_total_input.value.replace(/\s+/g, "");
								if (/^[0-9]+$/.test(parsednum)) {
									parsednum = parseInt(parsednum);
									trigger_gallery(parsednum - (prev_images + 1));
								}

								imagestotal_input_disable();

								e.stopPropagation();
								e.preventDefault();
								return false;
							}
						}, true);

						topbarel.appendChild(images_total);
					}

					if (settings.mouseover_ui_optionsbtn) {
						// \u2699 = ⚙
						var optionsbtn = addbtn("\u2699", _("Options"), options_page, true);
						topbarel.appendChild(optionsbtn);
					}

					if (settings.mouseover_ui_downloadbtn) {
						// \uD83E\uDC47 = 🡇
						var downloadbtn = addbtn("\uD83E\uDC47", _("Download (" + get_trigger_key_text(settings.mouseover_download_key) + ")"), download_popup_image, true);
						topbarel.appendChild(downloadbtn);
					}

					if (settings.mouseover_ui_rotationbtns) {
						var get_rotate_title = function(leftright) {
							return _("rotate_" + leftright + "_btn") + " (" + get_trigger_key_text(settings["mouseover_rotate_" + leftright + "_key"]) + ")";
						};

						// \u21B6 = ↶
						var rotateleftbtn = addbtn("\u21B6", get_rotate_title("left"), function() {rotate_gallery(-90)}, true);
						// \u21B7 = ↷
						var rotaterightbtn = addbtn("\u21B7", get_rotate_title("right"), function() {rotate_gallery(90)}, true);

						topbarel.appendChild(rotateleftbtn);
						topbarel.appendChild(rotaterightbtn);
					}

					var caption = get_caption(newobj, popup_el);
					if (caption && settings.mouseover_ui_caption) {
						var btntext = caption;

						if (settings.mouseover_ui_wrap_caption) {
							// /10 is arbitrary, but seems to work well
							var chars = parseInt(Math.max(10, Math.min(60, (popup_width - topbarel.clientWidth) / 10)));
							var caption_regex = new RegExp("^((?:.{" + chars + "}|.{0," + chars + "}[\\r\\n]))[\\s\\S]+?$");

							btntext = {
								truncated: caption.replace(caption_regex, "$1..."),
								full: caption
							};
						}

						var caption_btn = addbtn(btntext, caption, null, true);
						topbarel.appendChild(caption_btn);
					}

					var add_lrhover = function(isleft, btnel, action, title) {
						if (!settings.mouseover_enable_gallery || popup_width < 200)
							return;

						var img_height = get_img_height();
						var bottom_heights = 0;
						var top_heights = 20;
						if (is_video) {
							bottom_heights = 60;
						}

						if (img_height < 100) {
							top_heights = 0;
						}

						var lrheight = img_height - top_heights - bottom_heights;
						if (lrheight < 10)
							return;

						var lrhover = document.createElement("div");
						set_el_all_initial(lrhover);
						lrhover.title = title;
						if (isleft) {
							lrhover.style.left = "0em";
						} else {
							lrhover.style.right = "0em";
						}

						lrhover.style.top = top_heights + "px";
						lrhover.style.height = lrheight + "px";
						lrhover.style.position = "absolute";
						lrhover.style.width = "15%";
						lrhover.style.maxWidth = "200px";
						//lrhover.style.height = "100%";
						lrhover.style.zIndex = maxzindex - 2;
						lrhover.style.cursor = "pointer";

						opacity_hover(lrhover, btnel, true);
						our_addEventListener(lrhover, "click", function(e) {
							if (dragged) {
								return false;
							}

							estop(e);
							action(e);
							return false;
						}, true);
						outerdiv.appendChild(lrhover);
						ui_els.push(lrhover);
						return lrhover;
					};

					var add_leftright_gallery_button = function(leftright) {
						if (!settings.mouseover_enable_gallery || !settings.mouseover_ui_gallerybtns)
							return;

						var action = function() {
							return lraction(leftright);
						};

						var name = leftright ? "Next" : "Previous";

						// \u2192 = →
						// \u2190 = ←
						// Using the following arrows as they're more consistent throughout fonts
						// \ud83e\udc52 = 🡒
						// \ud83e\udc50 = 🡐
						var icon = leftright ? "\ud83e\udc52" : "\ud83e\udc50";

						var keybinding = leftright ? settings.mouseover_gallery_next_key : settings.mouseover_gallery_prev_key;
						var keybinding_text = get_trigger_key_text(keybinding);

						var title = _(name) + " (" + _(keybinding_text) + ")";

						var btn = addbtn(icon, title, action);
						btn.style.top = "calc(50% - 7px - " + emhalf + ")";
						if (!leftright) {
							btn.style.left = "-" + em1;
						} else {
							btn.style.left = "initial";
							btn.style.right = "-" + em1;
						}
						outerdiv.appendChild(btn);
						ui_els.push(btn);

						add_lrhover(!leftright, btn, action, title);

						if (settings.mouseover_enable_gallery && settings.mouseover_ui_gallerycounter) {
							if (use_cached_gallery) {
								if (!leftright) {
									prev_images = cached_previmages;
								} else {
									next_images = cached_nextimages;
								}

								update_imagestotal();
							} else {
								count_gallery(leftright, undefined, undefined, undefined, function(total) {
									if (!leftright) {
										prev_images = total;
										cached_previmages = prev_images;
									} else {
										next_images = total;
										cached_nextimages = next_images;
									}

									update_imagestotal();
								});
							}
						}
					};

					var add_leftright_gallery_button_if_valid = function(leftright) {
						if (!settings.mouseover_enable_gallery)
							return;

						is_nextprev_valid(leftright, function(valid) {
							if (valid) {
								add_leftright_gallery_button(leftright);
							}
						});
					};

					add_leftright_gallery_button_if_valid(false);
					add_leftright_gallery_button_if_valid(true);
				}

				create_ui();

				if (typeof newobj.filename !== "string")
					newobj.filename = "";

				var newobj_ext = null;
				if (newobj.filename.length === 0 && data.data.respdata) {
					try {
						var headers = parse_headers(data.data.respdata.responseHeaders);
						for (var h_i = 0; h_i < headers.length; h_i++) {
							var header_name = headers[h_i].name.toLowerCase();
							var header_value = headers[h_i].value;

							if (header_name === "content-disposition") {
								// http://cfile7.uf.tistory.com/original/227CF24E57ABEC701869E7
								// Content-Disposition: inline; filename="160731 LA Kcon stage - 15 copy.jpg"; filename*=UTF-8''160731%20LA%20Kcon%20stage%20-%2015%20copy.jpg
								var loops = 0;
								while (loops < 100 && typeof header_value === "string" && header_value.length > 0) {
									var current_value = header_value.replace(/^\s*([^;]*?)\s*(?:;.*)?$/, "$1");
									//header_value = header_value.replace(/^[^;]*(?:;\s*(.*))?$/, "$1");

									var attr = current_value.replace(/^\s*([^=;]*?)\s*(?:[=;].*)?$/, "$1").toLowerCase();
									var a_match = header_value.match(/^[^=;]*(?:(?:=\s*(?:(?:["']([^'"]*?)["'])|([^;]*?))\s*(;.*)?\s*)|;\s*(.*))?$/);
									if (!a_match) {
										console_error("Header value does not match pattern:", header_value);
										break;
									}
									var a_value = a_match[1] || a_match[2];

									// TODO: implement properly
									/*if (attr === "filename*") {
										newobj.filename = a_value;
									}*/

									if (newobj.filename.length === 0 && attr === "filename" && typeof a_value === "string" && a_value.length > 0) {
										newobj.filename = a_value;
									}

									header_value = a_match[3] || a_match[4];
									loops++;
								}
							} else if (header_name === "content-type") {
								newobj_ext = get_ext_from_contenttype(header_value);
							}

							if (newobj.filename.length > 0)
								break;
						}
					} catch (e) {
						console_error(e);
					}

					if (newobj.filename.length === 0) {
						newobj.filename = url.replace(/.*\/([^?#/]*)(?:[?#].*)?$/, "$1");

						// Disable as there's no use for this
						if (false && (newobj.filename.split(".").length - 1) === 1) {
							newobj.filename = newobj.filename.replace(/(.*)\.[^.]*?$/, "$1");
						}
					}

					// e.g. for /?...
					if (newobj.filename.length === 0) {
						newobj.filename = "download";
					}

					if (newobj.filename.indexOf(".") < 0 && newobj_ext) {
						newobj.filename += "." + newobj_ext;
					}
				}

				var a = document.createElement("a");
				set_el_all_initial(a);

				if (add_link) {
					a.style.cursor = "pointer";

					//a.addEventListener("click", function(e) {
					a.onclick = function(e) {
						e.stopPropagation();
						e.stopImmediatePropagation();
						return true;
					};
				}

				a.style.setProperty("vertical-align", "bottom", "important");
				a.style.setProperty("display", "block", "important");

				if (add_link) {
					a.href = url;
					if (settings.mouseover_download) {
						if (false) {
							a.href = img.src;

							if (newobj.filename.length > 0) {
								a.setAttribute("download", newobj.filename);
							} else {
								var attr = document.createAttribute("download");
								a.setAttributeNode(attr);
							}
						} else {
							a.href = "#";
							our_addEventListener(a, "click", function(e) {
								download_popup_image();

								e.preventDefault();
								e.stopPropagation();
								return false;
							}, true);
						}
					}
				}

				a.target = "_blank";
				a.appendChild(img);
				div.appendChild(a);

				popup_hidecursor_timer = null;
				var orig_a_cursor = a.style.cursor;
				var orig_img_cursor = img.style.cursor;
				popup_hidecursor_func = function(hide) {
					if (settings.mouseover_hide_cursor && hide) {
						a.style.cursor = "none";
						img.style.cursor = "none";
					} else {
						if (popup_hidecursor_timer) {
							clearTimeout(popup_hidecursor_timer);
							popup_hidecursor_timer = null;
						}

						a.style.cursor = orig_a_cursor;
						img.style.cursor = orig_img_cursor;
					}
				};

				if (settings.mouseover_hide_cursor && settings.mouseover_hide_cursor_after <= 0) {
					popup_hidecursor_func(true);
				}

				div.onmouseover = div.onmousemove = function(e) {
					if (settings.mouseover_hide_cursor_after > 0 || !settings.mouseover_hide_cursor) {
						popup_hidecursor_func(false);

						if (settings.mouseover_hide_cursor) {
							popup_hidecursor_timer = setTimeout(function() {
								popup_hidecursor_func(true);
							}, settings.mouseover_hide_cursor_after);
						}
					}
				};

				function startdrag(e) {
					dragstart = true;
					dragged = false;
					dragstartX = e.clientX;
					dragstartY = e.clientY;
					dragoffsetX = dragstartX - parseFloat(outerdiv.style.left);
					dragoffsetY = dragstartY - parseFloat(outerdiv.style.top);
				}

				// TODO: allow this to be live-reloaded
				if (get_single_setting("mouseover_pan_behavior") === "drag") {
					if (is_video) {
						img.onseeking = function(e) {
							seekstart = true;
						};

						img.onseeked = function(e) {
							seekstart = false;
						};
					}

					div.ondragstart = a.ondragstart = img.ondragstart = function(e) {
						if (seekstart)
							return;

						//dragstart = true;
						//dragged = false;
						startdrag(e);
						//e.stopPropagation();
						estop(e);
						return false;
					};

					//div.ondrop = estop;

					div.onmousedown = div.onpointerdown = a.onmousedown = a.onpointerdown = function(e) {
						//console_log("(div,a).mousedown", e);
						if (btndown || e.button !== 0 || seekstart)
							return;

						//dragstart = true;
						//dragged = false;
						startdrag(e);

						e.preventDefault();
						estop(e);
						return false;
					};

					img.onmousedown = img.onpointerdown = function(e) {
						//console_log("img.onmousedown", e);
						if (btndown || e.button !== 0 || seekstart)
							return;

						//dragstart = true;
						//dragged = false;
						startdrag(e);

						estop(e);
						return true;
					};

					a.onclick = function(e) {
						//console_log("a.onclick", e);
						dragstart = false;

						if (dragged) {
							estop(e);
							dragged = false;
							return false;
						}

						e.stopPropagation();
						e.stopImmediatePropagation();
						return true;
					};

					div.onmouseup = div.onpointerup = div.onclick = a.onmouseup = a.onpointerup = /*a.onclick =*/ function(e) {
						//console_log("(div,a).mouseup", e);
						dragstart = false;

						if (dragged) {
							//estop(e);
							return false;
						}

						e.stopPropagation();
						e.stopImmediatePropagation();
						return true;
					};

					// Enabling this makes buttons not work after clicking the link
					//div.addEventListener("click", div.onclick, true);
					//a.addEventListener("click", a.onclick, true);

					img.onmouseup = img.onpointerup = img.onclick = function(e) {
						//console_log("img.mouseup", e);
						dragstart = false;
						//estop(e);
						return true;
					};

					if (is_video && !settings.mouseover_video_controls) {
						img.onclick = function(e) {
							if (!dragged) {
								if (!img.paused) {
									img.pause();
								} else {
									img.play();
								}
							}

							//console_log("img.mouseup", e);
							dragstart = false;
							//estop(e);
							return true;
						};
					}
				}

				var currentmode = initial_zoom_behavior;

				popup_zoom_func = function(zoom_mode, zoomdir, x, y, zoom_out_to_close) {
					var changed = false;

					var popup_left = parseFloat(outerdiv.style.left);
					var popup_top = parseFloat(outerdiv.style.top);

					var popup_width = outerdiv.clientWidth;
					var popup_height = outerdiv.clientHeight;

					if (x === undefined && y === undefined) {
						// TODO: if mouse is within the popup, use the mouse's coordinates instead
						//   This will have to check the zoom_origin setting.
						//x = mouseAbsX;
						//y = mouseAbsY;

						var visible_left = Math.max(popup_left, 0);
						var visible_top = Math.max(popup_top, 0);

						var visible_right = Math.min(visible_left + popup_width, vw);
						var visible_bottom = Math.min(visible_top + popup_height, vh);

						// get the middle of the visible portion of the popup
						x = visible_left + (visible_right - visible_left) / 2;
						y = visible_top + (visible_bottom - visible_top) / 2;
					} else {
						// ensure it's clamped, e.g. when scrolling on the document instead of the popup
						if (x < popup_left)
							x = popup_left;
						else if (x > popup_left + popup_width)
							x = popup_left + popup_width;

						if (y < popup_top)
							y = popup_top;
						else if (y > popup_top + popup_height)
							y = popup_top + popup_height;
					}

					var offsetX = x - popup_left;
					var offsetY = y - popup_top;

					var percentX = offsetX / outerdiv.clientWidth;
					var percentY = offsetY / outerdiv.clientHeight;

					if (zoom_mode === "fitfull") {
						if (zoom_out_to_close && currentmode === "fit" && zoomdir > 0) {
							resetpopups();
							return false;
						}

						if (zoomdir > 0 && currentmode !== "fit") {
							update_vwh();

							imgh = img_naturalHeight;
							imgw = img_naturalWidth;
							calc_imghw_for_fit();

							set_popup_width(imgw, vw);
							set_popup_height(imgh, vh);

							currentmode = "fit";
							changed = true;
						} else if (zoomdir < 0 && currentmode !== "full") {
							set_popup_width(img_naturalWidth, "initial");
							set_popup_height(img_naturalHeight, "initial");

							currentmode = "full";
							changed = true;
						}
					} else if (zoom_mode === "incremental") {
						var imgwidth = img.clientWidth;
						var imgheight = img.clientHeight;

						var mult = 1;
						if (imgwidth < img_naturalWidth) {
							mult = img_naturalWidth / imgwidth;
						} else {
							mult = imgwidth / img_naturalWidth;
						}

						var increment = settings.scroll_incremental_mult - 1;

						mult = Math.round(mult / increment);
						mult *= increment;

						if (imgwidth < img_naturalWidth) {
							if (mult !== 0)
								mult = 1 / mult;
						}

						if (zoomdir > 0) {
							mult /= 1 + increment;
						} else {
							mult *= 1 + increment;
						}

						imgwidth = img_naturalWidth * mult;
						imgheight = img_naturalHeight * mult;

						var too_small = zoomdir > 0 && (imgwidth < 64 || imgheight < 64);
						var too_big = zoomdir < 0 && (imgwidth > img_naturalWidth * 512 || imgheight > img_naturalHeight * 512);

						if (too_small || too_big) {
							if (zoom_out_to_close && too_small)
								resetpopups();
							return false;
						}

						set_popup_width(imgwidth);
						set_popup_height(imgheight);
						changed = true;
					}

					if (!changed)
						return false;

					var imgwidth = outerdiv.clientWidth;
					var imgheight = outerdiv.clientHeight;

					var newx, newy;

					if (true || (imgwidth <= vw && imgheight <= vh) || zoom_mode === "incremental") {
						// centers wanted region to pointer
						newx = (x - percentX * imgwidth);
						newy = (y - percentY * imgheight);
					} else if (imgwidth > vw || imgheight > vh) {
						// centers wanted region to center of screen
						newx = (vw / 2) - percentX * imgwidth;
						var endx = newx + imgwidth;
						if (newx > border_thresh && endx > (vw - border_thresh))
							newx = Math.max(border_thresh, (vw + border_thresh) - imgwidth);

						if (newx < border_thresh && endx < (vw - border_thresh))
							newx = Math.min(border_thresh, (vw + border_thresh) - imgwidth);

						newy = (vh / 2) - percentY * imgheight;
						var endy = newy + imgheight;
						if (newy > border_thresh && endy > (vh - border_thresh))
							newy = Math.max(border_thresh, (vh + border_thresh) - imgheight);

						if (newy < border_thresh && endy < (vh - border_thresh))
							newy = Math.min(border_thresh, (vh + border_thresh) - imgheight);
					}

					if (imgwidth <= vw && imgheight <= vh) {
						newx = Math.max(newx, border_thresh);
						if (newx + imgwidth > (vw - border_thresh)) {
							newx = (vw + border_thresh) - imgwidth;
						}

						newy = Math.max(newy, border_thresh);
						if (newy + imgheight > (vh - border_thresh)) {
							newy = (vh + border_thresh) - imgheight;
						}
					}

					//var lefttop = get_lefttopouter();
					outerdiv.style.left = (newx/* - lefttop[0]*/) + "px";
					outerdiv.style.top = (newy/* - lefttop[1]*/) + "px";

					create_ui(true);

					// The mouse could accidentally land outside the image in theory
					mouse_in_image_yet = false;

					return false;
				};

				outerdiv.onwheel = popup_wheel_cb = function(e, is_document) {
					var handledx = false;
					var handledy = false;

					var handle_seek = function(xy) {
						var isright = false;

						if (xy) {
							if (e.deltaX < 0)
								isright = false;
							else if (e.deltaX > 0)
								isright = true;
							else return;
						} else {
							if (e.deltaY < 0)
								isright = false;
							else if (e.deltaY > 0)
								isright = true;
							else return;
						}

						seek_popup_video(!isright);
						estop_pd(e);
						return true;
					};

					if (is_video) {
						if (!handledx && settings.mouseover_video_seek_horizontal_scroll) {
							if (handle_seek(true)) {
								handledx = true;
							}
						}

						if (!handledy && settings.mouseover_video_seek_vertical_scroll) {
							if (handle_seek(false)) {
								handledy = true;
							}
						}
					}

					var scrollx_behavior = get_single_setting("mouseover_scrollx_behavior");
					var scrolly_behavior = get_single_setting("mouseover_scrolly_behavior");

					if (scrollx_behavior === "pan" && !handledx) {
						outerdiv.style.left = (parseInt(outerdiv.style.left) + e.deltaX) + "px";
						handledx = true;
					}

					if (scrolly_behavior === "pan" && !handledy) {
						outerdiv.style.top = (parseInt(outerdiv.style.top) + e.deltaY) + "px";
						handledy = true;
					}

					var handle_gallery = function(xy) {
						if (!settings.mouseover_enable_gallery)
							return;

						var isright = false;

						if (xy) {
							if (e.deltaX < 0)
								isright = false;
							else if (e.deltaX > 0)
								isright = true;
							else return;
						} else {
							if (e.deltaY < 0)
								isright = false;
							else if (e.deltaY > 0)
								isright = true;
							else return;
						}

						lraction(isright);
						estop_pd(e);
						return true;
					};

					if (scrollx_behavior === "gallery" && !handledx) {
						if (handle_gallery(true)) {
							return;
						}

						handledx = true;
					}

					if (scrolly_behavior === "gallery" && !handledy) {
						if (handle_gallery(false)) {
							return;
						}

						handledy = true;
					}

					if (handledy) {
						estop_pd(e);
						return false;
					}

					if (scrolly_behavior !== "zoom" || e.deltaY === 0) {
						return;
					}

					estop_pd(e);

					var cursor_x = e.clientX;
					var cursor_y = e.clientY;
					if (get_single_setting("scroll_zoom_origin") === "center") {
						cursor_x = undefined;
						cursor_y = undefined;
					}

					var zoom_mode = get_single_setting("scroll_zoom_behavior");
					if (popup_zoom_func(zoom_mode, e.deltaY, cursor_x, cursor_y, settings.zoom_out_to_close) === false)
						return false;
				};

				if (mask_el) {
					document.documentElement.appendChild(mask_el);
				}

				document.documentElement.appendChild(outerdiv);

				removepopups();
				popups.push(outerdiv);
				popupshown = true;
				popup_obj = newobj;
				can_close_popup = [false, false];
				popup_hold = false;
				mouse_in_image_yet = false;
				delay_handle_triggering = false;

				if (popup_trigger_reason !== "mouse") {
					can_close_popup[1] = true;
				}

				stop_waiting();
				popups_active = true;
				//console_log(div);

				add_resetpopup_timeout();
			}

			cb(data.data.img, data.data.newurl, obj);
		}

		function getUnit(unit) {
			if (unit.match(/^ *([0-9]+)px *$/)) {
				return unit.replace(/^ *([0-9]+)px *$/, "$1");
			}

			// https://github.com/tysonmatanich/getEmPixels/blob/master/getEmPixels.js
			var important = "!important;";
			var style = "position:absolute!important;visibility:hidden!important;width:" + unit + "!important;font-size:" + unit + "!important;padding:0!important";

			var extraBody;

			var unitel;
			if (!unitel) {
				// Emulate the documentElement to get rem value (documentElement does not work in IE6-7)
				unitel = extraBody = document.createElement("body");
				extraBody.style.cssText = "font-size:" + unit + "!important;";
				document.documentElement.insertBefore(extraBody, document.body);
			}

			// Create and style a test element
			var testElement = document.createElement("i");
			testElement.style.cssText = style;
			unitel.appendChild(testElement);

			// Get the client width of the test element
			var value = testElement.clientWidth;

			if (extraBody) {
				// Remove the extra body element
				document.documentElement.removeChild(extraBody);
			}
			else {
				// Remove the test element
				unitel.removeChild(testElement);
			}

			// Return the em value in pixels
			return value;
		}

		function valid_source(source) {
			var thresh = 20;

			if (source.tagName !== "PICTURE" &&
				source.tagName !== "VIDEO" &&
				source.tagName !== "IMG" &&
				source.tagName !== "SOURCE") {
				var style = get_computed_style(source);
				if (style.getPropertyValue("background-image")) {
					var bgimg = style.getPropertyValue("background-image");
					if (!bgimg.match(/^(.*?\)\s*,)?\s*url[(]/)) {
						return false;
					}
				} else {
					return false;
				}
			}

			return !(source.width && source.width < thresh ||
					 source.height && source.height < thresh);
		}

		function get_computed_style(el) {
			return el.currentStyle || window.getComputedStyle(el);
		}

		var recalculate_rect = function(rect) {
			rect.left = rect.x;
			rect.top = rect.y;
			rect.right = rect.left + rect.width;
			rect.bottom = rect.top + rect.height;

			return rect;
		};

		var copy_rect = function(rect) {
			// simplified copy, need to use recalculate_rect after
			return {
				x: rect.x,
				y: rect.y,
				width: rect.width,
				height: rect.height
			};
		};

		var parse_zoom = function(zoom) {
			if (typeof zoom === "number")
				return zoom;

			var match = zoom.match(/^([-0-9.]+)%$/);
			if (match)
				return parseFloat(match[1]) / 100.;

			match = zoom.match(/^([-0-9.]+)$/);
			if (match) {
				return parseFloat(match[1]);
			}

			return null;
		};

		function get_bounding_client_rect_inner(el, mapcache, need_rect) {
			// test: https://4seasonstaeyeon.tumblr.com/post/190710743124 (bottom images)
			if (!el)
				return null;

			if (mapcache && mapcache.has(el)) {
				var value = mapcache.get(el);

				if (need_rect) {
					if (value.orig_rect)
						return value;
				} else {
					return value;
				}
			}

			var parent = {};
			var parentel = el.parentElement;
			if (parentel) {
				parent = get_bounding_client_rect_inner(parentel, mapcache, false);
			}

			var orig_rect = null;
			if (need_rect)
				orig_rect = el.getBoundingClientRect();

			var rect = null;
			var zoom = 1;

			//var computed_style = get_computed_style(el);
			// computed_style is slow, and also might not be what we're looking for, as it might contain the parent's zoom
			// this is still very slow though (50ms on facebook)
			if (el.style.zoom) {
				zoom = parse_zoom(el.style.zoom);
				if (zoom && zoom !== 1) {
					if (!orig_rect)
						orig_rect = el.getBoundingClientRect();

					rect = copy_rect(orig_rect);

					rect.width *= zoom;
					rect.height *= zoom;
				}
			}

			if (parent.zoom && parent.zoom !== 1) {
				if (!orig_rect)
					orig_rect = el.getBoundingClientRect();

				if (!rect)
					rect = copy_rect(orig_rect);

				rect.x *= parent.zoom;
				rect.y *= parent.zoom;
				rect.width *= parent.zoom;
				rect.height *= parent.zoom;

				zoom *= parent.zoom;
				//console.log(el, zoom, deepcopy(rect));
			}

			// this is surprisingly slow, so rect is optimized out if possible
			if (false && parent.rect && parent.orig_rect) {
				if (!orig_rect)
					orig_rect = el.getBoundingClientRect();

				if (!rect)
					rect = copy_rect(orig_rect);

				rect.x += parent.rect.x - parent.orig_rect.x;
				rect.y += parent.rect.y - parent.orig_rect.y;
			}

			if (rect)
				recalculate_rect(rect);

			var result = {
				zoom: zoom
			};

			if (orig_rect)
				result.orig_rect = orig_rect

			if (rect)
				result.rect = rect;

			if (mapcache) {
				mapcache.set(el, result);
			}

			return result;
		}

		function get_bounding_client_rect(el, mapcache) {
			var obj = get_bounding_client_rect_inner(el, mapcache, true);
			return obj.rect || obj.orig_rect;
		}

		function get_popup_client_rect() {
			if (!popups || !popups[0])
				return null;

			var current_date = Date.now();
			if (!popup_client_rect_cache || (current_date - last_popup_client_rect_cache) > 50) {
				popup_client_rect_cache = get_bounding_client_rect(popups[0]);
				last_popup_client_rect_cache = current_date;
			}

			return popup_client_rect_cache;
		};

		function is_popup_el(el) {
			var current = el;
			do {
				if (array_indexof(popups, current) >= 0) {
					//console_error("Trying to find popup");
					return true;
				}
			} while ((current = current.parentElement))

			return false;
		}

		var get_svg_src = function(el) {
			var remove_attributes = [
				"class",
				"id",
				"tabindex",
				"style"
			];

			var newel = document.createElementNS("http://www.w3.org/2000/svg", "svg");
			newel.innerHTML = el.innerHTML;

			newel.setAttribute("xmlns", "http://www.w3.org/2000/svg");

			var attrs = el.attributes;
			for (var i = 0; i < attrs.length; i++) {
				var attr_name = attrs[i].name;

				if (array_indexof(remove_attributes, attr_name) >= 0 || /^on/.test(attr_name) || /^aria-/.test(attr_name)) {
					continue;
				}

				newel.setAttribute(attr_name, attrs[i].value);
			}

			var computed_style = get_computed_style(el);
			if (computed_style.color) {
				newel.setAttribute("fill", computed_style.color);
			}

			var header = '<?xml version="1.0" encoding="UTF-8" standalone="no" ?>\n';
			var svgdoc = header + newel.outerHTML;

			return "data:image/svg+xml;base64," + base64_encode(svgdoc);
		};

		function get_img_src(el) {
			if (get_tagname(el) === "A")
				return el.href;

			if (get_tagname(el) === "CANVAS") {
				try {
					return el.toDataURL();
				} catch (e) {
					// "Tainted canvases may not be exported", CORS error in some pages
					return;
				}
			}

			if (get_tagname(el) === "SVG") {
				if (settings.mouseover_allow_svg_el) {
					return get_svg_src(el);
				} else {
					return null;
				}
			}

			if (get_tagname(el) === "VIDEO") {
				return el.currentSrc || el.src || el.poster;
			}

			// currentSrc is used if another image is used in the srcset
			return el.currentSrc || el.src;
		}

		function is_valid_src(src, isvideo) {
			return src && (!(/^blob:/.test(src)) || !isvideo);
		}

		function find_source(els) {
			//console_log(els);
			var ok_els = [];
			var result = _find_source(els, ok_els);

			if (_nir_debug_)
				console_log("find_source: result =", result, "ok_els =", ok_els);

			if (!result)
				return result;

			var ret_bad = function() {
				if (ok_els.length > 0)
					return ok_els[0];
				return null;
			};

			if (result.el) {
				if (is_popup_el(result.el)) {
					if (_nir_debug_)
						console_log("find_source: result.el is popup el", result.el);

					return ret_bad();
				}
			}

			if (!result.is_ok_el && !is_valid_src(result.src, is_video_el(result.el))) {
				if (_nir_debug_)
					console_log("find_source: invalid src", result);

				return ret_bad();
			}

			var thresh = parseInt(settings.mouseover_minimum_size);
			// if it can be imu'd, ignore the treshold because the image could be any size
			if (isNaN(thresh) || result.imu)
				thresh = 0;

			if ((!isNaN(result.width) && result.width > 0 && result.width < thresh) ||
				(!isNaN(result.height) && result.height > 0 && result.height < thresh)) {
				if (_nir_debug_)
					console_log("find_source: result size is too small");

				return ret_bad();
			}

			return result;
		}

		function _find_source(els, ok_els) {
			// resetpopups() is already called in trigger_popup()
			/*if (popups_active)
				return;*/

			if (_nir_debug_)
				console_log("_find_source (els)", els);

			var sources = {};
			//var picture_sources = {};
			var links = {};
			var layers = [];

			var id = 0;

			var thresh = parseInt(settings.mouseover_minimum_size);
			if (isNaN(thresh))
				thresh = 0;

			var helpers = do_get_helpers({});

			var source;

			function check_visible(el) {
				do {
					if (!el)
						break;

					var style = get_computed_style(el);
					if (!style)
						break;

					if (style.opacity.toString().match(/^0(?:\.0*)?$/))
						return false;
					if (style.visibility === "hidden")
						return false;
				} while (el = el.parentElement);

				return true;
			}

			function getsource() {
				var thesource = null;
				var first = false;
				for (var source in sources) {
					if (first)
						return;
					first = true;
					thesource = sources[source];
				}
				return thesource;
			}

			function getfirstsource(sources) {
				var smallestid = Number.MAX_SAFE_INTEGER;
				var thesource = null;
				for (var source_url in sources) {
					var source = sources[source_url];
					if (source.id < smallestid) {
						smallestid = source.id;
						thesource = sources[source_url];
					}
				}

				return thesource;
			}

			function norm(src) {
				return urljoin(window.location.href, src, true);
			}

			function imu_check(src, el) {
				var result = bigimage_recursive(src, {
					fill_object: true,
					use_cache: "read",
					do_request: null,
					document: document,
					window: get_window(),
					host_url: window.location.href,
					element: el,
					include_pastobjs: true,
					iterations: 2,
					cb: null
				});

				for (var i = 0; i < result.length; i++) {
					if (result[i].url !== src)
						continue;

					if (result[i].bad)
						return false;

					// if result.length > 1, then it can be imu'd
					if (result.length > 1) {
						return true;
					} else {
						return undefined;
					}
				}

				return true;
			}

			function addImage(src, el, options) {
				if (_nir_debug_) {
					console_log("_find_source (addImage)", src, el, check_visible(el), options);
				}

				if (!is_valid_resource_url(src))
					return false;

				if (src && settings.mouseover_apply_blacklist && !bigimage_filter(src)) {
					if (_nir_debug_)
						console_log("blacklisted");

					return false;
				}

				if (!(src in sources)) {
					sources[src] = {
						count: 0,
						src: src,
						el: el,
						id: id++,
					};
				}

				// blank images
				// https://www.harpersbazaar.com/celebrity/red-carpet-dresses/g7565/selena-gomez-style-transformation/?slide=2
				var el_style = null;
				if (el) {
					el_style = window.getComputedStyle(el) || el.style;
				}

				if (src && (src.match(/^data:/) && !(/^data:image\/svg\+xml;/.test(src)) && src.length <= 500) ||
					// https://www.smugmug.com/
					// https://www.vogue.com/article/lady-gaga-met-gala-2019-entrance-behind-the-scenes-video
					!check_visible(el)) {
					if (_nir_debug_)
						console_log("Invisible or tiny data: image");

					return false;
				}

				if (!options) {
					options = {};
				}

				if (options.isbg && settings.mouseover_exclude_backgroundimages) {
					return false;
				}

				var imucheck = imu_check(src, el);
				if (imucheck === false) {
					if (_nir_debug_)
						console_log("Bad image");

					return false;
				}

				if (!("imu" in sources[src])) {
					sources[src].imu = imucheck === true;
				}

				if (settings.mouseover_only_links) {
					if (!el)
						return false;

					var has_link = false;
					var current = el;
					do {
						if (get_tagname(current) === "A") {
							has_link = true;
							break;
						}
					} while (current = current.parentElement);

					if (!has_link)
						return false;
				}

				if ("layer" in options) {
					if (!(options.layer in layers)) {
						layers[options.layer] = [];
					}

					layers[options.layer].push(src);
				}

				sources[src].count++;

				return true;
			}

			function addTagElement(el, layer) {
				if (helpers && helpers.element_ok) {
					if (helpers.element_ok(el) === true) {
						ok_els.push({
							count: 1,
							src: null,
							el: el,
							id: id++,
							is_ok_el: true
						});
					}
				}

				var el_tagname = get_tagname(el);
				if (el_tagname === "PICTURE" || el_tagname === "VIDEO") {
					for (var i = 0; i < el.children.length; i++) {
						addElement(el.children[i], layer);
					}
				}

				if (el_tagname === "SOURCE" || el_tagname === "IMG" || el_tagname === "VIDEO" ||
					(settings.mouseover_allow_canvas_el && el_tagname === "CANVAS") ||
					(settings.mouseover_allow_svg_el && el_tagname === "SVG")) {
					var el_src = get_img_src(el);

					if (el_src) {
						var src = norm(el_src);

						addImage(src, el, { layer: layer });

						if (!el.srcset && src in sources) {
							var dimensions = get_el_dimensions(el);
							sources[src].width = dimensions[0];
							sources[src].height = dimensions[1];
						}
					}

					if (!el.srcset)
						return;

					var ssources = [];
					var srcset = el.srcset;

					// https://www.erinyamagata.com/art-direction/kiernan-shipka
					// https://format-com-cld-res.cloudinary.com/image/private/s--hNRUHHWH--/c_crop,h_1596,w_1249,x_0,y_0/c_fill,g_center,w_2500/fl_keep_iptc.progressive,q_95/v1/986aaf7dd74bd5041ddfc495c430bf0d/KShipkaR29MW_FULL_3468.jpg?2500 2500w 3194h, https://format-com-cld-res.cloudinary.com/image/private/s--VSup0NR3--/c_crop,h_1596,w_1249,x_0,y_0/c_fill,g_center,w_900/fl_keep_iptc.progressive,q_95/v1/986aaf7dd74bd5041ddfc495c430bf0d/KShipkaR29MW_FULL_3468.jpg?900 900w 1150h
					// newlines: https://www.rt.com/russia/447357-miss-moscow-2018-photos/
					//
					// https://cdni.rt.com/files/2018.12/xxs/5c221e1ffc7e9397018b4600.jpg 280w,
					// https://cdni.rt.com/files/2018.12/xs/5c221e1ffc7e9397018b4601.jpg 320w,
					// https://cdni.rt.com/files/2018.12/thumbnail/5c221e1ffc7e9397018b45ff.jpg 460w,
					// https://cdni.rt.com/files/2018.12/m/5c221e1ffc7e9397018b4602.jpg 540w,
					// https://cdni.rt.com/files/2018.12/l/5c221e1ffc7e9397018b4603.jpg 768w,
					// https://cdni.rt.com/files/2018.12/article/5c221e1ffc7e9397018b45fe.jpg 980w,
					// https://cdni.rt.com/files/2018.12/xxl/5c221e20fc7e9397018b4604.jpg 1240w
					while (srcset.length > 0) {
						var old_srcset = srcset;
						srcset = srcset.replace(/^\s+/, "");
						var match = srcset.match(/^(\S+(?:\s+[^,]+)?)(?:,[\s\S]*)?\s*$/);
						if (match) {
							ssources.push(match[1].replace(/\s*$/, ""));
							srcset = srcset.substr(match[1].length);
						}

						srcset = srcset.replace(/^\s*,/, "");
						if (srcset === old_srcset)
							break;
					}
					//var ssources = el.srcset.split(/ +[^ ,/],/);

					var sizes = [];
					if (el.sizes) {
						sizes = el.sizes.split(",");
					}

					// https://www.gamestar.de/artikel/red-dead-redemption-2-pc-vorabversion-mit-limit-bei-120-fps-directx-12-und-vulkan,3350718.html
					// sidebar articles: //8images.cgames.de/images/gamestar/256/red-dead-redemption-2_6062507.jpg, //8images.cgames.de/images/gamestar/210/red-dead-redemption-2_6062507.jpg 2x
					for (var i = 0; i < ssources.length; i++) {
						var src = norm(ssources[i].replace(/^(\S+)(?:\s+[\s\S]+)?\s*$/, "$1"));
						var desc = ssources[i].slice(src.length).replace(/^\s*([\s\S]*?)\s*$/, "$1");

						if (!addImage(src, el, {layer:layer}))
							continue;

						//picture_sources[src] = sources[src];

						sources[src].picture = el.parentElement;

						if (desc) {
							sources[src].desc = desc;

							// https://format-com-cld-res.cloudinary.com/image/pr…dc82/004_003_03-000083520001.jpg?2500 2500w 1831h
							while (desc.length > 0) {
								desc = desc.replace(/^\s+/, "");
								var whxmatch = desc.match(/^([0-9.]+)([whx])(?:\s+[0-9.]+[\s\S]*)?\s*$/);
								if (whxmatch) {
									var number = parseFloat(whxmatch[1]);

									if (number > 0) {
										// if width/height/desc_x > number, then number is probably more accurate (multiple els, see rt link above)
										if (whxmatch[2] === "w" && (!sources[src].width || sources[src].width > number))
											sources[src].width = number;
										else if (whxmatch[2] === "h" && (!sources[src].height || sources[src].height > number))
											sources[src].height = number;
										else if (whxmatch[2] === "x" && (!sources[src].desc_x || sources[src].desc_x > number))
											sources[src].desc_x = number;
									}

									desc = desc.substr(whxmatch[1].length + whxmatch[2].length);
								} else {
									break;
								}
							}
						}

						if (el.media) {
							sources[src].media = el.media;
							if (el.media.match(/min-width:\s*([0-9]+)/)) {
								picture_minw = true;
								var minWidth = getUnit(el.media.replace(/.*min-width:\s*([0-9.a-z]+).*/, "$1"));
								if (!sources[src].minWidth || sources[src].minWidth > minWidth)
									sources[src].minWidth = minWidth;
							}

							if (el.media.match(/max-width:\s*([0-9]+)/)) {
								picture_maxw = true;
								var maxWidth = getUnit(el.media.replace(/.*max-width:\s*([0-9.a-z]+).*/, "$1"));
								if (!sources[src].maxWidth || sources[src].maxWidth > maxWidth)
									sources[src].maxWidth = maxWidth;
							}

							if (el.media.match(/min-height:\s*([0-9]+)/)) {
								picture_minh = true;
								var minHeight = getUnit(el.media.replace(/.*min-height:\s*([0-9.a-z]+).*/, "$1"));
								if (!sources[src].minHeight || sources[src].minHeight > minHeight)
									sources[src].minHeight = minHeight;
							}

							if (el.media.match(/max-height:\s*([0-9]+)/)) {
								picture_maxh = true;
								var maxHeight = getUnit(el.media.replace(/.*max-height:\s*([0-9.a-z]+).*/, "$1"));
								if (!sources[src].maxHeight || sources[src].maxHeight > maxHeight)
									sources[src].maxHeight = maxHeight;
							}
						}
					}
				}

				if (el_tagname === "A") {
					var src = el.href;
					links[src] = {
						count: 1,
						src: src,
						el: el,
						id: id++
					};
				}
			}

			function _tokenize_css_value(str) {
				var tokensets = [];
				var tokens = [];

				var current_token = "";
				var quote = null;
				var escaping = false;
				for (var i = 0; i < str.length; i++) {
					var char = str[i];

					if (escaping) {
						current_token += char;
						escaping = false;
						continue;
					}

					if (quote) {
						if (char === quote) {
							quote = null;

							tokens.push(current_token);
							current_token = "";
						} else {
							current_token += char;
						}

						continue;
					}

					if (/\s/.test(char)) {
						if (current_token.length > 0) {
							tokens.push(current_token);
							current_token = "";
						}

						continue;
					} else if (char === '\\') {
						escaping = true;
						continue;
					} else if (char === '"' || char === "'") {
						quote = char;
						continue;
					} else if (char === '(') {
						var subtokens = _tokenize_css_value(str.substr(i + 1));
						tokens.push({name: current_token, tokens: subtokens[0]});
						i += subtokens[1];
						current_token = "";
						continue;
					} else if (char === ')') {
						i++;
						break;
					} else if (char === ',') {
						if (current_token)
							tokens.push(current_token);

						tokensets.push(tokens);

						tokens = [];
						current_token = "";
						continue;
					}

					current_token += char;
				}

				if (current_token)
					tokens.push(current_token);

				if (tokens)
					tokensets.push(tokens);

				return [tokensets, i];
			}

			function has_bgimage_url(tokenized) {
				for (var i = 0; i < tokenized.length; i++) {
					if (tokenized[i].length < 1)
						continue;

					if (typeof tokenized[i][0] !== "object")
						continue;

					var our_func = tokenized[i][0];

					var funcname = our_func.name;

					// TODO: support image() and cross-fade()
					var allowed = [
						"url",
						"-webkit-image-set",
						"image-set"
					];

					if (array_indexof(allowed, funcname) < 0)
						continue;

					if (funcname === "url") {
						if (our_func.tokens.length >= 1 && our_func.tokens[0].length > 0 && our_func.tokens[0][0].length > 0)
							return true;
					} else {
						if (has_bgimage_url(our_func.tokens))
							return true;
					}
				}

				return false;
			}

			function get_urlfunc_url(func) {
				if (typeof func !== "object")
					return null;;

				if (func.name !== "url")
					return null;

				if (func.tokens.length < 1 || func.tokens[0].length < 1 || func.tokens[0][0].length === 0)
					return null;

				return func.tokens[0][0];
			}

			function get_imageset_urls(func) {
				var urls = [];
				for (var i = 0; i < func.tokens.length; i++) {
					if (func.tokens[i].length < 1) {
						continue;
					}

					var url = get_urlfunc_url(func.tokens[i][0]);
					if (!url)
						continue;

					var source = {
						src: url
					};

					for (var j = 1; j < func.tokens[i].length; j++) {
						var desc = func.tokens[i][j];
						var whxmatch = desc.match(/^([0-9.]+)(x|dp(?:px|i|cm))$/);

						if (whxmatch) {
							var number = parseFloat(whxmatch[1]);

							if (number > 0) {
								var unit = whxmatch[2];

								// https://drafts.csswg.org/css-values-3/#cm
								if (unit === "dppx") {
									number *= 96;
									unit = "dpi";
								} else if (unit === "dpcm") {
									number *= 96/2.54;
									unit = "dpi";
								}

								if (unit === "x")
									source.desc_x = number;
								else if (unit === "dpi")
									source.dpi = number;
							}
						} else {
							console_warn("Unknown descriptor: " + desc);
						}
					}

					urls.push(source);
				}

				return urls;
			}

			function get_bgimage_urls(tokenized) {
				var urls = [];

				for (var i = 0; i < tokenized.length; i++) {
					if (tokenized[i].length < 1)
						continue;

					if (typeof tokenized[i][0] !== "object")
						continue;

					var our_func = tokenized[i][0];

					var funcname = our_func.name;

					// TODO: support image() and cross-fade()
					var allowed = [
						"url",
						"-webkit-image-set",
						"image-set"
					];

					if (array_indexof(allowed, funcname) < 0)
						continue;

					if (funcname === "url") {
						var url = get_urlfunc_url(our_func);
						if (url) {
							urls.push(url);
						}
					} else if (funcname === "-webkit-image-set" || funcname === "image-set") {
						var newurls = get_imageset_urls(our_func);
						if (newurls) {
							[].push.apply(urls, newurls);
						}
					}
				}

				return urls;
			}

			function get_urls_from_css(str, elstr) {
				var str_tokenized = _tokenize_css_value(str)[0];

				if (!has_bgimage_url(str_tokenized))
					return null;

				// -webkit-image-set(url('https://carbonmade-media.accelerator.net/34754698;460x194/lossless.webp') 1x, url('https://carbonmade-media.accelerator.net/34754698;920x388/lossless.webp') 2x)

				//var emptystrregex = /^(.*?\)\s*,)?\s*url[(]["']{2}[)]/;
				//if (!str.match(/^(.*?\)\s*,)?\s*url[(]/) || emptystrregex.test(str))
				//	return null;

				// window.getComputedStyle returns the window's URL in this case for some reason, so we need the element's style to find the empty string
				var elstr_tokenized;
				if (elstr) {
					elstr_tokenized = _tokenize_css_value(elstr)[0];
					if (!has_bgimage_url(elstr_tokenized))
						return null;
				}

				return get_bgimage_urls(str_tokenized);
			}

			function add_urls_from_css(el, str, elstr, layer, bg) {
				var urls = get_urls_from_css(str, elstr);
				if (urls) {
					var url;

					for (var i = 0; i < urls.length; i++) {
						url = urls[i];
						if (typeof url !== "string") {
							url = url.src;
						}

						addImage(url, el, {
							isbg: bg,
							layer: layer
						});

						if (typeof urls[i] !== "string") {
							var props = ["desc_x", "dpi"];

							for (var j = 0; j < props.length; j++) {
								var prop = props[j];

								if (urls[i][prop] && (!sources[url][prop] || sources[url][prop] > urls[i][prop])) {
									sources[url][prop] = urls[i][prop];
								}
							}
						}
					}
				}
			}

			function add_bgimage(layer, el, style, beforeafter) {
				if (style.getPropertyValue("background-image")) {
					var bgimg = style.getPropertyValue("background-image");
					add_urls_from_css(el, bgimg, el.style.getPropertyValue("background-image"), layer, beforeafter || true);
				}

				if (beforeafter) {
					if (style.getPropertyValue("content")) {
						add_urls_from_css(el, style.getPropertyValue("content"), undefined, layer, beforeafter);
					}
				}
			}

			function addElement(el, layer) {
				if (settings.mouseover_exclude_page_bg && el.tagName === "BODY") {
					return;
				}

				if (typeof layer === "undefined")
					layer = layers.length;

				addTagElement(el, layer);

				add_bgimage(layer, el, window.getComputedStyle(el));
				add_bgimage(layer, el, window.getComputedStyle(el, ":before"), "before");
				add_bgimage(layer, el, window.getComputedStyle(el, ":after"), "after");
			}

			for (var i = 0; i < els.length; i++) {
				// sidebar articles on https://www.rt.com/russia/447357-miss-moscow-2018-photos/
				// the <picture> element has a size of 0, and hence isn't added to find_els_at_point
				if (els[i].tagName === "IMG" && els[i].parentElement && els[i].parentElement.tagName === "PICTURE" && array_indexof(els, els[i].parentElement) < 0) {
					els.splice(i + 1, 0, els[i].parentElement);
				}

				// remove every element before PICTURE as they will be added automatically anyways
				// this messes up the layering
				if (els[i].tagName === "PICTURE" && i == 1) {
					els.splice(0, i);
					i = 0;
					break;
				}
			}

			for (var i = 0; i < els.length; i++) {
				var el = els[i];
				addElement(el);
			}

			if (_nir_debug_) {
				//console_log(els);
				console_log("_find_source (sources)", deepcopy(sources));
				console_log("_find_source (layers)", deepcopy(layers));
			}

			// remove sources that aren't used
			var activesources = [];
			for (var i = 0; i < layers.length; i++) {
				for (var j = 0; j < layers[i].length; j++) {
					if (array_indexof(activesources, layers[i][j]) < 0)
						activesources.push(layers[i][j]);
				}
			}

			for (var source in sources) {
				for (var i = 0; i < ok_els.length; i++) {
					if (sources[source].el === ok_els[i].el) {
						ok_els[i] = sources[source];
					}
				}

				if (array_indexof(activesources, source) < 0)
					delete sources[source];
			}

			if ((source = getsource()) !== undefined) {
				if (_nir_debug_)
					console_log("_find_source (getsource())", source);

				if (source === null) {
					if (ok_els.length > 0) {
						return ok_els[0];
					} else if (get_single_setting("mouseover_links")) {
						if (Object.keys(links).length > 0) {
							var our_key = null;

							for (var link in links) {
								if (!settings.mouseover_only_valid_links) {
									our_key = link;
									break;
								}

								if (looks_like_valid_link(link, links[link].el)) {
									our_key = link;
									break;
								}
							}

							if (our_key)
								return links[our_key];
						}
					}
				}

				return source;
			}

			for (var i = 0; i < layers.length; i++) {
				var minW = 0;
				var elW = null;
				var minH = 0;
				var elH = null;
				var minMinW = 0;
				var elMinW = null;
				var minMinH = 0;
				var elMinH = null;
				var minMaxW = 0;
				var elMaxW = null;
				var minMaxH = 0;
				var elMaxH = null;
				var minX = 0;
				var elX = null;
				var minDpi = 0;
				var elDpi = null;

				var okurls = {};

				var have_something = false;

				for (var j = 0; j < layers[i].length; j++) {
					var source_url = layers[i][j];

					var source = sources[source_url];

					if (source.width && source.width > minW) {
						minW = source.width;
						elW = source;
						have_something = true;
					}

					if (source.height && source.height > minH) {
						minH = source.height;
						elH = source;
						have_something = true;
					}

					if (source.minWidth && source.minWidth > minMinW) {
						minMinW = source.minWidth;
						elMinW = source;
						have_something = true;
					}

					if (source.minHeight && source.minHeight > minMinH) {
						minMinH = source.minHeight;
						elMinH = source;
						have_something = true;
					}

					if (source.maxWidth && source.maxWidth > minMaxW) {
						minMaxW = source.maxWidth;
						elMaxW = source;
					}

					if (source.maxHeight && source.maxHeight > minMaxH) {
						minMaxH = source.maxHeight;
						elMaxH = source;
					}

					if (source.desc_x && source.desc_x > minX) {
						minX = source.desc_x;
						elX = source;
						have_something = true;
					}

					if (source.dpi && source.dpi > minDpi) {
						dpiX = source.dpi;
						elDpi = source;
						have_something = true;
					}

					if (source.isbg) {
						okurls[source.src] = true;
						have_something = true;
					}
				}

				if (!have_something)
					continue;

				if (minX > 1) {
					okurls[elX.src] = true;
				}

				if (minDpi > 96) {
					okurls[elDpi.src] = true;
				}

				if (minW > thresh && minW > minMinW) {
					okurls[elW.src] = true;
				}

				if (minH > thresh && minH > minMinH) {
					okurls[elH.src] = true;
				}

				if (minMinW > thresh && minMinW >= minW) {
					okurls[elMinW.src] = true;
				}

				if (minMinH > thresh && minMinH >= minH) {
					okurls[elMinH.src] = true;
				}

				layers[i] = [];
				for (var url in okurls) {
					layers[i].push(url);
				}
			}

			// TODO: improve?
			function pickbest(layer) {
				for (var i = 0; i < layer.length; i++) {
					var source_url = layer[i];
					var source = sources[source_url];

					if (source.desc_x)
						return source;
				}

				return sources[layer[0]];
			}

			function rebuildlayers() {
				var newlayers = [];
				for (var i = 0; i < layers.length; i++) {
					if (layers[i].length === 0)
						continue;
					newlayers.push(layers[i]);
				}
				layers = newlayers;
			}

			if (_nir_debug_)
				console_log("_find_source (new layers)", deepcopy(layers));

			rebuildlayers();

			if (_nir_debug_)
				console_log("_find_source (rebuilt layers)", deepcopy(layers));

			// If there are background images ahead of an image, it's likely to be masks
			// Maybe check if there's more than one element of ancestry between them?
			// Except: https://www.flickr.com/account/upgrade/pro (featured pro, the avatar's image is a bg image, while the image behind isn't)
			if (layers.length > 1 && layers[0].length === 1 && sources[layers[0][0]].isbg) {
				for (var i = 1; i < layers.length; i++) {
					if (layers[i].length === 1 && sources[layers[i][0]].isbg)
						continue;
					return pickbest(layers[i]);
				}
			}

			if (layers.length > 0) {
				return pickbest(layers[0]);
			}

			if (source = getsource())
				return source;
			else
				return getfirstsource(sources);
		}

		get_next_in_gallery = function(el, nextprev) {
			if (!el)
				return null;

			// https://www.gog.com/game/shadow_warrior_complete
			// "Buy series" gallery
			if (el.tagName === "SOURCE" && el.parentElement && array_indexof(["PICTURE", "VIDEO"], el.parentElement.tagName) >= 0) {
				el = el.parentElement;
			}

			var stack = [el.tagName];
			var current_el = el;
			var firstchild = false;

			while (true) {
				if (!firstchild) {
					var next = current_el.nextElementSibling;
					if (!nextprev)
						next = current_el.previousElementSibling;

					if (!next) {
						current_el = current_el.parentElement;
						if (!current_el)
							break;

						stack.unshift(current_el.tagName);
						continue;
					}

					current_el = next;
				} else {
					firstchild = false;
				}

				if (current_el.tagName === stack[0]) {
					if (stack.length === 1) {
						//if (valid_source(current_el))
						if (is_valid_el(current_el))
							return current_el;
						continue;
					}

					if (nextprev) {
						for (var i = 0; i < current_el.children.length; i++) {
							if (current_el.children[i].tagName === stack[1]) {
								current_el = current_el.children[i];
								stack.shift();
								firstchild = true;
								break;
							}
						}
					} else {
						for (var i = current_el.children.length - 1; i >= 0; i--) {
							if (current_el.children[i].tagName === stack[1]) {
								current_el = current_el.children[i];
								stack.shift();
								firstchild = true;
								break;
							}
						}
					}
				}
			}

			return null;
		}

		/*function normalize_trigger() {
			if (!is_array(settings.mouseover_trigger)) {
				settings.mouseover_trigger = [settings.mouseover_trigger];
			}
		}

		normalize_trigger();*/

		function update_mouseover_trigger_delay() {
			delay = settings.mouseover_trigger_delay;
			if (delay < 0 || isNaN(delay))
				delay = false;
			if (typeof delay === "number" && delay >= 10)
				delay = 10;

			if (settings.mouseover_trigger_behavior === "mouse") {
				delay_mouseonly = true;
			} else {
				delay = false;
				delay_mouseonly = false;
			}

			if (delay_handle) {
				clearTimeout(delay_handle);
				delay_handle = null;
			}
		}

		update_mouseover_trigger_delay();

		settings_meta.mouseover_trigger_delay.onupdate = update_mouseover_trigger_delay;
		settings_meta.mouseover_trigger_behavior.onupdate = update_mouseover_trigger_delay;

		function can_add_to_chord(str) {
			if (!keystr_is_wheel(str))
				return true;

			return !chord_is_only_wheel(current_chord);
		}

		function clear_chord_wheel() {
			for (var i = 0; i < current_chord.length; i++) {
				if (keystr_is_wheel(current_chord[i])) {
					current_chord.splice(i, 1);
					i--;
				}
			}
		}

		function clear_chord() {
			current_chord = [];
			current_chord_timeout = {};
		}

		function clear_chord_if_only_wheel() {
			if (chord_is_only_wheel(current_chord))
				clear_chord();
		}

		function keystr_in_trigger(str, wanted_chord) {
			if (wanted_chord === undefined)
				wanted_chord = settings.mouseover_trigger_key;

			return array_indexof(wanted_chord, str) >= 0;
		}

		function key_would_modify_single_chord(str, value) {
			if (value) {
				if (!can_add_to_chord(str))
					return false;

				if (array_indexof(current_chord, str) < 0)
					return true;
			} else {
				if (array_indexof(current_chord, str) >= 0)
					return true;
			}

			return false;
		}

		function set_chord_sub(str, value) {
			if (value) {
				if (!can_add_to_chord(str))
					return false;

				current_chord_timeout[str] = Date.now();
				if (array_indexof(current_chord, str) < 0) {
					current_chord.push(str);
					//console_log("+" + str);
					return true;
				}
			} else {
				delete current_chord_timeout[str];
				if (array_indexof(current_chord, str) >= 0) {
					current_chord.splice(array_indexof(current_chord, str), 1);
					clear_chord_if_only_wheel();
					//console_log("-" + str);
					return true;
				}
			}

			return false;
		}

		function event_in_single_chord(e, wanted_chord) {
			var map = get_keystrs_map(e, true);

			for (var key in map) {
				if (keystr_in_trigger(key, wanted_chord))
					return true;
			}

			return false;
		}

		function event_in_chord(e, wanted_chord) {
			wanted_chord = normalize_keychord(wanted_chord);

			for (var i = 0; i < wanted_chord.length; i++) {
				if (event_in_single_chord(e, wanted_chord[i]))
					return true;
			}

			return false;
		}

		function remove_old_keys() {
			var now = Date.now();

			for (var key in current_chord_timeout) {
				if (now - current_chord_timeout[key] > 5000)
					set_chord_sub(key, false);
			}
		}

		function update_chord(e, value) {
			var map = get_keystrs_map(e, value);

			remove_old_keys();

			var changed = false;
			for (var key in map) {
				if (set_chord_sub(key, map[key]))
					changed = true;
			}

			return changed;
		}

		function event_would_modify_single_chord(e, value, wanted_chord) {
			var map = get_keystrs_map(e, value)

			for (var key in map) {
				if (wanted_chord !== undefined && !keystr_in_trigger(key, wanted_chord))
					continue;

				if (key_would_modify_single_chord(key, map[key]))
					return true;
			}

			return false;
		}

		function event_would_modify_chord(e, value, wanted_chord) {
			wanted_chord = normalize_keychord(wanted_chord);

			for (var i = 0; i < wanted_chord.length; i++) {
				if (event_would_modify_single_chord(e, value, wanted_chord[i]))
					return true;
			}

			return false;
		}

		function trigger_complete_single(wanted_chord) {
			for (var i = 0; i < wanted_chord.length; i++) {
				var key = wanted_chord[i];

				if (array_indexof(current_chord, key) < 0)
					return false;
			}

			// e.g. if the user presses shift+r, but the chord is r, then it should fail
			for (var i = 0; i < current_chord.length; i++) {
				if (keystr_is_wheel(current_chord[i]))
					continue;

				if (array_indexof(wanted_chord, current_chord[i]) < 0)
					return false;
			}

			return true;
		}

		function trigger_complete(wanted_chord) {
			if (wanted_chord === undefined)
				wanted_chord = settings.mouseover_trigger_key;

			wanted_chord = normalize_keychord(wanted_chord);

			for (var i = 0; i < wanted_chord.length; i++) {
				if (trigger_complete_single(wanted_chord[i]))
					return true;
			}

			return false;
		}

		function trigger_partially_complete_single(e, wanted_chord) {
			for (var i = 0; i < wanted_chord.length; i++) {
				var key = wanted_chord[i];

				if (array_indexof(current_chord, key) >= 0)
					return true;
			}

			return false;
		}

		function trigger_partially_complete(e, wanted_chord) {
			if (wanted_chord === undefined)
				wanted_chord = settings.mouseover_trigger_key;

			wanted_chord = normalize_keychord(wanted_chord);

			for (var i = 0; i < wanted_chord.length; i++) {
				if (trigger_partially_complete_single(e, wanted_chord[i]))
					return true;
			}

			return false;
		}

		function get_close_behavior() {
			return get_single_setting("mouseover_close_behavior");
		}

		function get_close_need_mouseout() {
			return settings.mouseover_close_need_mouseout && get_close_behavior() !== "esc";
		}

		function get_close_on_leave_el() {
			return settings.mouseover_close_on_leave_el && get_single_setting("mouseover_position") === "beside_cursor";
		}

		function should_exclude_imagetab() {
			return settings.mouseover_exclude_imagetab && get_single_setting("mouseover_trigger_behavior") === "mouse" &&
				   currenttab_is_image() && !imagetab_ok_override;
		}

		function find_els_at_point(xy, els, prev, zoom_cache) {
			// test for pointer-events: none: https://www.shacknews.com/article/114834/should-you-choose-vulkan-or-directx-12-in-red-dead-redemption-2

			if (false && _nir_debug_)
				console_log("find_els_at_point", deepcopy(xy), deepcopy(els), deepcopy(prev));

			if (!prev) {
				prev = new_set();
			}

			if (zoom_cache === undefined) {
				try {
					zoom_cache = new Map();
				} catch (e) {
					zoom_cache = null;
				}
			}

			var ret = [];
			var afterret = [];

			if (!els) {
				els = document.elementsFromPoint(xy[0], xy[1]);
				afterret = els;

				if (!settings.mouseover_support_pointerevents_none)
					return els;
			}

			for (var i = 0; i < els.length; i++) {
				var el = els[i];

				if (set_has(prev, el))
					continue;

				set_add(prev, el);

				// FIXME: should we stop checking if not in bounding client rect?
				// this would depend on the fact that children are always within the bounding rect
				//  - probably not, there are cases where the parent div has a size of 0, but children have proper sizes
				if (el.childElementCount > 0) {
					// reverse, because the last element is (usually) the highest z
					var newchildren = [];
					for (var j = el.children.length - 1; j >= 0; j--) {
						newchildren.push(el.children[j]);
					}

					var newels = find_els_at_point(xy, newchildren, prev, zoom_cache);
					for (var j = 0; j < newels.length; j++) {
						var newel = newels[j];
						//console_log("about to add", newel, deepcopy(ret))
						if (array_indexof(ret, newel) < 0) {
							//console_log("adding", newel);
							ret.push(newel);
						}
					}
				}

				var rect = get_bounding_client_rect(el, zoom_cache);
				if (rect && rect.width > 0 && rect.height > 0 &&
					rect.left <= xy[0] && rect.right >= xy[0] &&
					rect.top <= xy[1] && rect.bottom >= xy[1] &&
					array_indexof(ret, el) < 0) {
					ret.push(el);
				}
			}

			for (var i = 0; i < afterret.length; i++) {
				if (array_indexof(ret, afterret[i]) < 0)
					ret.push(afterret[i]);
			}

			if (_nir_debug_ && ret.length > 0) {
				console_log("find_els_at_point (unsorted ret)", shallowcopy(ret));
			}

			var get_zindex_raw = function(el) {
				var zindex = get_computed_style(el).zIndex;

				if (zindex === "auto") {
					if (el.parentElement) {
						return get_zindex(el.parentElement);// + 0.001; // hack: child elements appear above parent elements
						// don't use the above hack, it breaks z-ordering, the indexOf thing works already
					} else {
						return 0;
					}
				} else {
					return parseFloat(zindex);
				}
			};

			var get_zindex = function(el) {
				if (zoom_cache) {
					var cached = map_get(zoom_cache, el);
					if (!cached || !("zIndex" in cached)) {
						var zindex = get_zindex_raw(el);

						if (cached) {
							cached.zIndex = zindex;
						} else {
							map_set(zoom_cache, el, {
								zIndex: zindex
							});
						}

						return zindex;
					} else {
						return cached.zIndex;
					}
				} else {
					return get_zindex_raw(el);
				}
			};

			// TODO: only sort elements that were added outside of elementsFromPoint
			ret.sort(function(a, b) {
				var a_zindex, b_zindex;

				a_zindex = get_zindex(a);
				b_zindex = get_zindex(b);

				//console_log(a_zindex, b_zindex, a, b);
				if (b_zindex === a_zindex) {
					// Don't modify the sort order
					return array_indexof(ret, a) - array_indexof(ret, b);
				} else {
					// opposite because we want it to be reversed (largest first)
					return b_zindex - a_zindex;
				}
			});

			if (_nir_debug_ && ret.length > 0)
				console_log("find_els_at_point (ret)", els, shallowcopy(ret), xy);

			return ret;
		}

		var get_physical_popup_el = function(el) {
			if (el.parentElement && el.tagName === "SOURCE")
				return el.parentElement;
			return el;
		}

		function trigger_popup(is_contextmenu) {
			if (_nir_debug_)
				console_log("trigger_popup (is_contextmenu=" + is_contextmenu + ")", current_frame_id);

			controlPressed = true;
			delay_handle_triggering = true;
			//var els = document.elementsFromPoint(mouseX, mouseY);
			var point = null;

			if (mousepos_initialized)
				point = [mouseX, mouseY];
			if (is_contextmenu)
				point = [mouseContextX, mouseContextY];

			if (point === null) {
				delay_handle_triggering = false;
				return;
			}

			var els = find_els_at_point(point);
			//console_log(els);

			if (_nir_debug_)
				console_log("trigger_popup: els =", els);

			var source = find_source(els);

			if (_nir_debug_)
				console_log("trigger_popup: source =", source);

			if (source && (popup_trigger_reason !== "mouse" || get_physical_popup_el(source.el) !== last_popup_el)) {
				trigger_popup_with_source(source);
			} else {
				if (popup_trigger_reason === "keyboard") {
					cursor_not_allowed();
				}

				delay_handle_triggering = false;
			}
		}

		function trigger_popup_with_source(source, automatic, use_last_pos) {
			next_popup_el = get_physical_popup_el(source.el);

			return get_final_from_source(source, automatic, false, use_last_pos, function(source_imu, source, processing, data) {
				if (!source_imu && !source && !processing && !data) {
					delay_handle_triggering = false;
					return;
				}

				//console_log(source_imu);
				resetpopups();

				if (automatic) {
					popup_el_automatic = true;
					removepopups(); // don't fade out
				}

				real_popup_el = source.el;
				popup_el = get_physical_popup_el(real_popup_el);
				if (popup_el.parentElement) // check if it's a fake element returned by a gallery helper
					popup_orig_el = popup_el;
				popup_el_is_video = is_video_el(popup_el);

				popup_orig_url = get_img_src(popup_el);

				if (is_in_iframe && can_iframe_popout() && get_single_setting("mouseover_open_behavior") === "popup") {
					data.data.img = serialize_img(data.data.img);
					remote_send_message("top", {
						type: "make_popup",
						data: {
							source_imu: source_imu,
							src: source.src,
							processing: processing,
							data: data
						}
					});
				} else {
					makePopup(source_imu, source.src, processing, data);

					if (is_in_iframe && can_use_remote() && get_single_setting("mouseover_open_behavior") === "popup") {
						remote_send_message("top", {
							type: "popup_open"
						});
					}
				}
			});
		}

		function get_final_from_source(source, automatic, multi, use_last_pos, cb) {
			// FIXME: "multi" is used purely for replace_images,
			//   so it's used as a check for many options that have nothing to do with multi

			var processing = {running: true};

			if (!multi) {
				stop_processing();
				processing_list = [processing];
			}

			//console_log(source);

			var do_popup = function() {
				if (!multi)
					start_waiting(source.el);

				var x = mouseX;
				var y = mouseY;

				if (use_last_pos) {
					x = null;
					y = null;
				}

				var realcb = function(source_imu, data) {
					//console_log(source_imu);
					//console_log(data);
					if ((!source_imu && false) || !data) {
						if (!multi)
							stop_waiting();
						return cb();
					}

					// In case the user has dragged while loading the next image (#154)
					if (use_last_pos) {
						x = null;
						y = null;
					}

					cb(source_imu, source, processing, {
						data: data,
						x: x,
						y: y
					});
				};

				try {
					var force_page = settings.mouseover_ui_caption && settings.redirect_force_page;

					bigimage_recursive_loop(source.src, {
						fill_object: true,
						host_url: window.location.href,
						document: document,
						window: get_window(),
						element: source.el,
						force_page: force_page,
						cb: realcb
					}, function(obj, finalcb) {
						var orig_obj = obj;

						if (_nir_debug_)
							console_log("do_popup: brl query:", obj);

						if (multi && obj[0].url === source.src) {
							return finalcb(source.src, obj[0], null);
						}

						var newobj = deepcopy(obj);

						if (!settings.mouseover_exclude_sameimage) {
							if (source.src && obj_indexOf(newobj, source.src) < 0)
								newobj.push(fillobj(source.src)[0]);
						} else if (source.src) {
							var index;

							while ((index = obj_indexOf(newobj, source.src)) >= 0) {
								newobj.splice(index, 1);
							}
						}

						var usehead = false;

						if (!multi && get_single_setting("mouseover_open_behavior") === "newtab") {
							usehead = true;
						} else if (multi && !get_single_setting("replaceimgs_usedata")) {
							usehead = true;
						} else if (!multi) {
							var partial = get_single_setting("mouseover_allow_partial");

							if (partial === "media") {
								processing.incomplete_image = true;
								processing.incomplete_video = true;
							} else if (partial === "video") {
								processing.incomplete_video = true;
							}
						}

						if (is_in_iframe && can_iframe_popout()) {
							processing.incomplete_image = true;
							processing.incomplete_video = true;
						}

						if (usehead) {
							processing.head = true;
						}

						processing.source = source;

						check_image_get(newobj, function(img, newurl, obj, respdata) {
							if (_nir_debug_)
								console_log("do_popup: check_image_get response:", img, newurl, obj, respdata);

							if (!img) {
								return finalcb(null);
							}

							var data = {img: img, newurl: newurl, obj: obj, respdata: respdata};
							var newurl1 = newurl;

							if (usehead) {
								data = {resp: img, obj: newurl};
								newurl1 = data.resp.finalUrl;
							}

							if (settings.print_imu_obj)
								console_log(orig_obj);

							finalcb(newurl1, data.obj, data);

							if (false) {
								// why?
								if (newurl == source.src) {
									realcb(obj, data);
								} else {
									finalcb(newurl, data);
								}
							}
						}, processing);
					});
				} catch (e) {
					console_error(e);
					//console.trace();
					// this doesn't work
					//makePopup(source.src);
				}
			};

			if (delay && !delay_mouseonly && !automatic) {
				start_progress(source.el);
				delay_handle = setTimeout(function() {
					if (delay_handle_triggering)
						return;

					delay_handle = null;
					do_popup();
				}, delay * 1000);
			} else {
				do_popup();
			}
		}

		function do_get_helpers(options) {
			var baseoptions = {
				document: document,
				window: get_window(),
				host_url: window.location.href,
				do_request: do_request,
				rule_specific: {}
			};

			for (var option in options) {
				baseoptions[option] = options[option];
			}

			return get_helpers(baseoptions);
		}

		function wrap_gallery_func(nextprev, origel, el, cb, new_options) {
			if (!el)
				el = real_popup_el;

			if (!origel)
				origel = popup_orig_el;

			var options = {
				element: origel,
				document: document,
				window: get_window(),
				host_url: window.location.href,
				do_request: do_request,
				rule_specific: {},
				cb: function(result) {
					if (result === undefined || result === "default") {
						return cb(get_next_in_gallery(el, nextprev));
					} else {
						cb(result);
					}
				}
			};

			if (new_options) {
				for (var key in new_options) {
					options[key] = new_options[key];
				}
			}

			get_bigimage_extoptions_first(options);
			get_bigimage_extoptions(options);

			var helpers = get_helpers(options);
			var gallery = get_next_in_gallery;

			if (helpers && helpers.gallery) {
				gallery = function(el, nextprev) {
					var value = helpers.gallery(el, nextprev);
					if (value || value === null)
						return value;

					return get_next_in_gallery(el, nextprev);
				};
			}

			var value = gallery(el, nextprev);
			if (value === "waiting") {
				return;
			} else if (value === "default") {
				return cb(get_next_in_gallery(el, nextprev));
			}

			return cb(value);
		}

		function is_valid_el(el) {
			if (!el)
				return false;

			return !!find_source([el]);
		}

		function count_gallery(nextprev, max, origel, el, cb) {
			var count = 0;

			if (max === undefined)
				max = settings.mouseover_ui_gallerymax;

			var firstel = el;
			if (!firstel)
				firstel = real_popup_el;

			if (!firstel && popup_el_remote && can_iframe_popout() && !is_in_iframe) {
				return remote_send_message(popup_el_remote, {
					type: "count_gallery",
					data: {
						nextprev: nextprev
					}
				}, function(count) {
					cb(count);
				});
			}

			var loop = function() {
				wrap_gallery_func(nextprev, origel, el, function(newel) {
					if (!newel || !is_valid_el(newel))
						return cb(count, el);

					count++;

					if (count >= max)
						return cb(count, newel);

					el = newel;
					loop();
				}, {is_counting: true, counting_firstel: firstel});
			};

			loop();
		}

		function wrap_gallery_cycle(dir, origel, el, cb) {
			if (!el)
				el = real_popup_el;

			if (dir === 0)
				return cb();

			var nextprev = true;
			var max = dir;
			if (dir < 0) {
				nextprev = false;
				max = -dir;
			}

			count_gallery(nextprev, max, origel, el, function(count, newel) {
				if (count < max) {
					if (settings.mouseover_gallery_cycle) {
						count_gallery(!nextprev, undefined, origel, el, function(count, newel) {
							cb(newel);
						});
					} else {
						cb(null);
					}
				} else {
					cb(newel);
				}
			});
		}

		function is_nextprev_valid(nextprev, cb) {
			if (popup_el_remote && can_iframe_popout() && !is_in_iframe) {
				return remote_send_message(popup_el_remote, {
					type: "is_nextprev_valid",
					data: {
						nextprev: nextprev
					}
				}, function(valid) {
					cb(valid);
				});
			}

			wrap_gallery_cycle(nextprev ? 1 : -1, undefined, undefined, function(el) {
				cb(is_valid_el(el));
			});
		}

		trigger_gallery = function(dir, cb) {
			if (!cb) {
				cb = nullfunc;
			}

			if (popup_el_remote && can_iframe_popout() && !is_in_iframe) {
				return remote_send_message(popup_el_remote, {
					type: "trigger_gallery",
					data: {
						dir: dir
					}
				}, function(triggered) {
					cb(triggered);
				});
			}

			wrap_gallery_cycle(dir, undefined, undefined, function(newel) {
				if (newel) {
					var source = find_source([newel]);
					if (source) {
						trigger_popup_with_source(source, true, true);
						return cb(true);
					}
				}

				return cb(false);
			});
		}

		var parse_transforms = function(transform) {
			var transforms = [];
			var transform_types = {};

			var last = 0;
			for (var i = 0; i < transform.length; i++) {
				if (transform[i] === ')') {
					var our_transform = strip_whitespace(transform.substr(last, (i - last) + 1));
					var type = our_transform.replace(/\(.*/, "");
					transforms.push(our_transform);

					if (!(type in transform_types)) {
						transform_types[type] = [];
					}
					transform_types[type].push(transforms.length - 1);

					last = i + 1;
					continue;
				}
			}

			return {transforms: transforms, types: transform_types};
		};

		var get_popup_transforms = function() {
			var style = popups[0].querySelector("img").parentElement.parentElement.style;
			if (style.transform) {
				return parse_transforms(style.transform);
			} else {
				return {transforms: [], types: {}};
			}
		}

		var stringify_transforms = function(transforms) {
			return transforms.transforms.join(" ");
		};

		var set_popup_transforms = function(transforms) {
			popups[0].querySelector("img").parentElement.parentElement.style.transform = stringify_transforms(transforms);
		};

		function rotate_gallery(dir) {
			if (!popups_active)
				return;

			var transforms = get_popup_transforms();

			var index = 0;
			if ("rotate" in transforms.types) {
				index = transforms.types.rotate[0];
			} else {
				transforms.transforms.unshift("rotate(0deg)");
			}

			var match = transforms.transforms[index].match(/^rotate\(([-0-9]+)deg\)$/);
			var deg = 0;
			if (match) {
				deg = parseInt(match[1]);
			}

			transforms.transforms[index] = "rotate(" + (deg + dir) + "deg)";
			set_popup_transforms(transforms);
		}

		// hv: vertical = true, horizontal = false
		var flip_gallery = function(hv) {
			if (!popups_active)
				return;

			var transforms = get_popup_transforms();

			var index = transforms.transforms.length;
			if ("scale" in transforms.types) {
				index = transforms.types.scale[0];
			} else {
				transforms.transforms.push("scale(1,1)");
			}

			var match = transforms.transforms[index].match(/^scale\(([-0-9.]+)\s*,\s*([-0-9.]+)\)$/);
			var scaleh = 1;
			var scalev = 1;

			if (match) {
				scaleh = parseFloat(match[1]);
				scalev = parseFloat(match[2]);
			}

			if (hv) {
				scalev = -scalev;
			} else {
				scaleh = -scaleh;
			}

			transforms.transforms[index] = "scale(" + scaleh + ", " + scalev + ")";
			set_popup_transforms(transforms);
		};

		function create_progress_el() {
			var progressc_el = document.createElement("div");
			set_el_all_initial(progressc_el);
			progressc_el.style.backgroundColor = "rgba(0,0,0,0.7)";
			//progressc_el.style.padding = "1em";
			progressc_el.style.height = "2em";
			progressc_el.style.zIndex = maxzindex - 2;

			var progressb_el = document.createElement("div");
			set_el_all_initial(progressb_el);
			progressb_el.style.position = "absolute";
			progressb_el.style.top = "0px";
			progressb_el.style.left = "0px";
			//progressb_el.style.backgroundColor = "#00aa00";
			progressb_el.style.backgroundColor = "#00aaff";
			progressb_el.style.height = "100%";
			progressb_el.style.width = "0%";
			progressb_el.style.zIndex = maxzindex - 1;

			progressc_el.appendChild(progressb_el);

			return progressc_el;
		}

		function update_progress_el(el, percent) {
			var bar = el.children[0];

			if (typeof percent === "number") {
				if (bar.getAttribute("data-timer")) {
					clearInterval(parseInt(bar.getAttribute("data-timer")));
					bar.removeAttribute("data-timer");
				}

				bar.style.width = (percent * 100) + "%";
			} else if (percent == "unknown") {
				bar.style.width = "10%";

				if (!bar.getAttribute("data-timer")) {
					bar.style.left = "0%";
					bar.setAttribute("data-dir", "right");
					var timer = setInterval(function() {
						var left = parseFloat(bar.style.left);
						var delta = (15/1000) * 1;
						var size = 90;

						if (bar.getAttribute("data-dir") == "right") {
							left += (delta * size);
							if (left >= size) {
								left = size - (left - size);
								bar.setAttribute("data-dir", "left");
							}
						} else {
							left -= (delta * size);
							if (left <= 0) {
								left = -left;
								bar.setAttribute("data-dir", "right");
							}
						}

						bar.style.left = left + "%";
					}, 15);

					bar.setAttribute("data-timer", timer);
				}
			}
		}

		var looks_like_valid_link = function(src, el) {
			if (/\.(?:jpe?g|png|web[mp]|gif|mp4|mkv|og[gv]|svg)(?:[?#].*)?$/i.test(src))
				return true;

			if (el && check_highlightimgs_supported_image(el))
				return true;

			return false;
		}

		var is_img_pic_vid = function(el) {
			return el.tagName === "IMG" || el.tagName === "PICTURE" || el.tagName === "VIDEO";
		};

		var is_img_pic_vid_link = function(el) {
			if (is_img_pic_vid(el))
				return true;

			if (settings.mouseover_links && el.tagName === "A")
				return true;

			return false;
		}

		var get_all_valid_els = function(el) {
			if (!el)
				el = document;
			return el.querySelectorAll("img, picture, video");
		};

		var get_all_valid_els_link = function(el) {
			var query = "img, picture, video";
			if (settings.mouseover_links) {
				query += ", a";
			}

			if (!el)
				el = document;
			return el.querySelectorAll(query);
		};

		var replacing_imgs = false;
		var replaceimgs_elcache = new Cache();
		function replace_images(options) {
			if (replacing_imgs || currenttab_is_image())
				return;

			var raw_imgs = options.images;

			if (raw_imgs === undefined) {
				raw_imgs = get_all_valid_els();
			}

			// remove non-images/videos
			var imgs = [];
			for (var i = 0; i < raw_imgs.length; i++) {
				if (is_img_pic_vid(raw_imgs[i])) {
					imgs.push(raw_imgs[i]);
				}
			}

			if (imgs.length === 0)
				return;

			if (options.use_progressbar)
				console_log("Replacing images");

			var finished = 0;

			var finish_img = function() {
				finished++;

				if (options.use_progressbar) {
					update_progress_el(progressc_el, finished / total_imgs);
					console_log("Finished " + finished + "/" + total_imgs);
				}

				if (finished >= total_imgs) {
					if (options.use_progressbar)
						progressc_el.parentElement.removeChild(progressc_el);

					replacing_imgs = false;
				} else {
					next_img();
				}
			};

			var next_img = function () {
				var total_limit = parseInt(settings.replaceimgs_totallimit);
				if (currently_processing > total_limit) {
					currently_processing--;
					return;
				} else if (currently_processing < total_limit) {
					currently_processing++;
					next_img();
				}


				var our_source = null;
				var our_domain = null;

				for (var domain in domains) {
					if (domains_processing[domain] >= parseInt(settings.replaceimgs_domainlimit)) {
						continue;
					}

					our_domain = domain;

					domains_processing[domain]++;

					our_source = domains[domain][0];
					domains[domain].splice(0, 1);

					if (domains[domain].length === 0) {
						delete domains[domain];
					}

					break;
				}

				if (our_source === null && other.length > 0) {
					our_source = other[0];
					other.splice(0, 1);
				}

				if (options.use_elcache && our_source) {
					if (replaceimgs_elcache.has(our_source.el)) {
						return finish_img();
					} else {
						// Not perfect, but 5 seconds should be enough
						replaceimgs_elcache.set(our_source.el, true, 5);
					}
				}

				if (our_source) {
					get_final_from_source(our_source, true, true, false, function (source_imu, source, processing, data) {
						if (our_domain)
							domains_processing[our_domain]--;

						var replace_func = function(el, newsrc, url) {
							if (options.replace_imgs && get_img_src(el) !== newsrc) {
								el.src = newsrc;
							}

							if (options.add_links) {
								var current = el;

								while (current = current.parentElement) {
									if (current.tagName === "A") {
										if (!options.replace_links)
											return;
										else
											break;
									}
								}

								if (!current) {
									current = document.createElement("a");

									el.parentElement.insertBefore(current, el);
									current.appendChild(el);
								}

								if (current.href !== url) {
									current.href = url;
								}
							}
						};

						if (!data) {
							replace_func(our_source.el, our_source.src, our_source.src);
							return finish_img();
						}

						var waiting = false;
						if (data.data.img) {
							replace_func(source.el, data.data.img.src, data.data.obj.url);
						} else if (data.data.obj) {
							var load_image = function() {
								if (settings.replaceimgs_wait_fullyloaded && options.replace_imgs) {
									// Preload the image, as adding onload/onerror to existing images won't fire the event
									var image = new Image();
									var finish_image = function () {
										replace_func(source.el, image.src, data.data.obj.url);
										finish_img();
									};

									image.onload = finish_image;
									image.onerror = finish_img;
									image.src = data.data.obj.url;
								} else {
									replace_func(source.el, data.data.obj.url, data.data.obj.url);
									finish_img();
								}
							};

							if (is_extension) {
								extension_send_message({
									type: "override_next_headers",
									data: {
										url: data.data.obj.url,
										headers: data.data.obj.headers,
										method: "GET"
									}
								}, function() {
									load_image();
								});
							} else {
								load_image();
							}

							waiting = true;
						}

						if (!waiting)
							finish_img();
					});
				} else {
					currently_processing--;
				}
			};

			var progressc_el;

			if (options.use_progressbar) {
				progressc_el = create_progress_el();
				progressc_el.style.position = "fixed";
				progressc_el.style.top = "0px";
				progressc_el.style.left = "0px";
				progressc_el.style.width = "80%";
				progressc_el.style.marginTop = "100px";
				progressc_el.style.marginLeft = "10%";
				document.documentElement.appendChild(progressc_el);
			}

			var domains = {};
			var domains_processing = {};
			var other = [];

			var total_imgs = imgs.length;

			for (var i = 0; i < imgs.length; i++) {
				var source = find_source([imgs[i]]);
				if (!source) {
					total_imgs--;
					continue;
				}

				if (!source.src) {
					other.push(source);
					continue;
				}

				var domain = source.src.match(/^https?:\/\/([^/]+)\//);
				if (!domain) {
					other.push(source);
					continue;
				}

				if (!(domain[1] in domains)) {
					domains[domain[1]] = [];
					domains_processing[domain[1]] = 0;
				}

				domains[domain[1]].push(source);
			}

			var currently_processing = 1;
			next_img();
		}

		var replace_images_full = function(options) {
			var base_options = {
				replace_imgs: settings.replaceimgs_replaceimgs,
				add_links: settings.replaceimgs_addlinks,
				replace_links: settings.replaceimgs_replacelinks,
				use_progressbar: true
			};

			if (!options)
				options = {};

			for (var key in options) {
				base_options[key] = options[key];
			}

			return replace_images(base_options);
		};

		register_menucommand("Replace images", replace_images_full);

		var generate_random_class = function(name) {
			return "imu-" + get_random_text(10) + "-" + name;
		};

		var highlightimgs_styleel = null;
		var highlightimgs_classname = generate_random_class("highlight");
		var update_highlight_styleel = function() {
			if (!highlightimgs_styleel) {
				highlightimgs_styleel = document.createElement("style");
				document.documentElement.appendChild(highlightimgs_styleel);
			}

			highlightimgs_styleel.innerText = "." + highlightimgs_classname + "{" + get_styletag_styles(settings.highlightimgs_css) + "}";
		}

		onload(function() {
			update_highlight_styleel();
		});

		(function() {
			var oldfunc = settings_meta.highlightimgs_css.onupdate;
			settings_meta.highlightimgs_css.onupdate = function() {
				update_highlight_styleel();

				if (oldfunc)
					return oldfunc.apply(this, arguments);
			};
		})();

		var apply_highlight_style = function(target) {
			target.classList.add(highlightimgs_classname);
		};

		var remove_highlight_style = function(target) {
			target.classList.remove(highlightimgs_classname);
		}

		var check_highlightimgs_valid_image = function(el) {
			var src = get_img_src(el);
			if (!is_valid_src(src, is_video_el(el)) || (el.tagName === "A" && !looks_like_valid_link(src, el)))
				return false;
			return true;
		};

		var get_highlightimgs_valid_image = function(el) {
			// TODO: dynamic attribute name
			if (el.hasAttribute("data-imu-valid")) {
				return !!parse_boolean(el.getAttribute("data-imu-valid"));
			}

			var valid = check_highlightimgs_valid_image(el);
			el.setAttribute("data-imu-valid", valid + "");
			return valid;
		};

		var check_highlightimgs_supported_image = function(el) {
			var src = get_img_src(el);

			var imu_output = bigimage_recursive(src, {
				fill_object: true,
				exclude_problems: [], // todo: use settings' exclude_problems instead
				use_cache: "read",
				//use_cache: false,
				use_api_cache: false,
				host_url: window.location.href,
				element: el,
				document: document,
				window: get_window(),
				cb: function() {},
				do_request: function() {}
			});

			// TODO: consolidate into its own routine
			if (imu_output.length !== 1)
				return true;

			var imu_obj = imu_output[0];
			if (imu_obj.url !== src)
				return true;

			for (var key in imu_obj) {
				if (key === "url")
					continue;

				if (!(key in default_object))
					return true;

				// e.g. for []
				if (JSON_stringify(default_object[key]) !== JSON_stringify(imu_obj[key]))
					return true;
			}

			return false;
		};

		var get_highlightimgs_supported_image = function(el) {
			// TODO: dynamic attribute name
			if (el.hasAttribute("data-imu-supported")) {
				return !!parse_boolean(el.getAttribute("data-imu-supported"));
			}

			var supported = check_highlightimgs_supported_image(el);
			el.setAttribute("data-imu-supported", supported + "");
			return supported;
		};

		var auto_highlighted_imgs = [];
		var highlight_images = function(options) {
			if (currenttab_is_image() && settings.mouseover_exclude_imagetab)
				return;

			if (!options) {
				options = {}
			}

			var images = options.images;
			if (images === undefined) {
				images = get_all_valid_els_link();
			}

			if (!images.length)
				return;

			for (var i = 0; i < images.length; i++) {
				if (!get_highlightimgs_valid_image(images[i]))
					continue;

				supported = !settings.highlightimgs_onlysupported;

				if (settings.highlightimgs_onlysupported) {
					supported = get_highlightimgs_supported_image(images[i]);
				}

				if (!options.hoveronly) {
					if (supported) {
						if (options.is_auto && array_indexof(auto_highlighted_imgs, images[i]) < 0) {
							auto_highlighted_imgs.push(images[i]);
						}

						apply_highlight_style(images[i]);
					} else {
						remove_highlight_style(images[i]);
					}
				}
			}
		};

		(function() {
			var added = false;
			var id = null;

			var update_button = function() {
				if (settings.highlightimgs_enable) {
					if (!added) {
						id = register_menucommand("Highlight images", highlight_images);
						added = true;
					}
				} else {
					if (added) {
						unregister_menucommand(id);
						added = false;
					}
				}
			};

			update_button();

			var origfunc = settings_meta.highlightimgs_enable.onupdate;
			settings_meta.highlightimgs_enable.onupdate = function() {
				update_button();

				if (origfunc)
					return origfunc.apply(this, arguments);
			};
		})();

		var popup_mouse_head = function() {
			if (delay_handle_triggering || trigger_complete(settings.mouseover_trigger_prevent_key))
				return false;

			popup_trigger_reason = "mouse";
			return true;
		};

		var image_mouseover = function(e) {
			if (currenttab_is_image() && settings.mouseover_exclude_imagetab)
				return;

			if (get_single_setting("highlightimgs_auto") === "hover" && get_highlightimgs_valid_image(e.target)) {
				var supported = !settings.highlightimgs_onlysupported;
				if (!supported) {
					supported = get_highlightimgs_supported_image(e.target);
				}

				if (supported) {
					if (array_indexof(auto_highlighted_imgs, e.target) < 0)
						auto_highlighted_imgs.push(e.target);
					apply_highlight_style(e.target);
				}
			}

			if (mouseover_mouse_enabled() && settings.mouseover_trigger_mouseover && !delay_handle && !should_exclude_imagetab()) {
				delay_el = e.target;
				delay_handle = setTimeout(function() {
					delay_el = null;

					if (delay_handle) {
						clearTimeout(delay_handle);
						delay_handle = null;
					}

					if (!popup_mouse_head())
						return;

					var source = find_source([e.target]);

					if (source && get_physical_popup_el(source.el) !== last_popup_el) {
						trigger_popup_with_source(source);
					}
				}, delay * 1000);
			}
		};

		var image_mouseout = function(e) {
			if (get_single_setting("highlightimgs_auto") === "hover" && get_highlightimgs_valid_image(e.target)) {
				remove_highlight_style(e.target);
			}

			if (mouseover_mouse_enabled() && settings.mouseover_trigger_mouseover && delay_handle) {
				if (delay_el === e.target) {
					clearTimeout(delay_handle);
					delay_handle = null;
				}
			}
		};

		function on_new_images(images) {
			var highlight = get_single_setting("highlightimgs_auto");
			if (highlight === "always" || highlight === "hover")
				highlight_images({images: images, hoveronly: highlight === "hover", is_auto: true});

			for (var i = 0; i < images.length; i++) {
				// apparently this isn't needed to ensure no duplicate event listeners?
				our_removeEventListener(images[i], "mouseover", image_mouseover);
				our_removeEventListener(images[i], "mouseout", image_mouseout);

				our_addEventListener(images[i], "mouseover", image_mouseover);
				our_addEventListener(images[i], "mouseout", image_mouseout);
			}

			if (settings.replaceimgs_auto)
				replace_images_full({images: images, use_progressbar: false, use_elcache: true});
		};

		(function() {
			if (!settings.imu_enabled)
				return;

			// TODO: allow this to be automatically updated
			if (settings.apply_blacklist_host && !bigimage_filter(window.location.href))
				return;

			var observer;

			var observe_options = {childList: true, subtree: true, attributes: true};

			var new_mutationobserver = function() {
				return new MutationObserver(function(mutations, observer) {
					var images = [];

					var add_nodes = function(nodes) {
						for (var i = 0; i < nodes.length; i++) {
							if (is_img_pic_vid_link(nodes[i])) {
								images.push(nodes[i]);
							}

							if (nodes[i].children) {
								add_nodes(nodes[i].children);
							}
						}
					};

					for (var i = 0; i < mutations.length; i++) {
						var mutation = mutations[i];

						if (mutation.addedNodes) {
							add_nodes(mutation.addedNodes);
						}

						if (mutation.target && mutation.type === "attributes") {
							if (mutation.attributeName === "src" || mutation.attributeName === "href" || mutation.attributeName === "srcset") {
								add_nodes([mutation.target]);
							}
						}
					}

					if (images.length > 0) {
						on_new_images(images);
					}
				});
			}

			var observe = function() {
				if (!settings.imu_enabled)
					return;

				on_new_images(get_all_valid_els_link());

				if (!observer)
					return;
				observer.observe(document, observe_options);
			};

			var remove_all_highlights = function() {
				for (var i = 0; i < auto_highlighted_imgs.length; i++) {
					remove_highlight_style(auto_highlighted_imgs[i]);
				}

				auto_highlighted_imgs = [];
			}

			var disconnect = function() {
				remove_all_highlights();

				if (!observer)
					return;

				observer.disconnect();
			};

			var needs_observer = function() {
				var highlight = get_single_setting("highlightimgs_auto");
				return highlight === "always" || highlight === "hover" || settings.replaceimgs_auto || (mouseover_mouse_enabled() && settings.mouseover_trigger_mouseover);
			};

			var create_mutationobserver = function() {
				try {
					// In case the browser doesn't support MutationObservers
					observer = new_mutationobserver();
				} catch (e) {
					console_warn(e);
				}

				if (needs_observer()) {
					observe();
				}
			};

			create_mutationobserver();

			var update_highlightimgs_func = function() {
				if (needs_observer()) {
					if (get_single_setting("highlightimgs_auto") !== "always") {
						remove_all_highlights();
					}

					observe();
				} else {
					disconnect();
				}
			};

			// replaceimgs_auto is intentionally not added here due to the warning
			var orig_highlightfunc = settings_meta.highlightimgs_auto.onupdate;
			settings_meta.highlightimgs_auto.onupdate = function() {
				if (orig_highlightfunc)
					orig_highlightfunc();

				update_highlightimgs_func();
			};

			var orig_imuenabledfunc = settings_meta.imu_enabled.onupdate;
			settings_meta.imu_enabled.onupdate = function() {
				if (orig_imuenabledfunc)
					orig_imuenabledfunc.apply(this, arguments);

				if (settings.imu_enabled) {
					update_highlightimgs_func();
				} else {
					disconnect();
				}
			};
		})();

		var get_popup_media_url = function() {
			var imgels = popups[0].getElementsByTagName("video");
			if (imgels.length === 0)
				imgels = popups[0].getElementsByTagName("img");
			var url = imgels[0].src;

			return url;
		};

		var do_browser_download = function(imu, filename, cb) {
			var a = document.createElement("a");

			a.href = imu.url;

			if (filename && filename.length > 0) {
				a.setAttribute("download", filename);
			} else {
				var attr = document.createAttribute("download");
				a.setAttributeNode(attr);
			}

			a.style.display = "none";
			a.onclick = function(e) {
				e.stopPropagation();
				e.stopImmediatePropagation();
				return true;
			};

			document.body.appendChild(a);
			a.click();

			setTimeout(function() {
				document.body.removeChild(a);
			}, 500);

			if (cb)
				cb();
		};

		var do_download = function(imu, filename, cb) {
			if (_nir_debug_) {
				console_log("do_download", imu, filename, cb);
			}

			if (is_extension) {
				extension_send_message({
					type: "download",
					data: {
						imu: imu
					}
				}, function() {
					if (cb)
						cb();
				});
			} else if (is_userscript && typeof GM_download !== "undefined") {
				var headers;

				if (imu.headers)
					headers = headers_dict_to_list(imu.headers);

				var download_obj = {
					url: imu.url,
					headers: headers,
					saveAs: true,
					onerror: function(error) {
						if (error && error.error && error.error !== "not_succeeded") {
							do_browser_download(imu, filename, cb);
						}
					}
				};

				if (filename) {
					download_obj.name = filename;
				} else {
					download_obj.name = "download"; // it can't be blank
				}

				GM_download(download_obj);
			} else {
				do_browser_download(imu, filename, cb);
			}
		};

		var download_popup_image = function() {
			do_download(popup_obj, popup_obj.filename);
		};

		var get_popup_video = function() {
			var videoel = popups[0].getElementsByTagName("video");
			if (!videoel || videoel.length === 0)
				return null;

			return videoel[0];
		};

		var seek_popup_video = function(leftright) {
			var timemul = leftright ? -1 : 1;
			var time = timemul * settings.mouseover_video_seek_amount;

			var videoel = get_popup_video();
			if (!videoel)
				return;

			videoel.currentTime += time;
		};

		var popup_video_speed = function(downup) {
			var videoel = get_popup_video();
			if (!videoel)
				return;

			var amount = settings.mouseover_video_speed_amount;
			if (downup)
				amount = -amount;

			videoel.playbackRate += amount;
		};

		var toggle_video_muted = function() {
			var videoel = get_popup_video();
			if (!videoel)
				return;

			videoel.muted = !videoel.muted;
		};

		var toggle_video_controls = function() {
			var videoel = get_popup_video();
			if (!videoel)
				return;

			if (videoel.getAttribute("controls") === "controls") {
				videoel.removeAttribute("controls");
			} else {
				videoel.setAttribute("controls", "controls");
			}
		};

		var toggle_video_playing = function() {
			var videoel = get_popup_video();
			if (!videoel)
				return;

			if (videoel.paused) {
				videoel.play();
			} else {
				videoel.pause();
			}
		};

		var popup_active = function() {
			return popups_active && popup_el;
		};

		var can_use_hold_key = function() {
			if (!popups_active)
				return false;

			if (popup_trigger_reason !== "mouse" && (!settings.mouseover_auto_close_popup || !settings.mouseover_auto_close_popup_time))
				return false;

			return settings.mouseover_use_hold_key;
		};

		var update_popup_hold = function() {
			if (!popups_active)
				return;

			if (popup_update_pos_func && settings.mouseover_hold_position_center) {
				var newpos = popup_update_pos_func(mouseX, mouseY, true);
				popups[0].style.top = newpos[1] + "px";
				popups[0].style.left = newpos[0] + "px";
			}
		};

		var action_handler = function(action) {
			if (action.needs_popup && !popup_active())
				return;

			switch (action.type) {
				case "resetpopups":
					resetpopups();
					return true;
				case "replace_images":
					replace_images_full();
					return true;
				case "highlight_images":
					highlight_images();
					return true;
				case "trigger_popup":
					if (action.trigger === "keyboard") {
						popup_trigger_reason = "keyboard";
					}

					trigger_popup();
					return true;
				case "gallery_prev":
					trigger_gallery(-1);
					return true;
				case "gallery_next":
					trigger_gallery(1);
					return true;
				case "download":
					download_popup_image();
					return true;
				case "open_in_new_tab":
					open_in_tab_imu(popup_obj, action.background_tab);
					return true;
				case "rotate_left":
					rotate_gallery(-90);
					return true;
				case "rotate_right":
					rotate_gallery(90);
					return true;
				case "flip_horizontal":
					flip_gallery(false);
					return true;
				case "flip_vertical":
					flip_gallery(true);
					return true;
				case "zoom_in":
					popup_zoom_func("incremental", -1);
					return true;
				case "zoom_out":
					popup_zoom_func("incremental", 1);
					return true;
				case "zoom_full":
					popup_zoom_func("fitfull", -1);
					return true;
				case "zoom_fit":
					popup_zoom_func("fitfull", 1);
					return true;
				case "seek_left":
					seek_popup_video(true);
					return true;
				case "seek_right":
					seek_popup_video(false);
					return true;
				case "speed_down":
					popup_video_speed(true);
					return true;
				case "speed_up":
					popup_video_speed(false);
					return true;
				case "toggle_mute":
					toggle_video_muted();
					return true;
				case "toggle_controls":
					toggle_video_controls();
					return true;
				case "toggle_play_pause":
					toggle_video_playing();
					return true;
				case "open_options":
					open_in_tab_imu({url: preferred_options_page}, false);
					return true;
				case "hold":
					update_popup_hold();
					return true;
			}

			return false;
		};

		var action_remote = function(actions) {
			// use can_use_remote instead of can_iframe_popout because this doesn't necessarily pop out of iframes
			if (can_use_remote()) {
				var recipient = "top";
				var has_mouse = true;

				if (!is_in_iframe) {
					recipient = mouse_frame_id;
					if (recipient === "top") {
						has_mouse = false;

						if (popup_el_remote) {
							recipient = popup_el_remote;
						} else {
							return;
						}
					}
				}

				for (var i = 0; i < actions.length; i++) {
					if (!has_mouse && actions[i].requires_mouse) {
						actions.splice(i, 1);
						i--;
					}
				}

				remote_send_message(recipient, {
					type: "action",
					data: actions
				});
			}
		};

		var keydown_cb = function(event) {
			if (!mouseover_enabled())
				return;

			if (event.type === "wheel" && chord_is_only_wheel(current_chord))
				return;

			if (event.type === "keydown") {
				if (editing_text)
					return;

				if (settings.disable_keybind_when_editing) {
					if (event.target.tagName === "TEXTAREA")
						return;

					if (event.target.tagName === "INPUT") {
						if (!event.target.hasAttribute("type") || event.target.getAttribute("type") === "text") {
							return;
						}
					}

					// TODO: support editable divs?
				}
			}

			var ret = undefined;
			var actions = [];

			update_chord(event, true);

			if (settings.mouseover_trigger_behavior === "keyboard" && event_in_chord(event, settings.mouseover_trigger_key)) {
				if (trigger_complete(settings.mouseover_trigger_key) && !popups_active) {
					// clear timeout so that all/any close behavior works
					current_chord_timeout = {};
					if (!delay_handle) {
						actions.push({
							requires_mouse: true,
							type: "trigger_popup",
							trigger: "keyboard"
						});

						ret = false;
						release_ignore = settings.mouseover_trigger_key;
					}
				}

				var close_behavior = get_close_behavior();
				if (close_behavior === "all" || (close_behavior === "any" && trigger_complete(settings.mouseover_trigger_key))) {
					can_close_popup[0] = false;
				}
			}

			if (can_use_hold_key() && event_in_chord(event, settings.mouseover_hold_key)) {
				if (trigger_complete(settings.mouseover_hold_key)) {
					popup_hold = !popup_hold;
					clear_resetpopup_timeout();

					if (!popup_hold && can_close_popup[1]) {
						actions.push({type: "resetpopups"});
					} else {
						actions.push({type: "hold"});
					}
				}
			}


			if (settings.replaceimgs_enable_keybinding && trigger_complete(settings.replaceimgs_keybinding)) {
				actions.push({type: "replace_images"});

				ret = false;
				release_ignore = settings.replaceimgs_keybinding;
			}

			if (settings.highlightimgs_enable_keybinding && trigger_complete(settings.highlightimgs_keybinding)) {
				actions.push({type: "highlight_images"});

				ret = false;
				release_ignore = settings.highlightimgs_keybinding;
			}


			if (true) {
				var is_popup_active = popup_el_remote || (popup_active());

				var keybinds = [
					{
						key: settings.mouseover_gallery_prev_key,
						action: {type: "gallery_prev"},
						requires: settings.mouseover_enable_gallery
					},
					{
						key: settings.mouseover_gallery_next_key,
						action: {type: "gallery_next"},
						requires: settings.mouseover_enable_gallery
					},
					{
						key: settings.mouseover_download_key,
						// Clear the chord because keyup might not be called due to the save dialog popup
						clear: true,
						action: {type: "download"}
					},
					{
						key: settings.mouseover_open_new_tab_key,
						// Clear the chord because opening in a new tab will not release the keys
						clear: true,
						action: {type: "open_in_new_tab"}
					},
					{
						key: settings.mouseover_open_bg_tab_key,
						action: {type: "open_in_new_tab", background_tab: true}
					},
					{
						key: settings.mouseover_open_options_key,
						// Clear the chord because opening in a new tab will not release the keys
						clear: true,
						action: {type: "open_options"}
					},
					{
						key: settings.mouseover_rotate_left_key,
						action: {type: "rotate_left"}
					},
					{
						key: settings.mouseover_rotate_right_key,
						action: {type: "rotate_right"}
					},
					{
						key: settings.mouseover_flip_horizontal_key,
						action: {type: "flip_horizontal"}
					},
					{
						key: settings.mouseover_flip_vertical_key,
						action: {type: "flip_vertical"}
					},
					{
						key: settings.mouseover_zoom_in_key,
						action: {type: "zoom_in"}
					},
					{
						key: settings.mouseover_zoom_out_key,
						action: {type: "zoom_out"}
					},
					{
						key: settings.mouseover_zoom_full_key,
						action: {type: "zoom_full"}
					},
					{
						key: settings.mouseover_zoom_fit_key,
						action: {type: "zoom_fit"}
					},
					{
						key: settings.mouseover_video_seek_left_key,
						action: {type: "seek_left"}
					},
					{
						key: settings.mouseover_video_seek_right_key,
						action: {type: "seek_right"}
					},
					{
						key: settings.mouseover_video_speed_down_key,
						action: {type: "speed_down"}
					},
					{
						key: settings.mouseover_video_speed_up_key,
						action: {type: "speed_up"}
					},
					{
						key: settings.mouseover_video_mute_key,
						action: {type: "toggle_mute"}
					},
					{
						key: settings.mouseover_video_controls_key,
						action: {type: "toggle_controls"}
					},
					{
						key: settings.mouseover_video_playpause_key,
						action: {type: "toggle_play_pause"}
					}
				];

				for (var i = 0; i < keybinds.length; i++) {
					if (trigger_complete(keybinds[i].key)) {
						if ("requires" in keybinds[i]) {
							if (!keybinds[i].requires)
								continue;
						}

						var action = keybinds[i].action;
						action.needs_popup = true;

						actions.push(action);

						if (keybinds[i].clear)
							clear_chord();

						if (is_popup_active) {
							release_ignore = keybinds[i].key;
							ret = false;
						}

						break;
					}
				}
			}

			if (!release_ignore || !release_ignore.length) {
				release_ignore = [];
			} else {
				release_ignore = deepcopy(release_ignore);
			}

			if (actions && actions.length > 0) {
				clear_chord_wheel();

				for (var i = 0; i < actions.length; i++) {
					action_handler(actions[i]);
				}

				action_remote(actions);
			}

			if (ret === false) {
				try {
					event.preventDefault();
					event.stopImmediatePropagation();
					event.stopPropagation();
				} catch (e) {}
			}

			return ret;
		};

		our_addEventListener(document, 'keydown', keydown_cb, true);
		our_addEventListener(document, 'mousedown', keydown_cb, true);
		our_addEventListener(document, 'contextmenu', keydown_cb, true);
		our_addEventListener(document, 'wheel', keydown_cb, {
			capture: true,
			passive: false
		});

		var keyup_cb = function(event) {
			if (!mouseover_enabled())
				return;

			var ret = undefined;

			var condition = event_would_modify_chord(event, false, settings.mouseover_trigger_key);

			update_chord(event, false);

			var close_behavior = get_close_behavior();
			if (condition && close_behavior === "all") {
				condition = !trigger_partially_complete(event);
			}

			if (condition && close_behavior !== "esc" && popup_trigger_reason === "keyboard" && popups_active) {
				controlPressed = false;

				if (!settings.mouseover_close_need_mouseout || can_close_popup[1]) {
					stop_waiting();
					resetpopups();
				} else {
					can_close_popup[0] = true;
				}

				return;
			}

			// 27 = esc
			if (event.which === 27) {
				if (delay_handle_triggering && popup_trigger_reason === "mouse" && !settings.mouseover_cancel_popup_with_esc)
					return;

				stop_waiting();
				resetpopups();
			}

			if (release_ignore.length > 0) {
				var map = get_keystrs_map(event, false);
				for (var key in map) {
					var index = array_indexof(release_ignore, key);
					if (index >= 0) {
						release_ignore.splice(index, 1);
						ret = false;
					}
				}
			}

			if (ret === false) {
				try {
					event.preventDefault();
					event.stopImmediatePropagation();
					event.stopPropagation();
				} catch (e) {}
			}

			return ret;
		};

		our_addEventListener(document, 'keyup', keyup_cb, true);
		our_addEventListener(document, 'mouseup', keyup_cb, true);

		function scrollLeft() {
			var doc = document.documentElement;
			var body = document.body;
			return (doc && doc.scrollLeft || body && body.scrollLeft || 0) -
				(doc && doc.clientLeft || body && body.clientLeft || 0);
		}

		function scrollTop() {
			var doc = document.documentElement;
			var body = document.body;
			return (doc && doc.scrollTop || body && body.scrollTop || 0) -
				(doc && doc.clientTop || body && body.clientTop || 0);
		}

		var get_move_with_cursor = function() {
			// don't require this for now, because esc can also be used to close the popup
			//var close_el_policy = get_single_setting("mouseover_close_el_policy");
			// maybe disable if popup position == center, and "move within page" is activated?
			return settings.mouseover_move_with_cursor && !popup_hold;// && close_el_policy === "thumbnail";
		};

		function do_popup_pan(popup, event, mouseX, mouseY) {
			var pan_behavior = get_single_setting("mouseover_pan_behavior");
			var move_with_cursor = get_move_with_cursor();
			if (pan_behavior === "drag" && (event.buttons === 0 || !dragstart) && !move_with_cursor)
				return;

			var viewport = get_viewport();
			var edge_buffer = 40;
			var border_thresh = 20;
			var min_move_amt = parseInt(settings.mouseover_drag_min);
			var moved = false;

			// lefttop: true = top, false = left

			var dodrag = function(lefttop) {
				var orig = parseInt(lefttop ? popup.style.top : popup.style.left);

				var mousepos = lefttop ? mouseY : mouseX;
				var dragoffset = lefttop ? dragoffsetY : dragoffsetX;
				var last = lefttop ? lastY : lastX;

				var current = mousepos - dragoffset;

				if (current !== orig) {
					if (dragged || Math.abs(current - orig) >= min_move_amt) {
						var newlast = current - (orig - last);

						if (lefttop) {
							lastY = newlast;
							popup.style.top = current + "px";
						} else {
							lastX = newlast;
							popup.style.left = current + "px";
						}

						dragged = true;
						moved = true;
					}
				}
			};

			var popup_clientrect = null;
			var domovement = function(lefttop) {
				if (!popup_clientrect) {
					popup_clientrect = get_popup_client_rect();
				}

				// offset* is very slow, slower than setting top/left! 250ms vs 30ms after a while
				//var offsetD = lefttop ? popup.offsetHeight : popup.offsetWidth;
				var offsetD = lefttop ? popup_clientrect.height : popup_clientrect.width;
				var viewportD = lefttop ? viewport[1] : viewport[0];
				var mousepos = lefttop ? mouseY : mouseX;

				if (!settings.mouseover_movement_inverted)
					mousepos = viewportD - mousepos;

				if (offsetD > viewportD) {
					var mouse_edge = Math.min(Math.max((mousepos - edge_buffer), 0), viewportD - edge_buffer * 2);
					var percent = mouse_edge / (viewportD - (edge_buffer * 2));

					var newpos = (percent * (viewportD - offsetD - border_thresh * 2) + border_thresh) + "px";

					if (lefttop)
						popup.style.top = newpos;
					else
						popup.style.left = newpos;

					moved = true;
				}
			};

			var update_pos_cache = null;
			var domovewith = function(lefttop) {
				var orig = parseInt(lefttop ? popup.style.top : popup.style.left);

				var mousepos = lefttop ? mouseY : mouseX;
				//var popupopen = lefttop ? popupOpenY : popupOpenX;
				var last = lefttop ? popupOpenLastY : popupOpenLastX;

				var current = mousepos - last + orig;

				if (settings.mouseover_move_within_page) {
					if (false) {
						var offsetD = lefttop ? popup.offsetHeight : popup.offsetWidth;
						var viewportD = lefttop ? viewport[1] : viewport[0];

						current = Math.max(current, border_thresh);

						if (current + offsetD > (viewportD - border_thresh)) {
							current = viewportD - border_thresh - offsetD;
						}
					} else if (popup_update_pos_func) {
						if (!update_pos_cache)
							update_pos_cache = popup_update_pos_func(mouseX, mouseY, false);

						current = update_pos_cache[lefttop ? 1 : 0];
					}
				}

				if (current === orig)
					return;

				var newlast = mousepos;

				if (lefttop) {
					popupOpenLastY = newlast;
					popup.style.top = current + "px";
				} else {
					popupOpenLastX = newlast;
					popup.style.left = current + "px";
				}
			};

			if (pan_behavior === "drag" && dragstart) {
				dodrag(false);
				dodrag(true);
			} else if (pan_behavior === "movement") {
				domovement(false);
				domovement(true);
			}

			if (move_with_cursor) {
				// make sure to fix this for remote (this is called at the top frame, but popup_el is remote)
				//var popup_el_rect = popup_el.getBoundingClientRect();
				var popup_el_rect = null;
				// don't check for now, maybe add this as an option later?
				if (true || in_clientrect(mouseX, mouseY, popup_el_rect)) {
					domovewith(false);
					domovewith(true);
				}
			}

			if (moved) {
				mouse_in_image_yet = false;
			}
		}

		var remote_handle_message = function(message, sender, respond) {
			//console_log("ON_REMOTE_MESSAGE", message, sender, respond);
			if (message.type === "make_popup") {
				if (!is_in_iframe) {
					resetpopups();

					deserialize_img(message.data.data.data.img, function(el) {
						message.data.data.data.img = el;

						popup_el_remote = sender;
						popup_el = null;
						real_popup_el = null;
						//console_log("Making popup", message);
						makePopup(message.data.source_imu, message.data.src, message.data.processing, message.data.data);
					});
				}
			} else if (message.type === "count_gallery") {
				count_gallery(message.data.nextprev, undefined, undefined, undefined, function(count) {
					respond(count);
				});
			} else if (message.type === "is_nextprev_valid") {
				is_nextprev_valid(message.data.nextprev, function(valid) {
					respond(valid);
				});
			} else if (message.type === "trigger_gallery") {
				trigger_gallery(message.data.dir, function(triggered) {
					respond(triggered);
				});
			} else if (message.type === "resetpopups") {
				resetpopups(true);
			} else if (message.type === "mousemove") {
				// todo: offset iframe location
				mousemove_cb(message.data);
			} else if (message.type === "action") {
				for (var i = 0; i < message.data.length; i++) {
					action_handler(message.data[i]);
				}
			} else if (message.type === "popup_open") {
				popup_el_remote = sender;
				last_popup_el = null;
			}
		};

		var handle_remote_event = function(message) {
			if (message.type === "remote") {
				var respond = function() {};
				if (message.response_id) {
					var response_id = message.response_id;

					respond = function(data) {
						remote_send_reply(message.from, response_id, data);
					};
				}

				if (message.from === current_frame_id || (message.to && current_frame_id !== message.to)) {
					return true;
				}

				remote_handle_message(message.data, message.from, respond);
				return true;
			} else if (message.type === "remote_reply") {
				if (message.response_id in remote_reply_ids) {
					var response_id = message.response_id;
					delete message.response_id;

					remote_reply_ids[response_id](message.data);
				}

				return true;
			}

			return false;
		}

		if (is_extension) {
			// TODO: move out of do_mouseover
			chrome.runtime.onMessage.addListener(function(message, sender, respond) {
				//console_log("ON_MESSAGE", message);
				if (message.type === "context_imu") {
					popup_trigger_reason = "contextmenu";
					trigger_popup(true);
				} else if (message.type === "popupaction") {
					if (message.data.action === "replace_images") {
						replace_images_full();
					} else if (message.data.action === "highlight_images") {
						highlight_images();
					}
				} else if (message.type === "settings_update") {
					//console_log(message);
					settings_updated_cb(message.data.changes);
				} else if (message.type === "request") {
					var response = message.data;

					if (!(response.id in extension_requests)) {
						// this happens when there's more than one frame per tab
						//console_error("Request ID " + response.id + " not in extension_requests");
						return;
					}

					if (response.data && response.data.responseType === "blob") {
						var enc = response.data._responseEncoded;

						if (enc) {
							var array = new Uint8Array(enc.value.length);
							for (var i = 0; i < enc.value.length; i++) {
								array[i] = enc.value.charCodeAt(i);
							}
							response.data.response = new native_blob([array.buffer], { type: enc.type });
						} else {
							response.data.response = null;
						}
					} else if (response.data && response.data.responseText && !response.data.response) {
						response.data.response = response.data.responseText;
					}

					var reqdata = extension_requests[response.id].data;

					var handle_event = function(event) {
						if (response.event === event && reqdata[event]) {
							reqdata[event](response.data);
						}
					};

					handle_event("onload");
					handle_event("onerror");
					handle_event("onprogress");
					handle_event("onabort");

					if (response.final) {
						delete extension_requests[response.id];
					}
				} else if (message.type === "remote" || message.type === "remote_reply") {
					handle_remote_event(message);
				}
			});
		} else {
			our_addEventListener(window, "message", function(event) {
				if (!can_use_remote() || !event.data || typeof event.data !== "object" || !(imu_message_key in event.data))
					return;

				// TODO: update id_to_iframe with event.source (remember that event.source is a window object, not an iframe)

				handle_remote_event(event.data[imu_message_key]);
			}, false);
		}

		our_addEventListener(document, 'contextmenu', function(event) {
			mouseContextX = event.clientX;
			mouseContextY = event.clientY;

			mouseAbsContextX = event.pageX;
			mouseAbsContextY = event.pageY;
		});

		var last_remote_mousemove = 0;
		var last_remote_mousemove_timer = null;
		var last_remote_mousemove_event = null;

		var wheel_cb = function(event) {
			if (settings.scroll_override_page && popups_active && popup_wheel_cb) {
				return popup_wheel_cb(event, true);
			}
		};

		our_addEventListener(document, "wheel", wheel_cb, {
			passive: false
		});

		var mousemove_cb = function(event) {
			mousepos_initialized = true;

			// https://stackoverflow.com/a/7790764
			event = event || window.event;

			if (event.pageX === null && event.clientX !== null) {
				eventDoc = (event.target && event.target.ownerDocument) || document;
				doc = eventDoc.documentElement;
				body = eventDoc.body;

				event.pageX = event.clientX + scrollLeft();
				event.pageY = event.clientY + scrollTop();
			}

			if (can_use_remote()) {
				if (!("remote_info" in event)) {
					event.remote_info = get_frame_info();
				}

				if (event.remote_info.id !== current_frame_id) {
					var iframe = find_iframe_for_info(event.remote_info);
					if (!iframe) {
						return;
					}

					//console_log(iframe);
					var bb = get_bounding_client_rect(iframe);

					// fixme: should this be done?
					event.clientX += bb.left;
					event.clientY += bb.top;

					event.pageX += bb.left + window.scrollX;
					event.pageY += bb.top + window.scrollY;
				} else if (is_in_iframe) {
					// todo: add timeouts to avoid too much cpu usage
					last_remote_mousemove_event = event;

					var mindelta = 16;
					if (!settings.mouseover_use_remote) {
						mindelta = 300; // we don't need precise movements, all we need is to inform the top frame that the mouse is here
					}

					var current_time = Date.now();
					var timeout = mindelta - (current_time - last_remote_mousemove);
					if (timeout < 1)
						timeout = 1;

					if (!last_remote_mousemove_timer) {
						last_remote_mousemove_timer = setTimeout(function() {
							last_remote_mousemove_timer = null;
							last_remote_mousemove = Date.now();
							remote_send_message("top", {
								type: "mousemove",
								data: serialize_event(last_remote_mousemove_event)
							});
						}, timeout);
					}
				}

				mouse_frame_id = event.remote_info.id;
			}

			mouseX = event.clientX;
			mouseY = event.clientY;

			mouseAbsX = event.pageX;
			mouseAbsY = event.pageY;

			if (waiting) {
				update_waiting();
			}

			if (popups_active) {
				do_popup_pan(popups[0], event, mouseX, mouseY);
			}

			var jitter_base = settings.mouseover_jitter_threshold;

			if (settings.mouseover_trigger_behavior === "keyboard" && get_close_need_mouseout() && popups_active &&
				popup_trigger_reason === "keyboard") {
				var img = popups[0].getElementsByTagName("img")[0];
				if (img) {
					var rect = get_popup_client_rect();

					var our_jitter = jitter_base;

					if (in_clientrect(mouseX, mouseY, rect, our_jitter)) {
						can_close_popup[1] = false;
					} else {
						if (can_close_popup[0]) {
							stop_waiting();
							resetpopups();
						} else {
							can_close_popup[1] = true;
						}
					}
				}
			}

			if (mouseover_mouse_enabled()) {
				if (popup_trigger_reason === "mouse") {
					if (popups_active) {
						// FIXME: why was this not in if (popups_active)?
						// The reason for putting it here is that if the mouse moves (even within jitter thresh) after a single popup is open, it will cancel the popup request
						if (delay_handle) {
							clearTimeout(delay_handle);
							delay_handle = null;

							if (false && waiting)
								stop_waiting();
						}

						var jitter_threshx = 40;
						var jitter_threshy = jitter_threshx;

						var img = popups[0].getElementsByTagName("img")[0];
						var imgmiddleX = null;
						var imgmiddleY = null;
						var in_img_jitter = false;
						if (img) {
							// TODO: call this less often
							var rect = img.getBoundingClientRect();

							in_img_jitter = in_clientrect(mouseX, mouseY, rect, jitter_base);

							// why this instead of getBoundingClientRect?
							//var w = Math.min(parseInt(img.style.maxWidth), img.naturalWidth);
							//var h = Math.min(parseInt(img.style.maxHeight), img.naturalHeight);
							var w = rect.width;
							var h = rect.height;

							imgmiddleX = rect.x + rect.width / 2;
							imgmiddleY = rect.y + rect.height / 2;

							jitter_threshx = Math.max(jitter_threshx, w / 2);
							jitter_threshy = Math.max(jitter_threshy, h / 2);

							jitter_threshx += jitter_base;
							jitter_threshy += jitter_base;

							/*console_log(jitter_threshx, img.naturalWidth, w);
							console_log(jitter_threshy, img.naturalHeight, h);*/
							if (mouse_in_image_yet === false) {
								if (in_clientrect(mouseX, mouseY, rect)) {
									mouse_in_image_yet = true;
									//mouseDelayX = mouseX;
									//mouseDelayY = mouseY;

									//mouseDelayX = imgmiddleX;
									//mouseDelayY = imgmiddleY;

									if (false) {
										var viewport = get_viewport();
										if ((mouseDelayX + jitter_threshx) > viewport[0]) {
											mouseDelayX = viewport[0] - jitter_threshx;
										}
										if ((mouseDelayX - jitter_threshx) < 0) {
											mouseDelayX = jitter_threshx;
										}

										if ((mouseDelayY + jitter_threshy) > viewport[1]) {
											mouseDelayY = viewport[1] - jitter_threshy;
										}
										if ((mouseDelayY - jitter_threshy) < 0) {
											mouseDelayY = jitter_threshy;
										}
									}
								}
							}
						}

						var do_mouse_reset = function() {
							if (popup_hold) {
								can_close_popup[1] = true;
							} else {
								resetpopups();
							}
						};

						var close_el_policy = get_single_setting("mouseover_close_el_policy");
						var close_on_leave_el = (close_el_policy === "thumbnail" || close_el_policy === "both") && popup_el && !popup_el_automatic;
						var outside_of_popup_el = false;
						var popup_el_hidden = false;

						if (close_on_leave_el) {
							var popup_el_rect = get_bounding_client_rect(popup_el);
							if (popup_el_rect && popup_el_rect.width > 0 && popup_el_rect.height > 0) {
								var our_in_img_jitter = in_img_jitter;
								if (close_el_policy === "thumbnail")
									our_in_img_jitter = false;

								if (!in_clientrect(mouseX, mouseY, popup_el_rect) && !our_in_img_jitter) {
									outside_of_popup_el = true;

									if (close_el_policy === "thumbnail") {
										return do_mouse_reset();
									}
								}
							} else {
								// the element must be hidden
								popup_el_hidden = true;
							}
						}

						can_close_popup[1] = false;
						if (mouse_in_image_yet && (!close_on_leave_el || outside_of_popup_el || popup_el_hidden)) {
							if (imgmiddleX && imgmiddleY &&
								(Math.abs(mouseX - imgmiddleX) > jitter_threshx ||
								 Math.abs(mouseY - imgmiddleY) > jitter_threshy)) {
								//console_log(mouseX, imgmiddleX, jitter_threshx);
								//console_log(mouseY, imgmiddleY, jitter_threshy);

								do_mouse_reset();
							}
						} else if (close_on_leave_el) {
							if (outside_of_popup_el) {
								do_mouse_reset();
							}
						}
					} else if (delay_handle_triggering) {
						if (next_popup_el && settings.mouseover_cancel_popup_when_elout) {
							var popup_el_rect = get_bounding_client_rect(next_popup_el);
							if (!in_clientrect(mouseX, mouseY, popup_el_rect)) {
								resetpopups();
							}
						}
					}
				}

				// FIXME: this is rather weird. Less CPU usage, but doesn't behave in the way one would expect
				if ((!popups_active || popup_el_automatic) && !should_exclude_imagetab()) {
					if (delay_handle && !settings.mouseover_trigger_mouseover) {
						var trigger_mouse_jitter_thresh = 10;

						if (Math.abs(mouseX - mouseDelayX) < trigger_mouse_jitter_thresh &&
							Math.abs(mouseY - mouseDelayY) < trigger_mouse_jitter_thresh)
							return;

						clearTimeout(delay_handle);
						delay_handle = null;
					}

					mouseDelayX = mouseX;
					mouseDelayY = mouseY;
					//mouse_in_image_yet = false;

					if (last_popup_el) {
						var popup_el_rect = get_bounding_client_rect(last_popup_el);
						if (!in_clientrect(mouseX, mouseY, popup_el_rect)) {
							last_popup_el = null;
						}
					}

					if (!settings.mouseover_trigger_mouseover) {
						delay_handle = setTimeout(function() {
							if (!popup_mouse_head())
								return;

							trigger_popup();
						}, delay * 1000);
					}
				}
			}
		};

		our_addEventListener(document, 'mousemove', mousemove_cb);
	}

	function do_websitehome() {
		unsafeWindow.imu_variable = bigimage_recursive;
		unsafeWindow.imu_inject = 1;
		unsafeWindow.do_imu = function(url, cb) {
			return bigimage_recursive(url, {
				fill_object: true,
				do_request: do_request,
				cb: cb
			});
		};

		var orig_set_max = unsafeWindow.set_max;
		unsafeWindow.set_max = function(obj) {
			if (!obj || !obj[0].url) {
				orig_set_max(obj);
				return;
			}

			var loop_url = function(obj, cb, options, lasturl) {
				check_image_get(obj, function(img, newurl) {
					var finalurl = newurl;
					if (!newurl && img && img.finalUrl) {
						finalurl = img.finalUrl;
					}

					if (!finalurl) {
						cb(img, newurl);
						return;
					}

					if (obj_indexOf(obj, finalurl) < 0 &&
						lasturl !== finalurl) {
						bigimage_recursive(finalurl, {
							fill_object: true,
							do_request: do_request,
							cb: function(obj) {
								revoke_objecturl(img);
								loop_url(obj, cb, options, finalurl);
							}
						});
					} else {
						cb(img, newurl, obj[obj_indexOf(obj, finalurl)]);
					}
				}, options);
			};

			if (settings.website_image) {
				loop_url(obj, function(img, url, obj) {
					if (!img) {
						orig_set_max("broken");
					} else {
						var newobj = obj;
						if (!newobj)
							newobj = {url: url};

						orig_set_max([newobj]);
						maximgel.src = img.src;
					}
				}, {running: true});
			} else if (obj.can_head) {
				loop_url(obj, function(resp) {
					if (!resp) {
						orig_set_max("broken");
					} else {
						var newobj = obj;
						if (!newobj)
							newobj = {url: resp.finalUrl};

						orig_set_max([newobj]);
					}
				}, {running: true, head: true});
			} else {
				orig_set_max(obj);
			}
		};
	}

	var do_userscript_page = function(imgel, latest_version) {
		var status_container_el = document.createElement("div");
		status_container_el.style.marginBottom = "2em";

		var version_el = document.createElement("span");
		version_el.style.fontSize = "90%";
		version_el.style.fontWeight = 800;
		version_el.style.marginRight = "2em";

		var version = null;
		try {
			version = gm_info.script.version;
		} catch (e) {
		}

		version_el.innerText = "Installed";

		if (version !== null) {
			version_el.innerText += " (v" + version;

			if (latest_version) {
				var compared = version_compare(latest_version, version);
				if (compared === -1) {
					version_el.innerText += ", update available";
				}
			}

			version_el.innerText += ")";
		}

		options_el = document.createElement("a");
		options_el.innerText = "Options";
		options_el.style.background = "#0af";
		options_el.style.padding = "0.5em 1em";
		options_el.style.color = "white";
		options_el.style.display = "inline-block";
		options_el.style.textDecoration = "none";
		options_el.target = "_blank";
		options_el.href = "https://qsniyg.github.io/maxurl/options.html";

		status_container_el.appendChild(version_el);
		status_container_el.appendChild(options_el);
		imgel.parentElement.appendChild(status_container_el);
	};

	var do_greasyfork_page = function() {
		var imgel = document.querySelector("div.script-author-description > center > img[alt='Image Max URL']");
		if (!imgel)
			return;

		// make sure it's the same general layout
		if (imgel.parentElement.previousElementSibling ||
			imgel.parentElement.nextElementSibling.tagName !== "UL")
			return;

		var gf_version = null;
		var gf_version_el = document.querySelector("dd.script-show-version");
		if (gf_version_el) {
			gf_version = gf_version_el.innerText.replace(/^\s*|\s*$/, "");
		}

		do_userscript_page(imgel, gf_version);
	};

	var do_oujs_page = function() {
		var imgel = document.querySelector("div#user-content > p[align='center'] img[alt='Image Max URL']");
		if (!imgel)
			return;

		// make sure it's the same general layout
		if (imgel.parentElement.previousElementSibling ||
			imgel.parentElement.nextElementSibling.tagName !== "UL")
			return;

		var latest_version = null;
		var version_icon_el = document.querySelector("div.script-meta i.fa-history");
		if (version_icon_el) {
			var code_el = version_icon_el.parentElement.querySelector("code");
			if (code_el) {
				latest_version = code_el.innerText.replace(/[+].*/, "");
				if (!/^[0-9.]+$/.test(latest_version))
					latest_version = null;
			}
		}

		do_userscript_page(imgel, latest_version);
	};

	function start() {
		do_export();

		if (is_userscript || is_extension) {
			if (is_maxurl_website || is_options_page) {
				onload(function() {
					update_dark_mode();
				});
			}

			if (is_options_page) {
				onload(function() {
					try {
						do_options();
					} catch (e) {
						console_error(e);

						var error_pre = document.createElement("pre");
						error_pre.style.fontFamily = "monospace";
						error_pre.style.margin = "1em";
						error_pre.innerText = get_crashlog_info() + "\n" + e.toString() + "\n" + e.stack;

						var error_div = document.createElement("div");
						var error_div_text = "Error loading options page, please report this to <a href='" + github_issues_page + "'>" + github_issues_page + "</a>, ";
						error_div_text += "and include the following information in the report:";
						error_div.innerHTML = error_div_text;

						error_div.appendChild(error_pre);

						document.body.innerHTML = error_div.outerHTML;
					}
				});
			}

			if (settings.imu_enabled) {
				if (settings.redirect) {
					do_redirect();
				}

				if (settings.website_inject_imu &&
					(window.location.href.match(/^https?:\/\/qsniyg\.github\.io\/+maxurl(\/+|\/+index\.html)?(?:[?#].*)?$/) ||
						window.location.href.match(/^file:\/\/.*\/maxurl\/site\/index\.html/))) {
					if (typeof (unsafeWindow) !== "undefined") {
						onload(function () {
							do_websitehome();
						});
					}
				}
			}

			if (is_userscript) {
				if (window.location.href.match(/^https?:\/\/(?:www\.)?greasyfork\.org\/+[^/]*\/+scripts\/+36662(?:-[^/]*)?(?:[?#].*)?$/)) {
					onload(function() {
						do_greasyfork_page();
					});
				} else if (window.location.href.match(/^https?:\/\/(?:www\.)?openuserjs\.org\/+scripts\/+qsniyg\/+Image_Max_URL(?:[?#].*)?$/)) {
					onload(function() {
						do_oujs_page();
					});
				}
			}

			do_mouseover();

			if (is_extension) {
				extension_send_message({
					type: "ready"
				});
			}
		}
	}

	do_config();
})();

// @license-end
